import path from 'path';
import fs from 'fs';

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, basedir, module) {
	return module = {
	  path: basedir,
	  exports: {},
	  require: function (path, base) {
      return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
    }
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */

var intToCharMap = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'.split('');

/**
 * Encode an integer in the range of 0 to 63 to a single base 64 digit.
 */
var encode = function (number) {
  if (0 <= number && number < intToCharMap.length) {
    return intToCharMap[number];
  }
  throw new TypeError("Must be between 0 and 63: " + number);
};

/**
 * Decode a single base 64 character code digit to an integer. Returns -1 on
 * failure.
 */
var decode = function (charCode) {
  var bigA = 65;     // 'A'
  var bigZ = 90;     // 'Z'

  var littleA = 97;  // 'a'
  var littleZ = 122; // 'z'

  var zero = 48;     // '0'
  var nine = 57;     // '9'

  var plus = 43;     // '+'
  var slash = 47;    // '/'

  var littleOffset = 26;
  var numberOffset = 52;

  // 0 - 25: ABCDEFGHIJKLMNOPQRSTUVWXYZ
  if (bigA <= charCode && charCode <= bigZ) {
    return (charCode - bigA);
  }

  // 26 - 51: abcdefghijklmnopqrstuvwxyz
  if (littleA <= charCode && charCode <= littleZ) {
    return (charCode - littleA + littleOffset);
  }

  // 52 - 61: 0123456789
  if (zero <= charCode && charCode <= nine) {
    return (charCode - zero + numberOffset);
  }

  // 62: +
  if (charCode == plus) {
    return 62;
  }

  // 63: /
  if (charCode == slash) {
    return 63;
  }

  // Invalid base64 digit.
  return -1;
};

var base64 = {
	encode: encode,
	decode: decode
};

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 *
 * Based on the Base 64 VLQ implementation in Closure Compiler:
 * https://code.google.com/p/closure-compiler/source/browse/trunk/src/com/google/debugging/sourcemap/Base64VLQ.java
 *
 * Copyright 2011 The Closure Compiler Authors. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *  * Neither the name of Google Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



// A single base 64 digit can contain 6 bits of data. For the base 64 variable
// length quantities we use in the source map spec, the first bit is the sign,
// the next four bits are the actual value, and the 6th bit is the
// continuation bit. The continuation bit tells us whether there are more
// digits in this value following this digit.
//
//   Continuation
//   |    Sign
//   |    |
//   V    V
//   101011

var VLQ_BASE_SHIFT = 5;

// binary: 100000
var VLQ_BASE = 1 << VLQ_BASE_SHIFT;

// binary: 011111
var VLQ_BASE_MASK = VLQ_BASE - 1;

// binary: 100000
var VLQ_CONTINUATION_BIT = VLQ_BASE;

/**
 * Converts from a two-complement value to a value where the sign bit is
 * placed in the least significant bit.  For example, as decimals:
 *   1 becomes 2 (10 binary), -1 becomes 3 (11 binary)
 *   2 becomes 4 (100 binary), -2 becomes 5 (101 binary)
 */
function toVLQSigned(aValue) {
  return aValue < 0
    ? ((-aValue) << 1) + 1
    : (aValue << 1) + 0;
}

/**
 * Converts to a two-complement value from a value where the sign bit is
 * placed in the least significant bit.  For example, as decimals:
 *   2 (10 binary) becomes 1, 3 (11 binary) becomes -1
 *   4 (100 binary) becomes 2, 5 (101 binary) becomes -2
 */
function fromVLQSigned(aValue) {
  var isNegative = (aValue & 1) === 1;
  var shifted = aValue >> 1;
  return isNegative
    ? -shifted
    : shifted;
}

/**
 * Returns the base 64 VLQ encoded value.
 */
var encode$1 = function base64VLQ_encode(aValue) {
  var encoded = "";
  var digit;

  var vlq = toVLQSigned(aValue);

  do {
    digit = vlq & VLQ_BASE_MASK;
    vlq >>>= VLQ_BASE_SHIFT;
    if (vlq > 0) {
      // There are still more digits in this value, so we must make sure the
      // continuation bit is marked.
      digit |= VLQ_CONTINUATION_BIT;
    }
    encoded += base64.encode(digit);
  } while (vlq > 0);

  return encoded;
};

/**
 * Decodes the next base 64 VLQ value from the given string and returns the
 * value and the rest of the string via the out parameter.
 */
var decode$1 = function base64VLQ_decode(aStr, aIndex, aOutParam) {
  var strLen = aStr.length;
  var result = 0;
  var shift = 0;
  var continuation, digit;

  do {
    if (aIndex >= strLen) {
      throw new Error("Expected more digits in base 64 VLQ value.");
    }

    digit = base64.decode(aStr.charCodeAt(aIndex++));
    if (digit === -1) {
      throw new Error("Invalid base64 digit: " + aStr.charAt(aIndex - 1));
    }

    continuation = !!(digit & VLQ_CONTINUATION_BIT);
    digit &= VLQ_BASE_MASK;
    result = result + (digit << shift);
    shift += VLQ_BASE_SHIFT;
  } while (continuation);

  aOutParam.value = fromVLQSigned(result);
  aOutParam.rest = aIndex;
};

var base64Vlq = {
	encode: encode$1,
	decode: decode$1
};

var util = createCommonjsModule(function (module, exports) {
/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */

/**
 * This is a helper function for getting values from parameter/options
 * objects.
 *
 * @param args The object we are extracting values from
 * @param name The name of the property we are getting.
 * @param defaultValue An optional value to return if the property is missing
 * from the object. If this is not specified and the property is missing, an
 * error will be thrown.
 */
function getArg(aArgs, aName, aDefaultValue) {
  if (aName in aArgs) {
    return aArgs[aName];
  } else if (arguments.length === 3) {
    return aDefaultValue;
  } else {
    throw new Error('"' + aName + '" is a required argument.');
  }
}
exports.getArg = getArg;

var urlRegexp = /^(?:([\w+\-.]+):)?\/\/(?:(\w+:\w+)@)?([\w.-]*)(?::(\d+))?(.*)$/;
var dataUrlRegexp = /^data:.+\,.+$/;

function urlParse(aUrl) {
  var match = aUrl.match(urlRegexp);
  if (!match) {
    return null;
  }
  return {
    scheme: match[1],
    auth: match[2],
    host: match[3],
    port: match[4],
    path: match[5]
  };
}
exports.urlParse = urlParse;

function urlGenerate(aParsedUrl) {
  var url = '';
  if (aParsedUrl.scheme) {
    url += aParsedUrl.scheme + ':';
  }
  url += '//';
  if (aParsedUrl.auth) {
    url += aParsedUrl.auth + '@';
  }
  if (aParsedUrl.host) {
    url += aParsedUrl.host;
  }
  if (aParsedUrl.port) {
    url += ":" + aParsedUrl.port;
  }
  if (aParsedUrl.path) {
    url += aParsedUrl.path;
  }
  return url;
}
exports.urlGenerate = urlGenerate;

/**
 * Normalizes a path, or the path portion of a URL:
 *
 * - Replaces consecutive slashes with one slash.
 * - Removes unnecessary '.' parts.
 * - Removes unnecessary '<dir>/..' parts.
 *
 * Based on code in the Node.js 'path' core module.
 *
 * @param aPath The path or url to normalize.
 */
function normalize(aPath) {
  var path = aPath;
  var url = urlParse(aPath);
  if (url) {
    if (!url.path) {
      return aPath;
    }
    path = url.path;
  }
  var isAbsolute = exports.isAbsolute(path);

  var parts = path.split(/\/+/);
  for (var part, up = 0, i = parts.length - 1; i >= 0; i--) {
    part = parts[i];
    if (part === '.') {
      parts.splice(i, 1);
    } else if (part === '..') {
      up++;
    } else if (up > 0) {
      if (part === '') {
        // The first part is blank if the path is absolute. Trying to go
        // above the root is a no-op. Therefore we can remove all '..' parts
        // directly after the root.
        parts.splice(i + 1, up);
        up = 0;
      } else {
        parts.splice(i, 2);
        up--;
      }
    }
  }
  path = parts.join('/');

  if (path === '') {
    path = isAbsolute ? '/' : '.';
  }

  if (url) {
    url.path = path;
    return urlGenerate(url);
  }
  return path;
}
exports.normalize = normalize;

/**
 * Joins two paths/URLs.
 *
 * @param aRoot The root path or URL.
 * @param aPath The path or URL to be joined with the root.
 *
 * - If aPath is a URL or a data URI, aPath is returned, unless aPath is a
 *   scheme-relative URL: Then the scheme of aRoot, if any, is prepended
 *   first.
 * - Otherwise aPath is a path. If aRoot is a URL, then its path portion
 *   is updated with the result and aRoot is returned. Otherwise the result
 *   is returned.
 *   - If aPath is absolute, the result is aPath.
 *   - Otherwise the two paths are joined with a slash.
 * - Joining for example 'http://' and 'www.example.com' is also supported.
 */
function join(aRoot, aPath) {
  if (aRoot === "") {
    aRoot = ".";
  }
  if (aPath === "") {
    aPath = ".";
  }
  var aPathUrl = urlParse(aPath);
  var aRootUrl = urlParse(aRoot);
  if (aRootUrl) {
    aRoot = aRootUrl.path || '/';
  }

  // `join(foo, '//www.example.org')`
  if (aPathUrl && !aPathUrl.scheme) {
    if (aRootUrl) {
      aPathUrl.scheme = aRootUrl.scheme;
    }
    return urlGenerate(aPathUrl);
  }

  if (aPathUrl || aPath.match(dataUrlRegexp)) {
    return aPath;
  }

  // `join('http://', 'www.example.com')`
  if (aRootUrl && !aRootUrl.host && !aRootUrl.path) {
    aRootUrl.host = aPath;
    return urlGenerate(aRootUrl);
  }

  var joined = aPath.charAt(0) === '/'
    ? aPath
    : normalize(aRoot.replace(/\/+$/, '') + '/' + aPath);

  if (aRootUrl) {
    aRootUrl.path = joined;
    return urlGenerate(aRootUrl);
  }
  return joined;
}
exports.join = join;

exports.isAbsolute = function (aPath) {
  return aPath.charAt(0) === '/' || urlRegexp.test(aPath);
};

/**
 * Make a path relative to a URL or another path.
 *
 * @param aRoot The root path or URL.
 * @param aPath The path or URL to be made relative to aRoot.
 */
function relative(aRoot, aPath) {
  if (aRoot === "") {
    aRoot = ".";
  }

  aRoot = aRoot.replace(/\/$/, '');

  // It is possible for the path to be above the root. In this case, simply
  // checking whether the root is a prefix of the path won't work. Instead, we
  // need to remove components from the root one by one, until either we find
  // a prefix that fits, or we run out of components to remove.
  var level = 0;
  while (aPath.indexOf(aRoot + '/') !== 0) {
    var index = aRoot.lastIndexOf("/");
    if (index < 0) {
      return aPath;
    }

    // If the only part of the root that is left is the scheme (i.e. http://,
    // file:///, etc.), one or more slashes (/), or simply nothing at all, we
    // have exhausted all components, so the path is not relative to the root.
    aRoot = aRoot.slice(0, index);
    if (aRoot.match(/^([^\/]+:\/)?\/*$/)) {
      return aPath;
    }

    ++level;
  }

  // Make sure we add a "../" for each component we removed from the root.
  return Array(level + 1).join("../") + aPath.substr(aRoot.length + 1);
}
exports.relative = relative;

var supportsNullProto = (function () {
  var obj = Object.create(null);
  return !('__proto__' in obj);
}());

function identity (s) {
  return s;
}

/**
 * Because behavior goes wacky when you set `__proto__` on objects, we
 * have to prefix all the strings in our set with an arbitrary character.
 *
 * See https://github.com/mozilla/source-map/pull/31 and
 * https://github.com/mozilla/source-map/issues/30
 *
 * @param String aStr
 */
function toSetString(aStr) {
  if (isProtoString(aStr)) {
    return '$' + aStr;
  }

  return aStr;
}
exports.toSetString = supportsNullProto ? identity : toSetString;

function fromSetString(aStr) {
  if (isProtoString(aStr)) {
    return aStr.slice(1);
  }

  return aStr;
}
exports.fromSetString = supportsNullProto ? identity : fromSetString;

function isProtoString(s) {
  if (!s) {
    return false;
  }

  var length = s.length;

  if (length < 9 /* "__proto__".length */) {
    return false;
  }

  if (s.charCodeAt(length - 1) !== 95  /* '_' */ ||
      s.charCodeAt(length - 2) !== 95  /* '_' */ ||
      s.charCodeAt(length - 3) !== 111 /* 'o' */ ||
      s.charCodeAt(length - 4) !== 116 /* 't' */ ||
      s.charCodeAt(length - 5) !== 111 /* 'o' */ ||
      s.charCodeAt(length - 6) !== 114 /* 'r' */ ||
      s.charCodeAt(length - 7) !== 112 /* 'p' */ ||
      s.charCodeAt(length - 8) !== 95  /* '_' */ ||
      s.charCodeAt(length - 9) !== 95  /* '_' */) {
    return false;
  }

  for (var i = length - 10; i >= 0; i--) {
    if (s.charCodeAt(i) !== 36 /* '$' */) {
      return false;
    }
  }

  return true;
}

/**
 * Comparator between two mappings where the original positions are compared.
 *
 * Optionally pass in `true` as `onlyCompareGenerated` to consider two
 * mappings with the same original source/line/column, but different generated
 * line and column the same. Useful when searching for a mapping with a
 * stubbed out mapping.
 */
function compareByOriginalPositions(mappingA, mappingB, onlyCompareOriginal) {
  var cmp = strcmp(mappingA.source, mappingB.source);
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.originalLine - mappingB.originalLine;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.originalColumn - mappingB.originalColumn;
  if (cmp !== 0 || onlyCompareOriginal) {
    return cmp;
  }

  cmp = mappingA.generatedColumn - mappingB.generatedColumn;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.generatedLine - mappingB.generatedLine;
  if (cmp !== 0) {
    return cmp;
  }

  return strcmp(mappingA.name, mappingB.name);
}
exports.compareByOriginalPositions = compareByOriginalPositions;

/**
 * Comparator between two mappings with deflated source and name indices where
 * the generated positions are compared.
 *
 * Optionally pass in `true` as `onlyCompareGenerated` to consider two
 * mappings with the same generated line and column, but different
 * source/name/original line and column the same. Useful when searching for a
 * mapping with a stubbed out mapping.
 */
function compareByGeneratedPositionsDeflated(mappingA, mappingB, onlyCompareGenerated) {
  var cmp = mappingA.generatedLine - mappingB.generatedLine;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.generatedColumn - mappingB.generatedColumn;
  if (cmp !== 0 || onlyCompareGenerated) {
    return cmp;
  }

  cmp = strcmp(mappingA.source, mappingB.source);
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.originalLine - mappingB.originalLine;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.originalColumn - mappingB.originalColumn;
  if (cmp !== 0) {
    return cmp;
  }

  return strcmp(mappingA.name, mappingB.name);
}
exports.compareByGeneratedPositionsDeflated = compareByGeneratedPositionsDeflated;

function strcmp(aStr1, aStr2) {
  if (aStr1 === aStr2) {
    return 0;
  }

  if (aStr1 === null) {
    return 1; // aStr2 !== null
  }

  if (aStr2 === null) {
    return -1; // aStr1 !== null
  }

  if (aStr1 > aStr2) {
    return 1;
  }

  return -1;
}

/**
 * Comparator between two mappings with inflated source and name strings where
 * the generated positions are compared.
 */
function compareByGeneratedPositionsInflated(mappingA, mappingB) {
  var cmp = mappingA.generatedLine - mappingB.generatedLine;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.generatedColumn - mappingB.generatedColumn;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = strcmp(mappingA.source, mappingB.source);
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.originalLine - mappingB.originalLine;
  if (cmp !== 0) {
    return cmp;
  }

  cmp = mappingA.originalColumn - mappingB.originalColumn;
  if (cmp !== 0) {
    return cmp;
  }

  return strcmp(mappingA.name, mappingB.name);
}
exports.compareByGeneratedPositionsInflated = compareByGeneratedPositionsInflated;

/**
 * Strip any JSON XSSI avoidance prefix from the string (as documented
 * in the source maps specification), and then parse the string as
 * JSON.
 */
function parseSourceMapInput(str) {
  return JSON.parse(str.replace(/^\)]}'[^\n]*\n/, ''));
}
exports.parseSourceMapInput = parseSourceMapInput;

/**
 * Compute the URL of a source given the the source root, the source's
 * URL, and the source map's URL.
 */
function computeSourceURL(sourceRoot, sourceURL, sourceMapURL) {
  sourceURL = sourceURL || '';

  if (sourceRoot) {
    // This follows what Chrome does.
    if (sourceRoot[sourceRoot.length - 1] !== '/' && sourceURL[0] !== '/') {
      sourceRoot += '/';
    }
    // The spec says:
    //   Line 4: An optional source root, useful for relocating source
    //   files on a server or removing repeated values in the
    //   “sources” entry.  This value is prepended to the individual
    //   entries in the “source” field.
    sourceURL = sourceRoot + sourceURL;
  }

  // Historically, SourceMapConsumer did not take the sourceMapURL as
  // a parameter.  This mode is still somewhat supported, which is why
  // this code block is conditional.  However, it's preferable to pass
  // the source map URL to SourceMapConsumer, so that this function
  // can implement the source URL resolution algorithm as outlined in
  // the spec.  This block is basically the equivalent of:
  //    new URL(sourceURL, sourceMapURL).toString()
  // ... except it avoids using URL, which wasn't available in the
  // older releases of node still supported by this library.
  //
  // The spec says:
  //   If the sources are not absolute URLs after prepending of the
  //   “sourceRoot”, the sources are resolved relative to the
  //   SourceMap (like resolving script src in a html document).
  if (sourceMapURL) {
    var parsed = urlParse(sourceMapURL);
    if (!parsed) {
      throw new Error("sourceMapURL could not be parsed");
    }
    if (parsed.path) {
      // Strip the last path component, but keep the "/".
      var index = parsed.path.lastIndexOf('/');
      if (index >= 0) {
        parsed.path = parsed.path.substring(0, index + 1);
      }
    }
    sourceURL = join(urlGenerate(parsed), sourceURL);
  }

  return normalize(sourceURL);
}
exports.computeSourceURL = computeSourceURL;
});
var util_1 = util.getArg;
var util_2 = util.urlParse;
var util_3 = util.urlGenerate;
var util_4 = util.normalize;
var util_5 = util.join;
var util_6 = util.isAbsolute;
var util_7 = util.relative;
var util_8 = util.toSetString;
var util_9 = util.fromSetString;
var util_10 = util.compareByOriginalPositions;
var util_11 = util.compareByGeneratedPositionsDeflated;
var util_12 = util.compareByGeneratedPositionsInflated;
var util_13 = util.parseSourceMapInput;
var util_14 = util.computeSourceURL;

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */


var has = Object.prototype.hasOwnProperty;
var hasNativeMap = typeof Map !== "undefined";

/**
 * A data structure which is a combination of an array and a set. Adding a new
 * member is O(1), testing for membership is O(1), and finding the index of an
 * element is O(1). Removing elements from the set is not supported. Only
 * strings are supported for membership.
 */
function ArraySet() {
  this._array = [];
  this._set = hasNativeMap ? new Map() : Object.create(null);
}

/**
 * Static method for creating ArraySet instances from an existing array.
 */
ArraySet.fromArray = function ArraySet_fromArray(aArray, aAllowDuplicates) {
  var set = new ArraySet();
  for (var i = 0, len = aArray.length; i < len; i++) {
    set.add(aArray[i], aAllowDuplicates);
  }
  return set;
};

/**
 * Return how many unique items are in this ArraySet. If duplicates have been
 * added, than those do not count towards the size.
 *
 * @returns Number
 */
ArraySet.prototype.size = function ArraySet_size() {
  return hasNativeMap ? this._set.size : Object.getOwnPropertyNames(this._set).length;
};

/**
 * Add the given string to this set.
 *
 * @param String aStr
 */
ArraySet.prototype.add = function ArraySet_add(aStr, aAllowDuplicates) {
  var sStr = hasNativeMap ? aStr : util.toSetString(aStr);
  var isDuplicate = hasNativeMap ? this.has(aStr) : has.call(this._set, sStr);
  var idx = this._array.length;
  if (!isDuplicate || aAllowDuplicates) {
    this._array.push(aStr);
  }
  if (!isDuplicate) {
    if (hasNativeMap) {
      this._set.set(aStr, idx);
    } else {
      this._set[sStr] = idx;
    }
  }
};

/**
 * Is the given string a member of this set?
 *
 * @param String aStr
 */
ArraySet.prototype.has = function ArraySet_has(aStr) {
  if (hasNativeMap) {
    return this._set.has(aStr);
  } else {
    var sStr = util.toSetString(aStr);
    return has.call(this._set, sStr);
  }
};

/**
 * What is the index of the given string in the array?
 *
 * @param String aStr
 */
ArraySet.prototype.indexOf = function ArraySet_indexOf(aStr) {
  if (hasNativeMap) {
    var idx = this._set.get(aStr);
    if (idx >= 0) {
        return idx;
    }
  } else {
    var sStr = util.toSetString(aStr);
    if (has.call(this._set, sStr)) {
      return this._set[sStr];
    }
  }

  throw new Error('"' + aStr + '" is not in the set.');
};

/**
 * What is the element at the given index?
 *
 * @param Number aIdx
 */
ArraySet.prototype.at = function ArraySet_at(aIdx) {
  if (aIdx >= 0 && aIdx < this._array.length) {
    return this._array[aIdx];
  }
  throw new Error('No element indexed by ' + aIdx);
};

/**
 * Returns the array representation of this set (which has the proper indices
 * indicated by indexOf). Note that this is a copy of the internal array used
 * for storing the members so that no one can mess with internal state.
 */
ArraySet.prototype.toArray = function ArraySet_toArray() {
  return this._array.slice();
};

var ArraySet_1 = ArraySet;

var arraySet = {
	ArraySet: ArraySet_1
};

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2014 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */



/**
 * Determine whether mappingB is after mappingA with respect to generated
 * position.
 */
function generatedPositionAfter(mappingA, mappingB) {
  // Optimized for most common case
  var lineA = mappingA.generatedLine;
  var lineB = mappingB.generatedLine;
  var columnA = mappingA.generatedColumn;
  var columnB = mappingB.generatedColumn;
  return lineB > lineA || lineB == lineA && columnB >= columnA ||
         util.compareByGeneratedPositionsInflated(mappingA, mappingB) <= 0;
}

/**
 * A data structure to provide a sorted view of accumulated mappings in a
 * performance conscious manner. It trades a neglibable overhead in general
 * case for a large speedup in case of mappings being added in order.
 */
function MappingList() {
  this._array = [];
  this._sorted = true;
  // Serves as infimum
  this._last = {generatedLine: -1, generatedColumn: 0};
}

/**
 * Iterate through internal items. This method takes the same arguments that
 * `Array.prototype.forEach` takes.
 *
 * NOTE: The order of the mappings is NOT guaranteed.
 */
MappingList.prototype.unsortedForEach =
  function MappingList_forEach(aCallback, aThisArg) {
    this._array.forEach(aCallback, aThisArg);
  };

/**
 * Add the given source mapping.
 *
 * @param Object aMapping
 */
MappingList.prototype.add = function MappingList_add(aMapping) {
  if (generatedPositionAfter(this._last, aMapping)) {
    this._last = aMapping;
    this._array.push(aMapping);
  } else {
    this._sorted = false;
    this._array.push(aMapping);
  }
};

/**
 * Returns the flat, sorted array of mappings. The mappings are sorted by
 * generated position.
 *
 * WARNING: This method returns internal data without copying, for
 * performance. The return value must NOT be mutated, and should be treated as
 * an immutable borrow. If you want to take ownership, you must make your own
 * copy.
 */
MappingList.prototype.toArray = function MappingList_toArray() {
  if (!this._sorted) {
    this._array.sort(util.compareByGeneratedPositionsInflated);
    this._sorted = true;
  }
  return this._array;
};

var MappingList_1 = MappingList;

var mappingList = {
	MappingList: MappingList_1
};

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */



var ArraySet$1 = arraySet.ArraySet;
var MappingList$1 = mappingList.MappingList;

/**
 * An instance of the SourceMapGenerator represents a source map which is
 * being built incrementally. You may pass an object with the following
 * properties:
 *
 *   - file: The filename of the generated source.
 *   - sourceRoot: A root for all relative URLs in this source map.
 */
function SourceMapGenerator(aArgs) {
  if (!aArgs) {
    aArgs = {};
  }
  this._file = util.getArg(aArgs, 'file', null);
  this._sourceRoot = util.getArg(aArgs, 'sourceRoot', null);
  this._skipValidation = util.getArg(aArgs, 'skipValidation', false);
  this._sources = new ArraySet$1();
  this._names = new ArraySet$1();
  this._mappings = new MappingList$1();
  this._sourcesContents = null;
}

SourceMapGenerator.prototype._version = 3;

/**
 * Creates a new SourceMapGenerator based on a SourceMapConsumer
 *
 * @param aSourceMapConsumer The SourceMap.
 */
SourceMapGenerator.fromSourceMap =
  function SourceMapGenerator_fromSourceMap(aSourceMapConsumer) {
    var sourceRoot = aSourceMapConsumer.sourceRoot;
    var generator = new SourceMapGenerator({
      file: aSourceMapConsumer.file,
      sourceRoot: sourceRoot
    });
    aSourceMapConsumer.eachMapping(function (mapping) {
      var newMapping = {
        generated: {
          line: mapping.generatedLine,
          column: mapping.generatedColumn
        }
      };

      if (mapping.source != null) {
        newMapping.source = mapping.source;
        if (sourceRoot != null) {
          newMapping.source = util.relative(sourceRoot, newMapping.source);
        }

        newMapping.original = {
          line: mapping.originalLine,
          column: mapping.originalColumn
        };

        if (mapping.name != null) {
          newMapping.name = mapping.name;
        }
      }

      generator.addMapping(newMapping);
    });
    aSourceMapConsumer.sources.forEach(function (sourceFile) {
      var sourceRelative = sourceFile;
      if (sourceRoot !== null) {
        sourceRelative = util.relative(sourceRoot, sourceFile);
      }

      if (!generator._sources.has(sourceRelative)) {
        generator._sources.add(sourceRelative);
      }

      var content = aSourceMapConsumer.sourceContentFor(sourceFile);
      if (content != null) {
        generator.setSourceContent(sourceFile, content);
      }
    });
    return generator;
  };

/**
 * Add a single mapping from original source line and column to the generated
 * source's line and column for this source map being created. The mapping
 * object should have the following properties:
 *
 *   - generated: An object with the generated line and column positions.
 *   - original: An object with the original line and column positions.
 *   - source: The original source file (relative to the sourceRoot).
 *   - name: An optional original token name for this mapping.
 */
SourceMapGenerator.prototype.addMapping =
  function SourceMapGenerator_addMapping(aArgs) {
    var generated = util.getArg(aArgs, 'generated');
    var original = util.getArg(aArgs, 'original', null);
    var source = util.getArg(aArgs, 'source', null);
    var name = util.getArg(aArgs, 'name', null);

    if (!this._skipValidation) {
      this._validateMapping(generated, original, source, name);
    }

    if (source != null) {
      source = String(source);
      if (!this._sources.has(source)) {
        this._sources.add(source);
      }
    }

    if (name != null) {
      name = String(name);
      if (!this._names.has(name)) {
        this._names.add(name);
      }
    }

    this._mappings.add({
      generatedLine: generated.line,
      generatedColumn: generated.column,
      originalLine: original != null && original.line,
      originalColumn: original != null && original.column,
      source: source,
      name: name
    });
  };

/**
 * Set the source content for a source file.
 */
SourceMapGenerator.prototype.setSourceContent =
  function SourceMapGenerator_setSourceContent(aSourceFile, aSourceContent) {
    var source = aSourceFile;
    if (this._sourceRoot != null) {
      source = util.relative(this._sourceRoot, source);
    }

    if (aSourceContent != null) {
      // Add the source content to the _sourcesContents map.
      // Create a new _sourcesContents map if the property is null.
      if (!this._sourcesContents) {
        this._sourcesContents = Object.create(null);
      }
      this._sourcesContents[util.toSetString(source)] = aSourceContent;
    } else if (this._sourcesContents) {
      // Remove the source file from the _sourcesContents map.
      // If the _sourcesContents map is empty, set the property to null.
      delete this._sourcesContents[util.toSetString(source)];
      if (Object.keys(this._sourcesContents).length === 0) {
        this._sourcesContents = null;
      }
    }
  };

/**
 * Applies the mappings of a sub-source-map for a specific source file to the
 * source map being generated. Each mapping to the supplied source file is
 * rewritten using the supplied source map. Note: The resolution for the
 * resulting mappings is the minimium of this map and the supplied map.
 *
 * @param aSourceMapConsumer The source map to be applied.
 * @param aSourceFile Optional. The filename of the source file.
 *        If omitted, SourceMapConsumer's file property will be used.
 * @param aSourceMapPath Optional. The dirname of the path to the source map
 *        to be applied. If relative, it is relative to the SourceMapConsumer.
 *        This parameter is needed when the two source maps aren't in the same
 *        directory, and the source map to be applied contains relative source
 *        paths. If so, those relative source paths need to be rewritten
 *        relative to the SourceMapGenerator.
 */
SourceMapGenerator.prototype.applySourceMap =
  function SourceMapGenerator_applySourceMap(aSourceMapConsumer, aSourceFile, aSourceMapPath) {
    var sourceFile = aSourceFile;
    // If aSourceFile is omitted, we will use the file property of the SourceMap
    if (aSourceFile == null) {
      if (aSourceMapConsumer.file == null) {
        throw new Error(
          'SourceMapGenerator.prototype.applySourceMap requires either an explicit source file, ' +
          'or the source map\'s "file" property. Both were omitted.'
        );
      }
      sourceFile = aSourceMapConsumer.file;
    }
    var sourceRoot = this._sourceRoot;
    // Make "sourceFile" relative if an absolute Url is passed.
    if (sourceRoot != null) {
      sourceFile = util.relative(sourceRoot, sourceFile);
    }
    // Applying the SourceMap can add and remove items from the sources and
    // the names array.
    var newSources = new ArraySet$1();
    var newNames = new ArraySet$1();

    // Find mappings for the "sourceFile"
    this._mappings.unsortedForEach(function (mapping) {
      if (mapping.source === sourceFile && mapping.originalLine != null) {
        // Check if it can be mapped by the source map, then update the mapping.
        var original = aSourceMapConsumer.originalPositionFor({
          line: mapping.originalLine,
          column: mapping.originalColumn
        });
        if (original.source != null) {
          // Copy mapping
          mapping.source = original.source;
          if (aSourceMapPath != null) {
            mapping.source = util.join(aSourceMapPath, mapping.source);
          }
          if (sourceRoot != null) {
            mapping.source = util.relative(sourceRoot, mapping.source);
          }
          mapping.originalLine = original.line;
          mapping.originalColumn = original.column;
          if (original.name != null) {
            mapping.name = original.name;
          }
        }
      }

      var source = mapping.source;
      if (source != null && !newSources.has(source)) {
        newSources.add(source);
      }

      var name = mapping.name;
      if (name != null && !newNames.has(name)) {
        newNames.add(name);
      }

    }, this);
    this._sources = newSources;
    this._names = newNames;

    // Copy sourcesContents of applied map.
    aSourceMapConsumer.sources.forEach(function (sourceFile) {
      var content = aSourceMapConsumer.sourceContentFor(sourceFile);
      if (content != null) {
        if (aSourceMapPath != null) {
          sourceFile = util.join(aSourceMapPath, sourceFile);
        }
        if (sourceRoot != null) {
          sourceFile = util.relative(sourceRoot, sourceFile);
        }
        this.setSourceContent(sourceFile, content);
      }
    }, this);
  };

/**
 * A mapping can have one of the three levels of data:
 *
 *   1. Just the generated position.
 *   2. The Generated position, original position, and original source.
 *   3. Generated and original position, original source, as well as a name
 *      token.
 *
 * To maintain consistency, we validate that any new mapping being added falls
 * in to one of these categories.
 */
SourceMapGenerator.prototype._validateMapping =
  function SourceMapGenerator_validateMapping(aGenerated, aOriginal, aSource,
                                              aName) {
    // When aOriginal is truthy but has empty values for .line and .column,
    // it is most likely a programmer error. In this case we throw a very
    // specific error message to try to guide them the right way.
    // For example: https://github.com/Polymer/polymer-bundler/pull/519
    if (aOriginal && typeof aOriginal.line !== 'number' && typeof aOriginal.column !== 'number') {
        throw new Error(
            'original.line and original.column are not numbers -- you probably meant to omit ' +
            'the original mapping entirely and only map the generated position. If so, pass ' +
            'null for the original mapping instead of an object with empty or null values.'
        );
    }

    if (aGenerated && 'line' in aGenerated && 'column' in aGenerated
        && aGenerated.line > 0 && aGenerated.column >= 0
        && !aOriginal && !aSource && !aName) {
      // Case 1.
      return;
    }
    else if (aGenerated && 'line' in aGenerated && 'column' in aGenerated
             && aOriginal && 'line' in aOriginal && 'column' in aOriginal
             && aGenerated.line > 0 && aGenerated.column >= 0
             && aOriginal.line > 0 && aOriginal.column >= 0
             && aSource) {
      // Cases 2 and 3.
      return;
    }
    else {
      throw new Error('Invalid mapping: ' + JSON.stringify({
        generated: aGenerated,
        source: aSource,
        original: aOriginal,
        name: aName
      }));
    }
  };

/**
 * Serialize the accumulated mappings in to the stream of base 64 VLQs
 * specified by the source map format.
 */
SourceMapGenerator.prototype._serializeMappings =
  function SourceMapGenerator_serializeMappings() {
    var previousGeneratedColumn = 0;
    var previousGeneratedLine = 1;
    var previousOriginalColumn = 0;
    var previousOriginalLine = 0;
    var previousName = 0;
    var previousSource = 0;
    var result = '';
    var next;
    var mapping;
    var nameIdx;
    var sourceIdx;

    var mappings = this._mappings.toArray();
    for (var i = 0, len = mappings.length; i < len; i++) {
      mapping = mappings[i];
      next = '';

      if (mapping.generatedLine !== previousGeneratedLine) {
        previousGeneratedColumn = 0;
        while (mapping.generatedLine !== previousGeneratedLine) {
          next += ';';
          previousGeneratedLine++;
        }
      }
      else {
        if (i > 0) {
          if (!util.compareByGeneratedPositionsInflated(mapping, mappings[i - 1])) {
            continue;
          }
          next += ',';
        }
      }

      next += base64Vlq.encode(mapping.generatedColumn
                                 - previousGeneratedColumn);
      previousGeneratedColumn = mapping.generatedColumn;

      if (mapping.source != null) {
        sourceIdx = this._sources.indexOf(mapping.source);
        next += base64Vlq.encode(sourceIdx - previousSource);
        previousSource = sourceIdx;

        // lines are stored 0-based in SourceMap spec version 3
        next += base64Vlq.encode(mapping.originalLine - 1
                                   - previousOriginalLine);
        previousOriginalLine = mapping.originalLine - 1;

        next += base64Vlq.encode(mapping.originalColumn
                                   - previousOriginalColumn);
        previousOriginalColumn = mapping.originalColumn;

        if (mapping.name != null) {
          nameIdx = this._names.indexOf(mapping.name);
          next += base64Vlq.encode(nameIdx - previousName);
          previousName = nameIdx;
        }
      }

      result += next;
    }

    return result;
  };

SourceMapGenerator.prototype._generateSourcesContent =
  function SourceMapGenerator_generateSourcesContent(aSources, aSourceRoot) {
    return aSources.map(function (source) {
      if (!this._sourcesContents) {
        return null;
      }
      if (aSourceRoot != null) {
        source = util.relative(aSourceRoot, source);
      }
      var key = util.toSetString(source);
      return Object.prototype.hasOwnProperty.call(this._sourcesContents, key)
        ? this._sourcesContents[key]
        : null;
    }, this);
  };

/**
 * Externalize the source map.
 */
SourceMapGenerator.prototype.toJSON =
  function SourceMapGenerator_toJSON() {
    var map = {
      version: this._version,
      sources: this._sources.toArray(),
      names: this._names.toArray(),
      mappings: this._serializeMappings()
    };
    if (this._file != null) {
      map.file = this._file;
    }
    if (this._sourceRoot != null) {
      map.sourceRoot = this._sourceRoot;
    }
    if (this._sourcesContents) {
      map.sourcesContent = this._generateSourcesContent(map.sources, map.sourceRoot);
    }

    return map;
  };

/**
 * Render the source map being generated to a string.
 */
SourceMapGenerator.prototype.toString =
  function SourceMapGenerator_toString() {
    return JSON.stringify(this.toJSON());
  };

var SourceMapGenerator_1 = SourceMapGenerator;

var sourceMapGenerator = {
	SourceMapGenerator: SourceMapGenerator_1
};

var binarySearch = createCommonjsModule(function (module, exports) {
/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */

exports.GREATEST_LOWER_BOUND = 1;
exports.LEAST_UPPER_BOUND = 2;

/**
 * Recursive implementation of binary search.
 *
 * @param aLow Indices here and lower do not contain the needle.
 * @param aHigh Indices here and higher do not contain the needle.
 * @param aNeedle The element being searched for.
 * @param aHaystack The non-empty array being searched.
 * @param aCompare Function which takes two elements and returns -1, 0, or 1.
 * @param aBias Either 'binarySearch.GREATEST_LOWER_BOUND' or
 *     'binarySearch.LEAST_UPPER_BOUND'. Specifies whether to return the
 *     closest element that is smaller than or greater than the one we are
 *     searching for, respectively, if the exact element cannot be found.
 */
function recursiveSearch(aLow, aHigh, aNeedle, aHaystack, aCompare, aBias) {
  // This function terminates when one of the following is true:
  //
  //   1. We find the exact element we are looking for.
  //
  //   2. We did not find the exact element, but we can return the index of
  //      the next-closest element.
  //
  //   3. We did not find the exact element, and there is no next-closest
  //      element than the one we are searching for, so we return -1.
  var mid = Math.floor((aHigh - aLow) / 2) + aLow;
  var cmp = aCompare(aNeedle, aHaystack[mid], true);
  if (cmp === 0) {
    // Found the element we are looking for.
    return mid;
  }
  else if (cmp > 0) {
    // Our needle is greater than aHaystack[mid].
    if (aHigh - mid > 1) {
      // The element is in the upper half.
      return recursiveSearch(mid, aHigh, aNeedle, aHaystack, aCompare, aBias);
    }

    // The exact needle element was not found in this haystack. Determine if
    // we are in termination case (3) or (2) and return the appropriate thing.
    if (aBias == exports.LEAST_UPPER_BOUND) {
      return aHigh < aHaystack.length ? aHigh : -1;
    } else {
      return mid;
    }
  }
  else {
    // Our needle is less than aHaystack[mid].
    if (mid - aLow > 1) {
      // The element is in the lower half.
      return recursiveSearch(aLow, mid, aNeedle, aHaystack, aCompare, aBias);
    }

    // we are in termination case (3) or (2) and return the appropriate thing.
    if (aBias == exports.LEAST_UPPER_BOUND) {
      return mid;
    } else {
      return aLow < 0 ? -1 : aLow;
    }
  }
}

/**
 * This is an implementation of binary search which will always try and return
 * the index of the closest element if there is no exact hit. This is because
 * mappings between original and generated line/col pairs are single points,
 * and there is an implicit region between each of them, so a miss just means
 * that you aren't on the very start of a region.
 *
 * @param aNeedle The element you are looking for.
 * @param aHaystack The array that is being searched.
 * @param aCompare A function which takes the needle and an element in the
 *     array and returns -1, 0, or 1 depending on whether the needle is less
 *     than, equal to, or greater than the element, respectively.
 * @param aBias Either 'binarySearch.GREATEST_LOWER_BOUND' or
 *     'binarySearch.LEAST_UPPER_BOUND'. Specifies whether to return the
 *     closest element that is smaller than or greater than the one we are
 *     searching for, respectively, if the exact element cannot be found.
 *     Defaults to 'binarySearch.GREATEST_LOWER_BOUND'.
 */
exports.search = function search(aNeedle, aHaystack, aCompare, aBias) {
  if (aHaystack.length === 0) {
    return -1;
  }

  var index = recursiveSearch(-1, aHaystack.length, aNeedle, aHaystack,
                              aCompare, aBias || exports.GREATEST_LOWER_BOUND);
  if (index < 0) {
    return -1;
  }

  // We have found either the exact element, or the next-closest element than
  // the one we are searching for. However, there may be more than one such
  // element. Make sure we always return the smallest of these.
  while (index - 1 >= 0) {
    if (aCompare(aHaystack[index], aHaystack[index - 1], true) !== 0) {
      break;
    }
    --index;
  }

  return index;
};
});
var binarySearch_1 = binarySearch.GREATEST_LOWER_BOUND;
var binarySearch_2 = binarySearch.LEAST_UPPER_BOUND;
var binarySearch_3 = binarySearch.search;

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */

// It turns out that some (most?) JavaScript engines don't self-host
// `Array.prototype.sort`. This makes sense because C++ will likely remain
// faster than JS when doing raw CPU-intensive sorting. However, when using a
// custom comparator function, calling back and forth between the VM's C++ and
// JIT'd JS is rather slow *and* loses JIT type information, resulting in
// worse generated code for the comparator function than would be optimal. In
// fact, when sorting with a comparator, these costs outweigh the benefits of
// sorting in C++. By using our own JS-implemented Quick Sort (below), we get
// a ~3500ms mean speed-up in `bench/bench.html`.

/**
 * Swap the elements indexed by `x` and `y` in the array `ary`.
 *
 * @param {Array} ary
 *        The array.
 * @param {Number} x
 *        The index of the first item.
 * @param {Number} y
 *        The index of the second item.
 */
function swap(ary, x, y) {
  var temp = ary[x];
  ary[x] = ary[y];
  ary[y] = temp;
}

/**
 * Returns a random integer within the range `low .. high` inclusive.
 *
 * @param {Number} low
 *        The lower bound on the range.
 * @param {Number} high
 *        The upper bound on the range.
 */
function randomIntInRange(low, high) {
  return Math.round(low + (Math.random() * (high - low)));
}

/**
 * The Quick Sort algorithm.
 *
 * @param {Array} ary
 *        An array to sort.
 * @param {function} comparator
 *        Function to use to compare two items.
 * @param {Number} p
 *        Start index of the array
 * @param {Number} r
 *        End index of the array
 */
function doQuickSort(ary, comparator, p, r) {
  // If our lower bound is less than our upper bound, we (1) partition the
  // array into two pieces and (2) recurse on each half. If it is not, this is
  // the empty array and our base case.

  if (p < r) {
    // (1) Partitioning.
    //
    // The partitioning chooses a pivot between `p` and `r` and moves all
    // elements that are less than or equal to the pivot to the before it, and
    // all the elements that are greater than it after it. The effect is that
    // once partition is done, the pivot is in the exact place it will be when
    // the array is put in sorted order, and it will not need to be moved
    // again. This runs in O(n) time.

    // Always choose a random pivot so that an input array which is reverse
    // sorted does not cause O(n^2) running time.
    var pivotIndex = randomIntInRange(p, r);
    var i = p - 1;

    swap(ary, pivotIndex, r);
    var pivot = ary[r];

    // Immediately after `j` is incremented in this loop, the following hold
    // true:
    //
    //   * Every element in `ary[p .. i]` is less than or equal to the pivot.
    //
    //   * Every element in `ary[i+1 .. j-1]` is greater than the pivot.
    for (var j = p; j < r; j++) {
      if (comparator(ary[j], pivot) <= 0) {
        i += 1;
        swap(ary, i, j);
      }
    }

    swap(ary, i + 1, j);
    var q = i + 1;

    // (2) Recurse on each half.

    doQuickSort(ary, comparator, p, q - 1);
    doQuickSort(ary, comparator, q + 1, r);
  }
}

/**
 * Sort the given array in-place with the given comparator function.
 *
 * @param {Array} ary
 *        An array to sort.
 * @param {function} comparator
 *        Function to use to compare two items.
 */
var quickSort_1 = function (ary, comparator) {
  doQuickSort(ary, comparator, 0, ary.length - 1);
};

var quickSort = {
	quickSort: quickSort_1
};

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */



var ArraySet$2 = arraySet.ArraySet;

var quickSort$1 = quickSort.quickSort;

function SourceMapConsumer(aSourceMap, aSourceMapURL) {
  var sourceMap = aSourceMap;
  if (typeof aSourceMap === 'string') {
    sourceMap = util.parseSourceMapInput(aSourceMap);
  }

  return sourceMap.sections != null
    ? new IndexedSourceMapConsumer(sourceMap, aSourceMapURL)
    : new BasicSourceMapConsumer(sourceMap, aSourceMapURL);
}

SourceMapConsumer.fromSourceMap = function(aSourceMap, aSourceMapURL) {
  return BasicSourceMapConsumer.fromSourceMap(aSourceMap, aSourceMapURL);
};

/**
 * The version of the source mapping spec that we are consuming.
 */
SourceMapConsumer.prototype._version = 3;

// `__generatedMappings` and `__originalMappings` are arrays that hold the
// parsed mapping coordinates from the source map's "mappings" attribute. They
// are lazily instantiated, accessed via the `_generatedMappings` and
// `_originalMappings` getters respectively, and we only parse the mappings
// and create these arrays once queried for a source location. We jump through
// these hoops because there can be many thousands of mappings, and parsing
// them is expensive, so we only want to do it if we must.
//
// Each object in the arrays is of the form:
//
//     {
//       generatedLine: The line number in the generated code,
//       generatedColumn: The column number in the generated code,
//       source: The path to the original source file that generated this
//               chunk of code,
//       originalLine: The line number in the original source that
//                     corresponds to this chunk of generated code,
//       originalColumn: The column number in the original source that
//                       corresponds to this chunk of generated code,
//       name: The name of the original symbol which generated this chunk of
//             code.
//     }
//
// All properties except for `generatedLine` and `generatedColumn` can be
// `null`.
//
// `_generatedMappings` is ordered by the generated positions.
//
// `_originalMappings` is ordered by the original positions.

SourceMapConsumer.prototype.__generatedMappings = null;
Object.defineProperty(SourceMapConsumer.prototype, '_generatedMappings', {
  configurable: true,
  enumerable: true,
  get: function () {
    if (!this.__generatedMappings) {
      this._parseMappings(this._mappings, this.sourceRoot);
    }

    return this.__generatedMappings;
  }
});

SourceMapConsumer.prototype.__originalMappings = null;
Object.defineProperty(SourceMapConsumer.prototype, '_originalMappings', {
  configurable: true,
  enumerable: true,
  get: function () {
    if (!this.__originalMappings) {
      this._parseMappings(this._mappings, this.sourceRoot);
    }

    return this.__originalMappings;
  }
});

SourceMapConsumer.prototype._charIsMappingSeparator =
  function SourceMapConsumer_charIsMappingSeparator(aStr, index) {
    var c = aStr.charAt(index);
    return c === ";" || c === ",";
  };

/**
 * Parse the mappings in a string in to a data structure which we can easily
 * query (the ordered arrays in the `this.__generatedMappings` and
 * `this.__originalMappings` properties).
 */
SourceMapConsumer.prototype._parseMappings =
  function SourceMapConsumer_parseMappings(aStr, aSourceRoot) {
    throw new Error("Subclasses must implement _parseMappings");
  };

SourceMapConsumer.GENERATED_ORDER = 1;
SourceMapConsumer.ORIGINAL_ORDER = 2;

SourceMapConsumer.GREATEST_LOWER_BOUND = 1;
SourceMapConsumer.LEAST_UPPER_BOUND = 2;

/**
 * Iterate over each mapping between an original source/line/column and a
 * generated line/column in this source map.
 *
 * @param Function aCallback
 *        The function that is called with each mapping.
 * @param Object aContext
 *        Optional. If specified, this object will be the value of `this` every
 *        time that `aCallback` is called.
 * @param aOrder
 *        Either `SourceMapConsumer.GENERATED_ORDER` or
 *        `SourceMapConsumer.ORIGINAL_ORDER`. Specifies whether you want to
 *        iterate over the mappings sorted by the generated file's line/column
 *        order or the original's source/line/column order, respectively. Defaults to
 *        `SourceMapConsumer.GENERATED_ORDER`.
 */
SourceMapConsumer.prototype.eachMapping =
  function SourceMapConsumer_eachMapping(aCallback, aContext, aOrder) {
    var context = aContext || null;
    var order = aOrder || SourceMapConsumer.GENERATED_ORDER;

    var mappings;
    switch (order) {
    case SourceMapConsumer.GENERATED_ORDER:
      mappings = this._generatedMappings;
      break;
    case SourceMapConsumer.ORIGINAL_ORDER:
      mappings = this._originalMappings;
      break;
    default:
      throw new Error("Unknown order of iteration.");
    }

    var sourceRoot = this.sourceRoot;
    mappings.map(function (mapping) {
      var source = mapping.source === null ? null : this._sources.at(mapping.source);
      source = util.computeSourceURL(sourceRoot, source, this._sourceMapURL);
      return {
        source: source,
        generatedLine: mapping.generatedLine,
        generatedColumn: mapping.generatedColumn,
        originalLine: mapping.originalLine,
        originalColumn: mapping.originalColumn,
        name: mapping.name === null ? null : this._names.at(mapping.name)
      };
    }, this).forEach(aCallback, context);
  };

/**
 * Returns all generated line and column information for the original source,
 * line, and column provided. If no column is provided, returns all mappings
 * corresponding to a either the line we are searching for or the next
 * closest line that has any mappings. Otherwise, returns all mappings
 * corresponding to the given line and either the column we are searching for
 * or the next closest column that has any offsets.
 *
 * The only argument is an object with the following properties:
 *
 *   - source: The filename of the original source.
 *   - line: The line number in the original source.  The line number is 1-based.
 *   - column: Optional. the column number in the original source.
 *    The column number is 0-based.
 *
 * and an array of objects is returned, each with the following properties:
 *
 *   - line: The line number in the generated source, or null.  The
 *    line number is 1-based.
 *   - column: The column number in the generated source, or null.
 *    The column number is 0-based.
 */
SourceMapConsumer.prototype.allGeneratedPositionsFor =
  function SourceMapConsumer_allGeneratedPositionsFor(aArgs) {
    var line = util.getArg(aArgs, 'line');

    // When there is no exact match, BasicSourceMapConsumer.prototype._findMapping
    // returns the index of the closest mapping less than the needle. By
    // setting needle.originalColumn to 0, we thus find the last mapping for
    // the given line, provided such a mapping exists.
    var needle = {
      source: util.getArg(aArgs, 'source'),
      originalLine: line,
      originalColumn: util.getArg(aArgs, 'column', 0)
    };

    needle.source = this._findSourceIndex(needle.source);
    if (needle.source < 0) {
      return [];
    }

    var mappings = [];

    var index = this._findMapping(needle,
                                  this._originalMappings,
                                  "originalLine",
                                  "originalColumn",
                                  util.compareByOriginalPositions,
                                  binarySearch.LEAST_UPPER_BOUND);
    if (index >= 0) {
      var mapping = this._originalMappings[index];

      if (aArgs.column === undefined) {
        var originalLine = mapping.originalLine;

        // Iterate until either we run out of mappings, or we run into
        // a mapping for a different line than the one we found. Since
        // mappings are sorted, this is guaranteed to find all mappings for
        // the line we found.
        while (mapping && mapping.originalLine === originalLine) {
          mappings.push({
            line: util.getArg(mapping, 'generatedLine', null),
            column: util.getArg(mapping, 'generatedColumn', null),
            lastColumn: util.getArg(mapping, 'lastGeneratedColumn', null)
          });

          mapping = this._originalMappings[++index];
        }
      } else {
        var originalColumn = mapping.originalColumn;

        // Iterate until either we run out of mappings, or we run into
        // a mapping for a different line than the one we were searching for.
        // Since mappings are sorted, this is guaranteed to find all mappings for
        // the line we are searching for.
        while (mapping &&
               mapping.originalLine === line &&
               mapping.originalColumn == originalColumn) {
          mappings.push({
            line: util.getArg(mapping, 'generatedLine', null),
            column: util.getArg(mapping, 'generatedColumn', null),
            lastColumn: util.getArg(mapping, 'lastGeneratedColumn', null)
          });

          mapping = this._originalMappings[++index];
        }
      }
    }

    return mappings;
  };

var SourceMapConsumer_1 = SourceMapConsumer;

/**
 * A BasicSourceMapConsumer instance represents a parsed source map which we can
 * query for information about the original file positions by giving it a file
 * position in the generated source.
 *
 * The first parameter is the raw source map (either as a JSON string, or
 * already parsed to an object). According to the spec, source maps have the
 * following attributes:
 *
 *   - version: Which version of the source map spec this map is following.
 *   - sources: An array of URLs to the original source files.
 *   - names: An array of identifiers which can be referrenced by individual mappings.
 *   - sourceRoot: Optional. The URL root from which all sources are relative.
 *   - sourcesContent: Optional. An array of contents of the original source files.
 *   - mappings: A string of base64 VLQs which contain the actual mappings.
 *   - file: Optional. The generated file this source map is associated with.
 *
 * Here is an example source map, taken from the source map spec[0]:
 *
 *     {
 *       version : 3,
 *       file: "out.js",
 *       sourceRoot : "",
 *       sources: ["foo.js", "bar.js"],
 *       names: ["src", "maps", "are", "fun"],
 *       mappings: "AA,AB;;ABCDE;"
 *     }
 *
 * The second parameter, if given, is a string whose value is the URL
 * at which the source map was found.  This URL is used to compute the
 * sources array.
 *
 * [0]: https://docs.google.com/document/d/1U1RGAehQwRypUTovF1KRlpiOFze0b-_2gc6fAH0KY0k/edit?pli=1#
 */
function BasicSourceMapConsumer(aSourceMap, aSourceMapURL) {
  var sourceMap = aSourceMap;
  if (typeof aSourceMap === 'string') {
    sourceMap = util.parseSourceMapInput(aSourceMap);
  }

  var version = util.getArg(sourceMap, 'version');
  var sources = util.getArg(sourceMap, 'sources');
  // Sass 3.3 leaves out the 'names' array, so we deviate from the spec (which
  // requires the array) to play nice here.
  var names = util.getArg(sourceMap, 'names', []);
  var sourceRoot = util.getArg(sourceMap, 'sourceRoot', null);
  var sourcesContent = util.getArg(sourceMap, 'sourcesContent', null);
  var mappings = util.getArg(sourceMap, 'mappings');
  var file = util.getArg(sourceMap, 'file', null);

  // Once again, Sass deviates from the spec and supplies the version as a
  // string rather than a number, so we use loose equality checking here.
  if (version != this._version) {
    throw new Error('Unsupported version: ' + version);
  }

  if (sourceRoot) {
    sourceRoot = util.normalize(sourceRoot);
  }

  sources = sources
    .map(String)
    // Some source maps produce relative source paths like "./foo.js" instead of
    // "foo.js".  Normalize these first so that future comparisons will succeed.
    // See bugzil.la/1090768.
    .map(util.normalize)
    // Always ensure that absolute sources are internally stored relative to
    // the source root, if the source root is absolute. Not doing this would
    // be particularly problematic when the source root is a prefix of the
    // source (valid, but why??). See github issue #199 and bugzil.la/1188982.
    .map(function (source) {
      return sourceRoot && util.isAbsolute(sourceRoot) && util.isAbsolute(source)
        ? util.relative(sourceRoot, source)
        : source;
    });

  // Pass `true` below to allow duplicate names and sources. While source maps
  // are intended to be compressed and deduplicated, the TypeScript compiler
  // sometimes generates source maps with duplicates in them. See Github issue
  // #72 and bugzil.la/889492.
  this._names = ArraySet$2.fromArray(names.map(String), true);
  this._sources = ArraySet$2.fromArray(sources, true);

  this._absoluteSources = this._sources.toArray().map(function (s) {
    return util.computeSourceURL(sourceRoot, s, aSourceMapURL);
  });

  this.sourceRoot = sourceRoot;
  this.sourcesContent = sourcesContent;
  this._mappings = mappings;
  this._sourceMapURL = aSourceMapURL;
  this.file = file;
}

BasicSourceMapConsumer.prototype = Object.create(SourceMapConsumer.prototype);
BasicSourceMapConsumer.prototype.consumer = SourceMapConsumer;

/**
 * Utility function to find the index of a source.  Returns -1 if not
 * found.
 */
BasicSourceMapConsumer.prototype._findSourceIndex = function(aSource) {
  var relativeSource = aSource;
  if (this.sourceRoot != null) {
    relativeSource = util.relative(this.sourceRoot, relativeSource);
  }

  if (this._sources.has(relativeSource)) {
    return this._sources.indexOf(relativeSource);
  }

  // Maybe aSource is an absolute URL as returned by |sources|.  In
  // this case we can't simply undo the transform.
  var i;
  for (i = 0; i < this._absoluteSources.length; ++i) {
    if (this._absoluteSources[i] == aSource) {
      return i;
    }
  }

  return -1;
};

/**
 * Create a BasicSourceMapConsumer from a SourceMapGenerator.
 *
 * @param SourceMapGenerator aSourceMap
 *        The source map that will be consumed.
 * @param String aSourceMapURL
 *        The URL at which the source map can be found (optional)
 * @returns BasicSourceMapConsumer
 */
BasicSourceMapConsumer.fromSourceMap =
  function SourceMapConsumer_fromSourceMap(aSourceMap, aSourceMapURL) {
    var smc = Object.create(BasicSourceMapConsumer.prototype);

    var names = smc._names = ArraySet$2.fromArray(aSourceMap._names.toArray(), true);
    var sources = smc._sources = ArraySet$2.fromArray(aSourceMap._sources.toArray(), true);
    smc.sourceRoot = aSourceMap._sourceRoot;
    smc.sourcesContent = aSourceMap._generateSourcesContent(smc._sources.toArray(),
                                                            smc.sourceRoot);
    smc.file = aSourceMap._file;
    smc._sourceMapURL = aSourceMapURL;
    smc._absoluteSources = smc._sources.toArray().map(function (s) {
      return util.computeSourceURL(smc.sourceRoot, s, aSourceMapURL);
    });

    // Because we are modifying the entries (by converting string sources and
    // names to indices into the sources and names ArraySets), we have to make
    // a copy of the entry or else bad things happen. Shared mutable state
    // strikes again! See github issue #191.

    var generatedMappings = aSourceMap._mappings.toArray().slice();
    var destGeneratedMappings = smc.__generatedMappings = [];
    var destOriginalMappings = smc.__originalMappings = [];

    for (var i = 0, length = generatedMappings.length; i < length; i++) {
      var srcMapping = generatedMappings[i];
      var destMapping = new Mapping;
      destMapping.generatedLine = srcMapping.generatedLine;
      destMapping.generatedColumn = srcMapping.generatedColumn;

      if (srcMapping.source) {
        destMapping.source = sources.indexOf(srcMapping.source);
        destMapping.originalLine = srcMapping.originalLine;
        destMapping.originalColumn = srcMapping.originalColumn;

        if (srcMapping.name) {
          destMapping.name = names.indexOf(srcMapping.name);
        }

        destOriginalMappings.push(destMapping);
      }

      destGeneratedMappings.push(destMapping);
    }

    quickSort$1(smc.__originalMappings, util.compareByOriginalPositions);

    return smc;
  };

/**
 * The version of the source mapping spec that we are consuming.
 */
BasicSourceMapConsumer.prototype._version = 3;

/**
 * The list of original sources.
 */
Object.defineProperty(BasicSourceMapConsumer.prototype, 'sources', {
  get: function () {
    return this._absoluteSources.slice();
  }
});

/**
 * Provide the JIT with a nice shape / hidden class.
 */
function Mapping() {
  this.generatedLine = 0;
  this.generatedColumn = 0;
  this.source = null;
  this.originalLine = null;
  this.originalColumn = null;
  this.name = null;
}

/**
 * Parse the mappings in a string in to a data structure which we can easily
 * query (the ordered arrays in the `this.__generatedMappings` and
 * `this.__originalMappings` properties).
 */
BasicSourceMapConsumer.prototype._parseMappings =
  function SourceMapConsumer_parseMappings(aStr, aSourceRoot) {
    var generatedLine = 1;
    var previousGeneratedColumn = 0;
    var previousOriginalLine = 0;
    var previousOriginalColumn = 0;
    var previousSource = 0;
    var previousName = 0;
    var length = aStr.length;
    var index = 0;
    var cachedSegments = {};
    var temp = {};
    var originalMappings = [];
    var generatedMappings = [];
    var mapping, str, segment, end, value;

    while (index < length) {
      if (aStr.charAt(index) === ';') {
        generatedLine++;
        index++;
        previousGeneratedColumn = 0;
      }
      else if (aStr.charAt(index) === ',') {
        index++;
      }
      else {
        mapping = new Mapping();
        mapping.generatedLine = generatedLine;

        // Because each offset is encoded relative to the previous one,
        // many segments often have the same encoding. We can exploit this
        // fact by caching the parsed variable length fields of each segment,
        // allowing us to avoid a second parse if we encounter the same
        // segment again.
        for (end = index; end < length; end++) {
          if (this._charIsMappingSeparator(aStr, end)) {
            break;
          }
        }
        str = aStr.slice(index, end);

        segment = cachedSegments[str];
        if (segment) {
          index += str.length;
        } else {
          segment = [];
          while (index < end) {
            base64Vlq.decode(aStr, index, temp);
            value = temp.value;
            index = temp.rest;
            segment.push(value);
          }

          if (segment.length === 2) {
            throw new Error('Found a source, but no line and column');
          }

          if (segment.length === 3) {
            throw new Error('Found a source and line, but no column');
          }

          cachedSegments[str] = segment;
        }

        // Generated column.
        mapping.generatedColumn = previousGeneratedColumn + segment[0];
        previousGeneratedColumn = mapping.generatedColumn;

        if (segment.length > 1) {
          // Original source.
          mapping.source = previousSource + segment[1];
          previousSource += segment[1];

          // Original line.
          mapping.originalLine = previousOriginalLine + segment[2];
          previousOriginalLine = mapping.originalLine;
          // Lines are stored 0-based
          mapping.originalLine += 1;

          // Original column.
          mapping.originalColumn = previousOriginalColumn + segment[3];
          previousOriginalColumn = mapping.originalColumn;

          if (segment.length > 4) {
            // Original name.
            mapping.name = previousName + segment[4];
            previousName += segment[4];
          }
        }

        generatedMappings.push(mapping);
        if (typeof mapping.originalLine === 'number') {
          originalMappings.push(mapping);
        }
      }
    }

    quickSort$1(generatedMappings, util.compareByGeneratedPositionsDeflated);
    this.__generatedMappings = generatedMappings;

    quickSort$1(originalMappings, util.compareByOriginalPositions);
    this.__originalMappings = originalMappings;
  };

/**
 * Find the mapping that best matches the hypothetical "needle" mapping that
 * we are searching for in the given "haystack" of mappings.
 */
BasicSourceMapConsumer.prototype._findMapping =
  function SourceMapConsumer_findMapping(aNeedle, aMappings, aLineName,
                                         aColumnName, aComparator, aBias) {
    // To return the position we are searching for, we must first find the
    // mapping for the given position and then return the opposite position it
    // points to. Because the mappings are sorted, we can use binary search to
    // find the best mapping.

    if (aNeedle[aLineName] <= 0) {
      throw new TypeError('Line must be greater than or equal to 1, got '
                          + aNeedle[aLineName]);
    }
    if (aNeedle[aColumnName] < 0) {
      throw new TypeError('Column must be greater than or equal to 0, got '
                          + aNeedle[aColumnName]);
    }

    return binarySearch.search(aNeedle, aMappings, aComparator, aBias);
  };

/**
 * Compute the last column for each generated mapping. The last column is
 * inclusive.
 */
BasicSourceMapConsumer.prototype.computeColumnSpans =
  function SourceMapConsumer_computeColumnSpans() {
    for (var index = 0; index < this._generatedMappings.length; ++index) {
      var mapping = this._generatedMappings[index];

      // Mappings do not contain a field for the last generated columnt. We
      // can come up with an optimistic estimate, however, by assuming that
      // mappings are contiguous (i.e. given two consecutive mappings, the
      // first mapping ends where the second one starts).
      if (index + 1 < this._generatedMappings.length) {
        var nextMapping = this._generatedMappings[index + 1];

        if (mapping.generatedLine === nextMapping.generatedLine) {
          mapping.lastGeneratedColumn = nextMapping.generatedColumn - 1;
          continue;
        }
      }

      // The last mapping for each line spans the entire line.
      mapping.lastGeneratedColumn = Infinity;
    }
  };

/**
 * Returns the original source, line, and column information for the generated
 * source's line and column positions provided. The only argument is an object
 * with the following properties:
 *
 *   - line: The line number in the generated source.  The line number
 *     is 1-based.
 *   - column: The column number in the generated source.  The column
 *     number is 0-based.
 *   - bias: Either 'SourceMapConsumer.GREATEST_LOWER_BOUND' or
 *     'SourceMapConsumer.LEAST_UPPER_BOUND'. Specifies whether to return the
 *     closest element that is smaller than or greater than the one we are
 *     searching for, respectively, if the exact element cannot be found.
 *     Defaults to 'SourceMapConsumer.GREATEST_LOWER_BOUND'.
 *
 * and an object is returned with the following properties:
 *
 *   - source: The original source file, or null.
 *   - line: The line number in the original source, or null.  The
 *     line number is 1-based.
 *   - column: The column number in the original source, or null.  The
 *     column number is 0-based.
 *   - name: The original identifier, or null.
 */
BasicSourceMapConsumer.prototype.originalPositionFor =
  function SourceMapConsumer_originalPositionFor(aArgs) {
    var needle = {
      generatedLine: util.getArg(aArgs, 'line'),
      generatedColumn: util.getArg(aArgs, 'column')
    };

    var index = this._findMapping(
      needle,
      this._generatedMappings,
      "generatedLine",
      "generatedColumn",
      util.compareByGeneratedPositionsDeflated,
      util.getArg(aArgs, 'bias', SourceMapConsumer.GREATEST_LOWER_BOUND)
    );

    if (index >= 0) {
      var mapping = this._generatedMappings[index];

      if (mapping.generatedLine === needle.generatedLine) {
        var source = util.getArg(mapping, 'source', null);
        if (source !== null) {
          source = this._sources.at(source);
          source = util.computeSourceURL(this.sourceRoot, source, this._sourceMapURL);
        }
        var name = util.getArg(mapping, 'name', null);
        if (name !== null) {
          name = this._names.at(name);
        }
        return {
          source: source,
          line: util.getArg(mapping, 'originalLine', null),
          column: util.getArg(mapping, 'originalColumn', null),
          name: name
        };
      }
    }

    return {
      source: null,
      line: null,
      column: null,
      name: null
    };
  };

/**
 * Return true if we have the source content for every source in the source
 * map, false otherwise.
 */
BasicSourceMapConsumer.prototype.hasContentsOfAllSources =
  function BasicSourceMapConsumer_hasContentsOfAllSources() {
    if (!this.sourcesContent) {
      return false;
    }
    return this.sourcesContent.length >= this._sources.size() &&
      !this.sourcesContent.some(function (sc) { return sc == null; });
  };

/**
 * Returns the original source content. The only argument is the url of the
 * original source file. Returns null if no original source content is
 * available.
 */
BasicSourceMapConsumer.prototype.sourceContentFor =
  function SourceMapConsumer_sourceContentFor(aSource, nullOnMissing) {
    if (!this.sourcesContent) {
      return null;
    }

    var index = this._findSourceIndex(aSource);
    if (index >= 0) {
      return this.sourcesContent[index];
    }

    var relativeSource = aSource;
    if (this.sourceRoot != null) {
      relativeSource = util.relative(this.sourceRoot, relativeSource);
    }

    var url;
    if (this.sourceRoot != null
        && (url = util.urlParse(this.sourceRoot))) {
      // XXX: file:// URIs and absolute paths lead to unexpected behavior for
      // many users. We can help them out when they expect file:// URIs to
      // behave like it would if they were running a local HTTP server. See
      // https://bugzilla.mozilla.org/show_bug.cgi?id=885597.
      var fileUriAbsPath = relativeSource.replace(/^file:\/\//, "");
      if (url.scheme == "file"
          && this._sources.has(fileUriAbsPath)) {
        return this.sourcesContent[this._sources.indexOf(fileUriAbsPath)]
      }

      if ((!url.path || url.path == "/")
          && this._sources.has("/" + relativeSource)) {
        return this.sourcesContent[this._sources.indexOf("/" + relativeSource)];
      }
    }

    // This function is used recursively from
    // IndexedSourceMapConsumer.prototype.sourceContentFor. In that case, we
    // don't want to throw if we can't find the source - we just want to
    // return null, so we provide a flag to exit gracefully.
    if (nullOnMissing) {
      return null;
    }
    else {
      throw new Error('"' + relativeSource + '" is not in the SourceMap.');
    }
  };

/**
 * Returns the generated line and column information for the original source,
 * line, and column positions provided. The only argument is an object with
 * the following properties:
 *
 *   - source: The filename of the original source.
 *   - line: The line number in the original source.  The line number
 *     is 1-based.
 *   - column: The column number in the original source.  The column
 *     number is 0-based.
 *   - bias: Either 'SourceMapConsumer.GREATEST_LOWER_BOUND' or
 *     'SourceMapConsumer.LEAST_UPPER_BOUND'. Specifies whether to return the
 *     closest element that is smaller than or greater than the one we are
 *     searching for, respectively, if the exact element cannot be found.
 *     Defaults to 'SourceMapConsumer.GREATEST_LOWER_BOUND'.
 *
 * and an object is returned with the following properties:
 *
 *   - line: The line number in the generated source, or null.  The
 *     line number is 1-based.
 *   - column: The column number in the generated source, or null.
 *     The column number is 0-based.
 */
BasicSourceMapConsumer.prototype.generatedPositionFor =
  function SourceMapConsumer_generatedPositionFor(aArgs) {
    var source = util.getArg(aArgs, 'source');
    source = this._findSourceIndex(source);
    if (source < 0) {
      return {
        line: null,
        column: null,
        lastColumn: null
      };
    }

    var needle = {
      source: source,
      originalLine: util.getArg(aArgs, 'line'),
      originalColumn: util.getArg(aArgs, 'column')
    };

    var index = this._findMapping(
      needle,
      this._originalMappings,
      "originalLine",
      "originalColumn",
      util.compareByOriginalPositions,
      util.getArg(aArgs, 'bias', SourceMapConsumer.GREATEST_LOWER_BOUND)
    );

    if (index >= 0) {
      var mapping = this._originalMappings[index];

      if (mapping.source === needle.source) {
        return {
          line: util.getArg(mapping, 'generatedLine', null),
          column: util.getArg(mapping, 'generatedColumn', null),
          lastColumn: util.getArg(mapping, 'lastGeneratedColumn', null)
        };
      }
    }

    return {
      line: null,
      column: null,
      lastColumn: null
    };
  };

var BasicSourceMapConsumer_1 = BasicSourceMapConsumer;

/**
 * An IndexedSourceMapConsumer instance represents a parsed source map which
 * we can query for information. It differs from BasicSourceMapConsumer in
 * that it takes "indexed" source maps (i.e. ones with a "sections" field) as
 * input.
 *
 * The first parameter is a raw source map (either as a JSON string, or already
 * parsed to an object). According to the spec for indexed source maps, they
 * have the following attributes:
 *
 *   - version: Which version of the source map spec this map is following.
 *   - file: Optional. The generated file this source map is associated with.
 *   - sections: A list of section definitions.
 *
 * Each value under the "sections" field has two fields:
 *   - offset: The offset into the original specified at which this section
 *       begins to apply, defined as an object with a "line" and "column"
 *       field.
 *   - map: A source map definition. This source map could also be indexed,
 *       but doesn't have to be.
 *
 * Instead of the "map" field, it's also possible to have a "url" field
 * specifying a URL to retrieve a source map from, but that's currently
 * unsupported.
 *
 * Here's an example source map, taken from the source map spec[0], but
 * modified to omit a section which uses the "url" field.
 *
 *  {
 *    version : 3,
 *    file: "app.js",
 *    sections: [{
 *      offset: {line:100, column:10},
 *      map: {
 *        version : 3,
 *        file: "section.js",
 *        sources: ["foo.js", "bar.js"],
 *        names: ["src", "maps", "are", "fun"],
 *        mappings: "AAAA,E;;ABCDE;"
 *      }
 *    }],
 *  }
 *
 * The second parameter, if given, is a string whose value is the URL
 * at which the source map was found.  This URL is used to compute the
 * sources array.
 *
 * [0]: https://docs.google.com/document/d/1U1RGAehQwRypUTovF1KRlpiOFze0b-_2gc6fAH0KY0k/edit#heading=h.535es3xeprgt
 */
function IndexedSourceMapConsumer(aSourceMap, aSourceMapURL) {
  var sourceMap = aSourceMap;
  if (typeof aSourceMap === 'string') {
    sourceMap = util.parseSourceMapInput(aSourceMap);
  }

  var version = util.getArg(sourceMap, 'version');
  var sections = util.getArg(sourceMap, 'sections');

  if (version != this._version) {
    throw new Error('Unsupported version: ' + version);
  }

  this._sources = new ArraySet$2();
  this._names = new ArraySet$2();

  var lastOffset = {
    line: -1,
    column: 0
  };
  this._sections = sections.map(function (s) {
    if (s.url) {
      // The url field will require support for asynchronicity.
      // See https://github.com/mozilla/source-map/issues/16
      throw new Error('Support for url field in sections not implemented.');
    }
    var offset = util.getArg(s, 'offset');
    var offsetLine = util.getArg(offset, 'line');
    var offsetColumn = util.getArg(offset, 'column');

    if (offsetLine < lastOffset.line ||
        (offsetLine === lastOffset.line && offsetColumn < lastOffset.column)) {
      throw new Error('Section offsets must be ordered and non-overlapping.');
    }
    lastOffset = offset;

    return {
      generatedOffset: {
        // The offset fields are 0-based, but we use 1-based indices when
        // encoding/decoding from VLQ.
        generatedLine: offsetLine + 1,
        generatedColumn: offsetColumn + 1
      },
      consumer: new SourceMapConsumer(util.getArg(s, 'map'), aSourceMapURL)
    }
  });
}

IndexedSourceMapConsumer.prototype = Object.create(SourceMapConsumer.prototype);
IndexedSourceMapConsumer.prototype.constructor = SourceMapConsumer;

/**
 * The version of the source mapping spec that we are consuming.
 */
IndexedSourceMapConsumer.prototype._version = 3;

/**
 * The list of original sources.
 */
Object.defineProperty(IndexedSourceMapConsumer.prototype, 'sources', {
  get: function () {
    var sources = [];
    for (var i = 0; i < this._sections.length; i++) {
      for (var j = 0; j < this._sections[i].consumer.sources.length; j++) {
        sources.push(this._sections[i].consumer.sources[j]);
      }
    }
    return sources;
  }
});

/**
 * Returns the original source, line, and column information for the generated
 * source's line and column positions provided. The only argument is an object
 * with the following properties:
 *
 *   - line: The line number in the generated source.  The line number
 *     is 1-based.
 *   - column: The column number in the generated source.  The column
 *     number is 0-based.
 *
 * and an object is returned with the following properties:
 *
 *   - source: The original source file, or null.
 *   - line: The line number in the original source, or null.  The
 *     line number is 1-based.
 *   - column: The column number in the original source, or null.  The
 *     column number is 0-based.
 *   - name: The original identifier, or null.
 */
IndexedSourceMapConsumer.prototype.originalPositionFor =
  function IndexedSourceMapConsumer_originalPositionFor(aArgs) {
    var needle = {
      generatedLine: util.getArg(aArgs, 'line'),
      generatedColumn: util.getArg(aArgs, 'column')
    };

    // Find the section containing the generated position we're trying to map
    // to an original position.
    var sectionIndex = binarySearch.search(needle, this._sections,
      function(needle, section) {
        var cmp = needle.generatedLine - section.generatedOffset.generatedLine;
        if (cmp) {
          return cmp;
        }

        return (needle.generatedColumn -
                section.generatedOffset.generatedColumn);
      });
    var section = this._sections[sectionIndex];

    if (!section) {
      return {
        source: null,
        line: null,
        column: null,
        name: null
      };
    }

    return section.consumer.originalPositionFor({
      line: needle.generatedLine -
        (section.generatedOffset.generatedLine - 1),
      column: needle.generatedColumn -
        (section.generatedOffset.generatedLine === needle.generatedLine
         ? section.generatedOffset.generatedColumn - 1
         : 0),
      bias: aArgs.bias
    });
  };

/**
 * Return true if we have the source content for every source in the source
 * map, false otherwise.
 */
IndexedSourceMapConsumer.prototype.hasContentsOfAllSources =
  function IndexedSourceMapConsumer_hasContentsOfAllSources() {
    return this._sections.every(function (s) {
      return s.consumer.hasContentsOfAllSources();
    });
  };

/**
 * Returns the original source content. The only argument is the url of the
 * original source file. Returns null if no original source content is
 * available.
 */
IndexedSourceMapConsumer.prototype.sourceContentFor =
  function IndexedSourceMapConsumer_sourceContentFor(aSource, nullOnMissing) {
    for (var i = 0; i < this._sections.length; i++) {
      var section = this._sections[i];

      var content = section.consumer.sourceContentFor(aSource, true);
      if (content) {
        return content;
      }
    }
    if (nullOnMissing) {
      return null;
    }
    else {
      throw new Error('"' + aSource + '" is not in the SourceMap.');
    }
  };

/**
 * Returns the generated line and column information for the original source,
 * line, and column positions provided. The only argument is an object with
 * the following properties:
 *
 *   - source: The filename of the original source.
 *   - line: The line number in the original source.  The line number
 *     is 1-based.
 *   - column: The column number in the original source.  The column
 *     number is 0-based.
 *
 * and an object is returned with the following properties:
 *
 *   - line: The line number in the generated source, or null.  The
 *     line number is 1-based. 
 *   - column: The column number in the generated source, or null.
 *     The column number is 0-based.
 */
IndexedSourceMapConsumer.prototype.generatedPositionFor =
  function IndexedSourceMapConsumer_generatedPositionFor(aArgs) {
    for (var i = 0; i < this._sections.length; i++) {
      var section = this._sections[i];

      // Only consider this section if the requested source is in the list of
      // sources of the consumer.
      if (section.consumer._findSourceIndex(util.getArg(aArgs, 'source')) === -1) {
        continue;
      }
      var generatedPosition = section.consumer.generatedPositionFor(aArgs);
      if (generatedPosition) {
        var ret = {
          line: generatedPosition.line +
            (section.generatedOffset.generatedLine - 1),
          column: generatedPosition.column +
            (section.generatedOffset.generatedLine === generatedPosition.line
             ? section.generatedOffset.generatedColumn - 1
             : 0)
        };
        return ret;
      }
    }

    return {
      line: null,
      column: null
    };
  };

/**
 * Parse the mappings in a string in to a data structure which we can easily
 * query (the ordered arrays in the `this.__generatedMappings` and
 * `this.__originalMappings` properties).
 */
IndexedSourceMapConsumer.prototype._parseMappings =
  function IndexedSourceMapConsumer_parseMappings(aStr, aSourceRoot) {
    this.__generatedMappings = [];
    this.__originalMappings = [];
    for (var i = 0; i < this._sections.length; i++) {
      var section = this._sections[i];
      var sectionMappings = section.consumer._generatedMappings;
      for (var j = 0; j < sectionMappings.length; j++) {
        var mapping = sectionMappings[j];

        var source = section.consumer._sources.at(mapping.source);
        source = util.computeSourceURL(section.consumer.sourceRoot, source, this._sourceMapURL);
        this._sources.add(source);
        source = this._sources.indexOf(source);

        var name = null;
        if (mapping.name) {
          name = section.consumer._names.at(mapping.name);
          this._names.add(name);
          name = this._names.indexOf(name);
        }

        // The mappings coming from the consumer for the section have
        // generated positions relative to the start of the section, so we
        // need to offset them to be relative to the start of the concatenated
        // generated file.
        var adjustedMapping = {
          source: source,
          generatedLine: mapping.generatedLine +
            (section.generatedOffset.generatedLine - 1),
          generatedColumn: mapping.generatedColumn +
            (section.generatedOffset.generatedLine === mapping.generatedLine
            ? section.generatedOffset.generatedColumn - 1
            : 0),
          originalLine: mapping.originalLine,
          originalColumn: mapping.originalColumn,
          name: name
        };

        this.__generatedMappings.push(adjustedMapping);
        if (typeof adjustedMapping.originalLine === 'number') {
          this.__originalMappings.push(adjustedMapping);
        }
      }
    }

    quickSort$1(this.__generatedMappings, util.compareByGeneratedPositionsDeflated);
    quickSort$1(this.__originalMappings, util.compareByOriginalPositions);
  };

var IndexedSourceMapConsumer_1 = IndexedSourceMapConsumer;

var sourceMapConsumer = {
	SourceMapConsumer: SourceMapConsumer_1,
	BasicSourceMapConsumer: BasicSourceMapConsumer_1,
	IndexedSourceMapConsumer: IndexedSourceMapConsumer_1
};

/* -*- Mode: js; js-indent-level: 2; -*- */
/*
 * Copyright 2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE or:
 * http://opensource.org/licenses/BSD-3-Clause
 */

var SourceMapGenerator$1 = sourceMapGenerator.SourceMapGenerator;


// Matches a Windows-style `\r\n` newline or a `\n` newline used by all other
// operating systems these days (capturing the result).
var REGEX_NEWLINE = /(\r?\n)/;

// Newline character code for charCodeAt() comparisons
var NEWLINE_CODE = 10;

// Private symbol for identifying `SourceNode`s when multiple versions of
// the source-map library are loaded. This MUST NOT CHANGE across
// versions!
var isSourceNode = "$$$isSourceNode$$$";

/**
 * SourceNodes provide a way to abstract over interpolating/concatenating
 * snippets of generated JavaScript source code while maintaining the line and
 * column information associated with the original source code.
 *
 * @param aLine The original line number.
 * @param aColumn The original column number.
 * @param aSource The original source's filename.
 * @param aChunks Optional. An array of strings which are snippets of
 *        generated JS, or other SourceNodes.
 * @param aName The original identifier.
 */
function SourceNode(aLine, aColumn, aSource, aChunks, aName) {
  this.children = [];
  this.sourceContents = {};
  this.line = aLine == null ? null : aLine;
  this.column = aColumn == null ? null : aColumn;
  this.source = aSource == null ? null : aSource;
  this.name = aName == null ? null : aName;
  this[isSourceNode] = true;
  if (aChunks != null) this.add(aChunks);
}

/**
 * Creates a SourceNode from generated code and a SourceMapConsumer.
 *
 * @param aGeneratedCode The generated code
 * @param aSourceMapConsumer The SourceMap for the generated code
 * @param aRelativePath Optional. The path that relative sources in the
 *        SourceMapConsumer should be relative to.
 */
SourceNode.fromStringWithSourceMap =
  function SourceNode_fromStringWithSourceMap(aGeneratedCode, aSourceMapConsumer, aRelativePath) {
    // The SourceNode we want to fill with the generated code
    // and the SourceMap
    var node = new SourceNode();

    // All even indices of this array are one line of the generated code,
    // while all odd indices are the newlines between two adjacent lines
    // (since `REGEX_NEWLINE` captures its match).
    // Processed fragments are accessed by calling `shiftNextLine`.
    var remainingLines = aGeneratedCode.split(REGEX_NEWLINE);
    var remainingLinesIndex = 0;
    var shiftNextLine = function() {
      var lineContents = getNextLine();
      // The last line of a file might not have a newline.
      var newLine = getNextLine() || "";
      return lineContents + newLine;

      function getNextLine() {
        return remainingLinesIndex < remainingLines.length ?
            remainingLines[remainingLinesIndex++] : undefined;
      }
    };

    // We need to remember the position of "remainingLines"
    var lastGeneratedLine = 1, lastGeneratedColumn = 0;

    // The generate SourceNodes we need a code range.
    // To extract it current and last mapping is used.
    // Here we store the last mapping.
    var lastMapping = null;

    aSourceMapConsumer.eachMapping(function (mapping) {
      if (lastMapping !== null) {
        // We add the code from "lastMapping" to "mapping":
        // First check if there is a new line in between.
        if (lastGeneratedLine < mapping.generatedLine) {
          // Associate first line with "lastMapping"
          addMappingWithCode(lastMapping, shiftNextLine());
          lastGeneratedLine++;
          lastGeneratedColumn = 0;
          // The remaining code is added without mapping
        } else {
          // There is no new line in between.
          // Associate the code between "lastGeneratedColumn" and
          // "mapping.generatedColumn" with "lastMapping"
          var nextLine = remainingLines[remainingLinesIndex] || '';
          var code = nextLine.substr(0, mapping.generatedColumn -
                                        lastGeneratedColumn);
          remainingLines[remainingLinesIndex] = nextLine.substr(mapping.generatedColumn -
                                              lastGeneratedColumn);
          lastGeneratedColumn = mapping.generatedColumn;
          addMappingWithCode(lastMapping, code);
          // No more remaining code, continue
          lastMapping = mapping;
          return;
        }
      }
      // We add the generated code until the first mapping
      // to the SourceNode without any mapping.
      // Each line is added as separate string.
      while (lastGeneratedLine < mapping.generatedLine) {
        node.add(shiftNextLine());
        lastGeneratedLine++;
      }
      if (lastGeneratedColumn < mapping.generatedColumn) {
        var nextLine = remainingLines[remainingLinesIndex] || '';
        node.add(nextLine.substr(0, mapping.generatedColumn));
        remainingLines[remainingLinesIndex] = nextLine.substr(mapping.generatedColumn);
        lastGeneratedColumn = mapping.generatedColumn;
      }
      lastMapping = mapping;
    }, this);
    // We have processed all mappings.
    if (remainingLinesIndex < remainingLines.length) {
      if (lastMapping) {
        // Associate the remaining code in the current line with "lastMapping"
        addMappingWithCode(lastMapping, shiftNextLine());
      }
      // and add the remaining lines without any mapping
      node.add(remainingLines.splice(remainingLinesIndex).join(""));
    }

    // Copy sourcesContent into SourceNode
    aSourceMapConsumer.sources.forEach(function (sourceFile) {
      var content = aSourceMapConsumer.sourceContentFor(sourceFile);
      if (content != null) {
        if (aRelativePath != null) {
          sourceFile = util.join(aRelativePath, sourceFile);
        }
        node.setSourceContent(sourceFile, content);
      }
    });

    return node;

    function addMappingWithCode(mapping, code) {
      if (mapping === null || mapping.source === undefined) {
        node.add(code);
      } else {
        var source = aRelativePath
          ? util.join(aRelativePath, mapping.source)
          : mapping.source;
        node.add(new SourceNode(mapping.originalLine,
                                mapping.originalColumn,
                                source,
                                code,
                                mapping.name));
      }
    }
  };

/**
 * Add a chunk of generated JS to this source node.
 *
 * @param aChunk A string snippet of generated JS code, another instance of
 *        SourceNode, or an array where each member is one of those things.
 */
SourceNode.prototype.add = function SourceNode_add(aChunk) {
  if (Array.isArray(aChunk)) {
    aChunk.forEach(function (chunk) {
      this.add(chunk);
    }, this);
  }
  else if (aChunk[isSourceNode] || typeof aChunk === "string") {
    if (aChunk) {
      this.children.push(aChunk);
    }
  }
  else {
    throw new TypeError(
      "Expected a SourceNode, string, or an array of SourceNodes and strings. Got " + aChunk
    );
  }
  return this;
};

/**
 * Add a chunk of generated JS to the beginning of this source node.
 *
 * @param aChunk A string snippet of generated JS code, another instance of
 *        SourceNode, or an array where each member is one of those things.
 */
SourceNode.prototype.prepend = function SourceNode_prepend(aChunk) {
  if (Array.isArray(aChunk)) {
    for (var i = aChunk.length-1; i >= 0; i--) {
      this.prepend(aChunk[i]);
    }
  }
  else if (aChunk[isSourceNode] || typeof aChunk === "string") {
    this.children.unshift(aChunk);
  }
  else {
    throw new TypeError(
      "Expected a SourceNode, string, or an array of SourceNodes and strings. Got " + aChunk
    );
  }
  return this;
};

/**
 * Walk over the tree of JS snippets in this node and its children. The
 * walking function is called once for each snippet of JS and is passed that
 * snippet and the its original associated source's line/column location.
 *
 * @param aFn The traversal function.
 */
SourceNode.prototype.walk = function SourceNode_walk(aFn) {
  var chunk;
  for (var i = 0, len = this.children.length; i < len; i++) {
    chunk = this.children[i];
    if (chunk[isSourceNode]) {
      chunk.walk(aFn);
    }
    else {
      if (chunk !== '') {
        aFn(chunk, { source: this.source,
                     line: this.line,
                     column: this.column,
                     name: this.name });
      }
    }
  }
};

/**
 * Like `String.prototype.join` except for SourceNodes. Inserts `aStr` between
 * each of `this.children`.
 *
 * @param aSep The separator.
 */
SourceNode.prototype.join = function SourceNode_join(aSep) {
  var newChildren;
  var i;
  var len = this.children.length;
  if (len > 0) {
    newChildren = [];
    for (i = 0; i < len-1; i++) {
      newChildren.push(this.children[i]);
      newChildren.push(aSep);
    }
    newChildren.push(this.children[i]);
    this.children = newChildren;
  }
  return this;
};

/**
 * Call String.prototype.replace on the very right-most source snippet. Useful
 * for trimming whitespace from the end of a source node, etc.
 *
 * @param aPattern The pattern to replace.
 * @param aReplacement The thing to replace the pattern with.
 */
SourceNode.prototype.replaceRight = function SourceNode_replaceRight(aPattern, aReplacement) {
  var lastChild = this.children[this.children.length - 1];
  if (lastChild[isSourceNode]) {
    lastChild.replaceRight(aPattern, aReplacement);
  }
  else if (typeof lastChild === 'string') {
    this.children[this.children.length - 1] = lastChild.replace(aPattern, aReplacement);
  }
  else {
    this.children.push(''.replace(aPattern, aReplacement));
  }
  return this;
};

/**
 * Set the source content for a source file. This will be added to the SourceMapGenerator
 * in the sourcesContent field.
 *
 * @param aSourceFile The filename of the source file
 * @param aSourceContent The content of the source file
 */
SourceNode.prototype.setSourceContent =
  function SourceNode_setSourceContent(aSourceFile, aSourceContent) {
    this.sourceContents[util.toSetString(aSourceFile)] = aSourceContent;
  };

/**
 * Walk over the tree of SourceNodes. The walking function is called for each
 * source file content and is passed the filename and source content.
 *
 * @param aFn The traversal function.
 */
SourceNode.prototype.walkSourceContents =
  function SourceNode_walkSourceContents(aFn) {
    for (var i = 0, len = this.children.length; i < len; i++) {
      if (this.children[i][isSourceNode]) {
        this.children[i].walkSourceContents(aFn);
      }
    }

    var sources = Object.keys(this.sourceContents);
    for (var i = 0, len = sources.length; i < len; i++) {
      aFn(util.fromSetString(sources[i]), this.sourceContents[sources[i]]);
    }
  };

/**
 * Return the string representation of this source node. Walks over the tree
 * and concatenates all the various snippets together to one string.
 */
SourceNode.prototype.toString = function SourceNode_toString() {
  var str = "";
  this.walk(function (chunk) {
    str += chunk;
  });
  return str;
};

/**
 * Returns the string representation of this source node along with a source
 * map.
 */
SourceNode.prototype.toStringWithSourceMap = function SourceNode_toStringWithSourceMap(aArgs) {
  var generated = {
    code: "",
    line: 1,
    column: 0
  };
  var map = new SourceMapGenerator$1(aArgs);
  var sourceMappingActive = false;
  var lastOriginalSource = null;
  var lastOriginalLine = null;
  var lastOriginalColumn = null;
  var lastOriginalName = null;
  this.walk(function (chunk, original) {
    generated.code += chunk;
    if (original.source !== null
        && original.line !== null
        && original.column !== null) {
      if(lastOriginalSource !== original.source
         || lastOriginalLine !== original.line
         || lastOriginalColumn !== original.column
         || lastOriginalName !== original.name) {
        map.addMapping({
          source: original.source,
          original: {
            line: original.line,
            column: original.column
          },
          generated: {
            line: generated.line,
            column: generated.column
          },
          name: original.name
        });
      }
      lastOriginalSource = original.source;
      lastOriginalLine = original.line;
      lastOriginalColumn = original.column;
      lastOriginalName = original.name;
      sourceMappingActive = true;
    } else if (sourceMappingActive) {
      map.addMapping({
        generated: {
          line: generated.line,
          column: generated.column
        }
      });
      lastOriginalSource = null;
      sourceMappingActive = false;
    }
    for (var idx = 0, length = chunk.length; idx < length; idx++) {
      if (chunk.charCodeAt(idx) === NEWLINE_CODE) {
        generated.line++;
        generated.column = 0;
        // Mappings end at eol
        if (idx + 1 === length) {
          lastOriginalSource = null;
          sourceMappingActive = false;
        } else if (sourceMappingActive) {
          map.addMapping({
            source: original.source,
            original: {
              line: original.line,
              column: original.column
            },
            generated: {
              line: generated.line,
              column: generated.column
            },
            name: original.name
          });
        }
      } else {
        generated.column++;
      }
    }
  });
  this.walkSourceContents(function (sourceFile, sourceContent) {
    map.setSourceContent(sourceFile, sourceContent);
  });

  return { code: generated.code, map: map };
};

var SourceNode_1 = SourceNode;

var sourceNode = {
	SourceNode: SourceNode_1
};

/*
 * Copyright 2009-2011 Mozilla Foundation and contributors
 * Licensed under the New BSD license. See LICENSE.txt or:
 * http://opensource.org/licenses/BSD-3-Clause
 */
var SourceMapGenerator$2 = sourceMapGenerator.SourceMapGenerator;
var SourceMapConsumer$1 = sourceMapConsumer.SourceMapConsumer;
var SourceNode$1 = sourceNode.SourceNode;

var sourceMap = {
	SourceMapGenerator: SourceMapGenerator$2,
	SourceMapConsumer: SourceMapConsumer$1,
	SourceNode: SourceNode$1
};

var toString = Object.prototype.toString;

var isModern = (
  typeof Buffer.alloc === 'function' &&
  typeof Buffer.allocUnsafe === 'function' &&
  typeof Buffer.from === 'function'
);

function isArrayBuffer (input) {
  return toString.call(input).slice(8, -1) === 'ArrayBuffer'
}

function fromArrayBuffer (obj, byteOffset, length) {
  byteOffset >>>= 0;

  var maxLength = obj.byteLength - byteOffset;

  if (maxLength < 0) {
    throw new RangeError("'offset' is out of bounds")
  }

  if (length === undefined) {
    length = maxLength;
  } else {
    length >>>= 0;

    if (length > maxLength) {
      throw new RangeError("'length' is out of bounds")
    }
  }

  return isModern
    ? Buffer.from(obj.slice(byteOffset, byteOffset + length))
    : new Buffer(new Uint8Array(obj.slice(byteOffset, byteOffset + length)))
}

function fromString (string, encoding) {
  if (typeof encoding !== 'string' || encoding === '') {
    encoding = 'utf8';
  }

  if (!Buffer.isEncoding(encoding)) {
    throw new TypeError('"encoding" must be a valid string encoding')
  }

  return isModern
    ? Buffer.from(string, encoding)
    : new Buffer(string, encoding)
}

function bufferFrom (value, encodingOrOffset, length) {
  if (typeof value === 'number') {
    throw new TypeError('"value" argument must not be a number')
  }

  if (isArrayBuffer(value)) {
    return fromArrayBuffer(value, encodingOrOffset, length)
  }

  if (typeof value === 'string') {
    return fromString(value, encodingOrOffset)
  }

  return isModern
    ? Buffer.from(value)
    : new Buffer(value)
}

var bufferFrom_1 = bufferFrom;

var sourceMapSupport = createCommonjsModule(function (module, exports) {
var SourceMapConsumer = sourceMap.SourceMapConsumer;
var path$1 = path;

var fs$1;
try {
  fs$1 = fs;
  if (!fs$1.existsSync || !fs$1.readFileSync) {
    // fs doesn't have all methods we need
    fs$1 = null;
  }
} catch (err) {
  /* nop */
}



/**
 * Requires a module which is protected against bundler minification.
 *
 * @param {NodeModule} mod
 * @param {string} request
 */
function dynamicRequire(mod, request) {
  return mod.require(request);
}

// Only install once if called multiple times
var errorFormatterInstalled = false;
var uncaughtShimInstalled = false;

// If true, the caches are reset before a stack trace formatting operation
var emptyCacheBetweenOperations = false;

// Supports {browser, node, auto}
var environment = "auto";

// Maps a file path to a string containing the file contents
var fileContentsCache = {};

// Maps a file path to a source map for that file
var sourceMapCache = {};

// Regex for detecting source maps
var reSourceMap = /^data:application\/json[^,]+base64,/;

// Priority list of retrieve handlers
var retrieveFileHandlers = [];
var retrieveMapHandlers = [];

function isInBrowser() {
  if (environment === "browser")
    return true;
  if (environment === "node")
    return false;
  return ((typeof window !== 'undefined') && (typeof XMLHttpRequest === 'function') && !(window.require && window.module && window.process && window.process.type === "renderer"));
}

function hasGlobalProcessEventEmitter() {
  return ((typeof process === 'object') && (process !== null) && (typeof process.on === 'function'));
}

function handlerExec(list) {
  return function(arg) {
    for (var i = 0; i < list.length; i++) {
      var ret = list[i](arg);
      if (ret) {
        return ret;
      }
    }
    return null;
  };
}

var retrieveFile = handlerExec(retrieveFileHandlers);

retrieveFileHandlers.push(function(path) {
  // Trim the path to make sure there is no extra whitespace.
  path = path.trim();
  if (/^file:/.test(path)) {
    // existsSync/readFileSync can't handle file protocol, but once stripped, it works
    path = path.replace(/file:\/\/\/(\w:)?/, function(protocol, drive) {
      return drive ?
        '' : // file:///C:/dir/file -> C:/dir/file
        '/'; // file:///root-dir/file -> /root-dir/file
    });
  }
  if (path in fileContentsCache) {
    return fileContentsCache[path];
  }

  var contents = '';
  try {
    if (!fs$1) {
      // Use SJAX if we are in the browser
      var xhr = new XMLHttpRequest();
      xhr.open('GET', path, /** async */ false);
      xhr.send(null);
      if (xhr.readyState === 4 && xhr.status === 200) {
        contents = xhr.responseText;
      }
    } else if (fs$1.existsSync(path)) {
      // Otherwise, use the filesystem
      contents = fs$1.readFileSync(path, 'utf8');
    }
  } catch (er) {
    /* ignore any errors */
  }

  return fileContentsCache[path] = contents;
});

// Support URLs relative to a directory, but be careful about a protocol prefix
// in case we are in the browser (i.e. directories may start with "http://" or "file:///")
function supportRelativeURL(file, url) {
  if (!file) return url;
  var dir = path$1.dirname(file);
  var match = /^\w+:\/\/[^\/]*/.exec(dir);
  var protocol = match ? match[0] : '';
  var startPath = dir.slice(protocol.length);
  if (protocol && /^\/\w\:/.test(startPath)) {
    // handle file:///C:/ paths
    protocol += '/';
    return protocol + path$1.resolve(dir.slice(protocol.length), url).replace(/\\/g, '/');
  }
  return protocol + path$1.resolve(dir.slice(protocol.length), url);
}

function retrieveSourceMapURL(source) {
  var fileData;

  if (isInBrowser()) {
     try {
       var xhr = new XMLHttpRequest();
       xhr.open('GET', source, false);
       xhr.send(null);
       fileData = xhr.readyState === 4 ? xhr.responseText : null;

       // Support providing a sourceMappingURL via the SourceMap header
       var sourceMapHeader = xhr.getResponseHeader("SourceMap") ||
                             xhr.getResponseHeader("X-SourceMap");
       if (sourceMapHeader) {
         return sourceMapHeader;
       }
     } catch (e) {
     }
  }

  // Get the URL of the source map
  fileData = retrieveFile(source);
  var re = /(?:\/\/[@#][\s]*sourceMappingURL=([^\s'"]+)[\s]*$)|(?:\/\*[@#][\s]*sourceMappingURL=([^\s*'"]+)[\s]*(?:\*\/)[\s]*$)/mg;
  // Keep executing the search to find the *last* sourceMappingURL to avoid
  // picking up sourceMappingURLs from comments, strings, etc.
  var lastMatch, match;
  while (match = re.exec(fileData)) lastMatch = match;
  if (!lastMatch) return null;
  return lastMatch[1];
}
// Can be overridden by the retrieveSourceMap option to install. Takes a
// generated source filename; returns a {map, optional url} object, or null if
// there is no source map.  The map field may be either a string or the parsed
// JSON object (ie, it must be a valid argument to the SourceMapConsumer
// constructor).
var retrieveSourceMap = handlerExec(retrieveMapHandlers);
retrieveMapHandlers.push(function(source) {
  var sourceMappingURL = retrieveSourceMapURL(source);
  if (!sourceMappingURL) return null;

  // Read the contents of the source map
  var sourceMapData;
  if (reSourceMap.test(sourceMappingURL)) {
    // Support source map URL as a data url
    var rawData = sourceMappingURL.slice(sourceMappingURL.indexOf(',') + 1);
    sourceMapData = bufferFrom_1(rawData, "base64").toString();
    sourceMappingURL = source;
  } else {
    // Support source map URLs relative to the source URL
    sourceMappingURL = supportRelativeURL(source, sourceMappingURL);
    sourceMapData = retrieveFile(sourceMappingURL);
  }

  if (!sourceMapData) {
    return null;
  }

  return {
    url: sourceMappingURL,
    map: sourceMapData
  };
});

function mapSourcePosition(position) {
  var sourceMap = sourceMapCache[position.source];
  if (!sourceMap) {
    // Call the (overrideable) retrieveSourceMap function to get the source map.
    var urlAndMap = retrieveSourceMap(position.source);
    if (urlAndMap) {
      sourceMap = sourceMapCache[position.source] = {
        url: urlAndMap.url,
        map: new SourceMapConsumer(urlAndMap.map)
      };

      // Load all sources stored inline with the source map into the file cache
      // to pretend like they are already loaded. They may not exist on disk.
      if (sourceMap.map.sourcesContent) {
        sourceMap.map.sources.forEach(function(source, i) {
          var contents = sourceMap.map.sourcesContent[i];
          if (contents) {
            var url = supportRelativeURL(sourceMap.url, source);
            fileContentsCache[url] = contents;
          }
        });
      }
    } else {
      sourceMap = sourceMapCache[position.source] = {
        url: null,
        map: null
      };
    }
  }

  // Resolve the source URL relative to the URL of the source map
  if (sourceMap && sourceMap.map && typeof sourceMap.map.originalPositionFor === 'function') {
    var originalPosition = sourceMap.map.originalPositionFor(position);

    // Only return the original position if a matching line was found. If no
    // matching line is found then we return position instead, which will cause
    // the stack trace to print the path and line for the compiled file. It is
    // better to give a precise location in the compiled file than a vague
    // location in the original file.
    if (originalPosition.source !== null) {
      originalPosition.source = supportRelativeURL(
        sourceMap.url, originalPosition.source);
      return originalPosition;
    }
  }

  return position;
}

// Parses code generated by FormatEvalOrigin(), a function inside V8:
// https://code.google.com/p/v8/source/browse/trunk/src/messages.js
function mapEvalOrigin(origin) {
  // Most eval() calls are in this format
  var match = /^eval at ([^(]+) \((.+):(\d+):(\d+)\)$/.exec(origin);
  if (match) {
    var position = mapSourcePosition({
      source: match[2],
      line: +match[3],
      column: match[4] - 1
    });
    return 'eval at ' + match[1] + ' (' + position.source + ':' +
      position.line + ':' + (position.column + 1) + ')';
  }

  // Parse nested eval() calls using recursion
  match = /^eval at ([^(]+) \((.+)\)$/.exec(origin);
  if (match) {
    return 'eval at ' + match[1] + ' (' + mapEvalOrigin(match[2]) + ')';
  }

  // Make sure we still return useful information if we didn't find anything
  return origin;
}

// This is copied almost verbatim from the V8 source code at
// https://code.google.com/p/v8/source/browse/trunk/src/messages.js. The
// implementation of wrapCallSite() used to just forward to the actual source
// code of CallSite.prototype.toString but unfortunately a new release of V8
// did something to the prototype chain and broke the shim. The only fix I
// could find was copy/paste.
function CallSiteToString() {
  var fileName;
  var fileLocation = "";
  if (this.isNative()) {
    fileLocation = "native";
  } else {
    fileName = this.getScriptNameOrSourceURL();
    if (!fileName && this.isEval()) {
      fileLocation = this.getEvalOrigin();
      fileLocation += ", ";  // Expecting source position to follow.
    }

    if (fileName) {
      fileLocation += fileName;
    } else {
      // Source code does not originate from a file and is not native, but we
      // can still get the source position inside the source string, e.g. in
      // an eval string.
      fileLocation += "<anonymous>";
    }
    var lineNumber = this.getLineNumber();
    if (lineNumber != null) {
      fileLocation += ":" + lineNumber;
      var columnNumber = this.getColumnNumber();
      if (columnNumber) {
        fileLocation += ":" + columnNumber;
      }
    }
  }

  var line = "";
  var functionName = this.getFunctionName();
  var addSuffix = true;
  var isConstructor = this.isConstructor();
  var isMethodCall = !(this.isToplevel() || isConstructor);
  if (isMethodCall) {
    var typeName = this.getTypeName();
    // Fixes shim to be backward compatable with Node v0 to v4
    if (typeName === "[object Object]") {
      typeName = "null";
    }
    var methodName = this.getMethodName();
    if (functionName) {
      if (typeName && functionName.indexOf(typeName) != 0) {
        line += typeName + ".";
      }
      line += functionName;
      if (methodName && functionName.indexOf("." + methodName) != functionName.length - methodName.length - 1) {
        line += " [as " + methodName + "]";
      }
    } else {
      line += typeName + "." + (methodName || "<anonymous>");
    }
  } else if (isConstructor) {
    line += "new " + (functionName || "<anonymous>");
  } else if (functionName) {
    line += functionName;
  } else {
    line += fileLocation;
    addSuffix = false;
  }
  if (addSuffix) {
    line += " (" + fileLocation + ")";
  }
  return line;
}

function cloneCallSite(frame) {
  var object = {};
  Object.getOwnPropertyNames(Object.getPrototypeOf(frame)).forEach(function(name) {
    object[name] = /^(?:is|get)/.test(name) ? function() { return frame[name].call(frame); } : frame[name];
  });
  object.toString = CallSiteToString;
  return object;
}

function wrapCallSite(frame, state) {
  // provides interface backward compatibility
  if (state === undefined) {
    state = { nextPosition: null, curPosition: null };
  }
  if(frame.isNative()) {
    state.curPosition = null;
    return frame;
  }

  // Most call sites will return the source file from getFileName(), but code
  // passed to eval() ending in "//# sourceURL=..." will return the source file
  // from getScriptNameOrSourceURL() instead
  var source = frame.getFileName() || frame.getScriptNameOrSourceURL();
  if (source) {
    var line = frame.getLineNumber();
    var column = frame.getColumnNumber() - 1;

    // Fix position in Node where some (internal) code is prepended.
    // See https://github.com/evanw/node-source-map-support/issues/36
    // Header removed in node at ^10.16 || >=11.11.0
    // v11 is not an LTS candidate, we can just test the one version with it.
    // Test node versions for: 10.16-19, 10.20+, 12-19, 20-99, 100+, or 11.11
    var noHeader = /^v(10\.1[6-9]|10\.[2-9][0-9]|10\.[0-9]{3,}|1[2-9]\d*|[2-9]\d|\d{3,}|11\.11)/;
    var headerLength = noHeader.test(process.version) ? 0 : 62;
    if (line === 1 && column > headerLength && !isInBrowser() && !frame.isEval()) {
      column -= headerLength;
    }

    var position = mapSourcePosition({
      source: source,
      line: line,
      column: column
    });
    state.curPosition = position;
    frame = cloneCallSite(frame);
    var originalFunctionName = frame.getFunctionName;
    frame.getFunctionName = function() {
      if (state.nextPosition == null) {
        return originalFunctionName();
      }
      return state.nextPosition.name || originalFunctionName();
    };
    frame.getFileName = function() { return position.source; };
    frame.getLineNumber = function() { return position.line; };
    frame.getColumnNumber = function() { return position.column + 1; };
    frame.getScriptNameOrSourceURL = function() { return position.source; };
    return frame;
  }

  // Code called using eval() needs special handling
  var origin = frame.isEval() && frame.getEvalOrigin();
  if (origin) {
    origin = mapEvalOrigin(origin);
    frame = cloneCallSite(frame);
    frame.getEvalOrigin = function() { return origin; };
    return frame;
  }

  // If we get here then we were unable to change the source position
  return frame;
}

// This function is part of the V8 stack trace API, for more info see:
// https://v8.dev/docs/stack-trace-api
function prepareStackTrace(error, stack) {
  if (emptyCacheBetweenOperations) {
    fileContentsCache = {};
    sourceMapCache = {};
  }

  var name = error.name || 'Error';
  var message = error.message || '';
  var errorString = name + ": " + message;

  var state = { nextPosition: null, curPosition: null };
  var processedStack = [];
  for (var i = stack.length - 1; i >= 0; i--) {
    processedStack.push('\n    at ' + wrapCallSite(stack[i], state));
    state.nextPosition = state.curPosition;
  }
  state.curPosition = state.nextPosition = null;
  return errorString + processedStack.reverse().join('');
}

// Generate position and snippet of original source with pointer
function getErrorSource(error) {
  var match = /\n    at [^(]+ \((.*):(\d+):(\d+)\)/.exec(error.stack);
  if (match) {
    var source = match[1];
    var line = +match[2];
    var column = +match[3];

    // Support the inline sourceContents inside the source map
    var contents = fileContentsCache[source];

    // Support files on disk
    if (!contents && fs$1 && fs$1.existsSync(source)) {
      try {
        contents = fs$1.readFileSync(source, 'utf8');
      } catch (er) {
        contents = '';
      }
    }

    // Format the line from the original source code like node does
    if (contents) {
      var code = contents.split(/(?:\r\n|\r|\n)/)[line - 1];
      if (code) {
        return source + ':' + line + '\n' + code + '\n' +
          new Array(column).join(' ') + '^';
      }
    }
  }
  return null;
}

function printErrorAndExit (error) {
  var source = getErrorSource(error);

  // Ensure error is printed synchronously and not truncated
  if (process.stderr._handle && process.stderr._handle.setBlocking) {
    process.stderr._handle.setBlocking(true);
  }

  if (source) {
    console.error();
    console.error(source);
  }

  console.error(error.stack);
  process.exit(1);
}

function shimEmitUncaughtException () {
  var origEmit = process.emit;

  process.emit = function (type) {
    if (type === 'uncaughtException') {
      var hasStack = (arguments[1] && arguments[1].stack);
      var hasListeners = (this.listeners(type).length > 0);

      if (hasStack && !hasListeners) {
        return printErrorAndExit(arguments[1]);
      }
    }

    return origEmit.apply(this, arguments);
  };
}

var originalRetrieveFileHandlers = retrieveFileHandlers.slice(0);
var originalRetrieveMapHandlers = retrieveMapHandlers.slice(0);

exports.wrapCallSite = wrapCallSite;
exports.getErrorSource = getErrorSource;
exports.mapSourcePosition = mapSourcePosition;
exports.retrieveSourceMap = retrieveSourceMap;

exports.install = function(options) {
  options = options || {};

  if (options.environment) {
    environment = options.environment;
    if (["node", "browser", "auto"].indexOf(environment) === -1) {
      throw new Error("environment " + environment + " was unknown. Available options are {auto, browser, node}")
    }
  }

  // Allow sources to be found by methods other than reading the files
  // directly from disk.
  if (options.retrieveFile) {
    if (options.overrideRetrieveFile) {
      retrieveFileHandlers.length = 0;
    }

    retrieveFileHandlers.unshift(options.retrieveFile);
  }

  // Allow source maps to be found by methods other than reading the files
  // directly from disk.
  if (options.retrieveSourceMap) {
    if (options.overrideRetrieveSourceMap) {
      retrieveMapHandlers.length = 0;
    }

    retrieveMapHandlers.unshift(options.retrieveSourceMap);
  }

  // Support runtime transpilers that include inline source maps
  if (options.hookRequire && !isInBrowser()) {
    // Use dynamicRequire to avoid including in browser bundles
    var Module = dynamicRequire(module, 'module');
    var $compile = Module.prototype._compile;

    if (!$compile.__sourceMapSupport) {
      Module.prototype._compile = function(content, filename) {
        fileContentsCache[filename] = content;
        sourceMapCache[filename] = undefined;
        return $compile.call(this, content, filename);
      };

      Module.prototype._compile.__sourceMapSupport = true;
    }
  }

  // Configure options
  if (!emptyCacheBetweenOperations) {
    emptyCacheBetweenOperations = 'emptyCacheBetweenOperations' in options ?
      options.emptyCacheBetweenOperations : false;
  }

  // Install the error reformatter
  if (!errorFormatterInstalled) {
    errorFormatterInstalled = true;
    Error.prepareStackTrace = prepareStackTrace;
  }

  if (!uncaughtShimInstalled) {
    var installHandler = 'handleUncaughtExceptions' in options ?
      options.handleUncaughtExceptions : true;

    // Do not override 'uncaughtException' with our own handler in Node.js
    // Worker threads. Workers pass the error to the main thread as an event,
    // rather than printing something to stderr and exiting.
    try {
      // We need to use `dynamicRequire` because `require` on it's own will be optimized by WebPack/Browserify.
      var worker_threads = dynamicRequire(module, 'worker_threads');
      if (worker_threads.isMainThread === false) {
        installHandler = false;
      }
    } catch(e) {}

    // Provide the option to not install the uncaught exception handler. This is
    // to support other uncaught exception handlers (in test frameworks, for
    // example). If this handler is not installed and there are no other uncaught
    // exception handlers, uncaught exceptions will be caught by node's built-in
    // exception handler and the process will still be terminated. However, the
    // generated JavaScript code will be shown above the stack trace instead of
    // the original source code.
    if (installHandler && hasGlobalProcessEventEmitter()) {
      uncaughtShimInstalled = true;
      shimEmitUncaughtException();
    }
  }
};

exports.resetRetrieveHandlers = function() {
  retrieveFileHandlers.length = 0;
  retrieveMapHandlers.length = 0;

  retrieveFileHandlers = originalRetrieveFileHandlers.slice(0);
  retrieveMapHandlers = originalRetrieveMapHandlers.slice(0);

  retrieveSourceMap = handlerExec(retrieveMapHandlers);
  retrieveFile = handlerExec(retrieveFileHandlers);
};
});
var sourceMapSupport_1 = sourceMapSupport.wrapCallSite;
var sourceMapSupport_2 = sourceMapSupport.getErrorSource;
var sourceMapSupport_3 = sourceMapSupport.mapSourcePosition;
var sourceMapSupport_4 = sourceMapSupport.retrieveSourceMap;
var sourceMapSupport_5 = sourceMapSupport.install;
var sourceMapSupport_6 = sourceMapSupport.resetRetrieveHandlers;

sourceMapSupport.install();

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var grapes_min = createCommonjsModule(function (module, exports) {
/*! grapesjs - 0.16.12 */
!function(t,e){module.exports=e();}(window,(function(){return function(t){var e={};function n(r){if(e[r])return e[r].exports;var i=e[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=t,n.c=e,n.d=function(t,e,r){n.o(t,e)||Object.defineProperty(t,e,{enumerable:!0,get:r});},n.r=function(t){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0});},n.t=function(t,e){if(1&e&&(t=n(t)),8&e)return t;if(4&e&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&e&&"string"!=typeof t)for(var i in t)n.d(r,i,function(e){return t[e]}.bind(null,i));return r},n.n=function(t){var e=t&&t.__esModule?function(){return t.default}:function(){return t};return n.d(e,"a",e),e},n.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},n.p="",n(n.s=91)}([function(t,e,n){(function(t,n){var r;!function(){var i="object"==typeof self&&self.self===self&&self||"object"==typeof t&&t.global===t&&t||this||{},o=i._,a=Array.prototype,s=Object.prototype,l="undefined"!=typeof Symbol?Symbol.prototype:null,c=a.push,u=a.slice,h=s.toString,d=s.hasOwnProperty,f=Array.isArray,p=Object.keys,g=Object.create,v=function(){},m=function(t){return t instanceof m?t:this instanceof m?void(this._wrapped=t):new m(t)};e.nodeType?i._=m:(!n.nodeType&&n.exports&&(e=n.exports=m),e._=m),m.VERSION="1.9.1";var y,b=function(t,e,n){if(void 0===e)return t;switch(null==n?3:n){case 1:return function(n){return t.call(e,n)};case 3:return function(n,r,i){return t.call(e,n,r,i)};case 4:return function(n,r,i,o){return t.call(e,n,r,i,o)}}return function(){return t.apply(e,arguments)}},w=function(t,e,n){return m.iteratee!==y?m.iteratee(t,e):null==t?m.identity:m.isFunction(t)?b(t,e,n):m.isObject(t)&&!m.isArray(t)?m.matcher(t):m.property(t)};m.iteratee=y=function(t,e){return w(t,e,1/0)};var x=function(t,e){return e=null==e?t.length-1:+e,function(){for(var n=Math.max(arguments.length-e,0),r=Array(n),i=0;i<n;i++)r[i]=arguments[i+e];switch(e){case 0:return t.call(this,r);case 1:return t.call(this,arguments[0],r);case 2:return t.call(this,arguments[0],arguments[1],r)}var o=Array(e+1);for(i=0;i<e;i++)o[i]=arguments[i];return o[e]=r,t.apply(this,o)}},O=function(t){if(!m.isObject(t))return {};if(g)return g(t);v.prototype=t;var e=new v;return v.prototype=null,e},C=function(t){return function(e){return null==e?void 0:e[t]}},S=function(t,e){return null!=t&&d.call(t,e)},k=function(t,e){for(var n=e.length,r=0;r<n;r++){if(null==t)return;t=t[e[r]];}return n?t:void 0},T=Math.pow(2,53)-1,j=C("length"),P=function(t){var e=j(t);return "number"==typeof e&&e>=0&&e<=T};m.each=m.forEach=function(t,e,n){var r,i;if(e=b(e,n),P(t))for(r=0,i=t.length;r<i;r++)e(t[r],r,t);else {var o=m.keys(t);for(r=0,i=o.length;r<i;r++)e(t[o[r]],o[r],t);}return t},m.map=m.collect=function(t,e,n){e=w(e,n);for(var r=!P(t)&&m.keys(t),i=(r||t).length,o=Array(i),a=0;a<i;a++){var s=r?r[a]:a;o[a]=e(t[s],s,t);}return o};var E=function(t){var e=function(e,n,r,i){var o=!P(e)&&m.keys(e),a=(o||e).length,s=t>0?0:a-1;for(i||(r=e[o?o[s]:s],s+=t);s>=0&&s<a;s+=t){var l=o?o[s]:s;r=n(r,e[l],l,e);}return r};return function(t,n,r,i){var o=arguments.length>=3;return e(t,b(n,i,4),r,o)}};m.reduce=m.foldl=m.inject=E(1),m.reduceRight=m.foldr=E(-1),m.find=m.detect=function(t,e,n){var r=(P(t)?m.findIndex:m.findKey)(t,e,n);if(void 0!==r&&-1!==r)return t[r]},m.filter=m.select=function(t,e,n){var r=[];return e=w(e,n),m.each(t,(function(t,n,i){e(t,n,i)&&r.push(t);})),r},m.reject=function(t,e,n){return m.filter(t,m.negate(w(e)),n)},m.every=m.all=function(t,e,n){e=w(e,n);for(var r=!P(t)&&m.keys(t),i=(r||t).length,o=0;o<i;o++){var a=r?r[o]:o;if(!e(t[a],a,t))return !1}return !0},m.some=m.any=function(t,e,n){e=w(e,n);for(var r=!P(t)&&m.keys(t),i=(r||t).length,o=0;o<i;o++){var a=r?r[o]:o;if(e(t[a],a,t))return !0}return !1},m.contains=m.includes=m.include=function(t,e,n,r){return P(t)||(t=m.values(t)),("number"!=typeof n||r)&&(n=0),m.indexOf(t,e,n)>=0},m.invoke=x((function(t,e,n){var r,i;return m.isFunction(e)?i=e:m.isArray(e)&&(r=e.slice(0,-1),e=e[e.length-1]),m.map(t,(function(t){var o=i;if(!o){if(r&&r.length&&(t=k(t,r)),null==t)return;o=t[e];}return null==o?o:o.apply(t,n)}))})),m.pluck=function(t,e){return m.map(t,m.property(e))},m.where=function(t,e){return m.filter(t,m.matcher(e))},m.findWhere=function(t,e){return m.find(t,m.matcher(e))},m.max=function(t,e,n){var r,i,o=-1/0,a=-1/0;if(null==e||"number"==typeof e&&"object"!=typeof t[0]&&null!=t)for(var s=0,l=(t=P(t)?t:m.values(t)).length;s<l;s++)null!=(r=t[s])&&r>o&&(o=r);else e=w(e,n),m.each(t,(function(t,n,r){((i=e(t,n,r))>a||i===-1/0&&o===-1/0)&&(o=t,a=i);}));return o},m.min=function(t,e,n){var r,i,o=1/0,a=1/0;if(null==e||"number"==typeof e&&"object"!=typeof t[0]&&null!=t)for(var s=0,l=(t=P(t)?t:m.values(t)).length;s<l;s++)null!=(r=t[s])&&r<o&&(o=r);else e=w(e,n),m.each(t,(function(t,n,r){((i=e(t,n,r))<a||i===1/0&&o===1/0)&&(o=t,a=i);}));return o},m.shuffle=function(t){return m.sample(t,1/0)},m.sample=function(t,e,n){if(null==e||n)return P(t)||(t=m.values(t)),t[m.random(t.length-1)];var r=P(t)?m.clone(t):m.values(t),i=j(r);e=Math.max(Math.min(e,i),0);for(var o=i-1,a=0;a<e;a++){var s=m.random(a,o),l=r[a];r[a]=r[s],r[s]=l;}return r.slice(0,e)},m.sortBy=function(t,e,n){var r=0;return e=w(e,n),m.pluck(m.map(t,(function(t,n,i){return {value:t,index:r++,criteria:e(t,n,i)}})).sort((function(t,e){var n=t.criteria,r=e.criteria;if(n!==r){if(n>r||void 0===n)return 1;if(n<r||void 0===r)return -1}return t.index-e.index})),"value")};var M=function(t,e){return function(n,r,i){var o=e?[[],[]]:{};return r=w(r,i),m.each(n,(function(e,i){var a=r(e,i,n);t(o,e,a);})),o}};m.groupBy=M((function(t,e,n){S(t,n)?t[n].push(e):t[n]=[e];})),m.indexBy=M((function(t,e,n){t[n]=e;})),m.countBy=M((function(t,e,n){S(t,n)?t[n]++:t[n]=1;}));var D=/[^\ud800-\udfff]|[\ud800-\udbff][\udc00-\udfff]|[\ud800-\udfff]/g;m.toArray=function(t){return t?m.isArray(t)?u.call(t):m.isString(t)?t.match(D):P(t)?m.map(t,m.identity):m.values(t):[]},m.size=function(t){return null==t?0:P(t)?t.length:m.keys(t).length},m.partition=M((function(t,e,n){t[n?0:1].push(e);}),!0),m.first=m.head=m.take=function(t,e,n){return null==t||t.length<1?null==e?void 0:[]:null==e||n?t[0]:m.initial(t,t.length-e)},m.initial=function(t,e,n){return u.call(t,0,Math.max(0,t.length-(null==e||n?1:e)))},m.last=function(t,e,n){return null==t||t.length<1?null==e?void 0:[]:null==e||n?t[t.length-1]:m.rest(t,Math.max(0,t.length-e))},m.rest=m.tail=m.drop=function(t,e,n){return u.call(t,null==e||n?1:e)},m.compact=function(t){return m.filter(t,Boolean)};var A=function(t,e,n,r){for(var i=(r=r||[]).length,o=0,a=j(t);o<a;o++){var s=t[o];if(P(s)&&(m.isArray(s)||m.isArguments(s)))if(e)for(var l=0,c=s.length;l<c;)r[i++]=s[l++];else A(s,e,n,r),i=r.length;else n||(r[i++]=s);}return r};m.flatten=function(t,e){return A(t,e,!1)},m.without=x((function(t,e){return m.difference(t,e)})),m.uniq=m.unique=function(t,e,n,r){m.isBoolean(e)||(r=n,n=e,e=!1),null!=n&&(n=w(n,r));for(var i=[],o=[],a=0,s=j(t);a<s;a++){var l=t[a],c=n?n(l,a,t):l;e&&!n?(a&&o===c||i.push(l),o=c):n?m.contains(o,c)||(o.push(c),i.push(l)):m.contains(i,l)||i.push(l);}return i},m.union=x((function(t){return m.uniq(A(t,!0,!0))})),m.intersection=function(t){for(var e=[],n=arguments.length,r=0,i=j(t);r<i;r++){var o=t[r];if(!m.contains(e,o)){var a;for(a=1;a<n&&m.contains(arguments[a],o);a++);a===n&&e.push(o);}}return e},m.difference=x((function(t,e){return e=A(e,!0,!0),m.filter(t,(function(t){return !m.contains(e,t)}))})),m.unzip=function(t){for(var e=t&&m.max(t,j).length||0,n=Array(e),r=0;r<e;r++)n[r]=m.pluck(t,r);return n},m.zip=x(m.unzip),m.object=function(t,e){for(var n={},r=0,i=j(t);r<i;r++)e?n[t[r]]=e[r]:n[t[r][0]]=t[r][1];return n};var L=function(t){return function(e,n,r){n=w(n,r);for(var i=j(e),o=t>0?0:i-1;o>=0&&o<i;o+=t)if(n(e[o],o,e))return o;return -1}};m.findIndex=L(1),m.findLastIndex=L(-1),m.sortedIndex=function(t,e,n,r){for(var i=(n=w(n,r,1))(e),o=0,a=j(t);o<a;){var s=Math.floor((o+a)/2);n(t[s])<i?o=s+1:a=s;}return o};var N=function(t,e,n){return function(r,i,o){var a=0,s=j(r);if("number"==typeof o)t>0?a=o>=0?o:Math.max(o+s,a):s=o>=0?Math.min(o+1,s):o+s+1;else if(n&&o&&s)return r[o=n(r,i)]===i?o:-1;if(i!=i)return (o=e(u.call(r,a,s),m.isNaN))>=0?o+a:-1;for(o=t>0?a:s-1;o>=0&&o<s;o+=t)if(r[o]===i)return o;return -1}};m.indexOf=N(1,m.findIndex,m.sortedIndex),m.lastIndexOf=N(-1,m.findLastIndex),m.range=function(t,e,n){null==e&&(e=t||0,t=0),n||(n=e<t?-1:1);for(var r=Math.max(Math.ceil((e-t)/n),0),i=Array(r),o=0;o<r;o++,t+=n)i[o]=t;return i},m.chunk=function(t,e){if(null==e||e<1)return [];for(var n=[],r=0,i=t.length;r<i;)n.push(u.call(t,r,r+=e));return n};var I=function(t,e,n,r,i){if(!(r instanceof e))return t.apply(n,i);var o=O(t.prototype),a=t.apply(o,i);return m.isObject(a)?a:o};m.bind=x((function(t,e,n){if(!m.isFunction(t))throw new TypeError("Bind must be called on a function");var r=x((function(i){return I(t,r,e,this,n.concat(i))}));return r})),m.partial=x((function(t,e){var n=m.partial.placeholder,r=function(){for(var i=0,o=e.length,a=Array(o),s=0;s<o;s++)a[s]=e[s]===n?arguments[i++]:e[s];for(;i<arguments.length;)a.push(arguments[i++]);return I(t,r,this,this,a)};return r})),m.partial.placeholder=m,m.bindAll=x((function(t,e){var n=(e=A(e,!1,!1)).length;if(n<1)throw new Error("bindAll must be passed function names");for(;n--;){var r=e[n];t[r]=m.bind(t[r],t);}})),m.memoize=function(t,e){var n=function(r){var i=n.cache,o=""+(e?e.apply(this,arguments):r);return S(i,o)||(i[o]=t.apply(this,arguments)),i[o]};return n.cache={},n},m.delay=x((function(t,e,n){return setTimeout((function(){return t.apply(null,n)}),e)})),m.defer=m.partial(m.delay,m,1),m.throttle=function(t,e,n){var r,i,o,a,s=0;n||(n={});var l=function(){s=!1===n.leading?0:m.now(),r=null,a=t.apply(i,o),r||(i=o=null);},c=function(){var c=m.now();s||!1!==n.leading||(s=c);var u=e-(c-s);return i=this,o=arguments,u<=0||u>e?(r&&(clearTimeout(r),r=null),s=c,a=t.apply(i,o),r||(i=o=null)):r||!1===n.trailing||(r=setTimeout(l,u)),a};return c.cancel=function(){clearTimeout(r),s=0,r=i=o=null;},c},m.debounce=function(t,e,n){var r,i,o=function(e,n){r=null,n&&(i=t.apply(e,n));},a=x((function(a){if(r&&clearTimeout(r),n){var s=!r;r=setTimeout(o,e),s&&(i=t.apply(this,a));}else r=m.delay(o,e,this,a);return i}));return a.cancel=function(){clearTimeout(r),r=null;},a},m.wrap=function(t,e){return m.partial(e,t)},m.negate=function(t){return function(){return !t.apply(this,arguments)}},m.compose=function(){var t=arguments,e=t.length-1;return function(){for(var n=e,r=t[e].apply(this,arguments);n--;)r=t[n].call(this,r);return r}},m.after=function(t,e){return function(){if(--t<1)return e.apply(this,arguments)}},m.before=function(t,e){var n;return function(){return --t>0&&(n=e.apply(this,arguments)),t<=1&&(e=null),n}},m.once=m.partial(m.before,2),m.restArguments=x;var F=!{toString:null}.propertyIsEnumerable("toString"),_=["valueOf","isPrototypeOf","toString","propertyIsEnumerable","hasOwnProperty","toLocaleString"],V=function(t,e){var n=_.length,r=t.constructor,i=m.isFunction(r)&&r.prototype||s,o="constructor";for(S(t,o)&&!m.contains(e,o)&&e.push(o);n--;)(o=_[n])in t&&t[o]!==i[o]&&!m.contains(e,o)&&e.push(o);};m.keys=function(t){if(!m.isObject(t))return [];if(p)return p(t);var e=[];for(var n in t)S(t,n)&&e.push(n);return F&&V(t,e),e},m.allKeys=function(t){if(!m.isObject(t))return [];var e=[];for(var n in t)e.push(n);return F&&V(t,e),e},m.values=function(t){for(var e=m.keys(t),n=e.length,r=Array(n),i=0;i<n;i++)r[i]=t[e[i]];return r},m.mapObject=function(t,e,n){e=w(e,n);for(var r=m.keys(t),i=r.length,o={},a=0;a<i;a++){var s=r[a];o[s]=e(t[s],s,t);}return o},m.pairs=function(t){for(var e=m.keys(t),n=e.length,r=Array(n),i=0;i<n;i++)r[i]=[e[i],t[e[i]]];return r},m.invert=function(t){for(var e={},n=m.keys(t),r=0,i=n.length;r<i;r++)e[t[n[r]]]=n[r];return e},m.functions=m.methods=function(t){var e=[];for(var n in t)m.isFunction(t[n])&&e.push(n);return e.sort()};var R=function(t,e){return function(n){var r=arguments.length;if(e&&(n=Object(n)),r<2||null==n)return n;for(var i=1;i<r;i++)for(var o=arguments[i],a=t(o),s=a.length,l=0;l<s;l++){var c=a[l];e&&void 0!==n[c]||(n[c]=o[c]);}return n}};m.extend=R(m.allKeys),m.extendOwn=m.assign=R(m.keys),m.findKey=function(t,e,n){e=w(e,n);for(var r,i=m.keys(t),o=0,a=i.length;o<a;o++)if(e(t[r=i[o]],r,t))return r};var H,z,$=function(t,e,n){return e in n};m.pick=x((function(t,e){var n={},r=e[0];if(null==t)return n;m.isFunction(r)?(e.length>1&&(r=b(r,e[1])),e=m.allKeys(t)):(r=$,e=A(e,!1,!1),t=Object(t));for(var i=0,o=e.length;i<o;i++){var a=e[i],s=t[a];r(s,a,t)&&(n[a]=s);}return n})),m.omit=x((function(t,e){var n,r=e[0];return m.isFunction(r)?(r=m.negate(r),e.length>1&&(n=e[1])):(e=m.map(A(e,!1,!1),String),r=function(t,n){return !m.contains(e,n)}),m.pick(t,r,n)})),m.defaults=R(m.allKeys,!0),m.create=function(t,e){var n=O(t);return e&&m.extendOwn(n,e),n},m.clone=function(t){return m.isObject(t)?m.isArray(t)?t.slice():m.extend({},t):t},m.tap=function(t,e){return e(t),t},m.isMatch=function(t,e){var n=m.keys(e),r=n.length;if(null==t)return !r;for(var i=Object(t),o=0;o<r;o++){var a=n[o];if(e[a]!==i[a]||!(a in i))return !1}return !0},H=function(t,e,n,r){if(t===e)return 0!==t||1/t==1/e;if(null==t||null==e)return !1;if(t!=t)return e!=e;var i=typeof t;return ("function"===i||"object"===i||"object"==typeof e)&&z(t,e,n,r)},z=function(t,e,n,r){t instanceof m&&(t=t._wrapped),e instanceof m&&(e=e._wrapped);var i=h.call(t);if(i!==h.call(e))return !1;switch(i){case"[object RegExp]":case"[object String]":return ""+t==""+e;case"[object Number]":return +t!=+t?+e!=+e:0==+t?1/+t==1/e:+t==+e;case"[object Date]":case"[object Boolean]":return +t==+e;case"[object Symbol]":return l.valueOf.call(t)===l.valueOf.call(e)}var o="[object Array]"===i;if(!o){if("object"!=typeof t||"object"!=typeof e)return !1;var a=t.constructor,s=e.constructor;if(a!==s&&!(m.isFunction(a)&&a instanceof a&&m.isFunction(s)&&s instanceof s)&&"constructor"in t&&"constructor"in e)return !1}r=r||[];for(var c=(n=n||[]).length;c--;)if(n[c]===t)return r[c]===e;if(n.push(t),r.push(e),o){if((c=t.length)!==e.length)return !1;for(;c--;)if(!H(t[c],e[c],n,r))return !1}else {var u,d=m.keys(t);if(c=d.length,m.keys(e).length!==c)return !1;for(;c--;)if(u=d[c],!S(e,u)||!H(t[u],e[u],n,r))return !1}return n.pop(),r.pop(),!0},m.isEqual=function(t,e){return H(t,e)},m.isEmpty=function(t){return null==t||(P(t)&&(m.isArray(t)||m.isString(t)||m.isArguments(t))?0===t.length:0===m.keys(t).length)},m.isElement=function(t){return !(!t||1!==t.nodeType)},m.isArray=f||function(t){return "[object Array]"===h.call(t)},m.isObject=function(t){var e=typeof t;return "function"===e||"object"===e&&!!t},m.each(["Arguments","Function","String","Number","Date","RegExp","Error","Symbol","Map","WeakMap","Set","WeakSet"],(function(t){m["is"+t]=function(e){return h.call(e)==="[object "+t+"]"};})),m.isArguments(arguments)||(m.isArguments=function(t){return S(t,"callee")});var W=i.document&&i.document.childNodes;"object"!=typeof Int8Array&&"function"!=typeof W&&(m.isFunction=function(t){return "function"==typeof t||!1}),m.isFinite=function(t){return !m.isSymbol(t)&&isFinite(t)&&!isNaN(parseFloat(t))},m.isNaN=function(t){return m.isNumber(t)&&isNaN(t)},m.isBoolean=function(t){return !0===t||!1===t||"[object Boolean]"===h.call(t)},m.isNull=function(t){return null===t},m.isUndefined=function(t){return void 0===t},m.has=function(t,e){if(!m.isArray(e))return S(t,e);for(var n=e.length,r=0;r<n;r++){var i=e[r];if(null==t||!d.call(t,i))return !1;t=t[i];}return !!n},m.noConflict=function(){return i._=o,this},m.identity=function(t){return t},m.constant=function(t){return function(){return t}},m.noop=function(){},m.property=function(t){return m.isArray(t)?function(e){return k(e,t)}:C(t)},m.propertyOf=function(t){return null==t?function(){}:function(e){return m.isArray(e)?k(t,e):t[e]}},m.matcher=m.matches=function(t){return t=m.extendOwn({},t),function(e){return m.isMatch(e,t)}},m.times=function(t,e,n){var r=Array(Math.max(0,t));e=b(e,n,1);for(var i=0;i<t;i++)r[i]=e(i);return r},m.random=function(t,e){return null==e&&(e=t,t=0),t+Math.floor(Math.random()*(e-t+1))},m.now=Date.now||function(){return (new Date).getTime()};var B={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#x27;","`":"&#x60;"},U=m.invert(B),q=function(t){var e=function(e){return t[e]},n="(?:"+m.keys(t).join("|")+")",r=RegExp(n),i=RegExp(n,"g");return function(t){return t=null==t?"":""+t,r.test(t)?t.replace(i,e):t}};m.escape=q(B),m.unescape=q(U),m.result=function(t,e,n){m.isArray(e)||(e=[e]);var r=e.length;if(!r)return m.isFunction(n)?n.call(t):n;for(var i=0;i<r;i++){var o=null==t?void 0:t[e[i]];void 0===o&&(o=n,i=r),t=m.isFunction(o)?o.call(t):o;}return t};var G=0;m.uniqueId=function(t){var e=++G+"";return t?t+e:e},m.templateSettings={evaluate:/<%([\s\S]+?)%>/g,interpolate:/<%=([\s\S]+?)%>/g,escape:/<%-([\s\S]+?)%>/g};var K=/(.)^/,Y={"'":"'","\\":"\\","\r":"r","\n":"n","\u2028":"u2028","\u2029":"u2029"},X=/\\|'|\r|\n|\u2028|\u2029/g,J=function(t){return "\\"+Y[t]};m.template=function(t,e,n){!e&&n&&(e=n),e=m.defaults({},e,m.templateSettings);var r,i=RegExp([(e.escape||K).source,(e.interpolate||K).source,(e.evaluate||K).source].join("|")+"|$","g"),o=0,a="__p+='";t.replace(i,(function(e,n,r,i,s){return a+=t.slice(o,s).replace(X,J),o=s+e.length,n?a+="'+\n((__t=("+n+"))==null?'':_.escape(__t))+\n'":r?a+="'+\n((__t=("+r+"))==null?'':__t)+\n'":i&&(a+="';\n"+i+"\n__p+='"),e})),a+="';\n",e.variable||(a="with(obj||{}){\n"+a+"}\n"),a="var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};\n"+a+"return __p;\n";try{r=new Function(e.variable||"obj","_",a);}catch(t){throw t.source=a,t}var s=function(t){return r.call(this,t,m)},l=e.variable||"obj";return s.source="function("+l+"){\n"+a+"}",s},m.chain=function(t){var e=m(t);return e._chain=!0,e};var Z=function(t,e){return t._chain?m(e).chain():e};m.mixin=function(t){return m.each(m.functions(t),(function(e){var n=m[e]=t[e];m.prototype[e]=function(){var t=[this._wrapped];return c.apply(t,arguments),Z(this,n.apply(m,t))};})),m},m.mixin(m),m.each(["pop","push","reverse","shift","sort","splice","unshift"],(function(t){var e=a[t];m.prototype[t]=function(){var n=this._wrapped;return e.apply(n,arguments),"shift"!==t&&"splice"!==t||0!==n.length||delete n[0],Z(this,n)};})),m.each(["concat","join","slice"],(function(t){var e=a[t];m.prototype[t]=function(){return Z(this,e.apply(this._wrapped,arguments))};})),m.prototype.value=function(){return this._wrapped},m.prototype.valueOf=m.prototype.toJSON=m.prototype.value,m.prototype.toString=function(){return String(this._wrapped)},void 0===(r=function(){return m}.apply(e,[]))||(n.exports=r);}();}).call(this,n(24),n(70)(t));},function(t,e,n){(function(r){var i,o,a;a="object"==typeof self&&self.self===self&&self||"object"==typeof r&&r.global===r&&r,i=[n(0),n(10),e],void 0===(o=function(t,e,n){a.Backbone=function(t,e,n,r){var i=t.Backbone,o=Array.prototype.slice;e.VERSION="1.3.3",e.$=r,e.noConflict=function(){return t.Backbone=i,this},e.emulateHTTP=!1,e.emulateJSON=!1;var a=function(t,e,r){n.each(e,(function(e,i){n[i]&&(t.prototype[i]=function(t,e,r){switch(t){case 1:return function(){return n[e](this[r])};case 2:return function(t){return n[e](this[r],t)};case 3:return function(t,i){return n[e](this[r],s(t,this),i)};case 4:return function(t,i,o){return n[e](this[r],s(t,this),i,o)};default:return function(){var t=o.call(arguments);return t.unshift(this[r]),n[e].apply(n,t)}}}(e,i,r));}));},s=function(t,e){return n.isFunction(t)?t:n.isObject(t)&&!e._isModel(t)?l(t):n.isString(t)?function(e){return e.get(t)}:t},l=function(t){var e=n.matches(t);return function(t){return e(t.attributes)}},c=e.Events={},u=/\s+/,h=function(t,e,r,i,o){var a,s=0;if(r&&"object"==typeof r){void 0!==i&&"context"in o&&void 0===o.context&&(o.context=i);for(a=n.keys(r);s<a.length;s++)e=h(t,e,a[s],r[a[s]],o);}else if(r&&u.test(r))for(a=r.split(u);s<a.length;s++)e=t(e,a[s],i,o);else e=t(e,r,i,o);return e};c.on=function(t,e,n){return d(this,t,e,n)};var d=function(t,e,n,r,i){return t._events=h(f,t._events||{},e,n,{context:r,ctx:t,listening:i}),i&&((t._listeners||(t._listeners={}))[i.id]=i),t};c.listenTo=function(t,e,r){if(!t)return this;var i=t._listenId||(t._listenId=n.uniqueId("l")),o=this._listeningTo||(this._listeningTo={}),a=o[i];if(!a){var s=this._listenId||(this._listenId=n.uniqueId("l"));a=o[i]={obj:t,objId:i,id:s,listeningTo:o,count:0};}return d(t,e,r,this,a),this};var f=function(t,e,n,r){if(n){var i=t[e]||(t[e]=[]),o=r.context,a=r.ctx,s=r.listening;s&&s.count++,i.push({callback:n,context:o,ctx:o||a,listening:s});}return t};c.off=function(t,e,n){return this._events?(this._events=h(p,this._events,t,e,{context:n,listeners:this._listeners}),this):this},c.stopListening=function(t,e,r){var i=this._listeningTo;if(!i)return this;for(var o=t?[t._listenId]:n.keys(i),a=0;a<o.length;a++){var s=i[o[a]];if(!s)break;s.obj.off(e,r,this);}return this};var p=function(t,e,r,i){if(t){var o,a=0,s=i.context,l=i.listeners;if(e||r||s){for(var c=e?[e]:n.keys(t);a<c.length;a++){var u=t[e=c[a]];if(!u)break;for(var h=[],d=0;d<u.length;d++){var f=u[d];r&&r!==f.callback&&r!==f.callback._callback||s&&s!==f.context?h.push(f):(o=f.listening)&&0==--o.count&&(delete l[o.id],delete o.listeningTo[o.objId]);}h.length?t[e]=h:delete t[e];}return t}for(var p=n.keys(l);a<p.length;a++)delete l[(o=l[p[a]]).id],delete o.listeningTo[o.objId];}};c.once=function(t,e,r){var i=h(g,{},t,e,n.bind(this.off,this));return "string"==typeof t&&null==r&&(e=void 0),this.on(i,e,r)},c.listenToOnce=function(t,e,r){var i=h(g,{},e,r,n.bind(this.stopListening,this,t));return this.listenTo(t,i)};var g=function(t,e,r,i){if(r){var o=t[e]=n.once((function(){i(e,o),r.apply(this,arguments);}));o._callback=r;}return t};c.trigger=function(t){if(!this._events)return this;for(var e=Math.max(0,arguments.length-1),n=Array(e),r=0;r<e;r++)n[r]=arguments[r+1];return h(v,this._events,t,void 0,n),this};var v=function(t,e,n,r){if(t){var i=t[e],o=t.all;i&&o&&(o=o.slice()),i&&m(i,r),o&&m(o,[e].concat(r));}return t},m=function(t,e){var n,r=-1,i=t.length,o=e[0],a=e[1],s=e[2];switch(e.length){case 0:for(;++r<i;)(n=t[r]).callback.call(n.ctx);return;case 1:for(;++r<i;)(n=t[r]).callback.call(n.ctx,o);return;case 2:for(;++r<i;)(n=t[r]).callback.call(n.ctx,o,a);return;case 3:for(;++r<i;)(n=t[r]).callback.call(n.ctx,o,a,s);return;default:for(;++r<i;)(n=t[r]).callback.apply(n.ctx,e);return}};c.bind=c.on,c.unbind=c.off,n.extend(e,c);var y=e.Model=function(t,e){var r=t||{};e||(e={}),this.cid=n.uniqueId(this.cidPrefix),this.attributes={},e.collection&&(this.collection=e.collection),e.parse&&(r=this.parse(r,e)||{});var i=n.result(this,"defaults");r=n.defaults(n.extend({},i,r),i),this.set(r,e),this.changed={},this.initialize.apply(this,arguments);};n.extend(y.prototype,c,{changed:null,validationError:null,idAttribute:"id",cidPrefix:"c",initialize:function(){},toJSON:function(t){return n.clone(this.attributes)},sync:function(){return e.sync.apply(this,arguments)},get:function(t){return this.attributes[t]},escape:function(t){return n.escape(this.get(t))},has:function(t){return null!=this.get(t)},matches:function(t){return !!n.iteratee(t,this)(this.attributes)},set:function(t,e,r){if(null==t)return this;var i;if("object"==typeof t?(i=t,r=e):(i={})[t]=e,r||(r={}),!this._validate(i,r))return !1;var o=r.unset,a=r.silent,s=[],l=this._changing;this._changing=!0,l||(this._previousAttributes=n.clone(this.attributes),this.changed={});var c=this.attributes,u=this.changed,h=this._previousAttributes;for(var d in i)e=i[d],n.isEqual(c[d],e)||s.push(d),n.isEqual(h[d],e)?delete u[d]:u[d]=e,o?delete c[d]:c[d]=e;if(this.idAttribute in i&&(this.id=this.get(this.idAttribute)),!a){s.length&&(this._pending=r);for(var f=0;f<s.length;f++)this.trigger("change:"+s[f],this,c[s[f]],r);}if(l)return this;if(!a)for(;this._pending;)r=this._pending,this._pending=!1,this.trigger("change",this,r);return this._pending=!1,this._changing=!1,this},unset:function(t,e){return this.set(t,void 0,n.extend({},e,{unset:!0}))},clear:function(t){var e={};for(var r in this.attributes)e[r]=void 0;return this.set(e,n.extend({},t,{unset:!0}))},hasChanged:function(t){return null==t?!n.isEmpty(this.changed):n.has(this.changed,t)},changedAttributes:function(t){if(!t)return !!this.hasChanged()&&n.clone(this.changed);var e=this._changing?this._previousAttributes:this.attributes,r={};for(var i in t){var o=t[i];n.isEqual(e[i],o)||(r[i]=o);}return !!n.size(r)&&r},previous:function(t){return null!=t&&this._previousAttributes?this._previousAttributes[t]:null},previousAttributes:function(){return n.clone(this._previousAttributes)},fetch:function(t){t=n.extend({parse:!0},t);var e=this,r=t.success;return t.success=function(n){var i=t.parse?e.parse(n,t):n;if(!e.set(i,t))return !1;r&&r.call(t.context,e,n,t),e.trigger("sync",e,n,t);},_(this,t),this.sync("read",this,t)},save:function(t,e,r){var i;null==t||"object"==typeof t?(i=t,r=e):(i={})[t]=e;var o=(r=n.extend({validate:!0,parse:!0},r)).wait;if(i&&!o){if(!this.set(i,r))return !1}else if(!this._validate(i,r))return !1;var a=this,s=r.success,l=this.attributes;r.success=function(t){a.attributes=l;var e=r.parse?a.parse(t,r):t;if(o&&(e=n.extend({},i,e)),e&&!a.set(e,r))return !1;s&&s.call(r.context,a,t,r),a.trigger("sync",a,t,r);},_(this,r),i&&o&&(this.attributes=n.extend({},l,i));var c=this.isNew()?"create":r.patch?"patch":"update";"patch"!==c||r.attrs||(r.attrs=i);var u=this.sync(c,this,r);return this.attributes=l,u},destroy:function(t){t=t?n.clone(t):{};var e=this,r=t.success,i=t.wait,o=function(){e.stopListening(),e.trigger("destroy",e,e.collection,t);};t.success=function(n){i&&o(),r&&r.call(t.context,e,n,t),e.isNew()||e.trigger("sync",e,n,t);};var a=!1;return this.isNew()?n.defer(t.success):(_(this,t),a=this.sync("delete",this,t)),i||o(),a},url:function(){var t=n.result(this,"urlRoot")||n.result(this.collection,"url")||F();if(this.isNew())return t;var e=this.get(this.idAttribute);return t.replace(/[^\/]$/,"$&/")+encodeURIComponent(e)},parse:function(t,e){return t},clone:function(){return new this.constructor(this.attributes)},isNew:function(){return !this.has(this.idAttribute)},isValid:function(t){return this._validate({},n.extend({},t,{validate:!0}))},_validate:function(t,e){if(!e.validate||!this.validate)return !0;t=n.extend({},this.attributes,t);var r=this.validationError=this.validate(t,e)||null;return !r||(this.trigger("invalid",this,r,n.extend(e,{validationError:r})),!1)}}),a(y,{keys:1,values:1,pairs:1,invert:1,pick:0,omit:0,chain:1,isEmpty:1},"attributes");var b=e.Collection=function(t,e){e||(e={}),e.model&&(this.model=e.model),void 0!==e.comparator&&(this.comparator=e.comparator),this._reset(),this.initialize.apply(this,arguments),t&&this.reset(t,n.extend({silent:!0},e));},w={add:!0,remove:!0,merge:!0},x={add:!0,remove:!1},O=function(t,e,n){n=Math.min(Math.max(n,0),t.length);var r,i=Array(t.length-n),o=e.length;for(r=0;r<i.length;r++)i[r]=t[r+n];for(r=0;r<o;r++)t[r+n]=e[r];for(r=0;r<i.length;r++)t[r+o+n]=i[r];};n.extend(b.prototype,c,{model:y,initialize:function(){},toJSON:function(t){return this.map((function(e){return e.toJSON(t)}))},sync:function(){return e.sync.apply(this,arguments)},add:function(t,e){return this.set(t,n.extend({merge:!1},e,x))},remove:function(t,e){e=n.extend({},e);var r=!n.isArray(t);t=r?[t]:t.slice();var i=this._removeModels(t,e);return !e.silent&&i.length&&(e.changes={added:[],merged:[],removed:i},this.trigger("update",this,e)),r?i[0]:i},set:function(t,e){if(null!=t){(e=n.extend({},w,e)).parse&&!this._isModel(t)&&(t=this.parse(t,e)||[]);var r=!n.isArray(t);t=r?[t]:t.slice();var i=e.at;null!=i&&(i=+i),i>this.length&&(i=this.length),i<0&&(i+=this.length+1);var o,a,s=[],l=[],c=[],u=[],h={},d=e.add,f=e.merge,p=e.remove,g=!1,v=this.comparator&&null==i&&!1!==e.sort,m=n.isString(this.comparator)?this.comparator:null;for(a=0;a<t.length;a++){o=t[a];var y=this.get(o);if(y){if(f&&o!==y){var b=this._isModel(o)?o.attributes:o;e.parse&&(b=y.parse(b,e)),y.set(b,e),c.push(y),v&&!g&&(g=y.hasChanged(m));}h[y.cid]||(h[y.cid]=!0,s.push(y)),t[a]=y;}else d&&(o=t[a]=this._prepareModel(o,e))&&(l.push(o),this._addReference(o,e),h[o.cid]=!0,s.push(o));}if(p){for(a=0;a<this.length;a++)h[(o=this.models[a]).cid]||u.push(o);u.length&&this._removeModels(u,e);}var x=!1,C=!v&&d&&p;if(s.length&&C?(x=this.length!==s.length||n.some(this.models,(function(t,e){return t!==s[e]})),this.models.length=0,O(this.models,s,0),this.length=this.models.length):l.length&&(v&&(g=!0),O(this.models,l,null==i?this.length:i),this.length=this.models.length),g&&this.sort({silent:!0}),!e.silent){for(a=0;a<l.length;a++)null!=i&&(e.index=i+a),(o=l[a]).trigger("add",o,this,e);(g||x)&&this.trigger("sort",this,e),(l.length||u.length||c.length)&&(e.changes={added:l,removed:u,merged:c},this.trigger("update",this,e));}return r?t[0]:t}},reset:function(t,e){e=e?n.clone(e):{};for(var r=0;r<this.models.length;r++)this._removeReference(this.models[r],e);return e.previousModels=this.models,this._reset(),t=this.add(t,n.extend({silent:!0},e)),e.silent||this.trigger("reset",this,e),t},push:function(t,e){return this.add(t,n.extend({at:this.length},e))},pop:function(t){var e=this.at(this.length-1);return this.remove(e,t)},unshift:function(t,e){return this.add(t,n.extend({at:0},e))},shift:function(t){var e=this.at(0);return this.remove(e,t)},slice:function(){return o.apply(this.models,arguments)},get:function(t){if(null!=t)return this._byId[t]||this._byId[this.modelId(t.attributes||t)]||t.cid&&this._byId[t.cid]},has:function(t){return null!=this.get(t)},at:function(t){return t<0&&(t+=this.length),this.models[t]},where:function(t,e){return this[e?"find":"filter"](t)},findWhere:function(t){return this.where(t,!0)},sort:function(t){var e=this.comparator;if(!e)throw new Error("Cannot sort a set without a comparator");t||(t={});var r=e.length;return n.isFunction(e)&&(e=n.bind(e,this)),1===r||n.isString(e)?this.models=this.sortBy(e):this.models.sort(e),t.silent||this.trigger("sort",this,t),this},pluck:function(t){return this.map(t+"")},fetch:function(t){var e=(t=n.extend({parse:!0},t)).success,r=this;return t.success=function(n){var i=t.reset?"reset":"set";r[i](n,t),e&&e.call(t.context,r,n,t),r.trigger("sync",r,n,t);},_(this,t),this.sync("read",this,t)},create:function(t,e){var r=(e=e?n.clone(e):{}).wait;if(!(t=this._prepareModel(t,e)))return !1;r||this.add(t,e);var i=this,o=e.success;return e.success=function(t,e,n){r&&i.add(t,n),o&&o.call(n.context,t,e,n);},t.save(null,e),t},parse:function(t,e){return t},clone:function(){return new this.constructor(this.models,{model:this.model,comparator:this.comparator})},modelId:function(t){return t[this.model.prototype.idAttribute||"id"]},_reset:function(){this.length=0,this.models=[],this._byId={};},_prepareModel:function(t,e){if(this._isModel(t))return t.collection||(t.collection=this),t;(e=e?n.clone(e):{}).collection=this;var r=new this.model(t,e);return r.validationError?(this.trigger("invalid",this,r.validationError,e),!1):r},_removeModels:function(t,e){for(var n=[],r=0;r<t.length;r++){var i=this.get(t[r]);if(i){var o=this.indexOf(i);this.models.splice(o,1),this.length--,delete this._byId[i.cid];var a=this.modelId(i.attributes);null!=a&&delete this._byId[a],e.silent||(e.index=o,i.trigger("remove",i,this,e)),n.push(i),this._removeReference(i,e);}}return n},_isModel:function(t){return t instanceof y},_addReference:function(t,e){this._byId[t.cid]=t;var n=this.modelId(t.attributes);null!=n&&(this._byId[n]=t),t.on("all",this._onModelEvent,this);},_removeReference:function(t,e){delete this._byId[t.cid];var n=this.modelId(t.attributes);null!=n&&delete this._byId[n],this===t.collection&&delete t.collection,t.off("all",this._onModelEvent,this);},_onModelEvent:function(t,e,n,r){if(e){if(("add"===t||"remove"===t)&&n!==this)return;if("destroy"===t&&this.remove(e,r),"change"===t){var i=this.modelId(e.previousAttributes()),o=this.modelId(e.attributes);i!==o&&(null!=i&&delete this._byId[i],null!=o&&(this._byId[o]=e));}}this.trigger.apply(this,arguments);}}),a(b,{forEach:3,each:3,map:3,collect:3,reduce:0,foldl:0,inject:0,reduceRight:0,foldr:0,find:3,detect:3,filter:3,select:3,reject:3,every:3,all:3,some:3,any:3,include:3,includes:3,contains:3,invoke:0,max:3,min:3,toArray:1,size:1,first:3,head:3,take:3,initial:3,rest:3,tail:3,drop:3,last:3,without:0,difference:0,indexOf:3,shuffle:1,lastIndexOf:3,isEmpty:1,chain:1,sample:3,partition:3,groupBy:3,countBy:3,sortBy:3,indexBy:3,findIndex:3,findLastIndex:3},"models");var C=e.View=function(t){this.cid=n.uniqueId("view"),n.extend(this,n.pick(t,k)),this._ensureElement(),this.initialize.apply(this,arguments);},S=/^(\S+)\s*(.*)$/,k=["model","collection","el","id","attributes","className","tagName","events"];n.extend(C.prototype,c,{tagName:"div",$:function(t){return this.$el.find(t)},initialize:function(){},render:function(){return this},remove:function(){return this._removeElement(),this.stopListening(),this},_removeElement:function(){this.$el.remove();},setElement:function(t){return this.undelegateEvents(),this._setElement(t),this.delegateEvents(),this},_setElement:function(t){this.$el=t instanceof e.$?t:e.$(t),this.el=this.$el[0];},delegateEvents:function(t){if(t||(t=n.result(this,"events")),!t)return this;for(var e in this.undelegateEvents(),t){var r=t[e];if(n.isFunction(r)||(r=this[r]),r){var i=e.match(S);this.delegate(i[1],i[2],n.bind(r,this));}}return this},delegate:function(t,e,n){return this.$el.on(t+".delegateEvents"+this.cid,e,n),this},undelegateEvents:function(){return this.$el&&this.$el.off(".delegateEvents"+this.cid),this},undelegate:function(t,e,n){return this.$el.off(t+".delegateEvents"+this.cid,e,n),this},_createElement:function(t){return document.createElement(t)},_ensureElement:function(){if(this.el)this.setElement(n.result(this,"el"));else {var t=n.extend({},n.result(this,"attributes"));this.id&&(t.id=n.result(this,"id")),this.className&&(t.class=n.result(this,"className")),this.setElement(this._createElement(n.result(this,"tagName"))),this._setAttributes(t);}},_setAttributes:function(t){this.$el.attr(t);}}),e.sync=function(t,r,i){var o=T[t];n.defaults(i||(i={}),{emulateHTTP:e.emulateHTTP,emulateJSON:e.emulateJSON});var a={type:o,dataType:"json"};if(i.url||(a.url=n.result(r,"url")||F()),null!=i.data||!r||"create"!==t&&"update"!==t&&"patch"!==t||(a.contentType="application/json",a.data=JSON.stringify(i.attrs||r.toJSON(i))),i.emulateJSON&&(a.contentType="application/x-www-form-urlencoded",a.data=a.data?{model:a.data}:{}),i.emulateHTTP&&("PUT"===o||"DELETE"===o||"PATCH"===o)){a.type="POST",i.emulateJSON&&(a.data._method=o);var s=i.beforeSend;i.beforeSend=function(t){if(t.setRequestHeader("X-HTTP-Method-Override",o),s)return s.apply(this,arguments)};}"GET"===a.type||i.emulateJSON||(a.processData=!1);var l=i.error;i.error=function(t,e,n){i.textStatus=e,i.errorThrown=n,l&&l.call(i.context,t,e,n);};var c=i.xhr=e.ajax(n.extend(a,i));return r.trigger("request",r,c,i),c};var T={create:"POST",update:"PUT",patch:"PATCH",delete:"DELETE",read:"GET"};e.ajax=function(){return e.$.ajax.apply(e.$,arguments)};var j=e.Router=function(t){t||(t={}),t.routes&&(this.routes=t.routes),this._bindRoutes(),this.initialize.apply(this,arguments);},P=/\((.*?)\)/g,E=/(\(\?)?:\w+/g,M=/\*\w+/g,D=/[\-{}\[\]+?.,\\\^$|#\s]/g;n.extend(j.prototype,c,{initialize:function(){},route:function(t,r,i){n.isRegExp(t)||(t=this._routeToRegExp(t)),n.isFunction(r)&&(i=r,r=""),i||(i=this[r]);var o=this;return e.history.route(t,(function(n){var a=o._extractParameters(t,n);!1!==o.execute(i,a,r)&&(o.trigger.apply(o,["route:"+r].concat(a)),o.trigger("route",r,a),e.history.trigger("route",o,r,a));})),this},execute:function(t,e,n){t&&t.apply(this,e);},navigate:function(t,n){return e.history.navigate(t,n),this},_bindRoutes:function(){if(this.routes){this.routes=n.result(this,"routes");for(var t,e=n.keys(this.routes);null!=(t=e.pop());)this.route(t,this.routes[t]);}},_routeToRegExp:function(t){return t=t.replace(D,"\\$&").replace(P,"(?:$1)?").replace(E,(function(t,e){return e?t:"([^/?]+)"})).replace(M,"([^?]*?)"),new RegExp("^"+t+"(?:\\?([\\s\\S]*))?$")},_extractParameters:function(t,e){var r=t.exec(e).slice(1);return n.map(r,(function(t,e){return e===r.length-1?t||null:t?decodeURIComponent(t):null}))}});var A=e.History=function(){this.handlers=[],this.checkUrl=n.bind(this.checkUrl,this),"undefined"!=typeof window&&(this.location=window.location,this.history=window.history);},L=/^[#\/]|\s+$/g,N=/^\/+|\/+$/g,I=/#.*$/;A.started=!1,n.extend(A.prototype,c,{interval:50,atRoot:function(){return this.location.pathname.replace(/[^\/]$/,"$&/")===this.root&&!this.getSearch()},matchRoot:function(){return this.decodeFragment(this.location.pathname).slice(0,this.root.length-1)+"/"===this.root},decodeFragment:function(t){return decodeURI(t.replace(/%25/g,"%2525"))},getSearch:function(){var t=this.location.href.replace(/#.*/,"").match(/\?.+/);return t?t[0]:""},getHash:function(t){var e=(t||this).location.href.match(/#(.*)$/);return e?e[1]:""},getPath:function(){var t=this.decodeFragment(this.location.pathname+this.getSearch()).slice(this.root.length-1);return "/"===t.charAt(0)?t.slice(1):t},getFragment:function(t){return null==t&&(t=this._usePushState||!this._wantsHashChange?this.getPath():this.getHash()),t.replace(L,"")},start:function(t){if(A.started)throw new Error("Backbone.history has already been started");if(A.started=!0,this.options=n.extend({root:"/"},this.options,t),this.root=this.options.root,this._wantsHashChange=!1!==this.options.hashChange,this._hasHashChange="onhashchange"in window&&(void 0===document.documentMode||document.documentMode>7),this._useHashChange=this._wantsHashChange&&this._hasHashChange,this._wantsPushState=!!this.options.pushState,this._hasPushState=!(!this.history||!this.history.pushState),this._usePushState=this._wantsPushState&&this._hasPushState,this.fragment=this.getFragment(),this.root=("/"+this.root+"/").replace(N,"/"),this._wantsHashChange&&this._wantsPushState){if(!this._hasPushState&&!this.atRoot()){var e=this.root.slice(0,-1)||"/";return this.location.replace(e+"#"+this.getPath()),!0}this._hasPushState&&this.atRoot()&&this.navigate(this.getHash(),{replace:!0});}if(!this._hasHashChange&&this._wantsHashChange&&!this._usePushState){this.iframe=document.createElement("iframe"),this.iframe.src="javascript:0",this.iframe.style.display="none",this.iframe.tabIndex=-1;var r=document.body,i=r.insertBefore(this.iframe,r.firstChild).contentWindow;i.document.open(),i.document.close(),i.location.hash="#"+this.fragment;}var o=window.addEventListener||function(t,e){return attachEvent("on"+t,e)};if(this._usePushState?o("popstate",this.checkUrl,!1):this._useHashChange&&!this.iframe?o("hashchange",this.checkUrl,!1):this._wantsHashChange&&(this._checkUrlInterval=setInterval(this.checkUrl,this.interval)),!this.options.silent)return this.loadUrl()},stop:function(){var t=window.removeEventListener||function(t,e){return detachEvent("on"+t,e)};this._usePushState?t("popstate",this.checkUrl,!1):this._useHashChange&&!this.iframe&&t("hashchange",this.checkUrl,!1),this.iframe&&(document.body.removeChild(this.iframe),this.iframe=null),this._checkUrlInterval&&clearInterval(this._checkUrlInterval),A.started=!1;},route:function(t,e){this.handlers.unshift({route:t,callback:e});},checkUrl:function(t){var e=this.getFragment();if(e===this.fragment&&this.iframe&&(e=this.getHash(this.iframe.contentWindow)),e===this.fragment)return !1;this.iframe&&this.navigate(e),this.loadUrl();},loadUrl:function(t){return !!this.matchRoot()&&(t=this.fragment=this.getFragment(t),n.some(this.handlers,(function(e){if(e.route.test(t))return e.callback(t),!0})))},navigate:function(t,e){if(!A.started)return !1;e&&!0!==e||(e={trigger:!!e}),t=this.getFragment(t||"");var n=this.root;""!==t&&"?"!==t.charAt(0)||(n=n.slice(0,-1)||"/");var r=n+t;if(t=this.decodeFragment(t.replace(I,"")),this.fragment!==t){if(this.fragment=t,this._usePushState)this.history[e.replace?"replaceState":"pushState"]({},document.title,r);else {if(!this._wantsHashChange)return this.location.assign(r);if(this._updateHash(this.location,t,e.replace),this.iframe&&t!==this.getHash(this.iframe.contentWindow)){var i=this.iframe.contentWindow;e.replace||(i.document.open(),i.document.close()),this._updateHash(i.location,t,e.replace);}}return e.trigger?this.loadUrl(t):void 0}},_updateHash:function(t,e,n){if(n){var r=t.href.replace(/(javascript:|#).*$/,"");t.replace(r+"#"+e);}else t.hash="#"+e;}}),e.history=new A,y.extend=b.extend=j.extend=C.extend=A.extend=function(t,e){var r,i=this;return r=t&&n.has(t,"constructor")?t.constructor:function(){return i.apply(this,arguments)},n.extend(r,i,e),r.prototype=n.create(i.prototype,t),r.prototype.constructor=r,r.__super__=i.prototype,r};var F=function(){throw new Error('A "url" property or function must be specified')},_=function(t,e){var n=e.error;e.error=function(r){n&&n.call(e.context,t,r,e),t.trigger("error",t,r,e);};};return e}(a,n,t,e);}.apply(e,i))||(t.exports=o);}).call(this,n(24));},function(t,e){t.exports=function(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t};},function(t,e,n){n.d(e,"o",(function(){return b})),n.d(e,"t",(function(){return u})),n.d(e,"s",(function(){return h})),n.d(e,"k",(function(){return v})),n.d(e,"q",(function(){return s})),n.d(e,"g",(function(){return w})),n.d(e,"d",(function(){return x})),n.d(e,"b",(function(){return p})),n.d(e,"p",(function(){return y})),n.d(e,"f",(function(){return S})),n.d(e,"m",(function(){return k})),n.d(e,"e",(function(){return m})),n.d(e,"v",(function(){return c})),n.d(e,"r",(function(){return g})),n.d(e,"h",(function(){return O})),n.d(e,"i",(function(){return d})),n.d(e,"c",(function(){return T})),n.d(e,"j",(function(){return E})),n.d(e,"u",(function(){return M})),n.d(e,"a",(function(){return l})),n.d(e,"l",(function(){return j})),n.d(e,"n",(function(){return P}));var r=n(6),i=n.n(r),o=n(0),a=window.Element.prototype,s=a.matches||a.webkitMatchesSelector||a.mozMatchesSelector||a.msMatchesSelector,l=function t(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=Object(o.isArray)(e)?i()(e):[e];if(r.length){var a=r.shift();if(a&&(!n.unique||!document.querySelector('link[href="'.concat(a,'"]')))){var s=document,l=s.head,c=document.createElement("link");c.href=a,c.rel="stylesheet",n.prepand?l.insertBefore(c,l.firstChild):l.appendChild(c);}t(r);}},c=function(t,e){var n={},r=Object(o.keys)(e);for(var i in t)if(t.hasOwnProperty(i)){var a=t[i],s=e[i];r.indexOf(i)>=0?a!==s&&(n[i]=s):n[i]=null;}for(var l in e)e.hasOwnProperty(l)&&Object(o.isUndefined)(t[l])&&(n[l]=e[l]);return n},u=function(t,e,n){e=e.split(/\s+/),t=t instanceof Array?t:[t];for(var r=function(r){t.forEach((function(t){return t.addEventListener(e[r],n)}));},i=0;i<e.length;++i)r(i);},h=function(t,e,n){e=e.split(/\s+/),t=t instanceof Array?t:[t];for(var r=function(r){t.forEach((function(t){return t.removeEventListener(e[r],n)}));},i=0;i<e.length;++i)r(i);},d=function(t){return t.replace(parseFloat(t),"")},f=function(t){return t[0].toUpperCase()+t.toLowerCase().slice(1)},p=function(t){var e=t.split("-").filter(String);return e[0].toLowerCase()+e.slice(1).map(f)},g=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0,r=0;if(isNaN(t))return n;if(t=parseFloat(t),Math.floor(t)!==t){var i=e.toString().split(".")[1];r=i?i.length:0;}return r?parseFloat(t.toFixed(r)):t},v=function(t){return "draggable"in document.createElement("i")&&(t?t.get("Config").nativeDnD:1)},m=function(t){return Object(o.isElement)(t)||y(t)?t:t&&t.getEl?t.getEl():void 0},y=function(t){return t&&3===t.nodeType},b=function(t){return t&&!y(t)&&!function(t){return t&&8===t.nodeType}(t)},w=function(t,e){var n=t;return Object(o.isElement)(t)&&(n=e(t).data("model")),n},x=function(t){var e,n={top:0,left:0,width:0,height:0};if(!t)return n;if(y(t)){var r=document.createRange();r.selectNode(t),e=r.getBoundingClientRect(),r.detach();}return e||(t.getBoundingClientRect?t.getBoundingClientRect():n)},O=function(t){return t.touches&&t.touches[0]?t.touches[0]:t},C=function(t){return t.which||t.keyCode},S=function(t){return String.fromCharCode(C(t))},k=function(t){return 27===C(t)},T=function(t){return t&&t.charAt(0).toUpperCase()+t.substring(1)},j=function(t){return t&&t.toHTML},P=function(t){return t&&t.toCSS},E=function(t){return t.__gjsv},M=function(t,e){t.__gjsv=e;};},function(t,e,n){n.r(e);var r=n(8),i=n.n(r),o=n(2),a=n.n(o),s=n(0),l=n(3),c=n(22),u=n(1),h=n.n(u),d=n(20),f=n(9),p=n(11);function g(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function v(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?g(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):g(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var m=h.a.Model.extend({defaults:{type:"text",label:"",name:"",min:"",max:"",unit:"",step:1,value:"",target:"",default:"",placeholder:"",changeProp:0,options:[]},initialize:function(){var t=this.get("target"),e=this.get("name"),n=this.get("changeProp");if(t){this.target=t,this.unset("target");var r=n?"change:".concat(e):"change:attributes:".concat(e);this.listenTo(t,r,this.targetUpdated);}},props:function(){return this.attributes},targetUpdated:function(){var t=this.getTargetValue();this.set({value:t},{fromTarget:1});},getTargetValue:function(){var t,e=this.get("name"),n=this.target;return t=this.get("changeProp")?n.get(e):n.getAttributes()[e],Object(s.isUndefined)(t)?"":t},setTargetValue:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.target,r=this.get("name");if(!Object(s.isUndefined)(t)){var i=t;if("false"===t?i=!1:"true"===t&&(i=!0),this.get("changeProp"))n.set(r,i,e);else {var o=v({},n.get("attributes"));o[r]=i,n.set("attributes",o,e);}}},setValueFromInput:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r={value:t};this.set(r,v({},n,{avoidStore:1})),e&&(this.set("value","",n),this.set(r,n));},getInitValue:function(){var t,e=this.target,n=this.get("name");if(e){var r=e.get("attributes");t=this.get("changeProp")?e.get(n):r[n];}return t||this.get("value")||this.get("default")}}),y=h.a.Collection.extend({model:m,initialize:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.em=e.em||"",this.listenTo(this,"add",this.handleAdd),this.listenTo(this,"reset",this.handleReset);},handleReset:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.previousModels,r=void 0===n?[]:n;r.forEach((function(t){return t.trigger("remove")}));},handleAdd:function(t){var e=this.target;e&&(t.target=e);},setTarget:function(t){this.target=t;},add:function(t,e){var n=this.em;if(Object(s.isString)(t)||Object(s.isArray)(t)){var r=n&&n.get&&n.get("TraitManager"),i=function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return {build:function(e){var n=[];"string"==typeof e&&(e=[e]);for(var r=0;r<e.length;r++){var i={},o=e[r];switch(i.name=o,o){case"target":i.type="select";}switch(o){case"target":i.options=t.optionsTarget;}n.push(i);}return n}}}(r&&r.getConfig());Object(s.isString)(t)&&(t=[t]);for(var o=0,a=t.length;o<a;o++){var l=t[o],c=Object(s.isString)(l)?i.build(l)[0]:l;c.target=this.target,t[o]=c;}}return h.a.Collection.prototype.add.apply(this,[t,e])}});function b(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function w(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?b(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):b(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}n.d(e,"eventDrag",(function(){return O}));var x=function(t){return t.replace(/[|\\{}()[\]^$+*?.]/g,"\\$&")},O="component:drag",C=h.a.Model.extend(c.a).extend({defaults:{tagName:"div",type:"",name:"",removable:!0,draggable:!0,droppable:!0,badgable:!0,stylable:!0,"stylable-require":"","style-signature":"",unstylable:"",highlightable:!0,copyable:!0,resizable:!1,editable:!1,layerable:!0,selectable:!0,hoverable:!0,void:!1,state:"",status:"",content:"",icon:"",style:"",classes:"",script:"","script-export":"",attributes:"",traits:["id","title"],propagate:"",dmode:"",toolbar:null},init:function(){},updated:function(t,e,n){},removed:function(){},initialize:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.em,i=this.parent(),o=i&&i.attributes;if(o&&o.propagate){var a={},l=o.propagate;l.forEach((function(t){return a[t]=i.get(t)})),a.propagate=l,a=w({},a,{},e),this.set(a);}var c=this.get("propagate");c&&this.set("propagate",Object(s.isArray)(c)?c:[c]),n&&n.config&&n.config.voidElements.indexOf(this.get("tagName"))>=0&&this.set("void",!0),n.em=r,this.opt=n,this.em=r,this.frame=n.frame,this.config=n.config||{},this.set("attributes",w({},this.defaults.attributes||{},{},this.get("attributes")||{})),this.ccid=C.createId(this),this.initClasses(),this.initTraits(),this.initComponents(),this.initToolbar(),this.listenTo(this,"change:script",this.scriptUpdated),this.listenTo(this,"change:tagName",this.tagUpdated),this.listenTo(this,"change:attributes",this.attrUpdated),this.listenTo(this,"change:attributes:id",this._idUpdated),this.set("status",""),this.views=[],["classes","traits","components"].forEach((function(e){var n="add remove ".concat("components"!==e?"change":"");t.listenTo(t.get(e),n.trim(),(function(){for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];return t.emitUpdate.apply(t,[e].concat(r))}));})),n.temporary||(this.init(),r&&r.trigger("component:create",this));},is:function(t){return !(this.get("type")!=t)},props:function(){return this.attributes},index:function(){var t=this.collection;return t&&t.indexOf(this)},setDragMode:function(t){return this.set("dmode",t)},find:function(t){var e=[],n=this.view.$el.find(t);return n.each((function(t){var r=n.eq(t).data("model");r&&e.push(r);})),e},findType:function(t){var e=[];return function n(r){return r.forEach((function(r){r.is(t)&&e.push(r),n(r.components());}))}(this.components()),e},closest:function(t){var e=this.view.$el.closest(t);return e.length&&e.data("model")},closestType:function(t){for(var e=this.parent();e&&!e.is(t);)e=e.parent();return e},tagUpdated:function(){var t=this.collection,e=t.indexOf(this);t.remove(this),t.add(this,{at:e});},replaceWith:function(t){var e=this.collection,n=e.indexOf(this);return e.remove(this),e.add(t,{at:n})},attrUpdated:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=this.get("attributes"),o=i.class;o&&this.setClass(o),delete i.class;var a=i.style;a&&this.setStyle(a),delete i.style;var c=w({},this.previous("attributes")),u=Object(l.v)(c,this.get("attributes"));Object(s.keys)(u).forEach((function(t){return n.trigger("change:attributes:".concat(t),n,u[t],r)}));},setAttributes:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return this.set("attributes",w({},t),e),this},addAttributes:function(t){var e=w({},this.getAttributes(),{},t);return this.setAttributes(e),this},getStyle:function(){var t=this.em;if(t&&t.getConfig("avoidInlineStyle")){var e=t.get("state"),n=t.get("CssComposer").getIdRule(this.getId(),{state:e});if(this.rule=n,n)return n.getStyle()}return c.a.getStyle.call(this)},setStyle:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=this.em,i=this.opt;if(r&&r.getConfig("avoidInlineStyle")&&!i.temporary){var o=this.get("style")||{};e=w({},e=Object(s.isString)(e)?this.parseStyle(e):e,{},o);var a=r.get("state"),u=r.get("CssComposer"),h=this.getStyle();this.rule=u.setIdRule(this.getId(),e,w({},n,{state:a}));var d=Object(l.v)(h,e);this.set("style",{},{silent:1}),Object(s.keys)(d).forEach((function(e){return t.trigger("change:style:".concat(e))}));}else e=c.a.setStyle.apply(this,arguments);return e},getAttributes:function(){var t,e=this.em,n=[],r=w({},this.get("attributes")),i=e&&e.get("SelectorManager"),o=this.getId();(this.get("classes").forEach((function(t){return n.push(Object(s.isString)(t)?t:t.get("name"))})),n.length&&(r.class=n.join(" ")),Object(s.has)(r,"id"))||(!function(t){return t&&t.getConfig("avoidInlineStyle")}(e)?Object(s.isEmpty)(this.getStyle())||(t=1):t=i&&i.get(o,i.Selector.TYPE_ID),t&&(r.id=this.getId()));return r},addClass:function(t){var e=this.em.get("SelectorManager").addClass(t);return this.get("classes").add(e)},setClass:function(t){return this.get("classes").reset(),this.addClass(t)},removeClass:function(t){var e=[];t=Object(s.isArray)(t)?t:[t];var n=this.get("classes"),r=f.a.TYPE_CLASS;return t.forEach((function(t){t.split(" ").forEach((function(t){var i=n.where({name:t,type:r})[0];i&&e.push(n.remove(i));}));})),e},getClasses:function(){var t=this.getAttributes().class;return t?t.split(" "):[]},initClasses:function(){var t=[this,"change:classes",this.initClasses],e=this.get("classes")||[],n=Object(s.isString)(e)?e.split(" "):e;this.stopListening.apply(this,t);var r=this.normalizeClasses(n),i=new p.a([]);return this.set("classes",i),i.add(r),this.listenTo.apply(this,t),this},initComponents:function(){var t=[this,"change:components",this.initComponents];this.stopListening.apply(this,t);var e=new d.a(null,this.opt);e.parent=this;var n=this.get("components"),r=!this.opt.avoidChildren;return this.set("components",e),r&&e.add(Object(s.isFunction)(n)?n(this):n),this.listenTo.apply(this,t),this},initTraits:function(t){var e=this.em,n=[this,"change:traits",this.initTraits];this.stopListening.apply(this,n),this.loadTraits();var r=w({},this.get("attributes")),i=this.get("traits");return i.each((function(t){if(!t.get("changeProp")){var e=t.get("name"),n=t.getInitValue();e&&n&&(r[e]=n);}})),i.length&&this.set("attributes",r),this.listenTo.apply(this,n),t&&e&&e.trigger("component:toggled"),this},append:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.components().add(t,e);return Object(s.isArray)(n)?n:[n]},components:function(t){var e=this.get("components");return Object(s.isUndefined)(t)?e:(e.reset(),t&&this.append(t))},empty:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return this.components().reset(null,t),this},parent:function(){var t=this.collection;return t&&t.parent},scriptUpdated:function(){this.set("scriptUpdated",1);},initToolbar:function(){var t=this.em,e=t&&t.getConfig("stylePrefix")||"";if(!this.get("toolbar")){var n=[];this.collection&&n.push({attributes:{class:"fa fa-arrow-up"},command:function(t){return t.runCommand("core:component-exit",{force:1})}}),this.get("draggable")&&n.push({attributes:{class:"fa fa-arrows ".concat(e,"no-touch-actions"),draggable:!0},command:"tlb-move"}),this.get("copyable")&&n.push({attributes:{class:"fa fa-clone"},command:"tlb-clone"}),this.get("removable")&&n.push({attributes:{class:"fa fa-trash-o"},command:"tlb-delete"}),this.set("toolbar",n);}},loadTraits:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(t=t||this.get("traits"),!((t=Object(s.isFunction)(t)?t(this):t)instanceof y)){var n=new y([],this.opt);n.setTarget(this),t.length&&(t.forEach((function(t){return t.attributes&&delete t.attributes.value})),n.add(t)),this.set("traits",n,e);}return this},getTrait:function(t){return this.get("traits").filter((function(e){return e.get("id")===t||e.get("name")===t}))[0]},updateTrait:function(t,e){var n=this.em,r=this.getTrait(t);return r&&r.set(e),n&&n.trigger("component:toggled"),this},getTraitIndex:function(t){var e=this.getTrait(t);return e?this.get("traits").indexOf(e):e},removeTrait:function(t){var e=this,n=this.em,r=(Object(s.isArray)(t)?t:[t]).map((function(t){return e.getTrait(t)})),i=this.get("traits").remove(r);return n&&n.trigger("component:toggled"),i},addTrait:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.em,r=this.get("traits").add(t,e);return n&&n.trigger("component:toggled"),r},normalizeClasses:function(t){var e=[],n=this.em;if(n){var r=n.get("SelectorManager");if(r)return t.forEach((function(t){var n="";n="string"==typeof t?t:t.name;var i=r.add(n);e.push(i);})),e}},clone:function(){var t=this.em,e=this.getStyle(),n=w({},this.attributes),r=w({},this.opt);n.attributes=w({},n.attributes),delete n.attributes.id,n.components=[],n.classes=[],n.traits=[],this.get("components").each((function(t,e){n.components[e]=t.clone();})),this.get("traits").each((function(t,e){n.traits[e]=t.clone();})),this.get("classes").each((function(t,e){n.classes[e]=t.get("name");})),n.status="",n.view="",r.collection=null,t&&t.getConfig("avoidInlineStyle")&&!Object(s.isEmpty)(e)&&(n.style=e);var i=new this.constructor(n,r);return t&&t.trigger("component:clone",i),this.trigger("component:clone",i),i},getName:function(){var t=this.em,e=this.attributes,n=e.type,r=e.tagName,i=this.get("name"),o="div"==r,a=n||(o?"box":r),s=!n&&r&&!o&&r,c="domComponents.names.",u=i&&t&&t.t("".concat(c).concat(i)),h=s&&t&&t.t("".concat(c).concat(s)),d=t&&(t.t("".concat(c).concat(n))||t.t("".concat(c).concat(r)));return this.get("custom-name")||u||i||h||Object(l.c)(s)||d||Object(l.c)(a)},getIcon:function(){var t=this.get("icon");return t?t+" ":""},toHTML:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this,n=[],r=t.tag,i=r||e.get("tagName"),o=e.get("void"),a=t.attributes,l=this.getAttrToHTML();for(var c in delete t.tag,a&&(Object(s.isFunction)(a)?l=a(e,l)||{}:Object(s.isObject)(a)&&(l=a)),l){var u=l[c],h=Object(s.isString)(u)?u.replace(/"/g,"&quot;"):u;Object(s.isUndefined)(h)||(Object(s.isBoolean)(h)?h&&n.push(c):n.push("".concat(c,'="').concat(h,'"')));}var d=n.length?" ".concat(n.join(" ")):"",f="<".concat(i).concat(d).concat(o?"/":"",">").concat(e.get("content"));return e.get("components").each((function(e){return f+=e.toHTML(t)})),!o&&(f+="</".concat(i,">")),f},getAttrToHTML:function(){var t=this.getAttributes();return delete t.style,t},toJSON:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=h.a.Model.prototype.toJSON.apply(this,e);if(r.attributes=this.getAttributes(),delete r.attributes.class,delete r.toolbar,delete r.traits,this.em.getConfig("avoidDefaults")){var i=Object(s.result)(this,"defaults");Object(s.forEach)(i,(function(t,e){-1===["type","content"].indexOf(e)&&r[e]===t&&delete r[e];})),Object(s.isEmpty)(r.type)&&delete r.type,Object(s.forEach)(["attributes","style"],(function(t){Object(s.isEmpty)(i[t])&&Object(s.isEmpty)(r[t])&&delete r[t];})),Object(s.forEach)(["classes","components"],(function(t){Object(s.isEmpty)(i[t])&&!r[t].length&&delete r[t];}));}return r},getId:function(){return (this.get("attributes")||{}).id||this.ccid||this.cid},setId:function(t,e){var n=w({},this.get("attributes"));return n.id=t,this.set("attributes",n,e),this},getEl:function(t){var e=this.getView(t);return e&&e.el},getView:function(t){var e=this.view,n=this.views;return t&&(e=n.filter((function(e){return e._getFrame()===t.view}))[0]),e},getCurrentView:function(){var t=(this.em.get("currentFrame")||{}).model;return this.getView(t)},getScriptString:function(t){var e=this,n=t||this.get("script");if(!n)return n;if("function"==typeof n){var r=n.toString().trim();n=(r=r.replace(/^function[\s\w]*\(\)\s?\{/,"").replace(/\}$/,"")).trim();}var o=this.em.getConfig(),a=x(o.tagVarStart||"{[ "),l=x(o.tagVarEnd||" ]}"),c=new RegExp("".concat(a,"([\\w\\d-]*)").concat(l),"g");return n=n.replace(c,(function(t,n){e.scriptUpdated();var r=e.attributes[n]||"";return Object(s.isArray)(r)||"object"==i()(r)?JSON.stringify(r):r}))},emitUpdate:function(t){for(var e=this.em,n="component:update"+(t?":".concat(t):""),r=arguments.length,i=new Array(r>1?r-1:0),o=1;o<r;o++)i[o-1]=arguments[o];t&&this.updated.apply(this,[t,t&&this.get(t),t&&this.previous(t)].concat(i)),this.trigger.apply(this,[n].concat(i)),e&&e.trigger.apply(e,[n,this].concat(i));},onAll:function(t){return Object(s.isFunction)(t)&&(t(this),this.components().forEach((function(e){return e.onAll(t)}))),this},remove:function(){var t=this.collection;return t&&t.remove(this)},resetId:function(){var t=this.em,e=this.getId();if(e){var n=C.createId(this);this.setId(n);var r=t&&t.get("CssComposer").getIdRule(e),i=r&&r.get("selectors").at(0);return i&&i.set("name",n),this}},_getStyleRule:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.id,n=this.em,r=e||this.getId();return n&&n.get("CssComposer").getIdRule(r)},_getStyleSelector:function(t){var e=this._getStyleRule(t);return e&&e.get("selectors").at(0)},_idUpdated:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(!n.idUpdate){var r=this.ccid,i=this.get("attributes")||{},o=i.id,a=(this.previous("attributes")||{}).id||r,s=C.getList(this);if(s[o])return this.setId(a,{idUpdate:1});delete s[a],s[o]=this,this.ccid=o;var l=this._getStyleSelector({id:a});l&&l.set({name:o,label:o});}}},{isComponent:function(t){return {tagName:t.tagName?t.tagName.toLowerCase():""}},ensureInList:function(t){var e=C.getList(t),n=t.getId(),r=e[n];if(r){if(r!==t){var i=C.getIncrementId(n,e);t.setId(i),e[i]=t;}}else e[n]=t;t.components().forEach((function(t){return C.ensureInList(t)}));},createId:function(t){var e,n=C.getList(t),r=t.get("attributes").id;return r?(e=C.getIncrementId(r,n),t.setId(e)):e=C.getNewId(n),n[e]=t,e},getNewId:function(t){for(var e=Object.keys(t).length.toString().length+2,n=(Math.random()+1.1).toString(36).slice(-e),r="i".concat(n);t[r];)r=C.getNewId(t);return r},getIncrementId:function(t,e){for(var n=1,r=t;e[r];)n++,r="".concat(t,"-").concat(n);return r},getList:function(t){var e=t.opt&&t.opt.domc;return e?e.componentsById:{}},checkId:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[],n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=Object(s.isArray)(t)?t:[t];r.forEach((function(t){var r=t.attributes,i=void 0===r?{}:r,o=t.components,a=i.id;if(a&&n[a]){var l=C.getIncrementId(a,n);i.id=l,Object(s.isArray)(e)&&e.forEach((function(t){var e=t.selectors;e.forEach((function(t,n){t==="#".concat(a)&&(e[n]="#".concat(l));}));}));}o&&C.checkId(o,e,n);}));}});e.default=C;},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(1),a=n.n(o),s=n(0),l=n(20),c=n(34),u=n(11),h=n(7),d=n(3);function f(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function p(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?f(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):f(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=a.a.View.extend({className:function(){return this.getClasses()},tagName:function(){return this.model.get("tagName")},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model,n=t.config||{},r=n.em,i=e.opt||{},o=this.$el,a=this.el,s=n.draggableComponents;this.opts=t,this.modelOpt=i,this.config=n,this.em=r||"",this.pfx=n.stylePrefix||"",this.ppfx=n.pStylePrefix||"",this.attr=e.get("attributes"),this.classe=this.attr.class||[],this.listenTo(e,"change:style",this.updateStyle),this.listenTo(e,"change:attributes change:_innertext",this.renderAttributes),this.listenTo(e,"change:highlightable",this.updateHighlight),this.listenTo(e,"change:status",this.updateStatus),this.listenTo(e,"change:script",this.reset),this.listenTo(e,"change:content",this.updateContent),this.listenTo(e,"change",this.handleChange),this.listenTo(e,"active",this.onActive),this.listenTo(e,"disable",this.onDisable),o.data("model",e),Object(d.u)(a,this),e.view=this,this._getFrame()&&e.views.push(this),this.initClasses(),this.initComponents({avoidRender:1}),this.events=p({},this.events,{},s&&{dragstart:"handleDragStart"}),this.delegateEvents(),!i.temporary&&this.init(this._clbObj());},_clbObj:function(){var t=this.em,e=this.model,n=this.el;return {editor:t&&t.getEditor(),model:e,el:n}},init:function(){},removed:function(){},onActive:function(){},onDisable:function(){},remove:function(){var t=this;a.a.View.prototype.remove.apply(t,arguments);var e=t.model,n=t._getFrame()||{},r=n.model;e.components().forEach((function(t){var e=t.getView(r);e&&e.remove();}));var i=e.views;return i.splice(i.indexOf(t),1),t.removed(t._clbObj()),t.$el.data({model:"",collection:"",view:""}),delete t.model,delete t.$el,delete t.el.__gjsv,delete t.childrenView,delete t.scriptContainer,delete t.opts,t},handleDragStart:function(t){t.preventDefault(),t.stopPropagation(),this.em.get("Commands").run("tlb-move",{target:this.model,event:t});},initClasses:function(){var t=this.model,e=t.get("classes");e instanceof u.a&&(this.stopListening(t,"change:classes",this.initClasses),this.listenTo(t,"change:classes",this.initClasses),this.listenTo(e,"add remove change",this.updateClasses),e.length&&this.importClasses());},initComponents:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model,n=this.$el,r=this.childrenView,i="change:components",o=e.get("components"),a=[e,i,this.initComponents];o instanceof l.a&&(n.data("collection",o),r&&r.remove(),this.stopListening.apply(this,a),!t.avoidRender&&this.renderChildren(),this.listenTo.apply(this,a));},handleChange:function(){var t=this.model,e=Object(s.keys)(t.changed);if(1!==e.length||"status"!==e[0])for(var n in t.emitUpdate(),t.changed)t.emitUpdate(n);},importClasses:function(){var t=this.config.em.get("SelectorManager");t&&this.model.get("classes").each((function(e){t.add(e.get("name"));}));},updateStatus:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=(this.em,this.el),n=this.model.get("status"),r=(this.pfx,this.ppfx),i="".concat(r,"selected"),o="".concat(i,"-parent"),a="".concat(r,"freezed"),s="".concat(r,"hovered"),l=[i,o,a,s];this.$el.removeClass(l.join(" "));var c=e.getAttribute("class")||"",u="";switch(n){case"selected":u="".concat(c," ").concat(i);break;case"selected-parent":u="".concat(c," ").concat(o);break;case"freezed":u="".concat(c," ").concat(a);break;case"freezed-selected":u="".concat(c," ").concat(a," ").concat(i);break;case"hovered":u=t.avoidHover?"":"".concat(c," ").concat(s);}(u=u.trim())&&e.setAttribute("class",u);},updateHighlight:function(){var t=this.model.get("highlightable");this.setAttribute("data-highlightable",t?1:"");},updateStyle:function(){var t=this.model,e=this.em,n=this.el;if(e&&e.getConfig("avoidInlineStyle")){t.get("_innertext")?n.removeAttribute("id"):n.id=t.getId();var r=t.getStyle();!Object(s.isEmpty)(r)&&t.setStyle(r);}else this.setAttribute("style",t.styleToString());},updateClasses:function(){var t=this.model.get("classes").pluck("name").join(" ");this.setAttribute("class",t),this.updateStatus();},setAttribute:function(t,e){var n=this.$el;e?n.attr(t,e):n.removeAttr(t);},getClasses:function(){return this.model.getClasses().join(" ")},updateAttributes:function(){var t=[],e=this.model,n=this.$el,r=this.el,i=this.config,o=e.attributes,a=o.highlightable,l=o.textable,c=o.type,u=o._innertext,h=p({"data-gjs-type":c||"default"},i.draggableComponents&&!u?{draggable:!0}:{},{},a?{"data-highlightable":1}:{},{},l?{contenteditable:"false","data-gjs-textable":"true"}:{});Object(s.each)(r.attributes,(function(e){return t.push(e.nodeName)})),t.forEach((function(t){return n.removeAttr(t)}));var d=p({},h,{},e.getAttributes());Object(s.keys)(d).forEach((function(t){return !1===d[t]&&delete d[t]})),n.attr(d),this.updateStyle();},updateContent:function(){this.getChildrenContainer().innerHTML=this.model.get("content");},prevDef:function(t){t.preventDefault();},updateScript:function(){var t=this.model,e=this.em;t.get("script")&&e&&e.get("Canvas").getCanvasView().updateScript(this);},getChildrenContainer:function(){var t=this.el;return "function"==typeof this.getChildrenSelector?t=this.el.querySelector(this.getChildrenSelector()):this.getTemplate,t},getOffsetRect:function(){var t={},e=this.el,n=0,r=0;return function i(o){var a=o.offsetParent;a?(n+=a.offsetTop,r+=a.offsetLeft,i(a)):(t.top=e.offsetTop+n,t.left=e.offsetLeft+r,t.bottom=t.top+e.offsetHeight,t.right=t.left+e.offsetWidth);}(e),t},isInViewport:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.rect,n=this.el,r=n.ownerDocument,i=r.body,o=r.defaultView.frameElement,a=e||this.getOffsetRect(),s=a.top,l=a.left,c=this._getFrame().getOffsetRect();return s>=c.scrollTop&&l>=c.scrollLeft&&s<=c.scrollBottom&&l<=o.offsetWidth+i.scrollLeft},scrollIntoView:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.getOffsetRect(),n=this.isInViewport({rect:e});if(!n||t.force){var r=this.el;"smooth"!==t.behavior?r.ownerDocument.defaultView.scrollTo(0,e.top):r.scrollIntoView(p({behavior:"smooth",block:"nearest"},t));}},reset:function(){var t=this.el;this.el="",this._ensureElement(),this._setData(),Object(h.h)(t,this.el),this.render();},_setData:function(){var t=this.model,e=t.components();this.$el.data({model:t,collection:e,view:this});},_getFrame:function(){return this.config.frameView},renderChildren:function(){this.updateContent();var t=this.getChildrenContainer(),e=this.childrenView||new c.a({collection:this.model.get("components"),config:this.config,componentTypes:this.opts.componentTypes});e.render(t),this.childrenView=e;for(var n=Array.prototype.slice.call(e.el.childNodes),r=0,i=n.length;r<i;r++)t.appendChild(n.shift());},renderAttributes:function(){this.updateAttributes(),this.updateClasses();},render:function(){return this.renderAttributes(),this.modelOpt.temporary?this:(this.renderChildren(),this.updateScript(),Object(d.u)(this.el,this),this.postRender(),this)},postRender:function(){var t=this.em,e=this.model;this.modelOpt.temporary||(this.onRender(this._clbObj()),t&&t.trigger("component:mount",e));},onRender:function(){}});},function(t,e,n){var r=n(71),i=n(72),o=n(73);t.exports=function(t){return r(t)||i(t)||o()};},function(t,e,n){n.d(e,"g",(function(){return i})),n.d(e,"f",(function(){return o})),n.d(e,"h",(function(){return a})),n.d(e,"b",(function(){return s})),n.d(e,"a",(function(){return l})),n.d(e,"e",(function(){return c})),n.d(e,"d",(function(){return u})),n.d(e,"c",(function(){return h}));var r=n(0),i="transitionend oTransitionEnd transitionend webkitTransitionEnd",o=function(t){for(;t.firstChild;)t.removeChild(t.firstChild);},a=function(t,e){t.parentNode.replaceChild(e,t);},s=function(t,e,n){var i=t.childNodes,o=i.length,a=Object(r.isUndefined)(n)?o:n;Object(r.isString)(e)&&(t.insertAdjacentHTML("beforeEnd",e),e=t.lastChild,t.removeChild(e)),a>=o?t.appendChild(e):t.insertBefore(e,i[a]);},l=function(t,e){return s(t,e)},c=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=arguments.length>2?arguments[2]:void 0,i=document.createElement(t);return e&&Object(r.each)(e,(function(t,e){return i.setAttribute(e,t)})),n&&(Object(r.isString)(n)?i.innerHTML=n:i.appendChild(n)),i},u=function(t,e){var n;try{n=new window[e](t.type,t);}catch(t){(n=document.createEvent(e)).initEvent(t.type,!0,!0);}return n.keyCodeVal=t.keyCode,n._parentEvent=t,["keyCode","which"].forEach((function(t){Object.defineProperty(n,t,{get:function(){return this.keyCodeVal}});})),n},h=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:[],n=Array.isArray(e)?e:[e];n.forEach((function(e){var n=e.tag||"div",i=e.attributes||{},o=document.createElement(n);Object(r.each)(i,(function(t,e){o.setAttribute(e,t);})),t.appendChild(o);}));};},function(t,e){function n(t){return (n="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t})(t)}function r(e){return "function"==typeof Symbol&&"symbol"===n(Symbol.iterator)?t.exports=r=function(t){return n(t)}:t.exports=r=function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":n(t)},r(e)}t.exports=r;},function(t,e,n){var r=n(1),i=n.n(r).a.Model.extend({idAttribute:"name",defaults:{name:"",label:"",type:1,active:!0,private:!1,protected:!1},initialize:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.config,r=void 0===n?{}:n,o=this.get("name"),a=this.get("label");o?a||this.set("label",o):this.set("name",a);var s=this.get("name"),l=r.escapeName,c=l?l(s):i.escapeName(s);this.set("name",c);},getFullName:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.escape,n=this.get("name"),r="";switch(this.get("type")){case 1:r=".";break;case 2:r="#";}return r+(e?e(n):n)}},{TYPE_CLASS:1,TYPE_ID:2,escapeName:function(t){return "".concat(t).trim().replace(/([^a-z0-9\w-\:]+)/gi,"-")}});e.a=i;},function(t,e,n){n.r(e);var r=document,i=window,o=Array.prototype,a=o.filter,s=o.indexOf,l=o.map,c=o.push,u=o.reverse,h=o.slice,d=o.splice,f=/^#[\w-]*$/,p=/^\.[\w-]*$/,g=/<.+>/,v=/^\w+$/;function m(t,e){return void 0===e&&(e=r),p.test(t)?e.getElementsByClassName(t.slice(1)):v.test(t)?e.getElementsByTagName(t):e.querySelectorAll(t)}function y(t,e){if(void 0===e&&(e=r),t){if(t.__cash)return t;var n=t;if(P(t)){if(e.__cash&&(e=e[0]),!(n=f.test(t)?e.getElementById(t.slice(1)):g.test(t)?dt(t):m(t,e)))return}else if(j(t))return this.ready(t);(n.nodeType||n===i)&&(n=[n]),this.length=n.length;for(var o=0,a=this.length;o<a;o++)this[o]=n[o];}}function b(t,e){return new y(t,e)}var w=b.fn=b.prototype=y.prototype={constructor:b,__cash:!0,length:0,splice:d};w.get=function(t){return void 0===t?h.call(this):this[t<0?t+this.length:t]},w.eq=function(t){return b(this.get(t))},w.first=function(){return this.eq(0)},w.last=function(){return this.eq(-1)},w.map=function(t){return b(l.call(this,(function(e,n){return t.call(e,n,e)})))},w.slice=function(){return b(h.apply(this,arguments))};var x=/(?:^\w|[A-Z]|\b\w)/g,O=/[\s-_]+/g;function C(t){return t.replace(x,(function(t,e){return t[e?"toUpperCase":"toLowerCase"]()})).replace(O,"")}function S(t,e){for(var n=0,r=t.length;n<r&&!1!==e.call(t[n],t[n],n,t);n++);}b.camelCase=C,b.each=S,w.each=function(t){return S(this,(function(e,n){return t.call(e,n,e)})),this},w.removeProp=function(t){return this.each((function(e,n){delete n[t];}))},b.extend=w.extend=function(t){void 0===t&&(t=this);for(var e=arguments,n=e.length,r=n<2?0:1;r<n;r++)for(var i in e[r])t[i]=e[r][i];return t};var k=1;function T(t,e){var n=t&&(t.matches||t.webkitMatchesSelector||t.mozMatchesSelector||t.msMatchesSelector||t.oMatchesSelector);return !!n&&n.call(t,e)}function j(t){return "function"==typeof t}function P(t){return "string"==typeof t}function E(t){return !isNaN(parseFloat(t))&&isFinite(t)}b.guid=k,b.matches=T,b.isFunction=j,b.isString=P,b.isNumeric=E;var M=Array.isArray;function D(t){return P(t)?function(e,n){return T(n,t)}:t.__cash?function(e,n){return t.is(n)}:function(t,e,n){return e===n}}b.isArray=M,w.prop=function(t,e){if(t){if(P(t))return arguments.length<2?this[0]&&this[0][t]:this.each((function(n,r){r[t]=e;}));for(var n in t)this.prop(n,t[n]);return this}},w.filter=function(t){if(!t)return b();var e=j(t)?t:D(t);return b(a.call(this,(function(n,r){return e.call(n,r,n,t)})))};var A=/\S+/g;function L(t){return P(t)&&t.match(A)||[]}function N(t){return t.filter((function(t,e,n){return n.indexOf(t)===e}))}function I(t,e,n){if(1===t.nodeType){var r=i.getComputedStyle(t,null);return e?n?r.getPropertyValue(e):r[e]:r}}function F(t,e){return parseInt(I(t,e),10)||0}w.hasClass=function(t){var e=L(t),n=!1;return e.length&&this.each((function(t,r){return !(n=r.classList.contains(e[0]))})),n},w.removeAttr=function(t){var e=L(t);return e.length?this.each((function(t,n){S(e,(function(t){n.removeAttribute(t);}));})):this},w.attr=function(t,e){if(t){if(P(t)){if(arguments.length<2){if(!this[0])return;var n=this[0].getAttribute(t);return null===n?void 0:n}return null===e?this.removeAttr(t):this.each((function(n,r){r.setAttribute(t,e);}))}for(var r in t)this.attr(r,t[r]);return this}},w.toggleClass=function(t,e){var n=L(t),r=void 0!==e;return n.length?this.each((function(t,i){S(n,(function(t){r?e?i.classList.add(t):i.classList.remove(t):i.classList.toggle(t);}));})):this},w.addClass=function(t){return this.toggleClass(t,!0)},w.removeClass=function(t){return arguments.length?this.toggleClass(t,!1):this.attr("class","")},b.unique=N,w.add=function(t,e){return b(N(this.get().concat(b(t,e).get())))};var _=/^--/;function V(t){return _.test(t)}var R={},H=r.createElement("div").style,z=["webkit","moz","ms","o"];function $(t,e){if(void 0===e&&(e=V(t)),e)return t;if(!R[t]){var n=C(t),r=""+n.charAt(0).toUpperCase()+n.slice(1);S((n+" "+z.join(r+" ")+r).split(" "),(function(e){if(e in H)return R[t]=e,!1}));}return R[t]}b.prefixedProp=$;var W={animationIterationCount:!0,columnCount:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0};function B(t,e,n){return void 0===n&&(n=V(t)),n||W[t]||!E(e)?e:e+"px"}w.css=function(t,e){if(P(t)){var n=V(t);return t=$(t,n),arguments.length<2?this[0]&&I(this[0],t,n):t?(e=B(t,e,n),this.each((function(r,i){1===i.nodeType&&(n?i.style.setProperty(t,e):i.style[t]=e);}))):this}for(var r in t)this.css(r,t[r]);return this};var U="__cashData",q=/^data-(.*)/;function G(t){return t[U]=t[U]||{}}function K(t,e){var n=G(t);if(e){if(!(e in n)){var r=t.dataset?t.dataset[e]||t.dataset[C(e)]:b(t).attr("data-"+e);if(void 0!==r){try{r=JSON.parse(r);}catch(t){}n[e]=r;}}return n[e]}return n}function Y(t,e){return F(t,"border"+(e?"Left":"Top")+"Width")+F(t,"padding"+(e?"Left":"Top"))+F(t,"padding"+(e?"Right":"Bottom"))+F(t,"border"+(e?"Right":"Bottom")+"Width")}function X(t,e){for(var n=0,r=e.length;n<r;n++)if(t.indexOf(e[n])<0)return !1;return !0}function J(t,e,n){S(t[n],(function(t){t[0];var r=t[1];e.removeEventListener(n,r);})),delete t[n];}b.hasData=function(t){return U in t},w.data=function(t,e){var n=this;if(!t){if(!this[0])return;return S(this[0].attributes,(function(t){var e=t.name.match(q);e&&n.data(e[1]);})),K(this[0])}if(P(t))return void 0===e?this[0]&&K(this[0],t):this.each((function(n,r){return function(t,e,n){G(t)[e]=n;}(r,t,e)}));for(var r in t)this.data(r,t[r]);return this},w.removeData=function(t){return this.each((function(e,n){return function(t,e){void 0===e?delete t[U]:delete G(t)[e];}(n,t)}))},S(["Width","Height"],(function(t){w["inner"+t]=function(){if(this[0])return this[0]===i?i["inner"+t]:this[0]["client"+t]};})),S(["width","height"],(function(t,e){w[t]=function(n){return this[0]?arguments.length?(n=parseInt(n,10),this.each((function(r,i){if(1===i.nodeType){var o=I(i,"boxSizing");i.style[t]=B(t,n+("border-box"===o?Y(i,!e):0));}}))):this[0]===i?this[0][C("outer-"+t)]:this[0].getBoundingClientRect()[t]-Y(this[0],!e):void 0===n?void 0:this};})),S(["Width","Height"],(function(t,e){w["outer"+t]=function(n){if(this[0])return this[0]===i?i["outer"+t]:this[0]["offset"+t]+(n?F(this[0],"margin"+(e?"Top":"Left"))+F(this[0],"margin"+(e?"Bottom":"Right")):0)};}));var Z="__cashEvents",Q=".";function tt(t){return t[Z]=t[Z]||{}}function et(t){var e=t.split(Q);return [e[0],e.slice(1).sort()]}function nt(t,e,n,r){var i=tt(t);if(e){var o=i[e];if(!o)return;r&&(r.guid=r.guid||k++),i[e]=o.filter((function(i){var o=i[0],a=i[1];if(r&&a.guid!==r.guid||!X(o,n))return !0;t.removeEventListener(e,a);}));}else if(n&&n.length)for(e in i)nt(t,e,n,r);else for(e in i)J(i,t,e);}w.off=function(t,e){var n=this;return void 0===t?this.each((function(t,e){return nt(e)})):S(L(t),(function(t){var r=et(t),i=r[0],o=r[1];n.each((function(t,n){return nt(n,i,o,e)}));})),this},w.on=function(t,e,n,r){var i=this;if(!P(t)){for(var o in t)this.on(o,e,t[o]);return this}return j(e)&&(n=e,e=!1),S(L(t),(function(t){var o=et(t),a=o[0],s=o[1];i.each((function(t,i){var o=function t(o){if(!o.namespace||X(s,o.namespace.split(Q))){var l=i;if(e){for(var c=o.target;!T(c,e);){if(c===i)return;if(!(c=c.parentNode))return}l=c;}o.namespace=o.namespace||"";var u=n.call(l,o,o.data);r&&nt(i,a,s,t),!1===u&&(o.preventDefault(),o.stopPropagation());}};o.guid=n.guid=n.guid||k++,function(t,e,n,r){r.guid=r.guid||k++;var i=tt(t);i[e]=i[e]||[],i[e].push([n,r]),t.addEventListener(e,r);}(i,a,s,o);}));})),this},w.one=function(t,e,n){return this.on(t,e,n,!0)},w.ready=function(t){var e=function(){return t(b)};return "loading"!==r.readyState?setTimeout(e):r.addEventListener("DOMContentLoaded",e),this},w.trigger=function(t,e){var n=t;if(P(t)){var i=et(t),o=i[0],a=i[1];(n=r.createEvent("HTMLEvents")).initEvent(o,!0,!0),n.namespace=a.join(Q);}return n.data=e,this.each((function(t,e){e.dispatchEvent(n);}))};var rt=/select-one/i,it=/select-multiple/i;function ot(t){var e=t.type;return rt.test(e)?function(t){return t.selectedIndex<0?null:t.options[t.selectedIndex].value}(t):it.test(e)?function(t){var e=[];return S(t.options,(function(t){!t.selected||t.disabled||t.parentNode.disabled||e.push(t.value);})),e}(t):t.value}var at=/%20/g;var st=/file|reset|submit|button|image/i,lt=/radio|checkbox/i;w.serialize=function(){var t="";return this.each((function(e,n){S(n.elements||[n],(function(e){if(!e.disabled&&e.name&&"FIELDSET"!==e.tagName&&!st.test(e.type)&&(!lt.test(e.type)||e.checked)){var n=ot(e);if(void 0!==n)S(M(n)?n:[n],(function(n){t+=function(t,e){return "&"+encodeURIComponent(t)+"="+encodeURIComponent(e).replace(at,"+")}(e.name,n);}));}}));})),t.substr(1)},w.val=function(t){return void 0===t?this[0]&&ot(this[0]):this.each((function(e,n){var r=it.test(n.type),i=null===t?r?[]:"":t;r&&M(i)?S(n.options,(function(t){t.selected=i.indexOf(t.value)>=0;})):n.value=i;}))},w.clone=function(){return this.map((function(t,e){return e.cloneNode(!0)}))},w.detach=function(){return this.each((function(t,e){e.parentNode&&e.parentNode.removeChild(e);}))};var ct,ut=/^\s*<(\w+)[^>]*>/,ht=/^\s*<(\w+)\s*\/?>(?:<\/\1>)?\s*$/;function dt(t){if(function(){if(!ct){var t=r.createElement("table"),e=r.createElement("tr");ct={"*":r.createElement("div"),tr:r.createElement("tbody"),td:e,th:e,thead:t,tbody:t,tfoot:t};}}(),!P(t))return [];if(ht.test(t))return [r.createElement(RegExp.$1)];var e=ut.test(t)&&RegExp.$1,n=ct[e]||ct["*"];return n.innerHTML=t,b(n.childNodes).detach().get()}function ft(t,e,n){if(void 0!==e){var r=P(e);!r&&e.length?S(e,(function(e){return ft(t,e,n)})):S(t,r?function(t){t.insertAdjacentHTML(n?"afterbegin":"beforeend",e);}:function(t,r){return function(t,e,n){n?t.insertBefore(e,t.childNodes[0]):t.appendChild(e);}(t,r?e.cloneNode(!0):e,n)});}}b.parseHTML=dt,w.empty=function(){var t=this[0];if(t)for(;t.firstChild;)t.removeChild(t.firstChild);return this},w.append=function(){var t=this;return S(arguments,(function(e){ft(t,e);})),this},w.appendTo=function(t){return ft(b(t),this),this},w.html=function(t){if(void 0===t)return this[0]&&this[0].innerHTML;var e=t.nodeType?t[0].outerHTML:t;return this.each((function(t,n){n.innerHTML=e;}))},w.insertAfter=function(t){var e=this;return b(t).each((function(t,n){var r=n.parentNode;e.each((function(e,i){r.insertBefore(t?i.cloneNode(!0):i,n.nextSibling);}));})),this},w.after=function(){var t=this;return S(u.apply(arguments),(function(e){u.apply(b(e).slice()).insertAfter(t);})),this},w.insertBefore=function(t){var e=this;return b(t).each((function(t,n){var r=n.parentNode;e.each((function(e,i){r.insertBefore(t?i.cloneNode(!0):i,n);}));})),this},w.before=function(){var t=this;return S(arguments,(function(e){b(e).insertBefore(t);})),this},w.prepend=function(){var t=this;return S(arguments,(function(e){ft(t,e,!0);})),this},w.prependTo=function(t){return ft(b(t),u.apply(this.slice()),!0),this},w.remove=function(){return this.detach().off()},w.replaceWith=function(t){var e=this;return this.each((function(n,r){var i=r.parentNode;if(i){var o=n?b(t).clone():b(t);if(!o[0])return e.remove(),!1;i.replaceChild(o[0],r),b(o[0]).after(o.slice(1));}}))},w.replaceAll=function(t){return b(t).replaceWith(this),this},w.text=function(t){return void 0===t?this[0]?this[0].textContent:"":this.each((function(e,n){n.textContent=t;}))};var pt=r.documentElement;w.offset=function(){var t=this[0];if(t){var e=t.getBoundingClientRect();return {top:e.top+i.pageYOffset-pt.clientTop,left:e.left+i.pageXOffset-pt.clientLeft}}},w.offsetParent=function(){return b(this[0]&&this[0].offsetParent)},w.position=function(){var t=this[0];if(t)return {left:t.offsetLeft,top:t.offsetTop}},w.children=function(t){var e=[];return this.each((function(t,n){c.apply(e,n.children);})),e=b(N(e)),t?e.filter((function(e,n){return T(n,t)})):e},w.contents=function(){var t=[];return this.each((function(e,n){c.apply(t,"IFRAME"===n.tagName?[n.contentDocument]:n.childNodes);})),b(t.length&&N(t))},w.find=function(t){for(var e=[],n=0,r=this.length;n<r;n++){var i=m(t,this[n]);i.length&&c.apply(e,i);}return b(e.length&&N(e))},w.has=function(t){var e=P(t)?function(e,n){return !!m(t,n).length}:function(e,n){return n.contains(t)};return this.filter(e)},w.is=function(t){if(!t||!this[0])return !1;var e=D(t),n=!1;return this.each((function(r,i){return !(n=e(r,i,t))})),n},w.next=function(){return b(this[0]&&this[0].nextElementSibling)},w.not=function(t){if(!t||!this[0])return this;var e=D(t);return this.filter((function(n,r){return !e(n,r,t)}))},w.parent=function(){var t=[];return this.each((function(e,n){n&&n.parentNode&&t.push(n.parentNode);})),b(N(t))},w.index=function(t){var e=t?b(t)[0]:this[0],n=t?this:b(e).parent().children();return s.call(n,e)},w.closest=function(t){return t&&this[0]?this.is(t)?this.filter(t):this.parent().closest(t):b()},w.parents=function(t){var e,n=[];return this.each((function(i,o){for(e=o;e&&e.parentNode&&e!==r.body.parentNode;)e=e.parentNode,(!t||t&&T(e,t))&&n.push(e);})),b(N(n))},w.prev=function(){return b(this[0]&&this[0].previousElementSibling)},w.siblings=function(){var t=this[0];return this.parent().children().filter((function(e,n){return n!==t}))},e.default=b;},function(t,e,n){var r=n(0),i=n(1),o=n.n(i),a=n(9);e.a=o.a.Collection.extend({model:a.a,modelId:function(t){return "".concat(t.name,"_").concat(t.type||a.a.TYPE_CLASS)},getStyleable:function(){return Object(r.filter)(this.models,(function(t){return t.get("active")&&!t.get("private")}))},getValid:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.noDisabled;return Object(r.filter)(this.models,(function(t){return !t.get("private")})).filter((function(t){return e?t.get("active"):1}))},getFullString:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=[],r=t||this;return r.forEach((function(t){return n.push(t.getFullName(e))})),n.join("").trim()}});},function(t,e,n){t.exports=function(){var t=navigator.userAgent,e=navigator.platform,n=/gecko\/\d/i.test(t),r=/MSIE \d/.test(t),i=/Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(t),o=/Edge\/(\d+)/.exec(t),a=r||i||o,s=a&&(r?document.documentMode||6:+(o||i)[1]),l=!o&&/WebKit\//.test(t),c=l&&/Qt\/\d+\.\d+/.test(t),u=!o&&/Chrome\//.test(t),h=/Opera\//.test(t),d=/Apple Computer/.test(navigator.vendor),f=/Mac OS X 1\d\D([8-9]|\d\d)\D/.test(t),p=/PhantomJS/.test(t),g=!o&&/AppleWebKit/.test(t)&&/Mobile\/\w+/.test(t),v=/Android/.test(t),m=g||v||/webOS|BlackBerry|Opera Mini|Opera Mobi|IEMobile/i.test(t),y=g||/Mac/.test(e),b=/\bCrOS\b/.test(t),w=/win/i.test(e),x=h&&t.match(/Version\/(\d*\.\d*)/);x&&(x=Number(x[1])),x&&x>=15&&(h=!1,l=!0);var O=y&&(c||h&&(null==x||x<12.11)),C=n||a&&s>=9;function S(t){return new RegExp("(^|\\s)"+t+"(?:$|\\s)\\s*")}var k,T=function(t,e){var n=t.className,r=S(e).exec(n);if(r){var i=n.slice(r.index+r[0].length);t.className=n.slice(0,r.index)+(i?r[1]+i:"");}};function j(t){for(var e=t.childNodes.length;e>0;--e)t.removeChild(t.firstChild);return t}function P(t,e){return j(t).appendChild(e)}function E(t,e,n,r){var i=document.createElement(t);if(n&&(i.className=n),r&&(i.style.cssText=r),"string"==typeof e)i.appendChild(document.createTextNode(e));else if(e)for(var o=0;o<e.length;++o)i.appendChild(e[o]);return i}function M(t,e,n,r){var i=E(t,e,n,r);return i.setAttribute("role","presentation"),i}function D(t,e){if(3==e.nodeType&&(e=e.parentNode),t.contains)return t.contains(e);do{if(11==e.nodeType&&(e=e.host),e==t)return !0}while(e=e.parentNode)}function A(){var t;try{t=document.activeElement;}catch(e){t=document.body||null;}for(;t&&t.shadowRoot&&t.shadowRoot.activeElement;)t=t.shadowRoot.activeElement;return t}function L(t,e){var n=t.className;S(e).test(n)||(t.className+=(n?" ":"")+e);}function N(t,e){for(var n=t.split(" "),r=0;r<n.length;r++)n[r]&&!S(n[r]).test(e)&&(e+=" "+n[r]);return e}k=document.createRange?function(t,e,n,r){var i=document.createRange();return i.setEnd(r||t,n),i.setStart(t,e),i}:function(t,e,n){var r=document.body.createTextRange();try{r.moveToElementText(t.parentNode);}catch(t){return r}return r.collapse(!0),r.moveEnd("character",n),r.moveStart("character",e),r};var I=function(t){t.select();};function F(t){var e=Array.prototype.slice.call(arguments,1);return function(){return t.apply(null,e)}}function _(t,e,n){for(var r in e||(e={}),t)!t.hasOwnProperty(r)||!1===n&&e.hasOwnProperty(r)||(e[r]=t[r]);return e}function V(t,e,n,r,i){null==e&&-1==(e=t.search(/[^\s\u00a0]/))&&(e=t.length);for(var o=r||0,a=i||0;;){var s=t.indexOf("\t",o);if(s<0||s>=e)return a+(e-o);a+=s-o,a+=n-a%n,o=s+1;}}g?I=function(t){t.selectionStart=0,t.selectionEnd=t.value.length;}:a&&(I=function(t){try{t.select();}catch(t){}});var R=function(){this.id=null,this.f=null,this.time=0,this.handler=F(this.onTimeout,this);};function H(t,e){for(var n=0;n<t.length;++n)if(t[n]==e)return n;return -1}R.prototype.onTimeout=function(t){t.id=0,t.time<=+new Date?t.f():setTimeout(t.handler,t.time-+new Date);},R.prototype.set=function(t,e){this.f=e;var n=+new Date+t;(!this.id||n<this.time)&&(clearTimeout(this.id),this.id=setTimeout(this.handler,t),this.time=n);};var z=30,$={toString:function(){return "CodeMirror.Pass"}},W={scroll:!1},B={origin:"*mouse"},U={origin:"+move"};function q(t,e,n){for(var r=0,i=0;;){var o=t.indexOf("\t",r);-1==o&&(o=t.length);var a=o-r;if(o==t.length||i+a>=e)return r+Math.min(a,e-i);if(i+=o-r,r=o+1,(i+=n-i%n)>=e)return r}}var G=[""];function K(t){for(;G.length<=t;)G.push(Y(G)+" ");return G[t]}function Y(t){return t[t.length-1]}function X(t,e){for(var n=[],r=0;r<t.length;r++)n[r]=e(t[r],r);return n}function J(){}function Z(t,e){var n;return Object.create?n=Object.create(t):(J.prototype=t,n=new J),e&&_(e,n),n}var Q=/[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;function tt(t){return /\w/.test(t)||t>""&&(t.toUpperCase()!=t.toLowerCase()||Q.test(t))}function et(t,e){return e?!!(e.source.indexOf("\\w")>-1&&tt(t))||e.test(t):tt(t)}function nt(t){for(var e in t)if(t.hasOwnProperty(e)&&t[e])return !1;return !0}var rt=/[\u0300-\u036f\u0483-\u0489\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u065e\u0670\u06d6-\u06dc\u06de-\u06e4\u06e7\u06e8\u06ea-\u06ed\u0711\u0730-\u074a\u07a6-\u07b0\u07eb-\u07f3\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0900-\u0902\u093c\u0941-\u0948\u094d\u0951-\u0955\u0962\u0963\u0981\u09bc\u09be\u09c1-\u09c4\u09cd\u09d7\u09e2\u09e3\u0a01\u0a02\u0a3c\u0a41\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a70\u0a71\u0a75\u0a81\u0a82\u0abc\u0ac1-\u0ac5\u0ac7\u0ac8\u0acd\u0ae2\u0ae3\u0b01\u0b3c\u0b3e\u0b3f\u0b41-\u0b44\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b82\u0bbe\u0bc0\u0bcd\u0bd7\u0c3e-\u0c40\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0cbc\u0cbf\u0cc2\u0cc6\u0ccc\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0d3e\u0d41-\u0d44\u0d4d\u0d57\u0d62\u0d63\u0dca\u0dcf\u0dd2-\u0dd4\u0dd6\u0ddf\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0eb1\u0eb4-\u0eb9\u0ebb\u0ebc\u0ec8-\u0ecd\u0f18\u0f19\u0f35\u0f37\u0f39\u0f71-\u0f7e\u0f80-\u0f84\u0f86\u0f87\u0f90-\u0f97\u0f99-\u0fbc\u0fc6\u102d-\u1030\u1032-\u1037\u1039\u103a\u103d\u103e\u1058\u1059\u105e-\u1060\u1071-\u1074\u1082\u1085\u1086\u108d\u109d\u135f\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b7-\u17bd\u17c6\u17c9-\u17d3\u17dd\u180b-\u180d\u18a9\u1920-\u1922\u1927\u1928\u1932\u1939-\u193b\u1a17\u1a18\u1a56\u1a58-\u1a5e\u1a60\u1a62\u1a65-\u1a6c\u1a73-\u1a7c\u1a7f\u1b00-\u1b03\u1b34\u1b36-\u1b3a\u1b3c\u1b42\u1b6b-\u1b73\u1b80\u1b81\u1ba2-\u1ba5\u1ba8\u1ba9\u1c2c-\u1c33\u1c36\u1c37\u1cd0-\u1cd2\u1cd4-\u1ce0\u1ce2-\u1ce8\u1ced\u1dc0-\u1de6\u1dfd-\u1dff\u200c\u200d\u20d0-\u20f0\u2cef-\u2cf1\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua66f-\ua672\ua67c\ua67d\ua6f0\ua6f1\ua802\ua806\ua80b\ua825\ua826\ua8c4\ua8e0-\ua8f1\ua926-\ua92d\ua947-\ua951\ua980-\ua982\ua9b3\ua9b6-\ua9b9\ua9bc\uaa29-\uaa2e\uaa31\uaa32\uaa35\uaa36\uaa43\uaa4c\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uabe5\uabe8\uabed\udc00-\udfff\ufb1e\ufe00-\ufe0f\ufe20-\ufe26\uff9e\uff9f]/;function it(t){return t.charCodeAt(0)>=768&&rt.test(t)}function ot(t,e,n){for(;(n<0?e>0:e<t.length)&&it(t.charAt(e));)e+=n;return e}function at(t,e,n){for(var r=e>n?-1:1;;){if(e==n)return e;var i=(e+n)/2,o=r<0?Math.ceil(i):Math.floor(i);if(o==e)return t(o)?e:n;t(o)?n=o:e=o+r;}}var st=null;function lt(t,e,n){var r;st=null;for(var i=0;i<t.length;++i){var o=t[i];if(o.from<e&&o.to>e)return i;o.to==e&&(o.from!=o.to&&"before"==n?r=i:st=i),o.from==e&&(o.from!=o.to&&"before"!=n?r=i:st=i);}return null!=r?r:st}var ct=function(){var t="bbbbbbbbbtstwsbbbbbbbbbbbbbbssstwNN%%%NNNNNN,N,N1111111111NNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNNNLLLLLLLLLLLLLLLLLLLLLLLLLLNNNNbbbbbbsbbbbbbbbbbbbbbbbbbbbbbbbbb,N%%%%NNNNLNNNNN%%11NLNNN1LNNNNNLLLLLLLLLLLLLLLLLLLLLLLNLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLN",e="nnnnnnNNr%%r,rNNmmmmmmmmmmmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmmmmmmmmmmmmmmmnnnnnnnnnn%nnrrrmrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrmmmmmmmnNmmmmmmrrmmNmmmmrr1111111111",n=/[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac]/,r=/[stwN]/,i=/[LRr]/,o=/[Lb1n]/,a=/[1n]/;function s(t,e,n){this.level=t,this.from=e,this.to=n;}return function(l,c){var u="ltr"==c?"L":"R";if(0==l.length||"ltr"==c&&!n.test(l))return !1;for(var h,d=l.length,f=[],p=0;p<d;++p)f.push((h=l.charCodeAt(p))<=247?t.charAt(h):1424<=h&&h<=1524?"R":1536<=h&&h<=1785?e.charAt(h-1536):1774<=h&&h<=2220?"r":8192<=h&&h<=8203?"w":8204==h?"b":"L");for(var g=0,v=u;g<d;++g){var m=f[g];"m"==m?f[g]=v:v=m;}for(var y=0,b=u;y<d;++y){var w=f[y];"1"==w&&"r"==b?f[y]="n":i.test(w)&&(b=w,"r"==w&&(f[y]="R"));}for(var x=1,O=f[0];x<d-1;++x){var C=f[x];"+"==C&&"1"==O&&"1"==f[x+1]?f[x]="1":","!=C||O!=f[x+1]||"1"!=O&&"n"!=O||(f[x]=O),O=C;}for(var S=0;S<d;++S){var k=f[S];if(","==k)f[S]="N";else if("%"==k){var T=void 0;for(T=S+1;T<d&&"%"==f[T];++T);for(var j=S&&"!"==f[S-1]||T<d&&"1"==f[T]?"1":"N",P=S;P<T;++P)f[P]=j;S=T-1;}}for(var E=0,M=u;E<d;++E){var D=f[E];"L"==M&&"1"==D?f[E]="L":i.test(D)&&(M=D);}for(var A=0;A<d;++A)if(r.test(f[A])){var L=void 0;for(L=A+1;L<d&&r.test(f[L]);++L);for(var N="L"==(A?f[A-1]:u),I=N==("L"==(L<d?f[L]:u))?N?"L":"R":u,F=A;F<L;++F)f[F]=I;A=L-1;}for(var _,V=[],R=0;R<d;)if(o.test(f[R])){var H=R;for(++R;R<d&&o.test(f[R]);++R);V.push(new s(0,H,R));}else {var z=R,$=V.length;for(++R;R<d&&"L"!=f[R];++R);for(var W=z;W<R;)if(a.test(f[W])){z<W&&V.splice($,0,new s(1,z,W));var B=W;for(++W;W<R&&a.test(f[W]);++W);V.splice($,0,new s(2,B,W)),z=W;}else ++W;z<R&&V.splice($,0,new s(1,z,R));}return "ltr"==c&&(1==V[0].level&&(_=l.match(/^\s+/))&&(V[0].from=_[0].length,V.unshift(new s(0,0,_[0].length))),1==Y(V).level&&(_=l.match(/\s+$/))&&(Y(V).to-=_[0].length,V.push(new s(0,d-_[0].length,d)))),"rtl"==c?V.reverse():V}}();function ut(t,e){var n=t.order;return null==n&&(n=t.order=ct(t.text,e)),n}var ht=[],dt=function(t,e,n){if(t.addEventListener)t.addEventListener(e,n,!1);else if(t.attachEvent)t.attachEvent("on"+e,n);else {var r=t._handlers||(t._handlers={});r[e]=(r[e]||ht).concat(n);}};function ft(t,e){return t._handlers&&t._handlers[e]||ht}function pt(t,e,n){if(t.removeEventListener)t.removeEventListener(e,n,!1);else if(t.detachEvent)t.detachEvent("on"+e,n);else {var r=t._handlers,i=r&&r[e];if(i){var o=H(i,n);o>-1&&(r[e]=i.slice(0,o).concat(i.slice(o+1)));}}}function gt(t,e){var n=ft(t,e);if(n.length)for(var r=Array.prototype.slice.call(arguments,2),i=0;i<n.length;++i)n[i].apply(null,r);}function vt(t,e,n){return "string"==typeof e&&(e={type:e,preventDefault:function(){this.defaultPrevented=!0;}}),gt(t,n||e.type,t,e),Ot(e)||e.codemirrorIgnore}function mt(t){var e=t._handlers&&t._handlers.cursorActivity;if(e)for(var n=t.curOp.cursorActivityHandlers||(t.curOp.cursorActivityHandlers=[]),r=0;r<e.length;++r)-1==H(n,e[r])&&n.push(e[r]);}function yt(t,e){return ft(t,e).length>0}function bt(t){t.prototype.on=function(t,e){dt(this,t,e);},t.prototype.off=function(t,e){pt(this,t,e);};}function wt(t){t.preventDefault?t.preventDefault():t.returnValue=!1;}function xt(t){t.stopPropagation?t.stopPropagation():t.cancelBubble=!0;}function Ot(t){return null!=t.defaultPrevented?t.defaultPrevented:0==t.returnValue}function Ct(t){wt(t),xt(t);}function St(t){return t.target||t.srcElement}function kt(t){var e=t.which;return null==e&&(1&t.button?e=1:2&t.button?e=3:4&t.button&&(e=2)),y&&t.ctrlKey&&1==e&&(e=3),e}var Tt,jt,Pt=function(){if(a&&s<9)return !1;var t=E("div");return "draggable"in t||"dragDrop"in t}();function Et(t){if(null==Tt){var e=E("span","​");P(t,E("span",[e,document.createTextNode("x")])),0!=t.firstChild.offsetHeight&&(Tt=e.offsetWidth<=1&&e.offsetHeight>2&&!(a&&s<8));}var n=Tt?E("span","​"):E("span"," ",null,"display: inline-block; width: 1px; margin-right: -1px");return n.setAttribute("cm-text",""),n}function Mt(t){if(null!=jt)return jt;var e=P(t,document.createTextNode("AخA")),n=k(e,0,1).getBoundingClientRect(),r=k(e,1,2).getBoundingClientRect();return j(t),!(!n||n.left==n.right)&&(jt=r.right-n.right<3)}var Dt,At=3!="\n\nb".split(/\n/).length?function(t){for(var e=0,n=[],r=t.length;e<=r;){var i=t.indexOf("\n",e);-1==i&&(i=t.length);var o=t.slice(e,"\r"==t.charAt(i-1)?i-1:i),a=o.indexOf("\r");-1!=a?(n.push(o.slice(0,a)),e+=a+1):(n.push(o),e=i+1);}return n}:function(t){return t.split(/\r\n?|\n/)},Lt=window.getSelection?function(t){try{return t.selectionStart!=t.selectionEnd}catch(t){return !1}}:function(t){var e;try{e=t.ownerDocument.selection.createRange();}catch(t){}return !(!e||e.parentElement()!=t)&&0!=e.compareEndPoints("StartToEnd",e)},Nt="oncopy"in(Dt=E("div"))||(Dt.setAttribute("oncopy","return;"),"function"==typeof Dt.oncopy),It=null,Ft={},_t={};function Vt(t,e){arguments.length>2&&(e.dependencies=Array.prototype.slice.call(arguments,2)),Ft[t]=e;}function Rt(t){if("string"==typeof t&&_t.hasOwnProperty(t))t=_t[t];else if(t&&"string"==typeof t.name&&_t.hasOwnProperty(t.name)){var e=_t[t.name];"string"==typeof e&&(e={name:e}),(t=Z(e,t)).name=e.name;}else {if("string"==typeof t&&/^[\w\-]+\/[\w\-]+\+xml$/.test(t))return Rt("application/xml");if("string"==typeof t&&/^[\w\-]+\/[\w\-]+\+json$/.test(t))return Rt("application/json")}return "string"==typeof t?{name:t}:t||{name:"null"}}function Ht(t,e){e=Rt(e);var n=Ft[e.name];if(!n)return Ht(t,"text/plain");var r=n(t,e);if(zt.hasOwnProperty(e.name)){var i=zt[e.name];for(var o in i)i.hasOwnProperty(o)&&(r.hasOwnProperty(o)&&(r["_"+o]=r[o]),r[o]=i[o]);}if(r.name=e.name,e.helperType&&(r.helperType=e.helperType),e.modeProps)for(var a in e.modeProps)r[a]=e.modeProps[a];return r}var zt={};function $t(t,e){_(e,zt.hasOwnProperty(t)?zt[t]:zt[t]={});}function Wt(t,e){if(!0===e)return e;if(t.copyState)return t.copyState(e);var n={};for(var r in e){var i=e[r];i instanceof Array&&(i=i.concat([])),n[r]=i;}return n}function Bt(t,e){for(var n;t.innerMode&&(n=t.innerMode(e))&&n.mode!=t;)e=n.state,t=n.mode;return n||{mode:t,state:e}}function Ut(t,e,n){return !t.startState||t.startState(e,n)}var qt=function(t,e,n){this.pos=this.start=0,this.string=t,this.tabSize=e||8,this.lastColumnPos=this.lastColumnValue=0,this.lineStart=0,this.lineOracle=n;};function Gt(t,e){if((e-=t.first)<0||e>=t.size)throw new Error("There is no line "+(e+t.first)+" in the document.");for(var n=t;!n.lines;)for(var r=0;;++r){var i=n.children[r],o=i.chunkSize();if(e<o){n=i;break}e-=o;}return n.lines[e]}function Kt(t,e,n){var r=[],i=e.line;return t.iter(e.line,n.line+1,(function(t){var o=t.text;i==n.line&&(o=o.slice(0,n.ch)),i==e.line&&(o=o.slice(e.ch)),r.push(o),++i;})),r}function Yt(t,e,n){var r=[];return t.iter(e,n,(function(t){r.push(t.text);})),r}function Xt(t,e){var n=e-t.height;if(n)for(var r=t;r;r=r.parent)r.height+=n;}function Jt(t){if(null==t.parent)return null;for(var e=t.parent,n=H(e.lines,t),r=e.parent;r;e=r,r=r.parent)for(var i=0;r.children[i]!=e;++i)n+=r.children[i].chunkSize();return n+e.first}function Zt(t,e){var n=t.first;t:do{for(var r=0;r<t.children.length;++r){var i=t.children[r],o=i.height;if(e<o){t=i;continue t}e-=o,n+=i.chunkSize();}return n}while(!t.lines);for(var a=0;a<t.lines.length;++a){var s=t.lines[a].height;if(e<s)break;e-=s;}return n+a}function Qt(t,e){return e>=t.first&&e<t.first+t.size}function te(t,e){return String(t.lineNumberFormatter(e+t.firstLineNumber))}function ee(t,e,n){if(void 0===n&&(n=null),!(this instanceof ee))return new ee(t,e,n);this.line=t,this.ch=e,this.sticky=n;}function ne(t,e){return t.line-e.line||t.ch-e.ch}function re(t,e){return t.sticky==e.sticky&&0==ne(t,e)}function ie(t){return ee(t.line,t.ch)}function oe(t,e){return ne(t,e)<0?e:t}function ae(t,e){return ne(t,e)<0?t:e}function se(t,e){return Math.max(t.first,Math.min(e,t.first+t.size-1))}function le(t,e){if(e.line<t.first)return ee(t.first,0);var n=t.first+t.size-1;return e.line>n?ee(n,Gt(t,n).text.length):function(t,e){var n=t.ch;return null==n||n>e?ee(t.line,e):n<0?ee(t.line,0):t}(e,Gt(t,e.line).text.length)}function ce(t,e){for(var n=[],r=0;r<e.length;r++)n[r]=le(t,e[r]);return n}qt.prototype.eol=function(){return this.pos>=this.string.length},qt.prototype.sol=function(){return this.pos==this.lineStart},qt.prototype.peek=function(){return this.string.charAt(this.pos)||void 0},qt.prototype.next=function(){if(this.pos<this.string.length)return this.string.charAt(this.pos++)},qt.prototype.eat=function(t){var e=this.string.charAt(this.pos);if("string"==typeof t?e==t:e&&(t.test?t.test(e):t(e)))return ++this.pos,e},qt.prototype.eatWhile=function(t){for(var e=this.pos;this.eat(t););return this.pos>e},qt.prototype.eatSpace=function(){for(var t=this.pos;/[\s\u00a0]/.test(this.string.charAt(this.pos));)++this.pos;return this.pos>t},qt.prototype.skipToEnd=function(){this.pos=this.string.length;},qt.prototype.skipTo=function(t){var e=this.string.indexOf(t,this.pos);if(e>-1)return this.pos=e,!0},qt.prototype.backUp=function(t){this.pos-=t;},qt.prototype.column=function(){return this.lastColumnPos<this.start&&(this.lastColumnValue=V(this.string,this.start,this.tabSize,this.lastColumnPos,this.lastColumnValue),this.lastColumnPos=this.start),this.lastColumnValue-(this.lineStart?V(this.string,this.lineStart,this.tabSize):0)},qt.prototype.indentation=function(){return V(this.string,null,this.tabSize)-(this.lineStart?V(this.string,this.lineStart,this.tabSize):0)},qt.prototype.match=function(t,e,n){if("string"!=typeof t){var r=this.string.slice(this.pos).match(t);return r&&r.index>0?null:(r&&!1!==e&&(this.pos+=r[0].length),r)}var i=function(t){return n?t.toLowerCase():t};if(i(this.string.substr(this.pos,t.length))==i(t))return !1!==e&&(this.pos+=t.length),!0},qt.prototype.current=function(){return this.string.slice(this.start,this.pos)},qt.prototype.hideFirstChars=function(t,e){this.lineStart+=t;try{return e()}finally{this.lineStart-=t;}},qt.prototype.lookAhead=function(t){var e=this.lineOracle;return e&&e.lookAhead(t)},qt.prototype.baseToken=function(){var t=this.lineOracle;return t&&t.baseToken(this.pos)};var ue=function(t,e){this.state=t,this.lookAhead=e;},he=function(t,e,n,r){this.state=e,this.doc=t,this.line=n,this.maxLookAhead=r||0,this.baseTokens=null,this.baseTokenPos=1;};function de(t,e,n,r){var i=[t.state.modeGen],o={};xe(t,e.text,t.doc.mode,n,(function(t,e){return i.push(t,e)}),o,r);for(var a=n.state,s=function(r){n.baseTokens=i;var s=t.state.overlays[r],l=1,c=0;n.state=!0,xe(t,e.text,s.mode,n,(function(t,e){for(var n=l;c<t;){var r=i[l];r>t&&i.splice(l,1,t,i[l+1],r),l+=2,c=Math.min(t,r);}if(e)if(s.opaque)i.splice(n,l-n,t,"overlay "+e),l=n+2;else for(;n<l;n+=2){var o=i[n+1];i[n+1]=(o?o+" ":"")+"overlay "+e;}}),o),n.state=a,n.baseTokens=null,n.baseTokenPos=1;},l=0;l<t.state.overlays.length;++l)s(l);return {styles:i,classes:o.bgClass||o.textClass?o:null}}function fe(t,e,n){if(!e.styles||e.styles[0]!=t.state.modeGen){var r=pe(t,Jt(e)),i=e.text.length>t.options.maxHighlightLength&&Wt(t.doc.mode,r.state),o=de(t,e,r);i&&(r.state=i),e.stateAfter=r.save(!i),e.styles=o.styles,o.classes?e.styleClasses=o.classes:e.styleClasses&&(e.styleClasses=null),n===t.doc.highlightFrontier&&(t.doc.modeFrontier=Math.max(t.doc.modeFrontier,++t.doc.highlightFrontier));}return e.styles}function pe(t,e,n){var r=t.doc,i=t.display;if(!r.mode.startState)return new he(r,!0,e);var o=function(t,e,n){for(var r,i,o=t.doc,a=n?-1:e-(t.doc.mode.innerMode?1e3:100),s=e;s>a;--s){if(s<=o.first)return o.first;var l=Gt(o,s-1),c=l.stateAfter;if(c&&(!n||s+(c instanceof ue?c.lookAhead:0)<=o.modeFrontier))return s;var u=V(l.text,null,t.options.tabSize);(null==i||r>u)&&(i=s-1,r=u);}return i}(t,e,n),a=o>r.first&&Gt(r,o-1).stateAfter,s=a?he.fromSaved(r,a,o):new he(r,Ut(r.mode),o);return r.iter(o,e,(function(n){ge(t,n.text,s);var r=s.line;n.stateAfter=r==e-1||r%5==0||r>=i.viewFrom&&r<i.viewTo?s.save():null,s.nextLine();})),n&&(r.modeFrontier=s.line),s}function ge(t,e,n,r){var i=t.doc.mode,o=new qt(e,t.options.tabSize,n);for(o.start=o.pos=r||0,""==e&&ve(i,n.state);!o.eol();)me(i,o,n.state),o.start=o.pos;}function ve(t,e){if(t.blankLine)return t.blankLine(e);if(t.innerMode){var n=Bt(t,e);return n.mode.blankLine?n.mode.blankLine(n.state):void 0}}function me(t,e,n,r){for(var i=0;i<10;i++){r&&(r[0]=Bt(t,n).mode);var o=t.token(e,n);if(e.pos>e.start)return o}throw new Error("Mode "+t.name+" failed to advance stream.")}he.prototype.lookAhead=function(t){var e=this.doc.getLine(this.line+t);return null!=e&&t>this.maxLookAhead&&(this.maxLookAhead=t),e},he.prototype.baseToken=function(t){if(!this.baseTokens)return null;for(;this.baseTokens[this.baseTokenPos]<=t;)this.baseTokenPos+=2;var e=this.baseTokens[this.baseTokenPos+1];return {type:e&&e.replace(/( |^)overlay .*/,""),size:this.baseTokens[this.baseTokenPos]-t}},he.prototype.nextLine=function(){this.line++,this.maxLookAhead>0&&this.maxLookAhead--;},he.fromSaved=function(t,e,n){return e instanceof ue?new he(t,Wt(t.mode,e.state),n,e.lookAhead):new he(t,Wt(t.mode,e),n)},he.prototype.save=function(t){var e=!1!==t?Wt(this.doc.mode,this.state):this.state;return this.maxLookAhead>0?new ue(e,this.maxLookAhead):e};var ye=function(t,e,n){this.start=t.start,this.end=t.pos,this.string=t.current(),this.type=e||null,this.state=n;};function be(t,e,n,r){var i,o,a=t.doc,s=a.mode,l=Gt(a,(e=le(a,e)).line),c=pe(t,e.line,n),u=new qt(l.text,t.options.tabSize,c);for(r&&(o=[]);(r||u.pos<e.ch)&&!u.eol();)u.start=u.pos,i=me(s,u,c.state),r&&o.push(new ye(u,i,Wt(a.mode,c.state)));return r?o:new ye(u,i,c.state)}function we(t,e){if(t)for(;;){var n=t.match(/(?:^|\s+)line-(background-)?(\S+)/);if(!n)break;t=t.slice(0,n.index)+t.slice(n.index+n[0].length);var r=n[1]?"bgClass":"textClass";null==e[r]?e[r]=n[2]:new RegExp("(?:^|s)"+n[2]+"(?:$|s)").test(e[r])||(e[r]+=" "+n[2]);}return t}function xe(t,e,n,r,i,o,a){var s=n.flattenSpans;null==s&&(s=t.options.flattenSpans);var l,c=0,u=null,h=new qt(e,t.options.tabSize,r),d=t.options.addModeClass&&[null];for(""==e&&we(ve(n,r.state),o);!h.eol();){if(h.pos>t.options.maxHighlightLength?(s=!1,a&&ge(t,e,r,h.pos),h.pos=e.length,l=null):l=we(me(n,h,r.state,d),o),d){var f=d[0].name;f&&(l="m-"+(l?f+" "+l:f));}if(!s||u!=l){for(;c<h.start;)i(c=Math.min(h.start,c+5e3),u);u=l;}h.start=h.pos;}for(;c<h.pos;){var p=Math.min(h.pos,c+5e3);i(p,u),c=p;}}var Oe=!1,Ce=!1;function Se(t,e,n){this.marker=t,this.from=e,this.to=n;}function ke(t,e){if(t)for(var n=0;n<t.length;++n){var r=t[n];if(r.marker==e)return r}}function Te(t,e){for(var n,r=0;r<t.length;++r)t[r]!=e&&(n||(n=[])).push(t[r]);return n}function je(t,e){if(e.full)return null;var n=Qt(t,e.from.line)&&Gt(t,e.from.line).markedSpans,r=Qt(t,e.to.line)&&Gt(t,e.to.line).markedSpans;if(!n&&!r)return null;var i=e.from.ch,o=e.to.ch,a=0==ne(e.from,e.to),s=function(t,e,n){var r;if(t)for(var i=0;i<t.length;++i){var o=t[i],a=o.marker;if(null==o.from||(a.inclusiveLeft?o.from<=e:o.from<e)||o.from==e&&"bookmark"==a.type&&(!n||!o.marker.insertLeft)){var s=null==o.to||(a.inclusiveRight?o.to>=e:o.to>e);(r||(r=[])).push(new Se(a,o.from,s?null:o.to));}}return r}(n,i,a),l=function(t,e,n){var r;if(t)for(var i=0;i<t.length;++i){var o=t[i],a=o.marker;if(null==o.to||(a.inclusiveRight?o.to>=e:o.to>e)||o.from==e&&"bookmark"==a.type&&(!n||o.marker.insertLeft)){var s=null==o.from||(a.inclusiveLeft?o.from<=e:o.from<e);(r||(r=[])).push(new Se(a,s?null:o.from-e,null==o.to?null:o.to-e));}}return r}(r,o,a),c=1==e.text.length,u=Y(e.text).length+(c?i:0);if(s)for(var h=0;h<s.length;++h){var d=s[h];if(null==d.to){var f=ke(l,d.marker);f?c&&(d.to=null==f.to?null:f.to+u):d.to=i;}}if(l)for(var p=0;p<l.length;++p){var g=l[p];null!=g.to&&(g.to+=u),null==g.from?ke(s,g.marker)||(g.from=u,c&&(s||(s=[])).push(g)):(g.from+=u,c&&(s||(s=[])).push(g));}s&&(s=Pe(s)),l&&l!=s&&(l=Pe(l));var v=[s];if(!c){var m,y=e.text.length-2;if(y>0&&s)for(var b=0;b<s.length;++b)null==s[b].to&&(m||(m=[])).push(new Se(s[b].marker,null,null));for(var w=0;w<y;++w)v.push(m);v.push(l);}return v}function Pe(t){for(var e=0;e<t.length;++e){var n=t[e];null!=n.from&&n.from==n.to&&!1!==n.marker.clearWhenEmpty&&t.splice(e--,1);}return t.length?t:null}function Ee(t){var e=t.markedSpans;if(e){for(var n=0;n<e.length;++n)e[n].marker.detachLine(t);t.markedSpans=null;}}function Me(t,e){if(e){for(var n=0;n<e.length;++n)e[n].marker.attachLine(t);t.markedSpans=e;}}function De(t){return t.inclusiveLeft?-1:0}function Ae(t){return t.inclusiveRight?1:0}function Le(t,e){var n=t.lines.length-e.lines.length;if(0!=n)return n;var r=t.find(),i=e.find(),o=ne(r.from,i.from)||De(t)-De(e);if(o)return -o;var a=ne(r.to,i.to)||Ae(t)-Ae(e);return a||e.id-t.id}function Ne(t,e){var n,r=Ce&&t.markedSpans;if(r)for(var i=void 0,o=0;o<r.length;++o)(i=r[o]).marker.collapsed&&null==(e?i.from:i.to)&&(!n||Le(n,i.marker)<0)&&(n=i.marker);return n}function Ie(t){return Ne(t,!0)}function Fe(t){return Ne(t,!1)}function _e(t,e){var n,r=Ce&&t.markedSpans;if(r)for(var i=0;i<r.length;++i){var o=r[i];o.marker.collapsed&&(null==o.from||o.from<e)&&(null==o.to||o.to>e)&&(!n||Le(n,o.marker)<0)&&(n=o.marker);}return n}function Ve(t,e,n,r,i){var o=Gt(t,e),a=Ce&&o.markedSpans;if(a)for(var s=0;s<a.length;++s){var l=a[s];if(l.marker.collapsed){var c=l.marker.find(0),u=ne(c.from,n)||De(l.marker)-De(i),h=ne(c.to,r)||Ae(l.marker)-Ae(i);if(!(u>=0&&h<=0||u<=0&&h>=0)&&(u<=0&&(l.marker.inclusiveRight&&i.inclusiveLeft?ne(c.to,n)>=0:ne(c.to,n)>0)||u>=0&&(l.marker.inclusiveRight&&i.inclusiveLeft?ne(c.from,r)<=0:ne(c.from,r)<0)))return !0}}}function Re(t){for(var e;e=Ie(t);)t=e.find(-1,!0).line;return t}function He(t,e){var n=Gt(t,e),r=Re(n);return n==r?e:Jt(r)}function ze(t,e){if(e>t.lastLine())return e;var n,r=Gt(t,e);if(!$e(t,r))return e;for(;n=Fe(r);)r=n.find(1,!0).line;return Jt(r)+1}function $e(t,e){var n=Ce&&e.markedSpans;if(n)for(var r=void 0,i=0;i<n.length;++i)if((r=n[i]).marker.collapsed){if(null==r.from)return !0;if(!r.marker.widgetNode&&0==r.from&&r.marker.inclusiveLeft&&We(t,e,r))return !0}}function We(t,e,n){if(null==n.to){var r=n.marker.find(1,!0);return We(t,r.line,ke(r.line.markedSpans,n.marker))}if(n.marker.inclusiveRight&&n.to==e.text.length)return !0;for(var i=void 0,o=0;o<e.markedSpans.length;++o)if((i=e.markedSpans[o]).marker.collapsed&&!i.marker.widgetNode&&i.from==n.to&&(null==i.to||i.to!=n.from)&&(i.marker.inclusiveLeft||n.marker.inclusiveRight)&&We(t,e,i))return !0}function Be(t){for(var e=0,n=(t=Re(t)).parent,r=0;r<n.lines.length;++r){var i=n.lines[r];if(i==t)break;e+=i.height;}for(var o=n.parent;o;o=(n=o).parent)for(var a=0;a<o.children.length;++a){var s=o.children[a];if(s==n)break;e+=s.height;}return e}function Ue(t){if(0==t.height)return 0;for(var e,n=t.text.length,r=t;e=Ie(r);){var i=e.find(0,!0);r=i.from.line,n+=i.from.ch-i.to.ch;}for(r=t;e=Fe(r);){var o=e.find(0,!0);n-=r.text.length-o.from.ch,n+=(r=o.to.line).text.length-o.to.ch;}return n}function qe(t){var e=t.display,n=t.doc;e.maxLine=Gt(n,n.first),e.maxLineLength=Ue(e.maxLine),e.maxLineChanged=!0,n.iter((function(t){var n=Ue(t);n>e.maxLineLength&&(e.maxLineLength=n,e.maxLine=t);}));}var Ge=function(t,e,n){this.text=t,Me(this,e),this.height=n?n(this):1;};function Ke(t){t.parent=null,Ee(t);}Ge.prototype.lineNo=function(){return Jt(this)},bt(Ge);var Ye={},Xe={};function Je(t,e){if(!t||/^\s*$/.test(t))return null;var n=e.addModeClass?Xe:Ye;return n[t]||(n[t]=t.replace(/\S+/g,"cm-$&"))}function Ze(t,e){var n=M("span",null,null,l?"padding-right: .1px":null),r={pre:M("pre",[n],"CodeMirror-line"),content:n,col:0,pos:0,cm:t,trailingSpace:!1,splitSpaces:t.getOption("lineWrapping")};e.measure={};for(var i=0;i<=(e.rest?e.rest.length:0);i++){var o=i?e.rest[i-1]:e.line,a=void 0;r.pos=0,r.addToken=tn,Mt(t.display.measure)&&(a=ut(o,t.doc.direction))&&(r.addToken=en(r.addToken,a)),r.map=[],rn(o,r,fe(t,o,e!=t.display.externalMeasured&&Jt(o))),o.styleClasses&&(o.styleClasses.bgClass&&(r.bgClass=N(o.styleClasses.bgClass,r.bgClass||"")),o.styleClasses.textClass&&(r.textClass=N(o.styleClasses.textClass,r.textClass||""))),0==r.map.length&&r.map.push(0,0,r.content.appendChild(Et(t.display.measure))),0==i?(e.measure.map=r.map,e.measure.cache={}):((e.measure.maps||(e.measure.maps=[])).push(r.map),(e.measure.caches||(e.measure.caches=[])).push({}));}if(l){var s=r.content.lastChild;(/\bcm-tab\b/.test(s.className)||s.querySelector&&s.querySelector(".cm-tab"))&&(r.content.className="cm-tab-wrap-hack");}return gt(t,"renderLine",t,e.line,r.pre),r.pre.className&&(r.textClass=N(r.pre.className,r.textClass||"")),r}function Qe(t){var e=E("span","•","cm-invalidchar");return e.title="\\u"+t.charCodeAt(0).toString(16),e.setAttribute("aria-label",e.title),e}function tn(t,e,n,r,i,o,l){if(e){var c,u=t.splitSpaces?function(t,e){if(t.length>1&&!/  /.test(t))return t;for(var n=e,r="",i=0;i<t.length;i++){var o=t.charAt(i);" "!=o||!n||i!=t.length-1&&32!=t.charCodeAt(i+1)||(o=" "),r+=o,n=" "==o;}return r}(e,t.trailingSpace):e,h=t.cm.state.specialChars,d=!1;if(h.test(e)){c=document.createDocumentFragment();for(var f=0;;){h.lastIndex=f;var p=h.exec(e),g=p?p.index-f:e.length-f;if(g){var v=document.createTextNode(u.slice(f,f+g));a&&s<9?c.appendChild(E("span",[v])):c.appendChild(v),t.map.push(t.pos,t.pos+g,v),t.col+=g,t.pos+=g;}if(!p)break;f+=g+1;var m=void 0;if("\t"==p[0]){var y=t.cm.options.tabSize,b=y-t.col%y;(m=c.appendChild(E("span",K(b),"cm-tab"))).setAttribute("role","presentation"),m.setAttribute("cm-text","\t"),t.col+=b;}else "\r"==p[0]||"\n"==p[0]?((m=c.appendChild(E("span","\r"==p[0]?"␍":"␤","cm-invalidchar"))).setAttribute("cm-text",p[0]),t.col+=1):((m=t.cm.options.specialCharPlaceholder(p[0])).setAttribute("cm-text",p[0]),a&&s<9?c.appendChild(E("span",[m])):c.appendChild(m),t.col+=1);t.map.push(t.pos,t.pos+1,m),t.pos++;}}else t.col+=e.length,c=document.createTextNode(u),t.map.push(t.pos,t.pos+e.length,c),a&&s<9&&(d=!0),t.pos+=e.length;if(t.trailingSpace=32==u.charCodeAt(e.length-1),n||r||i||d||o){var w=n||"";r&&(w+=r),i&&(w+=i);var x=E("span",[c],w,o);if(l)for(var O in l)l.hasOwnProperty(O)&&"style"!=O&&"class"!=O&&x.setAttribute(O,l[O]);return t.content.appendChild(x)}t.content.appendChild(c);}}function en(t,e){return function(n,r,i,o,a,s,l){i=i?i+" cm-force-border":"cm-force-border";for(var c=n.pos,u=c+r.length;;){for(var h=void 0,d=0;d<e.length&&!((h=e[d]).to>c&&h.from<=c);d++);if(h.to>=u)return t(n,r,i,o,a,s,l);t(n,r.slice(0,h.to-c),i,o,null,s,l),o=null,r=r.slice(h.to-c),c=h.to;}}}function nn(t,e,n,r){var i=!r&&n.widgetNode;i&&t.map.push(t.pos,t.pos+e,i),!r&&t.cm.display.input.needsContentAttribute&&(i||(i=t.content.appendChild(document.createElement("span"))),i.setAttribute("cm-marker",n.id)),i&&(t.cm.display.input.setUneditable(i),t.content.appendChild(i)),t.pos+=e,t.trailingSpace=!1;}function rn(t,e,n){var r=t.markedSpans,i=t.text,o=0;if(r)for(var a,s,l,c,u,h,d,f=i.length,p=0,g=1,v="",m=0;;){if(m==p){l=c=u=s="",d=null,h=null,m=1/0;for(var y=[],b=void 0,w=0;w<r.length;++w){var x=r[w],O=x.marker;if("bookmark"==O.type&&x.from==p&&O.widgetNode)y.push(O);else if(x.from<=p&&(null==x.to||x.to>p||O.collapsed&&x.to==p&&x.from==p)){if(null!=x.to&&x.to!=p&&m>x.to&&(m=x.to,c=""),O.className&&(l+=" "+O.className),O.css&&(s=(s?s+";":"")+O.css),O.startStyle&&x.from==p&&(u+=" "+O.startStyle),O.endStyle&&x.to==m&&(b||(b=[])).push(O.endStyle,x.to),O.title&&((d||(d={})).title=O.title),O.attributes)for(var C in O.attributes)(d||(d={}))[C]=O.attributes[C];O.collapsed&&(!h||Le(h.marker,O)<0)&&(h=x);}else x.from>p&&m>x.from&&(m=x.from);}if(b)for(var S=0;S<b.length;S+=2)b[S+1]==m&&(c+=" "+b[S]);if(!h||h.from==p)for(var k=0;k<y.length;++k)nn(e,0,y[k]);if(h&&(h.from||0)==p){if(nn(e,(null==h.to?f+1:h.to)-p,h.marker,null==h.from),null==h.to)return;h.to==p&&(h=!1);}}if(p>=f)break;for(var T=Math.min(f,m);;){if(v){var j=p+v.length;if(!h){var P=j>T?v.slice(0,T-p):v;e.addToken(e,P,a?a+l:l,u,p+P.length==m?c:"",s,d);}if(j>=T){v=v.slice(T-p),p=T;break}p=j,u="";}v=i.slice(o,o=n[g++]),a=Je(n[g++],e.cm.options);}}else for(var E=1;E<n.length;E+=2)e.addToken(e,i.slice(o,o=n[E]),Je(n[E+1],e.cm.options));}function on(t,e,n){this.line=e,this.rest=function(t){for(var e,n;e=Fe(t);)t=e.find(1,!0).line,(n||(n=[])).push(t);return n}(e),this.size=this.rest?Jt(Y(this.rest))-n+1:1,this.node=this.text=null,this.hidden=$e(t,e);}function an(t,e,n){for(var r,i=[],o=e;o<n;o=r){var a=new on(t.doc,Gt(t.doc,o),o);r=o+a.size,i.push(a);}return i}var sn=null,ln=null;function cn(t,e){var n=ft(t,e);if(n.length){var r,i=Array.prototype.slice.call(arguments,2);sn?r=sn.delayedCallbacks:ln?r=ln:(r=ln=[],setTimeout(un,0));for(var o=function(t){r.push((function(){return n[t].apply(null,i)}));},a=0;a<n.length;++a)o(a);}}function un(){var t=ln;ln=null;for(var e=0;e<t.length;++e)t[e]();}function hn(t,e,n,r){for(var i=0;i<e.changes.length;i++){var o=e.changes[i];"text"==o?pn(t,e):"gutter"==o?vn(t,e,n,r):"class"==o?gn(t,e):"widget"==o&&mn(t,e,r);}e.changes=null;}function dn(t){return t.node==t.text&&(t.node=E("div",null,null,"position: relative"),t.text.parentNode&&t.text.parentNode.replaceChild(t.node,t.text),t.node.appendChild(t.text),a&&s<8&&(t.node.style.zIndex=2)),t.node}function fn(t,e){var n=t.display.externalMeasured;return n&&n.line==e.line?(t.display.externalMeasured=null,e.measure=n.measure,n.built):Ze(t,e)}function pn(t,e){var n=e.text.className,r=fn(t,e);e.text==e.node&&(e.node=r.pre),e.text.parentNode.replaceChild(r.pre,e.text),e.text=r.pre,r.bgClass!=e.bgClass||r.textClass!=e.textClass?(e.bgClass=r.bgClass,e.textClass=r.textClass,gn(t,e)):n&&(e.text.className=n);}function gn(t,e){!function(t,e){var n=e.bgClass?e.bgClass+" "+(e.line.bgClass||""):e.line.bgClass;if(n&&(n+=" CodeMirror-linebackground"),e.background)n?e.background.className=n:(e.background.parentNode.removeChild(e.background),e.background=null);else if(n){var r=dn(e);e.background=r.insertBefore(E("div",null,n),r.firstChild),t.display.input.setUneditable(e.background);}}(t,e),e.line.wrapClass?dn(e).className=e.line.wrapClass:e.node!=e.text&&(e.node.className="");var n=e.textClass?e.textClass+" "+(e.line.textClass||""):e.line.textClass;e.text.className=n||"";}function vn(t,e,n,r){if(e.gutter&&(e.node.removeChild(e.gutter),e.gutter=null),e.gutterBackground&&(e.node.removeChild(e.gutterBackground),e.gutterBackground=null),e.line.gutterClass){var i=dn(e);e.gutterBackground=E("div",null,"CodeMirror-gutter-background "+e.line.gutterClass,"left: "+(t.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px; width: "+r.gutterTotalWidth+"px"),t.display.input.setUneditable(e.gutterBackground),i.insertBefore(e.gutterBackground,e.text);}var o=e.line.gutterMarkers;if(t.options.lineNumbers||o){var a=dn(e),s=e.gutter=E("div",null,"CodeMirror-gutter-wrapper","left: "+(t.options.fixedGutter?r.fixedPos:-r.gutterTotalWidth)+"px");if(t.display.input.setUneditable(s),a.insertBefore(s,e.text),e.line.gutterClass&&(s.className+=" "+e.line.gutterClass),!t.options.lineNumbers||o&&o["CodeMirror-linenumbers"]||(e.lineNumber=s.appendChild(E("div",te(t.options,n),"CodeMirror-linenumber CodeMirror-gutter-elt","left: "+r.gutterLeft["CodeMirror-linenumbers"]+"px; width: "+t.display.lineNumInnerWidth+"px"))),o)for(var l=0;l<t.display.gutterSpecs.length;++l){var c=t.display.gutterSpecs[l].className,u=o.hasOwnProperty(c)&&o[c];u&&s.appendChild(E("div",[u],"CodeMirror-gutter-elt","left: "+r.gutterLeft[c]+"px; width: "+r.gutterWidth[c]+"px"));}}}function mn(t,e,n){e.alignable&&(e.alignable=null);for(var r=e.node.firstChild,i=void 0;r;r=i)i=r.nextSibling,"CodeMirror-linewidget"==r.className&&e.node.removeChild(r);bn(t,e,n);}function yn(t,e,n,r){var i=fn(t,e);return e.text=e.node=i.pre,i.bgClass&&(e.bgClass=i.bgClass),i.textClass&&(e.textClass=i.textClass),gn(t,e),vn(t,e,n,r),bn(t,e,r),e.node}function bn(t,e,n){if(wn(t,e.line,e,n,!0),e.rest)for(var r=0;r<e.rest.length;r++)wn(t,e.rest[r],e,n,!1);}function wn(t,e,n,r,i){if(e.widgets)for(var o=dn(n),a=0,s=e.widgets;a<s.length;++a){var l=s[a],c=E("div",[l.node],"CodeMirror-linewidget");l.handleMouseEvents||c.setAttribute("cm-ignore-events","true"),xn(l,c,n,r),t.display.input.setUneditable(c),i&&l.above?o.insertBefore(c,n.gutter||n.text):o.appendChild(c),cn(l,"redraw");}}function xn(t,e,n,r){if(t.noHScroll){(n.alignable||(n.alignable=[])).push(e);var i=r.wrapperWidth;e.style.left=r.fixedPos+"px",t.coverGutter||(i-=r.gutterTotalWidth,e.style.paddingLeft=r.gutterTotalWidth+"px"),e.style.width=i+"px";}t.coverGutter&&(e.style.zIndex=5,e.style.position="relative",t.noHScroll||(e.style.marginLeft=-r.gutterTotalWidth+"px"));}function On(t){if(null!=t.height)return t.height;var e=t.doc.cm;if(!e)return 0;if(!D(document.body,t.node)){var n="position: relative;";t.coverGutter&&(n+="margin-left: -"+e.display.gutters.offsetWidth+"px;"),t.noHScroll&&(n+="width: "+e.display.wrapper.clientWidth+"px;"),P(e.display.measure,E("div",[t.node],null,n));}return t.height=t.node.parentNode.offsetHeight}function Cn(t,e){for(var n=St(e);n!=t.wrapper;n=n.parentNode)if(!n||1==n.nodeType&&"true"==n.getAttribute("cm-ignore-events")||n.parentNode==t.sizer&&n!=t.mover)return !0}function Sn(t){return t.lineSpace.offsetTop}function kn(t){return t.mover.offsetHeight-t.lineSpace.offsetHeight}function Tn(t){if(t.cachedPaddingH)return t.cachedPaddingH;var e=P(t.measure,E("pre","x","CodeMirror-line-like")),n=window.getComputedStyle?window.getComputedStyle(e):e.currentStyle,r={left:parseInt(n.paddingLeft),right:parseInt(n.paddingRight)};return isNaN(r.left)||isNaN(r.right)||(t.cachedPaddingH=r),r}function jn(t){return z-t.display.nativeBarWidth}function Pn(t){return t.display.scroller.clientWidth-jn(t)-t.display.barWidth}function En(t){return t.display.scroller.clientHeight-jn(t)-t.display.barHeight}function Mn(t,e,n){if(t.line==e)return {map:t.measure.map,cache:t.measure.cache};for(var r=0;r<t.rest.length;r++)if(t.rest[r]==e)return {map:t.measure.maps[r],cache:t.measure.caches[r]};for(var i=0;i<t.rest.length;i++)if(Jt(t.rest[i])>n)return {map:t.measure.maps[i],cache:t.measure.caches[i],before:!0}}function Dn(t,e,n,r){return Nn(t,Ln(t,e),n,r)}function An(t,e){if(e>=t.display.viewFrom&&e<t.display.viewTo)return t.display.view[hr(t,e)];var n=t.display.externalMeasured;return n&&e>=n.lineN&&e<n.lineN+n.size?n:void 0}function Ln(t,e){var n=Jt(e),r=An(t,n);r&&!r.text?r=null:r&&r.changes&&(hn(t,r,n,ar(t)),t.curOp.forceUpdate=!0),r||(r=function(t,e){var n=Jt(e=Re(e)),r=t.display.externalMeasured=new on(t.doc,e,n);r.lineN=n;var i=r.built=Ze(t,r);return r.text=i.pre,P(t.display.lineMeasure,i.pre),r}(t,e));var i=Mn(r,e,n);return {line:e,view:r,rect:null,map:i.map,cache:i.cache,before:i.before,hasHeights:!1}}function Nn(t,e,n,r,i){e.before&&(n=-1);var o,l=n+(r||"");return e.cache.hasOwnProperty(l)?o=e.cache[l]:(e.rect||(e.rect=e.view.text.getBoundingClientRect()),e.hasHeights||(function(t,e,n){var r=t.options.lineWrapping,i=r&&Pn(t);if(!e.measure.heights||r&&e.measure.width!=i){var o=e.measure.heights=[];if(r){e.measure.width=i;for(var a=e.text.firstChild.getClientRects(),s=0;s<a.length-1;s++){var l=a[s],c=a[s+1];Math.abs(l.bottom-c.bottom)>2&&o.push((l.bottom+c.top)/2-n.top);}}o.push(n.bottom-n.top);}}(t,e.view,e.rect),e.hasHeights=!0),(o=function(t,e,n,r){var i,o=_n(e.map,n,r),l=o.node,c=o.start,u=o.end,h=o.collapse;if(3==l.nodeType){for(var d=0;d<4;d++){for(;c&&it(e.line.text.charAt(o.coverStart+c));)--c;for(;o.coverStart+u<o.coverEnd&&it(e.line.text.charAt(o.coverStart+u));)++u;if((i=a&&s<9&&0==c&&u==o.coverEnd-o.coverStart?l.parentNode.getBoundingClientRect():Vn(k(l,c,u).getClientRects(),r)).left||i.right||0==c)break;u=c,c-=1,h="right";}a&&s<11&&(i=function(t,e){if(!window.screen||null==screen.logicalXDPI||screen.logicalXDPI==screen.deviceXDPI||!function(t){if(null!=It)return It;var e=P(t,E("span","x")),n=e.getBoundingClientRect(),r=k(e,0,1).getBoundingClientRect();return It=Math.abs(n.left-r.left)>1}(t))return e;var n=screen.logicalXDPI/screen.deviceXDPI,r=screen.logicalYDPI/screen.deviceYDPI;return {left:e.left*n,right:e.right*n,top:e.top*r,bottom:e.bottom*r}}(t.display.measure,i));}else {var f;c>0&&(h=r="right"),i=t.options.lineWrapping&&(f=l.getClientRects()).length>1?f["right"==r?f.length-1:0]:l.getBoundingClientRect();}if(a&&s<9&&!c&&(!i||!i.left&&!i.right)){var p=l.parentNode.getClientRects()[0];i=p?{left:p.left,right:p.left+or(t.display),top:p.top,bottom:p.bottom}:Fn;}for(var g=i.top-e.rect.top,v=i.bottom-e.rect.top,m=(g+v)/2,y=e.view.measure.heights,b=0;b<y.length-1&&!(m<y[b]);b++);var w=b?y[b-1]:0,x=y[b],O={left:("right"==h?i.right:i.left)-e.rect.left,right:("left"==h?i.left:i.right)-e.rect.left,top:w,bottom:x};return i.left||i.right||(O.bogus=!0),t.options.singleCursorHeightPerLine||(O.rtop=g,O.rbottom=v),O}(t,e,n,r)).bogus||(e.cache[l]=o)),{left:o.left,right:o.right,top:i?o.rtop:o.top,bottom:i?o.rbottom:o.bottom}}var In,Fn={left:0,right:0,top:0,bottom:0};function _n(t,e,n){for(var r,i,o,a,s,l,c=0;c<t.length;c+=3)if(s=t[c],l=t[c+1],e<s?(i=0,o=1,a="left"):e<l?o=1+(i=e-s):(c==t.length-3||e==l&&t[c+3]>e)&&(i=(o=l-s)-1,e>=l&&(a="right")),null!=i){if(r=t[c+2],s==l&&n==(r.insertLeft?"left":"right")&&(a=n),"left"==n&&0==i)for(;c&&t[c-2]==t[c-3]&&t[c-1].insertLeft;)r=t[2+(c-=3)],a="left";if("right"==n&&i==l-s)for(;c<t.length-3&&t[c+3]==t[c+4]&&!t[c+5].insertLeft;)r=t[(c+=3)+2],a="right";break}return {node:r,start:i,end:o,collapse:a,coverStart:s,coverEnd:l}}function Vn(t,e){var n=Fn;if("left"==e)for(var r=0;r<t.length&&(n=t[r]).left==n.right;r++);else for(var i=t.length-1;i>=0&&(n=t[i]).left==n.right;i--);return n}function Rn(t){if(t.measure&&(t.measure.cache={},t.measure.heights=null,t.rest))for(var e=0;e<t.rest.length;e++)t.measure.caches[e]={};}function Hn(t){t.display.externalMeasure=null,j(t.display.lineMeasure);for(var e=0;e<t.display.view.length;e++)Rn(t.display.view[e]);}function zn(t){Hn(t),t.display.cachedCharWidth=t.display.cachedTextHeight=t.display.cachedPaddingH=null,t.options.lineWrapping||(t.display.maxLineChanged=!0),t.display.lineNumChars=null;}function $n(){return u&&v?-(document.body.getBoundingClientRect().left-parseInt(getComputedStyle(document.body).marginLeft)):window.pageXOffset||(document.documentElement||document.body).scrollLeft}function Wn(){return u&&v?-(document.body.getBoundingClientRect().top-parseInt(getComputedStyle(document.body).marginTop)):window.pageYOffset||(document.documentElement||document.body).scrollTop}function Bn(t){var e=0;if(t.widgets)for(var n=0;n<t.widgets.length;++n)t.widgets[n].above&&(e+=On(t.widgets[n]));return e}function Un(t,e,n,r,i){if(!i){var o=Bn(e);n.top+=o,n.bottom+=o;}if("line"==r)return n;r||(r="local");var a=Be(e);if("local"==r?a+=Sn(t.display):a-=t.display.viewOffset,"page"==r||"window"==r){var s=t.display.lineSpace.getBoundingClientRect();a+=s.top+("window"==r?0:Wn());var l=s.left+("window"==r?0:$n());n.left+=l,n.right+=l;}return n.top+=a,n.bottom+=a,n}function qn(t,e,n){if("div"==n)return e;var r=e.left,i=e.top;if("page"==n)r-=$n(),i-=Wn();else if("local"==n||!n){var o=t.display.sizer.getBoundingClientRect();r+=o.left,i+=o.top;}var a=t.display.lineSpace.getBoundingClientRect();return {left:r-a.left,top:i-a.top}}function Gn(t,e,n,r,i){return r||(r=Gt(t.doc,e.line)),Un(t,r,Dn(t,r,e.ch,i),n)}function Kn(t,e,n,r,i,o){function a(e,a){var s=Nn(t,i,e,a?"right":"left",o);return a?s.left=s.right:s.right=s.left,Un(t,r,s,n)}r=r||Gt(t.doc,e.line),i||(i=Ln(t,r));var s=ut(r,t.doc.direction),l=e.ch,c=e.sticky;if(l>=r.text.length?(l=r.text.length,c="before"):l<=0&&(l=0,c="after"),!s)return a("before"==c?l-1:l,"before"==c);function u(t,e,n){return a(n?t-1:t,1==s[e].level!=n)}var h=lt(s,l,c),d=st,f=u(l,h,"before"==c);return null!=d&&(f.other=u(l,d,"before"!=c)),f}function Yn(t,e){var n=0;e=le(t.doc,e),t.options.lineWrapping||(n=or(t.display)*e.ch);var r=Gt(t.doc,e.line),i=Be(r)+Sn(t.display);return {left:n,right:n,top:i,bottom:i+r.height}}function Xn(t,e,n,r,i){var o=ee(t,e,n);return o.xRel=i,r&&(o.outside=r),o}function Jn(t,e,n){var r=t.doc;if((n+=t.display.viewOffset)<0)return Xn(r.first,0,null,-1,-1);var i=Zt(r,n),o=r.first+r.size-1;if(i>o)return Xn(r.first+r.size-1,Gt(r,o).text.length,null,1,1);e<0&&(e=0);for(var a=Gt(r,i);;){var s=er(t,a,i,e,n),l=_e(a,s.ch+(s.xRel>0||s.outside>0?1:0));if(!l)return s;var c=l.find(1);if(c.line==i)return c;a=Gt(r,i=c.line);}}function Zn(t,e,n,r){r-=Bn(e);var i=e.text.length,o=at((function(e){return Nn(t,n,e-1).bottom<=r}),i,0);return {begin:o,end:i=at((function(e){return Nn(t,n,e).top>r}),o,i)}}function Qn(t,e,n,r){return n||(n=Ln(t,e)),Zn(t,e,n,Un(t,e,Nn(t,n,r),"line").top)}function tr(t,e,n,r){return !(t.bottom<=n)&&(t.top>n||(r?t.left:t.right)>e)}function er(t,e,n,r,i){i-=Be(e);var o=Ln(t,e),a=Bn(e),s=0,l=e.text.length,c=!0,u=ut(e,t.doc.direction);if(u){var h=(t.options.lineWrapping?rr:nr)(t,e,n,o,u,r,i);s=(c=1!=h.level)?h.from:h.to-1,l=c?h.to:h.from-1;}var d,f,p=null,g=null,v=at((function(e){var n=Nn(t,o,e);return n.top+=a,n.bottom+=a,!!tr(n,r,i,!1)&&(n.top<=i&&n.left<=r&&(p=e,g=n),!0)}),s,l),m=!1;if(g){var y=r-g.left<g.right-r,b=y==c;v=p+(b?0:1),f=b?"after":"before",d=y?g.left:g.right;}else {c||v!=l&&v!=s||v++,f=0==v?"after":v==e.text.length?"before":Nn(t,o,v-(c?1:0)).bottom+a<=i==c?"after":"before";var w=Kn(t,ee(n,v,f),"line",e,o);d=w.left,m=i<w.top?-1:i>=w.bottom?1:0;}return Xn(n,v=ot(e.text,v,1),f,m,r-d)}function nr(t,e,n,r,i,o,a){var s=at((function(s){var l=i[s],c=1!=l.level;return tr(Kn(t,ee(n,c?l.to:l.from,c?"before":"after"),"line",e,r),o,a,!0)}),0,i.length-1),l=i[s];if(s>0){var c=1!=l.level,u=Kn(t,ee(n,c?l.from:l.to,c?"after":"before"),"line",e,r);tr(u,o,a,!0)&&u.top>a&&(l=i[s-1]);}return l}function rr(t,e,n,r,i,o,a){var s=Zn(t,e,r,a),l=s.begin,c=s.end;/\s/.test(e.text.charAt(c-1))&&c--;for(var u=null,h=null,d=0;d<i.length;d++){var f=i[d];if(!(f.from>=c||f.to<=l)){var p=Nn(t,r,1!=f.level?Math.min(c,f.to)-1:Math.max(l,f.from)).right,g=p<o?o-p+1e9:p-o;(!u||h>g)&&(u=f,h=g);}}return u||(u=i[i.length-1]),u.from<l&&(u={from:l,to:u.to,level:u.level}),u.to>c&&(u={from:u.from,to:c,level:u.level}),u}function ir(t){if(null!=t.cachedTextHeight)return t.cachedTextHeight;if(null==In){In=E("pre",null,"CodeMirror-line-like");for(var e=0;e<49;++e)In.appendChild(document.createTextNode("x")),In.appendChild(E("br"));In.appendChild(document.createTextNode("x"));}P(t.measure,In);var n=In.offsetHeight/50;return n>3&&(t.cachedTextHeight=n),j(t.measure),n||1}function or(t){if(null!=t.cachedCharWidth)return t.cachedCharWidth;var e=E("span","xxxxxxxxxx"),n=E("pre",[e],"CodeMirror-line-like");P(t.measure,n);var r=e.getBoundingClientRect(),i=(r.right-r.left)/10;return i>2&&(t.cachedCharWidth=i),i||10}function ar(t){for(var e=t.display,n={},r={},i=e.gutters.clientLeft,o=e.gutters.firstChild,a=0;o;o=o.nextSibling,++a){var s=t.display.gutterSpecs[a].className;n[s]=o.offsetLeft+o.clientLeft+i,r[s]=o.clientWidth;}return {fixedPos:sr(e),gutterTotalWidth:e.gutters.offsetWidth,gutterLeft:n,gutterWidth:r,wrapperWidth:e.wrapper.clientWidth}}function sr(t){return t.scroller.getBoundingClientRect().left-t.sizer.getBoundingClientRect().left}function lr(t){var e=ir(t.display),n=t.options.lineWrapping,r=n&&Math.max(5,t.display.scroller.clientWidth/or(t.display)-3);return function(i){if($e(t.doc,i))return 0;var o=0;if(i.widgets)for(var a=0;a<i.widgets.length;a++)i.widgets[a].height&&(o+=i.widgets[a].height);return n?o+(Math.ceil(i.text.length/r)||1)*e:o+e}}function cr(t){var e=t.doc,n=lr(t);e.iter((function(t){var e=n(t);e!=t.height&&Xt(t,e);}));}function ur(t,e,n,r){var i=t.display;if(!n&&"true"==St(e).getAttribute("cm-not-content"))return null;var o,a,s=i.lineSpace.getBoundingClientRect();try{o=e.clientX-s.left,a=e.clientY-s.top;}catch(e){return null}var l,c=Jn(t,o,a);if(r&&1==c.xRel&&(l=Gt(t.doc,c.line).text).length==c.ch){var u=V(l,l.length,t.options.tabSize)-l.length;c=ee(c.line,Math.max(0,Math.round((o-Tn(t.display).left)/or(t.display))-u));}return c}function hr(t,e){if(e>=t.display.viewTo)return null;if((e-=t.display.viewFrom)<0)return null;for(var n=t.display.view,r=0;r<n.length;r++)if((e-=n[r].size)<0)return r}function dr(t,e,n,r){null==e&&(e=t.doc.first),null==n&&(n=t.doc.first+t.doc.size),r||(r=0);var i=t.display;if(r&&n<i.viewTo&&(null==i.updateLineNumbers||i.updateLineNumbers>e)&&(i.updateLineNumbers=e),t.curOp.viewChanged=!0,e>=i.viewTo)Ce&&He(t.doc,e)<i.viewTo&&pr(t);else if(n<=i.viewFrom)Ce&&ze(t.doc,n+r)>i.viewFrom?pr(t):(i.viewFrom+=r,i.viewTo+=r);else if(e<=i.viewFrom&&n>=i.viewTo)pr(t);else if(e<=i.viewFrom){var o=gr(t,n,n+r,1);o?(i.view=i.view.slice(o.index),i.viewFrom=o.lineN,i.viewTo+=r):pr(t);}else if(n>=i.viewTo){var a=gr(t,e,e,-1);a?(i.view=i.view.slice(0,a.index),i.viewTo=a.lineN):pr(t);}else {var s=gr(t,e,e,-1),l=gr(t,n,n+r,1);s&&l?(i.view=i.view.slice(0,s.index).concat(an(t,s.lineN,l.lineN)).concat(i.view.slice(l.index)),i.viewTo+=r):pr(t);}var c=i.externalMeasured;c&&(n<c.lineN?c.lineN+=r:e<c.lineN+c.size&&(i.externalMeasured=null));}function fr(t,e,n){t.curOp.viewChanged=!0;var r=t.display,i=t.display.externalMeasured;if(i&&e>=i.lineN&&e<i.lineN+i.size&&(r.externalMeasured=null),!(e<r.viewFrom||e>=r.viewTo)){var o=r.view[hr(t,e)];if(null!=o.node){var a=o.changes||(o.changes=[]);-1==H(a,n)&&a.push(n);}}}function pr(t){t.display.viewFrom=t.display.viewTo=t.doc.first,t.display.view=[],t.display.viewOffset=0;}function gr(t,e,n,r){var i,o=hr(t,e),a=t.display.view;if(!Ce||n==t.doc.first+t.doc.size)return {index:o,lineN:n};for(var s=t.display.viewFrom,l=0;l<o;l++)s+=a[l].size;if(s!=e){if(r>0){if(o==a.length-1)return null;i=s+a[o].size-e,o++;}else i=s-e;e+=i,n+=i;}for(;He(t.doc,n)!=n;){if(o==(r<0?0:a.length-1))return null;n+=r*a[o-(r<0?1:0)].size,o+=r;}return {index:o,lineN:n}}function vr(t){for(var e=t.display.view,n=0,r=0;r<e.length;r++){var i=e[r];i.hidden||i.node&&!i.changes||++n;}return n}function mr(t){t.display.input.showSelection(t.display.input.prepareSelection());}function yr(t,e){void 0===e&&(e=!0);for(var n=t.doc,r={},i=r.cursors=document.createDocumentFragment(),o=r.selection=document.createDocumentFragment(),a=0;a<n.sel.ranges.length;a++)if(e||a!=n.sel.primIndex){var s=n.sel.ranges[a];if(!(s.from().line>=t.display.viewTo||s.to().line<t.display.viewFrom)){var l=s.empty();(l||t.options.showCursorWhenSelecting)&&br(t,s.head,i),l||xr(t,s,o);}}return r}function br(t,e,n){var r=Kn(t,e,"div",null,null,!t.options.singleCursorHeightPerLine),i=n.appendChild(E("div"," ","CodeMirror-cursor"));if(i.style.left=r.left+"px",i.style.top=r.top+"px",i.style.height=Math.max(0,r.bottom-r.top)*t.options.cursorHeight+"px",r.other){var o=n.appendChild(E("div"," ","CodeMirror-cursor CodeMirror-secondarycursor"));o.style.display="",o.style.left=r.other.left+"px",o.style.top=r.other.top+"px",o.style.height=.85*(r.other.bottom-r.other.top)+"px";}}function wr(t,e){return t.top-e.top||t.left-e.left}function xr(t,e,n){var r=t.display,i=t.doc,o=document.createDocumentFragment(),a=Tn(t.display),s=a.left,l=Math.max(r.sizerWidth,Pn(t)-r.sizer.offsetLeft)-a.right,c="ltr"==i.direction;function u(t,e,n,r){e<0&&(e=0),e=Math.round(e),r=Math.round(r),o.appendChild(E("div",null,"CodeMirror-selected","position: absolute; left: "+t+"px;\n                             top: "+e+"px; width: "+(null==n?l-t:n)+"px;\n                             height: "+(r-e)+"px"));}function h(e,n,r){var o,a,h=Gt(i,e),d=h.text.length;function f(n,r){return Gn(t,ee(e,n),"div",h,r)}function p(e,n,r){var i=Qn(t,h,null,e),o="ltr"==n==("after"==r)?"left":"right";return f("after"==r?i.begin:i.end-(/\s/.test(h.text.charAt(i.end-1))?2:1),o)[o]}var g=ut(h,i.direction);return function(t,e,n,r){if(!t)return r(e,n,"ltr",0);for(var i=!1,o=0;o<t.length;++o){var a=t[o];(a.from<n&&a.to>e||e==n&&a.to==e)&&(r(Math.max(a.from,e),Math.min(a.to,n),1==a.level?"rtl":"ltr",o),i=!0);}i||r(e,n,"ltr");}(g,n||0,null==r?d:r,(function(t,e,i,h){var v="ltr"==i,m=f(t,v?"left":"right"),y=f(e-1,v?"right":"left"),b=null==n&&0==t,w=null==r&&e==d,x=0==h,O=!g||h==g.length-1;if(y.top-m.top<=3){var C=(c?w:b)&&O,S=(c?b:w)&&x?s:(v?m:y).left,k=C?l:(v?y:m).right;u(S,m.top,k-S,m.bottom);}else {var T,j,P,E;v?(T=c&&b&&x?s:m.left,j=c?l:p(t,i,"before"),P=c?s:p(e,i,"after"),E=c&&w&&O?l:y.right):(T=c?p(t,i,"before"):s,j=!c&&b&&x?l:m.right,P=!c&&w&&O?s:y.left,E=c?p(e,i,"after"):l),u(T,m.top,j-T,m.bottom),m.bottom<y.top&&u(s,m.bottom,null,y.top),u(P,y.top,E-P,y.bottom);}(!o||wr(m,o)<0)&&(o=m),wr(y,o)<0&&(o=y),(!a||wr(m,a)<0)&&(a=m),wr(y,a)<0&&(a=y);})),{start:o,end:a}}var d=e.from(),f=e.to();if(d.line==f.line)h(d.line,d.ch,f.ch);else {var p=Gt(i,d.line),g=Gt(i,f.line),v=Re(p)==Re(g),m=h(d.line,d.ch,v?p.text.length+1:null).end,y=h(f.line,v?0:null,f.ch).start;v&&(m.top<y.top-2?(u(m.right,m.top,null,m.bottom),u(s,y.top,y.left,y.bottom)):u(m.right,m.top,y.left-m.right,m.bottom)),m.bottom<y.top&&u(s,m.bottom,null,y.top);}n.appendChild(o);}function Or(t){if(t.state.focused){var e=t.display;clearInterval(e.blinker);var n=!0;e.cursorDiv.style.visibility="",t.options.cursorBlinkRate>0?e.blinker=setInterval((function(){return e.cursorDiv.style.visibility=(n=!n)?"":"hidden"}),t.options.cursorBlinkRate):t.options.cursorBlinkRate<0&&(e.cursorDiv.style.visibility="hidden");}}function Cr(t){t.state.focused||(t.display.input.focus(),kr(t));}function Sr(t){t.state.delayingBlurEvent=!0,setTimeout((function(){t.state.delayingBlurEvent&&(t.state.delayingBlurEvent=!1,Tr(t));}),100);}function kr(t,e){t.state.delayingBlurEvent&&(t.state.delayingBlurEvent=!1),"nocursor"!=t.options.readOnly&&(t.state.focused||(gt(t,"focus",t,e),t.state.focused=!0,L(t.display.wrapper,"CodeMirror-focused"),t.curOp||t.display.selForContextMenu==t.doc.sel||(t.display.input.reset(),l&&setTimeout((function(){return t.display.input.reset(!0)}),20)),t.display.input.receivedFocus()),Or(t));}function Tr(t,e){t.state.delayingBlurEvent||(t.state.focused&&(gt(t,"blur",t,e),t.state.focused=!1,T(t.display.wrapper,"CodeMirror-focused")),clearInterval(t.display.blinker),setTimeout((function(){t.state.focused||(t.display.shift=!1);}),150));}function jr(t){for(var e=t.display,n=e.lineDiv.offsetTop,r=0;r<e.view.length;r++){var i=e.view[r],o=t.options.lineWrapping,l=void 0,c=0;if(!i.hidden){if(a&&s<8){var u=i.node.offsetTop+i.node.offsetHeight;l=u-n,n=u;}else {var h=i.node.getBoundingClientRect();l=h.bottom-h.top,!o&&i.text.firstChild&&(c=i.text.firstChild.getBoundingClientRect().right-h.left-1);}var d=i.line.height-l;if((d>.005||d<-.005)&&(Xt(i.line,l),Pr(i.line),i.rest))for(var f=0;f<i.rest.length;f++)Pr(i.rest[f]);if(c>t.display.sizerWidth){var p=Math.ceil(c/or(t.display));p>t.display.maxLineLength&&(t.display.maxLineLength=p,t.display.maxLine=i.line,t.display.maxLineChanged=!0);}}}}function Pr(t){if(t.widgets)for(var e=0;e<t.widgets.length;++e){var n=t.widgets[e],r=n.node.parentNode;r&&(n.height=r.offsetHeight);}}function Er(t,e,n){var r=n&&null!=n.top?Math.max(0,n.top):t.scroller.scrollTop;r=Math.floor(r-Sn(t));var i=n&&null!=n.bottom?n.bottom:r+t.wrapper.clientHeight,o=Zt(e,r),a=Zt(e,i);if(n&&n.ensure){var s=n.ensure.from.line,l=n.ensure.to.line;s<o?(o=s,a=Zt(e,Be(Gt(e,s))+t.wrapper.clientHeight)):Math.min(l,e.lastLine())>=a&&(o=Zt(e,Be(Gt(e,l))-t.wrapper.clientHeight),a=l);}return {from:o,to:Math.max(a,o+1)}}function Mr(t,e){var n=t.display,r=ir(t.display);e.top<0&&(e.top=0);var i=t.curOp&&null!=t.curOp.scrollTop?t.curOp.scrollTop:n.scroller.scrollTop,o=En(t),a={};e.bottom-e.top>o&&(e.bottom=e.top+o);var s=t.doc.height+kn(n),l=e.top<r,c=e.bottom>s-r;if(e.top<i)a.scrollTop=l?0:e.top;else if(e.bottom>i+o){var u=Math.min(e.top,(c?s:e.bottom)-o);u!=i&&(a.scrollTop=u);}var h=t.curOp&&null!=t.curOp.scrollLeft?t.curOp.scrollLeft:n.scroller.scrollLeft,d=Pn(t)-(t.options.fixedGutter?n.gutters.offsetWidth:0),f=e.right-e.left>d;return f&&(e.right=e.left+d),e.left<10?a.scrollLeft=0:e.left<h?a.scrollLeft=Math.max(0,e.left-(f?0:10)):e.right>d+h-3&&(a.scrollLeft=e.right+(f?0:10)-d),a}function Dr(t,e){null!=e&&(Nr(t),t.curOp.scrollTop=(null==t.curOp.scrollTop?t.doc.scrollTop:t.curOp.scrollTop)+e);}function Ar(t){Nr(t);var e=t.getCursor();t.curOp.scrollToPos={from:e,to:e,margin:t.options.cursorScrollMargin};}function Lr(t,e,n){null==e&&null==n||Nr(t),null!=e&&(t.curOp.scrollLeft=e),null!=n&&(t.curOp.scrollTop=n);}function Nr(t){var e=t.curOp.scrollToPos;e&&(t.curOp.scrollToPos=null,Ir(t,Yn(t,e.from),Yn(t,e.to),e.margin));}function Ir(t,e,n,r){var i=Mr(t,{left:Math.min(e.left,n.left),top:Math.min(e.top,n.top)-r,right:Math.max(e.right,n.right),bottom:Math.max(e.bottom,n.bottom)+r});Lr(t,i.scrollLeft,i.scrollTop);}function Fr(t,e){Math.abs(t.doc.scrollTop-e)<2||(n||li(t,{top:e}),_r(t,e,!0),n&&li(t),ri(t,100));}function _r(t,e,n){e=Math.min(t.display.scroller.scrollHeight-t.display.scroller.clientHeight,e),(t.display.scroller.scrollTop!=e||n)&&(t.doc.scrollTop=e,t.display.scrollbars.setScrollTop(e),t.display.scroller.scrollTop!=e&&(t.display.scroller.scrollTop=e));}function Vr(t,e,n,r){e=Math.min(e,t.display.scroller.scrollWidth-t.display.scroller.clientWidth),(n?e==t.doc.scrollLeft:Math.abs(t.doc.scrollLeft-e)<2)&&!r||(t.doc.scrollLeft=e,hi(t),t.display.scroller.scrollLeft!=e&&(t.display.scroller.scrollLeft=e),t.display.scrollbars.setScrollLeft(e));}function Rr(t){var e=t.display,n=e.gutters.offsetWidth,r=Math.round(t.doc.height+kn(t.display));return {clientHeight:e.scroller.clientHeight,viewHeight:e.wrapper.clientHeight,scrollWidth:e.scroller.scrollWidth,clientWidth:e.scroller.clientWidth,viewWidth:e.wrapper.clientWidth,barLeft:t.options.fixedGutter?n:0,docHeight:r,scrollHeight:r+jn(t)+e.barHeight,nativeBarWidth:e.nativeBarWidth,gutterWidth:n}}var Hr=function(t,e,n){this.cm=n;var r=this.vert=E("div",[E("div",null,null,"min-width: 1px")],"CodeMirror-vscrollbar"),i=this.horiz=E("div",[E("div",null,null,"height: 100%; min-height: 1px")],"CodeMirror-hscrollbar");r.tabIndex=i.tabIndex=-1,t(r),t(i),dt(r,"scroll",(function(){r.clientHeight&&e(r.scrollTop,"vertical");})),dt(i,"scroll",(function(){i.clientWidth&&e(i.scrollLeft,"horizontal");})),this.checkedZeroWidth=!1,a&&s<8&&(this.horiz.style.minHeight=this.vert.style.minWidth="18px");};Hr.prototype.update=function(t){var e=t.scrollWidth>t.clientWidth+1,n=t.scrollHeight>t.clientHeight+1,r=t.nativeBarWidth;if(n){this.vert.style.display="block",this.vert.style.bottom=e?r+"px":"0";var i=t.viewHeight-(e?r:0);this.vert.firstChild.style.height=Math.max(0,t.scrollHeight-t.clientHeight+i)+"px";}else this.vert.style.display="",this.vert.firstChild.style.height="0";if(e){this.horiz.style.display="block",this.horiz.style.right=n?r+"px":"0",this.horiz.style.left=t.barLeft+"px";var o=t.viewWidth-t.barLeft-(n?r:0);this.horiz.firstChild.style.width=Math.max(0,t.scrollWidth-t.clientWidth+o)+"px";}else this.horiz.style.display="",this.horiz.firstChild.style.width="0";return !this.checkedZeroWidth&&t.clientHeight>0&&(0==r&&this.zeroWidthHack(),this.checkedZeroWidth=!0),{right:n?r:0,bottom:e?r:0}},Hr.prototype.setScrollLeft=function(t){this.horiz.scrollLeft!=t&&(this.horiz.scrollLeft=t),this.disableHoriz&&this.enableZeroWidthBar(this.horiz,this.disableHoriz,"horiz");},Hr.prototype.setScrollTop=function(t){this.vert.scrollTop!=t&&(this.vert.scrollTop=t),this.disableVert&&this.enableZeroWidthBar(this.vert,this.disableVert,"vert");},Hr.prototype.zeroWidthHack=function(){var t=y&&!f?"12px":"18px";this.horiz.style.height=this.vert.style.width=t,this.horiz.style.pointerEvents=this.vert.style.pointerEvents="none",this.disableHoriz=new R,this.disableVert=new R;},Hr.prototype.enableZeroWidthBar=function(t,e,n){t.style.pointerEvents="auto",e.set(1e3,(function r(){var i=t.getBoundingClientRect();("vert"==n?document.elementFromPoint(i.right-1,(i.top+i.bottom)/2):document.elementFromPoint((i.right+i.left)/2,i.bottom-1))!=t?t.style.pointerEvents="none":e.set(1e3,r);}));},Hr.prototype.clear=function(){var t=this.horiz.parentNode;t.removeChild(this.horiz),t.removeChild(this.vert);};var zr=function(){};function $r(t,e){e||(e=Rr(t));var n=t.display.barWidth,r=t.display.barHeight;Wr(t,e);for(var i=0;i<4&&n!=t.display.barWidth||r!=t.display.barHeight;i++)n!=t.display.barWidth&&t.options.lineWrapping&&jr(t),Wr(t,Rr(t)),n=t.display.barWidth,r=t.display.barHeight;}function Wr(t,e){var n=t.display,r=n.scrollbars.update(e);n.sizer.style.paddingRight=(n.barWidth=r.right)+"px",n.sizer.style.paddingBottom=(n.barHeight=r.bottom)+"px",n.heightForcer.style.borderBottom=r.bottom+"px solid transparent",r.right&&r.bottom?(n.scrollbarFiller.style.display="block",n.scrollbarFiller.style.height=r.bottom+"px",n.scrollbarFiller.style.width=r.right+"px"):n.scrollbarFiller.style.display="",r.bottom&&t.options.coverGutterNextToScrollbar&&t.options.fixedGutter?(n.gutterFiller.style.display="block",n.gutterFiller.style.height=r.bottom+"px",n.gutterFiller.style.width=e.gutterWidth+"px"):n.gutterFiller.style.display="";}zr.prototype.update=function(){return {bottom:0,right:0}},zr.prototype.setScrollLeft=function(){},zr.prototype.setScrollTop=function(){},zr.prototype.clear=function(){};var Br={native:Hr,null:zr};function Ur(t){t.display.scrollbars&&(t.display.scrollbars.clear(),t.display.scrollbars.addClass&&T(t.display.wrapper,t.display.scrollbars.addClass)),t.display.scrollbars=new Br[t.options.scrollbarStyle]((function(e){t.display.wrapper.insertBefore(e,t.display.scrollbarFiller),dt(e,"mousedown",(function(){t.state.focused&&setTimeout((function(){return t.display.input.focus()}),0);})),e.setAttribute("cm-not-content","true");}),(function(e,n){"horizontal"==n?Vr(t,e):Fr(t,e);}),t),t.display.scrollbars.addClass&&L(t.display.wrapper,t.display.scrollbars.addClass);}var qr=0;function Gr(t){var e;t.curOp={cm:t,viewChanged:!1,startHeight:t.doc.height,forceUpdate:!1,updateInput:0,typing:!1,changeObjs:null,cursorActivityHandlers:null,cursorActivityCalled:0,selectionChanged:!1,updateMaxLine:!1,scrollLeft:null,scrollTop:null,scrollToPos:null,focus:!1,id:++qr},e=t.curOp,sn?sn.ops.push(e):e.ownsGroup=sn={ops:[e],delayedCallbacks:[]};}function Kr(t){var e=t.curOp;e&&function(t,e){var n=t.ownsGroup;if(n)try{!function(t){var e=t.delayedCallbacks,n=0;do{for(;n<e.length;n++)e[n].call(null);for(var r=0;r<t.ops.length;r++){var i=t.ops[r];if(i.cursorActivityHandlers)for(;i.cursorActivityCalled<i.cursorActivityHandlers.length;)i.cursorActivityHandlers[i.cursorActivityCalled++].call(null,i.cm);}}while(n<e.length)}(n);}finally{sn=null,e(n);}}(e,(function(t){for(var e=0;e<t.ops.length;e++)t.ops[e].cm.curOp=null;!function(t){for(var e=t.ops,n=0;n<e.length;n++)Yr(e[n]);for(var r=0;r<e.length;r++)(i=e[r]).updatedDisplay=i.mustUpdate&&ai(i.cm,i.update);for(var i,o=0;o<e.length;o++)Xr(e[o]);for(var a=0;a<e.length;a++)Jr(e[a]);for(var s=0;s<e.length;s++)Zr(e[s]);}(t);}));}function Yr(t){var e=t.cm,n=e.display;!function(t){var e=t.display;!e.scrollbarsClipped&&e.scroller.offsetWidth&&(e.nativeBarWidth=e.scroller.offsetWidth-e.scroller.clientWidth,e.heightForcer.style.height=jn(t)+"px",e.sizer.style.marginBottom=-e.nativeBarWidth+"px",e.sizer.style.borderRightWidth=jn(t)+"px",e.scrollbarsClipped=!0);}(e),t.updateMaxLine&&qe(e),t.mustUpdate=t.viewChanged||t.forceUpdate||null!=t.scrollTop||t.scrollToPos&&(t.scrollToPos.from.line<n.viewFrom||t.scrollToPos.to.line>=n.viewTo)||n.maxLineChanged&&e.options.lineWrapping,t.update=t.mustUpdate&&new oi(e,t.mustUpdate&&{top:t.scrollTop,ensure:t.scrollToPos},t.forceUpdate);}function Xr(t){var e=t.cm,n=e.display;t.updatedDisplay&&jr(e),t.barMeasure=Rr(e),n.maxLineChanged&&!e.options.lineWrapping&&(t.adjustWidthTo=Dn(e,n.maxLine,n.maxLine.text.length).left+3,e.display.sizerWidth=t.adjustWidthTo,t.barMeasure.scrollWidth=Math.max(n.scroller.clientWidth,n.sizer.offsetLeft+t.adjustWidthTo+jn(e)+e.display.barWidth),t.maxScrollLeft=Math.max(0,n.sizer.offsetLeft+t.adjustWidthTo-Pn(e))),(t.updatedDisplay||t.selectionChanged)&&(t.preparedSelection=n.input.prepareSelection());}function Jr(t){var e=t.cm;null!=t.adjustWidthTo&&(e.display.sizer.style.minWidth=t.adjustWidthTo+"px",t.maxScrollLeft<e.doc.scrollLeft&&Vr(e,Math.min(e.display.scroller.scrollLeft,t.maxScrollLeft),!0),e.display.maxLineChanged=!1);var n=t.focus&&t.focus==A();t.preparedSelection&&e.display.input.showSelection(t.preparedSelection,n),(t.updatedDisplay||t.startHeight!=e.doc.height)&&$r(e,t.barMeasure),t.updatedDisplay&&ui(e,t.barMeasure),t.selectionChanged&&Or(e),e.state.focused&&t.updateInput&&e.display.input.reset(t.typing),n&&Cr(t.cm);}function Zr(t){var e=t.cm,n=e.display,r=e.doc;t.updatedDisplay&&si(e,t.update),null==n.wheelStartX||null==t.scrollTop&&null==t.scrollLeft&&!t.scrollToPos||(n.wheelStartX=n.wheelStartY=null),null!=t.scrollTop&&_r(e,t.scrollTop,t.forceScroll),null!=t.scrollLeft&&Vr(e,t.scrollLeft,!0,!0),t.scrollToPos&&function(t,e){if(!vt(t,"scrollCursorIntoView")){var n=t.display,r=n.sizer.getBoundingClientRect(),i=null;if(e.top+r.top<0?i=!0:e.bottom+r.top>(window.innerHeight||document.documentElement.clientHeight)&&(i=!1),null!=i&&!p){var o=E("div","​",null,"position: absolute;\n                         top: "+(e.top-n.viewOffset-Sn(t.display))+"px;\n                         height: "+(e.bottom-e.top+jn(t)+n.barHeight)+"px;\n                         left: "+e.left+"px; width: "+Math.max(2,e.right-e.left)+"px;");t.display.lineSpace.appendChild(o),o.scrollIntoView(i),t.display.lineSpace.removeChild(o);}}}(e,function(t,e,n,r){var i;null==r&&(r=0),t.options.lineWrapping||e!=n||(n="before"==(e=e.ch?ee(e.line,"before"==e.sticky?e.ch-1:e.ch,"after"):e).sticky?ee(e.line,e.ch+1,"before"):e);for(var o=0;o<5;o++){var a=!1,s=Kn(t,e),l=n&&n!=e?Kn(t,n):s,c=Mr(t,i={left:Math.min(s.left,l.left),top:Math.min(s.top,l.top)-r,right:Math.max(s.left,l.left),bottom:Math.max(s.bottom,l.bottom)+r}),u=t.doc.scrollTop,h=t.doc.scrollLeft;if(null!=c.scrollTop&&(Fr(t,c.scrollTop),Math.abs(t.doc.scrollTop-u)>1&&(a=!0)),null!=c.scrollLeft&&(Vr(t,c.scrollLeft),Math.abs(t.doc.scrollLeft-h)>1&&(a=!0)),!a)break}return i}(e,le(r,t.scrollToPos.from),le(r,t.scrollToPos.to),t.scrollToPos.margin));var i=t.maybeHiddenMarkers,o=t.maybeUnhiddenMarkers;if(i)for(var a=0;a<i.length;++a)i[a].lines.length||gt(i[a],"hide");if(o)for(var s=0;s<o.length;++s)o[s].lines.length&&gt(o[s],"unhide");n.wrapper.offsetHeight&&(r.scrollTop=e.display.scroller.scrollTop),t.changeObjs&&gt(e,"changes",e,t.changeObjs),t.update&&t.update.finish();}function Qr(t,e){if(t.curOp)return e();Gr(t);try{return e()}finally{Kr(t);}}function ti(t,e){return function(){if(t.curOp)return e.apply(t,arguments);Gr(t);try{return e.apply(t,arguments)}finally{Kr(t);}}}function ei(t){return function(){if(this.curOp)return t.apply(this,arguments);Gr(this);try{return t.apply(this,arguments)}finally{Kr(this);}}}function ni(t){return function(){var e=this.cm;if(!e||e.curOp)return t.apply(this,arguments);Gr(e);try{return t.apply(this,arguments)}finally{Kr(e);}}}function ri(t,e){t.doc.highlightFrontier<t.display.viewTo&&t.state.highlight.set(e,F(ii,t));}function ii(t){var e=t.doc;if(!(e.highlightFrontier>=t.display.viewTo)){var n=+new Date+t.options.workTime,r=pe(t,e.highlightFrontier),i=[];e.iter(r.line,Math.min(e.first+e.size,t.display.viewTo+500),(function(o){if(r.line>=t.display.viewFrom){var a=o.styles,s=o.text.length>t.options.maxHighlightLength?Wt(e.mode,r.state):null,l=de(t,o,r,!0);s&&(r.state=s),o.styles=l.styles;var c=o.styleClasses,u=l.classes;u?o.styleClasses=u:c&&(o.styleClasses=null);for(var h=!a||a.length!=o.styles.length||c!=u&&(!c||!u||c.bgClass!=u.bgClass||c.textClass!=u.textClass),d=0;!h&&d<a.length;++d)h=a[d]!=o.styles[d];h&&i.push(r.line),o.stateAfter=r.save(),r.nextLine();}else o.text.length<=t.options.maxHighlightLength&&ge(t,o.text,r),o.stateAfter=r.line%5==0?r.save():null,r.nextLine();if(+new Date>n)return ri(t,t.options.workDelay),!0})),e.highlightFrontier=r.line,e.modeFrontier=Math.max(e.modeFrontier,r.line),i.length&&Qr(t,(function(){for(var e=0;e<i.length;e++)fr(t,i[e],"text");}));}}var oi=function(t,e,n){var r=t.display;this.viewport=e,this.visible=Er(r,t.doc,e),this.editorIsHidden=!r.wrapper.offsetWidth,this.wrapperHeight=r.wrapper.clientHeight,this.wrapperWidth=r.wrapper.clientWidth,this.oldDisplayWidth=Pn(t),this.force=n,this.dims=ar(t),this.events=[];};function ai(t,e){var n=t.display,r=t.doc;if(e.editorIsHidden)return pr(t),!1;if(!e.force&&e.visible.from>=n.viewFrom&&e.visible.to<=n.viewTo&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo)&&n.renderedView==n.view&&0==vr(t))return !1;di(t)&&(pr(t),e.dims=ar(t));var i=r.first+r.size,o=Math.max(e.visible.from-t.options.viewportMargin,r.first),a=Math.min(i,e.visible.to+t.options.viewportMargin);n.viewFrom<o&&o-n.viewFrom<20&&(o=Math.max(r.first,n.viewFrom)),n.viewTo>a&&n.viewTo-a<20&&(a=Math.min(i,n.viewTo)),Ce&&(o=He(t.doc,o),a=ze(t.doc,a));var s=o!=n.viewFrom||a!=n.viewTo||n.lastWrapHeight!=e.wrapperHeight||n.lastWrapWidth!=e.wrapperWidth;!function(t,e,n){var r=t.display;0==r.view.length||e>=r.viewTo||n<=r.viewFrom?(r.view=an(t,e,n),r.viewFrom=e):(r.viewFrom>e?r.view=an(t,e,r.viewFrom).concat(r.view):r.viewFrom<e&&(r.view=r.view.slice(hr(t,e))),r.viewFrom=e,r.viewTo<n?r.view=r.view.concat(an(t,r.viewTo,n)):r.viewTo>n&&(r.view=r.view.slice(0,hr(t,n)))),r.viewTo=n;}(t,o,a),n.viewOffset=Be(Gt(t.doc,n.viewFrom)),t.display.mover.style.top=n.viewOffset+"px";var c=vr(t);if(!s&&0==c&&!e.force&&n.renderedView==n.view&&(null==n.updateLineNumbers||n.updateLineNumbers>=n.viewTo))return !1;var u=function(t){if(t.hasFocus())return null;var e=A();if(!e||!D(t.display.lineDiv,e))return null;var n={activeElt:e};if(window.getSelection){var r=window.getSelection();r.anchorNode&&r.extend&&D(t.display.lineDiv,r.anchorNode)&&(n.anchorNode=r.anchorNode,n.anchorOffset=r.anchorOffset,n.focusNode=r.focusNode,n.focusOffset=r.focusOffset);}return n}(t);return c>4&&(n.lineDiv.style.display="none"),function(t,e,n){var r=t.display,i=t.options.lineNumbers,o=r.lineDiv,a=o.firstChild;function s(e){var n=e.nextSibling;return l&&y&&t.display.currentWheelTarget==e?e.style.display="none":e.parentNode.removeChild(e),n}for(var c=r.view,u=r.viewFrom,h=0;h<c.length;h++){var d=c[h];if(d.hidden);else if(d.node&&d.node.parentNode==o){for(;a!=d.node;)a=s(a);var f=i&&null!=e&&e<=u&&d.lineNumber;d.changes&&(H(d.changes,"gutter")>-1&&(f=!1),hn(t,d,u,n)),f&&(j(d.lineNumber),d.lineNumber.appendChild(document.createTextNode(te(t.options,u)))),a=d.node.nextSibling;}else {var p=yn(t,d,u,n);o.insertBefore(p,a);}u+=d.size;}for(;a;)a=s(a);}(t,n.updateLineNumbers,e.dims),c>4&&(n.lineDiv.style.display=""),n.renderedView=n.view,function(t){if(t&&t.activeElt&&t.activeElt!=A()&&(t.activeElt.focus(),t.anchorNode&&D(document.body,t.anchorNode)&&D(document.body,t.focusNode))){var e=window.getSelection(),n=document.createRange();n.setEnd(t.anchorNode,t.anchorOffset),n.collapse(!1),e.removeAllRanges(),e.addRange(n),e.extend(t.focusNode,t.focusOffset);}}(u),j(n.cursorDiv),j(n.selectionDiv),n.gutters.style.height=n.sizer.style.minHeight=0,s&&(n.lastWrapHeight=e.wrapperHeight,n.lastWrapWidth=e.wrapperWidth,ri(t,400)),n.updateLineNumbers=null,!0}function si(t,e){for(var n=e.viewport,r=!0;(r&&t.options.lineWrapping&&e.oldDisplayWidth!=Pn(t)||(n&&null!=n.top&&(n={top:Math.min(t.doc.height+kn(t.display)-En(t),n.top)}),e.visible=Er(t.display,t.doc,n),!(e.visible.from>=t.display.viewFrom&&e.visible.to<=t.display.viewTo)))&&ai(t,e);r=!1){jr(t);var i=Rr(t);mr(t),$r(t,i),ui(t,i),e.force=!1;}e.signal(t,"update",t),t.display.viewFrom==t.display.reportedViewFrom&&t.display.viewTo==t.display.reportedViewTo||(e.signal(t,"viewportChange",t,t.display.viewFrom,t.display.viewTo),t.display.reportedViewFrom=t.display.viewFrom,t.display.reportedViewTo=t.display.viewTo);}function li(t,e){var n=new oi(t,e);if(ai(t,n)){jr(t),si(t,n);var r=Rr(t);mr(t),$r(t,r),ui(t,r),n.finish();}}function ci(t){var e=t.gutters.offsetWidth;t.sizer.style.marginLeft=e+"px";}function ui(t,e){t.display.sizer.style.minHeight=e.docHeight+"px",t.display.heightForcer.style.top=e.docHeight+"px",t.display.gutters.style.height=e.docHeight+t.display.barHeight+jn(t)+"px";}function hi(t){var e=t.display,n=e.view;if(e.alignWidgets||e.gutters.firstChild&&t.options.fixedGutter){for(var r=sr(e)-e.scroller.scrollLeft+t.doc.scrollLeft,i=e.gutters.offsetWidth,o=r+"px",a=0;a<n.length;a++)if(!n[a].hidden){t.options.fixedGutter&&(n[a].gutter&&(n[a].gutter.style.left=o),n[a].gutterBackground&&(n[a].gutterBackground.style.left=o));var s=n[a].alignable;if(s)for(var l=0;l<s.length;l++)s[l].style.left=o;}t.options.fixedGutter&&(e.gutters.style.left=r+i+"px");}}function di(t){if(!t.options.lineNumbers)return !1;var e=t.doc,n=te(t.options,e.first+e.size-1),r=t.display;if(n.length!=r.lineNumChars){var i=r.measure.appendChild(E("div",[E("div",n)],"CodeMirror-linenumber CodeMirror-gutter-elt")),o=i.firstChild.offsetWidth,a=i.offsetWidth-o;return r.lineGutter.style.width="",r.lineNumInnerWidth=Math.max(o,r.lineGutter.offsetWidth-a)+1,r.lineNumWidth=r.lineNumInnerWidth+a,r.lineNumChars=r.lineNumInnerWidth?n.length:-1,r.lineGutter.style.width=r.lineNumWidth+"px",ci(t.display),!0}return !1}function fi(t,e){for(var n=[],r=!1,i=0;i<t.length;i++){var o=t[i],a=null;if("string"!=typeof o&&(a=o.style,o=o.className),"CodeMirror-linenumbers"==o){if(!e)continue;r=!0;}n.push({className:o,style:a});}return e&&!r&&n.push({className:"CodeMirror-linenumbers",style:null}),n}function pi(t){var e=t.gutters,n=t.gutterSpecs;j(e),t.lineGutter=null;for(var r=0;r<n.length;++r){var i=n[r],o=i.className,a=i.style,s=e.appendChild(E("div",null,"CodeMirror-gutter "+o));a&&(s.style.cssText=a),"CodeMirror-linenumbers"==o&&(t.lineGutter=s,s.style.width=(t.lineNumWidth||1)+"px");}e.style.display=n.length?"":"none",ci(t);}function gi(t){pi(t.display),dr(t),hi(t);}function vi(t,e,r,i){var o=this;this.input=r,o.scrollbarFiller=E("div",null,"CodeMirror-scrollbar-filler"),o.scrollbarFiller.setAttribute("cm-not-content","true"),o.gutterFiller=E("div",null,"CodeMirror-gutter-filler"),o.gutterFiller.setAttribute("cm-not-content","true"),o.lineDiv=M("div",null,"CodeMirror-code"),o.selectionDiv=E("div",null,null,"position: relative; z-index: 1"),o.cursorDiv=E("div",null,"CodeMirror-cursors"),o.measure=E("div",null,"CodeMirror-measure"),o.lineMeasure=E("div",null,"CodeMirror-measure"),o.lineSpace=M("div",[o.measure,o.lineMeasure,o.selectionDiv,o.cursorDiv,o.lineDiv],null,"position: relative; outline: none");var c=M("div",[o.lineSpace],"CodeMirror-lines");o.mover=E("div",[c],null,"position: relative"),o.sizer=E("div",[o.mover],"CodeMirror-sizer"),o.sizerWidth=null,o.heightForcer=E("div",null,null,"position: absolute; height: "+z+"px; width: 1px;"),o.gutters=E("div",null,"CodeMirror-gutters"),o.lineGutter=null,o.scroller=E("div",[o.sizer,o.heightForcer,o.gutters],"CodeMirror-scroll"),o.scroller.setAttribute("tabIndex","-1"),o.wrapper=E("div",[o.scrollbarFiller,o.gutterFiller,o.scroller],"CodeMirror"),a&&s<8&&(o.gutters.style.zIndex=-1,o.scroller.style.paddingRight=0),l||n&&m||(o.scroller.draggable=!0),t&&(t.appendChild?t.appendChild(o.wrapper):t(o.wrapper)),o.viewFrom=o.viewTo=e.first,o.reportedViewFrom=o.reportedViewTo=e.first,o.view=[],o.renderedView=null,o.externalMeasured=null,o.viewOffset=0,o.lastWrapHeight=o.lastWrapWidth=0,o.updateLineNumbers=null,o.nativeBarWidth=o.barHeight=o.barWidth=0,o.scrollbarsClipped=!1,o.lineNumWidth=o.lineNumInnerWidth=o.lineNumChars=null,o.alignWidgets=!1,o.cachedCharWidth=o.cachedTextHeight=o.cachedPaddingH=null,o.maxLine=null,o.maxLineLength=0,o.maxLineChanged=!1,o.wheelDX=o.wheelDY=o.wheelStartX=o.wheelStartY=null,o.shift=!1,o.selForContextMenu=null,o.activeTouch=null,o.gutterSpecs=fi(i.gutters,i.lineNumbers),pi(o),r.init(o);}oi.prototype.signal=function(t,e){yt(t,e)&&this.events.push(arguments);},oi.prototype.finish=function(){for(var t=0;t<this.events.length;t++)gt.apply(null,this.events[t]);};var mi=0,yi=null;function bi(t){var e=t.wheelDeltaX,n=t.wheelDeltaY;return null==e&&t.detail&&t.axis==t.HORIZONTAL_AXIS&&(e=t.detail),null==n&&t.detail&&t.axis==t.VERTICAL_AXIS?n=t.detail:null==n&&(n=t.wheelDelta),{x:e,y:n}}function wi(t){var e=bi(t);return e.x*=yi,e.y*=yi,e}function xi(t,e){var r=bi(e),i=r.x,o=r.y,a=t.display,s=a.scroller,c=s.scrollWidth>s.clientWidth,u=s.scrollHeight>s.clientHeight;if(i&&c||o&&u){if(o&&y&&l)t:for(var d=e.target,f=a.view;d!=s;d=d.parentNode)for(var p=0;p<f.length;p++)if(f[p].node==d){t.display.currentWheelTarget=d;break t}if(i&&!n&&!h&&null!=yi)return o&&u&&Fr(t,Math.max(0,s.scrollTop+o*yi)),Vr(t,Math.max(0,s.scrollLeft+i*yi)),(!o||o&&u)&&wt(e),void(a.wheelStartX=null);if(o&&null!=yi){var g=o*yi,v=t.doc.scrollTop,m=v+a.wrapper.clientHeight;g<0?v=Math.max(0,v+g-50):m=Math.min(t.doc.height,m+g+50),li(t,{top:v,bottom:m});}mi<20&&(null==a.wheelStartX?(a.wheelStartX=s.scrollLeft,a.wheelStartY=s.scrollTop,a.wheelDX=i,a.wheelDY=o,setTimeout((function(){if(null!=a.wheelStartX){var t=s.scrollLeft-a.wheelStartX,e=s.scrollTop-a.wheelStartY,n=e&&a.wheelDY&&e/a.wheelDY||t&&a.wheelDX&&t/a.wheelDX;a.wheelStartX=a.wheelStartY=null,n&&(yi=(yi*mi+n)/(mi+1),++mi);}}),200)):(a.wheelDX+=i,a.wheelDY+=o));}}a?yi=-.53:n?yi=15:u?yi=-.7:d&&(yi=-1/3);var Oi=function(t,e){this.ranges=t,this.primIndex=e;};Oi.prototype.primary=function(){return this.ranges[this.primIndex]},Oi.prototype.equals=function(t){if(t==this)return !0;if(t.primIndex!=this.primIndex||t.ranges.length!=this.ranges.length)return !1;for(var e=0;e<this.ranges.length;e++){var n=this.ranges[e],r=t.ranges[e];if(!re(n.anchor,r.anchor)||!re(n.head,r.head))return !1}return !0},Oi.prototype.deepCopy=function(){for(var t=[],e=0;e<this.ranges.length;e++)t[e]=new Ci(ie(this.ranges[e].anchor),ie(this.ranges[e].head));return new Oi(t,this.primIndex)},Oi.prototype.somethingSelected=function(){for(var t=0;t<this.ranges.length;t++)if(!this.ranges[t].empty())return !0;return !1},Oi.prototype.contains=function(t,e){e||(e=t);for(var n=0;n<this.ranges.length;n++){var r=this.ranges[n];if(ne(e,r.from())>=0&&ne(t,r.to())<=0)return n}return -1};var Ci=function(t,e){this.anchor=t,this.head=e;};function Si(t,e,n){var r=t&&t.options.selectionsMayTouch,i=e[n];e.sort((function(t,e){return ne(t.from(),e.from())})),n=H(e,i);for(var o=1;o<e.length;o++){var a=e[o],s=e[o-1],l=ne(s.to(),a.from());if(r&&!a.empty()?l>0:l>=0){var c=ae(s.from(),a.from()),u=oe(s.to(),a.to()),h=s.empty()?a.from()==a.head:s.from()==s.head;o<=n&&--n,e.splice(--o,2,new Ci(h?u:c,h?c:u));}}return new Oi(e,n)}function ki(t,e){return new Oi([new Ci(t,e||t)],0)}function Ti(t){return t.text?ee(t.from.line+t.text.length-1,Y(t.text).length+(1==t.text.length?t.from.ch:0)):t.to}function ji(t,e){if(ne(t,e.from)<0)return t;if(ne(t,e.to)<=0)return Ti(e);var n=t.line+e.text.length-(e.to.line-e.from.line)-1,r=t.ch;return t.line==e.to.line&&(r+=Ti(e).ch-e.to.ch),ee(n,r)}function Pi(t,e){for(var n=[],r=0;r<t.sel.ranges.length;r++){var i=t.sel.ranges[r];n.push(new Ci(ji(i.anchor,e),ji(i.head,e)));}return Si(t.cm,n,t.sel.primIndex)}function Ei(t,e,n){return t.line==e.line?ee(n.line,t.ch-e.ch+n.ch):ee(n.line+(t.line-e.line),t.ch)}function Mi(t){t.doc.mode=Ht(t.options,t.doc.modeOption),Di(t);}function Di(t){t.doc.iter((function(t){t.stateAfter&&(t.stateAfter=null),t.styles&&(t.styles=null);})),t.doc.modeFrontier=t.doc.highlightFrontier=t.doc.first,ri(t,100),t.state.modeGen++,t.curOp&&dr(t);}function Ai(t,e){return 0==e.from.ch&&0==e.to.ch&&""==Y(e.text)&&(!t.cm||t.cm.options.wholeLineUpdateBefore)}function Li(t,e,n,r){function i(t){return n?n[t]:null}function o(t,n,i){!function(t,e,n,r){t.text=e,t.stateAfter&&(t.stateAfter=null),t.styles&&(t.styles=null),null!=t.order&&(t.order=null),Ee(t),Me(t,n);var i=r?r(t):1;i!=t.height&&Xt(t,i);}(t,n,i,r),cn(t,"change",t,e);}function a(t,e){for(var n=[],o=t;o<e;++o)n.push(new Ge(c[o],i(o),r));return n}var s=e.from,l=e.to,c=e.text,u=Gt(t,s.line),h=Gt(t,l.line),d=Y(c),f=i(c.length-1),p=l.line-s.line;if(e.full)t.insert(0,a(0,c.length)),t.remove(c.length,t.size-c.length);else if(Ai(t,e)){var g=a(0,c.length-1);o(h,h.text,f),p&&t.remove(s.line,p),g.length&&t.insert(s.line,g);}else if(u==h)if(1==c.length)o(u,u.text.slice(0,s.ch)+d+u.text.slice(l.ch),f);else {var v=a(1,c.length-1);v.push(new Ge(d+u.text.slice(l.ch),f,r)),o(u,u.text.slice(0,s.ch)+c[0],i(0)),t.insert(s.line+1,v);}else if(1==c.length)o(u,u.text.slice(0,s.ch)+c[0]+h.text.slice(l.ch),i(0)),t.remove(s.line+1,p);else {o(u,u.text.slice(0,s.ch)+c[0],i(0)),o(h,d+h.text.slice(l.ch),f);var m=a(1,c.length-1);p>1&&t.remove(s.line+1,p-1),t.insert(s.line+1,m);}cn(t,"change",t,e);}function Ni(t,e,n){!function t(r,i,o){if(r.linked)for(var a=0;a<r.linked.length;++a){var s=r.linked[a];if(s.doc!=i){var l=o&&s.sharedHist;n&&!l||(e(s.doc,l),t(s.doc,r,l));}}}(t,null,!0);}function Ii(t,e){if(e.cm)throw new Error("This document is already in use.");t.doc=e,e.cm=t,cr(t),Mi(t),Fi(t),t.options.lineWrapping||qe(t),t.options.mode=e.modeOption,dr(t);}function Fi(t){("rtl"==t.doc.direction?L:T)(t.display.lineDiv,"CodeMirror-rtl");}function _i(t){this.done=[],this.undone=[],this.undoDepth=1/0,this.lastModTime=this.lastSelTime=0,this.lastOp=this.lastSelOp=null,this.lastOrigin=this.lastSelOrigin=null,this.generation=this.maxGeneration=t||1;}function Vi(t,e){var n={from:ie(e.from),to:Ti(e),text:Kt(t,e.from,e.to)};return Wi(t,n,e.from.line,e.to.line+1),Ni(t,(function(t){return Wi(t,n,e.from.line,e.to.line+1)}),!0),n}function Ri(t){for(;t.length&&Y(t).ranges;)t.pop();}function Hi(t,e,n,r){var i=t.history;i.undone.length=0;var o,a,s=+new Date;if((i.lastOp==r||i.lastOrigin==e.origin&&e.origin&&("+"==e.origin.charAt(0)&&i.lastModTime>s-(t.cm?t.cm.options.historyEventDelay:500)||"*"==e.origin.charAt(0)))&&(o=function(t,e){return e?(Ri(t.done),Y(t.done)):t.done.length&&!Y(t.done).ranges?Y(t.done):t.done.length>1&&!t.done[t.done.length-2].ranges?(t.done.pop(),Y(t.done)):void 0}(i,i.lastOp==r)))a=Y(o.changes),0==ne(e.from,e.to)&&0==ne(e.from,a.to)?a.to=Ti(e):o.changes.push(Vi(t,e));else {var l=Y(i.done);for(l&&l.ranges||$i(t.sel,i.done),o={changes:[Vi(t,e)],generation:i.generation},i.done.push(o);i.done.length>i.undoDepth;)i.done.shift(),i.done[0].ranges||i.done.shift();}i.done.push(n),i.generation=++i.maxGeneration,i.lastModTime=i.lastSelTime=s,i.lastOp=i.lastSelOp=r,i.lastOrigin=i.lastSelOrigin=e.origin,a||gt(t,"historyAdded");}function zi(t,e,n,r){var i=t.history,o=r&&r.origin;n==i.lastSelOp||o&&i.lastSelOrigin==o&&(i.lastModTime==i.lastSelTime&&i.lastOrigin==o||function(t,e,n,r){var i=e.charAt(0);return "*"==i||"+"==i&&n.ranges.length==r.ranges.length&&n.somethingSelected()==r.somethingSelected()&&new Date-t.history.lastSelTime<=(t.cm?t.cm.options.historyEventDelay:500)}(t,o,Y(i.done),e))?i.done[i.done.length-1]=e:$i(e,i.done),i.lastSelTime=+new Date,i.lastSelOrigin=o,i.lastSelOp=n,r&&!1!==r.clearRedo&&Ri(i.undone);}function $i(t,e){var n=Y(e);n&&n.ranges&&n.equals(t)||e.push(t);}function Wi(t,e,n,r){var i=e["spans_"+t.id],o=0;t.iter(Math.max(t.first,n),Math.min(t.first+t.size,r),(function(n){n.markedSpans&&((i||(i=e["spans_"+t.id]={}))[o]=n.markedSpans),++o;}));}function Bi(t){if(!t)return null;for(var e,n=0;n<t.length;++n)t[n].marker.explicitlyCleared?e||(e=t.slice(0,n)):e&&e.push(t[n]);return e?e.length?e:null:t}function Ui(t,e){var n=function(t,e){var n=e["spans_"+t.id];if(!n)return null;for(var r=[],i=0;i<e.text.length;++i)r.push(Bi(n[i]));return r}(t,e),r=je(t,e);if(!n)return r;if(!r)return n;for(var i=0;i<n.length;++i){var o=n[i],a=r[i];if(o&&a)t:for(var s=0;s<a.length;++s){for(var l=a[s],c=0;c<o.length;++c)if(o[c].marker==l.marker)continue t;o.push(l);}else a&&(n[i]=a);}return n}function qi(t,e,n){for(var r=[],i=0;i<t.length;++i){var o=t[i];if(o.ranges)r.push(n?Oi.prototype.deepCopy.call(o):o);else {var a=o.changes,s=[];r.push({changes:s});for(var l=0;l<a.length;++l){var c=a[l],u=void 0;if(s.push({from:c.from,to:c.to,text:c.text}),e)for(var h in c)(u=h.match(/^spans_(\d+)$/))&&H(e,Number(u[1]))>-1&&(Y(s)[h]=c[h],delete c[h]);}}}return r}function Gi(t,e,n,r){if(r){var i=t.anchor;if(n){var o=ne(e,i)<0;o!=ne(n,i)<0?(i=e,e=n):o!=ne(e,n)<0&&(e=n);}return new Ci(i,e)}return new Ci(n||e,e)}function Ki(t,e,n,r,i){null==i&&(i=t.cm&&(t.cm.display.shift||t.extend)),Qi(t,new Oi([Gi(t.sel.primary(),e,n,i)],0),r);}function Yi(t,e,n){for(var r=[],i=t.cm&&(t.cm.display.shift||t.extend),o=0;o<t.sel.ranges.length;o++)r[o]=Gi(t.sel.ranges[o],e[o],null,i);Qi(t,Si(t.cm,r,t.sel.primIndex),n);}function Xi(t,e,n,r){var i=t.sel.ranges.slice(0);i[e]=n,Qi(t,Si(t.cm,i,t.sel.primIndex),r);}function Ji(t,e,n,r){Qi(t,ki(e,n),r);}function Zi(t,e,n){var r=t.history.done,i=Y(r);i&&i.ranges?(r[r.length-1]=e,to(t,e,n)):Qi(t,e,n);}function Qi(t,e,n){to(t,e,n),zi(t,t.sel,t.cm?t.cm.curOp.id:NaN,n);}function to(t,e,n){(yt(t,"beforeSelectionChange")||t.cm&&yt(t.cm,"beforeSelectionChange"))&&(e=function(t,e,n){var r={ranges:e.ranges,update:function(e){this.ranges=[];for(var n=0;n<e.length;n++)this.ranges[n]=new Ci(le(t,e[n].anchor),le(t,e[n].head));},origin:n&&n.origin};return gt(t,"beforeSelectionChange",t,r),t.cm&&gt(t.cm,"beforeSelectionChange",t.cm,r),r.ranges!=e.ranges?Si(t.cm,r.ranges,r.ranges.length-1):e}(t,e,n));var r=n&&n.bias||(ne(e.primary().head,t.sel.primary().head)<0?-1:1);eo(t,ro(t,e,r,!0)),n&&!1===n.scroll||!t.cm||Ar(t.cm);}function eo(t,e){e.equals(t.sel)||(t.sel=e,t.cm&&(t.cm.curOp.updateInput=1,t.cm.curOp.selectionChanged=!0,mt(t.cm)),cn(t,"cursorActivity",t));}function no(t){eo(t,ro(t,t.sel,null,!1));}function ro(t,e,n,r){for(var i,o=0;o<e.ranges.length;o++){var a=e.ranges[o],s=e.ranges.length==t.sel.ranges.length&&t.sel.ranges[o],l=oo(t,a.anchor,s&&s.anchor,n,r),c=oo(t,a.head,s&&s.head,n,r);(i||l!=a.anchor||c!=a.head)&&(i||(i=e.ranges.slice(0,o)),i[o]=new Ci(l,c));}return i?Si(t.cm,i,e.primIndex):e}function io(t,e,n,r,i){var o=Gt(t,e.line);if(o.markedSpans)for(var a=0;a<o.markedSpans.length;++a){var s=o.markedSpans[a],l=s.marker,c="selectLeft"in l?!l.selectLeft:l.inclusiveLeft,u="selectRight"in l?!l.selectRight:l.inclusiveRight;if((null==s.from||(c?s.from<=e.ch:s.from<e.ch))&&(null==s.to||(u?s.to>=e.ch:s.to>e.ch))){if(i&&(gt(l,"beforeCursorEnter"),l.explicitlyCleared)){if(o.markedSpans){--a;continue}break}if(!l.atomic)continue;if(n){var h=l.find(r<0?1:-1),d=void 0;if((r<0?u:c)&&(h=ao(t,h,-r,h&&h.line==e.line?o:null)),h&&h.line==e.line&&(d=ne(h,n))&&(r<0?d<0:d>0))return io(t,h,e,r,i)}var f=l.find(r<0?-1:1);return (r<0?c:u)&&(f=ao(t,f,r,f.line==e.line?o:null)),f?io(t,f,e,r,i):null}}return e}function oo(t,e,n,r,i){var o=r||1,a=io(t,e,n,o,i)||!i&&io(t,e,n,o,!0)||io(t,e,n,-o,i)||!i&&io(t,e,n,-o,!0);return a||(t.cantEdit=!0,ee(t.first,0))}function ao(t,e,n,r){return n<0&&0==e.ch?e.line>t.first?le(t,ee(e.line-1)):null:n>0&&e.ch==(r||Gt(t,e.line)).text.length?e.line<t.first+t.size-1?ee(e.line+1,0):null:new ee(e.line,e.ch+n)}function so(t){t.setSelection(ee(t.firstLine(),0),ee(t.lastLine()),W);}function lo(t,e,n){var r={canceled:!1,from:e.from,to:e.to,text:e.text,origin:e.origin,cancel:function(){return r.canceled=!0}};return n&&(r.update=function(e,n,i,o){e&&(r.from=le(t,e)),n&&(r.to=le(t,n)),i&&(r.text=i),void 0!==o&&(r.origin=o);}),gt(t,"beforeChange",t,r),t.cm&&gt(t.cm,"beforeChange",t.cm,r),r.canceled?(t.cm&&(t.cm.curOp.updateInput=2),null):{from:r.from,to:r.to,text:r.text,origin:r.origin}}function co(t,e,n){if(t.cm){if(!t.cm.curOp)return ti(t.cm,co)(t,e,n);if(t.cm.state.suppressEdits)return}if(!(yt(t,"beforeChange")||t.cm&&yt(t.cm,"beforeChange"))||(e=lo(t,e,!0))){var r=Oe&&!n&&function(t,e,n){var r=null;if(t.iter(e.line,n.line+1,(function(t){if(t.markedSpans)for(var e=0;e<t.markedSpans.length;++e){var n=t.markedSpans[e].marker;!n.readOnly||r&&-1!=H(r,n)||(r||(r=[])).push(n);}})),!r)return null;for(var i=[{from:e,to:n}],o=0;o<r.length;++o)for(var a=r[o],s=a.find(0),l=0;l<i.length;++l){var c=i[l];if(!(ne(c.to,s.from)<0||ne(c.from,s.to)>0)){var u=[l,1],h=ne(c.from,s.from),d=ne(c.to,s.to);(h<0||!a.inclusiveLeft&&!h)&&u.push({from:c.from,to:s.from}),(d>0||!a.inclusiveRight&&!d)&&u.push({from:s.to,to:c.to}),i.splice.apply(i,u),l+=u.length-3;}}return i}(t,e.from,e.to);if(r)for(var i=r.length-1;i>=0;--i)uo(t,{from:r[i].from,to:r[i].to,text:i?[""]:e.text,origin:e.origin});else uo(t,e);}}function uo(t,e){if(1!=e.text.length||""!=e.text[0]||0!=ne(e.from,e.to)){var n=Pi(t,e);Hi(t,e,n,t.cm?t.cm.curOp.id:NaN),po(t,e,n,je(t,e));var r=[];Ni(t,(function(t,n){n||-1!=H(r,t.history)||(yo(t.history,e),r.push(t.history)),po(t,e,null,je(t,e));}));}}function ho(t,e,n){var r=t.cm&&t.cm.state.suppressEdits;if(!r||n){for(var i,o=t.history,a=t.sel,s="undo"==e?o.done:o.undone,l="undo"==e?o.undone:o.done,c=0;c<s.length&&(i=s[c],n?!i.ranges||i.equals(t.sel):i.ranges);c++);if(c!=s.length){for(o.lastOrigin=o.lastSelOrigin=null;;){if(!(i=s.pop()).ranges){if(r)return void s.push(i);break}if($i(i,l),n&&!i.equals(t.sel))return void Qi(t,i,{clearRedo:!1});a=i;}var u=[];$i(a,l),l.push({changes:u,generation:o.generation}),o.generation=i.generation||++o.maxGeneration;for(var h=yt(t,"beforeChange")||t.cm&&yt(t.cm,"beforeChange"),d=function(n){var r=i.changes[n];if(r.origin=e,h&&!lo(t,r,!1))return s.length=0,{};u.push(Vi(t,r));var o=n?Pi(t,r):Y(s);po(t,r,o,Ui(t,r)),!n&&t.cm&&t.cm.scrollIntoView({from:r.from,to:Ti(r)});var a=[];Ni(t,(function(t,e){e||-1!=H(a,t.history)||(yo(t.history,r),a.push(t.history)),po(t,r,null,Ui(t,r));}));},f=i.changes.length-1;f>=0;--f){var p=d(f);if(p)return p.v}}}}function fo(t,e){if(0!=e&&(t.first+=e,t.sel=new Oi(X(t.sel.ranges,(function(t){return new Ci(ee(t.anchor.line+e,t.anchor.ch),ee(t.head.line+e,t.head.ch))})),t.sel.primIndex),t.cm)){dr(t.cm,t.first,t.first-e,e);for(var n=t.cm.display,r=n.viewFrom;r<n.viewTo;r++)fr(t.cm,r,"gutter");}}function po(t,e,n,r){if(t.cm&&!t.cm.curOp)return ti(t.cm,po)(t,e,n,r);if(e.to.line<t.first)fo(t,e.text.length-1-(e.to.line-e.from.line));else if(!(e.from.line>t.lastLine())){if(e.from.line<t.first){var i=e.text.length-1-(t.first-e.from.line);fo(t,i),e={from:ee(t.first,0),to:ee(e.to.line+i,e.to.ch),text:[Y(e.text)],origin:e.origin};}var o=t.lastLine();e.to.line>o&&(e={from:e.from,to:ee(o,Gt(t,o).text.length),text:[e.text[0]],origin:e.origin}),e.removed=Kt(t,e.from,e.to),n||(n=Pi(t,e)),t.cm?function(t,e,n){var r=t.doc,i=t.display,o=e.from,a=e.to,s=!1,l=o.line;t.options.lineWrapping||(l=Jt(Re(Gt(r,o.line))),r.iter(l,a.line+1,(function(t){if(t==i.maxLine)return s=!0,!0}))),r.sel.contains(e.from,e.to)>-1&&mt(t),Li(r,e,n,lr(t)),t.options.lineWrapping||(r.iter(l,o.line+e.text.length,(function(t){var e=Ue(t);e>i.maxLineLength&&(i.maxLine=t,i.maxLineLength=e,i.maxLineChanged=!0,s=!1);})),s&&(t.curOp.updateMaxLine=!0)),function(t,e){if(t.modeFrontier=Math.min(t.modeFrontier,e),!(t.highlightFrontier<e-10)){for(var n=t.first,r=e-1;r>n;r--){var i=Gt(t,r).stateAfter;if(i&&(!(i instanceof ue)||r+i.lookAhead<e)){n=r+1;break}}t.highlightFrontier=Math.min(t.highlightFrontier,n);}}(r,o.line),ri(t,400);var c=e.text.length-(a.line-o.line)-1;e.full?dr(t):o.line!=a.line||1!=e.text.length||Ai(t.doc,e)?dr(t,o.line,a.line+1,c):fr(t,o.line,"text");var u=yt(t,"changes"),h=yt(t,"change");if(h||u){var d={from:o,to:a,text:e.text,removed:e.removed,origin:e.origin};h&&cn(t,"change",t,d),u&&(t.curOp.changeObjs||(t.curOp.changeObjs=[])).push(d);}t.display.selForContextMenu=null;}(t.cm,e,r):Li(t,e,r),to(t,n,W),t.cantEdit&&oo(t,ee(t.firstLine(),0))&&(t.cantEdit=!1);}}function go(t,e,n,r,i){var o;r||(r=n),ne(r,n)<0&&(n=(o=[r,n])[0],r=o[1]),"string"==typeof e&&(e=t.splitLines(e)),co(t,{from:n,to:r,text:e,origin:i});}function vo(t,e,n,r){n<t.line?t.line+=r:e<t.line&&(t.line=e,t.ch=0);}function mo(t,e,n,r){for(var i=0;i<t.length;++i){var o=t[i],a=!0;if(o.ranges){o.copied||((o=t[i]=o.deepCopy()).copied=!0);for(var s=0;s<o.ranges.length;s++)vo(o.ranges[s].anchor,e,n,r),vo(o.ranges[s].head,e,n,r);}else {for(var l=0;l<o.changes.length;++l){var c=o.changes[l];if(n<c.from.line)c.from=ee(c.from.line+r,c.from.ch),c.to=ee(c.to.line+r,c.to.ch);else if(e<=c.to.line){a=!1;break}}a||(t.splice(0,i+1),i=0);}}}function yo(t,e){var n=e.from.line,r=e.to.line,i=e.text.length-(r-n)-1;mo(t.done,n,r,i),mo(t.undone,n,r,i);}function bo(t,e,n,r){var i=e,o=e;return "number"==typeof e?o=Gt(t,se(t,e)):i=Jt(e),null==i?null:(r(o,i)&&t.cm&&fr(t.cm,i,n),o)}function wo(t){this.lines=t,this.parent=null;for(var e=0,n=0;n<t.length;++n)t[n].parent=this,e+=t[n].height;this.height=e;}function xo(t){this.children=t;for(var e=0,n=0,r=0;r<t.length;++r){var i=t[r];e+=i.chunkSize(),n+=i.height,i.parent=this;}this.size=e,this.height=n,this.parent=null;}Ci.prototype.from=function(){return ae(this.anchor,this.head)},Ci.prototype.to=function(){return oe(this.anchor,this.head)},Ci.prototype.empty=function(){return this.head.line==this.anchor.line&&this.head.ch==this.anchor.ch},wo.prototype={chunkSize:function(){return this.lines.length},removeInner:function(t,e){for(var n=t,r=t+e;n<r;++n){var i=this.lines[n];this.height-=i.height,Ke(i),cn(i,"delete");}this.lines.splice(t,e);},collapse:function(t){t.push.apply(t,this.lines);},insertInner:function(t,e,n){this.height+=n,this.lines=this.lines.slice(0,t).concat(e).concat(this.lines.slice(t));for(var r=0;r<e.length;++r)e[r].parent=this;},iterN:function(t,e,n){for(var r=t+e;t<r;++t)if(n(this.lines[t]))return !0}},xo.prototype={chunkSize:function(){return this.size},removeInner:function(t,e){this.size-=e;for(var n=0;n<this.children.length;++n){var r=this.children[n],i=r.chunkSize();if(t<i){var o=Math.min(e,i-t),a=r.height;if(r.removeInner(t,o),this.height-=a-r.height,i==o&&(this.children.splice(n--,1),r.parent=null),0==(e-=o))break;t=0;}else t-=i;}if(this.size-e<25&&(this.children.length>1||!(this.children[0]instanceof wo))){var s=[];this.collapse(s),this.children=[new wo(s)],this.children[0].parent=this;}},collapse:function(t){for(var e=0;e<this.children.length;++e)this.children[e].collapse(t);},insertInner:function(t,e,n){this.size+=e.length,this.height+=n;for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(t<=o){if(i.insertInner(t,e,n),i.lines&&i.lines.length>50){for(var a=i.lines.length%25+25,s=a;s<i.lines.length;){var l=new wo(i.lines.slice(s,s+=25));i.height-=l.height,this.children.splice(++r,0,l),l.parent=this;}i.lines=i.lines.slice(0,a),this.maybeSpill();}break}t-=o;}},maybeSpill:function(){if(!(this.children.length<=10)){var t=this;do{var e=new xo(t.children.splice(t.children.length-5,5));if(t.parent){t.size-=e.size,t.height-=e.height;var n=H(t.parent.children,t);t.parent.children.splice(n+1,0,e);}else {var r=new xo(t.children);r.parent=t,t.children=[r,e],t=r;}e.parent=t.parent;}while(t.children.length>10);t.parent.maybeSpill();}},iterN:function(t,e,n){for(var r=0;r<this.children.length;++r){var i=this.children[r],o=i.chunkSize();if(t<o){var a=Math.min(e,o-t);if(i.iterN(t,a,n))return !0;if(0==(e-=a))break;t=0;}else t-=o;}}};var Oo=function(t,e,n){if(n)for(var r in n)n.hasOwnProperty(r)&&(this[r]=n[r]);this.doc=t,this.node=e;};function Co(t,e,n){Be(e)<(t.curOp&&t.curOp.scrollTop||t.doc.scrollTop)&&Dr(t,n);}Oo.prototype.clear=function(){var t=this.doc.cm,e=this.line.widgets,n=this.line,r=Jt(n);if(null!=r&&e){for(var i=0;i<e.length;++i)e[i]==this&&e.splice(i--,1);e.length||(n.widgets=null);var o=On(this);Xt(n,Math.max(0,n.height-o)),t&&(Qr(t,(function(){Co(t,n,-o),fr(t,r,"widget");})),cn(t,"lineWidgetCleared",t,this,r));}},Oo.prototype.changed=function(){var t=this,e=this.height,n=this.doc.cm,r=this.line;this.height=null;var i=On(this)-e;i&&($e(this.doc,r)||Xt(r,r.height+i),n&&Qr(n,(function(){n.curOp.forceUpdate=!0,Co(n,r,i),cn(n,"lineWidgetChanged",n,t,Jt(r));})));},bt(Oo);var So=0,ko=function(t,e){this.lines=[],this.type=e,this.doc=t,this.id=++So;};function To(t,e,n,r,i){if(r&&r.shared)return function(t,e,n,r,i){(r=_(r)).shared=!1;var o=[To(t,e,n,r,i)],a=o[0],s=r.widgetNode;return Ni(t,(function(t){s&&(r.widgetNode=s.cloneNode(!0)),o.push(To(t,le(t,e),le(t,n),r,i));for(var l=0;l<t.linked.length;++l)if(t.linked[l].isParent)return;a=Y(o);})),new jo(o,a)}(t,e,n,r,i);if(t.cm&&!t.cm.curOp)return ti(t.cm,To)(t,e,n,r,i);var o=new ko(t,i),a=ne(e,n);if(r&&_(r,o,!1),a>0||0==a&&!1!==o.clearWhenEmpty)return o;if(o.replacedWith&&(o.collapsed=!0,o.widgetNode=M("span",[o.replacedWith],"CodeMirror-widget"),r.handleMouseEvents||o.widgetNode.setAttribute("cm-ignore-events","true"),r.insertLeft&&(o.widgetNode.insertLeft=!0)),o.collapsed){if(Ve(t,e.line,e,n,o)||e.line!=n.line&&Ve(t,n.line,e,n,o))throw new Error("Inserting collapsed marker partially overlapping an existing one");Ce=!0;}o.addToHistory&&Hi(t,{from:e,to:n,origin:"markText"},t.sel,NaN);var s,l=e.line,c=t.cm;if(t.iter(l,n.line+1,(function(t){c&&o.collapsed&&!c.options.lineWrapping&&Re(t)==c.display.maxLine&&(s=!0),o.collapsed&&l!=e.line&&Xt(t,0),function(t,e){t.markedSpans=t.markedSpans?t.markedSpans.concat([e]):[e],e.marker.attachLine(t);}(t,new Se(o,l==e.line?e.ch:null,l==n.line?n.ch:null)),++l;})),o.collapsed&&t.iter(e.line,n.line+1,(function(e){$e(t,e)&&Xt(e,0);})),o.clearOnEnter&&dt(o,"beforeCursorEnter",(function(){return o.clear()})),o.readOnly&&(Oe=!0,(t.history.done.length||t.history.undone.length)&&t.clearHistory()),o.collapsed&&(o.id=++So,o.atomic=!0),c){if(s&&(c.curOp.updateMaxLine=!0),o.collapsed)dr(c,e.line,n.line+1);else if(o.className||o.startStyle||o.endStyle||o.css||o.attributes||o.title)for(var u=e.line;u<=n.line;u++)fr(c,u,"text");o.atomic&&no(c.doc),cn(c,"markerAdded",c,o);}return o}ko.prototype.clear=function(){if(!this.explicitlyCleared){var t=this.doc.cm,e=t&&!t.curOp;if(e&&Gr(t),yt(this,"clear")){var n=this.find();n&&cn(this,"clear",n.from,n.to);}for(var r=null,i=null,o=0;o<this.lines.length;++o){var a=this.lines[o],s=ke(a.markedSpans,this);t&&!this.collapsed?fr(t,Jt(a),"text"):t&&(null!=s.to&&(i=Jt(a)),null!=s.from&&(r=Jt(a))),a.markedSpans=Te(a.markedSpans,s),null==s.from&&this.collapsed&&!$e(this.doc,a)&&t&&Xt(a,ir(t.display));}if(t&&this.collapsed&&!t.options.lineWrapping)for(var l=0;l<this.lines.length;++l){var c=Re(this.lines[l]),u=Ue(c);u>t.display.maxLineLength&&(t.display.maxLine=c,t.display.maxLineLength=u,t.display.maxLineChanged=!0);}null!=r&&t&&this.collapsed&&dr(t,r,i+1),this.lines.length=0,this.explicitlyCleared=!0,this.atomic&&this.doc.cantEdit&&(this.doc.cantEdit=!1,t&&no(t.doc)),t&&cn(t,"markerCleared",t,this,r,i),e&&Kr(t),this.parent&&this.parent.clear();}},ko.prototype.find=function(t,e){var n,r;null==t&&"bookmark"==this.type&&(t=1);for(var i=0;i<this.lines.length;++i){var o=this.lines[i],a=ke(o.markedSpans,this);if(null!=a.from&&(n=ee(e?o:Jt(o),a.from),-1==t))return n;if(null!=a.to&&(r=ee(e?o:Jt(o),a.to),1==t))return r}return n&&{from:n,to:r}},ko.prototype.changed=function(){var t=this,e=this.find(-1,!0),n=this,r=this.doc.cm;e&&r&&Qr(r,(function(){var i=e.line,o=Jt(e.line),a=An(r,o);if(a&&(Rn(a),r.curOp.selectionChanged=r.curOp.forceUpdate=!0),r.curOp.updateMaxLine=!0,!$e(n.doc,i)&&null!=n.height){var s=n.height;n.height=null;var l=On(n)-s;l&&Xt(i,i.height+l);}cn(r,"markerChanged",r,t);}));},ko.prototype.attachLine=function(t){if(!this.lines.length&&this.doc.cm){var e=this.doc.cm.curOp;e.maybeHiddenMarkers&&-1!=H(e.maybeHiddenMarkers,this)||(e.maybeUnhiddenMarkers||(e.maybeUnhiddenMarkers=[])).push(this);}this.lines.push(t);},ko.prototype.detachLine=function(t){if(this.lines.splice(H(this.lines,t),1),!this.lines.length&&this.doc.cm){var e=this.doc.cm.curOp;(e.maybeHiddenMarkers||(e.maybeHiddenMarkers=[])).push(this);}},bt(ko);var jo=function(t,e){this.markers=t,this.primary=e;for(var n=0;n<t.length;++n)t[n].parent=this;};function Po(t){return t.findMarks(ee(t.first,0),t.clipPos(ee(t.lastLine())),(function(t){return t.parent}))}function Eo(t){for(var e=function(e){var n=t[e],r=[n.primary.doc];Ni(n.primary.doc,(function(t){return r.push(t)}));for(var i=0;i<n.markers.length;i++){var o=n.markers[i];-1==H(r,o.doc)&&(o.parent=null,n.markers.splice(i--,1));}},n=0;n<t.length;n++)e(n);}jo.prototype.clear=function(){if(!this.explicitlyCleared){this.explicitlyCleared=!0;for(var t=0;t<this.markers.length;++t)this.markers[t].clear();cn(this,"clear");}},jo.prototype.find=function(t,e){return this.primary.find(t,e)},bt(jo);var Mo=0,Do=function(t,e,n,r,i){if(!(this instanceof Do))return new Do(t,e,n,r,i);null==n&&(n=0),xo.call(this,[new wo([new Ge("",null)])]),this.first=n,this.scrollTop=this.scrollLeft=0,this.cantEdit=!1,this.cleanGeneration=1,this.modeFrontier=this.highlightFrontier=n;var o=ee(n,0);this.sel=ki(o),this.history=new _i(null),this.id=++Mo,this.modeOption=e,this.lineSep=r,this.direction="rtl"==i?"rtl":"ltr",this.extend=!1,"string"==typeof t&&(t=this.splitLines(t)),Li(this,{from:o,to:o,text:t}),Qi(this,ki(o),W);};Do.prototype=Z(xo.prototype,{constructor:Do,iter:function(t,e,n){n?this.iterN(t-this.first,e-t,n):this.iterN(this.first,this.first+this.size,t);},insert:function(t,e){for(var n=0,r=0;r<e.length;++r)n+=e[r].height;this.insertInner(t-this.first,e,n);},remove:function(t,e){this.removeInner(t-this.first,e);},getValue:function(t){var e=Yt(this,this.first,this.first+this.size);return !1===t?e:e.join(t||this.lineSeparator())},setValue:ni((function(t){var e=ee(this.first,0),n=this.first+this.size-1;co(this,{from:e,to:ee(n,Gt(this,n).text.length),text:this.splitLines(t),origin:"setValue",full:!0},!0),this.cm&&Lr(this.cm,0,0),Qi(this,ki(e),W);})),replaceRange:function(t,e,n,r){go(this,t,e=le(this,e),n=n?le(this,n):e,r);},getRange:function(t,e,n){var r=Kt(this,le(this,t),le(this,e));return !1===n?r:r.join(n||this.lineSeparator())},getLine:function(t){var e=this.getLineHandle(t);return e&&e.text},getLineHandle:function(t){if(Qt(this,t))return Gt(this,t)},getLineNumber:function(t){return Jt(t)},getLineHandleVisualStart:function(t){return "number"==typeof t&&(t=Gt(this,t)),Re(t)},lineCount:function(){return this.size},firstLine:function(){return this.first},lastLine:function(){return this.first+this.size-1},clipPos:function(t){return le(this,t)},getCursor:function(t){var e=this.sel.primary();return null==t||"head"==t?e.head:"anchor"==t?e.anchor:"end"==t||"to"==t||!1===t?e.to():e.from()},listSelections:function(){return this.sel.ranges},somethingSelected:function(){return this.sel.somethingSelected()},setCursor:ni((function(t,e,n){Ji(this,le(this,"number"==typeof t?ee(t,e||0):t),null,n);})),setSelection:ni((function(t,e,n){Ji(this,le(this,t),le(this,e||t),n);})),extendSelection:ni((function(t,e,n){Ki(this,le(this,t),e&&le(this,e),n);})),extendSelections:ni((function(t,e){Yi(this,ce(this,t),e);})),extendSelectionsBy:ni((function(t,e){Yi(this,ce(this,X(this.sel.ranges,t)),e);})),setSelections:ni((function(t,e,n){if(t.length){for(var r=[],i=0;i<t.length;i++)r[i]=new Ci(le(this,t[i].anchor),le(this,t[i].head));null==e&&(e=Math.min(t.length-1,this.sel.primIndex)),Qi(this,Si(this.cm,r,e),n);}})),addSelection:ni((function(t,e,n){var r=this.sel.ranges.slice(0);r.push(new Ci(le(this,t),le(this,e||t))),Qi(this,Si(this.cm,r,r.length-1),n);})),getSelection:function(t){for(var e,n=this.sel.ranges,r=0;r<n.length;r++){var i=Kt(this,n[r].from(),n[r].to());e=e?e.concat(i):i;}return !1===t?e:e.join(t||this.lineSeparator())},getSelections:function(t){for(var e=[],n=this.sel.ranges,r=0;r<n.length;r++){var i=Kt(this,n[r].from(),n[r].to());!1!==t&&(i=i.join(t||this.lineSeparator())),e[r]=i;}return e},replaceSelection:function(t,e,n){for(var r=[],i=0;i<this.sel.ranges.length;i++)r[i]=t;this.replaceSelections(r,e,n||"+input");},replaceSelections:ni((function(t,e,n){for(var r=[],i=this.sel,o=0;o<i.ranges.length;o++){var a=i.ranges[o];r[o]={from:a.from(),to:a.to(),text:this.splitLines(t[o]),origin:n};}for(var s=e&&"end"!=e&&function(t,e,n){for(var r=[],i=ee(t.first,0),o=i,a=0;a<e.length;a++){var s=e[a],l=Ei(s.from,i,o),c=Ei(Ti(s),i,o);if(i=s.to,o=c,"around"==n){var u=t.sel.ranges[a],h=ne(u.head,u.anchor)<0;r[a]=new Ci(h?c:l,h?l:c);}else r[a]=new Ci(l,l);}return new Oi(r,t.sel.primIndex)}(this,r,e),l=r.length-1;l>=0;l--)co(this,r[l]);s?Zi(this,s):this.cm&&Ar(this.cm);})),undo:ni((function(){ho(this,"undo");})),redo:ni((function(){ho(this,"redo");})),undoSelection:ni((function(){ho(this,"undo",!0);})),redoSelection:ni((function(){ho(this,"redo",!0);})),setExtending:function(t){this.extend=t;},getExtending:function(){return this.extend},historySize:function(){for(var t=this.history,e=0,n=0,r=0;r<t.done.length;r++)t.done[r].ranges||++e;for(var i=0;i<t.undone.length;i++)t.undone[i].ranges||++n;return {undo:e,redo:n}},clearHistory:function(){this.history=new _i(this.history.maxGeneration);},markClean:function(){this.cleanGeneration=this.changeGeneration(!0);},changeGeneration:function(t){return t&&(this.history.lastOp=this.history.lastSelOp=this.history.lastOrigin=null),this.history.generation},isClean:function(t){return this.history.generation==(t||this.cleanGeneration)},getHistory:function(){return {done:qi(this.history.done),undone:qi(this.history.undone)}},setHistory:function(t){var e=this.history=new _i(this.history.maxGeneration);e.done=qi(t.done.slice(0),null,!0),e.undone=qi(t.undone.slice(0),null,!0);},setGutterMarker:ni((function(t,e,n){return bo(this,t,"gutter",(function(t){var r=t.gutterMarkers||(t.gutterMarkers={});return r[e]=n,!n&&nt(r)&&(t.gutterMarkers=null),!0}))})),clearGutter:ni((function(t){var e=this;this.iter((function(n){n.gutterMarkers&&n.gutterMarkers[t]&&bo(e,n,"gutter",(function(){return n.gutterMarkers[t]=null,nt(n.gutterMarkers)&&(n.gutterMarkers=null),!0}));}));})),lineInfo:function(t){var e;if("number"==typeof t){if(!Qt(this,t))return null;if(e=t,!(t=Gt(this,t)))return null}else if(null==(e=Jt(t)))return null;return {line:e,handle:t,text:t.text,gutterMarkers:t.gutterMarkers,textClass:t.textClass,bgClass:t.bgClass,wrapClass:t.wrapClass,widgets:t.widgets}},addLineClass:ni((function(t,e,n){return bo(this,t,"gutter"==e?"gutter":"class",(function(t){var r="text"==e?"textClass":"background"==e?"bgClass":"gutter"==e?"gutterClass":"wrapClass";if(t[r]){if(S(n).test(t[r]))return !1;t[r]+=" "+n;}else t[r]=n;return !0}))})),removeLineClass:ni((function(t,e,n){return bo(this,t,"gutter"==e?"gutter":"class",(function(t){var r="text"==e?"textClass":"background"==e?"bgClass":"gutter"==e?"gutterClass":"wrapClass",i=t[r];if(!i)return !1;if(null==n)t[r]=null;else {var o=i.match(S(n));if(!o)return !1;var a=o.index+o[0].length;t[r]=i.slice(0,o.index)+(o.index&&a!=i.length?" ":"")+i.slice(a)||null;}return !0}))})),addLineWidget:ni((function(t,e,n){return function(t,e,n,r){var i=new Oo(t,n,r),o=t.cm;return o&&i.noHScroll&&(o.display.alignWidgets=!0),bo(t,e,"widget",(function(e){var n=e.widgets||(e.widgets=[]);if(null==i.insertAt?n.push(i):n.splice(Math.min(n.length-1,Math.max(0,i.insertAt)),0,i),i.line=e,o&&!$e(t,e)){var r=Be(e)<t.scrollTop;Xt(e,e.height+On(i)),r&&Dr(o,i.height),o.curOp.forceUpdate=!0;}return !0})),o&&cn(o,"lineWidgetAdded",o,i,"number"==typeof e?e:Jt(e)),i}(this,t,e,n)})),removeLineWidget:function(t){t.clear();},markText:function(t,e,n){return To(this,le(this,t),le(this,e),n,n&&n.type||"range")},setBookmark:function(t,e){var n={replacedWith:e&&(null==e.nodeType?e.widget:e),insertLeft:e&&e.insertLeft,clearWhenEmpty:!1,shared:e&&e.shared,handleMouseEvents:e&&e.handleMouseEvents};return To(this,t=le(this,t),t,n,"bookmark")},findMarksAt:function(t){var e=[],n=Gt(this,(t=le(this,t)).line).markedSpans;if(n)for(var r=0;r<n.length;++r){var i=n[r];(null==i.from||i.from<=t.ch)&&(null==i.to||i.to>=t.ch)&&e.push(i.marker.parent||i.marker);}return e},findMarks:function(t,e,n){t=le(this,t),e=le(this,e);var r=[],i=t.line;return this.iter(t.line,e.line+1,(function(o){var a=o.markedSpans;if(a)for(var s=0;s<a.length;s++){var l=a[s];null!=l.to&&i==t.line&&t.ch>=l.to||null==l.from&&i!=t.line||null!=l.from&&i==e.line&&l.from>=e.ch||n&&!n(l.marker)||r.push(l.marker.parent||l.marker);}++i;})),r},getAllMarks:function(){var t=[];return this.iter((function(e){var n=e.markedSpans;if(n)for(var r=0;r<n.length;++r)null!=n[r].from&&t.push(n[r].marker);})),t},posFromIndex:function(t){var e,n=this.first,r=this.lineSeparator().length;return this.iter((function(i){var o=i.text.length+r;if(o>t)return e=t,!0;t-=o,++n;})),le(this,ee(n,e))},indexFromPos:function(t){var e=(t=le(this,t)).ch;if(t.line<this.first||t.ch<0)return 0;var n=this.lineSeparator().length;return this.iter(this.first,t.line,(function(t){e+=t.text.length+n;})),e},copy:function(t){var e=new Do(Yt(this,this.first,this.first+this.size),this.modeOption,this.first,this.lineSep,this.direction);return e.scrollTop=this.scrollTop,e.scrollLeft=this.scrollLeft,e.sel=this.sel,e.extend=!1,t&&(e.history.undoDepth=this.history.undoDepth,e.setHistory(this.getHistory())),e},linkedDoc:function(t){t||(t={});var e=this.first,n=this.first+this.size;null!=t.from&&t.from>e&&(e=t.from),null!=t.to&&t.to<n&&(n=t.to);var r=new Do(Yt(this,e,n),t.mode||this.modeOption,e,this.lineSep,this.direction);return t.sharedHist&&(r.history=this.history),(this.linked||(this.linked=[])).push({doc:r,sharedHist:t.sharedHist}),r.linked=[{doc:this,isParent:!0,sharedHist:t.sharedHist}],function(t,e){for(var n=0;n<e.length;n++){var r=e[n],i=r.find(),o=t.clipPos(i.from),a=t.clipPos(i.to);if(ne(o,a)){var s=To(t,o,a,r.primary,r.primary.type);r.markers.push(s),s.parent=r;}}}(r,Po(this)),r},unlinkDoc:function(t){if(t instanceof ja&&(t=t.doc),this.linked)for(var e=0;e<this.linked.length;++e)if(this.linked[e].doc==t){this.linked.splice(e,1),t.unlinkDoc(this),Eo(Po(this));break}if(t.history==this.history){var n=[t.id];Ni(t,(function(t){return n.push(t.id)}),!0),t.history=new _i(null),t.history.done=qi(this.history.done,n),t.history.undone=qi(this.history.undone,n);}},iterLinkedDocs:function(t){Ni(this,t);},getMode:function(){return this.mode},getEditor:function(){return this.cm},splitLines:function(t){return this.lineSep?t.split(this.lineSep):At(t)},lineSeparator:function(){return this.lineSep||"\n"},setDirection:ni((function(t){var e;"rtl"!=t&&(t="ltr"),t!=this.direction&&(this.direction=t,this.iter((function(t){return t.order=null})),this.cm&&Qr(e=this.cm,(function(){Fi(e),dr(e);})));}))}),Do.prototype.eachLine=Do.prototype.iter;var Ao=0;function Lo(t){var e=this;if(No(e),!vt(e,t)&&!Cn(e.display,t)){wt(t),a&&(Ao=+new Date);var n=ur(e,t,!0),r=t.dataTransfer.files;if(n&&!e.isReadOnly())if(r&&r.length&&window.FileReader&&window.File)for(var i=r.length,o=Array(i),s=0,l=function(t,r){if(!e.options.allowDropFileTypes||-1!=H(e.options.allowDropFileTypes,t.type)){var a=new FileReader;a.onload=ti(e,(function(){var t=a.result;if(/[\x00-\x08\x0e-\x1f]{2}/.test(t)&&(t=""),o[r]=t,++s==i){var l={from:n=le(e.doc,n),to:n,text:e.doc.splitLines(o.join(e.doc.lineSeparator())),origin:"paste"};co(e.doc,l),Zi(e.doc,ki(n,Ti(l)));}})),a.readAsText(t);}},c=0;c<i;++c)l(r[c],c);else {if(e.state.draggingText&&e.doc.sel.contains(n)>-1)return e.state.draggingText(t),void setTimeout((function(){return e.display.input.focus()}),20);try{var u=t.dataTransfer.getData("Text");if(u){var h;if(e.state.draggingText&&!e.state.draggingText.copy&&(h=e.listSelections()),to(e.doc,ki(n,n)),h)for(var d=0;d<h.length;++d)go(e.doc,"",h[d].anchor,h[d].head,"drag");e.replaceSelection(u,"around","paste"),e.display.input.focus();}}catch(t){}}}}function No(t){t.display.dragCursor&&(t.display.lineSpace.removeChild(t.display.dragCursor),t.display.dragCursor=null);}function Io(t){if(document.getElementsByClassName){for(var e=document.getElementsByClassName("CodeMirror"),n=[],r=0;r<e.length;r++){var i=e[r].CodeMirror;i&&n.push(i);}n.length&&n[0].operation((function(){for(var e=0;e<n.length;e++)t(n[e]);}));}}var Fo=!1;function _o(){var t;Fo||(dt(window,"resize",(function(){null==t&&(t=setTimeout((function(){t=null,Io(Vo);}),100));})),dt(window,"blur",(function(){return Io(Tr)})),Fo=!0);}function Vo(t){var e=t.display;e.cachedCharWidth=e.cachedTextHeight=e.cachedPaddingH=null,e.scrollbarsClipped=!1,t.setSize();}for(var Ro={3:"Pause",8:"Backspace",9:"Tab",13:"Enter",16:"Shift",17:"Ctrl",18:"Alt",19:"Pause",20:"CapsLock",27:"Esc",32:"Space",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"Left",38:"Up",39:"Right",40:"Down",44:"PrintScrn",45:"Insert",46:"Delete",59:";",61:"=",91:"Mod",92:"Mod",93:"Mod",106:"*",107:"=",109:"-",110:".",111:"/",145:"ScrollLock",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'",63232:"Up",63233:"Down",63234:"Left",63235:"Right",63272:"Delete",63273:"Home",63275:"End",63276:"PageUp",63277:"PageDown",63302:"Insert"},Ho=0;Ho<10;Ho++)Ro[Ho+48]=Ro[Ho+96]=String(Ho);for(var zo=65;zo<=90;zo++)Ro[zo]=String.fromCharCode(zo);for(var $o=1;$o<=12;$o++)Ro[$o+111]=Ro[$o+63235]="F"+$o;var Wo={};function Bo(t){var e,n,r,i,o=t.split(/-(?!$)/);t=o[o.length-1];for(var a=0;a<o.length-1;a++){var s=o[a];if(/^(cmd|meta|m)$/i.test(s))i=!0;else if(/^a(lt)?$/i.test(s))e=!0;else if(/^(c|ctrl|control)$/i.test(s))n=!0;else {if(!/^s(hift)?$/i.test(s))throw new Error("Unrecognized modifier name: "+s);r=!0;}}return e&&(t="Alt-"+t),n&&(t="Ctrl-"+t),i&&(t="Cmd-"+t),r&&(t="Shift-"+t),t}function Uo(t){var e={};for(var n in t)if(t.hasOwnProperty(n)){var r=t[n];if(/^(name|fallthrough|(de|at)tach)$/.test(n))continue;if("..."==r){delete t[n];continue}for(var i=X(n.split(" "),Bo),o=0;o<i.length;o++){var a=void 0,s=void 0;o==i.length-1?(s=i.join(" "),a=r):(s=i.slice(0,o+1).join(" "),a="...");var l=e[s];if(l){if(l!=a)throw new Error("Inconsistent bindings for "+s)}else e[s]=a;}delete t[n];}for(var c in e)t[c]=e[c];return t}function qo(t,e,n,r){var i=(e=Xo(e)).call?e.call(t,r):e[t];if(!1===i)return "nothing";if("..."===i)return "multi";if(null!=i&&n(i))return "handled";if(e.fallthrough){if("[object Array]"!=Object.prototype.toString.call(e.fallthrough))return qo(t,e.fallthrough,n,r);for(var o=0;o<e.fallthrough.length;o++){var a=qo(t,e.fallthrough[o],n,r);if(a)return a}}}function Go(t){var e="string"==typeof t?t:Ro[t.keyCode];return "Ctrl"==e||"Alt"==e||"Shift"==e||"Mod"==e}function Ko(t,e,n){var r=t;return e.altKey&&"Alt"!=r&&(t="Alt-"+t),(O?e.metaKey:e.ctrlKey)&&"Ctrl"!=r&&(t="Ctrl-"+t),(O?e.ctrlKey:e.metaKey)&&"Cmd"!=r&&(t="Cmd-"+t),!n&&e.shiftKey&&"Shift"!=r&&(t="Shift-"+t),t}function Yo(t,e){if(h&&34==t.keyCode&&t.char)return !1;var n=Ro[t.keyCode];return null!=n&&!t.altGraphKey&&(3==t.keyCode&&t.code&&(n=t.code),Ko(n,t,e))}function Xo(t){return "string"==typeof t?Wo[t]:t}function Jo(t,e){for(var n=t.doc.sel.ranges,r=[],i=0;i<n.length;i++){for(var o=e(n[i]);r.length&&ne(o.from,Y(r).to)<=0;){var a=r.pop();if(ne(a.from,o.from)<0){o.from=a.from;break}}r.push(o);}Qr(t,(function(){for(var e=r.length-1;e>=0;e--)go(t.doc,"",r[e].from,r[e].to,"+delete");Ar(t);}));}function Zo(t,e,n){var r=ot(t.text,e+n,n);return r<0||r>t.text.length?null:r}function Qo(t,e,n){var r=Zo(t,e.ch,n);return null==r?null:new ee(e.line,r,n<0?"after":"before")}function ta(t,e,n,r,i){if(t){var o=ut(n,e.doc.direction);if(o){var a,s=i<0?Y(o):o[0],l=i<0==(1==s.level)?"after":"before";if(s.level>0||"rtl"==e.doc.direction){var c=Ln(e,n);a=i<0?n.text.length-1:0;var u=Nn(e,c,a).top;a=at((function(t){return Nn(e,c,t).top==u}),i<0==(1==s.level)?s.from:s.to-1,a),"before"==l&&(a=Zo(n,a,1));}else a=i<0?s.to:s.from;return new ee(r,a,l)}}return new ee(r,i<0?n.text.length:0,i<0?"before":"after")}Wo.basic={Left:"goCharLeft",Right:"goCharRight",Up:"goLineUp",Down:"goLineDown",End:"goLineEnd",Home:"goLineStartSmart",PageUp:"goPageUp",PageDown:"goPageDown",Delete:"delCharAfter",Backspace:"delCharBefore","Shift-Backspace":"delCharBefore",Tab:"defaultTab","Shift-Tab":"indentAuto",Enter:"newlineAndIndent",Insert:"toggleOverwrite",Esc:"singleSelection"},Wo.pcDefault={"Ctrl-A":"selectAll","Ctrl-D":"deleteLine","Ctrl-Z":"undo","Shift-Ctrl-Z":"redo","Ctrl-Y":"redo","Ctrl-Home":"goDocStart","Ctrl-End":"goDocEnd","Ctrl-Up":"goLineUp","Ctrl-Down":"goLineDown","Ctrl-Left":"goGroupLeft","Ctrl-Right":"goGroupRight","Alt-Left":"goLineStart","Alt-Right":"goLineEnd","Ctrl-Backspace":"delGroupBefore","Ctrl-Delete":"delGroupAfter","Ctrl-S":"save","Ctrl-F":"find","Ctrl-G":"findNext","Shift-Ctrl-G":"findPrev","Shift-Ctrl-F":"replace","Shift-Ctrl-R":"replaceAll","Ctrl-[":"indentLess","Ctrl-]":"indentMore","Ctrl-U":"undoSelection","Shift-Ctrl-U":"redoSelection","Alt-U":"redoSelection",fallthrough:"basic"},Wo.emacsy={"Ctrl-F":"goCharRight","Ctrl-B":"goCharLeft","Ctrl-P":"goLineUp","Ctrl-N":"goLineDown","Alt-F":"goWordRight","Alt-B":"goWordLeft","Ctrl-A":"goLineStart","Ctrl-E":"goLineEnd","Ctrl-V":"goPageDown","Shift-Ctrl-V":"goPageUp","Ctrl-D":"delCharAfter","Ctrl-H":"delCharBefore","Alt-D":"delWordAfter","Alt-Backspace":"delWordBefore","Ctrl-K":"killLine","Ctrl-T":"transposeChars","Ctrl-O":"openLine"},Wo.macDefault={"Cmd-A":"selectAll","Cmd-D":"deleteLine","Cmd-Z":"undo","Shift-Cmd-Z":"redo","Cmd-Y":"redo","Cmd-Home":"goDocStart","Cmd-Up":"goDocStart","Cmd-End":"goDocEnd","Cmd-Down":"goDocEnd","Alt-Left":"goGroupLeft","Alt-Right":"goGroupRight","Cmd-Left":"goLineLeft","Cmd-Right":"goLineRight","Alt-Backspace":"delGroupBefore","Ctrl-Alt-Backspace":"delGroupAfter","Alt-Delete":"delGroupAfter","Cmd-S":"save","Cmd-F":"find","Cmd-G":"findNext","Shift-Cmd-G":"findPrev","Cmd-Alt-F":"replace","Shift-Cmd-Alt-F":"replaceAll","Cmd-[":"indentLess","Cmd-]":"indentMore","Cmd-Backspace":"delWrappedLineLeft","Cmd-Delete":"delWrappedLineRight","Cmd-U":"undoSelection","Shift-Cmd-U":"redoSelection","Ctrl-Up":"goDocStart","Ctrl-Down":"goDocEnd",fallthrough:["basic","emacsy"]},Wo.default=y?Wo.macDefault:Wo.pcDefault;var ea={selectAll:so,singleSelection:function(t){return t.setSelection(t.getCursor("anchor"),t.getCursor("head"),W)},killLine:function(t){return Jo(t,(function(e){if(e.empty()){var n=Gt(t.doc,e.head.line).text.length;return e.head.ch==n&&e.head.line<t.lastLine()?{from:e.head,to:ee(e.head.line+1,0)}:{from:e.head,to:ee(e.head.line,n)}}return {from:e.from(),to:e.to()}}))},deleteLine:function(t){return Jo(t,(function(e){return {from:ee(e.from().line,0),to:le(t.doc,ee(e.to().line+1,0))}}))},delLineLeft:function(t){return Jo(t,(function(t){return {from:ee(t.from().line,0),to:t.from()}}))},delWrappedLineLeft:function(t){return Jo(t,(function(e){var n=t.charCoords(e.head,"div").top+5;return {from:t.coordsChar({left:0,top:n},"div"),to:e.from()}}))},delWrappedLineRight:function(t){return Jo(t,(function(e){var n=t.charCoords(e.head,"div").top+5,r=t.coordsChar({left:t.display.lineDiv.offsetWidth+100,top:n},"div");return {from:e.from(),to:r}}))},undo:function(t){return t.undo()},redo:function(t){return t.redo()},undoSelection:function(t){return t.undoSelection()},redoSelection:function(t){return t.redoSelection()},goDocStart:function(t){return t.extendSelection(ee(t.firstLine(),0))},goDocEnd:function(t){return t.extendSelection(ee(t.lastLine()))},goLineStart:function(t){return t.extendSelectionsBy((function(e){return na(t,e.head.line)}),{origin:"+move",bias:1})},goLineStartSmart:function(t){return t.extendSelectionsBy((function(e){return ra(t,e.head)}),{origin:"+move",bias:1})},goLineEnd:function(t){return t.extendSelectionsBy((function(e){return function(t,e){var n=Gt(t.doc,e),r=function(t){for(var e;e=Fe(t);)t=e.find(1,!0).line;return t}(n);return r!=n&&(e=Jt(r)),ta(!0,t,n,e,-1)}(t,e.head.line)}),{origin:"+move",bias:-1})},goLineRight:function(t){return t.extendSelectionsBy((function(e){var n=t.cursorCoords(e.head,"div").top+5;return t.coordsChar({left:t.display.lineDiv.offsetWidth+100,top:n},"div")}),U)},goLineLeft:function(t){return t.extendSelectionsBy((function(e){var n=t.cursorCoords(e.head,"div").top+5;return t.coordsChar({left:0,top:n},"div")}),U)},goLineLeftSmart:function(t){return t.extendSelectionsBy((function(e){var n=t.cursorCoords(e.head,"div").top+5,r=t.coordsChar({left:0,top:n},"div");return r.ch<t.getLine(r.line).search(/\S/)?ra(t,e.head):r}),U)},goLineUp:function(t){return t.moveV(-1,"line")},goLineDown:function(t){return t.moveV(1,"line")},goPageUp:function(t){return t.moveV(-1,"page")},goPageDown:function(t){return t.moveV(1,"page")},goCharLeft:function(t){return t.moveH(-1,"char")},goCharRight:function(t){return t.moveH(1,"char")},goColumnLeft:function(t){return t.moveH(-1,"column")},goColumnRight:function(t){return t.moveH(1,"column")},goWordLeft:function(t){return t.moveH(-1,"word")},goGroupRight:function(t){return t.moveH(1,"group")},goGroupLeft:function(t){return t.moveH(-1,"group")},goWordRight:function(t){return t.moveH(1,"word")},delCharBefore:function(t){return t.deleteH(-1,"char")},delCharAfter:function(t){return t.deleteH(1,"char")},delWordBefore:function(t){return t.deleteH(-1,"word")},delWordAfter:function(t){return t.deleteH(1,"word")},delGroupBefore:function(t){return t.deleteH(-1,"group")},delGroupAfter:function(t){return t.deleteH(1,"group")},indentAuto:function(t){return t.indentSelection("smart")},indentMore:function(t){return t.indentSelection("add")},indentLess:function(t){return t.indentSelection("subtract")},insertTab:function(t){return t.replaceSelection("\t")},insertSoftTab:function(t){for(var e=[],n=t.listSelections(),r=t.options.tabSize,i=0;i<n.length;i++){var o=n[i].from(),a=V(t.getLine(o.line),o.ch,r);e.push(K(r-a%r));}t.replaceSelections(e);},defaultTab:function(t){t.somethingSelected()?t.indentSelection("add"):t.execCommand("insertTab");},transposeChars:function(t){return Qr(t,(function(){for(var e=t.listSelections(),n=[],r=0;r<e.length;r++)if(e[r].empty()){var i=e[r].head,o=Gt(t.doc,i.line).text;if(o)if(i.ch==o.length&&(i=new ee(i.line,i.ch-1)),i.ch>0)i=new ee(i.line,i.ch+1),t.replaceRange(o.charAt(i.ch-1)+o.charAt(i.ch-2),ee(i.line,i.ch-2),i,"+transpose");else if(i.line>t.doc.first){var a=Gt(t.doc,i.line-1).text;a&&(i=new ee(i.line,1),t.replaceRange(o.charAt(0)+t.doc.lineSeparator()+a.charAt(a.length-1),ee(i.line-1,a.length-1),i,"+transpose"));}n.push(new Ci(i,i));}t.setSelections(n);}))},newlineAndIndent:function(t){return Qr(t,(function(){for(var e=t.listSelections(),n=e.length-1;n>=0;n--)t.replaceRange(t.doc.lineSeparator(),e[n].anchor,e[n].head,"+input");e=t.listSelections();for(var r=0;r<e.length;r++)t.indentLine(e[r].from().line,null,!0);Ar(t);}))},openLine:function(t){return t.replaceSelection("\n","start")},toggleOverwrite:function(t){return t.toggleOverwrite()}};function na(t,e){var n=Gt(t.doc,e),r=Re(n);return r!=n&&(e=Jt(r)),ta(!0,t,r,e,1)}function ra(t,e){var n=na(t,e.line),r=Gt(t.doc,n.line),i=ut(r,t.doc.direction);if(!i||0==i[0].level){var o=Math.max(0,r.text.search(/\S/)),a=e.line==n.line&&e.ch<=o&&e.ch;return ee(n.line,a?0:o,n.sticky)}return n}function ia(t,e,n){if("string"==typeof e&&!(e=ea[e]))return !1;t.display.input.ensurePolled();var r=t.display.shift,i=!1;try{t.isReadOnly()&&(t.state.suppressEdits=!0),n&&(t.display.shift=!1),i=e(t)!=$;}finally{t.display.shift=r,t.state.suppressEdits=!1;}return i}var oa=new R;function aa(t,e,n,r){var i=t.state.keySeq;if(i){if(Go(e))return "handled";if(/\'$/.test(e)?t.state.keySeq=null:oa.set(50,(function(){t.state.keySeq==i&&(t.state.keySeq=null,t.display.input.reset());})),sa(t,i+" "+e,n,r))return !0}return sa(t,e,n,r)}function sa(t,e,n,r){var i=function(t,e,n){for(var r=0;r<t.state.keyMaps.length;r++){var i=qo(e,t.state.keyMaps[r],n,t);if(i)return i}return t.options.extraKeys&&qo(e,t.options.extraKeys,n,t)||qo(e,t.options.keyMap,n,t)}(t,e,r);return "multi"==i&&(t.state.keySeq=e),"handled"==i&&cn(t,"keyHandled",t,e,n),"handled"!=i&&"multi"!=i||(wt(n),Or(t)),!!i}function la(t,e){var n=Yo(e,!0);return !!n&&(e.shiftKey&&!t.state.keySeq?aa(t,"Shift-"+n,e,(function(e){return ia(t,e,!0)}))||aa(t,n,e,(function(e){if("string"==typeof e?/^go[A-Z]/.test(e):e.motion)return ia(t,e)})):aa(t,n,e,(function(e){return ia(t,e)})))}var ca=null;function ua(t){var e=this;if(e.curOp.focus=A(),!vt(e,t)){a&&s<11&&27==t.keyCode&&(t.returnValue=!1);var n=t.keyCode;e.display.shift=16==n||t.shiftKey;var r=la(e,t);h&&(ca=r?n:null,!r&&88==n&&!Nt&&(y?t.metaKey:t.ctrlKey)&&e.replaceSelection("",null,"cut")),18!=n||/\bCodeMirror-crosshair\b/.test(e.display.lineDiv.className)||function(t){var e=t.display.lineDiv;function n(t){18!=t.keyCode&&t.altKey||(T(e,"CodeMirror-crosshair"),pt(document,"keyup",n),pt(document,"mouseover",n));}L(e,"CodeMirror-crosshair"),dt(document,"keyup",n),dt(document,"mouseover",n);}(e);}}function ha(t){16==t.keyCode&&(this.doc.sel.shift=!1),vt(this,t);}function da(t){var e=this;if(!(Cn(e.display,t)||vt(e,t)||t.ctrlKey&&!t.altKey||y&&t.metaKey)){var n=t.keyCode,r=t.charCode;if(h&&n==ca)return ca=null,void wt(t);if(!h||t.which&&!(t.which<10)||!la(e,t)){var i=String.fromCharCode(null==r?n:r);"\b"!=i&&(function(t,e,n){return aa(t,"'"+n+"'",e,(function(e){return ia(t,e,!0)}))}(e,t,i)||e.display.input.onKeyPress(t));}}}var fa,pa,ga=function(t,e,n){this.time=t,this.pos=e,this.button=n;};function va(t){var e=this,n=e.display;if(!(vt(e,t)||n.activeTouch&&n.input.supportsTouch()))if(n.input.ensurePolled(),n.shift=t.shiftKey,Cn(n,t))l||(n.scroller.draggable=!1,setTimeout((function(){return n.scroller.draggable=!0}),100));else if(!ba(e,t)){var r=ur(e,t),i=kt(t),o=r?function(t,e){var n=+new Date;return pa&&pa.compare(n,t,e)?(fa=pa=null,"triple"):fa&&fa.compare(n,t,e)?(pa=new ga(n,t,e),fa=null,"double"):(fa=new ga(n,t,e),pa=null,"single")}(r,i):"single";window.focus(),1==i&&e.state.selectingText&&e.state.selectingText(t),r&&function(t,e,n,r,i){var o="Click";return "double"==r?o="Double"+o:"triple"==r&&(o="Triple"+o),aa(t,Ko(o=(1==e?"Left":2==e?"Middle":"Right")+o,i),i,(function(e){if("string"==typeof e&&(e=ea[e]),!e)return !1;var r=!1;try{t.isReadOnly()&&(t.state.suppressEdits=!0),r=e(t,n)!=$;}finally{t.state.suppressEdits=!1;}return r}))}(e,i,r,o,t)||(1==i?r?function(t,e,n,r){a?setTimeout(F(Cr,t),0):t.curOp.focus=A();var i,o=function(t,e,n){var r=t.getOption("configureMouse"),i=r?r(t,e,n):{};if(null==i.unit){var o=b?n.shiftKey&&n.metaKey:n.altKey;i.unit=o?"rectangle":"single"==e?"char":"double"==e?"word":"line";}return (null==i.extend||t.doc.extend)&&(i.extend=t.doc.extend||n.shiftKey),null==i.addNew&&(i.addNew=y?n.metaKey:n.ctrlKey),null==i.moveOnDrag&&(i.moveOnDrag=!(y?n.altKey:n.ctrlKey)),i}(t,n,r),c=t.doc.sel;t.options.dragDrop&&Pt&&!t.isReadOnly()&&"single"==n&&(i=c.contains(e))>-1&&(ne((i=c.ranges[i]).from(),e)<0||e.xRel>0)&&(ne(i.to(),e)>0||e.xRel<0)?function(t,e,n,r){var i=t.display,o=!1,c=ti(t,(function(e){l&&(i.scroller.draggable=!1),t.state.draggingText=!1,pt(i.wrapper.ownerDocument,"mouseup",c),pt(i.wrapper.ownerDocument,"mousemove",u),pt(i.scroller,"dragstart",h),pt(i.scroller,"drop",c),o||(wt(e),r.addNew||Ki(t.doc,n,null,null,r.extend),l||a&&9==s?setTimeout((function(){i.wrapper.ownerDocument.body.focus(),i.input.focus();}),20):i.input.focus());})),u=function(t){o=o||Math.abs(e.clientX-t.clientX)+Math.abs(e.clientY-t.clientY)>=10;},h=function(){return o=!0};l&&(i.scroller.draggable=!0),t.state.draggingText=c,c.copy=!r.moveOnDrag,i.scroller.dragDrop&&i.scroller.dragDrop(),dt(i.wrapper.ownerDocument,"mouseup",c),dt(i.wrapper.ownerDocument,"mousemove",u),dt(i.scroller,"dragstart",h),dt(i.scroller,"drop",c),Sr(t),setTimeout((function(){return i.input.focus()}),20);}(t,r,e,o):function(t,e,n,r){var i=t.display,o=t.doc;wt(e);var a,s,l=o.sel,c=l.ranges;if(r.addNew&&!r.extend?(s=o.sel.contains(n),a=s>-1?c[s]:new Ci(n,n)):(a=o.sel.primary(),s=o.sel.primIndex),"rectangle"==r.unit)r.addNew||(a=new Ci(n,n)),n=ur(t,e,!0,!0),s=-1;else {var u=ma(t,n,r.unit);a=r.extend?Gi(a,u.anchor,u.head,r.extend):u;}r.addNew?-1==s?(s=c.length,Qi(o,Si(t,c.concat([a]),s),{scroll:!1,origin:"*mouse"})):c.length>1&&c[s].empty()&&"char"==r.unit&&!r.extend?(Qi(o,Si(t,c.slice(0,s).concat(c.slice(s+1)),0),{scroll:!1,origin:"*mouse"}),l=o.sel):Xi(o,s,a,B):(s=0,Qi(o,new Oi([a],0),B),l=o.sel);var h=n;function d(e){if(0!=ne(h,e))if(h=e,"rectangle"==r.unit){for(var i=[],c=t.options.tabSize,u=V(Gt(o,n.line).text,n.ch,c),d=V(Gt(o,e.line).text,e.ch,c),f=Math.min(u,d),p=Math.max(u,d),g=Math.min(n.line,e.line),v=Math.min(t.lastLine(),Math.max(n.line,e.line));g<=v;g++){var m=Gt(o,g).text,y=q(m,f,c);f==p?i.push(new Ci(ee(g,y),ee(g,y))):m.length>y&&i.push(new Ci(ee(g,y),ee(g,q(m,p,c))));}i.length||i.push(new Ci(n,n)),Qi(o,Si(t,l.ranges.slice(0,s).concat(i),s),{origin:"*mouse",scroll:!1}),t.scrollIntoView(e);}else {var b,w=a,x=ma(t,e,r.unit),O=w.anchor;ne(x.anchor,O)>0?(b=x.head,O=ae(w.from(),x.anchor)):(b=x.anchor,O=oe(w.to(),x.head));var C=l.ranges.slice(0);C[s]=function(t,e){var n=e.anchor,r=e.head,i=Gt(t.doc,n.line);if(0==ne(n,r)&&n.sticky==r.sticky)return e;var o=ut(i);if(!o)return e;var a=lt(o,n.ch,n.sticky),s=o[a];if(s.from!=n.ch&&s.to!=n.ch)return e;var l,c=a+(s.from==n.ch==(1!=s.level)?0:1);if(0==c||c==o.length)return e;if(r.line!=n.line)l=(r.line-n.line)*("ltr"==t.doc.direction?1:-1)>0;else {var u=lt(o,r.ch,r.sticky),h=u-a||(r.ch-n.ch)*(1==s.level?-1:1);l=u==c-1||u==c?h<0:h>0;}var d=o[c+(l?-1:0)],f=l==(1==d.level),p=f?d.from:d.to,g=f?"after":"before";return n.ch==p&&n.sticky==g?e:new Ci(new ee(n.line,p,g),r)}(t,new Ci(le(o,O),b)),Qi(o,Si(t,C,s),B);}}var f=i.wrapper.getBoundingClientRect(),p=0;function g(e){t.state.selectingText=!1,p=1/0,e&&(wt(e),i.input.focus()),pt(i.wrapper.ownerDocument,"mousemove",v),pt(i.wrapper.ownerDocument,"mouseup",m),o.history.lastSelOrigin=null;}var v=ti(t,(function(e){0!==e.buttons&&kt(e)?function e(n){var a=++p,s=ur(t,n,!0,"rectangle"==r.unit);if(s)if(0!=ne(s,h)){t.curOp.focus=A(),d(s);var l=Er(i,o);(s.line>=l.to||s.line<l.from)&&setTimeout(ti(t,(function(){p==a&&e(n);})),150);}else {var c=n.clientY<f.top?-20:n.clientY>f.bottom?20:0;c&&setTimeout(ti(t,(function(){p==a&&(i.scroller.scrollTop+=c,e(n));})),50);}}(e):g(e);})),m=ti(t,g);t.state.selectingText=m,dt(i.wrapper.ownerDocument,"mousemove",v),dt(i.wrapper.ownerDocument,"mouseup",m);}(t,r,e,o);}(e,r,o,t):St(t)==n.scroller&&wt(t):2==i?(r&&Ki(e.doc,r),setTimeout((function(){return n.input.focus()}),20)):3==i&&(C?e.display.input.onContextMenu(t):Sr(e)));}}function ma(t,e,n){if("char"==n)return new Ci(e,e);if("word"==n)return t.findWordAt(e);if("line"==n)return new Ci(ee(e.line,0),le(t.doc,ee(e.line+1,0)));var r=n(t,e);return new Ci(r.from,r.to)}function ya(t,e,n,r){var i,o;if(e.touches)i=e.touches[0].clientX,o=e.touches[0].clientY;else try{i=e.clientX,o=e.clientY;}catch(e){return !1}if(i>=Math.floor(t.display.gutters.getBoundingClientRect().right))return !1;r&&wt(e);var a=t.display,s=a.lineDiv.getBoundingClientRect();if(o>s.bottom||!yt(t,n))return Ot(e);o-=s.top-a.viewOffset;for(var l=0;l<t.display.gutterSpecs.length;++l){var c=a.gutters.childNodes[l];if(c&&c.getBoundingClientRect().right>=i)return gt(t,n,t,Zt(t.doc,o),t.display.gutterSpecs[l].className,e),Ot(e)}}function ba(t,e){return ya(t,e,"gutterClick",!0)}function wa(t,e){Cn(t.display,e)||function(t,e){return !!yt(t,"gutterContextMenu")&&ya(t,e,"gutterContextMenu",!1)}(t,e)||vt(t,e,"contextmenu")||C||t.display.input.onContextMenu(e);}function xa(t){t.display.wrapper.className=t.display.wrapper.className.replace(/\s*cm-s-\S+/g,"")+t.options.theme.replace(/(^|\s)\s*/g," cm-s-"),zn(t);}ga.prototype.compare=function(t,e,n){return this.time+400>t&&0==ne(e,this.pos)&&n==this.button};var Oa={toString:function(){return "CodeMirror.Init"}},Ca={},Sa={};function ka(t,e,n){if(!e!=!(n&&n!=Oa)){var r=t.display.dragFunctions,i=e?dt:pt;i(t.display.scroller,"dragstart",r.start),i(t.display.scroller,"dragenter",r.enter),i(t.display.scroller,"dragover",r.over),i(t.display.scroller,"dragleave",r.leave),i(t.display.scroller,"drop",r.drop);}}function Ta(t){t.options.lineWrapping?(L(t.display.wrapper,"CodeMirror-wrap"),t.display.sizer.style.minWidth="",t.display.sizerWidth=null):(T(t.display.wrapper,"CodeMirror-wrap"),qe(t)),cr(t),dr(t),zn(t),setTimeout((function(){return $r(t)}),100);}function ja(t,e){var n=this;if(!(this instanceof ja))return new ja(t,e);this.options=e=e?_(e):{},_(Ca,e,!1);var r=e.value;"string"==typeof r?r=new Do(r,e.mode,null,e.lineSeparator,e.direction):e.mode&&(r.modeOption=e.mode),this.doc=r;var i=new ja.inputStyles[e.inputStyle](this),o=this.display=new vi(t,r,i,e);for(var c in o.wrapper.CodeMirror=this,xa(this),e.lineWrapping&&(this.display.wrapper.className+=" CodeMirror-wrap"),Ur(this),this.state={keyMaps:[],overlays:[],modeGen:0,overwrite:!1,delayingBlurEvent:!1,focused:!1,suppressEdits:!1,pasteIncoming:-1,cutIncoming:-1,selectingText:!1,draggingText:!1,highlight:new R,keySeq:null,specialChars:null},e.autofocus&&!m&&o.input.focus(),a&&s<11&&setTimeout((function(){return n.display.input.reset(!0)}),20),function(t){var e=t.display;dt(e.scroller,"mousedown",ti(t,va)),dt(e.scroller,"dblclick",a&&s<11?ti(t,(function(e){if(!vt(t,e)){var n=ur(t,e);if(n&&!ba(t,e)&&!Cn(t.display,e)){wt(e);var r=t.findWordAt(n);Ki(t.doc,r.anchor,r.head);}}})):function(e){return vt(t,e)||wt(e)}),dt(e.scroller,"contextmenu",(function(e){return wa(t,e)}));var n,r={end:0};function i(){e.activeTouch&&(n=setTimeout((function(){return e.activeTouch=null}),1e3),(r=e.activeTouch).end=+new Date);}function o(t,e){if(null==e.left)return !0;var n=e.left-t.left,r=e.top-t.top;return n*n+r*r>400}dt(e.scroller,"touchstart",(function(i){if(!vt(t,i)&&!function(t){if(1!=t.touches.length)return !1;var e=t.touches[0];return e.radiusX<=1&&e.radiusY<=1}(i)&&!ba(t,i)){e.input.ensurePolled(),clearTimeout(n);var o=+new Date;e.activeTouch={start:o,moved:!1,prev:o-r.end<=300?r:null},1==i.touches.length&&(e.activeTouch.left=i.touches[0].pageX,e.activeTouch.top=i.touches[0].pageY);}})),dt(e.scroller,"touchmove",(function(){e.activeTouch&&(e.activeTouch.moved=!0);})),dt(e.scroller,"touchend",(function(n){var r=e.activeTouch;if(r&&!Cn(e,n)&&null!=r.left&&!r.moved&&new Date-r.start<300){var a,s=t.coordsChar(e.activeTouch,"page");a=!r.prev||o(r,r.prev)?new Ci(s,s):!r.prev.prev||o(r,r.prev.prev)?t.findWordAt(s):new Ci(ee(s.line,0),le(t.doc,ee(s.line+1,0))),t.setSelection(a.anchor,a.head),t.focus(),wt(n);}i();})),dt(e.scroller,"touchcancel",i),dt(e.scroller,"scroll",(function(){e.scroller.clientHeight&&(Fr(t,e.scroller.scrollTop),Vr(t,e.scroller.scrollLeft,!0),gt(t,"scroll",t));})),dt(e.scroller,"mousewheel",(function(e){return xi(t,e)})),dt(e.scroller,"DOMMouseScroll",(function(e){return xi(t,e)})),dt(e.wrapper,"scroll",(function(){return e.wrapper.scrollTop=e.wrapper.scrollLeft=0})),e.dragFunctions={enter:function(e){vt(t,e)||Ct(e);},over:function(e){vt(t,e)||(function(t,e){var n=ur(t,e);if(n){var r=document.createDocumentFragment();br(t,n,r),t.display.dragCursor||(t.display.dragCursor=E("div",null,"CodeMirror-cursors CodeMirror-dragcursors"),t.display.lineSpace.insertBefore(t.display.dragCursor,t.display.cursorDiv)),P(t.display.dragCursor,r);}}(t,e),Ct(e));},start:function(e){return function(t,e){if(a&&(!t.state.draggingText||+new Date-Ao<100))Ct(e);else if(!vt(t,e)&&!Cn(t.display,e)&&(e.dataTransfer.setData("Text",t.getSelection()),e.dataTransfer.effectAllowed="copyMove",e.dataTransfer.setDragImage&&!d)){var n=E("img",null,null,"position: fixed; left: 0; top: 0;");n.src="data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==",h&&(n.width=n.height=1,t.display.wrapper.appendChild(n),n._top=n.offsetTop),e.dataTransfer.setDragImage(n,0,0),h&&n.parentNode.removeChild(n);}}(t,e)},drop:ti(t,Lo),leave:function(e){vt(t,e)||No(t);}};var l=e.input.getField();dt(l,"keyup",(function(e){return ha.call(t,e)})),dt(l,"keydown",ti(t,ua)),dt(l,"keypress",ti(t,da)),dt(l,"focus",(function(e){return kr(t,e)})),dt(l,"blur",(function(e){return Tr(t,e)}));}(this),_o(),Gr(this),this.curOp.forceUpdate=!0,Ii(this,r),e.autofocus&&!m||this.hasFocus()?setTimeout(F(kr,this),20):Tr(this),Sa)Sa.hasOwnProperty(c)&&Sa[c](n,e[c],Oa);di(this),e.finishInit&&e.finishInit(this);for(var u=0;u<Pa.length;++u)Pa[u](n);Kr(this),l&&e.lineWrapping&&"optimizelegibility"==getComputedStyle(o.lineDiv).textRendering&&(o.lineDiv.style.textRendering="auto");}ja.defaults=Ca,ja.optionHandlers=Sa;var Pa=[];function Ea(t,e,n,r){var i,o=t.doc;null==n&&(n="add"),"smart"==n&&(o.mode.indent?i=pe(t,e).state:n="prev");var a=t.options.tabSize,s=Gt(o,e),l=V(s.text,null,a);s.stateAfter&&(s.stateAfter=null);var c,u=s.text.match(/^\s*/)[0];if(r||/\S/.test(s.text)){if("smart"==n&&((c=o.mode.indent(i,s.text.slice(u.length),s.text))==$||c>150)){if(!r)return;n="prev";}}else c=0,n="not";"prev"==n?c=e>o.first?V(Gt(o,e-1).text,null,a):0:"add"==n?c=l+t.options.indentUnit:"subtract"==n?c=l-t.options.indentUnit:"number"==typeof n&&(c=l+n),c=Math.max(0,c);var h="",d=0;if(t.options.indentWithTabs)for(var f=Math.floor(c/a);f;--f)d+=a,h+="\t";if(d<c&&(h+=K(c-d)),h!=u)return go(o,h,ee(e,0),ee(e,u.length),"+input"),s.stateAfter=null,!0;for(var p=0;p<o.sel.ranges.length;p++){var g=o.sel.ranges[p];if(g.head.line==e&&g.head.ch<u.length){var v=ee(e,u.length);Xi(o,p,new Ci(v,v));break}}}ja.defineInitHook=function(t){return Pa.push(t)};var Ma=null;function Da(t){Ma=t;}function Aa(t,e,n,r,i){var o=t.doc;t.display.shift=!1,r||(r=o.sel);var a=+new Date-200,s="paste"==i||t.state.pasteIncoming>a,l=At(e),c=null;if(s&&r.ranges.length>1)if(Ma&&Ma.text.join("\n")==e){if(r.ranges.length%Ma.text.length==0){c=[];for(var u=0;u<Ma.text.length;u++)c.push(o.splitLines(Ma.text[u]));}}else l.length==r.ranges.length&&t.options.pasteLinesPerSelection&&(c=X(l,(function(t){return [t]})));for(var h=t.curOp.updateInput,d=r.ranges.length-1;d>=0;d--){var f=r.ranges[d],p=f.from(),g=f.to();f.empty()&&(n&&n>0?p=ee(p.line,p.ch-n):t.state.overwrite&&!s?g=ee(g.line,Math.min(Gt(o,g.line).text.length,g.ch+Y(l).length)):s&&Ma&&Ma.lineWise&&Ma.text.join("\n")==e&&(p=g=ee(p.line,0)));var v={from:p,to:g,text:c?c[d%c.length]:l,origin:i||(s?"paste":t.state.cutIncoming>a?"cut":"+input")};co(t.doc,v),cn(t,"inputRead",t,v);}e&&!s&&Na(t,e),Ar(t),t.curOp.updateInput<2&&(t.curOp.updateInput=h),t.curOp.typing=!0,t.state.pasteIncoming=t.state.cutIncoming=-1;}function La(t,e){var n=t.clipboardData&&t.clipboardData.getData("Text");if(n)return t.preventDefault(),e.isReadOnly()||e.options.disableInput||Qr(e,(function(){return Aa(e,n,0,null,"paste")})),!0}function Na(t,e){if(t.options.electricChars&&t.options.smartIndent)for(var n=t.doc.sel,r=n.ranges.length-1;r>=0;r--){var i=n.ranges[r];if(!(i.head.ch>100||r&&n.ranges[r-1].head.line==i.head.line)){var o=t.getModeAt(i.head),a=!1;if(o.electricChars){for(var s=0;s<o.electricChars.length;s++)if(e.indexOf(o.electricChars.charAt(s))>-1){a=Ea(t,i.head.line,"smart");break}}else o.electricInput&&o.electricInput.test(Gt(t.doc,i.head.line).text.slice(0,i.head.ch))&&(a=Ea(t,i.head.line,"smart"));a&&cn(t,"electricInput",t,i.head.line);}}}function Ia(t){for(var e=[],n=[],r=0;r<t.doc.sel.ranges.length;r++){var i=t.doc.sel.ranges[r].head.line,o={anchor:ee(i,0),head:ee(i+1,0)};n.push(o),e.push(t.getRange(o.anchor,o.head));}return {text:e,ranges:n}}function Fa(t,e,n,r){t.setAttribute("autocorrect",n?"":"off"),t.setAttribute("autocapitalize",r?"":"off"),t.setAttribute("spellcheck",!!e);}function _a(){var t=E("textarea",null,null,"position: absolute; bottom: -1em; padding: 0; width: 1px; height: 1em; outline: none"),e=E("div",[t],null,"overflow: hidden; position: relative; width: 3px; height: 0px;");return l?t.style.width="1000px":t.setAttribute("wrap","off"),g&&(t.style.border="1px solid black"),Fa(t),e}function Va(t,e,n,r,i){var o=e,a=n,s=Gt(t,e.line);function l(r){var o,a;if(null==(o=i?function(t,e,n,r){var i=ut(e,t.doc.direction);if(!i)return Qo(e,n,r);n.ch>=e.text.length?(n.ch=e.text.length,n.sticky="before"):n.ch<=0&&(n.ch=0,n.sticky="after");var o=lt(i,n.ch,n.sticky),a=i[o];if("ltr"==t.doc.direction&&a.level%2==0&&(r>0?a.to>n.ch:a.from<n.ch))return Qo(e,n,r);var s,l=function(t,n){return Zo(e,t instanceof ee?t.ch:t,n)},c=function(n){return t.options.lineWrapping?(s=s||Ln(t,e),Qn(t,e,s,n)):{begin:0,end:e.text.length}},u=c("before"==n.sticky?l(n,-1):n.ch);if("rtl"==t.doc.direction||1==a.level){var h=1==a.level==r<0,d=l(n,h?1:-1);if(null!=d&&(h?d<=a.to&&d<=u.end:d>=a.from&&d>=u.begin)){var f=h?"before":"after";return new ee(n.line,d,f)}}var p=function(t,e,r){for(var o=function(t,e){return e?new ee(n.line,l(t,1),"before"):new ee(n.line,t,"after")};t>=0&&t<i.length;t+=e){var a=i[t],s=e>0==(1!=a.level),c=s?r.begin:l(r.end,-1);if(a.from<=c&&c<a.to)return o(c,s);if(c=s?a.from:l(a.to,-1),r.begin<=c&&c<r.end)return o(c,s)}},g=p(o+r,r,u);if(g)return g;var v=r>0?u.end:l(u.begin,-1);return null==v||r>0&&v==e.text.length||!(g=p(r>0?0:i.length-1,r,c(v)))?null:g}(t.cm,s,e,n):Qo(s,e,n))){if(r||(a=e.line+n)<t.first||a>=t.first+t.size||(e=new ee(a,e.ch,e.sticky),!(s=Gt(t,a))))return !1;e=ta(i,t.cm,s,e.line,n);}else e=o;return !0}if("char"==r)l();else if("column"==r)l(!0);else if("word"==r||"group"==r)for(var c=null,u="group"==r,h=t.cm&&t.cm.getHelper(e,"wordChars"),d=!0;!(n<0)||l(!d);d=!1){var f=s.text.charAt(e.ch)||"\n",p=et(f,h)?"w":u&&"\n"==f?"n":!u||/\s/.test(f)?null:"p";if(!u||d||p||(p="s"),c&&c!=p){n<0&&(n=1,l(),e.sticky="after");break}if(p&&(c=p),n>0&&!l(!d))break}var g=oo(t,e,o,a,!0);return re(o,g)&&(g.hitSide=!0),g}function Ra(t,e,n,r){var i,o,a=t.doc,s=e.left;if("page"==r){var l=Math.min(t.display.wrapper.clientHeight,window.innerHeight||document.documentElement.clientHeight),c=Math.max(l-.5*ir(t.display),3);i=(n>0?e.bottom:e.top)+n*c;}else "line"==r&&(i=n>0?e.bottom+3:e.top-3);for(;(o=Jn(t,s,i)).outside;){if(n<0?i<=0:i>=a.height){o.hitSide=!0;break}i+=5*n;}return o}var Ha=function(t){this.cm=t,this.lastAnchorNode=this.lastAnchorOffset=this.lastFocusNode=this.lastFocusOffset=null,this.polling=new R,this.composing=null,this.gracePeriod=!1,this.readDOMTimeout=null;};function za(t,e){var n=An(t,e.line);if(!n||n.hidden)return null;var r=Gt(t.doc,e.line),i=Mn(n,r,e.line),o=ut(r,t.doc.direction),a="left";o&&(a=lt(o,e.ch)%2?"right":"left");var s=_n(i.map,e.ch,a);return s.offset="right"==s.collapse?s.end:s.start,s}function $a(t,e){return e&&(t.bad=!0),t}function Wa(t,e,n){var r;if(e==t.display.lineDiv){if(!(r=t.display.lineDiv.childNodes[n]))return $a(t.clipPos(ee(t.display.viewTo-1)),!0);e=null,n=0;}else for(r=e;;r=r.parentNode){if(!r||r==t.display.lineDiv)return null;if(r.parentNode&&r.parentNode==t.display.lineDiv)break}for(var i=0;i<t.display.view.length;i++){var o=t.display.view[i];if(o.node==r)return Ba(o,e,n)}}function Ba(t,e,n){var r=t.text.firstChild,i=!1;if(!e||!D(r,e))return $a(ee(Jt(t.line),0),!0);if(e==r&&(i=!0,e=r.childNodes[n],n=0,!e)){var o=t.rest?Y(t.rest):t.line;return $a(ee(Jt(o),o.text.length),i)}var a=3==e.nodeType?e:null,s=e;for(a||1!=e.childNodes.length||3!=e.firstChild.nodeType||(a=e.firstChild,n&&(n=a.nodeValue.length));s.parentNode!=r;)s=s.parentNode;var l=t.measure,c=l.maps;function u(e,n,r){for(var i=-1;i<(c?c.length:0);i++)for(var o=i<0?l.map:c[i],a=0;a<o.length;a+=3){var s=o[a+2];if(s==e||s==n){var u=Jt(i<0?t.line:t.rest[i]),h=o[a]+r;return (r<0||s!=e)&&(h=o[a+(r?1:0)]),ee(u,h)}}}var h=u(a,s,n);if(h)return $a(h,i);for(var d=s.nextSibling,f=a?a.nodeValue.length-n:0;d;d=d.nextSibling){if(h=u(d,d.firstChild,0))return $a(ee(h.line,h.ch-f),i);f+=d.textContent.length;}for(var p=s.previousSibling,g=n;p;p=p.previousSibling){if(h=u(p,p.firstChild,-1))return $a(ee(h.line,h.ch+g),i);g+=p.textContent.length;}}Ha.prototype.init=function(t){var e=this,n=this,r=n.cm,i=n.div=t.lineDiv;function o(t){if(!vt(r,t)){if(r.somethingSelected())Da({lineWise:!1,text:r.getSelections()}),"cut"==t.type&&r.replaceSelection("",null,"cut");else {if(!r.options.lineWiseCopyCut)return;var e=Ia(r);Da({lineWise:!0,text:e.text}),"cut"==t.type&&r.operation((function(){r.setSelections(e.ranges,0,W),r.replaceSelection("",null,"cut");}));}if(t.clipboardData){t.clipboardData.clearData();var o=Ma.text.join("\n");if(t.clipboardData.setData("Text",o),t.clipboardData.getData("Text")==o)return void t.preventDefault()}var a=_a(),s=a.firstChild;r.display.lineSpace.insertBefore(a,r.display.lineSpace.firstChild),s.value=Ma.text.join("\n");var l=document.activeElement;I(s),setTimeout((function(){r.display.lineSpace.removeChild(a),l.focus(),l==i&&n.showPrimarySelection();}),50);}}Fa(i,r.options.spellcheck,r.options.autocorrect,r.options.autocapitalize),dt(i,"paste",(function(t){vt(r,t)||La(t,r)||s<=11&&setTimeout(ti(r,(function(){return e.updateFromDOM()})),20);})),dt(i,"compositionstart",(function(t){e.composing={data:t.data,done:!1};})),dt(i,"compositionupdate",(function(t){e.composing||(e.composing={data:t.data,done:!1});})),dt(i,"compositionend",(function(t){e.composing&&(t.data!=e.composing.data&&e.readFromDOMSoon(),e.composing.done=!0);})),dt(i,"touchstart",(function(){return n.forceCompositionEnd()})),dt(i,"input",(function(){e.composing||e.readFromDOMSoon();})),dt(i,"copy",o),dt(i,"cut",o);},Ha.prototype.prepareSelection=function(){var t=yr(this.cm,!1);return t.focus=this.cm.state.focused,t},Ha.prototype.showSelection=function(t,e){t&&this.cm.display.view.length&&((t.focus||e)&&this.showPrimarySelection(),this.showMultipleSelections(t));},Ha.prototype.getSelection=function(){return this.cm.display.wrapper.ownerDocument.getSelection()},Ha.prototype.showPrimarySelection=function(){var t=this.getSelection(),e=this.cm,r=e.doc.sel.primary(),i=r.from(),o=r.to();if(e.display.viewTo==e.display.viewFrom||i.line>=e.display.viewTo||o.line<e.display.viewFrom)t.removeAllRanges();else {var a=Wa(e,t.anchorNode,t.anchorOffset),s=Wa(e,t.focusNode,t.focusOffset);if(!a||a.bad||!s||s.bad||0!=ne(ae(a,s),i)||0!=ne(oe(a,s),o)){var l=e.display.view,c=i.line>=e.display.viewFrom&&za(e,i)||{node:l[0].measure.map[2],offset:0},u=o.line<e.display.viewTo&&za(e,o);if(!u){var h=l[l.length-1].measure,d=h.maps?h.maps[h.maps.length-1]:h.map;u={node:d[d.length-1],offset:d[d.length-2]-d[d.length-3]};}if(c&&u){var f,p=t.rangeCount&&t.getRangeAt(0);try{f=k(c.node,c.offset,u.offset,u.node);}catch(t){}f&&(!n&&e.state.focused?(t.collapse(c.node,c.offset),f.collapsed||(t.removeAllRanges(),t.addRange(f))):(t.removeAllRanges(),t.addRange(f)),p&&null==t.anchorNode?t.addRange(p):n&&this.startGracePeriod()),this.rememberSelection();}else t.removeAllRanges();}}},Ha.prototype.startGracePeriod=function(){var t=this;clearTimeout(this.gracePeriod),this.gracePeriod=setTimeout((function(){t.gracePeriod=!1,t.selectionChanged()&&t.cm.operation((function(){return t.cm.curOp.selectionChanged=!0}));}),20);},Ha.prototype.showMultipleSelections=function(t){P(this.cm.display.cursorDiv,t.cursors),P(this.cm.display.selectionDiv,t.selection);},Ha.prototype.rememberSelection=function(){var t=this.getSelection();this.lastAnchorNode=t.anchorNode,this.lastAnchorOffset=t.anchorOffset,this.lastFocusNode=t.focusNode,this.lastFocusOffset=t.focusOffset;},Ha.prototype.selectionInEditor=function(){var t=this.getSelection();if(!t.rangeCount)return !1;var e=t.getRangeAt(0).commonAncestorContainer;return D(this.div,e)},Ha.prototype.focus=function(){"nocursor"!=this.cm.options.readOnly&&(this.selectionInEditor()||this.showSelection(this.prepareSelection(),!0),this.div.focus());},Ha.prototype.blur=function(){this.div.blur();},Ha.prototype.getField=function(){return this.div},Ha.prototype.supportsTouch=function(){return !0},Ha.prototype.receivedFocus=function(){var t=this;this.selectionInEditor()?this.pollSelection():Qr(this.cm,(function(){return t.cm.curOp.selectionChanged=!0})),this.polling.set(this.cm.options.pollInterval,(function e(){t.cm.state.focused&&(t.pollSelection(),t.polling.set(t.cm.options.pollInterval,e));}));},Ha.prototype.selectionChanged=function(){var t=this.getSelection();return t.anchorNode!=this.lastAnchorNode||t.anchorOffset!=this.lastAnchorOffset||t.focusNode!=this.lastFocusNode||t.focusOffset!=this.lastFocusOffset},Ha.prototype.pollSelection=function(){if(null==this.readDOMTimeout&&!this.gracePeriod&&this.selectionChanged()){var t=this.getSelection(),e=this.cm;if(v&&u&&this.cm.display.gutterSpecs.length&&function(t){for(var e=t;e;e=e.parentNode)if(/CodeMirror-gutter-wrapper/.test(e.className))return !0;return !1}(t.anchorNode))return this.cm.triggerOnKeyDown({type:"keydown",keyCode:8,preventDefault:Math.abs}),this.blur(),void this.focus();if(!this.composing){this.rememberSelection();var n=Wa(e,t.anchorNode,t.anchorOffset),r=Wa(e,t.focusNode,t.focusOffset);n&&r&&Qr(e,(function(){Qi(e.doc,ki(n,r),W),(n.bad||r.bad)&&(e.curOp.selectionChanged=!0);}));}}},Ha.prototype.pollContent=function(){null!=this.readDOMTimeout&&(clearTimeout(this.readDOMTimeout),this.readDOMTimeout=null);var t,e,n,r=this.cm,i=r.display,o=r.doc.sel.primary(),a=o.from(),s=o.to();if(0==a.ch&&a.line>r.firstLine()&&(a=ee(a.line-1,Gt(r.doc,a.line-1).length)),s.ch==Gt(r.doc,s.line).text.length&&s.line<r.lastLine()&&(s=ee(s.line+1,0)),a.line<i.viewFrom||s.line>i.viewTo-1)return !1;a.line==i.viewFrom||0==(t=hr(r,a.line))?(e=Jt(i.view[0].line),n=i.view[0].node):(e=Jt(i.view[t].line),n=i.view[t-1].node.nextSibling);var l,c,u=hr(r,s.line);if(u==i.view.length-1?(l=i.viewTo-1,c=i.lineDiv.lastChild):(l=Jt(i.view[u+1].line)-1,c=i.view[u+1].node.previousSibling),!n)return !1;for(var h=r.doc.splitLines(function(t,e,n,r,i){var o="",a=!1,s=t.doc.lineSeparator(),l=!1;function c(){a&&(o+=s,l&&(o+=s),a=l=!1);}function u(t){t&&(c(),o+=t);}function h(e){if(1==e.nodeType){var n=e.getAttribute("cm-text");if(n)return void u(n);var o,d=e.getAttribute("cm-marker");if(d){var f=t.findMarks(ee(r,0),ee(i+1,0),(v=+d,function(t){return t.id==v}));return void(f.length&&(o=f[0].find(0))&&u(Kt(t.doc,o.from,o.to).join(s)))}if("false"==e.getAttribute("contenteditable"))return;var p=/^(pre|div|p|li|table|br)$/i.test(e.nodeName);if(!/^br$/i.test(e.nodeName)&&0==e.textContent.length)return;p&&c();for(var g=0;g<e.childNodes.length;g++)h(e.childNodes[g]);/^(pre|p)$/i.test(e.nodeName)&&(l=!0),p&&(a=!0);}else 3==e.nodeType&&u(e.nodeValue.replace(/\u200b/g,"").replace(/\u00a0/g," "));var v;}for(;h(e),e!=n;)e=e.nextSibling,l=!1;return o}(r,n,c,e,l)),d=Kt(r.doc,ee(e,0),ee(l,Gt(r.doc,l).text.length));h.length>1&&d.length>1;)if(Y(h)==Y(d))h.pop(),d.pop(),l--;else {if(h[0]!=d[0])break;h.shift(),d.shift(),e++;}for(var f=0,p=0,g=h[0],v=d[0],m=Math.min(g.length,v.length);f<m&&g.charCodeAt(f)==v.charCodeAt(f);)++f;for(var y=Y(h),b=Y(d),w=Math.min(y.length-(1==h.length?f:0),b.length-(1==d.length?f:0));p<w&&y.charCodeAt(y.length-p-1)==b.charCodeAt(b.length-p-1);)++p;if(1==h.length&&1==d.length&&e==a.line)for(;f&&f>a.ch&&y.charCodeAt(y.length-p-1)==b.charCodeAt(b.length-p-1);)f--,p++;h[h.length-1]=y.slice(0,y.length-p).replace(/^\u200b+/,""),h[0]=h[0].slice(f).replace(/\u200b+$/,"");var x=ee(e,f),O=ee(l,d.length?Y(d).length-p:0);return h.length>1||h[0]||ne(x,O)?(go(r.doc,h,x,O,"+input"),!0):void 0},Ha.prototype.ensurePolled=function(){this.forceCompositionEnd();},Ha.prototype.reset=function(){this.forceCompositionEnd();},Ha.prototype.forceCompositionEnd=function(){this.composing&&(clearTimeout(this.readDOMTimeout),this.composing=null,this.updateFromDOM(),this.div.blur(),this.div.focus());},Ha.prototype.readFromDOMSoon=function(){var t=this;null==this.readDOMTimeout&&(this.readDOMTimeout=setTimeout((function(){if(t.readDOMTimeout=null,t.composing){if(!t.composing.done)return;t.composing=null;}t.updateFromDOM();}),80));},Ha.prototype.updateFromDOM=function(){var t=this;!this.cm.isReadOnly()&&this.pollContent()||Qr(this.cm,(function(){return dr(t.cm)}));},Ha.prototype.setUneditable=function(t){t.contentEditable="false";},Ha.prototype.onKeyPress=function(t){0==t.charCode||this.composing||(t.preventDefault(),this.cm.isReadOnly()||ti(this.cm,Aa)(this.cm,String.fromCharCode(null==t.charCode?t.keyCode:t.charCode),0));},Ha.prototype.readOnlyChanged=function(t){this.div.contentEditable=String("nocursor"!=t);},Ha.prototype.onContextMenu=function(){},Ha.prototype.resetPosition=function(){},Ha.prototype.needsContentAttribute=!0;var Ua=function(t){this.cm=t,this.prevInput="",this.pollingFast=!1,this.polling=new R,this.hasSelection=!1,this.composing=null;};Ua.prototype.init=function(t){var e=this,n=this,r=this.cm;this.createField(t);var i=this.textarea;function o(t){if(!vt(r,t)){if(r.somethingSelected())Da({lineWise:!1,text:r.getSelections()});else {if(!r.options.lineWiseCopyCut)return;var e=Ia(r);Da({lineWise:!0,text:e.text}),"cut"==t.type?r.setSelections(e.ranges,null,W):(n.prevInput="",i.value=e.text.join("\n"),I(i));}"cut"==t.type&&(r.state.cutIncoming=+new Date);}}t.wrapper.insertBefore(this.wrapper,t.wrapper.firstChild),g&&(i.style.width="0px"),dt(i,"input",(function(){a&&s>=9&&e.hasSelection&&(e.hasSelection=null),n.poll();})),dt(i,"paste",(function(t){vt(r,t)||La(t,r)||(r.state.pasteIncoming=+new Date,n.fastPoll());})),dt(i,"cut",o),dt(i,"copy",o),dt(t.scroller,"paste",(function(e){if(!Cn(t,e)&&!vt(r,e)){if(!i.dispatchEvent)return r.state.pasteIncoming=+new Date,void n.focus();var o=new Event("paste");o.clipboardData=e.clipboardData,i.dispatchEvent(o);}})),dt(t.lineSpace,"selectstart",(function(e){Cn(t,e)||wt(e);})),dt(i,"compositionstart",(function(){var t=r.getCursor("from");n.composing&&n.composing.range.clear(),n.composing={start:t,range:r.markText(t,r.getCursor("to"),{className:"CodeMirror-composing"})};})),dt(i,"compositionend",(function(){n.composing&&(n.poll(),n.composing.range.clear(),n.composing=null);}));},Ua.prototype.createField=function(t){this.wrapper=_a(),this.textarea=this.wrapper.firstChild;},Ua.prototype.prepareSelection=function(){var t=this.cm,e=t.display,n=t.doc,r=yr(t);if(t.options.moveInputWithCursor){var i=Kn(t,n.sel.primary().head,"div"),o=e.wrapper.getBoundingClientRect(),a=e.lineDiv.getBoundingClientRect();r.teTop=Math.max(0,Math.min(e.wrapper.clientHeight-10,i.top+a.top-o.top)),r.teLeft=Math.max(0,Math.min(e.wrapper.clientWidth-10,i.left+a.left-o.left));}return r},Ua.prototype.showSelection=function(t){var e=this.cm.display;P(e.cursorDiv,t.cursors),P(e.selectionDiv,t.selection),null!=t.teTop&&(this.wrapper.style.top=t.teTop+"px",this.wrapper.style.left=t.teLeft+"px");},Ua.prototype.reset=function(t){if(!this.contextMenuPending&&!this.composing){var e=this.cm;if(e.somethingSelected()){this.prevInput="";var n=e.getSelection();this.textarea.value=n,e.state.focused&&I(this.textarea),a&&s>=9&&(this.hasSelection=n);}else t||(this.prevInput=this.textarea.value="",a&&s>=9&&(this.hasSelection=null));}},Ua.prototype.getField=function(){return this.textarea},Ua.prototype.supportsTouch=function(){return !1},Ua.prototype.focus=function(){if("nocursor"!=this.cm.options.readOnly&&(!m||A()!=this.textarea))try{this.textarea.focus();}catch(t){}},Ua.prototype.blur=function(){this.textarea.blur();},Ua.prototype.resetPosition=function(){this.wrapper.style.top=this.wrapper.style.left=0;},Ua.prototype.receivedFocus=function(){this.slowPoll();},Ua.prototype.slowPoll=function(){var t=this;this.pollingFast||this.polling.set(this.cm.options.pollInterval,(function(){t.poll(),t.cm.state.focused&&t.slowPoll();}));},Ua.prototype.fastPoll=function(){var t=!1,e=this;e.pollingFast=!0,e.polling.set(20,(function n(){e.poll()||t?(e.pollingFast=!1,e.slowPoll()):(t=!0,e.polling.set(60,n));}));},Ua.prototype.poll=function(){var t=this,e=this.cm,n=this.textarea,r=this.prevInput;if(this.contextMenuPending||!e.state.focused||Lt(n)&&!r&&!this.composing||e.isReadOnly()||e.options.disableInput||e.state.keySeq)return !1;var i=n.value;if(i==r&&!e.somethingSelected())return !1;if(a&&s>=9&&this.hasSelection===i||y&&/[\uf700-\uf7ff]/.test(i))return e.display.input.reset(),!1;if(e.doc.sel==e.display.selForContextMenu){var o=i.charCodeAt(0);if(8203!=o||r||(r="​"),8666==o)return this.reset(),this.cm.execCommand("undo")}for(var l=0,c=Math.min(r.length,i.length);l<c&&r.charCodeAt(l)==i.charCodeAt(l);)++l;return Qr(e,(function(){Aa(e,i.slice(l),r.length-l,null,t.composing?"*compose":null),i.length>1e3||i.indexOf("\n")>-1?n.value=t.prevInput="":t.prevInput=i,t.composing&&(t.composing.range.clear(),t.composing.range=e.markText(t.composing.start,e.getCursor("to"),{className:"CodeMirror-composing"}));})),!0},Ua.prototype.ensurePolled=function(){this.pollingFast&&this.poll()&&(this.pollingFast=!1);},Ua.prototype.onKeyPress=function(){a&&s>=9&&(this.hasSelection=null),this.fastPoll();},Ua.prototype.onContextMenu=function(t){var e=this,n=e.cm,r=n.display,i=e.textarea;e.contextMenuPending&&e.contextMenuPending();var o=ur(n,t),c=r.scroller.scrollTop;if(o&&!h){n.options.resetSelectionOnContextMenu&&-1==n.doc.sel.contains(o)&&ti(n,Qi)(n.doc,ki(o),W);var u,d=i.style.cssText,f=e.wrapper.style.cssText,p=e.wrapper.offsetParent.getBoundingClientRect();if(e.wrapper.style.cssText="position: static",i.style.cssText="position: absolute; width: 30px; height: 30px;\n      top: "+(t.clientY-p.top-5)+"px; left: "+(t.clientX-p.left-5)+"px;\n      z-index: 1000; background: "+(a?"rgba(255, 255, 255, .05)":"transparent")+";\n      outline: none; border-width: 0; outline: none; overflow: hidden; opacity: .05; filter: alpha(opacity=5);",l&&(u=window.scrollY),r.input.focus(),l&&window.scrollTo(null,u),r.input.reset(),n.somethingSelected()||(i.value=e.prevInput=" "),e.contextMenuPending=m,r.selForContextMenu=n.doc.sel,clearTimeout(r.detectingSelectAll),a&&s>=9&&v(),C){Ct(t);var g=function(){pt(window,"mouseup",g),setTimeout(m,20);};dt(window,"mouseup",g);}else setTimeout(m,50);}function v(){if(null!=i.selectionStart){var t=n.somethingSelected(),o="​"+(t?i.value:"");i.value="⇚",i.value=o,e.prevInput=t?"":"​",i.selectionStart=1,i.selectionEnd=o.length,r.selForContextMenu=n.doc.sel;}}function m(){if(e.contextMenuPending==m&&(e.contextMenuPending=!1,e.wrapper.style.cssText=f,i.style.cssText=d,a&&s<9&&r.scrollbars.setScrollTop(r.scroller.scrollTop=c),null!=i.selectionStart)){(!a||a&&s<9)&&v();var t=0,o=function(){r.selForContextMenu==n.doc.sel&&0==i.selectionStart&&i.selectionEnd>0&&"​"==e.prevInput?ti(n,so)(n):t++<10?r.detectingSelectAll=setTimeout(o,500):(r.selForContextMenu=null,r.input.reset());};r.detectingSelectAll=setTimeout(o,200);}}},Ua.prototype.readOnlyChanged=function(t){t||this.reset(),this.textarea.disabled="nocursor"==t;},Ua.prototype.setUneditable=function(){},Ua.prototype.needsContentAttribute=!1,function(t){var e=t.optionHandlers;function n(n,r,i,o){t.defaults[n]=r,i&&(e[n]=o?function(t,e,n){n!=Oa&&i(t,e,n);}:i);}t.defineOption=n,t.Init=Oa,n("value","",(function(t,e){return t.setValue(e)}),!0),n("mode",null,(function(t,e){t.doc.modeOption=e,Mi(t);}),!0),n("indentUnit",2,Mi,!0),n("indentWithTabs",!1),n("smartIndent",!0),n("tabSize",4,(function(t){Di(t),zn(t),dr(t);}),!0),n("lineSeparator",null,(function(t,e){if(t.doc.lineSep=e,e){var n=[],r=t.doc.first;t.doc.iter((function(t){for(var i=0;;){var o=t.text.indexOf(e,i);if(-1==o)break;i=o+e.length,n.push(ee(r,o));}r++;}));for(var i=n.length-1;i>=0;i--)go(t.doc,e,n[i],ee(n[i].line,n[i].ch+e.length));}})),n("specialChars",/[\u0000-\u001f\u007f-\u009f\u00ad\u061c\u200b-\u200f\u2028\u2029\ufeff\ufff9-\ufffc]/g,(function(t,e,n){t.state.specialChars=new RegExp(e.source+(e.test("\t")?"":"|\t"),"g"),n!=Oa&&t.refresh();})),n("specialCharPlaceholder",Qe,(function(t){return t.refresh()}),!0),n("electricChars",!0),n("inputStyle",m?"contenteditable":"textarea",(function(){throw new Error("inputStyle can not (yet) be changed in a running editor")}),!0),n("spellcheck",!1,(function(t,e){return t.getInputField().spellcheck=e}),!0),n("autocorrect",!1,(function(t,e){return t.getInputField().autocorrect=e}),!0),n("autocapitalize",!1,(function(t,e){return t.getInputField().autocapitalize=e}),!0),n("rtlMoveVisually",!w),n("wholeLineUpdateBefore",!0),n("theme","default",(function(t){xa(t),gi(t);}),!0),n("keyMap","default",(function(t,e,n){var r=Xo(e),i=n!=Oa&&Xo(n);i&&i.detach&&i.detach(t,r),r.attach&&r.attach(t,i||null);})),n("extraKeys",null),n("configureMouse",null),n("lineWrapping",!1,Ta,!0),n("gutters",[],(function(t,e){t.display.gutterSpecs=fi(e,t.options.lineNumbers),gi(t);}),!0),n("fixedGutter",!0,(function(t,e){t.display.gutters.style.left=e?sr(t.display)+"px":"0",t.refresh();}),!0),n("coverGutterNextToScrollbar",!1,(function(t){return $r(t)}),!0),n("scrollbarStyle","native",(function(t){Ur(t),$r(t),t.display.scrollbars.setScrollTop(t.doc.scrollTop),t.display.scrollbars.setScrollLeft(t.doc.scrollLeft);}),!0),n("lineNumbers",!1,(function(t,e){t.display.gutterSpecs=fi(t.options.gutters,e),gi(t);}),!0),n("firstLineNumber",1,gi,!0),n("lineNumberFormatter",(function(t){return t}),gi,!0),n("showCursorWhenSelecting",!1,mr,!0),n("resetSelectionOnContextMenu",!0),n("lineWiseCopyCut",!0),n("pasteLinesPerSelection",!0),n("selectionsMayTouch",!1),n("readOnly",!1,(function(t,e){"nocursor"==e&&(Tr(t),t.display.input.blur()),t.display.input.readOnlyChanged(e);})),n("disableInput",!1,(function(t,e){e||t.display.input.reset();}),!0),n("dragDrop",!0,ka),n("allowDropFileTypes",null),n("cursorBlinkRate",530),n("cursorScrollMargin",0),n("cursorHeight",1,mr,!0),n("singleCursorHeightPerLine",!0,mr,!0),n("workTime",100),n("workDelay",100),n("flattenSpans",!0,Di,!0),n("addModeClass",!1,Di,!0),n("pollInterval",100),n("undoDepth",200,(function(t,e){return t.doc.history.undoDepth=e})),n("historyEventDelay",1250),n("viewportMargin",10,(function(t){return t.refresh()}),!0),n("maxHighlightLength",1e4,Di,!0),n("moveInputWithCursor",!0,(function(t,e){e||t.display.input.resetPosition();})),n("tabindex",null,(function(t,e){return t.display.input.getField().tabIndex=e||""})),n("autofocus",null),n("direction","ltr",(function(t,e){return t.doc.setDirection(e)}),!0),n("phrases",null);}(ja),function(t){var e=t.optionHandlers,n=t.helpers={};t.prototype={constructor:t,focus:function(){window.focus(),this.display.input.focus();},setOption:function(t,n){var r=this.options,i=r[t];r[t]==n&&"mode"!=t||(r[t]=n,e.hasOwnProperty(t)&&ti(this,e[t])(this,n,i),gt(this,"optionChange",this,t));},getOption:function(t){return this.options[t]},getDoc:function(){return this.doc},addKeyMap:function(t,e){this.state.keyMaps[e?"push":"unshift"](Xo(t));},removeKeyMap:function(t){for(var e=this.state.keyMaps,n=0;n<e.length;++n)if(e[n]==t||e[n].name==t)return e.splice(n,1),!0},addOverlay:ei((function(e,n){var r=e.token?e:t.getMode(this.options,e);if(r.startState)throw new Error("Overlays may not be stateful.");!function(t,e,n){for(var r=0,i=n(e);r<t.length&&n(t[r])<=i;)r++;t.splice(r,0,e);}(this.state.overlays,{mode:r,modeSpec:e,opaque:n&&n.opaque,priority:n&&n.priority||0},(function(t){return t.priority})),this.state.modeGen++,dr(this);})),removeOverlay:ei((function(t){for(var e=this.state.overlays,n=0;n<e.length;++n){var r=e[n].modeSpec;if(r==t||"string"==typeof t&&r.name==t)return e.splice(n,1),this.state.modeGen++,void dr(this)}})),indentLine:ei((function(t,e,n){"string"!=typeof e&&"number"!=typeof e&&(e=null==e?this.options.smartIndent?"smart":"prev":e?"add":"subtract"),Qt(this.doc,t)&&Ea(this,t,e,n);})),indentSelection:ei((function(t){for(var e=this.doc.sel.ranges,n=-1,r=0;r<e.length;r++){var i=e[r];if(i.empty())i.head.line>n&&(Ea(this,i.head.line,t,!0),n=i.head.line,r==this.doc.sel.primIndex&&Ar(this));else {var o=i.from(),a=i.to(),s=Math.max(n,o.line);n=Math.min(this.lastLine(),a.line-(a.ch?0:1))+1;for(var l=s;l<n;++l)Ea(this,l,t);var c=this.doc.sel.ranges;0==o.ch&&e.length==c.length&&c[r].from().ch>0&&Xi(this.doc,r,new Ci(o,c[r].to()),W);}}})),getTokenAt:function(t,e){return be(this,t,e)},getLineTokens:function(t,e){return be(this,ee(t),e,!0)},getTokenTypeAt:function(t){t=le(this.doc,t);var e,n=fe(this,Gt(this.doc,t.line)),r=0,i=(n.length-1)/2,o=t.ch;if(0==o)e=n[2];else for(;;){var a=r+i>>1;if((a?n[2*a-1]:0)>=o)i=a;else {if(!(n[2*a+1]<o)){e=n[2*a+2];break}r=a+1;}}var s=e?e.indexOf("overlay "):-1;return s<0?e:0==s?null:e.slice(0,s-1)},getModeAt:function(e){var n=this.doc.mode;return n.innerMode?t.innerMode(n,this.getTokenAt(e).state).mode:n},getHelper:function(t,e){return this.getHelpers(t,e)[0]},getHelpers:function(t,e){var r=[];if(!n.hasOwnProperty(e))return r;var i=n[e],o=this.getModeAt(t);if("string"==typeof o[e])i[o[e]]&&r.push(i[o[e]]);else if(o[e])for(var a=0;a<o[e].length;a++){var s=i[o[e][a]];s&&r.push(s);}else o.helperType&&i[o.helperType]?r.push(i[o.helperType]):i[o.name]&&r.push(i[o.name]);for(var l=0;l<i._global.length;l++){var c=i._global[l];c.pred(o,this)&&-1==H(r,c.val)&&r.push(c.val);}return r},getStateAfter:function(t,e){var n=this.doc;return pe(this,(t=se(n,null==t?n.first+n.size-1:t))+1,e).state},cursorCoords:function(t,e){var n=this.doc.sel.primary();return Kn(this,null==t?n.head:"object"==typeof t?le(this.doc,t):t?n.from():n.to(),e||"page")},charCoords:function(t,e){return Gn(this,le(this.doc,t),e||"page")},coordsChar:function(t,e){return Jn(this,(t=qn(this,t,e||"page")).left,t.top)},lineAtHeight:function(t,e){return t=qn(this,{top:t,left:0},e||"page").top,Zt(this.doc,t+this.display.viewOffset)},heightAtLine:function(t,e,n){var r,i=!1;if("number"==typeof t){var o=this.doc.first+this.doc.size-1;t<this.doc.first?t=this.doc.first:t>o&&(t=o,i=!0),r=Gt(this.doc,t);}else r=t;return Un(this,r,{top:0,left:0},e||"page",n||i).top+(i?this.doc.height-Be(r):0)},defaultTextHeight:function(){return ir(this.display)},defaultCharWidth:function(){return or(this.display)},getViewport:function(){return {from:this.display.viewFrom,to:this.display.viewTo}},addWidget:function(t,e,n,r,i){var o,a,s,l=this.display,c=(t=Kn(this,le(this.doc,t))).bottom,u=t.left;if(e.style.position="absolute",e.setAttribute("cm-ignore-events","true"),this.display.input.setUneditable(e),l.sizer.appendChild(e),"over"==r)c=t.top;else if("above"==r||"near"==r){var h=Math.max(l.wrapper.clientHeight,this.doc.height),d=Math.max(l.sizer.clientWidth,l.lineSpace.clientWidth);("above"==r||t.bottom+e.offsetHeight>h)&&t.top>e.offsetHeight?c=t.top-e.offsetHeight:t.bottom+e.offsetHeight<=h&&(c=t.bottom),u+e.offsetWidth>d&&(u=d-e.offsetWidth);}e.style.top=c+"px",e.style.left=e.style.right="","right"==i?(u=l.sizer.clientWidth-e.offsetWidth,e.style.right="0px"):("left"==i?u=0:"middle"==i&&(u=(l.sizer.clientWidth-e.offsetWidth)/2),e.style.left=u+"px"),n&&(o=this,a={left:u,top:c,right:u+e.offsetWidth,bottom:c+e.offsetHeight},null!=(s=Mr(o,a)).scrollTop&&Fr(o,s.scrollTop),null!=s.scrollLeft&&Vr(o,s.scrollLeft));},triggerOnKeyDown:ei(ua),triggerOnKeyPress:ei(da),triggerOnKeyUp:ha,triggerOnMouseDown:ei(va),execCommand:function(t){if(ea.hasOwnProperty(t))return ea[t].call(null,this)},triggerElectric:ei((function(t){Na(this,t);})),findPosH:function(t,e,n,r){var i=1;e<0&&(i=-1,e=-e);for(var o=le(this.doc,t),a=0;a<e&&!(o=Va(this.doc,o,i,n,r)).hitSide;++a);return o},moveH:ei((function(t,e){var n=this;this.extendSelectionsBy((function(r){return n.display.shift||n.doc.extend||r.empty()?Va(n.doc,r.head,t,e,n.options.rtlMoveVisually):t<0?r.from():r.to()}),U);})),deleteH:ei((function(t,e){var n=this.doc.sel,r=this.doc;n.somethingSelected()?r.replaceSelection("",null,"+delete"):Jo(this,(function(n){var i=Va(r,n.head,t,e,!1);return t<0?{from:i,to:n.head}:{from:n.head,to:i}}));})),findPosV:function(t,e,n,r){var i=1,o=r;e<0&&(i=-1,e=-e);for(var a=le(this.doc,t),s=0;s<e;++s){var l=Kn(this,a,"div");if(null==o?o=l.left:l.left=o,(a=Ra(this,l,i,n)).hitSide)break}return a},moveV:ei((function(t,e){var n=this,r=this.doc,i=[],o=!this.display.shift&&!r.extend&&r.sel.somethingSelected();if(r.extendSelectionsBy((function(a){if(o)return t<0?a.from():a.to();var s=Kn(n,a.head,"div");null!=a.goalColumn&&(s.left=a.goalColumn),i.push(s.left);var l=Ra(n,s,t,e);return "page"==e&&a==r.sel.primary()&&Dr(n,Gn(n,l,"div").top-s.top),l}),U),i.length)for(var a=0;a<r.sel.ranges.length;a++)r.sel.ranges[a].goalColumn=i[a];})),findWordAt:function(t){var e=Gt(this.doc,t.line).text,n=t.ch,r=t.ch;if(e){var i=this.getHelper(t,"wordChars");"before"!=t.sticky&&r!=e.length||!n?++r:--n;for(var o=e.charAt(n),a=et(o,i)?function(t){return et(t,i)}:/\s/.test(o)?function(t){return /\s/.test(t)}:function(t){return !/\s/.test(t)&&!et(t)};n>0&&a(e.charAt(n-1));)--n;for(;r<e.length&&a(e.charAt(r));)++r;}return new Ci(ee(t.line,n),ee(t.line,r))},toggleOverwrite:function(t){null!=t&&t==this.state.overwrite||((this.state.overwrite=!this.state.overwrite)?L(this.display.cursorDiv,"CodeMirror-overwrite"):T(this.display.cursorDiv,"CodeMirror-overwrite"),gt(this,"overwriteToggle",this,this.state.overwrite));},hasFocus:function(){return this.display.input.getField()==A()},isReadOnly:function(){return !(!this.options.readOnly&&!this.doc.cantEdit)},scrollTo:ei((function(t,e){Lr(this,t,e);})),getScrollInfo:function(){var t=this.display.scroller;return {left:t.scrollLeft,top:t.scrollTop,height:t.scrollHeight-jn(this)-this.display.barHeight,width:t.scrollWidth-jn(this)-this.display.barWidth,clientHeight:En(this),clientWidth:Pn(this)}},scrollIntoView:ei((function(t,e){null==t?(t={from:this.doc.sel.primary().head,to:null},null==e&&(e=this.options.cursorScrollMargin)):"number"==typeof t?t={from:ee(t,0),to:null}:null==t.from&&(t={from:t,to:null}),t.to||(t.to=t.from),t.margin=e||0,null!=t.from.line?function(t,e){Nr(t),t.curOp.scrollToPos=e;}(this,t):Ir(this,t.from,t.to,t.margin);})),setSize:ei((function(t,e){var n=this,r=function(t){return "number"==typeof t||/^\d+$/.test(String(t))?t+"px":t};null!=t&&(this.display.wrapper.style.width=r(t)),null!=e&&(this.display.wrapper.style.height=r(e)),this.options.lineWrapping&&Hn(this);var i=this.display.viewFrom;this.doc.iter(i,this.display.viewTo,(function(t){if(t.widgets)for(var e=0;e<t.widgets.length;e++)if(t.widgets[e].noHScroll){fr(n,i,"widget");break}++i;})),this.curOp.forceUpdate=!0,gt(this,"refresh",this);})),operation:function(t){return Qr(this,t)},startOperation:function(){return Gr(this)},endOperation:function(){return Kr(this)},refresh:ei((function(){var t=this.display.cachedTextHeight;dr(this),this.curOp.forceUpdate=!0,zn(this),Lr(this,this.doc.scrollLeft,this.doc.scrollTop),ci(this.display),(null==t||Math.abs(t-ir(this.display))>.5)&&cr(this),gt(this,"refresh",this);})),swapDoc:ei((function(t){var e=this.doc;return e.cm=null,this.state.selectingText&&this.state.selectingText(),Ii(this,t),zn(this),this.display.input.reset(),Lr(this,t.scrollLeft,t.scrollTop),this.curOp.forceScroll=!0,cn(this,"swapDoc",this,e),e})),phrase:function(t){var e=this.options.phrases;return e&&Object.prototype.hasOwnProperty.call(e,t)?e[t]:t},getInputField:function(){return this.display.input.getField()},getWrapperElement:function(){return this.display.wrapper},getScrollerElement:function(){return this.display.scroller},getGutterElement:function(){return this.display.gutters}},bt(t),t.registerHelper=function(e,r,i){n.hasOwnProperty(e)||(n[e]=t[e]={_global:[]}),n[e][r]=i;},t.registerGlobalHelper=function(e,r,i,o){t.registerHelper(e,r,o),n[e]._global.push({pred:i,val:o});};}(ja);var qa="iter insert remove copy getEditor constructor".split(" ");for(var Ga in Do.prototype)Do.prototype.hasOwnProperty(Ga)&&H(qa,Ga)<0&&(ja.prototype[Ga]=function(t){return function(){return t.apply(this.doc,arguments)}}(Do.prototype[Ga]));return bt(Do),ja.inputStyles={textarea:Ua,contenteditable:Ha},ja.defineMode=function(t){ja.defaults.mode||"null"==t||(ja.defaults.mode=t),Vt.apply(this,arguments);},ja.defineMIME=function(t,e){_t[t]=e;},ja.defineMode("null",(function(){return {token:function(t){return t.skipToEnd()}}})),ja.defineMIME("text/plain","null"),ja.defineExtension=function(t,e){ja.prototype[t]=e;},ja.defineDocExtension=function(t,e){Do.prototype[t]=e;},ja.fromTextArea=function(t,e){if((e=e?_(e):{}).value=t.value,!e.tabindex&&t.tabIndex&&(e.tabindex=t.tabIndex),!e.placeholder&&t.placeholder&&(e.placeholder=t.placeholder),null==e.autofocus){var n=A();e.autofocus=n==t||null!=t.getAttribute("autofocus")&&n==document.body;}function r(){t.value=s.getValue();}var i;if(t.form&&(dt(t.form,"submit",r),!e.leaveSubmitMethodAlone)){var o=t.form;i=o.submit;try{var a=o.submit=function(){r(),o.submit=i,o.submit(),o.submit=a;};}catch(t){}}e.finishInit=function(n){n.save=r,n.getTextArea=function(){return t},n.toTextArea=function(){n.toTextArea=isNaN,r(),t.parentNode.removeChild(n.getWrapperElement()),t.style.display="",t.form&&(pt(t.form,"submit",r),e.leaveSubmitMethodAlone||"function"!=typeof t.form.submit||(t.form.submit=i));};},t.style.display="none";var s=ja((function(e){return t.parentNode.insertBefore(e,t.nextSibling)}),e);return s},function(t){t.off=pt,t.on=dt,t.wheelEventPixels=wi,t.Doc=Do,t.splitLines=At,t.countColumn=V,t.findColumn=q,t.isWordChar=tt,t.Pass=$,t.signal=gt,t.Line=Ge,t.changeEnd=Ti,t.scrollbarModel=Br,t.Pos=ee,t.cmpPos=ne,t.modes=Ft,t.mimeModes=_t,t.resolveMode=Rt,t.getMode=Ht,t.modeExtensions=zt,t.extendMode=$t,t.copyState=Wt,t.startState=Ut,t.innerMode=Bt,t.commands=ea,t.keyMap=Wo,t.keyName=Yo,t.isModifierKey=Go,t.lookupKey=qo,t.normalizeKeyMap=Uo,t.StringStream=qt,t.SharedTextMarker=jo,t.TextMarker=ko,t.LineWidget=Oo,t.e_preventDefault=wt,t.e_stopPropagation=xt,t.e_stop=Ct,t.addClass=L,t.contains=D,t.rmClass=T,t.keyNames=Ro;}(ja),ja.version="5.49.2",ja}();},function(t,e,n){n.r(e);var r=n(8),i=n.n(r),o=n(2),a=n.n(o),s=n(1),l=n.n(s),c=n(0),u=n(3),h=n(19);function d(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var f=l.a.View.extend({events:function(){return this.model.get("events")||{mousedown:"handleClick"}},attributes:function(){return this.model.get("attributes")},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.config,n=void 0===e?{}:e;this.em=n.em,this.editor=n.editor;},handleClick:function(t){t.preventDefault(),t.stopPropagation();var e=this.editor,n=this.em,r=e.Canvas.getFrameEl().getBoundingClientRect(),i=r.left,o=r.top,s=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?d(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):d(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},t,{clientX:t.clientX-i,clientY:t.clientY-o});n.trigger("toolbar:run:before"),this.execCommand(s);},execCommand:function(t){var e={event:t},n=this.model.get("command"),r=this.editor;"function"==typeof n&&n(r,null,e),"string"==typeof n&&r.runCommand(n,e);},render:function(){var t=this.editor,e=this.$el,n=this.model,r=n.get("id"),i=n.get("label"),o=t.getConfig("stylePrefix");return e.addClass("".concat(o,"toolbar-item")),r&&e.addClass("".concat(o,"toolbar-item__").concat(r)),i&&e.append(i),this}}),p=h.a.extend({itemView:f,initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.config={editor:t.editor||"",em:t.em},this.listenTo(this.collection,"reset",this.render);}}),g=l.a.Model.extend({defaults:{command:"",attributes:{}}}),v=l.a.Collection.extend({model:g});function m(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function y(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?m(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):m(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var b,w=l.a.$;e.default={init:function(t){Object(c.bindAll)(this,"onHover","onOut","onClick","onFrameScroll","onFrameUpdated");},enable:function(){this.frameOff=this.canvasOff=this.adjScroll=null,this.startSelectComponent(),b=1;},startSelectComponent:function(){this.toggleSelectComponent(1),this.em.getSelected()&&this.onSelect();},stopSelectComponent:function(){this.toggleSelectComponent();},toggleSelectComponent:function(t){var e=this,n=this.em,r=t?"on":"off",i={on:u.t,off:u.s};i[r](window,"resize",this.onFrameUpdated),n[r]("component:toggled",this.onSelect,this),n[r]("change:componentHovered",this.onHovered,this),n[r]("component:resize component:styleUpdate component:input",this.updateGlobalPos,this),n[r]("change:canvasOffset",this.updateAttached,this),n[r]("frame:updated",this.onFrameUpdated,this),n.get("Canvas").getFrames().forEach((function(t){var n,o,a=t.view;a&&(n=a.getWindow(),o=a.getBody(),i[r](o,"mouseover",e.onHover),i[r](o,"mouseleave",e.onOut),i[r](o,"click touchend",e.onClick),i[r](n,"scroll",e.onFrameScroll));}));},onHover:function(t){t.stopPropagation();var e=t.target,n=Object(u.j)(e),r=n&&n._getFrame(),i=w(e),o=i.data("model");if(!o)for(var a=i.parent();!o&&a.length>0;)o=a.data("model"),a=a.parent();if(o&&!o.get("hoverable")){for(var s=o&&o.parent();s&&!s.get("hoverable");)s=s.parent();o=s;}this.currentDoc=e.ownerDocument,this.em.setHovered(o),r&&this.em.set("currentFrame",r);},onFrameUpdated:function(){this.updateLocalPos(),this.updateGlobalPos();},onHovered:function(t,e){var n=this,r={};e&&e.views.forEach((function(t){var i=t.el,o=n.getElementPos(i);r={el:i,pos:o,component:e,view:Object(u.j)(i)},n.updateToolsLocal(r),i.ownerDocument===n.currentDoc&&(n.elHovered=r);}));},onSelect:Object(c.debounce)((function(){var t=this.em,e=t.getSelected(),n=t.get("currentFrame")||{},r=e&&e.getView(n.model),i=r&&r.el,o={};i&&(o={el:i,pos:this.getElementPos(i),component:e,view:Object(u.j)(i)});this.elSelected=o,this.updateToolsGlobal(),this.updateToolsLocal(o);})),updateGlobalPos:function(){var t=this.getElSelected();t.el&&(t.pos=this.getElementPos(t.el),this.updateToolsGlobal());},updateLocalPos:function(){var t=this.getElHovered();t.el&&(t.pos=this.getElementPos(t.el),this.updateToolsLocal());},getElHovered:function(){return this.elHovered||{}},getElSelected:function(){return this.elSelected||{}},onOut:function(){var t=this;this.currentDoc=null,this.em.setHovered(0),this.canvas.getFrames().forEach((function(e){var n=e.view,r=n&&n.getToolsEl();r&&t.toggleToolsEl(0,0,{el:r});}));},toggleToolsEl:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n.el||this.canvas.getToolsEl(e);return r&&(r.style.opacity=t?1:0),r||{}},showElementOffset:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};b&&this.editor.runCommand("show-offset",{el:t,elPos:e,view:n.view,force:1,top:0,left:0});},hideElementOffset:function(t){this.editor.stopCommand("show-offset",{view:t});},showFixedElementOffset:function(t,e){this.editor.runCommand("show-offset",{el:t,elPos:e,state:"Fixed"});},hideFixedElementOffset:function(t,e){this.editor&&this.editor.stopCommand("show-offset",{state:"Fixed"});},hideHighlighter:function(t){this.canvas.getHighlighter(t).style.opacity=0;},onClick:function(t){t.stopPropagation(),t.preventDefault();var e=this.em;if(e.get("_cmpDrag"))return e.set("_cmpDrag");var n=w(t.target),r=n.data("model");if(!r)for(var i=n.parent();!r&&i.length>0;)r=i.data("model"),i=i.parent();if(r)if(r.get("selectable"))this.select(r,t);else {for(var o=r.parent();o&&!o.get("selectable");)o=o.parent();this.select(o,t);}},select:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(t){var n=e.ctrlKey||e.metaKey,r=e.shiftKey,i=this.editor,o=this.em,a=i.getConfig("multipleSelection");if(n&&a)i.selectToggle(t);else if(r&&a){o.clearSelection(i.Canvas.getWindow());var s,l,u=t.collection,h=u.indexOf(t);i.getSelectedAll();if(i.getSelectedAll().forEach((function(t){var e=t.collection,n=e.indexOf(t);e===u&&(n<h?s=Object(c.isUndefined)(s)?n:Math.max(s,n):n>h&&(l=Object(c.isUndefined)(l)?n:Math.min(l,n)));})),!Object(c.isUndefined)(s))for(;s!==h;)i.selectAdd(u.at(s)),s++;if(!Object(c.isUndefined)(l))for(;l!==h;)i.selectAdd(u.at(l)),l--;i.selectAdd(t);}else i.select(t,{scroll:{}});this.initResize(t);}},updateBadge:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=w(t).data("model");if(r&&r.get("badgable")){var i=this.getBadge(n);if(!n.posOnly){var o=this.canvas.getConfig(),a=r.getIcon(),s=o.pStylePrefix||"",l="".concat(s,"badge"),c=o.customBadgeLabel,u="".concat(a?'<div class="'.concat(l,'__icon">').concat(a,"</div>"):"",'\n        <div class="').concat(l,'__name">').concat(r.getName(),"</div>");i.innerHTML=c?c(r):u;}var h="px",d=i.style;d.display="block";var f=i?i.offsetHeight:0,p=0-f,g=n.topOff-f<0?-n.topOff:p,v=n.leftOff<0?-n.leftOff:0;d.top=g+h,d.left=v+h;}},showHighlighter:function(t){this.canvas.getHighlighter(t).style.opacity="";},initResize:function(t){var e,n=this.em,r=this.canvas,o=n?n.get("Editor"):"",a=(n?n.get("Config"):"").stylePrefix||"",s="".concat(a,"resizing"),l=!Object(c.isElement)(t)&&Object(u.o)(t)?t:n.getSelected(),h=l.get("resizable"),d=Object(c.isElement)(t)?t:l.getEl(),f={},p=function(t,e,n){var r=n.docs;r&&r.forEach((function(e){var n=e.body,r=n.className||"";n.className=("add"==t?"".concat(r," ").concat(s):r.replace(s,"")).trim();}));};o&&h?(f={onStart:function(t){var i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},o=i.el,a=i.config,s=i.resizer,c=a.keyHeight,h=a.keyWidth,d=a.currentUnit,f=a.keepAutoHeight,g=a.keepAutoWidth;p("add",0,i),e=n.get("StyleManager").getModelToStyle(l),r.toggleFramesEvents();var v=getComputedStyle(o),m=e.getStyle(),y=m[h];a.autoWidth=g&&"auto"===y,isNaN(parseFloat(y))&&(y=v[h]);var w=m[c];a.autoHeight=f&&"auto"===w,isNaN(parseFloat(w))&&(w=v[c]),s.startDim.w=parseFloat(y),s.startDim.h=parseFloat(w),b=0,d&&(a.unitHeight=Object(u.i)(w),a.unitWidth=Object(u.i)(y));},onMove:function(){o.trigger("component:resize");},onEnd:function(t,e){p("remove",0,e),o.trigger("component:resize"),r.toggleFramesEvents(1),b=1;},updateTarget:function(t,i){var o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(e){var a=o.store,s=o.selectedHandler,l=o.config,c=l.keyHeight,u=l.keyWidth,h=l.autoHeight,d=l.autoWidth,f=l.unitWidth,p=l.unitHeight,g=["tc","bc"].indexOf(s)>=0,v=["cl","cr"].indexOf(s)>=0,m={},b=a?"":1;if(!g){var w=r.getBody().offsetWidth,x=i.w<w?i.w:w;m[u]=d?"auto":"".concat(x).concat(f);}v||(m[c]=h?"auto":"".concat(i.h).concat(p)),e.addStyle(y({},m,{en:b}),{avoidStore:!a});var O="update:component:style",C="".concat(O,":").concat(c," ").concat(O,":").concat(u);n&&n.trigger(C,null,null,{noEmit:1});}}},"object"==i()(h)&&(f=y({},f,{},h)),this.resizer=o.runCommand("resize",{el:d,options:f,force:1})):(o.stopCommand("resize"),this.resizer=null);},updateToolbar:function(t){var e=this.config.em,n=t==e?e.getSelected():t,r=this.canvas.getToolbarEl(),i=r.style;if(n){var o=n.get("toolbar");if(e.get("Config").showToolbar&&o&&o.length){if(i.opacity="",i.display="",!this.toolbar){r.innerHTML="",this.toolbar=new v(o);var a=new p({collection:this.toolbar,editor:this.editor,em:e});r.appendChild(a.render().el);}this.toolbar.reset(o),i.top="-100px",i.left=0;}else i.display="none";}else i.opacity=0;},updateToolbarPos:function(t){var e=this.canvas.getToolbarEl().style;e.top="".concat(t.top).concat("px"),e.left="".concat(t.left).concat("px"),e.opacity="";},getCanvasPosition:function(){return this.canvas.getCanvasView().getPosition()},getBadge:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return this.canvas.getBadgeEl(t.view)},onFrameScroll:function(){this.updateTools();},updateTools:function(){this.updateToolsLocal(),this.updateGlobalPos();},isCompSelected:function(t){return t&&"selected"===t.get("status")},updateToolsLocal:function(t){var e=t||this.getElHovered(),n=e.el,r=e.pos,i=e.view,o=e.component;if(n){var a=o.get("hoverable"),s=this.lastHovered!==n,l=s?{}:{posOnly:1};s&&a&&(this.lastHovered=n,this.showHighlighter(i),this.showElementOffset(n,r,{view:i})),this.isCompSelected(o)&&(this.hideHighlighter(i),this.hideElementOffset(i));var c=this.toggleToolsEl(1,i).style,u=this.canvas.canvasRectOffset(n,r),h=u.top,d=u.left;this.updateBadge(n,r,y({},l,{view:i,topOff:h,leftOff:d})),c.top=h+"px",c.left=d+"px",c.width=r.width+"px",c.height=r.height+"px";}else this.lastHovered=0;},updateToolsGlobal:function(){var t=this.getElSelected(),e=t.el,n=t.pos,r=t.component;if(!e)return this.toggleToolsEl(),void(this.lastSelected=0);var i=this.canvas;this.lastSelected!==e&&(this.lastSelected=e,this.updateToolbar(r));var o=this.toggleToolsEl(1).style,a=i.getTargetToElementFixed(e,i.getToolbarEl(),{pos:n}),s=a.canvasOffsetTop,l=a.canvasOffsetLeft;o.top=s+"px",o.left=l+"px",o.width=n.width+"px",o.height=n.height+"px",this.updateToolbarPos({top:a.top,left:a.left});},updateAttached:Object(c.debounce)((function(){this.updateToolsGlobal();})),getElementPos:function(t){return this.canvas.getCanvasView().getElementPos(t)},hideBadge:function(){this.getBadge().style.display="none";},cleanPrevious:function(t){t&&t.set({status:"",state:""});},getContentWindow:function(){return this.canvas.getWindow()},run:function(t){this.editor=t&&t.get("Editor"),this.enable();},stop:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.em,i=this.editor;this.stopSelectComponent(),!n.preserveSelected&&r.setSelected(null),this.onOut(),this.toggleToolsEl(),i&&i.stopCommand("resize"),this.editor=0;}};},function(t,e){t.exports=function(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")};},function(t,e){function n(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r);}}t.exports=function(t,e,r){return e&&n(t.prototype,e),r&&n(t,r),t};},function(t,e,n){n.d(e,"a",(function(){return p}));var r=n(2),i=n.n(r),o=n(14),a=n.n(o),s=n(15),l=n.n(s),c=n(0),u=n(3);function h(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function d(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?h(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):h(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var f=function(){return {x:0,y:0}},p=function(){function t(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return a()(this,t),this.opts={container:null,onStart:null,onDrag:null,onEnd:null,setPosition:null,getPosition:null,guidesStatic:null,guidesTarget:null,snapOffset:5,doc:0,scale:1},Object(c.bindAll)(this,"drag","stop","keyHandle","handleScroll"),this.setOptions(e),this.delta=f(),this}return l()(t,[{key:"setOptions",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.opts=d({},this.opts,{},t);}},{key:"toggleDrag",value:function(t){var e=this.getDocumentEl(),n=this.getContainerEl(),r=this.getWindowEl(),i=t?"on":"off",o={on:u.t,off:u.s};o[i](n,"mousemove dragover",this.drag),o[i](e,"mouseup dragend touchend",this.stop),o[i](e,"keydown",this.keyHandle),o[i](r,"scroll",this.handleScroll);}},{key:"handleScroll",value:function(){var t=this.lastScroll,e=this.delta,n=this.getScrollInfo(),r={x:n.x-t.x,y:n.y-t.y};this.move(e.x+r.x,e.y+r.y),this.lastScrollDiff=r;}},{key:"start",value:function(t){var e=this.opts,n=e.onStart;this.toggleDrag(1),this.startPointer=this.getPointerPos(t),this.guidesStatic=Object(c.result)(e,"guidesStatic")||[],this.guidesTarget=Object(c.result)(e,"guidesTarget")||[],Object(c.isFunction)(n)&&n(t,this),this.startPosition=this.getStartPosition(),this.lastScrollDiff=f(),this.globScrollDiff=f(),this.drag(t);}},{key:"drag",value:function(t){var e=this,n=this.opts,r=this.lastScrollDiff,i=this.globScrollDiff,o=n.onDrag,a=this.startPointer,s=this.getPointerPos(t),l={x:i.x+r.x,y:i.y+r.y};this.globScrollDiff=l;var u={x:s.x-a.x+l.x,y:s.y-a.y+l.y};this.lastScrollDiff=f();var h=this.lockedAxis;"x"===(h=t.shiftKey?!h&&this.detectAxisLock(u.x,u.y):null)?u.x=a.x:"y"===h&&(u.y=a.y);var p=function(r){["x","y"].forEach((function(t){return r[t]=r[t]*Object(c.result)(n,"scale")})),e.delta=r,e.move(r.x,r.y),Object(c.isFunction)(o)&&o(t,e);},g=d({},u);if(this.currentPointer=s,this.lockedAxis=h,this.lastScroll=this.getScrollInfo(),p(u),this.guidesTarget.length){var v=this.snapGuides(g),m=v.newDelta,y=v.trgX,b=v.trgY;(y||b)&&p(m);}0===t.which&&this.stop(t);}},{key:"snapGuides",value:function(t){var e=this,n=t,r=this.trgX,i=this.trgY;return this.guidesTarget.forEach((function(n){n.x&&e.trgX||n.y&&e.trgY||(n.active=0,e.guidesStatic.forEach((function(o){if(!(n.y&&o.x||n.x&&o.y)){var a=n.y&&o.y,s=a?"y":"x",l=n[s],u=o[s],h=t[s],d=a?i:r;if(e.isPointIn(l,u)&&Object(c.isUndefined)(d)){var f=h-(l-u);e.setGuideLock(n,f);}}})));})),r=this.trgX,i=this.trgY,["x","y"].forEach((function(r){var i=r.toUpperCase(),o=e["trg".concat(i)];o&&!e.isPointIn(t[r],o.lock)&&(e.setGuideLock(o,null),o=null),o&&!Object(c.isUndefined)(o.lock)&&(n[r]=o.lock);})),{newDelta:n,trgX:this.trgX,trgY:this.trgY}}},{key:"isPointIn",value:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n.offset,i=r||this.opts.snapOffset;return t>=e&&t<=e+i||t<=e&&t>=e-i}},{key:"setGuideLock",value:function(t,e){var n=Object(c.isUndefined)(t.x)?"Y":"X",r="trg".concat(n);return null!==e?(t.active=1,t.lock=e,this[r]=t):(delete t.active,delete t.lock,delete this[r]),t}},{key:"stop",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.delta,r=e.cancel,i=r?0:n.x,o=r?0:n.y;this.toggleDrag(),this.lockedAxis=null,this.move(i,o,1);var a=this.opts.onEnd;Object(c.isFunction)(a)&&a(t,this,{cancelled:r});}},{key:"keyHandle",value:function(t){Object(u.m)(t)&&this.stop(t,{cancel:1});}},{key:"move",value:function(t,e,n){var r=this.el,i=this.opts,o=this.startPosition;if(o){var a=i.setPosition,s=o.x+t,l=o.y+e;this.position={x:s,y:l,end:n},Object(c.isFunction)(a)&&a(this.position),r&&(r.style.left="".concat(s,"px"),r.style.top="".concat(l,"px"));}}},{key:"getContainerEl",value:function(){var t=this.opts.container;return t?[t]:this.getDocumentEl()}},{key:"getWindowEl",value:function(){return this.getContainerEl().map((function(t){var e=t.ownerDocument||t;return e.defaultView||e.parentWindow}))}},{key:"getDocumentEl",value:function(t){var e=this.opts.doc;if(t=t||this.el,!this.docs){var n=[document];t&&n.push(t.ownerDocument),e&&n.push(e),this.docs=n;}return this.docs}},{key:"getPointerPos",value:function(t){var e=this.opts.getPointerPosition,n=Object(u.h)(t);return e?e(t):{x:n.clientX,y:n.clientY}}},{key:"getStartPosition",value:function(){var t=this.el,e=this.opts.getPosition,n=f();return Object(c.isFunction)(e)?n=e():t&&(n={x:parseFloat(t.style.left),y:parseFloat(t.style.top)}),n}},{key:"getScrollInfo",value:function(){var t=this.opts.doc,e=t&&t.body;return {y:e?e.scrollTop:0,x:e?e.scrollLeft:0}}},{key:"detectAxisLock",value:function(t,e){var n=t,r=e,i=Math.abs(n),o=Math.abs(r);return r>=i||r<=-i?"x":n>o||n<-o?"y":void 0}}]),t}();},function(t,e,n){var r=n(1),i=n.n(r),o=i.a.$;e.a=i.a.View.extend({events:{change:"handleChange"},template:function(){return '<span class="'.concat(this.holderClass(),'"></span>')},inputClass:function(){return "".concat(this.ppfx,"field")},holderClass:function(){return "".concat(this.ppfx,"input-holder")},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.ppfx||"";this.opts=t,this.ppfx=e,this.em=t.target||{},this.listenTo(this.model,"change:value",this.handleModelChange);},elementUpdated:function(){this.model.trigger("el:change");},setValue:function(t){var e=this.model,n=t||e.get("defaults"),r=this.getInputEl();r&&(r.value=n);},handleModelChange:function(t,e,n){this.setValue(e,n);},handleChange:function(t){t.stopPropagation();var e=this.getInputEl().value;this.model.set({value:e},{fromInput:1}),this.elementUpdated();},getInputEl:function(){if(!this.inputEl){var t=this.model,e=t.get("placeholder")||t.get("defaults")||"";this.inputEl=o('<input type="text" placeholder="'.concat(e,'">'));}return this.inputEl.get(0)},render:function(){this.inputEl=null;var t=this.$el;return t.addClass(this.inputClass()),t.html(this.template()),t.find(".".concat(this.holderClass())).append(this.getInputEl()),this}});},function(t,e,n){n.r(e);var r=n(1),i=n.n(r),o=n(7);e.default=i.a.View.extend({initialize:function(t){this.config=t.config||{},this.pfx=this.config.stylePrefix||"",this.target=t.target||{},this.propTarget=t.propTarget||{},this.onChange=t.onChange,this.onInputRender=t.onInputRender||{},this.customValue=t.customValue||{},this.properties=[];var e=this.collection;this.listenTo(e,"add",this.addTo),this.listenTo(e,"reset",this.render);},addTo:function(t,e,n){this.add(t,null,n);},add:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=e||this.el,i=new t.typeView({model:t,name:t.get("name"),id:this.pfx+t.get("property"),target:this.target,propTarget:this.propTarget,onChange:this.onChange,onInputRender:this.onInputRender,config:this.config});"composite"!=t.get("type")&&(i.customValue=this.customValue),i.render();var a=i.el;this.properties.push(i),i.updateVisibility(),Object(o.b)(r,a,n.at);},render:function(){var t=this,e=this.$el;this.clearItems();var n=document.createDocumentFragment();return this.collection.each((function(e){return t.add(e,n)})),e.empty(),e.append(n),e.attr("class","".concat(this.pfx,"properties")),this},remove:function(){i.a.View.prototype.remove.apply(this,arguments),this.clearItems();},clearItems:function(){this.properties.forEach((function(t){return t.remove()})),this.properties=[];}});},function(t,e,n){var r=n(0),i=n(1),o=n.n(i);e.a=o.a.View.extend({itemView:"",itemsView:"",itemType:"type",autoAdd:0,initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=arguments.length>1?arguments[1]:void 0;this.config=e||t.config||{},this.autoAdd&&this.listenTo(this.collection,"add",this.addTo),this.items=[],this.init();},init:function(){},addTo:function(t){this.add(t);},itemViewNotFound:function(t){var e=this.config,n=this.ns,r=e.em,i="".concat(n?"[".concat(n,"]: "):"","'").concat(t,"' type not found");r&&r.logWarning(i);},add:function(t,e){var n,i=this.config,o=this.reuseView,a=this.items,s=this.itemsView,l=void 0===s?{}:s,c=e||null,u=this.itemView,h=t.get(this.itemType);l[h]?u=l[h]:!h||l[h]||Object(r.includes)(["button","checkbox","color","date","datetime-local","email","file","hidden","image","month","number","password","radio","range","reset","search","submit","tel","text","time","url","week"],h)||this.itemViewNotFound(h),n=t.view&&o?t.view:new u({model:t,config:i},i),a&&a.push(n);var d=n.render().el;c?c.appendChild(d):this.$el.append(d);},render:function(){var t=document.createDocumentFragment();return this.clearItems(),this.$el.empty(),this.collection.length&&this.collection.each((function(e){this.add(e,t);}),this),this.$el.append(t),this.onRender(),this},onRender:function(){},remove:function(){this.clearItems(),o.a.View.prototype.remove.apply(this,arguments);},clearItems:function(){this.items;}});},function(t,e,n){var r,i=n(8),o=n.n(i),a=n(2),s=n.n(a),l=n(1),c=n.n(l),u=n(0);function h(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function d(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?h(n,!0).forEach((function(e){s()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):h(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.a=c.a.Collection.extend({initialize:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.opt=e,this.listenTo(this,"add",this.onAdd),this.listenTo(this,"remove",this.removeChildren),this.listenTo(this,"reset",this.resetChildren),this.config=e.config,this.em=e.em,this.domc=e.domc;},resetChildren:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=this,i=n.previousModels,o=void 0===i?[]:i;o.forEach((function(t){return e.removeChildren(t,r,n)})),t.each((function(t){return e.onAdd(t)}));},removeChildren:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=this.domc,o=this.em,a=i?i.allById():{};if(!r.temporary){var s=t.getId(),l=o.get("SelectorManager").getAll(),c=o.get("CssComposer").getAll();delete a[s];var u=c.remove(c.filter((function(t){return t.getSelectors().getFullString()==="#".concat(s)})));if(l.remove(u.map((function(t){return t.getSelectors().at(0)}))),!t.opt.temporary){var h=o.get("Commands"),d=t.get("style-signature"),f={target:t};d&&h.run("core:component-style-clear",f),t.removed(),o.trigger("component:remove",t);}var p=t.components();p.forEach((function(t){return n.removeChildren(t,e,r)}));}var g=t.components(),v=o.get("UndoManager");o.stopListening(g),o.stopListening(t),o.stopListening(t.get("classes")),v.remove(t),v.remove(g);},model:function(t,e){var n,r=e.collection.opt,i=r.em,o=i.get("DomComponents").componentTypes;e.em=i,e.config=r.config,e.componentTypes=o,e.domc=r.domc;for(var a=0;a<o.length;a++){if(o[a].id==t.type){n=o[a].model;break}}return n||(n=o[o.length-1].model,i&&t.type&&i.logWarning("Component type '".concat(t.type,"' not found"),{attrs:t,options:e})),new n(t,e)},parseString:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},i=this.em,o=this.opt.domc,a=i.get("CssComposer"),s=i.get("Parser").parseHtml(t);return r||(r=n(4).default),r.checkId(s.html,s.css,o.componentsById),s.css&&a&&!e.temporary&&a.addCollection(s.css,d({},e,{extend:1})),s.html},add:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};Object(u.isString)(t)?t=this.parseString(t,n):Object(u.isArray)(t)&&t.forEach((function(r,i){Object(u.isString)(r)&&(t[i]=e.parseString(r,n));}));var r=Object(u.isArray)(t);return t=(r?t:[t]).filter((function(t){return t})).map((function(t){return e.processDef(t)})),t=r?t:t[0],c.a.Collection.prototype.add.apply(this,[t,n])},processDef:function(t){if(t.cid&&t.ccid)return t;var e=this.em,n=this.config,r=(void 0===n?{}:n).processor,i=t;if(r){var a=r(i=d({},i));a&&(Object(u.each)(i,(function(t,e){return delete i[e]})),Object(u.extend)(i,a));}if(i.$$typeof&&"object"==o()(i.props)){(i=d({},i)).props=d({},i.props);var s=e.get("DomComponents"),l=e.get("Parser").parserHtml;Object(u.each)(i,(function(t,e){Object(u.includes)(["props","type"],e)||delete i[e];}));var c=i.props,h=c.children;delete c.children,delete i.props;var f=l.splitPropsFromAttr(c);i.attributes=f.attrs,h&&(i.components=h),i.type?s.getType(i.type)||(i.tagName=i.type,delete i.type):i.type="textnode",Object(u.extend)(i,f.props);}return i},onAdd:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.domc,i=this.em,o=t.getStyle(),a=i&&i.getConfig("avoidInlineStyle");if(r&&r.Component.ensureInList(t),!Object(u.isEmpty)(o)&&!a&&i&&i.get&&i.getConfig("forceClass")&&!n.temporary){var s=t.cid;i.get("CssComposer").setClassRule(s,o);t.setStyle({}),t.addClass(s);}}});},function(t,e,n){n.r(e);var r=n(1),i=n.n(r);i.a.$;e.default=i.a.View.extend({initialize:function(t){this.config=t||{},this.editorModel=this.em=this.config.em||{},this.pfx=this.config.stylePrefix,this.ppfx=this.config.pStylePrefix,this.hoverClass=this.pfx+"hover",this.badgeClass=this.pfx+"badge",this.plhClass=this.pfx+"placeholder",this.freezClass=this.ppfx+"freezed",this.canvas=this.em.get&&this.em.get("Canvas"),this.em.get&&this.setElement(this.getCanvas()),this.canvas&&(this.$canvas=this.$el,this.canvasTool=this.getCanvasTools()),this.init(this.config);},onFrameScroll:function(t){},getCanvas:function(){return this.canvas.getElement()},getCanvasBody:function(){return this.canvas.getBody()},getCanvasWrapper:function(){return this.canvas.getWrapperEl()},getCanvasTools:function(){return this.canvas.getToolsEl()},offset:function(t){var e=t.getBoundingClientRect();return {top:e.top+t.ownerDocument.body.scrollTop,left:e.left+t.ownerDocument.body.scrollLeft}},init:function(t){},callRun:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.id;if(t.trigger("run:".concat(n,":before"),e),!e||!e.abort){var r=e.sender||t,i=this.run(t,r,e);return t.trigger("run:".concat(n),i,e),t.trigger("run",n,i,e),i}t.trigger("abort:".concat(n),e);},callStop:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.id,r=e.sender||t;t.trigger("stop:".concat(n,":before"),e);var i=this.stop(t,r,e);return t.trigger("stop:".concat(n),i,e),t.trigger("stop",n,i,e),i},stopCommand:function(){this.em.get("Commands").stop(this.id);},run:function(t,e){},stop:function(t,e){}});},function(t,e,n){var r=n(2),i=n.n(r),o=n(0),a=n(3),s=n(27);function l(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function c(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?l(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):l(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var u=Object(s.a)().parseStyle;e.a={parseStyle:u,extendStyle:function(t){return c({},this.getStyle(),{},t)},getStyle:function(){return c({},this.get("style")||{})},setStyle:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};Object(o.isString)(e)&&(e=u(e));var r=this.getStyle(),i=c({},e);this.set("style",i,n);var s=Object(a.v)(r,i);return Object(o.keys)(s).forEach((function(e){var n=t.em;t.trigger("change:style:".concat(e)),n&&(n.trigger("styleable:change",t,e),n.trigger("styleable:change:".concat(e),t,e));})),i},addStyle:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};"string"==typeof t?t={prop:e}:n=e||{},t=this.extendStyle(t),this.setStyle(t,n);},removeStyle:function(t){var e=this.getStyle();delete e[t],this.setStyle(e);},styleToString:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=[],n=this.getStyle();for(var r in n){var i=t.important,a=Object(o.isArray)(i)?i.indexOf(r)>=0:i,s="".concat(n[r]).concat(a?" !important":""),l="__"==r.substr(0,2);s&&!l&&e.push("".concat(r,":").concat(s,";"));}return e.join("")},getSelectors:function(){return this.get("selectors")||this.get("classes")},getSelectorsString:function(){return this.selectorsToString?this.selectorsToString():this.getSelectors().getFullString()}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(1),a=n.n(o),s=n(28),l=n(0),c=n(3);function u(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function h(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?u(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):u(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var d=a.a.Model.extend({defaults:{name:"",property:"",type:"",defaults:"",info:"",value:"",icon:"",functionName:"",status:"",visible:!0,fixedValues:["initial","inherit"],full:0,important:0,toRequire:0,requires:null,requiresParent:null},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.get("id")||"",r=this.get("name")||"";!this.get("property")&&this.set("property",(r||n).replace(/ /g,"-"));var i=this.get("property");!this.get("id")&&this.set("id",i),!r&&this.set("name",Object(c.c)(i).replace(/-/g," ")),d.callInit(this,t,e);},init:function(){},clearValue:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return this.set({value:void 0,status:""},t),this},setValue:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1,n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.parseValue(t),i=!e;!i&&this.set({value:""},{avoidStore:i,silent:!0}),this.set(r,h({avoidStore:i},n));},setValueFromInput:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};this.setValue(t,e,h({},n,{fromInput:1}));},parseValue:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n={value:t},r="!important";if(Object(l.isString)(t)&&-1!==t.indexOf(r)&&(n.value=t.replace(r,"").trim(),n.important=1),!this.get("functionName")&&!e.complete)return n;var i=[],o="".concat(n.value),a=o.indexOf("(")+1,s=o.lastIndexOf(")"),c=o.substring(0,a-1);if(c&&(n.functionName=c),i.push(a),s>=0&&i.push(s),n.value=String.prototype.substring.apply(o,i),e.numeric){var u=parseFloat(n.value);n.unit=n.value.replace(u,""),n.value=u;}return n},splitValues:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:",",n=[],r="(",i=")",o="",a=0;return (t||"").split("").forEach((function(t){t==r?(a++,o+=r):t==i&&a>0?(a--,o+=i):t===e&&0==a?(n.push(o),o=""):o+=t;})),""!==o&&n.push(o),n.map((function(t){return t.trim()}))},getDefaultValue:function(){return this.get("defaults")},getFullValue:function(t){var e=this.get("functionName"),n=this.getDefaultValue(),r=Object(l.isUndefined)(t)?this.get("value"):t,i=!Object(l.isUndefined)(r)&&""!==r;return r&&n&&r===n?n:(e&&i&&(r="".concat(e,"(").concat(r,")")),i&&this.get("important")&&(r="".concat(r," !important")),r||"")}},{callParentInit:function(t,e,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{};t.prototype.initialize.apply(e,[n,h({},r,{skipInit:1})]);},callInit:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};!n.skipInit&&t.init(e,n);}}),f=d;function p(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var g=f.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?p(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):p(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},f.prototype.defaults,{detached:0,properties:[],separator:" "}),initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};f.callParentInit(f,this,t,e);var r=this.get("properties")||[],i=n(23).default;this.set("properties",new i(r)),this.listenTo(this,"change:value",this.updateValues),f.callInit(this,t,e);},clearValue:function(){return this.get("properties").each((function(t){return t.clearValue()})),f.prototype.clearValue.apply(this,arguments)},updateValues:function(){var t=this.getFullValue().split(this.getSplitSeparator());this.get("properties").each((function(e,n){var r=t.length;t[n]||t[n%r+(1!=r&&r%2?1:0)];}));},getSplitSeparator:function(){return new RegExp("".concat(this.get("separator"),"(?![^\\(]*\\))"))},getDefaultValue:function(t){var e=this.get("defaults");return e&&!t?e:(e="",this.get("properties").each((function(t,n){return e+="".concat(t.getDefaultValue()," ")})),e.trim())},getFullValue:function(){return this.get("detached")?"":this.get("properties").getFullValue()},getPropertyAt:function(t){return this.get("properties").at(t)}}),v=a.a.Model.extend({defaults:{index:"",value:"",values:{},active:!1,preview:!1,properties:[]},initialize:function(){var t=this.get("properties"),e=this.get("value");this.set("properties",t instanceof st?t:new st(t));var n=this.get("properties");if(n.forEach(this.onPropAdd,this),this.listenTo(n,"add",this.onPropAdd),!e){var r="",i=this.get("values");for(var o in i)r+=" "+i[o];this.set("value",r.trim());}},onPropAdd:function(t){var e=this.collection;t.parent=e&&e.property;},getPropertyAt:function(t){return this.get("properties").at(t)},getPropertyValue:function(t){var e="";return this.get("properties").each((function(n){n.get("property")==t&&(e=n.getFullValue());})),e},getFullValue:function(){var t=[];return this.get("properties").each((function(e){return t.push(e.getFullValue())})),t.join(" ").trim()}});function m(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function y(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?m(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):m(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var b=a.a.Collection.extend({model:v,initialize:function(){this.idx=1,this.on("add",this.onAdd),this.on("reset",this.onReset);},onAdd:function(t,e,n){n.noIncrement||t.set("index",this.idx++),n.active&&this.active(this.indexOf(t));},onReset:function(){this.idx=1;},getSeparator:function(){var t=this.property;return t?t.get("layerSeparator"):", "},getLayersFromValue:function(t){var e=this,n=[];return t.replace(/\(([\w\s,.]*)\)/g,(function(e){var n=e.replace(/,\s*/g,",");t=t.replace(e,n);})),(t?t.split(this.getSeparator()):[]).forEach((function(t){n.push({properties:e.properties.parseValue(t)});})),n},getLayersFromStyle:function(t){var e=[],n=this.properties;return n.each((function(n){var r=t[n.get("property")];(r?r.split(", "):[]).forEach((function(t,r){t=n.parseValue(t.trim()).value;var i=e[r],o=y({},n.attributes,{},{value:t});i?i.properties.push(o):e[r]={properties:[o]};}));})),e.forEach((function(t){var e=t.properties.map((function(t){return t.property}));n.each((function(n){var r=n.get("property");e.indexOf(r)<0&&t.properties.push(y({},n.attributes));}));})),e},active:function(t){this.each((function(t){return t.set("active",0)}));var e=this.at(t);e&&e.set("active",1);},getFullValue:function(){var t=[];return this.each((function(e){return t.push(e.getFullValue())})),t.join(this.getSeparator())},getPropertyValues:function(t,e){var n=[];return this.each((function(r){var i=r.getPropertyValue(t);i?n.push(i):!Object(l.isUndefined)(e)&&n.push(e);})),n.join(", ")}});function w(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var x=g.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?w(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):w(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},g.prototype.defaults,{layers:[],layerSeparator:", ",prepend:0,preview:0}),initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};g.callParentInit(g,this,t,e);var n=this.get("layers"),r=new b(n);r.property=this,r.properties=this.get("properties"),this.set("layers",r),g.callInit(this,t,e);},getLayers:function(){return this.get("layers")},getCurrentLayer:function(){return this.getLayers().filter((function(t){return t.get("active")}))[0]},getFullValue:function(){return this.get("detached")?"":this.get("layers").getFullValue()},getValueFromStyle:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.getLayers().getLayersFromStyle(t);return new b(e).getFullValue()},clearValue:function(){return this.getLayers().reset(),g.prototype.clearValue.apply(this,arguments)},getValueFromTarget:function(t){var e=this.attributes,n=e.detached,r=e.property,i=e.properties,o=t.getStyle(),a={};return i.forEach((function(t){var e=t.get("property"),n=o[e];n&&(a[e]=n);})),n?Object(l.keys)(a).length?a:"":o[r]},getLayersFromTarget:function(t){}}),O=n(30),C=n.n(O);function S(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function k(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?S(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):S(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var T=a.a.View.extend({template:function(t){var e=this.pfx;return '\n      <div class="'.concat(e,'label">\n        ').concat(this.templateLabel(t),'\n      </div>\n      <div class="').concat(this.ppfx,'fields">\n        ').concat(this.templateInput(t),"\n      </div>\n    ")},templateLabel:function(t){var e=this.pfx,n=this.em,r=t.parent,i=t.attributes,o=i.icon,a=void 0===o?"":o,s=i.info,l=void 0===s?"":s,c=i.id,u=i.name,h=n&&n.t("styleManager.properties.".concat(c))||u;return '\n      <span class="'.concat(e,"icon ").concat(a,'" title="').concat(l,'">\n        ').concat(h,"\n      </span>\n      ").concat(r?"":'<b class="'.concat(e,'clear" ').concat("data-clear-style",">&Cross;</b>"),"\n    ")},templateInput:function(t){return '\n      <div class="'.concat(this.ppfx,'field">\n        <input placeholder="').concat(t.getDefaultValue(),'"/>\n      </div>\n    ')},events:i()({change:"inputValueChanged"},"click [".concat("data-clear-style","]"),"clear"),initialize:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};Object(l.bindAll)(this,"targetUpdated"),this.config=e.config||{};var n=this.config.em;this.em=n,this.pfx=this.config.stylePrefix||"",this.ppfx=this.config.pStylePrefix||"",this.target=e.target||{},this.propTarget=e.propTarget||{},this.onChange=e.onChange,this.onInputRender=e.onInputRender||{},this.customValue=e.customValue||{};var r=this.model;this.property=r.get("property"),this.input=null;var i=this.pfx;this.inputHolderId="#"+i+"input-holder",this.sector=r.collection&&r.collection.sector,r.view=this,r.get("value")||r.set("value",r.getDefaultValue()),n&&n.on("update:component:style:".concat(this.property),this.targetUpdated);var o=r.get("requires");o&&Object.keys(o).forEach((function(e){n&&n.on("component:styleUpdate:".concat(e),t.targetUpdated);})),this.listenTo(this.propTarget,"update styleManager:update",this.targetUpdated),this.listenTo(r,"destroy remove",this.remove),this.listenTo(r,"change:value",this.modelValueChanged),this.listenTo(r,"targetUpdated",this.targetUpdated),this.listenTo(r,"change:visible",this.updateVisibility),this.listenTo(r,"change:status",this.updateStatus),this.listenTo(r,"change:name change:className change:full",this.render);var a=this.init&&this.init.bind(this);a&&a();},updateStatus:function(){var t=this.model,e=t.get("status"),n=t.parent,r=this.pfx,i=this.ppfx,o=this.config,a="".concat(i,"four-color"),s="".concat(i,"color-warn"),l=this.$el.children(".".concat(r,"label")),c=this.getClearEl(),u=c?c.style:{};switch(l.removeClass("".concat(a," ").concat(s)),u.display="none",e){case"updated":!n&&l.addClass(a),o.clearProperties&&(u.display="inline");break;case"computed":l.addClass(s);}},clear:function(t){var e=this;t&&t.stopPropagation(),this.model.clearValue(),setTimeout((function(){return e.targetUpdated()}));},getClearEl:function(){return this.clearEl||(this.clearEl=this.el.querySelector("[".concat("data-clear-style","]"))),this.clearEl},getTarget:function(){return this.getTargetModel()},getTargets:function(){return this.propTarget.targets||[this.getTarget()]},getTargetModel:function(){return this.propTarget&&this.propTarget.model},getHelperModel:function(){return this.propTarget&&this.propTarget.helper},inputValueChanged:function(t){t&&t.stopPropagation(),this.model.setValueFromInput(this.getInputValue()),this.elementUpdated();},elementUpdated:function(){this.setStatus("updated");},setStatus:function(t){this.model.set("status",t);var e=this.model.parent;e&&"updated"==t&&e.set("status",t);},emitUpdateTarget:Object(l.debounce)((function(){var t=this.config.em;t&&t.trigger("styleManager:update:target",this.getTarget());})),_getTargetData:function(){var t=this.model,e=this.config,n=this.getTargetValue({ignoreDefault:1}),r=t.getDefaultValue(),i=this.getComputedValue(),o="",a="";return n?(o=n,e.highlightChanged&&(a="updated")):i&&e.showComputed&&i!=r?(o=i,e.highlightComputed&&(a="computed")):(o=r,a=""),{value:o,status:a,targetValue:n,defaultValue:r,computedValue:i}},targetUpdated:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(this.emitUpdateTarget(),this.checkVisibility()){var r=this.config,i=r.em,o=this.model,a=o.get("property"),s=this._getTargetData(),l=s.status,c=s.value,u=C()(s,["status","value"]),h=k({status:l,value:c},u);return this.setStatus(l),o.setValue(c,0,k({fromTarget:1},n)),i&&(i.trigger("styleManager:change",this,a,c,h),i.trigger("styleManager:change:".concat(a),this,c,h),this._emitUpdate(h)),h}},_emitUpdate:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.em,n=this.model;if(e){var r=n.get("property"),i=k({},this._getEventData(),{},t),o=i.id;e.trigger("style:update",i),e.trigger("style:update:".concat(r),i),r!==o&&e.trigger("style:update:".concat(o),i);}},_getEventData:function(){var t=this.model;return {propertyView:this,targets:this.getTargets(),value:t.getFullValue(),property:t,id:t.get("id"),name:t.get("property")}},checkVisibility:function(){var t=1;return this.config.hideNotStylable&&(this.isTargetStylable()&&this.isComponentStylable()?this.show():(this.hide(),t=0),this.sector&&this.sector.trigger("updateVisibility")),t},getTargetValue:function(){var t,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=this.model,r=this.getTargetModel(),i=this.customValue;if(!r)return t;if((t=r.getStyle()[n.get("property")])||e.ignoreDefault||(t=n.getDefaultValue()),"function"==typeof i&&!e.ignoreCustomValue){var o=n.collection.indexOf(n),a=i(this,o,t);a&&(t=a);}return t},getComputedValue:function(){var t=this.propTarget,e=t.computed||{},n=t.computedDefault||{},r=this.config.avoidComputed||[],i=this.model.get("property"),o=r.indexOf(i)<0,a=e[i],s=n[Object(c.b)(i)];return e&&o&&s!==a&&a||""},getInputValue:function(){var t=this.getInputEl();return t?t.value:""},modelValueChanged:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=this.model,o=i.getFullValue();r.fromInput||this.setValue(o),r.fromTarget||this.getTargets().forEach((function(t){return n.__updateTarget(t,r)}));},__updateTarget:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.model,r=this.config.em,i=n.get("property"),o=n.getFullValue(),a=this.onChange;if(t&&this.isTargetStylable(t)&&this.isComponentStylable()){e.fromTarget||(a&&!e.fromParent?a(t,this,e):this.updateTargetStyle(o,null,k({},e,{target:t})));var s=r&&r.getSelected();r&&s&&(!e.noEmit&&r.trigger("component:update",s),r.trigger("component:styleUpdate",s,i),r.trigger("component:styleUpdate:".concat(i),s)),this._emitUpdate();}},updateTargetStyle:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=e||this.model.get("property"),i=n.target||this.getTarget(),o=i.getStyle();t?o[r]=t:delete o[r],n.avoidStore?o.__=1:delete o.__,i.setStyle(o,n);var a=this.getHelperModel();a&&a.setStyle(o,n);},isTargetStylable:function(t){var e=t||this.getTarget(),n=this.model,r=n.get("id"),i=n.get("property"),o=n.get("toRequire"),a=e.get("unstylable"),s=e.get("stylable-require"),c=n.get("requires"),u=n.get("requiresParent"),h=this.sector?this.sector.collection:null,d=this.em?this.em.getSelected():null,f=e.get("stylable");if(Object(l.isArray)(f)&&(f=f.indexOf(i)>=0),Object(l.isArray)(a)&&(f=a.indexOf(i)<0),o&&(f=!t||s&&(s.indexOf(r)>=0||s.indexOf(i)>=0)),h&&c){var p=Object.keys(c);h.each((function(t){t.get("properties").each((function(t){if(Object(l.includes)(p,t.id)){var e=c[t.id];f=f&&Object(l.includes)(e,t.get("value"));}}));}));}if(u){var g=d&&d.parent(),v=g&&g.getEl();if(v){var m=window.getComputedStyle(v);Object(l.each)(u,(function(t,e){f=f&&m[e]&&Object(l.includes)(t,m[e]);}));}else f=!1;}return f},isComponentStylable:function(){var t=this.em,e=t&&t.getSelected();return !e||this.isTargetStylable(e)},setRawValue:function(t){this.setValue(this.model.parseValue(t));},setValue:function(t){var e=this.model,n=Object(l.isUndefined)(t)?e.getDefaultValue():t,r=this.getInputEl();r&&(r.value=n);},getInputEl:function(){return this.input||(this.input=this.el.querySelector("input")),this.input},updateVisibility:function(){this.el.style.display=this.model.get("visible")?"block":"none";},show:function(){this.model.set("visible",1);},hide:function(){this.model.set("visible",0);},cleanValue:function(){this.setValue("");},clearCached:function(){this.clearEl=null,this.input=null,this.$input=null;},render:function(){this.clearCached();var t=this.pfx,e=this.model,n=this.el,r=e.get("property"),i=e.get("full"),o=e.get("className")||"",a="".concat(t,"property");n.innerHTML=this.template(e),n.className="".concat(a," ").concat(t).concat(e.get("type")," ").concat(a,"__").concat(r," ").concat(o).trim(),n.className+=i?" ".concat(a,"--full"):"",this.updateStatus();var s=this.onRender&&this.onRender.bind(this);s&&s(),this.setValue(e.get("value"),{fromTarget:1});}});function j(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function P(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?j(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):j(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var E=a.a.$,M=T.extend({templateInput:function(){var t=this.pfx;return '\n      <div class="'.concat(t,"field ").concat(t,'composite">\n        <span id="').concat(t,'input-holder"></span>\n      </div>\n    ')},inputValueChanged:function(){if(!this.model.get("detached")){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];T.prototype.inputValueChanged.apply(this,e);}},clear:function(t){var e=this.properties;e&&e.forEach((function(t){return t.clear()})),T.prototype.clear.apply(this,arguments);},onRender:function(){var t=this.model,e=t.get("properties")||[];if(this.properties=[],e.length&&(this.$input||(this.$input=E('<input type="hidden" value="0">'),this.input=this.$input.get(0)),this.props||(this.props=t.get("properties")),!this.$props)){this.props.each((function(e,n){e&&"composite"==e.get("type")&&(this.props.remove(e),console.warn("Nested composite types not yet allowed.")),e.parent=t;}),this);var r=new(n(18).default)(this.getPropsConfig());this.$props=r.render().$el,this.properties=r.properties,this.$el.find("#".concat(this.pfx,"input-holder")).append(this.$props);}},getPropsConfig:function(t){var e=this,n=this.model,r={config:P({},this.config,{highlightComputed:0}),collection:this.props,target:this.target,propTarget:this.propTarget,onChange:function(t,e,r){n.set("value",n.getFullValue(),r);},customValue:function(t,n){return e.valueOnIndex(n,t)}};return n.get("detached")&&delete r.onChange,r},valueOnIndex:function(t,e){var n,r=this.getTargetValue({ignoreDefault:1});r?n=r.split(this.model.getSplitSeparator())[t]:n=e&&e.getTargetValue({ignoreCustomValue:1,ignoreDefault:1});return n},clearCached:function(){T.prototype.clearCached.apply(this,arguments),this.$input=null,this.props=null,this.$props=null;}}),D=n(18),A=a.a.View.extend({events:{click:"active","click [data-close-layer]":"removeItem","mousedown [data-move-layer]":"initSorter","touchstart [data-move-layer]":"initSorter"},template:function(t){var e=this.pfx,n=this.ppfx,r=this.em,i="".concat(r&&r.t("styleManager.layer")," ").concat(t.get("index"));return '\n      <div id="'.concat(e,'move" class="').concat(n,'no-touch-actions" data-move-layer>\n        <i class="fa fa-arrows"></i>\n      </div>\n      <div id="').concat(e,'label">').concat(i,'</div>\n      <div id="').concat(e,'preview-box">\n      \t<div id="').concat(e,'preview" data-preview></div>\n      </div>\n      <div id="').concat(e,'close-layer" class="').concat(e,'btn-close" data-close-layer>\n        &Cross;\n      </div>\n      <div id="').concat(e,'inputs" data-properties></div>\n      <div style="clear:both"></div>\n    ')},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model;this.stackModel=t.stackModel,this.config=t.config||{},this.em=this.config.em,this.pfx=this.config.stylePrefix||"",this.ppfx=this.config.pStylePrefix||"",this.sorter=t.sorter||null,this.propsConfig=t.propsConfig||{},this.customPreview=t.onPreview,this.listenTo(e,"destroy remove",this.remove),this.listenTo(e,"change:active",this.updateVisibility),this.listenTo(e.get("properties"),"change",this.updatePreview),e.view=this,e.set({droppable:0,draggable:1}),this.$el.data("model",e);},initSorter:function(t){this.sorter&&this.sorter.startSort(this.el);},removeItem:function(t){t&&t.stopPropagation(),this.remove();},remove:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model,n=this.props,r=e.collection,i=this.stackModel;a.a.View.prototype.remove.apply(this,arguments),r&&r.contains(e)&&r.remove(e),i&&i.set&&(i.set({stackIndex:null},{silent:!0}),!t.fromTarget&&i.trigger("updateValue")),n&&n.remove();},onPreview:function(t){var e=this.stackModel,n=e&&e.get("detached"),r=t.split(" "),i=[],o={};return this.model.get("properties").each((function(t,e){var a=t.get("property"),s=n?t.getFullValue():r[e]||"";if(s&&"integer"==t.get("type")){var l=parseInt(s,10),c=s.replace(l,"");s=(l=(l=(l=isNaN(l)?0:l)>3?3:l)<-3?-3:l)+c;}i.push(s),o[a]=s;})),n?o:i.join(" ")},updatePreview:function(){var t=this.stackModel,e=this.customPreview,n=this.getPreviewEl(),r=this.model.getFullValue(),i=e?e(r):this.onPreview(r);if(i&&t&&n){var o=n.style;if(Object(l.isString)(i))o[t.get("property")]=i;else {var a=[];Object(l.each)(i,(function(t,e){return a.push("".concat(e,":").concat(t))})),n.setAttribute("style",a.join(";"));}}},getPropertiesWrapper:function(){return this.propsWrapEl||(this.propsWrapEl=this.el.querySelector("[data-properties]")),this.propsWrapEl},getPreviewEl:function(){return this.previewEl||(this.previewEl=this.el.querySelector("[data-preview]")),this.previewEl},active:function(){var t=this.model,e=t.collection;e.active(e.indexOf(t));},updateVisibility:function(){var t=this.pfx,e=this.getPropertiesWrapper(),n=this.model.get("active");e.style.display=n?"":"none",this.$el[n?"addClass":"removeClass"]("".concat(t,"active"));},render:function(){var t=this.propsConfig,e=this.model,n=this.el,r=this.pfx,i=e.get("preview"),o=new D.default({collection:e.get("properties"),config:this.config,target:t.target,customValue:t.customValue,propTarget:t.propTarget,onChange:t.onChange}),a=o.render().el;return n.innerHTML=this.template(e),n.className="".concat(r,"layer").concat(i?"":" ".concat(r,"no-preview")),this.props=o,this.getPropertiesWrapper().appendChild(a),this.updateVisibility(),this.updatePreview(),this}}),L=a.a.View.extend({initialize:function(t){this.config=t.config||{},this.stackModel=t.stackModel,this.preview=t.preview,this.pfx=this.config.stylePrefix||"",this.ppfx=this.config.pStylePrefix||"",this.propsConfig=t.propsConfig;var e=this.pfx,n=this.ppfx,r=this.collection;this.className="".concat(e,"layers ").concat(n,"field"),this.listenTo(r,"add",this.addTo),this.listenTo(r,"deselectAll",this.deselectAll),this.listenTo(r,"reset",this.reset),this.items=[];var i=this.config.em||"",o=i?i.get("Utils"):"";this.sorter=o?new o.Sorter({container:this.el,ignoreViewChildren:1,containerSel:".".concat(e,"layers"),itemSel:".".concat(e,"layer"),pfx:this.config.pStylePrefix}):"",r.view=this,this.$el.data("model",r),this.$el.data("collection",r);},addTo:function(t){var e=this.collection.indexOf(t);this.addToCollection(t,null,e);},addToCollection:function(t,e,n){var r=e||null,i=this.stackModel,o=this.config,a=this.sorter,s=this.propsConfig;void 0!==this.preview&&t.set("preview",this.preview);var l=new A({model:t,config:o,sorter:a,stackModel:i,propsConfig:s}),c=l.render().el;if(this.items.push(l),r)r.appendChild(c);else if(void 0!==n){var u="before";this.$el.children().length==n&&(n--,u="after"),n<0?this.$el.append(c):this.$el.children().eq(n)[u](c);}else this.$el.append(c);return c},deselectAll:function(){this.$el.find("."+this.pfx+"layer").removeClass(this.pfx+"active");},reset:function(t,e){this.clearItems(e),this.render();},render:function(){var t=document.createDocumentFragment();return this.$el.empty(),this.collection.each((function(e){this.addToCollection(e,t);}),this),this.$el.append(t),this.$el.attr("class",this.className),this.sorter&&(this.sorter.plh=null),this},remove:function(){this.clearItems(),a.a.View.prototype.remove.apply(this,arguments);},clearItems:function(t){this.items.forEach((function(e){return e.remove(t)})),this.items=[];}});function N(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var I=new(n(29).a),F=M.extend({templateInput:function(){var t=this.pfx;this.ppfx;return '\n      <div class="'.concat(t,"field ").concat(t,'stack">\n        <button type="button" id="').concat(t,'add" data-add-layer>+</button>\n        <div data-layers-wrapper></div>\n      </div>\n    ')},init:function(){var t=this.model;this.pfx;t.set("stackIndex",null),this.events["click [data-add-layer]"]="addLayer",this.listenTo(t,"change:stackIndex",this.indexChanged),this.listenTo(t,"updateValue",this.inputValueChanged),this.delegateEvents();var e=this.getPropsConfig();this.layers=new L({collection:this.getLayers(),stackModel:t,preview:t.get("preview"),config:this.config,propsConfig:e});var r=n(18).default;this.propsView=new r({target:this.target,collection:t.get("properties"),stackModel:t,config:this.config,onChange:e.onChange,propTarget:e.propTarget});},targetUpdated:function(){var t,e=this;if(this.model.get("detached"))t=this._getTargetData(),this.setStatus(t.status),this.checkVisibility();else {for(var n=arguments.length,r=new Array(n),i=0;i<n;i++)r[i]=arguments[i];t=M.prototype.targetUpdated.apply(this,r);}setTimeout((function(){return e.refreshLayers(t)}));},getLayers:function(){return this.model.get("layers")},indexChanged:function(t){var e=this.model;this.getLayers().active(e.get("stackIndex"));},addLayer:function(){var t=this.model,e=this.getLayers(),n=t.get("prepend"),r=t.get("properties").deepClone();r.each((function(t){return t.set("value","")}));var o=e.add({properties:r},function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?N(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):N(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({active:1},n&&{at:0}));this.inputValueChanged({up:1}),t.set("stackIndex",e.indexOf(o));},inputValueChanged:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model;t.up&&this.elementUpdated(),e.get("detached")?e.get("properties").each((function(t){return t.trigger("change:value")})):e.set("value",this.getLayerValues());},setValue:function(){},getLayerValues:function(){return this.getLayers().getFullValue()},_getClassRule:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.em,n=t.skipAdd,r=void 0===n?1:n,i=e.getSelected(),o=e.get("StyleManager").getModelToStyle(i,{skipAdd:r,useClasses:1});return o!==i&&o},_getParentTarget:function(t){for(var e,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=this.em,i=this.model,o=i.get("property"),a=n.isValid||function(t){return t.getStyle()[o]},s=r.get("CssComposer").getAll().filter((function(e){return e.selectorsToString()===t.getSelectorsString()})),l=s.reduce((function(t,e){return t[e.getAtRule()]=e,t}),{}),c=I.sortMediaObject(l),u=c.map((function(t){return t.value})),h=u.indexOf(t),d=u.splice(0,h),f=d.length-1;f>-1;f--){var p=d[f];if(a(p)){e=p;break}}return e},refreshLayers:function(){var t,e,n,r,i,o,a=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},s=[],c=this.model,u=this.em,h=this.getLayers(),d=c.get("detached"),f=c.get("property"),p=this.getTarget(),g=this.getComputedValue(),v=u.getSelected(),m={fromTarget:1};if(d){e=a.targetValue||{};var y=function(t){var e=c.get("properties").at(0).get("property");return t&&!Object(l.isUndefined)(t.getStyle()[e])};if(!Object(l.keys)(e).length&&g&&v){var b={isValid:function(t){return y(t)}};(r=this._getParentTarget(p,b))?e=r.getStyle():(n=this._getClassRule(),r=!(i=y(n)&&n.getStyle())&&this._getParentTarget(this._getClassRule({skipAdd:0}),b),o=y(r)&&r.getStyle(),e=i||o||{});}t=e,s=h.getLayersFromStyle(e);}else {var w=this.getTargetValue({ignoreDefault:1}),x=w;!x&&g&&((r=this._getParentTarget(p))?x=r.getStyle()[f]:(o=(r=!(i=(n=this._getClassRule())&&n.getStyle()[f])&&this._getParentTarget(this._getClassRule({skipAdd:0})))&&r.getStyle()[f],x=i||o||g)),t=x=x==c.getDefaultValue()?"":x,s=h.getLayersFromValue(x);}var O=c.getLayersFromTarget(p,{resultValue:t,layersObj:s})||s;h.reset(null,m),h.add(O,m),c.set({stackIndex:null},{silent:!0});},getTargetValue:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model,n=e.attributes.detached,r=this.getTarget(),i=M.prototype.getTargetValue.call(this,t);return Object(l.isUndefined)(i)&&!n?i=e.getValueFromStyle(r.getStyle()):n&&(i=e.getValueFromTarget(r)),i},getPropsConfig:function(){var t=this,e=t.model;return {target:t.target,propTarget:t.propTarget,onChange:function(n,r,i){var o=r.model;if(e.get("detached")){var a=o.get("property"),s=o.getDefaultValue(),l=t.getLayers().getPropertyValues(a,s);r.updateTargetStyle(l,null,i);}else if("updated"==e.get("status")){var c=e.getFullValue();e.set("value",c,i),!c&&r.updateTargetStyle(c,null,i);}}}},onRender:function(){var t=this.el,e=this.layers,n=this.propsView,r=t.querySelector("[data-layers-wrapper]");n.render(),r.appendChild(e.render().el);}}),_=a.a.$,V=T.extend({templateInput:function(){var t=this.pfx,e=this.em;return '\n    <div class="'.concat(t,"field ").concat(t,"file\">\n      <div id='").concat(t,"input-holder'>\n        <div class=\"").concat(t,'btn-c">\n          <button class="').concat(t,'btn" id="').concat(t,'images" type="button">\n            ').concat(e.t("styleManager.fileButton"),'\n          </button>\n        </div>\n        <div style="clear:both;"></div>\n      </div>\n      <div id="').concat(t,'preview-box">\n        <div id="').concat(t,'preview-file"></div>\n        <div id="').concat(t,'close">&Cross;</div>\n      </div>\n    </div>\n    ')},init:function(){var t=this.em;this.modal=t.get("Modal"),this.am=t.get("AssetManager"),this.events["click #"+this.pfx+"close"]="removeFile",this.events["click #"+this.pfx+"images"]="openAssetManager",this.delegateEvents();},onRender:function(){if(!this.$input){var t=this.model.getDefaultValue();this.$input=_('<input placeholder="'.concat(t,'">'));}this.$preview||(this.$preview=this.$el.find("#"+this.pfx+"preview-file")),this.$previewBox||(this.$previewBox=this.$el.find("#"+this.pfx+"preview-box")),this.setValue(this.componentValue,0);},clearCached:function(){T.prototype.clearCached.apply(this,arguments),this.$preview=null,this.$previewBox=null;},setValue:function(t,e){T.prototype.setValue.apply(this,arguments),this.setPreviewView(t&&t!=this.model.getDefaultValue()),this.setPreview(t);},setPreviewView:function(t){var e=this.$previewBox;e&&e[t?"addClass":"removeClass"]("".concat(this.pfx,"show")),e&&e.css({display:t?"block":"none"});},spreadUrl:function(t){this.model.set("value",t),this.setPreviewView(1);},setPreview:function(t){var e=this.$preview;t=t&&t.indexOf("url(")<0?"url(".concat(t,")"):t,e&&e.css("background-image",t);},cleanValue:function(){this.setPreviewView(0),this.model.set({value:""},{silent:!0});},removeFile:function(){this.model.set("value",this.model.getDefaultValue());for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];T.prototype.cleanValue.apply(this,e),this.setPreviewView(0);},openAssetManager:function(t){var e=this,n=this.em,r=this.modal,i=n?n.get("Editor"):"";i&&i.runCommand("open-assets",{types:["image"],accept:"image/*",target:this.getTargetModel(),onClick:function(){},onDblClick:function(){},onSelect:function(t){r.close();var n=Object(l.isString)(t)?t:t.get("src");e.spreadUrl(n);}});}}),R=a.a.$,H=T.extend({templateInput:function(){return ""},init:function(){var t=this.model;this.listenTo(t,"change:unit",this.modelValueChanged),this.listenTo(t,"el:change",this.elementUpdated),this.listenTo(t,"change:units",this.render);},setValue:function(t){var e=this.model.parseValue(t);t="".concat(e.value).concat(e.unit),this.inputInst.setValue(t,{silent:1});},onRender:function(){var t=this.ppfx;if(!this.input){var e=this.model.input;e.ppfx=t,e.render(),this.el.querySelector(".".concat(t,"fields")).appendChild(e.el),this.$input=e.inputEl,this.unit=e.unitEl,this.$unit=R(this.unit),this.input=this.$input.get(0),this.inputInst=e;}},clearCached:function(){T.prototype.clearCached.apply(this,arguments),this.unit=null,this.$unit=null;}}),z=n(37);function $(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function W(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?$(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):$(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var B=H.extend({setValue:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};e=W({},e,{silent:1}),this.inputInst.setValue(t,e);},onRender:function(){if(!this.input){var t=this.ppfx,e=new z.a({target:this.target,model:this.model,ppfx:t}).render();this.el.querySelector(".".concat(t,"fields")).appendChild(e.el),this.$input=e.inputEl,this.$color=e.colorEl,this.input=this.$input.get(0),this.inputInst=e;}}}),U=n(6),q=n.n(U);function G(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var K=f.extend({defaults:function(){return function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?G(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):G(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},f.prototype.defaults,{options:[],full:1})},initialize:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];f.prototype.initialize.apply(this,e),this.listenTo(this,"change:options",this.onOptionChange);},onOptionChange:function(){this.set("list",this.get("options"));},getOptions:function(){var t=this.attributes,e=t.options,n=t.list;return e&&e.length?e:n},setOptions:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[];return this.set("options",t),this},addOption:function(t){if(t){var e=this.getOptions();this.setOptions([].concat(q()(e),[t]));}return this}});function Y(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var X=K.extend({defaults:function(){return function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?Y(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):Y(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},K.prototype.defaults(),{full:0})}}),J=(a.a.$,T.extend({templateInput:function(){var t=this.pfx,e=this.ppfx;return '\n      <div class="'.concat(e,"field ").concat(e,'select">\n        <span id="').concat(t,'input-holder"></span>\n        <div class="').concat(e,'sel-arrow">\n          <div class="').concat(e,'d-s-arrow"></div>\n        </div>\n      </div>\n    ')},initialize:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];T.prototype.initialize.apply(this,e),this.listenTo(this.model,"change:options",this.updateOptions);},updateOptions:function(){this.input=null,this.onRender();},onRender:function(){var t=this.pfx,e=this.model.getOptions();if(!this.input){var n="";e.forEach((function(t){var e=t.name||t.value,r=t.style?t.style.replace(/"/g,"&quot;"):"",i=r?'style="'.concat(r,'"'):"",o=t.value.replace(/"/g,"&quot;");n+='<option value="'.concat(o,'" ').concat(i,">").concat(e,"</option>");}));var r=this.el.querySelector("#".concat(t,"input-holder"));r.innerHTML="<select>".concat(n,"</select>"),this.input=r.firstChild;}}})),Z=T.extend({templateInput:function(){this.pfx;var t=this.ppfx;return '\n      <div class="'.concat(t,"field ").concat(t,'field-radio">\n      </div>\n    ')},onRender:function(){var t=this.pfx,e=this.ppfx,n="".concat(e,"radio-item-label"),r=this.model,i=r.get("property"),o=r.get("list")||r.get("options")||[],a=r.cid,s="".concat(t,"radio ").concat(t,"radio-").concat(i);if(!this.input&&o&&o.length){var l="";o.forEach((function(r){var o=r.className?"".concat(r.className," ").concat(t,"icon ").concat(n):"",c="".concat(i,"-").concat(r.value,"-").concat(a),u=r.name||r.value,h=r.title?'title="'.concat(r.title,'"'):"";l+='\n            <div class="'.concat(e,'radio-item">\n              <input type="radio" class="').concat(s,'" id="').concat(c,'" name="').concat(i,"-").concat(a,'" value="').concat(r.value,'"/>\n              <label class="').concat(o||n,'" ').concat(h,' for="').concat(c,'">').concat(o?"":u,"</label>\n            </div>\n          ");}));var c=this.el.querySelector(".".concat(e,"field"));c.innerHTML='<div class="'.concat(e,'radio-items">').concat(l,"</div>"),this.input=c.firstChild;}},getInputValue:function(){var t=this.getCheckedEl();return t?t.value:""},getCheckedEl:function(){var t=this.getInputEl();return t?t.querySelector("input:checked"):""},setValue:function(t){var e=this.model,n=t||e.get("value")||e.getDefaultValue(),r=this.getInputEl(),i=r?r.querySelector('[value="'.concat(n,'"]')):"";if(i)i.checked=!0;else {var o=this.getCheckedEl();o&&(o.checked=!1);}}}),Q=n(31);function tt(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var et=f.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?tt(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):tt(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},f.prototype.defaults,{units:[],unit:"",step:1,min:"",max:""}),initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};f.callParentInit(f,this,t,e);var n=this.get("unit"),r=this.get("units");this.input=new Q.a({model:this}),r.length&&!n&&this.set("unit",r[0]),f.callInit(this,t,e);},clearValue:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return this.set({value:void 0,unit:void 0},t),this},parseValue:function(t){var e=f.prototype.parseValue.apply(this,arguments),n=this.input.validateInputValue(e.value,{deepCheck:1}),r=n.value,i=n.unit;return e.value=r,e.unit=i,e},getFullValue:function(){var t=this.get("value"),e=this.get("unit");return t=Object(l.isUndefined)(t)?"":t,e=!Object(l.isUndefined)(e)&&t?e:"",t="".concat(t).concat(e),f.prototype.getFullValue.apply(this,[t])}});function nt(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var rt=et.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?nt(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):nt(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},et.prototype.defaults,{showInput:1})});function it(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var ot=H.extend({events:function(){return function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?it(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):it(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},H.prototype.events,{"change [type=range]":"inputValueChanged","input [type=range]":"inputValueChangedSoft",change:""})},templateInput:function(t){var e=this.ppfx;return '\n      <div class="'.concat(e,"field ").concat(e,'field-range">\n        <input type="range"\n          min="').concat(t.get("min"),'"\n          max="').concat(t.get("max"),'"\n          step="').concat(t.get("step"),'"/>\n      </div>\n    ')},getSliderEl:function(){return this.slider||(this.slider=this.el.querySelector("input[type=range]")),this.slider},inputValueChanged:function(){var t=this.model,e=t.get("step");this.getInputEl().value=this.getSliderEl().value;var n=this.getInputValue()-e;t.set("value",n,{avoidStore:1}).set("value",n+e),this.elementUpdated();},inputValueChangedSoft:function(){this.getInputEl().value=this.getSliderEl().value,this.model.set("value",this.getInputValue(),{avoidStore:1}),this.elementUpdated();},setValue:function(t){var e=this.model.parseValue(t);this.getSliderEl().value=parseFloat(e.value),H.prototype.setValue.apply(this,arguments);},onRender:function(){H.prototype.onRender.apply(this,arguments),this.model.get("showInput")||(this.inputInst.el.style.display="none");},clearCached:function(){H.prototype.clearCached.apply(this,arguments),this.slider=null;}});function at(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var st=e.default=a.a.Collection.extend(s.a).extend({types:[{id:"stack",model:x,view:F,isType:function(t){if(t&&"stack"==t.type)return t}},{id:"composite",model:g,view:M,isType:function(t){if(t&&"composite"==t.type)return t}},{id:"file",model:f,view:V,isType:function(t){if(t&&"file"==t.type)return t}},{id:"color",model:f,view:B,isType:function(t){if(t&&"color"==t.type)return t}},{id:"select",model:X,view:J,isType:function(t){if(t&&"select"==t.type)return t}},{id:"radio",model:K,view:Z,isType:function(t){if(t&&"radio"==t.type)return t}},{id:"slider",model:rt,view:ot,isType:function(t){if(t&&"slider"==t.type)return t}},{id:"integer",model:et,view:H,isType:function(t){if(t&&"integer"==t.type)return t}},{id:"base",model:f,view:T,isType:function(t){return t.type="base",t}}],deepClone:function(){var t=this.clone();return t.reset(t.map((function(t){var e=t.clone();return e.typeView=t.typeView,e}))),t},parseValue:function(t){var e=this,n=[];return t.split(" ").forEach((function(t,r){var o=e.at(r);o&&n.push(function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?at(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):at(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},o.attributes,{},{value:t}));})),n},getFullValue:function(){var t="";return this.each((function(e){return t+="".concat(e.getFullValue()," ")})),t.trim()}});},function(t,e){var n;n=function(){return this}();try{n=n||new Function("return this")();}catch(t){"object"==typeof window&&(n=window);}t.exports=n;},function(t,e,n){n.r(e);var r=n(1),i=n.n(r).a.$;e.default={startSelectPosition:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};this.isPointed=!1;var i=this.editorModel.get("Utils"),o=t.ownerDocument.body;i&&!this.sorter&&(this.sorter=new i.Sorter({container:o,placer:this.canvas.getPlacerEl(),containerSel:"*",itemSel:"*",pfx:this.ppfx,direction:"a",document:e,wmargin:1,nested:1,em:this.editorModel,canvasRelative:1,scale:function(){return n.em.getZoomDecimal()}})),r.onStart&&(this.sorter.onStart=r.onStart),t&&this.sorter.startSort(t,{container:o});},getOffsetDim:function(){var t=this.offset(this.canvas.getFrameEl()),e=this.offset(this.canvas.getElement());return {top:t.top-e.top,left:t.left-e.left}},stopSelectPosition:function(){this.posTargetCollection=null,this.posIndex="after"==this.posMethod&&0!==this.cDim.length?this.posIndex+1:this.posIndex,this.sorter&&(this.sorter.moved=0,this.sorter.endMove()),this.cDim&&(this.posIsLastEl=0!==this.cDim.length&&"after"==this.posMethod&&this.posIndex==this.cDim.length,this.posTargetEl=0===this.cDim.length?i(this.outsideElem):!this.posIsLastEl&&this.cDim[this.posIndex]?i(this.cDim[this.posIndex][5]).parent():i(this.outsideElem),this.posTargetModel=this.posTargetEl.data("model"),this.posTargetCollection=this.posTargetEl.data("model-comp"));},enable:function(){this.startSelectPosition();},nearFloat:function(t,e,n){var r=t||0,i=e||"before",o=n.length,a=0!==o&&"after"==i&&r==o;return 0!==o&&(!a&&!n[r][4]||n[r-1]&&!n[r-1][4]||a&&!n[r-1][4])?1:0},run:function(){this.enable();},stop:function(){this.stopSelectPosition(),this.$wrapper.css("cursor",""),this.$wrapper.unbind();}};},function(t,e,n){var r=n(39);window.Promise=window.Promise||r.a,e.a="function"==typeof fetch?fetch.bind():function(t,e){return new r.a((function(n,i){var o=new XMLHttpRequest;for(var a in o.open(e.method||"get",t),o.withCredentials="include"==e.credentials,e.headers||{})o.setRequestHeader(a,e.headers[a]);o.onload=function(t){return n({status:o.status,statusText:o.statusText,text:function(){return r.a.resolve(o.responseText)}})},o.onerror=i,o.upload&&e.onProgress&&(o.upload.onprogress=e.onProgress),e.body?o.send(e.body):o.send();}))};},function(t,e,n){var r=n(8),i=n.n(r),o=n(0);e.a=function(t){var e=t;return {compTypes:"",modelAttrStart:"data-gjs-",splitPropsFromAttr:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n={},r={};return Object(o.each)(e,(function(e,i){if(0===i.indexOf(t.modelAttrStart)){var a=i.replace("data-gjs-",""),s=e.length,l=e&&Object(o.isString)(e),c=l&&e.substr(0,1),u=l&&e.substr(s-1);e="false"!==(e="true"===e||e)&&e;try{e="{"==c&&"}"==u||"["==c&&"]"==u?JSON.parse(e):e;}catch(t){}n[a]=e;}else r[i]=e;})),{props:n,attrs:r}},parseStyle:function(t){for(var e={},n=t.split(";"),r=0,i=n.length;r<i;r++){var o=n[r].trim();if(o){var a=o.split(":");e[a[0].trim()]=a.slice(1).join(":").trim();}}return e},parseClass:function(t){for(var e=[],n=t.split(" "),r=0,i=n.length;r<i;r++){var o=n[r].trim();o&&e.push(o);}return e},parseNode:function(n){for(var r=[],o=n.childNodes,a=0,s=o.length;a<s;a++){var l=o[a],c=l.attributes||[],u=c.length,h=r[r.length-1],d=l.childNodes.length,f=this.compTypes,p={};if(f){var g="",v=l.getAttribute&&l.getAttribute("".concat("data-gjs-","type"));if(v)p={type:v};else {for(var m=0;m<f.length;m++){var y=f[m];if(g=y.model.isComponent(l)){"object"!==i()(g)&&(g={type:y.id});break}}p=g;}}p.tagName||(p.tagName=l.tagName?l.tagName.toLowerCase():""),u&&(p.attributes={});for(var b=0;b<u;b++){var w=c[b].nodeName,x=c[b].nodeValue;if("style"==w)p.style=this.parseStyle(x);else if("class"==w)p.classes=this.parseClass(x);else {if("contenteditable"==w)continue;if(0===w.indexOf("data-gjs-")){var O=w.replace("data-gjs-",""),C=x.length,S=x&&x.substr(0,1),k=x&&x.substr(C-1);x="false"!==(x="true"===x||x)&&x;try{x="{"==S&&"}"==k||"["==S&&"]"==k?JSON.parse(x):x;}catch(t){}p[O]=x;}else p.attributes[w]=x;}}if(d&&!p.components){var T=l.childNodes[0];1===d&&3===T.nodeType?(!p.type&&(p.type="text"),p.content=T.nodeValue):p.components=this.parseNode(l);}if("textnode"==p.type){if(h&&"textnode"==h.type){h.content+=p.content;continue}if(!t.keepEmptyTextNodes){var j=l.nodeValue;if(" "!=j&&!j.trim())continue}}var P=p.components;if(!p.type&&P){for(var E=1,M=0,D=0;D<P.length;D++){var A=P[D],L=A.type;if(["text","textnode"].indexOf(L)<0&&e.textTags.indexOf(A.tagName)<0){E=0;break}"textnode"==L&&(M=1);}E&&M&&(p.type="text");}(p.tagName||"textnode"==p.type)&&r.push(p);}return r},parse:function(t,n){var r=e.em&&e.em.get("Config")||{},i={html:"",css:""},o=document.createElement("div");o.innerHTML=t;var a=o.querySelectorAll("script"),s=a.length;if(!r.allowScripts)for(;s--;)a[s].parentNode.removeChild(a[s]);if(n){for(var l="",c=o.querySelectorAll("style"),u=c.length;u--;)l=c[u].innerHTML+l,c[u].parentNode.removeChild(c[u]);l&&(i.css=n.parse(l));}var h=this.parseNode(o);return 1==h.length&&(h=h[0]),i.html=h,i}}};},function(t,e,n){var r=n(0),i=n(1),o=n.n(i),a=o.a.Model,s=o.a.View;e.a={types:[],initialize:function(t,e){var n=this;this.model=function(){var t,e,r,i=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},o=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(i&&i.type){var a=n.getBaseType();t=(r=n.getType(i.type))?r.model:a.model,e=r?r.view:a.view;}else {var s=n.recognizeType(i);t=(r=s.type).model,e=r.view,i=s.attributes;}var l=new t(i,o);return l.typeView=e,l};var r=this.init&&this.init.bind(this);r&&r();},recognizeType:function(t){for(var e=this.getTypes(),n=0;n<e.length;n++){var r=e[n],i=r.isType(t);if(i="boolean"==typeof i&&i?{type:r.id}:i)return {type:r,attributes:i}}return {type:this.getBaseType(),attributes:t}},getBaseType:function(){var t=this.getTypes();return t[t.length-1]},getTypes:function(){return this.types},getType:function(t){for(var e=this.getTypes(),n=0;n<e.length;n++){var r=e[n];if(r.id===t)return r}},addType:function(t,e){var n=this.getType(t),i=this.getBaseType(),o=n?n.model:i.model,l=n?n.view:i.view,c=e.model,u=e.view,h=e.isType;c=c instanceof a||Object(r.isFunction)(c)?c:o.extend(c||{}),u=u instanceof s||Object(r.isFunction)(u)?u:l.extend(u||{}),n?(n.model=c,n.view=u,n.isType=h||n.isType):(e.id=t,e.model=c,e.view=u,e.isType=h||function(e){if(e&&e.type==t)return !0},this.getTypes().unshift(e));}};},function(t,e,n){var r=n(1),i=n.n(r),o=n(0),a=Number.MAX_VALUE;e.a=i.a.Model.extend({initialize:function(){this.compCls=[],this.ids=[];},buildFromModel:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r="",i=this.em,o=i&&i.getConfig("avoidInlineStyle"),a=t.styleToString(),s=t.get("classes"),l=n.wrapperIsBody,c=t.get("wrapper");if(this.ids.push("#".concat(t.getId())),s.each((function(t){return e.compCls.push(t.getFullName())})),!o&&a){var u="#".concat(t.getId());r="".concat(u=l&&c?"body":u,"{").concat(a,"}");}var h=t.components();return h.each((function(t){return r+=e.buildFromModel(t,n)})),r},build:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.cssc,i=n.em||"";this.em=i,this.compCls=[],this.ids=[];var a=this.buildFromModel(t,n),s=Object(o.isUndefined)(n.clearStyles)&&i?i.getConfig("clearStyles"):n.clearStyles;if(r){var l=r.getAll(),c={},u=[];l.each((function(t){var r=t.getAtRule();if(r){var i=c[r];i?i.push(t):c[r]=[t];}else a+=e.buildFromRule(t,u,n);})),this.sortMediaObject(c).forEach((function(t){var r="",i=t.key;t.value.forEach((function(t){var o=e.buildFromRule(t,u,n);t.get("singleAtRule")?a+="".concat(i,"{").concat(o,"}"):r+=o;})),r&&(a+="".concat(i,"{").concat(r,"}"));})),i&&s&&l.remove(u);}return a},buildFromRule:function(t,e){var n,r=this,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},o="",a=t.selectorsToString({skipAdd:1}),s=t.get("selectorsAdd"),l=t.get("singleAtRule");if(t.get("selectors").each((function(t){var e=t.getFullName();(r.compCls.indexOf(e)>=0||r.ids.indexOf(e)>=0||i.keepUnusedStyles)&&(n=1);})),a&&n||s||l){var c=t.getDeclaration();c&&(o+=c);}else e.push(t);return o},getQueryLength:function(t){var e=/(-?\d*\.?\d+)\w{0,}/.exec(t);return e?parseFloat(e[1]):a},sortMediaObject:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=[];return Object(o.each)(e,(function(t,e){return n.push({key:e,value:t})})),n.sort((function(e,n){var r=[e.key,n.key].every((function(t){return -1!==t.indexOf("min-width")})),i=r?e.key:n.key,o=r?n.key:e.key;return t.getQueryLength(i)-t.getQueryLength(o)}))}});},function(t,e,n){var r=n(79);t.exports=function(t,e){if(null==t)return {};var n,i,o=r(t,e);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(t);for(i=0;i<a.length;i++)n=a[i],e.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(t,n)&&(o[n]=t[n]);}return o};},function(t,e,n){var r=n(1),i=n.n(r),o=n(0),a=n(3),s=n(17);i.a.$;e.a=s.a.extend({events:{"change input":"handleChange","change select":"handleUnitChange","click [data-arrow-up]":"upArrowClick","click [data-arrow-down]":"downArrowClick","mousedown [data-arrows]":"downIncrement"},template:function(){var t=this.ppfx;return '\n      <span class="'.concat(t,'input-holder"></span>\n      <span class="').concat(t,'field-units"></span>\n      <div class="').concat(t,'field-arrows" data-arrows>\n        <div class="').concat(t,'field-arrow-u" data-arrow-up></div>\n        <div class="').concat(t,'field-arrow-d" data-arrow-down></div>\n      </div>\n    ')},inputClass:function(){var t=this.ppfx;return this.opts.contClass||"".concat(t,"field ").concat(t,"field-integer")},initialize:function(){s.a.prototype.initialize.apply(this,arguments),Object(o.bindAll)(this,"moveIncrement","upIncrement"),this.doc=document,this.listenTo(this.model,"change:unit",this.handleModelChange);},setValue:function(t,e){var n=e||{},r=this.validateInputValue(t,{deepCheck:1}),i={value:r.value};(r.unit||r.force)&&(i.unit=r.unit),this.model.set(i,n),n.silent&&this.handleModelChange();},handleChange:function(t){t.stopPropagation(),this.setValue(this.getInputEl().value),this.elementUpdated();},handleUnitChange:function(t){t.stopPropagation();var e=this.getUnitEl().value;this.model.set("unit",e),this.elementUpdated();},elementUpdated:function(){this.model.trigger("el:change");},handleModelChange:function(){var t=this.model;this.getInputEl().value=t.get("value");var e=this.getUnitEl();e&&(e.value=t.get("unit")||"");},getUnitEl:function(){if(!this.unitEl){var t=this.model,e=t.get("units")||[];if(e.length){var n=[];e.forEach((function(e){var r=e==t.get("unit")?"selected":"";n.push("<option ".concat(r,">").concat(e,"</option>"));}));var r=document.createElement("div");r.innerHTML='<select class="'.concat(this.ppfx,'input-unit">').concat(n.join(""),"</select>"),this.unitEl=r.firstChild;}}return this.unitEl},upArrowClick:function(){var t=this.model,e=t.get("step"),n=parseInt(t.get("value"),10);n=this.normalizeValue(n+e);var r=this.validateInputValue(n);t.set("value",r.value),this.elementUpdated();},downArrowClick:function(){var t=this.model,e=t.get("step"),n=parseInt(t.get("value"),10),r=this.normalizeValue(n-e),i=this.validateInputValue(r);t.set("value",i.value),this.elementUpdated();},downIncrement:function(t){t.preventDefault(),this.moved=0;var e=this.model.get("value");e=this.normalizeValue(e),this.current={y:t.pageY,val:e},Object(a.t)(this.doc,"mousemove",this.moveIncrement),Object(a.t)(this.doc,"mouseup",this.upIncrement);},moveIncrement:function(t){this.moved=1;var e=this.model,n=e.get("step"),r=this.current,i=this.normalizeValue(r.val+(r.y-t.pageY)*n);return this.prValue=this.validateInputValue(i).value,e.set("value",this.prValue,{avoidStore:1}),!1},upIncrement:function(){var t=this.model,e=t.get("step");if(Object(a.s)(this.doc,"mouseup",this.upIncrement),Object(a.s)(this.doc,"mousemove",this.moveIncrement),this.prValue&&this.moved){var n=this.prValue-e;t.set("value",n,{avoidStore:1}).set("value",n+e),this.elementUpdated();}},normalizeValue:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,n=this.model,r=n.get("step"),i=0;if(isNaN(t))return e;if(t=parseFloat(t),Math.floor(t)!==t){var o=r.toString().split(".")[1];i=o?o.length:0;}return i?parseFloat(t.toFixed(i)):t},validateInputValue:function(t,e){var n=0,r=e||{},i=this.model,a=Object(o.isUndefined)(t)?"":t,s=i.get("units")||[],l=i.get("unit")||s.length&&s[0]||"",c=i.get("max"),u=i.get("min");if(r.deepCheck){var h=i.get("fixedValues")||[];if(a){var d=new RegExp("^"+h.join("|"),"g");if(h.length&&d.test(a))a=a.match(d)[0],l="",n=1;else {var f=a+"";a+="",a=parseFloat(a.replace(",",".")),a=isNaN(a)?"":a;var p=f.replace(a,"");Object(o.indexOf)(s,p)>=0&&(l=p);}}}return Object(o.isUndefined)(c)||""===c||(a=a>c?c:a),Object(o.isUndefined)(u)||""===u||(a=a<u?u:a),{force:n,value:a,unit:l}},render:function(){s.a.prototype.render.call(this),this.unitEl=null;var t=this.getUnitEl();return t&&this.$el.find(".".concat(this.ppfx,"field-units")).get(0).appendChild(t),this}});},function(t,e,n){var r=n(1),i=n.n(r),o=n(33);e.a=i.a.Collection.extend({model:o.a,initialize:function(t,e){var n=this;e&&e.em&&(this.editor=e.em),setTimeout((function(){return n.on("remove",n.onRemove)}));},onRemove:function(t){var e=this.editor;e.stopListening(t),e.get("UndoManager").remove(t);},add:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return "string"==typeof t&&(t=this.editor.get("Parser").parseCss(t)),e.em=this.editor,i.a.Collection.prototype.add.apply(this,[t,e])}});},function(t,e,n){var r=n(0),i=n(1),o=n.n(i),a=n(22),s=n(11),l=window.CSS;e.a=o.a.Model.extend(a.a).extend({defaults:{selectors:{},selectorsAdd:"",style:{},mediaText:"",state:"",stylable:!0,atRuleType:"",singleAtRule:0,important:0},initialize:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.config=t||{};var n=e.em,r=this.config.selectors||[];if(this.em=n,n){var i=n.get("SelectorManager"),o=[];r.forEach((function(t){o.push(i.add(t));})),r=o;}this.set("selectors",new s.a(r));},getAtRule:function(){var t=this.get("atRuleType"),e=this.get("mediaText"),n=t?"@".concat(t):e?"@media":"";return n+(e&&n?" ".concat(e):"")},selectorsToString:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=[],n=this.em,r=this.get("state"),i=this.get("wrapper"),o=this.get("selectorsAdd"),a=i&&n&&n.getConfig("wrapperIsBody"),s={escape:function(t){return l&&l.escape?l.escape(t):t}},c=a?"body":this.get("selectors").getFullString(0,s),u=r&&!t.skipState?":".concat(r):"";return c&&e.push("".concat(c).concat(u)),o&&!t.skipAdd&&e.push(o),e.join(", ")},getDeclaration:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e="",n=this.selectorsToString(),r=this.styleToString(t),i=this.get("singleAtRule");return (n||i)&&r&&(e=i?r:"".concat(n,"{").concat(r,"}")),e},toCSS:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e="",n=this.getAtRule(),r=this.getDeclaration(t);return r&&(e=r),n&&e&&(e="".concat(n,"{").concat(e,"}")),e},toJSON:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var i=o.a.Model.prototype.toJSON.apply(this,e);if(this.em.getConfig("avoidDefaults")){var a=this.defaults;Object(r.forEach)(a,(function(t,e){i[e]===t&&delete i[e];})),Object(r.isEmpty)(i.selectors)&&delete i.selectors,Object(r.isEmpty)(i.style)&&delete i.style;}return i},compare:function(t,e,n){var i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},o=e||"",a=n||"",s=i.selectorsAdd||"",l=i.atRuleType||"";t instanceof Array||t.models||(t=[t]);var c=Object(r.map)(t.models||t,(function(t){return t.get("name")})),u=Object(r.map)(this.get("selectors").models,(function(t){return t.get("name")})),h=!1;if(c.length!==u.length)return h;for(var d=0;d<c.length;d++){for(var f=0,p=0;p<u.length;p++)c[d]===u[p]&&(f=1);if(0===f)return h}return this.get("state")===o&&this.get("mediaText")===a&&this.get("selectorsAdd")===s&&this.get("atRuleType")===l||h}});},function(t,e,n){var r=n(1),i=n.n(r),o=n(0);e.a=i.a.View.extend({initialize:function(t){this.opts=t||{},this.config=t.config||{},this.em=this.config.em;var e=this.collection;this.listenTo(e,"add",this.addTo),this.listenTo(e,"reset",this.resetChildren),this.listenTo(e,"remove",this.removeChildren);},removeChildren:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};t.views.forEach((function(t){if(t){var e=t.childrenView,n=t.scriptContainer;e&&e.stopListening(),n&&n.remove(),t.remove.apply(t);}}));var i=t.components();i.forEach((function(t){return n.removeChildren(t,e,r)}));},addTo:function(t){var e=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},n=this.config.em,r=this.collection.indexOf(t);if(this.addToCollection(t,null,r),n&&!e.temporary){var i=function t(e){n.trigger("component:add",e),e.components().forEach((function(e){return t(e)}));};i(t);}},addToCollection:function(t,e,r){this.compView||(this.compView=n(5).default);for(var i=this.config,a=this.opts,s=this.em,l=e||null,c=i.frameView,u=void 0===c?{}:c,h=u.model&&t.getView(u.model),d=a.componentTypes||s&&s.get("DomComponents").getTypes(),f=t.get("type"),p=this.compView,g=0;g<d.length;g++)if(d[g].id==f){p=d[g].view;break}var v=(h||new p({model:t,config:i,componentTypes:d})).render().el;if(l)l.appendChild(v);else {var m=this.parentEl,y=m.childNodes;if(Object(o.isUndefined)(r))m.appendChild(v);else {var b=y.length==r;b&&r--,b||!y.length?m.appendChild(v):m.insertBefore(v,y[r]);}}return v},resetChildren:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.previousModels,i=void 0===r?[]:r;this.parentEl.innerHTML="",i.forEach((function(t){return e.removeChildren(t,e.collection)})),t.each((function(t){return e.addToCollection(t)}));},render:function(t){var e=this,n=this.el,r=document.createDocumentFragment();return this.parentEl=t||this.el,this.collection.each((function(t){return e.addToCollection(t,r)})),n.innerHTML="",n.appendChild(r),this}});},function(t,e,n){var r,i=n(0),o=n(3),a=n(1),s=n.n(a),l=n(5),c=n(4),u=s.a.$;e.a=s.a.View.extend({events:{"mousedown [data-toggle-move]":"startSort","touchstart [data-toggle-move]":"startSort","click [data-toggle-visible]":"toggleVisibility","click [data-toggle-select]":"handleSelect","mouseover [data-toggle-select]":"handleHover","click [data-toggle-open]":"toggleOpening","dblclick [data-name]":"handleEdit","focusout [data-name]":"handleEditEnd"},template:function(t){var e=this.pfx,n=this.ppfx,r=this.config,i=this.clsNoEdit,o=r.hidable,a=this.countChildren(t),s=a?"":this.clsNoChild,l="".concat(this.clsTitle," ").concat(s),c="".concat(this.clsTitleC," ").concat(n,"one-bg"),u="".concat(this.clsCaret," fa fa-chevron-right"),h="".concat(this.inputNameCls," ").concat(i," ").concat(n,"no-app"),d=this.level+1,f="".concat(30+10*d,"px"),p=t.getName(),g=t.getIcon(),v="".concat(e,"layer");return "\n      ".concat(o?'<i class="'.concat(e,"layer-vis fa fa-eye ").concat(this.isVisible()?"":"fa-eye-slash",'" data-toggle-visible></i>'):"",'\n      <div class="').concat(c,'">\n        <div class="').concat(l,'" style="padding-left: ').concat(f,'" data-toggle-select>\n          <div class="').concat(e,'layer-title-inn">\n            <i class="').concat(u,'" data-toggle-open></i>\n            ').concat(g?'<span class="'.concat(v,'__icon">').concat(g,"</span>"):"",'\n            <span class="').concat(h,'" data-name>').concat(p,'</span>\n          </div>\n        </div>\n      </div>\n      <div class="').concat(this.clsCount,'" data-count>').concat(a||"",'</div>\n      <div class="').concat(this.clsMove,'" data-toggle-move>\n        <i class="fa fa-arrows"></i>\n      </div>\n      <div class="').concat(this.clsChildren,'"></div>')},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.opt=t,this.level=t.level,this.config=t.config,this.em=t.config.em,this.ppfx=this.em.get("Config").stylePrefix,this.sorter=t.sorter||"",this.pfx=this.config.stylePrefix,this.parentView=t.parentView;var e=this.pfx,n=this.ppfx,r=this.model,i=r.get("components"),o=r.get("type")||"default";r.set("open",!1),this.listenTo(i,"remove add reset",this.checkChildren),this.listenTo(r,"change:status",this.updateStatus),this.listenTo(r,"change:open",this.updateOpening),this.listenTo(r,"change:layerable",this.updateLayerable),this.listenTo(r,"change:style:display",this.updateVisibility),this.className="".concat(e,"layer ").concat(e,"layer__t-").concat(o," no-select ").concat(n,"two-color"),this.inputNameCls="".concat(n,"layer-name"),this.clsTitleC="".concat(e,"layer-title-c"),this.clsTitle="".concat(e,"layer-title"),this.clsCaret="".concat(e,"layer-caret"),this.clsCount="".concat(e,"layer-count"),this.clsMove="".concat(e,"layer-move"),this.clsChildren="".concat(e,"layer-children"),this.clsNoChild="".concat(e,"layer-no-chld"),this.clsEdit="".concat(this.inputNameCls,"--edit"),this.clsNoEdit="".concat(this.inputNameCls,"--no-edit"),this.$el.data("model",r),this.$el.data("collection",i),r.viewLayer=this;},getVisibilityEl:function(){return this.eyeEl||(this.eyeEl=this.$el.children(".".concat(this.pfx,"layer-vis"))),this.eyeEl},updateVisibility:function(){var t=this.pfx,e=this.model,n="".concat(t,"layer-hidden"),r="none"==e.getStyle().display?"addClass":"removeClass";this.$el[r](n),this.getVisibilityEl()[r]("fa-eye-slash");},toggleVisibility:function(t){t&&t.stopPropagation();var e=this.model,n=e.get("__prev-display"),r=e.getStyle(),i=r.display;"none"==i?(delete r.display,n&&(r.display=n,e.unset("__prev-display"))):(i&&e.set("__prev-display",i),r.display="none"),e.setStyle(r);},handleEdit:function(t){t&&t.stopPropagation();var e=this.em,n=this.$el,r=this.clsNoEdit,i=this.clsEdit,o=this.getInputName();o.contentEditable=!0,o.focus(),e&&e.setEditing(1),n.find(".".concat(this.inputNameCls)).removeClass(r).addClass(i);},handleEditEnd:function(t){t&&t.stopPropagation();var e=this.em,n=this.$el,r=this.clsNoEdit,i=this.clsEdit,o=this.getInputName(),a=o.textContent;o.scrollLeft=0,o.contentEditable=!1,this.model.set({"custom-name":a}),e&&e.setEditing(0),n.find(".".concat(this.inputNameCls)).addClass(r).removeClass(i);},getInputName:function(){return this.inputName||(this.inputName=this.el.querySelector(".".concat(this.inputNameCls))),this.inputName},updateOpening:function(){var t=this.opt.opened||{},e=this.model;e.get("open")?(this.$el.addClass("open"),this.getCaret().addClass("fa-chevron-down"),t[e.cid]=e):(this.$el.removeClass("open"),this.getCaret().removeClass("fa-chevron-down"),delete t[e.cid]);},toggleOpening:function(t){t.stopPropagation(),this.model.get("components").length&&this.model.set("open",!this.model.get("open"));},handleSelect:function(t){t.stopPropagation();var e=this.em,n=this.config;if(e){var r=this.model;e.setSelected(r,{fromLayers:1});var i=n.scrollCanvas;i&&r.views.forEach((function(t){return t.scrollIntoView(i)}));}},handleHover:function(t){t.stopPropagation();var e=this.em,n=this.config,r=this.model;e&&n.showHover&&e.setHovered(r,{fromLayers:1});},startSort:function(t){t.stopPropagation();var e=this.em,n=this.sorter;t.button&&0!==t.button||n&&(n.onStart=function(t){return e.trigger("".concat(c.eventDrag,":start"),t)},n.onMoveClb=function(t){return e.trigger(c.eventDrag,t)},n.startSort(t.target));},freeze:function(){this.$el.addClass(this.pfx+"opac50"),this.model.set("open",0);},unfreeze:function(){this.$el.removeClass(this.pfx+"opac50");},updateStatus:function(t){l.default.prototype.updateStatus.apply(this,[{avoidHover:!this.config.highlightHover}]);},isVisible:function(){var t=this.model.get("style").display;if(!t||"none"!=t)return 1},checkChildren:function(){var t=this.model,e=this.clsNoChild,n=this.countChildren(t),r=this.$el.children(".".concat(this.clsTitleC)).children(".".concat(this.clsTitle)),i=this.cnt;i||(i=this.$el.children("[data-count]").get(0),this.cnt=i),r[n?"removeClass":"addClass"](e),i&&(i.innerHTML=n||""),!n&&t.set("open",0);},countChildren:function(t){var e=0;return t.get("components").each((function(t){var n=this.opt.isCountable,r=this.config.hideTextnode;n&&!n(t,r)||e++;}),this),e},getCaret:function(){if(!this.caret||!this.caret.length){this.pfx;this.caret=this.$el.children(".".concat(this.clsTitleC)).find(".".concat(this.clsCaret));}return this.caret},setRoot:function(t){t=Object(i.isString)(t)?this.em.getWrapper().find(t)[0]:t;var e=Object(o.g)(t,u);e&&(this.stopListening(),this.model=e,this.initialize(this.opt),this.render());},updateLayerable:function(){(this.parentView||this).render();},render:function(){var t=this.model,e=this.config,o=this.pfx,a=this.ppfx,s=this.opt.isCountable,l=s&&!s(t,e.hideTextnode),c=this.isVisible(),u=this.$el.empty(),h=this.level+1;Object(i.isUndefined)(r)&&(r=n(87).default);var d=new r({collection:t.get("components"),config:this.config,sorter:this.sorter,opened:this.opt.opened,parentView:this,parent:t,level:h}).render().$el;return this.config.showWrapper||1!==h?(u.html(this.template(t)),u.find(".".concat(this.clsChildren)).append(d)):u.append(d),t.get("draggable")&&this.config.sortable||u.children(".".concat(this.clsMove)).remove(),!c&&(this.className+=" ".concat(o,"hide")),l&&(this.className+=" ".concat(a,"hidden")),u.attr("class",this.className),this.updateOpening(),this.updateStatus(),this.updateVisibility(),this}});},function(t,e,n){var r=n(1),i=n.n(r),o=i.a.View.extend({tagName:"style",initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.config=t.config||{};var e=this.model,n="change:style change:state change:mediaText";this.listenTo(e,n,this.render),this.listenTo(e,"destroy remove",this.remove),this.listenTo(e.get("selectors"),"change",this.render);},render:function(){var t=this.model.get("important");return this.el.innerHTML=this.model.toCSS({important:t}),this}}),a=o.extend({_createElement:function(t){return document.createTextNode("")},render:function(){var t=this.model,e=t.get("important");return this.el.textContent=t.getDeclaration({important:e}),this}}),s=i.a.$,l=function(t,e){return "".concat(t).concat(e?"-".concat(parseFloat(e)):"")};e.a=i.a.View.extend({initialize:function(t){var e=t.config||{};this.atRules={},this.config=e,this.em=e.em,this.pfx=e.stylePrefix||"",this.className=this.pfx+"rules";var n=this.collection;this.listenTo(n,"add",this.addTo),this.listenTo(n,"reset",this.render);},addTo:function(t){this.addToCollection(t);},addToCollection:function(t,e){if(this.renderStarted){var n,r,i=e||null,s={model:t,config:this.config};if("keyframes"===t.get("atRuleType")){var c=t.getAtRule(),u=this.atRules[c];if(!u){var h=document.createElement("style");u=document.createTextNode(""),h.appendChild(document.createTextNode("".concat(c,"{"))),h.appendChild(u),h.appendChild(document.createTextNode("}")),this.atRules[c]=u,n=h;}r=new a(s),u.appendData(r.render().el.textContent);}else n=(r=new o(s)).render().el;var d=this.className,f=t.get("mediaText"),p=l(d),g=p;if(f&&(g=l(d,this.getMediaWidth(f))),n){var v,m=i||this.el;try{v=m.querySelector("#".concat(g));}catch(t){}v||(v=m.querySelector("#".concat(p))),v.appendChild(n);}return n}},getMediaWidth:function(t){return t&&t.replace("(".concat(this.em.getConfig("mediaCondition"),": "),"").replace(")","")},render:function(){var t=this;this.renderStarted=1,this.atRules={};var e=this.em,n=this.$el,r=this.className,i=this.collection,o=document.createDocumentFragment();n.empty();var a=e.get("DeviceManager").getAll().pluck("priority");return a.every((function(t){return t}))&&a.unshift(0),a.forEach((function(t){return s('<div id="'.concat(l(r,t),'"></div>')).appendTo(o)})),i.each((function(e){return t.addToCollection(e,o)})),n.append(o),n.attr("class",r),this}});},function(t,e,n){var r=n(2),i=n.n(r),o=n(1),a=n.n(o),s=n(0),l=n(8),c=n.n(l),u=n(17);function h(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var d=a.a.$;!function(t,e){var n={beforeShow:h,move:h,change:h,show:h,hide:h,color:!1,flat:!1,showInput:!1,allowEmpty:!1,showButtons:!0,clickoutFiresChange:!0,showInitial:!1,showPalette:!1,showPaletteOnly:!1,hideAfterPaletteSelect:!1,togglePaletteOnly:!1,showSelectionPalette:!0,localStorageKey:!1,appendTo:"body",maxSelectionSize:7,cancelText:"cancel",chooseText:"choose",togglePaletteMoreText:"more",togglePaletteLessText:"less",clearText:"Clear Color Selection",noColorSelectedText:"No Color Selected",preferredFormat:!1,className:"",containerClassName:"",replacerClassName:"",showAlpha:!1,theme:"sp-light",palette:[["#ffffff","#000000","#ff0000","#ff8000","#ffff00","#008000","#0000ff","#4b0082","#9400d3"]],selectionPalette:[],disabled:!1,offset:null},r=[],i=!!/msie/i.exec(window.navigator.userAgent),o=function(){function t(t,e){return !!~(""+t).indexOf(e)}var e=document.createElement("div").style;return e.cssText="background-color:rgba(0,0,0,.5)",t(e.backgroundColor,"rgba")||t(e.backgroundColor,"hsla")}(),a=["<div class='sp-replacer'>","<div class='sp-preview'><div class='sp-preview-inner'></div></div>","<div class='sp-dd'>&#9660;</div>","</div>"].join(""),s=function(){var t="";if(i)for(var e=1;e<=6;e++)t+="<div class='sp-"+e+"'></div>";return ["<div class='sp-container sp-hidden'>","<div class='sp-palette-container'>","<div class='sp-palette sp-thumb sp-cf'></div>","<div class='sp-palette-button-container sp-cf'>","<button type='button' class='sp-palette-toggle'></button>","</div>","</div>","<div class='sp-picker-container'>","<div class='sp-top sp-cf'>","<div class='sp-fill'></div>","<div class='sp-top-inner'>","<div class='sp-color'>","<div class='sp-sat'>","<div class='sp-val'>","<div class='sp-dragger'></div>","</div>","</div>","</div>","<div class='sp-clear sp-clear-display'>","</div>","<div class='sp-hue'>","<div class='sp-slider'></div>",t,"</div>","</div>","<div class='sp-alpha'><div class='sp-alpha-inner'><div class='sp-alpha-handle'></div></div></div>","</div>","<div class='sp-input-container sp-cf'>","<input class='sp-input' type='text' spellcheck='false'  />","</div>","<div class='sp-initial sp-thumb sp-cf'></div>","<div class='sp-button-container sp-cf'>","<a class='sp-cancel' href='#'></a>","<button type='button' class='sp-choose'></button>","</div>","</div>","</div>"].join("")}();function l(e,n,r,i){for(var a=[],s=0;s<e.length;s++){var l=e[s];if(l){var c=k(l),u=c.toHsl().l<.5?"sp-thumb-el sp-thumb-dark":"sp-thumb-el sp-thumb-light";u+=k.equals(n,l)?" sp-thumb-active":"";var h=c.toString(i.preferredFormat||"rgb"),d=o?"background-color:"+c.toRgbString():"filter:"+c.toFilter();a.push('<span title="'+h+'" data-color="'+c.toRgbString()+'" class="'+u+'"><span class="sp-thumb-inner" style="'+d+';"></span></span>');}else {a.push(t("<div />").append(t('<span data-color="" style="background-color:transparent;" class="sp-clear-display"></span>').attr("title",i.noColorSelectedText)).html());}}return "<div class='sp-cf "+r+"'>"+a.join("")+"</div>"}function u(c,u){var h,v,y,b=function(e,r){var i=t.extend({},n,e);return i.callbacks={move:f(i.move,r),change:f(i.change,r),show:f(i.show,r),hide:f(i.hide,r),beforeShow:f(i.beforeShow,r)},i}(u,c),w=b.flat,x=b.showSelectionPalette,O=b.localStorageKey,C=b.theme,S=b.callbacks,T=(h=Bt,v=10,function(){var t=this,e=arguments,n=function(){y=null,h.apply(t,e);};y||(y=setTimeout(n,v));}),j=!1,P=!1,E=0,M=0,D=0,A=0,L=0,N=0,I=0,F=0,_=0,V=0,R=1,H=[],z=[],$={},W=b.selectionPalette.slice(0),B=b.maxSelectionSize,U="sp-dragging",q=null,G=c.ownerDocument,K=(G.body,t(c)),Y=!1,X=t(s,G).addClass(C),J=X.find(".sp-picker-container"),Z=X.find(".sp-color"),Q=X.find(".sp-dragger"),tt=X.find(".sp-hue"),et=X.find(".sp-slider"),nt=X.find(".sp-alpha-inner"),rt=X.find(".sp-alpha"),it=X.find(".sp-alpha-handle"),ot=X.find(".sp-input"),at=X.find(".sp-palette"),st=X.find(".sp-initial"),lt=X.find(".sp-cancel"),ct=X.find(".sp-clear"),ut=X.find(".sp-choose"),ht=X.find(".sp-palette-toggle"),dt=K.is("input"),ft=dt&&"color"===K.attr("type")&&g(),pt=dt&&!w,gt=pt?t(a).addClass(C).addClass(b.className).addClass(b.replacerClassName):t([]),vt=pt?gt:K,mt=gt.find(".sp-preview-inner"),yt=b.color||dt&&K.val(),bt=!1,wt=b.preferredFormat,xt=!b.showButtons||b.clickoutFiresChange,Ot=!yt,Ct=b.allowEmpty&&!ft;function St(){if(b.showPaletteOnly&&(b.showPalette=!0),ht.text(b.showPaletteOnly?b.togglePaletteMoreText:b.togglePaletteLessText),b.palette){H=b.palette.slice(0),z=t.isArray(H[0])?H:[H],$={};for(var e=0;e<z.length;e++)for(var n=0;n<z[e].length;n++){var r=k(z[e][n]).toRgbString();$[r]=!0;}}X.toggleClass("sp-flat",w),X.toggleClass("sp-input-disabled",!b.showInput),X.toggleClass("sp-alpha-enabled",b.showAlpha),X.toggleClass("sp-clear-enabled",Ct),X.toggleClass("sp-buttons-disabled",!b.showButtons),X.toggleClass("sp-palette-buttons-disabled",!b.togglePaletteOnly),X.toggleClass("sp-palette-disabled",!b.showPalette),X.toggleClass("sp-palette-only",b.showPaletteOnly),X.toggleClass("sp-initial-disabled",!b.showInitial),X.addClass(b.className).addClass(b.containerClassName),Bt();}function kt(){if(O&&window.localStorage){try{var e=window.localStorage[O].split(",#");e.length>1&&(delete window.localStorage[O],t.each(e,(function(t,e){Tt(e);})));}catch(t){}try{W=window.localStorage[O].split(";");}catch(t){}}}function Tt(e){if(x){var n=k(e).toRgbString();if(!$[n]&&-1===t.inArray(n,W))for(W.push(n);W.length>B;)W.shift();if(O&&window.localStorage)try{window.localStorage[O]=W.join(";");}catch(t){}}}function jt(){var e=Rt(),n=t.map(z,(function(t,n){return l(t,e,"sp-palette-row sp-palette-row-"+n,b)}));kt(),W&&n.push(l(function(){var t=[];if(b.showPalette)for(var e=0;e<W.length;e++){var n=k(W[e]).toRgbString();$[n]||t.push(W[e]);}return t.reverse().slice(0,b.maxSelectionSize)}(),e,"sp-palette-row sp-palette-row-selection",b)),at.html(n.join(""));}function Pt(){if(b.showInitial){var t=bt,e=Rt();st.html(l([t,e],e,"sp-palette-row-initial",b));}}function Et(){(M<=0||E<=0||A<=0)&&Bt(),P=!0,X.addClass(U),q=null,K.trigger("dragstart.spectrum",[Rt()]);}function Mt(){P=!1,X.removeClass(U),K.trigger("dragstop.spectrum",[Rt()]);}function Dt(){var t=ot.val();if(null!==t&&""!==t||!Ct){var e=k(t);e.isValid()?(Vt(e),Wt(!0)):ot.addClass("sp-validation-error");}else Vt(null),Wt(!0);}function At(){j?Ft():Lt();}function Lt(){var e=t.Event("beforeShow.spectrum");if(j)Bt();else if(K.trigger("beforeShow.spectrum",[Rt()]),!1!==S.beforeShow(Rt())&&!e.isDefaultPrevented()){!function(){for(var t=0;t<r.length;t++)r[t]&&r[t].hide();}(),j=!0;var n=t(G);n.bind("keydown.spectrum",Nt),n.bind("click.spectrum",It),t(window).bind("resize.spectrum",T),gt.addClass("sp-active"),X.removeClass("sp-hidden"),Bt(),zt(),bt=Rt(),Pt(),S.show(bt),K.trigger("show.spectrum",[bt]);}}function Nt(t){27===t.keyCode&&Ft();}function It(t){2!=t.button&&(P||(xt?Wt(!0):_t(),Ft()));}function Ft(){j&&!w&&(j=!1,t(G).unbind("keydown.spectrum",Nt),t(G).unbind("click.spectrum",It),t(window).unbind("resize.spectrum",T),gt.removeClass("sp-active"),X.addClass("sp-hidden"),S.hide(Rt()),K.trigger("hide.spectrum",[Rt()]));}function _t(){Vt(bt,!0);}function Vt(t,e){var n,r;k.equals(t,Rt())?zt():(!t&&Ct?Ot=!0:(Ot=!1,r=(n=k(t)).toHsv(),F=r.h%360/360,_=r.s,V=r.v,R=r.a),zt(),n&&n.isValid()&&!e&&(wt=b.preferredFormat||n.getFormat()));}function Rt(t){return t=t||{},Ct&&Ot?null:k.fromRatio({h:F,s:_,v:V,a:Math.round(100*R)/100},{format:t.format||wt})}function Ht(){zt(),S.move(Rt()),K.trigger("move.spectrum",[Rt()]);}function zt(){ot.removeClass("sp-validation-error"),$t();var t=k.fromRatio({h:F,s:1,v:1});Z.css("background-color",t.toHexString());var e=wt;R<1&&(0!==R||"name"!==e)&&("hex"!==e&&"hex3"!==e&&"hex6"!==e&&"name"!==e||(e="rgb"));var n=Rt({format:e}),r="";if(mt.removeClass("sp-clear-display"),mt.css("background-color","transparent"),!n&&Ct)mt.addClass("sp-clear-display");else {var a=n.toHexString(),s=n.toRgbString();if(o||1===n.alpha?mt.css("background-color",s):(mt.css("background-color","transparent"),mt.css("filter",n.toFilter())),b.showAlpha){var l=n.toRgb();l.a=0;var c=k(l).toRgbString(),u="linear-gradient(left, "+c+", "+a+")";i?nt.css("filter",k(c).toFilter({gradientType:1},a)):(nt.css("background","-webkit-"+u),nt.css("background","-moz-"+u),nt.css("background","-ms-"+u),nt.css("background","linear-gradient(to right, "+c+", "+a+")"));}r=n.toString(e);}b.showInput&&ot.val(r),b.showPalette&&jt(),Pt();}function $t(){var t=_,e=V;if(Ct&&Ot)it.hide(),et.hide(),Q.hide();else {it.show(),et.show(),Q.show();var n=t*E,r=M-e*M;n=Math.max(-D,Math.min(E-D,n-D)),r=Math.max(-D,Math.min(M-D,r-D)),Q.css({top:r+"px",left:n+"px"});var i=R*L;it.css({left:i-N/2+"px"});var o=F*A;et.css({top:o-I+"px"});}}function Wt(t){var e=Rt(),n="",r=!k.equals(e,bt);e&&(n=e.toString(wt),Tt(e)),dt&&K.val(n),t&&r&&(S.change(e),K.trigger("change",[e]));}function Bt(){j&&(E=Z.width(),M=Z.height(),D=Q.height(),tt.width(),A=tt.height(),I=et.height(),L=rt.width(),N=it.width(),w||(X.css("position","absolute"),b.offset?X.offset(b.offset):X.offset(function(e,n){var r=e.outerWidth(),i=e.outerHeight(),o=n.outerHeight(),a=e[0].ownerDocument,s=a.documentElement,l=s.clientWidth,c=s.clientHeight,u=t(a).scrollLeft(),h=t(a).scrollTop(),d=l+u,f=c+h,p=n.offset();return p.top+=o,p.left-=Math.min(p.left,p.left+r>d&&d>r?Math.abs(p.left+r-d):0),p.top-=Math.min(p.top,p.top+i>f&&f>i?Math.abs(i+o-0):0),p}(X,vt))),$t(),b.showPalette&&jt(),K.trigger("reflow.spectrum"));}function Ut(){Ft(),Y=!0,K.attr("disabled",!0),vt.addClass("sp-disabled");}!function(){if(i&&X.find("*:not(input)").attr("unselectable","on"),St(),pt&&K.after(gt).hide(),Ct||ct.hide(),w)K.after(X).hide();else {var e="parent"===b.appendTo?K.parent():t(b.appendTo);1!==e.length&&(e=t("body")),e.append(X);}function n(e){return e.data&&e.data.ignore?(Vt(t(e.target).closest(".sp-thumb-el").data("color")),Ht()):(Vt(t(e.target).closest(".sp-thumb-el").data("color")),Ht(),Wt(!0),b.hideAfterPaletteSelect&&Ft()),!1}kt(),vt.bind("click.spectrum touchstart.spectrum",(function(e){Y||At(),e.stopPropagation(),t(e.target).is("input")||e.preventDefault();})),(K.is(":disabled")||!0===b.disabled)&&Ut(),X.click(d),ot.change(Dt),ot.bind("paste",(function(){setTimeout(Dt,1);})),ot.keydown((function(t){13==t.keyCode&&Dt();})),lt.text(b.cancelText),lt.bind("click.spectrum",(function(t){t.stopPropagation(),t.preventDefault(),_t(),Ft();})),ct.attr("title",b.clearText),ct.bind("click.spectrum",(function(t){t.stopPropagation(),t.preventDefault(),Ot=!0,Ht(),w&&Wt(!0);})),ut.text(b.chooseText),ut.bind("click.spectrum",(function(t){t.stopPropagation(),t.preventDefault(),i&&ot.is(":focus")&&ot.trigger("change"),ot.hasClass("sp-validation-error")||(Wt(!0),Ft());})),ht.text(b.showPaletteOnly?b.togglePaletteMoreText:b.togglePaletteLessText),ht.bind("click.spectrum",(function(t){t.stopPropagation(),t.preventDefault(),b.showPaletteOnly=!b.showPaletteOnly,b.showPaletteOnly||w||X.css("left","-="+(J.outerWidth(!0)+5)),St();})),p(rt,(function(t,e,n){R=t/L,Ot=!1,n.shiftKey&&(R=Math.round(10*R)/10),Ht();}),Et,Mt),p(tt,(function(t,e){F=parseFloat(e/A),Ot=!1,b.showAlpha||(R=1),Ht();}),Et,Mt),p(Z,(function(t,e,n){if(n.shiftKey){if(!q){var r=_*E,i=M-V*M,o=Math.abs(t-r)>Math.abs(e-i);q=o?"x":"y";}}else q=null;var a=!q||"y"===q;(!q||"x"===q)&&(_=parseFloat(t/E)),a&&(V=parseFloat((M-e)/M)),Ot=!1,b.showAlpha||(R=1),Ht();}),Et,Mt),yt?(Vt(yt),zt(),wt=b.preferredFormat||k(yt).format,Tt(yt)):zt(),w&&Lt();var r=i?"mousedown.spectrum":"click.spectrum touchstart.spectrum";at.delegate(".sp-thumb-el",r,n),st.delegate(".sp-thumb-el:nth-child(1)",r,{ignore:!0},n);}();var qt={show:Lt,hide:Ft,toggle:At,reflow:Bt,option:function(n,r){return n===e?t.extend({},b):r===e?b[n]:(b[n]=r,"preferredFormat"===n&&(wt=b.preferredFormat),void St())},enable:function(){Y=!1,K.attr("disabled",!1),vt.removeClass("sp-disabled");},disable:Ut,offset:function(t){b.offset=t,Bt();},set:function(t){Vt(t),Wt();},get:Rt,destroy:function(){K.show(),vt.unbind("click.spectrum touchstart.spectrum"),X.remove(),gt.remove(),r[qt.id]=null;},container:X};return qt.id=r.push(qt)-1,qt}function h(){}function d(t){t.stopPropagation();}function f(t,e){var n=Array.prototype.slice,r=n.call(arguments,2);return function(){return t.apply(e,r.concat(n.call(arguments)))}}function p(e,n,r,o){n=n||function(){},r=r||function(){},o=o||function(){};var a=document,s=!1,l={},c=0,u=0,h="ontouchstart"in window,d={};function f(t){t.stopPropagation&&t.stopPropagation(),t.preventDefault&&t.preventDefault(),t.returnValue=!1;}function p(t){if(s){if(i&&a.documentMode<9&&!t.button)return g();var r=t&&t.touches&&t.touches[0],o=r&&r.pageX||t.pageX,d=r&&r.pageY||t.pageY,p=Math.max(0,Math.min(o-l.left,u)),v=Math.max(0,Math.min(d-l.top,c));h&&f(t),n.apply(e,[p,v,t]);}}function g(){s&&(t(a).unbind(d),t(a.body).removeClass("sp-dragging"),setTimeout((function(){o.apply(e,arguments);}),0)),s=!1;}d.selectstart=f,d.dragstart=f,d["touchmove mousemove"]=p,d["touchend mouseup"]=g,t(e).bind("touchstart mousedown",(function(n){(n.which?3==n.which:2==n.button)||s||!1!==r.apply(e,arguments)&&(s=!0,c=t(e).height(),u=t(e).width(),l=t(e).offset(),t(a).bind(d),t(a.body).addClass("sp-dragging"),p(n),f(n));}));}function g(){return t.fn.spectrum.inputTypeColorSupport()}var v="spectrum.id";t.fn.spectrum=function(e,n){if("string"==typeof e){var i=this,o=Array.prototype.slice.call(arguments,1);return this.each((function(){var n=r[t(this).data(v)];if(n){var a=n[e];if(!a)throw new Error("Spectrum: no such method: '"+e+"'");"get"==e?i=n.get():"container"==e?i=n.container:"option"==e?i=n.option.apply(n,o):"destroy"==e?(n.destroy(),t(this).removeData(v)):a.apply(n,o);}})),i}return this.spectrum("destroy").each((function(){var n=u(this,t.extend({},e,t(this).data()));t(this).data(v,n.id);}))},t.fn.spectrum.load=!0,t.fn.spectrum.loadOpts={},t.fn.spectrum.draggable=p,t.fn.spectrum.defaults=n,t.fn.spectrum.inputTypeColorSupport=function e(){if(void 0===e._cachedResult){var n=t("<input type='color'/>")[0];e._cachedResult="color"===n.type&&""!==n.value;}return e._cachedResult},t.spectrum={},t.spectrum.localization={},t.spectrum.palettes={},t.fn.spectrum.processNativeColorInputs=function(){var e=t("input[type=color]");e.length&&!g()&&e.spectrum({preferredFormat:"hex6"});};var m=/^[\s,#]+/,y=/\s+$/,b=0,w=Math,x=w.round,O=w.min,C=w.max,S=w.random,k=function t(e,n){if(n=n||{},(e=e||"")instanceof t)return e;if(!(this instanceof t))return new t(e,n);var r=function(t){var e={r:0,g:0,b:0},n=1,r=!1,i=!1;"string"==typeof t&&(t=function(t){t=t.replace(m,"").replace(y,"").toLowerCase();var e,n=!1;if(W[t])t=W[t],n=!0;else if("transparent"==t)return {r:0,g:0,b:0,a:0,format:"name"};if(e=tt.rgb.exec(t))return {r:e[1],g:e[2],b:e[3]};if(e=tt.rgba.exec(t))return {r:e[1],g:e[2],b:e[3],a:e[4]};if(e=tt.hsl.exec(t))return {h:e[1],s:e[2],l:e[3]};if(e=tt.hsla.exec(t))return {h:e[1],s:e[2],l:e[3],a:e[4]};if(e=tt.hsv.exec(t))return {h:e[1],s:e[2],v:e[3]};if(e=tt.hsva.exec(t))return {h:e[1],s:e[2],v:e[3],a:e[4]};if(e=tt.hex8.exec(t))return {a:(r=e[1],K(r)/255),r:K(e[2]),g:K(e[3]),b:K(e[4]),format:n?"name":"hex8"};var r;if(e=tt.hex6.exec(t))return {r:K(e[1]),g:K(e[2]),b:K(e[3]),format:n?"name":"hex"};if(e=tt.hex3.exec(t))return {r:K(e[1]+""+e[1]),g:K(e[2]+""+e[2]),b:K(e[3]+""+e[3]),format:n?"name":"hex"};return !1}(t));"object"==c()(t)&&(t.hasOwnProperty("r")&&t.hasOwnProperty("g")&&t.hasOwnProperty("b")?(o=t.r,a=t.g,s=t.b,e={r:255*q(o,255),g:255*q(a,255),b:255*q(s,255)},r=!0,i="%"===String(t.r).substr(-1)?"prgb":"rgb"):t.hasOwnProperty("h")&&t.hasOwnProperty("s")&&t.hasOwnProperty("v")?(t.s=X(t.s),t.v=X(t.v),e=function(t,e,n){t=6*q(t,360),e=q(e,100),n=q(n,100);var r=w.floor(t),i=t-r,o=n*(1-e),a=n*(1-i*e),s=n*(1-(1-i)*e),l=r%6;return {r:255*[n,a,o,o,s,n][l],g:255*[s,n,n,a,o,o][l],b:255*[o,o,s,n,n,a][l]}}(t.h,t.s,t.v),r=!0,i="hsv"):t.hasOwnProperty("h")&&t.hasOwnProperty("s")&&t.hasOwnProperty("l")&&(t.s=X(t.s),t.l=X(t.l),e=function(t,e,n){var r,i,o;function a(t,e,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?t+6*(e-t)*n:n<.5?e:n<2/3?t+(e-t)*(2/3-n)*6:t}if(t=q(t,360),e=q(e,100),n=q(n,100),0===e)r=i=o=n;else {var s=n<.5?n*(1+e):n+e-n*e,l=2*n-s;r=a(l,s,t+1/3),i=a(l,s,t),o=a(l,s,t-1/3);}return {r:255*r,g:255*i,b:255*o}}(t.h,t.s,t.l),r=!0,i="hsl"),t.hasOwnProperty("a")&&(n=t.a));var o,a,s;return n=U(n),{ok:r,format:t.format||i,r:O(255,C(e.r,0)),g:O(255,C(e.g,0)),b:O(255,C(e.b,0)),a:n}}(e);this._originalInput=e,this._r=r.r,this._g=r.g,this._b=r.b,this._a=r.a,this._roundA=x(100*this._a)/100,this._format=n.format||r.format,this._gradientType=n.gradientType,this._r<1&&(this._r=x(this._r)),this._g<1&&(this._g=x(this._g)),this._b<1&&(this._b=x(this._b)),this._ok=r.ok,this._tc_id=b++;};function T(t,e,n){t=q(t,255),e=q(e,255),n=q(n,255);var r,i,o=C(t,e,n),a=O(t,e,n),s=(o+a)/2;if(o==a)r=i=0;else {var l=o-a;switch(i=s>.5?l/(2-o-a):l/(o+a),o){case t:r=(e-n)/l+(e<n?6:0);break;case e:r=(n-t)/l+2;break;case n:r=(t-e)/l+4;}r/=6;}return {h:r,s:i,l:s}}function j(t,e,n){t=q(t,255),e=q(e,255),n=q(n,255);var r,i,o=C(t,e,n),a=O(t,e,n),s=o,l=o-a;if(i=0===o?0:l/o,o==a)r=0;else {switch(o){case t:r=(e-n)/l+(e<n?6:0);break;case e:r=(n-t)/l+2;break;case n:r=(t-e)/l+4;}r/=6;}return {h:r,s:i,v:s}}function P(t,e,n,r){var i=[Y(x(t).toString(16)),Y(x(e).toString(16)),Y(x(n).toString(16))];return r&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)?i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0):i.join("")}function E(t,e,n,r){var i;return [Y((i=r,Math.round(255*parseFloat(i)).toString(16))),Y(x(t).toString(16)),Y(x(e).toString(16)),Y(x(n).toString(16))].join("")}function M(t,e){e=0===e?0:e||10;var n=k(t).toHsl();return n.s-=e/100,n.s=G(n.s),k(n)}function D(t,e){e=0===e?0:e||10;var n=k(t).toHsl();return n.s+=e/100,n.s=G(n.s),k(n)}function A(t){return k(t).desaturate(100)}function L(t,e){e=0===e?0:e||10;var n=k(t).toHsl();return n.l+=e/100,n.l=G(n.l),k(n)}function N(t,e){e=0===e?0:e||10;var n=k(t).toRgb();return n.r=C(0,O(255,n.r-x(-e/100*255))),n.g=C(0,O(255,n.g-x(-e/100*255))),n.b=C(0,O(255,n.b-x(-e/100*255))),k(n)}function I(t,e){e=0===e?0:e||10;var n=k(t).toHsl();return n.l-=e/100,n.l=G(n.l),k(n)}function F(t,e){var n=k(t).toHsl(),r=(x(n.h)+e)%360;return n.h=r<0?360+r:r,k(n)}function _(t){var e=k(t).toHsl();return e.h=(e.h+180)%360,k(e)}function V(t){var e=k(t).toHsl(),n=e.h;return [k(t),k({h:(n+120)%360,s:e.s,l:e.l}),k({h:(n+240)%360,s:e.s,l:e.l})]}function R(t){var e=k(t).toHsl(),n=e.h;return [k(t),k({h:(n+90)%360,s:e.s,l:e.l}),k({h:(n+180)%360,s:e.s,l:e.l}),k({h:(n+270)%360,s:e.s,l:e.l})]}function H(t){var e=k(t).toHsl(),n=e.h;return [k(t),k({h:(n+72)%360,s:e.s,l:e.l}),k({h:(n+216)%360,s:e.s,l:e.l})]}function z(t,e,n){e=e||6,n=n||30;var r=k(t).toHsl(),i=360/n,o=[k(t)];for(r.h=(r.h-(i*e>>1)+720)%360;--e;)r.h=(r.h+i)%360,o.push(k(r));return o}function $(t,e){e=e||6;for(var n=k(t).toHsv(),r=n.h,i=n.s,o=n.v,a=[],s=1/e;e--;)a.push(k({h:r,s:i,v:o})),o=(o+s)%1;return a}k.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return !this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var t=this.toRgb();return (299*t.r+587*t.g+114*t.b)/1e3},setAlpha:function(t){return this._a=U(t),this._roundA=x(100*this._a)/100,this},toHsv:function(){var t=j(this._r,this._g,this._b);return {h:360*t.h,s:t.s,v:t.v,a:this._a}},toHsvString:function(){var t=j(this._r,this._g,this._b),e=x(360*t.h),n=x(100*t.s),r=x(100*t.v);return 1==this._a?"hsv("+e+", "+n+"%, "+r+"%)":"hsva("+e+", "+n+"%, "+r+"%, "+this._roundA+")"},toHsl:function(){var t=T(this._r,this._g,this._b);return {h:360*t.h,s:t.s,l:t.l,a:this._a}},toHslString:function(){var t=T(this._r,this._g,this._b),e=x(360*t.h),n=x(100*t.s),r=x(100*t.l);return 1==this._a?"hsl("+e+", "+n+"%, "+r+"%)":"hsla("+e+", "+n+"%, "+r+"%, "+this._roundA+")"},toHex:function(t){return P(this._r,this._g,this._b,t)},toHexString:function(t){return "#"+this.toHex(t)},toHex8:function(){return E(this._r,this._g,this._b,this._a)},toHex8String:function(){return "#"+this.toHex8()},toRgb:function(){return {r:x(this._r),g:x(this._g),b:x(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+x(this._r)+", "+x(this._g)+", "+x(this._b)+")":"rgba("+x(this._r)+", "+x(this._g)+", "+x(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return {r:x(100*q(this._r,255))+"%",g:x(100*q(this._g,255))+"%",b:x(100*q(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+x(100*q(this._r,255))+"%, "+x(100*q(this._g,255))+"%, "+x(100*q(this._b,255))+"%)":"rgba("+x(100*q(this._r,255))+"%, "+x(100*q(this._g,255))+"%, "+x(100*q(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(B[P(this._r,this._g,this._b,!0)]||!1)},toFilter:function(t){var e="#"+E(this._r,this._g,this._b,this._a),n=e,r=this._gradientType?"GradientType = 1, ":"";t&&(n=k(t).toHex8String());return "progid:DXImageTransform.Microsoft.gradient("+r+"startColorstr="+e+",endColorstr="+n+")"},toString:function(t){var e=!!t;t=t||this._format;var n=!1,r=this._a<1&&this._a>=0;return e||!r||"hex"!==t&&"hex6"!==t&&"hex3"!==t&&"name"!==t?("rgb"===t&&(n=this.toRgbString()),"prgb"===t&&(n=this.toPercentageRgbString()),"hex"!==t&&"hex6"!==t||(n=this.toHexString()),"hex3"===t&&(n=this.toHexString(!0)),"hex8"===t&&(n=this.toHex8String()),"name"===t&&(n=this.toName()),"hsl"===t&&(n=this.toHslString()),"hsv"===t&&(n=this.toHsvString()),n||this.toHexString()):"name"===t&&0===this._a?this.toName():this.toRgbString()},_applyModification:function(t,e){var n=t.apply(null,[this].concat([].slice.call(e)));return this._r=n._r,this._g=n._g,this._b=n._b,this.setAlpha(n._a),this},lighten:function(){return this._applyModification(L,arguments)},brighten:function(){return this._applyModification(N,arguments)},darken:function(){return this._applyModification(I,arguments)},desaturate:function(){return this._applyModification(M,arguments)},saturate:function(){return this._applyModification(D,arguments)},greyscale:function(){return this._applyModification(A,arguments)},spin:function(){return this._applyModification(F,arguments)},_applyCombination:function(t,e){return t.apply(null,[this].concat([].slice.call(e)))},analogous:function(){return this._applyCombination(z,arguments)},complement:function(){return this._applyCombination(_,arguments)},monochromatic:function(){return this._applyCombination($,arguments)},splitcomplement:function(){return this._applyCombination(H,arguments)},triad:function(){return this._applyCombination(V,arguments)},tetrad:function(){return this._applyCombination(R,arguments)}},k.fromRatio=function(t,e){if("object"==c()(t)){var n={};for(var r in t)t.hasOwnProperty(r)&&(n[r]="a"===r?t[r]:X(t[r]));t=n;}return k(t,e)},k.equals=function(t,e){return !(!t||!e)&&k(t).toRgbString()==k(e).toRgbString()},k.random=function(){return k.fromRatio({r:S(),g:S(),b:S()})},k.mix=function(t,e,n){n=0===n?0:n||50;var r,i=k(t).toRgb(),o=k(e).toRgb(),a=n/100,s=2*a-1,l=o.a-i.a,c=1-(r=((r=s*l==-1?s:(s+l)/(1+s*l))+1)/2),u={r:o.r*r+i.r*c,g:o.g*r+i.g*c,b:o.b*r+i.b*c,a:o.a*a+i.a*(1-a)};return k(u)},k.readability=function(t,e){var n=k(t),r=k(e),i=n.toRgb(),o=r.toRgb(),a=n.getBrightness(),s=r.getBrightness(),l=Math.max(i.r,o.r)-Math.min(i.r,o.r)+Math.max(i.g,o.g)-Math.min(i.g,o.g)+Math.max(i.b,o.b)-Math.min(i.b,o.b);return {brightness:Math.abs(a-s),color:l}},k.isReadable=function(t,e){var n=k.readability(t,e);return n.brightness>125&&n.color>500},k.mostReadable=function(t,e){for(var n=null,r=0,i=!1,o=0;o<e.length;o++){var a=k.readability(t,e[o]),s=a.brightness>125&&a.color>500,l=a.brightness/125*3+a.color/500;(s&&!i||s&&i&&l>r||!s&&!i&&l>r)&&(i=s,r=l,n=k(e[o]));}return n};var W=k.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},B=k.hexNames=function(t){var e={};for(var n in t)t.hasOwnProperty(n)&&(e[t[n]]=n);return e}(W);function U(t){return t=parseFloat(t),(isNaN(t)||t<0||t>1)&&(t=1),t}function q(t,e){(function(t){return "string"==typeof t&&-1!=t.indexOf(".")&&1===parseFloat(t)})(t)&&(t="100%");var n=function(t){return "string"==typeof t&&-1!=t.indexOf("%")}(t);return t=O(e,C(0,parseFloat(t))),n&&(t=parseInt(t*e,10)/100),w.abs(t-e)<1e-6?1:t%e/parseFloat(e)}function G(t){return O(1,C(0,t))}function K(t){return parseInt(t,16)}function Y(t){return 1==t.length?"0"+t:""+t}function X(t){return t<=1&&(t=100*t+"%"),t}var J,Z,Q,tt=(Z="[\\s|\\(]+("+(J="(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)")+")[,|\\s]+("+J+")[,|\\s]+("+J+")\\s*\\)?",Q="[\\s|\\(]+("+J+")[,|\\s]+("+J+")[,|\\s]+("+J+")[,|\\s]+("+J+")\\s*\\)?",{rgb:new RegExp("rgb"+Z),rgba:new RegExp("rgba"+Q),hsl:new RegExp("hsl"+Z),hsla:new RegExp("hsla"+Q),hsv:new RegExp("hsv"+Z),hsva:new RegExp("hsva"+Q),hex3:/^([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex8:/^([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/});window.tinycolor=k,t((function(){t.fn.spectrum.load&&t.fn.spectrum.processNativeColorInputs();}));}(d);e.a=u.a.extend({template:function(){var t=this.ppfx;return '\n      <div class="'.concat(this.holderClass(),'"></div>\n      <div class="').concat(t,'field-colorp">\n        <div class="').concat(t,'field-colorp-c" data-colorp-c>\n          <div class="').concat(t,'checker-bg"></div>\n        </div>\n      </div>\n    ')},inputClass:function(){var t=this.ppfx;return "".concat(t,"field ").concat(t,"field-color")},holderClass:function(){return "".concat(this.ppfx,"input-holder")},setValue:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.model,r=n.get("defaults"),i=Object(s.isUndefined)(t)?Object(s.isUndefined)(r)?"":r:t,o=this.getInputEl(),a=this.getColorEl(),l="none"!=i?i:"";o.value=i,a.get(0).style.backgroundColor=l,(e.fromTarget||e.fromInput&&!e.avoidStore)&&(a.spectrum("set",l),this.noneColor="none"==i);},getColorEl:function(){if(!this.colorEl){var t,e=this.em,n=this,r=this.ppfx,o=this.model,a=d('<div class="'.concat(this.ppfx,'field-color-picker"></div>')),s=a.get(0).style,l=e&&e.config?e.config.el:"",c=e&&e.getConfig&&e.getConfig("colorPicker")||{},u=function(t){return (1==t.getAlpha()?t.toHexString():t.toRgbString()).replace(/ /g,"")},f=0;this.$el.find("[data-colorp-c]").append(a),a.spectrum(function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?h(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):h(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({containerClassName:"".concat(r,"one-bg ").concat(r,"two-color"),appendTo:l||"body",maxSelectionSize:8,showPalette:!0,showAlpha:!0,chooseText:"Ok",cancelText:"⨯",palette:[]},c,{move:function(t){var e=u(t);s.backgroundColor=e,o.setValueFromInput(e,0);},change:function(t){f=1;var e=u(t);s.backgroundColor=e,o.setValueFromInput(0,0),o.setValueFromInput(e),n.noneColor=0;},show:function(e){f=0,t=u(e);},hide:function(e){!f&&t&&(n.noneColor&&(t=""),s.backgroundColor=t,a.spectrum("set",t),o.setValueFromInput(t,0));}})),e&&e.on&&e.on("component:selected",(function(){f=1,a.spectrum("hide");})),this.colorEl=a;}return this.colorEl},render:function(){return u.a.prototype.render.call(this),this.getColorEl(),this}});},function(t,e,n){!function(e){var n,r={},i={16:!1,18:!1,17:!1,91:!1},o="all",a={"⇧":16,shift:16,"⌥":18,alt:18,option:18,"⌃":17,ctrl:17,control:17,"⌘":91,command:91},s={backspace:8,tab:9,clear:12,enter:13,return:13,esc:27,escape:27,space:32,left:37,up:38,right:39,down:40,del:46,delete:46,home:36,end:35,pageup:33,pagedown:34,",":188,".":190,"/":191,"`":192,"-":189,"=":187,";":186,"'":222,"[":219,"]":221,"\\":220},l=function(t){return s[t]||t.toUpperCase().charCodeAt(0)},c=[];for(n=1;n<20;n++)s["f"+n]=111+n;function u(t,e){for(var n=t.length;n--;)if(t[n]===e)return n;return -1}function h(t,e){if(t.length!=e.length)return !1;for(var n=0;n<t.length;n++)if(t[n]!==e[n])return !1;return !0}var d={16:"shiftKey",18:"altKey",17:"ctrlKey",91:"metaKey"};function f(t){for(n in i)i[n]=t[d[n]];}function p(t,e,n){var i,o;i=v(t),void 0===n&&(n=e,e="all");for(var a=0;a<i.length;a++)o=[],(t=i[a].split("+")).length>1&&(o=m(t),t=[t[t.length-1]]),t=t[0],(t=l(t))in r||(r[t]=[]),r[t].push({shortcut:i[a],scope:e,method:n,key:i[a],mods:o});}for(n in a)p[n]=!1;function g(){return o||"all"}function v(t){var e;return ""==(e=(t=t.replace(/\s/g,"")).split(","))[e.length-1]&&(e[e.length-2]+=","),e}function m(t){for(var e=t.slice(0,t.length-1),n=0;n<e.length;n++)e[n]=a[e[n]];return e}function y(t,e,n){t.addEventListener?t.addEventListener(e,n,!1):t.attachEvent&&t.attachEvent("on"+e,(function(){n(window.event);}));}y(document,"keydown",(function(t){!function(t){var e,n,o,s,l,h;if(e=t.keyCode,-1==u(c,e)&&c.push(e),93!=e&&224!=e||(e=91),e in i)for(o in i[e]=!0,a)a[o]==e&&(p[o]=!0);else if(f(t),p.filter.call(this,t)&&e in r)for(h=g(),s=0;s<r[e].length;s++)if((n=r[e][s]).scope==h||"all"==n.scope){for(o in l=n.mods.length>0,i)(!i[o]&&u(n.mods,+o)>-1||i[o]&&-1==u(n.mods,+o))&&(l=!1);(0!=n.mods.length||i[16]||i[18]||i[17]||i[91])&&!l||!1===n.method(t,n)&&(t.preventDefault?t.preventDefault():t.returnValue=!1,t.stopPropagation&&t.stopPropagation(),t.cancelBubble&&(t.cancelBubble=!0));}}(t);})),y(document,"keyup",(function(t){var e,n=t.keyCode,r=u(c,n);if(r>=0&&c.splice(r,1),93!=n&&224!=n||(n=91),n in i)for(e in i[n]=!1,a)a[e]==n&&(p[e]=!1);})),y(window,"focus",(function(){for(n in i)i[n]=!1;for(n in a)p[n]=!1;}));var b=e.key;e.key=p,e.key.setScope=function(t){o=t||"all";},e.key.getScope=g,e.key.deleteScope=function(t){var e,n,i;for(e in r)for(n=r[e],i=0;i<n.length;)n[i].scope===t?n.splice(i,1):i++;},e.key.filter=function(t){var e=(t.target||t.srcElement).tagName;return !("INPUT"==e||"SELECT"==e||"TEXTAREA"==e)},e.key.isPressed=function(t){return "string"==typeof t&&(t=l(t)),-1!=u(c,t)},e.key.getPressedKeyCodes=function(){return c.slice(0)},e.key.noConflict=function(){var t=e.key;return e.key=b,t},e.key.unbind=function(t,e){var n,i,o,a,s,c=[];for(n=v(t),a=0;a<n.length;a++){if((i=n[a].split("+")).length>1&&(c=m(i),t=i[i.length-1]),t=l(t),void 0===e&&(e=g()),!r[t])return;for(o=0;o<r[t].length;o++)(s=r[t][o]).scope===e&&h(s.mods,c)&&(r[t][o]={});}},t.exports=p;}(this);},function(t,e,n){(function(t){var r=n(68),i=setTimeout;function o(t){return Boolean(t&&void 0!==t.length)}function a(){}function s(t){if(!(this instanceof s))throw new TypeError("Promises must be constructed via new");if("function"!=typeof t)throw new TypeError("not a function");this._state=0,this._handled=!1,this._value=void 0,this._deferreds=[],f(t,this);}function l(t,e){for(;3===t._state;)t=t._value;0!==t._state?(t._handled=!0,s._immediateFn((function(){var n=1===t._state?e.onFulfilled:e.onRejected;if(null!==n){var r;try{r=n(t._value);}catch(t){return void u(e.promise,t)}c(e.promise,r);}else (1===t._state?c:u)(e.promise,t._value);}))):t._deferreds.push(e);}function c(t,e){try{if(e===t)throw new TypeError("A promise cannot be resolved with itself.");if(e&&("object"==typeof e||"function"==typeof e)){var n=e.then;if(e instanceof s)return t._state=3,t._value=e,void h(t);if("function"==typeof n)return void f((r=n,i=e,function(){r.apply(i,arguments);}),t)}t._state=1,t._value=e,h(t);}catch(e){u(t,e);}var r,i;}function u(t,e){t._state=2,t._value=e,h(t);}function h(t){2===t._state&&0===t._deferreds.length&&s._immediateFn((function(){t._handled||s._unhandledRejectionFn(t._value);}));for(var e=0,n=t._deferreds.length;e<n;e++)l(t,t._deferreds[e]);t._deferreds=null;}function d(t,e,n){this.onFulfilled="function"==typeof t?t:null,this.onRejected="function"==typeof e?e:null,this.promise=n;}function f(t,e){var n=!1;try{t((function(t){n||(n=!0,c(e,t));}),(function(t){n||(n=!0,u(e,t));}));}catch(t){if(n)return;n=!0,u(e,t);}}s.prototype.catch=function(t){return this.then(null,t)},s.prototype.then=function(t,e){var n=new this.constructor(a);return l(this,new d(t,e,n)),n},s.prototype.finally=r.a,s.all=function(t){return new s((function(e,n){if(!o(t))return n(new TypeError("Promise.all accepts an array"));var r=Array.prototype.slice.call(t);if(0===r.length)return e([]);var i=r.length;function a(t,o){try{if(o&&("object"==typeof o||"function"==typeof o)){var s=o.then;if("function"==typeof s)return void s.call(o,(function(e){a(t,e);}),n)}r[t]=o,0==--i&&e(r);}catch(t){n(t);}}for(var s=0;s<r.length;s++)a(s,r[s]);}))},s.resolve=function(t){return t&&"object"==typeof t&&t.constructor===s?t:new s((function(e){e(t);}))},s.reject=function(t){return new s((function(e,n){n(t);}))},s.race=function(t){return new s((function(e,n){if(!o(t))return n(new TypeError("Promise.race accepts an array"));for(var r=0,i=t.length;r<i;r++)s.resolve(t[r]).then(e,n);}))},s._immediateFn="function"==typeof t&&function(e){t(e);}||function(t){i(t,0);},s._unhandledRejectionFn=function(t){"undefined"!=typeof console&&console&&console.warn("Possible Unhandled Promise Rejection:",t);},e.a=s;}).call(this,n(76).setImmediate);},function(t,e,n){n.r(e);var r=n(1),i=n.n(r),o=i.a.Model.extend({defaults:{id:"",label:"",tagName:"span",className:"",command:"",context:"",buttons:[],attributes:{},options:{},active:!1,dragDrop:!1,togglable:!0,runDefaultCommand:!0,stopDefaultCommand:!1,disable:!1},initialize:function(t){if(this.get("buttons").length){var e=n(40).default;this.set("buttons",new e(this.get("buttons")));}}});e.default=i.a.Collection.extend({model:o,deactivateAllExceptOne:function(t,e){this.forEach((function(n,r){n!==t&&(n.set("active",!1),e&&n.get("buttons").length&&n.get("buttons").deactivateAllExceptOne(t,e));}));},deactivateAll:function(t,e){var n=t||"";this.forEach((function(t){t.get("context")==n&&t!==e&&(t.set("active",!1,{silent:1}),t.trigger("updateActive",{fromCollection:1}));}));},disableAllButtons:function(t){var e=t||"";this.forEach((function(t,n){t.get("context")==e&&t.set("disable",!0);}));},disableAllButtonsExceptOne:function(t,e){this.forEach((function(n,r){n!==t&&(n.set("disable",!0),e&&n.get("buttons").length&&n.get("buttons").disableAllButtonsExceptOne(t,e));}));}});},function(t,e,n){!function(t){function e(t){for(var e={},n=0;n<t.length;++n)e[t[n].toLowerCase()]=!0;return e}t.defineMode("css",(function(e,n){var r=n.inline;n.propertyKeywords||(n=t.resolveMode("text/css"));var i,o,a=e.indentUnit,s=n.tokenHooks,l=n.documentTypes||{},c=n.mediaTypes||{},u=n.mediaFeatures||{},h=n.mediaValueKeywords||{},d=n.propertyKeywords||{},f=n.nonStandardPropertyKeywords||{},p=n.fontProperties||{},g=n.counterDescriptors||{},v=n.colorKeywords||{},m=n.valueKeywords||{},y=n.allowNested,b=n.lineComment,w=!0===n.supportsAtComponent;function x(t,e){return i=e,t}function O(t,e){var n=t.next();if(s[n]){var r=s[n](t,e);if(!1!==r)return r}return "@"==n?(t.eatWhile(/[\w\\\-]/),x("def",t.current())):"="==n||("~"==n||"|"==n)&&t.eat("=")?x(null,"compare"):'"'==n||"'"==n?(e.tokenize=C(n),e.tokenize(t,e)):"#"==n?(t.eatWhile(/[\w\\\-]/),x("atom","hash")):"!"==n?(t.match(/^\s*\w*/),x("keyword","important")):/\d/.test(n)||"."==n&&t.eat(/\d/)?(t.eatWhile(/[\w.%]/),x("number","unit")):"-"!==n?/[,+>*\/]/.test(n)?x(null,"select-op"):"."==n&&t.match(/^-?[_a-z][_a-z0-9-]*/i)?x("qualifier","qualifier"):/[:;{}\[\]\(\)]/.test(n)?x(null,n):t.match(/[\w-.]+(?=\()/)?(/^(url(-prefix)?|domain|regexp)$/.test(t.current().toLowerCase())&&(e.tokenize=S),x("variable callee","variable")):/[\w\\\-]/.test(n)?(t.eatWhile(/[\w\\\-]/),x("property","word")):x(null,null):/[\d.]/.test(t.peek())?(t.eatWhile(/[\w.%]/),x("number","unit")):t.match(/^-[\w\\\-]*/)?(t.eatWhile(/[\w\\\-]/),t.match(/^\s*:/,!1)?x("variable-2","variable-definition"):x("variable-2","variable")):t.match(/^\w+-/)?x("meta","meta"):void 0}function C(t){return function(e,n){for(var r,i=!1;null!=(r=e.next());){if(r==t&&!i){")"==t&&e.backUp(1);break}i=!i&&"\\"==r;}return (r==t||!i&&")"!=t)&&(n.tokenize=null),x("string","string")}}function S(t,e){return t.next(),t.match(/\s*[\"\')]/,!1)?e.tokenize=null:e.tokenize=C(")"),x(null,"(")}function k(t,e,n){this.type=t,this.indent=e,this.prev=n;}function T(t,e,n,r){return t.context=new k(n,e.indentation()+(!1===r?0:a),t.context),n}function j(t){return t.context.prev&&(t.context=t.context.prev),t.context.type}function P(t,e,n){return D[n.context.type](t,e,n)}function E(t,e,n,r){for(var i=r||1;i>0;i--)n.context=n.context.prev;return P(t,e,n)}function M(t){var e=t.current().toLowerCase();o=m.hasOwnProperty(e)?"atom":v.hasOwnProperty(e)?"keyword":"variable";}var D={top:function(t,e,n){if("{"==t)return T(n,e,"block");if("}"==t&&n.context.prev)return j(n);if(w&&/@component/i.test(t))return T(n,e,"atComponentBlock");if(/^@(-moz-)?document$/i.test(t))return T(n,e,"documentTypes");if(/^@(media|supports|(-moz-)?document|import)$/i.test(t))return T(n,e,"atBlock");if(/^@(font-face|counter-style)/i.test(t))return n.stateArg=t,"restricted_atBlock_before";if(/^@(-(moz|ms|o|webkit)-)?keyframes$/i.test(t))return "keyframes";if(t&&"@"==t.charAt(0))return T(n,e,"at");if("hash"==t)o="builtin";else if("word"==t)o="tag";else {if("variable-definition"==t)return "maybeprop";if("interpolation"==t)return T(n,e,"interpolation");if(":"==t)return "pseudo";if(y&&"("==t)return T(n,e,"parens")}return n.context.type},block:function(t,e,n){if("word"==t){var r=e.current().toLowerCase();return d.hasOwnProperty(r)?(o="property","maybeprop"):f.hasOwnProperty(r)?(o="string-2","maybeprop"):y?(o=e.match(/^\s*:(?:\s|$)/,!1)?"property":"tag","block"):(o+=" error","maybeprop")}return "meta"==t?"block":y||"hash"!=t&&"qualifier"!=t?D.top(t,e,n):(o="error","block")},maybeprop:function(t,e,n){return ":"==t?T(n,e,"prop"):P(t,e,n)},prop:function(t,e,n){if(";"==t)return j(n);if("{"==t&&y)return T(n,e,"propBlock");if("}"==t||"{"==t)return E(t,e,n);if("("==t)return T(n,e,"parens");if("hash"!=t||/^#([0-9a-fA-f]{3,4}|[0-9a-fA-f]{6}|[0-9a-fA-f]{8})$/.test(e.current())){if("word"==t)M(e);else if("interpolation"==t)return T(n,e,"interpolation")}else o+=" error";return "prop"},propBlock:function(t,e,n){return "}"==t?j(n):"word"==t?(o="property","maybeprop"):n.context.type},parens:function(t,e,n){return "{"==t||"}"==t?E(t,e,n):")"==t?j(n):"("==t?T(n,e,"parens"):"interpolation"==t?T(n,e,"interpolation"):("word"==t&&M(e),"parens")},pseudo:function(t,e,n){return "meta"==t?"pseudo":"word"==t?(o="variable-3",n.context.type):P(t,e,n)},documentTypes:function(t,e,n){return "word"==t&&l.hasOwnProperty(e.current())?(o="tag",n.context.type):D.atBlock(t,e,n)},atBlock:function(t,e,n){if("("==t)return T(n,e,"atBlock_parens");if("}"==t||";"==t)return E(t,e,n);if("{"==t)return j(n)&&T(n,e,y?"block":"top");if("interpolation"==t)return T(n,e,"interpolation");if("word"==t){var r=e.current().toLowerCase();o="only"==r||"not"==r||"and"==r||"or"==r?"keyword":c.hasOwnProperty(r)?"attribute":u.hasOwnProperty(r)?"property":h.hasOwnProperty(r)?"keyword":d.hasOwnProperty(r)?"property":f.hasOwnProperty(r)?"string-2":m.hasOwnProperty(r)?"atom":v.hasOwnProperty(r)?"keyword":"error";}return n.context.type},atComponentBlock:function(t,e,n){return "}"==t?E(t,e,n):"{"==t?j(n)&&T(n,e,y?"block":"top",!1):("word"==t&&(o="error"),n.context.type)},atBlock_parens:function(t,e,n){return ")"==t?j(n):"{"==t||"}"==t?E(t,e,n,2):D.atBlock(t,e,n)},restricted_atBlock_before:function(t,e,n){return "{"==t?T(n,e,"restricted_atBlock"):"word"==t&&"@counter-style"==n.stateArg?(o="variable","restricted_atBlock_before"):P(t,e,n)},restricted_atBlock:function(t,e,n){return "}"==t?(n.stateArg=null,j(n)):"word"==t?(o="@font-face"==n.stateArg&&!p.hasOwnProperty(e.current().toLowerCase())||"@counter-style"==n.stateArg&&!g.hasOwnProperty(e.current().toLowerCase())?"error":"property","maybeprop"):"restricted_atBlock"},keyframes:function(t,e,n){return "word"==t?(o="variable","keyframes"):"{"==t?T(n,e,"top"):P(t,e,n)},at:function(t,e,n){return ";"==t?j(n):"{"==t||"}"==t?E(t,e,n):("word"==t?o="tag":"hash"==t&&(o="builtin"),"at")},interpolation:function(t,e,n){return "}"==t?j(n):"{"==t||";"==t?E(t,e,n):("word"==t?o="variable":"variable"!=t&&"("!=t&&")"!=t&&(o="error"),"interpolation")}};return {startState:function(t){return {tokenize:null,state:r?"block":"top",stateArg:null,context:new k(r?"block":"top",t||0,null)}},token:function(t,e){if(!e.tokenize&&t.eatSpace())return null;var n=(e.tokenize||O)(t,e);return n&&"object"==typeof n&&(i=n[1],n=n[0]),o=n,"comment"!=i&&(e.state=D[e.state](i,t,e)),o},indent:function(t,e){var n=t.context,r=e&&e.charAt(0),i=n.indent;return "prop"!=n.type||"}"!=r&&")"!=r||(n=n.prev),n.prev&&("}"!=r||"block"!=n.type&&"top"!=n.type&&"interpolation"!=n.type&&"restricted_atBlock"!=n.type?(")"!=r||"parens"!=n.type&&"atBlock_parens"!=n.type)&&("{"!=r||"at"!=n.type&&"atBlock"!=n.type)||(i=Math.max(0,n.indent-a)):i=(n=n.prev).indent),i},electricChars:"}",blockCommentStart:"/*",blockCommentEnd:"*/",blockCommentContinue:" * ",lineComment:b,fold:"brace"}}));var n=["domain","regexp","url","url-prefix"],r=e(n),i=["all","aural","braille","handheld","print","projection","screen","tty","tv","embossed"],o=e(i),a=["width","min-width","max-width","height","min-height","max-height","device-width","min-device-width","max-device-width","device-height","min-device-height","max-device-height","aspect-ratio","min-aspect-ratio","max-aspect-ratio","device-aspect-ratio","min-device-aspect-ratio","max-device-aspect-ratio","color","min-color","max-color","color-index","min-color-index","max-color-index","monochrome","min-monochrome","max-monochrome","resolution","min-resolution","max-resolution","scan","grid","orientation","device-pixel-ratio","min-device-pixel-ratio","max-device-pixel-ratio","pointer","any-pointer","hover","any-hover"],s=e(a),l=["landscape","portrait","none","coarse","fine","on-demand","hover","interlace","progressive"],c=e(l),u=["align-content","align-items","align-self","alignment-adjust","alignment-baseline","anchor-point","animation","animation-delay","animation-direction","animation-duration","animation-fill-mode","animation-iteration-count","animation-name","animation-play-state","animation-timing-function","appearance","azimuth","backface-visibility","background","background-attachment","background-blend-mode","background-clip","background-color","background-image","background-origin","background-position","background-repeat","background-size","baseline-shift","binding","bleed","bookmark-label","bookmark-level","bookmark-state","bookmark-target","border","border-bottom","border-bottom-color","border-bottom-left-radius","border-bottom-right-radius","border-bottom-style","border-bottom-width","border-collapse","border-color","border-image","border-image-outset","border-image-repeat","border-image-slice","border-image-source","border-image-width","border-left","border-left-color","border-left-style","border-left-width","border-radius","border-right","border-right-color","border-right-style","border-right-width","border-spacing","border-style","border-top","border-top-color","border-top-left-radius","border-top-right-radius","border-top-style","border-top-width","border-width","bottom","box-decoration-break","box-shadow","box-sizing","break-after","break-before","break-inside","caption-side","caret-color","clear","clip","color","color-profile","column-count","column-fill","column-gap","column-rule","column-rule-color","column-rule-style","column-rule-width","column-span","column-width","columns","content","counter-increment","counter-reset","crop","cue","cue-after","cue-before","cursor","direction","display","dominant-baseline","drop-initial-after-adjust","drop-initial-after-align","drop-initial-before-adjust","drop-initial-before-align","drop-initial-size","drop-initial-value","elevation","empty-cells","fit","fit-position","flex","flex-basis","flex-direction","flex-flow","flex-grow","flex-shrink","flex-wrap","float","float-offset","flow-from","flow-into","font","font-feature-settings","font-family","font-kerning","font-language-override","font-size","font-size-adjust","font-stretch","font-style","font-synthesis","font-variant","font-variant-alternates","font-variant-caps","font-variant-east-asian","font-variant-ligatures","font-variant-numeric","font-variant-position","font-weight","grid","grid-area","grid-auto-columns","grid-auto-flow","grid-auto-rows","grid-column","grid-column-end","grid-column-gap","grid-column-start","grid-gap","grid-row","grid-row-end","grid-row-gap","grid-row-start","grid-template","grid-template-areas","grid-template-columns","grid-template-rows","hanging-punctuation","height","hyphens","icon","image-orientation","image-rendering","image-resolution","inline-box-align","justify-content","justify-items","justify-self","left","letter-spacing","line-break","line-height","line-stacking","line-stacking-ruby","line-stacking-shift","line-stacking-strategy","list-style","list-style-image","list-style-position","list-style-type","margin","margin-bottom","margin-left","margin-right","margin-top","marks","marquee-direction","marquee-loop","marquee-play-count","marquee-speed","marquee-style","max-height","max-width","min-height","min-width","mix-blend-mode","move-to","nav-down","nav-index","nav-left","nav-right","nav-up","object-fit","object-position","opacity","order","orphans","outline","outline-color","outline-offset","outline-style","outline-width","overflow","overflow-style","overflow-wrap","overflow-x","overflow-y","padding","padding-bottom","padding-left","padding-right","padding-top","page","page-break-after","page-break-before","page-break-inside","page-policy","pause","pause-after","pause-before","perspective","perspective-origin","pitch","pitch-range","place-content","place-items","place-self","play-during","position","presentation-level","punctuation-trim","quotes","region-break-after","region-break-before","region-break-inside","region-fragment","rendering-intent","resize","rest","rest-after","rest-before","richness","right","rotation","rotation-point","ruby-align","ruby-overhang","ruby-position","ruby-span","shape-image-threshold","shape-inside","shape-margin","shape-outside","size","speak","speak-as","speak-header","speak-numeral","speak-punctuation","speech-rate","stress","string-set","tab-size","table-layout","target","target-name","target-new","target-position","text-align","text-align-last","text-decoration","text-decoration-color","text-decoration-line","text-decoration-skip","text-decoration-style","text-emphasis","text-emphasis-color","text-emphasis-position","text-emphasis-style","text-height","text-indent","text-justify","text-outline","text-overflow","text-shadow","text-size-adjust","text-space-collapse","text-transform","text-underline-position","text-wrap","top","transform","transform-origin","transform-style","transition","transition-delay","transition-duration","transition-property","transition-timing-function","unicode-bidi","user-select","vertical-align","visibility","voice-balance","voice-duration","voice-family","voice-pitch","voice-range","voice-rate","voice-stress","voice-volume","volume","white-space","widows","width","will-change","word-break","word-spacing","word-wrap","z-index","clip-path","clip-rule","mask","enable-background","filter","flood-color","flood-opacity","lighting-color","stop-color","stop-opacity","pointer-events","color-interpolation","color-interpolation-filters","color-rendering","fill","fill-opacity","fill-rule","image-rendering","marker","marker-end","marker-mid","marker-start","shape-rendering","stroke","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke-width","text-rendering","baseline-shift","dominant-baseline","glyph-orientation-horizontal","glyph-orientation-vertical","text-anchor","writing-mode"],h=e(u),d=["scrollbar-arrow-color","scrollbar-base-color","scrollbar-dark-shadow-color","scrollbar-face-color","scrollbar-highlight-color","scrollbar-shadow-color","scrollbar-3d-light-color","scrollbar-track-color","shape-inside","searchfield-cancel-button","searchfield-decoration","searchfield-results-button","searchfield-results-decoration","zoom"],f=e(d),p=e(["font-family","src","unicode-range","font-variant","font-feature-settings","font-stretch","font-weight","font-style"]),g=e(["additive-symbols","fallback","negative","pad","prefix","range","speak-as","suffix","symbols","system"]),v=["aliceblue","antiquewhite","aqua","aquamarine","azure","beige","bisque","black","blanchedalmond","blue","blueviolet","brown","burlywood","cadetblue","chartreuse","chocolate","coral","cornflowerblue","cornsilk","crimson","cyan","darkblue","darkcyan","darkgoldenrod","darkgray","darkgreen","darkkhaki","darkmagenta","darkolivegreen","darkorange","darkorchid","darkred","darksalmon","darkseagreen","darkslateblue","darkslategray","darkturquoise","darkviolet","deeppink","deepskyblue","dimgray","dodgerblue","firebrick","floralwhite","forestgreen","fuchsia","gainsboro","ghostwhite","gold","goldenrod","gray","grey","green","greenyellow","honeydew","hotpink","indianred","indigo","ivory","khaki","lavender","lavenderblush","lawngreen","lemonchiffon","lightblue","lightcoral","lightcyan","lightgoldenrodyellow","lightgray","lightgreen","lightpink","lightsalmon","lightseagreen","lightskyblue","lightslategray","lightsteelblue","lightyellow","lime","limegreen","linen","magenta","maroon","mediumaquamarine","mediumblue","mediumorchid","mediumpurple","mediumseagreen","mediumslateblue","mediumspringgreen","mediumturquoise","mediumvioletred","midnightblue","mintcream","mistyrose","moccasin","navajowhite","navy","oldlace","olive","olivedrab","orange","orangered","orchid","palegoldenrod","palegreen","paleturquoise","palevioletred","papayawhip","peachpuff","peru","pink","plum","powderblue","purple","rebeccapurple","red","rosybrown","royalblue","saddlebrown","salmon","sandybrown","seagreen","seashell","sienna","silver","skyblue","slateblue","slategray","snow","springgreen","steelblue","tan","teal","thistle","tomato","turquoise","violet","wheat","white","whitesmoke","yellow","yellowgreen"],m=e(v),y=["above","absolute","activeborder","additive","activecaption","afar","after-white-space","ahead","alias","all","all-scroll","alphabetic","alternate","always","amharic","amharic-abegede","antialiased","appworkspace","arabic-indic","armenian","asterisks","attr","auto","auto-flow","avoid","avoid-column","avoid-page","avoid-region","background","backwards","baseline","below","bidi-override","binary","bengali","blink","block","block-axis","bold","bolder","border","border-box","both","bottom","break","break-all","break-word","bullets","button","button-bevel","buttonface","buttonhighlight","buttonshadow","buttontext","calc","cambodian","capitalize","caps-lock-indicator","caption","captiontext","caret","cell","center","checkbox","circle","cjk-decimal","cjk-earthly-branch","cjk-heavenly-stem","cjk-ideographic","clear","clip","close-quote","col-resize","collapse","color","color-burn","color-dodge","column","column-reverse","compact","condensed","contain","content","contents","content-box","context-menu","continuous","copy","counter","counters","cover","crop","cross","crosshair","currentcolor","cursive","cyclic","darken","dashed","decimal","decimal-leading-zero","default","default-button","dense","destination-atop","destination-in","destination-out","destination-over","devanagari","difference","disc","discard","disclosure-closed","disclosure-open","document","dot-dash","dot-dot-dash","dotted","double","down","e-resize","ease","ease-in","ease-in-out","ease-out","element","ellipse","ellipsis","embed","end","ethiopic","ethiopic-abegede","ethiopic-abegede-am-et","ethiopic-abegede-gez","ethiopic-abegede-ti-er","ethiopic-abegede-ti-et","ethiopic-halehame-aa-er","ethiopic-halehame-aa-et","ethiopic-halehame-am-et","ethiopic-halehame-gez","ethiopic-halehame-om-et","ethiopic-halehame-sid-et","ethiopic-halehame-so-et","ethiopic-halehame-ti-er","ethiopic-halehame-ti-et","ethiopic-halehame-tig","ethiopic-numeric","ew-resize","exclusion","expanded","extends","extra-condensed","extra-expanded","fantasy","fast","fill","fixed","flat","flex","flex-end","flex-start","footnotes","forwards","from","geometricPrecision","georgian","graytext","grid","groove","gujarati","gurmukhi","hand","hangul","hangul-consonant","hard-light","hebrew","help","hidden","hide","higher","highlight","highlighttext","hiragana","hiragana-iroha","horizontal","hsl","hsla","hue","icon","ignore","inactiveborder","inactivecaption","inactivecaptiontext","infinite","infobackground","infotext","inherit","initial","inline","inline-axis","inline-block","inline-flex","inline-grid","inline-table","inset","inside","intrinsic","invert","italic","japanese-formal","japanese-informal","justify","kannada","katakana","katakana-iroha","keep-all","khmer","korean-hangul-formal","korean-hanja-formal","korean-hanja-informal","landscape","lao","large","larger","left","level","lighter","lighten","line-through","linear","linear-gradient","lines","list-item","listbox","listitem","local","logical","loud","lower","lower-alpha","lower-armenian","lower-greek","lower-hexadecimal","lower-latin","lower-norwegian","lower-roman","lowercase","ltr","luminosity","malayalam","match","matrix","matrix3d","media-controls-background","media-current-time-display","media-fullscreen-button","media-mute-button","media-play-button","media-return-to-realtime-button","media-rewind-button","media-seek-back-button","media-seek-forward-button","media-slider","media-sliderthumb","media-time-remaining-display","media-volume-slider","media-volume-slider-container","media-volume-sliderthumb","medium","menu","menulist","menulist-button","menulist-text","menulist-textfield","menutext","message-box","middle","min-intrinsic","mix","mongolian","monospace","move","multiple","multiply","myanmar","n-resize","narrower","ne-resize","nesw-resize","no-close-quote","no-drop","no-open-quote","no-repeat","none","normal","not-allowed","nowrap","ns-resize","numbers","numeric","nw-resize","nwse-resize","oblique","octal","opacity","open-quote","optimizeLegibility","optimizeSpeed","oriya","oromo","outset","outside","outside-shape","overlay","overline","padding","padding-box","painted","page","paused","persian","perspective","plus-darker","plus-lighter","pointer","polygon","portrait","pre","pre-line","pre-wrap","preserve-3d","progress","push-button","radial-gradient","radio","read-only","read-write","read-write-plaintext-only","rectangle","region","relative","repeat","repeating-linear-gradient","repeating-radial-gradient","repeat-x","repeat-y","reset","reverse","rgb","rgba","ridge","right","rotate","rotate3d","rotateX","rotateY","rotateZ","round","row","row-resize","row-reverse","rtl","run-in","running","s-resize","sans-serif","saturation","scale","scale3d","scaleX","scaleY","scaleZ","screen","scroll","scrollbar","scroll-position","se-resize","searchfield","searchfield-cancel-button","searchfield-decoration","searchfield-results-button","searchfield-results-decoration","self-start","self-end","semi-condensed","semi-expanded","separate","serif","show","sidama","simp-chinese-formal","simp-chinese-informal","single","skew","skewX","skewY","skip-white-space","slide","slider-horizontal","slider-vertical","sliderthumb-horizontal","sliderthumb-vertical","slow","small","small-caps","small-caption","smaller","soft-light","solid","somali","source-atop","source-in","source-out","source-over","space","space-around","space-between","space-evenly","spell-out","square","square-button","start","static","status-bar","stretch","stroke","sub","subpixel-antialiased","super","sw-resize","symbolic","symbols","system-ui","table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row","table-row-group","tamil","telugu","text","text-bottom","text-top","textarea","textfield","thai","thick","thin","threeddarkshadow","threedface","threedhighlight","threedlightshadow","threedshadow","tibetan","tigre","tigrinya-er","tigrinya-er-abegede","tigrinya-et","tigrinya-et-abegede","to","top","trad-chinese-formal","trad-chinese-informal","transform","translate","translate3d","translateX","translateY","translateZ","transparent","ultra-condensed","ultra-expanded","underline","unset","up","upper-alpha","upper-armenian","upper-greek","upper-hexadecimal","upper-latin","upper-norwegian","upper-roman","uppercase","urdu","url","var","vertical","vertical-text","visible","visibleFill","visiblePainted","visibleStroke","visual","w-resize","wait","wave","wider","window","windowframe","windowtext","words","wrap","wrap-reverse","x-large","x-small","xor","xx-large","xx-small"],b=e(y),w=n.concat(i).concat(a).concat(l).concat(u).concat(d).concat(v).concat(y);function x(t,e){for(var n,r=!1;null!=(n=t.next());){if(r&&"/"==n){e.tokenize=null;break}r="*"==n;}return ["comment","comment"]}t.registerHelper("hintWords","css",w),t.defineMIME("text/css",{documentTypes:r,mediaTypes:o,mediaFeatures:s,mediaValueKeywords:c,propertyKeywords:h,nonStandardPropertyKeywords:f,fontProperties:p,counterDescriptors:g,colorKeywords:m,valueKeywords:b,tokenHooks:{"/":function(t,e){return !!t.eat("*")&&(e.tokenize=x,x(t,e))}},name:"css"}),t.defineMIME("text/x-scss",{mediaTypes:o,mediaFeatures:s,mediaValueKeywords:c,propertyKeywords:h,nonStandardPropertyKeywords:f,colorKeywords:m,valueKeywords:b,fontProperties:p,allowNested:!0,lineComment:"//",tokenHooks:{"/":function(t,e){return t.eat("/")?(t.skipToEnd(),["comment","comment"]):t.eat("*")?(e.tokenize=x,x(t,e)):["operator","operator"]},":":function(t){return !!t.match(/\s*\{/,!1)&&[null,null]},$:function(t){return t.match(/^[\w-]+/),t.match(/^\s*:/,!1)?["variable-2","variable-definition"]:["variable-2","variable"]},"#":function(t){return !!t.eat("{")&&[null,"interpolation"]}},name:"css",helperType:"scss"}),t.defineMIME("text/x-less",{mediaTypes:o,mediaFeatures:s,mediaValueKeywords:c,propertyKeywords:h,nonStandardPropertyKeywords:f,colorKeywords:m,valueKeywords:b,fontProperties:p,allowNested:!0,lineComment:"//",tokenHooks:{"/":function(t,e){return t.eat("/")?(t.skipToEnd(),["comment","comment"]):t.eat("*")?(e.tokenize=x,x(t,e)):["operator","operator"]},"@":function(t){return t.eat("{")?[null,"interpolation"]:!t.match(/^(charset|document|font-face|import|(-(moz|ms|o|webkit)-)?keyframes|media|namespace|page|supports)\b/i,!1)&&(t.eatWhile(/[\w\\\-]/),t.match(/^\s*:/,!1)?["variable-2","variable-definition"]:["variable-2","variable"])},"&":function(){return ["atom","atom"]}},name:"css",helperType:"less"}),t.defineMIME("text/x-gss",{documentTypes:r,mediaTypes:o,mediaFeatures:s,propertyKeywords:h,nonStandardPropertyKeywords:f,fontProperties:p,counterDescriptors:g,colorKeywords:m,valueKeywords:b,supportsAtComponent:!0,tokenHooks:{"/":function(t,e){return !!t.eat("*")&&(e.tokenize=x,x(t,e))}},name:"css",helperType:"gss"});}(n(12));},function(t,e,n){n.r(e),e.default={run:function(t){t.DomComponents.clear(),t.CssComposer.clear();}};},function(t,e,n){n.r(e);var r=n(0),i=n(3),o=n(16);e.default={run:function(t){Object(r.bindAll)(this,"onKeyUp","enableDragger","disableDragger"),this.editor=t,this.canvasModel=this.canvas.getCanvasView().model,this.toggleMove(1);},stop:function(t){this.toggleMove(),this.disableDragger();},onKeyUp:function(t){" "===Object(i.f)(t)&&this.editor.stopCommand(this.id);},enableDragger:function(t){this.toggleDragger(1,t);},disableDragger:function(t){this.toggleDragger(0,t);},toggleDragger:function(t,e){var n=this.canvasModel,r=this.em,i=this.dragger,a=t?"add":"remove";this.getCanvas().classList[a]("".concat(this.ppfx,"is__grabbing")),i||(i=new o.a({getPosition:function(){return {x:n.get("x"),y:n.get("y")}},setPosition:function(t){var e=t.x,r=t.y;n.set({x:e,y:r});},onStart:function(t,e){r.trigger("canvas:move:start",e);},onDrag:function(t,e){r.trigger("canvas:move",e);},onEnd:function(t,e){r.trigger("canvas:move:end",e);}}),this.dragger=i),t?i.start(e):i.stop();},toggleMove:function(t){var e=this.ppfx,n=t?"add":"remove",r=t?"on":"off",o={on:i.t,off:i.s},a=this.getCanvas(),s=["".concat(e,"is__grab")];!t&&s.push("".concat(e,"is__grabbing")),s.forEach((function(t){return a.classList[n](t)})),o[r](document,"keyup",this.onKeyUp),o[r](a,"mousedown",this.enableDragger),o[r](document,"mouseup",this.disableDragger);}};},function(t,e,n){n.r(e);var r=n(6),i=n.n(r),o=n(0);e.default={run:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},a=r.component||t.getSelectedAll();return a=Object(o.isArray)(a)?i()(a):[a],t.select(null),a.forEach((function(t){if(!t||!t.get("removable"))return n.em.logWarning("The element is not removable",{component:t});t.remove();})),a}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(0),a=n(16);function s(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function l(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?s(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):s(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default={run:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};Object(o.bindAll)(this,"setPosition","onStart","onDrag","onEnd","getPosition","getGuidesStatic","renderGuide","getGuidesTarget");var i=r.target,s=r.event,c=r.mode,u=r.dragger,h=void 0===u?{}:u,d=i.getEl(),f=l({doc:d.ownerDocument,onStart:this.onStart,onEnd:this.onEnd,onDrag:this.onDrag,getPosition:this.getPosition,setPosition:this.setPosition,guidesStatic:function(){return n.guidesStatic},guidesTarget:function(){return n.guidesTarget}},h);this.setupGuides(),this.opts=r,this.editor=t,this.em=t.getModel(),this.target=i,this.isTran="translate"==c,this.guidesContainer=this.getGuidesContainer(),this.guidesTarget=this.getGuidesTarget(),this.guidesStatic=this.getGuidesStatic();var p=this.dragger;return p?p.setOptions(f):(p=new a.a(f),this.dragger=p),s&&p.start(s),this.toggleDrag(1),this.em.trigger("".concat("dmode",":start"),this.getEventOpts()),p},getEventOpts:function(){return {mode:this.opts.mode,target:this.target,guidesTarget:this.guidesTarget,guidesStatic:this.guidesStatic}},stop:function(){this.toggleDrag();},setupGuides:function(){(this.guides||[]).forEach((function(t){var e=t.guide;e&&e.parentNode.removeChild(e);})),this.guides=[];},getGuidesContainer:function(){var t=this,e=this.guidesEl;if(!e){var n=this.editor,r=this.em,i=this.opts,a=n.getConfig("stylePrefix"),s=document.createElement("div"),l=document.createElement("div"),c='<div class="'.concat(a,"guide-info__line ").concat(a,'danger-bg">\n        <div class="').concat(a,"guide-info__content ").concat(a,'danger-color"></div>\n      </div>');(e=document.createElement("div")).className="".concat(a,"guides"),s.className="".concat(a,"guide-info ").concat(a,"guide-info__x"),l.className="".concat(a,"guide-info ").concat(a,"guide-info__y"),s.innerHTML=c,l.innerHTML=c,e.appendChild(s),e.appendChild(l),n.Canvas.getGlobalToolsEl().appendChild(e),this.guidesEl=e,this.elGuideInfoX=s,this.elGuideInfoY=l,this.elGuideInfoContentX=s.querySelector(".".concat(a,"guide-info__content")),this.elGuideInfoContentY=l.querySelector(".".concat(a,"guide-info__content")),r.on("canvas:update frame:scroll",Object(o.debounce)((function(){t.updateGuides(),i.debug&&t.guides.forEach((function(e){return t.renderGuide(e)}));}),200));}return e},getGuidesStatic:function(){var t=this,e=[],n=this.target.getEl(),r=n.parentNode,i=void 0===r?{}:r;return Object(o.each)(i.children,(function(r){return e=e.concat(n!==r?t.getElementGuides(r):[])})),e.concat(this.getElementGuides(i))},getGuidesTarget:function(){return this.getElementGuides(this.target.getEl())},updateGuides:function(t){var e,n,r=this;(t||this.guides).forEach((function(t){var i=t.origin,a=e===i?n:r.getElementPos(i);e=i,n=a,Object(o.each)(r.getGuidePosUpdate(t,a),(function(e,n){return t[n]=e})),t.originRect=a;}));},getGuidePosUpdate:function(t,e){var n={},r=e.top,i=e.height,o=e.left,a=e.width;switch(t.type){case"t":n.y=r;break;case"b":n.y=r+i;break;case"l":n.x=o;break;case"r":n.x=o+a;break;case"x":n.x=o+a/2;break;case"y":n.y=r+i/2;}return n},renderGuide:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.guide||document.createElement("div"),n="px",r=t.active?2:1,i=e.children[0];return e.style="position: absolute; background-color: ".concat(t.active?"green":"red",";"),e.children.length||((i=document.createElement("div")).style="position: absolute; color: red; padding: 5px; top: 0; left: 0;",e.appendChild(i)),t.y?(e.style.width="100%",e.style.height="".concat(r).concat(n),e.style.top="".concat(t.y).concat(n),e.style.left=0):(e.style.width="".concat(r).concat(n),e.style.height="100%",e.style.left="".concat(t.x).concat(n),e.style.top="0".concat(n)),!t.guide&&this.guidesContainer.appendChild(e),e},getElementPos:function(t){return this.editor.Canvas.getElementPos(t,{noScroll:1})},getElementGuides:function(t){var e=this,n=this.opts,r=this.getElementPos(t),i=r.top,o=r.height,a=r.left,s=r.width,c=[{type:"t",y:i},{type:"b",y:i+o},{type:"l",x:a},{type:"r",x:a+s},{type:"x",x:a+s/2},{type:"y",y:i+o/2}].map((function(i){return l({},i,{origin:t,originRect:r,guide:n.debug&&e.renderGuide(i)})}));return c.forEach((function(t){return e.guides.push(t)})),c},getTranslate:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"x",n=0;return (t||"").split(" ").forEach((function(t){var r=t.trim(),i="translate".concat(e.toUpperCase(),"(");0===r.indexOf(i)&&(n=parseFloat(r.replace(i,"")));})),n},setTranslate:function(t,e,n){var r="translate".concat(e.toUpperCase(),"("),i="".concat(r).concat(n,")"),o=(t||"").split(" ").map((function(t){return 0===t.trim().indexOf(r)&&(t=i),t})).join(" ");return o.indexOf(r)<0&&(o+=" ".concat(i)),o},getPosition:function(){var t=this.target,e=this.isTran,n=t.getStyle(),r=n.left,i=n.top,o=n.transform,a=0,s=0;return e?(a=this.getTranslate(o),s=this.getTranslate(o,"y")):(a=parseFloat(r),s=parseFloat(i)),{x:a,y:s}},setPosition:function(t){var e=t.x,n=t.y,r=t.end,i=t.position,a=t.width,s=t.height,l=this.target,c=this.isTran,u=r?"":1,h="".concat(e).concat("px"),d="".concat(n).concat("px");if(c){var f=l.getStyle().transform||"";return f=this.setTranslate(f,"x",h),f=this.setTranslate(f,"y",d),l.addStyle({transform:f,en:u},{avoidStore:!r})}var p={position:i,width:a,height:s},g={left:h,top:d,en:u};Object(o.keys)(p).forEach((function(t){var e=p[t];e&&(g[t]=e);})),l.addStyle(g,{avoidStore:!r});},_getDragData:function(){var t=this.target;return {target:t,parent:t.parent(),index:t.index()}},onStart:function(){var t=this.target,e=this.editor,n=this.isTran,r=this.opts,i=r.center,o=r.onStart,a=e.Canvas,s=t.getStyle();if(o&&o(this._getDragData()),!n&&"absolute"!==s.position){var l=a.offset(t.getEl()),c=l.left,u=l.top,h=l.width,d=l.height;if(i){var f=a.getMouseRelativeCanvas(event);c=f.x,u=f.y;}this.setPosition({x:c,y:u,width:"".concat(h,"px"),height:"".concat(d,"px"),position:"absolute"});}},onDrag:function(){var t=this,e=this.guidesTarget,n=this.opts,r=n.onDrag;this.updateGuides(e),n.debug&&e.forEach((function(e){return t.renderGuide(e)})),n.guidesInfo&&this.renderGuideInfo(e.filter((function(t){return t.active}))),r&&r(this._getDragData());},onEnd:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.editor,i=this.opts,o=this.id,a=i.onEnd;a&&a(t,n,l({event:t},n,{},this._getDragData())),r.stopCommand(o),this.hideGuidesInfo(),this.em.trigger("".concat("dmode",":end"),this.getEventOpts());},hideGuidesInfo:function(){var t=this;["X","Y"].forEach((function(e){var n=t["elGuideInfo".concat(e)];n&&(n.style.display="none");}));},renderGuideInfo:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],n=this.guidesStatic;this.hideGuidesInfo(),e.forEach((function(e){var r=e.origin,i=e.x,a=t.getElementPos(r),s=Object(o.isUndefined)(i)?"y":"x",c="y"===s,u=a[c?"left":"top"],h=a.rect[c?"left":"top"],d=c?u+a.width:u+a.height,f=c?h+a.rect.width:h+a.rect.height,p=t["elGuideInfo".concat(s.toUpperCase())],g=t["elGuideInfoContent".concat(s.toUpperCase())],v=p.style,m=n.filter((function(t){return t.type===e.type})).map((function(t){var e=t.originRect,n=e.left,r=e.width,i=e.top,o=e.height,a=c?n+r:i+o;return {gap:a<u?u-a:(c?n:i)-d,guide:t}})).filter((function(t){return t.gap>0})).sort((function(t,e){return t.gap-e.gap})).map((function(t){return t.guide}))[0];if(m){var y=m.originRect,b=y.left,w=y.width,x=y.top,O=y.height,C=y.rect,S=c?b<a.left:x<a.top,k=c?b:x,T=c?C.left:C.top,j=c?b+w:x+O,P=c?C.left+C.width:C.top+C.height,E=c?e.y:e.x,M=S?j:d,D="".concat(E,"px"),A=S?u-j:k-d,L=S?h-P:T-f;v.display="",v[c?"top":"left"]=D,v[c?"left":"top"]="".concat(M,"px"),v[c?"width":"height"]="".concat(A,"px"),g.innerHTML="".concat(Math.round(L),"px"),t.em.trigger("".concat("dmode",":active"),l({},t.getEventOpts(),{guide:e,guidesStatic:n,matched:m,posFirst:E,posSecond:M,size:A,sizeRaw:L,elGuideInfo:p,elGuideInfoCnt:g}));}}));},toggleDrag:function(t){var e=this.ppfx,n=this.editor,r=t?"add":"remove",i=["".concat(e,"is__grabbing")],o=n.Canvas,a=o.getBody();i.forEach((function(t){return a.classList[r](t)})),o[t?"startAutoscroll":"stopAutoscroll"]();}};},function(t,e,n){n.r(e),e.default={run:function(t){if(t.Canvas.hasFocus()){var e=[];t.getSelectedAll().forEach((function(t){var n=t.components(),r=n&&n.at(0);r&&e.push(r);})),e.length&&t.select(e);}}};},function(t,e,n){n.r(e),e.default={run:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(t.Canvas.hasFocus()||n.force){var r=[];t.getSelectedAll().forEach((function(t){for(var e=t.parent();e&&!e.get("selectable");)e=e.parent();e&&r.push(e);})),r.length&&t.select(r);}}};},function(t,e,n){n.r(e),e.default={run:function(t){if(t.Canvas.hasFocus()){var e=[];t.getSelectedAll().forEach((function(t){var n=t.collection,r=n.indexOf(t),i=n.at(r+1);e.push(i||t);})),e.length&&t.select(e);}}};},function(t,e,n){n.r(e),e.default={run:function(t){if(t.Canvas.hasFocus()){var e=[];t.getSelectedAll().forEach((function(t){var n=t.collection,r=n.indexOf(t),i=n.at(r-1);e.push(i&&r-1>=0?i:t);})),e.length&&t.select(e);}}};},function(t,e,n){n.r(e);var r=n(0);e.default={run:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=n.target,o=t.DomComponents,a=i.get("type"),s=o.getWrapper().find('[data-gjs-type="'.concat(a,'"]')).length,l=[];if(!s){var c=t.CssComposer.getAll(),u=i.get("style-signature");u=Object(r.isArray)(u)?u:[u],c.forEach((function(t){var e=t.selectorsToString();u.forEach((function(n){n&&e.indexOf(n)>=0&&l.push(t);}));})),c.remove(l);}return l}};},function(t,e,n){n.r(e);var r=n(6),i=n.n(r);e.default={run:function(t){var e=t.getModel(),n=i()(t.getSelectedAll());n.length&&e.set("clipboard",n);}};},function(t,e,n){n.r(e);var r=n(0),i=n(1),o=n.n(i),a=n(13),s=o.a.$;e.default=Object(r.extend)({},a.default,{init:function(t){Object(r.bindAll)(this,"startDelete","stopDelete","onDelete"),this.hoverClass=this.pfx+"hover-delete",this.badgeClass=this.pfx+"badge-red";},enable:function(){this.$el.find("*").mouseover(this.startDelete).mouseout(this.stopDelete).click(this.onDelete);},startDelete:function(t){t.stopPropagation();var e=s(t.target);e.data("model").get("removable")&&(e.addClass(this.hoverClass),this.attachBadge(e.get(0)));},stopDelete:function(t){t.stopPropagation(),s(t.target).removeClass(this.hoverClass),this.badge&&this.badge.css({left:-1e3,top:-1e3});},onDelete:function(t){t.stopPropagation();var e=s(t.target);e.data("model").get("removable")&&(e.data("model").destroy(),this.removeBadge(),this.clean());},updateBadgeLabel:function(t){this.badge.html("Remove "+t.getName());}});},function(t,e,n){n.r(e);var r=n(1),i=n.n(r).a.$;e.default={run:function(t,e){var n=this;e&&e.set&&e.set("active",0);var r=t.getConfig(),o=t.Modal,a=r.stylePrefix;if(this.cm=t.CodeManager||null,!this.$editors){var s=this.buildEditor("htmlmixed","hopscotch","HTML"),l=this.buildEditor("css","hopscotch","CSS");this.htmlEditor=s.el,this.cssEditor=l.el;var c=i('<div class="'.concat(a,'export-dl"></div>'));c.append(s.$el).append(l.$el),this.$editors=c;}o.open({title:r.textViewCode,content:this.$editors}).getModel().once("change:open",(function(){return t.stopCommand(n.id)})),this.htmlEditor.setContent(t.getHtml()),this.cssEditor.setContent(t.getCss());},stop:function(t){var e=t.Modal;e&&e.close();},buildEditor:function(t,e,n){var r=document.createElement("textarea");!this.codeMirror&&(this.codeMirror=this.cm.getViewer("CodeMirror"));var i=this.codeMirror.clone().set({label:n,codeName:t,theme:e,input:r}),o=new this.cm.EditorView({model:i,config:this.cm.getConfig()}).render().$el;return i.init(r),{el:i,$el:o}}};},function(t,e,n){n.r(e);var r=n(0);e.default={isEnabled:function(){var t=document;return t.fullscreenElement||t.webkitFullscreenElement||t.mozFullScreenElement?1:0},enable:function(t){var e="";return t.requestFullscreen?t.requestFullscreen():t.webkitRequestFullscreen?(e="webkit",t.webkitRequestFullscreen()):t.mozRequestFullScreen?(e="moz",t.mozRequestFullScreen()):t.msRequestFullscreen?t.msRequestFullscreen():console.warn("Fullscreen not supported"),e},disable:function(){var t=document;this.isEnabled()&&(t.exitFullscreen?t.exitFullscreen():t.webkitExitFullscreen?t.webkitExitFullscreen():t.mozCancelFullScreen?t.mozCancelFullScreen():t.msExitFullscreen&&t.msExitFullscreen());},fsChanged:function(t,e){var n=(t||"")+"fullscreenchange";this.isEnabled()||(this.stop(null,this.sender),document.removeEventListener(n,this.fsChanged));},run:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};this.sender=e;var i=n.target,o=Object(r.isElement)(i)?i:document.querySelector(i),a=this.enable(o||t.getContainer());this.fsChanged=this.fsChanged.bind(this,a),document.addEventListener(a+"fullscreenchange",this.fsChanged),t.trigger("change:canvasOffset");},stop:function(t,e){e&&e.set&&e.set("active",!1),this.disable(),t&&t.trigger("change:canvasOffset");}};},function(t,e,n){n.r(e);var r=n(0),i=n(1),o=n.n(i),a=n(3),s=n(13),l=n(25),c=o.a.$;e.default=Object(r.extend)({},l.default,s.default,{init:function(t){s.default.init.apply(this,arguments),Object(r.bindAll)(this,"initSorter","rollback","onEndMove"),this.opt=t,this.hoverClass=this.ppfx+"highlighter-warning",this.badgeClass=this.ppfx+"badge-warning",this.noSelClass=this.ppfx+"no-select";},enable:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];s.default.enable.apply(this,e),this.getBadgeEl().addClass(this.badgeClass),this.getHighlighterEl().addClass(this.hoverClass);var r=this.$wrapper;r.css("cursor","move"),r.on("mousedown",this.initSorter),r.addClass(this.noSelClass);},toggleClipboard:function(){},initSorter:function(t){var e=c(t.target).data("model").get("draggable");e&&(this.cacheEl=null,this.startSelectPosition(t.target,this.frameEl.contentDocument),this.sorter.draggable=e,this.sorter.onEndMove=this.onEndMove.bind(this),this.stopSelectComponent(),this.$wrapper.off("mousedown",this.initSorter),Object(a.t)(this.getContentWindow(),"keydown",this.rollback));},initSorterFromModel:function(t){var e=t.get("draggable");if(e){this.cacheEl=null;var n=t.view.el;this.startSelectPosition(n,this.frameEl.contentDocument),this.sorter.draggable=e,this.sorter.onEndMove=this.onEndMoveFromModel.bind(this),this.stopSelectComponent(),Object(a.t)(this.getContentWindow(),"keydown",this.rollback);}},initSorterFromModels:function(t){this.cacheEl=null;var e=t[t.length-1],n=(this.em.get("currentFrame")||{}).model,r=e.getEl(n),i=r.ownerDocument;this.startSelectPosition(r,i,{onStart:this.onStart}),this.sorter.draggable=e.get("draggable"),this.sorter.toMove=t,this.sorter.onMoveClb=this.onDrag,this.sorter.onEndMove=this.onEndMoveFromModel.bind(this),this.stopSelectComponent(),Object(a.t)(this.getContentWindow(),"keydown",this.rollback);},onEndMoveFromModel:function(){Object(a.s)(this.getContentWindow(),"keydown",this.rollback);},onEndMove:function(){this.enable(),Object(a.s)(this.getContentWindow(),"keydown",this.rollback);},onSelect:function(t,e){},rollback:function(t,e){(27==(t.which||t.keyCode)||e)&&(this.sorter.moved=!1,this.sorter.endMove());},getBadgeEl:function(){return this.$badge||(this.$badge=c(this.getBadge())),this.$badge},getHighlighterEl:function(){return this.$hl||(this.$hl=c(this.canvas.getHighlighter())),this.$hl},stop:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];s.default.stop.apply(this,e),this.getBadgeEl().removeClass(this.badgeClass),this.getHighlighterEl().removeClass(this.hoverClass);var r=this.$wrapper;r.css("cursor","").unbind().removeClass(this.noSelClass);}});},function(t,e,n){n.r(e),e.default={run:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=t.Modal,o=t.AssetManager,a=o.getConfig(),s=o.getContainer(),l=r.modalTitle||t.t("assetManager.modalTitle")||"",c=r.types,u=r.accept;if(o.setTarget(r.target),o.onClick(r.onClick),o.onDblClick(r.onDblClick),o.onSelect(r.onSelect),!this.rendered||c){var h=o.getAll().filter((function(t){return 1}));c&&c.length&&(h=h.filter((function(t){return -1!==c.indexOf(t.get("type"))}))),o.render(h),this.rendered=1;}if(u){var d=s.querySelector("input#".concat(a.stylePrefix,"uploadFile"));d&&d.setAttribute("accept",u);}return i.open({title:l,content:s}).getModel().once("change:open",(function(){return t.stopCommand(n.id)})),this},stop:function(t){return t.Modal.close(),this}};},function(t,e,n){n.r(e),e.default={run:function(t,e){var n=t.BlockManager,r=t.Panels;if(!this.blocks){n.render();var i="views-container",o=document.createElement("div"),a=r.getPanel(i)||r.addPanel({id:i});o.appendChild(n.getContainer()),a.set("appendContent",o).trigger("change:appendContent"),this.blocks=o;}this.blocks.style.display="block";},stop:function(){var t=this.blocks;t&&(t.style.display="none");}};},function(t,e,n){n.r(e);var r=n(1);n.n(r).a.$;e.default={run:function(t){var e=t.LayerManager,n=t.Panels;if(!this.layers){var r="views-container",i=document.createElement("div"),o=n.getPanel(r)||n.addPanel({id:r});i.appendChild(e.render()),o.set("appendContent",i).trigger("change:appendContent"),this.layers=i;}this.layers.style.display="block";},stop:function(){var t=this.layers;t&&(t.style.display="none");}};},function(t,e,n){n.r(e);var r=n(1),i=n.n(r).a.$;e.default={run:function(t,e){if(this.sender=e,!this.$cn){var n=t.getConfig(),r=t.Panels;this.$cn=i("<div></div>"),this.$cn2=i("<div></div>"),this.$cn.append(this.$cn2);var o=t.DeviceManager;if(o&&n.showDevices)r.addPanel({id:"devices-c"}).set("appendContent",o.render()).trigger("change:appendContent");var a=t.SelectorManager;a&&this.$cn2.append(a.render([])),this.$cn2.append(t.StyleManager.render());var s=t.StyleManager.getConfig().stylePrefix;this.$header=i('<div class="'.concat(s,'header">').concat(t.t("styleManager.empty"),"</div>")),this.$cn.append(this.$header),r.getPanel("views-container")?this.panel=r.getPanel("views-container"):this.panel=r.addPanel({id:"views-container"}),this.panel.set("appendContent",this.$cn).trigger("change:appendContent"),this.target=t.editor,this.listenTo(this.target,"component:toggled",this.toggleSm);}this.toggleSm();},toggleSm:function(){var t=this.target,e=this.sender;if(!e||!e.get||e.get("active")){var n=t.get("SelectorManager").getConfig().componentFirst,r=t.getSelectedAll().length;1===r||r>1&&n?(this.$cn2.show(),this.$header.hide()):(this.$cn2.hide(),this.$header.show());}},stop:function(){this.$cn2&&this.$cn2.hide(),this.$header&&this.$header.hide();}};},function(t,e,n){n.r(e);var r=n(1),i=n.n(r).a.$;e.default={run:function(t,e){this.sender=e;var n=t.getModel(),r=t.Config.stylePrefix,o=t.TraitManager;if(!this.$cn){var a=o.getTraitsViewer(),s=o.getConfig();this.$cn=i("<div></div>"),this.$cn2=i("<div></div>"),this.$cn.append(this.$cn2),this.$header=i("<div>").append('<div class="'.concat(s.stylePrefix,'header">').concat(n.t("traitManager.empty"),"</div>")),this.$cn.append(this.$header),this.$cn2.append('<div class="'.concat(r,'traits-label">').concat(n.t("traitManager.label"),"</div>")),this.$cn2.append(a.render().el);var l=t.Panels;(l.getPanel("views-container")?l.getPanel("views-container"):l.addPanel({id:"views-container"})).set("appendContent",this.$cn.get(0)).trigger("change:appendContent"),this.target=t.getModel(),this.listenTo(this.target,"component:toggled",this.toggleTm);}this.toggleTm();},toggleTm:function(){var t=this.sender;t&&t.get&&!t.get("active")||(1===this.target.getSelectedAll().length?(this.$cn2.show(),this.$header.hide()):(this.$cn2.hide(),this.$header.show()));},stop:function(){this.$cn2&&this.$cn2.hide(),this.$header&&this.$header.hide();}};},function(t,e,n){n.r(e);var r=n(0);e.default={run:function(t){var e=t.getModel().get("clipboard"),n=t.getSelected();e&&n&&(t.getSelectedAll().forEach((function(n){if(n){var i,o=n.collection,a=o.indexOf(n)+1,s=e.filter((function(t){return t.get("copyable")}));i=Object(r.contains)(e,n)&&n.get("copyable")?o.add(n.clone(),{at:a}):o.add(s.map((function(t){return t.clone()})),{at:a}),(i=Object(r.isArray)(i)?i:[i]).forEach((function(e){return t.trigger("component:paste",e)}));}})),n.emitUpdate());}};},function(t,e,n){n.r(e);var r=n(0),i="sw-visibility";e.default={getPanels:function(t){return this.panels||(this.panels=t.Panels.getPanels()),this.panels},tglPointers:function(t,e){var n=t.Canvas.getBody().querySelectorAll(".".concat(this.ppfx,"no-pointer"));Object(r.each)(n,(function(t){return t.style.pointerEvents=e?"":"all"}));},run:function(t,e){var n=this;this.sender=e,this.shouldRunSwVisibility||(this.shouldRunSwVisibility=t.Commands.isActive(i)),this.shouldRunSwVisibility&&t.stopCommand(i),t.getModel().stopDefault();var r=this.getPanels(t),o=t.Canvas.getElement(),a=t.getEl(),s=t.Config.stylePrefix;if(!this.helper){var l=document.createElement("span");l.className="".concat(s,"off-prv fa fa-eye-slash"),a.appendChild(l),l.onclick=function(){return n.stopCommand()},this.helper=l;}this.helper.style.display="inline-block",this.tglPointers(t),r.forEach((function(t){return t.set("visible",!1)}));var c=o.style;c.width="100%",c.height="100%",c.top="0",c.left="0",c.padding="0",c.margin="0",t.refresh();},stop:function(t){var e=this.sender,n=void 0===e?{}:e;n.set&&n.set("active",0);var r=this.getPanels(t);this.shouldRunSwVisibility&&(t.runCommand(i),this.shouldRunSwVisibility=!1),t.getModel().runDefault(),r.forEach((function(t){return t.set("visible",!0)})),t.Canvas.getElement().setAttribute("style",""),this.helper&&(this.helper.style.display="none"),t.refresh(),this.tglPointers(t,1);}};},function(t,e,n){n.r(e),e.default={run:function(t,e,n){var r=n||{},i=r.el||"",o=t.Canvas,a=this.canvasResizer,s=r.options||{},l=o.getCanvasView();return s.appendTo=o.getResizerEl(),s.prefix=t.getConfig().stylePrefix,s.posFetcher=l.getElementPos.bind(l),s.mousePosFetcher=o.getMouseRelativePos,a&&!r.forceNew||(this.canvasResizer=t.Utils.Resizer.init(s),a=this.canvasResizer),a.setOptions(s),a.blur(),a.focus(i),a},stop:function(){var t=this.canvasResizer;t&&t.blur();}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(1),a=n.n(o),s=n(0),l=n(3);function c(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var u=a.a.$;e.default={getOffsetMethod:function(t){return "get"+(t||"")+"OffsetViewerEl"},run:function(t,e,n){var r=n||{},o=r.state||"",a=t.getConfig(),h=this.em.getZoomDecimal(),d=r.el||"";if(!a.showOffsets||Object(l.p)(d)||!a.showOffsetsSelected&&"Fixed"==o)t.stopCommand(this.id,n);else {var f=t.Canvas,p=function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?c(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):c(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},r.elPos||f.getElementPos(d));Object(s.isUndefined)(r.top)||(p.top=r.top),Object(s.isUndefined)(r.left)||(p.left=r.left);var g=window.getComputedStyle(d),v=this.ppfx,m=o+"State",y=f[this.getOffsetMethod(o)](n.view);y.style.opacity="";var b=this["marginT"+o],w=this["marginB"+o],x=this["marginL"+o],O=this["marginR"+o],C=this["padT"+o],S=this["padB"+o],k=this["padL"+o],T=this["padR"+o];if(y.childNodes.length&&(this[m]="1",b=y.querySelector("[data-offset-m-t]"),w=y.querySelector("[data-offset-m-b]"),x=y.querySelector("[data-offset-m-l]"),O=y.querySelector("[data-offset-m-r]"),C=y.querySelector("[data-offset-p-t]"),S=y.querySelector("[data-offset-p-b]"),k=y.querySelector("[data-offset-p-l]"),T=y.querySelector("[data-offset-p-r]")),!this[m]){var j=o.toLowerCase(),P=j+"margin-v",E=j+"padding-v",M=u('<div class="'.concat(v,'marginName">')).get(0),D=u('<div class="'.concat(v,'paddingName">')).get(0),A=v+E+"-el",L="".concat(v+P+"-el"," ").concat(v+P),N="".concat(A," ").concat(v+E);b=u('<div class="'.concat(L,'-top"></div>')).get(0),w=u('<div class="'.concat(L,'-bottom"></div>')).get(0),x=u('<div class="'.concat(L,'-left"></div>')).get(0),O=u('<div class="'.concat(L,'-right"></div>')).get(0),C=u('<div class="'.concat(N,'-top"></div>')).get(0),S=u('<div class="'.concat(N,'-bottom"></div>')).get(0),k=u('<div class="'.concat(N,'-left"></div>')).get(0),T=u('<div class="'.concat(N,'-right"></div>')).get(0),this["marginT"+o]=b,this["marginB"+o]=w,this["marginL"+o]=x,this["marginR"+o]=O,this["padT"+o]=C,this["padB"+o]=S,this["padL"+o]=k,this["padR"+o]=T,M.appendChild(b),M.appendChild(w),M.appendChild(x),M.appendChild(O),D.appendChild(C),D.appendChild(S),D.appendChild(k),D.appendChild(T),y.appendChild(M),y.appendChild(D),this[m]="1";}var I=parseFloat(g.marginLeft.replace("px",""))*h,F=parseFloat(g.marginRight.replace("px",""))*h,_=parseFloat(g.marginTop.replace("px",""))*h,V=parseFloat(g.marginBottom.replace("px",""))*h,R=b.style,H=w.style,z=x.style,$=O.style,W=C.style,B=S.style,U=k.style,q=T.style,G=parseFloat(p.left),K=parseFloat(g.width)*h+"px";R.height=_+"px",R.width=K,R.top=p.top-_+"px",R.left=G+"px",H.height=V+"px",H.width=K,H.top=p.top+p.height+"px",H.left=G+"px";var Y=p.height+_+V+"px",X=p.top-_+"px";z.height=Y,z.width=I+"px",z.top=X,z.left=G-I+"px",$.height=Y,$.width=F+"px",$.top=X,$.left=G+p.width+"px";var J=parseFloat(g.paddingTop)*h;W.height=J+"px";var Z=parseFloat(g.paddingBottom)*h;B.height=Z+"px";var Q=p.height-Z-J+"px",tt=p.top+J+"px";U.height=Q,U.width=parseFloat(g.paddingLeft)*h+"px",U.top=tt;var et=parseFloat(g.paddingRight)*h;q.height=Q,q.width=et+"px",q.top=tt;}},stop:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n||{},i=r.state||"",o=this.getOffsetMethod(i),a=t.Canvas,s=a[o](n.view);s.style.opacity=0;}};},function(t,e,n){n.r(e),e.default={run:function(t){this.toggleVis(t);},stop:function(t){this.toggleVis(t,0);},toggleVis:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1;if(!t.Commands.isActive("preview")){var r=n?"add":"remove";t.Canvas.getFrames().forEach((function(t){t.view.getBody().classList[r]("".concat(e.ppfx,"dashed"));}));}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o={stylePrefix:"",appendTo:"",sortable:1,hidable:1,hideTextnode:1,root:"",showWrapper:1,showHover:1,scrollCanvas:{behavior:"smooth",block:"nearest"},scrollLayers:{behavior:"auto",block:"nearest"},highlightHover:1},a=n(35),s=n(0);function l(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function c(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?l(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):l(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n={};return {name:"LayerManager",init:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return (n=c({},o,{},e)).stylePrefix=e.pStylePrefix,t=n.em,this},getConfig:function(){return n},onLoad:function(){e=new a.a({level:0,config:n,opened:n.opened||{},model:t.get("DomComponents").getWrapper()}),t&&t.on("component:selected",this.componentChanged),this.componentChanged();},postRender:function(){var t=n.appendTo,e=n.root;(e&&this.setRoot(e),t)&&(Object(s.isElement)(t)?t:document.querySelector(t)).appendChild(this.render());},setRoot:function(t){return e.setRoot(t),this},getRoot:function(){return e.model},getAll:function(){return e},componentChanged:function(e){var r=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!r.fromLayers){var i=t.get("opened"),o=t.getSelected(),a=n.scrollLayers,s=o&&o.collection?o.collection.parent:null;for(var l in i)i[l].set("open",0);for(;s;)s.set("open",1),i[s.cid]=s,s=s.collection?s.collection.parent:null;if(o&&a){var c=o.viewLayer&&o.viewLayer.el;c&&c.scrollIntoView(a);}}},render:function(){return e.render().el}}};},function(t,e,n){var r,i,o;
/*!
 * Backbone.Undo.js v0.2
 * 
 * Copyright (c)2013 Oliver Sartun
 * Released under the MIT License
 *
 * Documentation and full license available at
 * https://github.com/osartun/Backbone.Undo.js
 */i=[n(0),n(1)],void 0===(o="function"==typeof(r=function(t,e){var n=Array.prototype.slice;function r(t,e,n){return n.length<=4?t.call(e,n[0],n[1],n[2],n[3]):t.apply(e,n)}function i(t,e){return n.call(t,e)}function o(e,n){return null!=e&&(t.isArray(n)||(n=i(arguments,1)),t.all(n,(function(t){return t in e})))}var a,s,l=(a=!1,s=-1,function(){return a||(s++,a=!0,t.defer((function(){a=!1;}))),s});function c(){this.registeredObjects=[],this.cidIndexes=[];}function u(e,n,r,i){for(var o,a=0,s=n.length;a<s;a++)if(o=n[a]){if("on"===e){if(!i.objectRegistry.register(o))continue}else if(!i.objectRegistry.unregister(o))continue;t.isFunction(o[e])&&o[e]("all",r,i);}}function h(e,n){var r=n.type,i=n.undoTypes,o=!i[r]||i[r][e];t.isFunction(o)&&o(n.object,n.before,n.after,n.options);}function d(e,r,i,o,a){if(!(i.isCurrentlyUndoRedoing||"undo"===e&&-1===i.pointer||"redo"===e&&i.pointer===i.length-1)){i.isCurrentlyUndoRedoing=!0;var s,l,c="undo"===e;for(a?l=c&&i.pointer===i.length-1||!c&&-1===i.pointer?t.clone(i.models):n.apply(i.models,c?[0,i.pointer]:[i.pointer,i.length-1]):(s=i.at(c?i.pointer:i.pointer+1),l=o?i.where({magicFusionIndex:s.get("magicFusionIndex")}):[s]),i.pointer+=(c?-1:1)*l.length;s=c?l.pop():l.shift();)s[e]();i.isCurrentlyUndoRedoing=!1,r.trigger(e,r);}}function f(t,e,n,i){if(t.track&&!t.isCurrentlyUndoRedoing&&e in i&&function(t,e){var n=t.condition,i=typeof n;return "function"===i?!!r(n,t,e):"boolean"!==i||n}(i[e],n)){var a=r(i[e].on,i[e],n);if(o(a,"object","before","after")){if(a.type=e,a.magicFusionIndex=l(),a.undoTypes=i,t.pointer<t.length-1)for(var s=t.length-t.pointer-1;s--;)t.pop();t.pointer=t.length,t.add(a),t.length>t.maximumStackLength&&(t.shift(),t.pointer--);}}}c.prototype={isRegistered:function(e){return e&&e.cid?this.registeredObjects[e.cid]:t.contains(this.registeredObjects,e)},register:function(t){return !this.isRegistered(t)&&(t&&t.cid?(this.registeredObjects[t.cid]=t,this.cidIndexes.push(t.cid)):this.registeredObjects.push(t),!0)},unregister:function(e){if(this.isRegistered(e)){if(e&&e.cid)delete this.registeredObjects[e.cid],this.cidIndexes.splice(t.indexOf(this.cidIndexes,e.cid),1);else {var n=t.indexOf(this.registeredObjects,e);this.registeredObjects.splice(n,1);}return !0}return !1},get:function(){return t.map(this.cidIndexes,(function(t){return this.registeredObjects[t]}),this).concat(this.registeredObjects)}};var p={add:{undo:function(t,e,n,r){t.remove(n,r);},redo:function(t,e,n,r){r.index&&(r.at=r.index),t.add(n,r);},on:function(e,n,r){return {object:n,before:void 0,after:e,options:t.clone(r)}}},remove:{undo:function(t,e,n,r){"index"in r&&(r.at=r.index),t.add(e,r);},redo:function(t,e,n,r){t.remove(e,r);},on:function(e,n,r){return {object:n,before:e,after:void 0,options:t.clone(r)}}},change:{undo:function(e,n,r,i){t.isEmpty(n)?t.each(t.keys(r),e.unset,e):(e.set(n),i&&i.unsetData&&i.unsetData.before&&i.unsetData.before.length&&t.each(i.unsetData.before,e.unset,e));},redo:function(e,n,r,i){t.isEmpty(r)?t.each(t.keys(n),e.unset,e):(e.set(r),i&&i.unsetData&&i.unsetData.after&&i.unsetData.after.length&&t.each(i.unsetData.after,e.unset,e));},on:function(e,n){var r=e.changedAttributes(),i=t.keys(r),o=t.pick(e.previousAttributes(),i),a=t.keys(o),s=(n||(n={})).unsetData={after:[],before:[]};return i.length!=a.length&&(i.length>a.length?t.each(i,(function(t){t in o||s.before.push(t);}),this):t.each(a,(function(t){t in r||s.after.push(t);}))),{object:e,before:o,after:r,options:t.clone(n)}}},reset:{undo:function(t,e,n){t.reset(e);},redo:function(t,e,n){t.reset(n);},on:function(e,n){return {object:e,before:n.previousModels,after:t.clone(e.models)}}}};function g(){}function v(e,n,r,i){if("object"==typeof n)return t.each(n,(function(t,n){2===e?v(e,t,r,i):v(e,n,t,r);}));switch(e){case 0:o(r,"undo","redo","on")&&t.all(t.pick(r,"undo","redo","on"),t.isFunction)&&(i[n]=r);break;case 1:i[n]&&t.isObject(r)&&(i[n]=t.extend({},i[n],r));break;case 2:delete i[n];}return this}g.prototype=p;var m=e.Model.extend({defaults:{type:null,object:null,before:null,after:null,magicFusionIndex:null},undo:function(t){h("undo",this.attributes);},redo:function(t){h("redo",this.attributes);}}),y=e.Collection.extend({model:m,pointer:-1,track:!1,isCurrentlyUndoRedoing:!1,maximumStackLength:1/0,setMaxLength:function(t){this.maximumStackLength=t;}}),b=e.Model.extend({defaults:{maximumStackLength:1/0,track:!1},initialize:function(e){this.stack=new y,this.objectRegistry=new c,this.undoTypes=new g,this.stack.setMaxLength(this.get("maximumStackLength")),this.on("change:maximumStackLength",(function(t,e){this.stack.setMaxLength(e);}),this),e&&e.track&&this.startTracking(),e&&e.register&&(t.isArray(e.register)||t.isArguments(e.register)?r(this.register,this,e.register):this.register(e.register));},startTracking:function(){this.set("track",!0),this.stack.track=!0;},stopTracking:function(){this.set("track",!1),this.stack.track=!1;},isTracking:function(){return this.get("track")},_addToStack:function(t){f(this.stack,t,i(arguments,1),this.undoTypes);},register:function(){u("on",arguments,this._addToStack,this);},unregister:function(){u("off",arguments,this._addToStack,this);},unregisterAll:function(){r(this.unregister,this,this.objectRegistry.get());},undo:function(t){d("undo",this,this.stack,t);},undoAll:function(){d("undo",this,this.stack,!1,!0);},redo:function(t){d("redo",this,this.stack,t);},redoAll:function(){d("redo",this,this.stack,!1,!0);},isAvailable:function(t){var e=this.stack,n=e.length;switch(t){case"undo":return n>0&&e.pointer>-1;case"redo":return n>0&&e.pointer<n-1;default:return !1}},merge:function(e){for(var n,r=t.isArray(e)?e:i(arguments);n=r.pop();)n instanceof b&&n.stack instanceof y&&(n.stack=this.stack);},addUndoType:function(t,e){v(0,t,e,this.undoTypes);},changeUndoType:function(t,e){v(1,t,e,this.undoTypes);},removeUndoType:function(t){v(2,t,void 0,this.undoTypes);},clear:function(){this.stack.reset(),this.stack.pointer=-1;}});return t.extend(b,{defaults:function(e){t.extend(b.prototype.defaults,e);},addUndoType:function(t,e){v(0,t,e,p);},changeUndoType:function(t,e){v(1,t,e,p);},removeUndoType:function(t){v(2,t,void 0,p);}}),e.UndoManager=b})?r.apply(e,i):r)||(t.exports=o);},function(t,e,n){e.a=function(t){var e=this.constructor;return this.then((function(n){return e.resolve(t()).then((function(){return n}))}),(function(n){return e.resolve(t()).then((function(){return e.reject(n)}))}))};},function(t,e,n){var r=n(84),i=n(85),o=n(86);t.exports=function(t,e){return r(t)||i(t,e)||o()};},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t};},function(t,e){t.exports=function(t){if(Array.isArray(t)){for(var e=0,n=new Array(t.length);e<t.length;e++)n[e]=t[e];return n}};},function(t,e){t.exports=function(t){if(Symbol.iterator in Object(t)||"[object Arguments]"===Object.prototype.toString.call(t))return Array.from(t)};},function(t,e){t.exports=function(){throw new TypeError("Invalid attempt to spread non-iterable instance")};},function(t,e,n){n.r(e);var r=n(8),i=n.n(r),o=n(2),a=n.n(o),s=n(0),l=n(38),c=n.n(l);function u(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function h(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?u(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):u(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n={},r={defaults:{"core:undo":{keys:"⌘+z, ctrl+z",handler:"core:undo"},"core:redo":{keys:"⌘+shift+z, ctrl+shift+z",handler:"core:redo"},"core:copy":{keys:"⌘+c, ctrl+c",handler:"core:copy"},"core:paste":{keys:"⌘+v, ctrl+v",handler:"core:paste"},"core:component-next":{keys:"s",handler:"core:component-next"},"core:component-prev":{keys:"w",handler:"core:component-prev"},"core:component-enter":{keys:"d",handler:"core:component-enter"},"core:component-exit":{keys:"a",handler:"core:component-exit"},"core:component-delete":{keys:"backspace, delete",handler:"core:component-delete"}}};return {keymaster:c.a,name:"Keymaps",getConfig:function(){return e},init:function(){var n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return e=h({},r,{},n),t=e.em,this.em=t,this},onLoad:function(){var t=e.defaults;for(var n in t){var r=t[n];this.add(n,r.keys,r.handler);}},add:function(t,e,r){var o=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},a=this.em,l=a.get("Commands"),u=a.getEditor(),h=a.get("Canvas"),d={id:t,keys:e,handler:r},f=n[t];return f&&this.remove(t),n[t]=d,c()(e,(function(e,n){var c={event:e,h:n};if(r=Object(s.isString)(r)?l.get(r):r,o.prevent&&h.getCanvasView().preventDefault(e),!a.isEditing()&&!u.Canvas.isInputFocused()||o.force){"object"==i()(r)?r.run(u,0,c):r(u,0,c);var d=[t,n.shortcut,e];a.trigger.apply(a,["keymap:emit"].concat(d)),a.trigger.apply(a,["keymap:emit:".concat(t)].concat(d));}})),a.trigger("keymap:add",d),d},get:function(t){return n[t]},getAll:function(){return n},remove:function(t){var e=this.em,r=this.get(t);if(r)return delete n[t],c.a.unbind(r.keys),e&&e.trigger("keymap:remove",r),r},removeAll:function(){var t=this;return Object.keys(n).forEach((function(e){return t.remove(e)})),this}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(67),a=n.n(o);function s(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function l(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?s(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):s(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n,r,i={maximumStackLength:500};return {name:"UndoManager",init:function(){var o=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};n=l({},o,{},i),t=n.em,this.em=t,(e=new a.a(l({track:!0,register:[]},n))).changeUndoType("change",{condition:!1}),e.changeUndoType("add",{on:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(!n.avoidStore)return {object:e,before:void 0,after:t,options:l({},n)}}}),e.changeUndoType("remove",{on:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(!n.avoidStore)return {object:e,before:t,after:void 0,options:l({},n)}}});var s={on:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(!r&&(r=t.previousAttributes()),!n.avoidStore){var i={object:t,before:r,after:t.toJSON()};return r=null,i}},undo:function(t,e,n,r){t.set(e);},redo:function(t,e,n,r){t.set(n);}},c=["style","attributes","content","src"];return c.forEach((function(t){return e.addUndoType("change:".concat(t),s)})),e.on("undo redo",(function(){return t.trigger("component:toggled change:canvasOffset")})),["undo","redo"].forEach((function(n){return e.on(n,(function(){return t.trigger(n)}))})),this},getConfig:function(){return n},add:function(t){return e.register(t),this},remove:function(t){return e.unregister(t),this},removeAll:function(){return e.unregisterAll(),this},start:function(){return e.startTracking(),this},stop:function(){return e.stopTracking(),this},undo:function(){var n=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];return !t.isEditing()&&e.undo(n),this},undoAll:function(){return e.undoAll(),this},redo:function(){var n=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];return !t.isEditing()&&e.redo(n),this},redoAll:function(){return e.redoAll(),this},hasUndo:function(){return e.isAvailable("undo")},hasRedo:function(){return e.isAvailable("redo")},getStack:function(){return e.stack},getStackGroup:function(){var t=[],e=[];return this.getStack().forEach((function(n){var r=n.get("magicFusionIndex");e.indexOf(r)<0&&(e.push(r),t.push(n));})),t},getPointer:function(){return this.getStack().pointer},clear:function(){return e.clear(),this},getInstance:function(){return e}}};},function(t,e,n){(function(t){var r=void 0!==t&&t||"undefined"!=typeof self&&self||window,i=Function.prototype.apply;function o(t,e){this._id=t,this._clearFn=e;}e.setTimeout=function(){return new o(i.call(setTimeout,r,arguments),clearTimeout)},e.setInterval=function(){return new o(i.call(setInterval,r,arguments),clearInterval)},e.clearTimeout=e.clearInterval=function(t){t&&t.close();},o.prototype.unref=o.prototype.ref=function(){},o.prototype.close=function(){this._clearFn.call(r,this._id);},e.enroll=function(t,e){clearTimeout(t._idleTimeoutId),t._idleTimeout=e;},e.unenroll=function(t){clearTimeout(t._idleTimeoutId),t._idleTimeout=-1;},e._unrefActive=e.active=function(t){clearTimeout(t._idleTimeoutId);var e=t._idleTimeout;e>=0&&(t._idleTimeoutId=setTimeout((function(){t._onTimeout&&t._onTimeout();}),e));},n(77),e.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==t&&t.setImmediate||this&&this.setImmediate,e.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==t&&t.clearImmediate||this&&this.clearImmediate;}).call(this,n(24));},function(t,e,n){(function(t,e){!function(t,n){if(!t.setImmediate){var r,i,o,a,s,l=1,c={},u=!1,h=t.document,d=Object.getPrototypeOf&&Object.getPrototypeOf(t);d=d&&d.setTimeout?d:t,"[object process]"==={}.toString.call(t.process)?r=function(t){e.nextTick((function(){p(t);}));}:!function(){if(t.postMessage&&!t.importScripts){var e=!0,n=t.onmessage;return t.onmessage=function(){e=!1;},t.postMessage("","*"),t.onmessage=n,e}}()?t.MessageChannel?((o=new MessageChannel).port1.onmessage=function(t){p(t.data);},r=function(t){o.port2.postMessage(t);}):h&&"onreadystatechange"in h.createElement("script")?(i=h.documentElement,r=function(t){var e=h.createElement("script");e.onreadystatechange=function(){p(t),e.onreadystatechange=null,i.removeChild(e),e=null;},i.appendChild(e);}):r=function(t){setTimeout(p,0,t);}:(a="setImmediate$"+Math.random()+"$",s=function(e){e.source===t&&"string"==typeof e.data&&0===e.data.indexOf(a)&&p(+e.data.slice(a.length));},t.addEventListener?t.addEventListener("message",s,!1):t.attachEvent("onmessage",s),r=function(e){t.postMessage(a+e,"*");}),d.setImmediate=function(t){"function"!=typeof t&&(t=new Function(""+t));for(var e=new Array(arguments.length-1),n=0;n<e.length;n++)e[n]=arguments[n+1];var i={callback:t,args:e};return c[l]=i,r(l),l++},d.clearImmediate=f;}function f(t){delete c[t];}function p(t){if(u)setTimeout(p,0,t);else {var e=c[t];if(e){u=!0;try{!function(t){var e=t.callback,r=t.args;switch(r.length){case 0:e();break;case 1:e(r[0]);break;case 2:e(r[0],r[1]);break;case 3:e(r[0],r[1],r[2]);break;default:e.apply(n,r);}}(e);}finally{f(t),u=!1;}}}}}("undefined"==typeof self?void 0===t?this:t:self);}).call(this,n(24),n(78));},function(t,e){var n,r,i=t.exports={};function o(){throw new Error("setTimeout has not been defined")}function a(){throw new Error("clearTimeout has not been defined")}function s(t){if(n===setTimeout)return setTimeout(t,0);if((n===o||!n)&&setTimeout)return n=setTimeout,setTimeout(t,0);try{return n(t,0)}catch(e){try{return n.call(null,t,0)}catch(e){return n.call(this,t,0)}}}!function(){try{n="function"==typeof setTimeout?setTimeout:o;}catch(t){n=o;}try{r="function"==typeof clearTimeout?clearTimeout:a;}catch(t){r=a;}}();var l,c=[],u=!1,h=-1;function d(){u&&l&&(u=!1,l.length?c=l.concat(c):h=-1,c.length&&f());}function f(){if(!u){var t=s(d);u=!0;for(var e=c.length;e;){for(l=c,c=[];++h<e;)l&&l[h].run();h=-1,e=c.length;}l=null,u=!1,function(t){if(r===clearTimeout)return clearTimeout(t);if((r===a||!r)&&clearTimeout)return r=clearTimeout,clearTimeout(t);try{r(t);}catch(e){try{return r.call(null,t)}catch(e){return r.call(this,t)}}}(t);}}function p(t,e){this.fun=t,this.array=e;}function g(){}i.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];c.push(new p(t,e)),1!==c.length||u||s(f);},p.prototype.run=function(){this.fun.apply(null,this.array);},i.title="browser",i.browser=!0,i.env={},i.argv=[],i.version="",i.versions={},i.on=g,i.addListener=g,i.once=g,i.off=g,i.removeListener=g,i.removeAllListeners=g,i.emit=g,i.prependListener=g,i.prependOnceListener=g,i.listeners=function(t){return []},i.binding=function(t){throw new Error("process.binding is not supported")},i.cwd=function(){return "/"},i.chdir=function(t){throw new Error("process.chdir is not supported")},i.umask=function(){return 0};},function(t,e){t.exports=function(t,e){if(null==t)return {};var n,r,i={},o=Object.keys(t);for(r=0;r<o.length;r++)n=o[r],e.indexOf(n)>=0||(i[n]=t[n]);return i};},function(t,e,n){!function(t){var e={script:[["lang",/(javascript|babel)/i,"javascript"],["type",/^(?:text|application)\/(?:x-)?(?:java|ecma)script$|^module$|^$/i,"javascript"],["type",/./,"text/plain"],[null,null,"javascript"]],style:[["lang",/^css$/i,"css"],["type",/^(text\/)?(x-)?(stylesheet|css)$/i,"css"],["type",/./,"text/plain"],[null,null,"css"]]},n={};function r(t,e){var r=t.match(function(t){var e=n[t];return e||(n[t]=new RegExp("\\s+"+t+"\\s*=\\s*('|\")?([^'\"]+)('|\")?\\s*"))}(e));return r?/^\s*(.*?)\s*$/.exec(r[2])[1]:""}function i(t,e){return new RegExp((e?"^":"")+"</s*"+t+"s*>","i")}function o(t,e){for(var n in t)for(var r=e[n]||(e[n]=[]),i=t[n],o=i.length-1;o>=0;o--)r.unshift(i[o]);}t.defineMode("htmlmixed",(function(n,a){var s=t.getMode(n,{name:"xml",htmlMode:!0,multilineTagIndentFactor:a.multilineTagIndentFactor,multilineTagIndentPastTag:a.multilineTagIndentPastTag}),l={},c=a&&a.tags,u=a&&a.scriptTypes;if(o(e,l),c&&o(c,l),u)for(var h=u.length-1;h>=0;h--)l.script.unshift(["type",u[h].matches,u[h].mode]);function d(e,o){var a,c=s.token(e,o.htmlState),u=/\btag\b/.test(c);if(u&&!/[<>\s\/]/.test(e.current())&&(a=o.htmlState.tagName&&o.htmlState.tagName.toLowerCase())&&l.hasOwnProperty(a))o.inTag=a+" ";else if(o.inTag&&u&&/>$/.test(e.current())){var h=/^([\S]+) (.*)/.exec(o.inTag);o.inTag=null;var f=">"==e.current()&&function(t,e){for(var n=0;n<t.length;n++){var i=t[n];if(!i[0]||i[1].test(r(e,i[0])))return i[2]}}(l[h[1]],h[2]),p=t.getMode(n,f),g=i(h[1],!0),v=i(h[1],!1);o.token=function(t,e){return t.match(g,!1)?(e.token=d,e.localState=e.localMode=null,null):function(t,e,n){var r=t.current(),i=r.search(e);return i>-1?t.backUp(r.length-i):r.match(/<\/?$/)&&(t.backUp(r.length),t.match(e,!1)||t.match(r)),n}(t,v,e.localMode.token(t,e.localState))},o.localMode=p,o.localState=t.startState(p,s.indent(o.htmlState,"",""));}else o.inTag&&(o.inTag+=e.current(),e.eol()&&(o.inTag+=" "));return c}return {startState:function(){return {token:d,inTag:null,localMode:null,localState:null,htmlState:t.startState(s)}},copyState:function(e){var n;return e.localState&&(n=t.copyState(e.localMode,e.localState)),{token:e.token,inTag:e.inTag,localMode:e.localMode,localState:n,htmlState:t.copyState(s,e.htmlState)}},token:function(t,e){return e.token(t,e)},indent:function(e,n,r){return !e.localMode||/^\s*<\//.test(n)?s.indent(e.htmlState,n,r):e.localMode.indent?e.localMode.indent(e.localState,n,r):t.Pass},innerMode:function(t){return {state:t.localState||t.htmlState,mode:t.localMode||s}}}}),"xml","javascript","css"),t.defineMIME("text/html","htmlmixed");}(n(12),n(81),n(82),n(41));},function(t,e,n){!function(t){var e={autoSelfClosers:{area:!0,base:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0,menuitem:!0},implicitlyClosed:{dd:!0,li:!0,optgroup:!0,option:!0,p:!0,rp:!0,rt:!0,tbody:!0,td:!0,tfoot:!0,th:!0,tr:!0},contextGrabbers:{dd:{dd:!0,dt:!0},dt:{dd:!0,dt:!0},li:{li:!0},option:{option:!0,optgroup:!0},optgroup:{optgroup:!0},p:{address:!0,article:!0,aside:!0,blockquote:!0,dir:!0,div:!0,dl:!0,fieldset:!0,footer:!0,form:!0,h1:!0,h2:!0,h3:!0,h4:!0,h5:!0,h6:!0,header:!0,hgroup:!0,hr:!0,menu:!0,nav:!0,ol:!0,p:!0,pre:!0,section:!0,table:!0,ul:!0},rp:{rp:!0,rt:!0},rt:{rp:!0,rt:!0},tbody:{tbody:!0,tfoot:!0},td:{td:!0,th:!0},tfoot:{tbody:!0},th:{td:!0,th:!0},thead:{tbody:!0,tfoot:!0},tr:{tr:!0}},doNotIndent:{pre:!0},allowUnquoted:!0,allowMissing:!0,caseFold:!0},n={autoSelfClosers:{},implicitlyClosed:{},contextGrabbers:{},doNotIndent:{},allowUnquoted:!1,allowMissing:!1,allowMissingTagName:!1,caseFold:!1};t.defineMode("xml",(function(r,i){var o,a,s=r.indentUnit,l={},c=i.htmlMode?e:n;for(var u in c)l[u]=c[u];for(var u in i)l[u]=i[u];function h(t,e){function n(n){return e.tokenize=n,n(t,e)}var r=t.next();return "<"==r?t.eat("!")?t.eat("[")?t.match("CDATA[")?n(f("atom","]]>")):null:t.match("--")?n(f("comment","--\x3e")):t.match("DOCTYPE",!0,!0)?(t.eatWhile(/[\w\._\-]/),n(function t(e){return function(n,r){for(var i;null!=(i=n.next());){if("<"==i)return r.tokenize=t(e+1),r.tokenize(n,r);if(">"==i){if(1==e){r.tokenize=h;break}return r.tokenize=t(e-1),r.tokenize(n,r)}}return "meta"}}(1))):null:t.eat("?")?(t.eatWhile(/[\w\._\-]/),e.tokenize=f("meta","?>"),"meta"):(o=t.eat("/")?"closeTag":"openTag",e.tokenize=d,"tag bracket"):"&"==r?(t.eat("#")?t.eat("x")?t.eatWhile(/[a-fA-F\d]/)&&t.eat(";"):t.eatWhile(/[\d]/)&&t.eat(";"):t.eatWhile(/[\w\.\-:]/)&&t.eat(";"))?"atom":"error":(t.eatWhile(/[^&<]/),null)}function d(t,e){var n,r,i=t.next();if(">"==i||"/"==i&&t.eat(">"))return e.tokenize=h,o=">"==i?"endTag":"selfcloseTag","tag bracket";if("="==i)return o="equals",null;if("<"==i){e.tokenize=h,e.state=m,e.tagName=e.tagStart=null;var a=e.tokenize(t,e);return a?a+" tag error":"tag error"}return /[\'\"]/.test(i)?(e.tokenize=(n=i,(r=function(t,e){for(;!t.eol();)if(t.next()==n){e.tokenize=d;break}return "string"}).isInAttribute=!0,r),e.stringStartCol=t.column(),e.tokenize(t,e)):(t.match(/^[^\s\u00a0=<>\"\']*[^\s\u00a0=<>\"\'\/]/),"word")}function f(t,e){return function(n,r){for(;!n.eol();){if(n.match(e)){r.tokenize=h;break}n.next();}return t}}function p(t,e,n){this.prev=t.context,this.tagName=e,this.indent=t.indented,this.startOfLine=n,(l.doNotIndent.hasOwnProperty(e)||t.context&&t.context.noIndent)&&(this.noIndent=!0);}function g(t){t.context&&(t.context=t.context.prev);}function v(t,e){for(var n;;){if(!t.context)return;if(n=t.context.tagName,!l.contextGrabbers.hasOwnProperty(n)||!l.contextGrabbers[n].hasOwnProperty(e))return;g(t);}}function m(t,e,n){return "openTag"==t?(n.tagStart=e.column(),y):"closeTag"==t?b:m}function y(t,e,n){return "word"==t?(n.tagName=e.current(),a="tag",O):l.allowMissingTagName&&"endTag"==t?(a="tag bracket",O(t,0,n)):(a="error",y)}function b(t,e,n){if("word"==t){var r=e.current();return n.context&&n.context.tagName!=r&&l.implicitlyClosed.hasOwnProperty(n.context.tagName)&&g(n),n.context&&n.context.tagName==r||!1===l.matchClosing?(a="tag",w):(a="tag error",x)}return l.allowMissingTagName&&"endTag"==t?(a="tag bracket",w(t,0,n)):(a="error",x)}function w(t,e,n){return "endTag"!=t?(a="error",w):(g(n),m)}function x(t,e,n){return a="error",w(t,0,n)}function O(t,e,n){if("word"==t)return a="attribute",C;if("endTag"==t||"selfcloseTag"==t){var r=n.tagName,i=n.tagStart;return n.tagName=n.tagStart=null,"selfcloseTag"==t||l.autoSelfClosers.hasOwnProperty(r)?v(n,r):(v(n,r),n.context=new p(n,r,i==n.indented)),m}return a="error",O}function C(t,e,n){return "equals"==t?S:(l.allowMissing||(a="error"),O(t,0,n))}function S(t,e,n){return "string"==t?k:"word"==t&&l.allowUnquoted?(a="string",O):(a="error",O(t,0,n))}function k(t,e,n){return "string"==t?k:O(t,0,n)}return h.isInText=!0,{startState:function(t){var e={tokenize:h,state:m,indented:t||0,tagName:null,tagStart:null,context:null};return null!=t&&(e.baseIndent=t),e},token:function(t,e){if(!e.tagName&&t.sol()&&(e.indented=t.indentation()),t.eatSpace())return null;o=null;var n=e.tokenize(t,e);return (n||o)&&"comment"!=n&&(a=null,e.state=e.state(o||n,t,e),a&&(n="error"==a?n+" error":a)),n},indent:function(e,n,r){var i=e.context;if(e.tokenize.isInAttribute)return e.tagStart==e.indented?e.stringStartCol+1:e.indented+s;if(i&&i.noIndent)return t.Pass;if(e.tokenize!=d&&e.tokenize!=h)return r?r.match(/^(\s*)/)[0].length:0;if(e.tagName)return !1!==l.multilineTagIndentPastTag?e.tagStart+e.tagName.length+2:e.tagStart+s*(l.multilineTagIndentFactor||1);if(l.alignCDATA&&/<!\[CDATA\[/.test(n))return 0;var o=n&&/^<(\/)?([\w_:\.-]*)/.exec(n);if(o&&o[1])for(;i;){if(i.tagName==o[2]){i=i.prev;break}if(!l.implicitlyClosed.hasOwnProperty(i.tagName))break;i=i.prev;}else if(o)for(;i;){var a=l.contextGrabbers[i.tagName];if(!a||!a.hasOwnProperty(o[2]))break;i=i.prev;}for(;i&&i.prev&&!i.startOfLine;)i=i.prev;return i?i.indent+s:e.baseIndent||0},electricInput:/<\/[\s\w:]+>$/,blockCommentStart:"\x3c!--",blockCommentEnd:"--\x3e",configuration:l.htmlMode?"html":"xml",helperType:l.htmlMode?"html":"xml",skipAttribute:function(t){t.state==S&&(t.state=O);},xmlCurrentTag:function(t){return t.tagName?{name:t.tagName,close:"closeTag"==t.type}:null},xmlCurrentContext:function(t){for(var e=[],n=t.context;n;n=n.prev)n.tagName&&e.push(n.tagName);return e.reverse()}}})),t.defineMIME("text/xml","xml"),t.defineMIME("application/xml","xml"),t.mimeModes.hasOwnProperty("text/html")||t.defineMIME("text/html",{name:"xml",htmlMode:!0});}(n(12));},function(t,e,n){!function(t){t.defineMode("javascript",(function(e,n){var r,i,o=e.indentUnit,a=n.statementIndent,s=n.jsonld,l=n.json||s,c=n.typescript,u=n.wordCharacters||/[\w$\xa1-\uffff]/,h=function(){function t(t){return {type:t,style:"keyword"}}var e=t("keyword a"),n=t("keyword b"),r=t("keyword c"),i=t("keyword d"),o=t("operator"),a={type:"atom",style:"atom"};return {if:t("if"),while:e,with:e,else:n,do:n,try:n,finally:n,return:i,break:i,continue:i,new:t("new"),delete:r,void:r,throw:r,debugger:t("debugger"),var:t("var"),const:t("var"),let:t("var"),function:t("function"),catch:t("catch"),for:t("for"),switch:t("switch"),case:t("case"),default:t("default"),in:o,typeof:o,instanceof:o,true:a,false:a,null:a,undefined:a,NaN:a,Infinity:a,this:t("this"),class:t("class"),super:t("atom"),yield:r,export:t("export"),import:t("import"),extends:r,await:r}}(),d=/[+\-*&%=<>!?|~^@]/,f=/^@(context|id|value|language|type|container|list|set|reverse|index|base|vocab|graph)"/;function p(t,e,n){return r=t,i=n,e}function g(t,e){var n,r=t.next();if('"'==r||"'"==r)return e.tokenize=(n=r,function(t,e){var r,i=!1;if(s&&"@"==t.peek()&&t.match(f))return e.tokenize=g,p("jsonld-keyword","meta");for(;null!=(r=t.next())&&(r!=n||i);)i=!i&&"\\"==r;return i||(e.tokenize=g),p("string","string")}),e.tokenize(t,e);if("."==r&&t.match(/^\d[\d_]*(?:[eE][+\-]?[\d_]+)?/))return p("number","number");if("."==r&&t.match(".."))return p("spread","meta");if(/[\[\]{}\(\),;\:\.]/.test(r))return p(r);if("="==r&&t.eat(">"))return p("=>","operator");if("0"==r&&t.match(/^(?:x[\dA-Fa-f_]+|o[0-7_]+|b[01_]+)n?/))return p("number","number");if(/\d/.test(r))return t.match(/^[\d_]*(?:n|(?:\.[\d_]*)?(?:[eE][+\-]?[\d_]+)?)?/),p("number","number");if("/"==r)return t.eat("*")?(e.tokenize=v,v(t,e)):t.eat("/")?(t.skipToEnd(),p("comment","comment")):Kt(t,e,1)?(function(t){for(var e,n=!1,r=!1;null!=(e=t.next());){if(!n){if("/"==e&&!r)return;"["==e?r=!0:r&&"]"==e&&(r=!1);}n=!n&&"\\"==e;}}(t),t.match(/^\b(([gimyus])(?![gimyus]*\2))+\b/),p("regexp","string-2")):(t.eat("="),p("operator","operator",t.current()));if("`"==r)return e.tokenize=m,m(t,e);if("#"==r)return t.skipToEnd(),p("error","error");if("<"==r&&t.match("!--")||"-"==r&&t.match("->"))return t.skipToEnd(),p("comment","comment");if(d.test(r))return ">"==r&&e.lexical&&">"==e.lexical.type||(t.eat("=")?"!"!=r&&"="!=r||t.eat("="):/[<>*+\-]/.test(r)&&(t.eat(r),">"==r&&t.eat(r))),p("operator","operator",t.current());if(u.test(r)){t.eatWhile(u);var i=t.current();if("."!=e.lastType){if(h.propertyIsEnumerable(i)){var o=h[i];return p(o.type,o.style,i)}if("async"==i&&t.match(/^(\s|\/\*.*?\*\/)*[\[\(\w]/,!1))return p("async","keyword",i)}return p("variable","variable",i)}}function v(t,e){for(var n,r=!1;n=t.next();){if("/"==n&&r){e.tokenize=g;break}r="*"==n;}return p("comment","comment")}function m(t,e){for(var n,r=!1;null!=(n=t.next());){if(!r&&("`"==n||"$"==n&&t.eat("{"))){e.tokenize=g;break}r=!r&&"\\"==n;}return p("quasi","string-2",t.current())}var y="([{}])";function b(t,e){e.fatArrowAt&&(e.fatArrowAt=null);var n=t.string.indexOf("=>",t.start);if(!(n<0)){if(c){var r=/:\s*(?:\w+(?:<[^>]*>|\[\])?|\{[^}]*\})\s*$/.exec(t.string.slice(t.start,n));r&&(n=r.index);}for(var i=0,o=!1,a=n-1;a>=0;--a){var s=t.string.charAt(a),l=y.indexOf(s);if(l>=0&&l<3){if(!i){++a;break}if(0==--i){"("==s&&(o=!0);break}}else if(l>=3&&l<6)++i;else if(u.test(s))o=!0;else if(/["'\/`]/.test(s))for(;;--a){if(0==a)return;if(t.string.charAt(a-1)==s&&"\\"!=t.string.charAt(a-2)){a--;break}}else if(o&&!i){++a;break}}o&&!i&&(e.fatArrowAt=a);}}var w={atom:!0,number:!0,variable:!0,string:!0,regexp:!0,this:!0,"jsonld-keyword":!0};function x(t,e,n,r,i,o){this.indented=t,this.column=e,this.type=n,this.prev=i,this.info=o,null!=r&&(this.align=r);}function O(t,e){for(var n=t.localVars;n;n=n.next)if(n.name==e)return !0;for(var r=t.context;r;r=r.prev)for(n=r.vars;n;n=n.next)if(n.name==e)return !0}var C={state:null,column:null,marked:null,cc:null};function S(){for(var t=arguments.length-1;t>=0;t--)C.cc.push(arguments[t]);}function k(){return S.apply(null,arguments),!0}function T(t,e){for(var n=e;n;n=n.next)if(n.name==t)return !0;return !1}function j(t){var e=C.state;if(C.marked="def",e.context)if("var"==e.lexical.info&&e.context&&e.context.block){var r=function t(e,n){if(n){if(n.block){var r=t(e,n.prev);return r?r==n.prev?n:new E(r,n.vars,!0):null}return T(e,n.vars)?n:new E(n.prev,new M(e,n.vars),!1)}return null}(t,e.context);if(null!=r)return void(e.context=r)}else if(!T(t,e.localVars))return void(e.localVars=new M(t,e.localVars));n.globalVars&&!T(t,e.globalVars)&&(e.globalVars=new M(t,e.globalVars));}function P(t){return "public"==t||"private"==t||"protected"==t||"abstract"==t||"readonly"==t}function E(t,e,n){this.prev=t,this.vars=e,this.block=n;}function M(t,e){this.name=t,this.next=e;}var D=new M("this",new M("arguments",null));function A(){C.state.context=new E(C.state.context,C.state.localVars,!1),C.state.localVars=D;}function L(){C.state.context=new E(C.state.context,C.state.localVars,!0),C.state.localVars=null;}function N(){C.state.localVars=C.state.context.vars,C.state.context=C.state.context.prev;}function I(t,e){var n=function(){var n=C.state,r=n.indented;if("stat"==n.lexical.type)r=n.lexical.indented;else for(var i=n.lexical;i&&")"==i.type&&i.align;i=i.prev)r=i.indented;n.lexical=new x(r,C.stream.column(),t,null,n.lexical,e);};return n.lex=!0,n}function F(){var t=C.state;t.lexical.prev&&(")"==t.lexical.type&&(t.indented=t.lexical.indented),t.lexical=t.lexical.prev);}function _(t){return function e(n){return n==t?k():";"==t||"}"==n||")"==n||"]"==n?S():k(e)}}function V(t,e){return "var"==t?k(I("vardef",e),bt,_(";"),F):"keyword a"==t?k(I("form"),$,V,F):"keyword b"==t?k(I("form"),V,F):"keyword d"==t?C.stream.match(/^\s*$/,!1)?k():k(I("stat"),B,_(";"),F):"debugger"==t?k(_(";")):"{"==t?k(I("}"),L,at,F,N):";"==t?k():"if"==t?("else"==C.state.lexical.info&&C.state.cc[C.state.cc.length-1]==F&&C.state.cc.pop()(),k(I("form"),$,V,F,kt)):"function"==t?k(Et):"for"==t?k(I("form"),Tt,V,F):"class"==t||c&&"interface"==e?(C.marked="keyword",k(I("form","class"==t?t:e),Nt,F)):"variable"==t?c&&"declare"==e?(C.marked="keyword",k(V)):c&&("module"==e||"enum"==e||"type"==e)&&C.stream.match(/^\s*\w/,!1)?(C.marked="keyword","enum"==e?k(qt):"type"==e?k(Dt,_("operator"),ht,_(";")):k(I("form"),wt,_("{"),I("}"),at,F,F)):c&&"namespace"==e?(C.marked="keyword",k(I("form"),H,V,F)):c&&"abstract"==e?(C.marked="keyword",k(V)):k(I("stat"),Q):"switch"==t?k(I("form"),$,_("{"),I("}","switch"),L,at,F,F,N):"case"==t?k(H,_(":")):"default"==t?k(_(":")):"catch"==t?k(I("form"),A,R,V,F,N):"export"==t?k(I("stat"),Vt,F):"import"==t?k(I("stat"),Ht,F):"async"==t?k(V):"@"==e?k(H,V):S(I("stat"),H,_(";"),F)}function R(t){if("("==t)return k(At,_(")"))}function H(t,e){return W(t,e,!1)}function z(t,e){return W(t,e,!0)}function $(t){return "("!=t?S():k(I(")"),H,_(")"),F)}function W(t,e,n){if(C.state.fatArrowAt==C.stream.start){var r=n?X:Y;if("("==t)return k(A,I(")"),it(At,")"),F,_("=>"),r,N);if("variable"==t)return S(A,wt,_("=>"),r,N)}var i=n?q:U;return w.hasOwnProperty(t)?k(i):"function"==t?k(Et,i):"class"==t||c&&"interface"==e?(C.marked="keyword",k(I("form"),Lt,F)):"keyword c"==t||"async"==t?k(n?z:H):"("==t?k(I(")"),B,_(")"),F,i):"operator"==t||"spread"==t?k(n?z:H):"["==t?k(I("]"),Ut,F,i):"{"==t?ot(et,"}",null,i):"quasi"==t?S(G,i):"new"==t?k(function(t){return function(e){return "."==e?k(t?Z:J):"variable"==e&&c?k(vt,t?q:U):S(t?z:H)}}(n)):"import"==t?k(H):k()}function B(t){return t.match(/[;\}\)\],]/)?S():S(H)}function U(t,e){return ","==t?k(H):q(t,e,!1)}function q(t,e,n){var r=0==n?U:q,i=0==n?H:z;return "=>"==t?k(A,n?X:Y,N):"operator"==t?/\+\+|--/.test(e)||c&&"!"==e?k(r):c&&"<"==e&&C.stream.match(/^([^>]|<.*?>)*>\s*\(/,!1)?k(I(">"),it(ht,">"),F,r):"?"==e?k(H,_(":"),i):k(i):"quasi"==t?S(G,r):";"!=t?"("==t?ot(z,")","call",r):"."==t?k(tt,r):"["==t?k(I("]"),B,_("]"),F,r):c&&"as"==e?(C.marked="keyword",k(ht,r)):"regexp"==t?(C.state.lastType=C.marked="operator",C.stream.backUp(C.stream.pos-C.stream.start-1),k(i)):void 0:void 0}function G(t,e){return "quasi"!=t?S():"${"!=e.slice(e.length-2)?k(G):k(H,K)}function K(t){if("}"==t)return C.marked="string-2",C.state.tokenize=m,k(G)}function Y(t){return b(C.stream,C.state),S("{"==t?V:H)}function X(t){return b(C.stream,C.state),S("{"==t?V:z)}function J(t,e){if("target"==e)return C.marked="keyword",k(U)}function Z(t,e){if("target"==e)return C.marked="keyword",k(q)}function Q(t){return ":"==t?k(F,V):S(U,_(";"),F)}function tt(t){if("variable"==t)return C.marked="property",k()}function et(t,e){return "async"==t?(C.marked="property",k(et)):"variable"==t||"keyword"==C.style?(C.marked="property","get"==e||"set"==e?k(nt):(c&&C.state.fatArrowAt==C.stream.start&&(n=C.stream.match(/^\s*:\s*/,!1))&&(C.state.fatArrowAt=C.stream.pos+n[0].length),k(rt))):"number"==t||"string"==t?(C.marked=s?"property":C.style+" property",k(rt)):"jsonld-keyword"==t?k(rt):c&&P(e)?(C.marked="keyword",k(et)):"["==t?k(H,st,_("]"),rt):"spread"==t?k(z,rt):"*"==e?(C.marked="keyword",k(et)):":"==t?S(rt):void 0;var n;}function nt(t){return "variable"!=t?S(rt):(C.marked="property",k(Et))}function rt(t){return ":"==t?k(z):"("==t?S(Et):void 0}function it(t,e,n){function r(i,o){if(n?n.indexOf(i)>-1:","==i){var a=C.state.lexical;return "call"==a.info&&(a.pos=(a.pos||0)+1),k((function(n,r){return n==e||r==e?S():S(t)}),r)}return i==e||o==e?k():n&&n.indexOf(";")>-1?S(t):k(_(e))}return function(n,i){return n==e||i==e?k():S(t,r)}}function ot(t,e,n){for(var r=3;r<arguments.length;r++)C.cc.push(arguments[r]);return k(I(e,n),it(t,e),F)}function at(t){return "}"==t?k():S(V,at)}function st(t,e){if(c){if(":"==t)return k(ht);if("?"==e)return k(st)}}function lt(t,e){if(c&&(":"==t||"in"==e))return k(ht)}function ct(t){if(c&&":"==t)return C.stream.match(/^\s*\w+\s+is\b/,!1)?k(H,ut,ht):k(ht)}function ut(t,e){if("is"==e)return C.marked="keyword",k()}function ht(t,e){return "keyof"==e||"typeof"==e||"infer"==e?(C.marked="keyword",k("typeof"==e?z:ht)):"variable"==t||"void"==e?(C.marked="type",k(gt)):"|"==e||"&"==e?k(ht):"string"==t||"number"==t||"atom"==t?k(gt):"["==t?k(I("]"),it(ht,"]",","),F,gt):"{"==t?k(I("}"),it(ft,"}",",;"),F,gt):"("==t?k(it(pt,")"),dt,gt):"<"==t?k(it(ht,">"),ht):void 0}function dt(t){if("=>"==t)return k(ht)}function ft(t,e){return "variable"==t||"keyword"==C.style?(C.marked="property",k(ft)):"?"==e||"number"==t||"string"==t?k(ft):":"==t?k(ht):"["==t?k(_("variable"),lt,_("]"),ft):"("==t?S(Mt,ft):void 0}function pt(t,e){return "variable"==t&&C.stream.match(/^\s*[?:]/,!1)||"?"==e?k(pt):":"==t?k(ht):"spread"==t?k(pt):S(ht)}function gt(t,e){return "<"==e?k(I(">"),it(ht,">"),F,gt):"|"==e||"."==t||"&"==e?k(ht):"["==t?k(ht,_("]"),gt):"extends"==e||"implements"==e?(C.marked="keyword",k(ht)):"?"==e?k(ht,_(":"),ht):void 0}function vt(t,e){if("<"==e)return k(I(">"),it(ht,">"),F,gt)}function mt(){return S(ht,yt)}function yt(t,e){if("="==e)return k(ht)}function bt(t,e){return "enum"==e?(C.marked="keyword",k(qt)):S(wt,st,Ct,St)}function wt(t,e){return c&&P(e)?(C.marked="keyword",k(wt)):"variable"==t?(j(e),k()):"spread"==t?k(wt):"["==t?ot(Ot,"]"):"{"==t?ot(xt,"}"):void 0}function xt(t,e){return "variable"!=t||C.stream.match(/^\s*:/,!1)?("variable"==t&&(C.marked="property"),"spread"==t?k(wt):"}"==t?S():"["==t?k(H,_("]"),_(":"),xt):k(_(":"),wt,Ct)):(j(e),k(Ct))}function Ot(){return S(wt,Ct)}function Ct(t,e){if("="==e)return k(z)}function St(t){if(","==t)return k(bt)}function kt(t,e){if("keyword b"==t&&"else"==e)return k(I("form","else"),V,F)}function Tt(t,e){return "await"==e?k(Tt):"("==t?k(I(")"),jt,F):void 0}function jt(t){return "var"==t?k(bt,Pt):"variable"==t?k(Pt):S(Pt)}function Pt(t,e){return ")"==t?k():";"==t?k(Pt):"in"==e||"of"==e?(C.marked="keyword",k(H,Pt)):S(H,Pt)}function Et(t,e){return "*"==e?(C.marked="keyword",k(Et)):"variable"==t?(j(e),k(Et)):"("==t?k(A,I(")"),it(At,")"),F,ct,V,N):c&&"<"==e?k(I(">"),it(mt,">"),F,Et):void 0}function Mt(t,e){return "*"==e?(C.marked="keyword",k(Mt)):"variable"==t?(j(e),k(Mt)):"("==t?k(A,I(")"),it(At,")"),F,ct,N):c&&"<"==e?k(I(">"),it(mt,">"),F,Mt):void 0}function Dt(t,e){return "keyword"==t||"variable"==t?(C.marked="type",k(Dt)):"<"==e?k(I(">"),it(mt,">"),F):void 0}function At(t,e){return "@"==e&&k(H,At),"spread"==t?k(At):c&&P(e)?(C.marked="keyword",k(At)):c&&"this"==t?k(st,Ct):S(wt,st,Ct)}function Lt(t,e){return "variable"==t?Nt(t,e):It(t,e)}function Nt(t,e){if("variable"==t)return j(e),k(It)}function It(t,e){return "<"==e?k(I(">"),it(mt,">"),F,It):"extends"==e||"implements"==e||c&&","==t?("implements"==e&&(C.marked="keyword"),k(c?ht:H,It)):"{"==t?k(I("}"),Ft,F):void 0}function Ft(t,e){return "async"==t||"variable"==t&&("static"==e||"get"==e||"set"==e||c&&P(e))&&C.stream.match(/^\s+[\w$\xa1-\uffff]/,!1)?(C.marked="keyword",k(Ft)):"variable"==t||"keyword"==C.style?(C.marked="property",k(c?_t:Et,Ft)):"number"==t||"string"==t?k(c?_t:Et,Ft):"["==t?k(H,st,_("]"),c?_t:Et,Ft):"*"==e?(C.marked="keyword",k(Ft)):c&&"("==t?S(Mt,Ft):";"==t||","==t?k(Ft):"}"==t?k():"@"==e?k(H,Ft):void 0}function _t(t,e){if("?"==e)return k(_t);if(":"==t)return k(ht,Ct);if("="==e)return k(z);var n=C.state.lexical.prev;return S(n&&"interface"==n.info?Mt:Et)}function Vt(t,e){return "*"==e?(C.marked="keyword",k(Bt,_(";"))):"default"==e?(C.marked="keyword",k(H,_(";"))):"{"==t?k(it(Rt,"}"),Bt,_(";")):S(V)}function Rt(t,e){return "as"==e?(C.marked="keyword",k(_("variable"))):"variable"==t?S(z,Rt):void 0}function Ht(t){return "string"==t?k():"("==t?S(H):S(zt,$t,Bt)}function zt(t,e){return "{"==t?ot(zt,"}"):("variable"==t&&j(e),"*"==e&&(C.marked="keyword"),k(Wt))}function $t(t){if(","==t)return k(zt,$t)}function Wt(t,e){if("as"==e)return C.marked="keyword",k(zt)}function Bt(t,e){if("from"==e)return C.marked="keyword",k(H)}function Ut(t){return "]"==t?k():S(it(z,"]"))}function qt(){return S(I("form"),wt,_("{"),I("}"),it(Gt,"}"),F,F)}function Gt(){return S(wt,Ct)}function Kt(t,e,n){return e.tokenize==g&&/^(?:operator|sof|keyword [bcd]|case|new|export|default|spread|[\[{}\(,;:]|=>)$/.test(e.lastType)||"quasi"==e.lastType&&/\{\s*$/.test(t.string.slice(0,t.pos-(n||0)))}return N.lex=!0,F.lex=!0,{startState:function(t){var e={tokenize:g,lastType:"sof",cc:[],lexical:new x((t||0)-o,0,"block",!1),localVars:n.localVars,context:n.localVars&&new E(null,null,!1),indented:t||0};return n.globalVars&&"object"==typeof n.globalVars&&(e.globalVars=n.globalVars),e},token:function(t,e){if(t.sol()&&(e.lexical.hasOwnProperty("align")||(e.lexical.align=!1),e.indented=t.indentation(),b(t,e)),e.tokenize!=v&&t.eatSpace())return null;var n=e.tokenize(t,e);return "comment"==r?n:(e.lastType="operator"!=r||"++"!=i&&"--"!=i?r:"incdec",function(t,e,n,r,i){var o=t.cc;for(C.state=t,C.stream=i,C.marked=null,C.cc=o,C.style=e,t.lexical.hasOwnProperty("align")||(t.lexical.align=!0);;)if((o.length?o.pop():l?H:V)(n,r)){for(;o.length&&o[o.length-1].lex;)o.pop()();return C.marked?C.marked:"variable"==n&&O(t,r)?"variable-2":e}}(e,n,r,i,t))},indent:function(e,r){if(e.tokenize==v)return t.Pass;if(e.tokenize!=g)return 0;var i,s=r&&r.charAt(0),l=e.lexical;if(!/^\s*else\b/.test(r))for(var c=e.cc.length-1;c>=0;--c){var u=e.cc[c];if(u==F)l=l.prev;else if(u!=kt)break}for(;("stat"==l.type||"form"==l.type)&&("}"==s||(i=e.cc[e.cc.length-1])&&(i==U||i==q)&&!/^[,\.=+\-*:?[\(]/.test(r));)l=l.prev;a&&")"==l.type&&"stat"==l.prev.type&&(l=l.prev);var h=l.type,f=s==h;return "vardef"==h?l.indented+("operator"==e.lastType||","==e.lastType?l.info.length+1:0):"form"==h&&"{"==s?l.indented:"form"==h?l.indented+o:"stat"==h?l.indented+(function(t,e){return "operator"==t.lastType||","==t.lastType||d.test(e.charAt(0))||/[,.]/.test(e.charAt(0))}(e,r)?a||o:0):"switch"!=l.info||f||0==n.doubleIndentSwitch?l.align?l.column+(f?0:1):l.indented+(f?0:o):l.indented+(/^(?:case|default)\b/.test(r)?o:2*o)},electricInput:/^\s*(?:case .*?:|default:|\{|\})$/,blockCommentStart:l?null:"/*",blockCommentEnd:l?null:"*/",blockCommentContinue:l?null:" * ",lineComment:l?null:"//",fold:"brace",closeBrackets:"()[]{}''\"\"``",helperType:l?"json":"javascript",jsonldMode:s,jsonMode:l,expressionAllowed:Kt,skipExpression:function(t){var e=t.cc[t.cc.length-1];e!=H&&e!=z||t.cc.pop();}}})),t.registerHelper("wordChars","javascript",/[\w$]/),t.defineMIME("text/javascript","javascript"),t.defineMIME("text/ecmascript","javascript"),t.defineMIME("application/javascript","javascript"),t.defineMIME("application/x-javascript","javascript"),t.defineMIME("application/ecmascript","javascript"),t.defineMIME("application/json",{name:"javascript",json:!0}),t.defineMIME("application/x-json",{name:"javascript",json:!0}),t.defineMIME("application/ld+json",{name:"javascript",jsonld:!0}),t.defineMIME("text/typescript",{name:"javascript",typescript:!0}),t.defineMIME("application/typescript",{name:"javascript",typescript:!0});}(n(12));},function(t,e,n){!function(t){t.extendMode("css",{commentStart:"/*",commentEnd:"*/",newlineAfterToken:function(t,e){return /^[;{}]$/.test(e)}}),t.extendMode("javascript",{commentStart:"/*",commentEnd:"*/",newlineAfterToken:function(t,e,n,r){return this.jsonMode?/^[\[,{]$/.test(e)||/^}/.test(n):(";"!=e||!r.lexical||")"!=r.lexical.type)&&/^[;{}]$/.test(e)&&!/^;/.test(n)}});var e=/^(a|abbr|acronym|area|base|bdo|big|br|button|caption|cite|code|col|colgroup|dd|del|dfn|em|frame|hr|iframe|img|input|ins|kbd|label|legend|link|map|object|optgroup|option|param|q|samp|script|select|small|span|strong|sub|sup|textarea|tt|var)$/;t.extendMode("xml",{commentStart:"\x3c!--",commentEnd:"--\x3e",newlineAfterToken:function(t,n,r,i){var o=!1;return "html"==this.configuration&&(o=!!i.context&&e.test(i.context.tagName)),!o&&("tag"==t&&/>$/.test(n)&&i.context||/^</.test(r))}}),t.defineExtension("commentRange",(function(e,n,r){var i=this,o=t.innerMode(i.getMode(),i.getTokenAt(n).state).mode;i.operation((function(){if(e)i.replaceRange(o.commentEnd,r),i.replaceRange(o.commentStart,n),n.line==r.line&&n.ch==r.ch&&i.setCursor(n.line,n.ch+o.commentStart.length);else {var t=i.getRange(n,r),a=t.indexOf(o.commentStart),s=t.lastIndexOf(o.commentEnd);a>-1&&s>-1&&s>a&&(t=t.substr(0,a)+t.substring(a+o.commentStart.length,s)+t.substr(s+o.commentEnd.length)),i.replaceRange(t,n,r);}}));})),t.defineExtension("autoIndentRange",(function(t,e){var n=this;this.operation((function(){for(var r=t.line;r<=e.line;r++)n.indentLine(r,"smart");}));})),t.defineExtension("autoFormatRange",(function(e,n){var r=this,i=r.getMode(),o=r.getRange(e,n).split("\n"),a=t.copyState(i,r.getTokenAt(e).state),s=r.getOption("tabSize"),l="",c=0,u=0===e.ch;function h(){l+="\n",u=!0,++c;}for(var d=0;d<o.length;++d){for(var f=new t.StringStream(o[d],s);!f.eol();){var p=t.innerMode(i,a),g=i.token(f,a),v=f.current();f.start=f.pos,u&&!/\S/.test(v)||(l+=v,u=!1),!u&&p.mode.newlineAfterToken&&p.mode.newlineAfterToken(g,v,f.string.slice(f.pos)||o[d+1]||"",p.state)&&h();}!f.pos&&i.blankLine&&i.blankLine(a),!u&&d<o.length-1&&h();}r.operation((function(){r.replaceRange(l,e,n);for(var t=e.line+1,i=e.line+c;t<=i;++t)r.indentLine(t,"smart");r.setSelection(e,r.getCursor(!1));}));}));}(n(12));},function(t,e){t.exports=function(t){if(Array.isArray(t))return t};},function(t,e){t.exports=function(t,e){if(Symbol.iterator in Object(t)||"[object Arguments]"===Object.prototype.toString.call(t)){var n=[],r=!0,i=!1,o=void 0;try{for(var a,s=t[Symbol.iterator]();!(r=(a=s.next()).done)&&(n.push(a.value),!e||n.length!==e);r=!0);}catch(t){i=!0,o=t;}finally{try{r||null==s.return||s.return();}finally{if(i)throw o}}return n}};},function(t,e){t.exports=function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")};},function(t,e,n){n.r(e);var r=n(1),i=n.n(r),o=n(35),a=n(4);e.default=i.a.View.extend({initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.opt=t;var e=t.config||{};this.level=t.level,this.config=e,this.preview=t.preview,this.ppfx=e.pStylePrefix||"",this.pfx=e.stylePrefix||"",this.parent=t.parent,this.parentView=t.parentView;var n=this.pfx,r=this.ppfx,i=this.parent,o=this.collection;this.listenTo(o,"add",this.addTo),this.listenTo(o,"reset resetNavigator",this.render),this.listenTo(o,"remove",this.removeChildren),this.className="".concat(n,"layers");var s=e.em;if(e.sortable&&!this.opt.sorter){var l=s.get("Utils");this.opt.sorter=new l.Sorter({container:e.sortContainer||this.el,containerSel:".".concat(this.className),itemSel:".".concat(n,"layer"),ignoreViewChildren:1,onEndMove:function(t,e,n){var r=e.getSourceModel();s.setSelected(r,{forceChange:1}),s.trigger("".concat(a.eventDrag,":end"),n);},avoidSelectOnEnd:1,nested:1,ppfx:r,pfx:n});}this.sorter=this.opt.sorter||"",this.$el.data("collection",o),i&&this.$el.data("model",i);},removeChildren:function(t){var e=t.viewLayer;e&&(e.remove(),t.viewLayer=0);},addTo:function(t){var e=this.collection.indexOf(t);this.addToCollection(t,null,e);},addToCollection:function(t,e,n){var r=this.level,i=this.parentView,a=e||null,s=new(o.a)({level:r,model:t,parentView:i,config:this.config,sorter:this.sorter,isCountable:this.isCountable,opened:this.opt.opened}).render().el;if(a)a.appendChild(s);else if(void 0!==n){var l="before";this.$el.children().length==n&&(n--,l="after"),n<0?this.$el.append(s):this.$el.children().eq(n)[l](s);}else this.$el.append(s);return s},isCountable:function(t,e){var n=t.get("type"),r=t.get("tagName");return !(("textnode"==n||"br"==r)&&e||!t.get("layerable"))},render:function(){var t=this,e=document.createDocumentFragment(),n=this.el;return n.innerHTML="",this.collection.each((function(n){return t.addToCollection(n,e)})),n.appendChild(e),n.className=this.className,this}});},function(t,e,n){var r={"./CanvasClear":42,"./CanvasClear.js":42,"./CanvasMove":43,"./CanvasMove.js":43,"./CommandAbstract":21,"./CommandAbstract.js":21,"./ComponentDelete":44,"./ComponentDelete.js":44,"./ComponentDrag":45,"./ComponentDrag.js":45,"./ComponentEnter":46,"./ComponentEnter.js":46,"./ComponentExit":47,"./ComponentExit.js":47,"./ComponentNext":48,"./ComponentNext.js":48,"./ComponentPrev":49,"./ComponentPrev.js":49,"./ComponentStyleClear":50,"./ComponentStyleClear.js":50,"./CopyComponent":51,"./CopyComponent.js":51,"./DeleteComponent":52,"./DeleteComponent.js":52,"./ExportTemplate":53,"./ExportTemplate.js":53,"./Fullscreen":54,"./Fullscreen.js":54,"./MoveComponent":55,"./MoveComponent.js":55,"./OpenAssets":56,"./OpenAssets.js":56,"./OpenBlocks":57,"./OpenBlocks.js":57,"./OpenLayers":58,"./OpenLayers.js":58,"./OpenStyleManager":59,"./OpenStyleManager.js":59,"./OpenTraitManager":60,"./OpenTraitManager.js":60,"./PasteComponent":61,"./PasteComponent.js":61,"./Preview":62,"./Preview.js":62,"./Resize":63,"./Resize.js":63,"./SelectComponent":13,"./SelectComponent.js":13,"./SelectPosition":25,"./SelectPosition.js":25,"./ShowOffset":64,"./ShowOffset.js":64,"./SwitchVisibility":65,"./SwitchVisibility.js":65};function i(t){var e=o(t);return n(e)}function o(t){if(!n.o(r,t)){var e=new Error("Cannot find module '"+t+"'");throw e.code="MODULE_NOT_FOUND",e}return r[t]}i.keys=function(){return Object.keys(r)},i.resolve=o,t.exports=i,i.id=88;},function(t,e,n){n.r(e);var r=n(8),i=n.n(r),o=n(2),a=n.n(o),s=n(1),l=n.n(s),c=n(0),u={stylePrefix:"comp-",wrapperId:"wrapper",wrapperName:"Body",wrapper:{removable:!1,copyable:!1,draggable:!1,components:[],traits:[],stylable:["background","background-color","background-image","background-repeat","background-attachment","background-position","background-size"]},components:[],draggableComponents:1,storeWrapper:0,processor:0,voidElements:["area","base","br","col","embed","hr","img","input","keygen","link","menuitem","meta","param","source","track","wbr"]},h=n(4),d=n(20),f=n(5),p=n(34);function g(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var v=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?g(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):g(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{type:"cell",tagName:"td",draggable:["tr"]})},{isComponent:function(t){var e="",n=t.tagName;return "TD"!=n&&"TH"!=n||(e={type:"cell",tagName:n.toLowerCase()}),e}}),m=f.default.extend({});function y(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var b=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?y(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):y(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{tagName:"tr",draggable:["thead","tbody","tfoot"],droppable:["th","td"]})},{isComponent:function(t){return "TR"==t.tagName&&!0}}),w=f.default.extend({});function x(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var O=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?x(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):x(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{type:"table",tagName:"table",droppable:["tbody","thead","tfoot"]}),initialize:function(t,e){h.default.prototype.initialize.apply(this,arguments);var n=this.get("components");!n.length&&n.add({type:"tbody"});}},{isComponent:function(t){var e="";return "TABLE"==t.tagName&&(e={type:"table"}),e}}),C=f.default.extend({events:{}});function S(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var k=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?S(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):S(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{type:"tbody",tagName:"tbody",draggable:["table"],droppable:["tr"],columns:1,rows:1}),initialize:function(t,e){h.default.prototype.initialize.apply(this,arguments);var n=this.get("components"),r=this.get("columns"),i=this.get("rows");if(!n.length){for(var o=[];i--;){for(var a=[],s=r;s--;)a.push({type:"cell",classes:["cell"]});o.push({type:"row",classes:["row"],components:a});}n.add(o);}}},{isComponent:function(t){var e="";return "TBODY"==t.tagName&&(e={type:"tbody"}),e}});function T(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var j=k.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?T(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):T(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},k.prototype.defaults,{type:"thead",tagName:"thead"})},{isComponent:function(t){var e="";return "THEAD"==t.tagName&&(e={type:"thead"}),e}}),P=f.default.extend({}),E=f.default.extend({});function M(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var D=k.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?M(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):M(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},k.prototype.defaults,{type:"tfoot",tagName:"tfoot"})},{isComponent:function(t){var e="";return "TFOOT"==t.tagName&&(e={type:"tfoot"}),e}}),A=f.default.extend({});function L(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var N='xmlns="http://www.w3.org/2000/svg" width="100" viewBox="0 0 24 24" style="fill: rgba(0,0,0,0.15); transform: scale(0.75)"',I=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?L(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):L(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{type:"image",tagName:"img",void:1,droppable:0,editable:1,highlightable:0,resizable:{ratioDefault:1},traits:["alt"],src:"<svg ".concat(N,'>\n        <path d="M8.5 13.5l2.5 3 3.5-4.5 4.5 6H5m16 1V5a2 2 0 0 0-2-2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2z"></path>\n      </svg>'),fallback:"<svg ".concat(N,'>\n        <path d="M2.28 3L1 4.27l2 2V19c0 1.1.9 2 2 2h12.73l2 2L21 21.72 2.28 3m2.55 0L21 19.17V5a2 2 0 0 0-2-2H4.83M8.5 13.5l2.5 3 1-1.25L14.73 18H5l3.5-4.5z"></path>\n      </svg>'),file:""}),initialize:function(t,e){h.default.prototype.initialize.apply(this,arguments);var n=this.get("attributes");n.src&&this.set("src",n.src);},initToolbar:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];h.default.prototype.initToolbar.apply(this,e);var r=this.em;if(r){var i=r.get("Commands"),o="image-editor";if(i.has(o)){for(var a=!1,s=this.get("toolbar"),l=0;l<s.length;l++)if("image-editor"===s[l].command){a=!0;break}a||(s.push({attributes:{class:"fa fa-pencil"},command:o}),this.set("toolbar",s));}}},getAttrToHTML:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=h.default.prototype.getAttrToHTML.apply(this,e),i=this.getSrcResult();return i&&(r.src=i),r},getSrcResult:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.get(t.fallback?"fallback":"src")||"",n=e;return e&&"<svg"===e.substr(0,4)&&(n="data:image/svg+xml;base64,".concat(window.btoa(e))),n},isDefaultSrc:function(){return this.get("src")===Object(c.result)(this,"defaults").src},parseUri:function(t){var e=document.createElement("a");e.href=t;for(var n={},r=e.search.substring(1).split("&"),i=0;i<r.length;i++){var o=r[i].split("="),a=decodeURIComponent(o[0]);a&&(n[a]=decodeURIComponent(o[1]));}return {hostname:e.hostname,pathname:e.pathname,protocol:e.protocol,search:e.search,hash:e.hash,port:e.port,query:n}}},{isComponent:function(t){var e="";return "IMG"==t.tagName&&(e={type:"image"}),e}});function F(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var _=I.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?F(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):F(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},I.prototype.defaults,{type:"map",src:"",void:0,mapUrl:"https://maps.google.com/maps",tagName:"iframe",mapType:"q",address:"",zoom:"1",attributes:{frameborder:0},toolbar:h.default.prototype.defaults.toolbar,traits:[{label:"Address",name:"address",placeholder:"eg. London, UK",changeProp:1},{type:"select",label:"Map type",name:"mapType",changeProp:1,options:[{value:"q",name:"Roadmap"},{value:"w",name:"Satellite"}]},{label:"Zoom",name:"zoom",type:"range",min:"1",max:"20",changeProp:1}]}),initialize:function(t,e){this.get("src")?this.parseFromSrc():this.updateSrc(),I.prototype.initialize.apply(this,arguments),this.listenTo(this,"change:address change:zoom change:mapType",this.updateSrc);},updateSrc:function(){this.set("src",this.getMapUrl());},getMapUrl:function(){var t=this.get("address"),e=this.get("zoom"),n=this.get("mapType");t=t?"&q="+t:"",e=e?"&z="+e:"",n=n?"&t="+n:"";var r=this.get("mapUrl")+"?"+t+e+n;return r+="&output=embed"},parseFromSrc:function(){var t=this.parseUri(this.get("src")).query;t.q&&this.set("address",t.q),t.z&&this.set("zoom",t.z),t.t&&this.set("mapType",t.t);}},{isComponent:function(t){var e="";return "IFRAME"==t.tagName&&/maps\.google\.com/.test(t.src)&&(e={type:"map",src:t.src}),e}}),V=f.default.extend({tagName:"img",events:{dblclick:"onActive",click:"initResize",error:"onError",dragstart:"noDrag"},initialize:function(t){var e=this.model;f.default.prototype.initialize.apply(this,arguments),this.listenTo(e,"change:src",this.updateSrc),this.classEmpty="".concat(this.ppfx,"plh-image");var n=this.config;n.modal&&(this.modal=n.modal),n.am&&(this.am=n.am),this.fetchFile();},fetchFile:function(){if(!this.modelOpt.temporary){var t=this.model,e=t.get("file");if(e)this.em.get("AssetManager").FileUploader().uploadFile({dataTransfer:{files:[e]}},(function(e){var n=e&&e.data&&e.data[0],r=n&&(Object(c.isString)(n)?n:n.src);r&&t.set({src:r});})),t.set("file","");}},updateSrc:function(){var t=this.model,e=this.classEmpty,n=this.$el,r=t.getSrcResult(),i=r&&!t.isDefaultSrc();t.addAttributes({src:r}),n[i?"removeClass":"addClass"](e);},onActive:function(t){t&&t.stopPropagation();var e=this.opts.config.em,n=e?e.get("Editor"):"";n&&this.model.get("editable")&&n.runCommand("open-assets",{target:this.model,types:["image"],accept:"image/*",onSelect:function(){n.Modal.close(),n.AssetManager.setTarget(null);}});},onError:function(){var t=this.model.getSrcResult({fallback:1});t&&(this.el.src=t);},noDrag:function(t){return t.preventDefault(),!1},render:function(){this.renderAttributes(),this.updateSrc();var t=this.$el,e=this.model,n=t.attr("class")||"";return !e.get("src")&&t.attr("class","".concat(n," ").concat(this.classEmpty).trim()),this.postRender(),this}}),R=V.extend({tagName:"div",events:{},initialize:function(t){V.prototype.initialize.apply(this,arguments),this.classEmpty=this.ppfx+"plh-map";},updateSrc:function(){this.getIframe().src=this.model.get("src");},getIframe:function(){if(!this.iframe){var t=document.createElement("iframe");t.src=this.model.get("src"),t.frameBorder=0,t.style.height="100%",t.style.width="100%",t.className=this.ppfx+"no-pointer",this.iframe=t;}return this.iframe},render:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return V.prototype.render.apply(this,e),this.updateClasses(),this.el.appendChild(this.getIframe()),this}});function H(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var z=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?H(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):H(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{type:"text",droppable:!1,editable:!0}),toHTML:function(){return this.trigger("sync:content",{silent:1}),h.default.prototype.toHTML.apply(this,arguments)}});function $(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var W=z.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?$(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):$(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},z.prototype.defaults,{type:"link",tagName:"a",traits:["title","href","target"]}),getAttrToHTML:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=z.prototype.getAttrToHTML.apply(this,e);return delete r.onmousedown,r}},{isComponent:function(t){var e;if("A"==t.tagName){e={type:"link",editable:0};var n=t.childNodes,r=n.length;r||delete e.editable;for(var i=0;i<r;i++){var o=n[i];if(3==o.nodeType&&""!=o.textContent.trim()){delete e.editable;break}}}return e}}),B=n(3);function U(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function q(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?U(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):U(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var G=f.default.prototype,K=f.default.extend({events:{dblclick:"onActive",input:"onInput"},initialize:function(t){G.initialize.apply(this,arguments),this.disableEditing=this.disableEditing.bind(this);var e=this.model,n=this.em;this.listenTo(e,"focus",this.onActive),this.listenTo(e,"change:content",this.updateContentText),this.listenTo(e,"sync:content",this.syncContent),this.rte=n&&n.get("RichTextEditor");},updateContentText:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};!n.fromDisable&&this.disableEditing();},onActive:function(t){if(!this.rteEnabled&&this.model.get("editable")){t&&t.stopPropagation&&t.stopPropagation();var e=this.rte,n=this.em;if(e)try{this.activeRte=e.enable(this,this.activeRte);}catch(t){n.logError(t);}this.toggleEvents(1);}},onDisable:function(){this.disableEditing();},disableEditing:function(){var t=this.model,e=this.rte,n=this.activeRte,r=this.em,i=t.get("editable");if(e&&i){try{e.disable(this,n);}catch(t){r.logError(t);}this.syncContent();}this.toggleEvents();},getContent:function(){var t=(this.rte||{}).activeRte;return t&&"function"==typeof t.getContent?t.getContent():this.getChildrenContainer().innerHTML},syncContent:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model,n=this.rte,r=this.rteEnabled;if(r||t.force){var i=this.getContent(),o=e.components(),a=q({fromDisable:1},t);if(o.length&&o.reset(null,t),e.set("content","",a),n.customRte)e.set("content",i,a);else {var s=function e(n){var r=!!n.get("textable"),i=!["text","default",""].some((function(t){return n.is(t)}))||r;n.set(q({_innertext:!i,editable:i&&n.get("editable"),selectable:i,hoverable:i,removable:r,draggable:r,highlightable:0,copyable:r},!r&&{toolbar:""}),t),n.get("components").each((function(t){return e(t)}));};!t.silent&&e.trigger("change:content",e,"",a),o.add(i,t),o.each((function(t){return s(t)})),o.trigger("resetNavigator");}}},getModelsFromEl:function(t){for(var e=[],n=(t||this.el).childNodes,r=0;r<n.length;r++){var i=n[r],o=i.__cashData&&i.__cashData.model;o&&(o.components=this.getModelsFromEl(i),o.get("content")&&(o.attributes.content=i.textContent),e.push(o));}return e},onInput:function(){var t=this.em,e=["".concat("component",":update"),"".concat("component",":input")].join(" ");t&&t.trigger(e,this.model);},disablePropagation:function(t){t.stopPropagation();},toggleEvents:function(t){var e=this.em,n={on:B.t,off:B.s},r=t?"on":"off";e.setEditing(t),this.rteEnabled=!!t;var i=[this.el.ownerDocument,document];if(n.off(i,"mousedown",this.disableEditing),n[r](i,"mousedown",this.disableEditing),e[r]("toolbar:run:before",this.disableEditing),this.$el.off("mousedown",this.disablePropagation),this.$el[r]("mousedown",this.disablePropagation),this.config.draggableComponents)for(var o=this.el;o;)o.draggable=!t,(o=o.parentNode)&&"BODY"==o.tagName&&(o=0);}}),Y=K.extend({render:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return K.prototype.render.apply(this,e),this.el.addEventListener("click",this.prevDef,!0),this}});function X(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var J=z.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?X(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):X(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},z.prototype.defaults,{tagName:"label",traits:["id","title","for"]})},{isComponent:function(t){if("LABEL"==t.tagName)return {type:"label"}}}),Z=Y.extend({tagName:"span"});function Q(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var tt="yt",et="vi",nt="ytnc",rt=I.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?Q(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):Q(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},I.prototype.defaults,{type:"video",tagName:"video",videoId:"",void:0,provider:"so",ytUrl:"https://www.youtube.com/embed/",ytncUrl:"https://www.youtube-nocookie.com/embed/",viUrl:"https://player.vimeo.com/video/",loop:0,poster:"",muted:0,autoplay:0,controls:1,color:"",rel:1,modestbranding:0,sources:[],attributes:{allowfullscreen:"allowfullscreen"},toolbar:h.default.prototype.defaults.toolbar}),initialize:function(t,e){var n=[],r=this.get("provider");switch(r){case tt:case nt:n=this.getYoutubeTraits();break;case et:n=this.getVimeoTraits();break;default:n=this.getSourceTraits();}this.get("src")&&this.parseFromSrc(),this.set("traits",n),I.prototype.initialize.apply(this,arguments),this.listenTo(this,"change:provider",this.updateTraits),this.listenTo(this,"change:videoId change:provider",this.updateSrc);},initToolbar:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];h.default.prototype.initToolbar.apply(this,e);},parseFromSrc:function(){var t=this.get("provider"),e=this.parseUri(this.get("src")),n=e.query;switch(t){case tt:case nt:case et:var r=e.pathname.split("/").pop();this.set("videoId",r),n.autoplay&&this.set("autoplay",1),n.loop&&this.set("loop",1),0===parseInt(n.controls)&&this.set("controls",0),n.color&&this.set("color",n.color),"0"===n.rel&&this.set("rel",0),"1"===n.modestbranding&&this.set("modestbranding",1);}},updateSrc:function(){switch(this.get("provider")){case tt:this.set("src",this.getYoutubeSrc());break;case nt:this.set("src",this.getYoutubeNoCookieSrc());break;case et:this.set("src",this.getVimeoSrc());}},getAttrToHTML:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];var r=I.prototype.getAttrToHTML.apply(this,e),i=this.get("provider");switch(i){case tt:case nt:case et:break;default:this.get("loop")&&(r.loop="loop"),this.get("autoplay")&&(r.autoplay="autoplay"),this.get("controls")&&(r.controls="controls");}return r},updateTraits:function(){var t=this.get("provider"),e=this.getSourceTraits();switch(t){case tt:case nt:this.set("tagName","iframe"),e=this.getYoutubeTraits();break;case et:this.set("tagName","iframe"),e=this.getVimeoTraits();break;default:this.set("tagName","video");}this.loadTraits(e),this.em.trigger("component:toggled");},getProviderTrait:function(){return {type:"select",label:"Provider",name:"provider",changeProp:1,options:[{value:"so",name:"HTML5 Source"},{value:tt,name:"Youtube"},{value:nt,name:"Youtube (no cookie)"},{value:et,name:"Vimeo"}]}},getSourceTraits:function(){return [this.getProviderTrait(),{label:"Source",name:"src",placeholder:"eg. ./media/video.mp4",changeProp:1},{label:"Poster",name:"poster",placeholder:"eg. ./media/image.jpg"},this.getAutoplayTrait(),this.getLoopTrait(),this.getControlsTrait()]},getYoutubeTraits:function(){return [this.getProviderTrait(),{label:"Video ID",name:"videoId",placeholder:"eg. jNQXAC9IVRw",changeProp:1},this.getAutoplayTrait(),this.getLoopTrait(),this.getControlsTrait(),{type:"checkbox",label:"Related",name:"rel",changeProp:1},{type:"checkbox",label:"Modest",name:"modestbranding",changeProp:1}]},getVimeoTraits:function(){return [this.getProviderTrait(),{label:"Video ID",name:"videoId",placeholder:"eg. 123456789",changeProp:1},{label:"Color",name:"color",placeholder:"eg. FF0000",changeProp:1},this.getAutoplayTrait(),this.getLoopTrait()]},getAutoplayTrait:function(){return {type:"checkbox",label:"Autoplay",name:"autoplay",changeProp:1}},getLoopTrait:function(){return {type:"checkbox",label:"Loop",name:"loop",changeProp:1}},getControlsTrait:function(){return {type:"checkbox",label:"Controls",name:"controls",changeProp:1}},getYoutubeSrc:function(){var t=this.get("videoId"),e=this.get("ytUrl");return e+=t+"?",e+=this.get("autoplay")?"&autoplay=1":"",e+=this.get("controls")?"":"&controls=0&showinfo=0",e+=this.get("loop")?"&loop=1&playlist=".concat(t):"",e+=this.get("rel")?"":"&rel=0",e+=this.get("modestbranding")?"&modestbranding=1":""},getYoutubeNoCookieSrc:function(){var t=this.getYoutubeSrc();return t=t.replace(this.get("ytUrl"),this.get("ytncUrl"))},getVimeoSrc:function(){var t=this.get("viUrl");return t+=this.get("videoId")+"?",t+=this.get("autoplay")?"&autoplay=1":"",t+=this.get("loop")?"&loop=1":"",t+=this.get("controls")?"":"&title=0&portrait=0&badge=0",t+=this.get("color")?"&color="+this.get("color"):""}},{isComponent:function(t){var e="",n=/youtube\.com\/embed/.test(t.src),r=/youtube-nocookie\.com\/embed/.test(t.src),i=/player\.vimeo\.com\/video/.test(t.src),o=n||r||i;return ("VIDEO"==t.tagName||"IFRAME"==t.tagName&&o)&&(e={type:"video"},t.src&&(e.src=t.src),o&&(n?e.provider=tt:r?e.provider=nt:i&&(e.provider=et))),e}}),it=V.extend({tagName:"div",events:{},initialize:function(t){f.default.prototype.initialize.apply(this,arguments);var e=this.model,n=["loop","autoplay","controls","color","rel","modestbranding","poster"],r=n.map((function(t){return "change:".concat(t)})).join(" ");this.listenTo(e,"change:provider",this.updateProvider),this.listenTo(e,"change:src",this.updateSrc),this.listenTo(e,r,this.updateVideo);},updateProvider:function(){var t=this.model.get("provider");this.el.innerHTML="",this.el.appendChild(this.renderByProvider(t));},updateSrc:function(){var t=this.model,e=this.videoEl;if(e){var n=t.get("provider"),r=t.get("src");switch(n){case"yt":r=t.getYoutubeSrc();break;case"ytnc":r=t.getYoutubeNoCookieSrc();break;case"vi":r=t.getVimeoSrc();}e.src=r;}},updateVideo:function(){var t=this.model.get("provider"),e=this.videoEl,n=this.model;switch(t){case"yt":case"ytnc":case"vi":this.model.trigger("change:videoId");break;default:e.loop=n.get("loop"),e.autoplay=n.get("autoplay"),e.controls=n.get("controls"),e.poster=n.get("poster");}},renderByProvider:function(t){var e;switch(t){case"yt":e=this.renderYoutube();break;case"ytnc":e=this.renderYoutubeNoCookie();break;case"vi":e=this.renderVimeo();break;default:e=this.renderSource();}return this.videoEl=e,e},renderSource:function(){var t=document.createElement("video");return t.src=this.model.get("src"),this.initVideoEl(t),t},renderYoutube:function(){var t=document.createElement("iframe");return t.src=this.model.getYoutubeSrc(),t.frameBorder=0,t.setAttribute("allowfullscreen",!0),this.initVideoEl(t),t},renderYoutubeNoCookie:function(){var t=document.createElement("iframe");return t.src=this.model.getYoutubeNoCookieSrc(),t.frameBorder=0,t.setAttribute("allowfullscreen",!0),this.initVideoEl(t),t},renderVimeo:function(){var t=document.createElement("iframe");return t.src=this.model.getVimeoSrc(),t.frameBorder=0,t.setAttribute("allowfullscreen",!0),this.initVideoEl(t),t},initVideoEl:function(t){t.className=this.ppfx+"no-pointer",t.style.height="100%",t.style.width="100%";},render:function(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];V.prototype.render.apply(this,e),this.updateClasses();var r=this.model.get("provider");return this.el.appendChild(this.renderByProvider(r)),this.updateVideo(),this}});function ot(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var at=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?ot(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):ot(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{type:"script",droppable:!1,draggable:!1,layerable:!1})},{isComponent:function(t){if("SCRIPT"==t.tagName){var e={type:"script"};return t.src&&(e.src=t.src,e.onload=t.onload),e}}}),st=V.extend({tagName:"script",events:{},render:function(){var t=this.model,e=t.get("src"),n=this.em,r=n&&n.get("scriptCount")?n.get("scriptCount"):0,i="";if(e){var o=t.get("onload"),a="script"+r,s="script"+(r+1);i="var "+a+" = document.createElement('script');\n"+a+".onload = function(){\n"+(o?o+"();\n":"")+"typeof "+s+"Start == 'function' && "+s+"Start();\n};\n"+a+".src = '"+e+"';\nfunction "+a+"Start() { document.body.appendChild("+a+"); };\n"+(r?"":a+"Start();"),n&&n.set("scriptCount",r+1);}else i=t.get("content");return this.el.innerHTML=i,this}});function lt(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var ct=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?lt(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):lt(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{resizable:{ratioDefault:1},highlightable:0}),getName:function(){var t=this.get("tagName"),e=this.get("custom-name");return t=t.charAt(0).toUpperCase()+t.slice(1),e||t}},{isComponent:function(t){if(SVGElement&&t instanceof SVGElement)return {tagName:t.tagName,type:"svg"}}});function ut(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var ht=ct.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?ut(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):ut(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},ct.prototype.defaults,{selectable:!1,hoverable:!1,layerable:!1})},{isComponent:function(t){if(ct.isComponent(t)&&"svg"!==t.tagName.toLowerCase())return {tagName:t.tagName,type:"svg-in"}}}),dt=f.default.extend({_createElement:function(t){return document.createElementNS("http://www.w3.org/2000/svg",t)}});function ft(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var pt=h.default.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?ft(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):ft(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},h.default.prototype.defaults,{droppable:!1,layerable:!1,editable:!0}),toHTML:function(){return this.get("content").replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(/'/g,"&#039;")}},{isComponent:function(t){var e="";return 3===t.nodeType&&(e={type:"textnode",content:t.textContent}),e}});function gt(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var vt=pt.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?gt(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):gt(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},pt.prototype.defaults),toHTML:function(){return "\x3c!--".concat(this.get("content"),"--\x3e")}},{isComponent:function(t){if(8==t.nodeType)return {tagName:"NULL",type:"comment",content:t.textContent}}}),mt=l.a.View.extend({initialize:function(){var t=this.$el,e=this.model;t.data("model",e),e.view=this;},_createElement:function(){return document.createTextNode(this.model.get("content"))}}),yt=mt.extend({_createElement:function(){return document.createComment(this.model.get("content"))}}),bt=h.default.extend({},{isComponent:function(){return !1}});function wt(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function xt(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?wt(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):wt(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n,r={},o={},a=[{id:"cell",model:v,view:m},{id:"row",model:b,view:w},{id:"table",model:O,view:C},{id:"thead",model:j,view:P},{id:"tbody",model:k,view:E},{id:"tfoot",model:D,view:A},{id:"map",model:_,view:R},{id:"link",model:W,view:Y},{id:"label",model:J,view:Z},{id:"video",model:rt,view:it},{id:"image",model:I,view:V},{id:"script",model:at,view:st},{id:"svg-in",model:ht,view:dt},{id:"svg",model:ct,view:dt},{id:"comment",model:vt,view:yt},{id:"textnode",model:pt,view:mt},{id:"text",model:z,view:K},{id:"wrapper",model:bt,view:f.default},{id:"default",model:h.default,view:f.default}];return {Component:h.default,Components:d.a,ComponentsView:p.a,componentTypes:a,componentsById:o,name:"DomComponents",getConfig:function(){return r},storageKey:function(){var t=[],e=r.stm&&r.stm.getConfig()||{};return e.storeHtml&&t.push("html"),e.storeComponents&&t.push("components"),t},init:function(i){var o=this;for(var s in t=(r=i||{}).em,this.em=t,t&&(r.components=t.config.components||r.components),u)s in r||(r[s]=u[s]);var l=r.pStylePrefix;if(l&&(r.stylePrefix=l+r.stylePrefix),t){r.modal=t.get("Modal")||"",r.am=t.get("AssetManager")||"",t.get("Parser").compTypes=a,t.on("change:componentHovered",this.componentHovered,this);var c=t.get("selected");t.listenTo(c,"add",(function(t,e,n){return o.selectAdd(t,n)})),t.listenTo(c,"remove",(function(t,e,n){return o.selectRemove(t,n)}));}var d=r.components,p=xt({},r.wrapper);return p["custom-name"]=r.wrapperName,p.wrapper=1,p.type="wrapper",d&&d.constructor===Object&&d.wrapper&&(p=xt({},d),d=d.components||[],p.components=[],t&&(t.config.components=d,r.components=d)),(e=new h.default(p,{em:t,config:r,componentTypes:a,domc:this})).set({attributes:{id:"wrapper"}}),n=new f.default({model:e,config:r,componentTypes:a}),this},onLoad:function(){this.setComponents(r.components);},postLoad:function(t){this.handleChanges(this.getWrapper(),null,{avoidStore:1});},handleChanges:function(e,n){var r=this,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},o=e.components(),a=t.get("UndoManager"),s=t.handleUpdates.bind(t),l=this.handleChanges.bind(this),c=this.handleChangesColl.bind(this),u=this.handleRemoves.bind(this);a&&a.add(e),a&&o&&a.add(o);var h="change:style change:content change:attributes change:src";[[e,h,s],[e,"change:components",c],[o,"add",l],[o,"remove",u],[e.get("classes"),"add remove",s]].forEach((function(e){t.stopListening(e[0],e[1],e[2]),t.listenTo(e[0],e[1],e[2]);})),!i.avoidStore&&s("","",i),o.each((function(t){return r.handleChanges(t,n,i)}));},handleChangesColl:function(e,n){var r=t.get("UndoManager");if(r&&n instanceof l.a.Collection){var i=this.handleChanges.bind(this),o=this.handleRemoves.bind(this);r.add(n),[[n,"add",i],[n,"remove",o]].forEach((function(e){t.stopListening(e[0],e[1],e[2]),t.listenTo(e[0],e[1],e[2]);}));}},handleRemoves:function(e,n){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};!r.avoidStore&&t.handleUpdates(e,n,r);},load:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"",e=this.em,n="";!t&&r.stm&&(t=r.em.getCacheLoad());var i=t,o=i.components,a=i.html;if(o)if(Object(c.isObject)(o)||Object(c.isArray)(o))n=o;else try{n=JSON.parse(o);}catch(t){e&&e.logError(t);}else a&&(n=a);var s=n&&n.constructor===Object;return (n&&n.length||s)&&(this.clear(),s?this.getWrapper().set(n):this.getComponents().add(n)),n},store:function(t){if(r.stm){var e={},n=this.storageKey();if(n.indexOf("html")>=0&&(e.html=r.em.getHtml()),n.indexOf("components")>=0){this.em;var i=r.storeWrapper?this.getWrapper():this.getComponents();e.components=JSON.stringify(i);}return t||r.stm.store(e),e}},getComponent:function(){return e},getWrapper:function(){return this.getComponent()},getComponents:function(){return this.getWrapper().get("components")},addComponent:function(t){return this.getComponents().add(t)},render:function(){return n.render().el},clear:function(){return this.getComponents().map((function(t){return t})).forEach((function(t){return t.remove()})),this},setComponents:function(t){this.clear().addComponent(t);},addType:function(t,e){var n=this.em,r=e.model,o=void 0===r?{}:r,s=e.view,l=void 0===s?{}:s,u=e.isComponent,h=e.extend,d=e.extendView,f=e.extendFn,p=void 0===f?[]:f,g=e.extendFnView,v=void 0===g?[]:g,m=this.getType(t),y=this.getType(h),b=this.getType(d),w=y||(m||this.getType("default")),x=w.model,O=b?b.view:w.view,C=function(t,e,n){return t.reduce((function(t,r){var i=e[r],o=n.prototype[r];return i&&o&&(t[r]=function(){o.bind(this).apply(void 0,arguments),i.bind(this).apply(void 0,arguments);}),t}),{})};"object"===i()(o)&&(e.model=x.extend(xt({},o,{},C(p,o,x),{defaults:xt({},x.prototype.defaults,{},Object(c.result)(o,"defaults")||{})}),{isComponent:!m||y||u?u||function(){return 0}:x.isComponent})),"object"===i()(l)&&(e.view=O.extend(xt({},l,{},C(v,l,O)))),m?(m.model=e.model,m.view=e.view):(e.id=t,a.unshift(e));var S="component:type:".concat(m?"update":"add");return n&&n.trigger(S,m||e),this},getType:function(t){for(var e=a,n=0;n<e.length;n++){if(e[n].id==t)return e[n]}},removeType:function(t){var e=a,n=this.getType(t);if(n){var r=e.indexOf(n);return e.splice(r,1),n}},getTypes:function(){return a},selectAdd:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};t&&(t.set({status:"selected"}),["component:selected","component:toggled"].forEach((function(r){return e.em.trigger(r,t,n)})));},selectRemove:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(t){this.em;t.set({status:"",state:""}),["component:deselected","component:toggled"].forEach((function(r){return e.em.trigger(r,t,n)}));}},componentHovered:function(){var t=r.em,e=t.get("componentHovered"),n=t.previous("componentHovered");n&&"hovered"==n.get("status")&&n.set({status:"",state:""}),e&&Object(c.isEmpty)(e.get("status"))&&e.set("status","hovered");},allById:function(){return o}}};},function(t,e,n){n.r(e);var r=n(6),i=n.n(r),o=n(2),a=n.n(o),s=n(3),l=n(0),c=n(14),u=n.n(c),h=n(15),d=n.n(h),f=function(){function t(e,n){var r=this;u()(this,t),this.em=e;var i=n||e.get("Canvas").getFrames().map((function(t){return t.get("root").getEl()})),o=Array.isArray(i)?i:[i];return this.el=i,this.counter=0,Object(l.bindAll)(this,"handleDragEnter","handleDragOver","handleDrop","handleDragLeave"),o.forEach((function(t){return r.toggleEffects(t,1)})),this}return d()(t,[{key:"toggleEffects",value:function(t,e){var n={on:s.t,off:s.s},r=e?"on":"off";n[r](t,"dragenter",this.handleDragEnter),n[r](t,"dragover",this.handleDragOver),n[r](t,"drop",this.handleDrop),n[r](t,"dragleave",this.handleDragLeave);}},{key:"endDrop",value:function(t,e){var n=this.em,r=this.dragStop;this.counter=0,this.over=0,r&&r(t),n.runDefault({preserveSelected:1}),n.trigger("canvas:dragend",e);}},{key:"handleDragLeave",value:function(t){this.updateCounter(-1,t);}},{key:"updateCounter",value:function(t,e){this.counter+=t,0===this.counter&&this.endDrop(1,e);}},{key:"handleDragEnter",value:function(t){var e=this,n=this.em,r=t.dataTransfer;if(this.updateCounter(1,t),!this.over){this.over=1;var i,o,a=n.get("Utils"),s=n.get("Canvas"),l=s.getBody(),c=n.get("dragContent")||"<br>";if(n.stopDefault(),n.inAbsoluteMode()){var u=n.get("DomComponents").getWrapper(),h=u.append({})[0],d=n.get("Commands").run("core:component-drag",{event:t,guidesInfo:1,center:1,target:h,onEnd:function(t,n,i){if(!i.cancelled){var o=u.append(c)[0],a=h.getStyle(),s=a.left,l=a.top,d=a.position;o.addStyle({left:s,top:l,position:d}),e.handleDragEnd(o,r);}h.remove();}});i=function(e){return d.stop(t,{cancel:e})},o=function(t){return c=t};}else {var f=new a.Sorter({em:n,wmargin:1,nested:1,canvasRelative:1,direction:"a",container:l,placer:s.getPlacerEl(),containerSel:"*",itemSel:"*",pfx:"gjs-",onEndMove:function(t){return e.handleDragEnd(t,r)},document:s.getFrameEl().contentDocument});f.setDropContent(c),f.startSort(),this.sorter=f,i=function(t){t&&(f.moved=0),f.endMove();},o=function(t){return f.setDropContent(t)};}this.dragStop=i,this.dragContent=o,n.trigger("canvas:dragenter",r,c);}}},{key:"handleDragEnd",value:function(t,e){if(t){var n=this.em;n.set("dragResult",t),n.trigger("canvas:drop",e,t);}}},{key:"handleDragOver",value:function(t){t.preventDefault(),this.em.trigger("canvas:dragover",t);}},{key:"handleDrop",value:function(t){t.preventDefault();var e=this.dragContent,n=t.dataTransfer,r=this.getContentByData(n).content;t.target.style.border="",r&&e&&e(r),this.endDrop(!r,t);}},{key:"getContentByData",value:function(t){var e=this.em,n=t.types,r=t.files||[],i=e.get("dragContent"),o=t.getData("text");if(r.length){o=[];for(var a=0;a<r.length;a++){var s=r[a],c=s.type.split("/")[0];"image"==c&&o.push({type:c,file:s,attributes:{alt:s.name}});}}else if(i)o=i;else if(Object(l.indexOf)(n,"text/html")>=0)o=t.getData("text/html").replace(/<\/?meta[^>]*>/g,"");else if(Object(l.indexOf)(n,"text/uri-list")>=0)o={type:"link",attributes:{href:o},content:o};else if(Object(l.indexOf)(n,"text/json")>=0){var u=t.getData("text/json");u&&(o=JSON.parse(u));}var h={content:o};return e.trigger("canvas:dragdata",t,h),h}}]),t}(),p={stylePrefix:"cv-",scripts:[],styles:[],customBadgeLabel:"",autoscrollLimit:50,notTextable:["button","a","input[type=checkbox]","input[type=radio]"]},g=n(1),v=n.n(g),m=n(4),y=n(32);function b(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function w(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?b(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):b(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var x=v.a.Model.extend({defaults:{wrapper:"",width:null,height:null,head:"",x:0,y:0,root:0,components:0,styles:0,attributes:{}},initialize:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.attributes,r=n.root,i=n.styles,o=n.components;this.set("head",[]),this.em=e.em;var a={em:e.em,config:e.em.get("DomComponents").getConfig(),frame:this};!r&&this.set("root",new m.default({type:"wrapper",components:o||[]},a)),(!i||Object(l.isString)(i))&&this.set("styles",new y.a(i,a));},remove:function(){this.view=0;var t=this.collection;return t&&t.remove(this)},getHead:function(){return i()(this.get("head"))},setHead:function(t){return this.set("head",i()(t))},addHeadItem:function(t){var e=this.getHead();e.push(t),this.setHead(e);},getHeadByAttr:function(t,e,n){return this.getHead().filter((function(r){return r.attributes&&r.attributes[t]==e&&(!n||n===r.tag)}))[0]},removeHeadByAttr:function(t,e,n){var r=this.getHead(),i=this.getHeadByAttr(t,e,n),o=r.indexOf(i);o>=0&&(r.splice(o,1),this.setHead(r));},addLink:function(t){!this.getHeadByAttr("href",t,"link")&&this.addHeadItem({tag:"link",attributes:{href:t,rel:"stylesheet"}});},removeLink:function(t){this.removeHeadByAttr("href",t,"link");},addScript:function(t){!this.getHeadByAttr("src",t,"script")&&this.addHeadItem({tag:"script",attributes:{src:t}});},removeScript:function(t){this.removeHeadByAttr("src",t,"script");},_emitUpdated:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.em.trigger("frame:updated",w({frame:this},t));}}),O=v.a.Collection.extend({model:x,initialize:function(){Object(l.bindAll)(this,"itemLoaded");},itemLoaded:function(){this.loadedItems++,this.loadedItems>=this.itemsToLoad&&(this.trigger("loaded:all"),this.listenToLoadItems(0));},listenToLoad:function(){this.loadedItems=0,this.itemsToLoad=this.length,this.listenToLoadItems(1);},listenToLoadItems:function(t){var e=this;this.forEach((function(n){return n[t?"on":"off"]("loaded",e.itemLoaded)}));}}),C=v.a.Model.extend({defaults:{frame:"",frames:"",wrapper:"",rulers:!1,zoom:100,x:0,y:0},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.em,n=t.styles,r=void 0===n?[]:n,i=t.scripts,o=void 0===i?[]:i,a=e&&e.getWrapper(),s=e&&e.getStyle(),l=new x({root:a,styles:s},t);r.forEach((function(t){return l.addLink(t)})),o.forEach((function(t){return l.addScript(t)})),this.em=e,this.set("frame",l),this.set("frames",new O([l],t)),this.listenTo(this,"change:zoom",this.onZoomChange),this.listenTo(e,"change:device",this.updateDevice);},updateDevice:function(){var t=this.em,e=t.getDeviceModel(),n=t.getCurrentFrameModel();if(n&&e){var r=e.attributes,i=r.width,o=r.height;n.set({width:i,height:o});}},onZoomChange:function(){this.get("zoom")<1&&this.set("zoom",1);}}),S=n(19),k=n(36),T=n(5),j=n(7);function P(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function E(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?P(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):P(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var M=v.a.View.extend({tagName:"iframe",attributes:{allowfullscreen:"allowfullscreen","data-frame-el":!0},initialize:function(t){Object(l.bindAll)(this,"updateClientY","stopAutoscroll","autoscroll","_emitUpdate");var e=this.model,n=this.el;this.config=E({},t.config||{},{frameView:this}),this.ppfx=this.config.pStylePrefix||"",this.em=this.config.em,this.listenTo(e,"change:head",this.updateHead),e.view=this,Object(s.u)(n,this);},updateHead:function(){var t=this.getHead();Object(j.f)(t),Object(j.c)(t,this.model.getHead());},getEl:function(){return this.el},getWindow:function(){return this.getEl().contentWindow},getDoc:function(){return this.getEl().contentDocument},getHead:function(){return this.getDoc().querySelector("head")},getBody:function(){return this.getDoc().querySelector("body")},getWrapper:function(){return this.getBody().querySelector("[data-gjs-type=wrapper]")},getJsContainer:function(){return this.jsContainer||(this.jsContainer=Object(j.e)("div",{class:"".concat(this.ppfx,"js-cont")})),this.jsContainer},getToolsEl:function(){var t=this.config.frameWrapView;return t&&t.elTools},getGlobalToolsEl:function(){return this.em.get("Canvas").getGlobalToolsEl()},getHighlighter:function(){return this._getTool("[data-hl]")},getBadgeEl:function(){return this._getTool("[data-badge]")},getOffsetViewerEl:function(){return this._getTool("[data-offset]")},getRect:function(){return this.rect||(this.rect=this.el.getBoundingClientRect()),this.rect},getOffsetRect:function(){var t=this.el,e=this.getBody(),n=e.scrollTop,r=e.scrollLeft,i=t.offsetHeight,o=t.offsetWidth;return {top:t.offsetTop,left:t.offsetLeft,height:i,width:o,scrollTop:n,scrollLeft:r,scrollBottom:n+i,scrollRight:r+o}},_getTool:function(t){var e=this.getToolsEl();return this[t]||(this[t]=e.querySelector(t)),this[t]},remove:function(){var t=this.root,e=this.model;this._toggleEffects(),v.a.View.prototype.remove.apply(this,arguments),t.remove(),e.remove();},startAutoscroll:function(){var t=this;this.lastMaxHeight=this.getWrapper().offsetHeight-this.el.offsetHeight,setTimeout((function(){t._toggleAutoscrollFx(1),requestAnimationFrame(t.autoscroll);}),0);},autoscroll:function(){if(this.dragging){var t=this.em.get("Canvas"),e=this.getWindow(),n=this.getBody().scrollTop,r=this.lastClientY||0,i=t.getConfig().autoscrollLimit,o=this.getRect().height-i,a=n;if(r<i&&(a-=i-r),r>o&&(a+=r-o),a!==n&&a>0&&a<this.lastMaxHeight)this.getGlobalToolsEl().style.opacity=0,this.showGlobalTools(),e.scrollTo(0,a);requestAnimationFrame(this.autoscroll);}},updateClientY:function(t){t.preventDefault(),this.lastClientY=Object(s.h)(t).clientY*this.em.getZoomDecimal();},showGlobalTools:Object(l.debounce)((function(){this.getGlobalToolsEl().style.opacity="";}),50),stopAutoscroll:function(){this.dragging&&this._toggleAutoscrollFx();},_toggleAutoscrollFx:function(t){this.dragging=t;var e=this.getWindow(),n=t?"on":"off",r={on:s.t,off:s.s};r[n](e,"mousemove dragover",this.updateClientY),r[n](e,"mouseup",this.stopAutoscroll);},render:function(){var t=this.el,e=this.$el,n=this.ppfx,r=this.config;return e.attr({class:n+"frame"}),r.scripts.length?this.renderScripts():r.renderContent&&(t.onload=this.renderBody.bind(this)),this},renderScripts:function(){var t=this,e=this.el,n=this.config;e.onload=function(){return function n(r){if(r.length>0){var i=r.shift(),o=Object(j.e)("script",E({type:"text/javascript"},Object(l.isString)(i)?{src:i}:i));o.onerror=o.onload=n.bind(null,r),e.contentDocument.head.appendChild(o);}else t.renderBody();}(i()(n.scripts))};},renderBody:function(){var t=this,e=this.config,n=this.model,r=this.ppfx,i=n.get("root"),o=n.get("styles"),a=e.em,c=this.getDoc(),u=this.getHead(),h=this.getBody(),d=(this.getWindow(),a.get("Config")),f=[];e.styles.forEach((function(t){return f.push(Object(l.isString)(t)?{tag:"link",attributes:{href:t,rel:"stylesheet"}}:{tag:"link",attributes:E({rel:"stylesheet"},t)})})),f.length&&Object(j.c)(u,f);Object(j.a)(h,"<style>\n      ".concat(d.baseCss||"","\n\n      .").concat(r,"dashed *[data-highlightable] {\n        outline: 1px dashed rgba(170,170,170,0.7);\n        outline-offset: -2px;\n      }\n\n      .").concat(r,"selected {\n        outline: 3px solid #3b97e3 !important;\n        outline-offset: -3px;\n      }\n\n      .").concat(r,"selected-parent {\n        outline: 2px solid ").concat("#ffca6f"," !important\n      }\n\n      .").concat(r,"no-select {\n        user-select: none;\n        -webkit-user-select:none;\n        -moz-user-select: none;\n      }\n\n      .").concat(r,"freezed {\n        opacity: 0.5;\n        pointer-events: none;\n      }\n\n      .").concat(r,"no-pointer {\n        pointer-events: none;\n      }\n\n      .").concat(r,"plh-image {\n        background: #f5f5f5;\n        border: none;\n        height: 100px;\n        width: 100px;\n        display: block;\n        outline: 3px solid #ffca6f;\n        cursor: pointer;\n        outline-offset: -2px\n      }\n\n      .").concat(r,"grabbing {\n        cursor: grabbing;\n        cursor: -webkit-grabbing;\n      }\n\n      .").concat(r,"is__grabbing {\n        overflow-x: hidden;\n      }\n\n      .").concat(r,"is__grabbing,\n      .").concat(r,"is__grabbing * {\n        cursor: grabbing !important;\n      }\n\n      ").concat(d.canvasCss||"","\n      ").concat(d.protectedCss||"","\n    </style>")),this.root=new T.default({model:i,config:E({},i.config,{frameView:this})}).render(),Object(j.a)(h,this.root.el),Object(j.a)(h,new k.a({collection:o,config:E({},a.get("CssComposer").getConfig(),{frameView:this})}).render().el),Object(j.a)(h,this.getJsContainer()),Object(s.t)(h,"click",(function(t){return t&&"A"==t.target.tagName&&t.preventDefault()})),Object(s.t)(h,"submit",(function(t){return t&&t.preventDefault()})),[{event:"keydown keyup keypress",class:"KeyboardEvent"},{event:"mousemove",class:"MouseEvent"},{event:"wheel",class:"WheelEvent"}].forEach((function(e){return e.event.split(" ").forEach((function(n){c.addEventListener(n,(function(n){return t.el.dispatchEvent(Object(j.d)(n,e.class))}));}))})),this._toggleEffects(1),n.trigger("loaded");},_toggleEffects:function(t){(t?s.t:s.s)(this.getWindow(),"".concat(j.g," resize"),this._emitUpdate);},_emitUpdate:function(){this.model._emitUpdated();}}),D=n(16);function A(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function L(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?A(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):A(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var N=v.a.View.extend({events:{"click [data-action-remove]":"remove","mousedown [data-action-move]":"startDrag"},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};Object(l.bindAll)(this,"onScroll","frameLoaded","updateOffset","remove","startDrag");var n=this.model,r=L({},t.config||e,{frameWrapView:this}),i=r.canvasView,o=r.em;this.cv=i,this.config=r,this.em=o,this.canvas=o&&o.get("Canvas"),this.ppfx=r.pStylePrefix||"",this.frame=new M({model:n,config:r}),this.classAnim="".concat(this.ppfx,"frame-wrapper--anim"),this.listenTo(n,"loaded",this.frameLoaded),this.listenTo(n,"change:x change:y",this.updatePos),this.listenTo(n,"change:width change:height",this.updateSize),this.updatePos(),this.setupDragger();},setupDragger:function(){var t,e,n,r=this,i=this.canvas,o=this.model,a=function(t){i.toggleFramesEvents(t);};this.dragger=new D.a({onStart:function(){var i=o.attributes,s=i.x,l=i.y;n=r.em.getZoomMultiplier(),t=s,e=l,a();},onEnd:function(){return a(1)},setPosition:function(r){o.set({x:t+r.x*n,y:e+r.y*n});}});},startDrag:function(t){t&&this.dragger.start(t);},remove:function(){return v.a.View.prototype.remove.apply(this,arguments),this.frame.remove(),this},updateOffset:Object(l.debounce)((function(){var t=this.em,e=this.$el,n=this.frame;t.runDefault({preserveSelected:1}),e.removeClass(this.classAnim),n.model._emitUpdated();})),updatePos:function(t){var e=this.model,n=this.el,r=e.attributes,i=r.x,o=r.y,a=n.style;this.frame.rect=0,a.left=isNaN(i)?i:"".concat(i,"px"),a.top=isNaN(o)?o:"".concat(o,"px"),t&&this.updateOffset();},updateSize:Object(l.debounce)((function(){this.updateDim();})),updateDim:function(){var t=this.em,e=this.el,n=this.$el,r=this.model,i=this.classAnim,o=r.attributes,a=o.width,s=o.height,c=e.style,u=c.width||"",h=c.height||"",d=a||"",f=s||"",p=u==d&&h==f;if(this.frame.rect=0,n.addClass(i),c.width=Object(l.isNumber)(d)?"".concat(d).concat("px"):d,c.height=Object(l.isNumber)(f)?"".concat(f).concat("px"):f,Object(l.isNull)(a)||Object(l.isNull)(s)){var g=L({},a?{}:{width:e.offsetWidth},{},s?{}:{height:e.offsetHeight});r.set(g,{silent:1});}t.stopDefault({preserveSelected:1}),p?this.updateOffset():n.one(j.g,this.updateOffset);},onScroll:function(){var t=this.frame;this.em.trigger("frame:scroll",{frame:t,body:t.getBody(),target:t.getWindow()});},frameLoaded:function(){this.frame.getWindow().onscroll=this.onScroll,this.updateDim();},render:function(){var t=this.frame,e=this.$el,n=this.ppfx,r=this.cv,i=this.model,o=this.el,a=i.attributes.onRender;t.render(),e.empty().attr({class:"".concat(n,"frame-wrapper")}).append('\n      <div class="'.concat(n,'frame-wrapper__top gjs-two-color" data-frame-top>\n        <div class="').concat(n,'frame-wrapper__name" data-action-move>\n          ').concat(i.get("name")||"",'\n        </div>\n        <div class="').concat(n,'frame-wrapper__top-r">\n          <div class="').concat(n,'frame-wrapper__icon" data-action-remove style="display: none">\n            <svg viewBox="0 0 24 24"><path d="M19 4h-3.5l-1-1h-5l-1 1H5v2h14M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12z"></path></svg>\n          </div>\n        </div>\n      </div>\n      <div class="').concat(n,'frame-wrapper__right" data-frame-right></div>\n      <div class="').concat(n,'frame-wrapper__left" data-frame-left></div>\n      <div class="').concat(n,'frame-wrapper__bottom" data-frame-bottom></div>\n      ')).append(t.el);var s=Object(j.e)("div",{class:"".concat(n,"tools"),style:"pointer-events:none; opacity: 0"},'\n      <div class="'.concat(n,'highlighter" data-hl></div>\n      <div class="').concat(n,'badge" data-badge></div>\n      <div class="').concat(n,'placeholder">\n        <div class="').concat(n,'placeholder-int"></div>\n      </div>\n      <div class="').concat(n,'ghost"></div>\n      <div class="').concat(n,'toolbar" style="pointer-events:all"></div>\n      <div class="').concat(n,'resizer"></div>\n      <div class="').concat(n,'offset-v" data-offset>\n        <div class="gjs-marginName" data-offset-m>\n          <div class="gjs-margin-v-el gjs-margin-v-top" data-offset-m-t></div>\n          <div class="gjs-margin-v-el gjs-margin-v-bottom" data-offset-m-b></div>\n          <div class="gjs-margin-v-el gjs-margin-v-left" data-offset-m-l></div>\n          <div class="gjs-margin-v-el gjs-margin-v-right" data-offset-m-r></div>\n        </div>\n        <div class="gjs-paddingName" data-offset-m>\n          <div class="gjs-padding-v-el gjs-padding-v-top" data-offset-p-t></div>\n          <div class="gjs-padding-v-el gjs-padding-v-bottom" data-offset-p-b></div>\n          <div class="gjs-padding-v-el gjs-padding-v-left" data-offset-p-l></div>\n          <div class="gjs-padding-v-el gjs-padding-v-right" data-offset-p-r></div>\n        </div>\n      </div>\n      <div class="').concat(n,'offset-fixed-v"></div>\n    '));return this.elTools=s,r.toolsWrapper.appendChild(s),a&&a({el:o,elTop:o.querySelector("[data-frame-top]"),elRight:o.querySelector("[data-frame-right]"),elBottom:o.querySelector("[data-frame-bottom]"),elLeft:o.querySelector("[data-frame-left]"),frame:i,frameWrapperView:this,remove:this.remove,startDrag:this.startDrag}),this}}),I=S.a.extend({itemView:N,autoAdd:1,init:function(){this.listenTo(this.collection,"reset",this.render);},onRender:function(){var t=this.config,e=this.$el,n=t.em;n&&e.attr({class:"".concat(n.getConfig("stylePrefix"),"frames")});}});function F(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function _(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?F(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):F(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var V,R=v.a.$,H=v.a.View.extend({events:{wheel:"onWheel"},template:function(){var t=this.pfx;return '\n      <div class="'.concat(t,'canvas__frames" data-frames></div>\n      <div id="').concat(t,'tools" class="').concat(t,'canvas__tools" data-tools></div>\n    ')},initialize:function(t){Object(l.bindAll)(this,"clearOff","onKeyPress","onCanvasMove"),Object(s.t)(window,"scroll resize",this.clearOff);var e=this.model,n=e.get("frames");this.config=t.config||{},this.em=this.config.em||{},this.pfx=this.config.stylePrefix||"",this.ppfx=this.config.pStylePrefix||"",this.className=this.config.stylePrefix+"canvas";var r=this.em,i=this.config;this.frames=new I({collection:n,config:_({},i,{canvasView:this,renderContent:1})}),this.listenTo(r,"change:canvasOffset",this.clearOff),this.listenTo(r,"component:selected",this.checkSelected),this.listenTo(e,"change:zoom change:x change:y",this.updateFrames),this.listenTo(n,"loaded:all",(function(){return r.trigger("loaded")})),this.toggleListeners(1);},checkSelected:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.scroll,r=this.em.get("currentFrame");n&&t.views.forEach((function(t){t._getFrame()!==r&&t.scrollIntoView(n);}));},remove:function(){v.a.View.prototype.remove.apply(this,arguments),this.toggleListeners();},preventDefault:function(t){t&&(t.preventDefault(),t._parentEvent&&t._parentEvent.preventDefault());},onCanvasMove:function(t){},toggleListeners:function(t){this.el;(t?s.t:s.s)(document,"keypress",this.onKeyPress);},onKeyPress:function(t){var e=this.em;" "===Object(s.f)(t)&&1!==e.getZoomDecimal()&&(this.preventDefault(t),e.get("Editor").runCommand("core:canvas-move"));},onWheel:function(t){if((t.ctrlKey||t.metaKey)&&this.em.getConfig("multiFrames")){this.preventDefault(t);var e=this.model,n=Math.max(-1,Math.min(1,t.wheelDelta||-t.detail)),r=e.get("zoom");e.set("zoom",r+2*n);}},updateFrames:function(t){var e=this.em,n=this.model.attributes,r=n.x,i=n.y,o=this.getZoom(),a={preserveSelected:1},s=o?1/o:1;this.framesArea.style.transform="scale(".concat(o,") translate(").concat(r*s,"px, ").concat(i*s,"px)"),this.clearOff(),e.stopDefault(a),e.trigger("canvas:update",t),V&&clearTimeout(V),V=setTimeout((function(){return e.runDefault(a)}),300);},getZoom:function(){return this.em.getZoomDecimal()},isElInViewport:function(t){var e=Object(s.e)(t),n=Object(s.d)(e),r=this.getFrameOffset(e),i=n.top,o=n.left;return i>=0&&o>=0&&i<=r.height&&o<=r.width},offset:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=Object(s.d)(t),r=t.ownerDocument.body,i=e.noScroll;return {top:n.top+(i?0:r.scrollTop),left:n.left+(i?0:r.scrollLeft),width:n.width,height:n.height}},clearOff:function(){this.frmOff=null,this.cvsOff=null;},getFrameOffset:function(t){if(!this.frmOff||t){var e=t?t.ownerDocument.defaultView.frameElement:this.frame.el;this.frmOff=this.offset(e);}return this.frmOff},getCanvasOffset:function(){return this.cvsOff||(this.cvsOff=this.offset(this.el)),this.cvsOff},getElementPos:function(t,e){var n=this.getZoom(),r=e||{},i=this.getFrameOffset(t),o=this.getCanvasOffset(),a=this.offset(t,e),s=r.avoidFrameOffset?0:i.top,l=r.avoidFrameOffset?0:i.left;return {top:a.top*n+s-o.top,left:a.left*n+l-o.left,height:a.height*n,width:a.width*n,zoom:n,rect:a}},getElementOffsets:function(t){var e=this;if(!t||Object(s.p)(t))return {};var n={},r=window.getComputedStyle(t);return ["marginTop","marginRight","marginBottom","marginLeft","paddingTop","paddingRight","paddingBottom","paddingLeft"].forEach((function(t){n[t]=parseFloat(r[t])*e.getZoom();})),n},getPosition:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.frame.el.contentDocument;if(e){var n=e.body,r=this.getZoom(),i=this.getFrameOffset(),o=this.getCanvasOffset(),a=t.noScroll;return {top:i.top+(a?0:n.scrollTop)*r-o.top,left:i.left+(a?0:n.scrollLeft)*r-o.left,width:o.width,height:o.height}}},updateScript:function(t){var e=t.model,n=e.getId();t.scriptContainer||(t.scriptContainer=R('<div data-id="'.concat(n,'">')),this.getJsContainer().appendChild(t.scriptContainer.get(0))),t.el.id=n,t.scriptContainer.html("");var r=document.createElement("script");r.innerHTML="\n        setTimeout(function() {\n          var item = document.getElementById('".concat(n,"');\n          if (!item) return;\n          (function(){\n            ").concat(e.getScriptString(),";\n          }.bind(item))()\n        }, 1);"),setTimeout((function(){return t.scriptContainer.get(0).appendChild(r)}),0);},getJsContainer:function(t){var e=this.getFrameView(t);return e&&e.getJsContainer()},getFrameView:function(t){return t&&t._getFrame()||this.em.get("currentFrame")},render:function(){var t=this.el,e=this.$el,n=this.ppfx,r=this.model,i=this.em,o=this.frames,a=i.get("CssComposer"),s=r.get("wrapper");e.html(this.template());var l=e.find("[data-frames]");this.framesArea=l.get(0),this.wrapper=s,s&&"function"==typeof s.render&&r.get("frame").set({wrapper:s,root:s.getWrapper(),styles:a.getAll()});var c=e.find("[data-tools]");this.toolsWrapper=c.get(0),c.append('\n      <div class="'.concat(n,"tools ").concat(n,'tools-gl" style="pointer-events:none">\n        <div class="').concat(n,'placeholder">\n          <div class="').concat(n,'placeholder-int"></div>\n        </div>\n      </div>\n      <div id="').concat(n,'tools" style="pointer-events:none">\n        <div class="').concat(n,'badge"></div>\n        <div class="').concat(n,'ghost"></div>\n        <div class="').concat(n,'toolbar" style="pointer-events:all"></div>\n        <div class="').concat(n,'resizer"></div>\n        <div class="').concat(n,'offset-v"></div>\n        <div class="').concat(n,'offset-fixed-v"></div>\n      </div>\n    '));var u=t.querySelector("#".concat(n,"tools"));this.hlEl=t.querySelector(".".concat(n,"highlighter")),this.badgeEl=t.querySelector(".".concat(n,"badge")),this.placerEl=t.querySelector(".".concat(n,"placeholder")),this.ghostEl=t.querySelector(".".concat(n,"ghost")),this.toolbarEl=t.querySelector(".".concat(n,"toolbar")),this.resizerEl=t.querySelector(".".concat(n,"resizer")),this.offsetEl=t.querySelector(".".concat(n,"offset-v")),this.fixedOffsetEl=t.querySelector(".".concat(n,"offset-fixed-v")),this.toolsGlobEl=t.querySelector(".".concat(n,"tools-gl")),this.toolsEl=u,this.el.className=this.className;var h=r.get("frames");return h.listenToLoad(),o.render(),i.setCurrentFrame(h.at(0).view),l.append(o.el),this.frame=h.at(0).view,this}});function z(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function $(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?z(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):z(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n={};return {getCanvasView:function(){return e},name:"Canvas",init:function(){var r=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};n=$({},p,{},r,{module:this}),this.em=n.em;var i=n.pStylePrefix;i&&(n.stylePrefix=i+n.stylePrefix),t=new C(r),e=new H({model:t,config:n});var o=n.em.get("DomComponents");return o&&this.setWrapper(o),this.model=t,this.startAutoscroll=this.startAutoscroll.bind(this),this.stopAutoscroll=this.stopAutoscroll.bind(this),this},getConfig:function(){return n},setWrapper:function(e){t.set("wrapper",e);},getElement:function(){return e.el},getFrame:function(e){return e?this.getFrames()[e]:t.get("frame")},getFrameEl:function(){var t=e.frame;return t&&t.el},getFramesEl:function(){return e.framesArea},getWindow:function(){return this.getFrameEl().contentWindow},getDocument:function(){var t=this.getFrameEl();return t&&t.contentDocument},getBody:function(){var t=this.getDocument();return t&&t.body},getWrapperEl:function(){var t=this.getBody();return t&&t.querySelector("#wrapper")},_getCompFrame:function(t){return t&&t._getFrame()},_getLocalEl:function(t,e,n){var r=t,i=this._getCompFrame(e);return r=i?i[n]():r},getGlobalToolsEl:function(){return e.toolsGlobEl},getToolsEl:function(t){return this._getLocalEl(e.toolsEl,t,"getToolsEl")},getHighlighter:function(t){return this._getLocalEl(e.hlEl,t,"getHighlighter")},getBadgeEl:function(t){return this._getLocalEl(e.badgeEl,t,"getBadgeEl")},getPlacerEl:function(){return e.placerEl},getGhostEl:function(){return e.ghostEl},getToolbarEl:function(){return e.toolbarEl},getResizerEl:function(){return e.resizerEl},getOffsetViewerEl:function(t){return this._getLocalEl(e.offsetEl,t,"getOffsetViewerEl")},getFixedOffsetViewerEl:function(){return e.fixedOffsetEl},render:function(){return e.render().el},getOffset:function(){var t=this.offset(this.getFrameEl()),e=this.offset(this.getElement());return {top:t.top-e.top,left:t.left-e.left}},offset:function(t){return e.offset(t)},setCustomBadgeLabel:function(t){n.customBadgeLabel=t;},getElementPos:function(t,n){return e.getElementPos(t,n)},getElementOffsets:function(t){return e.getElementOffsets(t)},getRect:function(){var t=e.getPosition(),n=t.top,r=t.left;return $({},e.getCanvasOffset(),{topScroll:n,leftScroll:r})},getTargetToElementDim:function(t,r){var i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},o=i||{},a=e.getPosition();if(a){var s=o.elPos||e.getElementPos(r),l=i.toRight||0,c=o.targetHeight||t.offsetHeight,u=o.targetWidth||t.offsetWidth,h=o.event||null,d=s.top-c,f=s.left;f+=l?s.width:0;var p=(f=l?f-u:f)<a.left?a.left:f,g=d<a.top?a.top:d,v={top:g=g>s.top+s.height?s.top+s.height:g,left:p,elementTop:s.top,elementLeft:s.left,elementWidth:s.width,elementHeight:s.height,targetWidth:t.offsetWidth,targetHeight:t.offsetHeight,canvasTop:a.top,canvasLeft:a.left,canvasWidth:a.width,canvasHeight:a.height};return h&&n.em&&n.em.trigger(h,v),v}},canvasRectOffset:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=function(t){var e=t.defaultView;return e&&e.frameElement},o=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:1,o=arguments.length>2?arguments[2]:void 0,a=n.em.getZoomDecimal(),s=e?"top":"left",l=t.ownerDocument,c=r.offset?i(l):{},u=c.offsetTop,h=void 0===u?0:u,d=c.offsetLeft,f=void 0===d?0:d,p=l.body||{},g=p.scrollTop,v=void 0===g?0:g,m=p.scrollLeft,y=void 0===m?0:m,b=e?v:y,w=e?h:f;return o[s]-(b-w)*a};return {top:o(t,1,e),left:o(t,0,e)}},getTargetToElementFixed:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n.pos||this.getElementPos(t),i=n.canvasOff||this.canvasRectOffset(t,r),o=e.offsetHeight||0,a=e.offsetWidth||0,s=r.left+r.width,l=this.getCanvasView(),c=l.getPosition(),u=l.getFrameOffset(t),h=n.event,d=-o,f=r.width-a;f=r.left<-f?-r.left:f,f=s>c.width?f-(s-c.width):f;var p=r.height+o,g=p<u.height;i.top<o&&(g?d+=p:d=-i.top<r.height?-i.top:r.height);var v={top:d,left:f,canvasOffsetTop:i.top,canvasOffsetLeft:i.left};return h&&this.em.trigger(h,v),v},getMouseRelativePos:function(t,e){var n=0,r=0,i=(e||{}).subWinOffset,o=t.target.ownerDocument,a=o.defaultView||o.parentWindow,s=a.frameElement,l=i?a.pageYOffset:0,c=i?a.pageXOffset:0;if(s){var u=s.getBoundingClientRect();n=u.top||0,r=u.left||0;}return {y:t.clientY+n-l,x:t.clientX+r-c}},getMouseRelativeCanvas:function(t,n){var r=this.getZoomDecimal(),i=e.getPosition(n),o=i.top,a=i.left;return {y:t.clientY*r+o,x:t.clientX*r+a}},hasFocus:function(){return this.getDocument().hasFocus()},isInputFocused:function(){var t=this.getDocument(),e=["body"].concat(i()(this.getConfig().notTextable)),n=t&&t.activeElement;return n&&!e.some((function(t){return n.matches(t)}))},scrollTo:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=Object(s.e)(t),r=n&&Object(s.j)(n);r&&r.scrollIntoView(e);},startAutoscroll:function(t){var e=t&&t.view||this.em.getCurrentFrame();e&&e.startAutoscroll();},stopAutoscroll:function(t){var e=t&&t.view||this.em.getCurrentFrame();e&&e.stopAutoscroll();},postRender:function(){Object(s.k)(n.em)&&(this.droppable=new f(n.em));},setZoom:function(e){return t.set("zoom",parseFloat(e)),this},getZoom:function(){return parseFloat(t.get("zoom"))},getZoomDecimal:function(){return this.getZoom()/100},getZoomMultiplier:function(){var t=this.getZoomDecimal();return t?1/t:1},toggleFramesEvents:function(t){this.getFramesEl().style.pointerEvents=t?"":"none";},getFrameWrapperEl:function(){return e.frame.getWrapper()},getFrames:function(){return t.get("frames").map((function(t){return t}))},addFrame:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return t.get("frames").add($({},e),$({},n,{em:this.em}))}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(10),a={stylePrefix:"gjs-",components:"",style:"",fromElement:0,noticeOnUnload:!0,showOffsets:!1,showOffsetsSelected:!1,forceClass:!0,height:"900px",width:"100%",log:["warning","error"],baseCss:"\n    * {\n      box-sizing: border-box;\n    }\n    html, body, [data-gjs-type=wrapper] {\n      min-height: 100%;\n    }\n    body {\n      margin: 0;\n      height: 100%;\n      background-color: #fff\n    }\n    [data-gjs-type=wrapper] {\n      overflow: auto;\n      overflow-x: hidden;\n    }\n\n    * ::-webkit-scrollbar-track {\n      background: rgba(0, 0, 0, 0.1)\n    }\n\n    * ::-webkit-scrollbar-thumb {\n      background: rgba(255, 255, 255, 0.2)\n    }\n\n    * ::-webkit-scrollbar {\n      width: 10px\n    }\n  ",protectedCss:"* { box-sizing: border-box; } body {margin: 0;}",canvasCss:"",defaultCommand:"select-comp",showToolbar:1,allowScripts:0,showDevices:1,devicePreviewMode:0,mediaCondition:"max-width",tagVarStart:"{[ ",tagVarEnd:" ]}",keepEmptyTextNodes:0,jsInHtml:!0,nativeDnD:1,multipleSelection:1,exportWrapper:0,wrapperIsBody:1,avoidInlineStyle:1,avoidDefaults:1,clearStyles:0,dragMode:0,cssIcons:"https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css",el:"",i18n:{},undoManager:{},assetManager:{},canvas:{},layers:{},storageManager:{},richTextEditor:{},domComponents:{},modal:{},codeManager:{},panels:{},commands:{},cssComposer:{},selectorManager:{},deviceManager:{devices:[{id:"desktop",name:"Desktop",width:""},{id:"tablet",name:"Tablet",width:"768px",widthMedia:"992px"},{id:"mobileLandscape",name:"Mobile landscape",width:"568px",widthMedia:"768px"},{id:"mobilePortrait",name:"Mobile portrait",width:"320px",widthMedia:"480px"}]},styleManager:{sectors:[{name:"General",open:!1,buildProps:["float","display","position","top","right","left","bottom"]},{name:"Flex",open:!1,buildProps:["flex-direction","flex-wrap","justify-content","align-items","align-content","order","flex-basis","flex-grow","flex-shrink","align-self"]},{name:"Dimension",open:!1,buildProps:["width","height","max-width","min-height","margin","padding"]},{name:"Typography",open:!1,buildProps:["font-family","font-size","font-weight","letter-spacing","color","line-height","text-align","text-shadow"],properties:[{property:"text-align",list:[{value:"left",className:"fa fa-align-left"},{value:"center",className:"fa fa-align-center"},{value:"right",className:"fa fa-align-right"},{value:"justify",className:"fa fa-align-justify"}]}]},{name:"Decorations",open:!1,buildProps:["border-radius-c","background-color","border-radius","border","box-shadow","background"]},{name:"Extra",open:!1,buildProps:["transition","perspective","transform"]}]},blockManager:{},traitManager:{},textViewCode:"Code",keepUnusedStyles:0,multiFrames:0},s=n(0),l=n(1),c=n.n(l),u=n(3);function h(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function d(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?h(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):h(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}c.a.$=o.default;var f,p,g=[n(103),n(105),n(74),n(75),n(102),n(98),n(101),n(100),n(96),n(99),n(97),n(95),n(104),n(93),n(106),n(92),n(89),n(66),n(90),n(107),n(94)],v=c.a.Collection;!function(t){var e=t.$;if(e&&"jQuery"!==e.prototype.constructor.name){var n=e.fn;n.hide=function(){return this.css("display","none")},n.show=function(){return this.css("display","block")},n.focus=function(){var t=this.get(0);return t&&t.focus(),this},n.bind=function(t,e){return this.on(t,e)},n.unbind=function(t,e){if(Object(s.isObject)(t)){for(var n in t)t.hasOwnProperty(n)&&this.off(n,t[n]);return this}return this.off(t,e)},n.click=function(t){return t?this.on("click",t):this.trigger("click")},n.change=function(t){return t?this.on("change",t):this.trigger("change")},n.keydown=function(t){return t?this.on("keydown",t):this.trigger("keydown")},n.delegate=function(t,e,n,r){return r||(r=n),this.on(e,t,(function(t){t.data=n,r(t);}))},n.scrollLeft=function(){var t=this.get(0),e=(t=9==t.nodeType?t.defaultView:t)instanceof Window?t:null;return e?e.pageXOffset:t.scrollLeft||0},n.scrollTop=function(){var t=this.get(0),e=(t=9==t.nodeType?t.defaultView:t)instanceof Window?t:null;return e?e.pageYOffset:t.scrollTop||0};var r=e.prototype.offset;n.offset=function(t){var e,n;return t&&(e=t.top,n=t.left),void 0!==e&&this.css("top","".concat(e,"px")),void 0!==n&&this.css("left","".concat(n,"px")),r.call(this)},e.map=function(t,e){for(var n=[],r=0;r<t.length;r++)n.push(e(t[r],r));return n};var i=Array.prototype.indexOf;e.inArray=function(t,e,n){return null==e?-1:i.call(e,t,n)},e.Event=function(t,n){if(!(this instanceof e.Event))return new e.Event(t,n);this.type=t,this.isDefaultPrevented=function(){return !1};};}}({Backbone:c.a,$:c.a.$});var m={debug:console.log,info:console.info,warning:console.warn,error:console.error},y=c.a.Model.extend({defaults:function(){return {editing:0,selected:new v,clipboard:null,dmode:0,componentHovered:null,previousModel:null,changesCount:0,storables:[],modules:[],toLoad:[],opened:{},device:""}},initialize:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.config=e,this.set("Config",e),this.set("modules",[]),this.set("toLoad",[]),this.set("storables",[]),this.set("selected",new v),this.set("dmode",e.dragMode);var n=e.el,r=e.log,i=!0===r?Object(s.keys)(m):Object(s.isArray)(r)?r:[];Object(s.bindAll)(this,"initBaseColorPicker"),n&&e.fromElement&&(this.config.components=n.innerHTML),this.attrsOrig=n?Object(s.toArray)(n.attributes).reduce((function(t,e){return t[e.nodeName]=e.nodeValue,t}),{}):"",g.forEach((function(e){return t.loadModule(e)})),this.on("change:componentHovered",this.componentHovered,this),this.on("change:changesCount",this.updateChanges,this),i.forEach((function(e){return t.listenLog(e)})),[{from:"change:selectedComponent",to:"component:toggled"}].forEach((function(e){var n=e.from,r=e.to;t.listenTo(t,n,(function(){for(var e=arguments.length,i=new Array(e),o=0;o<e;o++)i[o]=arguments[o];t.trigger.apply(t,[r].concat(i)),t.logWarning("The event '".concat(n,"' is deprecated, replace it with '").concat(r,"'"));}));}));},getContainer:function(){return this.config.el},listenLog:function(t){this.listenTo(this,"log:".concat(t),m[t]);},getConfig:function(t){var e=this.config;return Object(s.isUndefined)(t)?e:e[t]},loadOnStart:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,n=this.get("StorageManager");this.get("toLoad").forEach((function(t){t.onLoad();}));var r=function(){t.get("modules").forEach((function(e){return e.postLoad&&e.postLoad(t)})),e&&e();};n&&n.canAutoload()?this.load(r):r();},updateChanges:function(){var t=this,e=this.get("StorageManager"),n=this.get("changesCount");p&&clearTimeout(p),p=setTimeout((function(){return t.trigger("update")})),this.config.noticeOnUnload&&(window.onbeforeunload=n?function(t){return 1}:null),e.isAutosave()&&n>=e.getStepsBeforeSave()&&this.store();},loadModule:function(t){var e=this.config,n=new(t.default||t),r=n.name.charAt(0).toLowerCase()+n.name.slice(1),i=Object(s.isUndefined)(e[r])?e[n.name]:e[r],o=i||{},a=this.get("StorageManager");if(o.pStylePrefix=e.pStylePrefix||"",Object(s.isUndefined)(i)||i||(o._disable=1),n.storageKey&&n.store&&n.load&&a){o.stm=a;var l="domComponents"==r?"unshift":"push";this.get("storables")[l](n);}return o.em=this,n.init(d({},o)),!n.private&&this.set(n.name,n),n.onLoad&&this.get("toLoad").push(n),this.get("modules").push(n),this},init:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.destroyed&&(this.initialize(e),this.destroyed=0),this.set("Editor",t);},getEditor:function(){return this.get("Editor")},handleUpdates:function(t,e){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};r.temporary||(f&&clearInterval(f),f=setTimeout((function(){r.avoidStore||n.set("changesCount",n.get("changesCount")+1,r);}),0));},componentHovered:function(t,e,n){var r=this.previous("componentHovered");r&&this.trigger("component:unhovered",r,n),e&&this.trigger("component:hovered",e,n);},getSelected:function(){return this.get("selected").last()},getSelectedAll:function(){return this.get("selected").models},setSelected:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=Object(s.isArray)(t),i=r?t:[t],a=this.get("selected");r&&this.removeSelected(a.filter((function(t){return !Object(s.contains)(i,t)}))),i.forEach((function(t){var i=Object(u.g)(t,o.default);i&&!i.get("selectable")||(!r&&e.removeSelected(a.filter((function(t){return t!==i}))),e.addSelected(i,n),i);}));},addSelected:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=Object(u.g)(t,o.default),i=Object(s.isArray)(r)?r:[r];i.forEach((function(t){if(!t||t.get("selectable")){var r=e.get("selected");n.forceChange&&r.remove(t,n),r.push(t,n);}}));},removeSelected:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.get("selected").remove(Object(u.g)(t,o.default),e);},toggleSelected:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=Object(u.g)(t,o.default),i=Object(s.isArray)(r)?r:[r];i.forEach((function(t){e.get("selected").contains(t)?e.removeSelected(t,n):e.addSelected(t,n);}));},setHovered:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=Object(u.g)(t,o.default);n&&!n.get("hoverable")||(e.forceChange&&this.set("componentHovered",""),this.set("componentHovered",n,e));},getHovered:function(){return this.get("componentHovered")},setComponents:function(t){return this.get("DomComponents").setComponents(t)},getComponents:function(){var t=this.get("DomComponents"),e=this.get("CodeManager");if(t&&e){var n=t.getComponents();return e.getCode(n,"json")}},setStyle:function(t){for(var e=this.get("CssComposer").getAll(),n=0,r=e.length;n<r;n++)e.pop();return e.add(t),this},getStyle:function(){return this.get("CssComposer").getAll()},setState:function(t){return this.set("state",t),this},getState:function(){return this.get("state")},getHtml:function(){var t=this.config,e=t.exportWrapper,n=t.wrapperIsBody,r=t.jsInHtml?this.getJs():"",i=this.get("DomComponents").getComponent(),o=this.get("CodeManager").getCode(i,"html",{exportWrapper:e,wrapperIsBody:n});return o+=r?"<script>".concat(r,"<\/script>"):""},getCss:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.config,n=e.wrapperIsBody,r=t.avoidProtected,i=Object(s.isUndefined)(t.keepUnusedStyles)?e.keepUnusedStyles:t.keepUnusedStyles,o=this.get("CssComposer"),a=this.get("DomComponents").getComponent(),l=r?"":e.protectedCss;return l+this.get("CodeManager").getCode(a,"css",{cssc:o,wrapperIsBody:n,keepUnusedStyles:i})},getJs:function(){var t=this.get("DomComponents").getWrapper();return this.get("CodeManager").getCode(t,"js").trim()},store:function(t){var e=this,n=this.get("StorageManager"),r={};if(n)return this.get("storables").forEach((function(t){var e=t.store(1);for(var n in e)r[n]=e[n];})),n.store(r,(function(n){t&&t(n),e.set("changesCount",0),e.trigger("storage:store",r);})),r},load:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null;this.getCacheLoad(1,(function(n){t.get("storables").forEach((function(t){return t.load(n)})),e&&e(n);}));},getCacheLoad:function(t,e){var n=this;if(this.cacheLoad&&!t)return this.cacheLoad;var r=this.get("StorageManager"),i=[];if(!r)return {};this.get("storables").forEach((function(t){var e=t.storageKey;e=Object(s.isFunction)(e)?e():e,(Object(s.isArray)(e)?e:[e]).forEach((function(t){return i.push(t)}));})),r.load(i,(function(t){n.cacheLoad=t,e&&e(t),setTimeout((function(){return n.trigger("storage:load",t)}));}));},getDeviceModel:function(){var t=this.get("device");return this.get("DeviceManager").get(t)},runDefault:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.get("Commands").get(this.config.defaultCommand);e&&!this.defaultRunning&&(e.stop(this,this,t),e.run(this,this,t),this.defaultRunning=1);},stopDefault:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.get("Commands"),n=e.get(this.config.defaultCommand);n&&(n.stop(this,this,t),this.defaultRunning=0);},refreshCanvas:function(){this.set("canvasOffset",null),this.set("canvasOffset",this.get("Canvas").getOffset());},clearSelection:function(t){(t||window).getSelection().removeAllRanges();},getCurrentMedia:function(){var t=this.config,e=this.getDeviceModel(),n=t.mediaCondition,r=t.devicePreviewMode,i=e&&e.get("widthMedia");return e&&i&&!r?"(".concat(n,": ").concat(i,")"):""},getWrapper:function(){return this.get("DomComponents").getWrapper()},setCurrentFrame:function(t){return this.set("currentFrame",t)},getCurrentFrame:function(){return this.get("currentFrame")},getCurrentFrameModel:function(){return (this.getCurrentFrame()||{}).model},getDirtyCount:function(){return this.get("changesCount")},getZoomDecimal:function(){return this.get("Canvas").getZoomDecimal()},getZoomMultiplier:function(){return this.get("Canvas").getZoomMultiplier()},setDragMode:function(t){return this.set("dmode",t)},t:function(){var t;return (t=this.get("I18n")).t.apply(t,arguments)},inAbsoluteMode:function(){return "absolute"===this.get("dmode")},destroyAll:function(){var t=this.config,e=this.getEditor(),n=(t.grapesjs||{}).editors,r=void 0===n?[]:n,i=this.attributes,a=i.DomComponents,s=i.CssComposer,l=i.UndoManager,c=i.Panels,u=i.Canvas,h=i.Keymaps,d=i.RichTextEditor;this.stopDefault(),a.clear(),s.clear(),l.clear().removeAll(),c.getPanels().reset(),u.getCanvasView().remove(),h.removeAll(),d.destroy(),this.view.remove(),this.stopListening(),this.clear({silent:!0}),this.destroyed=1,r.splice(r.indexOf(e),1),Object(o.default)(t.el).empty().attr(this.attrsOrig);},setEditing:function(t){return this.set("editing",t),this},isEditing:function(){return !!this.get("editing")},log:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.ns,r=e.level,i=void 0===r?"debug":r;if(this.trigger("log",t,e),i&&this.trigger("log:".concat(i),t,e),n){var o="log-".concat(n);this.trigger(o,t,e),i&&this.trigger("".concat(o,":").concat(i),t,e);}},logInfo:function(t,e){this.log(t,d({},e,{level:"info"}));},logWarning:function(t,e){this.log(t,d({},e,{level:"warning"}));},logError:function(t,e){this.log(t,d({},e,{level:"error"}));},initBaseColorPicker:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.getConfig(),r=n.colorPicker,i=void 0===r?{}:r,a=n.el,s=n.stylePrefix;return Object(o.default)(t).spectrum(d({containerClassName:"".concat(s,"one-bg ").concat(s,"two-color"),appendTo:a||"body",maxSelectionSize:8,showPalette:!0,palette:[],showAlpha:!0,chooseText:"Ok",cancelText:"⨯"},e,{},i))},data:function(t,e,n){if(t["_gjs-data"]||(t["_gjs-data"]={}),Object(s.isUndefined)(n))return t["_gjs-data"][e];t["_gjs-data"][e]=n;}}),b=c.a.$,w=c.a.View.extend({initialize:function(){var t=this,e=this.model;e.view=this,this.conf=e.config,this.pn=e.get("Panels"),this.cv=e.get("Canvas"),e.on("loaded",(function(){t.pn.active(),t.pn.disableButtons(),setTimeout((function(){e.runDefault(),e.trigger("load",e.get("Editor"));}));}));},render:function(){this.model;var t=this.$el,e=this.conf,n=e.stylePrefix,r=b(e.el||"body ".concat(e.container));return Object(u.a)(e.cssIcons,{unique:1,prepand:1}),t.empty(),e.width&&r.css("width",e.width),e.height&&r.css("height",e.height),t.append(this.cv.render()),t.append(this.pn.render()),t.attr("class","".concat(n,"editor ").concat(n,"one-bg ").concat(n,"two-color")),r.addClass("".concat(n,"editor-cont")).empty().append(t),this}});function x(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function O(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?x(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):x(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var C=function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=O({},a,{},t);e.pStylePrefix=e.stylePrefix;var n=new y(e),r=new w({model:n,config:e});return {$:o.default,editor:n,init:function(){var t=this,i=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return n.init(this,O({},e,{},i)),["I18n","Utils","Config","Commands","Keymaps","Modal","Panels","Canvas","Parser","CodeManager","UndoManager","RichTextEditor","DomComponents",["Components","DomComponents"],"LayerManager",["Layers","LayerManager"],"CssComposer",["Css","CssComposer"],"StorageManager",["Storage","StorageManager"],"AssetManager",["Assets","AssetManager"],"BlockManager",["Blocks","BlockManager"],"TraitManager",["Traits","TraitManager"],"SelectorManager",["Selectors","SelectorManager"],"StyleManager",["Styles","StyleManager"],"DeviceManager",["Devices","DeviceManager"]].forEach((function(e){Array.isArray(e)?t[e[0]]=n.get(e[1]):t[e]=n.get(e);})),n.on("loaded",(function(){t.UndoManager.clear(),n.get("modules").forEach((function(t){t.postRender&&t.postRender(r);}));})),this},getConfig:function(t){return n.getConfig(t)},getHtml:function(t){return n.getHtml(t)},getCss:function(t){return n.getCss(t)},getJs:function(){return n.getJs()},getComponents:function(){return n.get("DomComponents").getComponents()},getWrapper:function(){return n.get("DomComponents").getWrapper()},setComponents:function(t){return n.setComponents(t),this},addComponents:function(t,e){return this.getWrapper().append(t,e)},getStyle:function(){return n.get("CssComposer").getAll()},setStyle:function(t){return n.setStyle(t),this},getSelected:function(){return n.getSelected()},getSelectedAll:function(){return n.getSelectedAll()},getSelectedToStyle:function(){var t=n.getSelected();if(t)return this.StyleManager.getModelToStyle(t)},select:function(t,e){return n.setSelected(t,e),this},selectAdd:function(t){return n.addSelected(t),this},selectRemove:function(t){return n.removeSelected(t),this},selectToggle:function(t){return n.toggleSelected(t),this},setDevice:function(t){return n.set("device",t),this},getDevice:function(){return n.get("device")},runCommand:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return n.get("Commands").run(t,e)},stopCommand:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return n.get("Commands").stop(t,e)},store:function(t){return n.store(t)},load:function(t){return n.load(t)},getContainer:function(){return e.el},getDirtyCount:function(){return n.getDirtyCount()},refresh:function(){n.refreshCanvas();},setCustomRte:function(t){this.RichTextEditor.customRte=t;},setCustomParserCss:function(t){return this.Parser.getConfig().parserCss=t,this},setDragMode:function(t){return n.setDragMode(t),this},log:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return n.log(t,e),this},t:function(){return n.t.apply(n,arguments)},on:function(t,e){return n.on(t,e),this},once:function(t,e){return n.once(t,e),this},off:function(t,e){return n.off(t,e),this},trigger:function(t){return n.trigger.apply(n,arguments),this},destroy:function(){return n.destroyAll()},getEl:function(){return r.el},getModel:function(){return n},render:function(){return r.render(),r.el}}},S={plugins:[]};function k(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function T(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?k(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):k(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}!function(){if(function(){for(var t,e=window.navigator.userAgent,n=[["edge",/Edge\/([0-9\._]+)/],["ie",/MSIE\s(7\.0)/],["ie",/MSIE\s([0-9\.]+);.*Trident\/[4-7].0/],["ie",/Trident\/7\.0.*rv\:([0-9\.]+).*\).*Gecko$/]],r=0;r<n.length;r++){if(t=n[r][1].exec(e))break}return !!t}()){var t=DOMImplementation.prototype.createHTMLDocument;DOMImplementation.prototype.createHTMLDocument=function(e){return e||(e=""),t.apply(document.implementation,[e])};}}();var j=new function(t){var e=t||{};for(var n in S)n in e||(e[n]=S[n]);var r={};return {add:function(t,e){return r[t]?r[t]:(r[t]=e,e)},get:function(t){return r[t]},getAll:function(){return r}}},P=[],E={autorender:1,plugins:[],pluginsOpts:{}};e.default={$:o.default,editors:P,plugins:j,version:"0.16.12",init:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.container;if(!e)throw new Error("'container' is required");(t=T({},E,{},t,{grapesjs:this})).el=Object(s.isElement)(e)?e:document.querySelector(e);var n=new C(t).init();return t.plugins.forEach((function(e){var r=j.get(e),i=t.pluginsOpts[e]||{};if(!r){var o=window[e];r=o&&o.default?o.default:o;}r?r(n,i):Object(s.isFunction)(e)?e(n,i):console.warn("Plugin ".concat(e," not found"));})),n.getModel().loadOnStart(),t.autorender&&n.render(),P.push(n),n}};},function(t,e,n){n.r(e);var r=n(0),i={stylePrefix:"trt-",appendTo:"",optionsTarget:[{value:!1},{value:"_blank"}]},o=n(19),a=n(2),s=n.n(a),l=n(69),c=n.n(l),u=n(1),h=n.n(u),d=n(3);function f(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var p=h.a.$,g=h.a.View.extend({events:{},eventCapture:["change"],appendInput:1,attributes:function(){return this.model.get("attributes")},templateLabel:function(){var t=this.ppfx,e=this.getLabel();return '<div class="'.concat(t,'label" title="').concat(e,'">').concat(e,"</div>")},templateInput:function(){var t=this.clsField;return '<div class="'.concat(t,'" data-input></div>')},initialize:function(){var t=this,e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=e.config,r=void 0===n?{}:n,i=this.model,o=this.eventCapture,a=i.target,s=i.attributes.type;this.config=r,this.em=r.em,this.pfx=r.stylePrefix||"",this.ppfx=r.pStylePrefix||"",this.target=a;var l=this.ppfx;this.clsField="".concat(l,"field ").concat(l,"field-").concat(s),[["change:value",this.onValueChange],["remove",this.removeView]].forEach((function(e){var n=c()(e,2),r=n[0],o=n[1];i.off(r,o),t.listenTo(i,r,o);})),i.view=this,this.listenTo(i,"change:label",this.render),this.listenTo(i,"change:placeholder",this.rerender),o.forEach((function(e){return t.events[e]="onChange"})),this.delegateEvents(),this.init();},getClbOpts:function(){return {component:this.target,trait:this.model,elInput:this.getInputElem()}},removeView:function(){this.remove(),this.removed();},init:function(){},removed:function(){},onRender:function(){},onUpdate:function(){},onEvent:function(){},onChange:function(t){var e=this.getInputElem();e&&!Object(r.isUndefined)(e.value)&&this.model.set("value",e.value),this.onEvent(function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?f(n,!0).forEach((function(e){s()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):f(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},this.getClbOpts(),{event:t}));},getValueForTarget:function(){return this.model.get("value")},setInputValue:function(t){var e=this.getInputElem();e&&(e.value=t);},onValueChange:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if(n.fromTarget)this.setInputValue(t.get("value")),this.postUpdate();else {var r=this.getValueForTarget();t.setTargetValue(r,n);}},renderLabel:function(){var t=this.$el,e=this.target,n=this.getLabel(),r=this.templateLabel(e);this.createLabel&&(r=this.createLabel({label:n,component:e,trait:this})||""),t.find("[data-label]").append(r);},getLabel:function(){var t=this.em,e=this.model.attributes,n=e.label,r=e.name;return t.t("traitManager.traits.labels.".concat(r))||Object(d.c)(n||r).replace(/-/g," ")},getComponent:function(){return this.target},getInputEl:function(){if(!this.$input){var t=this.em,e=this.model,n=e,i=e.attributes.name,o=n.get("placeholder")||n.get("default")||"",a=n.get("type")||"text",s=n.get("min"),l=n.get("max"),c=this.getModelValue(),u=p('<input type="'.concat(a,'" placeholder="').concat(o,'">')),h=t.t("traitManager.traits.attributes.".concat(i))||{};u.attr(h),Object(r.isUndefined)(c)||(n.set({value:c},{silent:!0}),u.prop("value",c)),s&&u.prop("min",s),l&&u.prop("max",l),this.$input=u;}return this.$input.get(0)},getInputElem:function(){var t=this.input,e=this.$input;return t||e&&e.get&&e.get(0)||this.getElInput()},getModelValue:function(){var t,e=this.model,n=this.target,i=e.get("name");if(e.get("changeProp"))t=n.get(i);else {var o=n.get("attributes");t=e.get("value")||o[i];}return Object(r.isUndefined)(t)?"":t},getElInput:function(){return this.elInput},renderField:function(){var t=this.$el,e=this.appendInput,n=this.model,i=t.find("[data-input]"),o=i[i.length-1],a=n.el;a||(a=this.createInput?this.createInput(this.getClbOpts()):this.getInputEl()),Object(r.isString)(a)?(o.innerHTML=a,this.elInput=o.firstChild):(e?o.appendChild(a):o.insertBefore(a,o.firstChild),this.elInput=a),n.el=this.elInput;},hasLabel:function(){var t=this.model.attributes.label;return !this.noLabel&&!1!==t},rerender:function(){this.model.el=null,this.render();},postUpdate:function(){this.onUpdate(this.getClbOpts());},render:function(){var t=this.$el,e=this.pfx,n=this.ppfx,i=this.model.attributes.type,o=this.hasLabel&&this.hasLabel(),a="".concat(e,"trait");this.$input=null;var s='<div class="'.concat(a,'">\n      ').concat(o?'<div class="'.concat(n,'label-wrp" data-label></div>'):"",'\n      <div class="').concat(n,"field-wrp ").concat(n,"field-wrp--").concat(i,'" data-input>\n        ').concat(this.templateInput?Object(r.isFunction)(this.templateInput)?this.templateInput(this.getClbOpts()):this.templateInput:"","\n      </div>\n    </div>");return t.empty().append(s),o&&this.renderLabel(),this.renderField(),this.el.className="".concat(a,"__wrp"),this.postUpdate(),this.onRender(this.getClbOpts()),this}}),v=h.a.$,m=g.extend({init:function(){this.listenTo(this.model,"change:options",this.rerender);},templateInput:function(){var t=this.ppfx,e=this.clsField;return '<div class="'.concat(e,'">\n      <div data-input></div>\n      <div class="').concat(t,'sel-arrow">\n        <div class="').concat(t,'d-s-arrow"></div>\n      </div>\n    </div>')},getInputEl:function(){if(!this.$input){var t=this.model,e=this.em,n=t.get("name"),i=t.get("options")||[],o="<select>";i.forEach((function(t){var i,a,s,l="";Object(r.isString)(t)?(i=t,a=t):(i=t.name||t.label||t.value,a="".concat(Object(r.isUndefined)(t.value)?t.id:t.value).replace(/"/g,"&quot;"),l+=(s=t.style?t.style.replace(/"/g,"&quot;"):"")?' style="'.concat(s,'"'):"");var c=e.t("traitManager.traits.options.".concat(n,".").concat(a))||i;o+='<option value="'.concat(a,'"').concat(l,">").concat(c,"</option>");})),o+="</select>",this.$input=v(o);var a=t.getTargetValue();!Object(r.isUndefined)(a)&&this.$input.val(a);}return this.$input.get(0)}}),y=g.extend({appendInput:0,templateInput:function(){var t=this.ppfx,e=this.clsField;return '<label class="'.concat(e,'" data-input>\n    <i class="').concat(t,'chk-icon"></i>\n  </label>')},onChange:function(){var t=this.getInputElem().checked;this.model.set("value",this.getCheckedValue(t));},getCheckedValue:function(t){var e=t,n=this.model.attributes,i=n.valueTrue,o=n.valueFalse;return e&&!Object(r.isUndefined)(i)&&(e=i),e||Object(r.isUndefined)(o)||(e=o),e},getInputEl:function(){for(var t=!this.$input,e=arguments.length,n=new Array(e),i=0;i<e;i++)n[i]=arguments[i];var o=g.prototype.getInputEl.apply(this,n);if(t){var a,s,l=this.model,c=this.target,u=l.attributes,h=(u.valueTrue,u.valueFalse),d=l.get("name");l.get("changeProp")?s=a=c.get(d):a=!(!(s=c.get("attributes")[d])&&""!==s),Object(r.isUndefined)(h)||s!==h||(a=!1),o.checked=a;}return o}}),b=n(31),w=g.extend({getValueForTarget:function(){var t=this.model.attributes,e=t.value,n=t.unit;return e?e+n:""},getInputEl:function(){if(!this.input){var t=this.getModelValue(),e=new b.a({contClass:this.ppfx+"field-int",model:this.model,ppfx:this.ppfx});this.input=e.render(),this.$input=this.input.inputEl,this.$unit=this.input.unitEl,this.model.set("value",t),this.$input.val(t),this.input=e.el;}return this.input}}),x=n(37),O=g.extend({templateInput:"",getInputEl:function(){if(!this.input){var t=this.model,e=this.getModelValue(),n=new x.a({model:t,target:this.config.em,contClass:this.ppfx+"field-color",ppfx:this.ppfx}).render();n.setValue(e,{fromTarget:1}),this.input=n.el;}return this.input}}),C=g.extend({events:{"click button":"handleClick"},templateInput:"",handleClick:function(){var t=this.model,e=this.em,n=t.get("command");n&&(Object(r.isString)(n)?e.get("Commands").run(n):n(e.get("Editor"),t));},renderLabel:function(){this.model.get("label")&&g.prototype.renderLabel.apply(this,arguments);},getInputEl:function(){var t=this.model,e=this.ppfx,n=t.props(),r=n.labelButton,i=n.text,o=n.full,a=r||i,s="".concat(e,"btn");return '<button type="button" class="'.concat(s,"-prim").concat(o?" ".concat(s,"--full"):"",'">').concat(a,"</button>")}}),S=o.a.extend({ns:"Traits",itemView:g,reuseView:1,itemsView:{text:g,number:w,select:m,checkbox:y,color:O,button:C},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.config||{};this.config=e,this.em=t.editor,this.pfx=e.stylePrefix||"",this.ppfx=e.pStylePrefix||"",this.className=this.pfx+"traits";var n="component:toggled";this.listenTo(this.em,n,this.updatedCollection),this.updatedCollection();},updatedCollection:function(){var t=this.ppfx,e=this.em.getSelected();this.el.className="".concat(this.className," ").concat(t,"one-bg ").concat(t,"two-color"),this.collection=e?e.get("traits"):[],this.render();}});e.default=function(){var t,e={};return {TraitsView:S,name:"TraitManager",getConfig:function(){return e},init:function(){var n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};e=n,Object(r.defaults)(e,i);var o=e.pStylePrefix;return o&&(e.stylePrefix="".concat(o).concat(e.stylePrefix)),t=new S({collection:[],editor:e.em,config:e}),this},postRender:function(){var t=this.getConfig().appendTo;t&&(Object(r.isElement)(t)?t:document.querySelector(t)).appendChild(this.render());},getTraitsViewer:function(){return t},addType:function(e,n){var r=t.itemView;t.itemsView[e]=r.extend(n);},getType:function(e){return t.itemsView[e]},render:function(){return t.render().el}}};},function(t,e,n){n.r(e);var r={assets:[],noAssets:"",stylePrefix:"am-",upload:0,uploadName:"files",headers:{},params:{},credentials:"include",multiUpload:!0,autoAdd:1,customFetch:"",uploadFile:"",embedAsBase64:1,handleAdd:"",dropzone:0,openAssetsOnDrop:1,dropzoneContent:"",beforeUpload:null,showUrlInput:!0},i=n(1),o=n.n(i),a=n(2),s=n.n(a),l=o.a.Model.extend({idAttribute:"src",defaults:{type:"",src:""},getFilename:function(){return this.get("src").split("/").pop()},getExtension:function(){return this.getFilename().split(".").pop()}});function c(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var u=l.extend({defaults:function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?c(n,!0).forEach((function(e){s()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):c(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({},l.prototype.defaults,{type:"image",unitDim:"px",height:0,width:0})}),h=n(0),d=o.a.View.extend({initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.options=t,this.collection=t.collection;var e=t.config||{};this.config=e,this.pfx=e.stylePrefix||"",this.ppfx=e.pStylePrefix||"",this.em=e.em,this.className=this.pfx+"asset",this.listenTo(this.model,"destroy remove",this.remove),this.model.view=this;var n=this.init&&this.init.bind(this);n&&n(t);},template:function(){var t=this.pfx;return '\n      <div class="'.concat(t,'preview-cont">\n        ').concat(this.getPreview(),'\n      </div>\n      <div class="').concat(t,'meta">\n        ').concat(this.getInfo(),'\n      </div>\n      <div class="').concat(t,'close" data-toggle="asset-remove">\n        &Cross;\n      </div>\n    ')},updateTarget:function(t){t&&t.set&&(t.set("attributes",Object(h.clone)(t.get("attributes"))),t.set("src",this.model.get("src")));},getPreview:function(){return ""},getInfo:function(){return ""},render:function(){var t=this.el;return t.innerHTML=this.template(this,this.model),t.className=this.className,this}}).extend({events:{"click [data-toggle=asset-remove]":"onRemove",click:"onClick",dblclick:"onDblClick"},getPreview:function(){var t=this.pfx,e=this.model.get("src");return '\n      <div class="'.concat(t,'preview" style="background-image: url(\'').concat(e,'\');"></div>\n      <div class="').concat(t,"preview-bg ").concat(this.ppfx,'checker-bg"></div>\n    ')},getInfo:function(){var t=this.pfx,e=this.model,n=e.get("name"),r=e.get("width"),i=e.get("height"),o=e.get("unitDim"),a=r&&i?"".concat(r,"x").concat(i).concat(o):"";return n=n||e.getFilename(),'\n      <div class="'.concat(t,'name">').concat(n,'</div>\n      <div class="').concat(t,'dimensions">').concat(a,"</div>\n    ")},init:function(t){var e=this.pfx;this.className+=" ".concat(e,"asset-image");},onClick:function(){var t=this.config.onClick,e=this.model;this.collection.trigger("deselectAll"),this.$el.addClass(this.pfx+"highlight"),Object(h.isFunction)(t)?t(e):this.updateTarget(this.collection.target);},onDblClick:function(){var t=this.em,e=this.model,n=this.config.onDblClick;Object(h.isFunction)(n)?n(e):(this.updateTarget(this.collection.target),t&&t.get("Modal").close());var r=this.collection.onSelect;Object(h.isFunction)(r)&&r(e);},onRemove:function(t){t.stopImmediatePropagation(),this.model.collection.remove(this.model);}}),f=n(28),p=o.a.Collection.extend(f.a).extend({types:[{id:"image",model:u,view:d,isType:function(t){return "string"==typeof t?{type:"image",src:t}:t}}]}),g=n(30),v=n.n(g),m=o.a.View.extend({events:{submit:"handleSubmit"},template:function(t){var e=t.pfx,n=t.ppfx,r=t.em,i=(v()(t,["pfx","ppfx","em"]),"");return this.config.showUrlInput&&(i='\n          <form class="'.concat(e,'add-asset">\n            <div class="').concat(n,"field ").concat(e,'add-field">\n              <input placeholder="').concat(r&&r.t("assetManager.inputPlh"),'"/>\n            </div>\n            <button class="').concat(n,'btn-prim">').concat(r&&r.t("assetManager.addButton"),'</button>\n            <div style="clear:both"></div>\n          </form>\n      ')),'\n    <div class="'.concat(e,'assets-cont">\n      <div class="').concat(e,'assets-header">\n        ').concat(i,'\n      </div>\n      <div class="').concat(e,'assets" data-el="assets"></div>\n      <div style="clear:both"></div>\n    </div>\n    ')},initialize:function(t){this.options=t,this.config=t.config,this.pfx=this.config.stylePrefix||"",this.ppfx=this.config.pStylePrefix||"",this.em=this.config.em;var e=this.collection;this.listenTo(e,"reset",this.renderAssets),this.listenTo(e,"add",this.addToAsset),this.listenTo(e,"remove",this.removedAsset),this.listenTo(e,"deselectAll",this.deselectAll);},handleSubmit:function(t){t.preventDefault();var e=this.getAddInput(),n=e&&e.value.trim(),r=this.config.handleAdd;n&&(e.value="",this.getAssetsEl().scrollTop=0,r?r.bind(this)(n):this.options.globalCollection.add(n,{at:0}));},getAssetsEl:function(){return this.el.querySelector(".".concat(this.pfx,"assets"))},getAddInput:function(){return this.inputUrl&&this.inputUrl.value||(this.inputUrl=this.el.querySelector(".".concat(this.pfx,"add-asset input"))),this.inputUrl},removedAsset:function(t){this.collection.length||this.toggleNoAssets();},addToAsset:function(t){1==this.collection.length&&this.toggleNoAssets(1),this.addAsset(t);},addAsset:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,n=e,r=this.collection,i=this.config,o=new t.typeView({model:t,collection:r,config:i}).render().el;if(n)n.appendChild(o);else {var a=this.getAssetsEl();a&&a.insertBefore(o,a.firstChild);}return o},toggleNoAssets:function(t){var e=this.$el.find(".".concat(this.pfx,"assets"));if(t)e.empty();else {var n=this.config.noAssets;n&&e.append(n);}},deselectAll:function(){var t=this.pfx;this.$el.find(".".concat(t,"highlight")).removeClass("".concat(t,"highlight"));},renderAssets:function(){var t=this,e=document.createDocumentFragment(),n=this.$el.find(".".concat(this.pfx,"assets"));n.empty(),this.toggleNoAssets(this.collection.length),this.collection.each((function(n){return t.addAsset(n,e)})),n.append(e);},render:function(){var t=this.options.fu.render().el;return this.$el.empty(),this.$el.append(t).append(this.template(this)),this.el.className="".concat(this.ppfx,"asset-manager"),this.renderAssets(),this.rendered=1,this}}),y=n(26),b=o.a.View.extend({template:Object(h.template)('\n  <form>\n    <div id="<%= pfx %>title"><%= title %></div>\n    <input type="file" id="<%= uploadId %>" name="file" accept="*/*" <%= disabled ? \'disabled\' : \'\' %> <%= multiUpload ? \'multiple\' : \'\' %>/>\n    <div style="clear:both;"></div>\n  </form>\n  '),events:{},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.options=t;var e=t.config||{};this.config=e,this.em=this.config.em,this.pfx=e.stylePrefix||"",this.ppfx=e.pStylePrefix||"",this.target=this.options.globalCollection||{},this.uploadId=this.pfx+"uploadFile",this.disabled=void 0!==e.disableUpload?e.disableUpload:!e.upload&&!e.embedAsBase64,this.multiUpload=void 0===e.multiUpload||e.multiUpload,this.events["change #"+this.uploadId]="uploadFile";var n=e.uploadFile;n?this.uploadFile=n.bind(this):!e.upload&&e.embedAsBase64&&(this.uploadFile=this.constructor.embedAsBase64),this.delegateEvents();},onUploadStart:function(){var t=this.config.em;t&&t.trigger("asset:upload:start");},onUploadEnd:function(t){var e=this.$el,n=this.config.em;n&&n.trigger("asset:upload:end",t);var r=e.find("input");r&&r.val("");},onUploadError:function(t){var e=this.config.em;console.error(t),this.onUploadEnd(t),e&&e.trigger("asset:upload:error",t);},onUploadResponse:function(t,e){var n,r=this.config.em,i=this.config,o=this.target;try{n="string"==typeof t?JSON.parse(t):t;}catch(e){n=t;}r&&r.trigger("asset:upload:response",n),i.autoAdd&&o&&o.add(n.data,{at:0}),this.onUploadEnd(t),e&&e(n);},uploadFile:function(t,e){var n=this,r=t.dataTransfer?t.dataTransfer.files:t.target.files,i=this.config,o=i.beforeUpload;if(!1!==(o&&o(r))){var a=new FormData,s=i.params,l=i.customFetch;for(var c in s)a.append(c,s[c]);if(this.multiUpload)for(var u=0;u<r.length;u++)a.append("".concat(i.uploadName,"[]"),r[u]);else r.length&&a.append(i.uploadName,r[0]);this.target;var h=i.upload,d=i.headers;if(void 0===d["X-Requested-With"]&&(d["X-Requested-With"]="XMLHttpRequest"),h){this.onUploadStart();var f={method:"post",credentials:i.credentials||"include",headers:d,body:a};return (l?l(h,f):Object(y.a)(h,f).then((function(t){return 1==(t.status/200|0)?t.text():t.text().then((function(t){return Promise.reject(t)}))}))).then((function(t){return n.onUploadResponse(t,e)})).catch((function(t){return n.onUploadError(t)}))}}},initDrop:function(){var t=this;if(!this.uploadForm&&(this.uploadForm=this.$el.find("form").get(0),"draggable"in this.uploadForm)){this.uploadFile;this.uploadForm.ondragover=function(){return this.className=t.pfx+"hover",!1},this.uploadForm.ondragleave=function(){return this.className="",!1},this.uploadForm.ondrop=function(e){this.className="",e.preventDefault(),t.uploadFile(e);};}},initDropzone:function(t){var e=this,n=0,r=this.config,i=t.model,o=t.el,a=i.get("Editor"),s=(i.get("Config").el,i.get("Canvas").getBody()),l=this.ppfx,c="".concat(l,"dropzone-active"),u="".concat(l,"dropzone"),h=function(){o.className=o.className.replace(c,"").trim(),n=0;},d=function(){return n||(o.className+=" ".concat(c),n=1),!1},f=function(){return h(),!1},p=function(t){if(h(),t.preventDefault(),t.stopPropagation(),e.uploadFile(t),r.openAssetsOnDrop&&a){var n=a.getSelected();a.runCommand("open-assets",{target:n,onSelect:function(){a.Modal.close(),a.AssetManager.setTarget(null);}});}return !1};t.$el.append('<div class="'.concat(u,'">').concat(r.dropzoneContent,"</div>")),h(),"draggable"in o&&[o,s].forEach((function(t){t.ondragover=d,t.ondragleave=f,t.ondrop=p;}));},render:function(){var t=this.$el,e=this.pfx,n=this.em;return t.html(this.template({title:n&&n.t("assetManager.uploadTitle"),uploadId:this.uploadId,disabled:this.disabled,multiUpload:this.multiUpload,pfx:e})),this.initDrop(),t.attr("class",e+"file-uploader"),this}},{embedAsBase64:function(t,e){var n=this,r=t.dataTransfer?t.dataTransfer.files:t.target.files,i={data:[]};if(FileReader){var o=[],a=/^(.+)\/(.+)$/,s=!0,l=!1,c=void 0;try{for(var u,h=function(){var t=u.value,e=new Promise((function(e,n){var r=new FileReader;r.addEventListener("load",(function(i){var o,s=t.name,l=a.exec(t.type);if("image"===(o=l?l[1]:t.type)){var c={src:r.result,name:s,type:o,height:0,width:0},u=new Image;u.addEventListener("error",(function(t){n(t);})),u.addEventListener("load",(function(){c.height=u.height,c.width=u.width,e(c);})),u.src=c.src;}else e(o?{src:r.result,name:s,type:o}:r.result);})),r.addEventListener("error",(function(t){n(t);})),r.addEventListener("abort",(function(t){n("Aborted");})),r.readAsDataURL(t);}));o.push(e);},d=r[Symbol.iterator]();!(s=(u=d.next()).done);s=!0)h();}catch(t){l=!0,c=t;}finally{try{s||null==d.return||d.return();}finally{if(l)throw c}}Promise.all(o).then((function(t){i.data=t,n.onUploadResponse(i,e);}),(function(t){n.onUploadError(t);}));}else this.onUploadError(new Error("Unsupported platform, FileReader is not defined"));}});e.default=function(){var t,e,n,i={};return {name:"AssetManager",storageKey:"assets",getConfig:function(){return i},init:function(o){var a=this;for(var s in i=o||{},r)s in i||(i[s]=r[s]);var l=i.pStylePrefix,c=i.em;l&&(i.stylePrefix=l+i.stylePrefix),t=new p([]);var u={collection:new p([]),globalCollection:t,config:i};return n=new b(u),u.fu=n,e=new m(u),t.listenTo(t,"add",(function(t){a.getAllVisible().add(t),c&&c.trigger("asset:add",t);})),t.listenTo(t,"remove",(function(t){a.getAllVisible().remove(t),c&&c.trigger("asset:remove",t);})),this},add:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return void 0===n.at&&(n.at=0),t.add(e,n)},get:function(e){return t.where({src:e})[0]},getAll:function(){return t},getAllVisible:function(){return e.collection},remove:function(t){var e=this.get(t);return this.getAll().remove(e),this},store:function(t){var e={},n=JSON.stringify(this.getAll().toJSON());return e[this.storageKey]=n,!t&&i.stm&&i.stm.store(e),e},load:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.storageKey,n=t[e]||[];if("string"==typeof n)try{n=JSON.parse(t[e]);}catch(t){}return n&&n.length&&this.getAll().reset(n),n},getContainer:function(){return e.el},getAssetsEl:function(){return e.el.querySelector("[data-el=assets]")},render:function(t){var n=t||this.getAll().models;return e.rendered||e.render(),e.collection.reset(n),this.getContainer()},addType:function(t,e){this.getAll().addType(t,e);},getType:function(t){return this.getAll().getType(t)},getTypes:function(){return this.getAll().getTypes()},AssetsView:function(){return e},FileUploader:function(){return n},onLoad:function(){this.getAll().reset(i.assets);},postRender:function(t){i.dropzone&&n.initDropzone(t);},setTarget:function(t){e.collection.target=t;},onSelect:function(t){e.collection.onSelect=t;},onClick:function(t){i.onClick=t;},onDblClick:function(t){i.onDblClick=t;}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(0),a={appendTo:"",appendOnClick:0,blocks:[]},s=n(1),l=n.n(s),c=l.a.Model.extend({defaults:{id:"",label:"",open:!0,attributes:{}}}),u=l.a.Model.extend({defaults:{activate:0,select:0,resetId:0,label:"",disable:0,media:"",content:"",category:"",attributes:{}},initialize:function(){var t=this.get("category");if(t&&"string"==typeof t)new c({id:t,label:t});}}),h=l.a.Collection.extend({model:u}),d=l.a.Collection.extend({model:c}),f=n(3),p=l.a.View.extend({events:{click:"handleClick",mousedown:"startDrag",dragstart:"handleDragStart",drag:"handleDrag",dragend:"handleDragEnd"},initialize:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.model;this.em=e.em,this.config=e,this.endDrag=this.endDrag.bind(this),this.ppfx=e.pStylePrefix||"",this.listenTo(n,"destroy remove",this.remove),this.listenTo(n,"change",this.render);},handleClick:function(){var t=this.config,e=this.model,n=this.em;if(t.appendOnClick){var r,i=t.getSorter(),o=e.get("content"),a=n.getSelected();if(i.setDropContent(o),a)if(i.validTarget(a.getEl(),o).valid)r=a;else {var s=a.parent();i.validTarget(s.getEl(),o).valid&&(r=s);}if(!r){var l=n.getWrapper();i.validTarget(l.getEl(),o).valid&&(r=l);}var c=r&&r.append(o)[0];c&&n.setSelected(c,{scroll:1});}},startDrag:function(t){var e=this.config,n=this.em,r=this.model.get("disable");if(0===t.button&&e.getSorter&&!this.el.draggable&&!r){n.refreshCanvas();var i=e.getSorter();i.setDragHelper(this.el,t),i.setDropContent(this.model.get("content")),i.startSort(this.el),Object(f.t)(document,"mouseup",this.endDrag);}},handleDragStart:function(t){var e=this.em,n=this.model,r=n.get("content"),i=Object(o.isObject)(r)?JSON.stringify(r):r;e.set("dragResult"),t.dataTransfer.setData("text",i),e.set("dragContent",r),e.trigger("block:drag:start",n,t);},handleDrag:function(t){this.em.trigger("block:drag",this.model,t);},handleDragEnd:function(){var t=this.em,e=this.model,n=t.get("dragResult");if(n){var r=n.get&&n.get("activeOnRender");(e.get("activate")||r)&&(n.trigger("active"),n.set("activeOnRender",0)),e.get("select")&&t.setSelected(n),e.get("resetId")&&n.onAll((function(t){return t.resetId()}));}t.set({dragResult:null,dragContent:null}),t.trigger("block:drag:stop",n,e);},endDrag:function(t){Object(f.s)(document,"mouseup",this.endDrag);var e=this.config.getSorter();e.moved=0,e.endMove();},render:function(){var t=this.em,e=this.el,n=this.ppfx,r=this.model,i=r.get("disable"),o=(r.get("attributes")||{}).class||"",a="".concat(n,"block"),s=t&&t.t("blockManager.labels.".concat(r.id))||r.get("label"),l=r.get("render"),c=r.get("media"),u=i?"".concat(a,"--disable"):"".concat(n,"four-color-h");e.className="".concat(o," ").concat(a," ").concat(n,"one-bg ").concat(u).trim(),e.innerHTML="\n      ".concat(c?'<div class="'.concat(a,'__media">').concat(c,"</div>"):"",'\n      <div class="').concat(a,'-label">').concat(s,"</div>\n    "),e.title=e.textContent.trim(),e.setAttribute("draggable",!(!Object(f.k)(t)||i));var h=l&&l({el:e,model:r,className:a,prefix:n});return h&&(e.innerHTML=h),this}}),g=l.a.View.extend({template:Object(o.template)('\n  <div class="<%= pfx %>title">\n    <i class="<%= pfx %>caret-icon"></i>\n    <%= label %>\n  </div>\n  <div class="<%= pfx %>blocks-c"></div>\n  '),events:{},initialize:function(){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.config=t;var e=t.pStylePrefix||"";this.em=t.em,this.pfx=e,this.caretR="fa fa-caret-right",this.caretD="fa fa-caret-down",this.iconClass="".concat(e,"caret-icon"),this.activeClass="".concat(e,"open"),this.className="".concat(e,"block-category"),this.events["click .".concat(e,"title")]="toggle",this.listenTo(this.model,"change:open",this.updateVisibility),this.delegateEvents();},updateVisibility:function(){this.model.get("open")?this.open():this.close();},open:function(){this.el.className="".concat(this.className," ").concat(this.activeClass),this.getIconEl().className="".concat(this.iconClass," ").concat(this.caretD),this.getBlocksEl().style.display="";},close:function(){this.el.className=this.className,this.getIconEl().className="".concat(this.iconClass," ").concat(this.caretR),this.getBlocksEl().style.display="none";},toggle:function(){var t=this.model;t.set("open",!t.get("open"));},getIconEl:function(){return this.iconEl||(this.iconEl=this.el.querySelector("."+this.iconClass)),this.iconEl},getBlocksEl:function(){return this.blocksEl||(this.blocksEl=this.el.querySelector("."+this.pfx+"blocks-c")),this.blocksEl},append:function(t){this.getBlocksEl().appendChild(t);},render:function(){var t=this.em,e=this.el,n=this.$el,r=this.model,i=t.t("blockManager.categories.".concat(r.id))||r.get("label");return e.innerHTML=this.template({pfx:this.pfx,label:i}),e.className=this.className,n.css({order:r.get("order")}),this.updateVisibility(),this}});function v(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function m(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?v(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):v(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var y=l.a.View.extend({initialize:function(t,e){Object(o.bindAll)(this,"getSorter","onDrag","onDrop"),this.config=e||{},this.categories=t.categories||"",this.renderedCategories=[];var n=this.config.pStylePrefix||"";this.ppfx=n,this.noCatClass="".concat(n,"blocks-no-cat"),this.blockContClass="".concat(n,"blocks-c"),this.catsClass="".concat(n,"block-categories");var r=this.collection;this.listenTo(r,"add",this.addTo),this.listenTo(r,"reset",this.render),this.em=this.config.em,this.tac="test-tac",this.grabbingCls=this.ppfx+"grabbing",this.em&&(this.config.getSorter=this.getSorter,this.canvas=this.em.get("Canvas"));},updateConfig:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.config=m({},this.config,{},t);},getSorter:function(){if(this.em){if(!this.sorter){var t=this.em.get("Utils"),e=this.canvas;this.sorter=new t.Sorter({container:e.getBody(),placer:e.getPlacerEl(),containerSel:"*",itemSel:"*",pfx:this.ppfx,onStart:this.onDrag,onEndMove:this.onDrop,onMove:this.onMove,document:e.getFrameEl().contentDocument,direction:"a",wmargin:1,nested:1,em:this.em,canvasRelative:1});}return this.sorter}},onDrag:function(t){this.em.stopDefault(),this.em.trigger("block:drag:start",t);},onMove:function(t){this.em.trigger("block:drag:move",t);},onDrop:function(t){var e=this.em;e.runDefault(),t&&t.get&&(t.get("activeOnRender")&&(t.trigger("active"),t.set("activeOnRender",0)),e.trigger("block:drag:stop",t));},addTo:function(t){this.add(t);},add:function(t,e){var n=this.config,r=e||null,i=new p({model:t,attributes:t.get("attributes")},n).render().el,a=t.get("category");if(a&&this.categories&&!n.ignoreCategories){Object(o.isString)(a)?a={id:a,label:a}:Object(o.isObject)(a)&&!a.id&&(a.id=a.label);var s=this.categories.add(a),l=s.get("id"),c=this.renderedCategories[l],u=this.getCategoriesEl();return t.set("category",s),!c&&u&&(c=new g({model:s},this.config).render(),this.renderedCategories[l]=c,u.appendChild(c.el)),void(c&&c.append(i))}r?r.appendChild(i):this.append(i);},getCategoriesEl:function(){return this.catsEl||(this.catsEl=this.el.querySelector(".".concat(this.catsClass))),this.catsEl},getBlocksEl:function(){return this.blocksEl||(this.blocksEl=this.el.querySelector(".".concat(this.noCatClass," .").concat(this.blockContClass))),this.blocksEl},append:function(t){var e=this.getBlocksEl();e&&e.appendChild(t);},render:function(){var t=this,e=this.ppfx,n=document.createDocumentFragment();this.catsEl=null,this.blocksEl=null,this.renderedCategories=[],this.el.innerHTML='\n      <div class="'.concat(this.catsClass,'"></div>\n      <div class="').concat(this.noCatClass,'">\n        <div class="').concat(this.blockContClass,'"></div>\n      </div>\n    '),this.collection.each((function(e){return t.add(e,n)})),this.append(n);var r="".concat(this.blockContClass,"s ").concat(e,"one-bg ").concat(e,"two-color");return this.$el.addClass(r),this}});function b(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function w(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?b(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):b(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n,r={},i=[];return {name:"BlockManager",init:function(o){var s=(r=o||{}).em;for(var l in a)l in r||(r[l]=a[l]);return t=new h([]),e=new h([]),i=new d,n=new y({collection:e,categories:i},r),t.listenTo(t,"add",(function(t){e.add(t),s&&s.trigger("block:add",t);})),t.listenTo(t,"remove",(function(t){e.remove(t),s&&s.trigger("block:remove",t);})),t.listenTo(t,"reset",(function(t){e.reset(t.models);})),this},getConfig:function(){return r},onLoad:function(){var t=this.getAll();!t.length&&t.reset(r.blocks);},postRender:function(){var t=this.getConfig().appendTo;t&&(Object(o.isElement)(t)?t:document.querySelector(t)).appendChild(this.render());},add:function(e,n){var r=n||{};return r.id=e,t.add(r)},get:function(e){return t.get(e)},getAll:function(){return t},getAllVisible:function(){return e},remove:function(e){return t.remove(e)},getCategories:function(){return i},getContainer:function(){return n.el},render:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},o=t||this.getAll().models;return e.external?new y({collection:new h(o),categories:i},w({},r,{},e)).render().el:(n.rendered||(n.render(),n.rendered=1),n.updateConfig(e),n.collection.reset(o),this.getContainer())}}};},function(t,e,n){n.r(e);var r="sw-visibility",i="export-template",o="open-layers",a="open-blocks",s="fullscreen",l="preview",c={stylePrefix:"pn-",defaults:[{id:"commands",buttons:[{}]},{id:"options",buttons:[{active:!0,id:r,className:"fa fa-square-o",command:r,context:r,attributes:{title:"View components"}},{id:l,className:"fa fa-eye",command:l,context:l,attributes:{title:"Preview"}},{id:s,className:"fa fa-arrows-alt",command:s,context:s,attributes:{title:"Fullscreen"}},{id:i,className:"fa fa-code",command:i,attributes:{title:"View code"}}]},{id:"views",buttons:[{id:"open-sm",className:"fa fa-paint-brush",command:"open-sm",active:!0,togglable:0,attributes:{title:"Open Style Manager"}},{id:"open-tm",className:"fa fa-cog",command:"open-tm",togglable:0,attributes:{title:"Settings"}},{id:o,className:"fa fa-bars",command:o,togglable:0,attributes:{title:"Open Layer Manager"}},{id:a,className:"fa fa-th-large",command:a,togglable:0,attributes:{title:"Open Blocks"}}]}],em:null,delayBtnsShow:300},u=n(1),h=n.n(u),d=n(40),f=h.a.Model.extend({defaults:{id:"",content:"",visible:!0,buttons:[],attributes:{}},initialize:function(t){this.btn=this.get("buttons")||[],this.buttons=new d.default(this.btn),this.set("buttons",this.buttons);}}),p=h.a.Collection.extend({model:f}),g=n(2),v=n.n(g),m=n(0);function y(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function b(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?y(n,!0).forEach((function(e){v()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):y(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}h.a.$;var w=h.a.View.extend({tagName:function(){return this.model.get("tagName")},events:{click:"clicked"},initialize:function(t){var e=this.model.get("className");this.config=t.config||{},this.em=this.config.em||{};var n=this.config.stylePrefix||"",r=this.config.pStylePrefix||"";this.pfx=n,this.ppfx=this.config.pStylePrefix||"",this.id=n+this.model.get("id"),this.activeCls="".concat(n,"active ").concat(r,"four-color"),this.disableCls="".concat(r,"disabled"),this.btnsVisCls="".concat(n,"visible"),this.className=n+"btn"+(e?" "+e:""),this.listenTo(this.model,"change",this.render),this.listenTo(this.model,"change:active updateActive",this.updateActive),this.listenTo(this.model,"checkActive",this.checkActive),this.listenTo(this.model,"change:bntsVis",this.updateBtnsVis),this.listenTo(this.model,"change:attributes",this.updateAttributes),this.listenTo(this.model,"change:className",this.updateClassName),this.listenTo(this.model,"change:disable",this.updateDisable),this.em&&this.em.get&&(this.commands=this.em.get("Commands"));},updateClassName:function(){var t=this.model,e=this.pfx,n=t.get("className"),r=t.get("attributes").class,i="".concat(r||""," ").concat(e,"btn ").concat(n||"");this.$el.attr("class",i.trim());},updateAttributes:function(){var t=this.em,e=this.model,n=this.$el,r=e.get("attributes")||{},i=t&&t.t&&t.t("panels.buttons.titles.".concat(e.id));n.attr(r),i&&n.attr({title:i}),this.updateClassName();},updateBtnsVis:function(){this.$buttons&&(this.model.get("bntsVis")?this.$buttons.addClass(this.btnsVisCls):this.$buttons.removeClass(this.btnsVisCls));},updateActive:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.model,n=this.commands,r=this.$el,i=this.activeCls,o=t.fromCollection,a=e.get("context"),s=e.get("options"),l=e.get("command"),c={};n&&Object(m.isString)(l)?c=n.get(l)||{}:Object(m.isFunction)(l)?c=n.create({run:l}):null!==l&&Object(m.isObject)(l)&&(c=n.create(l)),e.get("active")?(!o&&e.collection.deactivateAll(a,e),e.set("active",!0,{silent:!0}).trigger("checkActive"),n.runCommand(c,b({},s,{sender:e})),c.noStop&&e.set("active",!1)):(r.removeClass(i),n.stopCommand(c,b({},s,{sender:e,force:1})));},updateDisable:function(){var t=this.disableCls,e=this.model.get("disable");this.$el[e?"addClass":"removeClass"](t);},checkActive:function(){var t=this.model,e=this.$el,n=this.activeCls;t.get("active")?e.addClass(n):e.removeClass(n);},clicked:function(t){this.model.get("bntsVis")||this.model.get("disable")||this.toggleActive();},toggleActive:function(){var t=this.model,e=t.attributes,n=e.active,r=e.togglable;if(!n||r){t.set("active",!n);this.em.get("Commands").get("select-comp");n?t.get("runDefaultCommand")&&this.em.runDefault():t.get("stopDefaultCommand")&&this.em.stopDefault();}},render:function(){var t=this.model.get("label"),e=this.$el;return e.empty(),this.updateAttributes(),t&&e.append(t),this.checkActive(),this.updateDisable(),this}}),x=h.a.View.extend({initialize:function(t){this.opt=t||{},this.config=this.opt.config||{},this.pfx=this.config.stylePrefix||"",this.parentM=this.opt.parentM||null,this.listenTo(this.collection,"add",this.addTo),this.listenTo(this.collection,"reset remove",this.render),this.className=this.pfx+"buttons";},addTo:function(t){this.addToCollection(t);},addToCollection:function(t,e){var n=e||null,r=new w({model:t,config:this.config,parentM:this.parentM}).render().el;return n?n.appendChild(r):this.$el.append(r),r},render:function(){var t=document.createDocumentFragment();return this.$el.empty(),this.collection.each((function(e){this.addToCollection(e,t);}),this),this.$el.append(t),this.$el.attr("class",Object(m.result)(this,"className")),this}});function O(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var C=h.a.View.extend({initialize:function(t){var e=t.config||{},n=this.model;this.config=e,this.pfx=e.stylePrefix||"",this.ppfx=e.pStylePrefix||"",this.buttons=n.get("buttons"),this.className=this.pfx+"panel",this.id=this.pfx+n.get("id"),this.listenTo(n,"change:appendContent",this.appendContent),this.listenTo(n,"change:content",this.updateContent),this.listenTo(n,"change:visible",this.toggleVisible),n.view=this;},appendContent:function(){this.$el.append(this.model.get("appendContent"));},updateContent:function(){this.$el.html(this.model.get("content"));},toggleVisible:function(){this.model.get("visible")?this.$el.removeClass("".concat(this.ppfx,"hidden")):this.$el.addClass("".concat(this.ppfx,"hidden"));},attributes:function(){return this.model.get("attributes")},initResize:function(){var t=this.config.em,e=t?t.get("Editor"):"",n=this.model.get("resizable");if(e&&n){var r,i,o,a=!0===n?[1,1,1,1]:n,s=a.length,l=0;2==s?(r=a[0],o=a[0],i=a[1],l=a[1]):4==s&&(r=a[0],i=a[1],o=a[2],l=a[3]);var c=e.Utils.Resizer.init(function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?O(n,!0).forEach((function(e){v()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):O(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({tc:r,cr:i,bc:o,cl:l,tl:0,tr:0,bl:0,br:0,appendTo:this.el,silentFrames:1,avoidContainerUpdate:1,prefix:e.getConfig().stylePrefix,onEnd:function(){t&&t.trigger("change:canvasOffset");},posFetcher:function(t,e){var n=e.target,r=t.style,i=c.getConfig(),o=i.keyWidth,a=i.keyHeight,s=t.getBoundingClientRect(),l="container"==n,u=r[o],h=r[a];return {left:0,top:0,width:u&&!l?parseFloat(u):s.width,height:h&&!l?parseFloat(h):s.height}}},n));c.blur=function(){},c.focus(this.el);}},render:function(){var t=this.$el,e=this.ppfx,n="".concat(this.className," ").concat(this.id," ").concat(e,"one-bg ").concat(e,"two-color");if(t.addClass(n),this.buttons.length){var r=new x({collection:this.buttons,config:this.config});t.append(r.render().el);}return t.append(this.model.get("content")),this}}),S=h.a.View.extend({initialize:function(t){this.opt=t||{},this.config=this.opt.config||{},this.pfx=this.config.stylePrefix||"";var e=this.collection;this.listenTo(e,"add",this.addTo),this.listenTo(e,"reset",this.render),this.listenTo(e,"remove",this.onRemove),this.className=this.pfx+"panels";},onRemove:function(t){var e=t.view;e&&e.remove();},addTo:function(t){this.addToCollection(t);},addToCollection:function(t,e){var n=e||null,r=this.config,i=t.get("el"),o=new C({el:i,model:t,config:r}),a=o.render().el,s=t.get("appendTo");if(i);else if(s){document.querySelector(s).appendChild(a);}else n?n.appendChild(a):this.$el.append(a);return o.initResize(),a},render:function(){var t=this,e=this.$el,n=document.createDocumentFragment();return e.empty(),this.collection.each((function(e){return t.addToCollection(e,n)})),e.append(n),e.attr("class",this.className),this}});e.default=function(){var t,e,n={};return {name:"Panels",init:function(r){for(var i in n=r||{},c)i in n||(n[i]=c[i]);var o=n.pStylePrefix;return o&&(n.stylePrefix=o+n.stylePrefix),t=new p(n.defaults),e=new S({collection:t,config:n}),this},getPanels:function(){return t},getPanelsEl:function(){return e.el},addPanel:function(e){return t.add(e)},removePanel:function(e){return t.remove(e)},getPanel:function(e){var n=t.where({id:e});return n.length?n[0]:null},addButton:function(t,e){var n=this.getPanel(t);return n?n.get("buttons").add(e):null},removeButton:function(t,e){var n=this.getPanel(t);return n&&n.get("buttons").remove(e)},getButton:function(t,e){var n=this.getPanel(t);if(n){var r=n.get("buttons").where({id:e});return r.length?r[0]:null}return null},render:function(){return e.render().el},active:function(){this.getPanels().each((function(t){t.get("buttons").each((function(t){t.get("active")&&t.trigger("updateActive");}));}));},disableButtons:function(){this.getPanels().each((function(t){t.get("buttons").each((function(t){t.get("disable")&&t.trigger("change:disable");}));}));},Panel:f}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(0),a={stylePrefix:"sm-",sectors:[],appendTo:"",hideNotStylable:!0,highlightChanged:!0,highlightComputed:!0,showComputed:!0,clearProperties:0,avoidComputed:["width","height"]},s=n(1),l=n.n(s),c=n(23),u=function(){return {build:function(t){var e=[];"string"==typeof t&&(t=[t]);for(var n=0,r=t.length;n<r;n++){var i={},o=t[n];switch(i.property=o,o){case"border-radius-c":i.property="border-radius";}switch(o){case"top":case"right":case"bottom":case"left":case"margin-top":case"margin-right":case"margin-bottom":case"margin-left":case"padding-top":case"padding-right":case"padding-bottom":case"padding-left":case"width":case"max-width":case"min-width":case"height":case"max-height":case"min-height":case"flex-basis":i.fixedValues=["initial","inherit","auto"];break;case"font-size":i.fixedValues=["medium","xx-small","x-small","small","large","x-large","xx-large","smaller","larger","length","initial","inherit"];break;case"letter-spacing":case"line-height":i.fixedValues=["normal","initial","inherit"];}switch(o){case"float":case"position":case"text-align":i.type="radio";break;case"display":case"flex-direction":case"flex-wrap":case"justify-content":case"align-items":case"align-content":case"align-self":case"font-family":case"font-weight":case"border-style":case"box-shadow-type":case"background-repeat":case"background-position":case"background-attachment":case"background-size":case"transition-property":case"transition-timing-function":case"cursor":case"overflow":case"overflow-x":case"overflow-y":i.type="select";break;case"top":case"right":case"bottom":case"left":case"margin-top":case"margin-right":case"margin-bottom":case"margin-left":case"padding-top":case"padding-right":case"padding-bottom":case"padding-left":case"min-height":case"min-width":case"max-height":case"max-width":case"width":case"height":case"font-size":case"letter-spacing":case"line-height":case"text-shadow-h":case"text-shadow-v":case"text-shadow-blur":case"border-radius-c":case"border-top-left-radius":case"border-top-right-radius":case"border-bottom-left-radius":case"border-bottom-right-radius":case"border-width":case"box-shadow-h":case"box-shadow-v":case"box-shadow-blur":case"box-shadow-spread":case"transition-duration":case"perspective":case"transform-rotate-x":case"transform-rotate-y":case"transform-rotate-z":case"transform-scale-x":case"transform-scale-y":case"transform-scale-z":case"order":case"flex-grow":case"flex-shrink":case"flex-basis":i.type="integer";break;case"margin":case"padding":case"border-radius":case"border":case"transform":i.type="composite";break;case"color":case"text-shadow-color":case"background-color":case"border-color":case"box-shadow-color":i.type="color";break;case"text-shadow":case"box-shadow":case"background":case"transition":i.type="stack";break;case"background-image":i.type="file";}switch(o){case"float":case"background-color":case"text-shadow":i.defaults="none";break;case"display":i.defaults="block";break;case"flex-direction":i.defaults="row";break;case"flex-wrap":i.defaults="nowrap";break;case"justify-content":i.defaults="flex-start";break;case"align-items":case"align-content":i.defaults="stretch";break;case"align-self":i.defaults="auto";break;case"position":i.defaults="static";break;case"margin-top":case"margin-right":case"margin-bottom":case"margin-left":case"padding-top":case"padding-right":case"padding-bottom":case"padding-left":case"text-shadow-h":case"text-shadow-v":case"text-shadow-blur":case"border-radius-c":case"box-shadow-h":case"box-shadow-v":case"box-shadow-spread":case"perspective":case"transform-rotate-x":case"transform-rotate-y":case"transform-rotate-z":case"order":case"flex-grow":i.defaults=0;break;case"border-top-left-radius":case"border-top-right-radius":case"border-bottom-left-radius":case"border-bottom-right-radius":i.defaults="0px";break;case"transform-scale-x":case"transform-scale-y":case"transform-scale-z":case"flex-shrink":i.defaults=1;break;case"box-shadow-blur":i.defaults="5px";break;case"top":case"right":case"bottom":case"left":case"min-height":case"min-width":case"max-height":case"max-width":case"width":case"height":case"background-size":case"cursor":case"flex-basis":i.defaults="auto";break;case"font-family":i.defaults="Arial, Helvetica, sans-serif";break;case"font-size":case"border-width":i.defaults="medium";break;case"font-weight":i.defaults="400";break;case"letter-spacing":case"line-height":i.defaults="normal";break;case"color":case"text-shadow-color":case"border-color":case"box-shadow-color":i.defaults="black";break;case"text-align":i.defaults="left";break;case"border-style":i.defaults="solid";break;case"box-shadow-type":i.defaults="";break;case"background-repeat":i.defaults="repeat";break;case"background-position":i.defaults="left top";break;case"background-attachment":i.defaults="scroll";break;case"transition-property":i.defaults="width";break;case"transition-duration":i.defaults="2";break;case"transition-timing-function":i.defaults="ease";break;case"overflow":case"overflow-x":case"overflow-y":i.defaults="visible";}switch(o){case"flex-direction":case"flex-wrap":case"justify-content":case"align-items":case"align-content":i.requires={display:["flex"]};break;case"order":case"flex-basis":case"flex-grow":case"flex-shrink":case"align-self":i.requiresParent={display:["flex"]};}switch(o){case"top":case"bottom":case"margin-top":case"margin-bottom":case"padding-top":case"padding-bottom":case"min-height":case"max-height":case"height":i.units=["px","%","vh"];break;case"right":case"left":case"margin-right":case"margin-left":case"padding-right":case"padding-left":case"min-width":case"max-width":case"width":i.units=["px","%","vw"];break;case"flex-basis":i.units=["px","%","vw","vh"];break;case"text-shadow-v":case"text-shadow-h":case"text-shadow-blur":case"border-radius-c":case"border-top-left-radius":case"border-top-right-radius":case"border-bottom-left-radius":case"border-bottom-right-radius":case"box-shadow-h":case"box-shadow-v":i.units=["px","%"];break;case"font-size":case"letter-spacing":case"line-height":i.units=["px","em","rem","%"];break;case"border-width":i.units=["px","em"];break;case"box-shadow-blur":case"box-shadow-spread":case"perspective":i.units=["px"];break;case"transition-duration":i.units=["s"];break;case"transform-rotate-x":case"transform-rotate-y":case"transform-rotate-z":i.units=["deg"];}switch(o){case"padding-top":case"padding-right":case"padding-bottom":case"padding-left":case"min-height":case"min-width":case"max-height":case"max-width":case"width":case"height":case"font-size":case"text-shadow-blur":case"border-radius-c":case"border-top-left-radius":case"border-top-right-radius":case"border-bottom-left-radius":case"border-bottom-right-radius":case"border-width":case"box-shadow-blur":case"transition-duration":case"perspective":case"flex-basis":i.min=0;}switch(o){case"text-shadow":case"box-shadow":case"background":i.preview=!0;}switch(o){case"background":i.detached=!0;}switch(o){case"transform-rotate-x":i.functionName="rotateX";break;case"transform-rotate-y":i.functionName="rotateY";break;case"transform-rotate-z":i.functionName="rotateZ";break;case"transform-scale-x":i.functionName="scaleX";break;case"transform-scale-y":i.functionName="scaleY";break;case"transform-scale-z":i.functionName="scaleZ";break;case"background-image":i.functionName="url";}switch(o){case"float":i.list=[{value:"none"},{value:"left"},{value:"right"}];break;case"display":i.list=[{value:"block"},{value:"inline"},{value:"inline-block"},{value:"flex"},{value:"none"}];break;case"flex-direction":i.list=[{value:"row"},{value:"row-reverse"},{value:"column"},{value:"column-reverse"}];break;case"flex-wrap":i.list=[{value:"nowrap"},{value:"wrap"},{value:"wrap-reverse"}];break;case"justify-content":i.list=[{value:"flex-start"},{value:"flex-end"},{value:"center"},{value:"space-between"},{value:"space-around"},{value:"space-evenly"}];break;case"align-items":i.list=[{value:"flex-start"},{value:"flex-end"},{value:"center"},{value:"baseline"},{value:"stretch"}];break;case"align-content":i.list=[{value:"flex-start"},{value:"flex-end"},{value:"center"},{value:"space-between"},{value:"space-around"},{value:"stretch"}];break;case"align-self":i.list=[{value:"auto"},{value:"flex-start"},{value:"flex-end"},{value:"center"},{value:"baseline"},{value:"stretch"}];break;case"position":i.list=[{value:"static"},{value:"relative"},{value:"absolute"},{value:"fixed"}];break;case"font-family":var a=", sans-serif",s=["Arial, Helvetica"+a,"Arial Black, Gadget"+a,"Brush Script MT"+a,"Comic Sans MS, cursive"+a,"Courier New, Courier, monospace","Georgia, serif","Helvetica, serif","Impact, Charcoal"+a,"Lucida Sans Unicode, Lucida Grande"+a,"Tahoma, Geneva"+a,"Times New Roman, Times, serif","Trebuchet MS, Helvetica"+a,"Verdana, Geneva"+a];i.list=[];for(var l=0,c=s.length;l<c;l++){var u={};u.value=s[l],u.name=s[l].split(",")[0],i.list.push(u);}break;case"font-weight":i.list=[{value:"100",name:"Thin"},{value:"200",name:"Extra-Light"},{value:"300",name:"Light"},{value:"400",name:"Normal"},{value:"500",name:"Medium"},{value:"600",name:"Semi-Bold"},{value:"700",name:"Bold"},{value:"800",name:"Extra-Bold"},{value:"900",name:"Ultra-Bold"}];break;case"text-align":i.list=[{value:"left"},{value:"center"},{value:"right"},{value:"justify"}];break;case"border-style":i.list=[{value:"none"},{value:"solid"},{value:"dotted"},{value:"dashed"},{value:"double"},{value:"groove"},{value:"ridge"},{value:"inset"},{value:"outset"}];break;case"box-shadow-type":i.list=[{value:"",name:"Outside"},{value:"inset",name:"Inside"}];break;case"background-repeat":i.list=[{value:"repeat"},{value:"repeat-x"},{value:"repeat-y"},{value:"no-repeat"}];break;case"background-position":i.list=[{value:"left top"},{value:"left center"},{value:"left bottom"},{value:"right top"},{value:"right center"},{value:"right bottom"},{value:"center top"},{value:"center center"},{value:"center bottom"}];break;case"background-attachment":i.list=[{value:"scroll"},{value:"fixed"},{value:"local"}];break;case"background-size":i.list=[{value:"auto"},{value:"cover"},{value:"contain"}];break;case"transition-property":i.list=[{value:"all"},{value:"width"},{value:"height"},{value:"background-color"},{value:"transform"},{value:"box-shadow"},{value:"opacity"}];break;case"transition-timing-function":i.list=[{value:"linear"},{value:"ease"},{value:"ease-in"},{value:"ease-out"},{value:"ease-in-out"}];break;case"cursor":i.list=[{value:"auto"},{value:"pointer"},{value:"copy"},{value:"crosshair"},{value:"grab"},{value:"grabbing"},{value:"help"},{value:"move"},{value:"text"}];break;case"overflow":case"overflow-x":case"overflow-y":i.list=[{value:"visible"},{value:"hidden"},{value:"scroll"},{value:"auto"}];}switch(o){case"margin":i.properties=this.build(["margin-top","margin-right","margin-bottom","margin-left"]);break;case"padding":i.properties=this.build(["padding-top","padding-right","padding-bottom","padding-left"]);break;case"text-shadow":i.properties=this.build(["text-shadow-h","text-shadow-v","text-shadow-blur","text-shadow-color"]);break;case"border":i.properties=this.build(["border-width","border-style","border-color"]);break;case"border-radius":i.properties=this.build(["border-top-left-radius","border-top-right-radius","border-bottom-right-radius","border-bottom-left-radius"]);break;case"box-shadow":i.properties=this.build(["box-shadow-h","box-shadow-v","box-shadow-blur","box-shadow-spread","box-shadow-color","box-shadow-type"]);break;case"background":i.properties=this.build(["background-image","background-repeat","background-position","background-attachment","background-size"]);break;case"transition":i.properties=this.build(["transition-property","transition-duration","transition-timing-function"]);break;case"transform":i.properties=this.build(["transform-rotate-x","transform-rotate-y","transform-rotate-z","transform-scale-x","transform-scale-y","transform-scale-z"]);}e.push(i);}return e}}},h=l.a.Model.extend({defaults:{id:"",name:"",open:!0,buildProps:"",extendBuilded:1,properties:[]},initialize:function(t){var e=t||{},n=this.buildProperties(e.buildProps),r=this.get("name")||"",i=[];!this.get("id")&&this.set("id",r.replace(/ /g,"_").toLowerCase()),i=n?this.extendProperties(n):this.get("properties");var o=new c.default(i);o.sector=this,this.set("properties",o);},extendProperties:function(t,e,n){for(var r=t.length,i=e||this.get("properties"),a=this.get("extendBuilded"),s=[],l=0,c=i.length;l<c;l++){for(var u=i[l],h=0,d=0;d<r;d++){var f=t[d];if(u.property!=f.property&&u.id!=f.property);else {var p=u.properties;p&&p.length&&(u.properties=this.extendProperties(f.properties||[],p,1)),t[d]=a?Object(o.extend)(f,u):u,s[d]=t[d],h=1;}}h||(t.push(u),s.push(u));}return n?s.filter((function(t){return t})):t},buildProperties:function(t){var e=t||[];if(e.length)return this.propFactory||(this.propFactory=new u),this.propFactory.build(e)}}),d=l.a.Collection.extend({model:h}),f=n(3),p=n(7),g=n(18),v=l.a.View.extend({template:Object(o.template)('\n  <div class="<%= pfx %>title" data-sector-title>\n    <i id="<%= pfx %>caret" class="fa"></i>\n    <%= label %>\n  </div>'),events:{"click [data-sector-title]":"toggle"},initialize:function(t){this.config=t.config||{},this.em=this.config.em,this.pfx=this.config.stylePrefix||"",this.target=t.target||{},this.propTarget=t.propTarget||{},this.caretR="fa-caret-right",this.caretD="fa-caret-down";var e=this.model;this.listenTo(e,"change:open",this.updateOpen),this.listenTo(e,"updateVisibility",this.updateVisibility),this.listenTo(e,"destroy remove",this.remove);},updateVisibility:function(){var t;this.model.get("properties").each((function(e){e.get("visible")&&(t=1);})),this.el.style.display=t?"block":"none";},updateOpen:function(){this.model.get("open")?this.show():this.hide();},show:function(){this.$el.addClass(this.pfx+"open"),this.getPropertiesEl().style.display="",this.$caret.removeClass(this.caretR).addClass(this.caretD);},hide:function(){this.$el.removeClass(this.pfx+"open"),this.getPropertiesEl().style.display="none",this.$caret.removeClass(this.caretD).addClass(this.caretR);},getPropertiesEl:function(){return this.$el.find(".".concat(this.pfx,"properties")).get(0)},toggle:function(t){var e=this.model.get("open")?0:1;this.model.set("open",e);},render:function(){var t=this.pfx,e=this.model,n=this.em,r=this.$el,i=e.attributes,o=i.id,a=i.name,s=n&&n.t("styleManager.sectors.".concat(o))||a;return r.html(this.template({pfx:t,label:s})),this.$caret=r.find("#".concat(t,"caret")),this.renderProperties(),r.attr("class","".concat(t,"sector ").concat(t,"sector__").concat(o," no-select")),this.updateOpen(),this},renderProperties:function(){var t=this.model.get("properties");if(t){var e=new g.default({collection:t,target:this.target,propTarget:this.propTarget,config:this.config});this.$el.append(e.render().el);}}});function m(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function y(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?m(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):m(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var b=l.a.View.extend({initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.config||{};this.pfx=e.stylePrefix||"",this.ppfx=e.pStylePrefix||"",this.target=t.target||{},this.config=e;var n={};Object(o.extend)(n,l.a.Events);var r=document.body,i=document.createElement("el-".concat((new Date).getTime()));r.appendChild(i),n.computedDefault=y({},window.getComputedStyle(i)),r.removeChild(i),this.propTarget=n;var a=this.collection,s="component:toggled component:update:classes change:state change:device frame:resized";this.listenTo(a,"add",this.addTo),this.listenTo(a,"reset",this.render),this.listenTo(this.target,s,this.targetUpdated);},addTo:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};this.addToCollection(t,null,n);},toggleStateCls:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],e=arguments.length>1?arguments[1]:void 0;t.forEach((function(t){var n=t.getEl();n&&n.classList&&n.classList[e?"add":"remove"]("hc-state");}));},targetUpdated:function(t){var e=this.target,n=this.propTarget,r=e.getSelectedAll(),i=e.getSelected(),o=t&&t.toHTML?t:i;if(o&&this.toggleStateCls([o]),i){var a=e.get("Config").devicePreviewMode?"":e.get("state"),s=e.get("SelectorManager").getConfig().componentFirst,l=i.getEl();if(n.helper=null,n.targets=null,l&&Object(f.o)(l)){var c=a?":".concat(a):null;n.computed=window.getComputedStyle(l,c);}i=e.get("StyleManager").getModelToStyle(i),a&&(!function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},r=e.get("CssComposer"),i=r.getAll(),o=r.getClassRule("hc-state");o?(i.remove(o),i.add(o)):o=r.setClassRule("hc-state"),o.set("important",1),o.setStyle(t),n.helper=o;}(i.getStyle()),this.toggleStateCls(r,1)),n.model=i,s&&(n.targets=r),n.trigger("update");}},setTarget:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.target,r=Object(o.isArray)(t)?t:[t],i=e.targetIsClass,a=e.stylable,s=[];r.forEach((function(t){var e=t;if(Object(o.isString)(t)){var r,l=n.get("CssComposer").getAll();i&&(r=l.filter((function(e){return e.get("selectors").getFullString()===t}))[0]),r||(r=l.filter((function(e){return e.get("selectorsAdd")===t}))[0]),r||(r=l.add({selectors:[],selectorsAdd:t})),a&&r.set({stylable:a}),e=r;}s.push(e);}));var l=this.propTarget;return l.targets=s,l.trigger("update"),s},addToCollection:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.pfx,i=this.target,o=this.propTarget,a=this.config,s=this.el,l=e||s,c=new v({model:t,id:"".concat(r).concat(t.get("id")),name:t.get("name"),properties:t.get("properties"),target:i,propTarget:o,config:a}).render().el;return Object(p.b)(l,c,n.at),c},render:function(){var t=this,e=document.createDocumentFragment(),n=this.$el,r=this.pfx,i=this.ppfx;return n.empty(),this.collection.each((function(n){return t.addToCollection(n,e)})),n.append(e),n.addClass("".concat(r,"sectors ").concat(i,"one-bg ").concat(i,"two-color")),this}});function w(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function x(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?w(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):w(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n,r={};return {name:"StyleManager",getConfig:function(){return r},init:function(i){var o=(r=x({},a,{},i)).pStylePrefix;return this.em=r.em,o&&(r.stylePrefix=o+r.stylePrefix),t=new c.default,e=new d([],r),n=new b({collection:e,target:r.em,config:r}),this},onLoad:function(){e.add(r.sectors,{silent:!0});},postRender:function(){var t=this.getConfig().appendTo;t&&(Object(o.isElement)(t)?t:document.querySelector(t)).appendChild(this.render());},addSector:function(t,n){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=this.getSector(t);return i||(n.id=t,i=e.add(n,r)),i},getSector:function(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=e.where({id:t})[0];return !r&&n.warn&&this._logNoSector(t),r},removeSector:function(t){return this.getSectors().remove(this.getSector(t,{warn:1}))},getSectors:function(){return e},addProperty:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.getSector(t,{warn:1}),i=null;return r&&(i=r.get("properties").add(e,n)),i},getProperty:function(t,e){var n=this.getSector(t,{warn:1}),r=null;return n&&(r=1==(r=n.get("properties").where({property:e})).length?r[0]:r),r},removeProperty:function(t,e){var n=this.getProperties(t);return n&&n.remove(this.getProperty(t,e))},getProperties:function(t){var e=null,n=this.getSector(t,{warn:1});return n&&(e=n.get("properties")),e},getModelToStyle:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=r.em,i=e.skipAdd,o=t.get("classes"),a=t.getId();if(n){var s,l=n.getConfig(),c=n.get("UndoManager"),u=n.get("CssComposer"),h=n.get("SelectorManager"),d=h?h.getConfig():{},f=l.devicePreviewMode?"":n.get("state"),p=o.getStyleable(),g=p.length,v=!d.componentFirst||e.useClasses,m={state:f};if(c.stop(),g&&v){var y=n.getCurrentMedia();(s=u.get(p,f,y))||i||(s=u.add(p,f,y));}else l.avoidInlineStyle&&(!(s=u.getIdRule(a,m))&&!i&&(s=u.setIdRule(a,{},m)),t.is("wrapper")&&s.set("wrapper",1));s&&(t=s),c.start();}return t},addType:function(e,n){t.addType(e,n);},getType:function(e){return t.getType(e)},getTypes:function(){return t.getTypes()},createType:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.model,i=void 0===n?{}:n,o=e.view,a=void 0===o?{}:o,s=this.getType(t);if(s)return new s.view(x({model:new s.model(i),config:r},a))},setTarget:function(t,e){return n.setTarget(t,e)},getEmitter:function(){return n.propTarget},render:function(){return n.render().el},_logNoSector:function(t){var e=this.em;e&&e.logWarning("'".concat(t,"' sector not found"));}}};},function(t,e,n){n.r(e);var r=n(0),i={stylePrefix:"cm-",inlineCss:!1},o=n(2),a=n.n(o),s=n(1),l=n.n(s);function c(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function u(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?c(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):c(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var h=l.a.Model.extend({build:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t.get("components");return e.exportWrapper?t.toHTML(u({},e.wrapperIsBody&&{tag:"body"})):this.buildModels(n)},buildModels:function(t){var e="";return t.each((function(t){e+=t.toHTML();})),e}}),d=n(29),f=l.a.Model.extend({build:function(t){var e=t.toJSON();return this.beforeEach(e),Object(r.each)(e,(function(t,n){var r=e[n];if(r instanceof l.a.Model)e[n]=this.build(r);else if(r instanceof l.a.Collection){var i=r;e[n]=[],i.length&&i.each((function(t,r){e[n][r]=this.build(t);}),this);}}),this),e},beforeEach:function(t){delete t.status;}}),p=l.a.Model.extend({mapModel:function(t){var e="",n=t.get("script-export")||t.get("script"),i=t.get("type"),o=t.get("components"),a=t.getId();if(n){var s=t.get("attributes");s=Object(r.extend)({},s,{id:a}),t.set("attributes",s,{silent:1});var l=t.getScriptString(n);if(t.get("scriptUpdated"))this.mapJs[i+"-"+a]={ids:[a],code:l};else {var c=this.mapJs[i];c?c.ids.push(a):this.mapJs[i]={ids:[a],code:l};}}return o.each((function(t){e+=this.mapModel(t);}),this),e},build:function(t){this.mapJs={},this.mapModel(t);var e="";for(var n in this.mapJs){var r=this.mapJs[n],i="#"+r.ids.join(", #");e+="\n        var items = document.querySelectorAll('".concat(i,"');\n        for (var i = 0, len = items.length; i < len; i++) {\n          (function(){").concat(r.code,"}.bind(items[i]))();\n        }");}return e}}),g=n(12),v=n.n(g);n(80),n(41),n(83);function m(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}var y=l.a.Model.extend({defaults:{input:"",label:"",codeName:"",theme:"hopscotch",readOnly:!0,lineNumbers:!0},init:function(t){return Object(r.bindAll)(this,"onChange"),this.editor=v.a.fromTextArea(t,function(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?m(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):m(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}({dragDrop:!1,lineWrapping:!0,mode:this.get("codeName")},this.attributes)),this.element=t,this.editor.on("change",this.onChange),this},onChange:function(){this.trigger("update",this);},getEditor:function(){return this.editor},getElement:function(){return this.element},setElement:function(t){return this.element=t,this},refresh:function(){return this.getEditor().refresh(),this},focus:function(){return this.getEditor().focus(),this},getContent:function(){var t=this.getEditor();return t&&t.getValue()},setContent:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=this.editor;r&&(r.setValue(t),r.autoFormatRange&&(v.a.commands.selectAll(r),r.autoFormatRange(r.getCursor(!0),r.getCursor(!1)),v.a.commands.goDocStart(r)),!n.noRefresh&&setTimeout((function(){return e.refresh()})));}}),b=l.a.View.extend({template:Object(r.template)('\n  <div class="<%= pfx %>editor" id="<%= pfx %><%= codeName %>">\n  \t<div id="<%= pfx %>title"><%= label %></div>\n  \t<div id="<%= pfx %>code"></div>\n  </div>'),initialize:function(t){this.config=t.config||{},this.pfx=this.config.stylePrefix;},render:function(){var t=this.model.toJSON();return t.pfx=this.pfx,this.$el.html(this.template(t)),this.$el.attr("class",this.pfx+"editor-c"),this.$el.find("#"+this.pfx+"code").append(this.model.get("input")),this}});e.default=function(){var t={},e={},n={},o={},a={};return {getConfig:function(){return t},config:t,EditorView:b,name:"CodeManager",init:function(e){for(var r in t=e||{},i)r in t||(t[r]=i[r]);var o=t.pStylePrefix;return o&&(t.stylePrefix=o+t.stylePrefix),n.html=new h,n.css=new d.a,n.json=new f,n.js=new p,a.CodeMirror=new y,this.loadDefaultGenerators().loadDefaultViewers(),this},addGenerator:function(t,n){return e[t]=n,this},getGenerator:function(t){return e[t]||null},getGenerators:function(){return e},addViewer:function(t,e){return o[t]=e,this},getViewer:function(t){return o[t]||null},getViewers:function(){return o},createViewer:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=Object(r.isUndefined)(t.type)?"CodeMirror":t.type,n=this.getViewer(e)&&this.getViewer(e).clone(),i=document.createElement("div"),o=document.createElement("textarea");return i.appendChild(o),n.set(t),n.init(o),n.setElement(i),n},updateViewer:function(t,e){t.setContent(e);},getCode:function(e,n){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};r.em=t.em;var i=this.getGenerator(n);return i?i.build(e,r):""},loadDefaultGenerators:function(){for(var t in n)this.addGenerator(t,n[t]);return this},loadDefaultViewers:function(){for(var t in a)this.addViewer(t,a[t]);return this}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o={devices:[]},a=n(1),s=n.n(a),l=s.a.Model.extend({idAttribute:"name",defaults:{name:"",width:null,height:"",widthMedia:null,priority:null},initialize:function(){var t=this;null===this.get("widthMedia")&&this.set("widthMedia",this.get("width")),null===this.get("width")&&this.set("width",this.get("widthMedia")),!this.get("priority")&&this.set("priority",parseFloat(this.get("widthMedia"))||0);["width","height","widthMedia"].forEach((function(e){return t.checkUnit(e)}));},checkUnit:function(t){var e=this.get(t)||"";(parseFloat(e)||0).toString()===e.toString()&&this.set(t,"".concat(e,"px"));}}),c=s.a.Collection.extend({model:l,comparator:function(t,e){var n=Number.MAX_VALUE;return (e.get("priority")||n)-(t.get("priority")||n)},getSorted:function(){return this.sort()}}),u=n(0),h=s.a.View.extend({template:Object(u.template)('\n    <div class="<%= ppfx %>device-label"><%= deviceLabel %></div>\n    <div class="<%= ppfx %>field <%= ppfx %>select">\n      <span id="<%= ppfx %>input-holder">\n        <select class="<%= ppfx %>devices"></select>\n      </span>\n      <div class="<%= ppfx %>sel-arrow">\n        <div class="<%= ppfx %>d-s-arrow"></div>\n      </div>\n    </div>\n    <button style="display:none" class="<%= ppfx %>add-trasp">+</button>'),events:{change:"updateDevice"},initialize:function(t){this.config=t.config||{},this.em=this.config.em,this.ppfx=this.config.pStylePrefix||"",this.events["click ."+this.ppfx+"add-trasp"]=this.startAdd,this.listenTo(this.em,"change:device",this.updateSelect),this.delegateEvents();},startAdd:function(){},updateDevice:function(){var t=this.em;if(t){var e=this.devicesEl,n=e?e.val():"";t.set("device",n);}},updateSelect:function(){var t=this.em,e=this.devicesEl;if(t&&t.getDeviceModel&&e){var n=t.getDeviceModel(),r=n?n.get("name"):"";e.val(r);}},getOptions:function(){var t=this.collection,e=this.em,n="";return t.each((function(t){var r=t.attributes,i=r.name,o=r.id,a=e&&e.t&&e.t("deviceManager.devices.".concat(o))||i;n+='<option value="'.concat(i,'">').concat(a,"</option>");})),n},render:function(){var t=this.em,e=this.ppfx,n=this.$el,r=this.el;return n.html(this.template({ppfx:e,deviceLabel:t&&t.t&&t.t("deviceManager.device")})),this.devicesEl=n.find(".".concat(e,"devices")),this.devicesEl.append(this.getOptions()),r.className="".concat(e,"devices-c"),this}});function d(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function f(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?d(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):d(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n={};return {name:"DeviceManager",init:function(r){var i=this;for(var a in n=r||{},o)a in n||(n[a]=o[a]);return t=new c,(n.devices||[]).forEach((function(t){return i.add(t.id||t.name,t.width,t)})),e=new h({collection:t,config:n}),this},add:function(e,n){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=f({},r,{id:e,name:r.name||e,width:n});return t.add(i)},get:function(e){return t.get(e)},getAll:function(){return t},render:function(){return e.render().el}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o={stylePrefix:"mdl-",title:"",content:"",backdrop:!0},a=n(1),s=n.n(a),l=s.a.Model.extend({defaults:{title:"",content:"",open:!1}}),c=s.a.View.extend({template:function(t){var e=t.pfx,n=t.ppfx,r=t.content,i=t.title;return '<div class="'.concat(e,"dialog ").concat(n,"one-bg ").concat(n,'two-color">\n      <div class="').concat(e,'header">\n        <div class="').concat(e,'title">').concat(i,'</div>\n        <div class="').concat(e,'btn-close" data-close-modal>&Cross;</div>\n      </div>\n      <div class="').concat(e,'content">\n        <div id="').concat(e,'c">').concat(r,'</div>\n        <div style="clear:both"></div>\n      </div>\n    </div>\n    <div class="').concat(e,'collector" style="display: none"></div>')},events:{click:"onClick","click [data-close-modal]":"hide"},initialize:function(t){var e=this.model,n=t.config||{},r=n.stylePrefix||"";this.config=n,this.pfx=r,this.ppfx=n.pStylePrefix||"",this.listenTo(e,"change:open",this.updateOpen),this.listenTo(e,"change:title",this.updateTitle),this.listenTo(e,"change:content",this.updateContent);},onClick:function(t){this.config.backdrop&&t.target===this.el&&this.hide();},getCollector:function(){return this.$collector||(this.$collector=this.$el.find("."+this.pfx+"collector")),this.$collector},getContent:function(){var t=this.pfx;return this.$content||(this.$content=this.$el.find(".".concat(t,"content #").concat(t,"c"))),this.$content},getTitle:function(){return this.$title||(this.$title=this.$el.find("."+this.pfx+"title")),this.$title.get(0)},updateContent:function(){var t=this.getContent(),e=t.children(),n=this.getCollector(),r=this.model.get("content");e.length&&n.append(e),t.empty().append(r);},updateTitle:function(){var t=this.getTitle();t&&(t.innerHTML=this.model.get("title"));},updateOpen:function(){this.el.style.display=this.model.get("open")?"":"none";},hide:function(){this.model.set("open",0);},show:function(){this.model.set("open",1);},render:function(){var t=this.$el,e=this.pfx,n=(this.ppfx,this.model.toJSON());return n.pfx=this.pfx,n.ppfx=this.ppfx,t.html(this.template(n)),t.attr("class","".concat(e,"container")),this.updateOpen(),this}});function u(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function h(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?u(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):u(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n={},r=function(t,e){e&&e.trigger("modal:".concat(t?"open":"close"));};return {name:"Modal",getConfig:function(){return n},init:function(){var i=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},a=(n=h({},o,{},i)).em;this.em=a;var s=n.pStylePrefix;return s&&(n.stylePrefix=s+n.stylePrefix),(t=new l(n)).on("change:open",(function(t,e){return r(e,a)})),e=new c({model:t,config:n}),this},postRender:function(t){var e=t.model.getConfig().el||t.el;this.render().appendTo(e);},open:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return t.title&&this.setTitle(t.title),t.content&&this.setContent(t.content),e.show(),this},close:function(){return e.hide(),this},onceClose:function(t){return this.em.once("modal:close",t),this},onceOpen:function(t){return this.em.once("modal:open",t),this},isOpen:function(){return !!t.get("open")},setTitle:function(e){return t.set("title",e),this},getTitle:function(){return t.get("title")},setContent:function(e){return t.set("content"," "),t.set("content",e),this},getContent:function(){return t.get("content")},getContentEl:function(){return e.getContent().get(0)},getModel:function(){return t},render:function(){return e.render().$el}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(0),a=n(3),s={stylePrefix:"clm-",appendTo:"",selectors:[],states:[{name:"hover"},{name:"active"},{name:"nth-of-type(2n)"}],escapeName:0,selectedName:0,iconAdd:'<svg viewBox="0 0 24 24"><path d="M19 13h-6v6h-2v-6H5v-2h6V5h2v6h6v2z"></path></svg>',iconSync:'<svg viewBox="0 0 24 24"><path d="M12 18c-3.31 0-6-2.69-6-6 0-1 .25-1.97.7-2.8L5.24 7.74A7.93 7.93 0 0 0 4 12c0 4.42 3.58 8 8 8v3l4-4-4-4m0-11V1L8 5l4 4V6c3.31 0 6 2.69 6 6 0 1-.25 1.97-.7 2.8l1.46 1.46A7.93 7.93 0 0 0 20 12c0-4.42-3.58-8-8-8z"></path></svg>',iconTagOn:'<svg viewBox="0 0 24 24"><path d="M19 19H5V5h10V3H5c-1.11 0-2 .89-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2v-8h-2m-11.09-.92L6.5 11.5 11 16 21 6l-1.41-1.42L11 13.17l-3.09-3.09z"></path></svg>',iconTagOff:'<svg viewBox="0 0 24 24"><path d="M19 3H5c-1.11 0-2 .89-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5a2 2 0 0 0-2-2m0 2v14H5V5h14z"></path></svg>',iconTagRemove:'<svg viewBox="0 0 24 24"><path d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12 19 6.41z"></path></svg>',render:0,componentFirst:0},l=n(9),c=n(11),u=n(6),h=n.n(u),d=n(1),f=n.n(d),p=f.a.View.extend({template:function(){var t=this.pfx,e=this.model,n=this.config,r=e.get("label")||"";return '\n      <span id="'.concat(t,'checkbox" class="').concat(t,'tag-status" data-tag-status></span>\n      <span id="').concat(t,'tag-label" data-tag-name>').concat(r,'</span>\n      <span id="').concat(t,'close" class="').concat(t,'tag-close" data-tag-remove>\n        ').concat(n.iconTagRemove,"\n      </span>\n    ")},events:{"click [data-tag-remove]":"removeTag","click [data-tag-status]":"changeStatus","dblclick [data-tag-name]":"startEditTag","focusout [data-tag-name]":"endEditTag"},initialize:function(t){var e=t.config||{};this.config=e,this.coll=t.coll||null,this.pfx=e.stylePrefix||"",this.ppfx=e.pStylePrefix||"",this.em=e.em,this.listenTo(this.model,"change:active",this.updateStatus);},getInputEl:function(){return this.inputEl||(this.inputEl=this.el.querySelector("[data-tag-name]")),this.inputEl},startEditTag:function(){var t=this.em,e=this.getInputEl();e.contentEditable=!0,e.focus(),t&&t.setEditing(1);},endEditTag:function(){var t=this.model,e=this.getInputEl(),n=e.textContent,r=this.em,i=r&&r.get("SelectorManager");if(e.contentEditable=!1,r&&r.setEditing(0),i){var o=i.escapeName(n);i.get(o)?e.innerText=t.get("label"):t.set({name:o,label:n});}},changeStatus:function(){var t=this.model;t.set("active",!t.get("active"));},removeTag:function(){var t=this.em,e=this.model;(t&&t.getSelectedAll()).forEach((function(t){!e.get("protected")&&t&&t.getSelectors().remove(e);}));},updateStatus:function(){var t=this.model,e=this.$el,n=this.config,r=n.iconTagOn,i=n.iconTagOff,o=e.find("[data-tag-status]");t.get("active")?(o.html(r),e.removeClass("opac50")):(o.html(i),e.addClass("opac50"));},render:function(){var t=this.pfx,e=this.ppfx;return this.$el.html(this.template()),this.$el.attr("class","".concat(t,"tag ").concat(e,"three-bg")),this.updateStatus(),this}}),g=f.a.View.extend({template:function(t){var e=t.labelInfo,n=t.labelStates,r=t.labelHead,i=t.iconSync,o=t.iconAdd,a=t.pfx,s=t.ppfx;return '\n    <div id="'.concat(a,'up" class="').concat(a,'header">\n      <div id="').concat(a,'label" class="').concat(a,'header-label">').concat(r,'</div>\n      <div id="').concat(a,'status-c" class="').concat(a,'header-status">\n        <span id="').concat(a,'input-c" data-states-c>\n          <div class="').concat(s,"field ").concat(s,'select">\n            <span id="').concat(s,'input-holder">\n              <select id="').concat(a,'states" data-states>\n                <option value="">').concat(n,'</option>\n              </select>\n            </span>\n            <div class="').concat(s,'sel-arrow">\n              <div class="').concat(s,'d-s-arrow"></div>\n            </div>\n          </div>\n        </span>\n      </div>\n    </div>\n    <div id="').concat(a,'tags-field" class="').concat(s,'field">\n      <div id="').concat(a,'tags-c" data-selectors></div>\n      <input id="').concat(a,'new" data-input/>\n      <span id="').concat(a,'add-tag" class="').concat(a,"tags-btn ").concat(a,'tags-btn__add" data-add>\n        ').concat(o,'\n      </span>\n      <span class="').concat(a,"tags-btn ").concat(a,'tags-btn__sync" style="display: none" data-sync-style>\n        ').concat(i,'\n      </span>\n    </div>\n    <div class="').concat(a,'sels-info">\n      <div class="').concat(a,'label-sel">').concat(e,':</div>\n      <div class="').concat(a,'sels" data-selected></div>\n      <div style="clear:both"></div>\n    </div>')},events:{"change [data-states]":"stateChanged","click [data-add]":"startNewTag","focusout [data-input]":"endNewTag","keyup [data-input]":"onInputKeyUp","click [data-sync-style]":"syncStyle"},initialize:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.config=t.config||{},this.pfx=this.config.stylePrefix||"",this.ppfx=this.config.pStylePrefix||"",this.className=this.pfx+"tags",this.stateInputId=this.pfx+"states",this.stateInputC=this.pfx+"input-c",this.states=this.config.states||[];var e=this.config.em,n=this.getStyleEmitter(),r=this.collection;this.target=this.config.em,this.em=e;var i="component:toggled component:update:classes",o="component:update:classes change:state";this.listenTo(e,i,this.componentChanged),this.listenTo(n,"styleManager:update",this.componentChanged),this.listenTo(e,o,this.__handleStateChange),this.listenTo(e,"styleable:change change:device",this.checkSync),this.listenTo(r,"add",this.addNew),this.listenTo(r,"reset",this.renderClasses),this.listenTo(r,"remove",this.tagRemoved),this.delegateEvents();},syncStyle:function(){var t,e=this.em,n=this.getTarget(),r=e.get("CssComposer"),i=this.getCommonSelectors({opts:{noDisabled:1}}),o=e.get("state"),a=e.getCurrentMedia(),s=[],l=r.get(i,o,a)||r.add(i,o,a);this.getTargets().forEach((function(e){var n=r.getIdRule(e.getId(),{state:o,mediaText:a});t=n.getStyle(),n.setStyle({}),s.push(n);})),t&&l.addStyle(t),e.trigger("component:toggled"),e.trigger("component:sync-style",{component:n,selectors:i,mediaText:a,rule:l,ruleComponents:s,state:o});},getStyleEmitter:function(){var t=this.em,e=t&&t.get("StyleManager");return e&&e.getEmitter()||{}},tagRemoved:function(t){this.updateStateVis();},getStateOptions:function(){var t=this.states,e=this.em,n=[];return t.forEach((function(t){return n.push('<option value="'.concat(t.name,'">').concat(e.t("selectorManager.states.".concat(t.name))||t.label||t.name,"</option>"))})),n.join("")},addNew:function(t){this.addToClasses(t);},startNewTag:function(){this.$addBtn.css({display:"none"}),this.$input.show().focus();},endNewTag:function(){this.$addBtn.css({display:""}),this.$input.hide().val("");},onInputKeyUp:function(t){13===t.keyCode?this.addNewTag(this.$input.val()):27===t.keyCode&&this.endNewTag();},checkStates:function(){var t=this.em.getState(),e=this.getStates();e&&e.val(t);},componentChanged:Object(o.debounce)((function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.targets;this.updateSelection(e);})),updateSelection:function(t){var e=t||this.getTargets(),n=[];return (e=Object(o.isArray)(e)?e:[e])&&e.length&&(n=this.getCommonSelectors({targets:e}),this.checkSync({validSelectors:n})),this.collection.reset(n),this.updateStateVis(e),n},getCommonSelectors:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.targets,n=t.opts,r=void 0===n?{}:n,i=e||this.getTargets(),o=i.map((function(t){return t.getSelectors&&t.getSelectors().getValid(r)})).filter((function(t){return t}));return this._commonSelectors.apply(this,h()(o))},_commonSelectors:function(){for(var t=this,e=arguments.length,n=new Array(e),r=0;r<e;r++)n[r]=arguments[r];return n.length?1===n.length?n[0]:2===n.length?n[0].filter((function(t){return n[1].indexOf(t)>=0})):n.slice(1).reduce((function(e,n){return t._commonSelectors(e,n)}),n[0]):[]},checkSync:Object(o.debounce)((function(){var t,e=this.$btnSyncEl,n=this.config,r=this.collection,i=this.getTarget();if(i&&n.componentFirst&&r.length){var a=i.getStyle();t=!Object(o.isEmpty)(a);}e&&e[t?"show":"hide"]();})),getTarget:function(){return this.target.getSelected()},getTargets:function(){return this.target.getSelectedAll()},updateStateVis:function(t){var e=this.em,n=e&&e.getConfig("avoidInlineStyle"),r=this.collection.length||n?"":"none";this.getStatesC().css("display",r),this.updateSelector(t);},__handleStateChange:function(){this.updateSelector(this.getTargets());},updateSelector:function(t){var e=this,n=this.el.querySelector("[data-selected]"),r=[],i=t||this.getTargets();(i=Object(o.isArray)(i)?i:[i]).forEach((function(t){return r.push(e.__getName(t))})),n&&(n.innerHTML=r.join(", ")),this.checkStates();},__getName:function(t){var e,n=this.pfx,r=this.config,i=this.em,a=r.selectedName,s=r.componentFirst;if(Object(o.isString)(t))e='<span class="'.concat(n,'sel-gen">').concat(t,"</span>");else {if(!t||!t.get)return;var l=t.getSelectors().getStyleable(),c=i.get("state"),u=t.getId?'<span class="'.concat(n,'sel-cmp">').concat(t.getName(),'</span><span class="').concat(n,'sel-id">#').concat(t.getId(),"</span>"):"";e=(e=this.collection.getFullString(l))?'<span class="'.concat(n,'sel-rule">').concat(e,"</span>"):t.get("selectorsAdd")||u,e=s&&u?u:e,e+=c?'<span class="'.concat(n,'sel-state">:').concat(c,"</span>"):"",e=a?a({result:e,state:c,target:t}):e;}return e&&'<span class="'.concat(n,'sel">').concat(e,"</span>")},stateChanged:function(t){var e=this.em,n=t.target.value;e.set("state",n);},addNewTag:function(t){var e=this,n=this.em;if(t.trim()){if(n){var r=n.get("SelectorManager").add({label:t});this.getTargets().forEach((function(t){t.getSelectors().add(r),e.collection.add(r),e.updateStateVis();}));}this.endNewTag();}},addToClasses:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,n=e,r=this.getClasses(),i=new p({model:t,config:this.config,coll:this.collection}).render().el;return n?n.appendChild(i):r.append(i),i},renderClasses:function(){var t=this,e=document.createDocumentFragment(),n=this.getClasses();n.empty(),this.collection.each((function(n){return t.addToClasses(n,e)})),n.append(e);},getClasses:function(){return this.$el.find("[data-selectors]")},getStates:function(){if(!this.$states){var t=this.$el.find("[data-states]");this.$states=t[0]&&t;}return this.$states},getStatesC:function(){return this.$statesC||(this.$statesC=this.$el.find("#"+this.stateInputC)),this.$statesC},render:function(){var t=this.em,e=this.pfx,n=this.ppfx,r=this.config,i=this.$el,o=this.el,a=r.render,s={iconSync:r.iconSync,iconAdd:r.iconAdd,labelHead:t.t("selectorManager.label"),labelStates:t.t("selectorManager.emptyState"),labelInfo:t.t("selectorManager.selected"),ppfx:n,pfx:e,el:o};i.html(this.template(s));var l=a&&a(s);l&&l!==o&&i.empty().append(l),this.$input=i.find("[data-input]"),this.$addBtn=i.find("[data-add]"),this.$classes=i.find("#"+e+"tags-c"),this.$btnSyncEl=i.find("[data-sync-style]"),this.$input.hide();var c=this.getStates();return c&&c.append(this.getStateOptions()),this.renderClasses(),i.attr("class","".concat(this.className," ").concat(n,"one-bg ").concat(n,"two-color")),this}});function v(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function m(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?v(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):v(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var y=function(t){return Object(o.isString)(t)&&"#"==t[0]},b=function(t){return Object(o.isString)(t)&&"."==t[0]};e.default=function(t){var e,n=t||{};return {Selector:l.a,Selectors:c.a,name:"SelectorManager",getConfig:function(){return n},init:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},r=(n=m({},s,{},t)).em,i=n.pStylePrefix;return this.em=r,i&&(n.stylePrefix=i+n.stylePrefix),this.selectorTags=new g({collection:new c.a([],{em:r,config:n}),config:n}),(e=new c.a(n.selectors)).on("add",(function(t){return r.trigger("selector:add",t)})),e.on("remove",(function(t){return r.trigger("selector:remove",t)})),e.on("change",(function(t){return r.trigger("selector:update",t,t.previousAttributes(),t.changedAttributes())})),r.on("change:state",(function(t,e){return r.trigger("selector:state",e)})),this},postRender:function(){var t=this.getConfig().appendTo;t&&(Object(o.isElement)(t)?t:document.querySelector(t)).appendChild(this.render([]));},select:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=Array.isArray(t)?t:[t],r=this.em.get("StyleManager").setTarget(n,e),i=r.filter((function(t){return t})).map((function(t){return Object(a.l)(t)?t:Object(a.n)(t)&&!t.get("selectorsAdd")?t:t.getSelectorsString()}));return this.selectorTags.componentChanged({targets:i}),this},setState:function(t){return this.em.setState(t),this},getState:function(){return this.em.setState()},addSelector:function(t){var r=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},i=m({},r);Object(o.isObject)(t)?i=t:i.name=t,y(i.name)?(i.name=i.name.substr(1),i.type=l.a.TYPE_ID):b(i.name)&&(i.name=i.name.substr(1)),i.label&&!i.name&&(i.name=this.escapeName(i.label));var a=i.name,s=a?this.get(a,i.type):e.where(i)[0];return s||e.add(i,{config:n})},getSelector:function(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:l.a.TYPE_CLASS;return y(t)?(t=t.substr(1),n=l.a.TYPE_ID):b(t)&&(t=t.substr(1)),e.where({name:t,type:n})[0]},add:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return Object(o.isArray)(t)?t.map((function(t){return e.addSelector(t,n)})):this.addSelector(t,n)},addClass:function(t){var e=this,n=[];return Object(o.isString)(t)&&(t=t.trim().split(" ")),t.forEach((function(t){return n.push(e.addSelector(t))})),n},get:function(t,e){var n=this;if(Object(o.isArray)(t)){var r=[];return t.map((function(t){return n.getSelector(t)})).filter((function(t){return t})).forEach((function(t){return r.indexOf(t)<0&&r.push(t)})),r}return this.getSelector(t,e)},getAll:function(){return e},escapeName:function(t){var e=n.escapeName;return e?e(t):l.a.escapeName(t)},render:function(t){return t?(this.selectorTags=new g({collection:new c.a(t),config:n}),this.selectorTags.render().el):this.selectorTags.render().el}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o={textTags:["br","b","i","u","a","ul","ol"],parserCss:null,parserHtml:null},a=n(0),s={4:"media",5:"font-face",6:"page",7:"keyframes",11:"counter-style",12:"supports",13:"document",14:"font-feature-values",15:"viewport"},l=Object(a.keys)(s),c=["5","6","11","15"],u=["font-face","page","counter-style","viewport"],h=function(){for(var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"",e=[],n=[],r=t.split(","),i=0,o=r.length;i<o;i++){var a=r[i].trim();if(/^(\.{1}[\w\-]+)+(:{1,2}[\w\-()]+)?$/gi.test(a)||/^(#{1}[\w\-]+){1}(:{1,2}[\w\-()]+)?$/gi.test(a)){var s=a.split(".").filter(Boolean);n.push(s);}else e.push(a);}return {result:n,add:e}},d=function(t){for(var e=t.style,n={},r=0,i=e.length;r<i;r++){var o=e[r],a=e.getPropertyValue(o),s=e.getPropertyPriority(o);n[o]="".concat(a).concat(s?" !".concat(s):"");}return n},f=function(t){return (t.conditionText||t.media&&t.media.mediaText||t.name||t.selectorText||"").trim()},p=function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r={},i=t.length,o=t[i-1],a=o?o.split(/:(.+)/):[],s=a[1],l=n.atRule,c=n.selectorsAdd,h=n.mediaText,d=u.indexOf(l)>=0;return d&&(r.singleAtRule=1),l&&(r.atRuleType=l),c&&(r.selectorsAdd=c),h&&(r.mediaText=h),s&&(t[i-1]=a[0],r.state=s,a.splice(a.length-1,1)),r.selectors=t,r.style=e,r},g=function(t){var e=document.createElement("style");e.innerHTML=t,document.head.appendChild(e);var n=e.sheet;return document.head.removeChild(e),function t(e){for(var n=[],r=e.cssRules||[],i=0,o=r.length;i<o;i++){var a=r[i],u=a.type.toString(),g=0,v="",m="",y=a.selectorText||a.keyText,b=c.indexOf(u)>=0;if(b)g=1,v=s[u],m=f(a);else if(l.indexOf(u)>=0){var w=t(a);m=f(a);for(var x=0,O=w.length;x<O;x++){var C=w[x];m&&(C.mediaText=m),C.atRuleType=s[u];}n=n.concat(w);}if(y||b){for(var S=d(a),k=h(y),T=k.add,j=void 0,P=0,E=(y=k.result).length;P<E;P++){var M=p(y[P],S,{atRule:s[u]});n.push(M),j=M;}if(T.length){var D=T.join(", ");if(j)j.selectorsAdd=D;else {var A={selectors:[],selectorsAdd:D,style:S};g&&(A.singleAtRule=g),v&&(A.atRuleType=v),m&&(A.mediaText=m),n.push(A);}}}}return n}(n)},v=function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return {parse:function(e){var n=this,r=[],i=t.parserCss,o=t.em,a=void 0===o?{}:o,s=a&&a.get&&a.get("Editor");return (i?i(e,s):g(e)).forEach((function(t){return r=r.concat(n.checkNode(t))})),r},checkNode:function(t){var e=t,n=e.selectors,r=e.style;if(Object(a.isString)(n)){var i=[],o=h(n),s=o.result,l=o.add.join(", "),c={atRule:t.atRule,mediaText:t.params};if(s.length?s.forEach((function(t){i.push(p(t,r,c));})):i.push(p([],r,c)),l)i[i.length-1].selectorsAdd=l;t=i;}return t}}},m=n(27);function y(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function b(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?y(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):y(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n={};return {compTypes:"",parserCss:null,parserHtml:null,name:"Parser",getConfig:function(){return n},init:function(){var r=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return (n=b({},o,{},r)).Parser=this,t=new m.a(n),e=new v(n),this.em=n.em,this.parserCss=e,this.parserHtml=t,this},parseHtml:function(n){var r=this.em,i=this.compTypes;return t.compTypes=r?r.get("DomComponents").getTypes():i,t.parse(n,e)},parseCss:function(t){return e.parse(t)}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o={id:"gjs-",autosave:1,autoload:1,type:"local",stepsBeforeSave:1,storeComponents:1,storeStyles:1,storeHtml:1,storeCss:1,checkLocal:1,params:{},headers:{},urlStore:"",urlLoad:"",beforeSend:function(t,e){},onComplete:function(t,e){},contentTypeJson:!0,credentials:"include",fetchOptions:""},a=n(1),s=n.n(a),l=s.a.Model.extend({defaults:{checkLocal:!0},store:function(t,e){for(var n in this.checkStorageEnvironment(),t)localStorage.setItem(n,t[n]);"function"==typeof e&&e();},load:function(t,e){this.checkStorageEnvironment();for(var n={},r=0,i=t.length;r<i;r++){var o=localStorage.getItem(t[r]);o&&(n[t[r]]=o);}return "function"==typeof e&&e(n),n},remove:function(t){this.checkStorageEnvironment();for(var e=0,n=t.length;e<n;e++)localStorage.removeItem(t[e]);},checkStorageEnvironment:function(){this.get("checkLocal")&&!localStorage&&console.warn("Your browser doesn't support localStorage");}}),c=n(26),u=n(0);function h(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function d(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?h(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):h(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var f=s.a.Model.extend({fetch:c.a,defaults:{urlStore:"",urlLoad:"",params:{},beforeSend:function(){},onComplete:function(){},contentTypeJson:!1,credentials:"include",fetchOptions:""},onStart:function(){this.get("em");var t=this.get("beforeSend");t&&t();},onError:function(t,e){if(e)e(t);else {var n=this.get("em");console.error(t),n&&n.trigger("storage:error",t);}},onResponse:function(t,e){var n=this.get("em"),r=this.get("onComplete"),i=this.get("contentTypeJson")&&(t&&"string"==typeof t)?JSON.parse(t):t;r&&r(i),e&&e(i),n&&n.trigger("storage:response",i);},store:function(t,e,n){var r={};for(var i in t)r[i]=t[i];this.request(this.get("urlStore"),{body:r},e,n);},load:function(t,e,n){this.request(this.get("urlLoad"),{method:"get"},e,n);},request:function(t){var e,n,r=this,i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},o=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null,a=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,s=this.get("contentTypeJson"),l=this.get("headers")||{},c=this.get("params"),h="X-Requested-With",f="Content-Type",p=i.body||{};for(var g in c)p[g]=c[g];if(Object(u.isUndefined)(l[h])&&(l[h]="XMLHttpRequest"),Object(u.isUndefined)(l[f])&&s&&(l[f]="application/json; charset=utf-8"),s)n=JSON.stringify(p);else for(var v in n=new FormData,p)n.append(v,p[v]);"post"===(e={method:i.method||"post",credentials:this.get("credentials"),headers:l}).method&&(e.body=n);var m=this.get("fetchOptions")||{},y=Object(u.isFunction)(m)?m(e):e;this.onStart(),this.fetch(t,d({},e,{},y||{})).then((function(t){return 1==(t.status/200|0)?t.text():t.text().then((function(t){return Promise.reject(t)}))})).then((function(t){return r.onResponse(t,o)})).catch((function(t){return r.onError(t,a)}));}});function p(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function g(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?p(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):p(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e={},n={},r={};return {name:"StorageManager",init:function(){var n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return e=g({},o,{},n),t=e.em,e._disable&&(e.type=0),r.remote=new f(e),r.local=new l(e),e.currentStorage=e.type,this.loadDefaultProviders().setCurrent(e.type),this},getConfig:function(){return e},isAutosave:function(){return !!e.autosave},setAutosave:function(t){return e.autosave=!!t,this},getStepsBeforeSave:function(){return e.stepsBeforeSave},setStepsBeforeSave:function(t){return e.stepsBeforeSave=t,this},add:function(t,e){return n[t]=e,this},get:function(t){return n[t]||null},getStorages:function(){return n},getCurrent:function(){return e.currentStorage},setCurrent:function(t){return e.currentStorage=t,this},store:function(t,n){var r=this,i=this.get(this.getCurrent()),o={};for(var a in this.onStart("store",t),t)o[e.id+a]=t[a];return i?i.store(o,(function(t){n&&n(t),r.onEnd("store",t);}),(function(t){r.onError("store",t);})):null},load:function(t,n){var r=this,i=this.get(this.getCurrent()),o=[],a={};"string"==typeof t&&(t=[t]),this.onStart("load",t);for(var s=0,l=t.length;s<l;s++)o.push(e.id+t[s]);i?i.load(o,(function(t){var i=new RegExp("^"+e.id);for(var o in t){var s=o.replace(i,"");a[s]=t[o];}n&&n(a),r.onEnd("load",a);}),(function(t){n&&n(a),r.onError("load",t);})):n&&n(a);},loadDefaultProviders:function(){for(var t in r)this.add(t,r[t]);return this},getCurrentStorage:function(){return this.get(this.getCurrent())},onStart:function(e,n){t&&(t.trigger("storage:start"),e&&t.trigger("".concat("storage:start",":").concat(e),n));},onEnd:function(e,n){t&&(t.trigger("storage:end"),e&&t.trigger("".concat("storage:end",":").concat(e),n));},onError:function(e,n){t&&(t.trigger("storage:error",n),e&&t.trigger("".concat("storage:error",":").concat(e),n),this.onEnd(e,n));},canAutoload:function(){return this.getCurrentStorage()&&this.getConfig().autoload}}};},function(t,e,n){n.r(e);var r=n(16),i=n(2),o=n.n(i),a=n(1),s=n.n(a),l=n(0),c=n(3);function u(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function h(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?u(n,!0).forEach((function(e){o()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):u(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var d=s.a.$,f=s.a.View.extend({initialize:function(t){this.opt=t||{},Object(l.bindAll)(this,"startSort","onMove","endMove","rollback","updateOffset","moveDragHelper");var e=t||{};this.elT=0,this.elL=0,this.borderOffset=e.borderOffset||10;var n=e.container;this.el="string"==typeof n?document.querySelector(n):n,this.$el=d(this.el),this.containerSel=e.containerSel||"div",this.itemSel=e.itemSel||"div",this.draggable=e.draggable||!0,this.nested=e.nested||0,this.pfx=e.pfx||"",this.ppfx=e.ppfx||"",this.freezeClass=e.freezeClass||this.pfx+"freezed",this.onStart=e.onStart||"",this.onEndMove=e.onEndMove||"",this.direction=e.direction||"v",this.onMoveClb=e.onMove||"",this.relative=e.relative||0,this.ignoreViewChildren=e.ignoreViewChildren||0,this.ignoreModels=e.ignoreModels||0,this.plh=e.placer||"",this.wmargin=e.wmargin||0,this.offTop=e.offsetTop||0,this.offLeft=e.offsetLeft||0,this.document=e.document||document,this.$document=d(this.document),this.dropContent=null,this.em=e.em||"",this.dragHelper=null,this.canvasRelative=e.canvasRelative||0,this.selectOnEnd=!e.avoidSelectOnEnd,this.scale=e.scale,this.activeTextModel=null,this.em&&this.em.on&&(this.em.on("change:canvasOffset",this.updateOffset),this.updateOffset());},getScale:function(){return Object(l.result)(this,scale)||1},getContainerEl:function(t){if(t&&(this.el=t),!this.el){var e=this.opt.container;this.el="string"==typeof e?document.querySelector(e):e,this.$el=d(this.el);}return this.el},getDocuments:function(t){var e=this.em,n=t?t.ownerDocument:e&&e.get("Canvas").getBody().ownerDocument,r=[document];return n&&r.push(n),r},updateOffset:function(){var t=this.em.get("canvasOffset")||{};this.offTop=t.top,this.offLeft=t.left;},setDropContent:function(t){this.dropModel=null,this.dropContent=t;},updateTextViewCursorPosition:function(t){var e=this.em.get("Canvas"),n=e.getDocument(),r=null;if(n.caretRangeFromPoint){var i=Object(c.h)(t);r=n.caretRangeFromPoint(i.clientX,i.clientY);}else t.rangeParent&&(r=n.createRange()).setStart(t.rangeParent,t.rangeOffset);var o=e.getWindow().getSelection();e.getFrameEl().focus(),o.removeAllRanges(),r&&o.addRange(r);},setContentEditable:function(t,e){if(t){var n=t.getEl();n.contentEditable!=e&&(n.contentEditable=e);}},toggleSortCursor:function(t){var e=this.em,n=e&&e.get("Canvas");n&&(t?n.startAutoscroll():n.stopAutoscroll());},setDragHelper:function(t,e){for(var n=e||"",r=t.cloneNode(1),i=t.getBoundingClientRect(),o=getComputedStyle(t),a="",s=0;s<o.length;s++){var l=o[s];a+="".concat(l,":").concat(o.getPropertyValue(l),";");}document.body.appendChild(r),r.className+=" ".concat(this.pfx,"bdrag"),r.setAttribute("style",a),this.dragHelper=r,r.style.width="".concat(i.width,"px"),r.style.height="".concat(i.height,"px"),n&&this.moveDragHelper(n),this.em&&d(this.em.get("Canvas").getBody().ownerDocument).off("mousemove",this.moveDragHelper).on("mousemove",this.moveDragHelper),d(document).off("mousemove",this.moveDragHelper).on("mousemove",this.moveDragHelper);},moveDragHelper:function(t){var e=t.target.ownerDocument;if(this.dragHelper&&e){var n=t.pageY,r=t.pageX,i=0,o=0,a=(e.defaultView||e.parentWindow).frameElement,s=this.dragHelper.style;if(a){var l=a.getBoundingClientRect();i=l.top+document.documentElement.scrollTop,o=l.left+document.documentElement.scrollLeft,n=t.clientY,r=t.clientX;}s.top=n+i+"px",s.left=r+o+"px";}},matches:function(t,e,n){return c.q.call(t,e)},closest:function(t,e){if(t){for(var n=t.parentNode;n&&1===n.nodeType;){if(this.matches(n,e))return n;n=n.parentNode;}return null}},offset:function(t){var e=t.getBoundingClientRect();return {top:e.top+document.body.scrollTop,left:e.left+document.body.scrollLeft}},createPlaceholder:function(){var t=this.pfx,e=document.createElement("div"),n=document.createElement("div");return e.className=t+"placeholder",e.style.display="none",e.style["pointer-events"]="none",n.className=t+"placeholder-int",e.appendChild(n),e},startSort:function(t){var e,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=this.em,i=this.itemSel,o=this.containerSel,a=this.getContainerEl(n.container),s=this.getDocuments(t),l=this.onStart,u=this.plh;this.dropModel=null,this.target=null,this.prevTarget=null,this.moved=0,t&&!this.matches(t,"".concat(i,", ").concat(o))&&(t=this.closest(t,i)),this.eV=t,u||(u=this.createPlaceholder(),a.appendChild(u),this.plh=u),t&&((e=this.getSourceModel(t))&&e.set&&e.set("status","freezed"),this.srcModel=e),Object(c.t)(a,"mousemove dragover",this.onMove),Object(c.t)(s,"mouseup dragend touchend",this.endMove),Object(c.t)(s,"keydown",this.rollback),l&&l({target:e,parent:e&&e.parent(),index:e&&e.index()}),r&&r.clearSelection(),this.toggleSortCursor(1),r&&r.trigger("sorter:drag:start",t,e);},getTargetModel:function(t){var e=t||this.target;return d(e).data("model")},getSourceModel:function(t){var e=this,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.target,i=n.avoidChildren,o=void 0===i?1:i,a=this.em,s=this.eV,l=t||s,c=this.dropModel,u=this.dropContent,f=function(t){return t&&r&&t.opt&&t.opt.avoidChildren&&e.isTextableActive(t,r)};if(u&&a){if(f(c)&&(c=null),!c){var p=a.get("DomComponents").getComponents(),g={avoidChildren:o,avoidStore:1,avoidUpdateStyle:1},v=p.add(u,h({},g,{temporary:1}));if(c=(c=p.remove(v,g))instanceof Array?c[0]:c,this.dropModel=c,f(c))return this.getSourceModel(l,{target:r,avoidChildren:0})}return c}return l&&d(l).data("model")},selectTargetModel:function(t){if(!(t instanceof s.a.Collection)){var e=this.targetModel;e&&e!==this.srcModel&&e.set("status",""),t&&t.set&&(t.set("status","selected-parent"),this.targetModel=t);}},onMove:function(t){var e=t,n=this.em,r=this.onMoveClb,i=this.plh;this.moved=1;var o=i.style.display;o&&"none"!==o||(i.style.display="block");var a=this.offset(this.el);this.elT=this.wmargin?Math.abs(a.top):a.top,this.elL=this.wmargin?Math.abs(a.left):a.left;var s=t.pageY-this.elT+this.el.scrollTop,c=t.pageX-this.elL+this.el.scrollLeft;if(this.canvasRelative&&n){var u=n.get("Canvas").getMouseRelativeCanvas(t,{noScroll:1});c=u.x,s=u.y;}this.rX=c,this.rY=s,this.eventMove=t;var h=this.getSourceModel(),f=this.dimsFromTarget(t.target,c,s),p=this.target,g=p&&this.getTargetModel(p);if(this.selectTargetModel(g),g||(i.style.display="none"),p){this.lastDims=f;var v=this.findPosition(f,c,s);this.isTextableActive(h,g)?(this.activeTextModel=g,this.setContentEditable(g,!0),i.style.display="none",this.lastPos=v,this.updateTextViewCursorPosition(e)):(this.disableTextable(),this.activeTextModel=null,this.lastPos&&this.lastPos.index==v.index&&this.lastPos.method==v.method||(this.movePlaceholder(this.plh,f,v,this.prevTargetDim),this.$plh||(this.$plh=d(this.plh)),this.canvasRelative||(this.offTop&&this.$plh.css("top","+="+this.offTop+"px"),this.offLeft&&this.$plh.css("left","+="+this.offLeft+"px")),this.lastPos=v)),Object(l.isFunction)(r)&&r({event:t,target:h,parent:g,index:v.index+("after"==v.method?1:0)}),n&&n.trigger("sorter:drag",{target:p,targetModel:g,sourceModel:h,dims:f,pos:v,x:c,y:s});}},isTextableActive:function(t,e){return t&&t.get&&t.get("textable")&&e&&e.is("text")},disableTextable:function(){var t=this.activeTextModel;t&&t.getView().disableEditing();},isInFlow:function(t,e){if(!t)return !1;e=e||document.body;var n=t;return n.offsetHeight,!!this.styleInFlow(n,e)},styleInFlow:function(t,e){if(!Object(c.p)(t)){var n=t.style||{},r=d(t),i=e&&d(e);if(!(n.overflow&&"visible"!==n.overflow||"none"!==r.css("float")||i&&"flex"==i.css("display")&&"column"!==i.css("flex-direction"))){switch(n.position){case"static":case"relative":case"":break;default:return}switch(t.tagName){case"TR":case"TBODY":case"THEAD":case"TFOOT":return !0}switch(r.css("display")){case"block":case"list-item":case"table":case"flex":return !0}}}},validTarget:function(t,e){var n=this.getTargetModel(t),r=this.getSourceModel(e,{target:n}),i={valid:!0,src:e=r&&r.view&&r.view.el,srcModel:r,trg:t=n&&n.view&&n.view.el,trgModel:n};if(!e||!t)return i.valid=!1,i;var o=r.get("draggable");o=o instanceof Array?o.join(", "):o,i.dragInfo=o,o=Object(l.isString)(o)?this.matches(t,o):o,i.draggable=o;var a=n.get("droppable");return a=(a=a instanceof s.a.Collection?1:a)instanceof Array?a.join(", "):a,i.dropInfo=a,a=Object(l.isString)(a)?this.matches(e,a):a,a=o&&this.isTextableActive(r,n)?1:a,i.droppable=a,a&&o||(i.valid=!1),i},dimsFromTarget:function(t,e,n){var r=this.em,i=[];if(!t)return i;if(this.matches(t,"".concat(this.itemSel,", ").concat(this.containerSel))||(t=this.closest(t,this.itemSel)),this.draggable instanceof Array&&(t=this.closest(t,this.draggable.join(","))),!t)return i;if(this.prevTarget&&this.prevTarget!=t&&(this.prevTarget=null),!this.prevTarget){this.targetP=this.closest(t,this.containerSel);var o=this.validTarget(t);if(r&&r.trigger("sorter:drag:validation",o),!o.valid&&this.targetP)return this.dimsFromTarget(this.targetP,e,n);this.prevTarget=t,this.prevTargetDim=this.getDim(t),this.cacheDimsP=this.getChildrenDim(this.targetP),this.cacheDims=this.getChildrenDim(t);}if(this.prevTarget==t&&(i=this.cacheDims),this.target=this.prevTarget,this.nearBorders(this.prevTargetDim,e,n)||!this.nested&&!this.cacheDims.length){var a=this.targetP;a&&this.validTarget(a).valid&&(i=this.cacheDimsP,this.target=a);}return this.lastPos=null,i},getTargetFromEl:function(t){var e,n=t,r=this.targetPrev,i=this.em,o=this.containerSel,a=this.itemSel;if(this.matches(n,"".concat(a,", ").concat(o))||(n=this.closest(n,a)),this.draggable instanceof Array&&(n=this.closest(n,this.draggable.join(","))),r&&r!=n&&(this.targetPrev=""),!this.targetPrev){e=this.closest(n,o);var s=this.validTarget(n);if(i&&i.trigger("sorter:drag:validation",s),!s.valid&&e)return this.getTargetFromEl(e);this.targetPrev=n;}return this.nearElBorders(n)&&(e=this.closest(n,o))&&this.validTarget(e).valid&&(n=e),n},nearElBorders:function(t){var e=t.getBoundingClientRect(),n=t.ownerDocument.body,r=this.getCurrentPos(),i=r.x,o=r.y,a=e.top+n.scrollTop,s=e.left+n.scrollLeft,l=e.width,c=e.height;if(o<a+10||o>a+c-10||i<s+10||i>s+l-10)return 1},getCurrentPos:function(){var t=this.eventMove;return {x:t.pageX||0,y:t.pageY||0}},getDim:function(t){var e,n,r,i,o=this.em;if(this.canvasRelative&&o){var a=o.get("Canvas"),s=a.getElementPos(t,{noScroll:1}),l=a.getElementOffsets(t);e=s.top-l.marginTop,n=s.left-l.marginLeft,r=s.height+l.marginTop+l.marginBottom,i=s.width+l.marginLeft+l.marginRight;}else {var c=this.offset(t);e=this.relative?t.offsetTop:c.top-(this.wmargin?-1:1)*this.elT,n=this.relative?t.offsetLeft:c.left-(this.wmargin?-1:1)*this.elL,r=t.offsetHeight,i=t.offsetWidth;}return [e,n,r,i]},getChildrenDim:function(t){var e=this,n=[];if(!t)return n;var r=this.getTargetModel(t);if(r&&r.view&&!this.ignoreViewChildren){var i=r.getCurrentView?r.getCurrentView():r.view;t=i.getChildrenContainer();}return Object(l.each)(t.children,(function(r,i){var o=Object(c.g)(r,d),a=o&&o.index?o.index():i;if(Object(c.p)(r)||e.matches(r,e.itemSel)){var s=e.getDim(r),l=e.direction;l="v"==l||"h"!=l&&e.isInFlow(r,t),s.push(l,r,a),n.push(s);}})),n},nearBorders:function(t,e,n){var r=0,i=this.borderOffset,o=e||0,a=n||0,s=t[0],l=t[1],c=t[2],u=t[3];return (s+i>a||a>s+c-i||l+i>o||o>l+u-i)&&(r=1),!!r},findPosition:function(t,e,n){for(var r={index:0,indexEl:0,method:"before"},i=0,o=0,a=0,s=0,l=0,c=0,u=0,h=0,d=0,f=t.length;d<f;d++)if(a=(h=t[d])[1]+h[3],u=h[0]+h[2],l=h[1]+h[3]/2,c=h[0]+h[2]/2,!(o&&h[1]>o||s&&c>=s||i&&a<i))if(r.index=d,r.indexEl=h[6],h[4]){if(n<c){r.method="before";break}r.method="after";}else n<u&&(s=u),e<l?(o=l,r.method="before"):(i=l,r.method="after");return r},movePlaceholder:function(t,e,n,r){var i=0,o=0,a=0,s=0,l="px",c=n.method,u=e[n.index];if(t.classList.remove("vertical"),t.classList.add("horizontal"),u)u[4]?(a=u[3]+l,s="auto",i="before"==c?u[0]-0:u[0]+u[2]-0,o=u[1]):(a="auto",s=u[2]-0+l,i=u[0]+0,o="before"==c?u[1]-0:u[1]+u[3]-0,t.classList.remove("horizontal"),t.classList.add("vertical"));else {if(!this.nested)return void(t.style.display="none");r&&(i=r[0]+5,o=r[1]+5,a=parseInt(r[3])-10+l,s="auto");}t.style.top=i+l,t.style.left=o+l,a&&(t.style.width=a),s&&(t.style.height=s);},endMove:function(t){var e,n=this,r=this.eV,i=[],o=this.getDocuments(),a=this.getContainerEl(),s=this.onEndMove,u=this.target,d=this.lastPos;if(Object(c.s)(a,"mousemove dragover",this.onMove),Object(c.s)(o,"mouseup dragend touchend",this.endMove),Object(c.s)(o,"keydown",this.rollback),this.plh.style.display="none",r&&(e=this.getSourceModel(),this.selectOnEnd&&e&&e.set&&(e.set("status",""),e.set("status","selected"))),this.moved){var f=this.toMove;(Object(l.isArray)(f)?f:f?[f]:[r]).forEach((function(t){i.push(n.move(u,t,d));}));}this.plh&&(this.plh.style.display="none");var p=this.dragHelper;if(p&&(p.parentNode.removeChild(p),this.dragHelper=null),this.disableTextable(),this.selectTargetModel(),this.toggleSortCursor(),this.toMove=null,this.eventMove=0,this.dropModel=null,Object(l.isFunction)(s)){var g={target:e,parent:e&&e.parent(),index:e&&e.index()};i.length?i.forEach((function(t){return s(t,n,g)})):s(null,this,h({},g,{cancelled:1}));}},move:function(t,e,n){var r=this.em,i=this.activeTextModel,o=this.dropContent,a=Object(c.e)(e);r&&r.trigger("component:dragEnd:before",t,a,n);var l,u,f,p=[],g=n.indexEl,v=this.validTarget(t,a),m=d(t).data("collection"),y=v.srcModel,b=v.droppable,w=v.draggable,x=v.dropInfo,O=v.dragInfo,C=v.trgModel;b=C instanceof s.a.Collection?1:b;var S=this.isTextableActive(y,C);if(m&&b&&w){var k={at:g="after"===n.method?g+1:g,noIncrement:1};if(o?(l=o,k.silent=!1,k.avoidUpdateStyle=1):(k.temporary=1,u=m.add({},h({},k)),y.collection&&(l=y.collection.remove(y,{temporary:1}))),S){var T=i.getView();i.trigger("active");var j=T.activeRte,P=y.getEl();delete y.opt.temporary,y.getView().render(),P.setAttribute("data-gjs-textable","true");var E=P.outerHTML;j.insertHTML&&j.insertHTML(E);}else f=m.add(l,k);o?this.dropContent=null:m.remove(u),this.prevTarget=null;}else m||p.push("Target collection not found"),b||p.push("Target is not droppable, accepts [".concat(x,"]")),w||p.push("Component not draggable, acceptable by [".concat(O,"]")),console.warn("Invalid target position: "+p.join(", "));return r&&r.trigger("component:dragEnd",m,l,p),r&&r.trigger("sorter:drag:end",{targetCollection:m,modelToDrop:l,warns:p,validResult:v,dst:t,srcEl:a}),f},rollback:function(t){Object(c.s)(this.getDocuments(),"keydown",this.rollback),27==(t.which||t.keyCode)&&(this.moved=0,this.endMove());}}),p=n(14),g=n.n(p),v=n(15),m=n.n(v);function y(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function b(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?y(n,!0).forEach((function(e){o()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):y(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var w={mousePosFetcher:null,updateTarget:null,ratioDefault:0,posFetcher:null,onStart:null,onMove:null,onEnd:null,onUpdateContainer:function(){},step:1,minDim:32,maxDim:"",unitHeight:"px",unitWidth:"px",keyHeight:"height",keyWidth:"width",currentUnit:1,silentFrames:0,avoidContainerUpdate:0,keepAutoHeight:!1,keepAutoWidth:!1,autoHeight:!1,autoWidth:!1,tl:1,tc:1,tr:1,cl:1,cr:1,bl:1,bc:1,br:1},x=function(t,e){var n=e||window,r=t.getBoundingClientRect();return {left:r.left+n.pageXOffset,top:r.top+n.pageYOffset,width:r.width,height:r.height}},O=function(){function t(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return g()(this,t),this.setOptions(e),Object(l.bindAll)(this,"handleKeyDown","handleMouseDown","move","stop"),this}return m()(t,[{key:"getConfig",value:function(){return this.opts}},{key:"setOptions",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};this.opts=Object(l.defaults)(t,w),this.setup();}},{key:"setup",value:function(){var t=this.opts,e=t.prefix||"",n=t.appendTo||document.body,r=this.container;for(r||((r=document.createElement("div")).className="".concat(e,"resizer-c"),n.appendChild(r),this.container=r);r.firstChild;)r.removeChild(r.firstChild);var i={};for(var o in ["tl","tc","tr","cl","cr","bl","bc","br"].forEach((function(e){return i[e]=t[e]?function(t,e){var n=e.prefix||"",r=document.createElement("i");return r.className=n+"resizer-h "+n+"resizer-h-"+t,r.setAttribute("data-"+n+"handler",t),r}(e,t):""})),i){var a=i[o];a&&r.appendChild(a);}this.handlers=i,this.mousePosFetcher=t.mousePosFetcher,this.updateTarget=t.updateTarget,this.posFetcher=t.posFetcher,this.onStart=t.onStart,this.onMove=t.onMove,this.onEnd=t.onEnd,this.onUpdateContainer=t.onUpdateContainer;}},{key:"toggleFrames",value:function(t){if(this.opts.silentFrames){var e=document.querySelectorAll("iframe");Object(l.each)(e,(function(e){return e.style.pointerEvents=t?"none":""}));}}},{key:"isHandler",value:function(t){var e=this.handlers;for(var n in e)if(e[n]===t)return !0;return !1}},{key:"getFocusedEl",value:function(){return this.el}},{key:"getDocumentEl",value:function(){return [this.el.ownerDocument,document]}},{key:"getElementPos",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.posFetcher||"";return n?n(t,e):x(t)}},{key:"focus",value:function(t){t&&t===this.el||(this.el=t,this.updateContainer({forceShow:1}),Object(c.t)(this.getDocumentEl(),"mousedown",this.handleMouseDown));}},{key:"blur",value:function(){this.container.style.display="none",this.el&&(Object(c.s)(this.getDocumentEl(),"mousedown",this.handleMouseDown),this.el=null);}},{key:"start",value:function(t){if(0===t.button){t.preventDefault(),t.stopPropagation();var e=this.el,n=this.opts||{},r="data-"+n.prefix+"handler",i=this.getElementPos(e,{target:"el"});this.handlerAttr=t.target.getAttribute(r),this.clickedHandler=t.target,this.startDim={t:i.top,l:i.left,w:i.width,h:i.height},this.rectDim={t:i.top,l:i.left,w:i.width,h:i.height},this.startPos={x:t.clientX,y:t.clientY};var o=this.getDocumentEl();Object(c.t)(o,"mousemove",this.move),Object(c.t)(o,"keydown",this.handleKeyDown),Object(c.t)(o,"mouseup",this.stop),Object(l.isFunction)(this.onStart)&&this.onStart(t,{docs:o,config:n,el:e,resizer:this}),this.toggleFrames(1),this.move(t);}}},{key:"move",value:function(t){var e=this.onMove,n=this.mousePosFetcher,r=n?n(t):{x:t.clientX,y:t.clientY};this.currentPos=r,this.delta={x:r.x-this.startPos.x,y:r.y-this.startPos.y},this.keys={shift:t.shiftKey,ctrl:t.ctrlKey,alt:t.altKey},this.rectDim=this.calc(this),this.updateRect(0),e&&e(t),0===t.which&&this.stop(t);}},{key:"stop",value:function(t){var e=this.opts,n=this.getDocumentEl();Object(c.s)(n,"mousemove",this.move),Object(c.s)(n,"keydown",this.handleKeyDown),Object(c.s)(n,"mouseup",this.stop),this.updateRect(1),this.toggleFrames(),Object(l.isFunction)(this.onEnd)&&this.onEnd(t,{docs:n,config:e});}},{key:"updateRect",value:function(t){var e=this.el,n=this.opts,r=this.rectDim,i=this.updateTarget,o=this.getSelectedHandler(),a=n.unitHeight,s=n.unitWidth,c=n.keyWidth,u=n.keyHeight;if(Object(l.isFunction)(i))i(e,r,{store:t,selectedHandler:o,resizer:this,config:n});else {var h=e.style;h[c]=r.w+s,h[u]=r.h+a;}this.updateContainer();}},{key:"updateContainer",value:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=this.opts,n=this.container,r=this.el,i=n.style;!e.avoidContainerUpdate&&r&&t.forceShow&&(i.display="block"),this.onUpdateContainer({el:n,resizer:this,opts:b({},e,{},t)});}},{key:"getSelectedHandler",value:function(){var t=this.handlers;if(this.selectedHandler)for(var e in t)if(t[e]===this.selectedHandler)return e}},{key:"handleKeyDown",value:function(t){27===t.keyCode&&(this.rectDim=this.startDim,this.stop(t));}},{key:"handleMouseDown",value:function(t){var e=t.target;this.isHandler(e)?(this.selectedHandler=e,this.start(t)):e!==this.el&&(this.selectedHandler="",this.blur());}},{key:"calc",value:function(t){var e,n=this.opts||{},r=n.step,i=this.startDim,o=n.minDim,a=n.maxDim,s=t.delta.x,l=t.delta.y,u=i.w,h=i.h,d={t:0,l:0,w:u,h:h};if(t){var f=t.handlerAttr;~f.indexOf("r")&&(e=Object(c.r)(u+s*r,r),e=Math.max(o,e),a&&(e=Math.min(a,e)),d.w=e),~f.indexOf("b")&&(e=Object(c.r)(h+l*r,r),e=Math.max(o,e),a&&(e=Math.min(a,e)),d.h=e),~f.indexOf("l")&&(e=Object(c.r)(u-s*r,r),e=Math.max(o,e),a&&(e=Math.min(a,e)),d.w=e),~f.indexOf("t")&&(e=Object(c.r)(h-l*r,r),e=Math.max(o,e),a&&(e=Math.min(a,e)),d.h=e);var p=n.ratioDefault?!t.keys.shift:t.keys.shift;if(f.indexOf("c")<0&&p){var g=i.w/i.h;d.w/d.h>g?d.h=Math.round(d.w/g):d.w=Math.round(d.h*g);}return ~f.indexOf("l")&&(d.l=i.w-d.w),~f.indexOf("t")&&(d.t=i.h-d.h),d}}}]),t}(),C={init:function(t){return new O(t)}};e.default=function(){return {name:"Utils",init:function(){return this},Sorter:f,Resizer:C,Dragger:r.a}};},function(t,e,n){n.r(e);var r=n(6),i=n.n(r),o=n(2),a=n.n(o),s=n(14),l=n.n(s),c=n(15),u=n.n(c),h=n(3);function d(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function f(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?d(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):d(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var p="_rte",g=1,v=0,m=-1,y=function(t){var e=t.selection().anchorNode,n=e&&e.parentNode,r=e&&e.nextSibling;return n&&"A"==n.nodeName||r&&"A"==r.nodeName},b={bold:{name:"bold",icon:"<b>B</b>",attributes:{title:"Bold"},result:function(t){return t.exec("bold")}},italic:{name:"italic",icon:"<i>I</i>",attributes:{title:"Italic"},result:function(t){return t.exec("italic")}},underline:{name:"underline",icon:"<u>U</u>",attributes:{title:"Underline"},result:function(t){return t.exec("underline")}},strikethrough:{name:"strikethrough",icon:"<strike>S</strike>",attributes:{title:"Strike-through"},result:function(t){return t.exec("strikeThrough")}},link:{icon:'<span style="transform:rotate(45deg)">&supdsub;</span>',name:"link",attributes:{style:"font-size:1.4rem;padding:0 4px 2px;",title:"Link"},state:function(t,e){return t&&t.selection()&&y(t)?g:v},result:function(t){y(t)?t.exec("unlink"):t.insertHTML('<a class="link" href="">'.concat(t.selection(),"</a>"));}}},w=function(){function t(){var e=this,n=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};l()(this,t);var r=n.el;if(r[p])return r[p];r[p]=this,this.setEl(r),this.updateActiveActions=this.updateActiveActions.bind(this);var i=n.actions||[];i.forEach((function(t,e){"string"==typeof t?t=b[t]:b[t.name]&&(t=f({},b[t.name],{},t)),i[e]=t;}));var o=i.length?i:Object.keys(b).map((function(t){return b[t]}));n.classes=f({},{actionbar:"actionbar",button:"action",active:"active",disabled:"disabled",inactive:"inactive"},{},n.classes);var a=n.classes,s=n.actionbar;if(this.actionbar=s,this.settings=n,this.classes=a,this.actions=o,!s){var c=n.actionbarContainer;(s=document.createElement("div")).className=a.actionbar,c.appendChild(s),this.actionbar=s,o.forEach((function(t){return e.addAction(t)}));}return n.styleWithCSS&&this.exec("styleWithCSS"),this.syncActions(),this}return u()(t,[{key:"destroy",value:function(){this.el=0,this.doc=0,this.actionbar=0,this.settings={},this.classes={},this.actions=[];}},{key:"setEl",value:function(t){this.el=t,this.doc=t.ownerDocument;}},{key:"updateActiveActions",value:function(){var t=this;this.getActions().forEach((function(e){var n=e.btn,r=e.update,i=f({},t.classes),o=i.active,a=i.inactive,s=i.disabled,l=e.state,c=e.name,u=t.doc;if(n.className=n.className.replace(o,"").trim(),n.className=n.className.replace(a,"").trim(),n.className=n.className.replace(s,"").trim(),l)switch(l(t,u)){case g:n.className+=" ".concat(o);break;case v:n.className+=" ".concat(a);break;case m:n.className+=" ".concat(s);}else u.queryCommandSupported(c)&&u.queryCommandState(c)&&(n.className+=" ".concat(o));r&&r(t,e);}));}},{key:"enable",value:function(){return this.enabled?this:(this.actionbarEl().style.display="",this.el.contentEditable=!0,Object(h.t)(this.el,"mouseup keyup",this.updateActiveActions),this.syncActions(),this.updateActiveActions(),this.el.focus(),this.enabled=1,this)}},{key:"disable",value:function(){return this.actionbarEl().style.display="none",this.el.contentEditable=!1,Object(h.s)(this.el,"mouseup keyup",this.updateActiveActions),this.enabled=0,this}},{key:"syncActions",value:function(){var t=this;this.getActions().forEach((function(e){if(t.settings.actionbar&&(!e.state||e.state&&e.state(t,t.doc)>=0)){var n=e.event||"click";e.btn["on".concat(n)]=function(n){e.result(t,e),t.updateActiveActions();};}}));}},{key:"addAction",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=e.sync,r=document.createElement("span"),i=t.icon,o=t.attributes||{};for(var a in r.className=this.classes.button,t.btn=r,o)r.setAttribute(a,o[a]);"string"==typeof i?r.innerHTML=i:r.appendChild(i),this.actionbarEl().appendChild(r),n&&(this.actions.push(t),this.syncActions());}},{key:"getActions",value:function(){return this.actions}},{key:"selection",value:function(){return this.doc.getSelection()}},{key:"exec",value:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;this.doc.execCommand(t,!1,e);}},{key:"actionbarEl",value:function(){return this.actionbar}},{key:"insertHTML",value:function(t){var e=this.doc,n=e.getSelection();if(n&&n.rangeCount){var r=e.createElement("div"),i=n.getRangeAt(0);i.deleteContents(),r.innerHTML=t,Array.prototype.slice.call(r.childNodes).forEach((function(t){i.insertNode(t),t;})),n.removeAllRanges(),n.addRange(i),this.el.focus();}}}]),t}(),x={stylePrefix:"rte-",adjustToolbar:1,actions:["bold","italic","underline","strikethrough","link"]};function O(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function C(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?O(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):O(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n,r={},o=function(){var e=t.style;e.top="-1000px",e.left="-1000px",e.display="none";};return {customRte:null,name:"RichTextEditor",getConfig:function(){return r},init:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},i=(r=C({},x,{},e)).pStylePrefix;return i&&(r.stylePrefix=i+r.stylePrefix),this.pfx=r.stylePrefix,r.actions||[],(t=document.createElement("div")).className="".concat(i,"rte-toolbar ").concat(i,"one-bg"),n=this.initRte(document.createElement("div")),Object(h.t)(t,"mousedown",(function(t){return t.stopPropagation()})),this},destroy:function(){var e=this.customRte;n&&n.destroy(),e&&e.destroy&&e.destroy(),t=0,n=0,this.actionbar=0,this.actions=0;},postRender:function(e){var n=e.model.get("Canvas");t.style.pointerEvents="all",o(),n.getToolsEl().appendChild(t);},initRte:function(e){var o=this.pfx,a=t,s=this.actionbar,l=this.actions||i()(r.actions),c={actionbar:"".concat(o,"actionbar"),button:"".concat(o,"action"),active:"".concat(o,"active"),inactive:"".concat(o,"inactive"),disabled:"".concat(o,"disabled")},u=new w({el:e,classes:c,actions:l,actionbar:s,actionbarContainer:a});return n&&n.setEl(e),u.actionbar&&(this.actionbar=u.actionbar),u.actions&&(this.actions=u.actions),u},add:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};e.name=t,n.addAction(e,{sync:1});},get:function(t){var e;return n.getActions().forEach((function(n){n.name==t&&(e=n);})),e},getAll:function(){return n.getActions()},remove:function(t){var e=this.getAll(),n=this.get(t);if(n){var r=n.btn,i=e.indexOf(n);r.parentNode.removeChild(r),e.splice(i,1);}return n},getToolbarEl:function(){return t},updatePosition:function(){var n=r.em.get("Canvas"),i=t.style,o=n.getTargetToElementFixed(e,t,{event:"rteToolbarPosUpdate"});i.top=o.top+"px",i.left="0px";},enable:function(n,i){e=n.el;var o=r.em.get("Canvas"),a=r.em,s=n.getChildrenContainer(),l=this.customRte;if(o.getElementPos(e),t.style.display="",i=l?l.enable(s,i):this.initRte(s).enable(),a){setTimeout(this.updatePosition.bind(this),0);var c="change:canvasOffset canvasScroll frame:scroll component:update";a.off(c,this.updatePosition,this),a.on(c,this.updatePosition,this),a.trigger("rte:enable",n,i);}return i},disable:function(t,e){var n=r.em,i=this.customRte,a=t.getChildrenContainer();i?i.disable(a,e):e&&e.disable(),o(),n&&n.trigger("rte:disable",t,e);}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(8),a=n.n(o),s=n(0),l={placeholder:"eg. Text here"},c={locale:"en",localeFallback:"en",detectLocale:1,debug:0,messages:{en:{assetManager:{addButton:"Add image",inputPlh:"http://path/to/the/image.jpg",modalTitle:"Select Image",uploadTitle:"Drop files here or click to upload"},blockManager:{labels:{},categories:{}},domComponents:{names:{"":"Box",wrapper:"Body",text:"Text",comment:"Comment",image:"Image",video:"Video",label:"Label",link:"Link",map:"Map",tfoot:"Table foot",tbody:"Table body",thead:"Table head",table:"Table",row:"Table row",cell:"Table cell"}},deviceManager:{device:"Device",devices:{desktop:"Desktop",tablet:"Tablet",mobileLandscape:"Mobile Landscape",mobilePortrait:"Mobile Portrait"}},panels:{buttons:{titles:{preview:"Preview",fullscreen:"Fullscreen","sw-visibility":"View components","export-template":"View code","open-sm":"Open Style Manager","open-tm":"Settings","open-layers":"Open Layer Manager","open-blocks":"Open Blocks"}}},selectorManager:{label:"Classes",selected:"Selected",emptyState:"- State -",states:{hover:"Hover",active:"Click","nth-of-type(2n)":"Even/Odd"}},styleManager:{empty:"Select an element before using Style Manager",layer:"Layer",fileButton:"Images",sectors:{general:"General",layout:"Layout",typography:"Typography",decorations:"Decorations",extra:"Extra",flex:"Flex",dimension:"Dimension"},properties:{}},traitManager:{empty:"Select an element before using Trait Manager",label:"Component settings",traits:{labels:{},attributes:{id:l,alt:l,title:l,href:{placeholder:"eg. https://google.com"}},options:{target:{false:"This window",_blank:"New window"}}}}}}};function u(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function h(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?u(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):u(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}var d=function(t){return !Array.isArray(t)&&null!==t&&"object"===a()(t)};e.default=function(){return {name:"I18n",config:c,init:function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};return this.config=h({},c,{},t,{messages:h({},c.messages,{},t.messages||{})}),this.config.detectLocale&&(this.config.locale=this._localLang()),this.em=t.em,this},getConfig:function(){return this.config},setLocale:function(t){var e=this.em,n=this.config,r={value:t,valuePrev:n.locale};return e&&e.trigger("i18n:locale",r),n.locale=t,this},getLocale:function(){return this.config.locale},getMessages:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.config.messages;return t&&!n[t]&&this._debug("'".concat(t,"' i18n lang not found"),e),t?n[t]:n},setMessages:function(t){var e=this.em;return this.config.messages=t,e&&e.trigger("i18n:update",t),this},addMessages:function(t){var e=this.em,n=this.config.messages;return e&&e.trigger("i18n:add",t),this.setMessages(function t(){for(var e=h({},arguments.length<=0?void 0:arguments[0]),n=1;n<arguments.length;n++){var r=h({},n<0||arguments.length<=n?void 0:arguments[n]);for(var i in r){var o=e[i],a=r[i];d(o)&&d(a)?e[i]=t(o,a):e[i]=a;}}return e}(n,t)),this},t:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.config,r=e.params||{},i=e.l||this.getLocale(),o=e.lFlb||n.localeFallback,a=this._getMsg(t,i,e);return a||(a=this._getMsg(t,o,e)),!a&&this._debug("'".concat(t,"' i18n key not found in '").concat(i,"' lang"),e),a=a&&Object(s.isString)(a)?this._addParams(a,r):a},_localLang:function(){var t=window.navigator||{},e=t.language||t.userLanguage;return e?e.split("-")[0]:"en"},_addParams:function(t,e){var n=new RegExp("{([\\w\\d-]*)}","g");return t.replace(n,(function(t,n){return e[n]||""})).trim()},_getMsg:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=this.getMessages(e,n);if(r){var i=r[t];return !i&&t.indexOf(".")>0&&(i=t.split(".").reduce((function(t,e){if(!Object(s.isUndefined)(t))return t[e]}),r)),i}},_debug:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.em,r=this.config;(e.debug||r.debug)&&n&&n.logWarning(t);}}};},function(t,e,n){n.r(e);var r=n(2),i=n.n(r),o=n(0),a={stylePrefix:"css-",staticRules:"",rules:[]},s=n(33),l=n(32),c=n(36),u=n(11),h=n(9);function d(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function f(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?d(n,!0).forEach((function(e){i()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):d(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e,n,r={};return {Selectors:u.a,name:"CssComposer",getConfig:function(){return r},storageKey:function(){var t=[],e=r.stm&&r.stm.getConfig()||{};return e.storeCss&&t.push("css"),e.storeStyles&&t.push("styles"),t},init:function(i){for(var o in r=i||{},a)o in r||(r[o]=a[o]);var s=r.pStylePrefix;s&&(r.stylePrefix=s+r.stylePrefix);var u=r.em&&r.em.config.style||"";return r.rules=u||r.rules,t=r.em,e=new l.a([],r),n=new c.a({collection:e,config:r}),this},onLoad:function(){e.add(r.rules);},postLoad:function(t){var e=this,n="add remove",r=this.getAll(),i=t.get("UndoManager");i&&i.add(r),t.stopListening(r,n,this.handleChange),t.listenTo(r,n,this.handleChange),r.each((function(t){return e.handleChange(t,{avoidStore:1})}));},handleChange:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r="change:style",i=t.get("UndoManager");i&&i.add(e);var o=t.handleUpdates.bind(t);t.stopListening(e,r,o),t.listenTo(e,r,o),!n.avoidStore&&o("","",n);},load:function(t){var n=t||"";!n&&r.stm&&(n=r.em.getCacheLoad());var i=n.styles||"";if(n.styles)try{i=JSON.parse(n.styles);}catch(t){}else n.css&&(i=r.em.get("Parser").parseCss(n.css));return Object(o.isArray)(i)?i.length&&e.reset(i):i&&e.reset(i),i},store:function(t){if(r.stm){var n={},i=this.storageKey();return i.indexOf("css")>=0&&(n.css=r.em.getCss()),i.indexOf("styles")>=0&&(n.styles=JSON.stringify(e)),t||r.stm.store(n),n}},add:function(t,n,i){var o=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},a=n||"",l=i||"",c=f({},o),u=this.get(t,a,l,c);return u&&u.config&&!u.config.singleAtRule?u:(c.state=a,c.mediaText=l,c.selectors="",(u=new s.a(c,r)).get("selectors").add(t),e.add(u),u)},get:function(t,n,r,i){var o=null;return e.each((function(e){o||e.compare(t,n,r,i)&&(o=e);})),o},getAll:function(){return e},clear:function(){return this.getAll().reset(),this},addCollection:function(t){for(var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=[],i=t instanceof Array?t:[t],o=0,a=i.length;o<a;o++){var s=i[o]||{};if(s.selectors){var l=r.em&&r.em.get("SelectorManager");l||console.warn("Selector Manager not found");for(var c=s.selectors,u=c instanceof Array?c:[c],h=[],d=0,p=u.length;d<p;d++){var g=l.add(u[d]);h.push(g);}var v=this.get(h,s.state,s.mediaText,s),m=this.add(h,s.state,s.mediaText,s),y=!v||!e.avoidUpdateStyle,b=s.style||{};if(y){var w=e.extend?f({},m.get("style"),{},b):b;m.set("style",w);}n.push(m);}}return n},setRule:function(e,n){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=r.atRuleType,o=r.atRuleParams,a=t.get("Parser").parserCss.checkNode({selectors:e,style:n})[0],s=a.state,l=a.selectorsAdd,c=t.get("SelectorManager"),u=c.add(a.selectors),h=this.add(u,s,o,{selectorsAdd:l,atRule:i});return h.setStyle(n,r),h},getRule:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=t.get("SelectorManager"),i=t.get("Parser").parserCss.checkNode({selectors:e})[0],o=r.get(i.selectors),a=i.state,s=i.selectorsAdd,l=n.atRuleType,c=n.atRuleParams;return o&&this.get(o,a,c,{selectorsAdd:s,atRule:l})},setIdRule:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=r.state||"",o=r.mediaText||t.getCurrentMedia(),a=t.get("SelectorManager"),s=a.add({name:e,type:h.a.TYPE_ID}),l=this.add(s,i,o);return l.setStyle(n,r),l},getIdRule:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.state||"",i=n.mediaText||t.getCurrentMedia(),o=t.get("SelectorManager").get(e,h.a.TYPE_ID);return o&&this.get(o,r,i)},setClassRule:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=r.state||"",o=r.mediaText||t.getCurrentMedia(),a=t.get("SelectorManager"),s=a.add({name:e,type:h.a.TYPE_CLASS}),l=this.add(s,i,o);return l.setStyle(n,r),l},getClassRule:function(e){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},r=n.state||"",i=n.mediaText||t.getCurrentMedia(),o=t.get("SelectorManager").get(e,h.a.TYPE_CLASS);return o&&this.get(o,r,i)},render:function(){return n.render().el}}};},function(t,e,n){n.r(e);var r=n(6),i=n.n(r),o=n(2),a=n.n(o),s=n(0),l=n(21),c={stylePrefix:"com-",defaults:[],strict:1},u=n(4);function h(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.push.apply(n,r);}return n}function d(t){for(var e=1;e<arguments.length;e++){var n=null!=arguments[e]?arguments[e]:{};e%2?h(n,!0).forEach((function(e){a()(t,e,n[e]);})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):h(n).forEach((function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e));}));}return t}e.default=function(){var t,e={},r={},o={},a={},h=[["preview","Preview","preview"],["resize","Resize","resize"],["fullscreen","Fullscreen","fullscreen"],["copy","CopyComponent"],["paste","PasteComponent"],["canvas-move","CanvasMove"],["canvas-clear","CanvasClear"],["open-code","ExportTemplate","export-template"],["open-layers","OpenLayers","open-layers"],["open-styles","OpenStyleManager","open-sm"],["open-traits","OpenTraitManager","open-tm"],["open-blocks","OpenBlocks","open-blocks"],["open-assets","OpenAssets","open-assets"],["component-select","SelectComponent","select-comp"],["component-outline","SwitchVisibility","sw-visibility"],["component-offset","ShowOffset","show-offset"],["component-move","MoveComponent","move-comp"],["component-next","ComponentNext"],["component-prev","ComponentPrev"],["component-enter","ComponentEnter"],["component-exit","ComponentExit","select-parent"],["component-delete","ComponentDelete"],["component-style-clear","ComponentStyleClear"],["component-drag","ComponentDrag"]];return {CommandAbstract:l.default,name:"Commands",init:function(){var r=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};e=d({},c,{},r),t=e.em;var a=e.pStylePrefix;for(var l in a&&(e.stylePrefix=a+e.stylePrefix),e.defaults){var f=e.defaults[l];f.id&&this.add(f.id,f);}return o["tlb-delete"]={run:function(t){return t.runCommand("core:component-delete")}},o["tlb-clone"]={run:function(t){t.runCommand("core:copy"),t.runCommand("core:paste");}},o["tlb-move"]={run:function(t,e){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=t.getModel(),o=n&&n.event,a=n.target,l=a||t.getSelected(),c=a?[a]:i()(t.getSelectedAll()),h=o&&"dragstart"==o.type,d={preserveSelected:1},f=["absolute","translate"],p=l.get("dmode")||r.get("dmode"),g=function(){return r.stopDefault(d)},v=Object(s.includes)(f,p);if(c.forEach((function(t){return t.trigger("disable")})),!l||!l.get("draggable"))return r.logWarning("The element is not draggable");h?setTimeout(g,0):g();var m=function(t){r.trigger("".concat(u.eventDrag,":start"),t);},y=function(t){r.trigger(u.eventDrag,t);},b=function(e,n,i){r.runDefault(d),c.forEach((function(t){return t.set("status","selected")})),t.select(c),l.emitUpdate(),r.trigger("".concat(u.eventDrag,":end"),i),(v||i.cancelled)&&r.set("_cmpDrag",1);};if(v)t.runCommand("core:component-drag",{guidesInfo:1,mode:p,target:l,onStart:m,onDrag:y,onEnd:b,event:o});else {h&&o.dataTransfer.setDragImage(l.view.el,0,0);var w=t.Commands.get("move-comp");w.onStart=m,w.onDrag=y,w.onEndMoveFromModel=b,w.initSorterFromModels(c);}c.forEach((function(t){return t.set("status","freezed-selected")}));}},o["core:undo"]=function(t){return t.UndoManager.undo()},o["core:redo"]=function(t){return t.UndoManager.redo()},h.forEach((function(e){var r=e[2],i=n(88)("./".concat(e[1])).default,a="core:".concat(e[0]);o[a]=i,r&&(o[r]=i,["run","stop"].forEach((function(e){t.on("".concat(e,":").concat(r),(function(){for(var n,r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];return (n=t).trigger.apply(n,["".concat(e,":").concat(a)].concat(i))}));})));})),e.em&&(e.model=e.em.get("Canvas")),this.loadDefaultCommands(),this},add:function(t,e){return Object(s.isFunction)(e)&&(e={run:e}),e.stop||(e.noStop=1),delete e.initialize,e.id=t,r[t]=l.default.extend(e),this},get:function(n){var i=r[n];return Object(s.isFunction)(i)?(i=new i(e),r[n]=i):i||t.logWarning("'".concat(n,"' command not found")),i},extend:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=this.get(t);if(n){var r=d({},n.constructor.prototype,{},e);this.add(t,r);var i=h.filter((function(e){return "core:".concat(e[0])===t&&e[2]}))[0];i&&this.add(i[2],r);}return this},has:function(t){return !!r[t]},getAll:function(){return r},run:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return this.runCommand(this.get(t),e)},stop:function(t){var e=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return this.stopCommand(this.get(t),e)},isActive:function(t){return this.getActive().hasOwnProperty(t)},getActive:function(){return a},loadDefaultCommands:function(){for(var t in o)this.add(t,o[t]);return this},runCommand:function(n){var r,i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(n&&n.run){var o=n.id,s=t.get("Editor");this.isActive(o)&&!i.force&&e.strict||(r=n.callRun(s,i),o&&n.stop&&!n.noStop&&!i.abort&&(a[o]=r));}return r},stopCommand:function(n){var r,i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(n&&n.run){var o=n.id,s=t.get("Editor");(this.isActive(o)||i.force||!e.strict)&&(o&&delete a[o],r=n.callStop(s,i));}return r},create:function(t){return t.stop||(t.noStop=1),new(l.default.extend(t))(e)}}};}]).default}));

});

var grapesjs = unwrapExports(grapes_min);
var grapes_min_1 = grapes_min.grapesjs;

var grapesjsPresetNewsletter_min = createCommonjsModule(function (module, exports) {
/*! grapesjs-preset-newsletter - 0.2.20 */
!function(t,e){module.exports=e(grapes_min);}("undefined"!=typeof self?self:commonjsGlobal,function(t){return function(t){function e(n){if(r[n])return r[n].exports;var i=r[n]={i:n,l:!1,exports:{}};return t[n].call(i.exports,i,i.exports,e),i.l=!0,i.exports}var r={};return e.m=t,e.c=r,e.d=function(t,r,n){e.o(t,r)||Object.defineProperty(t,r,{configurable:!1,enumerable:!0,get:n});},e.n=function(t){var r=t&&t.__esModule?function(){return t.default}:function(){return t};return e.d(r,"a",r),r},e.o=function(t,e){return Object.prototype.hasOwnProperty.call(t,e)},e.p="",e(e.s=46)}([function(t,e){var r;r=function(){return this}();try{r=r||Function("return this")()||(0,eval)("this");}catch(t){"object"==typeof window&&(r=window);}t.exports=r;},function(t,e){"function"==typeof Object.create?t.exports=function(t,e){t.super_=e,t.prototype=Object.create(e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}});}:t.exports=function(t,e){t.super_=e;var r=function(){};r.prototype=e.prototype,t.prototype=new r,t.prototype.constructor=t;};},function(t,e,r){function n(t){if(!(this instanceof n))return new n(t);c.call(this,t),l.call(this,t),t&&!1===t.readable&&(this.readable=!1),t&&!1===t.writable&&(this.writable=!1),this.allowHalfOpen=!0,t&&!1===t.allowHalfOpen&&(this.allowHalfOpen=!1),this.once("end",i);}function i(){this.allowHalfOpen||this._writableState.ended||a.nextTick(o,this);}function o(t){t.end();}var a=r(14),s=Object.keys||function(t){var e=[];for(var r in t)e.push(r);return e};t.exports=n;var u=r(8);u.inherits=r(1);var c=r(36),l=r(21);u.inherits(n,c);for(var f=s(l.prototype),h=0;h<f.length;h++){var p=f[h];n.prototype[p]||(n.prototype[p]=l.prototype[p]);}Object.defineProperty(n.prototype,"writableHighWaterMark",{enumerable:!1,get:function(){return this._writableState.highWaterMark}}),Object.defineProperty(n.prototype,"destroyed",{get:function(){return void 0!==this._readableState&&void 0!==this._writableState&&(this._readableState.destroyed&&this._writableState.destroyed)},set:function(t){void 0!==this._readableState&&void 0!==this._writableState&&(this._readableState.destroyed=t,this._writableState.destroyed=t);}}),n.prototype._destroy=function(t,e){this.push(null),this.end(),a.nextTick(e,t);};},function(t,e,r){function n(e,r){return delete t.exports[e],t.exports[e]=r,r}var i=r(30),o=r(58);t.exports={Parser:i,Tokenizer:r(31),ElementType:r(6),DomHandler:o,get FeedHandler(){return n("FeedHandler",r(60))},get Stream(){return n("Stream",r(61))},get WritableStream(){return n("WritableStream",r(35))},get ProxyHandler(){return n("ProxyHandler",r(75))},get DomUtils(){return n("DomUtils",r(4))},get CollectingHandler(){return n("CollectingHandler",r(86))},DefaultHandler:o,get RssHandler(){return n("RssHandler",this.FeedHandler)},parseDOM:function(t,e){var r=new o(e);return new i(r,e).end(t),r.dom},parseFeed:function(e,r){var n=new t.exports.FeedHandler(r);return new i(n,r).end(e),n.dom},createDomStream:function(t,e,r){var n=new o(t,e,r);return new i(n,e)},EVENTS:{attribute:2,cdatastart:0,cdataend:0,text:1,processinginstruction:2,comment:1,commentend:0,closetag:1,opentag:2,opentagname:1,error:1,end:0}};},function(t,e,r){var n=t.exports;[r(76),r(81),r(82),r(83),r(84),r(85)].forEach(function(t){Object.keys(t).forEach(function(e){n[e]=t[e].bind(n);});});},function(t,e){t.exports=function(t){return t.webpackPolyfill||(t.deprecate=function(){},t.paths=[],t.children||(t.children=[]),Object.defineProperty(t,"loaded",{enumerable:!0,get:function(){return t.l}}),Object.defineProperty(t,"id",{enumerable:!0,get:function(){return t.i}}),t.webpackPolyfill=1),t};},function(t,e){t.exports={Text:"text",Directive:"directive",Comment:"comment",Script:"script",Style:"style",Tag:"tag",CDATA:"cdata",Doctype:"doctype",isTag:function(t){return "tag"===t.type||"script"===t.type||"style"===t.type}};},function(t,e){function r(){throw new Error("setTimeout has not been defined")}function n(){throw new Error("clearTimeout has not been defined")}function i(t){if(l===setTimeout)return setTimeout(t,0);if((l===r||!l)&&setTimeout)return l=setTimeout,setTimeout(t,0);try{return l(t,0)}catch(e){try{return l.call(null,t,0)}catch(e){return l.call(this,t,0)}}}function o(t){if(f===clearTimeout)return clearTimeout(t);if((f===n||!f)&&clearTimeout)return f=clearTimeout,clearTimeout(t);try{return f(t)}catch(e){try{return f.call(null,t)}catch(e){return f.call(this,t)}}}function a(){g&&p&&(g=!1,p.length?d=p.concat(d):v=-1,d.length&&s());}function s(){if(!g){var t=i(a);g=!0;for(var e=d.length;e;){for(p=d,d=[];++v<e;)p&&p[v].run();v=-1,e=d.length;}p=null,g=!1,o(t);}}function u(t,e){this.fun=t,this.array=e;}function c(){}var l,f,h=t.exports={};!function(){try{l="function"==typeof setTimeout?setTimeout:r;}catch(t){l=r;}try{f="function"==typeof clearTimeout?clearTimeout:n;}catch(t){f=n;}}();var p,d=[],g=!1,v=-1;h.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var r=1;r<arguments.length;r++)e[r-1]=arguments[r];d.push(new u(t,e)),1!==d.length||g||i(s);},u.prototype.run=function(){this.fun.apply(null,this.array);},h.title="browser",h.browser=!0,h.env={},h.argv=[],h.version="",h.versions={},h.on=c,h.addListener=c,h.once=c,h.off=c,h.removeListener=c,h.removeAllListeners=c,h.emit=c,h.prependListener=c,h.prependOnceListener=c,h.listeners=function(t){return []},h.binding=function(t){throw new Error("process.binding is not supported")},h.cwd=function(){return "/"},h.chdir=function(t){throw new Error("process.chdir is not supported")},h.umask=function(){return 0};},function(t,e,r){(function(t){function r(t){return Array.isArray?Array.isArray(t):"[object Array]"===v(t)}function n(t){return "boolean"==typeof t}function i(t){return null===t}function o(t){return null==t}function a(t){return "number"==typeof t}function s(t){return "string"==typeof t}function u(t){return "symbol"==typeof t}function c(t){return void 0===t}function l(t){return "[object RegExp]"===v(t)}function f(t){return "object"==typeof t&&null!==t}function h(t){return "[object Date]"===v(t)}function p(t){return "[object Error]"===v(t)||t instanceof Error}function d(t){return "function"==typeof t}function g(t){return null===t||"boolean"==typeof t||"number"==typeof t||"string"==typeof t||"symbol"==typeof t||void 0===t}function v(t){return Object.prototype.toString.call(t)}e.isArray=r,e.isBoolean=n,e.isNull=i,e.isNullOrUndefined=o,e.isNumber=a,e.isString=s,e.isSymbol=u,e.isUndefined=c,e.isRegExp=l,e.isObject=f,e.isDate=h,e.isError=p,e.isFunction=d,e.isPrimitive=g,e.isBuffer=t.isBuffer;}).call(e,r(12).Buffer);},function(t,e,r){var n=r(11),i=r(23),o={tag:!0,script:!0,style:!0};e.isTag=function(t){return t.type&&(t=t.type),o[t]||!1},e.camelCase=function(t){return t.replace(/[_.-](\w|$)/g,function(t,e){return e.toUpperCase()})},e.cssCase=function(t){return t.replace(/[A-Z]/g,"-$&").toLowerCase()},e.domEach=function(t,e){for(var r=0,n=t.length;r<n&&!1!==e.call(t,r,t[r]);)++r;return t},e.cloneDom=function(t,e){return n(i(t,e),e).children};var a=/^(?:[^#<]*(<[\w\W]+>)[^>]*$|#([\w\-]*)$)/;e.isHtml=function(t){if("<"===t.charAt(0)&&">"===t.charAt(t.length-1)&&t.length>=3)return !0;var e=a.exec(t);return !(!e||!e[1])};},function(t,e){t.exports={trueFunc:function(){return !0},falseFunc:function(){return !1}};},function(t,e,r){(function(n){var i=r(3);e=t.exports=function(t,r){var n=e.evaluate(t,r),i=e.evaluate("<root></root>",r)[0];return i.type="root",e.update(n,i),i},e.evaluate=function(t,e){return "string"==typeof t||n.isBuffer(t)?i.parseDOM(t,e):t},e.update=function(t,e){Array.isArray(t)||(t=[t]),e?e.children=t:e=null;for(var r=0;r<t.length;r++){var n=t[r],i=n.parent||n.root,o=i&&i.children;o&&o!==t&&(o.splice(o.indexOf(n),1),n.prev&&(n.prev.next=n.next),n.next&&(n.next.prev=n.prev)),e?(n.prev=t[r-1]||null,n.next=t[r+1]||null):n.prev=n.next=null,e&&"root"===e.type?(n.root=e,n.parent=null):(n.root=null,n.parent=e);}return e};}).call(e,r(12).Buffer);},function(t,e,r){(function(t){function n(){return o.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function i(t,e){if(n()<e)throw new RangeError("Invalid typed array length");return o.TYPED_ARRAY_SUPPORT?(t=new Uint8Array(e),t.__proto__=o.prototype):(null===t&&(t=new o(e)),t.length=e),t}function o(t,e,r){if(!(o.TYPED_ARRAY_SUPPORT||this instanceof o))return new o(t,e,r);if("number"==typeof t){if("string"==typeof e)throw new Error("If encoding is specified then the first argument must be a string");return c(this,t)}return a(this,t,e,r)}function a(t,e,r,n){if("number"==typeof e)throw new TypeError('"value" argument must not be a number');return "undefined"!=typeof ArrayBuffer&&e instanceof ArrayBuffer?h(t,e,r,n):"string"==typeof e?l(t,e,r):p(t,e)}function s(t){if("number"!=typeof t)throw new TypeError('"size" argument must be a number');if(t<0)throw new RangeError('"size" argument must not be negative')}function u(t,e,r,n){return s(e),e<=0?i(t,e):void 0!==r?"string"==typeof n?i(t,e).fill(r,n):i(t,e).fill(r):i(t,e)}function c(t,e){if(s(e),t=i(t,e<0?0:0|d(e)),!o.TYPED_ARRAY_SUPPORT)for(var r=0;r<e;++r)t[r]=0;return t}function l(t,e,r){if("string"==typeof r&&""!==r||(r="utf8"),!o.isEncoding(r))throw new TypeError('"encoding" must be a valid string encoding');var n=0|v(e,r);t=i(t,n);var a=t.write(e,r);return a!==n&&(t=t.slice(0,a)),t}function f(t,e){var r=e.length<0?0:0|d(e.length);t=i(t,r);for(var n=0;n<r;n+=1)t[n]=255&e[n];return t}function h(t,e,r,n){if(e.byteLength,r<0||e.byteLength<r)throw new RangeError("'offset' is out of bounds");if(e.byteLength<r+(n||0))throw new RangeError("'length' is out of bounds");return e=void 0===r&&void 0===n?new Uint8Array(e):void 0===n?new Uint8Array(e,r):new Uint8Array(e,r,n),o.TYPED_ARRAY_SUPPORT?(t=e,t.__proto__=o.prototype):t=f(t,e),t}function p(t,e){if(o.isBuffer(e)){var r=0|d(e.length);return t=i(t,r),0===t.length?t:(e.copy(t,0,0,r),t)}if(e){if("undefined"!=typeof ArrayBuffer&&e.buffer instanceof ArrayBuffer||"length"in e)return "number"!=typeof e.length||J(e.length)?i(t,0):f(t,e);if("Buffer"===e.type&&Z(e.data))return f(t,e.data)}throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}function d(t){if(t>=n())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+n().toString(16)+" bytes");return 0|t}function g(t){return +t!=t&&(t=0),o.alloc(+t)}function v(t,e){if(o.isBuffer(t))return t.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(t)||t instanceof ArrayBuffer))return t.byteLength;"string"!=typeof t&&(t=""+t);var r=t.length;if(0===r)return 0;for(var n=!1;;)switch(e){case"ascii":case"latin1":case"binary":return r;case"utf8":case"utf-8":case void 0:return $(t).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*r;case"hex":return r>>>1;case"base64":return G(t).length;default:if(n)return $(t).length;e=(""+e).toLowerCase(),n=!0;}}function b(t,e,r){var n=!1;if((void 0===e||e<0)&&(e=0),e>this.length)return "";if((void 0===r||r>this.length)&&(r=this.length),r<=0)return "";if(r>>>=0,e>>>=0,r<=e)return "";for(t||(t="utf8");;)switch(t){case"hex":return B(this,e,r);case"utf8":case"utf-8":return T(this,e,r);case"ascii":return L(this,e,r);case"latin1":case"binary":return C(this,e,r);case"base64":return A(this,e,r);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return D(this,e,r);default:if(n)throw new TypeError("Unknown encoding: "+t);t=(t+"").toLowerCase(),n=!0;}}function y(t,e,r){var n=t[e];t[e]=t[r],t[r]=n;}function m(t,e,r,n,i){if(0===t.length)return -1;if("string"==typeof r?(n=r,r=0):r>2147483647?r=2147483647:r<-2147483648&&(r=-2147483648),r=+r,isNaN(r)&&(r=i?0:t.length-1),r<0&&(r=t.length+r),r>=t.length){if(i)return -1;r=t.length-1;}else if(r<0){if(!i)return -1;r=0;}if("string"==typeof e&&(e=o.from(e,n)),o.isBuffer(e))return 0===e.length?-1:_(t,e,r,n,i);if("number"==typeof e)return e&=255,o.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(t,e,r):Uint8Array.prototype.lastIndexOf.call(t,e,r):_(t,[e],r,n,i);throw new TypeError("val must be string, number or Buffer")}function _(t,e,r,n,i){function o(t,e){return 1===a?t[e]:t.readUInt16BE(e*a)}var a=1,s=t.length,u=e.length;if(void 0!==n&&("ucs2"===(n=String(n).toLowerCase())||"ucs-2"===n||"utf16le"===n||"utf-16le"===n)){if(t.length<2||e.length<2)return -1;a=2,s/=2,u/=2,r/=2;}var c;if(i){var l=-1;for(c=r;c<s;c++)if(o(t,c)===o(e,-1===l?0:c-l)){if(-1===l&&(l=c),c-l+1===u)return l*a}else -1!==l&&(c-=c-l),l=-1;}else for(r+u>s&&(r=s-u),c=r;c>=0;c--){for(var f=!0,h=0;h<u;h++)if(o(t,c+h)!==o(e,h)){f=!1;break}if(f)return c}return -1}function w(t,e,r,n){r=Number(r)||0;var i=t.length-r;n?(n=Number(n))>i&&(n=i):n=i;var o=e.length;if(o%2!=0)throw new TypeError("Invalid hex string");n>o/2&&(n=o/2);for(var a=0;a<n;++a){var s=parseInt(e.substr(2*a,2),16);if(isNaN(s))return a;t[r+a]=s;}return a}function x(t,e,r,n){return Y($(e,t.length-r),t,r,n)}function S(t,e,r,n){return Y(H(e),t,r,n)}function j(t,e,r,n){return S(t,e,r,n)}function k(t,e,r,n){return Y(G(e),t,r,n)}function E(t,e,r,n){return Y(W(e,t.length-r),t,r,n)}function A(t,e,r){return 0===e&&r===t.length?Q.fromByteArray(t):Q.fromByteArray(t.slice(e,r))}function T(t,e,r){r=Math.min(t.length,r);for(var n=[],i=e;i<r;){var o=t[i],a=null,s=o>239?4:o>223?3:o>191?2:1;if(i+s<=r){var u,c,l,f;switch(s){case 1:o<128&&(a=o);break;case 2:u=t[i+1],128==(192&u)&&(f=(31&o)<<6|63&u)>127&&(a=f);break;case 3:u=t[i+1],c=t[i+2],128==(192&u)&&128==(192&c)&&(f=(15&o)<<12|(63&u)<<6|63&c)>2047&&(f<55296||f>57343)&&(a=f);break;case 4:u=t[i+1],c=t[i+2],l=t[i+3],128==(192&u)&&128==(192&c)&&128==(192&l)&&(f=(15&o)<<18|(63&u)<<12|(63&c)<<6|63&l)>65535&&f<1114112&&(a=f);}}null===a?(a=65533,s=1):a>65535&&(a-=65536,n.push(a>>>10&1023|55296),a=56320|1023&a),n.push(a),i+=s;}return O(n)}function O(t){var e=t.length;if(e<=K)return String.fromCharCode.apply(String,t);for(var r="",n=0;n<e;)r+=String.fromCharCode.apply(String,t.slice(n,n+=K));return r}function L(t,e,r){var n="";r=Math.min(t.length,r);for(var i=e;i<r;++i)n+=String.fromCharCode(127&t[i]);return n}function C(t,e,r){var n="";r=Math.min(t.length,r);for(var i=e;i<r;++i)n+=String.fromCharCode(t[i]);return n}function B(t,e,r){var n=t.length;(!e||e<0)&&(e=0),(!r||r<0||r>n)&&(r=n);for(var i="",o=e;o<r;++o)i+=V(t[o]);return i}function D(t,e,r){for(var n=t.slice(e,r),i="",o=0;o<n.length;o+=2)i+=String.fromCharCode(n[o]+256*n[o+1]);return i}function q(t,e,r){if(t%1!=0||t<0)throw new RangeError("offset is not uint");if(t+e>r)throw new RangeError("Trying to access beyond buffer length")}function P(t,e,r,n,i,a){if(!o.isBuffer(t))throw new TypeError('"buffer" argument must be a Buffer instance');if(e>i||e<a)throw new RangeError('"value" argument is out of bounds');if(r+n>t.length)throw new RangeError("Index out of range")}function M(t,e,r,n){e<0&&(e=65535+e+1);for(var i=0,o=Math.min(t.length-r,2);i<o;++i)t[r+i]=(e&255<<8*(n?i:1-i))>>>8*(n?i:1-i);}function R(t,e,r,n){e<0&&(e=4294967295+e+1);for(var i=0,o=Math.min(t.length-r,4);i<o;++i)t[r+i]=e>>>8*(n?i:3-i)&255;}function I(t,e,r,n,i,o){if(r+n>t.length)throw new RangeError("Index out of range");if(r<0)throw new RangeError("Index out of range")}function N(t,e,r,n,i){return i||I(t,e,r,4),X.write(t,e,r,n,23,4),r+4}function U(t,e,r,n,i){return i||I(t,e,r,8),X.write(t,e,r,n,52,8),r+8}function F(t){if(t=z(t).replace(tt,""),t.length<2)return "";for(;t.length%4!=0;)t+="=";return t}function z(t){return t.trim?t.trim():t.replace(/^\s+|\s+$/g,"")}function V(t){return t<16?"0"+t.toString(16):t.toString(16)}function $(t,e){e=e||1/0;for(var r,n=t.length,i=null,o=[],a=0;a<n;++a){if((r=t.charCodeAt(a))>55295&&r<57344){if(!i){if(r>56319){(e-=3)>-1&&o.push(239,191,189);continue}if(a+1===n){(e-=3)>-1&&o.push(239,191,189);continue}i=r;continue}if(r<56320){(e-=3)>-1&&o.push(239,191,189),i=r;continue}r=65536+(i-55296<<10|r-56320);}else i&&(e-=3)>-1&&o.push(239,191,189);if(i=null,r<128){if((e-=1)<0)break;o.push(r);}else if(r<2048){if((e-=2)<0)break;o.push(r>>6|192,63&r|128);}else if(r<65536){if((e-=3)<0)break;o.push(r>>12|224,r>>6&63|128,63&r|128);}else {if(!(r<1114112))throw new Error("Invalid code point");if((e-=4)<0)break;o.push(r>>18|240,r>>12&63|128,r>>6&63|128,63&r|128);}}return o}function H(t){for(var e=[],r=0;r<t.length;++r)e.push(255&t.charCodeAt(r));return e}function W(t,e){for(var r,n,i,o=[],a=0;a<t.length&&!((e-=2)<0);++a)r=t.charCodeAt(a),n=r>>8,i=r%256,o.push(i),o.push(n);return o}function G(t){return Q.toByteArray(F(t))}function Y(t,e,r,n){for(var i=0;i<n&&!(i+r>=e.length||i>=t.length);++i)e[i+r]=t[i];return i}function J(t){return t!==t}/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <feross@feross.org> <http://feross.org>
 * @license  MIT
 */
var Q=r(55),X=r(56),Z=r(29);e.Buffer=o,e.SlowBuffer=g,e.INSPECT_MAX_BYTES=50,o.TYPED_ARRAY_SUPPORT=void 0!==t.TYPED_ARRAY_SUPPORT?t.TYPED_ARRAY_SUPPORT:function(){try{var t=new Uint8Array(1);return t.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===t.foo()&&"function"==typeof t.subarray&&0===t.subarray(1,1).byteLength}catch(t){return !1}}(),e.kMaxLength=n(),o.poolSize=8192,o._augment=function(t){return t.__proto__=o.prototype,t},o.from=function(t,e,r){return a(null,t,e,r)},o.TYPED_ARRAY_SUPPORT&&(o.prototype.__proto__=Uint8Array.prototype,o.__proto__=Uint8Array,"undefined"!=typeof Symbol&&Symbol.species&&o[Symbol.species]===o&&Object.defineProperty(o,Symbol.species,{value:null,configurable:!0})),o.alloc=function(t,e,r){return u(null,t,e,r)},o.allocUnsafe=function(t){return c(null,t)},o.allocUnsafeSlow=function(t){return c(null,t)},o.isBuffer=function(t){return !(null==t||!t._isBuffer)},o.compare=function(t,e){if(!o.isBuffer(t)||!o.isBuffer(e))throw new TypeError("Arguments must be Buffers");if(t===e)return 0;for(var r=t.length,n=e.length,i=0,a=Math.min(r,n);i<a;++i)if(t[i]!==e[i]){r=t[i],n=e[i];break}return r<n?-1:n<r?1:0},o.isEncoding=function(t){switch(String(t).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return !0;default:return !1}},o.concat=function(t,e){if(!Z(t))throw new TypeError('"list" argument must be an Array of Buffers');if(0===t.length)return o.alloc(0);var r;if(void 0===e)for(e=0,r=0;r<t.length;++r)e+=t[r].length;var n=o.allocUnsafe(e),i=0;for(r=0;r<t.length;++r){var a=t[r];if(!o.isBuffer(a))throw new TypeError('"list" argument must be an Array of Buffers');a.copy(n,i),i+=a.length;}return n},o.byteLength=v,o.prototype._isBuffer=!0,o.prototype.swap16=function(){var t=this.length;if(t%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var e=0;e<t;e+=2)y(this,e,e+1);return this},o.prototype.swap32=function(){var t=this.length;if(t%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var e=0;e<t;e+=4)y(this,e,e+3),y(this,e+1,e+2);return this},o.prototype.swap64=function(){var t=this.length;if(t%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var e=0;e<t;e+=8)y(this,e,e+7),y(this,e+1,e+6),y(this,e+2,e+5),y(this,e+3,e+4);return this},o.prototype.toString=function(){var t=0|this.length;return 0===t?"":0===arguments.length?T(this,0,t):b.apply(this,arguments)},o.prototype.equals=function(t){if(!o.isBuffer(t))throw new TypeError("Argument must be a Buffer");return this===t||0===o.compare(this,t)},o.prototype.inspect=function(){var t="",r=e.INSPECT_MAX_BYTES;return this.length>0&&(t=this.toString("hex",0,r).match(/.{2}/g).join(" "),this.length>r&&(t+=" ... ")),"<Buffer "+t+">"},o.prototype.compare=function(t,e,r,n,i){if(!o.isBuffer(t))throw new TypeError("Argument must be a Buffer");if(void 0===e&&(e=0),void 0===r&&(r=t?t.length:0),void 0===n&&(n=0),void 0===i&&(i=this.length),e<0||r>t.length||n<0||i>this.length)throw new RangeError("out of range index");if(n>=i&&e>=r)return 0;if(n>=i)return -1;if(e>=r)return 1;if(e>>>=0,r>>>=0,n>>>=0,i>>>=0,this===t)return 0;for(var a=i-n,s=r-e,u=Math.min(a,s),c=this.slice(n,i),l=t.slice(e,r),f=0;f<u;++f)if(c[f]!==l[f]){a=c[f],s=l[f];break}return a<s?-1:s<a?1:0},o.prototype.includes=function(t,e,r){return -1!==this.indexOf(t,e,r)},o.prototype.indexOf=function(t,e,r){return m(this,t,e,r,!0)},o.prototype.lastIndexOf=function(t,e,r){return m(this,t,e,r,!1)},o.prototype.write=function(t,e,r,n){if(void 0===e)n="utf8",r=this.length,e=0;else if(void 0===r&&"string"==typeof e)n=e,r=this.length,e=0;else {if(!isFinite(e))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");e|=0,isFinite(r)?(r|=0,void 0===n&&(n="utf8")):(n=r,r=void 0);}var i=this.length-e;if((void 0===r||r>i)&&(r=i),t.length>0&&(r<0||e<0)||e>this.length)throw new RangeError("Attempt to write outside buffer bounds");n||(n="utf8");for(var o=!1;;)switch(n){case"hex":return w(this,t,e,r);case"utf8":case"utf-8":return x(this,t,e,r);case"ascii":return S(this,t,e,r);case"latin1":case"binary":return j(this,t,e,r);case"base64":return k(this,t,e,r);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return E(this,t,e,r);default:if(o)throw new TypeError("Unknown encoding: "+n);n=(""+n).toLowerCase(),o=!0;}},o.prototype.toJSON=function(){return {type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var K=4096;o.prototype.slice=function(t,e){var r=this.length;t=~~t,e=void 0===e?r:~~e,t<0?(t+=r)<0&&(t=0):t>r&&(t=r),e<0?(e+=r)<0&&(e=0):e>r&&(e=r),e<t&&(e=t);var n;if(o.TYPED_ARRAY_SUPPORT)n=this.subarray(t,e),n.__proto__=o.prototype;else {var i=e-t;n=new o(i,void 0);for(var a=0;a<i;++a)n[a]=this[a+t];}return n},o.prototype.readUIntLE=function(t,e,r){t|=0,e|=0,r||q(t,e,this.length);for(var n=this[t],i=1,o=0;++o<e&&(i*=256);)n+=this[t+o]*i;return n},o.prototype.readUIntBE=function(t,e,r){t|=0,e|=0,r||q(t,e,this.length);for(var n=this[t+--e],i=1;e>0&&(i*=256);)n+=this[t+--e]*i;return n},o.prototype.readUInt8=function(t,e){return e||q(t,1,this.length),this[t]},o.prototype.readUInt16LE=function(t,e){return e||q(t,2,this.length),this[t]|this[t+1]<<8},o.prototype.readUInt16BE=function(t,e){return e||q(t,2,this.length),this[t]<<8|this[t+1]},o.prototype.readUInt32LE=function(t,e){return e||q(t,4,this.length),(this[t]|this[t+1]<<8|this[t+2]<<16)+16777216*this[t+3]},o.prototype.readUInt32BE=function(t,e){return e||q(t,4,this.length),16777216*this[t]+(this[t+1]<<16|this[t+2]<<8|this[t+3])},o.prototype.readIntLE=function(t,e,r){t|=0,e|=0,r||q(t,e,this.length);for(var n=this[t],i=1,o=0;++o<e&&(i*=256);)n+=this[t+o]*i;return i*=128,n>=i&&(n-=Math.pow(2,8*e)),n},o.prototype.readIntBE=function(t,e,r){t|=0,e|=0,r||q(t,e,this.length);for(var n=e,i=1,o=this[t+--n];n>0&&(i*=256);)o+=this[t+--n]*i;return i*=128,o>=i&&(o-=Math.pow(2,8*e)),o},o.prototype.readInt8=function(t,e){return e||q(t,1,this.length),128&this[t]?-1*(255-this[t]+1):this[t]},o.prototype.readInt16LE=function(t,e){e||q(t,2,this.length);var r=this[t]|this[t+1]<<8;return 32768&r?4294901760|r:r},o.prototype.readInt16BE=function(t,e){e||q(t,2,this.length);var r=this[t+1]|this[t]<<8;return 32768&r?4294901760|r:r},o.prototype.readInt32LE=function(t,e){return e||q(t,4,this.length),this[t]|this[t+1]<<8|this[t+2]<<16|this[t+3]<<24},o.prototype.readInt32BE=function(t,e){return e||q(t,4,this.length),this[t]<<24|this[t+1]<<16|this[t+2]<<8|this[t+3]},o.prototype.readFloatLE=function(t,e){return e||q(t,4,this.length),X.read(this,t,!0,23,4)},o.prototype.readFloatBE=function(t,e){return e||q(t,4,this.length),X.read(this,t,!1,23,4)},o.prototype.readDoubleLE=function(t,e){return e||q(t,8,this.length),X.read(this,t,!0,52,8)},o.prototype.readDoubleBE=function(t,e){return e||q(t,8,this.length),X.read(this,t,!1,52,8)},o.prototype.writeUIntLE=function(t,e,r,n){if(t=+t,e|=0,r|=0,!n){P(this,t,e,r,Math.pow(2,8*r)-1,0);}var i=1,o=0;for(this[e]=255&t;++o<r&&(i*=256);)this[e+o]=t/i&255;return e+r},o.prototype.writeUIntBE=function(t,e,r,n){if(t=+t,e|=0,r|=0,!n){P(this,t,e,r,Math.pow(2,8*r)-1,0);}var i=r-1,o=1;for(this[e+i]=255&t;--i>=0&&(o*=256);)this[e+i]=t/o&255;return e+r},o.prototype.writeUInt8=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,1,255,0),o.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),this[e]=255&t,e+1},o.prototype.writeUInt16LE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,2,65535,0),o.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):M(this,t,e,!0),e+2},o.prototype.writeUInt16BE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,2,65535,0),o.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):M(this,t,e,!1),e+2},o.prototype.writeUInt32LE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,4,4294967295,0),o.TYPED_ARRAY_SUPPORT?(this[e+3]=t>>>24,this[e+2]=t>>>16,this[e+1]=t>>>8,this[e]=255&t):R(this,t,e,!0),e+4},o.prototype.writeUInt32BE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,4,4294967295,0),o.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):R(this,t,e,!1),e+4},o.prototype.writeIntLE=function(t,e,r,n){if(t=+t,e|=0,!n){var i=Math.pow(2,8*r-1);P(this,t,e,r,i-1,-i);}var o=0,a=1,s=0;for(this[e]=255&t;++o<r&&(a*=256);)t<0&&0===s&&0!==this[e+o-1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+r},o.prototype.writeIntBE=function(t,e,r,n){if(t=+t,e|=0,!n){var i=Math.pow(2,8*r-1);P(this,t,e,r,i-1,-i);}var o=r-1,a=1,s=0;for(this[e+o]=255&t;--o>=0&&(a*=256);)t<0&&0===s&&0!==this[e+o+1]&&(s=1),this[e+o]=(t/a>>0)-s&255;return e+r},o.prototype.writeInt8=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,1,127,-128),o.TYPED_ARRAY_SUPPORT||(t=Math.floor(t)),t<0&&(t=255+t+1),this[e]=255&t,e+1},o.prototype.writeInt16LE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,2,32767,-32768),o.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8):M(this,t,e,!0),e+2},o.prototype.writeInt16BE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,2,32767,-32768),o.TYPED_ARRAY_SUPPORT?(this[e]=t>>>8,this[e+1]=255&t):M(this,t,e,!1),e+2},o.prototype.writeInt32LE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,4,2147483647,-2147483648),o.TYPED_ARRAY_SUPPORT?(this[e]=255&t,this[e+1]=t>>>8,this[e+2]=t>>>16,this[e+3]=t>>>24):R(this,t,e,!0),e+4},o.prototype.writeInt32BE=function(t,e,r){return t=+t,e|=0,r||P(this,t,e,4,2147483647,-2147483648),t<0&&(t=4294967295+t+1),o.TYPED_ARRAY_SUPPORT?(this[e]=t>>>24,this[e+1]=t>>>16,this[e+2]=t>>>8,this[e+3]=255&t):R(this,t,e,!1),e+4},o.prototype.writeFloatLE=function(t,e,r){return N(this,t,e,!0,r)},o.prototype.writeFloatBE=function(t,e,r){return N(this,t,e,!1,r)},o.prototype.writeDoubleLE=function(t,e,r){return U(this,t,e,!0,r)},o.prototype.writeDoubleBE=function(t,e,r){return U(this,t,e,!1,r)},o.prototype.copy=function(t,e,r,n){if(r||(r=0),n||0===n||(n=this.length),e>=t.length&&(e=t.length),e||(e=0),n>0&&n<r&&(n=r),n===r)return 0;if(0===t.length||0===this.length)return 0;if(e<0)throw new RangeError("targetStart out of bounds");if(r<0||r>=this.length)throw new RangeError("sourceStart out of bounds");if(n<0)throw new RangeError("sourceEnd out of bounds");n>this.length&&(n=this.length),t.length-e<n-r&&(n=t.length-e+r);var i,a=n-r;if(this===t&&r<e&&e<n)for(i=a-1;i>=0;--i)t[i+e]=this[i+r];else if(a<1e3||!o.TYPED_ARRAY_SUPPORT)for(i=0;i<a;++i)t[i+e]=this[i+r];else Uint8Array.prototype.set.call(t,this.subarray(r,r+a),e);return a},o.prototype.fill=function(t,e,r,n){if("string"==typeof t){if("string"==typeof e?(n=e,e=0,r=this.length):"string"==typeof r&&(n=r,r=this.length),1===t.length){var i=t.charCodeAt(0);i<256&&(t=i);}if(void 0!==n&&"string"!=typeof n)throw new TypeError("encoding must be a string");if("string"==typeof n&&!o.isEncoding(n))throw new TypeError("Unknown encoding: "+n)}else "number"==typeof t&&(t&=255);if(e<0||this.length<e||this.length<r)throw new RangeError("Out of range index");if(r<=e)return this;e>>>=0,r=void 0===r?this.length:r>>>0,t||(t=0);var a;if("number"==typeof t)for(a=e;a<r;++a)this[a]=t;else {var s=o.isBuffer(t)?t:$(new o(t,n).toString()),u=s.length;for(a=0;a<r-e;++a)this[a+e]=s[a%u];}return this};var tt=/[^+\/0-9A-Za-z-_]/g;}).call(e,r(0));},function(t,e){function r(){this._events=this._events||{},this._maxListeners=this._maxListeners||void 0;}function n(t){return "function"==typeof t}function i(t){return "number"==typeof t}function o(t){return "object"==typeof t&&null!==t}function a(t){return void 0===t}t.exports=r,r.EventEmitter=r,r.prototype._events=void 0,r.prototype._maxListeners=void 0,r.defaultMaxListeners=10,r.prototype.setMaxListeners=function(t){if(!i(t)||t<0||isNaN(t))throw TypeError("n must be a positive number");return this._maxListeners=t,this},r.prototype.emit=function(t){var e,r,i,s,u,c;if(this._events||(this._events={}),"error"===t&&(!this._events.error||o(this._events.error)&&!this._events.error.length)){if((e=arguments[1])instanceof Error)throw e;var l=new Error('Uncaught, unspecified "error" event. ('+e+")");throw l.context=e,l}if(r=this._events[t],a(r))return !1;if(n(r))switch(arguments.length){case 1:r.call(this);break;case 2:r.call(this,arguments[1]);break;case 3:r.call(this,arguments[1],arguments[2]);break;default:s=Array.prototype.slice.call(arguments,1),r.apply(this,s);}else if(o(r))for(s=Array.prototype.slice.call(arguments,1),c=r.slice(),i=c.length,u=0;u<i;u++)c[u].apply(this,s);return !0},r.prototype.addListener=function(t,e){var i;if(!n(e))throw TypeError("listener must be a function");return this._events||(this._events={}),this._events.newListener&&this.emit("newListener",t,n(e.listener)?e.listener:e),this._events[t]?o(this._events[t])?this._events[t].push(e):this._events[t]=[this._events[t],e]:this._events[t]=e,o(this._events[t])&&!this._events[t].warned&&(i=a(this._maxListeners)?r.defaultMaxListeners:this._maxListeners)&&i>0&&this._events[t].length>i&&(this._events[t].warned=!0,console.error("(node) warning: possible EventEmitter memory leak detected. %d listeners added. Use emitter.setMaxListeners() to increase limit.",this._events[t].length),"function"==typeof console.trace&&console.trace()),this},r.prototype.on=r.prototype.addListener,r.prototype.once=function(t,e){function r(){this.removeListener(t,r),i||(i=!0,e.apply(this,arguments));}if(!n(e))throw TypeError("listener must be a function");var i=!1;return r.listener=e,this.on(t,r),this},r.prototype.removeListener=function(t,e){var r,i,a,s;if(!n(e))throw TypeError("listener must be a function");if(!this._events||!this._events[t])return this;if(r=this._events[t],a=r.length,i=-1,r===e||n(r.listener)&&r.listener===e)delete this._events[t],this._events.removeListener&&this.emit("removeListener",t,e);else if(o(r)){for(s=a;s-- >0;)if(r[s]===e||r[s].listener&&r[s].listener===e){i=s;break}if(i<0)return this;1===r.length?(r.length=0,delete this._events[t]):r.splice(i,1),this._events.removeListener&&this.emit("removeListener",t,e);}return this},r.prototype.removeAllListeners=function(t){var e,r;if(!this._events)return this;if(!this._events.removeListener)return 0===arguments.length?this._events={}:this._events[t]&&delete this._events[t],this;if(0===arguments.length){for(e in this._events)"removeListener"!==e&&this.removeAllListeners(e);return this.removeAllListeners("removeListener"),this._events={},this}if(r=this._events[t],n(r))this.removeListener(t,r);else if(r)for(;r.length;)this.removeListener(t,r[r.length-1]);return delete this._events[t],this},r.prototype.listeners=function(t){return this._events&&this._events[t]?n(this._events[t])?[this._events[t]]:this._events[t].slice():[]},r.prototype.listenerCount=function(t){if(this._events){var e=this._events[t];if(n(e))return 1;if(e)return e.length}return 0},r.listenerCount=function(t,e){return t.listenerCount(e)};},function(t,e,r){(function(e){function r(t,r,n,i){if("function"!=typeof t)throw new TypeError('"callback" argument must be a function');var o,a,s=arguments.length;switch(s){case 0:case 1:return e.nextTick(t);case 2:return e.nextTick(function(){t.call(null,r);});case 3:return e.nextTick(function(){t.call(null,r,n);});case 4:return e.nextTick(function(){t.call(null,r,n,i);});default:for(o=new Array(s-1),a=0;a<o.length;)o[a++]=arguments[a];return e.nextTick(function(){t.apply(null,o);})}}!e.version||0===e.version.indexOf("v0.")||0===e.version.indexOf("v1.")&&0!==e.version.indexOf("v1.8.")?t.exports={nextTick:r}:t.exports=e;}).call(e,r(7));},function(t,e,r){function n(t,e){for(var r in t)e[r]=t[r];}function i(t,e,r){return a(t,e,r)}var o=r(12),a=o.Buffer;a.from&&a.alloc&&a.allocUnsafe&&a.allocUnsafeSlow?t.exports=o:(n(o,e),e.Buffer=i),n(a,i),i.from=function(t,e,r){if("number"==typeof t)throw new TypeError("Argument must not be a number");return a(t,e,r)},i.alloc=function(t,e,r){if("number"!=typeof t)throw new TypeError("Argument must be a number");var n=a(t);return void 0!==e?"string"==typeof r?n.fill(e,r):n.fill(e):n.fill(0),n},i.allocUnsafe=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return a(t)},i.allocUnsafeSlow=function(t){if("number"!=typeof t)throw new TypeError("Argument must be a number");return o.SlowBuffer(t)};},function(t,e){function r(t,e){for(var r=-1,n=t?t.length:0;++r<n&&!1!==e(t[r],r,t););return t}function n(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function i(t,e){var r=C(t)||l(t)?n(t.length,String):[],i=r.length,o=!!i;for(var a in t)!e&&!k.call(t,a)||o&&("length"==a||s(a,i))||r.push(a);return r}function o(t,e){return t&&L(t,e,b)}function a(t){if(!u(t))return T(t);var e=[];for(var r in Object(t))k.call(t,r)&&"constructor"!=r&&e.push(r);return e}function s(t,e){return !!(e=null==e?m:e)&&("number"==typeof t||S.test(t))&&t>-1&&t%1==0&&t<e}function u(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||j)}function c(t,e){return (C(t)?r:O)(t,"function"==typeof e?e:y)}function l(t){return h(t)&&k.call(t,"callee")&&(!A.call(t,"callee")||E.call(t)==_)}function f(t){return null!=t&&d(t.length)&&!p(t)}function h(t){return v(t)&&f(t)}function p(t){var e=g(t)?E.call(t):"";return e==w||e==x}function d(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=m}function g(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function v(t){return !!t&&"object"==typeof t}function b(t){return f(t)?i(t):a(t)}function y(t){return t}var m=9007199254740991,_="[object Arguments]",w="[object Function]",x="[object GeneratorFunction]",S=/^(?:0|[1-9]\d*)$/,j=Object.prototype,k=j.hasOwnProperty,E=j.toString,A=j.propertyIsEnumerable,T=function(t,e){return function(r){return t(e(r))}}(Object.keys,Object),O=function(t,e){return function(r,n){if(null==r)return r;if(!f(r))return t(r,n);for(var i=r.length,o=e?i:-1,a=Object(r);(e?o--:++o<i)&&!1!==n(a[o],o,a););return r}}(o),L=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),C=Array.isArray;t.exports=c;},function(t,e,r){var n=r(11),i=r(9).isHtml,o={extend:r(40),bind:r(24),forEach:r(16),defaults:r(41)},a=[r(87),r(97),r(101),r(103),r(105)],s=t.exports=function(t,e,r,a){return this instanceof s?(this.options=o.defaults(a||{},this.options),t?(r&&("string"==typeof r&&(r=n(r,this.options)),this._root=s.call(this,r)),t.cheerio?t:(u(t)&&(t=[t]),Array.isArray(t)?(o.forEach(t,o.bind(function(t,e){this[e]=t;},this)),this.length=t.length,this):"string"==typeof t&&i(t)?s.call(this,n(t,this.options).children):(e?"string"==typeof e?i(e)?(e=n(e,this.options),e=s.call(this,e)):(t=[e,t].join(" "),e=this._root):e.cheerio||(e=s.call(this,e)):e=this._root,e?e.find(t):this))):this):new s(t,e,r,a)};o.extend(s,r(25)),s.prototype.cheerio="[cheerio object]",s.prototype.options={withDomLvl1:!0,normalizeWhitespace:!1,xmlMode:!1,decodeEntities:!0},s.prototype.length=0,s.prototype.splice=Array.prototype.splice,s.prototype._make=function(t,e){var r=new this.constructor(t,e,this._root,this.options);return r.prevObject=this,r},s.prototype.toArray=function(){return this.get()},a.forEach(function(t){o.extend(s.prototype,t);});var u=function(t){return t.name||"text"===t.type||"comment"===t.type};},function(t,e){t.exports={Aacute:"Á",aacute:"á",Abreve:"Ă",abreve:"ă",ac:"∾",acd:"∿",acE:"∾̳",Acirc:"Â",acirc:"â",acute:"´",Acy:"А",acy:"а",AElig:"Æ",aelig:"æ",af:"⁡",Afr:"𝔄",afr:"𝔞",Agrave:"À",agrave:"à",alefsym:"ℵ",aleph:"ℵ",Alpha:"Α",alpha:"α",Amacr:"Ā",amacr:"ā",amalg:"⨿",amp:"&",AMP:"&",andand:"⩕",And:"⩓",and:"∧",andd:"⩜",andslope:"⩘",andv:"⩚",ang:"∠",ange:"⦤",angle:"∠",angmsdaa:"⦨",angmsdab:"⦩",angmsdac:"⦪",angmsdad:"⦫",angmsdae:"⦬",angmsdaf:"⦭",angmsdag:"⦮",angmsdah:"⦯",angmsd:"∡",angrt:"∟",angrtvb:"⊾",angrtvbd:"⦝",angsph:"∢",angst:"Å",angzarr:"⍼",Aogon:"Ą",aogon:"ą",Aopf:"𝔸",aopf:"𝕒",apacir:"⩯",ap:"≈",apE:"⩰",ape:"≊",apid:"≋",apos:"'",ApplyFunction:"⁡",approx:"≈",approxeq:"≊",Aring:"Å",aring:"å",Ascr:"𝒜",ascr:"𝒶",Assign:"≔",ast:"*",asymp:"≈",asympeq:"≍",Atilde:"Ã",atilde:"ã",Auml:"Ä",auml:"ä",awconint:"∳",awint:"⨑",backcong:"≌",backepsilon:"϶",backprime:"‵",backsim:"∽",backsimeq:"⋍",Backslash:"∖",Barv:"⫧",barvee:"⊽",barwed:"⌅",Barwed:"⌆",barwedge:"⌅",bbrk:"⎵",bbrktbrk:"⎶",bcong:"≌",Bcy:"Б",bcy:"б",bdquo:"„",becaus:"∵",because:"∵",Because:"∵",bemptyv:"⦰",bepsi:"϶",bernou:"ℬ",Bernoullis:"ℬ",Beta:"Β",beta:"β",beth:"ℶ",between:"≬",Bfr:"𝔅",bfr:"𝔟",bigcap:"⋂",bigcirc:"◯",bigcup:"⋃",bigodot:"⨀",bigoplus:"⨁",bigotimes:"⨂",bigsqcup:"⨆",bigstar:"★",bigtriangledown:"▽",bigtriangleup:"△",biguplus:"⨄",bigvee:"⋁",bigwedge:"⋀",bkarow:"⤍",blacklozenge:"⧫",blacksquare:"▪",blacktriangle:"▴",blacktriangledown:"▾",blacktriangleleft:"◂",blacktriangleright:"▸",blank:"␣",blk12:"▒",blk14:"░",blk34:"▓",block:"█",bne:"=⃥",bnequiv:"≡⃥",bNot:"⫭",bnot:"⌐",Bopf:"𝔹",bopf:"𝕓",bot:"⊥",bottom:"⊥",bowtie:"⋈",boxbox:"⧉",boxdl:"┐",boxdL:"╕",boxDl:"╖",boxDL:"╗",boxdr:"┌",boxdR:"╒",boxDr:"╓",boxDR:"╔",boxh:"─",boxH:"═",boxhd:"┬",boxHd:"╤",boxhD:"╥",boxHD:"╦",boxhu:"┴",boxHu:"╧",boxhU:"╨",boxHU:"╩",boxminus:"⊟",boxplus:"⊞",boxtimes:"⊠",boxul:"┘",boxuL:"╛",boxUl:"╜",boxUL:"╝",boxur:"└",boxuR:"╘",boxUr:"╙",boxUR:"╚",boxv:"│",boxV:"║",boxvh:"┼",boxvH:"╪",boxVh:"╫",boxVH:"╬",boxvl:"┤",boxvL:"╡",boxVl:"╢",boxVL:"╣",boxvr:"├",boxvR:"╞",boxVr:"╟",boxVR:"╠",bprime:"‵",breve:"˘",Breve:"˘",brvbar:"¦",bscr:"𝒷",Bscr:"ℬ",bsemi:"⁏",bsim:"∽",bsime:"⋍",bsolb:"⧅",bsol:"\\",bsolhsub:"⟈",bull:"•",bullet:"•",bump:"≎",bumpE:"⪮",bumpe:"≏",Bumpeq:"≎",bumpeq:"≏",Cacute:"Ć",cacute:"ć",capand:"⩄",capbrcup:"⩉",capcap:"⩋",cap:"∩",Cap:"⋒",capcup:"⩇",capdot:"⩀",CapitalDifferentialD:"ⅅ",caps:"∩︀",caret:"⁁",caron:"ˇ",Cayleys:"ℭ",ccaps:"⩍",Ccaron:"Č",ccaron:"č",Ccedil:"Ç",ccedil:"ç",Ccirc:"Ĉ",ccirc:"ĉ",Cconint:"∰",ccups:"⩌",ccupssm:"⩐",Cdot:"Ċ",cdot:"ċ",cedil:"¸",Cedilla:"¸",cemptyv:"⦲",cent:"¢",centerdot:"·",CenterDot:"·",cfr:"𝔠",Cfr:"ℭ",CHcy:"Ч",chcy:"ч",check:"✓",checkmark:"✓",Chi:"Χ",chi:"χ",circ:"ˆ",circeq:"≗",circlearrowleft:"↺",circlearrowright:"↻",circledast:"⊛",circledcirc:"⊚",circleddash:"⊝",CircleDot:"⊙",circledR:"®",circledS:"Ⓢ",CircleMinus:"⊖",CirclePlus:"⊕",CircleTimes:"⊗",cir:"○",cirE:"⧃",cire:"≗",cirfnint:"⨐",cirmid:"⫯",cirscir:"⧂",ClockwiseContourIntegral:"∲",CloseCurlyDoubleQuote:"”",CloseCurlyQuote:"’",clubs:"♣",clubsuit:"♣",colon:":",Colon:"∷",Colone:"⩴",colone:"≔",coloneq:"≔",comma:",",commat:"@",comp:"∁",compfn:"∘",complement:"∁",complexes:"ℂ",cong:"≅",congdot:"⩭",Congruent:"≡",conint:"∮",Conint:"∯",ContourIntegral:"∮",copf:"𝕔",Copf:"ℂ",coprod:"∐",Coproduct:"∐",copy:"©",COPY:"©",copysr:"℗",CounterClockwiseContourIntegral:"∳",crarr:"↵",cross:"✗",Cross:"⨯",Cscr:"𝒞",cscr:"𝒸",csub:"⫏",csube:"⫑",csup:"⫐",csupe:"⫒",ctdot:"⋯",cudarrl:"⤸",cudarrr:"⤵",cuepr:"⋞",cuesc:"⋟",cularr:"↶",cularrp:"⤽",cupbrcap:"⩈",cupcap:"⩆",CupCap:"≍",cup:"∪",Cup:"⋓",cupcup:"⩊",cupdot:"⊍",cupor:"⩅",cups:"∪︀",curarr:"↷",curarrm:"⤼",curlyeqprec:"⋞",curlyeqsucc:"⋟",curlyvee:"⋎",curlywedge:"⋏",curren:"¤",curvearrowleft:"↶",curvearrowright:"↷",cuvee:"⋎",cuwed:"⋏",cwconint:"∲",cwint:"∱",cylcty:"⌭",dagger:"†",Dagger:"‡",daleth:"ℸ",darr:"↓",Darr:"↡",dArr:"⇓",dash:"‐",Dashv:"⫤",dashv:"⊣",dbkarow:"⤏",dblac:"˝",Dcaron:"Ď",dcaron:"ď",Dcy:"Д",dcy:"д",ddagger:"‡",ddarr:"⇊",DD:"ⅅ",dd:"ⅆ",DDotrahd:"⤑",ddotseq:"⩷",deg:"°",Del:"∇",Delta:"Δ",delta:"δ",demptyv:"⦱",dfisht:"⥿",Dfr:"𝔇",dfr:"𝔡",dHar:"⥥",dharl:"⇃",dharr:"⇂",DiacriticalAcute:"´",DiacriticalDot:"˙",DiacriticalDoubleAcute:"˝",DiacriticalGrave:"`",DiacriticalTilde:"˜",diam:"⋄",diamond:"⋄",Diamond:"⋄",diamondsuit:"♦",diams:"♦",die:"¨",DifferentialD:"ⅆ",digamma:"ϝ",disin:"⋲",div:"÷",divide:"÷",divideontimes:"⋇",divonx:"⋇",DJcy:"Ђ",djcy:"ђ",dlcorn:"⌞",dlcrop:"⌍",dollar:"$",Dopf:"𝔻",dopf:"𝕕",Dot:"¨",dot:"˙",DotDot:"⃜",doteq:"≐",doteqdot:"≑",DotEqual:"≐",dotminus:"∸",dotplus:"∔",dotsquare:"⊡",doublebarwedge:"⌆",DoubleContourIntegral:"∯",DoubleDot:"¨",DoubleDownArrow:"⇓",DoubleLeftArrow:"⇐",DoubleLeftRightArrow:"⇔",DoubleLeftTee:"⫤",DoubleLongLeftArrow:"⟸",DoubleLongLeftRightArrow:"⟺",DoubleLongRightArrow:"⟹",DoubleRightArrow:"⇒",DoubleRightTee:"⊨",DoubleUpArrow:"⇑",DoubleUpDownArrow:"⇕",DoubleVerticalBar:"∥",DownArrowBar:"⤓",downarrow:"↓",DownArrow:"↓",Downarrow:"⇓",DownArrowUpArrow:"⇵",DownBreve:"̑",downdownarrows:"⇊",downharpoonleft:"⇃",downharpoonright:"⇂",DownLeftRightVector:"⥐",DownLeftTeeVector:"⥞",DownLeftVectorBar:"⥖",DownLeftVector:"↽",DownRightTeeVector:"⥟",DownRightVectorBar:"⥗",DownRightVector:"⇁",DownTeeArrow:"↧",DownTee:"⊤",drbkarow:"⤐",drcorn:"⌟",drcrop:"⌌",Dscr:"𝒟",dscr:"𝒹",DScy:"Ѕ",dscy:"ѕ",dsol:"⧶",Dstrok:"Đ",dstrok:"đ",dtdot:"⋱",dtri:"▿",dtrif:"▾",duarr:"⇵",duhar:"⥯",dwangle:"⦦",DZcy:"Џ",dzcy:"џ",dzigrarr:"⟿",Eacute:"É",eacute:"é",easter:"⩮",Ecaron:"Ě",ecaron:"ě",Ecirc:"Ê",ecirc:"ê",ecir:"≖",ecolon:"≕",Ecy:"Э",ecy:"э",eDDot:"⩷",Edot:"Ė",edot:"ė",eDot:"≑",ee:"ⅇ",efDot:"≒",Efr:"𝔈",efr:"𝔢",eg:"⪚",Egrave:"È",egrave:"è",egs:"⪖",egsdot:"⪘",el:"⪙",Element:"∈",elinters:"⏧",ell:"ℓ",els:"⪕",elsdot:"⪗",Emacr:"Ē",emacr:"ē",empty:"∅",emptyset:"∅",EmptySmallSquare:"◻",emptyv:"∅",EmptyVerySmallSquare:"▫",emsp13:" ",emsp14:" ",emsp:" ",ENG:"Ŋ",eng:"ŋ",ensp:" ",Eogon:"Ę",eogon:"ę",Eopf:"𝔼",eopf:"𝕖",epar:"⋕",eparsl:"⧣",eplus:"⩱",epsi:"ε",Epsilon:"Ε",epsilon:"ε",epsiv:"ϵ",eqcirc:"≖",eqcolon:"≕",eqsim:"≂",eqslantgtr:"⪖",eqslantless:"⪕",Equal:"⩵",equals:"=",EqualTilde:"≂",equest:"≟",Equilibrium:"⇌",equiv:"≡",equivDD:"⩸",eqvparsl:"⧥",erarr:"⥱",erDot:"≓",escr:"ℯ",Escr:"ℰ",esdot:"≐",Esim:"⩳",esim:"≂",Eta:"Η",eta:"η",ETH:"Ð",eth:"ð",Euml:"Ë",euml:"ë",euro:"€",excl:"!",exist:"∃",Exists:"∃",expectation:"ℰ",exponentiale:"ⅇ",ExponentialE:"ⅇ",fallingdotseq:"≒",Fcy:"Ф",fcy:"ф",female:"♀",ffilig:"ﬃ",fflig:"ﬀ",ffllig:"ﬄ",Ffr:"𝔉",ffr:"𝔣",filig:"ﬁ",FilledSmallSquare:"◼",FilledVerySmallSquare:"▪",fjlig:"fj",flat:"♭",fllig:"ﬂ",fltns:"▱",fnof:"ƒ",Fopf:"𝔽",fopf:"𝕗",forall:"∀",ForAll:"∀",fork:"⋔",forkv:"⫙",Fouriertrf:"ℱ",fpartint:"⨍",frac12:"½",frac13:"⅓",frac14:"¼",frac15:"⅕",frac16:"⅙",frac18:"⅛",frac23:"⅔",frac25:"⅖",frac34:"¾",frac35:"⅗",frac38:"⅜",frac45:"⅘",frac56:"⅚",frac58:"⅝",frac78:"⅞",frasl:"⁄",frown:"⌢",fscr:"𝒻",Fscr:"ℱ",gacute:"ǵ",Gamma:"Γ",gamma:"γ",Gammad:"Ϝ",gammad:"ϝ",gap:"⪆",Gbreve:"Ğ",gbreve:"ğ",Gcedil:"Ģ",Gcirc:"Ĝ",gcirc:"ĝ",Gcy:"Г",gcy:"г",Gdot:"Ġ",gdot:"ġ",ge:"≥",gE:"≧",gEl:"⪌",gel:"⋛",geq:"≥",geqq:"≧",geqslant:"⩾",gescc:"⪩",ges:"⩾",gesdot:"⪀",gesdoto:"⪂",gesdotol:"⪄",gesl:"⋛︀",gesles:"⪔",Gfr:"𝔊",gfr:"𝔤",gg:"≫",Gg:"⋙",ggg:"⋙",gimel:"ℷ",GJcy:"Ѓ",gjcy:"ѓ",gla:"⪥",gl:"≷",glE:"⪒",glj:"⪤",gnap:"⪊",gnapprox:"⪊",gne:"⪈",gnE:"≩",gneq:"⪈",gneqq:"≩",gnsim:"⋧",Gopf:"𝔾",gopf:"𝕘",grave:"`",GreaterEqual:"≥",GreaterEqualLess:"⋛",GreaterFullEqual:"≧",GreaterGreater:"⪢",GreaterLess:"≷",GreaterSlantEqual:"⩾",GreaterTilde:"≳",Gscr:"𝒢",gscr:"ℊ",gsim:"≳",gsime:"⪎",gsiml:"⪐",gtcc:"⪧",gtcir:"⩺",gt:">",GT:">",Gt:"≫",gtdot:"⋗",gtlPar:"⦕",gtquest:"⩼",gtrapprox:"⪆",gtrarr:"⥸",gtrdot:"⋗",gtreqless:"⋛",gtreqqless:"⪌",gtrless:"≷",gtrsim:"≳",gvertneqq:"≩︀",gvnE:"≩︀",Hacek:"ˇ",hairsp:" ",half:"½",hamilt:"ℋ",HARDcy:"Ъ",hardcy:"ъ",harrcir:"⥈",harr:"↔",hArr:"⇔",harrw:"↭",Hat:"^",hbar:"ℏ",Hcirc:"Ĥ",hcirc:"ĥ",hearts:"♥",heartsuit:"♥",hellip:"…",hercon:"⊹",hfr:"𝔥",Hfr:"ℌ",HilbertSpace:"ℋ",hksearow:"⤥",hkswarow:"⤦",hoarr:"⇿",homtht:"∻",hookleftarrow:"↩",hookrightarrow:"↪",hopf:"𝕙",Hopf:"ℍ",horbar:"―",HorizontalLine:"─",hscr:"𝒽",Hscr:"ℋ",hslash:"ℏ",Hstrok:"Ħ",hstrok:"ħ",HumpDownHump:"≎",HumpEqual:"≏",hybull:"⁃",hyphen:"‐",Iacute:"Í",iacute:"í",ic:"⁣",Icirc:"Î",icirc:"î",Icy:"И",icy:"и",Idot:"İ",IEcy:"Е",iecy:"е",iexcl:"¡",iff:"⇔",ifr:"𝔦",Ifr:"ℑ",Igrave:"Ì",igrave:"ì",ii:"ⅈ",iiiint:"⨌",iiint:"∭",iinfin:"⧜",iiota:"℩",IJlig:"Ĳ",ijlig:"ĳ",Imacr:"Ī",imacr:"ī",image:"ℑ",ImaginaryI:"ⅈ",imagline:"ℐ",imagpart:"ℑ",imath:"ı",Im:"ℑ",imof:"⊷",imped:"Ƶ",Implies:"⇒",incare:"℅",in:"∈",infin:"∞",infintie:"⧝",inodot:"ı",intcal:"⊺",int:"∫",Int:"∬",integers:"ℤ",Integral:"∫",intercal:"⊺",Intersection:"⋂",intlarhk:"⨗",intprod:"⨼",InvisibleComma:"⁣",InvisibleTimes:"⁢",IOcy:"Ё",iocy:"ё",Iogon:"Į",iogon:"į",Iopf:"𝕀",iopf:"𝕚",Iota:"Ι",iota:"ι",iprod:"⨼",iquest:"¿",iscr:"𝒾",Iscr:"ℐ",isin:"∈",isindot:"⋵",isinE:"⋹",isins:"⋴",isinsv:"⋳",isinv:"∈",it:"⁢",Itilde:"Ĩ",itilde:"ĩ",Iukcy:"І",iukcy:"і",Iuml:"Ï",iuml:"ï",Jcirc:"Ĵ",jcirc:"ĵ",Jcy:"Й",jcy:"й",Jfr:"𝔍",jfr:"𝔧",jmath:"ȷ",Jopf:"𝕁",jopf:"𝕛",Jscr:"𝒥",jscr:"𝒿",Jsercy:"Ј",jsercy:"ј",Jukcy:"Є",jukcy:"є",Kappa:"Κ",kappa:"κ",kappav:"ϰ",Kcedil:"Ķ",kcedil:"ķ",Kcy:"К",kcy:"к",Kfr:"𝔎",kfr:"𝔨",kgreen:"ĸ",KHcy:"Х",khcy:"х",KJcy:"Ќ",kjcy:"ќ",Kopf:"𝕂",kopf:"𝕜",Kscr:"𝒦",kscr:"𝓀",lAarr:"⇚",Lacute:"Ĺ",lacute:"ĺ",laemptyv:"⦴",lagran:"ℒ",Lambda:"Λ",lambda:"λ",lang:"⟨",Lang:"⟪",langd:"⦑",langle:"⟨",lap:"⪅",Laplacetrf:"ℒ",laquo:"«",larrb:"⇤",larrbfs:"⤟",larr:"←",Larr:"↞",lArr:"⇐",larrfs:"⤝",larrhk:"↩",larrlp:"↫",larrpl:"⤹",larrsim:"⥳",larrtl:"↢",latail:"⤙",lAtail:"⤛",lat:"⪫",late:"⪭",lates:"⪭︀",lbarr:"⤌",lBarr:"⤎",lbbrk:"❲",lbrace:"{",lbrack:"[",lbrke:"⦋",lbrksld:"⦏",lbrkslu:"⦍",Lcaron:"Ľ",lcaron:"ľ",Lcedil:"Ļ",lcedil:"ļ",lceil:"⌈",lcub:"{",Lcy:"Л",lcy:"л",ldca:"⤶",ldquo:"“",ldquor:"„",ldrdhar:"⥧",ldrushar:"⥋",ldsh:"↲",le:"≤",lE:"≦",LeftAngleBracket:"⟨",LeftArrowBar:"⇤",leftarrow:"←",LeftArrow:"←",Leftarrow:"⇐",LeftArrowRightArrow:"⇆",leftarrowtail:"↢",LeftCeiling:"⌈",LeftDoubleBracket:"⟦",LeftDownTeeVector:"⥡",LeftDownVectorBar:"⥙",LeftDownVector:"⇃",LeftFloor:"⌊",leftharpoondown:"↽",leftharpoonup:"↼",leftleftarrows:"⇇",leftrightarrow:"↔",LeftRightArrow:"↔",Leftrightarrow:"⇔",leftrightarrows:"⇆",leftrightharpoons:"⇋",leftrightsquigarrow:"↭",LeftRightVector:"⥎",LeftTeeArrow:"↤",LeftTee:"⊣",LeftTeeVector:"⥚",leftthreetimes:"⋋",LeftTriangleBar:"⧏",LeftTriangle:"⊲",LeftTriangleEqual:"⊴",LeftUpDownVector:"⥑",LeftUpTeeVector:"⥠",LeftUpVectorBar:"⥘",LeftUpVector:"↿",LeftVectorBar:"⥒",LeftVector:"↼",lEg:"⪋",leg:"⋚",leq:"≤",leqq:"≦",leqslant:"⩽",lescc:"⪨",les:"⩽",lesdot:"⩿",lesdoto:"⪁",lesdotor:"⪃",lesg:"⋚︀",lesges:"⪓",lessapprox:"⪅",lessdot:"⋖",lesseqgtr:"⋚",lesseqqgtr:"⪋",LessEqualGreater:"⋚",LessFullEqual:"≦",LessGreater:"≶",lessgtr:"≶",LessLess:"⪡",lesssim:"≲",LessSlantEqual:"⩽",LessTilde:"≲",lfisht:"⥼",lfloor:"⌊",Lfr:"𝔏",lfr:"𝔩",lg:"≶",lgE:"⪑",lHar:"⥢",lhard:"↽",lharu:"↼",lharul:"⥪",lhblk:"▄",LJcy:"Љ",ljcy:"љ",llarr:"⇇",ll:"≪",Ll:"⋘",llcorner:"⌞",Lleftarrow:"⇚",llhard:"⥫",lltri:"◺",Lmidot:"Ŀ",lmidot:"ŀ",lmoustache:"⎰",lmoust:"⎰",lnap:"⪉",lnapprox:"⪉",lne:"⪇",lnE:"≨",lneq:"⪇",lneqq:"≨",lnsim:"⋦",loang:"⟬",loarr:"⇽",lobrk:"⟦",longleftarrow:"⟵",LongLeftArrow:"⟵",Longleftarrow:"⟸",longleftrightarrow:"⟷",LongLeftRightArrow:"⟷",Longleftrightarrow:"⟺",longmapsto:"⟼",longrightarrow:"⟶",LongRightArrow:"⟶",Longrightarrow:"⟹",looparrowleft:"↫",looparrowright:"↬",lopar:"⦅",Lopf:"𝕃",lopf:"𝕝",loplus:"⨭",lotimes:"⨴",lowast:"∗",lowbar:"_",LowerLeftArrow:"↙",LowerRightArrow:"↘",loz:"◊",lozenge:"◊",lozf:"⧫",lpar:"(",lparlt:"⦓",lrarr:"⇆",lrcorner:"⌟",lrhar:"⇋",lrhard:"⥭",lrm:"‎",lrtri:"⊿",lsaquo:"‹",lscr:"𝓁",Lscr:"ℒ",lsh:"↰",Lsh:"↰",lsim:"≲",lsime:"⪍",lsimg:"⪏",lsqb:"[",lsquo:"‘",lsquor:"‚",Lstrok:"Ł",lstrok:"ł",ltcc:"⪦",ltcir:"⩹",lt:"<",LT:"<",Lt:"≪",ltdot:"⋖",lthree:"⋋",ltimes:"⋉",ltlarr:"⥶",ltquest:"⩻",ltri:"◃",ltrie:"⊴",ltrif:"◂",ltrPar:"⦖",lurdshar:"⥊",luruhar:"⥦",lvertneqq:"≨︀",lvnE:"≨︀",macr:"¯",male:"♂",malt:"✠",maltese:"✠",Map:"⤅",map:"↦",mapsto:"↦",mapstodown:"↧",mapstoleft:"↤",mapstoup:"↥",marker:"▮",mcomma:"⨩",Mcy:"М",mcy:"м",mdash:"—",mDDot:"∺",measuredangle:"∡",MediumSpace:" ",Mellintrf:"ℳ",Mfr:"𝔐",mfr:"𝔪",mho:"℧",micro:"µ",midast:"*",midcir:"⫰",mid:"∣",middot:"·",minusb:"⊟",minus:"−",minusd:"∸",minusdu:"⨪",MinusPlus:"∓",mlcp:"⫛",mldr:"…",mnplus:"∓",models:"⊧",Mopf:"𝕄",mopf:"𝕞",mp:"∓",mscr:"𝓂",Mscr:"ℳ",mstpos:"∾",Mu:"Μ",mu:"μ",multimap:"⊸",mumap:"⊸",nabla:"∇",Nacute:"Ń",nacute:"ń",nang:"∠⃒",nap:"≉",napE:"⩰̸",napid:"≋̸",napos:"ŉ",napprox:"≉",natural:"♮",naturals:"ℕ",natur:"♮",nbsp:" ",nbump:"≎̸",nbumpe:"≏̸",ncap:"⩃",Ncaron:"Ň",ncaron:"ň",Ncedil:"Ņ",ncedil:"ņ",ncong:"≇",ncongdot:"⩭̸",ncup:"⩂",Ncy:"Н",ncy:"н",ndash:"–",nearhk:"⤤",nearr:"↗",neArr:"⇗",nearrow:"↗",ne:"≠",nedot:"≐̸",NegativeMediumSpace:"​",NegativeThickSpace:"​",NegativeThinSpace:"​",NegativeVeryThinSpace:"​",nequiv:"≢",nesear:"⤨",nesim:"≂̸",NestedGreaterGreater:"≫",NestedLessLess:"≪",NewLine:"\n",nexist:"∄",nexists:"∄",Nfr:"𝔑",nfr:"𝔫",ngE:"≧̸",nge:"≱",ngeq:"≱",ngeqq:"≧̸",ngeqslant:"⩾̸",nges:"⩾̸",nGg:"⋙̸",ngsim:"≵",nGt:"≫⃒",ngt:"≯",ngtr:"≯",nGtv:"≫̸",nharr:"↮",nhArr:"⇎",nhpar:"⫲",ni:"∋",nis:"⋼",nisd:"⋺",niv:"∋",NJcy:"Њ",njcy:"њ",nlarr:"↚",nlArr:"⇍",nldr:"‥",nlE:"≦̸",nle:"≰",nleftarrow:"↚",nLeftarrow:"⇍",nleftrightarrow:"↮",nLeftrightarrow:"⇎",nleq:"≰",nleqq:"≦̸",nleqslant:"⩽̸",nles:"⩽̸",nless:"≮",nLl:"⋘̸",nlsim:"≴",nLt:"≪⃒",nlt:"≮",nltri:"⋪",nltrie:"⋬",nLtv:"≪̸",nmid:"∤",NoBreak:"⁠",NonBreakingSpace:" ",nopf:"𝕟",Nopf:"ℕ",Not:"⫬",not:"¬",NotCongruent:"≢",NotCupCap:"≭",NotDoubleVerticalBar:"∦",NotElement:"∉",NotEqual:"≠",NotEqualTilde:"≂̸",NotExists:"∄",NotGreater:"≯",NotGreaterEqual:"≱",NotGreaterFullEqual:"≧̸",NotGreaterGreater:"≫̸",NotGreaterLess:"≹",NotGreaterSlantEqual:"⩾̸",NotGreaterTilde:"≵",NotHumpDownHump:"≎̸",NotHumpEqual:"≏̸",notin:"∉",notindot:"⋵̸",notinE:"⋹̸",notinva:"∉",notinvb:"⋷",notinvc:"⋶",NotLeftTriangleBar:"⧏̸",NotLeftTriangle:"⋪",NotLeftTriangleEqual:"⋬",NotLess:"≮",NotLessEqual:"≰",NotLessGreater:"≸",NotLessLess:"≪̸",NotLessSlantEqual:"⩽̸",NotLessTilde:"≴",NotNestedGreaterGreater:"⪢̸",NotNestedLessLess:"⪡̸",notni:"∌",notniva:"∌",notnivb:"⋾",notnivc:"⋽",NotPrecedes:"⊀",NotPrecedesEqual:"⪯̸",NotPrecedesSlantEqual:"⋠",NotReverseElement:"∌",NotRightTriangleBar:"⧐̸",NotRightTriangle:"⋫",NotRightTriangleEqual:"⋭",NotSquareSubset:"⊏̸",NotSquareSubsetEqual:"⋢",NotSquareSuperset:"⊐̸",NotSquareSupersetEqual:"⋣",NotSubset:"⊂⃒",NotSubsetEqual:"⊈",NotSucceeds:"⊁",NotSucceedsEqual:"⪰̸",NotSucceedsSlantEqual:"⋡",NotSucceedsTilde:"≿̸",NotSuperset:"⊃⃒",NotSupersetEqual:"⊉",NotTilde:"≁",NotTildeEqual:"≄",NotTildeFullEqual:"≇",NotTildeTilde:"≉",NotVerticalBar:"∤",nparallel:"∦",npar:"∦",nparsl:"⫽⃥",npart:"∂̸",npolint:"⨔",npr:"⊀",nprcue:"⋠",nprec:"⊀",npreceq:"⪯̸",npre:"⪯̸",nrarrc:"⤳̸",nrarr:"↛",nrArr:"⇏",nrarrw:"↝̸",nrightarrow:"↛",nRightarrow:"⇏",nrtri:"⋫",nrtrie:"⋭",nsc:"⊁",nsccue:"⋡",nsce:"⪰̸",Nscr:"𝒩",nscr:"𝓃",nshortmid:"∤",nshortparallel:"∦",nsim:"≁",nsime:"≄",nsimeq:"≄",nsmid:"∤",nspar:"∦",nsqsube:"⋢",nsqsupe:"⋣",nsub:"⊄",nsubE:"⫅̸",nsube:"⊈",nsubset:"⊂⃒",nsubseteq:"⊈",nsubseteqq:"⫅̸",nsucc:"⊁",nsucceq:"⪰̸",nsup:"⊅",nsupE:"⫆̸",nsupe:"⊉",nsupset:"⊃⃒",nsupseteq:"⊉",nsupseteqq:"⫆̸",ntgl:"≹",Ntilde:"Ñ",ntilde:"ñ",ntlg:"≸",ntriangleleft:"⋪",ntrianglelefteq:"⋬",ntriangleright:"⋫",ntrianglerighteq:"⋭",Nu:"Ν",nu:"ν",num:"#",numero:"№",numsp:" ",nvap:"≍⃒",nvdash:"⊬",nvDash:"⊭",nVdash:"⊮",nVDash:"⊯",nvge:"≥⃒",nvgt:">⃒",nvHarr:"⤄",nvinfin:"⧞",nvlArr:"⤂",nvle:"≤⃒",nvlt:"<⃒",nvltrie:"⊴⃒",nvrArr:"⤃",nvrtrie:"⊵⃒",nvsim:"∼⃒",nwarhk:"⤣",nwarr:"↖",nwArr:"⇖",nwarrow:"↖",nwnear:"⤧",Oacute:"Ó",oacute:"ó",oast:"⊛",Ocirc:"Ô",ocirc:"ô",ocir:"⊚",Ocy:"О",ocy:"о",odash:"⊝",Odblac:"Ő",odblac:"ő",odiv:"⨸",odot:"⊙",odsold:"⦼",OElig:"Œ",oelig:"œ",ofcir:"⦿",Ofr:"𝔒",ofr:"𝔬",ogon:"˛",Ograve:"Ò",ograve:"ò",ogt:"⧁",ohbar:"⦵",ohm:"Ω",oint:"∮",olarr:"↺",olcir:"⦾",olcross:"⦻",oline:"‾",olt:"⧀",Omacr:"Ō",omacr:"ō",Omega:"Ω",omega:"ω",Omicron:"Ο",omicron:"ο",omid:"⦶",ominus:"⊖",Oopf:"𝕆",oopf:"𝕠",opar:"⦷",OpenCurlyDoubleQuote:"“",OpenCurlyQuote:"‘",operp:"⦹",oplus:"⊕",orarr:"↻",Or:"⩔",or:"∨",ord:"⩝",order:"ℴ",orderof:"ℴ",ordf:"ª",ordm:"º",origof:"⊶",oror:"⩖",orslope:"⩗",orv:"⩛",oS:"Ⓢ",Oscr:"𝒪",oscr:"ℴ",Oslash:"Ø",oslash:"ø",osol:"⊘",Otilde:"Õ",otilde:"õ",otimesas:"⨶",Otimes:"⨷",otimes:"⊗",Ouml:"Ö",ouml:"ö",ovbar:"⌽",OverBar:"‾",OverBrace:"⏞",OverBracket:"⎴",OverParenthesis:"⏜",para:"¶",parallel:"∥",par:"∥",parsim:"⫳",parsl:"⫽",part:"∂",PartialD:"∂",Pcy:"П",pcy:"п",percnt:"%",period:".",permil:"‰",perp:"⊥",pertenk:"‱",Pfr:"𝔓",pfr:"𝔭",Phi:"Φ",phi:"φ",phiv:"ϕ",phmmat:"ℳ",phone:"☎",Pi:"Π",pi:"π",pitchfork:"⋔",piv:"ϖ",planck:"ℏ",planckh:"ℎ",plankv:"ℏ",plusacir:"⨣",plusb:"⊞",pluscir:"⨢",plus:"+",plusdo:"∔",plusdu:"⨥",pluse:"⩲",PlusMinus:"±",plusmn:"±",plussim:"⨦",plustwo:"⨧",pm:"±",Poincareplane:"ℌ",pointint:"⨕",popf:"𝕡",Popf:"ℙ",pound:"£",prap:"⪷",Pr:"⪻",pr:"≺",prcue:"≼",precapprox:"⪷",prec:"≺",preccurlyeq:"≼",Precedes:"≺",PrecedesEqual:"⪯",PrecedesSlantEqual:"≼",PrecedesTilde:"≾",preceq:"⪯",precnapprox:"⪹",precneqq:"⪵",precnsim:"⋨",pre:"⪯",prE:"⪳",precsim:"≾",prime:"′",Prime:"″",primes:"ℙ",prnap:"⪹",prnE:"⪵",prnsim:"⋨",prod:"∏",Product:"∏",profalar:"⌮",profline:"⌒",profsurf:"⌓",prop:"∝",Proportional:"∝",Proportion:"∷",propto:"∝",prsim:"≾",prurel:"⊰",Pscr:"𝒫",pscr:"𝓅",Psi:"Ψ",psi:"ψ",puncsp:" ",Qfr:"𝔔",qfr:"𝔮",qint:"⨌",qopf:"𝕢",Qopf:"ℚ",qprime:"⁗",Qscr:"𝒬",qscr:"𝓆",quaternions:"ℍ",quatint:"⨖",quest:"?",questeq:"≟",quot:'"',QUOT:'"',rAarr:"⇛",race:"∽̱",Racute:"Ŕ",racute:"ŕ",radic:"√",raemptyv:"⦳",rang:"⟩",Rang:"⟫",rangd:"⦒",range:"⦥",rangle:"⟩",raquo:"»",rarrap:"⥵",rarrb:"⇥",rarrbfs:"⤠",rarrc:"⤳",rarr:"→",Rarr:"↠",rArr:"⇒",rarrfs:"⤞",rarrhk:"↪",rarrlp:"↬",rarrpl:"⥅",rarrsim:"⥴",Rarrtl:"⤖",rarrtl:"↣",rarrw:"↝",ratail:"⤚",rAtail:"⤜",ratio:"∶",rationals:"ℚ",rbarr:"⤍",rBarr:"⤏",RBarr:"⤐",rbbrk:"❳",rbrace:"}",rbrack:"]",rbrke:"⦌",rbrksld:"⦎",rbrkslu:"⦐",Rcaron:"Ř",rcaron:"ř",Rcedil:"Ŗ",rcedil:"ŗ",rceil:"⌉",rcub:"}",Rcy:"Р",rcy:"р",rdca:"⤷",rdldhar:"⥩",rdquo:"”",rdquor:"”",rdsh:"↳",real:"ℜ",realine:"ℛ",realpart:"ℜ",reals:"ℝ",Re:"ℜ",rect:"▭",reg:"®",REG:"®",ReverseElement:"∋",ReverseEquilibrium:"⇋",ReverseUpEquilibrium:"⥯",rfisht:"⥽",rfloor:"⌋",rfr:"𝔯",Rfr:"ℜ",rHar:"⥤",rhard:"⇁",rharu:"⇀",rharul:"⥬",Rho:"Ρ",rho:"ρ",rhov:"ϱ",RightAngleBracket:"⟩",RightArrowBar:"⇥",rightarrow:"→",RightArrow:"→",Rightarrow:"⇒",RightArrowLeftArrow:"⇄",rightarrowtail:"↣",RightCeiling:"⌉",RightDoubleBracket:"⟧",RightDownTeeVector:"⥝",RightDownVectorBar:"⥕",RightDownVector:"⇂",RightFloor:"⌋",rightharpoondown:"⇁",rightharpoonup:"⇀",rightleftarrows:"⇄",rightleftharpoons:"⇌",rightrightarrows:"⇉",rightsquigarrow:"↝",RightTeeArrow:"↦",RightTee:"⊢",RightTeeVector:"⥛",rightthreetimes:"⋌",RightTriangleBar:"⧐",RightTriangle:"⊳",RightTriangleEqual:"⊵",RightUpDownVector:"⥏",RightUpTeeVector:"⥜",RightUpVectorBar:"⥔",RightUpVector:"↾",RightVectorBar:"⥓",RightVector:"⇀",ring:"˚",risingdotseq:"≓",rlarr:"⇄",rlhar:"⇌",rlm:"‏",rmoustache:"⎱",rmoust:"⎱",rnmid:"⫮",roang:"⟭",roarr:"⇾",robrk:"⟧",ropar:"⦆",ropf:"𝕣",Ropf:"ℝ",roplus:"⨮",rotimes:"⨵",RoundImplies:"⥰",rpar:")",rpargt:"⦔",rppolint:"⨒",rrarr:"⇉",Rrightarrow:"⇛",rsaquo:"›",rscr:"𝓇",Rscr:"ℛ",rsh:"↱",Rsh:"↱",rsqb:"]",rsquo:"’",rsquor:"’",rthree:"⋌",rtimes:"⋊",rtri:"▹",rtrie:"⊵",rtrif:"▸",rtriltri:"⧎",RuleDelayed:"⧴",ruluhar:"⥨",rx:"℞",Sacute:"Ś",sacute:"ś",sbquo:"‚",scap:"⪸",Scaron:"Š",scaron:"š",Sc:"⪼",sc:"≻",sccue:"≽",sce:"⪰",scE:"⪴",Scedil:"Ş",scedil:"ş",Scirc:"Ŝ",scirc:"ŝ",scnap:"⪺",scnE:"⪶",scnsim:"⋩",scpolint:"⨓",scsim:"≿",Scy:"С",scy:"с",sdotb:"⊡",sdot:"⋅",sdote:"⩦",searhk:"⤥",searr:"↘",seArr:"⇘",searrow:"↘",sect:"§",semi:";",seswar:"⤩",setminus:"∖",setmn:"∖",sext:"✶",Sfr:"𝔖",sfr:"𝔰",sfrown:"⌢",sharp:"♯",SHCHcy:"Щ",shchcy:"щ",SHcy:"Ш",shcy:"ш",ShortDownArrow:"↓",ShortLeftArrow:"←",shortmid:"∣",shortparallel:"∥",ShortRightArrow:"→",ShortUpArrow:"↑",shy:"­",Sigma:"Σ",sigma:"σ",sigmaf:"ς",sigmav:"ς",sim:"∼",simdot:"⩪",sime:"≃",simeq:"≃",simg:"⪞",simgE:"⪠",siml:"⪝",simlE:"⪟",simne:"≆",simplus:"⨤",simrarr:"⥲",slarr:"←",SmallCircle:"∘",smallsetminus:"∖",smashp:"⨳",smeparsl:"⧤",smid:"∣",smile:"⌣",smt:"⪪",smte:"⪬",smtes:"⪬︀",SOFTcy:"Ь",softcy:"ь",solbar:"⌿",solb:"⧄",sol:"/",Sopf:"𝕊",sopf:"𝕤",spades:"♠",spadesuit:"♠",spar:"∥",sqcap:"⊓",sqcaps:"⊓︀",sqcup:"⊔",sqcups:"⊔︀",Sqrt:"√",sqsub:"⊏",sqsube:"⊑",sqsubset:"⊏",sqsubseteq:"⊑",sqsup:"⊐",sqsupe:"⊒",sqsupset:"⊐",sqsupseteq:"⊒",square:"□",Square:"□",SquareIntersection:"⊓",SquareSubset:"⊏",SquareSubsetEqual:"⊑",SquareSuperset:"⊐",SquareSupersetEqual:"⊒",SquareUnion:"⊔",squarf:"▪",squ:"□",squf:"▪",srarr:"→",Sscr:"𝒮",sscr:"𝓈",ssetmn:"∖",ssmile:"⌣",sstarf:"⋆",Star:"⋆",star:"☆",starf:"★",straightepsilon:"ϵ",straightphi:"ϕ",strns:"¯",sub:"⊂",Sub:"⋐",subdot:"⪽",subE:"⫅",sube:"⊆",subedot:"⫃",submult:"⫁",subnE:"⫋",subne:"⊊",subplus:"⪿",subrarr:"⥹",subset:"⊂",Subset:"⋐",subseteq:"⊆",subseteqq:"⫅",SubsetEqual:"⊆",subsetneq:"⊊",subsetneqq:"⫋",subsim:"⫇",subsub:"⫕",subsup:"⫓",succapprox:"⪸",succ:"≻",succcurlyeq:"≽",Succeeds:"≻",SucceedsEqual:"⪰",SucceedsSlantEqual:"≽",SucceedsTilde:"≿",succeq:"⪰",succnapprox:"⪺",succneqq:"⪶",succnsim:"⋩",succsim:"≿",SuchThat:"∋",sum:"∑",Sum:"∑",sung:"♪",sup1:"¹",sup2:"²",sup3:"³",sup:"⊃",Sup:"⋑",supdot:"⪾",supdsub:"⫘",supE:"⫆",supe:"⊇",supedot:"⫄",Superset:"⊃",SupersetEqual:"⊇",suphsol:"⟉",suphsub:"⫗",suplarr:"⥻",supmult:"⫂",supnE:"⫌",supne:"⊋",supplus:"⫀",supset:"⊃",Supset:"⋑",supseteq:"⊇",supseteqq:"⫆",supsetneq:"⊋",supsetneqq:"⫌",supsim:"⫈",supsub:"⫔",supsup:"⫖",swarhk:"⤦",swarr:"↙",swArr:"⇙",swarrow:"↙",swnwar:"⤪",szlig:"ß",Tab:"\t",target:"⌖",Tau:"Τ",tau:"τ",tbrk:"⎴",Tcaron:"Ť",tcaron:"ť",Tcedil:"Ţ",tcedil:"ţ",Tcy:"Т",tcy:"т",tdot:"⃛",telrec:"⌕",Tfr:"𝔗",tfr:"𝔱",there4:"∴",therefore:"∴",Therefore:"∴",Theta:"Θ",theta:"θ",thetasym:"ϑ",thetav:"ϑ",thickapprox:"≈",thicksim:"∼",ThickSpace:"  ",ThinSpace:" ",thinsp:" ",thkap:"≈",thksim:"∼",THORN:"Þ",thorn:"þ",tilde:"˜",Tilde:"∼",TildeEqual:"≃",TildeFullEqual:"≅",TildeTilde:"≈",timesbar:"⨱",timesb:"⊠",times:"×",timesd:"⨰",tint:"∭",toea:"⤨",topbot:"⌶",topcir:"⫱",top:"⊤",Topf:"𝕋",topf:"𝕥",topfork:"⫚",tosa:"⤩",tprime:"‴",trade:"™",TRADE:"™",triangle:"▵",triangledown:"▿",triangleleft:"◃",trianglelefteq:"⊴",triangleq:"≜",triangleright:"▹",trianglerighteq:"⊵",tridot:"◬",trie:"≜",triminus:"⨺",TripleDot:"⃛",triplus:"⨹",trisb:"⧍",tritime:"⨻",trpezium:"⏢",Tscr:"𝒯",tscr:"𝓉",TScy:"Ц",tscy:"ц",TSHcy:"Ћ",tshcy:"ћ",Tstrok:"Ŧ",tstrok:"ŧ",twixt:"≬",twoheadleftarrow:"↞",twoheadrightarrow:"↠",Uacute:"Ú",uacute:"ú",uarr:"↑",Uarr:"↟",uArr:"⇑",Uarrocir:"⥉",Ubrcy:"Ў",ubrcy:"ў",Ubreve:"Ŭ",ubreve:"ŭ",Ucirc:"Û",ucirc:"û",Ucy:"У",ucy:"у",udarr:"⇅",Udblac:"Ű",udblac:"ű",udhar:"⥮",ufisht:"⥾",Ufr:"𝔘",ufr:"𝔲",Ugrave:"Ù",ugrave:"ù",uHar:"⥣",uharl:"↿",uharr:"↾",uhblk:"▀",ulcorn:"⌜",ulcorner:"⌜",ulcrop:"⌏",ultri:"◸",Umacr:"Ū",umacr:"ū",uml:"¨",UnderBar:"_",UnderBrace:"⏟",UnderBracket:"⎵",UnderParenthesis:"⏝",Union:"⋃",UnionPlus:"⊎",Uogon:"Ų",uogon:"ų",Uopf:"𝕌",uopf:"𝕦",UpArrowBar:"⤒",uparrow:"↑",UpArrow:"↑",Uparrow:"⇑",UpArrowDownArrow:"⇅",updownarrow:"↕",UpDownArrow:"↕",Updownarrow:"⇕",UpEquilibrium:"⥮",upharpoonleft:"↿",upharpoonright:"↾",uplus:"⊎",UpperLeftArrow:"↖",UpperRightArrow:"↗",upsi:"υ",Upsi:"ϒ",upsih:"ϒ",Upsilon:"Υ",upsilon:"υ",UpTeeArrow:"↥",UpTee:"⊥",upuparrows:"⇈",urcorn:"⌝",urcorner:"⌝",urcrop:"⌎",Uring:"Ů",uring:"ů",urtri:"◹",Uscr:"𝒰",uscr:"𝓊",utdot:"⋰",Utilde:"Ũ",utilde:"ũ",utri:"▵",utrif:"▴",uuarr:"⇈",Uuml:"Ü",uuml:"ü",uwangle:"⦧",vangrt:"⦜",varepsilon:"ϵ",varkappa:"ϰ",varnothing:"∅",varphi:"ϕ",varpi:"ϖ",varpropto:"∝",varr:"↕",vArr:"⇕",varrho:"ϱ",varsigma:"ς",varsubsetneq:"⊊︀",varsubsetneqq:"⫋︀",varsupsetneq:"⊋︀",varsupsetneqq:"⫌︀",vartheta:"ϑ",vartriangleleft:"⊲",vartriangleright:"⊳",vBar:"⫨",Vbar:"⫫",vBarv:"⫩",Vcy:"В",vcy:"в",vdash:"⊢",vDash:"⊨",Vdash:"⊩",VDash:"⊫",Vdashl:"⫦",veebar:"⊻",vee:"∨",Vee:"⋁",veeeq:"≚",vellip:"⋮",verbar:"|",Verbar:"‖",vert:"|",Vert:"‖",VerticalBar:"∣",VerticalLine:"|",VerticalSeparator:"❘",VerticalTilde:"≀",VeryThinSpace:" ",Vfr:"𝔙",vfr:"𝔳",vltri:"⊲",vnsub:"⊂⃒",vnsup:"⊃⃒",Vopf:"𝕍",vopf:"𝕧",vprop:"∝",vrtri:"⊳",Vscr:"𝒱",vscr:"𝓋",vsubnE:"⫋︀",vsubne:"⊊︀",vsupnE:"⫌︀",vsupne:"⊋︀",Vvdash:"⊪",vzigzag:"⦚",Wcirc:"Ŵ",wcirc:"ŵ",wedbar:"⩟",wedge:"∧",Wedge:"⋀",wedgeq:"≙",weierp:"℘",Wfr:"𝔚",wfr:"𝔴",Wopf:"𝕎",wopf:"𝕨",wp:"℘",wr:"≀",wreath:"≀",Wscr:"𝒲",wscr:"𝓌",xcap:"⋂",xcirc:"◯",xcup:"⋃",xdtri:"▽",Xfr:"𝔛",xfr:"𝔵",xharr:"⟷",xhArr:"⟺",Xi:"Ξ",xi:"ξ",xlarr:"⟵",xlArr:"⟸",xmap:"⟼",xnis:"⋻",xodot:"⨀",Xopf:"𝕏",xopf:"𝕩",xoplus:"⨁",xotime:"⨂",xrarr:"⟶",xrArr:"⟹",Xscr:"𝒳",xscr:"𝓍",xsqcup:"⨆",xuplus:"⨄",xutri:"△",xvee:"⋁",xwedge:"⋀",Yacute:"Ý",yacute:"ý",YAcy:"Я",yacy:"я",Ycirc:"Ŷ",ycirc:"ŷ",Ycy:"Ы",ycy:"ы",yen:"¥",Yfr:"𝔜",yfr:"𝔶",YIcy:"Ї",yicy:"ї",Yopf:"𝕐",yopf:"𝕪",Yscr:"𝒴",yscr:"𝓎",YUcy:"Ю",yucy:"ю",yuml:"ÿ",Yuml:"Ÿ",Zacute:"Ź",zacute:"ź",Zcaron:"Ž",zcaron:"ž",Zcy:"З",zcy:"з",Zdot:"Ż",zdot:"ż",zeetrf:"ℨ",ZeroWidthSpace:"​",Zeta:"Ζ",zeta:"ζ",zfr:"𝔷",Zfr:"ℨ",ZHcy:"Ж",zhcy:"ж",zigrarr:"⇝",zopf:"𝕫",Zopf:"ℤ",Zscr:"𝒵",zscr:"𝓏",zwj:"‍",zwnj:"‌"};},function(t,e){t.exports={amp:"&",apos:"'",gt:">",lt:"<",quot:'"'};},function(t,e,r){e=t.exports=r(36),e.Stream=e,e.Readable=e,e.Writable=r(21),e.Duplex=r(2),e.Transform=r(39),e.PassThrough=r(69);},function(t,e,r){(function(e,n,i){function o(t){var e=this;this.next=null,this.entry=null,this.finish=function(){A(e,t);};}function a(t){return q.from(t)}function s(t){return q.isBuffer(t)||t instanceof P}function u(){}function c(t,e){O=O||r(2),t=t||{};var n=e instanceof O;this.objectMode=!!t.objectMode,n&&(this.objectMode=this.objectMode||!!t.writableObjectMode);var i=t.highWaterMark,a=t.writableHighWaterMark,s=this.objectMode?16:16384;this.highWaterMark=i||0===i?i:n&&(a||0===a)?a:s,this.highWaterMark=Math.floor(this.highWaterMark),this.finalCalled=!1,this.needDrain=!1,this.ending=!1,this.ended=!1,this.finished=!1,this.destroyed=!1;var u=!1===t.decodeStrings;this.decodeStrings=!u,this.defaultEncoding=t.defaultEncoding||"utf8",this.length=0,this.writing=!1,this.corked=0,this.sync=!0,this.bufferProcessing=!1,this.onwrite=function(t){y(e,t);},this.writecb=null,this.writelen=0,this.bufferedRequest=null,this.lastBufferedRequest=null,this.pendingcb=0,this.prefinished=!1,this.errorEmitted=!1,this.bufferedRequestCount=0,this.corkedRequestsFree=new o(this);}function l(t){if(O=O||r(2),!(R.call(l,this)||this instanceof O))return new l(t);this._writableState=new c(t,this),this.writable=!0,t&&("function"==typeof t.write&&(this._write=t.write),"function"==typeof t.writev&&(this._writev=t.writev),"function"==typeof t.destroy&&(this._destroy=t.destroy),"function"==typeof t.final&&(this._final=t.final)),D.call(this);}function f(t,e){var r=new Error("write after end");t.emit("error",r),T.nextTick(e,r);}function h(t,e,r,n){var i=!0,o=!1;return null===r?o=new TypeError("May not write null values to stream"):"string"==typeof r||void 0===r||e.objectMode||(o=new TypeError("Invalid non-string/buffer chunk")),o&&(t.emit("error",o),T.nextTick(n,o),i=!1),i}function p(t,e,r){return t.objectMode||!1===t.decodeStrings||"string"!=typeof e||(e=q.from(e,r)),e}function d(t,e,r,n,i,o){if(!r){var a=p(e,n,i);n!==a&&(r=!0,i="buffer",n=a);}var s=e.objectMode?1:n.length;e.length+=s;var u=e.length<e.highWaterMark;if(u||(e.needDrain=!0),e.writing||e.corked){var c=e.lastBufferedRequest;e.lastBufferedRequest={chunk:n,encoding:i,isBuf:r,callback:o,next:null},c?c.next=e.lastBufferedRequest:e.bufferedRequest=e.lastBufferedRequest,e.bufferedRequestCount+=1;}else g(t,e,!1,s,n,i,o);return u}function g(t,e,r,n,i,o,a){e.writelen=n,e.writecb=a,e.writing=!0,e.sync=!0,r?t._writev(i,e.onwrite):t._write(i,o,e.onwrite),e.sync=!1;}function v(t,e,r,n,i){--e.pendingcb,r?(T.nextTick(i,n),T.nextTick(k,t,e),t._writableState.errorEmitted=!0,t.emit("error",n)):(i(n),t._writableState.errorEmitted=!0,t.emit("error",n),k(t,e));}function b(t){t.writing=!1,t.writecb=null,t.length-=t.writelen,t.writelen=0;}function y(t,e){var r=t._writableState,n=r.sync,i=r.writecb;if(b(r),e)v(t,r,n,e,i);else {var o=x(r);o||r.corked||r.bufferProcessing||!r.bufferedRequest||w(t,r),n?L(m,t,r,o,i):m(t,r,o,i);}}function m(t,e,r,n){r||_(t,e),e.pendingcb--,n(),k(t,e);}function _(t,e){0===e.length&&e.needDrain&&(e.needDrain=!1,t.emit("drain"));}function w(t,e){e.bufferProcessing=!0;var r=e.bufferedRequest;if(t._writev&&r&&r.next){var n=e.bufferedRequestCount,i=new Array(n),a=e.corkedRequestsFree;a.entry=r;for(var s=0,u=!0;r;)i[s]=r,r.isBuf||(u=!1),r=r.next,s+=1;i.allBuffers=u,g(t,e,!0,e.length,i,"",a.finish),e.pendingcb++,e.lastBufferedRequest=null,a.next?(e.corkedRequestsFree=a.next,a.next=null):e.corkedRequestsFree=new o(e),e.bufferedRequestCount=0;}else {for(;r;){var c=r.chunk,l=r.encoding,f=r.callback;if(g(t,e,!1,e.objectMode?1:c.length,c,l,f),r=r.next,e.bufferedRequestCount--,e.writing)break}null===r&&(e.lastBufferedRequest=null);}e.bufferedRequest=r,e.bufferProcessing=!1;}function x(t){return t.ending&&0===t.length&&null===t.bufferedRequest&&!t.finished&&!t.writing}function S(t,e){t._final(function(r){e.pendingcb--,r&&t.emit("error",r),e.prefinished=!0,t.emit("prefinish"),k(t,e);});}function j(t,e){e.prefinished||e.finalCalled||("function"==typeof t._final?(e.pendingcb++,e.finalCalled=!0,T.nextTick(S,t,e)):(e.prefinished=!0,t.emit("prefinish")));}function k(t,e){var r=x(e);return r&&(j(t,e),0===e.pendingcb&&(e.finished=!0,t.emit("finish"))),r}function E(t,e,r){e.ending=!0,k(t,e),r&&(e.finished?T.nextTick(r):t.once("finish",r)),e.ended=!0,t.writable=!1;}function A(t,e,r){var n=t.entry;for(t.entry=null;n;){var i=n.callback;e.pendingcb--,i(r),n=n.next;}e.corkedRequestsFree?e.corkedRequestsFree.next=t:e.corkedRequestsFree=t;}var T=r(14);t.exports=l;var O,L=!e.browser&&["v0.10","v0.9."].indexOf(e.version.slice(0,5))>-1?n:T.nextTick;l.WritableState=c;var C=r(8);C.inherits=r(1);var B={deprecate:r(68)},D=r(37),q=r(15).Buffer,P=i.Uint8Array||function(){},M=r(38);C.inherits(l,D),c.prototype.getBuffer=function(){for(var t=this.bufferedRequest,e=[];t;)e.push(t),t=t.next;return e},function(){try{Object.defineProperty(c.prototype,"buffer",{get:B.deprecate(function(){return this.getBuffer()},"_writableState.buffer is deprecated. Use _writableState.getBuffer instead.","DEP0003")});}catch(t){}}();var R;"function"==typeof Symbol&&Symbol.hasInstance&&"function"==typeof Function.prototype[Symbol.hasInstance]?(R=Function.prototype[Symbol.hasInstance],Object.defineProperty(l,Symbol.hasInstance,{value:function(t){return !!R.call(this,t)||this===l&&(t&&t._writableState instanceof c)}})):R=function(t){return t instanceof this},l.prototype.pipe=function(){this.emit("error",new Error("Cannot pipe, not readable"));},l.prototype.write=function(t,e,r){var n=this._writableState,i=!1,o=!n.objectMode&&s(t);return o&&!q.isBuffer(t)&&(t=a(t)),"function"==typeof e&&(r=e,e=null),o?e="buffer":e||(e=n.defaultEncoding),"function"!=typeof r&&(r=u),n.ended?f(this,r):(o||h(this,n,t,r))&&(n.pendingcb++,i=d(this,n,o,t,e,r)),i},l.prototype.cork=function(){this._writableState.corked++;},l.prototype.uncork=function(){var t=this._writableState;t.corked&&(t.corked--,t.writing||t.corked||t.finished||t.bufferProcessing||!t.bufferedRequest||w(this,t));},l.prototype.setDefaultEncoding=function(t){if("string"==typeof t&&(t=t.toLowerCase()),!(["hex","utf8","utf-8","ascii","binary","base64","ucs2","ucs-2","utf16le","utf-16le","raw"].indexOf((t+"").toLowerCase())>-1))throw new TypeError("Unknown encoding: "+t);return this._writableState.defaultEncoding=t,this},Object.defineProperty(l.prototype,"writableHighWaterMark",{enumerable:!1,get:function(){return this._writableState.highWaterMark}}),l.prototype._write=function(t,e,r){r(new Error("_write() is not implemented"));},l.prototype._writev=null,l.prototype.end=function(t,e,r){var n=this._writableState;"function"==typeof t?(r=t,t=null,e=null):"function"==typeof e&&(r=e,e=null),null!==t&&void 0!==t&&this.write(t,e),n.corked&&(n.corked=1,this.uncork()),n.ending||n.finished||E(this,n,r);},Object.defineProperty(l.prototype,"destroyed",{get:function(){return void 0!==this._writableState&&this._writableState.destroyed},set:function(t){this._writableState&&(this._writableState.destroyed=t);}}),l.prototype.destroy=M.destroy,l.prototype._undestroy=M.undestroy,l.prototype._destroy=function(t,e){this.end(),e(t);};}).call(e,r(7),r(66).setImmediate,r(0));},function(t,e,r){function n(t){if(!t)return "utf8";for(var e;;)switch(t){case"utf8":case"utf-8":return "utf8";case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return "utf16le";case"latin1":case"binary":return "latin1";case"base64":case"ascii":case"hex":return t;default:if(e)return;t=(""+t).toLowerCase(),e=!0;}}function i(t){var e=n(t);if("string"!=typeof e&&(y.isEncoding===m||!m(t)))throw new Error("Unknown encoding: "+t);return e||t}function o(t){this.encoding=i(t);var e;switch(this.encoding){case"utf16le":this.text=h,this.end=p,e=4;break;case"utf8":this.fillLast=c,e=4;break;case"base64":this.text=d,this.end=g,e=3;break;default:return this.write=v,void(this.end=b)}this.lastNeed=0,this.lastTotal=0,this.lastChar=y.allocUnsafe(e);}function a(t){return t<=127?0:t>>5==6?2:t>>4==14?3:t>>3==30?4:t>>6==2?-1:-2}function s(t,e,r){var n=e.length-1;if(n<r)return 0;var i=a(e[n]);return i>=0?(i>0&&(t.lastNeed=i-1),i):--n<r||-2===i?0:(i=a(e[n]))>=0?(i>0&&(t.lastNeed=i-2),i):--n<r||-2===i?0:(i=a(e[n]),i>=0?(i>0&&(2===i?i=0:t.lastNeed=i-3),i):0)}function u(t,e,r){if(128!=(192&e[0]))return t.lastNeed=0,"�";if(t.lastNeed>1&&e.length>1){if(128!=(192&e[1]))return t.lastNeed=1,"�";if(t.lastNeed>2&&e.length>2&&128!=(192&e[2]))return t.lastNeed=2,"�"}}function c(t){var e=this.lastTotal-this.lastNeed,r=u(this,t);return void 0!==r?r:this.lastNeed<=t.length?(t.copy(this.lastChar,e,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal)):(t.copy(this.lastChar,e,0,t.length),void(this.lastNeed-=t.length))}function l(t,e){var r=s(this,t,e);if(!this.lastNeed)return t.toString("utf8",e);this.lastTotal=r;var n=t.length-(r-this.lastNeed);return t.copy(this.lastChar,0,n),t.toString("utf8",e,n)}function f(t){var e=t&&t.length?this.write(t):"";return this.lastNeed?e+"�":e}function h(t,e){if((t.length-e)%2==0){var r=t.toString("utf16le",e);if(r){var n=r.charCodeAt(r.length-1);if(n>=55296&&n<=56319)return this.lastNeed=2,this.lastTotal=4,this.lastChar[0]=t[t.length-2],this.lastChar[1]=t[t.length-1],r.slice(0,-1)}return r}return this.lastNeed=1,this.lastTotal=2,this.lastChar[0]=t[t.length-1],t.toString("utf16le",e,t.length-1)}function p(t){var e=t&&t.length?this.write(t):"";if(this.lastNeed){var r=this.lastTotal-this.lastNeed;return e+this.lastChar.toString("utf16le",0,r)}return e}function d(t,e){var r=(t.length-e)%3;return 0===r?t.toString("base64",e):(this.lastNeed=3-r,this.lastTotal=3,1===r?this.lastChar[0]=t[t.length-1]:(this.lastChar[0]=t[t.length-2],this.lastChar[1]=t[t.length-1]),t.toString("base64",e,t.length-r))}function g(t){var e=t&&t.length?this.write(t):"";return this.lastNeed?e+this.lastChar.toString("base64",0,3-this.lastNeed):e}function v(t){return t.toString(this.encoding)}function b(t){return t&&t.length?this.write(t):""}var y=r(15).Buffer,m=y.isEncoding||function(t){switch((t=""+t)&&t.toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":case"raw":return !0;default:return !1}};e.StringDecoder=o,o.prototype.write=function(t){if(0===t.length)return "";var e,r;if(this.lastNeed){if(void 0===(e=this.fillLast(t)))return "";r=this.lastNeed,this.lastNeed=0;}else r=0;return r<t.length?e?e+this.text(t,r):this.text(t,r):e||""},o.prototype.end=f,o.prototype.text=l,o.prototype.fillLast=function(t){if(this.lastNeed<=t.length)return t.copy(this.lastChar,this.lastTotal-this.lastNeed,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal);t.copy(this.lastChar,this.lastTotal-this.lastNeed,0,t.length),this.lastNeed-=t.length;};},function(t,e,r){function n(t,e){if(t){var r,n="";for(var i in t)r=t[i],n&&(n+=" "),!r&&f[i]?n+=i:n+=i+'="'+(e.decodeEntities?l.encodeXML(r):r)+'"';return n}}function i(t,e){"svg"===t.name&&(e={decodeEntities:e.decodeEntities,xmlMode:!0});var r="<"+t.name,i=n(t.attribs,e);return i&&(r+=" "+i),!e.xmlMode||t.children&&0!==t.children.length?(r+=">",t.children&&(r+=d(t.children,e)),p[t.name]&&!e.xmlMode||(r+="</"+t.name+">")):r+="/>",r}function o(t){return "<"+t.data+">"}function a(t,e){var r=t.data||"";return !e.decodeEntities||t.parent&&t.parent.name in h||(r=l.encodeXML(r)),r}function s(t){return "<![CDATA["+t.children[0].data+"]]>"}function u(t){return "\x3c!--"+t.data+"--\x3e"}var c=r(77),l=r(78),f={__proto__:null,allowfullscreen:!0,async:!0,autofocus:!0,autoplay:!0,checked:!0,controls:!0,default:!0,defer:!0,disabled:!0,hidden:!0,ismap:!0,loop:!0,multiple:!0,muted:!0,open:!0,readonly:!0,required:!0,reversed:!0,scoped:!0,seamless:!0,selected:!0,typemustmatch:!0},h={__proto__:null,style:!0,script:!0,xmp:!0,iframe:!0,noembed:!0,noframes:!0,plaintext:!0,noscript:!0},p={__proto__:null,area:!0,base:!0,basefont:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,isindex:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0},d=t.exports=function(t,e){Array.isArray(t)||t.cheerio||(t=[t]),e=e||{};for(var r="",n=0;n<t.length;n++){var l=t[n];"root"===l.type?r+=d(l.children,e):c.isTag(l)?r+=i(l,e):l.type===c.Directive?r+=o(l):l.type===c.Comment?r+=u(l):l.type===c.CDATA?r+=s(l):r+=a(l,e);}return r};},function(t,e,r){(function(e){function r(t,e,r){switch(r.length){case 0:return t.call(e);case 1:return t.call(e,r[0]);case 2:return t.call(e,r[0],r[1]);case 3:return t.call(e,r[0],r[1],r[2])}return t.apply(e,r)}function n(t,e){for(var r=-1,n=t?t.length:0;++r<n&&!1!==e(t[r],r,t););return t}function i(t,e){return !!(t?t.length:0)&&a(t,e,0)>-1}function o(t,e,r,n){for(var i=t.length,o=r+(n?1:-1);n?o--:++o<i;)if(e(t[o],o,t))return o;return -1}function a(t,e,r){if(e!==e)return o(t,s,r);for(var n=r-1,i=t.length;++n<i;)if(t[n]===e)return n;return -1}function s(t){return t!==t}function u(t,e){for(var r=t.length,n=0;r--;)t[r]===e&&n++;return n}function c(t,e){return null==t?void 0:t[e]}function l(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"");}catch(t){}return e}function f(t,e){for(var r=-1,n=t.length,i=0,o=[];++r<n;){var a=t[r];a!==e&&a!==V||(t[r]=V,o[i++]=r);}return o}function h(t){return q(t)?Ot(t):{}}function p(t){return !(!q(t)||O(t))&&(D(t)||l(t)?Tt:dt).test(C(t))}function d(t,e,r,n){for(var i=-1,o=t.length,a=r.length,s=-1,u=e.length,c=Lt(o-a,0),l=Array(u+c),f=!n;++s<u;)l[s]=e[s];for(;++i<a;)(f||i<o)&&(l[r[i]]=t[i]);for(;c--;)l[s++]=t[i++];return l}function g(t,e,r,n){for(var i=-1,o=t.length,a=-1,s=r.length,u=-1,c=e.length,l=Lt(o-s,0),f=Array(l+c),h=!n;++i<l;)f[i]=t[i];for(var p=i;++u<c;)f[p+u]=e[u];for(;++a<s;)(h||i<o)&&(f[p+r[a]]=t[i++]);return f}function v(t,e){var r=-1,n=t.length;for(e||(e=Array(n));++r<n;)e[r]=t[r];return e}function b(t,e,r){function n(){return (this&&this!==_t&&this instanceof n?o:t).apply(i?r:this,arguments)}var i=e&$,o=y(t);return n}function y(t){return function(){var e=arguments;switch(e.length){case 0:return new t;case 1:return new t(e[0]);case 2:return new t(e[0],e[1]);case 3:return new t(e[0],e[1],e[2]);case 4:return new t(e[0],e[1],e[2],e[3]);case 5:return new t(e[0],e[1],e[2],e[3],e[4]);case 6:return new t(e[0],e[1],e[2],e[3],e[4],e[5]);case 7:return new t(e[0],e[1],e[2],e[3],e[4],e[5],e[6])}var r=h(t.prototype),n=t.apply(r,e);return q(n)?n:r}}function m(t,e,n){function i(){for(var a=arguments.length,s=Array(a),u=a,c=j(i);u--;)s[u]=arguments[u];var l=a<3&&s[0]!==c&&s[a-1]!==c?[]:f(s,c);return (a-=l.length)<n?x(t,e,_,i.placeholder,void 0,s,l,void 0,void 0,n-a):r(this&&this!==_t&&this instanceof i?o:t,this,s)}var o=y(t);return i}function _(t,e,r,n,i,o,a,s,c,l){function h(){for(var k=arguments.length,E=Array(k),A=k;A--;)E[A]=arguments[A];if(m)var T=j(h),O=u(E,T);if(n&&(E=d(E,n,i,m)),o&&(E=g(E,o,a,m)),k-=O,m&&k<l){var C=f(E,T);return x(t,e,_,h.placeholder,r,E,C,s,c,l-k)}var B=v?r:this,D=b?B[t]:t;return k=E.length,s?E=L(E,s):w&&k>1&&E.reverse(),p&&c<k&&(E.length=c),this&&this!==_t&&this instanceof h&&(D=S||y(D)),D.apply(B,E)}var p=e&X,v=e&$,b=e&H,m=e&(G|Y),w=e&Z,S=b?void 0:y(t);return h}function w(t,e,n,i){function o(){for(var e=-1,u=arguments.length,c=-1,l=i.length,f=Array(l+u),h=this&&this!==_t&&this instanceof o?s:t;++c<l;)f[c]=i[c];for(;u--;)f[c++]=arguments[++e];return r(h,a?n:this,f)}var a=e&$,s=y(t);return o}function x(t,e,r,n,i,o,a,s,u,c){var l=e&G,f=l?a:void 0,h=l?void 0:a,p=l?o:void 0,d=l?void 0:o;e|=l?J:Q,(e&=~(l?Q:J))&W||(e&=~($|H));var g=r(t,e,i,p,f,d,h,s,u,c);return g.placeholder=n,Dt(g,t,e)}function S(t,e,r,n,i,o,a,s){var u=e&H;if(!u&&"function"!=typeof t)throw new TypeError(z);var c=n?n.length:0;if(c||(e&=~(J|Q),n=i=void 0),a=void 0===a?a:Lt(I(a),0),s=void 0===s?s:I(s),c-=i?i.length:0,e&Q){var l=n,f=i;n=i=void 0;}var h=[t,e,r,n,i,l,f,o,a,s];if(t=h[0],e=h[1],r=h[2],n=h[3],i=h[4],s=h[9]=null==h[9]?u?0:t.length:Lt(h[9]-c,0),!s&&e&(G|Y)&&(e&=~(G|Y)),e&&e!=$)p=e==G||e==Y?m(t,e,s):e!=J&&e!=($|J)||i.length?_.apply(void 0,h):w(t,e,r,n);else var p=b(t,e,r);return Dt(p,t,e)}function j(t){return t.placeholder}function k(t,e){var r=c(t,e);return p(r)?r:void 0}function E(t){var e=t.match(lt);return e?e[1].split(ft):[]}function A(t,e){var r=e.length,n=r-1;return e[n]=(r>1?"& ":"")+e[n],e=e.join(r>2?", ":" "),t.replace(ct,"{\n/* [wrapped with "+e+"] */\n")}function T(t,e){return !!(e=null==e?tt:e)&&("number"==typeof t||vt.test(t))&&t>-1&&t%1==0&&t<e}function O(t){return !!jt&&jt in t}function L(t,e){for(var r=t.length,n=Ct(e.length,r),i=v(t);n--;){var o=e[n];t[n]=T(o,r)?i[o]:void 0;}return t}function C(t){if(null!=t){try{return kt.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function B(t,e){return n(nt,function(r){var n="_."+r[0];e&r[1]&&!i(t,n)&&t.push(n);}),t.sort()}function D(t){var e=q(t)?At.call(t):"";return e==it||e==ot}function q(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function P(t){return !!t&&"object"==typeof t}function M(t){return "symbol"==typeof t||P(t)&&At.call(t)==at}function R(t){if(!t)return 0===t?t:0;if((t=N(t))===K||t===-K){return (t<0?-1:1)*et}return t===t?t:0}function I(t){var e=R(t),r=e%1;return e===e?r?e-r:e:0}function N(t){if("number"==typeof t)return t;if(M(t))return rt;if(q(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=q(e)?e+"":e;}if("string"!=typeof t)return 0===t?t:+t;t=t.replace(ut,"");var r=pt.test(t);return r||gt.test(t)?bt(t.slice(2),r?2:8):ht.test(t)?rt:+t}function U(t){return function(){return t}}function F(t){return t}var z="Expected a function",V="__lodash_placeholder__",$=1,H=2,W=4,G=8,Y=16,J=32,Q=64,X=128,Z=512,K=1/0,tt=9007199254740991,et=1.7976931348623157e308,rt=NaN,nt=[["ary",X],["bind",$],["bindKey",H],["curry",G],["curryRight",Y],["flip",Z],["partial",J],["partialRight",Q],["rearg",256]],it="[object Function]",ot="[object GeneratorFunction]",at="[object Symbol]",st=/[\\^$.*+?()[\]{}|]/g,ut=/^\s+|\s+$/g,ct=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,lt=/\{\n\/\* \[wrapped with (.+)\] \*/,ft=/,? & /,ht=/^[-+]0x[0-9a-f]+$/i,pt=/^0b[01]+$/i,dt=/^\[object .+?Constructor\]$/,gt=/^0o[0-7]+$/i,vt=/^(?:0|[1-9]\d*)$/,bt=parseInt,yt="object"==typeof e&&e&&e.Object===Object&&e,mt="object"==typeof self&&self&&self.Object===Object&&self,_t=yt||mt||Function("return this")(),wt=Function.prototype,xt=Object.prototype,St=_t["__core-js_shared__"],jt=function(){var t=/[^.]+$/.exec(St&&St.keys&&St.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),kt=wt.toString,Et=xt.hasOwnProperty,At=xt.toString,Tt=RegExp("^"+kt.call(Et).replace(st,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Ot=Object.create,Lt=Math.max,Ct=Math.min,Bt=function(){var t=k(Object,"defineProperty"),e=k.name;return e&&e.length>2?t:void 0}(),Dt=Bt?function(t,e,r){var n=e+"";return Bt(t,"toString",{configurable:!0,enumerable:!1,value:U(A(n,B(E(n),r)))})}:F,qt=function(t,e){return e=Lt(void 0===e?t.length-1:e,0),function(){for(var n=arguments,i=-1,o=Lt(n.length-e,0),a=Array(o);++i<o;)a[i]=n[e+i];i=-1;for(var s=Array(e+1);++i<e;)s[i]=n[i];return s[e]=a,r(t,this,s)}}(function(t,e,r){var n=$;if(r.length){var i=f(r,j(qt));n|=J;}return S(t,n,e,r,i)});qt.placeholder={},t.exports=qt;}).call(e,r(0));},function(t,e,r){function n(t,e,r){if(e)"string"==typeof e&&(e=o(e,t._root,r));else {if(!t._root||!t._root.children)return "";e=t._root.children;}return i(e,r)}var i=r(23),o=r(42),a=r(11),s={merge:r(95),defaults:r(41)};e.load=function(t,n){var i=r(17);n=s.defaults(n||{},i.prototype.options);var o=a(t,n),u=function(t,e,r,a){return this instanceof u?(a=s.defaults(a||{},n),i.call(this,t,e,r||o,a)):new u(t,e,r,a)};return u.prototype=Object.create(i.prototype),u.prototype.constructor=u,u.fn=u.prototype,u.prototype._originalRoot=o,s.merge(u,e),u._root=o,u._options=n,u},e.html=function(t,e){var i=r(17);return "[object Object]"!==Object.prototype.toString.call(t)||e||"length"in t||"type"in t||(e=t,t=void 0),e=s.defaults(e||{},this._options,i.prototype.options),n(this,t,e)},e.xml=function(t){return n(this,t,s.defaults({xmlMode:!0},this._options))},e.text=function(t){t||(t=this.root());for(var r,n="",i=t.length,o=0;o<i;o++)r=t[o],"text"===r.type?n+=r.data:r.children&&"comment"!==r.type&&(n+=e.text(r.children));return n},e.parseHTML=function(t,e,r){var n;return t&&"string"==typeof t?("boolean"==typeof e&&(r=e),n=this.load(t),r||n("script").remove(),n.root()[0].children.slice()):null},e.root=function(){return this(this._root)},e.contains=function(t,e){if(e===t)return !1;for(;e&&e!==e.parent;)if((e=e.parent)===t)return !0;return !1};},function(t,e,r){function n(t){for(var e=0;t&&e<t.length;e++)if(u(t[e]))return t[e]}function i(t,e){var r={name:t,value:e};return function(t){return b(t,r)}}function o(t){return function(e){return !!l(e)&&t(e)}}function a(t,e,r){if(null===r){if(t.length>1&&"scope"!==e)throw new SyntaxError("pseudo-selector :"+e+" requires an argument")}else if(1===t.length)throw new SyntaxError("pseudo-selector :"+e+" doesn't have any arguments")}var s=r(4),u=s.isTag,c=s.getText,l=s.getParent,f=s.getChildren,h=s.getSiblings,p=s.hasAttrib,d=s.getName,g=s.getAttributeValue,v=r(88),b=r(43).rules.equals,y=r(10),m=y.trueFunc,_=y.falseFunc,w={contains:function(t,e){return function(r){return t(r)&&c(r).indexOf(e)>=0}},icontains:function(t,e){var r=e.toLowerCase();return function(e){return t(e)&&c(e).toLowerCase().indexOf(r)>=0}},"nth-child":function(t,e){var r=v(e);return r===_?r:r===m?o(t):function(e){for(var n=h(e),i=0,o=0;i<n.length;i++)if(u(n[i])){if(n[i]===e)break;o++;}return r(o)&&t(e)}},"nth-last-child":function(t,e){var r=v(e);return r===_?r:r===m?o(t):function(e){for(var n=h(e),i=0,o=n.length-1;o>=0;o--)if(u(n[o])){if(n[o]===e)break;i++;}return r(i)&&t(e)}},"nth-of-type":function(t,e){var r=v(e);return r===_?r:r===m?o(t):function(e){for(var n=h(e),i=0,o=0;o<n.length;o++)if(u(n[o])){if(n[o]===e)break;d(n[o])===d(e)&&i++;}return r(i)&&t(e)}},"nth-last-of-type":function(t,e){var r=v(e);return r===_?r:r===m?o(t):function(e){for(var n=h(e),i=0,o=n.length-1;o>=0;o--)if(u(n[o])){if(n[o]===e)break;d(n[o])===d(e)&&i++;}return r(i)&&t(e)}},root:function(t){return function(e){return !l(e)&&t(e)}},scope:function(t,e,r,n){return n&&0!==n.length?1===n.length?function(e){return n[0]===e&&t(e)}:function(e){return n.indexOf(e)>=0&&t(e)}:w.root(t)},checkbox:i("type","checkbox"),file:i("type","file"),password:i("type","password"),radio:i("type","radio"),reset:i("type","reset"),image:i("type","image"),submit:i("type","submit")},x={empty:function(t){return !f(t).some(function(t){return u(t)||"text"===t.type})},"first-child":function(t){return n(h(t))===t},"last-child":function(t){for(var e=h(t),r=e.length-1;r>=0;r--){if(e[r]===t)return !0;if(u(e[r]))break}return !1},"first-of-type":function(t){for(var e=h(t),r=0;r<e.length;r++)if(u(e[r])){if(e[r]===t)return !0;if(d(e[r])===d(t))break}return !1},"last-of-type":function(t){for(var e=h(t),r=e.length-1;r>=0;r--)if(u(e[r])){if(e[r]===t)return !0;if(d(e[r])===d(t))break}return !1},"only-of-type":function(t){for(var e=h(t),r=0,n=e.length;r<n;r++)if(u(e[r])){if(e[r]===t)continue;if(d(e[r])===d(t))return !1}return !0},"only-child":function(t){for(var e=h(t),r=0;r<e.length;r++)if(u(e[r])&&e[r]!==t)return !1;return !0},link:function(t){return p(t,"href")},visited:_,selected:function(t){if(p(t,"selected"))return !0;if("option"!==d(t))return !1;var e=l(t);if(!e||"select"!==d(e)||p(e,"multiple"))return !1;for(var r=f(e),n=!1,i=0;i<r.length;i++)if(u(r[i]))if(r[i]===t)n=!0;else {if(!n)return !1;if(p(r[i],"selected"))return !1}return n},disabled:function(t){return p(t,"disabled")},enabled:function(t){return !p(t,"disabled")},checked:function(t){return p(t,"checked")||x.selected(t)},required:function(t){return p(t,"required")},optional:function(t){return !p(t,"required")},parent:function(t){return !x.empty(t)},header:function(t){var e=d(t);return "h1"===e||"h2"===e||"h3"===e||"h4"===e||"h5"===e||"h6"===e},button:function(t){var e=d(t);return "button"===e||"input"===e&&"button"===g(t,"type")},input:function(t){var e=d(t);return "input"===e||"textarea"===e||"select"===e||"button"===e},text:function(t){var e;return "input"===d(t)&&(!(e=g(t,"type"))||"text"===e.toLowerCase())}},S=/^(?:(?:nth|last|first|only)-(?:child|of-type)|root|empty|(?:en|dis)abled|checked|not)$/;t.exports={compile:function(t,e,r,n){var i=e.name,o=e.data;if(r&&r.strict&&!S.test(i))throw SyntaxError(":"+i+" isn't part of CSS3");if("function"==typeof w[i])return a(w[i],i,o),w[i](t,o,r,n);if("function"==typeof x[i]){var s=x[i];return a(s,i,o),t===m?s:function(e){return s(e,o)&&t(e)}}throw new SyntaxError("unmatched pseudo-class :"+i)},filters:w,pseudos:x};},function(t,e,r){var n=r(108),i={}.hasOwnProperty,o=r(111),a=r(113);e.Selector=o,e.Property=a,/**
 * Returns an array of the selectors.
 *
 * @license Sizzle CSS Selector Engine - MIT
 * @param {String} selectorText from mensch
 * @api public
 */
e.extract=function(t){for(var e=0,r=[],n="",i=0,o=t.length;i<o;i++){var a=t.charAt(i);e?("]"!==a&&")"!==a||e--,n+=a):","===a?(r.push(n),n=""):("["!==a&&"("!==a||e++,(n.length||","!==a&&"\n"!==a&&" "!==a)&&(n+=a));}return n.length&&r.push(n),r},e.parseCSS=function(t){for(var e=n.parse(t,{position:!0,comments:!0}),r=void 0!==e.stylesheet&&e.stylesheet.rules?e.stylesheet.rules:[],i=[],o=0,a=r.length;o<a;o++)if("rule"==r[o].type)for(var s=r[o],u=s.selectors,c=0,l=u.length;c<l;c++)i.push([u[c],s.declarations]);return i},e.getPreservedText=function(t,e){for(var r=n.parse(t,{position:!0,comments:!0}),i=void 0!==r.stylesheet&&r.stylesheet.rules?r.stylesheet.rules:[],o=[],a=i.length-1;a>=0;a--)(e.fontFaces&&"font-face"===i[a].type||e.mediaQueries&&"media"===i[a].type)&&o.unshift(n.stringify({stylesheet:{rules:[i[a]]}},{comments:!1,indentation:"  "})),i[a].position.start;return 0!==o.length&&"\n"+o.join("\n")+"\n"},e.normalizeLineEndings=function(t){return t.replace(/\r\n/g,"\n").replace(/\n/g,"\r\n")},e.compareFunc=function(t,e){for(var r=Math.min(t.length,e.length),n=0;n<r;n++)if(t[n]!==e[n])return t[n]>e[n]?1:-1;return t.length-e.length},e.compare=function(t,r){return 1==e.compareFunc(t,r)?t:r},e.extend=function(t,e){for(var r in e)i.call(e,r)&&(t[r]=e[r]);return t},e.getDefaultOptions=function(t){var r=e.extend({extraCss:"",insertPreservedExtraCss:!0,applyStyleTags:!0,removeStyleTags:!0,preserveMediaQueries:!0,preserveFontFaces:!0,applyWidthAttributes:!0,applyHeightAttributes:!0,applyAttributesTableElements:!0,url:""},t);return r.webResources=r.webResources||{},r};},function(t,e,r){(function(r){function n(t){return i.bind(null,t)}function i(t){var e=[].slice.call(arguments,1);e.unshift("["+t+"]"),r.stderr.write(e.join(" ")+"\n");}e=t.exports=n;}).call(e,r(7));},function(t,e){var r={}.toString;t.exports=Array.isArray||function(t){return "[object Array]"==r.call(t)};},function(t,e,r){function n(t,e){this._options=e||{},this._cbs=t||{},this._tagname="",this._attribname="",this._attribvalue="",this._attribs=null,this._stack=[],this.startIndex=0,this.endIndex=null,this._lowerCaseTagNames="lowerCaseTags"in this._options?!!this._options.lowerCaseTags:!this._options.xmlMode,this._lowerCaseAttributeNames="lowerCaseAttributeNames"in this._options?!!this._options.lowerCaseAttributeNames:!this._options.xmlMode,this._options.Tokenizer&&(i=this._options.Tokenizer),this._tokenizer=new i(this._options,this),this._cbs.onparserinit&&this._cbs.onparserinit(this);}var i=r(31),o={input:!0,option:!0,optgroup:!0,select:!0,button:!0,datalist:!0,textarea:!0},a={tr:{tr:!0,th:!0,td:!0},th:{th:!0},td:{thead:!0,th:!0,td:!0},body:{head:!0,link:!0,script:!0},li:{li:!0},p:{p:!0},h1:{p:!0},h2:{p:!0},h3:{p:!0},h4:{p:!0},h5:{p:!0},h6:{p:!0},select:o,input:o,output:o,button:o,datalist:o,textarea:o,option:{option:!0},optgroup:{optgroup:!0}},s={__proto__:null,area:!0,base:!0,basefont:!0,br:!0,col:!0,command:!0,embed:!0,frame:!0,hr:!0,img:!0,input:!0,isindex:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0,path:!0,circle:!0,ellipse:!0,line:!0,rect:!0,use:!0,stop:!0,polyline:!0,polygon:!0},u=/\s|\//;r(1)(n,r(13).EventEmitter),n.prototype._updatePosition=function(t){null===this.endIndex?this._tokenizer._sectionStart<=t?this.startIndex=0:this.startIndex=this._tokenizer._sectionStart-t:this.startIndex=this.endIndex+1,this.endIndex=this._tokenizer.getAbsoluteIndex();},n.prototype.ontext=function(t){this._updatePosition(1),this.endIndex--,this._cbs.ontext&&this._cbs.ontext(t);},n.prototype.onopentagname=function(t){if(this._lowerCaseTagNames&&(t=t.toLowerCase()),this._tagname=t,!this._options.xmlMode&&t in a)for(var e;(e=this._stack[this._stack.length-1])in a[t];this.onclosetag(e));!this._options.xmlMode&&t in s||this._stack.push(t),this._cbs.onopentagname&&this._cbs.onopentagname(t),this._cbs.onopentag&&(this._attribs={});},n.prototype.onopentagend=function(){this._updatePosition(1),this._attribs&&(this._cbs.onopentag&&this._cbs.onopentag(this._tagname,this._attribs),this._attribs=null),!this._options.xmlMode&&this._cbs.onclosetag&&this._tagname in s&&this._cbs.onclosetag(this._tagname),this._tagname="";},n.prototype.onclosetag=function(t){if(this._updatePosition(1),this._lowerCaseTagNames&&(t=t.toLowerCase()),!this._stack.length||t in s&&!this._options.xmlMode)this._options.xmlMode||"br"!==t&&"p"!==t||(this.onopentagname(t),this._closeCurrentTag());else {var e=this._stack.lastIndexOf(t);if(-1!==e)if(this._cbs.onclosetag)for(e=this._stack.length-e;e--;)this._cbs.onclosetag(this._stack.pop());else this._stack.length=e;else "p"!==t||this._options.xmlMode||(this.onopentagname(t),this._closeCurrentTag());}},n.prototype.onselfclosingtag=function(){this._options.xmlMode||this._options.recognizeSelfClosing?this._closeCurrentTag():this.onopentagend();},n.prototype._closeCurrentTag=function(){var t=this._tagname;this.onopentagend(),this._stack[this._stack.length-1]===t&&(this._cbs.onclosetag&&this._cbs.onclosetag(t),this._stack.pop());},n.prototype.onattribname=function(t){this._lowerCaseAttributeNames&&(t=t.toLowerCase()),this._attribname=t;},n.prototype.onattribdata=function(t){this._attribvalue+=t;},n.prototype.onattribend=function(){this._cbs.onattribute&&this._cbs.onattribute(this._attribname,this._attribvalue),this._attribs&&!Object.prototype.hasOwnProperty.call(this._attribs,this._attribname)&&(this._attribs[this._attribname]=this._attribvalue),this._attribname="",this._attribvalue="";},n.prototype._getInstructionName=function(t){var e=t.search(u),r=e<0?t:t.substr(0,e);return this._lowerCaseTagNames&&(r=r.toLowerCase()),r},n.prototype.ondeclaration=function(t){if(this._cbs.onprocessinginstruction){var e=this._getInstructionName(t);this._cbs.onprocessinginstruction("!"+e,"!"+t);}},n.prototype.onprocessinginstruction=function(t){if(this._cbs.onprocessinginstruction){var e=this._getInstructionName(t);this._cbs.onprocessinginstruction("?"+e,"?"+t);}},n.prototype.oncomment=function(t){this._updatePosition(4),this._cbs.oncomment&&this._cbs.oncomment(t),this._cbs.oncommentend&&this._cbs.oncommentend();},n.prototype.oncdata=function(t){this._updatePosition(1),this._options.xmlMode||this._options.recognizeCDATA?(this._cbs.oncdatastart&&this._cbs.oncdatastart(),this._cbs.ontext&&this._cbs.ontext(t),this._cbs.oncdataend&&this._cbs.oncdataend()):this.oncomment("[CDATA["+t+"]]");},n.prototype.onerror=function(t){this._cbs.onerror&&this._cbs.onerror(t);},n.prototype.onend=function(){if(this._cbs.onclosetag)for(var t=this._stack.length;t>0;this._cbs.onclosetag(this._stack[--t]));this._cbs.onend&&this._cbs.onend();},n.prototype.reset=function(){this._cbs.onreset&&this._cbs.onreset(),this._tokenizer.reset(),this._tagname="",this._attribname="",this._attribs=null,this._stack=[],this._cbs.onparserinit&&this._cbs.onparserinit(this);},n.prototype.parseComplete=function(t){this.reset(),this.end(t);},n.prototype.write=function(t){this._tokenizer.write(t);},n.prototype.end=function(t){this._tokenizer.end(t);},n.prototype.pause=function(){this._tokenizer.pause();},n.prototype.resume=function(){this._tokenizer.resume();},n.prototype.parseChunk=n.prototype.write,n.prototype.done=n.prototype.end,t.exports=n;},function(t,e,r){function n(t){return " "===t||"\n"===t||"\t"===t||"\f"===t||"\r"===t}function i(t,e,r){var n=t.toLowerCase();return t===n?function(t){t===n?this._state=e:(this._state=r,this._index--);}:function(i){i===n||i===t?this._state=e:(this._state=r,this._index--);}}function o(t,e){var r=t.toLowerCase();return function(n){n===r||n===t?this._state=e:(this._state=d,this._index--);}}function a(t,e){this._state=h,this._buffer="",this._sectionStart=0,this._index=0,this._bufferOffset=0,this._baseState=h,this._special=dt,this._cbs=e,this._running=!0,this._ended=!1,this._xmlMode=!(!t||!t.xmlMode),this._decodeEntities=!(!t||!t.decodeEntities);}t.exports=a;var s=r(32),u=r(18),c=r(33),l=r(19),f=0,h=f++,p=f++,d=f++,g=f++,v=f++,b=f++,y=f++,m=f++,_=f++,w=f++,x=f++,S=f++,j=f++,k=f++,E=f++,A=f++,T=f++,O=f++,L=f++,C=f++,B=f++,D=f++,q=f++,P=f++,M=f++,R=f++,I=f++,N=f++,U=f++,F=f++,z=f++,V=f++,$=f++,H=f++,W=f++,G=f++,Y=f++,J=f++,Q=f++,X=f++,Z=f++,K=f++,tt=f++,et=f++,rt=f++,nt=f++,it=f++,ot=f++,at=f++,st=f++,ut=f++,ct=f++,lt=f++,ft=f++,ht=f++,pt=0,dt=pt++,gt=pt++,vt=pt++;a.prototype._stateText=function(t){"<"===t?(this._index>this._sectionStart&&this._cbs.ontext(this._getSection()),this._state=p,this._sectionStart=this._index):this._decodeEntities&&this._special===dt&&"&"===t&&(this._index>this._sectionStart&&this._cbs.ontext(this._getSection()),this._baseState=h,this._state=ut,this._sectionStart=this._index);},a.prototype._stateBeforeTagName=function(t){"/"===t?this._state=v:"<"===t?(this._cbs.ontext(this._getSection()),this._sectionStart=this._index):">"===t||this._special!==dt||n(t)?this._state=h:"!"===t?(this._state=E,this._sectionStart=this._index+1):"?"===t?(this._state=T,this._sectionStart=this._index+1):(this._state=this._xmlMode||"s"!==t&&"S"!==t?d:z,this._sectionStart=this._index);},a.prototype._stateInTagName=function(t){("/"===t||">"===t||n(t))&&(this._emitToken("onopentagname"),this._state=m,this._index--);},a.prototype._stateBeforeCloseingTagName=function(t){n(t)||(">"===t?this._state=h:this._special!==dt?"s"===t||"S"===t?this._state=V:(this._state=h,this._index--):(this._state=b,this._sectionStart=this._index));},a.prototype._stateInCloseingTagName=function(t){(">"===t||n(t))&&(this._emitToken("onclosetag"),this._state=y,this._index--);},a.prototype._stateAfterCloseingTagName=function(t){">"===t&&(this._state=h,this._sectionStart=this._index+1);},a.prototype._stateBeforeAttributeName=function(t){">"===t?(this._cbs.onopentagend(),this._state=h,this._sectionStart=this._index+1):"/"===t?this._state=g:n(t)||(this._state=_,this._sectionStart=this._index);},a.prototype._stateInSelfClosingTag=function(t){">"===t?(this._cbs.onselfclosingtag(),this._state=h,this._sectionStart=this._index+1):n(t)||(this._state=m,this._index--);},a.prototype._stateInAttributeName=function(t){("="===t||"/"===t||">"===t||n(t))&&(this._cbs.onattribname(this._getSection()),this._sectionStart=-1,this._state=w,this._index--);},a.prototype._stateAfterAttributeName=function(t){"="===t?this._state=x:"/"===t||">"===t?(this._cbs.onattribend(),this._state=m,this._index--):n(t)||(this._cbs.onattribend(),this._state=_,this._sectionStart=this._index);},a.prototype._stateBeforeAttributeValue=function(t){'"'===t?(this._state=S,this._sectionStart=this._index+1):"'"===t?(this._state=j,this._sectionStart=this._index+1):n(t)||(this._state=k,this._sectionStart=this._index,this._index--);},a.prototype._stateInAttributeValueDoubleQuotes=function(t){'"'===t?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=m):this._decodeEntities&&"&"===t&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=ut,this._sectionStart=this._index);},a.prototype._stateInAttributeValueSingleQuotes=function(t){"'"===t?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=m):this._decodeEntities&&"&"===t&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=ut,this._sectionStart=this._index);},a.prototype._stateInAttributeValueNoQuotes=function(t){n(t)||">"===t?(this._emitToken("onattribdata"),this._cbs.onattribend(),this._state=m,this._index--):this._decodeEntities&&"&"===t&&(this._emitToken("onattribdata"),this._baseState=this._state,this._state=ut,this._sectionStart=this._index);},a.prototype._stateBeforeDeclaration=function(t){this._state="["===t?D:"-"===t?O:A;},a.prototype._stateInDeclaration=function(t){">"===t&&(this._cbs.ondeclaration(this._getSection()),this._state=h,this._sectionStart=this._index+1);},a.prototype._stateInProcessingInstruction=function(t){">"===t&&(this._cbs.onprocessinginstruction(this._getSection()),this._state=h,this._sectionStart=this._index+1);},a.prototype._stateBeforeComment=function(t){"-"===t?(this._state=L,this._sectionStart=this._index+1):this._state=A;},a.prototype._stateInComment=function(t){"-"===t&&(this._state=C);},a.prototype._stateAfterComment1=function(t){this._state="-"===t?B:L;},a.prototype._stateAfterComment2=function(t){">"===t?(this._cbs.oncomment(this._buffer.substring(this._sectionStart,this._index-2)),this._state=h,this._sectionStart=this._index+1):"-"!==t&&(this._state=L);},a.prototype._stateBeforeCdata1=i("C",q,A),a.prototype._stateBeforeCdata2=i("D",P,A),a.prototype._stateBeforeCdata3=i("A",M,A),a.prototype._stateBeforeCdata4=i("T",R,A),a.prototype._stateBeforeCdata5=i("A",I,A),a.prototype._stateBeforeCdata6=function(t){"["===t?(this._state=N,this._sectionStart=this._index+1):(this._state=A,this._index--);},a.prototype._stateInCdata=function(t){"]"===t&&(this._state=U);},a.prototype._stateAfterCdata1=function(t,e){return function(r){r===t&&(this._state=e);}}("]",F),a.prototype._stateAfterCdata2=function(t){">"===t?(this._cbs.oncdata(this._buffer.substring(this._sectionStart,this._index-2)),this._state=h,this._sectionStart=this._index+1):"]"!==t&&(this._state=N);},a.prototype._stateBeforeSpecial=function(t){"c"===t||"C"===t?this._state=$:"t"===t||"T"===t?this._state=tt:(this._state=d,this._index--);},a.prototype._stateBeforeSpecialEnd=function(t){this._special!==gt||"c"!==t&&"C"!==t?this._special!==vt||"t"!==t&&"T"!==t?this._state=h:this._state=it:this._state=J;},a.prototype._stateBeforeScript1=o("R",H),a.prototype._stateBeforeScript2=o("I",W),a.prototype._stateBeforeScript3=o("P",G),a.prototype._stateBeforeScript4=o("T",Y),a.prototype._stateBeforeScript5=function(t){("/"===t||">"===t||n(t))&&(this._special=gt),this._state=d,this._index--;},a.prototype._stateAfterScript1=i("R",Q,h),a.prototype._stateAfterScript2=i("I",X,h),a.prototype._stateAfterScript3=i("P",Z,h),a.prototype._stateAfterScript4=i("T",K,h),a.prototype._stateAfterScript5=function(t){">"===t||n(t)?(this._special=dt,this._state=b,this._sectionStart=this._index-6,this._index--):this._state=h;},a.prototype._stateBeforeStyle1=o("Y",et),a.prototype._stateBeforeStyle2=o("L",rt),a.prototype._stateBeforeStyle3=o("E",nt),a.prototype._stateBeforeStyle4=function(t){("/"===t||">"===t||n(t))&&(this._special=vt),this._state=d,this._index--;},a.prototype._stateAfterStyle1=i("Y",ot,h),a.prototype._stateAfterStyle2=i("L",at,h),a.prototype._stateAfterStyle3=i("E",st,h),a.prototype._stateAfterStyle4=function(t){">"===t||n(t)?(this._special=dt,this._state=b,this._sectionStart=this._index-5,this._index--):this._state=h;},a.prototype._stateBeforeEntity=i("#",ct,lt),a.prototype._stateBeforeNumericEntity=i("X",ht,ft),a.prototype._parseNamedEntityStrict=function(){if(this._sectionStart+1<this._index){var t=this._buffer.substring(this._sectionStart+1,this._index),e=this._xmlMode?l:u;e.hasOwnProperty(t)&&(this._emitPartial(e[t]),this._sectionStart=this._index+1);}},a.prototype._parseLegacyEntity=function(){var t=this._sectionStart+1,e=this._index-t;for(e>6&&(e=6);e>=2;){var r=this._buffer.substr(t,e);if(c.hasOwnProperty(r))return this._emitPartial(c[r]),void(this._sectionStart+=e+1);e--;}},a.prototype._stateInNamedEntity=function(t){";"===t?(this._parseNamedEntityStrict(),this._sectionStart+1<this._index&&!this._xmlMode&&this._parseLegacyEntity(),this._state=this._baseState):(t<"a"||t>"z")&&(t<"A"||t>"Z")&&(t<"0"||t>"9")&&(this._xmlMode||this._sectionStart+1===this._index||(this._baseState!==h?"="!==t&&this._parseNamedEntityStrict():this._parseLegacyEntity()),this._state=this._baseState,this._index--);},a.prototype._decodeNumericEntity=function(t,e){var r=this._sectionStart+t;if(r!==this._index){var n=this._buffer.substring(r,this._index),i=parseInt(n,e);this._emitPartial(s(i)),this._sectionStart=this._index;}else this._sectionStart--;this._state=this._baseState;},a.prototype._stateInNumericEntity=function(t){";"===t?(this._decodeNumericEntity(2,10),this._sectionStart++):(t<"0"||t>"9")&&(this._xmlMode?this._state=this._baseState:this._decodeNumericEntity(2,10),this._index--);},a.prototype._stateInHexEntity=function(t){";"===t?(this._decodeNumericEntity(3,16),this._sectionStart++):(t<"a"||t>"f")&&(t<"A"||t>"F")&&(t<"0"||t>"9")&&(this._xmlMode?this._state=this._baseState:this._decodeNumericEntity(3,16),this._index--);},a.prototype._cleanup=function(){this._sectionStart<0?(this._buffer="",this._bufferOffset+=this._index,this._index=0):this._running&&(this._state===h?(this._sectionStart!==this._index&&this._cbs.ontext(this._buffer.substr(this._sectionStart)),this._buffer="",this._bufferOffset+=this._index,this._index=0):this._sectionStart===this._index?(this._buffer="",this._bufferOffset+=this._index,this._index=0):(this._buffer=this._buffer.substr(this._sectionStart),this._index-=this._sectionStart,this._bufferOffset+=this._sectionStart),this._sectionStart=0);},a.prototype.write=function(t){this._ended&&this._cbs.onerror(Error(".write() after done!")),this._buffer+=t,this._parse();},a.prototype._parse=function(){for(;this._index<this._buffer.length&&this._running;){var t=this._buffer.charAt(this._index);this._state===h?this._stateText(t):this._state===p?this._stateBeforeTagName(t):this._state===d?this._stateInTagName(t):this._state===v?this._stateBeforeCloseingTagName(t):this._state===b?this._stateInCloseingTagName(t):this._state===y?this._stateAfterCloseingTagName(t):this._state===g?this._stateInSelfClosingTag(t):this._state===m?this._stateBeforeAttributeName(t):this._state===_?this._stateInAttributeName(t):this._state===w?this._stateAfterAttributeName(t):this._state===x?this._stateBeforeAttributeValue(t):this._state===S?this._stateInAttributeValueDoubleQuotes(t):this._state===j?this._stateInAttributeValueSingleQuotes(t):this._state===k?this._stateInAttributeValueNoQuotes(t):this._state===E?this._stateBeforeDeclaration(t):this._state===A?this._stateInDeclaration(t):this._state===T?this._stateInProcessingInstruction(t):this._state===O?this._stateBeforeComment(t):this._state===L?this._stateInComment(t):this._state===C?this._stateAfterComment1(t):this._state===B?this._stateAfterComment2(t):this._state===D?this._stateBeforeCdata1(t):this._state===q?this._stateBeforeCdata2(t):this._state===P?this._stateBeforeCdata3(t):this._state===M?this._stateBeforeCdata4(t):this._state===R?this._stateBeforeCdata5(t):this._state===I?this._stateBeforeCdata6(t):this._state===N?this._stateInCdata(t):this._state===U?this._stateAfterCdata1(t):this._state===F?this._stateAfterCdata2(t):this._state===z?this._stateBeforeSpecial(t):this._state===V?this._stateBeforeSpecialEnd(t):this._state===$?this._stateBeforeScript1(t):this._state===H?this._stateBeforeScript2(t):this._state===W?this._stateBeforeScript3(t):this._state===G?this._stateBeforeScript4(t):this._state===Y?this._stateBeforeScript5(t):this._state===J?this._stateAfterScript1(t):this._state===Q?this._stateAfterScript2(t):this._state===X?this._stateAfterScript3(t):this._state===Z?this._stateAfterScript4(t):this._state===K?this._stateAfterScript5(t):this._state===tt?this._stateBeforeStyle1(t):this._state===et?this._stateBeforeStyle2(t):this._state===rt?this._stateBeforeStyle3(t):this._state===nt?this._stateBeforeStyle4(t):this._state===it?this._stateAfterStyle1(t):this._state===ot?this._stateAfterStyle2(t):this._state===at?this._stateAfterStyle3(t):this._state===st?this._stateAfterStyle4(t):this._state===ut?this._stateBeforeEntity(t):this._state===ct?this._stateBeforeNumericEntity(t):this._state===lt?this._stateInNamedEntity(t):this._state===ft?this._stateInNumericEntity(t):this._state===ht?this._stateInHexEntity(t):this._cbs.onerror(Error("unknown _state"),this._state),this._index++;}this._cleanup();},a.prototype.pause=function(){this._running=!1;},a.prototype.resume=function(){this._running=!0,this._index<this._buffer.length&&this._parse(),this._ended&&this._finish();},a.prototype.end=function(t){this._ended&&this._cbs.onerror(Error(".end() after done!")),t&&this.write(t),this._ended=!0,this._running&&this._finish();},a.prototype._finish=function(){this._sectionStart<this._index&&this._handleTrailingData(),this._cbs.onend();},a.prototype._handleTrailingData=function(){var t=this._buffer.substr(this._sectionStart);this._state===N||this._state===U||this._state===F?this._cbs.oncdata(t):this._state===L||this._state===C||this._state===B?this._cbs.oncomment(t):this._state!==lt||this._xmlMode?this._state!==ft||this._xmlMode?this._state!==ht||this._xmlMode?this._state!==d&&this._state!==m&&this._state!==x&&this._state!==w&&this._state!==_&&this._state!==j&&this._state!==S&&this._state!==k&&this._state!==b&&this._cbs.ontext(t):(this._decodeNumericEntity(3,16),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData())):(this._decodeNumericEntity(2,10),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData())):(this._parseLegacyEntity(),this._sectionStart<this._index&&(this._state=this._baseState,this._handleTrailingData()));},a.prototype.reset=function(){a.call(this,{xmlMode:this._xmlMode,decodeEntities:this._decodeEntities},this._cbs);},a.prototype.getAbsoluteIndex=function(){return this._bufferOffset+this._index},a.prototype._getSection=function(){return this._buffer.substring(this._sectionStart,this._index)},a.prototype._emitToken=function(t){this._cbs[t](this._getSection()),this._sectionStart=-1;},a.prototype._emitPartial=function(t){this._baseState!==h?this._cbs.onattribdata(t):this._cbs.ontext(t);};},function(t,e,r){function n(t){if(t>=55296&&t<=57343||t>1114111)return "�";t in i&&(t=i[t]);var e="";return t>65535&&(t-=65536,e+=String.fromCharCode(t>>>10&1023|55296),t=56320|1023&t),e+=String.fromCharCode(t)}var i=r(57);t.exports=n;},function(t,e){t.exports={Aacute:"Á",aacute:"á",Acirc:"Â",acirc:"â",acute:"´",AElig:"Æ",aelig:"æ",Agrave:"À",agrave:"à",amp:"&",AMP:"&",Aring:"Å",aring:"å",Atilde:"Ã",atilde:"ã",Auml:"Ä",auml:"ä",brvbar:"¦",Ccedil:"Ç",ccedil:"ç",cedil:"¸",cent:"¢",copy:"©",COPY:"©",curren:"¤",deg:"°",divide:"÷",Eacute:"É",eacute:"é",Ecirc:"Ê",ecirc:"ê",Egrave:"È",egrave:"è",ETH:"Ð",eth:"ð",Euml:"Ë",euml:"ë",frac12:"½",frac14:"¼",frac34:"¾",gt:">",GT:">",Iacute:"Í",iacute:"í",Icirc:"Î",icirc:"î",iexcl:"¡",Igrave:"Ì",igrave:"ì",iquest:"¿",Iuml:"Ï",iuml:"ï",laquo:"«",lt:"<",LT:"<",macr:"¯",micro:"µ",middot:"·",nbsp:" ",not:"¬",Ntilde:"Ñ",ntilde:"ñ",Oacute:"Ó",oacute:"ó",Ocirc:"Ô",ocirc:"ô",Ograve:"Ò",ograve:"ò",ordf:"ª",ordm:"º",Oslash:"Ø",oslash:"ø",Otilde:"Õ",otilde:"õ",Ouml:"Ö",ouml:"ö",para:"¶",plusmn:"±",pound:"£",quot:'"',QUOT:'"',raquo:"»",reg:"®",REG:"®",sect:"§",shy:"­",sup1:"¹",sup2:"²",sup3:"³",szlig:"ß",THORN:"Þ",thorn:"þ",times:"×",Uacute:"Ú",uacute:"ú",Ucirc:"Û",ucirc:"û",Ugrave:"Ù",ugrave:"ù",uml:"¨",Uuml:"Ü",uuml:"ü",Yacute:"Ý",yacute:"ý",yen:"¥",yuml:"ÿ"};},function(t,e){var r=t.exports={get firstChild(){var t=this.children;return t&&t[0]||null},get lastChild(){var t=this.children;return t&&t[t.length-1]||null},get nodeType(){return i[this.type]||i.element}},n={tagName:"name",childNodes:"children",parentNode:"parent",previousSibling:"prev",nextSibling:"next",nodeValue:"data"},i={element:1,text:3,cdata:4,comment:8};Object.keys(n).forEach(function(t){var e=n[t];Object.defineProperty(r,t,{get:function(){return this[e]||null},set:function(t){return this[e]=t,t}});});},function(t,e,r){function n(t,e){var r=this._parser=new i(t,e),n=this._decoder=new a;o.call(this,{decodeStrings:!1}),this.once("finish",function(){r.end(n.end());});}t.exports=n;var i=r(30),o=r(62).Writable||r(74).Writable,a=r(22).StringDecoder,s=r(12).Buffer;r(1)(n,o),o.prototype._write=function(t,e,r){t instanceof s&&(t=this._decoder.write(t)),this._parser.write(t),r();};},function(t,e,r){(function(e,n){function i(t){return R.from(t)}function o(t){return R.isBuffer(t)||t instanceof I}function a(t,e,r){if("function"==typeof t.prependListener)return t.prependListener(e,r);t._events&&t._events[e]?q(t._events[e])?t._events[e].unshift(r):t._events[e]=[r,t._events[e]]:t.on(e,r);}function s(t,e){D=D||r(2),t=t||{};var n=e instanceof D;this.objectMode=!!t.objectMode,n&&(this.objectMode=this.objectMode||!!t.readableObjectMode);var i=t.highWaterMark,o=t.readableHighWaterMark,a=this.objectMode?16:16384;this.highWaterMark=i||0===i?i:n&&(o||0===o)?o:a,this.highWaterMark=Math.floor(this.highWaterMark),this.buffer=new V,this.length=0,this.pipes=null,this.pipesCount=0,this.flowing=null,this.ended=!1,this.endEmitted=!1,this.reading=!1,this.sync=!0,this.needReadable=!1,this.emittedReadable=!1,this.readableListening=!1,this.resumeScheduled=!1,this.destroyed=!1,this.defaultEncoding=t.defaultEncoding||"utf8",this.awaitDrain=0,this.readingMore=!1,this.decoder=null,this.encoding=null,t.encoding&&(z||(z=r(22).StringDecoder),this.decoder=new z(t.encoding),this.encoding=t.encoding);}function u(t){if(D=D||r(2),!(this instanceof u))return new u(t);this._readableState=new s(t,this),this.readable=!0,t&&("function"==typeof t.read&&(this._read=t.read),"function"==typeof t.destroy&&(this._destroy=t.destroy)),M.call(this);}function c(t,e,r,n,o){var a=t._readableState;if(null===e)a.reading=!1,g(t,a);else {var s;o||(s=f(a,e)),s?t.emit("error",s):a.objectMode||e&&e.length>0?("string"==typeof e||a.objectMode||Object.getPrototypeOf(e)===R.prototype||(e=i(e)),n?a.endEmitted?t.emit("error",new Error("stream.unshift() after end event")):l(t,a,e,!0):a.ended?t.emit("error",new Error("stream.push() after EOF")):(a.reading=!1,a.decoder&&!r?(e=a.decoder.write(e),a.objectMode||0!==e.length?l(t,a,e,!1):y(t,a)):l(t,a,e,!1))):n||(a.reading=!1);}return h(a)}function l(t,e,r,n){e.flowing&&0===e.length&&!e.sync?(t.emit("data",r),t.read(0)):(e.length+=e.objectMode?1:r.length,n?e.buffer.unshift(r):e.buffer.push(r),e.needReadable&&v(t)),y(t,e);}function f(t,e){var r;return o(e)||"string"==typeof e||void 0===e||t.objectMode||(r=new TypeError("Invalid non-string/buffer chunk")),r}function h(t){return !t.ended&&(t.needReadable||t.length<t.highWaterMark||0===t.length)}function p(t){return t>=W?t=W:(t--,t|=t>>>1,t|=t>>>2,t|=t>>>4,t|=t>>>8,t|=t>>>16,t++),t}function d(t,e){return t<=0||0===e.length&&e.ended?0:e.objectMode?1:t!==t?e.flowing&&e.length?e.buffer.head.data.length:e.length:(t>e.highWaterMark&&(e.highWaterMark=p(t)),t<=e.length?t:e.ended?e.length:(e.needReadable=!0,0))}function g(t,e){if(!e.ended){if(e.decoder){var r=e.decoder.end();r&&r.length&&(e.buffer.push(r),e.length+=e.objectMode?1:r.length);}e.ended=!0,v(t);}}function v(t){var e=t._readableState;e.needReadable=!1,e.emittedReadable||(F("emitReadable",e.flowing),e.emittedReadable=!0,e.sync?B.nextTick(b,t):b(t));}function b(t){F("emit readable"),t.emit("readable"),j(t);}function y(t,e){e.readingMore||(e.readingMore=!0,B.nextTick(m,t,e));}function m(t,e){for(var r=e.length;!e.reading&&!e.flowing&&!e.ended&&e.length<e.highWaterMark&&(F("maybeReadMore read 0"),t.read(0),r!==e.length);)r=e.length;e.readingMore=!1;}function _(t){return function(){var e=t._readableState;F("pipeOnDrain",e.awaitDrain),e.awaitDrain&&e.awaitDrain--,0===e.awaitDrain&&P(t,"data")&&(e.flowing=!0,j(t));}}function w(t){F("readable nexttick read 0"),t.read(0);}function x(t,e){e.resumeScheduled||(e.resumeScheduled=!0,B.nextTick(S,t,e));}function S(t,e){e.reading||(F("resume read 0"),t.read(0)),e.resumeScheduled=!1,e.awaitDrain=0,t.emit("resume"),j(t),e.flowing&&!e.reading&&t.read(0);}function j(t){var e=t._readableState;for(F("flow",e.flowing);e.flowing&&null!==t.read(););}function k(t,e){if(0===e.length)return null;var r;return e.objectMode?r=e.buffer.shift():!t||t>=e.length?(r=e.decoder?e.buffer.join(""):1===e.buffer.length?e.buffer.head.data:e.buffer.concat(e.length),e.buffer.clear()):r=E(t,e.buffer,e.decoder),r}function E(t,e,r){var n;return t<e.head.data.length?(n=e.head.data.slice(0,t),e.head.data=e.head.data.slice(t)):n=t===e.head.data.length?e.shift():r?A(t,e):T(t,e),n}function A(t,e){var r=e.head,n=1,i=r.data;for(t-=i.length;r=r.next;){var o=r.data,a=t>o.length?o.length:t;if(a===o.length?i+=o:i+=o.slice(0,t),0===(t-=a)){a===o.length?(++n,r.next?e.head=r.next:e.head=e.tail=null):(e.head=r,r.data=o.slice(a));break}++n;}return e.length-=n,i}function T(t,e){var r=R.allocUnsafe(t),n=e.head,i=1;for(n.data.copy(r),t-=n.data.length;n=n.next;){var o=n.data,a=t>o.length?o.length:t;if(o.copy(r,r.length-t,0,a),0===(t-=a)){a===o.length?(++i,n.next?e.head=n.next:e.head=e.tail=null):(e.head=n,n.data=o.slice(a));break}++i;}return e.length-=i,r}function O(t){var e=t._readableState;if(e.length>0)throw new Error('"endReadable()" called on non-empty stream');e.endEmitted||(e.ended=!0,B.nextTick(L,e,t));}function L(t,e){t.endEmitted||0!==t.length||(t.endEmitted=!0,e.readable=!1,e.emit("end"));}function C(t,e){for(var r=0,n=t.length;r<n;r++)if(t[r]===e)return r;return -1}var B=r(14);t.exports=u;var D,q=r(29);u.ReadableState=s;var P=(r(13).EventEmitter,function(t,e){return t.listeners(e).length}),M=r(37),R=r(15).Buffer,I=e.Uint8Array||function(){},N=r(8);N.inherits=r(1);var U=r(63),F=void 0;F=U&&U.debuglog?U.debuglog("stream"):function(){};var z,V=r(64),$=r(38);N.inherits(u,M);var H=["error","close","destroy","pause","resume"];Object.defineProperty(u.prototype,"destroyed",{get:function(){return void 0!==this._readableState&&this._readableState.destroyed},set:function(t){this._readableState&&(this._readableState.destroyed=t);}}),u.prototype.destroy=$.destroy,u.prototype._undestroy=$.undestroy,u.prototype._destroy=function(t,e){this.push(null),e(t);},u.prototype.push=function(t,e){var r,n=this._readableState;return n.objectMode?r=!0:"string"==typeof t&&(e=e||n.defaultEncoding,e!==n.encoding&&(t=R.from(t,e),e=""),r=!0),c(this,t,e,!1,r)},u.prototype.unshift=function(t){return c(this,t,null,!0,!1)},u.prototype.isPaused=function(){return !1===this._readableState.flowing},u.prototype.setEncoding=function(t){return z||(z=r(22).StringDecoder),this._readableState.decoder=new z(t),this._readableState.encoding=t,this};var W=8388608;u.prototype.read=function(t){F("read",t),t=parseInt(t,10);var e=this._readableState,r=t;if(0!==t&&(e.emittedReadable=!1),0===t&&e.needReadable&&(e.length>=e.highWaterMark||e.ended))return F("read: emitReadable",e.length,e.ended),0===e.length&&e.ended?O(this):v(this),null;if(0===(t=d(t,e))&&e.ended)return 0===e.length&&O(this),null;var n=e.needReadable;F("need readable",n),(0===e.length||e.length-t<e.highWaterMark)&&(n=!0,F("length less than watermark",n)),e.ended||e.reading?(n=!1,F("reading or ended",n)):n&&(F("do read"),e.reading=!0,e.sync=!0,0===e.length&&(e.needReadable=!0),this._read(e.highWaterMark),e.sync=!1,e.reading||(t=d(r,e)));var i;return i=t>0?k(t,e):null,null===i?(e.needReadable=!0,t=0):e.length-=t,0===e.length&&(e.ended||(e.needReadable=!0),r!==t&&e.ended&&O(this)),null!==i&&this.emit("data",i),i},u.prototype._read=function(t){this.emit("error",new Error("_read() is not implemented"));},u.prototype.pipe=function(t,e){function r(t,e){F("onunpipe"),t===h&&e&&!1===e.hasUnpiped&&(e.hasUnpiped=!0,o());}function i(){F("onend"),t.end();}function o(){F("cleanup"),t.removeListener("close",c),t.removeListener("finish",l),t.removeListener("drain",v),t.removeListener("error",u),t.removeListener("unpipe",r),h.removeListener("end",i),h.removeListener("end",f),h.removeListener("data",s),b=!0,!p.awaitDrain||t._writableState&&!t._writableState.needDrain||v();}function s(e){F("ondata"),y=!1,!1!==t.write(e)||y||((1===p.pipesCount&&p.pipes===t||p.pipesCount>1&&-1!==C(p.pipes,t))&&!b&&(F("false write response, pause",h._readableState.awaitDrain),h._readableState.awaitDrain++,y=!0),h.pause());}function u(e){F("onerror",e),f(),t.removeListener("error",u),0===P(t,"error")&&t.emit("error",e);}function c(){t.removeListener("finish",l),f();}function l(){F("onfinish"),t.removeListener("close",c),f();}function f(){F("unpipe"),h.unpipe(t);}var h=this,p=this._readableState;switch(p.pipesCount){case 0:p.pipes=t;break;case 1:p.pipes=[p.pipes,t];break;default:p.pipes.push(t);}p.pipesCount+=1,F("pipe count=%d opts=%j",p.pipesCount,e);var d=(!e||!1!==e.end)&&t!==n.stdout&&t!==n.stderr,g=d?i:f;p.endEmitted?B.nextTick(g):h.once("end",g),t.on("unpipe",r);var v=_(h);t.on("drain",v);var b=!1,y=!1;return h.on("data",s),a(t,"error",u),t.once("close",c),t.once("finish",l),t.emit("pipe",h),p.flowing||(F("pipe resume"),h.resume()),t},u.prototype.unpipe=function(t){var e=this._readableState,r={hasUnpiped:!1};if(0===e.pipesCount)return this;if(1===e.pipesCount)return t&&t!==e.pipes?this:(t||(t=e.pipes),e.pipes=null,e.pipesCount=0,e.flowing=!1,t&&t.emit("unpipe",this,r),this);if(!t){var n=e.pipes,i=e.pipesCount;e.pipes=null,e.pipesCount=0,e.flowing=!1;for(var o=0;o<i;o++)n[o].emit("unpipe",this,r);return this}var a=C(e.pipes,t);return -1===a?this:(e.pipes.splice(a,1),e.pipesCount-=1,1===e.pipesCount&&(e.pipes=e.pipes[0]),t.emit("unpipe",this,r),this)},u.prototype.on=function(t,e){var r=M.prototype.on.call(this,t,e);if("data"===t)!1!==this._readableState.flowing&&this.resume();else if("readable"===t){var n=this._readableState;n.endEmitted||n.readableListening||(n.readableListening=n.needReadable=!0,n.emittedReadable=!1,n.reading?n.length&&v(this):B.nextTick(w,this));}return r},u.prototype.addListener=u.prototype.on,u.prototype.resume=function(){var t=this._readableState;return t.flowing||(F("resume"),t.flowing=!0,x(this,t)),this},u.prototype.pause=function(){return F("call pause flowing=%j",this._readableState.flowing),!1!==this._readableState.flowing&&(F("pause"),this._readableState.flowing=!1,this.emit("pause")),this},u.prototype.wrap=function(t){var e=this,r=this._readableState,n=!1;t.on("end",function(){if(F("wrapped end"),r.decoder&&!r.ended){var t=r.decoder.end();t&&t.length&&e.push(t);}e.push(null);}),t.on("data",function(i){if(F("wrapped data"),r.decoder&&(i=r.decoder.write(i)),(!r.objectMode||null!==i&&void 0!==i)&&(r.objectMode||i&&i.length)){e.push(i)||(n=!0,t.pause());}});for(var i in t)void 0===this[i]&&"function"==typeof t[i]&&(this[i]=function(e){return function(){return t[e].apply(t,arguments)}}(i));for(var o=0;o<H.length;o++)t.on(H[o],this.emit.bind(this,H[o]));return this._read=function(e){F("wrapped _read",e),n&&(n=!1,t.resume());},this},Object.defineProperty(u.prototype,"readableHighWaterMark",{enumerable:!1,get:function(){return this._readableState.highWaterMark}}),u._fromList=k;}).call(e,r(0),r(7));},function(t,e,r){t.exports=r(13).EventEmitter;},function(t,e,r){function n(t,e){var r=this,n=this._readableState&&this._readableState.destroyed,i=this._writableState&&this._writableState.destroyed;return n||i?(e?e(t):!t||this._writableState&&this._writableState.errorEmitted||a.nextTick(o,this,t),this):(this._readableState&&(this._readableState.destroyed=!0),this._writableState&&(this._writableState.destroyed=!0),this._destroy(t||null,function(t){!e&&t?(a.nextTick(o,r,t),r._writableState&&(r._writableState.errorEmitted=!0)):e&&e(t);}),this)}function i(){this._readableState&&(this._readableState.destroyed=!1,this._readableState.reading=!1,this._readableState.ended=!1,this._readableState.endEmitted=!1),this._writableState&&(this._writableState.destroyed=!1,this._writableState.ended=!1,this._writableState.ending=!1,this._writableState.finished=!1,this._writableState.errorEmitted=!1);}function o(t,e){t.emit("error",e);}var a=r(14);t.exports={destroy:n,undestroy:i};},function(t,e,r){function n(t,e){var r=this._transformState;r.transforming=!1;var n=r.writecb;if(!n)return this.emit("error",new Error("write callback called multiple times"));r.writechunk=null,r.writecb=null,null!=e&&this.push(e),n(t);var i=this._readableState;i.reading=!1,(i.needReadable||i.length<i.highWaterMark)&&this._read(i.highWaterMark);}function i(t){if(!(this instanceof i))return new i(t);s.call(this,t),this._transformState={afterTransform:n.bind(this),needTransform:!1,transforming:!1,writecb:null,writechunk:null,writeencoding:null},this._readableState.needReadable=!0,this._readableState.sync=!1,t&&("function"==typeof t.transform&&(this._transform=t.transform),"function"==typeof t.flush&&(this._flush=t.flush)),this.on("prefinish",o);}function o(){var t=this;"function"==typeof this._flush?this._flush(function(e,r){a(t,e,r);}):a(this,null,null);}function a(t,e,r){if(e)return t.emit("error",e);if(null!=r&&t.push(r),t._writableState.length)throw new Error("Calling transform done when ws.length != 0");if(t._transformState.transforming)throw new Error("Calling transform done when still transforming");return t.push(null)}t.exports=i;var s=r(2),u=r(8);u.inherits=r(1),u.inherits(i,s),i.prototype.push=function(t,e){return this._transformState.needTransform=!1,s.prototype.push.call(this,t,e)},i.prototype._transform=function(t,e,r){throw new Error("_transform() is not implemented")},i.prototype._write=function(t,e,r){var n=this._transformState;if(n.writecb=r,n.writechunk=t,n.writeencoding=e,!n.transforming){var i=this._readableState;(n.needTransform||i.needReadable||i.length<i.highWaterMark)&&this._read(i.highWaterMark);}},i.prototype._read=function(t){var e=this._transformState;null!==e.writechunk&&e.writecb&&!e.transforming?(e.transforming=!0,this._transform(e.writechunk,e.writeencoding,e.afterTransform)):e.needTransform=!0;},i.prototype._destroy=function(t,e){var r=this;s.prototype._destroy.call(this,t,function(t){e(t),r.emit("close");});};},function(t,e){function r(t,e,r){switch(r.length){case 0:return t.call(e);case 1:return t.call(e,r[0]);case 2:return t.call(e,r[0],r[1]);case 3:return t.call(e,r[0],r[1],r[2])}return t.apply(e,r)}function n(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function i(t,e){var r=B(t)||d(t)?n(t.length,String):[],i=r.length,o=!!i;for(var a in t)!e&&!T.call(t,a)||o&&("length"==a||c(a,i))||r.push(a);return r}function o(t,e,r){var n=t[e];T.call(t,e)&&p(n,r)&&(void 0!==r||e in t)||(t[e]=r);}function a(t){if(!m(t))return h(t);var e=f(t),r=[];for(var n in t)("constructor"!=n||!e&&T.call(t,n))&&r.push(n);return r}function s(t,e){return e=C(void 0===e?t.length-1:e,0),function(){for(var n=arguments,i=-1,o=C(n.length-e,0),a=Array(o);++i<o;)a[i]=n[e+i];i=-1;for(var s=Array(e+1);++i<e;)s[i]=n[i];return s[e]=a,r(t,this,s)}}function u(t,e,r,n){r||(r={});for(var i=-1,a=e.length;++i<a;){var s=e[i],u=n?n(r[s],t[s],s,r,t):void 0;o(r,s,void 0===u?t[s]:u);}return r}function c(t,e){return !!(e=null==e?x:e)&&("number"==typeof t||E.test(t))&&t>-1&&t%1==0&&t<e}function l(t,e,r){if(!m(r))return !1;var n=typeof e;return !!("number"==n?g(r)&&c(e,r.length):"string"==n&&e in r)&&p(r[e],t)}function f(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||A)}function h(t){var e=[];if(null!=t)for(var r in Object(t))e.push(r);return e}function p(t,e){return t===e||t!==t&&e!==e}function d(t){return v(t)&&T.call(t,"callee")&&(!L.call(t,"callee")||O.call(t)==S)}function g(t){return null!=t&&y(t.length)&&!b(t)}function v(t){return _(t)&&g(t)}function b(t){var e=m(t)?O.call(t):"";return e==j||e==k}function y(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=x}function m(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function _(t){return !!t&&"object"==typeof t}function w(t){return g(t)?i(t,!0):a(t)}var x=9007199254740991,S="[object Arguments]",j="[object Function]",k="[object GeneratorFunction]",E=/^(?:0|[1-9]\d*)$/,A=Object.prototype,T=A.hasOwnProperty,O=A.toString,L=A.propertyIsEnumerable,C=Math.max,B=Array.isArray,D=function(t){return s(function(e,r){var n=-1,i=r.length,o=i>1?r[i-1]:void 0,a=i>2?r[2]:void 0;for(o=t.length>3&&"function"==typeof o?(i--,o):void 0,a&&l(r[0],r[1],a)&&(o=i<3?void 0:o,i=1),e=Object(e);++n<i;){var s=r[n];s&&t(e,s,n,o);}return e})}(function(t,e){u(e,w(e),t);});t.exports=D;},function(t,e){function r(t,e,r){switch(r.length){case 0:return t.call(e);case 1:return t.call(e,r[0]);case 2:return t.call(e,r[0],r[1]);case 3:return t.call(e,r[0],r[1],r[2])}return t.apply(e,r)}function n(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function i(t,e){var r=D(t)||g(t)?n(t.length,String):[],i=r.length,o=!!i;for(var a in t)!e&&!O.call(t,a)||o&&("length"==a||l(a,i))||r.push(a);return r}function o(t,e,r,n){return void 0===t||d(t,T[r])&&!O.call(n,r)?e:t}function a(t,e,r){var n=t[e];O.call(t,e)&&d(n,r)&&(void 0!==r||e in t)||(t[e]=r);}function s(t){if(!_(t))return p(t);var e=h(t),r=[];for(var n in t)("constructor"!=n||!e&&O.call(t,n))&&r.push(n);return r}function u(t,e){return e=B(void 0===e?t.length-1:e,0),function(){for(var n=arguments,i=-1,o=B(n.length-e,0),a=Array(o);++i<o;)a[i]=n[e+i];i=-1;for(var s=Array(e+1);++i<e;)s[i]=n[i];return s[e]=a,r(t,this,s)}}function c(t,e,r,n){r||(r={});for(var i=-1,o=e.length;++i<o;){var s=e[i],u=n?n(r[s],t[s],s,r,t):void 0;a(r,s,void 0===u?t[s]:u);}return r}function l(t,e){return !!(e=null==e?S:e)&&("number"==typeof t||A.test(t))&&t>-1&&t%1==0&&t<e}function f(t,e,r){if(!_(r))return !1;var n=typeof e;return !!("number"==n?v(r)&&l(e,r.length):"string"==n&&e in r)&&d(r[e],t)}function h(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||T)}function p(t){var e=[];if(null!=t)for(var r in Object(t))e.push(r);return e}function d(t,e){return t===e||t!==t&&e!==e}function g(t){return b(t)&&O.call(t,"callee")&&(!C.call(t,"callee")||L.call(t)==j)}function v(t){return null!=t&&m(t.length)&&!y(t)}function b(t){return w(t)&&v(t)}function y(t){var e=_(t)?L.call(t):"";return e==k||e==E}function m(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=S}function _(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function w(t){return !!t&&"object"==typeof t}function x(t){return v(t)?i(t,!0):s(t)}var S=9007199254740991,j="[object Arguments]",k="[object Function]",E="[object GeneratorFunction]",A=/^(?:0|[1-9]\d*)$/,T=Object.prototype,O=T.hasOwnProperty,L=T.toString,C=T.propertyIsEnumerable,B=Math.max,D=Array.isArray,q=function(t){return u(function(e,r){var n=-1,i=r.length,o=i>1?r[i-1]:void 0,a=i>2?r[2]:void 0;for(o=t.length>3&&"function"==typeof o?(i--,o):void 0,a&&f(r[0],r[1],a)&&(o=i<3?void 0:o,i=1),e=Object(e);++n<i;){var s=r[n];s&&t(e,s,n,o);}return e})}(function(t,e,r,n){c(e,x(e),t,n);}),P=u(function(t){return t.push(void 0,o),r(q,void 0,t)});t.exports=P;},function(t,e,r){function n(t){return function(e,r,n){return "function"!=typeof e&&(e=d(e,n,r)),r=Array.isArray(r)?f(r):l(r),t(e,r)}}function i(t,e,r){return ("function"==typeof e?e:p(e,r))(t)}function o(t,e,r){return v(t,e,r)}t.exports=o;var a=r(26),s=r(4),u=s.findOne,c=s.findAll,l=s.getChildren,f=s.removeSubsets,h=r(10).falseFunc,p=r(91),d=p.compileUnsafe,g=p.compileToken,v=n(function(t,e){return t!==h&&e&&0!==e.length?c(t,e):[]}),b=n(function(t,e){return t!==h&&e&&0!==e.length?u(t,e):null});o.compile=p,o.filters=a.filters,o.pseudos=a.pseudos,o.selectAll=v,o.selectOne=b,o.is=i,o.parse=p,o.iterate=v,o._compileUnsafe=d,o._compileToken=g;},function(t,e,r){var n=r(4),i=n.hasAttrib,o=n.getAttributeValue,a=r(10).falseFunc,s=/[-[\]{}()*+?.,\\^$|#\s]/g,u={__proto__:null,equals:function(t,e){var r=e.name,n=e.value;return e.ignoreCase?(n=n.toLowerCase(),function(e){var i=o(e,r);return null!=i&&i.toLowerCase()===n&&t(e)}):function(e){return o(e,r)===n&&t(e)}},hyphen:function(t,e){var r=e.name,n=e.value,i=n.length;return e.ignoreCase?(n=n.toLowerCase(),function(e){var a=o(e,r);return null!=a&&(a.length===i||"-"===a.charAt(i))&&a.substr(0,i).toLowerCase()===n&&t(e)}):function(e){var a=o(e,r);return null!=a&&a.substr(0,i)===n&&(a.length===i||"-"===a.charAt(i))&&t(e)}},element:function(t,e){var r=e.name,n=e.value;if(/\s/.test(n))return a;n=n.replace(s,"\\$&");var i="(?:^|\\s)"+n+"(?:$|\\s)",u=e.ignoreCase?"i":"",c=new RegExp(i,u);return function(e){var n=o(e,r);return null!=n&&c.test(n)&&t(e)}},exists:function(t,e){var r=e.name;return function(e){return i(e,r)&&t(e)}},start:function(t,e){var r=e.name,n=e.value,i=n.length;return 0===i?a:e.ignoreCase?(n=n.toLowerCase(),function(e){var a=o(e,r);return null!=a&&a.substr(0,i).toLowerCase()===n&&t(e)}):function(e){var a=o(e,r);return null!=a&&a.substr(0,i)===n&&t(e)}},end:function(t,e){var r=e.name,n=e.value,i=-n.length;return 0===i?a:e.ignoreCase?(n=n.toLowerCase(),function(e){var a=o(e,r);return null!=a&&a.substr(i).toLowerCase()===n&&t(e)}):function(e){var a=o(e,r);return null!=a&&a.substr(i)===n&&t(e)}},any:function(t,e){var r=e.name,n=e.value;if(""===n)return a;if(e.ignoreCase){var i=new RegExp(n.replace(s,"\\$&"),"i");return function(e){var n=o(e,r);return null!=n&&i.test(n)&&t(e)}}return function(e){var i=o(e,r);return null!=i&&i.indexOf(n)>=0&&t(e)}},not:function(t,e){var r=e.name,n=e.value;return ""===n?function(e){return !!o(e,r)&&t(e)}:e.ignoreCase?(n=n.toLowerCase(),function(e){var i=o(e,r);return null!=i&&i.toLowerCase()!==n&&t(e)}):function(e){return o(e,r)!==n&&t(e)}}};t.exports={compile:function(t,e,r){if(r&&r.strict&&(e.ignoreCase||"not"===e.action))throw SyntaxError("Unsupported attribute selector");return u[e.action](t,e)},rules:u};},function(t,e){t.exports={universal:50,tag:30,attribute:1,pseudo:0,descendant:-1,child:-1,parent:-1,sibling:-1,adjacent:-1};},function(t,e,r){function n(t){function e(t){return t?w[w.length-1-t]:_}function r(t){return t===n(1)}function n(e){return t[b+(e||1)]}function s(){var t=w.pop();return _=w[w.length-1],t}function u(t){return _=t,w.push(_),w.length}function c(t){var e=_;return w[w.length-1]=_=t,e}function l(e){if(1==(e||1))"\n"==t[b]?(m++,v=1):v++,b++;else {var r=t.slice(b,b+e).split("\n");r.length>1&&(m+=r.length-1,v=1),v+=r[r.length-1].length,b+=e;}}function f(){x.end={line:m,col:v},S.push(x),g="",x={};}function h(t){x={type:t,start:{line:m,col:v}};}var d,g="",v=0,b=-1,y=0,m=1,_="before-selector",w=[_],x={},S=[],j=["media","keyframes",{name:"-webkit-keyframes",type:"keyframes",prefix:"-webkit-"},{name:"-moz-keyframes",type:"keyframes",prefix:"-moz-"},{name:"-ms-keyframes",type:"keyframes",prefix:"-ms-"},{name:"-o-keyframes",type:"keyframes",prefix:"-o-"},"font-face",{name:"import",state:"before-at-value"},{name:"charset",state:"before-at-value"},"supports","viewport",{name:"namespace",state:"before-at-value"},"document",{name:"-moz-document",type:"document",prefix:"-moz-"},"page"];for(o;d=function(){return l(),t[b]}();)switch(d){case" ":switch(e()){case"selector":case"value":case"value-paren":case"at-group":case"at-value":case"comment":case"double-string":case"single-string":g+=d;}break;case"\n":case"\t":case"\r":case"\f":switch(e()){case"value":case"value-paren":case"at-group":case"comment":case"single-string":case"double-string":case"selector":g+=d;break;case"at-value":"\n"===d&&(x.value=g.trim(),f(),s());}break;case":":switch(e()){case"name":x.name=g.trim(),g="",c("before-value");break;case"before-selector":g+=d,h("selector"),u("selector");break;case"before-value":c("value"),g+=d;break;default:g+=d;}break;case";":switch(e()){case"name":case"before-value":case"value":g.trim().length>0&&(x.value=g.trim(),f()),c("before-name");break;case"value-paren":g+=d;break;case"at-value":x.value=g.trim(),f(),s();break;case"before-name":break;default:g+=d;}break;case"{":switch(e()){case"selector":if("\\"===n(-1)){g+=d;break}x.text=g.trim(),f(),c("before-name"),y+=1;break;case"at-group":switch(x.name=g.trim(),x.type){case"font-face":case"viewport":case"page":u("before-name");break;default:u("before-selector");}f(),y+=1;break;case"name":case"at-rule":x.name=g.trim(),f(),u("before-name"),y+=1;break;case"comment":case"double-string":case"single-string":g+=d;break;case"before-value":c("value"),g+=d;}break;case"}":switch(e()){case"before-name":case"name":case"before-value":case"value":g&&(x.value=g.trim()),x.name&&x.value&&f(),h("end"),f(),s(),"at-group"===e()&&(h("at-group-end"),f(),s()),y>0&&(y-=1);break;case"at-group":case"before-selector":case"selector":if("\\"===n(-1)){g+=d;break}y>0&&"at-group"===e(1)&&(h("at-group-end"),f()),y>1&&s(),y>0&&(y-=1);break;case"double-string":case"single-string":case"comment":g+=d;}break;case'"':case"'":switch(e()){case"double-string":'"'===d&&"\\"!==n(-1)&&s();break;case"single-string":"'"===d&&"\\"!==n(-1)&&s();break;case"before-at-value":c("at-value"),u('"'===d?"double-string":"single-string");break;case"before-value":c("value"),u('"'===d?"double-string":"single-string");break;case"comment":break;default:"\\"!==n(-1)&&u('"'===d?"double-string":"single-string");}g+=d;break;case"/":switch(e()){case"comment":case"double-string":case"single-string":g+=d;break;case"before-value":case"selector":case"name":case"value":if(r("*")){var k=function(e){var r=t.slice(b).indexOf(e);return r>0&&r}("*/");k&&l(k+1);}else "before-value"==e()&&c("value"),g+=d;break;default:r("*")?(h("comment"),u("comment"),l()):g+=d;}break;case"*":switch(e()){case"comment":r("/")?(x.text=g,l(),f(),s()):g+=d;break;case"before-selector":g+=d,h("selector"),u("selector");break;case"before-value":c("value"),g+=d;break;default:g+=d;}break;case"@":switch(e()){case"comment":case"double-string":case"single-string":g+=d;break;case"before-value":c("value"),g+=d;break;default:for(var E,A,T=!1,O=0,L=j.length;!T&&O<L;++O)A=j[O],E=A.name||A,function(e){var r=b+1;return e===t.slice(r,r+e.length)}(E)&&(T=!0,h(E),u(A.state||"at-group"),l(E.length),A.prefix&&(x.prefix=A.prefix),A.type&&(x.type=A.type));T||(g+=d);}break;case"(":switch(e()){case"value":u("value-paren");break;case"before-value":c("value");}g+=d;break;case")":switch(e()){case"value-paren":s();break;case"before-value":c("value");}g+=d;break;default:switch(e()){case"before-selector":h("selector"),u("selector");break;case"before-name":h("property"),c("name");break;case"before-value":c("value");break;case"before-at-value":c("at-value");}g+=d;}return S}var o=!1,a=r(28)("lex");t.exports=n;},function(t,e,r){Object.defineProperty(e,"__esModule",{value:!0});var n=r(47),i=function(t){return t&&t.__esModule?t:{default:t}}(n);e.default=i.default.plugins.add("gjs-preset-newsletter",function(t,e){var n=e||{},i=t.getConfig(),o=i.stylePrefix,a={editor:t,pfx:o||"",cmdOpenImport:"gjs-open-import-template",cmdTglImages:"gjs-toggle-images",cmdInlineHtml:"gjs-get-inlined-html",cmtTglImagesLabel:"Toggle Images",cmdBtnMoveLabel:"Move",cmdBtnUndoLabel:"Undo",cmdBtnRedoLabel:"Redo",cmdBtnDesktopLabel:"Desktop",cmdBtnTabletLabel:"Tablet",cmdBtnMobileLabel:"Mobile",modalTitleImport:"Import template",modalTitleExport:"Export template",modalLabelImport:"",modalLabelExport:"",modalBtnImport:"Import",codeViewerTheme:"hopscotch",openBlocksBtnTitle:n.openBlocksBtnTitle||"",openLayersBtnTitle:n.openLayersBtnTitle||"",openSmBtnTitle:n.openSmBtnTitle||"",openTmBtnTitle:n.openTmBtnTitle||"",expTplBtnTitle:n.expTplBtnTitle||"View Code",fullScrBtnTitle:n.fullScrBtnTitle||"FullScreen",swichtVwBtnTitle:n.swichtVwBtnTitle||"View Components",categoryLabel:n.categoryLabel||"",importPlaceholder:"",defaultTemplate:"",inlineCss:1,cellStyle:{padding:0,margin:0,"vertical-align":"top"},tableStyle:{height:"150px",margin:"0 auto 10px auto",padding:"5px 5px 5px 5px",width:"100%"},sect100BlkLabel:"1 Section",sect50BlkLabel:"1/2 Section",sect30BlkLabel:"1/3 Section",sect37BlkLabel:"3/7 Section",buttonBlkLabel:"Button",dividerBlkLabel:"Divider",textBlkLabel:"Text",textSectionBlkLabel:"Text Section",imageBlkLabel:"Image",quoteBlkLabel:"Quote",linkBlkLabel:"Link",linkBlockBlkLabel:"Link Block",gridItemsBlkLabel:"Grid Items",listItemsBlkLabel:"List Items",assetsModalTitle:n.assetsModalTitle||"Select image",styleManagerSectors:[{name:"Dimension",open:!1,buildProps:["width","height","max-width","min-height","margin","padding"],properties:[{property:"margin",properties:[{name:"Top",property:"margin-top"},{name:"Left",property:"margin-left"},{name:"Right",property:"margin-right"},{name:"Bottom",property:"margin-bottom"}]},{property:"padding",properties:[{name:"Top",property:"padding-top"},{name:"Right",property:"padding-right"},{name:"Bottom",property:"padding-bottom"},{name:"Left",property:"padding-left"}]}]},{name:"Typography",open:!1,buildProps:["font-family","font-size","font-weight","letter-spacing","color","line-height","text-align","text-decoration","font-style","vertical-align","text-shadow"],properties:[{name:"Font",property:"font-family"},{name:"Weight",property:"font-weight"},{name:"Font color",property:"color"},{property:"text-align",type:"radio",defaults:"left",list:[{value:"left",name:"Left",className:"fa fa-align-left"},{value:"center",name:"Center",className:"fa fa-align-center"},{value:"right",name:"Right",className:"fa fa-align-right"},{value:"justify",name:"Justify",className:"fa fa-align-justify"}]},{property:"text-decoration",type:"radio",defaults:"none",list:[{value:"none",name:"None",className:"fa fa-times"},{value:"underline",name:"underline",className:"fa fa-underline"},{value:"line-through",name:"Line-through",className:"fa fa-strikethrough"}]},{property:"font-style",type:"radio",defaults:"normal",list:[{value:"normal",name:"Normal",className:"fa fa-font"},{value:"italic",name:"Italic",className:"fa fa-italic"}]},{property:"vertical-align",type:"select",defaults:"baseline",list:[{value:"baseline"},{value:"top"},{value:"middle"},{value:"bottom"}]},{property:"text-shadow",properties:[{name:"X position",property:"text-shadow-h"},{name:"Y position",property:"text-shadow-v"},{name:"Blur",property:"text-shadow-blur"},{name:"Color",property:"text-shadow-color"}]}]},{name:"Decorations",open:!1,buildProps:["background-color","border-collapse","border-radius","border","background"],properties:[{property:"background-color",name:"Background"},{property:"border-radius",properties:[{name:"Top",property:"border-top-left-radius"},{name:"Right",property:"border-top-right-radius"},{name:"Bottom",property:"border-bottom-left-radius"},{name:"Left",property:"border-bottom-right-radius"}]},{property:"border-collapse",type:"radio",defaults:"separate",list:[{value:"separate",name:"No"},{value:"collapse",name:"Yes"}]},{property:"border",properties:[{name:"Width",property:"border-width",defaults:"0"},{name:"Style",property:"border-style"},{name:"Color",property:"border-color"}]},{property:"background",properties:[{name:"Image",property:"background-image"},{name:"Repeat",property:"background-repeat"},{name:"Position",property:"background-position"},{name:"Attachment",property:"background-attachment"},{name:"Size",property:"background-size"}]}]}]};i.devicePreviewMode=1;for(var s in a)s in n||(n[s]=a[s]);r(48)(n),r(115)(n),r(116)(n),r(117)(n),!t.getHtml()&&n.defaultTemplate&&(t.setComponents(n.defaultTemplate),t.editor.initChildrenComp(t.DomComponents.getWrapper())),t.on("change:selectedComponent",function(){var e=t.Panels.getButton("views","open-layers");if((!e||!e.get("active"))&&t.editor.get("selectedComponent")){var r=t.Panels.getButton("views","open-sm");r.set("attributes",{title:a.openSmBtnTitle}),r&&r.set("active",1);}}),t.on("run:open-assets",function(){t.Modal.setTitle(a.assetsModalTitle);}),t.on("load",function(){t.Panels.getButton("options","export-template").set("attributes",{title:a.expTplBtnTitle}),t.Panels.getButton("options","fullscreen").set("attributes",{title:a.fullScrBtnTitle}),t.Panels.getButton("options","sw-visibility").set("attributes",{title:a.swichtVwBtnTitle}),t.Panels.getButton("views","open-sm").set("attributes",{title:a.openSmBtnTitle}),t.Panels.getButton("views","open-tm").set("attributes",{title:a.openTmBtnTitle}),t.Panels.getButton("views","open-layers").set("attributes",{title:a.openLayersBtnTitle});var e=t.Panels.getButton("views","open-blocks");e.set("attributes",{title:a.openBlocksBtnTitle}),e&&e.set("active",1);});});},function(e,r){e.exports=t;},function(t,e,r){var n,i=r(49),o=function(t){return t&&t.__esModule?t:{default:t}}(i);void 0!==(n=function(){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.editor,n=e.Commands,i=r(50),a=r(51);n.add(t.cmdOpenImport,i(t)),n.add(t.cmdTglImages,(0, o.default)(t)),e.on("load",function(){n.add("export-template",a(t));}),n.add("undo",{run:function(t,e){e.set("active",0),t.UndoManager.undo(1);}}),n.add("redo",{run:function(t,e){e.set("active",0),t.UndoManager.redo(1);}}),n.add("set-device-desktop",{run:function(t){t.setDevice("Desktop");}}),n.add("set-device-tablet",{run:function(t){t.setDevice("Tablet");}}),n.add("set-device-mobile",{run:function(t){t.setDevice("Mobile portrait");}});}}.call(e,r,e,t))&&(t.exports=n);},function(t,e,r){Object.defineProperty(e,"__esModule",{value:!0}),e.default=function(){var t=function t(e,r){e.each(function(e){if("image"===e.get("type")){var n=e.get("src");r?"##"===n&&e.set("src",e.get("src_bkp")):"##"!==n&&(e.set("src_bkp",e.get("src")),e.set("src","##"));}t(e.get("components"),r);});};return {run:function(e){var r=e.getComponents();t(r);},stop:function(e){var r=e.getComponents();t(r,1);}}};},function(t,e,r){var n;void 0!==(n=function(){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.editor,r=e&&e.CodeManager.getViewer("CodeMirror").clone(),n=document.createElement("button"),i=document.createElement("div"),o=t.pfx||"";return n.innerHTML=t.modalBtnImport,n.className=o+"btn-prim "+o+"btn-import",n.onclick=function(){var t=r.editor.getValue();e.DomComponents.getWrapper().set("content",""),e.setComponents(t),e.Modal.close();},r.set({codeName:"htmlmixed",theme:t.codeViewerTheme,readOnly:0}),{run:function(e,a){var s=e.Modal,u=(s.getContentEl(),r.editor);if(s.setTitle(t.modalTitleImport),!u){var c=document.createElement("textarea");if(t.modalLabelImport){var l=document.createElement("div");l.className=o+"import-label",l.innerHTML=t.modalLabelImport,i.appendChild(l);}i.appendChild(c),i.appendChild(n),r.init(c),u=r.editor;}s.setContent(""),s.setContent(i),r.setContent(t.importPlaceholder||""),s.open(),u.refresh(),a&&a.set("active",0);}}}}.call(e,r,e,t))&&(t.exports=n);},function(t,e,r){var n;void 0!==(n=function(){var t=r(52);return function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},r=e.editor,n=r&&r.CodeManager.getViewer("CodeMirror").clone(),i=document.createElement("div"),o=e.pfx||"",a=r.Commands,s=e.juiceOpts||{};return n.set({codeName:"htmlmixed",theme:e.codeViewerTheme}),a.add(e.cmdInlineHtml,{run:function(e,r){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},i=e.getHtml()+"<style>"+e.getCss()+"</style>";return t(i,n)}}),{run:function(r,a){var u=r.Modal,c=(u.getContentEl(),n.editor);if(u.setTitle(e.modalTitleExport),!c){var l=document.createElement("textarea");if(e.modalLabelExport){var f=document.createElement("div");f.className=o+"export-label",f.innerHTML=e.modalLabelExport,i.appendChild(f);}i.appendChild(l),n.init(l),c=n.editor,c.setOption("lineWrapping",1);}u.setContent(i);var h=r.getHtml()+"<style>"+r.getCss()+"</style>";n.setContent(e.inlineCss?t(h,s):h),u.open(),c.refresh(),a&&a.set&&a.set("active",0);}}}}.call(e,r,e,t))&&(t.exports=n);},function(t,e,r){var n=r(53),i=r(114),o=i(function(t,e){return n(t,{xmlMode:e&&e.xmlMode},a,[e])}),a=function(t,e){return o.juiceDocument(t,e)};o.inlineContent=function(t,e,r){return n(t,{xmlMode:r&&r.xmlMode},o.inlineDocument,[e,r])},t.exports=o;},function(t,e,r){var n=r(54),i=r(27),o=function(t,e,r){return e=i.extend({decodeEntities:!1},e||{}),t=r(t),n.load(t,e)},a=function(){var e=[];return {encodeEntities:function(r){var n=t.exports.codeBlocks;return Object.keys(n).forEach(function(t){var i=new RegExp(n[t].start+"([\\S\\s]*?)"+n[t].end,"g");r=r.replace(i,function(t,r){return e.push(t),"JUICE_CODE_BLOCK_"+(e.length-1)+"_"});}),r},decodeEntities:function(t){for(var r=0;r<e.length;r++){var n=new RegExp("JUICE_CODE_BLOCK_"+r+'_(="")?',"gi");t=t.replace(n,function(){return e[r]});}return t}}};t.exports=function(t,e,r,n){var i=a(),s=o(t,e,i.encodeEntities),u=[s];u.push.apply(u,n);var c=r.apply(void 0,u)||s;return e&&e.xmlMode?c.xml():i.decodeEntities(c.html())},t.exports.codeBlocks={EJS:{start:"<%",end:"%>"},HBS:{start:"{{",end:"}}"}};},function(t,e,r){e=t.exports=r(17),e.version=r(107).version;},function(t,e,r){function n(t){var e=t.length;if(e%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var r=t.indexOf("=");return -1===r&&(r=e),[r,r===e?0:4-r%4]}function i(t){var e=n(t),r=e[0],i=e[1];return 3*(r+i)/4-i}function o(t,e,r){return 3*(e+r)/4-r}function a(t){for(var e,r=n(t),i=r[0],a=r[1],s=new h(o(t,i,a)),u=0,c=a>0?i-4:i,l=0;l<c;l+=4)e=f[t.charCodeAt(l)]<<18|f[t.charCodeAt(l+1)]<<12|f[t.charCodeAt(l+2)]<<6|f[t.charCodeAt(l+3)],s[u++]=e>>16&255,s[u++]=e>>8&255,s[u++]=255&e;return 2===a&&(e=f[t.charCodeAt(l)]<<2|f[t.charCodeAt(l+1)]>>4,s[u++]=255&e),1===a&&(e=f[t.charCodeAt(l)]<<10|f[t.charCodeAt(l+1)]<<4|f[t.charCodeAt(l+2)]>>2,s[u++]=e>>8&255,s[u++]=255&e),s}function s(t){return l[t>>18&63]+l[t>>12&63]+l[t>>6&63]+l[63&t]}function u(t,e,r){for(var n,i=[],o=e;o<r;o+=3)n=(t[o]<<16&16711680)+(t[o+1]<<8&65280)+(255&t[o+2]),i.push(s(n));return i.join("")}function c(t){for(var e,r=t.length,n=r%3,i=[],o=0,a=r-n;o<a;o+=16383)i.push(u(t,o,o+16383>a?a:o+16383));return 1===n?(e=t[r-1],i.push(l[e>>2]+l[e<<4&63]+"==")):2===n&&(e=(t[r-2]<<8)+t[r-1],i.push(l[e>>10]+l[e>>4&63]+l[e<<2&63]+"=")),i.join("")}e.byteLength=i,e.toByteArray=a,e.fromByteArray=c;for(var l=[],f=[],h="undefined"!=typeof Uint8Array?Uint8Array:Array,p="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",d=0,g=p.length;d<g;++d)l[d]=p[d],f[p.charCodeAt(d)]=d;f["-".charCodeAt(0)]=62,f["_".charCodeAt(0)]=63;},function(t,e){e.read=function(t,e,r,n,i){var o,a,s=8*i-n-1,u=(1<<s)-1,c=u>>1,l=-7,f=r?i-1:0,h=r?-1:1,p=t[e+f];for(f+=h,o=p&(1<<-l)-1,p>>=-l,l+=s;l>0;o=256*o+t[e+f],f+=h,l-=8);for(a=o&(1<<-l)-1,o>>=-l,l+=n;l>0;a=256*a+t[e+f],f+=h,l-=8);if(0===o)o=1-c;else {if(o===u)return a?NaN:1/0*(p?-1:1);a+=Math.pow(2,n),o-=c;}return (p?-1:1)*a*Math.pow(2,o-n)},e.write=function(t,e,r,n,i,o){var a,s,u,c=8*o-i-1,l=(1<<c)-1,f=l>>1,h=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,p=n?0:o-1,d=n?1:-1,g=e<0||0===e&&1/e<0?1:0;for(e=Math.abs(e),isNaN(e)||e===1/0?(s=isNaN(e)?1:0,a=l):(a=Math.floor(Math.log(e)/Math.LN2),e*(u=Math.pow(2,-a))<1&&(a--,u*=2),e+=a+f>=1?h/u:h*Math.pow(2,1-f),e*u>=2&&(a++,u/=2),a+f>=l?(s=0,a=l):a+f>=1?(s=(e*u-1)*Math.pow(2,i),a+=f):(s=e*Math.pow(2,f-1)*Math.pow(2,i),a=0));i>=8;t[r+p]=255&s,p+=d,s/=256,i-=8);for(a=a<<i|s,c+=i;c>0;t[r+p]=255&a,p+=d,a/=256,c-=8);t[r+p-d]|=128*g;};},function(t,e){t.exports={0:65533,128:8364,130:8218,131:402,132:8222,133:8230,134:8224,135:8225,136:710,137:8240,138:352,139:8249,140:338,142:381,145:8216,146:8217,147:8220,148:8221,149:8226,150:8211,151:8212,152:732,153:8482,154:353,155:8250,156:339,158:382,159:376};},function(t,e,r){function n(t,e,r){"object"==typeof t?(r=e,e=t,t=null):"function"==typeof e&&(r=e,e=u),this._callback=t,this._options=e||u,this._elementCB=r,this.dom=[],this._done=!1,this._tagStack=[],this._parser=this._parser||null;}var i=r(6),o=/\s+/g,a=r(34),s=r(59),u={normalizeWhitespace:!1,withStartIndices:!1,withEndIndices:!1};n.prototype.onparserinit=function(t){this._parser=t;},n.prototype.onreset=function(){n.call(this,this._callback,this._options,this._elementCB);},n.prototype.onend=function(){this._done||(this._done=!0,this._parser=null,this._handleCallback(null));},n.prototype._handleCallback=n.prototype.onerror=function(t){if("function"==typeof this._callback)this._callback(t,this.dom);else if(t)throw t},n.prototype.onclosetag=function(){var t=this._tagStack.pop();this._options.withEndIndices&&t&&(t.endIndex=this._parser.endIndex),this._elementCB&&this._elementCB(t);},n.prototype._createDomElement=function(t){if(!this._options.withDomLvl1)return t;var e;e="tag"===t.type?Object.create(s):Object.create(a);for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r]);return e},n.prototype._addDomElement=function(t){var e=this._tagStack[this._tagStack.length-1],r=e?e.children:this.dom,n=r[r.length-1];t.next=null,this._options.withStartIndices&&(t.startIndex=this._parser.startIndex),this._options.withEndIndices&&(t.endIndex=this._parser.endIndex),n?(t.prev=n,n.next=t):t.prev=null,r.push(t),t.parent=e||null;},n.prototype.onopentag=function(t,e){var r={type:"script"===t?i.Script:"style"===t?i.Style:i.Tag,name:t,attribs:e,children:[]},n=this._createDomElement(r);this._addDomElement(n),this._tagStack.push(n);},n.prototype.ontext=function(t){var e,r=this._options.normalizeWhitespace||this._options.ignoreWhitespace;if(!this._tagStack.length&&this.dom.length&&(e=this.dom[this.dom.length-1]).type===i.Text)r?e.data=(e.data+t).replace(o," "):e.data+=t;else if(this._tagStack.length&&(e=this._tagStack[this._tagStack.length-1])&&(e=e.children[e.children.length-1])&&e.type===i.Text)r?e.data=(e.data+t).replace(o," "):e.data+=t;else {r&&(t=t.replace(o," "));var n=this._createDomElement({data:t,type:i.Text});this._addDomElement(n);}},n.prototype.oncomment=function(t){var e=this._tagStack[this._tagStack.length-1];if(e&&e.type===i.Comment)return void(e.data+=t);var r={data:t,type:i.Comment},n=this._createDomElement(r);this._addDomElement(n),this._tagStack.push(n);},n.prototype.oncdatastart=function(){var t={children:[{data:"",type:i.Text}],type:i.CDATA},e=this._createDomElement(t);this._addDomElement(e),this._tagStack.push(e);},n.prototype.oncommentend=n.prototype.oncdataend=function(){this._tagStack.pop();},n.prototype.onprocessinginstruction=function(t,e){var r=this._createDomElement({name:t,data:e,type:i.Directive});this._addDomElement(r);},t.exports=n;},function(t,e,r){var n=r(34),i=t.exports=Object.create(n),o={tagName:"name"};Object.keys(o).forEach(function(t){var e=o[t];Object.defineProperty(i,t,{get:function(){return this[e]||null},set:function(t){return this[e]=t,t}});});},function(t,e,r){function n(t,e){this.init(t,e);}function i(t,e){return l.getElementsByTagName(t,e,!0)}function o(t,e){return l.getElementsByTagName(t,e,!0,1)[0]}function a(t,e,r){return l.getText(l.getElementsByTagName(t,e,r,1)).trim()}function s(t,e,r,n,i){var o=a(r,n,i);o&&(t[e]=o);}var u=r(3),c=u.DomHandler,l=u.DomUtils;r(1)(n,c),n.prototype.init=c;var f=function(t){return "rss"===t||"feed"===t||"rdf:RDF"===t};n.prototype.onend=function(){var t,e,r={},n=o(f,this.dom);n&&("feed"===n.name?(e=n.children,r.type="atom",s(r,"id","id",e),s(r,"title","title",e),(t=o("link",e))&&(t=t.attribs)&&(t=t.href)&&(r.link=t),s(r,"description","subtitle",e),(t=a("updated",e))&&(r.updated=new Date(t)),s(r,"author","email",e,!0),r.items=i("entry",e).map(function(t){var e,r={};return t=t.children,s(r,"id","id",t),s(r,"title","title",t),(e=o("link",t))&&(e=e.attribs)&&(e=e.href)&&(r.link=e),(e=a("summary",t)||a("content",t))&&(r.description=e),(e=a("updated",t))&&(r.pubDate=new Date(e)),r})):(e=o("channel",n.children).children,r.type=n.name.substr(0,3),r.id="",s(r,"title","title",e),s(r,"link","link",e),s(r,"description","description",e),(t=a("lastBuildDate",e))&&(r.updated=new Date(t)),s(r,"author","managingEditor",e,!0),r.items=i("item",n.children).map(function(t){var e,r={};return t=t.children,s(r,"id","guid",t),s(r,"title","title",t),s(r,"link","link",t),s(r,"description","description",t),(e=a("pubDate",t))&&(r.pubDate=new Date(e)),r}))),this.dom=r,c.prototype._handleCallback.call(this,n?null:Error("couldn't find root of feed"));},t.exports=n;},function(t,e,r){function n(t){o.call(this,new i(this),t);}function i(t){this.scope=t;}t.exports=n;var o=r(35);r(1)(n,o),n.prototype.readable=!0;var a=r(3).EVENTS;Object.keys(a).forEach(function(t){if(0===a[t])i.prototype["on"+t]=function(){this.scope.emit(t);};else if(1===a[t])i.prototype["on"+t]=function(e){this.scope.emit(t,e);};else {if(2!==a[t])throw Error("wrong number of arguments!");i.prototype["on"+t]=function(e,r){this.scope.emit(t,e,r);};}});},function(t,e,r){function n(){i.call(this);}t.exports=n;var i=r(13).EventEmitter;r(1)(n,i),n.Readable=r(20),n.Writable=r(70),n.Duplex=r(71),n.Transform=r(72),n.PassThrough=r(73),n.Stream=n,n.prototype.pipe=function(t,e){function r(e){t.writable&&!1===t.write(e)&&c.pause&&c.pause();}function n(){c.readable&&c.resume&&c.resume();}function o(){l||(l=!0,t.end());}function a(){l||(l=!0,"function"==typeof t.destroy&&t.destroy());}function s(t){if(u(),0===i.listenerCount(this,"error"))throw t}function u(){c.removeListener("data",r),t.removeListener("drain",n),c.removeListener("end",o),c.removeListener("close",a),c.removeListener("error",s),t.removeListener("error",s),c.removeListener("end",u),c.removeListener("close",u),t.removeListener("close",u);}var c=this;c.on("data",r),t.on("drain",n),t._isStdio||e&&!1===e.end||(c.on("end",o),c.on("close",a));var l=!1;return c.on("error",s),t.on("error",s),c.on("end",u),c.on("close",u),t.on("close",u),t.emit("pipe",c),t};},function(t,e){},function(t,e,r){function n(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}function i(t,e,r){t.copy(e,r);}var o=r(15).Buffer,a=r(65);t.exports=function(){function t(){n(this,t),this.head=null,this.tail=null,this.length=0;}return t.prototype.push=function(t){var e={data:t,next:null};this.length>0?this.tail.next=e:this.head=e,this.tail=e,++this.length;},t.prototype.unshift=function(t){var e={data:t,next:this.head};0===this.length&&(this.tail=e),this.head=e,++this.length;},t.prototype.shift=function(){if(0!==this.length){var t=this.head.data;return 1===this.length?this.head=this.tail=null:this.head=this.head.next,--this.length,t}},t.prototype.clear=function(){this.head=this.tail=null,this.length=0;},t.prototype.join=function(t){if(0===this.length)return "";for(var e=this.head,r=""+e.data;e=e.next;)r+=t+e.data;return r},t.prototype.concat=function(t){if(0===this.length)return o.alloc(0);if(1===this.length)return this.head.data;for(var e=o.allocUnsafe(t>>>0),r=this.head,n=0;r;)i(r.data,e,n),n+=r.data.length,r=r.next;return e},t}(),a&&a.inspect&&a.inspect.custom&&(t.exports.prototype[a.inspect.custom]=function(){var t=a.inspect({length:this.length});return this.constructor.name+" "+t});},function(t,e){},function(t,e,r){(function(t){function n(t,e){this._id=t,this._clearFn=e;}var i=void 0!==t&&t||"undefined"!=typeof self&&self||window,o=Function.prototype.apply;e.setTimeout=function(){return new n(o.call(setTimeout,i,arguments),clearTimeout)},e.setInterval=function(){return new n(o.call(setInterval,i,arguments),clearInterval)},e.clearTimeout=e.clearInterval=function(t){t&&t.close();},n.prototype.unref=n.prototype.ref=function(){},n.prototype.close=function(){this._clearFn.call(i,this._id);},e.enroll=function(t,e){clearTimeout(t._idleTimeoutId),t._idleTimeout=e;},e.unenroll=function(t){clearTimeout(t._idleTimeoutId),t._idleTimeout=-1;},e._unrefActive=e.active=function(t){clearTimeout(t._idleTimeoutId);var e=t._idleTimeout;e>=0&&(t._idleTimeoutId=setTimeout(function(){t._onTimeout&&t._onTimeout();},e));},r(67),e.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==t&&t.setImmediate||this&&this.setImmediate,e.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==t&&t.clearImmediate||this&&this.clearImmediate;}).call(e,r(0));},function(t,e,r){(function(t,e){!function(t,r){function n(t){"function"!=typeof t&&(t=new Function(""+t));for(var e=new Array(arguments.length-1),r=0;r<e.length;r++)e[r]=arguments[r+1];var n={callback:t,args:e};return c[u]=n,s(u),u++}function i(t){delete c[t];}function o(t){var e=t.callback,n=t.args;switch(n.length){case 0:e();break;case 1:e(n[0]);break;case 2:e(n[0],n[1]);break;case 3:e(n[0],n[1],n[2]);break;default:e.apply(r,n);}}function a(t){if(l)setTimeout(a,0,t);else {var e=c[t];if(e){l=!0;try{o(e);}finally{i(t),l=!1;}}}}if(!t.setImmediate){var s,u=1,c={},l=!1,f=t.document,h=Object.getPrototypeOf&&Object.getPrototypeOf(t);h=h&&h.setTimeout?h:t,"[object process]"==={}.toString.call(t.process)?function(){s=function(t){e.nextTick(function(){a(t);});};}():function(){if(t.postMessage&&!t.importScripts){var e=!0,r=t.onmessage;return t.onmessage=function(){e=!1;},t.postMessage("","*"),t.onmessage=r,e}}()?function(){var e="setImmediate$"+Math.random()+"$",r=function(r){r.source===t&&"string"==typeof r.data&&0===r.data.indexOf(e)&&a(+r.data.slice(e.length));};t.addEventListener?t.addEventListener("message",r,!1):t.attachEvent("onmessage",r),s=function(r){t.postMessage(e+r,"*");};}():t.MessageChannel?function(){var t=new MessageChannel;t.port1.onmessage=function(t){a(t.data);},s=function(e){t.port2.postMessage(e);};}():f&&"onreadystatechange"in f.createElement("script")?function(){var t=f.documentElement;s=function(e){var r=f.createElement("script");r.onreadystatechange=function(){a(e),r.onreadystatechange=null,t.removeChild(r),r=null;},t.appendChild(r);};}():function(){s=function(t){setTimeout(a,0,t);};}(),h.setImmediate=n,h.clearImmediate=i;}}("undefined"==typeof self?void 0===t?this:t:self);}).call(e,r(0),r(7));},function(t,e,r){(function(e){function r(t,e){function r(){if(!i){if(n("throwDeprecation"))throw new Error(e);n("traceDeprecation")?console.trace(e):console.warn(e),i=!0;}return t.apply(this,arguments)}if(n("noDeprecation"))return t;var i=!1;return r}function n(t){try{if(!e.localStorage)return !1}catch(t){return !1}var r=e.localStorage[t];return null!=r&&"true"===String(r).toLowerCase()}t.exports=r;}).call(e,r(0));},function(t,e,r){function n(t){if(!(this instanceof n))return new n(t);i.call(this,t);}t.exports=n;var i=r(39),o=r(8);o.inherits=r(1),o.inherits(n,i),n.prototype._transform=function(t,e,r){r(null,t);};},function(t,e,r){t.exports=r(21);},function(t,e,r){t.exports=r(2);},function(t,e,r){t.exports=r(20).Transform;},function(t,e,r){t.exports=r(20).PassThrough;},function(t,e){},function(t,e,r){function n(t){this._cbs=t||{};}t.exports=n;var i=r(3).EVENTS;Object.keys(i).forEach(function(t){if(0===i[t])t="on"+t,n.prototype[t]=function(){this._cbs[t]&&this._cbs[t]();};else if(1===i[t])t="on"+t,n.prototype[t]=function(e){this._cbs[t]&&this._cbs[t](e);};else {if(2!==i[t])throw Error("wrong number of arguments");t="on"+t,n.prototype[t]=function(e,r){this._cbs[t]&&this._cbs[t](e,r);};}});},function(t,e,r){function n(t,e){return t.children?t.children.map(function(t){return a(t,e)}).join(""):""}function i(t){return Array.isArray(t)?t.map(i).join(""):s(t)||t.type===o.CDATA?i(t.children):t.type===o.Text?t.data:""}var o=r(6),a=r(23),s=o.isTag;t.exports={getInnerHTML:n,getOuterHTML:a,getText:i};},function(t,e){t.exports={Text:"text",Directive:"directive",Comment:"comment",Script:"script",Style:"style",Tag:"tag",CDATA:"cdata",isTag:function(t){return "tag"===t.type||"script"===t.type||"style"===t.type}};},function(t,e,r){var n=r(79),i=r(80);e.decode=function(t,e){return (!e||e<=0?i.XML:i.HTML)(t)},e.decodeStrict=function(t,e){return (!e||e<=0?i.XML:i.HTMLStrict)(t)},e.encode=function(t,e){return (!e||e<=0?n.XML:n.HTML)(t)},e.encodeXML=n.XML,e.encodeHTML4=e.encodeHTML5=e.encodeHTML=n.HTML,e.decodeXML=e.decodeXMLStrict=i.XML,e.decodeHTML4=e.decodeHTML5=e.decodeHTML=i.HTML,e.decodeHTML4Strict=e.decodeHTML5Strict=e.decodeHTMLStrict=i.HTMLStrict,e.escape=n.escape;},function(t,e,r){function n(t){return Object.keys(t).sort().reduce(function(e,r){return e[t[r]]="&"+r+";",e},{})}function i(t){var e=[],r=[];return Object.keys(t).forEach(function(t){1===t.length?e.push("\\"+t):r.push(t);}),r.unshift("["+e.join("")+"]"),new RegExp(r.join("|"),"g")}function o(t){return "&#x"+t.charCodeAt(0).toString(16).toUpperCase()+";"}function a(t){return "&#x"+(1024*(t.charCodeAt(0)-55296)+t.charCodeAt(1)-56320+65536).toString(16).toUpperCase()+";"}function s(t,e){function r(e){return t[e]}return function(t){return t.replace(e,r).replace(d,a).replace(p,o)}}function u(t){return t.replace(g,o).replace(d,a).replace(p,o)}var c=n(r(19)),l=i(c);e.XML=s(c,l);var f=n(r(18)),h=i(f);e.HTML=s(f,h);var p=/[^\0-\x7F]/g,d=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,g=i(c);e.escape=u;},function(t,e,r){function n(t){var e=Object.keys(t).join("|"),r=o(t);e+="|#[xX][\\da-fA-F]+|#\\d+";var n=new RegExp("&(?:"+e+");","g");return function(t){return String(t).replace(n,r)}}function i(t,e){return t<e?1:-1}function o(t){return function(e){return "#"===e.charAt(1)?c("X"===e.charAt(2)||"x"===e.charAt(2)?parseInt(e.substr(3),16):parseInt(e.substr(2),10)):t[e.slice(1,-1)]}}var a=r(18),s=r(33),u=r(19),c=r(32),l=n(u),f=n(a),h=function(){function t(t){return ";"!==t.substr(-1)&&(t+=";"),l(t)}for(var e=Object.keys(s).sort(i),r=Object.keys(a).sort(i),n=0,u=0;n<r.length;n++)e[u]===r[n]?(r[n]+=";?",u++):r[n]+=";";var c=new RegExp("&(?:"+r.join("|")+"|#[xX][\\da-fA-F]+;?|#\\d+;?)","g"),l=o(a);return function(e){return String(e).replace(c,t)}}();t.exports={XML:l,HTML:h,HTMLStrict:f};},function(t,e){var r=e.getChildren=function(t){return t.children},n=e.getParent=function(t){return t.parent};e.getSiblings=function(t){var e=n(t);return e?r(e):[t]},e.getAttributeValue=function(t,e){return t.attribs&&t.attribs[e]},e.hasAttrib=function(t,e){return !!t.attribs&&hasOwnProperty.call(t.attribs,e)},e.getName=function(t){return t.name};},function(t,e){e.removeElement=function(t){if(t.prev&&(t.prev.next=t.next),t.next&&(t.next.prev=t.prev),t.parent){var e=t.parent.children;e.splice(e.lastIndexOf(t),1);}},e.replaceElement=function(t,e){var r=e.prev=t.prev;r&&(r.next=e);var n=e.next=t.next;n&&(n.prev=e);var i=e.parent=t.parent;if(i){var o=i.children;o[o.lastIndexOf(t)]=e;}},e.appendChild=function(t,e){if(e.parent=t,1!==t.children.push(e)){var r=t.children[t.children.length-2];r.next=e,e.prev=r,e.next=null;}},e.append=function(t,e){var r=t.parent,n=t.next;if(e.next=n,e.prev=t,t.next=e,e.parent=r,n){if(n.prev=e,r){var i=r.children;i.splice(i.lastIndexOf(n),0,e);}}else r&&r.children.push(e);},e.prepend=function(t,e){var r=t.parent;if(r){var n=r.children;n.splice(n.lastIndexOf(t),0,e);}t.prev&&(t.prev.next=e),e.parent=r,e.prev=t.prev,e.next=t,t.prev=e;};},function(t,e,r){function n(t,e,r,n){return Array.isArray(e)||(e=[e]),"number"==typeof n&&isFinite(n)||(n=1/0),i(t,e,!1!==r,n)}function i(t,e,r,n){for(var o,a=[],s=0,u=e.length;s<u&&!(t(e[s])&&(a.push(e[s]),--n<=0))&&(o=e[s].children,!(r&&o&&o.length>0&&(o=i(t,o,r,n),a=a.concat(o),(n-=o.length)<=0)));s++);return a}function o(t,e){for(var r=0,n=e.length;r<n;r++)if(t(e[r]))return e[r];return null}function a(t,e){for(var r=null,n=0,i=e.length;n<i&&!r;n++)c(e[n])&&(t(e[n])?r=e[n]:e[n].children.length>0&&(r=a(t,e[n].children)));return r}function s(t,e){for(var r=0,n=e.length;r<n;r++)if(c(e[r])&&(t(e[r])||e[r].children.length>0&&s(t,e[r].children)))return !0;return !1}function u(t,e){for(var r=[],n=0,i=e.length;n<i;n++)c(e[n])&&(t(e[n])&&r.push(e[n]),e[n].children.length>0&&(r=r.concat(u(t,e[n].children))));return r}var c=r(6).isTag;t.exports={filter:n,find:i,findOneChild:o,findOne:a,existsOne:s,findAll:u};},function(t,e,r){function n(t,e){return "function"==typeof e?function(r){return r.attribs&&e(r.attribs[t])}:function(r){return r.attribs&&r.attribs[t]===e}}function i(t,e){return function(r){return t(r)||e(r)}}var o=r(6),a=e.isTag=o.isTag;e.testElement=function(t,e){for(var r in t)if(t.hasOwnProperty(r)){if("tag_name"===r){if(!a(e)||!t.tag_name(e.name))return !1}else if("tag_type"===r){if(!t.tag_type(e.type))return !1}else if("tag_contains"===r){if(a(e)||!t.tag_contains(e.data))return !1}else if(!e.attribs||!t[r](e.attribs[r]))return !1}return !0};var s={tag_name:function(t){return "function"==typeof t?function(e){return a(e)&&t(e.name)}:"*"===t?a:function(e){return a(e)&&e.name===t}},tag_type:function(t){return "function"==typeof t?function(e){return t(e.type)}:function(e){return e.type===t}},tag_contains:function(t){return "function"==typeof t?function(e){return !a(e)&&t(e.data)}:function(e){return !a(e)&&e.data===t}}};e.getElements=function(t,e,r,o){var a=Object.keys(t).map(function(e){var r=t[e];return e in s?s[e](r):n(e,r)});return 0===a.length?[]:this.filter(a.reduce(i),e,r,o)},e.getElementById=function(t,e,r){return Array.isArray(e)||(e=[e]),this.findOne(n("id",t),e,!1!==r)},e.getElementsByTagName=function(t,e,r,n){return this.filter(s.tag_name(t),e,r,n)},e.getElementsByTagType=function(t,e,r,n){return this.filter(s.tag_type(t),e,r,n)};},function(t,e){e.removeSubsets=function(t){for(var e,r,n,i=t.length;--i>-1;){for(e=r=t[i],t[i]=null,n=!0;r;){if(t.indexOf(r)>-1){n=!1,t.splice(i,1);break}r=r.parent;}n&&(t[i]=e);}return t};var r={DISCONNECTED:1,PRECEDING:2,FOLLOWING:4,CONTAINS:8,CONTAINED_BY:16},n=e.compareDocumentPosition=function(t,e){var n,i,o,a,s,u,c=[],l=[];if(t===e)return 0;for(n=t;n;)c.unshift(n),n=n.parent;for(n=e;n;)l.unshift(n),n=n.parent;for(u=0;c[u]===l[u];)u++;return 0===u?r.DISCONNECTED:(i=c[u-1],o=i.children,a=c[u],s=l[u],o.indexOf(a)>o.indexOf(s)?i===e?r.FOLLOWING|r.CONTAINED_BY:r.FOLLOWING:i===t?r.PRECEDING|r.CONTAINS:r.PRECEDING)};e.uniqueSort=function(t){var e,i,o=t.length;for(t=t.slice();--o>-1;)e=t[o],(i=t.indexOf(e))>-1&&i<o&&t.splice(o,1);return t.sort(function(t,e){var i=n(t,e);return i&r.PRECEDING?-1:i&r.FOLLOWING?1:0}),t};},function(t,e,r){function n(t){this._cbs=t||{},this.events=[];}t.exports=n;var i=r(3).EVENTS;Object.keys(i).forEach(function(t){if(0===i[t])t="on"+t,n.prototype[t]=function(){this.events.push([t]),this._cbs[t]&&this._cbs[t]();};else if(1===i[t])t="on"+t,n.prototype[t]=function(e){this.events.push([t,e]),this._cbs[t]&&this._cbs[t](e);};else {if(2!==i[t])throw Error("wrong number of arguments");t="on"+t,n.prototype[t]=function(e,r){this.events.push([t,e,r]),this._cbs[t]&&this._cbs[t](e,r);};}}),n.prototype.onreset=function(){this.events=[],this._cbs.onreset&&this._cbs.onreset();},n.prototype.restart=function(){this._cbs.onreset&&this._cbs.onreset();for(var t=0,e=this.events.length;t<e;t++)if(this._cbs[this.events[t][0]]){var r=this.events[t].length;1===r?this._cbs[this.events[t][0]]():2===r?this._cbs[this.events[t][0]](this.events[t][1]):this._cbs[this.events[t][0]](this.events[t][1],this.events[t][2]);}};},function(t,e,r){var n=r(25),i=r(9),o=i.isTag,a=i.domEach,s=Object.prototype.hasOwnProperty,u=i.camelCase,c=i.cssCase,l=/\s+/,f={forEach:r(16),extend:r(40),some:r(96)},h={null:null,true:!0,false:!1},p=/^(?:autofocus|autoplay|async|checked|controls|defer|disabled|hidden|loop|multiple|open|readonly|required|scoped|selected)$/i,d=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,g=function(t,e){if(t&&o(t))return t.attribs||(t.attribs={}),e?s.call(t.attribs,e)?p.test(e)?e:t.attribs[e]:"option"===t.name&&"value"===e?n.text(t.children):"input"!==t.name||"radio"!==t.attribs.type&&"checkbox"!==t.attribs.type||"value"!==e?void 0:"on":t.attribs},v=function(t,e,r){null===r?w(t,e):t.attribs[e]=r+"";};e.attr=function(t,e){return "object"==typeof t||void 0!==e?"function"==typeof e?a(this,function(r,n){v(n,t,e.call(n,r,n.attribs[t]));}):a(this,function(r,n){o(n)&&("object"==typeof t?f.forEach(t,function(t,e){v(n,e,t);}):v(n,t,e));}):g(this[0],t)};var b=function(t,e){if(t&&o(t))return t.hasOwnProperty(e)?t[e]:p.test(e)?void 0!==g(t,e):g(t,e)},y=function(t,e,r){t[e]=p.test(e)?!!r:r;};e.prop=function(t,e){var r,n=0;if("string"==typeof t&&void 0===e){switch(t){case"style":r=this.css(),f.forEach(r,function(t,e){r[n++]=e;}),r.length=n;break;case"tagName":case"nodeName":r=this[0].name.toUpperCase();break;default:r=b(this[0],t);}return r}if("object"==typeof t||void 0!==e)return "function"==typeof e?a(this,function(r,n){y(n,t,e.call(n,r,b(n,t)));}):a(this,function(r,n){o(n)&&("object"==typeof t?f.forEach(t,function(t,e){y(n,e,t);}):y(n,t,e));})};var m=function(t,e,r){if(t.data||(t.data={}),"object"==typeof e)return f.extend(t.data,e);"string"==typeof e&&void 0!==r?t.data[e]=r:"object"==typeof e&&f.extend(t.data,e);},_=function(t,e){var r,n,i,o,a,l,f,p=1===arguments.length;for(p?(r=Object.keys(t.attribs).filter(function(t){return "data-"===t.slice(0,"data-".length)}),i=r.map(function(t){return u(t.slice("data-".length))})):(r=["data-"+c(e)],i=[e]),l=0,f=r.length;l<f;++l)if(n=r[l],o=i[l],s.call(t.attribs,n)){if(a=t.attribs[n],s.call(h,a))a=h[a];else if(a===String(Number(a)))a=Number(a);else if(d.test(a))try{a=JSON.parse(a);}catch(t){}t.data[o]=a;}return p?t.data:a};e.data=function(t,e){var r=this[0];if(r&&o(r))return r.data||(r.data={}),t?"object"==typeof t||void 0!==e?(a(this,function(r,n){m(n,t,e);}),this):s.call(r.data,t)?r.data[t]:_(r,t):_(r)},e.val=function(t){var e=0===arguments.length,r=this[0];if(r)switch(r.name){case"textarea":return this.text(t);case"input":switch(this.attr("type")){case"radio":return e?this.attr("value"):(this.attr("value",t),this);default:return this.attr("value",t)}case"select":var n,i=this.find("option:selected");if(void 0===i)return;if(!e){if(!this.attr().hasOwnProperty("multiple")&&"object"==typeof t)return this;"object"!=typeof t&&(t=[t]),this.find("option").removeAttr("selected");for(var o=0;o<t.length;o++)this.find('option[value="'+t[o]+'"]').attr("selected","");return this}return n=i.attr("value"),this.attr().hasOwnProperty("multiple")&&(n=[],a(i,function(t,e){n.push(g(e,"value"));})),n;case"option":return e?this.attr("value"):(this.attr("value",t),this)}};var w=function(t,e){t.attribs&&s.call(t.attribs,e)&&delete t.attribs[e];};e.removeAttr=function(t){return a(this,function(e,r){w(r,t);}),this},e.hasClass=function(t){return f.some(this,function(e){var r,n=e.attribs,i=n&&n.class,o=-1;if(i)for(;(o=i.indexOf(t,o+1))>-1;)if(r=o+t.length,(0===o||l.test(i[o-1]))&&(r===i.length||l.test(i[r])))return !0})},e.addClass=function(t){if("function"==typeof t)return a(this,function(r,n){var i=n.attribs.class||"";e.addClass.call([n],t.call(n,r,i));});if(!t||"string"!=typeof t)return this;for(var r=t.split(l),n=this.length,i=0;i<n;i++)if(o(this[i])){var s,u,c=g(this[i],"class");if(c){u=" "+c+" ",s=r.length;for(var f=0;f<s;f++){var h=r[f]+" ";u.indexOf(" "+h)<0&&(u+=h);}v(this[i],"class",u.trim());}else v(this[i],"class",r.join(" ").trim());}return this};var x=function(t){return t?t.trim().split(l):[]};e.removeClass=function(t){var r,n,i;return "function"==typeof t?a(this,function(r,n){e.removeClass.call([n],t.call(n,r,n.attribs.class||""));}):(r=x(t),n=r.length,i=0===arguments.length,a(this,function(t,e){if(o(e))if(i)e.attribs.class="";else {for(var a,s,u=x(e.attribs.class),c=0;c<n;c++)(a=u.indexOf(r[c]))>=0&&(u.splice(a,1),s=!0,c--);s&&(e.attribs.class=u.join(" "));}}))},e.toggleClass=function(t,r){if("function"==typeof t)return a(this,function(n,i){e.toggleClass.call([i],t.call(i,n,i.attribs.class||"",r),r);});if(!t||"string"!=typeof t)return this;for(var n,i,s=t.split(l),u=s.length,c="boolean"==typeof r?r?1:-1:0,f=this.length,h=0;h<f;h++)if(o(this[h])){n=x(this[h].attribs.class);for(var p=0;p<u;p++)i=n.indexOf(s[p]),c>=0&&i<0?n.push(s[p]):c<=0&&i>=0&&n.splice(i,1);this[h].attribs.class=n.join(" ");}return this},e.is=function(t){return !!t&&this.filter(t).length>0};},function(t,e,r){var n=r(89),i=r(90);t.exports=function(t){return i(n(t))},t.exports.parse=n,t.exports.compile=i;},function(t,e){function r(t){if("even"===(t=t.trim().toLowerCase()))return [2,0];if("odd"===t)return [2,1];var e=t.match(n);if(!e)throw new SyntaxError("n-th rule couldn't be parsed ('"+t+"')");var r;return e[1]?(r=parseInt(e[1],10),isNaN(r)&&(r="-"===e[1].charAt(0)?-1:1)):r=0,[r,e[3]?parseInt((e[2]||"")+e[3],10):0]}t.exports=r;var n=/^([+\-]?\d*n)?\s*(?:([+\-]?)\s*(\d+))?$/;},function(t,e,r){function n(t){var e=t[0],r=t[1]-1;if(r<0&&e<=0)return a;if(-1===e)return function(t){return t<=r};if(0===e)return function(t){return t===r};if(1===e)return r<0?o:function(t){return t>=r};var n=r%e;return n<0&&(n+=e),e>1?function(t){return t>=r&&t%e===n}:(e*=-1,function(t){return t<=r&&t%e===n})}t.exports=n;var i=r(10),o=i.trueFunc,a=i.falseFunc;},function(t,e,r){function n(t,e,r){return i(o(t,e,r))}function i(t){return function(e){return g(e)&&t(e)}}function o(t,e,r){return u(p(t,e),e,r)}function a(t){return "pseudo"===t.type&&("scope"===t.name||Array.isArray(t.data)&&t.data.some(function(t){return t.some(a)}))}function s(t,e){var r=!!e&&!!e.length&&e.every(function(t){return t===j||!!k(t)});t.forEach(function(t){if(t.length>0&&c(t[0])&&"descendant"!==t[0].type);else {if(!r||a(t))return;t.unshift(x);}t.unshift(S);});}function u(t,e,r){t=t.filter(function(t){return t.length>0}),t.forEach(b);var n=Array.isArray(r);return r=e&&e.context||r,r&&!n&&(r=[r]),s(t,r),t.map(function(t){return l(t,e,r,n)}).reduce(f,_)}function c(t){return w[t.type]<0}function l(t,e,r,n){var i=n&&"scope"===t[0].name&&"descendant"===t[1].type;return t.reduce(function(t,n,o){return t===_?t:v[n.type](t,n,e,r,i&&1===o)},e&&e.rootFunc||m)}function f(t,e){return e===_||t===m?t:t===_||e===m?e:function(r){return t(r)||e(r)}}function h(t){return t.some(c)}t.exports=n,t.exports.compileUnsafe=o,t.exports.compileToken=u;var p=r(92),d=r(4),g=d.isTag,v=r(93),b=r(94),y=r(10),m=y.trueFunc,_=y.falseFunc,w=r(44),x={type:"descendant"},S={type:"pseudo",name:"scope"},j={},k=d.getParent,E=r(26),A=E.filters,T=d.existsOne,g=d.isTag,O=d.getChildren;A.not=function(t,e,r,n){var i={xmlMode:!(!r||!r.xmlMode),strict:!(!r||!r.strict)};if(i.strict&&(e.length>1||e.some(h)))throw new SyntaxError("complex selectors in :not aren't allowed in strict mode");var o=u(e,i,n);return o===_?t:o===m?_:function(e){return !o(e)&&t(e)}},A.has=function(t,e,r){var n={xmlMode:!(!r||!r.xmlMode),strict:!(!r||!r.strict)},o=e.some(h)?[j]:null,a=u(e,n,o);return a===_?_:a===m?function(e){return O(e).some(g)&&t(e)}:(a=i(a),o?function(e){return t(e)&&(o[0]=e,T(a,O(e)))}:function(e){return t(e)&&T(a,O(e))})},A.matches=function(t,e,r,n){return u(e,{xmlMode:!(!r||!r.xmlMode),strict:!(!r||!r.strict),rootFunc:t},n)};},function(t,e,r){function n(t,e,r){var n="0x"+e-65536;return n!==n||r?e:n<0?String.fromCharCode(n+65536):String.fromCharCode(n>>10|55296,1023&n|56320)}function i(t){return t.replace(l,n)}function o(t){return " "===t||"\n"===t||"\t"===t||"\f"===t||"\r"===t}function a(t,e){var r=[];if(""!==(t=s(r,t+"",e)))throw new SyntaxError("Unmatched selector: "+t);return r}function s(t,e,r){function n(){var t=e.match(c)[0];return e=e.substr(t.length),i(t)}function a(t){for(;o(e.charAt(t));)t++;e=e.substr(t);}var l,y,m,_,w=[],x=!1;for(a(0);""!==e;)if(y=e.charAt(0),o(y))x=!0,a(1);else if(y in p)w.push({type:p[y]}),x=!1,a(1);else if(","===y){if(0===w.length)throw new SyntaxError("empty sub-selector");t.push(w),w=[],x=!1,a(1);}else if(x&&(w.length>0&&w.push({type:"descendant"}),x=!1),"*"===y)e=e.substr(1),w.push({type:"universal"});else if(y in d)e=e.substr(1),w.push({type:"attribute",name:d[y][0],action:d[y][1],value:n(),ignoreCase:!1});else if("["===y){if(e=e.substr(1),!(l=e.match(f)))throw new SyntaxError("Malformed attribute selector: "+e);e=e.substr(l[0].length),m=i(l[1]),r&&("lowerCaseAttributeNames"in r?!r.lowerCaseAttributeNames:r.xmlMode)||(m=m.toLowerCase()),w.push({type:"attribute",name:m,action:h[l[2]],value:i(l[4]||l[5]||""),ignoreCase:!!l[6]});}else if(":"===y){if(":"===e.charAt(1)){e=e.substr(2),w.push({type:"pseudo-element",name:n().toLowerCase()});continue}if(e=e.substr(1),m=n().toLowerCase(),l=null,"("===e.charAt(0))if(m in g){_=e.charAt(1);var S=_ in b;if(e=e.substr(S+1),l=[],e=s(l,e,r),S){if(e.charAt(0)!==_)throw new SyntaxError("unmatched quotes in :"+m);e=e.substr(1);}if(")"!==e.charAt(0))throw new SyntaxError("missing closing parenthesis in :"+m+" "+e);e=e.substr(1);}else {for(var j=1,k=1;k>0&&j<e.length;j++)"("===e.charAt(j)?k++:")"===e.charAt(j)&&k--;if(k)throw new SyntaxError("parenthesis not matched");l=e.substr(1,j-2),e=e.substr(j),m in v&&(_=l.charAt(0),_===l.slice(-1)&&_ in b&&(l=l.slice(1,-1)),l=i(l));}w.push({type:"pseudo",name:m,data:l});}else {if(!c.test(e))return w.length&&"descendant"===w[w.length-1].type&&w.pop(),u(t,w),e;m=n(),r&&("lowerCaseTags"in r?!r.lowerCaseTags:r.xmlMode)||(m=m.toLowerCase()),w.push({type:"tag",name:m});}return u(t,w),e}function u(t,e){if(t.length>0&&0===e.length)throw new SyntaxError("empty sub-selector");t.push(e);}t.exports=a;var c=/^(?:\\.|[\w\-\u00c0-\uFFFF])+/,l=/\\([\da-f]{1,6}\s?|(\s)|.)/gi,f=/^\s*((?:\\.|[\w\u00c0-\uFFFF\-])+)\s*(?:(\S?)=\s*(?:(['"])(.*?)\3|(#?(?:\\.|[\w\u00c0-\uFFFF\-])*)|)|)\s*(i)?\]/,h={__proto__:null,undefined:"exists","":"equals","~":"element","^":"start",$:"end","*":"any","!":"not","|":"hyphen"},p={__proto__:null,">":"child","<":"parent","~":"sibling","+":"adjacent"},d={__proto__:null,"#":["id","equals"],".":["class","element"]},g={__proto__:null,has:!0,not:!0,matches:!0},v={__proto__:null,contains:!0,icontains:!0},b={__proto__:null,'"':!0,"'":!0};},function(t,e,r){var n=r(4),i=n.isTag,o=n.getParent,a=n.getChildren,s=n.getSiblings,u=n.getName;t.exports={__proto__:null,attribute:r(43).compile,pseudo:r(26).compile,tag:function(t,e){var r=e.name;return function(e){return u(e)===r&&t(e)}},descendant:function(t,e,r,n,i){return function(e){if(i&&t(e))return !0;for(var r=!1;!r&&(e=o(e));)r=t(e);return r}},parent:function(t,e,r){function n(e){return i(e)&&t(e)}if(r&&r.strict)throw SyntaxError("Parent selector isn't part of CSS3");return function(t){return a(t).some(n)}},child:function(t){return function(e){var r=o(e);return !!r&&t(r)}},sibling:function(t){return function(e){for(var r=s(e),n=0;n<r.length;n++)if(i(r[n])){if(r[n]===e)break;if(t(r[n]))return !0}return !1}},adjacent:function(t){return function(e){for(var r,n=s(e),o=0;o<n.length;o++)if(i(n[o])){if(n[o]===e)break;r=n[o];}return !!r&&t(r)}},universal:function(t){return t}};},function(t,e,r){function n(t){for(var e=t.map(i),r=1;r<t.length;r++){var n=e[r];if(!(n<0))for(var o=r-1;o>=0&&n<e[o];o--){var a=t[o+1];t[o+1]=t[o],t[o]=a,e[o+1]=e[o],e[o]=n;}}}function i(t){var e=o[t.type];if(e===o.attribute)e=a[t.action],e===a.equals&&"id"===t.name&&(e=9),t.ignoreCase&&(e>>=1);else if(e===o.pseudo)if(t.data)if("has"===t.name||"contains"===t.name)e=0;else if("matches"===t.name||"not"===t.name){e=0;for(var r=0;r<t.data.length;r++)if(1===t.data[r].length){var n=i(t.data[r][0]);if(0===n){e=0;break}n>e&&(e=n);}t.data.length>1&&e>0&&(e-=1);}else e=1;else e=3;return e}t.exports=n;var o=r(44),a={__proto__:null,exists:10,equals:8,not:7,start:6,end:6,any:5,hyphen:4,element:4};},function(t,e,r){(function(t,r){function n(t,e,r){switch(r.length){case 0:return t.call(e);case 1:return t.call(e,r[0]);case 2:return t.call(e,r[0],r[1]);case 3:return t.call(e,r[0],r[1],r[2])}return t.apply(e,r)}function i(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function o(t,e){return null==t?void 0:t[e]}function a(t,e){return "__proto__"==e?void 0:t[e]}function s(t){var e=-1,r=null==t?0:t.length;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function u(){this.__data__=be?be(null):{},this.size=0;}function c(t){var e=this.has(t)&&delete this.__data__[t];return this.size-=e?1:0,e}function l(t){var e=this.__data__;if(be){var r=e[t];return r===xt?void 0:r}return Zt.call(e,t)?e[t]:void 0}function f(t){var e=this.__data__;return be?void 0!==e[t]:Zt.call(e,t)}function h(t,e){var r=this.__data__;return this.size+=this.has(t)?0:1,r[t]=be&&void 0===e?xt:e,this}function p(t){var e=-1,r=null==t?0:t.length;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function d(){this.__data__=[],this.size=0;}function g(t){var e=this.__data__,r=q(e,t);return !(r<0)&&(r==e.length-1?e.pop():le.call(e,r,1),--this.size,!0)}function v(t){var e=this.__data__,r=q(e,t);return r<0?void 0:e[r][1]}function b(t){return q(this.__data__,t)>-1}function y(t,e){var r=this.__data__,n=q(r,t);return n<0?(++this.size,r.push([t,e])):r[n][1]=e,this}function m(t){var e=-1,r=null==t?0:t.length;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function _(){this.size=0,this.__data__={hash:new s,map:new(ve||p),string:new s};}function w(t){var e=J(this,t).delete(t);return this.size-=e?1:0,e}function x(t){return J(this,t).get(t)}function S(t){return J(this,t).has(t)}function j(t,e){var r=J(this,t),n=r.size;return r.set(t,e),this.size+=r.size==n?0:1,this}function k(t){var e=this.__data__=new p(t);this.size=e.size;}function E(){this.__data__=new p,this.size=0;}function A(t){var e=this.__data__,r=e.delete(t);return this.size=e.size,r}function T(t){return this.__data__.get(t)}function O(t){return this.__data__.has(t)}function L(t,e){var r=this.__data__;if(r instanceof p){var n=r.__data__;if(!ve||n.length<wt-1)return n.push([t,e]),this.size=++r.size,this;r=this.__data__=new m(n);}return r.set(t,e),this.size=r.size,this}function C(t,e){var r=Se(t),n=!r&&xe(t),o=!r&&!n&&je(t),a=!r&&!n&&!o&&ke(t),s=r||n||o||a,u=s?i(t.length,String):[],c=u.length;for(var l in t)!e&&!Zt.call(t,l)||s&&("length"==l||o&&("offset"==l||"parent"==l)||a&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||K(l,c))||u.push(l);return u}function B(t,e,r){(void 0===r||ut(t[e],r))&&(void 0!==r||e in t)||P(t,e,r);}function D(t,e,r){var n=t[e];Zt.call(t,e)&&ut(n,r)&&(void 0!==r||e in t)||P(t,e,r);}function q(t,e){for(var r=t.length;r--;)if(ut(t[r][0],e))return r;return -1}function P(t,e,r){"__proto__"==e&&he?he(t,e,{configurable:!0,enumerable:!0,value:r,writable:!0}):t[e]=r;}function M(t){return null==t?void 0===t?Dt:Lt:fe&&fe in Object(t)?X(t):ot(t)}function R(t){return dt(t)&&M(t)==Et}function I(t){return !(!pt(t)||rt(t))&&(ft(t)?re:Pt).test(st(t))}function N(t){return dt(t)&&ht(t.length)&&!!Rt[M(t)]}function U(t){if(!pt(t))return it(t);var e=nt(t),r=[];for(var n in t)("constructor"!=n||!e&&Zt.call(t,n))&&r.push(n);return r}function F(t,e,r,n,i){t!==e&&me(e,function(o,s){if(pt(o))i||(i=new k),z(t,e,s,r,F,n,i);else {var u=n?n(a(t,s),o,s+"",t,e,i):void 0;void 0===u&&(u=o),B(t,s,u);}},bt);}function z(t,e,r,n,i,o,s){var u=a(t,r),c=a(e,r),l=s.get(c);if(l)return void B(t,r,l);var f=o?o(u,c,r+"",t,e,s):void 0,h=void 0===f;if(h){var p=Se(c),d=!p&&je(c),g=!p&&!d&&ke(c);f=c,p||d||g?Se(u)?f=u:lt(u)?f=G(u):d?(h=!1,f=$(c,!0)):g?(h=!1,f=W(c,!0)):f=[]:gt(c)||xe(c)?(f=u,xe(u)?f=vt(u):(!pt(u)||n&&ft(u))&&(f=Z(c))):h=!1;}h&&(s.set(c,f),i(f,c,n,o,s),s.delete(c)),B(t,r,f);}function V(t,e){return we(at(t,e,mt),t+"")}function $(t,e){if(e)return t.slice();var r=t.length,n=ae?ae(r):new t.constructor(r);return t.copy(n),n}function H(t){var e=new t.constructor(t.byteLength);return new oe(e).set(new oe(t)),e}function W(t,e){var r=e?H(t.buffer):t.buffer;return new t.constructor(r,t.byteOffset,t.length)}function G(t,e){var r=-1,n=t.length;for(e||(e=Array(n));++r<n;)e[r]=t[r];return e}function Y(t,e,r,n){var i=!r;r||(r={});for(var o=-1,a=e.length;++o<a;){var s=e[o],u=n?n(r[s],t[s],s,r,t):void 0;void 0===u&&(u=t[s]),i?P(r,s,u):D(r,s,u);}return r}function J(t,e){var r=t.__data__;return et(e)?r["string"==typeof e?"string":"hash"]:r.map}function Q(t,e){var r=o(t,e);return I(r)?r:void 0}function X(t){var e=Zt.call(t,fe),r=t[fe];try{t[fe]=void 0;var n=!0;}catch(t){}var i=te.call(t);return n&&(e?t[fe]=r:delete t[fe]),i}function Z(t){return "function"!=typeof t.constructor||nt(t)?{}:ye(se(t))}function K(t,e){var r=typeof t;return !!(e=null==e?kt:e)&&("number"==r||"symbol"!=r&&Mt.test(t))&&t>-1&&t%1==0&&t<e}function tt(t,e,r){if(!pt(r))return !1;var n=typeof e;return !!("number"==n?ct(r)&&K(e,r.length):"string"==n&&e in r)&&ut(r[e],t)}function et(t){var e=typeof t;return "string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}function rt(t){return !!Kt&&Kt in t}function nt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||Jt)}function it(t){var e=[];if(null!=t)for(var r in Object(t))e.push(r);return e}function ot(t){return te.call(t)}function at(t,e,r){return e=de(void 0===e?t.length-1:e,0),function(){for(var i=arguments,o=-1,a=de(i.length-e,0),s=Array(a);++o<a;)s[o]=i[e+o];o=-1;for(var u=Array(e+1);++o<e;)u[o]=i[o];return u[e]=r(s),n(t,this,u)}}function st(t){if(null!=t){try{return Xt.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function ut(t,e){return t===e||t!==t&&e!==e}function ct(t){return null!=t&&ht(t.length)&&!ft(t)}function lt(t){return dt(t)&&ct(t)}function ft(t){if(!pt(t))return !1;var e=M(t);return e==Tt||e==Ot||e==At||e==Bt}function ht(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=kt}function pt(t){var e=typeof t;return null!=t&&("object"==e||"function"==e)}function dt(t){return null!=t&&"object"==typeof t}function gt(t){if(!dt(t)||M(t)!=Ct)return !1;var e=se(t);if(null===e)return !0;var r=Zt.call(e,"constructor")&&e.constructor;return "function"==typeof r&&r instanceof r&&Xt.call(r)==ee}function vt(t){return Y(t,bt(t))}function bt(t){return ct(t)?C(t,!0):U(t)}function yt(t){return function(){return t}}function mt(t){return t}function _t(){return !1}var wt=200,xt="__lodash_hash_undefined__",St=800,jt=16,kt=9007199254740991,Et="[object Arguments]",At="[object AsyncFunction]",Tt="[object Function]",Ot="[object GeneratorFunction]",Lt="[object Null]",Ct="[object Object]",Bt="[object Proxy]",Dt="[object Undefined]",qt=/[\\^$.*+?()[\]{}|]/g,Pt=/^\[object .+?Constructor\]$/,Mt=/^(?:0|[1-9]\d*)$/,Rt={};Rt["[object Float32Array]"]=Rt["[object Float64Array]"]=Rt["[object Int8Array]"]=Rt["[object Int16Array]"]=Rt["[object Int32Array]"]=Rt["[object Uint8Array]"]=Rt["[object Uint8ClampedArray]"]=Rt["[object Uint16Array]"]=Rt["[object Uint32Array]"]=!0,Rt[Et]=Rt["[object Array]"]=Rt["[object ArrayBuffer]"]=Rt["[object Boolean]"]=Rt["[object DataView]"]=Rt["[object Date]"]=Rt["[object Error]"]=Rt[Tt]=Rt["[object Map]"]=Rt["[object Number]"]=Rt[Ct]=Rt["[object RegExp]"]=Rt["[object Set]"]=Rt["[object String]"]=Rt["[object WeakMap]"]=!1;var It="object"==typeof t&&t&&t.Object===Object&&t,Nt="object"==typeof self&&self&&self.Object===Object&&self,Ut=It||Nt||Function("return this")(),Ft="object"==typeof e&&e&&!e.nodeType&&e,zt=Ft&&"object"==typeof r&&r&&!r.nodeType&&r,Vt=zt&&zt.exports===Ft,$t=Vt&&It.process,Ht=function(){try{return $t&&$t.binding&&$t.binding("util")}catch(t){}}(),Wt=Ht&&Ht.isTypedArray,Gt=Array.prototype,Yt=Function.prototype,Jt=Object.prototype,Qt=Ut["__core-js_shared__"],Xt=Yt.toString,Zt=Jt.hasOwnProperty,Kt=function(){var t=/[^.]+$/.exec(Qt&&Qt.keys&&Qt.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),te=Jt.toString,ee=Xt.call(Object),re=RegExp("^"+Xt.call(Zt).replace(qt,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),ne=Vt?Ut.Buffer:void 0,ie=Ut.Symbol,oe=Ut.Uint8Array,ae=ne?ne.allocUnsafe:void 0,se=function(t,e){return function(r){return t(e(r))}}(Object.getPrototypeOf,Object),ue=Object.create,ce=Jt.propertyIsEnumerable,le=Gt.splice,fe=ie?ie.toStringTag:void 0,he=function(){try{var t=Q(Object,"defineProperty");return t({},"",{}),t}catch(t){}}(),pe=ne?ne.isBuffer:void 0,de=Math.max,ge=Date.now,ve=Q(Ut,"Map"),be=Q(Object,"create"),ye=function(){function t(){}return function(e){if(!pt(e))return {};if(ue)return ue(e);t.prototype=e;var r=new t;return t.prototype=void 0,r}}();s.prototype.clear=u,s.prototype.delete=c,s.prototype.get=l,s.prototype.has=f,s.prototype.set=h,p.prototype.clear=d,p.prototype.delete=g,p.prototype.get=v,p.prototype.has=b,p.prototype.set=y,m.prototype.clear=_,m.prototype.delete=w,m.prototype.get=x,m.prototype.has=S,m.prototype.set=j,k.prototype.clear=E,k.prototype.delete=A,k.prototype.get=T,k.prototype.has=O,k.prototype.set=L;var me=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),_e=he?function(t,e){return he(t,"toString",{configurable:!0,enumerable:!1,value:yt(e),writable:!0})}:mt,we=function(t){var e=0,r=0;return function(){var n=ge(),i=jt-(n-r);if(r=n,i>0){if(++e>=St)return arguments[0]}else e=0;return t.apply(void 0,arguments)}}(_e),xe=R(function(){return arguments}())?R:function(t){return dt(t)&&Zt.call(t,"callee")&&!ce.call(t,"callee")},Se=Array.isArray,je=pe||_t,ke=Wt?function(t){return function(e){return t(e)}}(Wt):N,Ee=function(t){return V(function(e,r){var n=-1,i=r.length,o=i>1?r[i-1]:void 0,a=i>2?r[2]:void 0;for(o=t.length>3&&"function"==typeof o?(i--,o):void 0,a&&tt(r[0],r[1],a)&&(o=i<3?void 0:o,i=1),e=Object(e);++n<i;){var s=r[n];s&&t(e,s,n,o);}return e})}(function(t,e,r){F(t,e,r);});r.exports=Ee;}).call(e,r(0),r(5)(t));},function(t,e,r){(function(t,r){function n(t,e){for(var r=-1,n=t?t.length:0;++r<n;)if(e(t[r],r,t))return !0;return !1}function i(t){return function(e){return null==e?void 0:e[t]}}function o(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function a(t,e){return null==t?void 0:t[e]}function s(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"");}catch(t){}return e}function u(t){var e=-1,r=Array(t.size);return t.forEach(function(t,n){r[++e]=[n,t];}),r}function c(t){var e=-1,r=Array(t.size);return t.forEach(function(t){r[++e]=t;}),r}function l(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function f(){this.__data__=ze?ze(null):{};}function h(t){return this.has(t)&&delete this.__data__[t]}function p(t){var e=this.__data__;if(ze){var r=e[t];return r===It?void 0:r}return Oe.call(e,t)?e[t]:void 0}function d(t){var e=this.__data__;return ze?void 0!==e[t]:Oe.call(e,t)}function g(t,e){return this.__data__[t]=ze&&void 0===e?It:e,this}function v(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function b(){this.__data__=[];}function y(t){var e=this.__data__,r=I(e,t);return !(r<0)&&(r==e.length-1?e.pop():Pe.call(e,r,1),!0)}function m(t){var e=this.__data__,r=I(e,t);return r<0?void 0:e[r][1]}function _(t){return I(this.__data__,t)>-1}function w(t,e){var r=this.__data__,n=I(r,t);return n<0?r.push([t,e]):r[n][1]=e,this}function x(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function S(){this.__data__={hash:new l,map:new(Ie||v),string:new l};}function j(t){return ot(this,t).delete(t)}function k(t){return ot(this,t).get(t)}function E(t){return ot(this,t).has(t)}function A(t,e){return ot(this,t).set(t,e),this}function T(t){var e=-1,r=t?t.length:0;for(this.__data__=new x;++e<r;)this.add(t[e]);}function O(t){return this.__data__.set(t,It),this}function L(t){return this.__data__.has(t)}function C(t){this.__data__=new v(t);}function B(){this.__data__=new v;}function D(t){return this.__data__.delete(t)}function q(t){return this.__data__.get(t)}function P(t){return this.__data__.has(t)}function M(t,e){var r=this.__data__;if(r instanceof v){var n=r.__data__;if(!Ie||n.length<Mt-1)return n.push([t,e]),this;r=this.__data__=new x(n);}return r.set(t,e),this}function R(t,e){var r=er(t)||xt(t)?o(t.length,String):[],n=r.length,i=!!n;for(var a in t)!e&&!Oe.call(t,a)||i&&("length"==a||ct(a,n))||r.push(a);return r}function I(t,e){for(var r=t.length;r--;)if(wt(t[r][0],e))return r;return -1}function N(t,e){return t&&Ze(t,e,Dt)}function U(t,e){e=ft(e,t)?[e]:et(e);for(var r=0,n=e.length;null!=t&&r<n;)t=t[bt(e[r++])];return r&&r==n?t:void 0}function F(t){return Le.call(t)}function z(t,e){return null!=t&&e in Object(t)}function V(t,e,r,n,i){return t===e||(null==t||null==e||!At(t)&&!Tt(e)?t!==t&&e!==e:$(t,e,V,r,n,i))}function $(t,e,r,n,i,o){var a=er(t),u=er(e),c=$t,l=$t;a||(c=Ke(t),c=c==Vt?Zt:c),u||(l=Ke(e),l=l==Vt?Zt:l);var f=c==Zt&&!s(t),h=l==Zt&&!s(e),p=c==l;if(p&&!f)return o||(o=new C),a||rr(t)?rt(t,e,r,n,i,o):nt(t,e,c,r,n,i,o);if(!(i&Ut)){var d=f&&Oe.call(t,"__wrapped__"),g=h&&Oe.call(e,"__wrapped__");if(d||g){var v=d?t.value():t,b=g?e.value():e;return o||(o=new C),r(v,b,n,i,o)}}return !!p&&(o||(o=new C),it(t,e,r,n,i,o))}function H(t,e,r,n){var i=r.length,o=i,a=!n;if(null==t)return !o;for(t=Object(t);i--;){var s=r[i];if(a&&s[2]?s[1]!==t[s[0]]:!(s[0]in t))return !1}for(;++i<o;){s=r[i];var u=s[0],c=t[u],l=s[1];if(a&&s[2]){if(void 0===c&&!(u in t))return !1}else {var f=new C;if(n)var h=n(c,l,u,t,e,f);if(!(void 0===h?V(l,c,n,Nt|Ut,f):h))return !1}}return !0}function W(t){return !(!At(t)||pt(t))&&(kt(t)||s(t)?Ce:fe).test(yt(t))}function G(t){return Tt(t)&&Et(t.length)&&!!pe[Le.call(t)]}function Y(t){return "function"==typeof t?t:null==t?qt:"object"==typeof t?er(t)?X(t[0],t[1]):Q(t):Pt(t)}function J(t){if(!dt(t))return Me(t);var e=[];for(var r in Object(t))Oe.call(t,r)&&"constructor"!=r&&e.push(r);return e}function Q(t){var e=at(t);return 1==e.length&&e[0][2]?vt(e[0][0],e[0][1]):function(r){return r===t||H(r,t,e)}}function X(t,e){return ft(t)&&gt(e)?vt(bt(t),e):function(r){var n=Ct(r,t);return void 0===n&&n===e?Bt(r,t):V(e,n,void 0,Nt|Ut)}}function Z(t){return function(e){return U(e,t)}}function K(t,e){var r;return Xe(t,function(t,n,i){return !(r=e(t,n,i))}),!!r}function tt(t){if("string"==typeof t)return t;if(Ot(t))return Qe?Qe.call(t):"";var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function et(t){return er(t)?t:tr(t)}function rt(t,e,r,i,o,a){var s=o&Ut,u=t.length,c=e.length;if(u!=c&&!(s&&c>u))return !1;var l=a.get(t);if(l&&a.get(e))return l==e;var f=-1,h=!0,p=o&Nt?new T:void 0;for(a.set(t,e),a.set(e,t);++f<u;){var d=t[f],g=e[f];if(i)var v=s?i(g,d,f,e,t,a):i(d,g,f,t,e,a);if(void 0!==v){if(v)continue;h=!1;break}if(p){if(!n(e,function(t,e){if(!p.has(e)&&(d===t||r(d,t,i,o,a)))return p.add(e)})){h=!1;break}}else if(d!==g&&!r(d,g,i,o,a)){h=!1;break}}return a.delete(t),a.delete(e),h}function nt(t,e,r,n,i,o,a){switch(r){case ie:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return !1;t=t.buffer,e=e.buffer;case ne:return !(t.byteLength!=e.byteLength||!n(new De(t),new De(e)));case Ht:case Wt:case Xt:return wt(+t,+e);case Gt:return t.name==e.name&&t.message==e.message;case Kt:case ee:return t==e+"";case Qt:var s=u;case te:var l=o&Ut;if(s||(s=c),t.size!=e.size&&!l)return !1;var f=a.get(t);if(f)return f==e;o|=Nt,a.set(t,e);var h=rt(s(t),s(e),n,i,o,a);return a.delete(t),h;case re:if(Je)return Je.call(t)==Je.call(e)}return !1}function it(t,e,r,n,i,o){var a=i&Ut,s=Dt(t),u=s.length;if(u!=Dt(e).length&&!a)return !1;for(var c=u;c--;){var l=s[c];if(!(a?l in e:Oe.call(e,l)))return !1}var f=o.get(t);if(f&&o.get(e))return f==e;var h=!0;o.set(t,e),o.set(e,t);for(var p=a;++c<u;){l=s[c];var d=t[l],g=e[l];if(n)var v=a?n(g,d,l,e,t,o):n(d,g,l,t,e,o);if(!(void 0===v?d===g||r(d,g,n,i,o):v)){h=!1;break}p||(p="constructor"==l);}if(h&&!p){var b=t.constructor,y=e.constructor;b!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof b&&b instanceof b&&"function"==typeof y&&y instanceof y)&&(h=!1);}return o.delete(t),o.delete(e),h}function ot(t,e){var r=t.__data__;return ht(e)?r["string"==typeof e?"string":"hash"]:r.map}function at(t){for(var e=Dt(t),r=e.length;r--;){var n=e[r],i=t[n];e[r]=[n,i,gt(i)];}return e}function st(t,e){var r=a(t,e);return W(r)?r:void 0}function ut(t,e,r){e=ft(e,t)?[e]:et(e);for(var n,i=-1,o=e.length;++i<o;){var a=bt(e[i]);if(!(n=null!=t&&r(t,a)))break;t=t[a];}if(n)return n;var o=t?t.length:0;return !!o&&Et(o)&&ct(a,o)&&(er(t)||xt(t))}function ct(t,e){return !!(e=null==e?zt:e)&&("number"==typeof t||he.test(t))&&t>-1&&t%1==0&&t<e}function lt(t,e,r){if(!At(r))return !1;var n=typeof e;return !!("number"==n?St(r)&&ct(e,r.length):"string"==n&&e in r)&&wt(r[e],t)}function ft(t,e){if(er(t))return !1;var r=typeof t;return !("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=t&&!Ot(t))||(ae.test(t)||!oe.test(t)||null!=e&&t in Object(e))}function ht(t){var e=typeof t;return "string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}function pt(t){return !!Ae&&Ae in t}function dt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||ke)}function gt(t){return t===t&&!At(t)}function vt(t,e){return function(r){return null!=r&&(r[t]===e&&(void 0!==e||t in Object(r)))}}function bt(t){if("string"==typeof t||Ot(t))return t;var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function yt(t){if(null!=t){try{return Te.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function mt(t,e,r){var i=er(t)?n:K;return r&&lt(t,e,r)&&(e=void 0),i(t,Y(e))}function _t(t,e){if("function"!=typeof t||e&&"function"!=typeof e)throw new TypeError(Rt);var r=function(){var n=arguments,i=e?e.apply(this,n):n[0],o=r.cache;if(o.has(i))return o.get(i);var a=t.apply(this,n);return r.cache=o.set(i,a),a};return r.cache=new(_t.Cache||x),r}function wt(t,e){return t===e||t!==t&&e!==e}function xt(t){return jt(t)&&Oe.call(t,"callee")&&(!qe.call(t,"callee")||Le.call(t)==Vt)}function St(t){return null!=t&&Et(t.length)&&!kt(t)}function jt(t){return Tt(t)&&St(t)}function kt(t){var e=At(t)?Le.call(t):"";return e==Yt||e==Jt}function Et(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=zt}function At(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function Tt(t){return !!t&&"object"==typeof t}function Ot(t){return "symbol"==typeof t||Tt(t)&&Le.call(t)==re}function Lt(t){return null==t?"":tt(t)}function Ct(t,e,r){var n=null==t?void 0:U(t,e);return void 0===n?r:n}function Bt(t,e){return null!=t&&ut(t,e,z)}function Dt(t){return St(t)?R(t):J(t)}function qt(t){return t}function Pt(t){return ft(t)?i(bt(t)):Z(t)}var Mt=200,Rt="Expected a function",It="__lodash_hash_undefined__",Nt=1,Ut=2,Ft=1/0,zt=9007199254740991,Vt="[object Arguments]",$t="[object Array]",Ht="[object Boolean]",Wt="[object Date]",Gt="[object Error]",Yt="[object Function]",Jt="[object GeneratorFunction]",Qt="[object Map]",Xt="[object Number]",Zt="[object Object]",Kt="[object RegExp]",te="[object Set]",ee="[object String]",re="[object Symbol]",ne="[object ArrayBuffer]",ie="[object DataView]",oe=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,ae=/^\w*$/,se=/^\./,ue=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,ce=/[\\^$.*+?()[\]{}|]/g,le=/\\(\\)?/g,fe=/^\[object .+?Constructor\]$/,he=/^(?:0|[1-9]\d*)$/,pe={};pe["[object Float32Array]"]=pe["[object Float64Array]"]=pe["[object Int8Array]"]=pe["[object Int16Array]"]=pe["[object Int32Array]"]=pe["[object Uint8Array]"]=pe["[object Uint8ClampedArray]"]=pe["[object Uint16Array]"]=pe["[object Uint32Array]"]=!0,pe[Vt]=pe[$t]=pe[ne]=pe[Ht]=pe[ie]=pe[Wt]=pe[Gt]=pe[Yt]=pe[Qt]=pe[Xt]=pe[Zt]=pe[Kt]=pe[te]=pe[ee]=pe["[object WeakMap]"]=!1;var de="object"==typeof t&&t&&t.Object===Object&&t,ge="object"==typeof self&&self&&self.Object===Object&&self,ve=de||ge||Function("return this")(),be="object"==typeof e&&e&&!e.nodeType&&e,ye=be&&"object"==typeof r&&r&&!r.nodeType&&r,me=ye&&ye.exports===be,_e=me&&de.process,we=function(){try{return _e&&_e.binding("util")}catch(t){}}(),xe=we&&we.isTypedArray,Se=Array.prototype,je=Function.prototype,ke=Object.prototype,Ee=ve["__core-js_shared__"],Ae=function(){var t=/[^.]+$/.exec(Ee&&Ee.keys&&Ee.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),Te=je.toString,Oe=ke.hasOwnProperty,Le=ke.toString,Ce=RegExp("^"+Te.call(Oe).replace(ce,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Be=ve.Symbol,De=ve.Uint8Array,qe=ke.propertyIsEnumerable,Pe=Se.splice,Me=function(t,e){return function(r){return t(e(r))}}(Object.keys,Object),Re=st(ve,"DataView"),Ie=st(ve,"Map"),Ne=st(ve,"Promise"),Ue=st(ve,"Set"),Fe=st(ve,"WeakMap"),ze=st(Object,"create"),Ve=yt(Re),$e=yt(Ie),He=yt(Ne),We=yt(Ue),Ge=yt(Fe),Ye=Be?Be.prototype:void 0,Je=Ye?Ye.valueOf:void 0,Qe=Ye?Ye.toString:void 0;l.prototype.clear=f,l.prototype.delete=h,l.prototype.get=p,l.prototype.has=d,l.prototype.set=g,v.prototype.clear=b,v.prototype.delete=y,v.prototype.get=m,v.prototype.has=_,v.prototype.set=w,x.prototype.clear=S,x.prototype.delete=j,x.prototype.get=k,x.prototype.has=E,x.prototype.set=A,T.prototype.add=T.prototype.push=O,T.prototype.has=L,C.prototype.clear=B,C.prototype.delete=D,C.prototype.get=q,C.prototype.has=P,C.prototype.set=M;var Xe=function(t,e){return function(r,n){if(null==r)return r;if(!St(r))return t(r,n);for(var i=r.length,o=e?i:-1,a=Object(r);(e?o--:++o<i)&&!1!==n(a[o],o,a););return r}}(N),Ze=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),Ke=F;(Re&&Ke(new Re(new ArrayBuffer(1)))!=ie||Ie&&Ke(new Ie)!=Qt||Ne&&"[object Promise]"!=Ke(Ne.resolve())||Ue&&Ke(new Ue)!=te||Fe&&"[object WeakMap]"!=Ke(new Fe))&&(Ke=function(t){var e=Le.call(t),r=e==Zt?t.constructor:void 0,n=r?yt(r):void 0;if(n)switch(n){case Ve:return ie;case $e:return Qt;case He:return "[object Promise]";case We:return te;case Ge:return "[object WeakMap]"}return e});var tr=_t(function(t){t=Lt(t);var e=[];return se.test(t)&&e.push(""),t.replace(ue,function(t,r,n,i){e.push(n?i.replace(le,"$1"):r||t);}),e});_t.Cache=x;var er=Array.isArray,rr=xe?function(t){return function(e){return t(e)}}(xe):G;r.exports=mt;}).call(e,r(0),r(5)(t));},function(t,e,r){function n(t,r,n,i){for(var o=[];r&&o.length<i;)n&&!e.filter.call([r],n,t).length||o.push(r),r=r.parent;return o}var i=r(42),o=r(9),a=o.domEach,s=r(3).DomUtils.uniqueSort,u=o.isTag,c={bind:r(24),forEach:r(16),reject:r(98),filter:r(99),reduce:r(100)};e.find=function(t){var e,r=c.reduce(this,function(t,e){return t.concat(c.filter(e.children,u))},[]),n=this.constructor.contains;if(t&&"string"!=typeof t)return e=t.cheerio?t.get():[t],this._make(e.filter(function(t){var e,r;for(e=0,r=this.length;e<r;++e)if(n(this[e],t))return !0},this));var o={__proto__:this.options,context:this.toArray()};return this._make(i(t,r,o))},e.parent=function(t){var r=[];return a(this,function(t,e){var n=e.parent;n&&r.indexOf(n)<0&&r.push(n);}),arguments.length&&(r=e.filter.call(r,t,this)),this._make(r)},e.parents=function(t){var e=[];return this.get().reverse().forEach(function(r){n(this,r.parent,t,1/0).forEach(function(t){-1===e.indexOf(t)&&e.push(t);});},this),this._make(e)},e.parentsUntil=function(t,e){var r,n,o=[];return "string"==typeof t?r=i(t,this.parents().toArray(),this.options)[0]:t&&t.cheerio?n=t.toArray():t&&(r=t),this.toArray().reverse().forEach(function(t){for(;(t=t.parent)&&(r&&t!==r||n&&-1===n.indexOf(t)||!r&&!n);)u(t)&&-1===o.indexOf(t)&&o.push(t);},this),this._make(e?i(e,o,this.options):o)},e.closest=function(t){var e=[];return t?(a(this,function(r,i){var o=n(this,i,t,1)[0];o&&e.indexOf(o)<0&&e.push(o);}.bind(this)),this._make(e)):this._make(e)},e.next=function(t){if(!this[0])return this;var r=[];return c.forEach(this,function(t){for(;t=t.next;)if(u(t))return void r.push(t)}),t?e.filter.call(r,t,this):this._make(r)},e.nextAll=function(t){if(!this[0])return this;var r=[];return c.forEach(this,function(t){for(;t=t.next;)u(t)&&-1===r.indexOf(t)&&r.push(t);}),t?e.filter.call(r,t,this):this._make(r)},e.nextUntil=function(t,r){if(!this[0])return this;var n,o,a=[];return "string"==typeof t?n=i(t,this.nextAll().get(),this.options)[0]:t&&t.cheerio?o=t.get():t&&(n=t),c.forEach(this,function(t){for(;(t=t.next)&&(n&&t!==n||o&&-1===o.indexOf(t)||!n&&!o);)u(t)&&-1===a.indexOf(t)&&a.push(t);}),r?e.filter.call(a,r,this):this._make(a)},e.prev=function(t){if(!this[0])return this;var r=[];return c.forEach(this,function(t){for(;t=t.prev;)if(u(t))return void r.push(t)}),t?e.filter.call(r,t,this):this._make(r)},e.prevAll=function(t){if(!this[0])return this;var r=[];return c.forEach(this,function(t){for(;t=t.prev;)u(t)&&-1===r.indexOf(t)&&r.push(t);}),t?e.filter.call(r,t,this):this._make(r)},e.prevUntil=function(t,r){if(!this[0])return this;var n,o,a=[];return "string"==typeof t?n=i(t,this.prevAll().get(),this.options)[0]:t&&t.cheerio?o=t.get():t&&(n=t),c.forEach(this,function(t){for(;(t=t.prev)&&(n&&t!==n||o&&-1===o.indexOf(t)||!n&&!o);)u(t)&&-1===a.indexOf(t)&&a.push(t);}),r?e.filter.call(a,r,this):this._make(a)},e.siblings=function(t){var r=this.parent(),n=c.filter(r?r.children():this.siblingsAndMe(),c.bind(function(t){return u(t)&&!this.is(t)},this));return void 0!==t?e.filter.call(n,t,this):this._make(n)},e.children=function(t){var r=c.reduce(this,function(t,e){return t.concat(c.filter(e.children,u))},[]);return void 0===t?this._make(r):e.filter.call(r,t,this)},e.contents=function(){return this._make(c.reduce(this,function(t,e){return t.push.apply(t,e.children),t},[]))},e.each=function(t){for(var e=0,r=this.length;e<r&&!1!==t.call(this[e],e,this[e]);)++e;return this},e.map=function(t){return this._make(c.reduce(this,function(e,r,n){var i=t.call(r,n,r);return null==i?e:e.concat(i)},[]))};var l=function(t){return function(e,r){var n;return r=r||this,n="string"==typeof e?i.compile(e,r.options):"function"==typeof e?function(t,r){return e.call(t,r,t)}:e.cheerio?e.is.bind(e):function(t){return e===t},r._make(t(this,n))}};e.filter=l(c.filter),e.not=l(c.reject),e.has=function(t){var r=this;return e.filter.call(this,function(){return r._make(this).find(t).length>0})},e.first=function(){return this.length>1?this._make(this[0]):this},e.last=function(){return this.length>1?this._make(this[this.length-1]):this},e.eq=function(t){return 0===(t=+t)&&this.length<=1?this:(t<0&&(t=this.length+t),this[t]?this._make(this[t]):this._make([]))},e.get=function(t){return null==t?Array.prototype.slice.call(this):this[t<0?this.length+t:t]},e.index=function(t){var e,r;return 0===arguments.length?(e=this.parent().children(),r=this[0]):"string"==typeof t?(e=this._make(t),r=this[0]):(e=this,r=t.cheerio?t[0]:t),e.get().indexOf(r)},e.slice=function(){return this._make([].slice.apply(this,arguments))},e.end=function(){return this.prevObject||this._make([])},e.add=function(t,e){for(var r=this._make(t,e),n=s(r.get().concat(this.get())),i=0;i<n.length;++i)r[i]=n[i];return r.length=n.length,r},e.addBack=function(t){return this.add(arguments.length?this.prevObject.filter(t):this.prevObject)};},function(t,e,r){(function(t,r){function n(t,e){for(var r=-1,n=t?t.length:0,i=0,o=[];++r<n;){var a=t[r];e(a,r,t)&&(o[i++]=a);}return o}function i(t,e){for(var r=-1,n=t?t.length:0;++r<n;)if(e(t[r],r,t))return !0;return !1}function o(t){return function(e){return null==e?void 0:e[t]}}function a(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function s(t,e){return null==t?void 0:t[e]}function u(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"");}catch(t){}return e}function c(t){var e=-1,r=Array(t.size);return t.forEach(function(t,n){r[++e]=[n,t];}),r}function l(t){var e=-1,r=Array(t.size);return t.forEach(function(t){r[++e]=t;}),r}function f(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function h(){this.__data__=Ve?Ve(null):{};}function p(t){return this.has(t)&&delete this.__data__[t]}function d(t){var e=this.__data__;if(Ve){var r=e[t];return r===Nt?void 0:r}return Le.call(e,t)?e[t]:void 0}function g(t){var e=this.__data__;return Ve?void 0!==e[t]:Le.call(e,t)}function v(t,e){return this.__data__[t]=Ve&&void 0===e?Nt:e,this}function b(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function y(){this.__data__=[];}function m(t){var e=this.__data__,r=N(e,t);return !(r<0)&&(r==e.length-1?e.pop():Me.call(e,r,1),!0)}function _(t){var e=this.__data__,r=N(e,t);return r<0?void 0:e[r][1]}function w(t){return N(this.__data__,t)>-1}function x(t,e){var r=this.__data__,n=N(r,t);return n<0?r.push([t,e]):r[n][1]=e,this}function S(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function j(){this.__data__={hash:new f,map:new(Ne||b),string:new f};}function k(t){return at(this,t).delete(t)}function E(t){return at(this,t).get(t)}function A(t){return at(this,t).has(t)}function T(t,e){return at(this,t).set(t,e),this}function O(t){var e=-1,r=t?t.length:0;for(this.__data__=new S;++e<r;)this.add(t[e]);}function L(t){return this.__data__.set(t,Nt),this}function C(t){return this.__data__.has(t)}function B(t){this.__data__=new b(t);}function D(){this.__data__=new b;}function q(t){return this.__data__.delete(t)}function P(t){return this.__data__.get(t)}function M(t){return this.__data__.has(t)}function R(t,e){var r=this.__data__;if(r instanceof b){var n=r.__data__;if(!Ne||n.length<Rt-1)return n.push([t,e]),this;r=this.__data__=new S(n);}return r.set(t,e),this}function I(t,e){var r=rr(t)||St(t)?a(t.length,String):[],n=r.length,i=!!n;for(var o in t)!e&&!Le.call(t,o)||i&&("length"==o||lt(o,n))||r.push(o);return r}function N(t,e){for(var r=t.length;r--;)if(xt(t[r][0],e))return r;return -1}function U(t,e){var r=[];return Ze(t,function(t,n,i){e(t,n,i)&&r.push(t);}),r}function F(t,e){return t&&Ke(t,e,qt)}function z(t,e){e=ft(e,t)?[e]:rt(e);for(var r=0,n=e.length;null!=t&&r<n;)t=t[bt(e[r++])];return r&&r==n?t:void 0}function V(t){return Ce.call(t)}function $(t,e){return null!=t&&e in Object(t)}function H(t,e,r,n,i){return t===e||(null==t||null==e||!Tt(t)&&!Ot(e)?t!==t&&e!==e:W(t,e,H,r,n,i))}function W(t,e,r,n,i,o){var a=rr(t),s=rr(e),c=Ht,l=Ht;a||(c=tr(t),c=c==$t?Kt:c),s||(l=tr(e),l=l==$t?Kt:l);var f=c==Kt&&!u(t),h=l==Kt&&!u(e),p=c==l;if(p&&!f)return o||(o=new B),a||nr(t)?nt(t,e,r,n,i,o):it(t,e,c,r,n,i,o);if(!(i&Ft)){var d=f&&Le.call(t,"__wrapped__"),g=h&&Le.call(e,"__wrapped__");if(d||g){var v=d?t.value():t,b=g?e.value():e;return o||(o=new B),r(v,b,n,i,o)}}return !!p&&(o||(o=new B),ot(t,e,r,n,i,o))}function G(t,e,r,n){var i=r.length,o=i,a=!n;if(null==t)return !o;for(t=Object(t);i--;){var s=r[i];if(a&&s[2]?s[1]!==t[s[0]]:!(s[0]in t))return !1}for(;++i<o;){s=r[i];var u=s[0],c=t[u],l=s[1];if(a&&s[2]){if(void 0===c&&!(u in t))return !1}else {var f=new B;if(n)var h=n(c,l,u,t,e,f);if(!(void 0===h?H(l,c,n,Ut|Ft,f):h))return !1}}return !0}function Y(t){return !(!Tt(t)||pt(t))&&(Et(t)||u(t)?Be:he).test(yt(t))}function J(t){return Ot(t)&&At(t.length)&&!!de[Ce.call(t)]}function Q(t){return "function"==typeof t?t:null==t?Pt:"object"==typeof t?rr(t)?K(t[0],t[1]):Z(t):Mt(t)}function X(t){if(!dt(t))return Re(t);var e=[];for(var r in Object(t))Le.call(t,r)&&"constructor"!=r&&e.push(r);return e}function Z(t){var e=st(t);return 1==e.length&&e[0][2]?vt(e[0][0],e[0][1]):function(r){return r===t||G(r,t,e)}}function K(t,e){return ft(t)&&gt(e)?vt(bt(t),e):function(r){var n=Bt(r,t);return void 0===n&&n===e?Dt(r,t):H(e,n,void 0,Ut|Ft)}}function tt(t){return function(e){return z(e,t)}}function et(t){if("string"==typeof t)return t;if(Lt(t))return Xe?Xe.call(t):"";var e=t+"";return "0"==e&&1/t==-zt?"-0":e}function rt(t){return rr(t)?t:er(t)}function nt(t,e,r,n,o,a){var s=o&Ft,u=t.length,c=e.length;if(u!=c&&!(s&&c>u))return !1;var l=a.get(t);if(l&&a.get(e))return l==e;var f=-1,h=!0,p=o&Ut?new O:void 0;for(a.set(t,e),a.set(e,t);++f<u;){var d=t[f],g=e[f];if(n)var v=s?n(g,d,f,e,t,a):n(d,g,f,t,e,a);if(void 0!==v){if(v)continue;h=!1;break}if(p){if(!i(e,function(t,e){if(!p.has(e)&&(d===t||r(d,t,n,o,a)))return p.add(e)})){h=!1;break}}else if(d!==g&&!r(d,g,n,o,a)){h=!1;break}}return a.delete(t),a.delete(e),h}function it(t,e,r,n,i,o,a){switch(r){case oe:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return !1;t=t.buffer,e=e.buffer;case ie:return !(t.byteLength!=e.byteLength||!n(new qe(t),new qe(e)));case Wt:case Gt:case Zt:return xt(+t,+e);case Yt:return t.name==e.name&&t.message==e.message;case te:case re:return t==e+"";case Xt:var s=c;case ee:var u=o&Ft;if(s||(s=l),t.size!=e.size&&!u)return !1;var f=a.get(t);if(f)return f==e;o|=Ut,a.set(t,e);var h=nt(s(t),s(e),n,i,o,a);return a.delete(t),h;case ne:if(Qe)return Qe.call(t)==Qe.call(e)}return !1}function ot(t,e,r,n,i,o){var a=i&Ft,s=qt(t),u=s.length;if(u!=qt(e).length&&!a)return !1;for(var c=u;c--;){var l=s[c];if(!(a?l in e:Le.call(e,l)))return !1}var f=o.get(t);if(f&&o.get(e))return f==e;var h=!0;o.set(t,e),o.set(e,t);for(var p=a;++c<u;){l=s[c];var d=t[l],g=e[l];if(n)var v=a?n(g,d,l,e,t,o):n(d,g,l,t,e,o);if(!(void 0===v?d===g||r(d,g,n,i,o):v)){h=!1;break}p||(p="constructor"==l);}if(h&&!p){var b=t.constructor,y=e.constructor;b!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof b&&b instanceof b&&"function"==typeof y&&y instanceof y)&&(h=!1);}return o.delete(t),o.delete(e),h}function at(t,e){var r=t.__data__;return ht(e)?r["string"==typeof e?"string":"hash"]:r.map}function st(t){for(var e=qt(t),r=e.length;r--;){var n=e[r],i=t[n];e[r]=[n,i,gt(i)];}return e}function ut(t,e){var r=s(t,e);return Y(r)?r:void 0}function ct(t,e,r){e=ft(e,t)?[e]:rt(e);for(var n,i=-1,o=e.length;++i<o;){var a=bt(e[i]);if(!(n=null!=t&&r(t,a)))break;t=t[a];}if(n)return n;var o=t?t.length:0;return !!o&&At(o)&&lt(a,o)&&(rr(t)||St(t))}function lt(t,e){return !!(e=null==e?Vt:e)&&("number"==typeof t||pe.test(t))&&t>-1&&t%1==0&&t<e}function ft(t,e){if(rr(t))return !1;var r=typeof t;return !("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=t&&!Lt(t))||(se.test(t)||!ae.test(t)||null!=e&&t in Object(e))}function ht(t){var e=typeof t;return "string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}function pt(t){return !!Te&&Te in t}function dt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||Ee)}function gt(t){return t===t&&!Tt(t)}function vt(t,e){return function(r){return null!=r&&(r[t]===e&&(void 0!==e||t in Object(r)))}}function bt(t){if("string"==typeof t||Lt(t))return t;var e=t+"";return "0"==e&&1/t==-zt?"-0":e}function yt(t){if(null!=t){try{return Oe.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function mt(t,e){return (rr(t)?n:U)(t,wt(Q(e)))}function _t(t,e){if("function"!=typeof t||e&&"function"!=typeof e)throw new TypeError(It);var r=function(){var n=arguments,i=e?e.apply(this,n):n[0],o=r.cache;if(o.has(i))return o.get(i);var a=t.apply(this,n);return r.cache=o.set(i,a),a};return r.cache=new(_t.Cache||S),r}function wt(t){if("function"!=typeof t)throw new TypeError(It);return function(){var e=arguments;switch(e.length){case 0:return !t.call(this);case 1:return !t.call(this,e[0]);case 2:return !t.call(this,e[0],e[1]);case 3:return !t.call(this,e[0],e[1],e[2])}return !t.apply(this,e)}}function xt(t,e){return t===e||t!==t&&e!==e}function St(t){return kt(t)&&Le.call(t,"callee")&&(!Pe.call(t,"callee")||Ce.call(t)==$t)}function jt(t){return null!=t&&At(t.length)&&!Et(t)}function kt(t){return Ot(t)&&jt(t)}function Et(t){var e=Tt(t)?Ce.call(t):"";return e==Jt||e==Qt}function At(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=Vt}function Tt(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function Ot(t){return !!t&&"object"==typeof t}function Lt(t){return "symbol"==typeof t||Ot(t)&&Ce.call(t)==ne}function Ct(t){return null==t?"":et(t)}function Bt(t,e,r){var n=null==t?void 0:z(t,e);return void 0===n?r:n}function Dt(t,e){return null!=t&&ct(t,e,$)}function qt(t){return jt(t)?I(t):X(t)}function Pt(t){return t}function Mt(t){return ft(t)?o(bt(t)):tt(t)}var Rt=200,It="Expected a function",Nt="__lodash_hash_undefined__",Ut=1,Ft=2,zt=1/0,Vt=9007199254740991,$t="[object Arguments]",Ht="[object Array]",Wt="[object Boolean]",Gt="[object Date]",Yt="[object Error]",Jt="[object Function]",Qt="[object GeneratorFunction]",Xt="[object Map]",Zt="[object Number]",Kt="[object Object]",te="[object RegExp]",ee="[object Set]",re="[object String]",ne="[object Symbol]",ie="[object ArrayBuffer]",oe="[object DataView]",ae=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,se=/^\w*$/,ue=/^\./,ce=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,le=/[\\^$.*+?()[\]{}|]/g,fe=/\\(\\)?/g,he=/^\[object .+?Constructor\]$/,pe=/^(?:0|[1-9]\d*)$/,de={};de["[object Float32Array]"]=de["[object Float64Array]"]=de["[object Int8Array]"]=de["[object Int16Array]"]=de["[object Int32Array]"]=de["[object Uint8Array]"]=de["[object Uint8ClampedArray]"]=de["[object Uint16Array]"]=de["[object Uint32Array]"]=!0,de[$t]=de[Ht]=de[ie]=de[Wt]=de[oe]=de[Gt]=de[Yt]=de[Jt]=de[Xt]=de[Zt]=de[Kt]=de[te]=de[ee]=de[re]=de["[object WeakMap]"]=!1;var ge="object"==typeof t&&t&&t.Object===Object&&t,ve="object"==typeof self&&self&&self.Object===Object&&self,be=ge||ve||Function("return this")(),ye="object"==typeof e&&e&&!e.nodeType&&e,me=ye&&"object"==typeof r&&r&&!r.nodeType&&r,_e=me&&me.exports===ye,we=_e&&ge.process,xe=function(){try{return we&&we.binding("util")}catch(t){}}(),Se=xe&&xe.isTypedArray,je=Array.prototype,ke=Function.prototype,Ee=Object.prototype,Ae=be["__core-js_shared__"],Te=function(){var t=/[^.]+$/.exec(Ae&&Ae.keys&&Ae.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),Oe=ke.toString,Le=Ee.hasOwnProperty,Ce=Ee.toString,Be=RegExp("^"+Oe.call(Le).replace(le,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),De=be.Symbol,qe=be.Uint8Array,Pe=Ee.propertyIsEnumerable,Me=je.splice,Re=function(t,e){return function(r){return t(e(r))}}(Object.keys,Object),Ie=ut(be,"DataView"),Ne=ut(be,"Map"),Ue=ut(be,"Promise"),Fe=ut(be,"Set"),ze=ut(be,"WeakMap"),Ve=ut(Object,"create"),$e=yt(Ie),He=yt(Ne),We=yt(Ue),Ge=yt(Fe),Ye=yt(ze),Je=De?De.prototype:void 0,Qe=Je?Je.valueOf:void 0,Xe=Je?Je.toString:void 0;f.prototype.clear=h,f.prototype.delete=p,f.prototype.get=d,f.prototype.has=g,f.prototype.set=v,b.prototype.clear=y,b.prototype.delete=m,b.prototype.get=_,b.prototype.has=w,b.prototype.set=x,S.prototype.clear=j,S.prototype.delete=k,S.prototype.get=E,S.prototype.has=A,S.prototype.set=T,O.prototype.add=O.prototype.push=L,O.prototype.has=C,B.prototype.clear=D,B.prototype.delete=q,B.prototype.get=P,B.prototype.has=M,B.prototype.set=R;var Ze=function(t,e){return function(r,n){if(null==r)return r;if(!jt(r))return t(r,n);for(var i=r.length,o=e?i:-1,a=Object(r);(e?o--:++o<i)&&!1!==n(a[o],o,a););return r}}(F),Ke=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),tr=V;(Ie&&tr(new Ie(new ArrayBuffer(1)))!=oe||Ne&&tr(new Ne)!=Xt||Ue&&"[object Promise]"!=tr(Ue.resolve())||Fe&&tr(new Fe)!=ee||ze&&"[object WeakMap]"!=tr(new ze))&&(tr=function(t){var e=Ce.call(t),r=e==Kt?t.constructor:void 0,n=r?yt(r):void 0;if(n)switch(n){case $e:return oe;case He:return Xt;case We:return "[object Promise]";case Ge:return ee;case Ye:return "[object WeakMap]"}return e});var er=_t(function(t){t=Ct(t);var e=[];return ue.test(t)&&e.push(""),t.replace(ce,function(t,r,n,i){e.push(n?i.replace(fe,"$1"):r||t);}),e});_t.Cache=S;var rr=Array.isArray,nr=Se?function(t){return function(e){return t(e)}}(Se):J;r.exports=mt;}).call(e,r(0),r(5)(t));},function(t,e,r){(function(t,r){function n(t,e){for(var r=-1,n=t?t.length:0,i=0,o=[];++r<n;){var a=t[r];e(a,r,t)&&(o[i++]=a);}return o}function i(t,e){for(var r=-1,n=t?t.length:0;++r<n;)if(e(t[r],r,t))return !0;return !1}function o(t){return function(e){return null==e?void 0:e[t]}}function a(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function s(t,e){return null==t?void 0:t[e]}function u(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"");}catch(t){}return e}function c(t){var e=-1,r=Array(t.size);return t.forEach(function(t,n){r[++e]=[n,t];}),r}function l(t){var e=-1,r=Array(t.size);return t.forEach(function(t){r[++e]=t;}),r}function f(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function h(){this.__data__=ze?ze(null):{};}function p(t){return this.has(t)&&delete this.__data__[t]}function d(t){var e=this.__data__;if(ze){var r=e[t];return r===It?void 0:r}return Oe.call(e,t)?e[t]:void 0}function g(t){var e=this.__data__;return ze?void 0!==e[t]:Oe.call(e,t)}function v(t,e){return this.__data__[t]=ze&&void 0===e?It:e,this}function b(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function y(){this.__data__=[];}function m(t){var e=this.__data__,r=N(e,t);return !(r<0)&&(r==e.length-1?e.pop():Pe.call(e,r,1),!0)}function _(t){var e=this.__data__,r=N(e,t);return r<0?void 0:e[r][1]}function w(t){return N(this.__data__,t)>-1}function x(t,e){var r=this.__data__,n=N(r,t);return n<0?r.push([t,e]):r[n][1]=e,this}function S(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function j(){this.__data__={hash:new f,map:new(Ie||b),string:new f};}function k(t){return at(this,t).delete(t)}function E(t){return at(this,t).get(t)}function A(t){return at(this,t).has(t)}function T(t,e){return at(this,t).set(t,e),this}function O(t){var e=-1,r=t?t.length:0;for(this.__data__=new S;++e<r;)this.add(t[e]);}function L(t){return this.__data__.set(t,It),this}function C(t){return this.__data__.has(t)}function B(t){this.__data__=new b(t);}function D(){this.__data__=new b;}function q(t){return this.__data__.delete(t)}function P(t){return this.__data__.get(t)}function M(t){return this.__data__.has(t)}function R(t,e){var r=this.__data__;if(r instanceof b){var n=r.__data__;if(!Ie||n.length<Mt-1)return n.push([t,e]),this;r=this.__data__=new S(n);}return r.set(t,e),this}function I(t,e){var r=er(t)||xt(t)?a(t.length,String):[],n=r.length,i=!!n;for(var o in t)!e&&!Oe.call(t,o)||i&&("length"==o||lt(o,n))||r.push(o);return r}function N(t,e){for(var r=t.length;r--;)if(wt(t[r][0],e))return r;return -1}function U(t,e){var r=[];return Xe(t,function(t,n,i){e(t,n,i)&&r.push(t);}),r}function F(t,e){return t&&Ze(t,e,Dt)}function z(t,e){e=ft(e,t)?[e]:rt(e);for(var r=0,n=e.length;null!=t&&r<n;)t=t[bt(e[r++])];return r&&r==n?t:void 0}function V(t){return Le.call(t)}function $(t,e){return null!=t&&e in Object(t)}function H(t,e,r,n,i){return t===e||(null==t||null==e||!At(t)&&!Tt(e)?t!==t&&e!==e:W(t,e,H,r,n,i))}function W(t,e,r,n,i,o){var a=er(t),s=er(e),c=$t,l=$t;a||(c=Ke(t),c=c==Vt?Zt:c),s||(l=Ke(e),l=l==Vt?Zt:l);var f=c==Zt&&!u(t),h=l==Zt&&!u(e),p=c==l;if(p&&!f)return o||(o=new B),a||rr(t)?nt(t,e,r,n,i,o):it(t,e,c,r,n,i,o);if(!(i&Ut)){var d=f&&Oe.call(t,"__wrapped__"),g=h&&Oe.call(e,"__wrapped__");if(d||g){var v=d?t.value():t,b=g?e.value():e;return o||(o=new B),r(v,b,n,i,o)}}return !!p&&(o||(o=new B),ot(t,e,r,n,i,o))}function G(t,e,r,n){var i=r.length,o=i,a=!n;if(null==t)return !o;for(t=Object(t);i--;){var s=r[i];if(a&&s[2]?s[1]!==t[s[0]]:!(s[0]in t))return !1}for(;++i<o;){s=r[i];var u=s[0],c=t[u],l=s[1];if(a&&s[2]){if(void 0===c&&!(u in t))return !1}else {var f=new B;if(n)var h=n(c,l,u,t,e,f);if(!(void 0===h?H(l,c,n,Nt|Ut,f):h))return !1}}return !0}function Y(t){return !(!At(t)||pt(t))&&(kt(t)||u(t)?Ce:fe).test(yt(t))}function J(t){return Tt(t)&&Et(t.length)&&!!pe[Le.call(t)]}function Q(t){return "function"==typeof t?t:null==t?qt:"object"==typeof t?er(t)?K(t[0],t[1]):Z(t):Pt(t)}function X(t){if(!dt(t))return Me(t);var e=[];for(var r in Object(t))Oe.call(t,r)&&"constructor"!=r&&e.push(r);return e}function Z(t){var e=st(t);return 1==e.length&&e[0][2]?vt(e[0][0],e[0][1]):function(r){return r===t||G(r,t,e)}}function K(t,e){return ft(t)&&gt(e)?vt(bt(t),e):function(r){var n=Ct(r,t);return void 0===n&&n===e?Bt(r,t):H(e,n,void 0,Nt|Ut)}}function tt(t){return function(e){return z(e,t)}}function et(t){if("string"==typeof t)return t;if(Ot(t))return Qe?Qe.call(t):"";var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function rt(t){return er(t)?t:tr(t)}function nt(t,e,r,n,o,a){var s=o&Ut,u=t.length,c=e.length;if(u!=c&&!(s&&c>u))return !1;var l=a.get(t);if(l&&a.get(e))return l==e;var f=-1,h=!0,p=o&Nt?new O:void 0;for(a.set(t,e),a.set(e,t);++f<u;){var d=t[f],g=e[f];if(n)var v=s?n(g,d,f,e,t,a):n(d,g,f,t,e,a);if(void 0!==v){if(v)continue;h=!1;break}if(p){if(!i(e,function(t,e){if(!p.has(e)&&(d===t||r(d,t,n,o,a)))return p.add(e)})){h=!1;break}}else if(d!==g&&!r(d,g,n,o,a)){h=!1;break}}return a.delete(t),a.delete(e),h}function it(t,e,r,n,i,o,a){switch(r){case ie:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return !1;t=t.buffer,e=e.buffer;case ne:return !(t.byteLength!=e.byteLength||!n(new De(t),new De(e)));case Ht:case Wt:case Xt:return wt(+t,+e);case Gt:return t.name==e.name&&t.message==e.message;case Kt:case ee:return t==e+"";case Qt:var s=c;case te:var u=o&Ut;if(s||(s=l),t.size!=e.size&&!u)return !1;var f=a.get(t);if(f)return f==e;o|=Nt,a.set(t,e);var h=nt(s(t),s(e),n,i,o,a);return a.delete(t),h;case re:if(Je)return Je.call(t)==Je.call(e)}return !1}function ot(t,e,r,n,i,o){var a=i&Ut,s=Dt(t),u=s.length;if(u!=Dt(e).length&&!a)return !1;for(var c=u;c--;){var l=s[c];if(!(a?l in e:Oe.call(e,l)))return !1}var f=o.get(t);if(f&&o.get(e))return f==e;var h=!0;o.set(t,e),o.set(e,t);for(var p=a;++c<u;){l=s[c];var d=t[l],g=e[l];if(n)var v=a?n(g,d,l,e,t,o):n(d,g,l,t,e,o);if(!(void 0===v?d===g||r(d,g,n,i,o):v)){h=!1;break}p||(p="constructor"==l);}if(h&&!p){var b=t.constructor,y=e.constructor;b!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof b&&b instanceof b&&"function"==typeof y&&y instanceof y)&&(h=!1);}return o.delete(t),o.delete(e),h}function at(t,e){var r=t.__data__;return ht(e)?r["string"==typeof e?"string":"hash"]:r.map}function st(t){for(var e=Dt(t),r=e.length;r--;){var n=e[r],i=t[n];e[r]=[n,i,gt(i)];}return e}function ut(t,e){var r=s(t,e);return Y(r)?r:void 0}function ct(t,e,r){e=ft(e,t)?[e]:rt(e);for(var n,i=-1,o=e.length;++i<o;){var a=bt(e[i]);if(!(n=null!=t&&r(t,a)))break;t=t[a];}if(n)return n;var o=t?t.length:0;return !!o&&Et(o)&&lt(a,o)&&(er(t)||xt(t))}function lt(t,e){return !!(e=null==e?zt:e)&&("number"==typeof t||he.test(t))&&t>-1&&t%1==0&&t<e}function ft(t,e){if(er(t))return !1;var r=typeof t;return !("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=t&&!Ot(t))||(ae.test(t)||!oe.test(t)||null!=e&&t in Object(e))}function ht(t){var e=typeof t;return "string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}function pt(t){return !!Ae&&Ae in t}function dt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||ke)}function gt(t){return t===t&&!At(t)}function vt(t,e){return function(r){return null!=r&&(r[t]===e&&(void 0!==e||t in Object(r)))}}function bt(t){if("string"==typeof t||Ot(t))return t;var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function yt(t){if(null!=t){try{return Te.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function mt(t,e){return (er(t)?n:U)(t,Q(e))}function _t(t,e){if("function"!=typeof t||e&&"function"!=typeof e)throw new TypeError(Rt);var r=function(){var n=arguments,i=e?e.apply(this,n):n[0],o=r.cache;if(o.has(i))return o.get(i);var a=t.apply(this,n);return r.cache=o.set(i,a),a};return r.cache=new(_t.Cache||S),r}function wt(t,e){return t===e||t!==t&&e!==e}function xt(t){return jt(t)&&Oe.call(t,"callee")&&(!qe.call(t,"callee")||Le.call(t)==Vt)}function St(t){return null!=t&&Et(t.length)&&!kt(t)}function jt(t){return Tt(t)&&St(t)}function kt(t){var e=At(t)?Le.call(t):"";return e==Yt||e==Jt}function Et(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=zt}function At(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function Tt(t){return !!t&&"object"==typeof t}function Ot(t){return "symbol"==typeof t||Tt(t)&&Le.call(t)==re}function Lt(t){return null==t?"":et(t)}function Ct(t,e,r){var n=null==t?void 0:z(t,e);return void 0===n?r:n}function Bt(t,e){return null!=t&&ct(t,e,$)}function Dt(t){return St(t)?I(t):X(t)}function qt(t){return t}function Pt(t){return ft(t)?o(bt(t)):tt(t)}var Mt=200,Rt="Expected a function",It="__lodash_hash_undefined__",Nt=1,Ut=2,Ft=1/0,zt=9007199254740991,Vt="[object Arguments]",$t="[object Array]",Ht="[object Boolean]",Wt="[object Date]",Gt="[object Error]",Yt="[object Function]",Jt="[object GeneratorFunction]",Qt="[object Map]",Xt="[object Number]",Zt="[object Object]",Kt="[object RegExp]",te="[object Set]",ee="[object String]",re="[object Symbol]",ne="[object ArrayBuffer]",ie="[object DataView]",oe=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,ae=/^\w*$/,se=/^\./,ue=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,ce=/[\\^$.*+?()[\]{}|]/g,le=/\\(\\)?/g,fe=/^\[object .+?Constructor\]$/,he=/^(?:0|[1-9]\d*)$/,pe={};pe["[object Float32Array]"]=pe["[object Float64Array]"]=pe["[object Int8Array]"]=pe["[object Int16Array]"]=pe["[object Int32Array]"]=pe["[object Uint8Array]"]=pe["[object Uint8ClampedArray]"]=pe["[object Uint16Array]"]=pe["[object Uint32Array]"]=!0,pe[Vt]=pe[$t]=pe[ne]=pe[Ht]=pe[ie]=pe[Wt]=pe[Gt]=pe[Yt]=pe[Qt]=pe[Xt]=pe[Zt]=pe[Kt]=pe[te]=pe[ee]=pe["[object WeakMap]"]=!1;var de="object"==typeof t&&t&&t.Object===Object&&t,ge="object"==typeof self&&self&&self.Object===Object&&self,ve=de||ge||Function("return this")(),be="object"==typeof e&&e&&!e.nodeType&&e,ye=be&&"object"==typeof r&&r&&!r.nodeType&&r,me=ye&&ye.exports===be,_e=me&&de.process,we=function(){try{return _e&&_e.binding("util")}catch(t){}}(),xe=we&&we.isTypedArray,Se=Array.prototype,je=Function.prototype,ke=Object.prototype,Ee=ve["__core-js_shared__"],Ae=function(){var t=/[^.]+$/.exec(Ee&&Ee.keys&&Ee.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),Te=je.toString,Oe=ke.hasOwnProperty,Le=ke.toString,Ce=RegExp("^"+Te.call(Oe).replace(ce,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Be=ve.Symbol,De=ve.Uint8Array,qe=ke.propertyIsEnumerable,Pe=Se.splice,Me=function(t,e){return function(r){return t(e(r))}}(Object.keys,Object),Re=ut(ve,"DataView"),Ie=ut(ve,"Map"),Ne=ut(ve,"Promise"),Ue=ut(ve,"Set"),Fe=ut(ve,"WeakMap"),ze=ut(Object,"create"),Ve=yt(Re),$e=yt(Ie),He=yt(Ne),We=yt(Ue),Ge=yt(Fe),Ye=Be?Be.prototype:void 0,Je=Ye?Ye.valueOf:void 0,Qe=Ye?Ye.toString:void 0;f.prototype.clear=h,f.prototype.delete=p,f.prototype.get=d,f.prototype.has=g,f.prototype.set=v,b.prototype.clear=y,b.prototype.delete=m,b.prototype.get=_,b.prototype.has=w,b.prototype.set=x,S.prototype.clear=j,S.prototype.delete=k,S.prototype.get=E,S.prototype.has=A,S.prototype.set=T,O.prototype.add=O.prototype.push=L,O.prototype.has=C,B.prototype.clear=D,B.prototype.delete=q,B.prototype.get=P,B.prototype.has=M,B.prototype.set=R;var Xe=function(t,e){return function(r,n){if(null==r)return r;if(!St(r))return t(r,n);for(var i=r.length,o=e?i:-1,a=Object(r);(e?o--:++o<i)&&!1!==n(a[o],o,a););return r}}(F),Ze=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),Ke=V;(Re&&Ke(new Re(new ArrayBuffer(1)))!=ie||Ie&&Ke(new Ie)!=Qt||Ne&&"[object Promise]"!=Ke(Ne.resolve())||Ue&&Ke(new Ue)!=te||Fe&&"[object WeakMap]"!=Ke(new Fe))&&(Ke=function(t){var e=Le.call(t),r=e==Zt?t.constructor:void 0,n=r?yt(r):void 0;if(n)switch(n){case Ve:return ie;case $e:return Qt;case He:return "[object Promise]";case We:return te;case Ge:return "[object WeakMap]"}return e});var tr=_t(function(t){t=Lt(t);var e=[];return se.test(t)&&e.push(""),t.replace(ue,function(t,r,n,i){e.push(n?i.replace(le,"$1"):r||t);}),e});_t.Cache=S;var er=Array.isArray,rr=xe?function(t){return function(e){return t(e)}}(xe):J;r.exports=mt;}).call(e,r(0),r(5)(t));},function(t,e,r){(function(t,r){function n(t,e,r,n){var i=-1,o=t?t.length:0;for(n&&o&&(r=t[++i]);++i<o;)r=e(r,t[i],i,t);return r}function i(t,e){for(var r=-1,n=t?t.length:0;++r<n;)if(e(t[r],r,t))return !0;return !1}function o(t){return function(e){return null==e?void 0:e[t]}}function a(t,e,r,n,i){return i(t,function(t,i,o){r=n?(n=!1,t):e(r,t,i,o);}),r}function s(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function u(t,e){return null==t?void 0:t[e]}function c(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"");}catch(t){}return e}function l(t){var e=-1,r=Array(t.size);return t.forEach(function(t,n){r[++e]=[n,t];}),r}function f(t){var e=-1,r=Array(t.size);return t.forEach(function(t){r[++e]=t;}),r}function h(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function p(){this.__data__=ze?ze(null):{};}function d(t){return this.has(t)&&delete this.__data__[t]}function g(t){var e=this.__data__;if(ze){var r=e[t];return r===It?void 0:r}return Oe.call(e,t)?e[t]:void 0}function v(t){var e=this.__data__;return ze?void 0!==e[t]:Oe.call(e,t)}function b(t,e){return this.__data__[t]=ze&&void 0===e?It:e,this}function y(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function m(){this.__data__=[];}function _(t){var e=this.__data__,r=U(e,t);return !(r<0)&&(r==e.length-1?e.pop():Pe.call(e,r,1),!0)}function w(t){var e=this.__data__,r=U(e,t);return r<0?void 0:e[r][1]}function x(t){return U(this.__data__,t)>-1}function S(t,e){var r=this.__data__,n=U(r,t);return n<0?r.push([t,e]):r[n][1]=e,this}function j(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function k(){this.__data__={hash:new h,map:new(Ie||y),string:new h};}function E(t){return at(this,t).delete(t)}function A(t){return at(this,t).get(t)}function T(t){return at(this,t).has(t)}function O(t,e){return at(this,t).set(t,e),this}function L(t){var e=-1,r=t?t.length:0;for(this.__data__=new j;++e<r;)this.add(t[e]);}function C(t){return this.__data__.set(t,It),this}function B(t){return this.__data__.has(t)}function D(t){this.__data__=new y(t);}function q(){this.__data__=new y;}function P(t){return this.__data__.delete(t)}function M(t){return this.__data__.get(t)}function R(t){return this.__data__.has(t)}function I(t,e){var r=this.__data__;if(r instanceof y){var n=r.__data__;if(!Ie||n.length<Mt-1)return n.push([t,e]),this;r=this.__data__=new j(n);}return r.set(t,e),this}function N(t,e){var r=er(t)||xt(t)?s(t.length,String):[],n=r.length,i=!!n;for(var o in t)!e&&!Oe.call(t,o)||i&&("length"==o||lt(o,n))||r.push(o);return r}function U(t,e){for(var r=t.length;r--;)if(wt(t[r][0],e))return r;return -1}function F(t,e){return t&&Ze(t,e,Dt)}function z(t,e){e=ft(e,t)?[e]:rt(e);for(var r=0,n=e.length;null!=t&&r<n;)t=t[bt(e[r++])];return r&&r==n?t:void 0}function V(t){return Le.call(t)}function $(t,e){return null!=t&&e in Object(t)}function H(t,e,r,n,i){return t===e||(null==t||null==e||!At(t)&&!Tt(e)?t!==t&&e!==e:W(t,e,H,r,n,i))}function W(t,e,r,n,i,o){var a=er(t),s=er(e),u=$t,l=$t;a||(u=Ke(t),u=u==Vt?Zt:u),s||(l=Ke(e),l=l==Vt?Zt:l);var f=u==Zt&&!c(t),h=l==Zt&&!c(e),p=u==l;if(p&&!f)return o||(o=new D),a||rr(t)?nt(t,e,r,n,i,o):it(t,e,u,r,n,i,o);if(!(i&Ut)){var d=f&&Oe.call(t,"__wrapped__"),g=h&&Oe.call(e,"__wrapped__");if(d||g){var v=d?t.value():t,b=g?e.value():e;return o||(o=new D),r(v,b,n,i,o)}}return !!p&&(o||(o=new D),ot(t,e,r,n,i,o))}function G(t,e,r,n){var i=r.length,o=i,a=!n;if(null==t)return !o;for(t=Object(t);i--;){var s=r[i];if(a&&s[2]?s[1]!==t[s[0]]:!(s[0]in t))return !1}for(;++i<o;){s=r[i];var u=s[0],c=t[u],l=s[1];if(a&&s[2]){if(void 0===c&&!(u in t))return !1}else {var f=new D;if(n)var h=n(c,l,u,t,e,f);if(!(void 0===h?H(l,c,n,Nt|Ut,f):h))return !1}}return !0}function Y(t){return !(!At(t)||pt(t))&&(kt(t)||c(t)?Ce:fe).test(yt(t))}function J(t){return Tt(t)&&Et(t.length)&&!!pe[Le.call(t)]}function Q(t){return "function"==typeof t?t:null==t?qt:"object"==typeof t?er(t)?K(t[0],t[1]):Z(t):Pt(t)}function X(t){if(!dt(t))return Me(t);var e=[];for(var r in Object(t))Oe.call(t,r)&&"constructor"!=r&&e.push(r);return e}function Z(t){var e=st(t);return 1==e.length&&e[0][2]?vt(e[0][0],e[0][1]):function(r){return r===t||G(r,t,e)}}function K(t,e){return ft(t)&&gt(e)?vt(bt(t),e):function(r){var n=Ct(r,t);return void 0===n&&n===e?Bt(r,t):H(e,n,void 0,Nt|Ut)}}function tt(t){return function(e){return z(e,t)}}function et(t){if("string"==typeof t)return t;if(Ot(t))return Qe?Qe.call(t):"";var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function rt(t){return er(t)?t:tr(t)}function nt(t,e,r,n,o,a){var s=o&Ut,u=t.length,c=e.length;if(u!=c&&!(s&&c>u))return !1;var l=a.get(t);if(l&&a.get(e))return l==e;var f=-1,h=!0,p=o&Nt?new L:void 0;for(a.set(t,e),a.set(e,t);++f<u;){var d=t[f],g=e[f];if(n)var v=s?n(g,d,f,e,t,a):n(d,g,f,t,e,a);if(void 0!==v){if(v)continue;h=!1;break}if(p){if(!i(e,function(t,e){if(!p.has(e)&&(d===t||r(d,t,n,o,a)))return p.add(e)})){h=!1;break}}else if(d!==g&&!r(d,g,n,o,a)){h=!1;break}}return a.delete(t),a.delete(e),h}function it(t,e,r,n,i,o,a){switch(r){case ie:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return !1;t=t.buffer,e=e.buffer;case ne:return !(t.byteLength!=e.byteLength||!n(new De(t),new De(e)));case Ht:case Wt:case Xt:return wt(+t,+e);case Gt:return t.name==e.name&&t.message==e.message;case Kt:case ee:return t==e+"";case Qt:var s=l;case te:var u=o&Ut;if(s||(s=f),t.size!=e.size&&!u)return !1;var c=a.get(t);if(c)return c==e;o|=Nt,a.set(t,e);var h=nt(s(t),s(e),n,i,o,a);return a.delete(t),h;case re:if(Je)return Je.call(t)==Je.call(e)}return !1}function ot(t,e,r,n,i,o){var a=i&Ut,s=Dt(t),u=s.length;if(u!=Dt(e).length&&!a)return !1;for(var c=u;c--;){var l=s[c];if(!(a?l in e:Oe.call(e,l)))return !1}var f=o.get(t);if(f&&o.get(e))return f==e;var h=!0;o.set(t,e),o.set(e,t);for(var p=a;++c<u;){l=s[c];var d=t[l],g=e[l];if(n)var v=a?n(g,d,l,e,t,o):n(d,g,l,t,e,o);if(!(void 0===v?d===g||r(d,g,n,i,o):v)){h=!1;break}p||(p="constructor"==l);}if(h&&!p){var b=t.constructor,y=e.constructor;b!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof b&&b instanceof b&&"function"==typeof y&&y instanceof y)&&(h=!1);}return o.delete(t),o.delete(e),h}function at(t,e){var r=t.__data__;return ht(e)?r["string"==typeof e?"string":"hash"]:r.map}function st(t){for(var e=Dt(t),r=e.length;r--;){var n=e[r],i=t[n];e[r]=[n,i,gt(i)];}return e}function ut(t,e){var r=u(t,e);return Y(r)?r:void 0}function ct(t,e,r){e=ft(e,t)?[e]:rt(e);for(var n,i=-1,o=e.length;++i<o;){var a=bt(e[i]);if(!(n=null!=t&&r(t,a)))break;t=t[a];}if(n)return n;var o=t?t.length:0;return !!o&&Et(o)&&lt(a,o)&&(er(t)||xt(t))}function lt(t,e){return !!(e=null==e?zt:e)&&("number"==typeof t||he.test(t))&&t>-1&&t%1==0&&t<e}function ft(t,e){if(er(t))return !1;var r=typeof t;return !("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=t&&!Ot(t))||(ae.test(t)||!oe.test(t)||null!=e&&t in Object(e))}function ht(t){var e=typeof t;return "string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}function pt(t){return !!Ae&&Ae in t}function dt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||ke)}function gt(t){return t===t&&!At(t)}function vt(t,e){return function(r){return null!=r&&(r[t]===e&&(void 0!==e||t in Object(r)))}}function bt(t){if("string"==typeof t||Ot(t))return t;var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function yt(t){if(null!=t){try{return Te.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function mt(t,e,r){var i=er(t)?n:a,o=arguments.length<3;return i(t,Q(e),r,o,Xe)}function _t(t,e){if("function"!=typeof t||e&&"function"!=typeof e)throw new TypeError(Rt);var r=function(){var n=arguments,i=e?e.apply(this,n):n[0],o=r.cache;if(o.has(i))return o.get(i);var a=t.apply(this,n);return r.cache=o.set(i,a),a};return r.cache=new(_t.Cache||j),r}function wt(t,e){return t===e||t!==t&&e!==e}function xt(t){return jt(t)&&Oe.call(t,"callee")&&(!qe.call(t,"callee")||Le.call(t)==Vt)}function St(t){return null!=t&&Et(t.length)&&!kt(t)}function jt(t){return Tt(t)&&St(t)}function kt(t){var e=At(t)?Le.call(t):"";return e==Yt||e==Jt}function Et(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=zt}function At(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function Tt(t){return !!t&&"object"==typeof t}function Ot(t){return "symbol"==typeof t||Tt(t)&&Le.call(t)==re}function Lt(t){return null==t?"":et(t)}function Ct(t,e,r){var n=null==t?void 0:z(t,e);return void 0===n?r:n}function Bt(t,e){return null!=t&&ct(t,e,$)}function Dt(t){return St(t)?N(t):X(t)}function qt(t){return t}function Pt(t){return ft(t)?o(bt(t)):tt(t)}var Mt=200,Rt="Expected a function",It="__lodash_hash_undefined__",Nt=1,Ut=2,Ft=1/0,zt=9007199254740991,Vt="[object Arguments]",$t="[object Array]",Ht="[object Boolean]",Wt="[object Date]",Gt="[object Error]",Yt="[object Function]",Jt="[object GeneratorFunction]",Qt="[object Map]",Xt="[object Number]",Zt="[object Object]",Kt="[object RegExp]",te="[object Set]",ee="[object String]",re="[object Symbol]",ne="[object ArrayBuffer]",ie="[object DataView]",oe=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,ae=/^\w*$/,se=/^\./,ue=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,ce=/[\\^$.*+?()[\]{}|]/g,le=/\\(\\)?/g,fe=/^\[object .+?Constructor\]$/,he=/^(?:0|[1-9]\d*)$/,pe={};pe["[object Float32Array]"]=pe["[object Float64Array]"]=pe["[object Int8Array]"]=pe["[object Int16Array]"]=pe["[object Int32Array]"]=pe["[object Uint8Array]"]=pe["[object Uint8ClampedArray]"]=pe["[object Uint16Array]"]=pe["[object Uint32Array]"]=!0,pe[Vt]=pe[$t]=pe[ne]=pe[Ht]=pe[ie]=pe[Wt]=pe[Gt]=pe[Yt]=pe[Qt]=pe[Xt]=pe[Zt]=pe[Kt]=pe[te]=pe[ee]=pe["[object WeakMap]"]=!1;var de="object"==typeof t&&t&&t.Object===Object&&t,ge="object"==typeof self&&self&&self.Object===Object&&self,ve=de||ge||Function("return this")(),be="object"==typeof e&&e&&!e.nodeType&&e,ye=be&&"object"==typeof r&&r&&!r.nodeType&&r,me=ye&&ye.exports===be,_e=me&&de.process,we=function(){try{return _e&&_e.binding("util")}catch(t){}}(),xe=we&&we.isTypedArray,Se=Array.prototype,je=Function.prototype,ke=Object.prototype,Ee=ve["__core-js_shared__"],Ae=function(){var t=/[^.]+$/.exec(Ee&&Ee.keys&&Ee.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),Te=je.toString,Oe=ke.hasOwnProperty,Le=ke.toString,Ce=RegExp("^"+Te.call(Oe).replace(ce,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Be=ve.Symbol,De=ve.Uint8Array,qe=ke.propertyIsEnumerable,Pe=Se.splice,Me=function(t,e){return function(r){return t(e(r))}}(Object.keys,Object),Re=ut(ve,"DataView"),Ie=ut(ve,"Map"),Ne=ut(ve,"Promise"),Ue=ut(ve,"Set"),Fe=ut(ve,"WeakMap"),ze=ut(Object,"create"),Ve=yt(Re),$e=yt(Ie),He=yt(Ne),We=yt(Ue),Ge=yt(Fe),Ye=Be?Be.prototype:void 0,Je=Ye?Ye.valueOf:void 0,Qe=Ye?Ye.toString:void 0;h.prototype.clear=p,h.prototype.delete=d,h.prototype.get=g,h.prototype.has=v,h.prototype.set=b,y.prototype.clear=m,y.prototype.delete=_,y.prototype.get=w,y.prototype.has=x,y.prototype.set=S,j.prototype.clear=k,j.prototype.delete=E,j.prototype.get=A,j.prototype.has=T,j.prototype.set=O,L.prototype.add=L.prototype.push=C,L.prototype.has=B,D.prototype.clear=q,D.prototype.delete=P,D.prototype.get=M,D.prototype.has=R,D.prototype.set=I;var Xe=function(t,e){return function(r,n){if(null==r)return r;if(!St(r))return t(r,n);for(var i=r.length,o=e?i:-1,a=Object(r);(e?o--:++o<i)&&!1!==n(a[o],o,a););return r}}(F),Ze=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),Ke=V;(Re&&Ke(new Re(new ArrayBuffer(1)))!=ie||Ie&&Ke(new Ie)!=Qt||Ne&&"[object Promise]"!=Ke(Ne.resolve())||Ue&&Ke(new Ue)!=te||Fe&&"[object WeakMap]"!=Ke(new Fe))&&(Ke=function(t){var e=Le.call(t),r=e==Zt?t.constructor:void 0,n=r?yt(r):void 0;if(n)switch(n){case Ve:return ie;case $e:return Qt;case He:return "[object Promise]";case We:return te;case Ge:return "[object WeakMap]"}return e});var tr=_t(function(t){t=Lt(t);var e=[];return se.test(t)&&e.push(""),t.replace(ue,function(t,r,n,i){e.push(n?i.replace(le,"$1"):r||t);}),e});_t.Cache=j;var er=Array.isArray,rr=xe?function(t){return function(e){return t(e)}}(xe):J;r.exports=mt;}).call(e,r(0),r(5)(t));},function(t,e,r){var n=r(11),i=r(25),o=n.update,a=n.evaluate,s=r(9),u=s.domEach,c=s.cloneDom,l=s.isHtml,f=Array.prototype.slice,h={flatten:r(102),bind:r(24),forEach:r(16)};e._makeDomArray=function(t,e){return null==t?[]:t.cheerio?e?c(t.get(),t.options):t.get():Array.isArray(t)?h.flatten(t.map(function(t){return this._makeDomArray(t,e)},this)):"string"==typeof t?a(t,this.options):e?c([t]):[t]};var p=function(t){return function(){var e=f.call(arguments),r=this.length-1;return u(this,function(n,o){var a,s;s="function"==typeof e[0]?e[0].call(o,n,i.html(o.children)):e,a=this._makeDomArray(s,n<r),t(a,o.children,o);})}},d=function(t,e,r,n,i){var o,a,s,u,c,l=[e,r].concat(n),f=t[e-1]||null,h=t[e]||null;for(o=0,a=n.length;o<a;++o)u=n[o],c=u.parent||u.root,s=c&&c.children.indexOf(n[o]),c&&s>-1&&(c.children.splice(s,1),i===c&&e>s&&l[0]--),u.root=null,u.parent=i,u.prev&&(u.prev.next=u.next||null),u.next&&(u.next.prev=u.prev||null),u.prev=n[o-1]||f,u.next=n[o+1]||h;return f&&(f.next=n[0]),h&&(h.prev=n[n.length-1]),t.splice.apply(t,l)};e.appendTo=function(t){return t.cheerio||(t=this.constructor.call(this.constructor,t,null,this._originalRoot)),t.append(this),this},e.prependTo=function(t){return t.cheerio||(t=this.constructor.call(this.constructor,t,null,this._originalRoot)),t.prepend(this),this},e.append=p(function(t,e,r){d(e,e.length,0,t,r);}),e.prepend=p(function(t,e,r){d(e,0,0,t,r);}),e.wrap=function(t){var e="function"==typeof t&&t,r=this.length-1;return h.forEach(this,h.bind(function(n,i){var a,s,u=n.parent||n.root,c=u.children;u&&(e&&(t=e.call(n,i)),"string"!=typeof t||l(t)||(t=this.parents().last().find(t).clone()),a=this._makeDomArray(t,i<r).slice(0,1),s=c.indexOf(n),o([n],a[0]),d(c,s,0,a,u));},this)),this},e.after=function(){var t=f.call(arguments),e=this.length-1;return u(this,function(r,n){var o=n.parent||n.root;if(o){var a,s,u=o.children,c=u.indexOf(n);c<0||(a="function"==typeof t[0]?t[0].call(n,r,i.html(n.children)):t,s=this._makeDomArray(a,r<e),d(u,c+1,0,s,o));}}),this},e.insertAfter=function(t){var e=[],r=this;return "string"==typeof t&&(t=this.constructor.call(this.constructor,t,null,this._originalRoot)),t=this._makeDomArray(t),r.remove(),u(t,function(t,n){var i=r._makeDomArray(r.clone()),o=n.parent||n.root;if(o){var a=o.children,s=a.indexOf(n);s<0||(d(a,s+1,0,i,o),e.push(i));}}),this.constructor.call(this.constructor,this._makeDomArray(e))},e.before=function(){var t=f.call(arguments),e=this.length-1;return u(this,function(r,n){var o=n.parent||n.root;if(o){var a,s,u=o.children,c=u.indexOf(n);c<0||(a="function"==typeof t[0]?t[0].call(n,r,i.html(n.children)):t,s=this._makeDomArray(a,r<e),d(u,c,0,s,o));}}),this},e.insertBefore=function(t){var e=[],r=this;return "string"==typeof t&&(t=this.constructor.call(this.constructor,t,null,this._originalRoot)),t=this._makeDomArray(t),r.remove(),u(t,function(t,n){var i=r._makeDomArray(r.clone()),o=n.parent||n.root;if(o){var a=o.children,s=a.indexOf(n);s<0||(d(a,s,0,i,o),e.push(i));}}),this.constructor.call(this.constructor,this._makeDomArray(e))},e.remove=function(t){var e=this;return t&&(e=e.filter(t)),u(e,function(t,e){var r=e.parent||e.root;if(r){var n=r.children,i=n.indexOf(e);i<0||(n.splice(i,1),e.prev&&(e.prev.next=e.next),e.next&&(e.next.prev=e.prev),e.prev=e.next=e.parent=e.root=null);}}),this},e.replaceWith=function(t){var e=this;return u(this,function(r,n){var i=n.parent||n.root;if(i){var a,s=i.children,u=e._makeDomArray("function"==typeof t?t.call(n,r,n):t);o(u,null),a=s.indexOf(n),d(s,a,1,u,i),n.parent=n.prev=n.next=n.root=null;}}),this},e.empty=function(){return u(this,function(t,e){h.forEach(e.children,function(t){t.next=t.prev=t.parent=null;}),e.children.length=0;}),this},e.html=function(t){if(void 0===t)return this[0]&&this[0].children?i.html(this[0].children,this.options):null;var e=this.options;return u(this,function(r,n){h.forEach(n.children,function(t){t.next=t.prev=t.parent=null;});var i=t.cheerio?t.clone().get():a(""+t,e);o(i,n);}),this},e.toString=function(){return i.html(this,this.options)},e.text=function(t){return void 0===t?i.text(this):"function"==typeof t?u(this,function(r,n){var o=[n];return e.text.call(o,t.call(n,r,i.text(o)))}):(u(this,function(e,r){h.forEach(r.children,function(t){t.next=t.prev=t.parent=null;}),o({data:""+t,type:"text",parent:r,prev:null,next:null,children:[]},r);}),this)},e.clone=function(){return this._make(c(this.get(),this.options))};},function(t,e,r){(function(e){function r(t,e){for(var r=-1,n=e.length,i=t.length;++r<n;)t[i+r]=e[r];return t}function n(t,e,o,a,s){var u=-1,c=t.length;for(o||(o=i),s||(s=[]);++u<c;){var l=t[u];e>0&&o(l)?e>1?n(l,e-1,o,a,s):r(s,l):a||(s[s.length]=l);}return s}function i(t){return E(t)||a(t)||!!(k&&t&&t[k])}function o(t){return (t?t.length:0)?n(t,1):[]}function a(t){return u(t)&&w.call(t,"callee")&&(!j.call(t,"callee")||x.call(t)==d)}function s(t){return null!=t&&l(t.length)&&!c(t)}function u(t){return h(t)&&s(t)}function c(t){var e=f(t)?x.call(t):"";return e==g||e==v}function l(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=p}function f(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function h(t){return !!t&&"object"==typeof t}var p=9007199254740991,d="[object Arguments]",g="[object Function]",v="[object GeneratorFunction]",b="object"==typeof e&&e&&e.Object===Object&&e,y="object"==typeof self&&self&&self.Object===Object&&self,m=b||y||Function("return this")(),_=Object.prototype,w=_.hasOwnProperty,x=_.toString,S=m.Symbol,j=_.propertyIsEnumerable,k=S?S.isConcatSpreadable:void 0,E=Array.isArray;t.exports=o;}).call(e,r(0));},function(t,e,r){function n(t,e,r,a){if("string"==typeof e){var s=i(t);"function"==typeof r&&(r=r.call(t,a,s[e])),""===r?delete s[e]:null!=r&&(s[e]=r),t.attribs.style=o(s);}else "object"==typeof e&&Object.keys(e).forEach(function(r){n(t,r,e[r]);});}function i(t,e){var r=a(t.attribs.style);return "string"==typeof e?r[e]:Array.isArray(e)?u.pick(r,e):r}function o(t){return Object.keys(t||{}).reduce(function(e,r){return e+=(e?" ":"")+r+": "+t[r]+";"},"")}function a(t){return t=(t||"").trim(),t?t.split(";").reduce(function(t,e){var r=e.indexOf(":");return r<1||r===e.length-1?t:(t[e.slice(0,r).trim()]=e.slice(r+1).trim(),t)},{}):{}}var s=r(9).domEach,u={pick:r(104)},c=Object.prototype.toString;e.css=function(t,e){return 2===arguments.length||"[object Object]"===c.call(t)?s(this,function(r,i){n(i,t,e,r);}):i(this[0],t)};},function(t,e,r){(function(e){function r(t,e,r){switch(r.length){case 0:return t.call(e);case 1:return t.call(e,r[0]);case 2:return t.call(e,r[0],r[1]);case 3:return t.call(e,r[0],r[1],r[2])}return t.apply(e,r)}function n(t,e){for(var r=-1,n=t?t.length:0,i=Array(n);++r<n;)i[r]=e(t[r],r,t);return i}function i(t,e){for(var r=-1,n=e.length,i=t.length;++r<n;)t[i+r]=e[r];return t}function o(t,e,r,n,a){var s=-1,c=t.length;for(r||(r=u),a||(a=[]);++s<c;){var l=t[s];e>0&&r(l)?e>1?o(l,e-1,r,n,a):i(a,l):n||(a[a.length]=l);}return a}function a(t,e){return t=Object(t),s(t,e,function(e,r){return r in t})}function s(t,e,r){for(var n=-1,i=e.length,o={};++n<i;){var a=e[n],s=t[a];r(s,a)&&(o[a]=s);}return o}function u(t){return q(t)||l(t)||!!(B&&t&&t[B])}function c(t){if("string"==typeof t||b(t))return t;var e=t+"";return "0"==e&&1/t==-y?"-0":e}function l(t){return h(t)&&T.call(t,"callee")&&(!C.call(t,"callee")||O.call(t)==_)}function f(t){return null!=t&&d(t.length)&&!p(t)}function h(t){return v(t)&&f(t)}function p(t){var e=g(t)?O.call(t):"";return e==w||e==x}function d(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=m}function g(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function v(t){return !!t&&"object"==typeof t}function b(t){return "symbol"==typeof t||v(t)&&O.call(t)==S}var y=1/0,m=9007199254740991,_="[object Arguments]",w="[object Function]",x="[object GeneratorFunction]",S="[object Symbol]",j="object"==typeof e&&e&&e.Object===Object&&e,k="object"==typeof self&&self&&self.Object===Object&&self,E=j||k||Function("return this")(),A=Object.prototype,T=A.hasOwnProperty,O=A.toString,L=E.Symbol,C=A.propertyIsEnumerable,B=L?L.isConcatSpreadable:void 0,D=Math.max,q=Array.isArray,P=function(t,e){return e=D(void 0===e?t.length-1:e,0),function(){for(var n=arguments,i=-1,o=D(n.length-e,0),a=Array(o);++i<o;)a[i]=n[e+i];i=-1;for(var s=Array(e+1);++i<e;)s[i]=n[i];return s[e]=a,r(t,this,s)}}(function(t,e){return null==t?{}:a(t,n(o(e,1),c))});t.exports=P;}).call(e,r(0));},function(t,e,r){var n=/%20/g,i=/\r?\n/g,o={map:r(106)};e.serialize=function(){var t=this.serializeArray();return o.map(t,function(t){return encodeURIComponent(t.name)+"="+encodeURIComponent(t.value)}).join("&").replace(n,"+")},e.serializeArray=function(){var t=this.constructor;return this.map(function(){var e=this,r=t(e);return "form"===e.name?r.find("input,select,textarea,keygen").toArray():r.filter("input,select,textarea,keygen").toArray()}).filter('[name!=""]:not(:disabled):not(:submit, :button, :image, :reset, :file):matches([checked], :not(:checkbox, :radio))').map(function(e,r){var n=t(r),a=n.attr("name"),s=n.val();return null==s?null:Array.isArray(s)?o.map(s,function(t){return {name:a,value:t.replace(i,"\r\n")}}):{name:a,value:s.replace(i,"\r\n")}}).get()};},function(t,e,r){(function(t,r){function n(t,e){for(var r=-1,n=t?t.length:0,i=Array(n);++r<n;)i[r]=e(t[r],r,t);return i}function i(t,e){for(var r=-1,n=t?t.length:0;++r<n;)if(e(t[r],r,t))return !0;return !1}function o(t){return function(e){return null==e?void 0:e[t]}}function a(t,e){for(var r=-1,n=Array(t);++r<t;)n[r]=e(r);return n}function s(t,e){return null==t?void 0:t[e]}function u(t){var e=!1;if(null!=t&&"function"!=typeof t.toString)try{e=!!(t+"");}catch(t){}return e}function c(t){var e=-1,r=Array(t.size);return t.forEach(function(t,n){r[++e]=[n,t];}),r}function l(t){var e=-1,r=Array(t.size);return t.forEach(function(t){r[++e]=t;}),r}function f(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function h(){this.__data__=ze?ze(null):{};}function p(t){return this.has(t)&&delete this.__data__[t]}function d(t){var e=this.__data__;if(ze){var r=e[t];return r===It?void 0:r}return Oe.call(e,t)?e[t]:void 0}function g(t){var e=this.__data__;return ze?void 0!==e[t]:Oe.call(e,t)}function v(t,e){return this.__data__[t]=ze&&void 0===e?It:e,this}function b(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function y(){this.__data__=[];}function m(t){var e=this.__data__,r=N(e,t);return !(r<0)&&(r==e.length-1?e.pop():Pe.call(e,r,1),!0)}function _(t){var e=this.__data__,r=N(e,t);return r<0?void 0:e[r][1]}function w(t){return N(this.__data__,t)>-1}function x(t,e){var r=this.__data__,n=N(r,t);return n<0?r.push([t,e]):r[n][1]=e,this}function S(t){var e=-1,r=t?t.length:0;for(this.clear();++e<r;){var n=t[e];this.set(n[0],n[1]);}}function j(){this.__data__={hash:new f,map:new(Ie||b),string:new f};}function k(t){return at(this,t).delete(t)}function E(t){return at(this,t).get(t)}function A(t){return at(this,t).has(t)}function T(t,e){return at(this,t).set(t,e),this}function O(t){var e=-1,r=t?t.length:0;for(this.__data__=new S;++e<r;)this.add(t[e]);}function L(t){return this.__data__.set(t,It),this}function C(t){return this.__data__.has(t)}function B(t){this.__data__=new b(t);}function D(){this.__data__=new b;}function q(t){return this.__data__.delete(t)}function P(t){return this.__data__.get(t)}function M(t){return this.__data__.has(t)}function R(t,e){var r=this.__data__;if(r instanceof b){var n=r.__data__;if(!Ie||n.length<Mt-1)return n.push([t,e]),this;r=this.__data__=new S(n);}return r.set(t,e),this}function I(t,e){var r=er(t)||xt(t)?a(t.length,String):[],n=r.length,i=!!n;for(var o in t)!e&&!Oe.call(t,o)||i&&("length"==o||lt(o,n))||r.push(o);return r}function N(t,e){for(var r=t.length;r--;)if(wt(t[r][0],e))return r;return -1}function U(t,e){return t&&Ze(t,e,Dt)}function F(t,e){e=ft(e,t)?[e]:rt(e);for(var r=0,n=e.length;null!=t&&r<n;)t=t[bt(e[r++])];return r&&r==n?t:void 0}function z(t){return Le.call(t)}function V(t,e){return null!=t&&e in Object(t)}function $(t,e,r,n,i){return t===e||(null==t||null==e||!At(t)&&!Tt(e)?t!==t&&e!==e:H(t,e,$,r,n,i))}function H(t,e,r,n,i,o){var a=er(t),s=er(e),c=$t,l=$t;a||(c=Ke(t),c=c==Vt?Zt:c),s||(l=Ke(e),l=l==Vt?Zt:l);var f=c==Zt&&!u(t),h=l==Zt&&!u(e),p=c==l;if(p&&!f)return o||(o=new B),a||rr(t)?nt(t,e,r,n,i,o):it(t,e,c,r,n,i,o);if(!(i&Ut)){var d=f&&Oe.call(t,"__wrapped__"),g=h&&Oe.call(e,"__wrapped__");if(d||g){var v=d?t.value():t,b=g?e.value():e;return o||(o=new B),r(v,b,n,i,o)}}return !!p&&(o||(o=new B),ot(t,e,r,n,i,o))}function W(t,e,r,n){var i=r.length,o=i,a=!n;if(null==t)return !o;for(t=Object(t);i--;){var s=r[i];if(a&&s[2]?s[1]!==t[s[0]]:!(s[0]in t))return !1}for(;++i<o;){s=r[i];var u=s[0],c=t[u],l=s[1];if(a&&s[2]){if(void 0===c&&!(u in t))return !1}else {var f=new B;if(n)var h=n(c,l,u,t,e,f);if(!(void 0===h?$(l,c,n,Nt|Ut,f):h))return !1}}return !0}function G(t){return !(!At(t)||pt(t))&&(kt(t)||u(t)?Ce:fe).test(yt(t))}function Y(t){return Tt(t)&&Et(t.length)&&!!pe[Le.call(t)]}function J(t){return "function"==typeof t?t:null==t?qt:"object"==typeof t?er(t)?K(t[0],t[1]):Z(t):Pt(t)}function Q(t){if(!dt(t))return Me(t);var e=[];for(var r in Object(t))Oe.call(t,r)&&"constructor"!=r&&e.push(r);return e}function X(t,e){var r=-1,n=St(t)?Array(t.length):[];return Xe(t,function(t,i,o){n[++r]=e(t,i,o);}),n}function Z(t){var e=st(t);return 1==e.length&&e[0][2]?vt(e[0][0],e[0][1]):function(r){return r===t||W(r,t,e)}}function K(t,e){return ft(t)&&gt(e)?vt(bt(t),e):function(r){var n=Ct(r,t);return void 0===n&&n===e?Bt(r,t):$(e,n,void 0,Nt|Ut)}}function tt(t){return function(e){return F(e,t)}}function et(t){if("string"==typeof t)return t;if(Ot(t))return Qe?Qe.call(t):"";var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function rt(t){return er(t)?t:tr(t)}function nt(t,e,r,n,o,a){var s=o&Ut,u=t.length,c=e.length;if(u!=c&&!(s&&c>u))return !1;var l=a.get(t);if(l&&a.get(e))return l==e;var f=-1,h=!0,p=o&Nt?new O:void 0;for(a.set(t,e),a.set(e,t);++f<u;){var d=t[f],g=e[f];if(n)var v=s?n(g,d,f,e,t,a):n(d,g,f,t,e,a);if(void 0!==v){if(v)continue;h=!1;break}if(p){if(!i(e,function(t,e){if(!p.has(e)&&(d===t||r(d,t,n,o,a)))return p.add(e)})){h=!1;break}}else if(d!==g&&!r(d,g,n,o,a)){h=!1;break}}return a.delete(t),a.delete(e),h}function it(t,e,r,n,i,o,a){switch(r){case ie:if(t.byteLength!=e.byteLength||t.byteOffset!=e.byteOffset)return !1;t=t.buffer,e=e.buffer;case ne:return !(t.byteLength!=e.byteLength||!n(new De(t),new De(e)));case Ht:case Wt:case Xt:return wt(+t,+e);case Gt:return t.name==e.name&&t.message==e.message;case Kt:case ee:return t==e+"";case Qt:var s=c;case te:var u=o&Ut;if(s||(s=l),t.size!=e.size&&!u)return !1;var f=a.get(t);if(f)return f==e;o|=Nt,a.set(t,e);var h=nt(s(t),s(e),n,i,o,a);return a.delete(t),h;case re:if(Je)return Je.call(t)==Je.call(e)}return !1}function ot(t,e,r,n,i,o){var a=i&Ut,s=Dt(t),u=s.length;if(u!=Dt(e).length&&!a)return !1;for(var c=u;c--;){var l=s[c];if(!(a?l in e:Oe.call(e,l)))return !1}var f=o.get(t);if(f&&o.get(e))return f==e;var h=!0;o.set(t,e),o.set(e,t);for(var p=a;++c<u;){l=s[c];var d=t[l],g=e[l];if(n)var v=a?n(g,d,l,e,t,o):n(d,g,l,t,e,o);if(!(void 0===v?d===g||r(d,g,n,i,o):v)){h=!1;break}p||(p="constructor"==l);}if(h&&!p){var b=t.constructor,y=e.constructor;b!=y&&"constructor"in t&&"constructor"in e&&!("function"==typeof b&&b instanceof b&&"function"==typeof y&&y instanceof y)&&(h=!1);}return o.delete(t),o.delete(e),h}function at(t,e){var r=t.__data__;return ht(e)?r["string"==typeof e?"string":"hash"]:r.map}function st(t){for(var e=Dt(t),r=e.length;r--;){var n=e[r],i=t[n];e[r]=[n,i,gt(i)];}return e}function ut(t,e){var r=s(t,e);return G(r)?r:void 0}function ct(t,e,r){e=ft(e,t)?[e]:rt(e);for(var n,i=-1,o=e.length;++i<o;){var a=bt(e[i]);if(!(n=null!=t&&r(t,a)))break;t=t[a];}if(n)return n;var o=t?t.length:0;return !!o&&Et(o)&&lt(a,o)&&(er(t)||xt(t))}function lt(t,e){return !!(e=null==e?zt:e)&&("number"==typeof t||he.test(t))&&t>-1&&t%1==0&&t<e}function ft(t,e){if(er(t))return !1;var r=typeof t;return !("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=t&&!Ot(t))||(ae.test(t)||!oe.test(t)||null!=e&&t in Object(e))}function ht(t){var e=typeof t;return "string"==e||"number"==e||"symbol"==e||"boolean"==e?"__proto__"!==t:null===t}function pt(t){return !!Ae&&Ae in t}function dt(t){var e=t&&t.constructor;return t===("function"==typeof e&&e.prototype||ke)}function gt(t){return t===t&&!At(t)}function vt(t,e){return function(r){return null!=r&&(r[t]===e&&(void 0!==e||t in Object(r)))}}function bt(t){if("string"==typeof t||Ot(t))return t;var e=t+"";return "0"==e&&1/t==-Ft?"-0":e}function yt(t){if(null!=t){try{return Te.call(t)}catch(t){}try{return t+""}catch(t){}}return ""}function mt(t,e){return (er(t)?n:X)(t,J(e))}function _t(t,e){if("function"!=typeof t||e&&"function"!=typeof e)throw new TypeError(Rt);var r=function(){var n=arguments,i=e?e.apply(this,n):n[0],o=r.cache;if(o.has(i))return o.get(i);var a=t.apply(this,n);return r.cache=o.set(i,a),a};return r.cache=new(_t.Cache||S),r}function wt(t,e){return t===e||t!==t&&e!==e}function xt(t){return jt(t)&&Oe.call(t,"callee")&&(!qe.call(t,"callee")||Le.call(t)==Vt)}function St(t){return null!=t&&Et(t.length)&&!kt(t)}function jt(t){return Tt(t)&&St(t)}function kt(t){var e=At(t)?Le.call(t):"";return e==Yt||e==Jt}function Et(t){return "number"==typeof t&&t>-1&&t%1==0&&t<=zt}function At(t){var e=typeof t;return !!t&&("object"==e||"function"==e)}function Tt(t){return !!t&&"object"==typeof t}function Ot(t){return "symbol"==typeof t||Tt(t)&&Le.call(t)==re}function Lt(t){return null==t?"":et(t)}function Ct(t,e,r){var n=null==t?void 0:F(t,e);return void 0===n?r:n}function Bt(t,e){return null!=t&&ct(t,e,V)}function Dt(t){return St(t)?I(t):Q(t)}function qt(t){return t}function Pt(t){return ft(t)?o(bt(t)):tt(t)}var Mt=200,Rt="Expected a function",It="__lodash_hash_undefined__",Nt=1,Ut=2,Ft=1/0,zt=9007199254740991,Vt="[object Arguments]",$t="[object Array]",Ht="[object Boolean]",Wt="[object Date]",Gt="[object Error]",Yt="[object Function]",Jt="[object GeneratorFunction]",Qt="[object Map]",Xt="[object Number]",Zt="[object Object]",Kt="[object RegExp]",te="[object Set]",ee="[object String]",re="[object Symbol]",ne="[object ArrayBuffer]",ie="[object DataView]",oe=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,ae=/^\w*$/,se=/^\./,ue=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,ce=/[\\^$.*+?()[\]{}|]/g,le=/\\(\\)?/g,fe=/^\[object .+?Constructor\]$/,he=/^(?:0|[1-9]\d*)$/,pe={};pe["[object Float32Array]"]=pe["[object Float64Array]"]=pe["[object Int8Array]"]=pe["[object Int16Array]"]=pe["[object Int32Array]"]=pe["[object Uint8Array]"]=pe["[object Uint8ClampedArray]"]=pe["[object Uint16Array]"]=pe["[object Uint32Array]"]=!0,pe[Vt]=pe[$t]=pe[ne]=pe[Ht]=pe[ie]=pe[Wt]=pe[Gt]=pe[Yt]=pe[Qt]=pe[Xt]=pe[Zt]=pe[Kt]=pe[te]=pe[ee]=pe["[object WeakMap]"]=!1;var de="object"==typeof t&&t&&t.Object===Object&&t,ge="object"==typeof self&&self&&self.Object===Object&&self,ve=de||ge||Function("return this")(),be="object"==typeof e&&e&&!e.nodeType&&e,ye=be&&"object"==typeof r&&r&&!r.nodeType&&r,me=ye&&ye.exports===be,_e=me&&de.process,we=function(){try{return _e&&_e.binding("util")}catch(t){}}(),xe=we&&we.isTypedArray,Se=Array.prototype,je=Function.prototype,ke=Object.prototype,Ee=ve["__core-js_shared__"],Ae=function(){var t=/[^.]+$/.exec(Ee&&Ee.keys&&Ee.keys.IE_PROTO||"");return t?"Symbol(src)_1."+t:""}(),Te=je.toString,Oe=ke.hasOwnProperty,Le=ke.toString,Ce=RegExp("^"+Te.call(Oe).replace(ce,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),Be=ve.Symbol,De=ve.Uint8Array,qe=ke.propertyIsEnumerable,Pe=Se.splice,Me=function(t,e){return function(r){return t(e(r))}}(Object.keys,Object),Re=ut(ve,"DataView"),Ie=ut(ve,"Map"),Ne=ut(ve,"Promise"),Ue=ut(ve,"Set"),Fe=ut(ve,"WeakMap"),ze=ut(Object,"create"),Ve=yt(Re),$e=yt(Ie),He=yt(Ne),We=yt(Ue),Ge=yt(Fe),Ye=Be?Be.prototype:void 0,Je=Ye?Ye.valueOf:void 0,Qe=Ye?Ye.toString:void 0;f.prototype.clear=h,f.prototype.delete=p,f.prototype.get=d,f.prototype.has=g,f.prototype.set=v,b.prototype.clear=y,b.prototype.delete=m,b.prototype.get=_,b.prototype.has=w,b.prototype.set=x,S.prototype.clear=j,S.prototype.delete=k,S.prototype.get=E,S.prototype.has=A,S.prototype.set=T,O.prototype.add=O.prototype.push=L,O.prototype.has=C,B.prototype.clear=D,B.prototype.delete=q,B.prototype.get=P,B.prototype.has=M,B.prototype.set=R;var Xe=function(t,e){return function(r,n){if(null==r)return r;if(!St(r))return t(r,n);for(var i=r.length,o=e?i:-1,a=Object(r);(e?o--:++o<i)&&!1!==n(a[o],o,a););return r}}(U),Ze=function(t){return function(e,r,n){for(var i=-1,o=Object(e),a=n(e),s=a.length;s--;){var u=a[t?s:++i];if(!1===r(o[u],u,o))break}return e}}(),Ke=z;(Re&&Ke(new Re(new ArrayBuffer(1)))!=ie||Ie&&Ke(new Ie)!=Qt||Ne&&"[object Promise]"!=Ke(Ne.resolve())||Ue&&Ke(new Ue)!=te||Fe&&"[object WeakMap]"!=Ke(new Fe))&&(Ke=function(t){var e=Le.call(t),r=e==Zt?t.constructor:void 0,n=r?yt(r):void 0;if(n)switch(n){case Ve:return ie;case $e:return Qt;case He:return "[object Promise]";case We:return te;case Ge:return "[object WeakMap]"}return e});var tr=_t(function(t){t=Lt(t);var e=[];return se.test(t)&&e.push(""),t.replace(ue,function(t,r,n,i){e.push(n?i.replace(le,"$1"):r||t);}),e});_t.Cache=S;var er=Array.isArray,rr=xe?function(t){return function(e){return t(e)}}(xe):Y;r.exports=mt;}).call(e,r(0),r(5)(t));},function(t,e){t.exports={_from:"cheerio@0.22.0",_id:"cheerio@0.22.0",_inBundle:!1,_integrity:"sha1-qbqoYKP5tZWmuBsahocxIe06Jp4=",_location:"/cheerio",_phantomChildren:{},_requested:{type:"version",registry:!0,raw:"cheerio@0.22.0",name:"cheerio",escapedName:"cheerio",rawSpec:"0.22.0",saveSpec:null,fetchSpec:"0.22.0"},_requiredBy:["/juice"],_resolved:"https://registry.npmjs.org/cheerio/-/cheerio-0.22.0.tgz",_shasum:"a9baa860a3f9b595a6b81b1a86873121ed3a269e",_spec:"cheerio@0.22.0",_where:"/Users/artur/Sites/grapesjs-plugins/grapesjs-preset-newsletter/node_modules/juice",author:{name:"Matt Mueller",email:"mattmuelle@gmail.com",url:"mat.io"},bugs:{url:"https://github.com/cheeriojs/cheerio/issues"},bundleDependencies:!1,dependencies:{"css-select":"~1.2.0","dom-serializer":"~0.1.0",entities:"~1.1.1",htmlparser2:"^3.9.1","lodash.assignin":"^4.0.9","lodash.bind":"^4.1.4","lodash.defaults":"^4.0.1","lodash.filter":"^4.4.0","lodash.flatten":"^4.2.0","lodash.foreach":"^4.3.0","lodash.map":"^4.4.0","lodash.merge":"^4.4.0","lodash.pick":"^4.2.1","lodash.reduce":"^4.4.0","lodash.reject":"^4.4.0","lodash.some":"^4.4.0"},deprecated:!1,description:"Tiny, fast, and elegant implementation of core jQuery designed specifically for the server",devDependencies:{benchmark:"^2.1.0",coveralls:"^2.11.9","expect.js":"~0.3.1",istanbul:"^0.4.3",jquery:"^3.0.0",jsdom:"^9.2.1",jshint:"^2.9.2",mocha:"^2.5.3",xyz:"~0.5.0"},engines:{node:">= 0.6"},files:["index.js","lib"],homepage:"https://github.com/cheeriojs/cheerio#readme",keywords:["htmlparser","jquery","selector","scraper","parser","html"],license:"MIT",main:"./index.js",name:"cheerio",repository:{type:"git",url:"git://github.com/cheeriojs/cheerio.git"},scripts:{test:"make test"},version:"0.22.0"};},function(t,e,r){t.exports={lex:r(45),parse:r(109),stringify:r(110)};},function(t,e,r){function n(t,e){e||(e={}),w=!!e.comments,S=!!e.position,x=0,j=Array.isArray(t)?t.slice():_(t);var n,i,a=[];for(y;i=o();)(n=p(i))&&a.push(n);return {type:"stylesheet",stylesheet:{rules:a}}}function i(t,e){e||(e={});for(var r,n=["type","name","value"],i={},o=0;o<n.length;++o)r=n[o],t[r]&&(i[r]=e[r]||t[r]);for(n=Object.keys(e),o=0;o<n.length;++o)r=n[o],i[r]||(i[r]=e[r]);return S&&(i.position={start:t.start,end:t.end}),i}function o(){var t=j.shift();return t}function a(t){x+=1;var e={};switch(t.type){case"font-face":case"viewport":e.declarations=g();break;case"page":e.prefix=t.prefix,e.declarations=g();break;default:e.prefix=t.prefix,e.rules=v();}return i(t,e)}function s(t){return i(t)}function u(t){return i(t)}function c(t){return i(t,{text:t.text})}function l(t){return i(t)}function f(t){return i(t)}function h(t){function e(t){return t.trim()}return i(t,{type:"rule",selectors:t.text.split(",").map(e),declarations:g()})}function p(t){switch(t.type){case"property":return f(t);case"selector":return h(t);case"at-group-end":return void(x-=1);case"media":case"keyframes":return a(t);case"comment":if(w)return c(t);break;case"charset":return u(t);case"import":return s(t);case"namespace":return l(t);case"font-face":case"supports":case"viewport":case"document":case"page":return a(t)}}function d(t){for(var e,r,n=[];(r=o())&&t&&t(r);)(e=p(r))&&n.push(e);return r&&"end"!==r.type&&j.unshift(r),n}function g(){return d(function(t){return "property"===t.type||"comment"===t.type})}function v(){return d(function(){return x})}var y=!1,m=r(28)("parse"),_=r(45);t.exports=n;var w,x,S,j;},function(t,e,r){function n(t,e){e||(e={}),b=e.indentation||"",v=!!e.compress,g=!!e.comments,v?y=m="":(y="\n",m=" "),w;var n=c(t.stylesheet.rules,p).join("\n").trim();return n}function i(t){return this.level||(this.level=1),t?void(this.level+=t):v?"":Array(this.level).join(b||"")}function o(t){return "@"+t.type+" "+t.value+";"+y}function a(t){var e="",r=t.prefix||"";t.name&&(e=" "+t.name);var n="page"!==t.type;return "@"+r+t.type+e+m+l(t,n)+y}function s(t){return g?"/*"+(t.text||"")+"*/"+y:""}function u(t){var e;return t.selectors?e=t.selectors.join(","+y):(e="@"+t.type,e+=t.name?" "+t.name:""),i()+e+m+l(t)+y}function c(t,e){return t.reduce(function(t,r){var n="comment"===r.type?s(r):e(r);return n&&t.push(n),t},[])}function l(t,e){var r=t.declarations,n=h;return t.rules&&(r=t.rules,n=u),r=f(r,n),r&&(r=y+r+(e?"":y)),"{"+r+i()+"}"}function f(t,e){if(!t)return "";i(1);var r=c(t,e);return i(-1),r.length?r.join(y):""}function h(t){if("property"===t.type)return d(t);}function p(t){switch(t.type){case"rule":return u(t);case"media":case"keyframes":return a(t);case"comment":return s(t);case"import":case"charset":case"namespace":return o(t);case"font-face":case"supports":case"viewport":case"document":case"page":return a(t)}}function d(t){var e=t.name?t.name+":"+m:"";return i()+e+t.value+";"}var g,v,b,y,m,w=!1,x=r(28)("stringify");t.exports=n;},function(t,e,r){function n(t,e){this.text=t,this.spec=void 0,this.styleAttribute=e||!1;}function i(t){try{return o(t)[0]}catch(t){return []}}var o=r(112);t.exports=n,n.prototype.parsed=function(){return this.tokens||(this.tokens=i(this.text)),this.tokens},n.prototype.specificity=function(){function t(r,n){for(var o=n||i(r),a=[e?1:0,0,0,0],s=[],u=0;u<o.length;u++){var c=o[u],l=c.pseudos;if(c.id&&a[1]++,c.attributes&&(a[2]+=c.attributes.length),c.classList&&(a[2]+=c.classList.length),c.tag&&"*"!==c.tag&&a[3]++,l){a[3]+=l.length;for(var f=0;f<l.length;f++)"not"===l[f].name&&(s.push(l[f].value),a[3]--);}}for(var h=s.length;h--;)for(var p=t(s[h]),d=4;d--;)a[d]+=p[d];return a}var e=this.styleAttribute;return this.spec||(this.spec=t(this.text,this.parsed())),this.spec};},function(t,e,r){var n=/([-.*+?^${}()|[\]\/\\])/g,i=/\\/g,o=function(t){return (t+"").replace(n,"\\$1")},a=function(t){return (t+"").replace(i,"")},s=RegExp("^(?:\\s*(,)\\s*|\\s*(<combinator>+)\\s*|(\\s+)|(<unicode>+|\\*)|\\#(<unicode>+)|\\.(<unicode>+)|\\[\\s*(<unicode1>+)(?:\\s*([*^$!~|]?=)(?:\\s*(?:([\"']?)(.*?)\\9)))?\\s*\\](?!\\])|(:+)(<unicode>+)(?:\\((?:(?:([\"'])([^\\13]*)\\13)|((?:\\([^)]+\\)|[^()]*)+))\\))?)".replace(/<combinator>/,"["+o(">+~`!@$%^&={}\\;</")+"]").replace(/<unicode>/g,"(?:[\\w\\u00a1-\\uFFFF-]|\\\\[^\\s0-9a-f])").replace(/<unicode1>/g,"(?:[:\\w\\u00a1-\\uFFFF-]|\\\\[^\\s0-9a-f])")),u=function(t){this.combinator=t||" ",this.tag="*";};u.prototype.toString=function(){if(!this.raw){var t,e,r="";if(r+=this.tag||"*",this.id&&(r+="#"+this.id),this.classes&&(r+="."+this.classList.join(".")),this.attributes)for(t=0;e=this.attributes[t++];)r+="["+e.name+(e.operator?e.operator+'"'+e.value+'"':"")+"]";if(this.pseudos)for(t=0;e=this.pseudos[t++];)r+=":"+e.name,e.value&&(r+="("+e.value+")");this.raw=r;}return this.raw};var c=function(){this.length=0;};c.prototype.toString=function(){if(!this.raw){for(var t,e="",r=0;t=this[r++];)1!==r&&(e+=" ")," "!==t.combinator&&(e+=t.combinator+" "),e+=t;this.raw=e;}return this.raw};var l=function(t,e,r,n,i,s,l,f,h,p,d,g,v,b,y,m){var _,w;if((e||!this.length)&&(_=this[this.length++]=new c,e))return "";if(_||(_=this[this.length-1]),(r||n||!_.length)&&(w=_[_.length++]=new u(r)),w||(w=_[_.length-1]),i)w.tag=a(i);else if(s)w.id=a(s);else if(l){var x=a(l),S=w.classes||(w.classes={});if(!S[x]){S[x]=o(l);var j=w.classList||(w.classList=[]);j.push(x),j.sort();}}else v?(m=m||y,(w.pseudos||(w.pseudos=[])).push({type:1==g.length?"class":"element",name:a(v),escapedName:o(v),value:m?a(m):null,escapedValue:m?o(m):null})):f&&(d=d?o(d):null,(w.attributes||(w.attributes=[])).push({operator:h,name:a(f),escapedName:o(f),value:d?a(d):null,escapedValue:d?o(d):null}));return ""},f=function(t){this.length=0;for(var e,r=this,n=t;t;){if((e=t.replace(s,function(){return l.apply(r,arguments)}))===t)throw new Error(n+" is an invalid expression");t=e;}};f.prototype.toString=function(){if(!this.raw){for(var t,e=[],r=0;t=this[r++];)e.push(t);this.raw=e.join(", ");}return this.raw};var h={},p=function(t){return null==t?null:(t=(""+t).replace(/^\s+|\s+$/g,""),h[t]||(h[t]=new f(t)))};t.exports=p;},function(t,e,r){function n(t,e,r,n,i){this.prop=t,this.value=e,this.selector=r,this.priority=n||0,this.additionalPriority=i||[];}t.exports=n;var i=r(27);n.prototype.compareFunc=function(t){var e=[];e.push.apply(e,this.selector.specificity()),e.push.apply(e,this.additionalPriority),e[0]+=this.priority;var r=[];return r.push.apply(r,t.selector.specificity()),r.push.apply(r,t.additionalPriority),r[0]+=t.priority,i.compareFunc(e,r)},n.prototype.compare=function(t){return 1===this.compareFunc(t)?this:t},n.prototype.toString=function(){return this.prop+": "+this.value.replace(/['"]+/g,"")+";"};},function(t,e,r){var n=r(27);t.exports=function(t){function e(e,o,s){function u(r){for(var o=r[0],u=r[1],c=new n.Selector(o),l=c.parsed(),f=i(l),h=0;h<l.length;++h){var p=l[h];if(p.pseudos)for(var d=0;d<p.pseudos.length;++d){var b=p.pseudos[d];if(t.ignoredPseudos.indexOf(b.name)>=0)return}}if(f){var y=l[l.length-1],m=y.pseudos;y.pseudos=a(y.pseudos),o=l.toString(),y.pseudos=m;}var _;try{_=e(o);}catch(t){return}_.each(function(){function r(e,r){for(var o=0,a=e.length;o<a;o++)if("property"==e[o].type){var u=e[o].name,c=e[o].value,l=null!==e[o].value.match(/!important$/);l&&!s.preserveImportant&&(c=c.replace(/\s*!important$/,""));var f=[e[o].position.start.line,e[o].position.start.col],h=new n.Property(u,c,r,l?2:0,f),p=i.styleProps[u];t.excludedProperties.indexOf(u)<0&&(p&&p.compare(h)===h||!p)&&(p&&p.selector!==r?delete i.styleProps[u]:p&&(h.nextProp=p),i.styleProps[u]=h);}}var i=this;if(!(i.name&&t.nonVisualElements.indexOf(i.name.toUpperCase())>=0)){if(f){var o="pseudo"+f,a=i[o];a||(a=i[o]=e("<span />").get(0),a.pseudoElementType=f,a.pseudoElementParent=i,i[o]=a),i=a;}if(!i.styleProps){if(i.styleProps={},e(i).attr(v)){var l="* { "+e(i).attr(v)+" } ";r(n.parseCSS(l)[0][1],new n.Selector("<style>",!0));}g.push(i);}r(u,c);}});}function c(t){var r=(Object.keys(t.styleProps).length,[]);Object.keys(t.styleProps).forEach(function(e){for(var n=t.styleProps[e];void 0!==n;)r.push(n),n=n.nextProp;}),r.sort(function(t,e){return t.compareFunc(e)});var n=r.filter(function(t){return "content"!==t.prop}).map(function(t){return t.prop+": "+t.value.replace(/["]/g,"'")+";"}).join(" ");n&&e(t).attr(v,n);}function l(t){if(t.pseudoElementType&&t.styleProps.content){var n=r(t.styleProps.content.value);n.img?(t.name="img",e(t).attr("src",n.img)):e(t).text(n);var i=t.pseudoElementParent;"before"===t.pseudoElementType?e(i).prepend(t):e(i).append(t);}}function f(r,n){if(r.name){var i=r.name.toUpperCase();if(t[n+"Elements"].indexOf(i)>-1)for(var o in r.styleProps)if(r.styleProps[o].prop===n){if(r.styleProps[o].value.match(/px/)){var a=r.styleProps[o].value.replace("px","");return void e(r).attr(n,a)}if(t.tableElements.indexOf(i)>-1&&r.styleProps[o].value.match(/\%/))return void e(r).attr(n,r.styleProps[o].value)}}}function h(t){return 0!==t.indexOf("url(")?t:t.replace(/^url\((["'])?([^"']+)\1\)$/,"$2")}function p(r){if(r.name){var n=r.name.toUpperCase(),i=Object.keys(t.styleToAttribute);if(t.tableElements.indexOf(n)>-1)for(var o in r.styleProps)if(i.indexOf(r.styleProps[o].prop)>-1){var a=t.styleToAttribute[r.styleProps[o].prop],s=r.styleProps[o].value;"background"===a&&(s=h(s)),e(r).attr(a,s);}}}s=s||{};var d=n.parseCSS(o),g=[],v="style";if(s.styleAttributeName&&(v=s.styleAttributeName),d.forEach(u),g.forEach(c),s.inlinePseudoElements&&g.forEach(l),s.applyWidthAttributes&&g.forEach(function(t){f(t,"width");}),s.applyHeightAttributes&&g.forEach(function(t){f(t,"height");}),s.applyAttributesTableElements&&g.forEach(p),s.insertPreservedExtraCss&&s.extraCss){var b=n.getPreservedText(s.extraCss,{mediaQueries:s.preserveMediaQueries,fontFaces:s.preserveFontFaces});if(b){var y=null;!0!==s.insertPreservedExtraCss?y=e(s.insertPreservedExtraCss):(y=e("head"),y.length||(y=e("body")),y.length||(y=e.root())),y.first().append("<style>"+b+"</style>");}}}function r(t){if("none"===t||"normal"===t)return "";var e=t.match(/^\s*url\s*\(\s*(.*?)\s*\)\s*$/i);if(e){return {img:e[1].replace(/^['"]|['"]$/g,"")}}return t=t.slice(1,t.length-1),t=t.replace(/\\/g,"")}function i(t){if(0!==t.length){var e=t[t.length-1].pseudos;if(e)for(var r=0;r<e.length;r++)if(o(e[r]))return e[r].name}}function o(t){return "before"===t.name||"after"===t.name}function a(t){return t.filter(function(t){return !o(t)})}function s(t,r){r=n.getDefaultOptions(r);var i=c(t,r);return i+="\n"+r.extraCss,e(t,i,r),t}function u(t,e){var r,i,o,a=[],s=t("style");return s.each(function(){if(o=this,r=o.childNodes,1===r.length){if(i=r[0].data,e.applyStyleTags&&void 0===t(o).attr("data-embed")&&a.push(i),e.removeStyleTags&&void 0===t(o).attr("data-embed")){var s=n.getPreservedText(o.childNodes[0].nodeValue,{mediaQueries:e.preserveMediaQueries,fontFaces:e.preserveFontFaces});s?o.childNodes[0].nodeValue=s:t(o).remove();}t(o).removeAttr("data-embed");}}),a}function c(t,e){return u(t,e).join("\n")}return t.ignoredPseudos=["hover","active","focus","visited","link"],t.widthElements=["TABLE","TD","IMG"],t.heightElements=["TABLE","TD","IMG"],t.tableElements=["TABLE","TD","TH","TR","TD","CAPTION","COLGROUP","COL","THEAD","TBODY","TFOOT"],t.nonVisualElements=["HEAD","TITLE","BASE","LINK","STYLE","META","SCRIPT","NOSCRIPT"],t.styleToAttribute={"background-color":"bgcolor","background-image":"background","text-align":"align","vertical-align":"valign"},t.excludedProperties=[],t.juiceDocument=s,t.inlineDocument=e,t};},function(t,e,r){var n;void 0!==(n=function(){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e="",r="",n=t.editor,i=t.tableStyle||{},o=t.cellStyle||{},a=n.BlockManager;for(var s in i)e+=s+": "+i[s]+"; ";for(var u in o)r+=u+": "+o[u]+"; ";a.getAll().reset(),a.add("sect100",{label:t.sect100BlkLabel,category:t.categoryLabel,attributes:{class:"gjs-fonts gjs-f-b1"},content:'<table style="'+e+'">\n        <tr>\n          <td style="'+r+'"></td>\n        </tr>\n        </table>'}),a.add("sect50",{label:t.sect50BlkLabel,category:t.categoryLabel,attributes:{class:"gjs-fonts gjs-f-b2"},content:'<table style="'+e+'">\n        <tr>\n          <td style="'+r+' width: 50%"></td>\n          <td style="'+r+' width: 50%"></td>\n        </tr>\n        </table>'}),a.add("sect30",{label:t.sect30BlkLabel,category:t.categoryLabel,attributes:{class:"gjs-fonts gjs-f-b3"},content:'<table style="'+e+'">\n        <tr>\n          <td style="'+r+' width: 33.3333%"></td>\n          <td style="'+r+' width: 33.3333%"></td>\n          <td style="'+r+' width: 33.3333%"></td>\n        </tr>\n        </table>'}),a.add("sect37",{label:t.sect37BlkLabel,category:t.categoryLabel,attributes:{class:"gjs-fonts gjs-f-b37"},content:'<table style="'+e+'">\n        <tr>\n          <td style="'+r+' width:30%"></td>\n          <td style="'+r+' width:70%"></td>\n        </tr>\n        </table>'}),a.add("button",{label:t.buttonBlkLabel,category:t.categoryLabel,content:'<a class="button">Button</a>',attributes:{class:"gjs-fonts gjs-f-button"}}),a.add("divider",{label:t.dividerBlkLabel,category:t.categoryLabel,content:'<table style="width: 100%; margin-top: 10px; margin-bottom: 10px;">\n        <tr>\n          <td class="divider"></td>\n        </tr>\n      </table>\n      <style>\n      .divider {\n        background-color: rgba(0, 0, 0, 0.1);\n        height: 1px;\n      }\n      </style>',attributes:{class:"gjs-fonts gjs-f-divider"}}),a.add("text",{label:t.textBlkLabel,category:t.categoryLabel,attributes:{class:"gjs-fonts gjs-f-text"},content:{type:"text",content:"Insert your text here",style:{padding:"10px"},activeOnRender:1}}),a.add("text-sect",{label:t.textSectionBlkLabel,category:t.categoryLabel,content:'<h1 class="heading">Insert title here</h1><p class="paragraph">Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua</p>',attributes:{class:"gjs-fonts gjs-f-h1p"}}),a.add("image",{label:t.imageBlkLabel,category:t.categoryLabel,attributes:{class:"gjs-fonts gjs-f-image"},content:{type:"image",style:{color:"black"},activeOnRender:1}}),a.add("quote",{label:t.quoteBlkLabel,category:t.categoryLabel,content:'<blockquote class="quote">Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore ipsum dolor sit</blockquote>',attributes:{class:"fa fa-quote-right"}}),a.add("link",{label:t.linkBlkLabel,category:t.categoryLabel,attributes:{class:"fa fa-link"},content:{type:"link",content:"Link",style:{color:"#3b97e3"}}}),a.add("link-block",{label:t.linkBlockBlkLabel,category:t.categoryLabel,attributes:{class:"fa fa-link"},content:{type:"link",editable:!1,droppable:!0,style:{display:"inline-block",padding:"5px","min-height":"50px","min-width":"50px"}}});var c='<table class="grid-item-card">\n        <tr>\n          <td class="grid-item-card-cell">\n            <img class="grid-item-image" src="http://placehold.it/250x150/78c5d6/fff/" alt="Image"/>\n            <table class="grid-item-card-body">\n              <tr>\n                <td class="grid-item-card-content">\n                  <h1 class="card-title">Title here</h1>\n                  <p class="card-text">Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt</p>\n                </td>\n              </tr>\n            </table>\n          </td>\n        </tr>\n      </table>';a.add("grid-items",{label:t.gridItemsBlkLabel,category:t.categoryLabel,content:'<table class="grid-item-row">\n        <tr>\n          <td class="grid-item-cell2-l">'+c+'</td>\n          <td class="grid-item-cell2-r">'+c+"</td>\n        </tr>\n      </table>",attributes:{class:"fa fa-th"}});var l='<table class="list-item">\n        <tr>\n          <td class="list-item-cell">\n            <table class="list-item-content">\n              <tr class="list-item-row">\n                <td class="list-cell-left">\n                  <img class="list-item-image" src="http://placehold.it/150x150/78c5d6/fff/" alt="Image"/>\n                </td>\n                <td class="list-cell-right">\n                  <h1 class="card-title">Title here</h1>\n                  <p class="card-text">Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt</p>\n                </td>\n              </tr>\n            </table>\n          </td>\n        </tr>\n      </table>';a.add("list-items",{label:t.listItemsBlkLabel,category:t.categoryLabel,content:l+l,attributes:{class:"fa fa-th-list"}});}}.call(e,r,e,t))&&(t.exports=n);},function(t,e,r){function n(t,e,r){return e in t?Object.defineProperty(t,e,{value:r,enumerable:!0,configurable:!0,writable:!0}):t[e]=r,t}var i;void 0!==(i=function(){var t=function(t){t.each(function(t){var e=t.get("attributes");e["data-tooltip-pos"]="bottom",t.set("attributes",e);});};return function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},r=e.editor,i=r.Panels,o=i.getPanel("options");if(i.addButton("options",{id:e.cmdOpenImport,className:"fa fa-download",command:e.cmdOpenImport,attributes:n({},"title",e.modalTitleImport)}),i.addButton("options",{id:e.cmdTglImages,className:"fa fa-warning",command:e.cmdTglImages,attributes:n({},"title",e.cmtTglImagesLabel)}),o){var a=o.get("buttons");a.each(function(t){var e=t.get("attributes");e["data-tooltip-pos"]="bottom",t.set("attributes",e);});var s=i.addButton("options","preview");s&&a.remove(s);}var u=i.getPanel("commands");if(u){var c=u.get("buttons");c.reset(),c.add([{id:"undo",className:"fa fa-undo",command:"undo",attributes:n({},"title",e.cmdBtnUndoLabel)},{id:"redo",className:"fa fa-repeat",command:"redo",attributes:n({},"title",e.cmdBtnRedoLabel)}]),t(c);}r.getConfig().showDevices=0;var l=i.addPanel({id:"devices-c"}),f=l.get("buttons");l.get("buttons").add([{id:"deviceDesktop",command:"set-device-desktop",className:"fa fa-desktop",attributes:n({},"title",e.cmdBtnDesktopLabel),active:1},{id:"deviceTablet",command:"set-device-tablet",className:"fa fa-tablet",attributes:n({},"title",e.cmdBtnTabletLabel)},{id:"deviceMobile",command:"set-device-mobile",className:"fa fa-mobile",attributes:n({},"title",e.cmdBtnMobileLabel)}]),t(f);}}.call(e,r,e,t))&&(t.exports=i);},function(t,e,r){var n;void 0!==(n=function(){return function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},e=t.editor,r=e.StyleManager.getSectors();e.on("load",function(){r.reset(),r.add(t.styleManagerSectors);});}}.call(e,r,e,t))&&(t.exports=n);}])});
});

var newsletter = unwrapExports(grapesjsPresetNewsletter_min);

var grapesjsPresetWebpage_min = createCommonjsModule(function (module, exports) {
/*! grapesjs-preset-webpage - 0.1.11 */
!function(e,t){module.exports=t(grapes_min);}("undefined"!=typeof self?self:commonjsGlobal,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=4)}([function(t,n){t.exports=e;},function(e,t,n){(function(e){function r(e,t){this._id=e,this._clearFn=t;}var i=Function.prototype.apply;t.setTimeout=function(){return new r(i.call(setTimeout,window,arguments),clearTimeout)},t.setInterval=function(){return new r(i.call(setInterval,window,arguments),clearInterval)},t.clearTimeout=t.clearInterval=function(e){e&&e.close();},r.prototype.unref=r.prototype.ref=function(){},r.prototype.close=function(){this._clearFn.call(window,this._id);},t.enroll=function(e,t){clearTimeout(e._idleTimeoutId),e._idleTimeout=t;},t.unenroll=function(e){clearTimeout(e._idleTimeoutId),e._idleTimeout=-1;},t._unrefActive=t.active=function(e){clearTimeout(e._idleTimeoutId);var t=e._idleTimeout;t>=0&&(e._idleTimeoutId=setTimeout(function(){e._onTimeout&&e._onTimeout();},t));},n(10),t.setImmediate="undefined"!=typeof self&&self.setImmediate||void 0!==e&&e.setImmediate||this&&this.setImmediate,t.clearImmediate="undefined"!=typeof self&&self.clearImmediate||void 0!==e&&e.clearImmediate||this&&this.clearImmediate;}).call(t,n(2));},function(e,t){var n;n=function(){return this}();try{n=n||Function("return this")()||(0,eval)("this");}catch(e){"object"==typeof window&&(n=window);}e.exports=n;},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});t.cmdImport="gjs-open-import-webpage",t.cmdDeviceDesktop="set-device-desktop",t.cmdDeviceTablet="set-device-tablet",t.cmdDeviceMobile="set-device-mobile",t.cmdClear="canvas-clear";},function(e,t,n){function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(0),a=r(i),o=n(5),s=r(o),l=n(6),u=r(l),c=n(7),f=r(c),h=n(8),d=r(h),p=n(9),m=r(p),g=n(12),v=r(g),b=n(13),y=r(b),w=n(14),_=r(w),k=n(16),x=r(k),S=n(17),E=r(S),T=n(18),C=r(T),A=n(19),O=r(A);t.default=a.default.plugins.add("gjs-preset-webpage",function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,r={blocks:["link-block","quote","text-basic"],modalImportTitle:"Import",modalImportButton:"Import",modalImportLabel:"",modalImportContent:"",importViewerOptions:{},textCleanCanvas:"Are you sure to clean the canvas?",showStylesOnChange:1,textGeneral:"General",textLayout:"Layout",textTypography:"Typography",textDecorations:"Decorations",textExtra:"Extra",customStyleManager:[],blocksBasicOpts:{},navbarOpts:{},countdownOpts:{},formsOpts:{},exportOpts:{},aviaryOpts:0,filestackOpts:0};for(var i in r)i in n||(n[i]=r[i]);var a=n.blocksBasicOpts,o=n.navbarOpts,l=n.countdownOpts,c=n.formsOpts,h=n.exportOpts,p=n.aviaryOpts,g=n.filestackOpts;a&&(0, s.default)(e,a),o&&(0, u.default)(e,o),l&&(0, f.default)(e,l),c&&(0, d.default)(e,c),h&&(0, m.default)(e,h),p&&(0, v.default)(e,p),g&&(0, y.default)(e,g),(0, E.default)(e,n),(0, x.default)(e,n),(0, _.default)(e,n),(0, C.default)(e,n),(0, O.default)(e,n);});},function(e,t,n){/*! grapesjs-blocks-basic - 0.1.7 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=0)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r]);}return e},i=n(1),a=function(e){return e&&e.__esModule?e:{default:e}}(i);t.default=a.default.plugins.add("gjs-blocks-basic",function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},i=r({blocks:["column1","column2","column3","column3-7","text","link","image","video","map"],flexGrid:0,stylePrefix:"",addBasicStyle:!0,category:"Basic",labelColumn1:"1 Column",labelColumn2:"2 Columns",labelColumn3:"3 Columns",labelColumn37:"2 Columns 3/7",labelText:"Text",labelLink:"Link",labelImage:"Image",labelVideo:"Video",labelMap:"Map"},t);n(2).default(e,i);});},function(t,n){t.exports=e;},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r]);}return e};t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,i=e.BlockManager,a=n.blocks,o=n.stylePrefix,s=n.flexGrid,l=n.addBasicStyle,u=o+"row",c=o+"cell",f=s?"\n    ."+u+" {\n      display: flex;\n      justify-content: flex-start;\n      align-items: stretch;\n      flex-wrap: nowrap;\n      padding: 10px;\n    }\n    @media (max-width: 768px) {\n      ."+u+" {\n        flex-wrap: wrap;\n      }\n    }":"\n    ."+u+" {\n      display: table;\n      padding: 10px;\n      width: 100%;\n    }\n    @media (max-width: 768px) {\n      ."+o+"cell, ."+o+"cell30, ."+o+"cell70 {\n        width: 100%;\n        display: block;\n      }\n    }",h=s?"\n    ."+c+" {\n      min-height: 75px;\n      flex-grow: 1;\n      flex-basis: 100%;\n    }":"\n    ."+c+" {\n      width: 8%;\n      display: table-cell;\n      height: 75px;\n    }",d="\n  ."+o+"cell30 {\n    width: 30%;\n  }",p="\n  ."+o+"cell70 {\n    width: 70%;\n  }",m={tl:0,tc:0,tr:0,cl:0,cr:0,bl:0,br:0,minDim:1},g=r({},m,{cr:1,bc:0,currentUnit:1,minDim:1,step:.2});s&&(g.keyWidth="flex-basis");var v={class:u,"data-gjs-droppable":"."+c,"data-gjs-resizable":m,"data-gjs-name":"Row"},b={class:c,"data-gjs-draggable":"."+u,"data-gjs-resizable":g,"data-gjs-name":"Cell"};s&&(b["data-gjs-unstylable"]=["width"],b["data-gjs-stylable-require"]=["flex-basis"]);var y=["."+u,"."+c];e.on("selector:add",function(e){return y.indexOf(e.getFullName())>=0&&e.set("private",1)});var w=function(e){var t=[];for(var n in e){var r=e[n],i=r instanceof Array||r instanceof Object;r=i?JSON.stringify(r):r,t.push(n+"="+(i?"'"+r+"'":'"'+r+'"'));}return t.length?" "+t.join(" "):""},_=function(e){return a.indexOf(e)>=0},k=w(v),x=w(b);_("column1")&&i.add("column1",{label:n.labelColumn1,category:n.category,attributes:{class:"gjs-fonts gjs-f-b1"},content:"<div "+k+">\n        <div "+x+"></div>\n      </div>\n      "+(l?"<style>\n          "+f+"\n          "+h+"\n        </style>":"")}),_("column2")&&i.add("column2",{label:n.labelColumn2,attributes:{class:"gjs-fonts gjs-f-b2"},category:n.category,content:"<div "+k+">\n        <div "+x+"></div>\n        <div "+x+"></div>\n      </div>\n      "+(l?"<style>\n          "+f+"\n          "+h+"\n        </style>":"")}),_("column3")&&i.add("column3",{label:n.labelColumn3,category:n.category,attributes:{class:"gjs-fonts gjs-f-b3"},content:"<div "+k+">\n        <div "+x+"></div>\n        <div "+x+"></div>\n        <div "+x+"></div>\n      </div>\n      "+(l?"<style>\n          "+f+"\n          "+h+"\n        </style>":"")}),_("column3-7")&&i.add("column3-7",{label:n.labelColumn37,category:n.category,attributes:{class:"gjs-fonts gjs-f-b37"},content:"<div "+k+">\n        <div "+x+' style="'+(s?"flex-basis":"width")+': 30%;"></div>\n        <div '+x+' style="'+(s?"flex-basis":"width")+': 70%;"></div>\n      </div>\n      '+(l?"<style>\n          "+f+"\n          "+h+"\n          "+d+"\n          "+p+"\n        </style>":"")}),_("text")&&i.add("text",{label:n.labelText,category:n.category,attributes:{class:"gjs-fonts gjs-f-text"},content:{type:"text",content:"Insert your text here",style:{padding:"10px"},activeOnRender:1}}),_("link")&&i.add("link",{label:n.labelLink,category:n.category,attributes:{class:"fa fa-link"},content:{type:"link",content:"Link",style:{color:"#d983a6"}}}),_("image")&&i.add("image",{label:n.labelImage,category:n.category,attributes:{class:"gjs-fonts gjs-f-image"},content:{style:{color:"black"},type:"image",activeOnRender:1}}),_("video")&&i.add("video",{label:n.labelVideo,category:n.category,attributes:{class:"fa fa-youtube-play"},content:{type:"video",src:"img/video2.webm",style:{height:"350px",width:"615px"}}}),_("map")&&i.add("map",{label:n.labelMap,category:n.category,attributes:{class:"fa fa-map-o"},content:{type:"map",style:{height:"350px"}}});};}])});},function(e,t,n){/*! grapesjs-navbar - 0.1.5 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=1)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.hNavbarRef="h-navbar",t.navbarRef="navbar",t.navbarItemsRef="navbar-items",t.menuRef="navbar-menu";},function(e,t,n){function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(2),a=r(i),o=n(3),s=r(o),l=n(4),u=r(l),c=n(0);t.default=a.default.plugins.add("gjs-navbar",function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,r={blocks:[c.hNavbarRef],defaultStyle:1,navbarClsPfx:"navbar",labelNavbar:"Navbar",labelNavbarContainer:"Navbar Container",labelMenu:"Navbar Menu",labelMenuLink:"Menu link",labelBurger:"Burger",labelBurgerLine:"Burger Line",labelNavbarBlock:"Navbar",labelNavbarCategory:"Extra",labelHome:"Home",labelAbout:"About",labelContact:"Contact"};for(var i in r)i in n||(n[i]=r[i]);(0, s.default)(e,n),(0, u.default)(e,n);});},function(t,n){t.exports=e;},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=n(0);t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,i=e.BlockManager,a=n.navbarClsPfx||"navbar",o=n.defaultStyle?"\n  <style>\n    ."+a+"-items-c {\n      display: inline-block;\n      float: right;\n    }\n\n    ."+a+" {\n      background-color: #222;\n      color: #ddd;\n      min-height: 50px;\n      width: 100%;\n    }\n\n    ."+a+"-container {\n      max-width: 950px;\n      margin: 0 auto;\n      width: 95%;\n    }\n\n    ."+a+'-container::after {\n      content: "";\n      clear: both;\n      display: block;\n    }\n\n    .'+a+"-brand {\n      vertical-align: top;\n      display: inline-block;\n      padding: 5px;\n      min-height: 50px;\n      min-width: 50px;\n      color: inherit;\n      text-decoration: none;\n    }\n\n    ."+a+"-menu {\n      padding: 10px 0;\n      display: block;\n      float: right;\n      margin: 0;\n    }\n\n    ."+a+"-menu-link {\n      margin: 0;\n      color: inherit;\n      text-decoration: none;\n      display: inline-block;\n      padding: 10px 15px;\n    }\n\n    ."+a+"-burger {\n      margin: 10px 0;\n      width: 45px;\n      padding: 5px 10px;\n      display: none;\n      float: right;\n      cursor: pointer;\n    }\n\n    ."+a+"-burger-line {\n      padding: 1px;\n      background-color: white;\n      margin: 5px 0;\n    }\n\n    @media (max-width: 768px) {\n      ."+a+"-burger {\n        display: block;\n      }\n\n      ."+a+"-items-c {\n        display: none;\n        width: 100%;\n      }\n\n      ."+a+"-menu {\n        width: 100%;\n      }\n\n      ."+a+"-menu-link {\n        display: block;\n      }\n    }\n  </style>\n  ":"";n.blocks.indexOf(r.hNavbarRef)>=0&&i.add(r.hNavbarRef,{label:'\n        <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n          <path class="gjs-block-svg-path" d="M22,9 C22,8.4 21.5,8 20.75,8 L3.25,8 C2.5,8 2,8.4 2,9 L2,15 C2,15.6 2.5,16 3.25,16 L20.75,16 C21.5,16 22,15.6 22,15 L22,9 Z M21,15 L3,15 L3,9 L21,9 L21,15 Z" fill-rule="nonzero"></path>\n          <rect class="gjs-block-svg-path" x="15" y="10" width="5" height="1"></rect>\n          <rect class="gjs-block-svg-path" x="15" y="13" width="5" height="1"></rect>\n          <rect class="gjs-block-svg-path" x="15" y="11.5" width="5" height="1"></rect>\n        </svg>\n        <div class="gjs-block-label">'+n.labelNavbarBlock+"</div>",category:n.labelNavbarCategory,content:'\n        <div class="'+a+'" data-gjs-droppable="false" data-gjs-custom-name="'+n.labelNavbar+'" data-gjs="'+r.navbarRef+'">\n          <div class="'+a+'-container" data-gjs-droppable="false" data-gjs-draggable="false"\n            data-gjs-removable="false" data-gjs-copyable="false" data-gjs-highlightable="false"\n            data-gjs-custom-name="'+n.labelNavbarContainer+'">\n\n            <a href="/" class="'+a+'-brand" data-gjs-droppable="true"></a>\n\n            <div class="'+a+'-burger" data-gjs-type="burger-menu">\n              <div class="'+a+'-burger-line" data-gjs-custom-name="'+n.labelBurgerLine+'" data-gjs-droppable="false" data-gjs-draggable="false"></div>\n              <div class="'+a+'-burger-line" data-gjs-custom-name="'+n.labelBurgerLine+'" data-gjs-droppable="false" data-gjs-draggable="false"></div>\n              <div class="'+a+'-burger-line" data-gjs-custom-name="'+n.labelBurgerLine+'" data-gjs-droppable="false" data-gjs-draggable="false"></div>\n            </div>\n\n            <div class="'+a+'-items-c" data-gjs="'+r.navbarItemsRef+'">\n              <nav class="'+a+'-menu" data-gjs="'+r.menuRef+'" data-gjs-custom-name="'+n.labelMenu+'">\n                <a href="#" class="'+a+'-menu-link" data-gjs-custom-name="'+n.labelMenuLink+'" data-gjs-draggable="[data-gjs='+r.menuRef+']">'+n.labelHome+'</a>\n                <a href="#" class="'+a+'-menu-link" data-gjs-custom-name="'+n.labelMenuLink+'" data-gjs-draggable="[data-gjs='+r.menuRef+']">'+n.labelAbout+'</a>\n                <a href="#" class="'+a+'-menu-link" data-gjs-custom-name="'+n.labelMenuLink+'" data-gjs-draggable="[data-gjs='+r.menuRef+']">'+n.labelContact+"</a>\n              </nav>\n            </div>\n\n          </div>\n        </div>\n        "+o+"\n      "});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r]);}return e};t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,i=e.DomComponents,a=i.getType("default"),o=a.model;i.addType("burger-menu",{model:o.extend({defaults:r({},o.prototype.defaults,{"custom-name":n.labelBurger,draggable:!1,droppable:!1,copyable:!1,removable:!1,script:function(){var e,t=0,n=function(){var e,t=document.createElement("void"),n={transition:"transitionend",OTransition:"oTransitionEnd",MozTransition:"transitionend",WebkitTransition:"webkitTransitionEnd"};for(e in n)if(void 0!==t.style[e])return n[e]}(),r=function(e){var t=window.getComputedStyle(e),n=t.display,r=(t.position,t.visibility,t.height,parseInt(t["max-height"]));if("none"!==n&&"0"!==r)return e.offsetHeight;e.style.height="auto",e.style.display="block",e.style.position="absolute",e.style.visibility="hidden";var i=e.offsetHeight;return e.style.height="",e.style.display="",e.style.position="",e.style.visibility="",i},i=function(e){t=1;var n=r(e),i=e.style;i.display="block",i.transition="max-height 0.25s ease-in-out",i.overflowY="hidden",""==i["max-height"]&&(i["max-height"]=0),0==parseInt(i["max-height"])?(i["max-height"]="0",setTimeout(function(){i["max-height"]=n+"px";},10)):i["max-height"]="0";},a=function(r){if(r.preventDefault(),!t){var a=this.closest("[data-gjs=navbar]"),o=a.querySelector("[data-gjs=navbar-items]");i(o),e||(o.addEventListener(n,function(){t=0;var e=o.style;0==parseInt(e["max-height"])&&(e.display="",e["max-height"]="");}),e=1);}};"gjs-collapse"in this||this.addEventListener("click",a),this["gjs-collapse"]=1;}})},{isComponent:function(e){if(e.getAttribute&&"burger-menu"==e.getAttribute("data-gjs-type"))return {type:"burger-menu"}}}),view:a.view});};}])});},function(e,t,n){/*! grapesjs-component-countdown - 0.1.2 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=1)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.countdownRef="countdown";},function(e,t,n){function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(2),a=r(i),o=n(3),s=r(o),l=n(4),u=r(l),c=n(0);t.default=a.default.plugins.add("gjs-component-countdown",function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,r={blocks:[c.countdownRef],defaultStyle:!0,startTime:"",endText:"EXPIRED",dateInputType:"date",countdownClsPfx:"countdown",labelCountdown:"Countdown",labelCountdownCategory:"Extra",labelDays:"days",labelHours:"hours",labelMinutes:"minutes",labelSeconds:"seconds"};for(var i in r)i in n||(n[i]=r[i]);(0, s.default)(e,n),(0, u.default)(e,n);});},function(t,n){t.exports=e;},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r]);}return e};t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,i=e.DomComponents,a=i.getType("default"),o=i.getType("text"),s=a.model,l=a.view,u=(o.model,o.view,n.countdownClsPfx);i.addType("countdown",{model:s.extend({defaults:r({},s.prototype.defaults,{startfrom:n.startTime,endText:n.endText,droppable:!1,traits:[{label:"Start",name:"startfrom",changeProp:1,type:n.dateInputType},{label:"End text",name:"endText",changeProp:1}],script:function(){var e=new Date("{[ startfrom ]}").getTime(),t=this.querySelector("[data-js=countdown]"),n=this.querySelector("[data-js=countdown-endtext]"),r=this.querySelector("[data-js=countdown-day]"),i=this.querySelector("[data-js=countdown-hour]"),a=this.querySelector("[data-js=countdown-minute]"),o=this.querySelector("[data-js=countdown-second]"),s=this.gjs_countdown_interval;s&&s&&clearInterval(s);var l=function(e,t,n,s){r.innerHTML=e<10?"0"+e:e,i.innerHTML=t<10?"0"+t:t,a.innerHTML=n<10?"0"+n:n,o.innerHTML=s<10?"0"+s:s;},u=function(){var r=(new Date).getTime(),i=e-r,a=Math.floor(i/864e5),o=Math.floor(i%864e5/36e5),s=Math.floor(i%36e4/6e4),u=Math.floor(i%6e4/1e3);l(a,o,s,u),i<0&&(clearInterval(c),n.innerHTML="{[ endText ]}",t.style.display="none",n.style.display="");};if(e){var c=setInterval(u,1e3);this.gjs_countdown_interval=c,n.style.display="none",t.style.display="",u();}else l(0,0,0,0);}})},{isComponent:function(e){if(e.getAttribute&&"countdown"==e.getAttribute("data-gjs-type"))return {type:"countdown"}}}),view:l.extend({init:function(){this.listenTo(this.model,"change:startfrom change:endText",this.updateScript);var e=this.model.get("components");e.length||(e.reset(),e.add('\n            <span data-js="countdown" class="'+u+'-cont">\n              <div class="'+u+'-block">\n                <div data-js="countdown-day" class="'+u+'-digit"></div>\n                <div class="'+u+'-label">'+n.labelDays+'</div>\n              </div>\n              <div class="'+u+'-block">\n                <div data-js="countdown-hour" class="'+u+'-digit"></div>\n                <div class="'+u+'-label">'+n.labelHours+'</div>\n              </div>\n              <div class="'+u+'-block">\n                <div data-js="countdown-minute" class="'+u+'-digit"></div>\n                <div class="'+u+'-label">'+n.labelMinutes+'</div>\n              </div>\n              <div class="'+u+'-block">\n                <div data-js="countdown-second" class="'+u+'-digit"></div>\n                <div class="'+u+'-label">'+n.labelSeconds+'</div>\n              </div>\n            </span>\n            <span data-js="countdown-endtext" class="'+u+'-endtext"></span>\n          '));}})});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,i=e.BlockManager,a=n.countdownClsPfx,o=n.defaultStyle?"<style>\n    ."+a+" {\n      text-align: center;\n      font-family: Helvetica, serif;\n    }\n\n    ."+a+"-block {\n      display: inline-block;\n      margin: 0 10px;\n      padding: 10px;\n    }\n\n    ."+a+"-digit {\n      font-size: 5rem;\n    }\n\n    ."+a+"-endtext {\n      font-size: 5rem;\n    }\n\n    ."+a+"-cont,\n    ."+a+"-block {\n      display: inline-block;\n    }\n  </style>":"";n.blocks.indexOf(r.countdownRef)>=0&&i.add(r.countdownRef,{label:n.labelCountdown,category:n.labelCountdownCategory,attributes:{class:"fa fa-clock-o"},content:'\n        <div class="'+a+'" data-gjs-type="countdown"></div>\n        '+o+"\n      "});};var r=n(0);}])});},function(e,t,n){/*! grapesjs-plugin-forms - 0.3.5 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=0)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=function(e){return e&&e.__esModule?e:{default:e}}(r);t.default=i.default.plugins.add("gjs-plugin-forms",function(e,t){var r=t||{},i=e.getConfig(),a=(i.stylePrefix,{blocks:["form","input","textarea","select","button","label","checkbox","radio"],labelTraitMethod:"Method",labelTraitAction:"Action",labelTraitState:"State",labelTraitId:"ID",labelTraitFor:"For",labelInputName:"Input",labelTextareaName:"Textarea",labelSelectName:"Select",labelCheckboxName:"Checkbox",labelRadioName:"Radio",labelButtonName:"Button",labelTraitName:"Name",labelTraitPlaceholder:"Placeholder",labelTraitValue:"Value",labelTraitRequired:"Required",labelTraitType:"Type",labelTraitOptions:"Options",labelTraitChecked:"Checked",labelTypeText:"Text",labelTypeEmail:"Email",labelTypePassword:"Password",labelTypeNumber:"Number",labelTypeSubmit:"Submit",labelTypeReset:"Reset",labelTypeButton:"Button",labelNameLabel:"Label",labelForm:"Form",labelSelectOption:"- Select option -",labelOption:"Option",labelStateNormal:"Normal",labelStateSuccess:"Success",labelStateError:"Error"});for(var o in a)o in r||(r[o]=a[o]);n(2).default(e,r),n(3).default(e,r),n(4).default(e,r);});},function(t,n){t.exports=e;},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r]);}return e};t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,i=e.DomComponents,a=i.getType("default"),o=i.getType("text"),s=a.model,l=a.view,u=o.model,c=o.view,f={name:"id",label:n.labelTraitId},h={name:"for",label:n.labelTraitFor},d={name:"name",label:n.labelTraitName},p={name:"placeholder",label:n.labelTraitPlaceholder},m={name:"value",label:n.labelTraitValue},g={type:"checkbox",name:"required",label:n.labelTraitRequired},v={label:n.labelTraitChecked,type:"checkbox",name:"checked",changeProp:1};i.addType("form",{model:s.extend({defaults:r({},s.prototype.defaults,{droppable:":not(form)",draggable:":not(form)",traits:[{type:"select",label:n.labelTraitMethod,name:"method",options:[{value:"post",name:"POST"},{value:"get",name:"GET"}]},{label:n.labelTraitAction,name:"action"}]}),init:function(){this.listenTo(this,"change:formState",this.updateFormState);},updateFormState:function(){switch(this.get("formState")){case"success":this.showState("success");break;case"error":this.showState("error");break;default:this.showState("normal");}},showState:function(e){var t,n,r=e||"normal";"success"==r?(t="none",n="block"):"error"==r?(t="block",n="none"):(t="none",n="none");var i=this.getStateModel("success"),a=this.getStateModel("error"),o=i.getStyle(),s=a.getStyle();o.display=n,s.display=t,i.setStyle(o),a.setStyle(s);},getStateModel:function(e){for(var t,n=e||"success",r=this.get("components"),i=0;i<r.length;i++){var a=r.models[i];if(a.get("form-state-type")==n){t=a;break}}if(!t){var o=formMsgSuccess;"error"==n&&(o=formMsgError),t=r.add({"form-state-type":n,type:"text",removable:!1,copyable:!1,draggable:!1,attributes:{"data-form-state":n},content:o});}return t}},{isComponent:function(e){if("FORM"==e.tagName)return {type:"form"}}}),view:l.extend({events:{submit:function(e){e.preventDefault();}}})}),i.addType("input",{model:s.extend({defaults:r({},s.prototype.defaults,{"custom-name":n.labelInputName,tagName:"input",draggable:"form, form *",droppable:!1,traits:[d,p,{label:n.labelTraitType,type:"select",name:"type",options:[{value:"text",name:n.labelTypeText},{value:"email",name:n.labelTypeEmail},{value:"password",name:n.labelTypePassword},{value:"number",name:n.labelTypeNumber}]},g]})},{isComponent:function(e){if("INPUT"==e.tagName)return {type:"input"}}}),view:l});var b=i.getType("input"),y=b.model;i.addType("textarea",{model:b.model.extend({defaults:r({},y.prototype.defaults,{"custom-name":n.labelTextareaName,tagName:"textarea",traits:[d,p,g]})},{isComponent:function(e){if("TEXTAREA"==e.tagName)return {type:"textarea"}}}),view:l}),i.addType("select",{model:s.extend({defaults:r({},y.prototype.defaults,{"custom-name":n.labelSelectName,tagName:"select",traits:[d,{label:n.labelTraitOptions,type:"select-options"},g]})},{isComponent:function(e){if("SELECT"==e.tagName)return {type:"select"}}}),view:function(){return a.view.extend({events:{mousedown:"handleClick"},handleClick:function(e){e.preventDefault();}})}()}),i.addType("checkbox",{model:s.extend({defaults:r({},y.prototype.defaults,{"custom-name":n.labelCheckboxName,copyable:!1,attributes:{type:"checkbox"},traits:[f,d,m,g,v]}),init:function(){this.listenTo(this,"change:checked",this.handleChecked);},handleChecked:function(){var e=this.get("checked"),t=this.get("attributes"),n=this.view;e?t.checked="checked":delete t.checked,n&&(n.el.checked=e),this.set("attributes",r({},t));}},{isComponent:function(e){if("INPUT"==e.tagName&&"checkbox"==e.type)return {type:"checkbox"}}}),view:l.extend({events:{click:"handleClick"},handleClick:function(e){e.preventDefault();}})});var w=i.getType("checkbox");i.addType("radio",{model:w.model.extend({defaults:r({},w.model.prototype.defaults,{"custom-name":n.labelRadioName,attributes:{type:"radio"}})},{isComponent:function(e){if("INPUT"==e.tagName&&"radio"==e.type)return {type:"radio"}}}),view:w.view}),i.addType("button",{model:s.extend({defaults:r({},y.prototype.defaults,{"custom-name":n.labelButtonName,tagName:"button",traits:[{type:"content",label:"Text"},{label:n.labelTraitType,type:"select",name:"type",options:[{value:"submit",name:n.labelTypeSubmit},{value:"reset",name:n.labelTypeReset},{value:"button",name:n.labelTypeButton}]}]})},{isComponent:function(e){if("BUTTON"==e.tagName)return {type:"button"}}}),view:l.extend({events:{click:"handleClick"},init:function(){this.listenTo(this.model,"change:content",this.updateContent);},updateContent:function(){this.el.innerHTML=this.model.get("content");},handleClick:function(e){e.preventDefault();}})}),i.addType("label",{model:u.extend({defaults:r({},u.prototype.defaults,{"custom-name":n.labelNameLabel,tagName:"label",traits:[h]})},{isComponent:function(e){if("LABEL"==e.tagName)return {type:"label"}}}),view:c});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=(e.TraitManager),n=t.getType("text");t.addType("content",{events:{keyup:"onChange"},onValueChange:function(){var e=this.model;e.target.set("content",e.get("value"));},getInputEl:function(){return this.inputEl||(this.inputEl=n.prototype.getInputEl.bind(this)(),this.inputEl.value=this.target.get("content")),this.inputEl}}),t.addType("select-options",{events:{keyup:"onChange"},onValueChange:function(){for(var e=this.model.get("value").trim(),t=e.split("\n"),n=[],r=0;r<t.length;r++){var i=t[r],a=i.split("::"),o={tagName:"option",attributes:{}};a[1]?(o.content=a[1],o.attributes.value=a[0]):(o.content=a[0],o.attributes.value=a[0]),n.push(o);}this.target.get("components").reset(n),this.target.view.render();},getInputEl:function(){if(!this.$input){for(var e=this.model,t=this.target,n=(e.get("name"),""),r=t.get("components"),i=0;i<r.length;i++){var a=r.models[i];n+=(a.get("attributes").value||"")+"::"+a.get("content")+"\n";}this.$input=document.createElement("textarea"),this.$input.value=n;}return this.$input}});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,r=e.BlockManager;n.blocks.indexOf("form")>=0&&r.add("form",{label:'\n      <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n        <path class="gjs-block-svg-path" d="M22,5.5 C22,5.2 21.5,5 20.75,5 L3.25,5 C2.5,5 2,5.2 2,5.5 L2,8.5 C2,8.8 2.5,9 3.25,9 L20.75,9 C21.5,9 22,8.8 22,8.5 L22,5.5 Z M21,8 L3,8 L3,6 L21,6 L21,8 Z" fill-rule="nonzero"></path>\n        <path class="gjs-block-svg-path" d="M22,10.5 C22,10.2 21.5,10 20.75,10 L3.25,10 C2.5,10 2,10.2 2,10.5 L2,13.5 C2,13.8 2.5,14 3.25,14 L20.75,14 C21.5,14 22,13.8 22,13.5 L22,10.5 Z M21,13 L3,13 L3,11 L21,11 L21,13 Z" fill-rule="nonzero"></path>\n        <rect class="gjs-block-svg-path" x="2" y="15" width="10" height="3" rx="0.5"></rect>\n      </svg>\n      <div class="gjs-block-label">'+n.labelForm+"</div>",category:"Forms",content:'\n        <form class="form">\n          <div class="form-group">\n            <label class="label">Name</label>\n            <input placeholder="Type here your name" class="input"/>\n          </div>\n          <div class="form-group">\n            <label class="label">Email</label>\n            <input type="email" placeholder="Type here your email" class="input"/>\n          </div>\n          <div class="form-group">\n            <label class="label">Gender</label>\n            <input type="checkbox" class="checkbox" value="M">\n            <label class="checkbox-label">M</label>\n            <input type="checkbox" class="checkbox" value="F">\n            <label class="checkbox-label">F</label>\n          </div>\n          <div class="form-group">\n            <label class="label">Message</label>\n            <textarea class="textarea"></textarea>\n          </div>\n          <div class="form-group">\n            <button type="submit" class="button">Send</button>\n          </div>\n        </form>\n      '}),n.blocks.indexOf("input")>=0&&r.add("input",{label:'\n      <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n        <path class="gjs-block-svg-path" d="M22,9 C22,8.4 21.5,8 20.75,8 L3.25,8 C2.5,8 2,8.4 2,9 L2,15 C2,15.6 2.5,16 3.25,16 L20.75,16 C21.5,16 22,15.6 22,15 L22,9 Z M21,15 L3,15 L3,9 L21,9 L21,15 Z"></path>\n        <polygon class="gjs-block-svg-path" points="4 10 5 10 5 14 4 14"></polygon>\n      </svg>\n      <div class="gjs-block-label">'+n.labelInputName+"</div>",category:"Forms",content:'<input class="input"/>'}),n.blocks.indexOf("textarea")>=0&&r.add("textarea",{label:'\n      <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n        <path class="gjs-block-svg-path" d="M22,7.5 C22,6.6 21.5,6 20.75,6 L3.25,6 C2.5,6 2,6.6 2,7.5 L2,16.5 C2,17.4 2.5,18 3.25,18 L20.75,18 C21.5,18 22,17.4 22,16.5 L22,7.5 Z M21,17 L3,17 L3,7 L21,7 L21,17 Z"></path>\n        <polygon class="gjs-block-svg-path" points="4 8 5 8 5 12 4 12"></polygon>\n        <polygon class="gjs-block-svg-path" points="19 7 20 7 20 17 19 17"></polygon>\n        <polygon class="gjs-block-svg-path" points="20 8 21 8 21 9 20 9"></polygon>\n        <polygon class="gjs-block-svg-path" points="20 15 21 15 21 16 20 16"></polygon>\n      </svg>\n      <div class="gjs-block-label">'+n.labelTextareaName+"</div>",category:"Forms",content:'<textarea class="textarea"></textarea>'}),n.blocks.indexOf("select")>=0&&r.add("select",{label:'\n      <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n        <path class="gjs-block-svg-path" d="M22,9 C22,8.4 21.5,8 20.75,8 L3.25,8 C2.5,8 2,8.4 2,9 L2,15 C2,15.6 2.5,16 3.25,16 L20.75,16 C21.5,16 22,15.6 22,15 L22,9 Z M21,15 L3,15 L3,9 L21,9 L21,15 Z" fill-rule="nonzero"></path>\n        <polygon class="gjs-block-svg-path" transform="translate(18.500000, 12.000000) scale(1, -1) translate(-18.500000, -12.000000) " points="18.5 11 20 13 17 13"></polygon>\n        <rect class="gjs-block-svg-path" x="4" y="11.5" width="11" height="1"></rect>\n      </svg>\n      <div class="gjs-block-label">'+n.labelSelectName+"</div>",category:"Forms",content:'<select class="select">\n        '+(n.labelSelectOption?'<option value="">'+n.labelSelectOption+"</option>":"")+'\n        <option value="1">'+n.labelOption+" 1</option>\n        </select>"}),n.blocks.indexOf("button")>=0&&r.add("button",{label:'\n      <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n        <path class="gjs-block-svg-path" d="M22,9 C22,8.4 21.5,8 20.75,8 L3.25,8 C2.5,8 2,8.4 2,9 L2,15 C2,15.6 2.5,16 3.25,16 L20.75,16 C21.5,16 22,15.6 22,15 L22,9 Z M21,15 L3,15 L3,9 L21,9 L21,15 Z" fill-rule="nonzero"></path>\n        <rect class="gjs-block-svg-path" x="4" y="11.5" width="16" height="1"></rect>\n      </svg>\n      <div class="gjs-block-label">'+n.labelButtonName+"</div>",category:"Forms",content:'<button class="button">Send</button>'}),n.blocks.indexOf("label")>=0&&r.add("label",{label:'\n      <svg class="gjs-block-svg" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">\n        <path class="gjs-block-svg-path" d="M22,11.875 C22,11.35 21.5,11 20.75,11 L3.25,11 C2.5,11 2,11.35 2,11.875 L2,17.125 C2,17.65 2.5,18 3.25,18 L20.75,18 C21.5,18 22,17.65 22,17.125 L22,11.875 Z M21,17 L3,17 L3,12 L21,12 L21,17 Z" fill-rule="nonzero"></path>\n        <rect class="gjs-block-svg-path" x="2" y="5" width="14" height="5" rx="0.5"></rect>\n        <polygon class="gjs-block-svg-path" fill-rule="nonzero" points="4 13 5 13 5 16 4 16"></polygon>\n      </svg>\n      <div class="gjs-block-label">'+n.labelNameLabel+"</div>",category:"Forms",content:'<label class="label">Label</label>'}),n.blocks.indexOf("checkbox")>=0&&r.add("checkbox",{label:n.labelCheckboxName,attributes:{class:"fa fa-check-square"},category:"Forms",content:'<input type="checkbox" class="checkbox"/>'}),n.blocks.indexOf("radio")>=0&&r.add("radio",{label:n.labelRadioName,attributes:{class:"fa fa-dot-circle-o"},category:"Forms",content:'<input type="radio" class="radio"/>'});};}])});},function(e,t,n){(function(t,r){/*! grapesjs-plugin-export - 0.1.5 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=50)}([function(e,t,n){function r(e){var t=null;return t=l.uint8array?new Uint8Array(e.length):new Array(e.length),a(e,t)}function i(e){return e}function a(e,t){for(var n=0;n<e.length;++n)t[n]=255&e.charCodeAt(n);return t}function o(e){var n=65536,r=t.getTypeOf(e),i=!0;if("uint8array"===r?i=d.applyCanBeUsed.uint8array:"nodebuffer"===r&&(i=d.applyCanBeUsed.nodebuffer),i)for(;n>1;)try{return d.stringifyByChunk(e,r,n)}catch(e){n=Math.floor(n/2);}return d.stringifyByChar(e)}function s(e,t){for(var n=0;n<e.length;n++)t[n]=e[n];return t}var l=n(3),u=n(29),c=n(14),f=n(66),h=n(9);t.newBlob=function(e,n){t.checkSupport("blob");try{return new Blob([e],{type:n})}catch(t){try{var r=self.BlobBuilder||self.WebKitBlobBuilder||self.MozBlobBuilder||self.MSBlobBuilder,i=new r;return i.append(e),i.getBlob(n)}catch(e){throw new Error("Bug : can't construct the Blob.")}}};var d={stringifyByChunk:function(e,t,n){var r=[],i=0,a=e.length;if(a<=n)return String.fromCharCode.apply(null,e);for(;i<a;)"array"===t||"nodebuffer"===t?r.push(String.fromCharCode.apply(null,e.slice(i,Math.min(i+n,a)))):r.push(String.fromCharCode.apply(null,e.subarray(i,Math.min(i+n,a)))),i+=n;return r.join("")},stringifyByChar:function(e){for(var t="",n=0;n<e.length;n++)t+=String.fromCharCode(e[n]);return t},applyCanBeUsed:{uint8array:function(){try{return l.uint8array&&1===String.fromCharCode.apply(null,new Uint8Array(1)).length}catch(e){return !1}}(),nodebuffer:function(){try{return l.nodebuffer&&1===String.fromCharCode.apply(null,c.allocBuffer(1)).length}catch(e){return !1}}()}};t.applyFromCharCode=o;var p={};p.string={string:i,array:function(e){return a(e,new Array(e.length))},arraybuffer:function(e){return p.string.uint8array(e).buffer},uint8array:function(e){return a(e,new Uint8Array(e.length))},nodebuffer:function(e){return a(e,c.allocBuffer(e.length))}},p.array={string:o,array:i,arraybuffer:function(e){return new Uint8Array(e).buffer},uint8array:function(e){return new Uint8Array(e)},nodebuffer:function(e){return c.newBufferFrom(e)}},p.arraybuffer={string:function(e){return o(new Uint8Array(e))},array:function(e){return s(new Uint8Array(e),new Array(e.byteLength))},arraybuffer:i,uint8array:function(e){return new Uint8Array(e)},nodebuffer:function(e){return c.newBufferFrom(new Uint8Array(e))}},p.uint8array={string:o,array:function(e){return s(e,new Array(e.length))},arraybuffer:function(e){return e.buffer},uint8array:i,nodebuffer:function(e){return c.newBufferFrom(e)}},p.nodebuffer={string:o,array:function(e){return s(e,new Array(e.length))},arraybuffer:function(e){return p.nodebuffer.uint8array(e).buffer},uint8array:function(e){return s(e,new Uint8Array(e.length))},nodebuffer:i},t.transformTo=function(e,n){if(n||(n=""),!e)return n;t.checkSupport(e);var r=t.getTypeOf(n);return p[r][e](n)},t.getTypeOf=function(e){return "string"==typeof e?"string":"[object Array]"===Object.prototype.toString.call(e)?"array":l.nodebuffer&&c.isBuffer(e)?"nodebuffer":l.uint8array&&e instanceof Uint8Array?"uint8array":l.arraybuffer&&e instanceof ArrayBuffer?"arraybuffer":void 0},t.checkSupport=function(e){if(!l[e.toLowerCase()])throw new Error(e+" is not supported by this platform")},t.MAX_VALUE_16BITS=65535,t.MAX_VALUE_32BITS=-1,t.pretty=function(e){var t,n,r="";for(n=0;n<(e||"").length;n++)t=e.charCodeAt(n),r+="\\x"+(t<16?"0":"")+t.toString(16).toUpperCase();return r},t.delay=function(e,t,n){f(function(){e.apply(n||null,t||[]);});},t.inherits=function(e,t){var n=function(){};n.prototype=t.prototype,e.prototype=new n;},t.extend=function(){var e,t,n={};for(e=0;e<arguments.length;e++)for(t in arguments[e])arguments[e].hasOwnProperty(t)&&void 0===n[t]&&(n[t]=arguments[e][t]);return n},t.prepareContent=function(e,n,i,a,o){return h.Promise.resolve(n).then(function(e){return l.blob&&(e instanceof Blob||-1!==["[object File]","[object Blob]"].indexOf(Object.prototype.toString.call(e)))&&"undefined"!=typeof FileReader?new h.Promise(function(t,n){var r=new FileReader;r.onload=function(e){t(e.target.result);},r.onerror=function(e){n(e.target.error);},r.readAsArrayBuffer(e);}):e}).then(function(n){var s=t.getTypeOf(n);return s?("arraybuffer"===s?n=t.transformTo("uint8array",n):"string"===s&&(o?n=u.decode(n):i&&!0!==a&&(n=r(n))),n):h.Promise.reject(new Error("Can't read the data of '"+e+"'. Is it in a supported JavaScript type (String, Blob, ArrayBuffer, etc) ?"))})};},function(e,t,n){function r(e){this.name=e||"default",this.streamInfo={},this.generatedError=null,this.extraStreamInfo={},this.isPaused=!0,this.isFinished=!1,this.isLocked=!1,this._listeners={data:[],end:[],error:[]},this.previous=null;}r.prototype={push:function(e){this.emit("data",e);},end:function(){if(this.isFinished)return !1;this.flush();try{this.emit("end"),this.cleanUp(),this.isFinished=!0;}catch(e){this.emit("error",e);}return !0},error:function(e){return !this.isFinished&&(this.isPaused?this.generatedError=e:(this.isFinished=!0,this.emit("error",e),this.previous&&this.previous.error(e),this.cleanUp()),!0)},on:function(e,t){return this._listeners[e].push(t),this},cleanUp:function(){this.streamInfo=this.generatedError=this.extraStreamInfo=null,this._listeners=[];},emit:function(e,t){if(this._listeners[e])for(var n=0;n<this._listeners[e].length;n++)this._listeners[e][n].call(this,t);},pipe:function(e){return e.registerPrevious(this)},registerPrevious:function(e){if(this.isLocked)throw new Error("The stream '"+this+"' has already been used.");this.streamInfo=e.streamInfo,this.mergeStreamInfo(),this.previous=e;var t=this;return e.on("data",function(e){t.processChunk(e);}),e.on("end",function(){t.end();}),e.on("error",function(e){t.error(e);}),this},pause:function(){return !this.isPaused&&!this.isFinished&&(this.isPaused=!0,this.previous&&this.previous.pause(),!0)},resume:function(){if(!this.isPaused||this.isFinished)return !1;this.isPaused=!1;var e=!1;return this.generatedError&&(this.error(this.generatedError),e=!0),this.previous&&this.previous.resume(),!e},flush:function(){},processChunk:function(e){this.push(e);},withStreamInfo:function(e,t){return this.extraStreamInfo[e]=t,this.mergeStreamInfo(),this},mergeStreamInfo:function(){for(var e in this.extraStreamInfo)this.extraStreamInfo.hasOwnProperty(e)&&(this.streamInfo[e]=this.extraStreamInfo[e]);},lock:function(){if(this.isLocked)throw new Error("The stream '"+this+"' has already been used.");this.isLocked=!0,this.previous&&this.previous.lock();},toString:function(){var e="Worker "+this.name;return this.previous?this.previous+" -> "+e:e}},e.exports=r;},function(e,t,n){(function(e){function r(){return a.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function i(e,t){if(r()<t)throw new RangeError("Invalid typed array length");return a.TYPED_ARRAY_SUPPORT?(e=new Uint8Array(t),e.__proto__=a.prototype):(null===e&&(e=new a(t)),e.length=t),e}function a(e,t,n){if(!(a.TYPED_ARRAY_SUPPORT||this instanceof a))return new a(e,t,n);if("number"==typeof e){if("string"==typeof t)throw new Error("If encoding is specified then the first argument must be a string");return u(this,e)}return o(this,e,t,n)}function o(e,t,n,r){if("number"==typeof t)throw new TypeError('"value" argument must not be a number');return "undefined"!=typeof ArrayBuffer&&t instanceof ArrayBuffer?h(e,t,n,r):"string"==typeof t?c(e,t,n):d(e,t)}function s(e){if("number"!=typeof e)throw new TypeError('"size" argument must be a number');if(e<0)throw new RangeError('"size" argument must not be negative')}function l(e,t,n,r){return s(t),t<=0?i(e,t):void 0!==n?"string"==typeof r?i(e,t).fill(n,r):i(e,t).fill(n):i(e,t)}function u(e,t){if(s(t),e=i(e,t<0?0:0|p(t)),!a.TYPED_ARRAY_SUPPORT)for(var n=0;n<t;++n)e[n]=0;return e}function c(e,t,n){if("string"==typeof n&&""!==n||(n="utf8"),!a.isEncoding(n))throw new TypeError('"encoding" must be a valid string encoding');var r=0|g(t,n);e=i(e,r);var o=e.write(t,n);return o!==r&&(e=e.slice(0,o)),e}function f(e,t){var n=t.length<0?0:0|p(t.length);e=i(e,n);for(var r=0;r<n;r+=1)e[r]=255&t[r];return e}function h(e,t,n,r){if(t.byteLength,n<0||t.byteLength<n)throw new RangeError("'offset' is out of bounds");if(t.byteLength<n+(r||0))throw new RangeError("'length' is out of bounds");return t=void 0===n&&void 0===r?new Uint8Array(t):void 0===r?new Uint8Array(t,n):new Uint8Array(t,n,r),a.TYPED_ARRAY_SUPPORT?(e=t,e.__proto__=a.prototype):e=f(e,t),e}function d(e,t){if(a.isBuffer(t)){var n=0|p(t.length);return e=i(e,n),0===e.length?e:(t.copy(e,0,0,n),e)}if(t){if("undefined"!=typeof ArrayBuffer&&t.buffer instanceof ArrayBuffer||"length"in t)return "number"!=typeof t.length||X(t.length)?i(e,0):f(e,t);if("Buffer"===t.type&&$(t.data))return f(e,t.data)}throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}function p(e){if(e>=r())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+r().toString(16)+" bytes");return 0|e}function m(e){return +e!=e&&(e=0),a.alloc(+e)}function g(e,t){if(a.isBuffer(e))return e.length;if("undefined"!=typeof ArrayBuffer&&"function"==typeof ArrayBuffer.isView&&(ArrayBuffer.isView(e)||e instanceof ArrayBuffer))return e.byteLength;"string"!=typeof e&&(e=""+e);var n=e.length;if(0===n)return 0;for(var r=!1;;)switch(t){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return Y(e).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*n;case"hex":return n>>>1;case"base64":return V(e).length;default:if(r)return Y(e).length;t=(""+t).toLowerCase(),r=!0;}}function v(e,t,n){var r=!1;if((void 0===t||t<0)&&(t=0),t>this.length)return "";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return "";if(n>>>=0,t>>>=0,n<=t)return "";for(e||(e="utf8");;)switch(e){case"hex":return L(this,t,n);case"utf8":case"utf-8":return A(this,t,n);case"ascii":return I(this,t,n);case"latin1":case"binary":return R(this,t,n);case"base64":return C(this,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return j(this,t,n);default:if(r)throw new TypeError("Unknown encoding: "+e);e=(e+"").toLowerCase(),r=!0;}}function b(e,t,n){var r=e[t];e[t]=e[n],e[n]=r;}function y(e,t,n,r,i){if(0===e.length)return -1;if("string"==typeof n?(r=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=i?0:e.length-1),n<0&&(n=e.length+n),n>=e.length){if(i)return -1;n=e.length-1;}else if(n<0){if(!i)return -1;n=0;}if("string"==typeof t&&(t=a.from(t,r)),a.isBuffer(t))return 0===t.length?-1:w(e,t,n,r,i);if("number"==typeof t)return t&=255,a.TYPED_ARRAY_SUPPORT&&"function"==typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(e,t,n):Uint8Array.prototype.lastIndexOf.call(e,t,n):w(e,[t],n,r,i);throw new TypeError("val must be string, number or Buffer")}function w(e,t,n,r,i){function a(e,t){return 1===o?e[t]:e.readUInt16BE(t*o)}var o=1,s=e.length,l=t.length;if(void 0!==r&&("ucs2"===(r=String(r).toLowerCase())||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(e.length<2||t.length<2)return -1;o=2,s/=2,l/=2,n/=2;}var u;if(i){var c=-1;for(u=n;u<s;u++)if(a(e,u)===a(t,-1===c?0:u-c)){if(-1===c&&(c=u),u-c+1===l)return c*o}else -1!==c&&(u-=u-c),c=-1;}else for(n+l>s&&(n=s-l),u=n;u>=0;u--){for(var f=!0,h=0;h<l;h++)if(a(e,u+h)!==a(t,h)){f=!1;break}if(f)return u}return -1}function _(e,t,n,r){n=Number(n)||0;var i=e.length-n;r?(r=Number(r))>i&&(r=i):r=i;var a=t.length;if(a%2!=0)throw new TypeError("Invalid hex string");r>a/2&&(r=a/2);for(var o=0;o<r;++o){var s=parseInt(t.substr(2*o,2),16);if(isNaN(s))return o;e[n+o]=s;}return o}function k(e,t,n,r){return K(Y(t,e.length-n),e,n,r)}function x(e,t,n,r){return K(q(t),e,n,r)}function S(e,t,n,r){return x(e,t,n,r)}function E(e,t,n,r){return K(V(t),e,n,r)}function T(e,t,n,r){return K(H(t,e.length-n),e,n,r)}function C(e,t,n){return 0===t&&n===e.length?G.fromByteArray(e):G.fromByteArray(e.slice(t,n))}function A(e,t,n){n=Math.min(e.length,n);for(var r=[],i=t;i<n;){var a=e[i],o=null,s=a>239?4:a>223?3:a>191?2:1;if(i+s<=n){var l,u,c,f;switch(s){case 1:a<128&&(o=a);break;case 2:128==(192&(l=e[i+1]))&&(f=(31&a)<<6|63&l)>127&&(o=f);break;case 3:l=e[i+1],u=e[i+2],128==(192&l)&&128==(192&u)&&(f=(15&a)<<12|(63&l)<<6|63&u)>2047&&(f<55296||f>57343)&&(o=f);break;case 4:l=e[i+1],u=e[i+2],c=e[i+3],128==(192&l)&&128==(192&u)&&128==(192&c)&&(f=(15&a)<<18|(63&l)<<12|(63&u)<<6|63&c)>65535&&f<1114112&&(o=f);}}null===o?(o=65533,s=1):o>65535&&(o-=65536,r.push(o>>>10&1023|55296),o=56320|1023&o),r.push(o),i+=s;}return O(r)}function O(e){var t=e.length;if(t<=Q)return String.fromCharCode.apply(String,e);for(var n="",r=0;r<t;)n+=String.fromCharCode.apply(String,e.slice(r,r+=Q));return n}function I(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(127&e[i]);return r}function R(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(e[i]);return r}function L(e,t,n){var r=e.length;(!t||t<0)&&(t=0),(!n||n<0||n>r)&&(n=r);for(var i="",a=t;a<n;++a)i+=W(e[a]);return i}function j(e,t,n){for(var r=e.slice(t,n),i="",a=0;a<r.length;a+=2)i+=String.fromCharCode(r[a]+256*r[a+1]);return i}function B(e,t,n){if(e%1!=0||e<0)throw new RangeError("offset is not uint");if(e+t>n)throw new RangeError("Trying to access beyond buffer length")}function M(e,t,n,r,i,o){if(!a.isBuffer(e))throw new TypeError('"buffer" argument must be a Buffer instance');if(t>i||t<o)throw new RangeError('"value" argument is out of bounds');if(n+r>e.length)throw new RangeError("Index out of range")}function P(e,t,n,r){t<0&&(t=65535+t+1);for(var i=0,a=Math.min(e.length-n,2);i<a;++i)e[n+i]=(t&255<<8*(r?i:1-i))>>>8*(r?i:1-i);}function N(e,t,n,r){t<0&&(t=4294967295+t+1);for(var i=0,a=Math.min(e.length-n,4);i<a;++i)e[n+i]=t>>>8*(r?i:3-i)&255;}function z(e,t,n,r,i,a){if(n+r>e.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function D(e,t,n,r,i){return i||z(e,t,n,4),J.write(e,t,n,r,23,4),n+4}function F(e,t,n,r,i){return i||z(e,t,n,8),J.write(e,t,n,r,52,8),n+8}function U(e){if(e=Z(e).replace(ee,""),e.length<2)return "";for(;e.length%4!=0;)e+="=";return e}function Z(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}function W(e){return e<16?"0"+e.toString(16):e.toString(16)}function Y(e,t){t=t||1/0;for(var n,r=e.length,i=null,a=[],o=0;o<r;++o){if((n=e.charCodeAt(o))>55295&&n<57344){if(!i){if(n>56319){(t-=3)>-1&&a.push(239,191,189);continue}if(o+1===r){(t-=3)>-1&&a.push(239,191,189);continue}i=n;continue}if(n<56320){(t-=3)>-1&&a.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320);}else i&&(t-=3)>-1&&a.push(239,191,189);if(i=null,n<128){if((t-=1)<0)break;a.push(n);}else if(n<2048){if((t-=2)<0)break;a.push(n>>6|192,63&n|128);}else if(n<65536){if((t-=3)<0)break;a.push(n>>12|224,n>>6&63|128,63&n|128);}else {if(!(n<1114112))throw new Error("Invalid code point");if((t-=4)<0)break;a.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128);}}return a}function q(e){for(var t=[],n=0;n<e.length;++n)t.push(255&e.charCodeAt(n));return t}function H(e,t){for(var n,r,i,a=[],o=0;o<e.length&&!((t-=2)<0);++o)n=e.charCodeAt(o),r=n>>8,i=n%256,a.push(i),a.push(r);return a}function V(e){return G.toByteArray(U(e))}function K(e,t,n,r){for(var i=0;i<r&&!(i+n>=t.length||i>=e.length);++i)t[i+n]=e[i];return i}function X(e){return e!==e}/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <feross@feross.org> <http://feross.org>
 * @license  MIT
 */
var G=n(54),J=n(55),$=n(24);t.Buffer=a,t.SlowBuffer=m,t.INSPECT_MAX_BYTES=50,a.TYPED_ARRAY_SUPPORT=void 0!==e.TYPED_ARRAY_SUPPORT?e.TYPED_ARRAY_SUPPORT:function(){try{var e=new Uint8Array(1);return e.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===e.foo()&&"function"==typeof e.subarray&&0===e.subarray(1,1).byteLength}catch(e){return !1}}(),t.kMaxLength=r(),a.poolSize=8192,a._augment=function(e){return e.__proto__=a.prototype,e},a.from=function(e,t,n){return o(null,e,t,n)},a.TYPED_ARRAY_SUPPORT&&(a.prototype.__proto__=Uint8Array.prototype,a.__proto__=Uint8Array,"undefined"!=typeof Symbol&&Symbol.species&&a[Symbol.species]===a&&Object.defineProperty(a,Symbol.species,{value:null,configurable:!0})),a.alloc=function(e,t,n){return l(null,e,t,n)},a.allocUnsafe=function(e){return u(null,e)},a.allocUnsafeSlow=function(e){return u(null,e)},a.isBuffer=function(e){return !(null==e||!e._isBuffer)},a.compare=function(e,t){if(!a.isBuffer(e)||!a.isBuffer(t))throw new TypeError("Arguments must be Buffers");if(e===t)return 0;for(var n=e.length,r=t.length,i=0,o=Math.min(n,r);i<o;++i)if(e[i]!==t[i]){n=e[i],r=t[i];break}return n<r?-1:r<n?1:0},a.isEncoding=function(e){switch(String(e).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return !0;default:return !1}},a.concat=function(e,t){if(!$(e))throw new TypeError('"list" argument must be an Array of Buffers');if(0===e.length)return a.alloc(0);var n;if(void 0===t)for(t=0,n=0;n<e.length;++n)t+=e[n].length;var r=a.allocUnsafe(t),i=0;for(n=0;n<e.length;++n){var o=e[n];if(!a.isBuffer(o))throw new TypeError('"list" argument must be an Array of Buffers');o.copy(r,i),i+=o.length;}return r},a.byteLength=g,a.prototype._isBuffer=!0,a.prototype.swap16=function(){var e=this.length;if(e%2!=0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var t=0;t<e;t+=2)b(this,t,t+1);return this},a.prototype.swap32=function(){var e=this.length;if(e%4!=0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var t=0;t<e;t+=4)b(this,t,t+3),b(this,t+1,t+2);return this},a.prototype.swap64=function(){var e=this.length;if(e%8!=0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var t=0;t<e;t+=8)b(this,t,t+7),b(this,t+1,t+6),b(this,t+2,t+5),b(this,t+3,t+4);return this},a.prototype.toString=function(){var e=0|this.length;return 0===e?"":0===arguments.length?A(this,0,e):v.apply(this,arguments)},a.prototype.equals=function(e){if(!a.isBuffer(e))throw new TypeError("Argument must be a Buffer");return this===e||0===a.compare(this,e)},a.prototype.inspect=function(){var e="",n=t.INSPECT_MAX_BYTES;return this.length>0&&(e=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(e+=" ... ")),"<Buffer "+e+">"},a.prototype.compare=function(e,t,n,r,i){if(!a.isBuffer(e))throw new TypeError("Argument must be a Buffer");if(void 0===t&&(t=0),void 0===n&&(n=e?e.length:0),void 0===r&&(r=0),void 0===i&&(i=this.length),t<0||n>e.length||r<0||i>this.length)throw new RangeError("out of range index");if(r>=i&&t>=n)return 0;if(r>=i)return -1;if(t>=n)return 1;if(t>>>=0,n>>>=0,r>>>=0,i>>>=0,this===e)return 0;for(var o=i-r,s=n-t,l=Math.min(o,s),u=this.slice(r,i),c=e.slice(t,n),f=0;f<l;++f)if(u[f]!==c[f]){o=u[f],s=c[f];break}return o<s?-1:s<o?1:0},a.prototype.includes=function(e,t,n){return -1!==this.indexOf(e,t,n)},a.prototype.indexOf=function(e,t,n){return y(this,e,t,n,!0)},a.prototype.lastIndexOf=function(e,t,n){return y(this,e,t,n,!1)},a.prototype.write=function(e,t,n,r){if(void 0===t)r="utf8",n=this.length,t=0;else if(void 0===n&&"string"==typeof t)r=t,n=this.length,t=0;else {if(!isFinite(t))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");t|=0,isFinite(n)?(n|=0,void 0===r&&(r="utf8")):(r=n,n=void 0);}var i=this.length-t;if((void 0===n||n>i)&&(n=i),e.length>0&&(n<0||t<0)||t>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var a=!1;;)switch(r){case"hex":return _(this,e,t,n);case"utf8":case"utf-8":return k(this,e,t,n);case"ascii":return x(this,e,t,n);case"latin1":case"binary":return S(this,e,t,n);case"base64":return E(this,e,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return T(this,e,t,n);default:if(a)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),a=!0;}},a.prototype.toJSON=function(){return {type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var Q=4096;a.prototype.slice=function(e,t){var n=this.length;e=~~e,t=void 0===t?n:~~t,e<0?(e+=n)<0&&(e=0):e>n&&(e=n),t<0?(t+=n)<0&&(t=0):t>n&&(t=n),t<e&&(t=e);var r;if(a.TYPED_ARRAY_SUPPORT)r=this.subarray(e,t),r.__proto__=a.prototype;else {var i=t-e;r=new a(i,void 0);for(var o=0;o<i;++o)r[o]=this[o+e];}return r},a.prototype.readUIntLE=function(e,t,n){e|=0,t|=0,n||B(e,t,this.length);for(var r=this[e],i=1,a=0;++a<t&&(i*=256);)r+=this[e+a]*i;return r},a.prototype.readUIntBE=function(e,t,n){e|=0,t|=0,n||B(e,t,this.length);for(var r=this[e+--t],i=1;t>0&&(i*=256);)r+=this[e+--t]*i;return r},a.prototype.readUInt8=function(e,t){return t||B(e,1,this.length),this[e]},a.prototype.readUInt16LE=function(e,t){return t||B(e,2,this.length),this[e]|this[e+1]<<8},a.prototype.readUInt16BE=function(e,t){return t||B(e,2,this.length),this[e]<<8|this[e+1]},a.prototype.readUInt32LE=function(e,t){return t||B(e,4,this.length),(this[e]|this[e+1]<<8|this[e+2]<<16)+16777216*this[e+3]},a.prototype.readUInt32BE=function(e,t){return t||B(e,4,this.length),16777216*this[e]+(this[e+1]<<16|this[e+2]<<8|this[e+3])},a.prototype.readIntLE=function(e,t,n){e|=0,t|=0,n||B(e,t,this.length);for(var r=this[e],i=1,a=0;++a<t&&(i*=256);)r+=this[e+a]*i;return i*=128,r>=i&&(r-=Math.pow(2,8*t)),r},a.prototype.readIntBE=function(e,t,n){e|=0,t|=0,n||B(e,t,this.length);for(var r=t,i=1,a=this[e+--r];r>0&&(i*=256);)a+=this[e+--r]*i;return i*=128,a>=i&&(a-=Math.pow(2,8*t)),a},a.prototype.readInt8=function(e,t){return t||B(e,1,this.length),128&this[e]?-1*(255-this[e]+1):this[e]},a.prototype.readInt16LE=function(e,t){t||B(e,2,this.length);var n=this[e]|this[e+1]<<8;return 32768&n?4294901760|n:n},a.prototype.readInt16BE=function(e,t){t||B(e,2,this.length);var n=this[e+1]|this[e]<<8;return 32768&n?4294901760|n:n},a.prototype.readInt32LE=function(e,t){return t||B(e,4,this.length),this[e]|this[e+1]<<8|this[e+2]<<16|this[e+3]<<24},a.prototype.readInt32BE=function(e,t){return t||B(e,4,this.length),this[e]<<24|this[e+1]<<16|this[e+2]<<8|this[e+3]},a.prototype.readFloatLE=function(e,t){return t||B(e,4,this.length),J.read(this,e,!0,23,4)},a.prototype.readFloatBE=function(e,t){return t||B(e,4,this.length),J.read(this,e,!1,23,4)},a.prototype.readDoubleLE=function(e,t){return t||B(e,8,this.length),J.read(this,e,!0,52,8)},a.prototype.readDoubleBE=function(e,t){return t||B(e,8,this.length),J.read(this,e,!1,52,8)},a.prototype.writeUIntLE=function(e,t,n,r){e=+e,t|=0,n|=0,r||M(this,e,t,n,Math.pow(2,8*n)-1,0);var i=1,a=0;for(this[t]=255&e;++a<n&&(i*=256);)this[t+a]=e/i&255;return t+n},a.prototype.writeUIntBE=function(e,t,n,r){e=+e,t|=0,n|=0,r||M(this,e,t,n,Math.pow(2,8*n)-1,0);var i=n-1,a=1;for(this[t+i]=255&e;--i>=0&&(a*=256);)this[t+i]=e/a&255;return t+n},a.prototype.writeUInt8=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,1,255,0),a.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),this[t]=255&e,t+1},a.prototype.writeUInt16LE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,2,65535,0),a.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):P(this,e,t,!0),t+2},a.prototype.writeUInt16BE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,2,65535,0),a.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):P(this,e,t,!1),t+2},a.prototype.writeUInt32LE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,4,4294967295,0),a.TYPED_ARRAY_SUPPORT?(this[t+3]=e>>>24,this[t+2]=e>>>16,this[t+1]=e>>>8,this[t]=255&e):N(this,e,t,!0),t+4},a.prototype.writeUInt32BE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,4,4294967295,0),a.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):N(this,e,t,!1),t+4},a.prototype.writeIntLE=function(e,t,n,r){if(e=+e,t|=0,!r){var i=Math.pow(2,8*n-1);M(this,e,t,n,i-1,-i);}var a=0,o=1,s=0;for(this[t]=255&e;++a<n&&(o*=256);)e<0&&0===s&&0!==this[t+a-1]&&(s=1),this[t+a]=(e/o>>0)-s&255;return t+n},a.prototype.writeIntBE=function(e,t,n,r){if(e=+e,t|=0,!r){var i=Math.pow(2,8*n-1);M(this,e,t,n,i-1,-i);}var a=n-1,o=1,s=0;for(this[t+a]=255&e;--a>=0&&(o*=256);)e<0&&0===s&&0!==this[t+a+1]&&(s=1),this[t+a]=(e/o>>0)-s&255;return t+n},a.prototype.writeInt8=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,1,127,-128),a.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),e<0&&(e=255+e+1),this[t]=255&e,t+1},a.prototype.writeInt16LE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,2,32767,-32768),a.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):P(this,e,t,!0),t+2},a.prototype.writeInt16BE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,2,32767,-32768),a.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):P(this,e,t,!1),t+2},a.prototype.writeInt32LE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,4,2147483647,-2147483648),a.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8,this[t+2]=e>>>16,this[t+3]=e>>>24):N(this,e,t,!0),t+4},a.prototype.writeInt32BE=function(e,t,n){return e=+e,t|=0,n||M(this,e,t,4,2147483647,-2147483648),e<0&&(e=4294967295+e+1),a.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):N(this,e,t,!1),t+4},a.prototype.writeFloatLE=function(e,t,n){return D(this,e,t,!0,n)},a.prototype.writeFloatBE=function(e,t,n){return D(this,e,t,!1,n)},a.prototype.writeDoubleLE=function(e,t,n){return F(this,e,t,!0,n)},a.prototype.writeDoubleBE=function(e,t,n){return F(this,e,t,!1,n)},a.prototype.copy=function(e,t,n,r){if(n||(n=0),r||0===r||(r=this.length),t>=e.length&&(t=e.length),t||(t=0),r>0&&r<n&&(r=n),r===n)return 0;if(0===e.length||0===this.length)return 0;if(t<0)throw new RangeError("targetStart out of bounds");if(n<0||n>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),e.length-t<r-n&&(r=e.length-t+n);var i,o=r-n;if(this===e&&n<t&&t<r)for(i=o-1;i>=0;--i)e[i+t]=this[i+n];else if(o<1e3||!a.TYPED_ARRAY_SUPPORT)for(i=0;i<o;++i)e[i+t]=this[i+n];else Uint8Array.prototype.set.call(e,this.subarray(n,n+o),t);return o},a.prototype.fill=function(e,t,n,r){if("string"==typeof e){if("string"==typeof t?(r=t,t=0,n=this.length):"string"==typeof n&&(r=n,n=this.length),1===e.length){var i=e.charCodeAt(0);i<256&&(e=i);}if(void 0!==r&&"string"!=typeof r)throw new TypeError("encoding must be a string");if("string"==typeof r&&!a.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else "number"==typeof e&&(e&=255);if(t<0||this.length<t||this.length<n)throw new RangeError("Out of range index");if(n<=t)return this;t>>>=0,n=void 0===n?this.length:n>>>0,e||(e=0);var o;if("number"==typeof e)for(o=t;o<n;++o)this[o]=e;else {var s=a.isBuffer(e)?e:Y(new a(e,r).toString()),l=s.length;for(o=0;o<n-t;++o)this[o+t]=s[o%l];}return this};var ee=/[^+\/0-9A-Za-z-_]/g;}).call(t,n(10));},function(e,t,n){(function(e){if(t.base64=!0,t.array=!0,t.string=!0,t.arraybuffer="undefined"!=typeof ArrayBuffer&&"undefined"!=typeof Uint8Array,t.nodebuffer=void 0!==e,t.uint8array="undefined"!=typeof Uint8Array,"undefined"==typeof ArrayBuffer)t.blob=!1;else {var r=new ArrayBuffer(0);try{t.blob=0===new Blob([r],{type:"application/zip"}).size;}catch(e){try{var i=self.BlobBuilder||self.WebKitBlobBuilder||self.MozBlobBuilder||self.MSBlobBuilder,a=new i;a.append(r),t.blob=0===a.getBlob("application/zip").size;}catch(e){t.blob=!1;}}}try{t.nodestream=!!n(25).Readable;}catch(e){t.nodestream=!1;}}).call(t,n(2).Buffer);},function(e,t,n){function r(e,t){return Object.prototype.hasOwnProperty.call(e,t)}var i="undefined"!=typeof Uint8Array&&"undefined"!=typeof Uint16Array&&"undefined"!=typeof Int32Array;t.assign=function(e){for(var t=Array.prototype.slice.call(arguments,1);t.length;){var n=t.shift();if(n){if("object"!=typeof n)throw new TypeError(n+"must be non-object");for(var i in n)r(n,i)&&(e[i]=n[i]);}}return e},t.shrinkBuf=function(e,t){return e.length===t?e:e.subarray?e.subarray(0,t):(e.length=t,e)};var a={arraySet:function(e,t,n,r,i){if(t.subarray&&e.subarray)return void e.set(t.subarray(n,n+r),i);for(var a=0;a<r;a++)e[i+a]=t[n+a];},flattenChunks:function(e){var t,n,r,i,a,o;for(r=0,t=0,n=e.length;t<n;t++)r+=e[t].length;for(o=new Uint8Array(r),i=0,t=0,n=e.length;t<n;t++)a=e[t],o.set(a,i),i+=a.length;return o}},o={arraySet:function(e,t,n,r,i){for(var a=0;a<r;a++)e[i+a]=t[n+a];},flattenChunks:function(e){return [].concat.apply([],e)}};t.setTyped=function(e){e?(t.Buf8=Uint8Array,t.Buf16=Uint16Array,t.Buf32=Int32Array,t.assign(t,a)):(t.Buf8=Array,t.Buf16=Array,t.Buf32=Array,t.assign(t,o));},t.setTyped(i);},function(e,t,n){function r(e){if(!(this instanceof r))return new r(e);u.call(this,e),c.call(this,e),e&&!1===e.readable&&(this.readable=!1),e&&!1===e.writable&&(this.writable=!1),this.allowHalfOpen=!0,e&&!1===e.allowHalfOpen&&(this.allowHalfOpen=!1),this.once("end",i);}function i(){this.allowHalfOpen||this._writableState.ended||s(a,this);}function a(e){e.end();}var o=Object.keys||function(e){var t=[];for(var n in e)t.push(n);return t};e.exports=r;var s=n(16),l=n(8);l.inherits=n(7);var u=n(26),c=n(17);l.inherits(r,u);for(var f=o(c.prototype),h=0;h<f.length;h++){var d=f[h];r.prototype[d]||(r.prototype[d]=c.prototype[d]);}},function(e,t,n){function r(){l.call(this,"utf-8 decode"),this.leftOver=null;}function i(){l.call(this,"utf-8 encode");}for(var a=n(0),o=n(3),s=n(14),l=n(1),u=new Array(256),c=0;c<256;c++)u[c]=c>=252?6:c>=248?5:c>=240?4:c>=224?3:c>=192?2:1;u[254]=u[254]=1;var f=function(e){var t,n,r,i,a,s=e.length,l=0;for(i=0;i<s;i++)n=e.charCodeAt(i),55296==(64512&n)&&i+1<s&&56320==(64512&(r=e.charCodeAt(i+1)))&&(n=65536+(n-55296<<10)+(r-56320),i++),l+=n<128?1:n<2048?2:n<65536?3:4;for(t=o.uint8array?new Uint8Array(l):new Array(l),a=0,i=0;a<l;i++)n=e.charCodeAt(i),55296==(64512&n)&&i+1<s&&56320==(64512&(r=e.charCodeAt(i+1)))&&(n=65536+(n-55296<<10)+(r-56320),i++),n<128?t[a++]=n:n<2048?(t[a++]=192|n>>>6,t[a++]=128|63&n):n<65536?(t[a++]=224|n>>>12,t[a++]=128|n>>>6&63,t[a++]=128|63&n):(t[a++]=240|n>>>18,t[a++]=128|n>>>12&63,t[a++]=128|n>>>6&63,t[a++]=128|63&n);return t},h=function(e,t){var n;for(t=t||e.length,t>e.length&&(t=e.length),n=t-1;n>=0&&128==(192&e[n]);)n--;return n<0?t:0===n?t:n+u[e[n]]>t?n:t},d=function(e){var t,n,r,i,o=e.length,s=new Array(2*o);for(n=0,t=0;t<o;)if((r=e[t++])<128)s[n++]=r;else if((i=u[r])>4)s[n++]=65533,t+=i-1;else {for(r&=2===i?31:3===i?15:7;i>1&&t<o;)r=r<<6|63&e[t++],i--;i>1?s[n++]=65533:r<65536?s[n++]=r:(r-=65536,s[n++]=55296|r>>10&1023,s[n++]=56320|1023&r);}return s.length!==n&&(s.subarray?s=s.subarray(0,n):s.length=n),a.applyFromCharCode(s)};t.utf8encode=function(e){return o.nodebuffer?s.newBufferFrom(e,"utf-8"):f(e)},t.utf8decode=function(e){return o.nodebuffer?a.transformTo("nodebuffer",e).toString("utf-8"):(e=a.transformTo(o.uint8array?"uint8array":"array",e),d(e))},a.inherits(r,l),r.prototype.processChunk=function(e){var n=a.transformTo(o.uint8array?"uint8array":"array",e.data);if(this.leftOver&&this.leftOver.length){if(o.uint8array){var r=n;n=new Uint8Array(r.length+this.leftOver.length),n.set(this.leftOver,0),n.set(r,this.leftOver.length);}else n=this.leftOver.concat(n);this.leftOver=null;}var i=h(n),s=n;i!==n.length&&(o.uint8array?(s=n.subarray(0,i),this.leftOver=n.subarray(i,n.length)):(s=n.slice(0,i),this.leftOver=n.slice(i,n.length))),this.push({data:t.utf8decode(s),meta:e.meta});},r.prototype.flush=function(){this.leftOver&&this.leftOver.length&&(this.push({data:t.utf8decode(this.leftOver),meta:{}}),this.leftOver=null);},t.Utf8DecodeWorker=r,a.inherits(i,l),i.prototype.processChunk=function(e){this.push({data:t.utf8encode(e.data),meta:e.meta});},t.Utf8EncodeWorker=i;},function(e,t){"function"==typeof Object.create?e.exports=function(e,t){e.super_=t,e.prototype=Object.create(t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}});}:e.exports=function(e,t){e.super_=t;var n=function(){};n.prototype=t.prototype,e.prototype=new n,e.prototype.constructor=e;};},function(e,t,n){(function(e){function n(e){return Array.isArray?Array.isArray(e):"[object Array]"===g(e)}function r(e){return "boolean"==typeof e}function i(e){return null===e}function a(e){return null==e}function o(e){return "number"==typeof e}function s(e){return "string"==typeof e}function l(e){return "symbol"==typeof e}function u(e){return void 0===e}function c(e){return "[object RegExp]"===g(e)}function f(e){return "object"==typeof e&&null!==e}function h(e){return "[object Date]"===g(e)}function d(e){return "[object Error]"===g(e)||e instanceof Error}function p(e){return "function"==typeof e}function m(e){return null===e||"boolean"==typeof e||"number"==typeof e||"string"==typeof e||"symbol"==typeof e||void 0===e}function g(e){return Object.prototype.toString.call(e)}t.isArray=n,t.isBoolean=r,t.isNull=i,t.isNullOrUndefined=a,t.isNumber=o,t.isString=s,t.isSymbol=l,t.isUndefined=u,t.isRegExp=c,t.isObject=f,t.isDate=h,t.isError=d,t.isFunction=p,t.isPrimitive=m,t.isBuffer=e.isBuffer;}).call(t,n(2).Buffer);},function(e,t,n){var r=null;r="undefined"!=typeof Promise?Promise:n(80),e.exports={Promise:r};},function(e,t){var n;n=function(){return this}();try{n=n||Function("return this")()||(0,eval)("this");}catch(e){"object"==typeof window&&(n=window);}e.exports=n;},function(e,t,n){function r(){i.call(this);}e.exports=r;var i=n(12).EventEmitter;n(7)(r,i),r.Readable=n(56),r.Writable=n(62),r.Duplex=n(63),r.Transform=n(64),r.PassThrough=n(65),r.Stream=r,r.prototype.pipe=function(e,t){function n(t){e.writable&&!1===e.write(t)&&u.pause&&u.pause();}function r(){u.readable&&u.resume&&u.resume();}function a(){c||(c=!0,e.end());}function o(){c||(c=!0,"function"==typeof e.destroy&&e.destroy());}function s(e){if(l(),0===i.listenerCount(this,"error"))throw e}function l(){u.removeListener("data",n),e.removeListener("drain",r),u.removeListener("end",a),u.removeListener("close",o),u.removeListener("error",s),e.removeListener("error",s),u.removeListener("end",l),u.removeListener("close",l),e.removeListener("close",l);}var u=this;u.on("data",n),e.on("drain",r),e._isStdio||t&&!1===t.end||(u.on("end",a),u.on("close",o));var c=!1;return u.on("error",s),e.on("error",s),u.on("end",l),u.on("close",l),e.on("close",l),e.emit("pipe",u),e};},function(e,t){function n(){this._events=this._events||{},this._maxListeners=this._maxListeners||void 0;}function r(e){return "function"==typeof e}function i(e){return "number"==typeof e}function a(e){return "object"==typeof e&&null!==e}function o(e){return void 0===e}e.exports=n,n.EventEmitter=n,n.prototype._events=void 0,n.prototype._maxListeners=void 0,n.defaultMaxListeners=10,n.prototype.setMaxListeners=function(e){if(!i(e)||e<0||isNaN(e))throw TypeError("n must be a positive number");return this._maxListeners=e,this},n.prototype.emit=function(e){var t,n,i,s,l,u;if(this._events||(this._events={}),"error"===e&&(!this._events.error||a(this._events.error)&&!this._events.error.length)){if((t=arguments[1])instanceof Error)throw t;var c=new Error('Uncaught, unspecified "error" event. ('+t+")");throw c.context=t,c}if(n=this._events[e],o(n))return !1;if(r(n))switch(arguments.length){case 1:n.call(this);break;case 2:n.call(this,arguments[1]);break;case 3:n.call(this,arguments[1],arguments[2]);break;default:s=Array.prototype.slice.call(arguments,1),n.apply(this,s);}else if(a(n))for(s=Array.prototype.slice.call(arguments,1),u=n.slice(),i=u.length,l=0;l<i;l++)u[l].apply(this,s);return !0},n.prototype.addListener=function(e,t){var i;if(!r(t))throw TypeError("listener must be a function");return this._events||(this._events={}),this._events.newListener&&this.emit("newListener",e,r(t.listener)?t.listener:t),this._events[e]?a(this._events[e])?this._events[e].push(t):this._events[e]=[this._events[e],t]:this._events[e]=t,a(this._events[e])&&!this._events[e].warned&&(i=o(this._maxListeners)?n.defaultMaxListeners:this._maxListeners)&&i>0&&this._events[e].length>i&&(this._events[e].warned=!0,console.error("(node) warning: possible EventEmitter memory leak detected. %d listeners added. Use emitter.setMaxListeners() to increase limit.",this._events[e].length),"function"==typeof console.trace&&console.trace()),this},n.prototype.on=n.prototype.addListener,n.prototype.once=function(e,t){function n(){this.removeListener(e,n),i||(i=!0,t.apply(this,arguments));}if(!r(t))throw TypeError("listener must be a function");var i=!1;return n.listener=t,this.on(e,n),this},n.prototype.removeListener=function(e,t){var n,i,o,s;if(!r(t))throw TypeError("listener must be a function");if(!this._events||!this._events[e])return this;if(n=this._events[e],o=n.length,i=-1,n===t||r(n.listener)&&n.listener===t)delete this._events[e],this._events.removeListener&&this.emit("removeListener",e,t);else if(a(n)){for(s=o;s-- >0;)if(n[s]===t||n[s].listener&&n[s].listener===t){i=s;break}if(i<0)return this;1===n.length?(n.length=0,delete this._events[e]):n.splice(i,1),this._events.removeListener&&this.emit("removeListener",e,t);}return this},n.prototype.removeAllListeners=function(e){var t,n;if(!this._events)return this;if(!this._events.removeListener)return 0===arguments.length?this._events={}:this._events[e]&&delete this._events[e],this;if(0===arguments.length){for(t in this._events)"removeListener"!==t&&this.removeAllListeners(t);return this.removeAllListeners("removeListener"),this._events={},this}if(n=this._events[e],r(n))this.removeListener(e,n);else if(n)for(;n.length;)this.removeListener(e,n[n.length-1]);return delete this._events[e],this},n.prototype.listeners=function(e){return this._events&&this._events[e]?r(this._events[e])?[this._events[e]]:this._events[e].slice():[]},n.prototype.listenerCount=function(e){if(this._events){var t=this._events[e];if(r(t))return 1;if(t)return t.length}return 0},n.listenerCount=function(e,t){return e.listenerCount(t)};},function(e,t){function n(){throw new Error("setTimeout has not been defined")}function r(){throw new Error("clearTimeout has not been defined")}function i(e){if(c===setTimeout)return setTimeout(e,0);if((c===n||!c)&&setTimeout)return c=setTimeout,setTimeout(e,0);try{return c(e,0)}catch(t){try{return c.call(null,e,0)}catch(t){return c.call(this,e,0)}}}function a(e){if(f===clearTimeout)return clearTimeout(e);if((f===r||!f)&&clearTimeout)return f=clearTimeout,clearTimeout(e);try{return f(e)}catch(t){try{return f.call(null,e)}catch(t){return f.call(this,e)}}}function o(){m&&d&&(m=!1,d.length?p=d.concat(p):g=-1,p.length&&s());}function s(){if(!m){var e=i(o);m=!0;for(var t=p.length;t;){for(d=p,p=[];++g<t;)d&&d[g].run();g=-1,t=p.length;}d=null,m=!1,a(e);}}function l(e,t){this.fun=e,this.array=t;}function u(){}var c,f,h=e.exports={};!function(){try{c="function"==typeof setTimeout?setTimeout:n;}catch(e){c=n;}try{f="function"==typeof clearTimeout?clearTimeout:r;}catch(e){f=r;}}();var d,p=[],m=!1,g=-1;h.nextTick=function(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];p.push(new l(e,t)),1!==p.length||m||i(s);},l.prototype.run=function(){this.fun.apply(null,this.array);},h.title="browser",h.browser=!0,h.env={},h.argv=[],h.version="",h.versions={},h.on=u,h.addListener=u,h.once=u,h.off=u,h.removeListener=u,h.removeAllListeners=u,h.emit=u,h.prependListener=u,h.prependOnceListener=u,h.listeners=function(e){return []},h.binding=function(e){throw new Error("process.binding is not supported")},h.cwd=function(){return "/"},h.chdir=function(e){throw new Error("process.chdir is not supported")},h.umask=function(){return 0};},function(e,t,n){(function(t){e.exports={isNode:void 0!==t,newBufferFrom:function(e,n){return new t(e,n)},allocBuffer:function(e){return t.alloc?t.alloc(e):new t(e)},isBuffer:function(e){return t.isBuffer(e)},isStream:function(e){return e&&"function"==typeof e.on&&"function"==typeof e.pause&&"function"==typeof e.resume}};}).call(t,n(2).Buffer);},function(e,t){var n=e.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n);},function(e,t,n){(function(t){function n(e,n,r,i){if("function"!=typeof e)throw new TypeError('"callback" argument must be a function');var a,o,s=arguments.length;switch(s){case 0:case 1:return t.nextTick(e);case 2:return t.nextTick(function(){e.call(null,n);});case 3:return t.nextTick(function(){e.call(null,n,r);});case 4:return t.nextTick(function(){e.call(null,n,r,i);});default:for(a=new Array(s-1),o=0;o<a.length;)a[o++]=arguments[o];return t.nextTick(function(){e.apply(null,a);})}}!t.version||0===t.version.indexOf("v0.")||0===t.version.indexOf("v1.")&&0!==t.version.indexOf("v1.8.")?e.exports=n:e.exports=t.nextTick;}).call(t,n(13));},function(e,t,n){(function(t,r){function i(){}function a(e,t,n){this.chunk=e,this.encoding=t,this.callback=n,this.next=null;}function o(e,t){I=I||n(5),e=e||{},this.objectMode=!!e.objectMode,t instanceof I&&(this.objectMode=this.objectMode||!!e.writableObjectMode);var r=e.highWaterMark,i=this.objectMode?16:16384;this.highWaterMark=r||0===r?r:i,this.highWaterMark=~~this.highWaterMark,this.needDrain=!1,this.ending=!1,this.ended=!1,this.finished=!1;var a=!1===e.decodeStrings;this.decodeStrings=!a,this.defaultEncoding=e.defaultEncoding||"utf8",this.length=0,this.writing=!1,this.corked=0,this.sync=!0,this.bufferProcessing=!1,this.onwrite=function(e){m(t,e);},this.writecb=null,this.writelen=0,this.bufferedRequest=null,this.lastBufferedRequest=null,this.pendingcb=0,this.prefinished=!1,this.errorEmitted=!1,this.bufferedRequestCount=0,this.corkedRequestsFree=new x(this),this.corkedRequestsFree.next=new x(this);}function s(e){if(I=I||n(5),!(this instanceof s||this instanceof I))return new s(e);this._writableState=new o(e,this),this.writable=!0,e&&("function"==typeof e.write&&(this._write=e.write),"function"==typeof e.writev&&(this._writev=e.writev)),A.call(this);}function l(e,t){var n=new Error("write after end");e.emit("error",n),S(t,n);}function u(e,t,n,r){var i=!0;if(!T.isBuffer(n)&&"string"!=typeof n&&null!==n&&void 0!==n&&!t.objectMode){var a=new TypeError("Invalid non-string/buffer chunk");e.emit("error",a),S(r,a),i=!1;}return i}function c(e,t,n){return e.objectMode||!1===e.decodeStrings||"string"!=typeof t||(t=new T(t,n)),t}function f(e,t,n,r,i){n=c(t,n,r),T.isBuffer(n)&&(r="buffer");var o=t.objectMode?1:n.length;t.length+=o;var s=t.length<t.highWaterMark;if(s||(t.needDrain=!0),t.writing||t.corked){var l=t.lastBufferedRequest;t.lastBufferedRequest=new a(n,r,i),l?l.next=t.lastBufferedRequest:t.bufferedRequest=t.lastBufferedRequest,t.bufferedRequestCount+=1;}else h(e,t,!1,o,n,r,i);return s}function h(e,t,n,r,i,a,o){t.writelen=r,t.writecb=o,t.writing=!0,t.sync=!0,n?e._writev(i,t.onwrite):e._write(i,a,t.onwrite),t.sync=!1;}function d(e,t,n,r,i){--t.pendingcb,n?S(i,r):i(r),e._writableState.errorEmitted=!0,e.emit("error",r);}function p(e){e.writing=!1,e.writecb=null,e.length-=e.writelen,e.writelen=0;}function m(e,t){var n=e._writableState,r=n.sync,i=n.writecb;if(p(n),t)d(e,n,r,t,i);else {var a=y(n);a||n.corked||n.bufferProcessing||!n.bufferedRequest||b(e,n),r?E(g,e,n,a,i):g(e,n,a,i);}}function g(e,t,n,r){n||v(e,t),t.pendingcb--,r(),_(e,t);}function v(e,t){0===t.length&&t.needDrain&&(t.needDrain=!1,e.emit("drain"));}function b(e,t){t.bufferProcessing=!0;var n=t.bufferedRequest;if(e._writev&&n&&n.next){var r=t.bufferedRequestCount,i=new Array(r),a=t.corkedRequestsFree;a.entry=n;for(var o=0;n;)i[o]=n,n=n.next,o+=1;h(e,t,!0,t.length,i,"",a.finish),t.pendingcb++,t.lastBufferedRequest=null,t.corkedRequestsFree=a.next,a.next=null;}else {for(;n;){var s=n.chunk,l=n.encoding,u=n.callback;if(h(e,t,!1,t.objectMode?1:s.length,s,l,u),n=n.next,t.writing)break}null===n&&(t.lastBufferedRequest=null);}t.bufferedRequestCount=0,t.bufferedRequest=n,t.bufferProcessing=!1;}function y(e){return e.ending&&0===e.length&&null===e.bufferedRequest&&!e.finished&&!e.writing}function w(e,t){t.prefinished||(t.prefinished=!0,e.emit("prefinish"));}function _(e,t){var n=y(t);return n&&(0===t.pendingcb?(w(e,t),t.finished=!0,e.emit("finish")):w(e,t)),n}function k(e,t,n){t.ending=!0,_(e,t),n&&(t.finished?S(n):e.once("finish",n)),t.ended=!0,e.writable=!1;}function x(e){var t=this;this.next=null,this.entry=null,this.finish=function(n){var r=t.entry;for(t.entry=null;r;){var i=r.callback;e.pendingcb--,i(n),r=r.next;}e.corkedRequestsFree?e.corkedRequestsFree.next=t:e.corkedRequestsFree=t;};}e.exports=s;var S=n(16),E=!t.browser&&["v0.10","v0.9."].indexOf(t.version.slice(0,5))>-1?r:S,T=n(2).Buffer;s.WritableState=o;var C=n(8);C.inherits=n(7);var A,O={deprecate:n(60)};!function(){try{A=n(11);}catch(e){}finally{A||(A=n(12).EventEmitter);}}();var T=n(2).Buffer;C.inherits(s,A);var I;o.prototype.getBuffer=function(){for(var e=this.bufferedRequest,t=[];e;)t.push(e),e=e.next;return t},function(){try{Object.defineProperty(o.prototype,"buffer",{get:O.deprecate(function(){return this.getBuffer()},"_writableState.buffer is deprecated. Use _writableState.getBuffer instead.")});}catch(e){}}();var I;s.prototype.pipe=function(){this.emit("error",new Error("Cannot pipe. Not readable."));},s.prototype.write=function(e,t,n){var r=this._writableState,a=!1;return "function"==typeof t&&(n=t,t=null),T.isBuffer(e)?t="buffer":t||(t=r.defaultEncoding),"function"!=typeof n&&(n=i),r.ended?l(this,n):u(this,r,e,n)&&(r.pendingcb++,a=f(this,r,e,t,n)),a},s.prototype.cork=function(){this._writableState.corked++;},s.prototype.uncork=function(){var e=this._writableState;e.corked&&(e.corked--,e.writing||e.corked||e.finished||e.bufferProcessing||!e.bufferedRequest||b(this,e));},s.prototype.setDefaultEncoding=function(e){if("string"==typeof e&&(e=e.toLowerCase()),!(["hex","utf8","utf-8","ascii","binary","base64","ucs2","ucs-2","utf16le","utf-16le","raw"].indexOf((e+"").toLowerCase())>-1))throw new TypeError("Unknown encoding: "+e);this._writableState.defaultEncoding=e;},s.prototype._write=function(e,t,n){n(new Error("not implemented"));},s.prototype._writev=null,s.prototype.end=function(e,t,n){var r=this._writableState;"function"==typeof e?(n=e,e=null,t=null):"function"==typeof t&&(n=t,t=null),null!==e&&void 0!==e&&this.write(e,t),r.corked&&(r.corked=1,this.uncork()),r.ending||r.finished||k(this,r,n);};}).call(t,n(13),n(58).setImmediate);},function(e,t,n){function r(e){this.afterTransform=function(t,n){return i(e,t,n)},this.needTransform=!1,this.transforming=!1,this.writecb=null,this.writechunk=null,this.writeencoding=null;}function i(e,t,n){var r=e._transformState;r.transforming=!1;var i=r.writecb;if(!i)return e.emit("error",new Error("no writecb in Transform class"));r.writechunk=null,r.writecb=null,null!==n&&void 0!==n&&e.push(n),i(t);var a=e._readableState;a.reading=!1,(a.needReadable||a.length<a.highWaterMark)&&e._read(a.highWaterMark);}function a(e){if(!(this instanceof a))return new a(e);s.call(this,e),this._transformState=new r(this);var t=this;this._readableState.needReadable=!0,this._readableState.sync=!1,e&&("function"==typeof e.transform&&(this._transform=e.transform),"function"==typeof e.flush&&(this._flush=e.flush)),this.once("prefinish",function(){"function"==typeof this._flush?this._flush(function(e){o(t,e);}):o(t);});}function o(e,t){if(t)return e.emit("error",t);var n=e._writableState,r=e._transformState;if(n.length)throw new Error("calling transform done when ws.length != 0");if(r.transforming)throw new Error("calling transform done when still transforming");return e.push(null)}e.exports=a;var s=n(5),l=n(8);l.inherits=n(7),l.inherits(a,s),a.prototype.push=function(e,t){return this._transformState.needTransform=!1,s.prototype.push.call(this,e,t)},a.prototype._transform=function(e,t,n){throw new Error("not implemented")},a.prototype._write=function(e,t,n){var r=this._transformState;if(r.writecb=n,r.writechunk=e,r.writeencoding=t,!r.transforming){var i=this._readableState;(r.needTransform||i.needReadable||i.length<i.highWaterMark)&&this._read(i.highWaterMark);}},a.prototype._read=function(e){var t=this._transformState;null!==t.writechunk&&t.writecb&&!t.transforming?(t.transforming=!0,this._transform(t.writechunk,t.writeencoding,t.afterTransform)):t.needTransform=!0;};},function(e,t){e.exports=function(e){return "object"==typeof e?null!==e:"function"==typeof e};},function(e,t,n){e.exports=!n(32)(function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a});},function(e,t,n){function r(e,t,n,r,i){this.compressedSize=e,this.uncompressedSize=t,this.crc32=n,this.compression=r,this.compressedContent=i;}var i=n(9),a=n(36),o=n(37),s=n(38),o=n(37);r.prototype={getContentWorker:function(){var e=new a(i.Promise.resolve(this.compressedContent)).pipe(this.compression.uncompressWorker()).pipe(new o("data_length")),t=this;return e.on("end",function(){if(this.streamInfo.data_length!==t.uncompressedSize)throw new Error("Bug : uncompressed data size mismatch")}),e},getCompressedWorker:function(){return new a(i.Promise.resolve(this.compressedContent)).withStreamInfo("compressedSize",this.compressedSize).withStreamInfo("uncompressedSize",this.uncompressedSize).withStreamInfo("crc32",this.crc32).withStreamInfo("compression",this.compression)}},r.createWorkerFrom=function(e,t,n){return e.pipe(new s).pipe(new o("uncompressedSize")).pipe(t.compressWorker(n)).pipe(new o("compressedSize")).withStreamInfo("compression",t)},e.exports=r;},function(e,t,n){function r(e,t,n,r){var i=o,a=r+n;e^=-1;for(var s=r;s<a;s++)e=e>>>8^i[255&(e^t[s])];return -1^e}function i(e,t,n,r){var i=o,a=r+n;e^=-1;for(var s=r;s<a;s++)e=e>>>8^i[255&(e^t.charCodeAt(s))];return -1^e}var a=n(0),o=function(){for(var e,t=[],n=0;n<256;n++){e=n;for(var r=0;r<8;r++)e=1&e?3988292384^e>>>1:e>>>1;t[n]=e;}return t}();e.exports=function(e,t){return void 0!==e&&e.length?"string"!==a.getTypeOf(e)?r(0|t,e,e.length,0):i(0|t,e,e.length,0):0};},function(e,t,n){e.exports={2:"need dictionary",1:"stream end",0:"","-1":"file error","-2":"stream error","-3":"data error","-4":"insufficient memory","-5":"buffer error","-6":"incompatible version"};},function(e,t){var n={}.toString;e.exports=Array.isArray||function(e){return "[object Array]"==n.call(e)};},function(e,t,n){e.exports=n(11);},function(e,t,n){(function(t){function r(e,t){B=B||n(5),e=e||{},this.objectMode=!!e.objectMode,t instanceof B&&(this.objectMode=this.objectMode||!!e.readableObjectMode);var r=e.highWaterMark,i=this.objectMode?16:16384;this.highWaterMark=r||0===r?r:i,this.highWaterMark=~~this.highWaterMark,this.buffer=[],this.length=0,this.pipes=null,this.pipesCount=0,this.flowing=null,this.ended=!1,this.endEmitted=!1,this.reading=!1,this.sync=!0,this.needReadable=!1,this.emittedReadable=!1,this.readableListening=!1,this.resumeScheduled=!1,this.defaultEncoding=e.defaultEncoding||"utf8",this.ranOut=!1,this.awaitDrain=0,this.readingMore=!1,this.decoder=null,this.encoding=null,e.encoding&&(j||(j=n(27).StringDecoder),this.decoder=new j(e.encoding),this.encoding=e.encoding);}function i(e){if(B=B||n(5),!(this instanceof i))return new i(e);this._readableState=new r(e,this),this.readable=!0,e&&"function"==typeof e.read&&(this._read=e.read),A.call(this);}function a(e,t,n,r,i){var a=u(t,n);if(a)e.emit("error",a);else if(null===n)t.reading=!1,c(e,t);else if(t.objectMode||n&&n.length>0)if(t.ended&&!i){var s=new Error("stream.push() after EOF");e.emit("error",s);}else if(t.endEmitted&&i){var s=new Error("stream.unshift() after end event");e.emit("error",s);}else {var l;!t.decoder||i||r||(n=t.decoder.write(n),l=!t.objectMode&&0===n.length),i||(t.reading=!1),l||(t.flowing&&0===t.length&&!t.sync?(e.emit("data",n),e.read(0)):(t.length+=t.objectMode?1:n.length,i?t.buffer.unshift(n):t.buffer.push(n),t.needReadable&&f(e))),d(e,t);}else i||(t.reading=!1);return o(t)}function o(e){return !e.ended&&(e.needReadable||e.length<e.highWaterMark||0===e.length)}function s(e){return e>=M?e=M:(e--,e|=e>>>1,e|=e>>>2,e|=e>>>4,e|=e>>>8,e|=e>>>16,e++),e}function l(e,t){return 0===t.length&&t.ended?0:t.objectMode?0===e?0:1:null===e||isNaN(e)?t.flowing&&t.buffer.length?t.buffer[0].length:t.length:e<=0?0:(e>t.highWaterMark&&(t.highWaterMark=s(e)),e>t.length?t.ended?t.length:(t.needReadable=!0,0):e)}function u(e,t){var n=null;return C.isBuffer(t)||"string"==typeof t||null===t||void 0===t||e.objectMode||(n=new TypeError("Invalid non-string/buffer chunk")),n}function c(e,t){if(!t.ended){if(t.decoder){var n=t.decoder.end();n&&n.length&&(t.buffer.push(n),t.length+=t.objectMode?1:n.length);}t.ended=!0,f(e);}}function f(e){var t=e._readableState;t.needReadable=!1,t.emittedReadable||(L("emitReadable",t.flowing),t.emittedReadable=!0,t.sync?E(h,e):h(e));}function h(e){L("emit readable"),e.emit("readable"),y(e);}function d(e,t){t.readingMore||(t.readingMore=!0,E(p,e,t));}function p(e,t){for(var n=t.length;!t.reading&&!t.flowing&&!t.ended&&t.length<t.highWaterMark&&(L("maybeReadMore read 0"),e.read(0),n!==t.length);)n=t.length;t.readingMore=!1;}function m(e){return function(){var t=e._readableState;L("pipeOnDrain",t.awaitDrain),t.awaitDrain&&t.awaitDrain--,0===t.awaitDrain&&O(e,"data")&&(t.flowing=!0,y(e));}}function g(e){L("readable nexttick read 0"),e.read(0);}function v(e,t){t.resumeScheduled||(t.resumeScheduled=!0,E(b,e,t));}function b(e,t){t.reading||(L("resume read 0"),e.read(0)),t.resumeScheduled=!1,e.emit("resume"),y(e),t.flowing&&!t.reading&&e.read(0);}function y(e){var t=e._readableState;if(L("flow",t.flowing),t.flowing)do{var n=e.read();}while(null!==n&&t.flowing)}function w(e,t){var n,r=t.buffer,i=t.length,a=!!t.decoder,o=!!t.objectMode;if(0===r.length)return null;if(0===i)n=null;else if(o)n=r.shift();else if(!e||e>=i)n=a?r.join(""):1===r.length?r[0]:C.concat(r,i),r.length=0;else if(e<r[0].length){var s=r[0];n=s.slice(0,e),r[0]=s.slice(e);}else if(e===r[0].length)n=r.shift();else {n=a?"":new C(e);for(var l=0,u=0,c=r.length;u<c&&l<e;u++){var s=r[0],f=Math.min(e-l,s.length);a?n+=s.slice(0,f):s.copy(n,l,0,f),f<s.length?r[0]=s.slice(f):r.shift(),l+=f;}}return n}function _(e){var t=e._readableState;if(t.length>0)throw new Error("endReadable called on non-empty stream");t.endEmitted||(t.ended=!0,E(k,t,e));}function k(e,t){e.endEmitted||0!==e.length||(e.endEmitted=!0,t.readable=!1,t.emit("end"));}function x(e,t){for(var n=0,r=e.length;n<r;n++)t(e[n],n);}function S(e,t){for(var n=0,r=e.length;n<r;n++)if(e[n]===t)return n;return -1}e.exports=i;var E=n(16),T=n(24),C=n(2).Buffer;i.ReadableState=r;var A,O=(n(12),function(e,t){return e.listeners(t).length});!function(){try{A=n(11);}catch(e){}finally{A||(A=n(12).EventEmitter);}}();var C=n(2).Buffer,I=n(8);I.inherits=n(7);var R=n(57),L=void 0;L=R&&R.debuglog?R.debuglog("stream"):function(){};var j;I.inherits(i,A);var B,B;i.prototype.push=function(e,t){var n=this._readableState;return n.objectMode||"string"!=typeof e||(t=t||n.defaultEncoding)!==n.encoding&&(e=new C(e,t),t=""),a(this,n,e,t,!1)},i.prototype.unshift=function(e){return a(this,this._readableState,e,"",!0)},i.prototype.isPaused=function(){return !1===this._readableState.flowing},i.prototype.setEncoding=function(e){return j||(j=n(27).StringDecoder),this._readableState.decoder=new j(e),this._readableState.encoding=e,this};var M=8388608;i.prototype.read=function(e){L("read",e);var t=this._readableState,n=e;if(("number"!=typeof e||e>0)&&(t.emittedReadable=!1),0===e&&t.needReadable&&(t.length>=t.highWaterMark||t.ended))return L("read: emitReadable",t.length,t.ended),0===t.length&&t.ended?_(this):f(this),null;if(0===(e=l(e,t))&&t.ended)return 0===t.length&&_(this),null;var r=t.needReadable;L("need readable",r),(0===t.length||t.length-e<t.highWaterMark)&&(r=!0,L("length less than watermark",r)),(t.ended||t.reading)&&(r=!1,L("reading or ended",r)),r&&(L("do read"),t.reading=!0,t.sync=!0,0===t.length&&(t.needReadable=!0),this._read(t.highWaterMark),t.sync=!1),r&&!t.reading&&(e=l(n,t));var i;return i=e>0?w(e,t):null,null===i&&(t.needReadable=!0,e=0),t.length-=e,0!==t.length||t.ended||(t.needReadable=!0),n!==e&&t.ended&&0===t.length&&_(this),null!==i&&this.emit("data",i),i},i.prototype._read=function(e){this.emit("error",new Error("not implemented"));},i.prototype.pipe=function(e,n){function r(e){L("onunpipe"),e===f&&a();}function i(){L("onend"),e.end();}function a(){L("cleanup"),e.removeListener("close",l),e.removeListener("finish",u),e.removeListener("drain",g),e.removeListener("error",s),e.removeListener("unpipe",r),f.removeListener("end",i),f.removeListener("end",a),f.removeListener("data",o),v=!0,!h.awaitDrain||e._writableState&&!e._writableState.needDrain||g();}function o(t){L("ondata"),!1===e.write(t)&&(1!==h.pipesCount||h.pipes[0]!==e||1!==f.listenerCount("data")||v||(L("false write response, pause",f._readableState.awaitDrain),f._readableState.awaitDrain++),f.pause());}function s(t){L("onerror",t),c(),e.removeListener("error",s),0===O(e,"error")&&e.emit("error",t);}function l(){e.removeListener("finish",u),c();}function u(){L("onfinish"),e.removeListener("close",l),c();}function c(){L("unpipe"),f.unpipe(e);}var f=this,h=this._readableState;switch(h.pipesCount){case 0:h.pipes=e;break;case 1:h.pipes=[h.pipes,e];break;default:h.pipes.push(e);}h.pipesCount+=1,L("pipe count=%d opts=%j",h.pipesCount,n);var d=(!n||!1!==n.end)&&e!==t.stdout&&e!==t.stderr,p=d?i:a;h.endEmitted?E(p):f.once("end",p),e.on("unpipe",r);var g=m(f);e.on("drain",g);var v=!1;return f.on("data",o),e._events&&e._events.error?T(e._events.error)?e._events.error.unshift(s):e._events.error=[s,e._events.error]:e.on("error",s),e.once("close",l),e.once("finish",u),e.emit("pipe",f),h.flowing||(L("pipe resume"),f.resume()),e},i.prototype.unpipe=function(e){var t=this._readableState;if(0===t.pipesCount)return this;if(1===t.pipesCount)return e&&e!==t.pipes?this:(e||(e=t.pipes),t.pipes=null,t.pipesCount=0,t.flowing=!1,e&&e.emit("unpipe",this),this);if(!e){var n=t.pipes,r=t.pipesCount;t.pipes=null,t.pipesCount=0,t.flowing=!1;for(var i=0;i<r;i++)n[i].emit("unpipe",this);return this}var a=S(t.pipes,e);return -1===a?this:(t.pipes.splice(a,1),t.pipesCount-=1,1===t.pipesCount&&(t.pipes=t.pipes[0]),e.emit("unpipe",this),this)},i.prototype.on=function(e,t){var n=A.prototype.on.call(this,e,t);if("data"===e&&!1!==this._readableState.flowing&&this.resume(),"readable"===e&&!this._readableState.endEmitted){var r=this._readableState;r.readableListening||(r.readableListening=!0,r.emittedReadable=!1,r.needReadable=!0,r.reading?r.length&&f(this):E(g,this));}return n},i.prototype.addListener=i.prototype.on,i.prototype.resume=function(){var e=this._readableState;return e.flowing||(L("resume"),e.flowing=!0,v(this,e)),this},i.prototype.pause=function(){return L("call pause flowing=%j",this._readableState.flowing),!1!==this._readableState.flowing&&(L("pause"),this._readableState.flowing=!1,this.emit("pause")),this},i.prototype.wrap=function(e){var t=this._readableState,n=!1,r=this;e.on("end",function(){if(L("wrapped end"),t.decoder&&!t.ended){var e=t.decoder.end();e&&e.length&&r.push(e);}r.push(null);}),e.on("data",function(i){L("wrapped data"),t.decoder&&(i=t.decoder.write(i)),(!t.objectMode||null!==i&&void 0!==i)&&(t.objectMode||i&&i.length)&&(r.push(i)||(n=!0,e.pause()));});for(var i in e)void 0===this[i]&&"function"==typeof e[i]&&(this[i]=function(t){return function(){return e[t].apply(e,arguments)}}(i));return x(["error","close","destroy","pause","resume"],function(t){e.on(t,r.emit.bind(r,t));}),r._read=function(t){L("wrapped _read",t),n&&(n=!1,e.resume());},r},i._fromList=w;}).call(t,n(13));},function(e,t,n){function r(e){if(!e)return "utf8";for(var t;;)switch(e){case"utf8":case"utf-8":return "utf8";case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return "utf16le";case"latin1":case"binary":return "latin1";case"base64":case"ascii":case"hex":return e;default:if(t)return;e=(""+e).toLowerCase(),t=!0;}}function i(e){var t=r(e);if("string"!=typeof t&&(b.isEncoding===y||!y(e)))throw new Error("Unknown encoding: "+e);return t||e}function a(e){this.encoding=i(e);var t;switch(this.encoding){case"utf16le":this.text=h,this.end=d,t=4;break;case"utf8":this.fillLast=u,t=4;break;case"base64":this.text=p,this.end=m,t=3;break;default:return this.write=g,void(this.end=v)}this.lastNeed=0,this.lastTotal=0,this.lastChar=b.allocUnsafe(t);}function o(e){return e<=127?0:e>>5==6?2:e>>4==14?3:e>>3==30?4:-1}function s(e,t,n){var r=t.length-1;if(r<n)return 0;var i=o(t[r]);return i>=0?(i>0&&(e.lastNeed=i-1),i):--r<n?0:(i=o(t[r]))>=0?(i>0&&(e.lastNeed=i-2),i):--r<n?0:(i=o(t[r]),i>=0?(i>0&&(2===i?i=0:e.lastNeed=i-3),i):0)}function l(e,t,n){if(128!=(192&t[0]))return e.lastNeed=0,"�".repeat(n);if(e.lastNeed>1&&t.length>1){if(128!=(192&t[1]))return e.lastNeed=1,"�".repeat(n+1);if(e.lastNeed>2&&t.length>2&&128!=(192&t[2]))return e.lastNeed=2,"�".repeat(n+2)}}function u(e){var t=this.lastTotal-this.lastNeed,n=l(this,e,t);return void 0!==n?n:this.lastNeed<=e.length?(e.copy(this.lastChar,t,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal)):(e.copy(this.lastChar,t,0,e.length),void(this.lastNeed-=e.length))}function c(e,t){var n=s(this,e,t);if(!this.lastNeed)return e.toString("utf8",t);this.lastTotal=n;var r=e.length-(n-this.lastNeed);return e.copy(this.lastChar,0,r),e.toString("utf8",t,r)}function f(e){var t=e&&e.length?this.write(e):"";return this.lastNeed?t+"�".repeat(this.lastTotal-this.lastNeed):t}function h(e,t){if((e.length-t)%2==0){var n=e.toString("utf16le",t);if(n){var r=n.charCodeAt(n.length-1);if(r>=55296&&r<=56319)return this.lastNeed=2,this.lastTotal=4,this.lastChar[0]=e[e.length-2],this.lastChar[1]=e[e.length-1],n.slice(0,-1)}return n}return this.lastNeed=1,this.lastTotal=2,this.lastChar[0]=e[e.length-1],e.toString("utf16le",t,e.length-1)}function d(e){var t=e&&e.length?this.write(e):"";if(this.lastNeed){var n=this.lastTotal-this.lastNeed;return t+this.lastChar.toString("utf16le",0,n)}return t}function p(e,t){var n=(e.length-t)%3;return 0===n?e.toString("base64",t):(this.lastNeed=3-n,this.lastTotal=3,1===n?this.lastChar[0]=e[e.length-1]:(this.lastChar[0]=e[e.length-2],this.lastChar[1]=e[e.length-1]),e.toString("base64",t,e.length-n))}function m(e){var t=e&&e.length?this.write(e):"";return this.lastNeed?t+this.lastChar.toString("base64",0,3-this.lastNeed):t}function g(e){return e.toString(this.encoding)}function v(e){return e&&e.length?this.write(e):""}var b=n(61).Buffer,y=b.isEncoding||function(e){switch((e=""+e)&&e.toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":case"raw":return !0;default:return !1}};t.StringDecoder=a,a.prototype.write=function(e){if(0===e.length)return "";var t,n;if(this.lastNeed){if(void 0===(t=this.fillLast(e)))return "";n=this.lastNeed,this.lastNeed=0;}else n=0;return n<e.length?t?t+this.text(e,n):this.text(e,n):t||""},a.prototype.end=f,a.prototype.text=c,a.prototype.fillLast=function(e){if(this.lastNeed<=e.length)return e.copy(this.lastChar,this.lastTotal-this.lastNeed,0,this.lastNeed),this.lastChar.toString(this.encoding,0,this.lastTotal);e.copy(this.lastChar,this.lastTotal-this.lastNeed,0,e.length),this.lastNeed-=e.length;};},function(e,t,n){function r(e){if(!(this instanceof r))return new r(e);i.call(this,e);}e.exports=r;var i=n(18),a=n(8);a.inherits=n(7),a.inherits(r,i),r.prototype._transform=function(e,t,n){n(null,e);};},function(e,t,n){var r=n(0),i=n(3),a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";t.encode=function(e){for(var t,n,i,o,s,l,u,c=[],f=0,h=e.length,d=h,p="string"!==r.getTypeOf(e);f<e.length;)d=h-f,p?(t=e[f++],n=f<h?e[f++]:0,i=f<h?e[f++]:0):(t=e.charCodeAt(f++),n=f<h?e.charCodeAt(f++):0,i=f<h?e.charCodeAt(f++):0),o=t>>2,s=(3&t)<<4|n>>4,l=d>1?(15&n)<<2|i>>6:64,u=d>2?63&i:64,c.push(a.charAt(o)+a.charAt(s)+a.charAt(l)+a.charAt(u));return c.join("")},t.decode=function(e){var t,n,r,o,s,l,u,c=0,f=0;if("data:"===e.substr(0,"data:".length))throw new Error("Invalid base64 input, it looks like a data url.");e=e.replace(/[^A-Za-z0-9\+\/\=]/g,"");var h=3*e.length/4;if(e.charAt(e.length-1)===a.charAt(64)&&h--,e.charAt(e.length-2)===a.charAt(64)&&h--,h%1!=0)throw new Error("Invalid base64 input, bad content length.");var d;for(d=i.uint8array?new Uint8Array(0|h):new Array(0|h);c<e.length;)o=a.indexOf(e.charAt(c++)),s=a.indexOf(e.charAt(c++)),l=a.indexOf(e.charAt(c++)),u=a.indexOf(e.charAt(c++)),t=o<<2|s>>4,n=(15&s)<<4|l>>2,r=(3&l)<<6|u,d[f++]=t,64!==l&&(d[f++]=n),64!==u&&(d[f++]=r);return d};},function(e,t){var n=e.exports={version:"2.3.0"};"number"==typeof __e&&(__e=n);},function(e,t,n){var r=n(69);e.exports=function(e,t,n){if(r(e),void 0===t)return e;switch(n){case 1:return function(n){return e.call(t,n)};case 2:return function(n,r){return e.call(t,n,r)};case 3:return function(n,r,i){return e.call(t,n,r,i)}}return function(){return e.apply(t,arguments)}};},function(e,t){e.exports=function(e){try{return !!e()}catch(e){return !0}};},function(e,t,n){var r=n(19),i=n(15).document,a=r(i)&&r(i.createElement);e.exports=function(e){return a?i.createElement(e):{}};},function(e,t,n){(function(t){function r(e,t,n){switch(e){case"blob":return s.newBlob(s.transformTo("arraybuffer",t),n);case"base64":return c.encode(t);default:return s.transformTo(e,t)}}function i(e,n){var r,i=0,a=null,o=0;for(r=0;r<n.length;r++)o+=n[r].length;switch(e){case"string":return n.join("");case"array":return Array.prototype.concat.apply([],n);case"uint8array":for(a=new Uint8Array(o),r=0;r<n.length;r++)a.set(n[r],i),i+=n[r].length;return a;case"nodebuffer":return t.concat(n);default:throw new Error("concat : unsupported type '"+e+"'")}}function a(e,t){return new h.Promise(function(n,a){var o=[],s=e._internalType,l=e._outputType,u=e._mimeType;e.on("data",function(e,n){o.push(e),t&&t(n);}).on("error",function(e){o=[],a(e);}).on("end",function(){try{var e=r(l,i(s,o),u);n(e);}catch(e){a(e);}o=[];}).resume();})}function o(e,t,n){var r=t;switch(t){case"blob":case"arraybuffer":r="uint8array";break;case"base64":r="string";}try{this._internalType=r,this._outputType=t,this._mimeType=n,s.checkSupport(r),this._worker=e.pipe(new l(r)),e.lock();}catch(e){this._worker=new u("error"),this._worker.error(e);}}var s=n(0),l=n(82),u=n(1),c=n(29),f=n(3),h=n(9),d=null;if(f.nodestream)try{d=n(83);}catch(e){}o.prototype={accumulate:function(e){return a(this,e)},on:function(e,t){var n=this;return "data"===e?this._worker.on(e,function(e){t.call(n,e.data,e.meta);}):this._worker.on(e,function(){s.delay(t,arguments,n);}),this},resume:function(){return s.delay(this._worker.resume,[],this._worker),this},pause:function(){return this._worker.pause(),this},toNodejsStream:function(e){if(s.checkSupport("nodestream"),"nodebuffer"!==this._outputType)throw new Error(this._outputType+" is not supported by this method");return new d(this,{objectMode:"nodebuffer"!==this._outputType},e)}},e.exports=o;}).call(t,n(2).Buffer);},function(e,t,n){t.base64=!1,t.binary=!1,t.dir=!1,t.createFolders=!0,t.date=null,t.compression=null,t.compressionOptions=null,t.comment=null,t.unixPermissions=null,t.dosPermissions=null;},function(e,t,n){function r(e){a.call(this,"DataWorker");var t=this;this.dataIsReady=!1,this.index=0,this.max=0,this.data=null,this.type="",this._tickScheduled=!1,e.then(function(e){t.dataIsReady=!0,t.data=e,t.max=e&&e.length||0,t.type=i.getTypeOf(e),t.isPaused||t._tickAndRepeat();},function(e){t.error(e);});}var i=n(0),a=n(1);i.inherits(r,a),r.prototype.cleanUp=function(){a.prototype.cleanUp.call(this),this.data=null;},r.prototype.resume=function(){return !!a.prototype.resume.call(this)&&(!this._tickScheduled&&this.dataIsReady&&(this._tickScheduled=!0,i.delay(this._tickAndRepeat,[],this)),!0)},r.prototype._tickAndRepeat=function(){this._tickScheduled=!1,this.isPaused||this.isFinished||(this._tick(),this.isFinished||(i.delay(this._tickAndRepeat,[],this),this._tickScheduled=!0));},r.prototype._tick=function(){if(this.isPaused||this.isFinished)return !1;var e=null,t=Math.min(this.max,this.index+16384);if(this.index>=this.max)return this.end();switch(this.type){case"string":e=this.data.substring(this.index,t);break;case"uint8array":e=this.data.subarray(this.index,t);break;case"array":case"nodebuffer":e=this.data.slice(this.index,t);}return this.index=t,this.push({data:e,meta:{percent:this.max?this.index/this.max*100:0}})},e.exports=r;},function(e,t,n){function r(e){a.call(this,"DataLengthProbe for "+e),this.propName=e,this.withStreamInfo(e,0);}var i=n(0),a=n(1);i.inherits(r,a),r.prototype.processChunk=function(e){if(e){var t=this.streamInfo[this.propName]||0;this.streamInfo[this.propName]=t+e.data.length;}a.prototype.processChunk.call(this,e);},e.exports=r;},function(e,t,n){function r(){i.call(this,"Crc32Probe"),this.withStreamInfo("crc32",0);}var i=n(1),a=n(22);n(0).inherits(r,i),r.prototype.processChunk=function(e){this.streamInfo.crc32=a(e.data,this.streamInfo.crc32||0),this.push(e);},e.exports=r;},function(e,t,n){var r=n(1);t.STORE={magic:"\0\0",compressWorker:function(e){return new r("STORE compression")},uncompressWorker:function(){return new r("STORE decompression")}},t.DEFLATE=n(86);},function(e,t,n){function r(e,t,n,r){for(var i=65535&e|0,a=e>>>16&65535|0,o=0;0!==n;){o=n>2e3?2e3:n,n-=o;do{i=i+t[r++]|0,a=a+i|0;}while(--o);i%=65521,a%=65521;}return i|a<<16|0}e.exports=r;},function(e,t,n){function r(e,t,n,r){var a=i,o=r+n;e^=-1;for(var s=r;s<o;s++)e=e>>>8^a[255&(e^t[s])];return -1^e}var i=function(){for(var e,t=[],n=0;n<256;n++){e=n;for(var r=0;r<8;r++)e=1&e?3988292384^e>>>1:e>>>1;t[n]=e;}return t}();e.exports=r;},function(e,t,n){function r(e,t){if(t<65537&&(e.subarray&&o||!e.subarray&&a))return String.fromCharCode.apply(null,i.shrinkBuf(e,t));for(var n="",r=0;r<t;r++)n+=String.fromCharCode(e[r]);return n}var i=n(4),a=!0,o=!0;try{String.fromCharCode.apply(null,[0]);}catch(e){a=!1;}try{String.fromCharCode.apply(null,new Uint8Array(1));}catch(e){o=!1;}for(var s=new i.Buf8(256),l=0;l<256;l++)s[l]=l>=252?6:l>=248?5:l>=240?4:l>=224?3:l>=192?2:1;s[254]=s[254]=1,t.string2buf=function(e){var t,n,r,a,o,s=e.length,l=0;for(a=0;a<s;a++)n=e.charCodeAt(a),55296==(64512&n)&&a+1<s&&56320==(64512&(r=e.charCodeAt(a+1)))&&(n=65536+(n-55296<<10)+(r-56320),a++),l+=n<128?1:n<2048?2:n<65536?3:4;for(t=new i.Buf8(l),o=0,a=0;o<l;a++)n=e.charCodeAt(a),55296==(64512&n)&&a+1<s&&56320==(64512&(r=e.charCodeAt(a+1)))&&(n=65536+(n-55296<<10)+(r-56320),a++),n<128?t[o++]=n:n<2048?(t[o++]=192|n>>>6,t[o++]=128|63&n):n<65536?(t[o++]=224|n>>>12,t[o++]=128|n>>>6&63,t[o++]=128|63&n):(t[o++]=240|n>>>18,t[o++]=128|n>>>12&63,t[o++]=128|n>>>6&63,t[o++]=128|63&n);return t},t.buf2binstring=function(e){return r(e,e.length)},t.binstring2buf=function(e){for(var t=new i.Buf8(e.length),n=0,r=t.length;n<r;n++)t[n]=e.charCodeAt(n);return t},t.buf2string=function(e,t){var n,i,a,o,l=t||e.length,u=new Array(2*l);for(i=0,n=0;n<l;)if((a=e[n++])<128)u[i++]=a;else if((o=s[a])>4)u[i++]=65533,n+=o-1;else {for(a&=2===o?31:3===o?15:7;o>1&&n<l;)a=a<<6|63&e[n++],o--;o>1?u[i++]=65533:a<65536?u[i++]=a:(a-=65536,u[i++]=55296|a>>10&1023,u[i++]=56320|1023&a);}return r(u,i)},t.utf8border=function(e,t){var n;for(t=t||e.length,t>e.length&&(t=e.length),n=t-1;n>=0&&128==(192&e[n]);)n--;return n<0?t:0===n?t:n+s[e[n]]>t?n:t};},function(e,t,n){function r(){this.input=null,this.next_in=0,this.avail_in=0,this.total_in=0,this.output=null,this.next_out=0,this.avail_out=0,this.total_out=0,this.msg="",this.state=null,this.data_type=2,this.adler=0;}e.exports=r;},function(e,t,n){e.exports={Z_NO_FLUSH:0,Z_PARTIAL_FLUSH:1,Z_SYNC_FLUSH:2,Z_FULL_FLUSH:3,Z_FINISH:4,Z_BLOCK:5,Z_TREES:6,Z_OK:0,Z_STREAM_END:1,Z_NEED_DICT:2,Z_ERRNO:-1,Z_STREAM_ERROR:-2,Z_DATA_ERROR:-3,Z_BUF_ERROR:-5,Z_NO_COMPRESSION:0,Z_BEST_SPEED:1,Z_BEST_COMPRESSION:9,Z_DEFAULT_COMPRESSION:-1,Z_FILTERED:1,Z_HUFFMAN_ONLY:2,Z_RLE:3,Z_FIXED:4,Z_DEFAULT_STRATEGY:0,Z_BINARY:0,Z_TEXT:1,Z_UNKNOWN:2,Z_DEFLATED:8};},function(e,t,n){t.LOCAL_FILE_HEADER="PK",t.CENTRAL_FILE_HEADER="PK",t.CENTRAL_DIRECTORY_END="PK",t.ZIP64_CENTRAL_DIRECTORY_LOCATOR="PK",t.ZIP64_CENTRAL_DIRECTORY_END="PK",t.DATA_DESCRIPTOR="PK\b";},function(e,t,n){var r=n(0),i=n(3),a=n(47),o=n(100),s=n(101),l=n(49);e.exports=function(e){var t=r.getTypeOf(e);return r.checkSupport(t),"string"!==t||i.uint8array?"nodebuffer"===t?new s(e):i.uint8array?new l(r.transformTo("uint8array",e)):new a(r.transformTo("array",e)):new o(e)};},function(e,t,n){function r(e){i.call(this,e);for(var t=0;t<this.data.length;t++)e[t]=255&e[t];}var i=n(48);n(0).inherits(r,i),r.prototype.byteAt=function(e){return this.data[this.zero+e]},r.prototype.lastIndexOfSignature=function(e){for(var t=e.charCodeAt(0),n=e.charCodeAt(1),r=e.charCodeAt(2),i=e.charCodeAt(3),a=this.length-4;a>=0;--a)if(this.data[a]===t&&this.data[a+1]===n&&this.data[a+2]===r&&this.data[a+3]===i)return a-this.zero;return -1},r.prototype.readAndCheckSignature=function(e){var t=e.charCodeAt(0),n=e.charCodeAt(1),r=e.charCodeAt(2),i=e.charCodeAt(3),a=this.readData(4);return t===a[0]&&n===a[1]&&r===a[2]&&i===a[3]},r.prototype.readData=function(e){if(this.checkOffset(e),0===e)return [];var t=this.data.slice(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t},e.exports=r;},function(e,t,n){function r(e){this.data=e,this.length=e.length,this.index=0,this.zero=0;}var i=n(0);r.prototype={checkOffset:function(e){this.checkIndex(this.index+e);},checkIndex:function(e){if(this.length<this.zero+e||e<0)throw new Error("End of data reached (data length = "+this.length+", asked index = "+e+"). Corrupted zip ?")},setIndex:function(e){this.checkIndex(e),this.index=e;},skip:function(e){this.setIndex(this.index+e);},byteAt:function(e){},readInt:function(e){var t,n=0;for(this.checkOffset(e),t=this.index+e-1;t>=this.index;t--)n=(n<<8)+this.byteAt(t);return this.index+=e,n},readString:function(e){return i.transformTo("string",this.readData(e))},readData:function(e){},lastIndexOfSignature:function(e){},readAndCheckSignature:function(e){},readDate:function(){var e=this.readInt(4);return new Date(Date.UTC(1980+(e>>25&127),(e>>21&15)-1,e>>16&31,e>>11&31,e>>5&63,(31&e)<<1))}},e.exports=r;},function(e,t,n){function r(e){i.call(this,e);}var i=n(47);n(0).inherits(r,i),r.prototype.readData=function(e){if(this.checkOffset(e),0===e)return new Uint8Array(0);var t=this.data.subarray(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t},e.exports=r;},function(e,t,n){function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(51),a=r(i),o=n(52),s=r(o),l=n(103),u=r(l);t.default=a.default.plugins.add("gjs-plugin-export",function(e,t){var n=t||{},r=e.getConfig(),i=r.stylePrefix,a=document.createElement("BUTTON"),o={addExportBtn:1,btnLabel:"Export to ZIP",preHtml:'<!doctype html><html lang="en"><head><meta charset="utf-8"><link rel="stylesheet" href="./css/style.css"></head><body>',postHtml:"</body><html>",preCss:"",postCss:""};for(var l in o)l in n||(n[l]=o[l]);a.innerHTML=n.btnLabel,a.className=i+"btn-prim",e.Commands.add("gjs-export-zip",{run:function(){var t=new s.default,r=t.folder("css"),i="grapesjs_template_"+Date.now()+".zip";t.file("index.html",n.preHtml+e.getHtml()+n.postHtml),r.file("style.css",n.preCss+e.getCss()+n.postCss),t.generateAsync({type:"blob"}).then(function(e){u.default.saveAs(e,i);});}}),n.addExportBtn&&e.on("run:export-template",function(){e.Modal.getContentEl().appendChild(a),a.onclick=function(){e.runCommand("gjs-export-zip");};});});},function(t,n){t.exports=e;},function(e,t,n){function r(){if(!(this instanceof r))return new r;if(arguments.length)throw new Error("The constructor with parameters has been removed in JSZip 3.0, please check the upgrade guide.");this.files={},this.comment=null,this.root="",this.clone=function(){var e=new r;for(var t in this)"function"!=typeof this[t]&&(e[t]=this[t]);return e};}r.prototype=n(53),r.prototype.loadAsync=n(98),r.support=n(3),r.defaults=n(35),r.version="3.1.5",r.loadAsync=function(e,t){return (new r).loadAsync(e,t)},r.external=n(9),e.exports=r;},function(e,t,n){function r(e){return "[object RegExp]"===Object.prototype.toString.call(e)}var i=n(6),a=n(0),o=n(1),s=n(34),l=n(35),u=n(21),c=n(84),f=n(85),h=n(14),d=n(97),p=function(e,t,n){var r,i=a.getTypeOf(t),s=a.extend(n||{},l);s.date=s.date||new Date,null!==s.compression&&(s.compression=s.compression.toUpperCase()),"string"==typeof s.unixPermissions&&(s.unixPermissions=parseInt(s.unixPermissions,8)),s.unixPermissions&&16384&s.unixPermissions&&(s.dir=!0),s.dosPermissions&&16&s.dosPermissions&&(s.dir=!0),s.dir&&(e=g(e)),s.createFolders&&(r=m(e))&&v.call(this,r,!0);var f="string"===i&&!1===s.binary&&!1===s.base64;n&&void 0!==n.binary||(s.binary=!f),(t instanceof u&&0===t.uncompressedSize||s.dir||!t||0===t.length)&&(s.base64=!1,s.binary=!0,t="",s.compression="STORE",i="string");var p=null;p=t instanceof u||t instanceof o?t:h.isNode&&h.isStream(t)?new d(e,t):a.prepareContent(e,t,s.binary,s.optimizedBinaryString,s.base64);var b=new c(e,p,s);this.files[e]=b;},m=function(e){"/"===e.slice(-1)&&(e=e.substring(0,e.length-1));var t=e.lastIndexOf("/");return t>0?e.substring(0,t):""},g=function(e){return "/"!==e.slice(-1)&&(e+="/"),e},v=function(e,t){return t=void 0!==t?t:l.createFolders,e=g(e),this.files[e]||p.call(this,e,null,{dir:!0,createFolders:t}),this.files[e]},b={load:function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},forEach:function(e){var t,n,r;for(t in this.files)this.files.hasOwnProperty(t)&&(r=this.files[t],(n=t.slice(this.root.length,t.length))&&t.slice(0,this.root.length)===this.root&&e(n,r));},filter:function(e){var t=[];return this.forEach(function(n,r){e(n,r)&&t.push(r);}),t},file:function(e,t,n){if(1===arguments.length){if(r(e)){var i=e;return this.filter(function(e,t){return !t.dir&&i.test(e)})}var a=this.files[this.root+e];return a&&!a.dir?a:null}return e=this.root+e,p.call(this,e,t,n),this},folder:function(e){if(!e)return this;if(r(e))return this.filter(function(t,n){return n.dir&&e.test(t)});var t=this.root+e,n=v.call(this,t),i=this.clone();return i.root=n.name,i},remove:function(e){e=this.root+e;var t=this.files[e];if(t||("/"!==e.slice(-1)&&(e+="/"),t=this.files[e]),t&&!t.dir)delete this.files[e];else for(var n=this.filter(function(t,n){return n.name.slice(0,e.length)===e}),r=0;r<n.length;r++)delete this.files[n[r].name];return this},generate:function(e){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},generateInternalStream:function(e){var t,n={};try{if(n=a.extend(e||{},{streamFiles:!1,compression:"STORE",compressionOptions:null,type:"",platform:"DOS",comment:null,mimeType:"application/zip",encodeFileName:i.utf8encode}),n.type=n.type.toLowerCase(),n.compression=n.compression.toUpperCase(),"binarystring"===n.type&&(n.type="string"),!n.type)throw new Error("No output type specified.");a.checkSupport(n.type),"darwin"!==n.platform&&"freebsd"!==n.platform&&"linux"!==n.platform&&"sunos"!==n.platform||(n.platform="UNIX"),"win32"===n.platform&&(n.platform="DOS");var r=n.comment||this.comment||"";t=f.generateWorker(this,n,r);}catch(e){t=new o("error"),t.error(e);}return new s(t,n.type||"string",n.mimeType)},generateAsync:function(e,t){return this.generateInternalStream(e).accumulate(t)},generateNodeStream:function(e,t){return e=e||{},e.type||(e.type="nodebuffer"),this.generateInternalStream(e).toNodejsStream(t)}};e.exports=b;},function(e,t,n){function r(e){var t=e.length;if(t%4>0)throw new Error("Invalid string. Length must be a multiple of 4");return "="===e[t-2]?2:"="===e[t-1]?1:0}function i(e){return 3*e.length/4-r(e)}function a(e){var t,n,i,a,o,s=e.length;a=r(e),o=new f(3*s/4-a),n=a>0?s-4:s;var l=0;for(t=0;t<n;t+=4)i=c[e.charCodeAt(t)]<<18|c[e.charCodeAt(t+1)]<<12|c[e.charCodeAt(t+2)]<<6|c[e.charCodeAt(t+3)],o[l++]=i>>16&255,o[l++]=i>>8&255,o[l++]=255&i;return 2===a?(i=c[e.charCodeAt(t)]<<2|c[e.charCodeAt(t+1)]>>4,o[l++]=255&i):1===a&&(i=c[e.charCodeAt(t)]<<10|c[e.charCodeAt(t+1)]<<4|c[e.charCodeAt(t+2)]>>2,o[l++]=i>>8&255,o[l++]=255&i),o}function o(e){return u[e>>18&63]+u[e>>12&63]+u[e>>6&63]+u[63&e]}function s(e,t,n){for(var r,i=[],a=t;a<n;a+=3)r=(e[a]<<16)+(e[a+1]<<8)+e[a+2],i.push(o(r));return i.join("")}function l(e){for(var t,n=e.length,r=n%3,i="",a=[],o=0,l=n-r;o<l;o+=16383)a.push(s(e,o,o+16383>l?l:o+16383));return 1===r?(t=e[n-1],i+=u[t>>2],i+=u[t<<4&63],i+="=="):2===r&&(t=(e[n-2]<<8)+e[n-1],i+=u[t>>10],i+=u[t>>4&63],i+=u[t<<2&63],i+="="),a.push(i),a.join("")}t.byteLength=i,t.toByteArray=a,t.fromByteArray=l;for(var u=[],c=[],f="undefined"!=typeof Uint8Array?Uint8Array:Array,h="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",d=0,p=h.length;d<p;++d)u[d]=h[d],c[h.charCodeAt(d)]=d;c["-".charCodeAt(0)]=62,c["_".charCodeAt(0)]=63;},function(e,t){t.read=function(e,t,n,r,i){var a,o,s=8*i-r-1,l=(1<<s)-1,u=l>>1,c=-7,f=n?i-1:0,h=n?-1:1,d=e[t+f];for(f+=h,a=d&(1<<-c)-1,d>>=-c,c+=s;c>0;a=256*a+e[t+f],f+=h,c-=8);for(o=a&(1<<-c)-1,a>>=-c,c+=r;c>0;o=256*o+e[t+f],f+=h,c-=8);if(0===a)a=1-u;else {if(a===l)return o?NaN:1/0*(d?-1:1);o+=Math.pow(2,r),a-=u;}return (d?-1:1)*o*Math.pow(2,a-r)},t.write=function(e,t,n,r,i,a){var o,s,l,u=8*a-i-1,c=(1<<u)-1,f=c>>1,h=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,d=r?0:a-1,p=r?1:-1,m=t<0||0===t&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(s=isNaN(t)?1:0,o=c):(o=Math.floor(Math.log(t)/Math.LN2),t*(l=Math.pow(2,-o))<1&&(o--,l*=2),t+=o+f>=1?h/l:h*Math.pow(2,1-f),t*l>=2&&(o++,l/=2),o+f>=c?(s=0,o=c):o+f>=1?(s=(t*l-1)*Math.pow(2,i),o+=f):(s=t*Math.pow(2,f-1)*Math.pow(2,i),o=0));i>=8;e[n+d]=255&s,d+=p,s/=256,i-=8);for(o=o<<i|s,u+=i;u>0;e[n+d]=255&o,d+=p,o/=256,u-=8);e[n+d-p]|=128*m;};},function(e,t,n){var r=function(){try{return n(11)}catch(e){}}();t=e.exports=n(26),t.Stream=r||t,t.Readable=t,t.Writable=n(17),t.Duplex=n(5),t.Transform=n(18),t.PassThrough=n(28);},function(e,t){},function(e,n,i){function a(e,t){this._id=e,this._clearFn=t;}var o=Function.prototype.apply;n.setTimeout=function(){return new a(o.call(setTimeout,window,arguments),clearTimeout)},n.setInterval=function(){return new a(o.call(setInterval,window,arguments),clearInterval)},n.clearTimeout=n.clearInterval=function(e){e&&e.close();},a.prototype.unref=a.prototype.ref=function(){},a.prototype.close=function(){this._clearFn.call(window,this._id);},n.enroll=function(e,t){clearTimeout(e._idleTimeoutId),e._idleTimeout=t;},n.unenroll=function(e){clearTimeout(e._idleTimeoutId),e._idleTimeout=-1;},n._unrefActive=n.active=function(e){clearTimeout(e._idleTimeoutId);var t=e._idleTimeout;t>=0&&(e._idleTimeoutId=setTimeout(function(){e._onTimeout&&e._onTimeout();},t));},i(59),n.setImmediate=t,n.clearImmediate=r;},function(e,t,n){(function(e,t){!function(e,n){function r(e){"function"!=typeof e&&(e=new Function(""+e));for(var t=new Array(arguments.length-1),n=0;n<t.length;n++)t[n]=arguments[n+1];var r={callback:e,args:t};return u[l]=r,s(l),l++}function i(e){delete u[e];}function a(e){var t=e.callback,r=e.args;switch(r.length){case 0:t();break;case 1:t(r[0]);break;case 2:t(r[0],r[1]);break;case 3:t(r[0],r[1],r[2]);break;default:t.apply(n,r);}}function o(e){if(c)setTimeout(o,0,e);else {var t=u[e];if(t){c=!0;try{a(t);}finally{i(e),c=!1;}}}}if(!e.setImmediate){var s,l=1,u={},c=!1,f=e.document,h=Object.getPrototypeOf&&Object.getPrototypeOf(e);h=h&&h.setTimeout?h:e,"[object process]"==={}.toString.call(e.process)?function(){s=function(e){t.nextTick(function(){o(e);});};}():function(){if(e.postMessage&&!e.importScripts){var t=!0,n=e.onmessage;return e.onmessage=function(){t=!1;},e.postMessage("","*"),e.onmessage=n,t}}()?function(){var t="setImmediate$"+Math.random()+"$",n=function(n){n.source===e&&"string"==typeof n.data&&0===n.data.indexOf(t)&&o(+n.data.slice(t.length));};e.addEventListener?e.addEventListener("message",n,!1):e.attachEvent("onmessage",n),s=function(n){e.postMessage(t+n,"*");};}():e.MessageChannel?function(){var e=new MessageChannel;e.port1.onmessage=function(e){o(e.data);},s=function(t){e.port2.postMessage(t);};}():f&&"onreadystatechange"in f.createElement("script")?function(){var e=f.documentElement;s=function(t){var n=f.createElement("script");n.onreadystatechange=function(){o(t),n.onreadystatechange=null,e.removeChild(n),n=null;},e.appendChild(n);};}():function(){s=function(e){setTimeout(o,0,e);};}(),h.setImmediate=r,h.clearImmediate=i;}}("undefined"==typeof self?void 0===e?this:e:self);}).call(t,n(10),n(13));},function(e,t,n){(function(t){function n(e,t){function n(){if(!i){if(r("throwDeprecation"))throw new Error(t);r("traceDeprecation")?console.trace(t):console.warn(t),i=!0;}return e.apply(this,arguments)}if(r("noDeprecation"))return e;var i=!1;return n}function r(e){try{if(!t.localStorage)return !1}catch(e){return !1}var n=t.localStorage[e];return null!=n&&"true"===String(n).toLowerCase()}e.exports=n;}).call(t,n(10));},function(e,t,n){function r(e,t){for(var n in e)t[n]=e[n];}function i(e,t,n){return o(e,t,n)}var a=n(2),o=a.Buffer;o.from&&o.alloc&&o.allocUnsafe&&o.allocUnsafeSlow?e.exports=a:(r(a,t),t.Buffer=i),r(o,i),i.from=function(e,t,n){if("number"==typeof e)throw new TypeError("Argument must not be a number");return o(e,t,n)},i.alloc=function(e,t,n){if("number"!=typeof e)throw new TypeError("Argument must be a number");var r=o(e);return void 0!==t?"string"==typeof n?r.fill(t,n):r.fill(t):r.fill(0),r},i.allocUnsafe=function(e){if("number"!=typeof e)throw new TypeError("Argument must be a number");return o(e)},i.allocUnsafeSlow=function(e){if("number"!=typeof e)throw new TypeError("Argument must be a number");return a.SlowBuffer(e)};},function(e,t,n){e.exports=n(17);},function(e,t,n){e.exports=n(5);},function(e,t,n){e.exports=n(18);},function(e,t,n){e.exports=n(28);},function(e,t,n){n(67),e.exports=n(30).setImmediate;},function(e,t,n){var r=n(68),i=n(76);r(r.G+r.B,{setImmediate:i.set,clearImmediate:i.clear});},function(e,t,n){var r=n(15),i=n(30),a=n(31),o=n(70),s=function(e,t,n){var l,u,c,f=e&s.F,h=e&s.G,d=e&s.S,p=e&s.P,m=e&s.B,g=e&s.W,v=h?i:i[t]||(i[t]={}),b=v.prototype,y=h?r:d?r[t]:(r[t]||{}).prototype;h&&(n=t);for(l in n)(u=!f&&y&&void 0!==y[l])&&l in v||(c=u?y[l]:n[l],v[l]=h&&"function"!=typeof y[l]?n[l]:m&&u?a(c,r):g&&y[l]==c?function(e){var t=function(t,n,r){if(this instanceof e){switch(arguments.length){case 0:return new e;case 1:return new e(t);case 2:return new e(t,n)}return new e(t,n,r)}return e.apply(this,arguments)};return t.prototype=e.prototype,t}(c):p&&"function"==typeof c?a(Function.call,c):c,p&&((v.virtual||(v.virtual={}))[l]=c,e&s.R&&b&&!b[l]&&o(b,l,c)));};s.F=1,s.G=2,s.S=4,s.P=8,s.B=16,s.W=32,s.U=64,s.R=128,e.exports=s;},function(e,t){e.exports=function(e){if("function"!=typeof e)throw TypeError(e+" is not a function!");return e};},function(e,t,n){var r=n(71),i=n(75);e.exports=n(20)?function(e,t,n){return r.f(e,t,i(1,n))}:function(e,t,n){return e[t]=n,e};},function(e,t,n){var r=n(72),i=n(73),a=n(74),o=Object.defineProperty;t.f=n(20)?Object.defineProperty:function(e,t,n){if(r(e),t=a(t,!0),r(n),i)try{return o(e,t,n)}catch(e){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return "value"in n&&(e[t]=n.value),e};},function(e,t,n){var r=n(19);e.exports=function(e){if(!r(e))throw TypeError(e+" is not an object!");return e};},function(e,t,n){e.exports=!n(20)&&!n(32)(function(){return 7!=Object.defineProperty(n(33)("div"),"a",{get:function(){return 7}}).a});},function(e,t,n){var r=n(19);e.exports=function(e,t){if(!r(e))return e;var n,i;if(t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;if("function"==typeof(n=e.valueOf)&&!r(i=n.call(e)))return i;if(!t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;throw TypeError("Can't convert object to primitive value")};},function(e,t){e.exports=function(e,t){return {enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}};},function(e,t,n){var r,i,a,o=n(31),s=n(77),l=n(78),u=n(33),c=n(15),f=c.process,h=c.setImmediate,d=c.clearImmediate,p=c.MessageChannel,m=0,g={},v=function(){var e=+this;if(g.hasOwnProperty(e)){var t=g[e];delete g[e],t();}},b=function(e){v.call(e.data);};h&&d||(h=function(e){for(var t=[],n=1;arguments.length>n;)t.push(arguments[n++]);return g[++m]=function(){s("function"==typeof e?e:Function(e),t);},r(m),m},d=function(e){delete g[e];},"process"==n(79)(f)?r=function(e){f.nextTick(o(v,e,1));}:p?(i=new p,a=i.port2,i.port1.onmessage=b,r=o(a.postMessage,a,1)):c.addEventListener&&"function"==typeof postMessage&&!c.importScripts?(r=function(e){c.postMessage(e+"","*");},c.addEventListener("message",b,!1)):r="onreadystatechange"in u("script")?function(e){l.appendChild(u("script")).onreadystatechange=function(){l.removeChild(this),v.call(e);};}:function(e){setTimeout(o(v,e,1),0);}),e.exports={set:h,clear:d};},function(e,t){e.exports=function(e,t,n){var r=void 0===n;switch(t.length){case 0:return r?e():e.call(n);case 1:return r?e(t[0]):e.call(n,t[0]);case 2:return r?e(t[0],t[1]):e.call(n,t[0],t[1]);case 3:return r?e(t[0],t[1],t[2]):e.call(n,t[0],t[1],t[2]);case 4:return r?e(t[0],t[1],t[2],t[3]):e.call(n,t[0],t[1],t[2],t[3])}return e.apply(n,t)};},function(e,t,n){e.exports=n(15).document&&document.documentElement;},function(e,t){var n={}.toString;e.exports=function(e){return n.call(e).slice(8,-1)};},function(e,t,n){function r(){}function i(e){if("function"!=typeof e)throw new TypeError("resolver must be a function");this.state=b,this.queue=[],this.outcome=void 0,e!==r&&l(this,e);}function a(e,t,n){this.promise=e,"function"==typeof t&&(this.onFulfilled=t,this.callFulfilled=this.otherCallFulfilled),"function"==typeof n&&(this.onRejected=n,this.callRejected=this.otherCallRejected);}function o(e,t,n){p(function(){var r;try{r=t(n);}catch(t){return m.reject(e,t)}r===e?m.reject(e,new TypeError("Cannot resolve promise with itself")):m.resolve(e,r);});}function s(e){var t=e&&e.then;if(e&&("object"==typeof e||"function"==typeof e)&&"function"==typeof t)return function(){t.apply(e,arguments);}}function l(e,t){function n(t){a||(a=!0,m.reject(e,t));}function r(t){a||(a=!0,m.resolve(e,t));}function i(){t(r,n);}var a=!1,o=u(i);"error"===o.status&&n(o.value);}function u(e,t){var n={};try{n.value=e(t),n.status="success";}catch(e){n.status="error",n.value=e;}return n}function c(e){return e instanceof this?e:m.resolve(new this(r),e)}function f(e){var t=new this(r);return m.reject(t,e)}function h(e){var t=this;if("[object Array]"!==Object.prototype.toString.call(e))return this.reject(new TypeError("must be an array"));var n=e.length,i=!1;if(!n)return this.resolve([]);for(var a=new Array(n),o=0,s=-1,l=new this(r);++s<n;)!function(e,r){function s(e){a[r]=e,++o!==n||i||(i=!0,m.resolve(l,a));}t.resolve(e).then(s,function(e){i||(i=!0,m.reject(l,e));});}(e[s],s);return l}function d(e){var t=this;if("[object Array]"!==Object.prototype.toString.call(e))return this.reject(new TypeError("must be an array"));var n=e.length,i=!1;if(!n)return this.resolve([]);for(var a=-1,o=new this(r);++a<n;)!function(e){t.resolve(e).then(function(e){i||(i=!0,m.resolve(o,e));},function(e){i||(i=!0,m.reject(o,e));});}(e[a]);return o}var p=n(81),m={},g=["REJECTED"],v=["FULFILLED"],b=["PENDING"];e.exports=i,i.prototype.catch=function(e){return this.then(null,e)},i.prototype.then=function(e,t){if("function"!=typeof e&&this.state===v||"function"!=typeof t&&this.state===g)return this;var n=new this.constructor(r);return this.state!==b?o(n,this.state===v?e:t,this.outcome):this.queue.push(new a(n,e,t)),n},a.prototype.callFulfilled=function(e){m.resolve(this.promise,e);},a.prototype.otherCallFulfilled=function(e){o(this.promise,this.onFulfilled,e);},a.prototype.callRejected=function(e){m.reject(this.promise,e);},a.prototype.otherCallRejected=function(e){o(this.promise,this.onRejected,e);},m.resolve=function(e,t){var n=u(s,t);if("error"===n.status)return m.reject(e,n.value);var r=n.value;if(r)l(e,r);else {e.state=v,e.outcome=t;for(var i=-1,a=e.queue.length;++i<a;)e.queue[i].callFulfilled(t);}return e},m.reject=function(e,t){e.state=g,e.outcome=t;for(var n=-1,r=e.queue.length;++n<r;)e.queue[n].callRejected(t);return e},i.resolve=c,i.reject=f,i.all=h,i.race=d;},function(e,t,n){(function(t){function n(){c=!0;for(var e,t,n=f.length;n;){for(t=f,f=[],e=-1;++e<n;)t[e]();n=f.length;}c=!1;}function r(e){1!==f.push(e)||c||i();}var i,a=t.MutationObserver||t.WebKitMutationObserver;if(a){var o=0,s=new a(n),l=t.document.createTextNode("");s.observe(l,{characterData:!0}),i=function(){l.data=o=++o%2;};}else if(t.setImmediate||void 0===t.MessageChannel)i="document"in t&&"onreadystatechange"in t.document.createElement("script")?function(){var e=t.document.createElement("script");e.onreadystatechange=function(){n(),e.onreadystatechange=null,e.parentNode.removeChild(e),e=null;},t.document.documentElement.appendChild(e);}:function(){setTimeout(n,0);};else {var u=new t.MessageChannel;u.port1.onmessage=n,i=function(){u.port2.postMessage(0);};}var c,f=[];e.exports=r;}).call(t,n(10));},function(e,t,n){function r(e){i.call(this,"ConvertWorker to "+e),this.destType=e;}var i=n(1),a=n(0);a.inherits(r,i),r.prototype.processChunk=function(e){this.push({data:a.transformTo(this.destType,e.data),meta:e.meta});},e.exports=r;},function(e,t,n){function r(e,t,n){i.call(this,t),this._helper=e;var r=this;e.on("data",function(e,t){r.push(e)||r._helper.pause(),n&&n(t);}).on("error",function(e){r.emit("error",e);}).on("end",function(){r.push(null);});}var i=n(25).Readable;n(0).inherits(r,i),r.prototype._read=function(){this._helper.resume();},e.exports=r;},function(e,t,n){var r=n(34),i=n(36),a=n(6),o=n(21),s=n(1),l=function(e,t,n){this.name=e,this.dir=n.dir,this.date=n.date,this.comment=n.comment,this.unixPermissions=n.unixPermissions,this.dosPermissions=n.dosPermissions,this._data=t,this._dataBinary=n.binary,this.options={compression:n.compression,compressionOptions:n.compressionOptions};};l.prototype={internalStream:function(e){var t=null,n="string";try{if(!e)throw new Error("No output type specified.");n=e.toLowerCase();var i="string"===n||"text"===n;"binarystring"!==n&&"text"!==n||(n="string"),t=this._decompressWorker();var o=!this._dataBinary;o&&!i&&(t=t.pipe(new a.Utf8EncodeWorker)),!o&&i&&(t=t.pipe(new a.Utf8DecodeWorker));}catch(e){t=new s("error"),t.error(e);}return new r(t,n,"")},async:function(e,t){return this.internalStream(e).accumulate(t)},nodeStream:function(e,t){return this.internalStream(e||"nodebuffer").toNodejsStream(t)},_compressWorker:function(e,t){if(this._data instanceof o&&this._data.compression.magic===e.magic)return this._data.getCompressedWorker();var n=this._decompressWorker();return this._dataBinary||(n=n.pipe(new a.Utf8EncodeWorker)),o.createWorkerFrom(n,e,t)},_decompressWorker:function(){return this._data instanceof o?this._data.getContentWorker():this._data instanceof s?this._data:new i(this._data)}};for(var u=["asText","asBinary","asNodeBuffer","asUint8Array","asArrayBuffer"],c=function(){throw new Error("This method has been removed in JSZip 3.0, please check the upgrade guide.")},f=0;f<u.length;f++)l.prototype[u[f]]=c;e.exports=l;},function(e,t,n){var r=n(39),i=n(96),a=function(e,t){var n=e||t,i=r[n];if(!i)throw new Error(n+" is not a valid compression method !");return i};t.generateWorker=function(e,t,n){var r=new i(t.streamFiles,n,t.platform,t.encodeFileName),o=0;try{e.forEach(function(e,n){o++;var i=a(n.options.compression,t.compression),s=n.options.compressionOptions||t.compressionOptions||{},l=n.dir,u=n.date;n._compressWorker(i,s).withStreamInfo("file",{name:e,dir:l,date:u,comment:n.comment||"",unixPermissions:n.unixPermissions,dosPermissions:n.dosPermissions}).pipe(r);}),r.entriesCount=o;}catch(e){r.error(e);}return r};},function(e,t,n){function r(e,t){s.call(this,"FlateWorker/"+e),this._pako=null,this._pakoAction=e,this._pakoOptions=t,this.meta={};}var i="undefined"!=typeof Uint8Array&&"undefined"!=typeof Uint16Array&&"undefined"!=typeof Uint32Array,a=n(87),o=n(0),s=n(1),l=i?"uint8array":"array";t.magic="\b\0",o.inherits(r,s),r.prototype.processChunk=function(e){this.meta=e.meta,null===this._pako&&this._createPako(),this._pako.push(o.transformTo(l,e.data),!1);},r.prototype.flush=function(){s.prototype.flush.call(this),null===this._pako&&this._createPako(),this._pako.push([],!0);},r.prototype.cleanUp=function(){s.prototype.cleanUp.call(this),this._pako=null;},r.prototype._createPako=function(){this._pako=new a[this._pakoAction]({raw:!0,level:this._pakoOptions.level||-1});var e=this;this._pako.onData=function(t){e.push({data:t,meta:e.meta});};},t.compressWorker=function(e){return new r("Deflate",e)},t.uncompressWorker=function(){return new r("Inflate",{})};},function(e,t,n){var r=n(4).assign,i=n(88),a=n(91),o=n(44),s={};r(s,i,a,o),e.exports=s;},function(e,t,n){function r(e){if(!(this instanceof r))return new r(e);this.options=l.assign({level:p,method:g,chunkSize:16384,windowBits:15,memLevel:8,strategy:m,to:""},e||{});var t=this.options;t.raw&&t.windowBits>0?t.windowBits=-t.windowBits:t.gzip&&t.windowBits>0&&t.windowBits<16&&(t.windowBits+=16),this.err=0,this.msg="",this.ended=!1,this.chunks=[],this.strm=new f,this.strm.avail_out=0;var n=s.deflateInit2(this.strm,t.level,t.method,t.windowBits,t.memLevel,t.strategy);if(n!==d)throw new Error(c[n]);if(t.header&&s.deflateSetHeader(this.strm,t.header),t.dictionary){var i;if(i="string"==typeof t.dictionary?u.string2buf(t.dictionary):"[object ArrayBuffer]"===h.call(t.dictionary)?new Uint8Array(t.dictionary):t.dictionary,(n=s.deflateSetDictionary(this.strm,i))!==d)throw new Error(c[n]);this._dict_set=!0;}}function i(e,t){var n=new r(t);if(n.push(e,!0),n.err)throw n.msg||c[n.err];return n.result}function a(e,t){return t=t||{},t.raw=!0,i(e,t)}function o(e,t){return t=t||{},t.gzip=!0,i(e,t)}var s=n(89),l=n(4),u=n(42),c=n(23),f=n(43),h=Object.prototype.toString,d=0,p=-1,m=0,g=8;r.prototype.push=function(e,t){var n,r,i=this.strm,a=this.options.chunkSize;if(this.ended)return !1;r=t===~~t?t:!0===t?4:0,"string"==typeof e?i.input=u.string2buf(e):"[object ArrayBuffer]"===h.call(e)?i.input=new Uint8Array(e):i.input=e,i.next_in=0,i.avail_in=i.input.length;do{if(0===i.avail_out&&(i.output=new l.Buf8(a),i.next_out=0,i.avail_out=a),1!==(n=s.deflate(i,r))&&n!==d)return this.onEnd(n),this.ended=!0,!1;0!==i.avail_out&&(0!==i.avail_in||4!==r&&2!==r)||("string"===this.options.to?this.onData(u.buf2binstring(l.shrinkBuf(i.output,i.next_out))):this.onData(l.shrinkBuf(i.output,i.next_out)));}while((i.avail_in>0||0===i.avail_out)&&1!==n);return 4===r?(n=s.deflateEnd(this.strm),this.onEnd(n),this.ended=!0,n===d):2!==r||(this.onEnd(d),i.avail_out=0,!0)},r.prototype.onData=function(e){this.chunks.push(e);},r.prototype.onEnd=function(e){e===d&&("string"===this.options.to?this.result=this.chunks.join(""):this.result=l.flattenChunks(this.chunks)),this.chunks=[],this.err=e,this.msg=this.strm.msg;},t.Deflate=r,t.deflate=i,t.deflateRaw=a,t.gzip=o;},function(e,t,n){function r(e,t){return e.msg=B[t],t}function i(e){return (e<<1)-(e>4?9:0)}function a(e){for(var t=e.length;--t>=0;)e[t]=0;}function o(e){var t=e.state,n=t.pending;n>e.avail_out&&(n=e.avail_out),0!==n&&(I.arraySet(e.output,t.pending_buf,t.pending_out,n,e.next_out),e.next_out+=n,t.pending_out+=n,e.total_out+=n,e.avail_out-=n,t.pending-=n,0===t.pending&&(t.pending_out=0));}function s(e,t){R._tr_flush_block(e,e.block_start>=0?e.block_start:-1,e.strstart-e.block_start,t),e.block_start=e.strstart,o(e.strm);}function l(e,t){e.pending_buf[e.pending++]=t;}function u(e,t){e.pending_buf[e.pending++]=t>>>8&255,e.pending_buf[e.pending++]=255&t;}function c(e,t,n,r){var i=e.avail_in;return i>r&&(i=r),0===i?0:(e.avail_in-=i,I.arraySet(t,e.input,e.next_in,i,n),1===e.state.wrap?e.adler=L(e.adler,t,i,n):2===e.state.wrap&&(e.adler=j(e.adler,t,i,n)),e.next_in+=i,e.total_in+=i,i)}function f(e,t){var n,r,i=e.max_chain_length,a=e.strstart,o=e.prev_length,s=e.nice_match,l=e.strstart>e.w_size-ue?e.strstart-(e.w_size-ue):0,u=e.window,c=e.w_mask,f=e.prev,h=e.strstart+le,d=u[a+o-1],p=u[a+o];e.prev_length>=e.good_match&&(i>>=2),s>e.lookahead&&(s=e.lookahead);do{if(n=t,u[n+o]===p&&u[n+o-1]===d&&u[n]===u[a]&&u[++n]===u[a+1]){a+=2,n++;do{}while(u[++a]===u[++n]&&u[++a]===u[++n]&&u[++a]===u[++n]&&u[++a]===u[++n]&&u[++a]===u[++n]&&u[++a]===u[++n]&&u[++a]===u[++n]&&u[++a]===u[++n]&&a<h);if(r=le-(h-a),a=h-le,r>o){if(e.match_start=t,o=r,r>=s)break;d=u[a+o-1],p=u[a+o];}}}while((t=f[t&c])>l&&0!=--i);return o<=e.lookahead?o:e.lookahead}function h(e){var t,n,r,i,a,o=e.w_size;do{if(i=e.window_size-e.lookahead-e.strstart,e.strstart>=o+(o-ue)){I.arraySet(e.window,e.window,o,o,0),e.match_start-=o,e.strstart-=o,e.block_start-=o,n=e.hash_size,t=n;do{r=e.head[--t],e.head[t]=r>=o?r-o:0;}while(--n);n=o,t=n;do{r=e.prev[--t],e.prev[t]=r>=o?r-o:0;}while(--n);i+=o;}if(0===e.strm.avail_in)break;if(n=c(e.strm,e.window,e.strstart+e.lookahead,i),e.lookahead+=n,e.lookahead+e.insert>=se)for(a=e.strstart-e.insert,e.ins_h=e.window[a],e.ins_h=(e.ins_h<<e.hash_shift^e.window[a+1])&e.hash_mask;e.insert&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[a+se-1])&e.hash_mask,e.prev[a&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=a,a++,e.insert--,!(e.lookahead+e.insert<se)););}while(e.lookahead<ue&&0!==e.strm.avail_in)}function d(e,t){var n=65535;for(n>e.pending_buf_size-5&&(n=e.pending_buf_size-5);;){if(e.lookahead<=1){if(h(e),0===e.lookahead&&t===M)return be;if(0===e.lookahead)break}e.strstart+=e.lookahead,e.lookahead=0;var r=e.block_start+n;if((0===e.strstart||e.strstart>=r)&&(e.lookahead=e.strstart-r,e.strstart=r,s(e,!1),0===e.strm.avail_out))return be;if(e.strstart-e.block_start>=e.w_size-ue&&(s(e,!1),0===e.strm.avail_out))return be}return e.insert=0,t===z?(s(e,!0),0===e.strm.avail_out?we:_e):(e.strstart>e.block_start&&(s(e,!1),e.strm.avail_out),be)}function p(e,t){for(var n,r;;){if(e.lookahead<ue){if(h(e),e.lookahead<ue&&t===M)return be;if(0===e.lookahead)break}if(n=0,e.lookahead>=se&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+se-1])&e.hash_mask,n=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart),0!==n&&e.strstart-n<=e.w_size-ue&&(e.match_length=f(e,n)),e.match_length>=se)if(r=R._tr_tally(e,e.strstart-e.match_start,e.match_length-se),e.lookahead-=e.match_length,e.match_length<=e.max_lazy_match&&e.lookahead>=se){e.match_length--;do{e.strstart++,e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+se-1])&e.hash_mask,n=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart;}while(0!=--e.match_length);e.strstart++;}else e.strstart+=e.match_length,e.match_length=0,e.ins_h=e.window[e.strstart],e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+1])&e.hash_mask;else r=R._tr_tally(e,0,e.window[e.strstart]),e.lookahead--,e.strstart++;if(r&&(s(e,!1),0===e.strm.avail_out))return be}return e.insert=e.strstart<se-1?e.strstart:se-1,t===z?(s(e,!0),0===e.strm.avail_out?we:_e):e.last_lit&&(s(e,!1),0===e.strm.avail_out)?be:ye}function m(e,t){for(var n,r,i;;){if(e.lookahead<ue){if(h(e),e.lookahead<ue&&t===M)return be;if(0===e.lookahead)break}if(n=0,e.lookahead>=se&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+se-1])&e.hash_mask,n=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart),e.prev_length=e.match_length,e.prev_match=e.match_start,e.match_length=se-1,0!==n&&e.prev_length<e.max_lazy_match&&e.strstart-n<=e.w_size-ue&&(e.match_length=f(e,n),e.match_length<=5&&(e.strategy===H||e.match_length===se&&e.strstart-e.match_start>4096)&&(e.match_length=se-1)),e.prev_length>=se&&e.match_length<=e.prev_length){i=e.strstart+e.lookahead-se,r=R._tr_tally(e,e.strstart-1-e.prev_match,e.prev_length-se),e.lookahead-=e.prev_length-1,e.prev_length-=2;do{++e.strstart<=i&&(e.ins_h=(e.ins_h<<e.hash_shift^e.window[e.strstart+se-1])&e.hash_mask,n=e.prev[e.strstart&e.w_mask]=e.head[e.ins_h],e.head[e.ins_h]=e.strstart);}while(0!=--e.prev_length);if(e.match_available=0,e.match_length=se-1,e.strstart++,r&&(s(e,!1),0===e.strm.avail_out))return be}else if(e.match_available){if(r=R._tr_tally(e,0,e.window[e.strstart-1]),r&&s(e,!1),e.strstart++,e.lookahead--,0===e.strm.avail_out)return be}else e.match_available=1,e.strstart++,e.lookahead--;}return e.match_available&&(r=R._tr_tally(e,0,e.window[e.strstart-1]),e.match_available=0),e.insert=e.strstart<se-1?e.strstart:se-1,t===z?(s(e,!0),0===e.strm.avail_out?we:_e):e.last_lit&&(s(e,!1),0===e.strm.avail_out)?be:ye}function g(e,t){for(var n,r,i,a,o=e.window;;){if(e.lookahead<=le){if(h(e),e.lookahead<=le&&t===M)return be;if(0===e.lookahead)break}if(e.match_length=0,e.lookahead>=se&&e.strstart>0&&(i=e.strstart-1,(r=o[i])===o[++i]&&r===o[++i]&&r===o[++i])){a=e.strstart+le;do{}while(r===o[++i]&&r===o[++i]&&r===o[++i]&&r===o[++i]&&r===o[++i]&&r===o[++i]&&r===o[++i]&&r===o[++i]&&i<a);e.match_length=le-(a-i),e.match_length>e.lookahead&&(e.match_length=e.lookahead);}if(e.match_length>=se?(n=R._tr_tally(e,1,e.match_length-se),e.lookahead-=e.match_length,e.strstart+=e.match_length,e.match_length=0):(n=R._tr_tally(e,0,e.window[e.strstart]),e.lookahead--,e.strstart++),n&&(s(e,!1),0===e.strm.avail_out))return be}return e.insert=0,t===z?(s(e,!0),0===e.strm.avail_out?we:_e):e.last_lit&&(s(e,!1),0===e.strm.avail_out)?be:ye}function v(e,t){for(var n;;){if(0===e.lookahead&&(h(e),0===e.lookahead)){if(t===M)return be;break}if(e.match_length=0,n=R._tr_tally(e,0,e.window[e.strstart]),e.lookahead--,e.strstart++,n&&(s(e,!1),0===e.strm.avail_out))return be}return e.insert=0,t===z?(s(e,!0),0===e.strm.avail_out?we:_e):e.last_lit&&(s(e,!1),0===e.strm.avail_out)?be:ye}function b(e,t,n,r,i){this.good_length=e,this.max_lazy=t,this.nice_length=n,this.max_chain=r,this.func=i;}function y(e){e.window_size=2*e.w_size,a(e.head),e.max_lazy_match=O[e.level].max_lazy,e.good_match=O[e.level].good_length,e.nice_match=O[e.level].nice_length,e.max_chain_length=O[e.level].max_chain,e.strstart=0,e.block_start=0,e.lookahead=0,e.insert=0,e.match_length=e.prev_length=se-1,e.match_available=0,e.ins_h=0;}function w(){this.strm=null,this.status=0,this.pending_buf=null,this.pending_buf_size=0,this.pending_out=0,this.pending=0,this.wrap=0,this.gzhead=null,this.gzindex=0,this.method=$,this.last_flush=-1,this.w_size=0,this.w_bits=0,this.w_mask=0,this.window=null,this.window_size=0,this.prev=null,this.head=null,this.ins_h=0,this.hash_size=0,this.hash_bits=0,this.hash_mask=0,this.hash_shift=0,this.block_start=0,this.match_length=0,this.prev_match=0,this.match_available=0,this.strstart=0,this.match_start=0,this.lookahead=0,this.prev_length=0,this.max_chain_length=0,this.max_lazy_match=0,this.level=0,this.strategy=0,this.good_match=0,this.nice_match=0,this.dyn_ltree=new I.Buf16(2*ae),this.dyn_dtree=new I.Buf16(2*(2*re+1)),this.bl_tree=new I.Buf16(2*(2*ie+1)),a(this.dyn_ltree),a(this.dyn_dtree),a(this.bl_tree),this.l_desc=null,this.d_desc=null,this.bl_desc=null,this.bl_count=new I.Buf16(oe+1),this.heap=new I.Buf16(2*ne+1),a(this.heap),this.heap_len=0,this.heap_max=0,this.depth=new I.Buf16(2*ne+1),a(this.depth),this.l_buf=0,this.lit_bufsize=0,this.last_lit=0,this.d_buf=0,this.opt_len=0,this.static_len=0,this.matches=0,this.insert=0,this.bi_buf=0,this.bi_valid=0;}function _(e){var t;return e&&e.state?(e.total_in=e.total_out=0,e.data_type=J,t=e.state,t.pending=0,t.pending_out=0,t.wrap<0&&(t.wrap=-t.wrap),t.status=t.wrap?fe:ge,e.adler=2===t.wrap?0:1,t.last_flush=M,R._tr_init(t),F):r(e,Z)}function k(e){var t=_(e);return t===F&&y(e.state),t}function x(e,t){return e&&e.state?2!==e.state.wrap?Z:(e.state.gzhead=t,F):Z}function S(e,t,n,i,a,o){if(!e)return Z;var s=1;if(t===q&&(t=6),i<0?(s=0,i=-i):i>15&&(s=2,i-=16),a<1||a>Q||n!==$||i<8||i>15||t<0||t>9||o<0||o>X)return r(e,Z);8===i&&(i=9);var l=new w;return e.state=l,l.strm=e,l.wrap=s,l.gzhead=null,l.w_bits=i,l.w_size=1<<l.w_bits,l.w_mask=l.w_size-1,l.hash_bits=a+7,l.hash_size=1<<l.hash_bits,l.hash_mask=l.hash_size-1,l.hash_shift=~~((l.hash_bits+se-1)/se),l.window=new I.Buf8(2*l.w_size),l.head=new I.Buf16(l.hash_size),l.prev=new I.Buf16(l.w_size),l.lit_bufsize=1<<a+6,l.pending_buf_size=4*l.lit_bufsize,l.pending_buf=new I.Buf8(l.pending_buf_size),l.d_buf=1*l.lit_bufsize,l.l_buf=3*l.lit_bufsize,l.level=t,l.strategy=o,l.method=n,k(e)}function E(e,t){return S(e,t,$,ee,te,G)}function T(e,t){var n,s,c,f;if(!e||!e.state||t>D||t<0)return e?r(e,Z):Z;if(s=e.state,!e.output||!e.input&&0!==e.avail_in||s.status===ve&&t!==z)return r(e,0===e.avail_out?Y:Z);if(s.strm=e,n=s.last_flush,s.last_flush=t,s.status===fe)if(2===s.wrap)e.adler=0,l(s,31),l(s,139),l(s,8),s.gzhead?(l(s,(s.gzhead.text?1:0)+(s.gzhead.hcrc?2:0)+(s.gzhead.extra?4:0)+(s.gzhead.name?8:0)+(s.gzhead.comment?16:0)),l(s,255&s.gzhead.time),l(s,s.gzhead.time>>8&255),l(s,s.gzhead.time>>16&255),l(s,s.gzhead.time>>24&255),l(s,9===s.level?2:s.strategy>=V||s.level<2?4:0),l(s,255&s.gzhead.os),s.gzhead.extra&&s.gzhead.extra.length&&(l(s,255&s.gzhead.extra.length),l(s,s.gzhead.extra.length>>8&255)),s.gzhead.hcrc&&(e.adler=j(e.adler,s.pending_buf,s.pending,0)),s.gzindex=0,s.status=he):(l(s,0),l(s,0),l(s,0),l(s,0),l(s,0),l(s,9===s.level?2:s.strategy>=V||s.level<2?4:0),l(s,ke),s.status=ge);else {var h=$+(s.w_bits-8<<4)<<8,d=-1;d=s.strategy>=V||s.level<2?0:s.level<6?1:6===s.level?2:3,h|=d<<6,0!==s.strstart&&(h|=ce),h+=31-h%31,s.status=ge,u(s,h),0!==s.strstart&&(u(s,e.adler>>>16),u(s,65535&e.adler)),e.adler=1;}if(s.status===he)if(s.gzhead.extra){for(c=s.pending;s.gzindex<(65535&s.gzhead.extra.length)&&(s.pending!==s.pending_buf_size||(s.gzhead.hcrc&&s.pending>c&&(e.adler=j(e.adler,s.pending_buf,s.pending-c,c)),o(e),c=s.pending,s.pending!==s.pending_buf_size));)l(s,255&s.gzhead.extra[s.gzindex]),s.gzindex++;s.gzhead.hcrc&&s.pending>c&&(e.adler=j(e.adler,s.pending_buf,s.pending-c,c)),s.gzindex===s.gzhead.extra.length&&(s.gzindex=0,s.status=de);}else s.status=de;if(s.status===de)if(s.gzhead.name){c=s.pending;do{if(s.pending===s.pending_buf_size&&(s.gzhead.hcrc&&s.pending>c&&(e.adler=j(e.adler,s.pending_buf,s.pending-c,c)),o(e),c=s.pending,s.pending===s.pending_buf_size)){f=1;break}f=s.gzindex<s.gzhead.name.length?255&s.gzhead.name.charCodeAt(s.gzindex++):0,l(s,f);}while(0!==f);s.gzhead.hcrc&&s.pending>c&&(e.adler=j(e.adler,s.pending_buf,s.pending-c,c)),0===f&&(s.gzindex=0,s.status=pe);}else s.status=pe;if(s.status===pe)if(s.gzhead.comment){c=s.pending;do{if(s.pending===s.pending_buf_size&&(s.gzhead.hcrc&&s.pending>c&&(e.adler=j(e.adler,s.pending_buf,s.pending-c,c)),o(e),c=s.pending,s.pending===s.pending_buf_size)){f=1;break}f=s.gzindex<s.gzhead.comment.length?255&s.gzhead.comment.charCodeAt(s.gzindex++):0,l(s,f);}while(0!==f);s.gzhead.hcrc&&s.pending>c&&(e.adler=j(e.adler,s.pending_buf,s.pending-c,c)),0===f&&(s.status=me);}else s.status=me;if(s.status===me&&(s.gzhead.hcrc?(s.pending+2>s.pending_buf_size&&o(e),s.pending+2<=s.pending_buf_size&&(l(s,255&e.adler),l(s,e.adler>>8&255),e.adler=0,s.status=ge)):s.status=ge),0!==s.pending){if(o(e),0===e.avail_out)return s.last_flush=-1,F}else if(0===e.avail_in&&i(t)<=i(n)&&t!==z)return r(e,Y);if(s.status===ve&&0!==e.avail_in)return r(e,Y);if(0!==e.avail_in||0!==s.lookahead||t!==M&&s.status!==ve){var p=s.strategy===V?v(s,t):s.strategy===K?g(s,t):O[s.level].func(s,t);if(p!==we&&p!==_e||(s.status=ve),p===be||p===we)return 0===e.avail_out&&(s.last_flush=-1),F;if(p===ye&&(t===P?R._tr_align(s):t!==D&&(R._tr_stored_block(s,0,0,!1),t===N&&(a(s.head),0===s.lookahead&&(s.strstart=0,s.block_start=0,s.insert=0))),o(e),0===e.avail_out))return s.last_flush=-1,F}return t!==z?F:s.wrap<=0?U:(2===s.wrap?(l(s,255&e.adler),l(s,e.adler>>8&255),l(s,e.adler>>16&255),l(s,e.adler>>24&255),l(s,255&e.total_in),l(s,e.total_in>>8&255),l(s,e.total_in>>16&255),l(s,e.total_in>>24&255)):(u(s,e.adler>>>16),u(s,65535&e.adler)),o(e),s.wrap>0&&(s.wrap=-s.wrap),0!==s.pending?F:U)}function C(e){var t;return e&&e.state?(t=e.state.status)!==fe&&t!==he&&t!==de&&t!==pe&&t!==me&&t!==ge&&t!==ve?r(e,Z):(e.state=null,t===ge?r(e,W):F):Z}function A(e,t){var n,r,i,o,s,l,u,c,f=t.length;if(!e||!e.state)return Z;if(n=e.state,2===(o=n.wrap)||1===o&&n.status!==fe||n.lookahead)return Z;for(1===o&&(e.adler=L(e.adler,t,f,0)),n.wrap=0,f>=n.w_size&&(0===o&&(a(n.head),n.strstart=0,n.block_start=0,n.insert=0),c=new I.Buf8(n.w_size),I.arraySet(c,t,f-n.w_size,n.w_size,0),t=c,f=n.w_size),s=e.avail_in,l=e.next_in,u=e.input,e.avail_in=f,e.next_in=0,e.input=t,h(n);n.lookahead>=se;){r=n.strstart,i=n.lookahead-(se-1);do{n.ins_h=(n.ins_h<<n.hash_shift^n.window[r+se-1])&n.hash_mask,n.prev[r&n.w_mask]=n.head[n.ins_h],n.head[n.ins_h]=r,r++;}while(--i);n.strstart=r,n.lookahead=se-1,h(n);}return n.strstart+=n.lookahead,n.block_start=n.strstart,n.insert=n.lookahead,n.lookahead=0,n.match_length=n.prev_length=se-1,n.match_available=0,e.next_in=l,e.input=u,e.avail_in=s,n.wrap=o,F}var O,I=n(4),R=n(90),L=n(40),j=n(41),B=n(23),M=0,P=1,N=3,z=4,D=5,F=0,U=1,Z=-2,W=-3,Y=-5,q=-1,H=1,V=2,K=3,X=4,G=0,J=2,$=8,Q=9,ee=15,te=8,ne=286,re=30,ie=19,ae=2*ne+1,oe=15,se=3,le=258,ue=le+se+1,ce=32,fe=42,he=69,de=73,pe=91,me=103,ge=113,ve=666,be=1,ye=2,we=3,_e=4,ke=3;O=[new b(0,0,0,0,d),new b(4,4,8,4,p),new b(4,5,16,8,p),new b(4,6,32,32,p),new b(4,4,16,16,m),new b(8,16,32,32,m),new b(8,16,128,128,m),new b(8,32,128,256,m),new b(32,128,258,1024,m),new b(32,258,258,4096,m)],t.deflateInit=E,t.deflateInit2=S,t.deflateReset=k,t.deflateResetKeep=_,t.deflateSetHeader=x,t.deflate=T,t.deflateEnd=C,t.deflateSetDictionary=A,t.deflateInfo="pako deflate (from Nodeca project)";},function(e,t,n){function r(e){for(var t=e.length;--t>=0;)e[t]=0;}function i(e,t,n,r,i){this.static_tree=e,this.extra_bits=t,this.extra_base=n,this.elems=r,this.max_length=i,this.has_stree=e&&e.length;}function a(e,t){this.dyn_tree=e,this.max_code=0,this.stat_desc=t;}function o(e){return e<256?ae[e]:ae[256+(e>>>7)]}function s(e,t){e.pending_buf[e.pending++]=255&t,e.pending_buf[e.pending++]=t>>>8&255;}function l(e,t,n){e.bi_valid>V-n?(e.bi_buf|=t<<e.bi_valid&65535,s(e,e.bi_buf),e.bi_buf=t>>V-e.bi_valid,e.bi_valid+=n-V):(e.bi_buf|=t<<e.bi_valid&65535,e.bi_valid+=n);}function u(e,t,n){l(e,n[2*t],n[2*t+1]);}function c(e,t){var n=0;do{n|=1&e,e>>>=1,n<<=1;}while(--t>0);return n>>>1}function f(e){16===e.bi_valid?(s(e,e.bi_buf),e.bi_buf=0,e.bi_valid=0):e.bi_valid>=8&&(e.pending_buf[e.pending++]=255&e.bi_buf,e.bi_buf>>=8,e.bi_valid-=8);}function h(e,t){var n,r,i,a,o,s,l=t.dyn_tree,u=t.max_code,c=t.stat_desc.static_tree,f=t.stat_desc.has_stree,h=t.stat_desc.extra_bits,d=t.stat_desc.extra_base,p=t.stat_desc.max_length,m=0;for(a=0;a<=H;a++)e.bl_count[a]=0;for(l[2*e.heap[e.heap_max]+1]=0,n=e.heap_max+1;n<q;n++)r=e.heap[n],a=l[2*l[2*r+1]+1]+1,a>p&&(a=p,m++),l[2*r+1]=a,r>u||(e.bl_count[a]++,o=0,r>=d&&(o=h[r-d]),s=l[2*r],e.opt_len+=s*(a+o),f&&(e.static_len+=s*(c[2*r+1]+o)));if(0!==m){do{for(a=p-1;0===e.bl_count[a];)a--;e.bl_count[a]--,e.bl_count[a+1]+=2,e.bl_count[p]--,m-=2;}while(m>0);for(a=p;0!==a;a--)for(r=e.bl_count[a];0!==r;)(i=e.heap[--n])>u||(l[2*i+1]!==a&&(e.opt_len+=(a-l[2*i+1])*l[2*i],l[2*i+1]=a),r--);}}function d(e,t,n){var r,i,a=new Array(H+1),o=0;for(r=1;r<=H;r++)a[r]=o=o+n[r-1]<<1;for(i=0;i<=t;i++){var s=e[2*i+1];0!==s&&(e[2*i]=c(a[s]++,s));}}function p(){var e,t,n,r,a,o=new Array(H+1);for(n=0,r=0;r<F-1;r++)for(se[r]=n,e=0;e<1<<Q[r];e++)oe[n++]=r;for(oe[n-1]=r,a=0,r=0;r<16;r++)for(le[r]=a,e=0;e<1<<ee[r];e++)ae[a++]=r;for(a>>=7;r<W;r++)for(le[r]=a<<7,e=0;e<1<<ee[r]-7;e++)ae[256+a++]=r;for(t=0;t<=H;t++)o[t]=0;for(e=0;e<=143;)re[2*e+1]=8,e++,o[8]++;for(;e<=255;)re[2*e+1]=9,e++,o[9]++;for(;e<=279;)re[2*e+1]=7,e++,o[7]++;for(;e<=287;)re[2*e+1]=8,e++,o[8]++;for(d(re,Z+1,o),e=0;e<W;e++)ie[2*e+1]=5,ie[2*e]=c(e,5);ue=new i(re,Q,U+1,Z,H),ce=new i(ie,ee,0,W,H),fe=new i(new Array(0),te,0,Y,K);}function m(e){var t;for(t=0;t<Z;t++)e.dyn_ltree[2*t]=0;for(t=0;t<W;t++)e.dyn_dtree[2*t]=0;for(t=0;t<Y;t++)e.bl_tree[2*t]=0;e.dyn_ltree[2*X]=1,e.opt_len=e.static_len=0,e.last_lit=e.matches=0;}function g(e){e.bi_valid>8?s(e,e.bi_buf):e.bi_valid>0&&(e.pending_buf[e.pending++]=e.bi_buf),e.bi_buf=0,e.bi_valid=0;}function v(e,t,n,r){g(e),r&&(s(e,n),s(e,~n)),L.arraySet(e.pending_buf,e.window,t,n,e.pending),e.pending+=n;}function b(e,t,n,r){var i=2*t,a=2*n;return e[i]<e[a]||e[i]===e[a]&&r[t]<=r[n]}function y(e,t,n){for(var r=e.heap[n],i=n<<1;i<=e.heap_len&&(i<e.heap_len&&b(t,e.heap[i+1],e.heap[i],e.depth)&&i++,!b(t,r,e.heap[i],e.depth));)e.heap[n]=e.heap[i],n=i,i<<=1;e.heap[n]=r;}function w(e,t,n){var r,i,a,s,c=0;if(0!==e.last_lit)do{r=e.pending_buf[e.d_buf+2*c]<<8|e.pending_buf[e.d_buf+2*c+1],i=e.pending_buf[e.l_buf+c],c++,0===r?u(e,i,t):(a=oe[i],u(e,a+U+1,t),s=Q[a],0!==s&&(i-=se[a],l(e,i,s)),r--,a=o(r),u(e,a,n),0!==(s=ee[a])&&(r-=le[a],l(e,r,s)));}while(c<e.last_lit);u(e,X,t);}function _(e,t){var n,r,i,a=t.dyn_tree,o=t.stat_desc.static_tree,s=t.stat_desc.has_stree,l=t.stat_desc.elems,u=-1;for(e.heap_len=0,e.heap_max=q,n=0;n<l;n++)0!==a[2*n]?(e.heap[++e.heap_len]=u=n,e.depth[n]=0):a[2*n+1]=0;for(;e.heap_len<2;)i=e.heap[++e.heap_len]=u<2?++u:0,a[2*i]=1,e.depth[i]=0,e.opt_len--,s&&(e.static_len-=o[2*i+1]);for(t.max_code=u,n=e.heap_len>>1;n>=1;n--)y(e,a,n);i=l;do{n=e.heap[1],e.heap[1]=e.heap[e.heap_len--],y(e,a,1),r=e.heap[1],e.heap[--e.heap_max]=n,e.heap[--e.heap_max]=r,a[2*i]=a[2*n]+a[2*r],e.depth[i]=(e.depth[n]>=e.depth[r]?e.depth[n]:e.depth[r])+1,a[2*n+1]=a[2*r+1]=i,e.heap[1]=i++,y(e,a,1);}while(e.heap_len>=2);e.heap[--e.heap_max]=e.heap[1],h(e,t),d(a,u,e.bl_count);}function k(e,t,n){var r,i,a=-1,o=t[1],s=0,l=7,u=4;for(0===o&&(l=138,u=3),t[2*(n+1)+1]=65535,r=0;r<=n;r++)i=o,o=t[2*(r+1)+1],++s<l&&i===o||(s<u?e.bl_tree[2*i]+=s:0!==i?(i!==a&&e.bl_tree[2*i]++,e.bl_tree[2*G]++):s<=10?e.bl_tree[2*J]++:e.bl_tree[2*$]++,s=0,a=i,0===o?(l=138,u=3):i===o?(l=6,u=3):(l=7,u=4));}function x(e,t,n){var r,i,a=-1,o=t[1],s=0,c=7,f=4;for(0===o&&(c=138,f=3),r=0;r<=n;r++)if(i=o,o=t[2*(r+1)+1],!(++s<c&&i===o)){if(s<f)do{u(e,i,e.bl_tree);}while(0!=--s);else 0!==i?(i!==a&&(u(e,i,e.bl_tree),s--),u(e,G,e.bl_tree),l(e,s-3,2)):s<=10?(u(e,J,e.bl_tree),l(e,s-3,3)):(u(e,$,e.bl_tree),l(e,s-11,7));s=0,a=i,0===o?(c=138,f=3):i===o?(c=6,f=3):(c=7,f=4);}}function S(e){var t;for(k(e,e.dyn_ltree,e.l_desc.max_code),k(e,e.dyn_dtree,e.d_desc.max_code),_(e,e.bl_desc),t=Y-1;t>=3&&0===e.bl_tree[2*ne[t]+1];t--);return e.opt_len+=3*(t+1)+5+5+4,t}function E(e,t,n,r){var i;for(l(e,t-257,5),l(e,n-1,5),l(e,r-4,4),i=0;i<r;i++)l(e,e.bl_tree[2*ne[i]+1],3);x(e,e.dyn_ltree,t-1),x(e,e.dyn_dtree,n-1);}function T(e){var t,n=4093624447;for(t=0;t<=31;t++,n>>>=1)if(1&n&&0!==e.dyn_ltree[2*t])return B;if(0!==e.dyn_ltree[18]||0!==e.dyn_ltree[20]||0!==e.dyn_ltree[26])return M;for(t=32;t<U;t++)if(0!==e.dyn_ltree[2*t])return M;return B}function C(e){he||(p(),he=!0),e.l_desc=new a(e.dyn_ltree,ue),e.d_desc=new a(e.dyn_dtree,ce),e.bl_desc=new a(e.bl_tree,fe),e.bi_buf=0,e.bi_valid=0,m(e);}function A(e,t,n,r){l(e,(N<<1)+(r?1:0),3),v(e,t,n,!0);}function O(e){l(e,z<<1,3),u(e,X,re),f(e);}function I(e,t,n,r){var i,a,o=0;e.level>0?(e.strm.data_type===P&&(e.strm.data_type=T(e)),_(e,e.l_desc),_(e,e.d_desc),o=S(e),i=e.opt_len+3+7>>>3,(a=e.static_len+3+7>>>3)<=i&&(i=a)):i=a=n+5,n+4<=i&&-1!==t?A(e,t,n,r):e.strategy===j||a===i?(l(e,(z<<1)+(r?1:0),3),w(e,re,ie)):(l(e,(D<<1)+(r?1:0),3),E(e,e.l_desc.max_code+1,e.d_desc.max_code+1,o+1),w(e,e.dyn_ltree,e.dyn_dtree)),m(e),r&&g(e);}function R(e,t,n){return e.pending_buf[e.d_buf+2*e.last_lit]=t>>>8&255,e.pending_buf[e.d_buf+2*e.last_lit+1]=255&t,e.pending_buf[e.l_buf+e.last_lit]=255&n,e.last_lit++,0===t?e.dyn_ltree[2*n]++:(e.matches++,t--,e.dyn_ltree[2*(oe[n]+U+1)]++,e.dyn_dtree[2*o(t)]++),e.last_lit===e.lit_bufsize-1}var L=n(4),j=4,B=0,M=1,P=2,N=0,z=1,D=2,F=29,U=256,Z=U+1+F,W=30,Y=19,q=2*Z+1,H=15,V=16,K=7,X=256,G=16,J=17,$=18,Q=[0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,0],ee=[0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13],te=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,3,7],ne=[16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15],re=new Array(2*(Z+2));r(re);var ie=new Array(2*W);r(ie);var ae=new Array(512);r(ae);var oe=new Array(256);r(oe);var se=new Array(F);r(se);var le=new Array(W);r(le);var ue,ce,fe,he=!1;t._tr_init=C,t._tr_stored_block=A,t._tr_flush_block=I,t._tr_tally=R,t._tr_align=O;},function(e,t,n){function r(e){if(!(this instanceof r))return new r(e);this.options=s.assign({chunkSize:16384,windowBits:0,to:""},e||{});var t=this.options;t.raw&&t.windowBits>=0&&t.windowBits<16&&(t.windowBits=-t.windowBits,0===t.windowBits&&(t.windowBits=-15)),!(t.windowBits>=0&&t.windowBits<16)||e&&e.windowBits||(t.windowBits+=32),t.windowBits>15&&t.windowBits<48&&0==(15&t.windowBits)&&(t.windowBits|=15),this.err=0,this.msg="",this.ended=!1,this.chunks=[],this.strm=new f,this.strm.avail_out=0;var n=o.inflateInit2(this.strm,t.windowBits);if(n!==u.Z_OK)throw new Error(c[n]);this.header=new h,o.inflateGetHeader(this.strm,this.header);}function i(e,t){var n=new r(t);if(n.push(e,!0),n.err)throw n.msg||c[n.err];return n.result}function a(e,t){return t=t||{},t.raw=!0,i(e,t)}var o=n(92),s=n(4),l=n(42),u=n(44),c=n(23),f=n(43),h=n(95),d=Object.prototype.toString;r.prototype.push=function(e,t){var n,r,i,a,c,f,h=this.strm,p=this.options.chunkSize,m=this.options.dictionary,g=!1;if(this.ended)return !1;r=t===~~t?t:!0===t?u.Z_FINISH:u.Z_NO_FLUSH,"string"==typeof e?h.input=l.binstring2buf(e):"[object ArrayBuffer]"===d.call(e)?h.input=new Uint8Array(e):h.input=e,h.next_in=0,h.avail_in=h.input.length;do{if(0===h.avail_out&&(h.output=new s.Buf8(p),h.next_out=0,h.avail_out=p),n=o.inflate(h,u.Z_NO_FLUSH),n===u.Z_NEED_DICT&&m&&(f="string"==typeof m?l.string2buf(m):"[object ArrayBuffer]"===d.call(m)?new Uint8Array(m):m,n=o.inflateSetDictionary(this.strm,f)),n===u.Z_BUF_ERROR&&!0===g&&(n=u.Z_OK,g=!1),n!==u.Z_STREAM_END&&n!==u.Z_OK)return this.onEnd(n),this.ended=!0,!1;h.next_out&&(0!==h.avail_out&&n!==u.Z_STREAM_END&&(0!==h.avail_in||r!==u.Z_FINISH&&r!==u.Z_SYNC_FLUSH)||("string"===this.options.to?(i=l.utf8border(h.output,h.next_out),a=h.next_out-i,c=l.buf2string(h.output,i),h.next_out=a,h.avail_out=p-a,a&&s.arraySet(h.output,h.output,i,a,0),this.onData(c)):this.onData(s.shrinkBuf(h.output,h.next_out)))),0===h.avail_in&&0===h.avail_out&&(g=!0);}while((h.avail_in>0||0===h.avail_out)&&n!==u.Z_STREAM_END);return n===u.Z_STREAM_END&&(r=u.Z_FINISH),r===u.Z_FINISH?(n=o.inflateEnd(this.strm),this.onEnd(n),this.ended=!0,n===u.Z_OK):r!==u.Z_SYNC_FLUSH||(this.onEnd(u.Z_OK),h.avail_out=0,!0)},r.prototype.onData=function(e){this.chunks.push(e);},r.prototype.onEnd=function(e){e===u.Z_OK&&("string"===this.options.to?this.result=this.chunks.join(""):this.result=s.flattenChunks(this.chunks)),this.chunks=[],this.err=e,this.msg=this.strm.msg;},t.Inflate=r,t.inflate=i,t.inflateRaw=a,t.ungzip=i;},function(e,t,n){function r(e){return (e>>>24&255)+(e>>>8&65280)+((65280&e)<<8)+((255&e)<<24)}function i(){this.mode=0,this.last=!1,this.wrap=0,this.havedict=!1,this.flags=0,this.dmax=0,this.check=0,this.total=0,this.head=null,this.wbits=0,this.wsize=0,this.whave=0,this.wnext=0,this.window=null,this.hold=0,this.bits=0,this.length=0,this.offset=0,this.extra=0,this.lencode=null,this.distcode=null,this.lenbits=0,this.distbits=0,this.ncode=0,this.nlen=0,this.ndist=0,this.have=0,this.next=null,this.lens=new b.Buf16(320),this.work=new b.Buf16(288),this.lendyn=null,this.distdyn=null,this.sane=0,this.back=0,this.was=0;}function a(e){var t;return e&&e.state?(t=e.state,e.total_in=e.total_out=t.total=0,e.msg="",t.wrap&&(e.adler=1&t.wrap),t.mode=N,t.last=0,t.havedict=0,t.dmax=32768,t.head=null,t.hold=0,t.bits=0,t.lencode=t.lendyn=new b.Buf32(me),t.distcode=t.distdyn=new b.Buf32(ge),t.sane=1,t.back=-1,O):L}function o(e){var t;return e&&e.state?(t=e.state,t.wsize=0,t.whave=0,t.wnext=0,a(e)):L}function s(e,t){var n,r;return e&&e.state?(r=e.state,t<0?(n=0,t=-t):(n=1+(t>>4),t<48&&(t&=15)),t&&(t<8||t>15)?L:(null!==r.window&&r.wbits!==t&&(r.window=null),r.wrap=n,r.wbits=t,o(e))):L}function l(e,t){var n,r;return e?(r=new i,e.state=r,r.window=null,n=s(e,t),n!==O&&(e.state=null),n):L}function u(e){return l(e,ve)}function c(e){if(be){var t;for(g=new b.Buf32(512),v=new b.Buf32(32),t=0;t<144;)e.lens[t++]=8;for(;t<256;)e.lens[t++]=9;for(;t<280;)e.lens[t++]=7;for(;t<288;)e.lens[t++]=8;for(k(S,e.lens,0,288,g,0,e.work,{bits:9}),t=0;t<32;)e.lens[t++]=5;k(E,e.lens,0,32,v,0,e.work,{bits:5}),be=!1;}e.lencode=g,e.lenbits=9,e.distcode=v,e.distbits=5;}function f(e,t,n,r){var i,a=e.state;return null===a.window&&(a.wsize=1<<a.wbits,a.wnext=0,a.whave=0,a.window=new b.Buf8(a.wsize)),r>=a.wsize?(b.arraySet(a.window,t,n-a.wsize,a.wsize,0),a.wnext=0,a.whave=a.wsize):(i=a.wsize-a.wnext,i>r&&(i=r),b.arraySet(a.window,t,n-r,i,a.wnext),r-=i,r?(b.arraySet(a.window,t,n-r,r,0),a.wnext=r,a.whave=a.wsize):(a.wnext+=i,a.wnext===a.wsize&&(a.wnext=0),a.whave<a.wsize&&(a.whave+=i))),0}function h(e,t){var n,i,a,o,s,l,u,h,d,p,m,g,v,me,ge,ve,be,ye,we,_e,ke,xe,Se,Ee,Te=0,Ce=new b.Buf8(4),Ae=[16,17,18,0,8,7,9,6,10,5,11,4,12,3,13,2,14,1,15];if(!e||!e.state||!e.output||!e.input&&0!==e.avail_in)return L;n=e.state,n.mode===K&&(n.mode=X),s=e.next_out,a=e.output,u=e.avail_out,o=e.next_in,i=e.input,l=e.avail_in,h=n.hold,d=n.bits,p=l,m=u,xe=O;e:for(;;)switch(n.mode){case N:if(0===n.wrap){n.mode=X;break}for(;d<16;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(2&n.wrap&&35615===h){n.check=0,Ce[0]=255&h,Ce[1]=h>>>8&255,n.check=w(n.check,Ce,2,0),h=0,d=0,n.mode=z;break}if(n.flags=0,n.head&&(n.head.done=!1),!(1&n.wrap)||(((255&h)<<8)+(h>>8))%31){e.msg="incorrect header check",n.mode=he;break}if((15&h)!==P){e.msg="unknown compression method",n.mode=he;break}if(h>>>=4,d-=4,ke=8+(15&h),0===n.wbits)n.wbits=ke;else if(ke>n.wbits){e.msg="invalid window size",n.mode=he;break}n.dmax=1<<ke,e.adler=n.check=1,n.mode=512&h?H:K,h=0,d=0;break;case z:for(;d<16;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(n.flags=h,(255&n.flags)!==P){e.msg="unknown compression method",n.mode=he;break}if(57344&n.flags){e.msg="unknown header flags set",n.mode=he;break}n.head&&(n.head.text=h>>8&1),512&n.flags&&(Ce[0]=255&h,Ce[1]=h>>>8&255,n.check=w(n.check,Ce,2,0)),h=0,d=0,n.mode=D;case D:for(;d<32;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}n.head&&(n.head.time=h),512&n.flags&&(Ce[0]=255&h,Ce[1]=h>>>8&255,Ce[2]=h>>>16&255,Ce[3]=h>>>24&255,n.check=w(n.check,Ce,4,0)),h=0,d=0,n.mode=F;case F:for(;d<16;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}n.head&&(n.head.xflags=255&h,n.head.os=h>>8),512&n.flags&&(Ce[0]=255&h,Ce[1]=h>>>8&255,n.check=w(n.check,Ce,2,0)),h=0,d=0,n.mode=U;case U:if(1024&n.flags){for(;d<16;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}n.length=h,n.head&&(n.head.extra_len=h),512&n.flags&&(Ce[0]=255&h,Ce[1]=h>>>8&255,n.check=w(n.check,Ce,2,0)),h=0,d=0;}else n.head&&(n.head.extra=null);n.mode=Z;case Z:if(1024&n.flags&&(g=n.length,g>l&&(g=l),g&&(n.head&&(ke=n.head.extra_len-n.length,n.head.extra||(n.head.extra=new Array(n.head.extra_len)),b.arraySet(n.head.extra,i,o,g,ke)),512&n.flags&&(n.check=w(n.check,i,g,o)),l-=g,o+=g,n.length-=g),n.length))break e;n.length=0,n.mode=W;case W:if(2048&n.flags){if(0===l)break e;g=0;do{ke=i[o+g++],n.head&&ke&&n.length<65536&&(n.head.name+=String.fromCharCode(ke));}while(ke&&g<l);if(512&n.flags&&(n.check=w(n.check,i,g,o)),l-=g,o+=g,ke)break e}else n.head&&(n.head.name=null);n.length=0,n.mode=Y;case Y:if(4096&n.flags){if(0===l)break e;g=0;do{ke=i[o+g++],n.head&&ke&&n.length<65536&&(n.head.comment+=String.fromCharCode(ke));}while(ke&&g<l);if(512&n.flags&&(n.check=w(n.check,i,g,o)),l-=g,o+=g,ke)break e}else n.head&&(n.head.comment=null);n.mode=q;case q:if(512&n.flags){for(;d<16;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(h!==(65535&n.check)){e.msg="header crc mismatch",n.mode=he;break}h=0,d=0;}n.head&&(n.head.hcrc=n.flags>>9&1,n.head.done=!0),e.adler=n.check=0,n.mode=K;break;case H:for(;d<32;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}e.adler=n.check=r(h),h=0,d=0,n.mode=V;case V:if(0===n.havedict)return e.next_out=s,e.avail_out=u,e.next_in=o,e.avail_in=l,n.hold=h,n.bits=d,R;e.adler=n.check=1,n.mode=K;case K:if(t===C||t===A)break e;case X:if(n.last){h>>>=7&d,d-=7&d,n.mode=ue;break}for(;d<3;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}switch(n.last=1&h,h>>>=1,d-=1,3&h){case 0:n.mode=G;break;case 1:if(c(n),n.mode=ne,t===A){h>>>=2,d-=2;break e}break;case 2:n.mode=Q;break;case 3:e.msg="invalid block type",n.mode=he;}h>>>=2,d-=2;break;case G:for(h>>>=7&d,d-=7&d;d<32;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if((65535&h)!=(h>>>16^65535)){e.msg="invalid stored block lengths",n.mode=he;break}if(n.length=65535&h,h=0,d=0,n.mode=J,t===A)break e;case J:n.mode=$;case $:if(g=n.length){if(g>l&&(g=l),g>u&&(g=u),0===g)break e;b.arraySet(a,i,o,g,s),l-=g,o+=g,u-=g,s+=g,n.length-=g;break}n.mode=K;break;case Q:for(;d<14;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(n.nlen=257+(31&h),h>>>=5,d-=5,n.ndist=1+(31&h),h>>>=5,d-=5,n.ncode=4+(15&h),h>>>=4,d-=4,n.nlen>286||n.ndist>30){e.msg="too many length or distance symbols",n.mode=he;break}n.have=0,n.mode=ee;case ee:for(;n.have<n.ncode;){for(;d<3;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}n.lens[Ae[n.have++]]=7&h,h>>>=3,d-=3;}for(;n.have<19;)n.lens[Ae[n.have++]]=0;if(n.lencode=n.lendyn,n.lenbits=7,Se={bits:n.lenbits},xe=k(x,n.lens,0,19,n.lencode,0,n.work,Se),n.lenbits=Se.bits,xe){e.msg="invalid code lengths set",n.mode=he;break}n.have=0,n.mode=te;case te:for(;n.have<n.nlen+n.ndist;){for(;Te=n.lencode[h&(1<<n.lenbits)-1],ge=Te>>>24,ve=Te>>>16&255,be=65535&Te,!(ge<=d);){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(be<16)h>>>=ge,d-=ge,n.lens[n.have++]=be;else {if(16===be){for(Ee=ge+2;d<Ee;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(h>>>=ge,d-=ge,0===n.have){e.msg="invalid bit length repeat",n.mode=he;break}ke=n.lens[n.have-1],g=3+(3&h),h>>>=2,d-=2;}else if(17===be){for(Ee=ge+3;d<Ee;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}h>>>=ge,d-=ge,ke=0,g=3+(7&h),h>>>=3,d-=3;}else {for(Ee=ge+7;d<Ee;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}h>>>=ge,d-=ge,ke=0,g=11+(127&h),h>>>=7,d-=7;}if(n.have+g>n.nlen+n.ndist){e.msg="invalid bit length repeat",n.mode=he;break}for(;g--;)n.lens[n.have++]=ke;}}if(n.mode===he)break;if(0===n.lens[256]){e.msg="invalid code -- missing end-of-block",n.mode=he;break}if(n.lenbits=9,Se={bits:n.lenbits},xe=k(S,n.lens,0,n.nlen,n.lencode,0,n.work,Se),n.lenbits=Se.bits,xe){e.msg="invalid literal/lengths set",n.mode=he;break}if(n.distbits=6,n.distcode=n.distdyn,Se={bits:n.distbits},xe=k(E,n.lens,n.nlen,n.ndist,n.distcode,0,n.work,Se),n.distbits=Se.bits,xe){e.msg="invalid distances set",n.mode=he;break}if(n.mode=ne,t===A)break e;case ne:n.mode=re;case re:if(l>=6&&u>=258){e.next_out=s,e.avail_out=u,e.next_in=o,e.avail_in=l,n.hold=h,n.bits=d,_(e,m),s=e.next_out,a=e.output,u=e.avail_out,o=e.next_in,i=e.input,l=e.avail_in,h=n.hold,d=n.bits,n.mode===K&&(n.back=-1);break}for(n.back=0;Te=n.lencode[h&(1<<n.lenbits)-1],ge=Te>>>24,ve=Te>>>16&255,be=65535&Te,!(ge<=d);){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(ve&&0==(240&ve)){for(ye=ge,we=ve,_e=be;Te=n.lencode[_e+((h&(1<<ye+we)-1)>>ye)],ge=Te>>>24,ve=Te>>>16&255,be=65535&Te,!(ye+ge<=d);){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}h>>>=ye,d-=ye,n.back+=ye;}if(h>>>=ge,d-=ge,n.back+=ge,n.length=be,0===ve){n.mode=le;break}if(32&ve){n.back=-1,n.mode=K;break}if(64&ve){e.msg="invalid literal/length code",n.mode=he;break}n.extra=15&ve,n.mode=ie;case ie:if(n.extra){for(Ee=n.extra;d<Ee;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}n.length+=h&(1<<n.extra)-1,h>>>=n.extra,d-=n.extra,n.back+=n.extra;}n.was=n.length,n.mode=ae;case ae:for(;Te=n.distcode[h&(1<<n.distbits)-1],ge=Te>>>24,ve=Te>>>16&255,be=65535&Te,!(ge<=d);){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(0==(240&ve)){for(ye=ge,we=ve,_e=be;Te=n.distcode[_e+((h&(1<<ye+we)-1)>>ye)],ge=Te>>>24,ve=Te>>>16&255,be=65535&Te,!(ye+ge<=d);){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}h>>>=ye,d-=ye,n.back+=ye;}if(h>>>=ge,d-=ge,n.back+=ge,64&ve){e.msg="invalid distance code",n.mode=he;break}n.offset=be,n.extra=15&ve,n.mode=oe;case oe:if(n.extra){for(Ee=n.extra;d<Ee;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}n.offset+=h&(1<<n.extra)-1,h>>>=n.extra,d-=n.extra,n.back+=n.extra;}if(n.offset>n.dmax){e.msg="invalid distance too far back",n.mode=he;break}n.mode=se;case se:if(0===u)break e;if(g=m-u,n.offset>g){if((g=n.offset-g)>n.whave&&n.sane){e.msg="invalid distance too far back",n.mode=he;break}g>n.wnext?(g-=n.wnext,v=n.wsize-g):v=n.wnext-g,g>n.length&&(g=n.length),me=n.window;}else me=a,v=s-n.offset,g=n.length;g>u&&(g=u),u-=g,n.length-=g;do{a[s++]=me[v++];}while(--g);0===n.length&&(n.mode=re);break;case le:if(0===u)break e;a[s++]=n.length,u--,n.mode=re;break;case ue:if(n.wrap){for(;d<32;){if(0===l)break e;l--,h|=i[o++]<<d,d+=8;}if(m-=u,e.total_out+=m,n.total+=m,m&&(e.adler=n.check=n.flags?w(n.check,a,m,s-m):y(n.check,a,m,s-m)),m=u,(n.flags?h:r(h))!==n.check){e.msg="incorrect data check",n.mode=he;break}h=0,d=0;}n.mode=ce;case ce:if(n.wrap&&n.flags){for(;d<32;){if(0===l)break e;l--,h+=i[o++]<<d,d+=8;}if(h!==(4294967295&n.total)){e.msg="incorrect length check",n.mode=he;break}h=0,d=0;}n.mode=fe;case fe:xe=I;break e;case he:xe=j;break e;case de:return B;case pe:default:return L}return e.next_out=s,e.avail_out=u,e.next_in=o,e.avail_in=l,n.hold=h,n.bits=d,(n.wsize||m!==e.avail_out&&n.mode<he&&(n.mode<ue||t!==T))&&f(e,e.output,e.next_out,m-e.avail_out)?(n.mode=de,B):(p-=e.avail_in,m-=e.avail_out,e.total_in+=p,e.total_out+=m,n.total+=m,n.wrap&&m&&(e.adler=n.check=n.flags?w(n.check,a,m,e.next_out-m):y(n.check,a,m,e.next_out-m)),e.data_type=n.bits+(n.last?64:0)+(n.mode===K?128:0)+(n.mode===ne||n.mode===J?256:0),(0===p&&0===m||t===T)&&xe===O&&(xe=M),xe)}function d(e){if(!e||!e.state)return L;var t=e.state;return t.window&&(t.window=null),e.state=null,O}function p(e,t){var n;return e&&e.state?(n=e.state,0==(2&n.wrap)?L:(n.head=t,t.done=!1,O)):L}function m(e,t){var n,r,i=t.length;return e&&e.state?(n=e.state,0!==n.wrap&&n.mode!==V?L:n.mode===V&&(r=1,(r=y(r,t,i,0))!==n.check)?j:f(e,t,i,i)?(n.mode=de,B):(n.havedict=1,O)):L}var g,v,b=n(4),y=n(40),w=n(41),_=n(93),k=n(94),x=0,S=1,E=2,T=4,C=5,A=6,O=0,I=1,R=2,L=-2,j=-3,B=-4,M=-5,P=8,N=1,z=2,D=3,F=4,U=5,Z=6,W=7,Y=8,q=9,H=10,V=11,K=12,X=13,G=14,J=15,$=16,Q=17,ee=18,te=19,ne=20,re=21,ie=22,ae=23,oe=24,se=25,le=26,ue=27,ce=28,fe=29,he=30,de=31,pe=32,me=852,ge=592,ve=15,be=!0;t.inflateReset=o,t.inflateReset2=s,t.inflateResetKeep=a,t.inflateInit=u,t.inflateInit2=l,t.inflate=h,t.inflateEnd=d,t.inflateGetHeader=p,t.inflateSetDictionary=m,t.inflateInfo="pako inflate (from Nodeca project)";},function(e,t,n){e.exports=function(e,t){var n,r,i,a,o,s,l,u,c,f,h,d,p,m,g,v,b,y,w,_,k,x,S,E,T;n=e.state,r=e.next_in,E=e.input,i=r+(e.avail_in-5),a=e.next_out,T=e.output,o=a-(t-e.avail_out),s=a+(e.avail_out-257),l=n.dmax,u=n.wsize,c=n.whave,f=n.wnext,h=n.window,d=n.hold,p=n.bits,m=n.lencode,g=n.distcode,v=(1<<n.lenbits)-1,b=(1<<n.distbits)-1;e:do{p<15&&(d+=E[r++]<<p,p+=8,d+=E[r++]<<p,p+=8),y=m[d&v];t:for(;;){if(w=y>>>24,d>>>=w,p-=w,0==(w=y>>>16&255))T[a++]=65535&y;else {if(!(16&w)){if(0==(64&w)){y=m[(65535&y)+(d&(1<<w)-1)];continue t}if(32&w){n.mode=12;break e}e.msg="invalid literal/length code",n.mode=30;break e}_=65535&y,w&=15,w&&(p<w&&(d+=E[r++]<<p,p+=8),_+=d&(1<<w)-1,d>>>=w,p-=w),p<15&&(d+=E[r++]<<p,p+=8,d+=E[r++]<<p,p+=8),y=g[d&b];n:for(;;){if(w=y>>>24,d>>>=w,p-=w,!(16&(w=y>>>16&255))){if(0==(64&w)){y=g[(65535&y)+(d&(1<<w)-1)];continue n}e.msg="invalid distance code",n.mode=30;break e}if(k=65535&y,w&=15,p<w&&(d+=E[r++]<<p,(p+=8)<w&&(d+=E[r++]<<p,p+=8)),(k+=d&(1<<w)-1)>l){e.msg="invalid distance too far back",n.mode=30;break e}if(d>>>=w,p-=w,w=a-o,k>w){if((w=k-w)>c&&n.sane){e.msg="invalid distance too far back",n.mode=30;break e}if(x=0,S=h,0===f){if(x+=u-w,w<_){_-=w;do{T[a++]=h[x++];}while(--w);x=a-k,S=T;}}else if(f<w){if(x+=u+f-w,(w-=f)<_){_-=w;do{T[a++]=h[x++];}while(--w);if(x=0,f<_){w=f,_-=w;do{T[a++]=h[x++];}while(--w);x=a-k,S=T;}}}else if(x+=f-w,w<_){_-=w;do{T[a++]=h[x++];}while(--w);x=a-k,S=T;}for(;_>2;)T[a++]=S[x++],T[a++]=S[x++],T[a++]=S[x++],_-=3;_&&(T[a++]=S[x++],_>1&&(T[a++]=S[x++]));}else {x=a-k;do{T[a++]=T[x++],T[a++]=T[x++],T[a++]=T[x++],_-=3;}while(_>2);_&&(T[a++]=T[x++],_>1&&(T[a++]=T[x++]));}break}}break}}while(r<i&&a<s);_=p>>3,r-=_,p-=_<<3,d&=(1<<p)-1,e.next_in=r,e.next_out=a,e.avail_in=r<i?i-r+5:5-(r-i),e.avail_out=a<s?s-a+257:257-(a-s),n.hold=d,n.bits=p;};},function(e,t,n){var r=n(4),i=[3,4,5,6,7,8,9,10,11,13,15,17,19,23,27,31,35,43,51,59,67,83,99,115,131,163,195,227,258,0,0],a=[16,16,16,16,16,16,16,16,17,17,17,17,18,18,18,18,19,19,19,19,20,20,20,20,21,21,21,21,16,72,78],o=[1,2,3,4,5,7,9,13,17,25,33,49,65,97,129,193,257,385,513,769,1025,1537,2049,3073,4097,6145,8193,12289,16385,24577,0,0],s=[16,16,16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,64,64];e.exports=function(e,t,n,l,u,c,f,h){var d,p,m,g,v,b,y,w,_,k=h.bits,x=0,S=0,E=0,T=0,C=0,A=0,O=0,I=0,R=0,L=0,j=null,B=0,M=new r.Buf16(16),P=new r.Buf16(16),N=null,z=0;for(x=0;x<=15;x++)M[x]=0;for(S=0;S<l;S++)M[t[n+S]]++;for(C=k,T=15;T>=1&&0===M[T];T--);if(C>T&&(C=T),0===T)return u[c++]=20971520,u[c++]=20971520,h.bits=1,0;for(E=1;E<T&&0===M[E];E++);for(C<E&&(C=E),I=1,x=1;x<=15;x++)if(I<<=1,(I-=M[x])<0)return -1;if(I>0&&(0===e||1!==T))return -1;for(P[1]=0,x=1;x<15;x++)P[x+1]=P[x]+M[x];for(S=0;S<l;S++)0!==t[n+S]&&(f[P[t[n+S]]++]=S);if(0===e?(j=N=f,b=19):1===e?(j=i,B-=257,N=a,z-=257,b=256):(j=o,N=s,b=-1),L=0,S=0,x=E,v=c,A=C,O=0,m=-1,R=1<<C,g=R-1,1===e&&R>852||2===e&&R>592)return 1;for(;;){y=x-O,f[S]<b?(w=0,_=f[S]):f[S]>b?(w=N[z+f[S]],_=j[B+f[S]]):(w=96,_=0),d=1<<x-O,p=1<<A,E=p;do{p-=d,u[v+(L>>O)+p]=y<<24|w<<16|_|0;}while(0!==p);for(d=1<<x-1;L&d;)d>>=1;if(0!==d?(L&=d-1,L+=d):L=0,S++,0==--M[x]){if(x===T)break;x=t[n+f[S]];}if(x>C&&(L&g)!==m){for(0===O&&(O=C),v+=E,A=x-O,I=1<<A;A+O<T&&!((I-=M[A+O])<=0);)A++,I<<=1;if(R+=1<<A,1===e&&R>852||2===e&&R>592)return 1;m=L&g,u[m]=C<<24|A<<16|v-c|0;}}return 0!==L&&(u[v+L]=x-O<<24|64<<16|0),h.bits=C,0};},function(e,t,n){function r(){this.text=0,this.time=0,this.xflags=0,this.os=0,this.extra=null,this.extra_len=0,this.name="",this.comment="",this.hcrc=0,this.done=!1;}e.exports=r;},function(e,t,n){function r(e,t,n,r){a.call(this,"ZipFileWorker"),this.bytesWritten=0,this.zipComment=t,this.zipPlatform=n,this.encodeFileName=r,this.streamFiles=e,this.accumulate=!1,this.contentBuffer=[],this.dirRecords=[],this.currentSourceOffset=0,this.entriesCount=0,this.currentFile=null,this._sources=[];}var i=n(0),a=n(1),o=n(6),s=n(22),l=n(45),u=function(e,t){var n,r="";for(n=0;n<t;n++)r+=String.fromCharCode(255&e),e>>>=8;return r},c=function(e,t){var n=e;return e||(n=t?16893:33204),(65535&n)<<16},f=function(e,t){return 63&(e||0)},h=function(e,t,n,r,a,h){var d,p,m=e.file,g=e.compression,v=h!==o.utf8encode,b=i.transformTo("string",h(m.name)),y=i.transformTo("string",o.utf8encode(m.name)),w=m.comment,_=i.transformTo("string",h(w)),k=i.transformTo("string",o.utf8encode(w)),x=y.length!==m.name.length,S=k.length!==w.length,E="",T="",C="",A=m.dir,O=m.date,I={crc32:0,compressedSize:0,uncompressedSize:0};t&&!n||(I.crc32=e.crc32,I.compressedSize=e.compressedSize,I.uncompressedSize=e.uncompressedSize);var R=0;t&&(R|=8),v||!x&&!S||(R|=2048);var L=0,j=0;A&&(L|=16),"UNIX"===a?(j=798,L|=c(m.unixPermissions,A)):(j=20,L|=f(m.dosPermissions)),d=O.getUTCHours(),d<<=6,d|=O.getUTCMinutes(),d<<=5,d|=O.getUTCSeconds()/2,p=O.getUTCFullYear()-1980,p<<=4,p|=O.getUTCMonth()+1,p<<=5,p|=O.getUTCDate(),x&&(T=u(1,1)+u(s(b),4)+y,E+="up"+u(T.length,2)+T),S&&(C=u(1,1)+u(s(_),4)+k,E+="uc"+u(C.length,2)+C);var B="";return B+="\n\0",B+=u(R,2),B+=g.magic,B+=u(d,2),B+=u(p,2),B+=u(I.crc32,4),B+=u(I.compressedSize,4),B+=u(I.uncompressedSize,4),B+=u(b.length,2),B+=u(E.length,2),{fileRecord:l.LOCAL_FILE_HEADER+B+b+E,dirRecord:l.CENTRAL_FILE_HEADER+u(j,2)+B+u(_.length,2)+"\0\0\0\0"+u(L,4)+u(r,4)+b+E+_}},d=function(e,t,n,r,a){var o=i.transformTo("string",a(r));return l.CENTRAL_DIRECTORY_END+"\0\0\0\0"+u(e,2)+u(e,2)+u(t,4)+u(n,4)+u(o.length,2)+o},p=function(e){return l.DATA_DESCRIPTOR+u(e.crc32,4)+u(e.compressedSize,4)+u(e.uncompressedSize,4)};i.inherits(r,a),r.prototype.push=function(e){var t=e.meta.percent||0,n=this.entriesCount,r=this._sources.length;this.accumulate?this.contentBuffer.push(e):(this.bytesWritten+=e.data.length,a.prototype.push.call(this,{data:e.data,meta:{currentFile:this.currentFile,percent:n?(t+100*(n-r-1))/n:100}}));},r.prototype.openedSource=function(e){this.currentSourceOffset=this.bytesWritten,this.currentFile=e.file.name;var t=this.streamFiles&&!e.file.dir;if(t){var n=h(e,t,!1,this.currentSourceOffset,this.zipPlatform,this.encodeFileName);this.push({data:n.fileRecord,meta:{percent:0}});}else this.accumulate=!0;},r.prototype.closedSource=function(e){this.accumulate=!1;var t=this.streamFiles&&!e.file.dir,n=h(e,t,!0,this.currentSourceOffset,this.zipPlatform,this.encodeFileName);if(this.dirRecords.push(n.dirRecord),t)this.push({data:p(e),meta:{percent:100}});else for(this.push({data:n.fileRecord,meta:{percent:0}});this.contentBuffer.length;)this.push(this.contentBuffer.shift());this.currentFile=null;},r.prototype.flush=function(){for(var e=this.bytesWritten,t=0;t<this.dirRecords.length;t++)this.push({data:this.dirRecords[t],meta:{percent:100}});var n=this.bytesWritten-e,r=d(this.dirRecords.length,n,e,this.zipComment,this.encodeFileName);this.push({data:r,meta:{percent:100}});},r.prototype.prepareNextSource=function(){this.previous=this._sources.shift(),this.openedSource(this.previous.streamInfo),this.isPaused?this.previous.pause():this.previous.resume();},r.prototype.registerPrevious=function(e){this._sources.push(e);var t=this;return e.on("data",function(e){t.processChunk(e);}),e.on("end",function(){t.closedSource(t.previous.streamInfo),t._sources.length?t.prepareNextSource():t.end();}),e.on("error",function(e){t.error(e);}),this},r.prototype.resume=function(){return !!a.prototype.resume.call(this)&&(!this.previous&&this._sources.length?(this.prepareNextSource(),!0):this.previous||this._sources.length||this.generatedError?void 0:(this.end(),!0))},r.prototype.error=function(e){var t=this._sources;if(!a.prototype.error.call(this,e))return !1;for(var n=0;n<t.length;n++)try{t[n].error(e);}catch(e){}return !0},r.prototype.lock=function(){a.prototype.lock.call(this);for(var e=this._sources,t=0;t<e.length;t++)e[t].lock();},e.exports=r;},function(e,t,n){function r(e,t){a.call(this,"Nodejs stream input adapter for "+e),this._upstreamEnded=!1,this._bindStream(t);}var i=n(0),a=n(1);i.inherits(r,a),r.prototype._bindStream=function(e){var t=this;this._stream=e,e.pause(),e.on("data",function(e){t.push({data:e,meta:{percent:0}});}).on("error",function(e){t.isPaused?this.generatedError=e:t.error(e);}).on("end",function(){t.isPaused?t._upstreamEnded=!0:t.end();});},r.prototype.pause=function(){return !!a.prototype.pause.call(this)&&(this._stream.pause(),!0)},r.prototype.resume=function(){return !!a.prototype.resume.call(this)&&(this._upstreamEnded?this.end():this._stream.resume(),!0)},e.exports=r;},function(e,t,n){function r(e){return new a.Promise(function(t,n){var r=e.decompressed.getContentWorker().pipe(new l);r.on("error",function(e){n(e);}).on("end",function(){r.streamInfo.crc32!==e.decompressed.crc32?n(new Error("Corrupted zip : CRC32 mismatch")):t();}).resume();})}var i=n(0),a=n(9),o=n(6),i=n(0),s=n(99),l=n(38),u=n(14);e.exports=function(e,t){var n=this;return t=i.extend(t||{},{base64:!1,checkCRC32:!1,optimizedBinaryString:!1,createFolders:!1,decodeFileName:o.utf8decode}),u.isNode&&u.isStream(e)?a.Promise.reject(new Error("JSZip can't accept a stream when loading a zip file.")):i.prepareContent("the loaded zip file",e,!0,t.optimizedBinaryString,t.base64).then(function(e){var n=new s(t);return n.load(e),n}).then(function(e){var n=[a.Promise.resolve(e)],i=e.files;if(t.checkCRC32)for(var o=0;o<i.length;o++)n.push(r(i[o]));return a.Promise.all(n)}).then(function(e){for(var r=e.shift(),i=r.files,a=0;a<i.length;a++){var o=i[a];n.file(o.fileNameStr,o.decompressed,{binary:!0,optimizedBinaryString:!0,date:o.date,dir:o.dir,comment:o.fileCommentStr.length?o.fileCommentStr:null,unixPermissions:o.unixPermissions,dosPermissions:o.dosPermissions,createFolders:t.createFolders});}return r.zipComment.length&&(n.comment=r.zipComment),n})};},function(e,t,n){function r(e){this.files=[],this.loadOptions=e;}var i=n(46),a=n(0),o=n(45),s=n(102),l=(n(6),n(3));r.prototype={checkSignature:function(e){if(!this.reader.readAndCheckSignature(e)){this.reader.index-=4;var t=this.reader.readString(4);throw new Error("Corrupted zip or bug: unexpected signature ("+a.pretty(t)+", expected "+a.pretty(e)+")")}},isSignature:function(e,t){var n=this.reader.index;this.reader.setIndex(e);var r=this.reader.readString(4),i=r===t;return this.reader.setIndex(n),i},readBlockEndOfCentral:function(){this.diskNumber=this.reader.readInt(2),this.diskWithCentralDirStart=this.reader.readInt(2),this.centralDirRecordsOnThisDisk=this.reader.readInt(2),this.centralDirRecords=this.reader.readInt(2),this.centralDirSize=this.reader.readInt(4),this.centralDirOffset=this.reader.readInt(4),this.zipCommentLength=this.reader.readInt(2);var e=this.reader.readData(this.zipCommentLength),t=l.uint8array?"uint8array":"array",n=a.transformTo(t,e);this.zipComment=this.loadOptions.decodeFileName(n);},readBlockZip64EndOfCentral:function(){this.zip64EndOfCentralSize=this.reader.readInt(8),this.reader.skip(4),this.diskNumber=this.reader.readInt(4),this.diskWithCentralDirStart=this.reader.readInt(4),this.centralDirRecordsOnThisDisk=this.reader.readInt(8),this.centralDirRecords=this.reader.readInt(8),this.centralDirSize=this.reader.readInt(8),this.centralDirOffset=this.reader.readInt(8),this.zip64ExtensibleData={};for(var e,t,n,r=this.zip64EndOfCentralSize-44;0<r;)e=this.reader.readInt(2),t=this.reader.readInt(4),n=this.reader.readData(t),this.zip64ExtensibleData[e]={id:e,length:t,value:n};},readBlockZip64EndOfCentralLocator:function(){if(this.diskWithZip64CentralDirStart=this.reader.readInt(4),this.relativeOffsetEndOfZip64CentralDir=this.reader.readInt(8),this.disksCount=this.reader.readInt(4),this.disksCount>1)throw new Error("Multi-volumes zip are not supported")},readLocalFiles:function(){var e,t;for(e=0;e<this.files.length;e++)t=this.files[e],this.reader.setIndex(t.localHeaderOffset),this.checkSignature(o.LOCAL_FILE_HEADER),t.readLocalPart(this.reader),t.handleUTF8(),t.processAttributes();},readCentralDir:function(){var e;for(this.reader.setIndex(this.centralDirOffset);this.reader.readAndCheckSignature(o.CENTRAL_FILE_HEADER);)e=new s({zip64:this.zip64},this.loadOptions),e.readCentralPart(this.reader),this.files.push(e);if(this.centralDirRecords!==this.files.length&&0!==this.centralDirRecords&&0===this.files.length)throw new Error("Corrupted zip or bug: expected "+this.centralDirRecords+" records in central dir, got "+this.files.length)},readEndOfCentral:function(){var e=this.reader.lastIndexOfSignature(o.CENTRAL_DIRECTORY_END);if(e<0)throw this.isSignature(0,o.LOCAL_FILE_HEADER)?new Error("Corrupted zip: can't find end of central directory"):new Error("Can't find end of central directory : is this a zip file ? If it is, see https://stuk.github.io/jszip/documentation/howto/read_zip.html");this.reader.setIndex(e);var t=e;if(this.checkSignature(o.CENTRAL_DIRECTORY_END),this.readBlockEndOfCentral(),this.diskNumber===a.MAX_VALUE_16BITS||this.diskWithCentralDirStart===a.MAX_VALUE_16BITS||this.centralDirRecordsOnThisDisk===a.MAX_VALUE_16BITS||this.centralDirRecords===a.MAX_VALUE_16BITS||this.centralDirSize===a.MAX_VALUE_32BITS||this.centralDirOffset===a.MAX_VALUE_32BITS){if(this.zip64=!0,(e=this.reader.lastIndexOfSignature(o.ZIP64_CENTRAL_DIRECTORY_LOCATOR))<0)throw new Error("Corrupted zip: can't find the ZIP64 end of central directory locator");if(this.reader.setIndex(e),this.checkSignature(o.ZIP64_CENTRAL_DIRECTORY_LOCATOR),this.readBlockZip64EndOfCentralLocator(),!this.isSignature(this.relativeOffsetEndOfZip64CentralDir,o.ZIP64_CENTRAL_DIRECTORY_END)&&(this.relativeOffsetEndOfZip64CentralDir=this.reader.lastIndexOfSignature(o.ZIP64_CENTRAL_DIRECTORY_END),this.relativeOffsetEndOfZip64CentralDir<0))throw new Error("Corrupted zip: can't find the ZIP64 end of central directory");this.reader.setIndex(this.relativeOffsetEndOfZip64CentralDir),this.checkSignature(o.ZIP64_CENTRAL_DIRECTORY_END),this.readBlockZip64EndOfCentral();}var n=this.centralDirOffset+this.centralDirSize;this.zip64&&(n+=20,n+=12+this.zip64EndOfCentralSize);var r=t-n;if(r>0)this.isSignature(t,o.CENTRAL_FILE_HEADER)||(this.reader.zero=r);else if(r<0)throw new Error("Corrupted zip: missing "+Math.abs(r)+" bytes.")},prepareReader:function(e){this.reader=i(e);},load:function(e){this.prepareReader(e),this.readEndOfCentral(),this.readCentralDir(),this.readLocalFiles();}},e.exports=r;},function(e,t,n){function r(e){i.call(this,e);}var i=n(48);n(0).inherits(r,i),r.prototype.byteAt=function(e){return this.data.charCodeAt(this.zero+e)},r.prototype.lastIndexOfSignature=function(e){return this.data.lastIndexOf(e)-this.zero},r.prototype.readAndCheckSignature=function(e){return e===this.readData(4)},r.prototype.readData=function(e){this.checkOffset(e);var t=this.data.slice(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t},e.exports=r;},function(e,t,n){function r(e){i.call(this,e);}var i=n(49);n(0).inherits(r,i),r.prototype.readData=function(e){this.checkOffset(e);var t=this.data.slice(this.zero+this.index,this.zero+this.index+e);return this.index+=e,t},e.exports=r;},function(e,t,n){function r(e,t){this.options=e,this.loadOptions=t;}var i=n(46),a=n(0),o=n(21),s=n(22),l=n(6),u=n(39),c=n(3),f=function(e){for(var t in u)if(u.hasOwnProperty(t)&&u[t].magic===e)return u[t];return null};r.prototype={isEncrypted:function(){return 1==(1&this.bitFlag)},useUTF8:function(){return 2048==(2048&this.bitFlag)},readLocalPart:function(e){var t,n;if(e.skip(22),this.fileNameLength=e.readInt(2),n=e.readInt(2),this.fileName=e.readData(this.fileNameLength),e.skip(n),-1===this.compressedSize||-1===this.uncompressedSize)throw new Error("Bug or corrupted zip : didn't get enough informations from the central directory (compressedSize === -1 || uncompressedSize === -1)");if(null===(t=f(this.compressionMethod)))throw new Error("Corrupted zip : compression "+a.pretty(this.compressionMethod)+" unknown (inner file : "+a.transformTo("string",this.fileName)+")");this.decompressed=new o(this.compressedSize,this.uncompressedSize,this.crc32,t,e.readData(this.compressedSize));},readCentralPart:function(e){this.versionMadeBy=e.readInt(2),e.skip(2),this.bitFlag=e.readInt(2),this.compressionMethod=e.readString(2),this.date=e.readDate(),this.crc32=e.readInt(4),this.compressedSize=e.readInt(4),this.uncompressedSize=e.readInt(4);var t=e.readInt(2);if(this.extraFieldsLength=e.readInt(2),this.fileCommentLength=e.readInt(2),this.diskNumberStart=e.readInt(2),this.internalFileAttributes=e.readInt(2),this.externalFileAttributes=e.readInt(4),this.localHeaderOffset=e.readInt(4),this.isEncrypted())throw new Error("Encrypted zip are not supported");e.skip(t),this.readExtraFields(e),this.parseZIP64ExtraField(e),this.fileComment=e.readData(this.fileCommentLength);},processAttributes:function(){this.unixPermissions=null,this.dosPermissions=null;var e=this.versionMadeBy>>8;this.dir=!!(16&this.externalFileAttributes),0===e&&(this.dosPermissions=63&this.externalFileAttributes),3===e&&(this.unixPermissions=this.externalFileAttributes>>16&65535),this.dir||"/"!==this.fileNameStr.slice(-1)||(this.dir=!0);},parseZIP64ExtraField:function(e){if(this.extraFields[1]){var t=i(this.extraFields[1].value);this.uncompressedSize===a.MAX_VALUE_32BITS&&(this.uncompressedSize=t.readInt(8)),this.compressedSize===a.MAX_VALUE_32BITS&&(this.compressedSize=t.readInt(8)),this.localHeaderOffset===a.MAX_VALUE_32BITS&&(this.localHeaderOffset=t.readInt(8)),this.diskNumberStart===a.MAX_VALUE_32BITS&&(this.diskNumberStart=t.readInt(4));}},readExtraFields:function(e){var t,n,r,i=e.index+this.extraFieldsLength;for(this.extraFields||(this.extraFields={});e.index<i;)t=e.readInt(2),n=e.readInt(2),r=e.readData(n),this.extraFields[t]={id:t,length:n,value:r};},handleUTF8:function(){var e=c.uint8array?"uint8array":"array";if(this.useUTF8())this.fileNameStr=l.utf8decode(this.fileName),this.fileCommentStr=l.utf8decode(this.fileComment);else {var t=this.findExtraFieldUnicodePath();if(null!==t)this.fileNameStr=t;else {var n=a.transformTo(e,this.fileName);this.fileNameStr=this.loadOptions.decodeFileName(n);}var r=this.findExtraFieldUnicodeComment();if(null!==r)this.fileCommentStr=r;else {var i=a.transformTo(e,this.fileComment);this.fileCommentStr=this.loadOptions.decodeFileName(i);}}},findExtraFieldUnicodePath:function(){var e=this.extraFields[28789];if(e){var t=i(e.value);return 1!==t.readInt(1)?null:s(this.fileName)!==t.readInt(4)?null:l.utf8decode(t.readData(e.length-5))}return null},findExtraFieldUnicodeComment:function(){var e=this.extraFields[25461];if(e){var t=i(e.value);return 1!==t.readInt(1)?null:s(this.fileComment)!==t.readInt(4)?null:l.utf8decode(t.readData(e.length-5))}return null}},e.exports=r;},function(e,t,n){var r,i=i||function(e){if(!(void 0===e||"undefined"!=typeof navigator&&/MSIE [1-9]\./.test(navigator.userAgent))){var t=e.document,n=function(){return e.URL||e.webkitURL||e},r=t.createElementNS("http://www.w3.org/1999/xhtml","a"),i="download"in r,a=function(e){var t=new MouseEvent("click");e.dispatchEvent(t);},o=/constructor/i.test(e.HTMLElement)||e.safari,s=/CriOS\/[\d]+/.test(navigator.userAgent),l=function(t){(e.setImmediate||e.setTimeout)(function(){throw t},0);},u=function(e){var t=function(){"string"==typeof e?n().revokeObjectURL(e):e.remove();};setTimeout(t,4e4);},c=function(e,t,n){t=[].concat(t);for(var r=t.length;r--;){var i=e["on"+t[r]];if("function"==typeof i)try{i.call(e,n||e);}catch(e){l(e);}}},f=function(e){return /^\s*(?:text\/\S*|application\/xml|\S*\/\S*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(e.type)?new Blob([String.fromCharCode(65279),e],{type:e.type}):e},h=function(t,l,h){h||(t=f(t));var d,p=this,m=t.type,g="application/octet-stream"===m,v=function(){c(p,"writestart progress write writeend".split(" "));};if(p.readyState=p.INIT,i)return d=n().createObjectURL(t),void setTimeout(function(){r.href=d,r.download=l,a(r),v(),u(d),p.readyState=p.DONE;});!function(){if((s||g&&o)&&e.FileReader){var r=new FileReader;return r.onloadend=function(){var t=s?r.result:r.result.replace(/^data:[^;]*;/,"data:attachment/file;");e.open(t,"_blank")||(e.location.href=t),t=void 0,p.readyState=p.DONE,v();},r.readAsDataURL(t),void(p.readyState=p.INIT)}d||(d=n().createObjectURL(t)),g?e.location.href=d:e.open(d,"_blank")||(e.location.href=d),p.readyState=p.DONE,v(),u(d);}();},d=h.prototype,p=function(e,t,n){return new h(e,t||e.name||"download",n)};return "undefined"!=typeof navigator&&navigator.msSaveOrOpenBlob?function(e,t,n){return t=t||e.name||"download",n||(e=f(e)),navigator.msSaveOrOpenBlob(e,t)}:(d.abort=function(){},d.readyState=d.INIT=0,d.WRITING=1,d.DONE=2,d.error=d.onwritestart=d.onprogress=d.onwrite=d.onabort=d.onerror=d.onwriteend=null,p)}}("undefined"!=typeof self&&self||"undefined"!=typeof window&&window||this.content);void 0!==e&&e.exports?e.exports.saveAs=i:null!==n(104)&&null!==n(105)&&void 0!==(r=function(){return i}.call(t,n,t,e))&&(e.exports=r);},function(e,t){e.exports=function(){throw new Error("define cannot be used indirect")};},function(e,t){(function(t){e.exports=t;}).call(t,{});}])});}).call(t,n(1).setImmediate,n(1).clearImmediate);},function(e,t,n){(function(e,t){!function(e,n){function r(e){"function"!=typeof e&&(e=new Function(""+e));for(var t=new Array(arguments.length-1),n=0;n<t.length;n++)t[n]=arguments[n+1];var r={callback:e,args:t};return u[l]=r,s(l),l++}function i(e){delete u[e];}function a(e){var t=e.callback,r=e.args;switch(r.length){case 0:t();break;case 1:t(r[0]);break;case 2:t(r[0],r[1]);break;case 3:t(r[0],r[1],r[2]);break;default:t.apply(n,r);}}function o(e){if(c)setTimeout(o,0,e);else {var t=u[e];if(t){c=!0;try{a(t);}finally{i(e),c=!1;}}}}if(!e.setImmediate){var s,l=1,u={},c=!1,f=e.document,h=Object.getPrototypeOf&&Object.getPrototypeOf(e);h=h&&h.setTimeout?h:e,"[object process]"==={}.toString.call(e.process)?function(){s=function(e){t.nextTick(function(){o(e);});};}():function(){if(e.postMessage&&!e.importScripts){var t=!0,n=e.onmessage;return e.onmessage=function(){t=!1;},e.postMessage("","*"),e.onmessage=n,t}}()?function(){var t="setImmediate$"+Math.random()+"$",n=function(n){n.source===e&&"string"==typeof n.data&&0===n.data.indexOf(t)&&o(+n.data.slice(t.length));};e.addEventListener?e.addEventListener("message",n,!1):e.attachEvent("onmessage",n),s=function(n){e.postMessage(t+n,"*");};}():e.MessageChannel?function(){var e=new MessageChannel;e.port1.onmessage=function(e){o(e.data);},s=function(t){e.port2.postMessage(t);};}():f&&"onreadystatechange"in f.createElement("script")?function(){var e=f.documentElement;s=function(t){var n=f.createElement("script");n.onreadystatechange=function(){o(t),n.onreadystatechange=null,e.removeChild(n),n=null;},e.appendChild(n);};}():function(){s=function(e){setTimeout(o,0,e);};}(),h.setImmediate=r,h.clearImmediate=i;}}("undefined"==typeof self?void 0===e?this:e:self);}).call(t,n(2),n(11));},function(e,t){function n(){throw new Error("setTimeout has not been defined")}function r(){throw new Error("clearTimeout has not been defined")}function i(e){if(c===setTimeout)return setTimeout(e,0);if((c===n||!c)&&setTimeout)return c=setTimeout,setTimeout(e,0);try{return c(e,0)}catch(t){try{return c.call(null,e,0)}catch(t){return c.call(this,e,0)}}}function a(e){if(f===clearTimeout)return clearTimeout(e);if((f===r||!f)&&clearTimeout)return f=clearTimeout,clearTimeout(e);try{return f(e)}catch(t){try{return f.call(null,e)}catch(t){return f.call(this,e)}}}function o(){m&&d&&(m=!1,d.length?p=d.concat(p):g=-1,p.length&&s());}function s(){if(!m){var e=i(o);m=!0;for(var t=p.length;t;){for(d=p,p=[];++g<t;)d&&d[g].run();g=-1,t=p.length;}d=null,m=!1,a(e);}}function l(e,t){this.fun=e,this.array=t;}function u(){}var c,f,h=e.exports={};!function(){try{c="function"==typeof setTimeout?setTimeout:n;}catch(e){c=n;}try{f="function"==typeof clearTimeout?clearTimeout:r;}catch(e){f=r;}}();var d,p=[],m=!1,g=-1;h.nextTick=function(e){var t=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];p.push(new l(e,t)),1!==p.length||m||i(s);},l.prototype.run=function(){this.fun.apply(null,this.array);},h.title="browser",h.browser=!0,h.env={},h.argv=[],h.version="",h.versions={},h.on=u,h.addListener=u,h.once=u,h.off=u,h.removeListener=u,h.removeAllListeners=u,h.emit=u,h.prependListener=u,h.prependOnceListener=u,h.listeners=function(e){return []},h.binding=function(e){throw new Error("process.binding is not supported")},h.cwd=function(){return "/"},h.chdir=function(e){throw new Error("process.chdir is not supported")},h.umask=function(){return 0};},function(e,t,n){/*! grapesjs-aviary - 0.1.2 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=0)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=function(e){return e&&e.__esModule?e:{default:e}}(r);t.default=i.default.plugins.add("gjs-aviary",function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,r=e.getModel(),i=void 0,a={key:"1",onApply:null,getFilename:null,closeOnApply:!0,config:{}};for(var o in a)o in n||(n[o]=a[o]);var s=n.config;s.apiKey=n.key,s.onSave=function(e,t){i.set("src",t);var r="function"==typeof n.getFilename?n.getFilename:c,a=r(i);("function"==typeof n.onApply?n.onApply:f)(t,a,i),n.closeOnApply&&l.close();};var l=new Aviary.Feather(s),u=e.Commands,c=function(e){var t=e.get("src").split("/").pop();return Date.now()+"_"+t.slice(-15)},f=function(t,n){e.AssetManager.add({src:t,name:n});},h=document.createElement("img");u.add("image-editor",{run:function(e,t,n){var a=n||{},o=a.model||e.getSelected();i=o,h.src=o.get("src"),l.launch({image:h}),r.trigger("gjs-aviary:launch",o,l);}});});},function(t,n){t.exports=e;}])});},function(e,t,n){/*! grapesjs-plugin-filestack - 0.1.1 */
!function(t,r){e.exports=r(n(0));}(0,function(e){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r});},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=0)}([function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=n(1),i=function(e){return e&&e.__esModule?e:{default:e}}(r);t.default=i.default.plugins.add("gjs-plugin-filestack",function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=t,r=e.getConfig(),i=r.stylePrefix||"",a=void 0,o={key:"",btnEl:"",btnText:"Add images",filestackOpts:{accept:"image/*",maxFiles:10},onComplete:function(e,t){}};for(var s in o)s in n||(n[s]=o[s]);if(!filestack)throw new Error("Filestack instance not found");if(!n.key)throw new Error("Filestack's API key not found");var l=filestack.init(n.key);e.on("run:open-assets",function(){var t=e.Modal,r=t.getContentEl(),o=r.querySelector("."+i+"am-file-uploader"),s=r.querySelector("."+i+"am-assets-header"),c=r.querySelector("."+i+"am-assets-cont");o&&(o.style.display="none"),s&&(s.style.display="none"),c.style.width="100%",a||(a=n.btnEl,a||(a=document.createElement("button"),a.className=i+"btn-prim "+i+"btn-filestack",a.innerHTML=n.btnText),a.onclick=function(){l.pick(n.filestackOpts).then(function(e){var t=e.filesUploaded,r=t instanceof Array?t:[t],i=u(r);n.onComplete(r,i);});}),c.insertBefore(a,s);});var u=function(t){var n=t.map(function(e){return e.src=e.url,e});return e.AssetManager.add(n)};});},function(t,n){t.exports=e;}])});},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=n(15),i=function(e){return e&&e.__esModule?e:{default:e}}(r),a=n(3);t.default=function(e,t){var n=e.Commands,r=t.textCleanCanvas;n.add(a.cmdImport,(0, i.default)(e,t)),n.add(a.cmdDeviceDesktop,function(e){return e.setDevice("Desktop")}),n.add(a.cmdDeviceTablet,function(e){return e.setDevice("Tablet")}),n.add(a.cmdDeviceMobile,function(e){return e.setDevice("Mobile portrait")}),n.add(a.cmdClear,function(e){return confirm(r)&&e.runCommand("core:canvas-clear")});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r]);}return e};t.default=function(e,t){var n=e.getConfig("stylePrefix"),i=e.Modal,a=e.CodeManager.getViewer("CodeMirror").clone(),o=document.createElement("div"),s=t.modalImportLabel,l=t.modalImportContent,u=a.editor,c=document.createElement("button");return c.type="button",c.innerHTML=t.modalImportButton,c.className=n+"btn-prim "+n+"btn-import",c.onclick=function(t){e.setComponents(u.getValue().trim()),i.close();},a.set(r({codeName:"htmlmixed",theme:"hopscotch",readOnly:0},t.importViewerOptions)),{run:function(e){var r=this;if(!u){var f=document.createElement("textarea");if(s){var h=document.createElement("div");h.className=n+"import-label",h.innerHTML=s,o.appendChild(h);}o.appendChild(f),o.appendChild(c),a.init(f),u=a.editor;}i.setTitle(t.modalImportTitle),i.setContent(o);var d="function"==typeof l?l(e):l;a.setContent(d||""),i.open().getModel().once("change:open",function(){return e.stopCommand(r.id)}),u.refresh();},stop:function(){i.close();}}};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){var n=e.BlockManager,r=function(e){return t.blocks.indexOf(e)>=0};r("link-block")&&n.add("link-block",{category:"Basic",label:"Link Block",attributes:{class:"fa fa-link"},content:{type:"link",editable:!1,droppable:!0,style:{display:"inline-block",padding:"5px","min-height":"50px","min-width":"50px"}}}),r("quote")&&n.add("quote",{label:"Quote",category:"Basic",attributes:{class:"fa fa-quote-right"},content:'<blockquote class="quote">\n        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore ipsum dolor sit\n      </blockquote>'}),r("text-basic")&&n.add("text-basic",{category:"Basic",label:"Text section",attributes:{class:"gjs-fonts gjs-f-h1p"},content:'<section class="bdg-sect">\n      <h1 class="heading">Insert title here</h1>\n      <p class="paragraph">Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua</p>\n      </section>'});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e){var t=(e.DomComponents),n=t.getType("default");n.model,n.view;};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0});var r=n(3);t.default=function(e,t){var n=e.Panels,i=e.getConfig(),a="sw-visibility",o="export-template",s="open-sm",l="open-layers",u="open-blocks",c="fullscreen",f="preview";i.showDevices=0,n.getPanels().reset([{id:"commands",buttons:[{}]},{id:"options",buttons:[{id:a,command:a,context:a,className:"fa fa-square-o"},{id:f,context:f,command:function(e){return e.runCommand(f)},className:"fa fa-eye"},{id:c,command:c,context:c,className:"fa fa-arrows-alt"},{id:o,className:"fa fa-code",command:function(e){return e.runCommand(o)}},{id:"undo",className:"fa fa-undo",command:function(e){return e.runCommand("core:undo")}},{id:"redo",className:"fa fa-repeat",command:function(e){return e.runCommand("core:redo")}},{id:r.cmdImport,className:"fa fa-download",command:function(e){return e.runCommand(r.cmdImport)}},{id:r.cmdClear,className:"fa fa-trash",command:function(e){return e.runCommand(r.cmdClear)}}]},{id:"views",buttons:[{id:s,command:s,active:!0,className:"fa fa-paint-brush"},{id:"open-tm",command:"open-tm",className:"fa fa-cog"},{id:l,command:l,className:"fa fa-bars"},{id:u,command:u,className:"fa fa-th-large"}]}]),n.addPanel({id:"devices-c"}).get("buttons").add([{id:r.cmdDeviceDesktop,command:r.cmdDeviceDesktop,className:"fa fa-desktop",active:1},{id:r.cmdDeviceTablet,command:r.cmdDeviceTablet,className:"fa fa-tablet"},{id:r.cmdDeviceMobile,command:r.cmdDeviceMobile,className:"fa fa-mobile"}]);var h=n.getButton("views",u);e.on("load",function(){return h&&h.set("active",1)}),t.showStylesOnChange&&e.on("component:selected",function(){var t=n.getButton("views",s),r=n.getButton("views",l);r&&r.get("active")||!e.getSelected()||t&&t.set("active",1);});};},function(e,t,n){Object.defineProperty(t,"__esModule",{value:!0}),t.default=function(e,t){var n=e.StyleManager,r=t.customStyleManager;n.getSectors().reset(r&&r.length?r:[{name:t.textGeneral,open:!1,buildProps:["float","display","position","top","right","left","bottom"]},{name:t.textLayout,open:!1,buildProps:["width","height","max-width","min-height","margin","padding"]},{name:t.textTypography,open:!1,buildProps:["font-family","font-size","font-weight","letter-spacing","color","line-height","text-align","text-shadow"],properties:[{property:"text-align",list:[{value:"left",className:"fa fa-align-left"},{value:"center",className:"fa fa-align-center"},{value:"right",className:"fa fa-align-right"},{value:"justify",className:"fa fa-align-justify"}]}]},{name:t.textDecorations,open:!1,buildProps:["border-radius-c","background-color","border-radius","border","box-shadow","background"]},{name:t.textExtra,open:!1,buildProps:["transition","perspective","transform"]}]);};}])});
});

var webpage = unwrapExports(grapesjsPresetWebpage_min);

/*
object-assign
(c) Sindre Sorhus
@license MIT
*/
/* eslint-disable no-unused-vars */
var getOwnPropertySymbols = Object.getOwnPropertySymbols;
var hasOwnProperty$1 = Object.prototype.hasOwnProperty;
var propIsEnumerable = Object.prototype.propertyIsEnumerable;

function toObject(val) {
	if (val === null || val === undefined) {
		throw new TypeError('Object.assign cannot be called with null or undefined');
	}

	return Object(val);
}

function shouldUseNative() {
	try {
		if (!Object.assign) {
			return false;
		}

		// Detect buggy property enumeration order in older V8 versions.

		// https://bugs.chromium.org/p/v8/issues/detail?id=4118
		var test1 = new String('abc');  // eslint-disable-line no-new-wrappers
		test1[5] = 'de';
		if (Object.getOwnPropertyNames(test1)[0] === '5') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test2 = {};
		for (var i = 0; i < 10; i++) {
			test2['_' + String.fromCharCode(i)] = i;
		}
		var order2 = Object.getOwnPropertyNames(test2).map(function (n) {
			return test2[n];
		});
		if (order2.join('') !== '0123456789') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test3 = {};
		'abcdefghijklmnopqrst'.split('').forEach(function (letter) {
			test3[letter] = letter;
		});
		if (Object.keys(Object.assign({}, test3)).join('') !==
				'abcdefghijklmnopqrst') {
			return false;
		}

		return true;
	} catch (err) {
		// We don't expect any of the above to throw, but better to be safe.
		return false;
	}
}

var objectAssign = shouldUseNative() ? Object.assign : function (target, source) {
	var from;
	var to = toObject(target);
	var symbols;

	for (var s = 1; s < arguments.length; s++) {
		from = Object(arguments[s]);

		for (var key in from) {
			if (hasOwnProperty$1.call(from, key)) {
				to[key] = from[key];
			}
		}

		if (getOwnPropertySymbols) {
			symbols = getOwnPropertySymbols(from);
			for (var i = 0; i < symbols.length; i++) {
				if (propIsEnumerable.call(from, symbols[i])) {
					to[symbols[i]] = from[symbols[i]];
				}
			}
		}
	}

	return to;
};

var n="function"===typeof Symbol&&Symbol.for,p=n?Symbol.for("react.element"):60103,q=n?Symbol.for("react.portal"):60106,r=n?Symbol.for("react.fragment"):60107,t=n?Symbol.for("react.strict_mode"):60108,u=n?Symbol.for("react.profiler"):60114,v=n?Symbol.for("react.provider"):60109,w=n?Symbol.for("react.context"):60110,x=n?Symbol.for("react.forward_ref"):60112,y=n?Symbol.for("react.suspense"):60113,z=n?Symbol.for("react.memo"):60115,A=n?Symbol.for("react.lazy"):
60116,B="function"===typeof Symbol&&Symbol.iterator;function C(a){for(var b="https://reactjs.org/docs/error-decoder.html?invariant="+a,c=1;c<arguments.length;c++)b+="&args[]="+encodeURIComponent(arguments[c]);return "Minified React error #"+a+"; visit "+b+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}
var D={isMounted:function(){return !1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},E={};function F(a,b,c){this.props=a;this.context=b;this.refs=E;this.updater=c||D;}F.prototype.isReactComponent={};F.prototype.setState=function(a,b){if("object"!==typeof a&&"function"!==typeof a&&null!=a)throw Error(C(85));this.updater.enqueueSetState(this,a,b,"setState");};F.prototype.forceUpdate=function(a){this.updater.enqueueForceUpdate(this,a,"forceUpdate");};
function G(){}G.prototype=F.prototype;function H(a,b,c){this.props=a;this.context=b;this.refs=E;this.updater=c||D;}var I=H.prototype=new G;I.constructor=H;objectAssign(I,F.prototype);I.isPureReactComponent=!0;var J={current:null},K=Object.prototype.hasOwnProperty,L={key:!0,ref:!0,__self:!0,__source:!0};
function M(a,b,c){var e,d={},g=null,k=null;if(null!=b)for(e in void 0!==b.ref&&(k=b.ref),void 0!==b.key&&(g=""+b.key),b)K.call(b,e)&&!L.hasOwnProperty(e)&&(d[e]=b[e]);var f=arguments.length-2;if(1===f)d.children=c;else if(1<f){for(var h=Array(f),m=0;m<f;m++)h[m]=arguments[m+2];d.children=h;}if(a&&a.defaultProps)for(e in f=a.defaultProps,f)void 0===d[e]&&(d[e]=f[e]);return {$$typeof:p,type:a,key:g,ref:k,props:d,_owner:J.current}}
function N(a,b){return {$$typeof:p,type:a.type,key:b,ref:a.ref,props:a.props,_owner:a._owner}}function O(a){return "object"===typeof a&&null!==a&&a.$$typeof===p}function escape(a){var b={"=":"=0",":":"=2"};return "$"+(""+a).replace(/[=:]/g,function(a){return b[a]})}var P=/\/+/g,Q=[];function R(a,b,c,e){if(Q.length){var d=Q.pop();d.result=a;d.keyPrefix=b;d.func=c;d.context=e;d.count=0;return d}return {result:a,keyPrefix:b,func:c,context:e,count:0}}
function S(a){a.result=null;a.keyPrefix=null;a.func=null;a.context=null;a.count=0;10>Q.length&&Q.push(a);}
function T(a,b,c,e){var d=typeof a;if("undefined"===d||"boolean"===d)a=null;var g=!1;if(null===a)g=!0;else switch(d){case "string":case "number":g=!0;break;case "object":switch(a.$$typeof){case p:case q:g=!0;}}if(g)return c(e,a,""===b?"."+U(a,0):b),1;g=0;b=""===b?".":b+":";if(Array.isArray(a))for(var k=0;k<a.length;k++){d=a[k];var f=b+U(d,k);g+=T(d,f,c,e);}else if(null===a||"object"!==typeof a?f=null:(f=B&&a[B]||a["@@iterator"],f="function"===typeof f?f:null),"function"===typeof f)for(a=f.call(a),k=
0;!(d=a.next()).done;)d=d.value,f=b+U(d,k++),g+=T(d,f,c,e);else if("object"===d)throw c=""+a,Error(C(31,"[object Object]"===c?"object with keys {"+Object.keys(a).join(", ")+"}":c,""));return g}function V(a,b,c){return null==a?0:T(a,"",b,c)}function U(a,b){return "object"===typeof a&&null!==a&&null!=a.key?escape(a.key):b.toString(36)}function W(a,b){a.func.call(a.context,b,a.count++);}
function aa(a,b,c){var e=a.result,d=a.keyPrefix;a=a.func.call(a.context,b,a.count++);Array.isArray(a)?X(a,e,c,function(a){return a}):null!=a&&(O(a)&&(a=N(a,d+(!a.key||b&&b.key===a.key?"":(""+a.key).replace(P,"$&/")+"/")+c)),e.push(a));}function X(a,b,c,e,d){var g="";null!=c&&(g=(""+c).replace(P,"$&/")+"/");b=R(b,g,e,d);V(a,aa,b);S(b);}var Y={current:null};function Z(){var a=Y.current;if(null===a)throw Error(C(321));return a}
var ba={ReactCurrentDispatcher:Y,ReactCurrentBatchConfig:{suspense:null},ReactCurrentOwner:J,IsSomeRendererActing:{current:!1},assign:objectAssign};var Children={map:function(a,b,c){if(null==a)return a;var e=[];X(a,e,null,b,c);return e},forEach:function(a,b,c){if(null==a)return a;b=R(null,null,b,c);V(a,W,b);S(b);},count:function(a){return V(a,function(){return null},null)},toArray:function(a){var b=[];X(a,b,null,function(a){return a});return b},only:function(a){if(!O(a))throw Error(C(143));return a}};
var Component=F;var Fragment=r;var Profiler=u;var PureComponent=H;var StrictMode=t;var Suspense=y;var __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=ba;
var cloneElement=function(a,b,c){if(null===a||void 0===a)throw Error(C(267,a));var e=objectAssign({},a.props),d=a.key,g=a.ref,k=a._owner;if(null!=b){void 0!==b.ref&&(g=b.ref,k=J.current);void 0!==b.key&&(d=""+b.key);if(a.type&&a.type.defaultProps)var f=a.type.defaultProps;for(h in b)K.call(b,h)&&!L.hasOwnProperty(h)&&(e[h]=void 0===b[h]&&void 0!==f?f[h]:b[h]);}var h=arguments.length-2;if(1===h)e.children=c;else if(1<h){f=Array(h);for(var m=0;m<h;m++)f[m]=arguments[m+2];e.children=f;}return {$$typeof:p,type:a.type,
key:d,ref:g,props:e,_owner:k}};var createContext=function(a,b){void 0===b&&(b=null);a={$$typeof:w,_calculateChangedBits:b,_currentValue:a,_currentValue2:a,_threadCount:0,Provider:null,Consumer:null};a.Provider={$$typeof:v,_context:a};return a.Consumer=a};var createElement=M;var createFactory=function(a){var b=M.bind(null,a);b.type=a;return b};var createRef=function(){return {current:null}};var forwardRef=function(a){return {$$typeof:x,render:a}};var isValidElement=O;
var lazy=function(a){return {$$typeof:A,_ctor:a,_status:-1,_result:null}};var memo=function(a,b){return {$$typeof:z,type:a,compare:void 0===b?null:b}};var useCallback=function(a,b){return Z().useCallback(a,b)};var useContext=function(a,b){return Z().useContext(a,b)};var useDebugValue=function(){};var useEffect=function(a,b){return Z().useEffect(a,b)};var useImperativeHandle=function(a,b,c){return Z().useImperativeHandle(a,b,c)};
var useLayoutEffect=function(a,b){return Z().useLayoutEffect(a,b)};var useMemo=function(a,b){return Z().useMemo(a,b)};var useReducer=function(a,b,c){return Z().useReducer(a,b,c)};var useRef=function(a){return Z().useRef(a)};var useState=function(a){return Z().useState(a)};var version="16.13.1";

var react_production_min = {
	Children: Children,
	Component: Component,
	Fragment: Fragment,
	Profiler: Profiler,
	PureComponent: PureComponent,
	StrictMode: StrictMode,
	Suspense: Suspense,
	__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED: __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
	cloneElement: cloneElement,
	createContext: createContext,
	createElement: createElement,
	createFactory: createFactory,
	createRef: createRef,
	forwardRef: forwardRef,
	isValidElement: isValidElement,
	lazy: lazy,
	memo: memo,
	useCallback: useCallback,
	useContext: useContext,
	useDebugValue: useDebugValue,
	useEffect: useEffect,
	useImperativeHandle: useImperativeHandle,
	useLayoutEffect: useLayoutEffect,
	useMemo: useMemo,
	useReducer: useReducer,
	useRef: useRef,
	useState: useState,
	version: version
};

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var ReactPropTypesSecret = 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED';

var ReactPropTypesSecret_1 = ReactPropTypesSecret;

var printWarning = function() {};

if (process.env.NODE_ENV !== 'production') {
  var ReactPropTypesSecret$1 = ReactPropTypesSecret_1;
  var loggedTypeFailures = {};
  var has$1 = Function.call.bind(Object.prototype.hasOwnProperty);

  printWarning = function(text) {
    var message = 'Warning: ' + text;
    if (typeof console !== 'undefined') {
      console.error(message);
    }
    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (x) {}
  };
}

/**
 * Assert that the values match with the type specs.
 * Error messages are memorized and will only be shown once.
 *
 * @param {object} typeSpecs Map of name to a ReactPropType
 * @param {object} values Runtime values that need to be type-checked
 * @param {string} location e.g. "prop", "context", "child context"
 * @param {string} componentName Name of the component for error messages.
 * @param {?Function} getStack Returns the component stack.
 * @private
 */
function checkPropTypes(typeSpecs, values, location, componentName, getStack) {
  if (process.env.NODE_ENV !== 'production') {
    for (var typeSpecName in typeSpecs) {
      if (has$1(typeSpecs, typeSpecName)) {
        var error;
        // Prop type validation may throw. In case they do, we don't want to
        // fail the render phase where it didn't fail before. So we log it.
        // After these have been cleaned up, we'll let them throw.
        try {
          // This is intentionally an invariant that gets caught. It's the same
          // behavior as without this statement except with a better message.
          if (typeof typeSpecs[typeSpecName] !== 'function') {
            var err = Error(
              (componentName || 'React class') + ': ' + location + ' type `' + typeSpecName + '` is invalid; ' +
              'it must be a function, usually from the `prop-types` package, but received `' + typeof typeSpecs[typeSpecName] + '`.'
            );
            err.name = 'Invariant Violation';
            throw err;
          }
          error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret$1);
        } catch (ex) {
          error = ex;
        }
        if (error && !(error instanceof Error)) {
          printWarning(
            (componentName || 'React class') + ': type specification of ' +
            location + ' `' + typeSpecName + '` is invalid; the type checker ' +
            'function must return `null` or an `Error` but returned a ' + typeof error + '. ' +
            'You may have forgotten to pass an argument to the type checker ' +
            'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' +
            'shape all require an argument).'
          );
        }
        if (error instanceof Error && !(error.message in loggedTypeFailures)) {
          // Only monitor this failure once because there tends to be a lot of the
          // same error.
          loggedTypeFailures[error.message] = true;

          var stack = getStack ? getStack() : '';

          printWarning(
            'Failed ' + location + ' type: ' + error.message + (stack != null ? stack : '')
          );
        }
      }
    }
  }
}

/**
 * Resets warning cache when testing.
 *
 * @private
 */
checkPropTypes.resetWarningCache = function() {
  if (process.env.NODE_ENV !== 'production') {
    loggedTypeFailures = {};
  }
};

var checkPropTypes_1 = checkPropTypes;

var react_development = createCommonjsModule(function (module, exports) {



if (process.env.NODE_ENV !== "production") {
  (function() {

var _assign = objectAssign;
var checkPropTypes = checkPropTypes_1;

var ReactVersion = '16.13.1';

// The Symbol used to tag the ReactElement-like types. If there is no native Symbol
// nor polyfill, then a plain number is used for performance.
var hasSymbol = typeof Symbol === 'function' && Symbol.for;
var REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;
var REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;
var REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;
var REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;
var REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;
var REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;
var REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace; // TODO: We don't use AsyncMode or ConcurrentMode anymore. They were temporary
var REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;
var REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;
var REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;
var REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for('react.suspense_list') : 0xead8;
var REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;
var REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;
var REACT_BLOCK_TYPE = hasSymbol ? Symbol.for('react.block') : 0xead9;
var REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for('react.fundamental') : 0xead5;
var REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for('react.responder') : 0xead6;
var REACT_SCOPE_TYPE = hasSymbol ? Symbol.for('react.scope') : 0xead7;
var MAYBE_ITERATOR_SYMBOL = typeof Symbol === 'function' && Symbol.iterator;
var FAUX_ITERATOR_SYMBOL = '@@iterator';
function getIteratorFn(maybeIterable) {
  if (maybeIterable === null || typeof maybeIterable !== 'object') {
    return null;
  }

  var maybeIterator = MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL];

  if (typeof maybeIterator === 'function') {
    return maybeIterator;
  }

  return null;
}

/**
 * Keeps track of the current dispatcher.
 */
var ReactCurrentDispatcher = {
  /**
   * @internal
   * @type {ReactComponent}
   */
  current: null
};

/**
 * Keeps track of the current batch's configuration such as how long an update
 * should suspend for if it needs to.
 */
var ReactCurrentBatchConfig = {
  suspense: null
};

/**
 * Keeps track of the current owner.
 *
 * The current owner is the component who should own any components that are
 * currently being constructed.
 */
var ReactCurrentOwner = {
  /**
   * @internal
   * @type {ReactComponent}
   */
  current: null
};

var BEFORE_SLASH_RE = /^(.*)[\\\/]/;
function describeComponentFrame (name, source, ownerName) {
  var sourceInfo = '';

  if (source) {
    var path = source.fileName;
    var fileName = path.replace(BEFORE_SLASH_RE, '');

    {
      // In DEV, include code for a common special case:
      // prefer "folder/index.js" instead of just "index.js".
      if (/^index\./.test(fileName)) {
        var match = path.match(BEFORE_SLASH_RE);

        if (match) {
          var pathBeforeSlash = match[1];

          if (pathBeforeSlash) {
            var folderName = pathBeforeSlash.replace(BEFORE_SLASH_RE, '');
            fileName = folderName + '/' + fileName;
          }
        }
      }
    }

    sourceInfo = ' (at ' + fileName + ':' + source.lineNumber + ')';
  } else if (ownerName) {
    sourceInfo = ' (created by ' + ownerName + ')';
  }

  return '\n    in ' + (name || 'Unknown') + sourceInfo;
}

var Resolved = 1;
function refineResolvedLazyComponent(lazyComponent) {
  return lazyComponent._status === Resolved ? lazyComponent._result : null;
}

function getWrappedName(outerType, innerType, wrapperName) {
  var functionName = innerType.displayName || innerType.name || '';
  return outerType.displayName || (functionName !== '' ? wrapperName + "(" + functionName + ")" : wrapperName);
}

function getComponentName(type) {
  if (type == null) {
    // Host root, text node or just invalid type.
    return null;
  }

  {
    if (typeof type.tag === 'number') {
      error('Received an unexpected object in getComponentName(). ' + 'This is likely a bug in React. Please file an issue.');
    }
  }

  if (typeof type === 'function') {
    return type.displayName || type.name || null;
  }

  if (typeof type === 'string') {
    return type;
  }

  switch (type) {
    case REACT_FRAGMENT_TYPE:
      return 'Fragment';

    case REACT_PORTAL_TYPE:
      return 'Portal';

    case REACT_PROFILER_TYPE:
      return "Profiler";

    case REACT_STRICT_MODE_TYPE:
      return 'StrictMode';

    case REACT_SUSPENSE_TYPE:
      return 'Suspense';

    case REACT_SUSPENSE_LIST_TYPE:
      return 'SuspenseList';
  }

  if (typeof type === 'object') {
    switch (type.$$typeof) {
      case REACT_CONTEXT_TYPE:
        return 'Context.Consumer';

      case REACT_PROVIDER_TYPE:
        return 'Context.Provider';

      case REACT_FORWARD_REF_TYPE:
        return getWrappedName(type, type.render, 'ForwardRef');

      case REACT_MEMO_TYPE:
        return getComponentName(type.type);

      case REACT_BLOCK_TYPE:
        return getComponentName(type.render);

      case REACT_LAZY_TYPE:
        {
          var thenable = type;
          var resolvedThenable = refineResolvedLazyComponent(thenable);

          if (resolvedThenable) {
            return getComponentName(resolvedThenable);
          }

          break;
        }
    }
  }

  return null;
}

var ReactDebugCurrentFrame = {};
var currentlyValidatingElement = null;
function setCurrentlyValidatingElement(element) {
  {
    currentlyValidatingElement = element;
  }
}

{
  // Stack implementation injected by the current renderer.
  ReactDebugCurrentFrame.getCurrentStack = null;

  ReactDebugCurrentFrame.getStackAddendum = function () {
    var stack = ''; // Add an extra top frame while an element is being validated

    if (currentlyValidatingElement) {
      var name = getComponentName(currentlyValidatingElement.type);
      var owner = currentlyValidatingElement._owner;
      stack += describeComponentFrame(name, currentlyValidatingElement._source, owner && getComponentName(owner.type));
    } // Delegate to the injected renderer-specific implementation


    var impl = ReactDebugCurrentFrame.getCurrentStack;

    if (impl) {
      stack += impl() || '';
    }

    return stack;
  };
}

/**
 * Used by act() to track whether you're inside an act() scope.
 */
var IsSomeRendererActing = {
  current: false
};

var ReactSharedInternals = {
  ReactCurrentDispatcher: ReactCurrentDispatcher,
  ReactCurrentBatchConfig: ReactCurrentBatchConfig,
  ReactCurrentOwner: ReactCurrentOwner,
  IsSomeRendererActing: IsSomeRendererActing,
  // Used by renderers to avoid bundling object-assign twice in UMD bundles:
  assign: _assign
};

{
  _assign(ReactSharedInternals, {
    // These should not be included in production.
    ReactDebugCurrentFrame: ReactDebugCurrentFrame,
    // Shim for React DOM 16.0.0 which still destructured (but not used) this.
    // TODO: remove in React 17.0.
    ReactComponentTreeHook: {}
  });
}

// by calls to these methods by a Babel plugin.
//
// In PROD (or in packages without access to React internals),
// they are left as they are instead.

function warn(format) {
  {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    printWarning('warn', format, args);
  }
}
function error(format) {
  {
    for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    printWarning('error', format, args);
  }
}

function printWarning(level, format, args) {
  // When changing this logic, you might want to also
  // update consoleWithStackDev.www.js as well.
  {
    var hasExistingStack = args.length > 0 && typeof args[args.length - 1] === 'string' && args[args.length - 1].indexOf('\n    in') === 0;

    if (!hasExistingStack) {
      var ReactDebugCurrentFrame = ReactSharedInternals.ReactDebugCurrentFrame;
      var stack = ReactDebugCurrentFrame.getStackAddendum();

      if (stack !== '') {
        format += '%s';
        args = args.concat([stack]);
      }
    }

    var argsWithFormat = args.map(function (item) {
      return '' + item;
    }); // Careful: RN currently depends on this prefix

    argsWithFormat.unshift('Warning: ' + format); // We intentionally don't use spread (or .apply) directly because it
    // breaks IE9: https://github.com/facebook/react/issues/13610
    // eslint-disable-next-line react-internal/no-production-logging

    Function.prototype.apply.call(console[level], console, argsWithFormat);

    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      var argIndex = 0;
      var message = 'Warning: ' + format.replace(/%s/g, function () {
        return args[argIndex++];
      });
      throw new Error(message);
    } catch (x) {}
  }
}

var didWarnStateUpdateForUnmountedComponent = {};

function warnNoop(publicInstance, callerName) {
  {
    var _constructor = publicInstance.constructor;
    var componentName = _constructor && (_constructor.displayName || _constructor.name) || 'ReactClass';
    var warningKey = componentName + "." + callerName;

    if (didWarnStateUpdateForUnmountedComponent[warningKey]) {
      return;
    }

    error("Can't call %s on a component that is not yet mounted. " + 'This is a no-op, but it might indicate a bug in your application. ' + 'Instead, assign to `this.state` directly or define a `state = {};` ' + 'class property with the desired state in the %s component.', callerName, componentName);

    didWarnStateUpdateForUnmountedComponent[warningKey] = true;
  }
}
/**
 * This is the abstract API for an update queue.
 */


var ReactNoopUpdateQueue = {
  /**
   * Checks whether or not this composite component is mounted.
   * @param {ReactClass} publicInstance The instance we want to test.
   * @return {boolean} True if mounted, false otherwise.
   * @protected
   * @final
   */
  isMounted: function (publicInstance) {
    return false;
  },

  /**
   * Forces an update. This should only be invoked when it is known with
   * certainty that we are **not** in a DOM transaction.
   *
   * You may want to call this when you know that some deeper aspect of the
   * component's state has changed but `setState` was not called.
   *
   * This will not invoke `shouldComponentUpdate`, but it will invoke
   * `componentWillUpdate` and `componentDidUpdate`.
   *
   * @param {ReactClass} publicInstance The instance that should rerender.
   * @param {?function} callback Called after component is updated.
   * @param {?string} callerName name of the calling function in the public API.
   * @internal
   */
  enqueueForceUpdate: function (publicInstance, callback, callerName) {
    warnNoop(publicInstance, 'forceUpdate');
  },

  /**
   * Replaces all of the state. Always use this or `setState` to mutate state.
   * You should treat `this.state` as immutable.
   *
   * There is no guarantee that `this.state` will be immediately updated, so
   * accessing `this.state` after calling this method may return the old value.
   *
   * @param {ReactClass} publicInstance The instance that should rerender.
   * @param {object} completeState Next state.
   * @param {?function} callback Called after component is updated.
   * @param {?string} callerName name of the calling function in the public API.
   * @internal
   */
  enqueueReplaceState: function (publicInstance, completeState, callback, callerName) {
    warnNoop(publicInstance, 'replaceState');
  },

  /**
   * Sets a subset of the state. This only exists because _pendingState is
   * internal. This provides a merging strategy that is not available to deep
   * properties which is confusing. TODO: Expose pendingState or don't use it
   * during the merge.
   *
   * @param {ReactClass} publicInstance The instance that should rerender.
   * @param {object} partialState Next partial state to be merged with state.
   * @param {?function} callback Called after component is updated.
   * @param {?string} Name of the calling function in the public API.
   * @internal
   */
  enqueueSetState: function (publicInstance, partialState, callback, callerName) {
    warnNoop(publicInstance, 'setState');
  }
};

var emptyObject = {};

{
  Object.freeze(emptyObject);
}
/**
 * Base class helpers for the updating state of a component.
 */


function Component(props, context, updater) {
  this.props = props;
  this.context = context; // If a component has string refs, we will assign a different object later.

  this.refs = emptyObject; // We initialize the default updater but the real one gets injected by the
  // renderer.

  this.updater = updater || ReactNoopUpdateQueue;
}

Component.prototype.isReactComponent = {};
/**
 * Sets a subset of the state. Always use this to mutate
 * state. You should treat `this.state` as immutable.
 *
 * There is no guarantee that `this.state` will be immediately updated, so
 * accessing `this.state` after calling this method may return the old value.
 *
 * There is no guarantee that calls to `setState` will run synchronously,
 * as they may eventually be batched together.  You can provide an optional
 * callback that will be executed when the call to setState is actually
 * completed.
 *
 * When a function is provided to setState, it will be called at some point in
 * the future (not synchronously). It will be called with the up to date
 * component arguments (state, props, context). These values can be different
 * from this.* because your function may be called after receiveProps but before
 * shouldComponentUpdate, and this new state, props, and context will not yet be
 * assigned to this.
 *
 * @param {object|function} partialState Next partial state or function to
 *        produce next partial state to be merged with current state.
 * @param {?function} callback Called after state is updated.
 * @final
 * @protected
 */

Component.prototype.setState = function (partialState, callback) {
  if (!(typeof partialState === 'object' || typeof partialState === 'function' || partialState == null)) {
    {
      throw Error( "setState(...): takes an object of state variables to update or a function which returns an object of state variables." );
    }
  }

  this.updater.enqueueSetState(this, partialState, callback, 'setState');
};
/**
 * Forces an update. This should only be invoked when it is known with
 * certainty that we are **not** in a DOM transaction.
 *
 * You may want to call this when you know that some deeper aspect of the
 * component's state has changed but `setState` was not called.
 *
 * This will not invoke `shouldComponentUpdate`, but it will invoke
 * `componentWillUpdate` and `componentDidUpdate`.
 *
 * @param {?function} callback Called after update is complete.
 * @final
 * @protected
 */


Component.prototype.forceUpdate = function (callback) {
  this.updater.enqueueForceUpdate(this, callback, 'forceUpdate');
};
/**
 * Deprecated APIs. These APIs used to exist on classic React classes but since
 * we would like to deprecate them, we're not going to move them over to this
 * modern base class. Instead, we define a getter that warns if it's accessed.
 */


{
  var deprecatedAPIs = {
    isMounted: ['isMounted', 'Instead, make sure to clean up subscriptions and pending requests in ' + 'componentWillUnmount to prevent memory leaks.'],
    replaceState: ['replaceState', 'Refactor your code to use setState instead (see ' + 'https://github.com/facebook/react/issues/3236).']
  };

  var defineDeprecationWarning = function (methodName, info) {
    Object.defineProperty(Component.prototype, methodName, {
      get: function () {
        warn('%s(...) is deprecated in plain JavaScript React classes. %s', info[0], info[1]);

        return undefined;
      }
    });
  };

  for (var fnName in deprecatedAPIs) {
    if (deprecatedAPIs.hasOwnProperty(fnName)) {
      defineDeprecationWarning(fnName, deprecatedAPIs[fnName]);
    }
  }
}

function ComponentDummy() {}

ComponentDummy.prototype = Component.prototype;
/**
 * Convenience component with default shallow equality check for sCU.
 */

function PureComponent(props, context, updater) {
  this.props = props;
  this.context = context; // If a component has string refs, we will assign a different object later.

  this.refs = emptyObject;
  this.updater = updater || ReactNoopUpdateQueue;
}

var pureComponentPrototype = PureComponent.prototype = new ComponentDummy();
pureComponentPrototype.constructor = PureComponent; // Avoid an extra prototype jump for these methods.

_assign(pureComponentPrototype, Component.prototype);

pureComponentPrototype.isPureReactComponent = true;

// an immutable object with a single mutable value
function createRef() {
  var refObject = {
    current: null
  };

  {
    Object.seal(refObject);
  }

  return refObject;
}

var hasOwnProperty = Object.prototype.hasOwnProperty;
var RESERVED_PROPS = {
  key: true,
  ref: true,
  __self: true,
  __source: true
};
var specialPropKeyWarningShown, specialPropRefWarningShown, didWarnAboutStringRefs;

{
  didWarnAboutStringRefs = {};
}

function hasValidRef(config) {
  {
    if (hasOwnProperty.call(config, 'ref')) {
      var getter = Object.getOwnPropertyDescriptor(config, 'ref').get;

      if (getter && getter.isReactWarning) {
        return false;
      }
    }
  }

  return config.ref !== undefined;
}

function hasValidKey(config) {
  {
    if (hasOwnProperty.call(config, 'key')) {
      var getter = Object.getOwnPropertyDescriptor(config, 'key').get;

      if (getter && getter.isReactWarning) {
        return false;
      }
    }
  }

  return config.key !== undefined;
}

function defineKeyPropWarningGetter(props, displayName) {
  var warnAboutAccessingKey = function () {
    {
      if (!specialPropKeyWarningShown) {
        specialPropKeyWarningShown = true;

        error('%s: `key` is not a prop. Trying to access it will result ' + 'in `undefined` being returned. If you need to access the same ' + 'value within the child component, you should pass it as a different ' + 'prop. (https://fb.me/react-special-props)', displayName);
      }
    }
  };

  warnAboutAccessingKey.isReactWarning = true;
  Object.defineProperty(props, 'key', {
    get: warnAboutAccessingKey,
    configurable: true
  });
}

function defineRefPropWarningGetter(props, displayName) {
  var warnAboutAccessingRef = function () {
    {
      if (!specialPropRefWarningShown) {
        specialPropRefWarningShown = true;

        error('%s: `ref` is not a prop. Trying to access it will result ' + 'in `undefined` being returned. If you need to access the same ' + 'value within the child component, you should pass it as a different ' + 'prop. (https://fb.me/react-special-props)', displayName);
      }
    }
  };

  warnAboutAccessingRef.isReactWarning = true;
  Object.defineProperty(props, 'ref', {
    get: warnAboutAccessingRef,
    configurable: true
  });
}

function warnIfStringRefCannotBeAutoConverted(config) {
  {
    if (typeof config.ref === 'string' && ReactCurrentOwner.current && config.__self && ReactCurrentOwner.current.stateNode !== config.__self) {
      var componentName = getComponentName(ReactCurrentOwner.current.type);

      if (!didWarnAboutStringRefs[componentName]) {
        error('Component "%s" contains the string ref "%s". ' + 'Support for string refs will be removed in a future major release. ' + 'This case cannot be automatically converted to an arrow function. ' + 'We ask you to manually fix this case by using useRef() or createRef() instead. ' + 'Learn more about using refs safely here: ' + 'https://fb.me/react-strict-mode-string-ref', getComponentName(ReactCurrentOwner.current.type), config.ref);

        didWarnAboutStringRefs[componentName] = true;
      }
    }
  }
}
/**
 * Factory method to create a new React element. This no longer adheres to
 * the class pattern, so do not use new to call it. Also, instanceof check
 * will not work. Instead test $$typeof field against Symbol.for('react.element') to check
 * if something is a React Element.
 *
 * @param {*} type
 * @param {*} props
 * @param {*} key
 * @param {string|object} ref
 * @param {*} owner
 * @param {*} self A *temporary* helper to detect places where `this` is
 * different from the `owner` when React.createElement is called, so that we
 * can warn. We want to get rid of owner and replace string `ref`s with arrow
 * functions, and as long as `this` and owner are the same, there will be no
 * change in behavior.
 * @param {*} source An annotation object (added by a transpiler or otherwise)
 * indicating filename, line number, and/or other information.
 * @internal
 */


var ReactElement = function (type, key, ref, self, source, owner, props) {
  var element = {
    // This tag allows us to uniquely identify this as a React Element
    $$typeof: REACT_ELEMENT_TYPE,
    // Built-in properties that belong on the element
    type: type,
    key: key,
    ref: ref,
    props: props,
    // Record the component responsible for creating this element.
    _owner: owner
  };

  {
    // The validation flag is currently mutative. We put it on
    // an external backing store so that we can freeze the whole object.
    // This can be replaced with a WeakMap once they are implemented in
    // commonly used development environments.
    element._store = {}; // To make comparing ReactElements easier for testing purposes, we make
    // the validation flag non-enumerable (where possible, which should
    // include every environment we run tests in), so the test framework
    // ignores it.

    Object.defineProperty(element._store, 'validated', {
      configurable: false,
      enumerable: false,
      writable: true,
      value: false
    }); // self and source are DEV only properties.

    Object.defineProperty(element, '_self', {
      configurable: false,
      enumerable: false,
      writable: false,
      value: self
    }); // Two elements created in two different places should be considered
    // equal for testing purposes and therefore we hide it from enumeration.

    Object.defineProperty(element, '_source', {
      configurable: false,
      enumerable: false,
      writable: false,
      value: source
    });

    if (Object.freeze) {
      Object.freeze(element.props);
      Object.freeze(element);
    }
  }

  return element;
};
/**
 * Create and return a new ReactElement of the given type.
 * See https://reactjs.org/docs/react-api.html#createelement
 */

function createElement(type, config, children) {
  var propName; // Reserved names are extracted

  var props = {};
  var key = null;
  var ref = null;
  var self = null;
  var source = null;

  if (config != null) {
    if (hasValidRef(config)) {
      ref = config.ref;

      {
        warnIfStringRefCannotBeAutoConverted(config);
      }
    }

    if (hasValidKey(config)) {
      key = '' + config.key;
    }

    self = config.__self === undefined ? null : config.__self;
    source = config.__source === undefined ? null : config.__source; // Remaining properties are added to a new props object

    for (propName in config) {
      if (hasOwnProperty.call(config, propName) && !RESERVED_PROPS.hasOwnProperty(propName)) {
        props[propName] = config[propName];
      }
    }
  } // Children can be more than one argument, and those are transferred onto
  // the newly allocated props object.


  var childrenLength = arguments.length - 2;

  if (childrenLength === 1) {
    props.children = children;
  } else if (childrenLength > 1) {
    var childArray = Array(childrenLength);

    for (var i = 0; i < childrenLength; i++) {
      childArray[i] = arguments[i + 2];
    }

    {
      if (Object.freeze) {
        Object.freeze(childArray);
      }
    }

    props.children = childArray;
  } // Resolve default props


  if (type && type.defaultProps) {
    var defaultProps = type.defaultProps;

    for (propName in defaultProps) {
      if (props[propName] === undefined) {
        props[propName] = defaultProps[propName];
      }
    }
  }

  {
    if (key || ref) {
      var displayName = typeof type === 'function' ? type.displayName || type.name || 'Unknown' : type;

      if (key) {
        defineKeyPropWarningGetter(props, displayName);
      }

      if (ref) {
        defineRefPropWarningGetter(props, displayName);
      }
    }
  }

  return ReactElement(type, key, ref, self, source, ReactCurrentOwner.current, props);
}
function cloneAndReplaceKey(oldElement, newKey) {
  var newElement = ReactElement(oldElement.type, newKey, oldElement.ref, oldElement._self, oldElement._source, oldElement._owner, oldElement.props);
  return newElement;
}
/**
 * Clone and return a new ReactElement using element as the starting point.
 * See https://reactjs.org/docs/react-api.html#cloneelement
 */

function cloneElement(element, config, children) {
  if (!!(element === null || element === undefined)) {
    {
      throw Error( "React.cloneElement(...): The argument must be a React element, but you passed " + element + "." );
    }
  }

  var propName; // Original props are copied

  var props = _assign({}, element.props); // Reserved names are extracted


  var key = element.key;
  var ref = element.ref; // Self is preserved since the owner is preserved.

  var self = element._self; // Source is preserved since cloneElement is unlikely to be targeted by a
  // transpiler, and the original source is probably a better indicator of the
  // true owner.

  var source = element._source; // Owner will be preserved, unless ref is overridden

  var owner = element._owner;

  if (config != null) {
    if (hasValidRef(config)) {
      // Silently steal the ref from the parent.
      ref = config.ref;
      owner = ReactCurrentOwner.current;
    }

    if (hasValidKey(config)) {
      key = '' + config.key;
    } // Remaining properties override existing props


    var defaultProps;

    if (element.type && element.type.defaultProps) {
      defaultProps = element.type.defaultProps;
    }

    for (propName in config) {
      if (hasOwnProperty.call(config, propName) && !RESERVED_PROPS.hasOwnProperty(propName)) {
        if (config[propName] === undefined && defaultProps !== undefined) {
          // Resolve default props
          props[propName] = defaultProps[propName];
        } else {
          props[propName] = config[propName];
        }
      }
    }
  } // Children can be more than one argument, and those are transferred onto
  // the newly allocated props object.


  var childrenLength = arguments.length - 2;

  if (childrenLength === 1) {
    props.children = children;
  } else if (childrenLength > 1) {
    var childArray = Array(childrenLength);

    for (var i = 0; i < childrenLength; i++) {
      childArray[i] = arguments[i + 2];
    }

    props.children = childArray;
  }

  return ReactElement(element.type, key, ref, self, source, owner, props);
}
/**
 * Verifies the object is a ReactElement.
 * See https://reactjs.org/docs/react-api.html#isvalidelement
 * @param {?object} object
 * @return {boolean} True if `object` is a ReactElement.
 * @final
 */

function isValidElement(object) {
  return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
}

var SEPARATOR = '.';
var SUBSEPARATOR = ':';
/**
 * Escape and wrap key so it is safe to use as a reactid
 *
 * @param {string} key to be escaped.
 * @return {string} the escaped key.
 */

function escape(key) {
  var escapeRegex = /[=:]/g;
  var escaperLookup = {
    '=': '=0',
    ':': '=2'
  };
  var escapedString = ('' + key).replace(escapeRegex, function (match) {
    return escaperLookup[match];
  });
  return '$' + escapedString;
}
/**
 * TODO: Test that a single child and an array with one item have the same key
 * pattern.
 */


var didWarnAboutMaps = false;
var userProvidedKeyEscapeRegex = /\/+/g;

function escapeUserProvidedKey(text) {
  return ('' + text).replace(userProvidedKeyEscapeRegex, '$&/');
}

var POOL_SIZE = 10;
var traverseContextPool = [];

function getPooledTraverseContext(mapResult, keyPrefix, mapFunction, mapContext) {
  if (traverseContextPool.length) {
    var traverseContext = traverseContextPool.pop();
    traverseContext.result = mapResult;
    traverseContext.keyPrefix = keyPrefix;
    traverseContext.func = mapFunction;
    traverseContext.context = mapContext;
    traverseContext.count = 0;
    return traverseContext;
  } else {
    return {
      result: mapResult,
      keyPrefix: keyPrefix,
      func: mapFunction,
      context: mapContext,
      count: 0
    };
  }
}

function releaseTraverseContext(traverseContext) {
  traverseContext.result = null;
  traverseContext.keyPrefix = null;
  traverseContext.func = null;
  traverseContext.context = null;
  traverseContext.count = 0;

  if (traverseContextPool.length < POOL_SIZE) {
    traverseContextPool.push(traverseContext);
  }
}
/**
 * @param {?*} children Children tree container.
 * @param {!string} nameSoFar Name of the key path so far.
 * @param {!function} callback Callback to invoke with each child found.
 * @param {?*} traverseContext Used to pass information throughout the traversal
 * process.
 * @return {!number} The number of children in this subtree.
 */


function traverseAllChildrenImpl(children, nameSoFar, callback, traverseContext) {
  var type = typeof children;

  if (type === 'undefined' || type === 'boolean') {
    // All of the above are perceived as null.
    children = null;
  }

  var invokeCallback = false;

  if (children === null) {
    invokeCallback = true;
  } else {
    switch (type) {
      case 'string':
      case 'number':
        invokeCallback = true;
        break;

      case 'object':
        switch (children.$$typeof) {
          case REACT_ELEMENT_TYPE:
          case REACT_PORTAL_TYPE:
            invokeCallback = true;
        }

    }
  }

  if (invokeCallback) {
    callback(traverseContext, children, // If it's the only child, treat the name as if it was wrapped in an array
    // so that it's consistent if the number of children grows.
    nameSoFar === '' ? SEPARATOR + getComponentKey(children, 0) : nameSoFar);
    return 1;
  }

  var child;
  var nextName;
  var subtreeCount = 0; // Count of children found in the current subtree.

  var nextNamePrefix = nameSoFar === '' ? SEPARATOR : nameSoFar + SUBSEPARATOR;

  if (Array.isArray(children)) {
    for (var i = 0; i < children.length; i++) {
      child = children[i];
      nextName = nextNamePrefix + getComponentKey(child, i);
      subtreeCount += traverseAllChildrenImpl(child, nextName, callback, traverseContext);
    }
  } else {
    var iteratorFn = getIteratorFn(children);

    if (typeof iteratorFn === 'function') {

      {
        // Warn about using Maps as children
        if (iteratorFn === children.entries) {
          if (!didWarnAboutMaps) {
            warn('Using Maps as children is deprecated and will be removed in ' + 'a future major release. Consider converting children to ' + 'an array of keyed ReactElements instead.');
          }

          didWarnAboutMaps = true;
        }
      }

      var iterator = iteratorFn.call(children);
      var step;
      var ii = 0;

      while (!(step = iterator.next()).done) {
        child = step.value;
        nextName = nextNamePrefix + getComponentKey(child, ii++);
        subtreeCount += traverseAllChildrenImpl(child, nextName, callback, traverseContext);
      }
    } else if (type === 'object') {
      var addendum = '';

      {
        addendum = ' If you meant to render a collection of children, use an array ' + 'instead.' + ReactDebugCurrentFrame.getStackAddendum();
      }

      var childrenString = '' + children;

      {
        {
          throw Error( "Objects are not valid as a React child (found: " + (childrenString === '[object Object]' ? 'object with keys {' + Object.keys(children).join(', ') + '}' : childrenString) + ")." + addendum );
        }
      }
    }
  }

  return subtreeCount;
}
/**
 * Traverses children that are typically specified as `props.children`, but
 * might also be specified through attributes:
 *
 * - `traverseAllChildren(this.props.children, ...)`
 * - `traverseAllChildren(this.props.leftPanelChildren, ...)`
 *
 * The `traverseContext` is an optional argument that is passed through the
 * entire traversal. It can be used to store accumulations or anything else that
 * the callback might find relevant.
 *
 * @param {?*} children Children tree object.
 * @param {!function} callback To invoke upon traversing each child.
 * @param {?*} traverseContext Context for traversal.
 * @return {!number} The number of children in this subtree.
 */


function traverseAllChildren(children, callback, traverseContext) {
  if (children == null) {
    return 0;
  }

  return traverseAllChildrenImpl(children, '', callback, traverseContext);
}
/**
 * Generate a key string that identifies a component within a set.
 *
 * @param {*} component A component that could contain a manual key.
 * @param {number} index Index that is used if a manual key is not provided.
 * @return {string}
 */


function getComponentKey(component, index) {
  // Do some typechecking here since we call this blindly. We want to ensure
  // that we don't block potential future ES APIs.
  if (typeof component === 'object' && component !== null && component.key != null) {
    // Explicit key
    return escape(component.key);
  } // Implicit key determined by the index in the set


  return index.toString(36);
}

function forEachSingleChild(bookKeeping, child, name) {
  var func = bookKeeping.func,
      context = bookKeeping.context;
  func.call(context, child, bookKeeping.count++);
}
/**
 * Iterates through children that are typically specified as `props.children`.
 *
 * See https://reactjs.org/docs/react-api.html#reactchildrenforeach
 *
 * The provided forEachFunc(child, index) will be called for each
 * leaf child.
 *
 * @param {?*} children Children tree container.
 * @param {function(*, int)} forEachFunc
 * @param {*} forEachContext Context for forEachContext.
 */


function forEachChildren(children, forEachFunc, forEachContext) {
  if (children == null) {
    return children;
  }

  var traverseContext = getPooledTraverseContext(null, null, forEachFunc, forEachContext);
  traverseAllChildren(children, forEachSingleChild, traverseContext);
  releaseTraverseContext(traverseContext);
}

function mapSingleChildIntoContext(bookKeeping, child, childKey) {
  var result = bookKeeping.result,
      keyPrefix = bookKeeping.keyPrefix,
      func = bookKeeping.func,
      context = bookKeeping.context;
  var mappedChild = func.call(context, child, bookKeeping.count++);

  if (Array.isArray(mappedChild)) {
    mapIntoWithKeyPrefixInternal(mappedChild, result, childKey, function (c) {
      return c;
    });
  } else if (mappedChild != null) {
    if (isValidElement(mappedChild)) {
      mappedChild = cloneAndReplaceKey(mappedChild, // Keep both the (mapped) and old keys if they differ, just as
      // traverseAllChildren used to do for objects as children
      keyPrefix + (mappedChild.key && (!child || child.key !== mappedChild.key) ? escapeUserProvidedKey(mappedChild.key) + '/' : '') + childKey);
    }

    result.push(mappedChild);
  }
}

function mapIntoWithKeyPrefixInternal(children, array, prefix, func, context) {
  var escapedPrefix = '';

  if (prefix != null) {
    escapedPrefix = escapeUserProvidedKey(prefix) + '/';
  }

  var traverseContext = getPooledTraverseContext(array, escapedPrefix, func, context);
  traverseAllChildren(children, mapSingleChildIntoContext, traverseContext);
  releaseTraverseContext(traverseContext);
}
/**
 * Maps children that are typically specified as `props.children`.
 *
 * See https://reactjs.org/docs/react-api.html#reactchildrenmap
 *
 * The provided mapFunction(child, key, index) will be called for each
 * leaf child.
 *
 * @param {?*} children Children tree container.
 * @param {function(*, int)} func The map function.
 * @param {*} context Context for mapFunction.
 * @return {object} Object containing the ordered map of results.
 */


function mapChildren(children, func, context) {
  if (children == null) {
    return children;
  }

  var result = [];
  mapIntoWithKeyPrefixInternal(children, result, null, func, context);
  return result;
}
/**
 * Count the number of children that are typically specified as
 * `props.children`.
 *
 * See https://reactjs.org/docs/react-api.html#reactchildrencount
 *
 * @param {?*} children Children tree container.
 * @return {number} The number of children.
 */


function countChildren(children) {
  return traverseAllChildren(children, function () {
    return null;
  }, null);
}
/**
 * Flatten a children object (typically specified as `props.children`) and
 * return an array with appropriately re-keyed children.
 *
 * See https://reactjs.org/docs/react-api.html#reactchildrentoarray
 */


function toArray(children) {
  var result = [];
  mapIntoWithKeyPrefixInternal(children, result, null, function (child) {
    return child;
  });
  return result;
}
/**
 * Returns the first child in a collection of children and verifies that there
 * is only one child in the collection.
 *
 * See https://reactjs.org/docs/react-api.html#reactchildrenonly
 *
 * The current implementation of this function assumes that a single child gets
 * passed without a wrapper, but the purpose of this helper function is to
 * abstract away the particular structure of children.
 *
 * @param {?object} children Child collection structure.
 * @return {ReactElement} The first and only `ReactElement` contained in the
 * structure.
 */


function onlyChild(children) {
  if (!isValidElement(children)) {
    {
      throw Error( "React.Children.only expected to receive a single React element child." );
    }
  }

  return children;
}

function createContext(defaultValue, calculateChangedBits) {
  if (calculateChangedBits === undefined) {
    calculateChangedBits = null;
  } else {
    {
      if (calculateChangedBits !== null && typeof calculateChangedBits !== 'function') {
        error('createContext: Expected the optional second argument to be a ' + 'function. Instead received: %s', calculateChangedBits);
      }
    }
  }

  var context = {
    $$typeof: REACT_CONTEXT_TYPE,
    _calculateChangedBits: calculateChangedBits,
    // As a workaround to support multiple concurrent renderers, we categorize
    // some renderers as primary and others as secondary. We only expect
    // there to be two concurrent renderers at most: React Native (primary) and
    // Fabric (secondary); React DOM (primary) and React ART (secondary).
    // Secondary renderers store their context values on separate fields.
    _currentValue: defaultValue,
    _currentValue2: defaultValue,
    // Used to track how many concurrent renderers this context currently
    // supports within in a single renderer. Such as parallel server rendering.
    _threadCount: 0,
    // These are circular
    Provider: null,
    Consumer: null
  };
  context.Provider = {
    $$typeof: REACT_PROVIDER_TYPE,
    _context: context
  };
  var hasWarnedAboutUsingNestedContextConsumers = false;
  var hasWarnedAboutUsingConsumerProvider = false;

  {
    // A separate object, but proxies back to the original context object for
    // backwards compatibility. It has a different $$typeof, so we can properly
    // warn for the incorrect usage of Context as a Consumer.
    var Consumer = {
      $$typeof: REACT_CONTEXT_TYPE,
      _context: context,
      _calculateChangedBits: context._calculateChangedBits
    }; // $FlowFixMe: Flow complains about not setting a value, which is intentional here

    Object.defineProperties(Consumer, {
      Provider: {
        get: function () {
          if (!hasWarnedAboutUsingConsumerProvider) {
            hasWarnedAboutUsingConsumerProvider = true;

            error('Rendering <Context.Consumer.Provider> is not supported and will be removed in ' + 'a future major release. Did you mean to render <Context.Provider> instead?');
          }

          return context.Provider;
        },
        set: function (_Provider) {
          context.Provider = _Provider;
        }
      },
      _currentValue: {
        get: function () {
          return context._currentValue;
        },
        set: function (_currentValue) {
          context._currentValue = _currentValue;
        }
      },
      _currentValue2: {
        get: function () {
          return context._currentValue2;
        },
        set: function (_currentValue2) {
          context._currentValue2 = _currentValue2;
        }
      },
      _threadCount: {
        get: function () {
          return context._threadCount;
        },
        set: function (_threadCount) {
          context._threadCount = _threadCount;
        }
      },
      Consumer: {
        get: function () {
          if (!hasWarnedAboutUsingNestedContextConsumers) {
            hasWarnedAboutUsingNestedContextConsumers = true;

            error('Rendering <Context.Consumer.Consumer> is not supported and will be removed in ' + 'a future major release. Did you mean to render <Context.Consumer> instead?');
          }

          return context.Consumer;
        }
      }
    }); // $FlowFixMe: Flow complains about missing properties because it doesn't understand defineProperty

    context.Consumer = Consumer;
  }

  {
    context._currentRenderer = null;
    context._currentRenderer2 = null;
  }

  return context;
}

function lazy(ctor) {
  var lazyType = {
    $$typeof: REACT_LAZY_TYPE,
    _ctor: ctor,
    // React uses these fields to store the result.
    _status: -1,
    _result: null
  };

  {
    // In production, this would just set it on the object.
    var defaultProps;
    var propTypes;
    Object.defineProperties(lazyType, {
      defaultProps: {
        configurable: true,
        get: function () {
          return defaultProps;
        },
        set: function (newDefaultProps) {
          error('React.lazy(...): It is not supported to assign `defaultProps` to ' + 'a lazy component import. Either specify them where the component ' + 'is defined, or create a wrapping component around it.');

          defaultProps = newDefaultProps; // Match production behavior more closely:

          Object.defineProperty(lazyType, 'defaultProps', {
            enumerable: true
          });
        }
      },
      propTypes: {
        configurable: true,
        get: function () {
          return propTypes;
        },
        set: function (newPropTypes) {
          error('React.lazy(...): It is not supported to assign `propTypes` to ' + 'a lazy component import. Either specify them where the component ' + 'is defined, or create a wrapping component around it.');

          propTypes = newPropTypes; // Match production behavior more closely:

          Object.defineProperty(lazyType, 'propTypes', {
            enumerable: true
          });
        }
      }
    });
  }

  return lazyType;
}

function forwardRef(render) {
  {
    if (render != null && render.$$typeof === REACT_MEMO_TYPE) {
      error('forwardRef requires a render function but received a `memo` ' + 'component. Instead of forwardRef(memo(...)), use ' + 'memo(forwardRef(...)).');
    } else if (typeof render !== 'function') {
      error('forwardRef requires a render function but was given %s.', render === null ? 'null' : typeof render);
    } else {
      if (render.length !== 0 && render.length !== 2) {
        error('forwardRef render functions accept exactly two parameters: props and ref. %s', render.length === 1 ? 'Did you forget to use the ref parameter?' : 'Any additional parameter will be undefined.');
      }
    }

    if (render != null) {
      if (render.defaultProps != null || render.propTypes != null) {
        error('forwardRef render functions do not support propTypes or defaultProps. ' + 'Did you accidentally pass a React component?');
      }
    }
  }

  return {
    $$typeof: REACT_FORWARD_REF_TYPE,
    render: render
  };
}

function isValidElementType(type) {
  return typeof type === 'string' || typeof type === 'function' || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
  type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);
}

function memo(type, compare) {
  {
    if (!isValidElementType(type)) {
      error('memo: The first argument must be a component. Instead ' + 'received: %s', type === null ? 'null' : typeof type);
    }
  }

  return {
    $$typeof: REACT_MEMO_TYPE,
    type: type,
    compare: compare === undefined ? null : compare
  };
}

function resolveDispatcher() {
  var dispatcher = ReactCurrentDispatcher.current;

  if (!(dispatcher !== null)) {
    {
      throw Error( "Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:\n1. You might have mismatching versions of React and the renderer (such as React DOM)\n2. You might be breaking the Rules of Hooks\n3. You might have more than one copy of React in the same app\nSee https://fb.me/react-invalid-hook-call for tips about how to debug and fix this problem." );
    }
  }

  return dispatcher;
}

function useContext(Context, unstable_observedBits) {
  var dispatcher = resolveDispatcher();

  {
    if (unstable_observedBits !== undefined) {
      error('useContext() second argument is reserved for future ' + 'use in React. Passing it is not supported. ' + 'You passed: %s.%s', unstable_observedBits, typeof unstable_observedBits === 'number' && Array.isArray(arguments[2]) ? '\n\nDid you call array.map(useContext)? ' + 'Calling Hooks inside a loop is not supported. ' + 'Learn more at https://fb.me/rules-of-hooks' : '');
    } // TODO: add a more generic warning for invalid values.


    if (Context._context !== undefined) {
      var realContext = Context._context; // Don't deduplicate because this legitimately causes bugs
      // and nobody should be using this in existing code.

      if (realContext.Consumer === Context) {
        error('Calling useContext(Context.Consumer) is not supported, may cause bugs, and will be ' + 'removed in a future major release. Did you mean to call useContext(Context) instead?');
      } else if (realContext.Provider === Context) {
        error('Calling useContext(Context.Provider) is not supported. ' + 'Did you mean to call useContext(Context) instead?');
      }
    }
  }

  return dispatcher.useContext(Context, unstable_observedBits);
}
function useState(initialState) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useState(initialState);
}
function useReducer(reducer, initialArg, init) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useReducer(reducer, initialArg, init);
}
function useRef(initialValue) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useRef(initialValue);
}
function useEffect(create, deps) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useEffect(create, deps);
}
function useLayoutEffect(create, deps) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useLayoutEffect(create, deps);
}
function useCallback(callback, deps) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useCallback(callback, deps);
}
function useMemo(create, deps) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useMemo(create, deps);
}
function useImperativeHandle(ref, create, deps) {
  var dispatcher = resolveDispatcher();
  return dispatcher.useImperativeHandle(ref, create, deps);
}
function useDebugValue(value, formatterFn) {
  {
    var dispatcher = resolveDispatcher();
    return dispatcher.useDebugValue(value, formatterFn);
  }
}

var propTypesMisspellWarningShown;

{
  propTypesMisspellWarningShown = false;
}

function getDeclarationErrorAddendum() {
  if (ReactCurrentOwner.current) {
    var name = getComponentName(ReactCurrentOwner.current.type);

    if (name) {
      return '\n\nCheck the render method of `' + name + '`.';
    }
  }

  return '';
}

function getSourceInfoErrorAddendum(source) {
  if (source !== undefined) {
    var fileName = source.fileName.replace(/^.*[\\\/]/, '');
    var lineNumber = source.lineNumber;
    return '\n\nCheck your code at ' + fileName + ':' + lineNumber + '.';
  }

  return '';
}

function getSourceInfoErrorAddendumForProps(elementProps) {
  if (elementProps !== null && elementProps !== undefined) {
    return getSourceInfoErrorAddendum(elementProps.__source);
  }

  return '';
}
/**
 * Warn if there's no key explicitly set on dynamic arrays of children or
 * object keys are not valid. This allows us to keep track of children between
 * updates.
 */


var ownerHasKeyUseWarning = {};

function getCurrentComponentErrorInfo(parentType) {
  var info = getDeclarationErrorAddendum();

  if (!info) {
    var parentName = typeof parentType === 'string' ? parentType : parentType.displayName || parentType.name;

    if (parentName) {
      info = "\n\nCheck the top-level render call using <" + parentName + ">.";
    }
  }

  return info;
}
/**
 * Warn if the element doesn't have an explicit key assigned to it.
 * This element is in an array. The array could grow and shrink or be
 * reordered. All children that haven't already been validated are required to
 * have a "key" property assigned to it. Error statuses are cached so a warning
 * will only be shown once.
 *
 * @internal
 * @param {ReactElement} element Element that requires a key.
 * @param {*} parentType element's parent's type.
 */


function validateExplicitKey(element, parentType) {
  if (!element._store || element._store.validated || element.key != null) {
    return;
  }

  element._store.validated = true;
  var currentComponentErrorInfo = getCurrentComponentErrorInfo(parentType);

  if (ownerHasKeyUseWarning[currentComponentErrorInfo]) {
    return;
  }

  ownerHasKeyUseWarning[currentComponentErrorInfo] = true; // Usually the current owner is the offender, but if it accepts children as a
  // property, it may be the creator of the child that's responsible for
  // assigning it a key.

  var childOwner = '';

  if (element && element._owner && element._owner !== ReactCurrentOwner.current) {
    // Give the component that originally created this child.
    childOwner = " It was passed a child from " + getComponentName(element._owner.type) + ".";
  }

  setCurrentlyValidatingElement(element);

  {
    error('Each child in a list should have a unique "key" prop.' + '%s%s See https://fb.me/react-warning-keys for more information.', currentComponentErrorInfo, childOwner);
  }

  setCurrentlyValidatingElement(null);
}
/**
 * Ensure that every element either is passed in a static location, in an
 * array with an explicit keys property defined, or in an object literal
 * with valid key property.
 *
 * @internal
 * @param {ReactNode} node Statically passed child of any type.
 * @param {*} parentType node's parent's type.
 */


function validateChildKeys(node, parentType) {
  if (typeof node !== 'object') {
    return;
  }

  if (Array.isArray(node)) {
    for (var i = 0; i < node.length; i++) {
      var child = node[i];

      if (isValidElement(child)) {
        validateExplicitKey(child, parentType);
      }
    }
  } else if (isValidElement(node)) {
    // This element was passed in a valid location.
    if (node._store) {
      node._store.validated = true;
    }
  } else if (node) {
    var iteratorFn = getIteratorFn(node);

    if (typeof iteratorFn === 'function') {
      // Entry iterators used to provide implicit keys,
      // but now we print a separate warning for them later.
      if (iteratorFn !== node.entries) {
        var iterator = iteratorFn.call(node);
        var step;

        while (!(step = iterator.next()).done) {
          if (isValidElement(step.value)) {
            validateExplicitKey(step.value, parentType);
          }
        }
      }
    }
  }
}
/**
 * Given an element, validate that its props follow the propTypes definition,
 * provided by the type.
 *
 * @param {ReactElement} element
 */


function validatePropTypes(element) {
  {
    var type = element.type;

    if (type === null || type === undefined || typeof type === 'string') {
      return;
    }

    var name = getComponentName(type);
    var propTypes;

    if (typeof type === 'function') {
      propTypes = type.propTypes;
    } else if (typeof type === 'object' && (type.$$typeof === REACT_FORWARD_REF_TYPE || // Note: Memo only checks outer props here.
    // Inner props are checked in the reconciler.
    type.$$typeof === REACT_MEMO_TYPE)) {
      propTypes = type.propTypes;
    } else {
      return;
    }

    if (propTypes) {
      setCurrentlyValidatingElement(element);
      checkPropTypes(propTypes, element.props, 'prop', name, ReactDebugCurrentFrame.getStackAddendum);
      setCurrentlyValidatingElement(null);
    } else if (type.PropTypes !== undefined && !propTypesMisspellWarningShown) {
      propTypesMisspellWarningShown = true;

      error('Component %s declared `PropTypes` instead of `propTypes`. Did you misspell the property assignment?', name || 'Unknown');
    }

    if (typeof type.getDefaultProps === 'function' && !type.getDefaultProps.isReactClassApproved) {
      error('getDefaultProps is only used on classic React.createClass ' + 'definitions. Use a static property named `defaultProps` instead.');
    }
  }
}
/**
 * Given a fragment, validate that it can only be provided with fragment props
 * @param {ReactElement} fragment
 */


function validateFragmentProps(fragment) {
  {
    setCurrentlyValidatingElement(fragment);
    var keys = Object.keys(fragment.props);

    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];

      if (key !== 'children' && key !== 'key') {
        error('Invalid prop `%s` supplied to `React.Fragment`. ' + 'React.Fragment can only have `key` and `children` props.', key);

        break;
      }
    }

    if (fragment.ref !== null) {
      error('Invalid attribute `ref` supplied to `React.Fragment`.');
    }

    setCurrentlyValidatingElement(null);
  }
}
function createElementWithValidation(type, props, children) {
  var validType = isValidElementType(type); // We warn in this case but don't throw. We expect the element creation to
  // succeed and there will likely be errors in render.

  if (!validType) {
    var info = '';

    if (type === undefined || typeof type === 'object' && type !== null && Object.keys(type).length === 0) {
      info += ' You likely forgot to export your component from the file ' + "it's defined in, or you might have mixed up default and named imports.";
    }

    var sourceInfo = getSourceInfoErrorAddendumForProps(props);

    if (sourceInfo) {
      info += sourceInfo;
    } else {
      info += getDeclarationErrorAddendum();
    }

    var typeString;

    if (type === null) {
      typeString = 'null';
    } else if (Array.isArray(type)) {
      typeString = 'array';
    } else if (type !== undefined && type.$$typeof === REACT_ELEMENT_TYPE) {
      typeString = "<" + (getComponentName(type.type) || 'Unknown') + " />";
      info = ' Did you accidentally export a JSX literal instead of a component?';
    } else {
      typeString = typeof type;
    }

    {
      error('React.createElement: type is invalid -- expected a string (for ' + 'built-in components) or a class/function (for composite ' + 'components) but got: %s.%s', typeString, info);
    }
  }

  var element = createElement.apply(this, arguments); // The result can be nullish if a mock or a custom function is used.
  // TODO: Drop this when these are no longer allowed as the type argument.

  if (element == null) {
    return element;
  } // Skip key warning if the type isn't valid since our key validation logic
  // doesn't expect a non-string/function type and can throw confusing errors.
  // We don't want exception behavior to differ between dev and prod.
  // (Rendering will throw with a helpful message and as soon as the type is
  // fixed, the key warnings will appear.)


  if (validType) {
    for (var i = 2; i < arguments.length; i++) {
      validateChildKeys(arguments[i], type);
    }
  }

  if (type === REACT_FRAGMENT_TYPE) {
    validateFragmentProps(element);
  } else {
    validatePropTypes(element);
  }

  return element;
}
var didWarnAboutDeprecatedCreateFactory = false;
function createFactoryWithValidation(type) {
  var validatedFactory = createElementWithValidation.bind(null, type);
  validatedFactory.type = type;

  {
    if (!didWarnAboutDeprecatedCreateFactory) {
      didWarnAboutDeprecatedCreateFactory = true;

      warn('React.createFactory() is deprecated and will be removed in ' + 'a future major release. Consider using JSX ' + 'or use React.createElement() directly instead.');
    } // Legacy hook: remove it


    Object.defineProperty(validatedFactory, 'type', {
      enumerable: false,
      get: function () {
        warn('Factory.type is deprecated. Access the class directly ' + 'before passing it to createFactory.');

        Object.defineProperty(this, 'type', {
          value: type
        });
        return type;
      }
    });
  }

  return validatedFactory;
}
function cloneElementWithValidation(element, props, children) {
  var newElement = cloneElement.apply(this, arguments);

  for (var i = 2; i < arguments.length; i++) {
    validateChildKeys(arguments[i], newElement.type);
  }

  validatePropTypes(newElement);
  return newElement;
}

{

  try {
    var frozenObject = Object.freeze({});
    var testMap = new Map([[frozenObject, null]]);
    var testSet = new Set([frozenObject]); // This is necessary for Rollup to not consider these unused.
    // https://github.com/rollup/rollup/issues/1771
    // TODO: we can remove these if Rollup fixes the bug.

    testMap.set(0, 0);
    testSet.add(0);
  } catch (e) {
  }
}

var createElement$1 =  createElementWithValidation ;
var cloneElement$1 =  cloneElementWithValidation ;
var createFactory =  createFactoryWithValidation ;
var Children = {
  map: mapChildren,
  forEach: forEachChildren,
  count: countChildren,
  toArray: toArray,
  only: onlyChild
};

exports.Children = Children;
exports.Component = Component;
exports.Fragment = REACT_FRAGMENT_TYPE;
exports.Profiler = REACT_PROFILER_TYPE;
exports.PureComponent = PureComponent;
exports.StrictMode = REACT_STRICT_MODE_TYPE;
exports.Suspense = REACT_SUSPENSE_TYPE;
exports.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = ReactSharedInternals;
exports.cloneElement = cloneElement$1;
exports.createContext = createContext;
exports.createElement = createElement$1;
exports.createFactory = createFactory;
exports.createRef = createRef;
exports.forwardRef = forwardRef;
exports.isValidElement = isValidElement;
exports.lazy = lazy;
exports.memo = memo;
exports.useCallback = useCallback;
exports.useContext = useContext;
exports.useDebugValue = useDebugValue;
exports.useEffect = useEffect;
exports.useImperativeHandle = useImperativeHandle;
exports.useLayoutEffect = useLayoutEffect;
exports.useMemo = useMemo;
exports.useReducer = useReducer;
exports.useRef = useRef;
exports.useState = useState;
exports.version = ReactVersion;
  })();
}
});
var react_development_1 = react_development.Children;
var react_development_2 = react_development.Component;
var react_development_3 = react_development.Fragment;
var react_development_4 = react_development.Profiler;
var react_development_5 = react_development.PureComponent;
var react_development_6 = react_development.StrictMode;
var react_development_7 = react_development.Suspense;
var react_development_8 = react_development.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;
var react_development_9 = react_development.cloneElement;
var react_development_10 = react_development.createContext;
var react_development_11 = react_development.createElement;
var react_development_12 = react_development.createFactory;
var react_development_13 = react_development.createRef;
var react_development_14 = react_development.forwardRef;
var react_development_15 = react_development.isValidElement;
var react_development_16 = react_development.lazy;
var react_development_17 = react_development.memo;
var react_development_18 = react_development.useCallback;
var react_development_19 = react_development.useContext;
var react_development_20 = react_development.useDebugValue;
var react_development_21 = react_development.useEffect;
var react_development_22 = react_development.useImperativeHandle;
var react_development_23 = react_development.useLayoutEffect;
var react_development_24 = react_development.useMemo;
var react_development_25 = react_development.useReducer;
var react_development_26 = react_development.useRef;
var react_development_27 = react_development.useState;
var react_development_28 = react_development.version;

var react = createCommonjsModule(function (module) {

if (process.env.NODE_ENV === 'production') {
  module.exports = react_production_min;
} else {
  module.exports = react_development;
}
});

var presets = {
    webpage: webpage,
    newsletter: newsletter,
};
var Editor = /** @class */ (function (_super) {
    __extends(Editor, _super);
    function Editor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Editor.prototype.componentDidMount = function () {
        var _a = this.props, id = _a.id, width = _a.width, height = _a.height, blockManager = _a.blockManager, storageManager = _a.storageManager, styleManager = _a.styleManager, presetType = _a.presetType, onInit = _a.onInit;
        var editor = grapesjs.init({
            container: "#" + id,
            fromElement: true,
            width: width,
            height: height,
            blockManager: blockManager,
            storageManager: storageManager,
            styleManager: styleManager,
            plugins: [
                presets[presetType],
            ],
        });
        if (typeof onInit === 'function') {
            onInit(editor);
        }
        editor.on('change:changesCount', this.handleOnUpdate.bind(this));
        this.setState({
            editor: editor,
        });
    };
    Editor.prototype.componentWillUnmount = function () {
        var editor = this.state.editor;
        var _a = this.props, onDestroy = _a.onDestroy, id = _a.id;
        if (editor) {
            if (typeof onDestroy === 'function') {
                onDestroy(editor);
            }
            setTimeout(function () {
                editor.destroy();
            }, 0);
            if (document) {
                var container = document.getElementById(id);
                if (container) {
                    container.innerHTML = '';
                }
            }
        }
    };
    Editor.prototype.handleOnUpdate = function () {
        this.props.onUpdate(this.state.editor);
    };
    Editor.prototype.render = function () {
        var _a = this.props, id = _a.id, children = _a.children;
        return (react.createElement("div", { id: id }, children));
    };
    Editor.defaultProps = {
        id: 'gjs',
        presetType: 'webpage',
        width: 'auto',
        height: '100vh',
        blockManager: {},
        storageManager: {},
        styleManager: {},
        blocks: [],
        components: [],
    };
    return Editor;
}(react.Component));

export { Editor };
//# sourceMappingURL=react-grapesjs.js.map
